/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: bufferbuilder.cxx,v 1.15 2001/04/05 18:40:22 stevek Exp $
 */

#include <wr_conf.h>

#include <wsi_types.h>

#include "bufferbuilder.h"

#include <iostream>

using namespace std;

void BufferBuilder::Dump()
{
   cout << nRows << " row(s)" << endl;

   for( int i = 0; i <= nRows; i++ )
   {
      if( ( i == nRows ) && !colCount )
         break;

      bbstring *row = rows.at( i );

      if( i < nRows )
      {
         cout << "Complete Row #" << i << ": ";

         for( int j = 0; j < nCols; j++ )
         {
            if( j ) cout << "\t";
            cout << j << ":" << row[j].c_str();
         }
      }
      else
      {
         cout << "Partial Row #" << i << ": ";

         for( int j = 0; j < colCount; j++ )
         {
            if( j ) cout << "\t";
            cout << j << ":" << row[j].c_str();
         }
      }

      cout << endl;
   }
}

void BufferBuilder::EndRow()
{
   if( colCount < nCols )
      for( ; colCount < nCols; )
         Add( (char *)NULL );

   colCount = 0;
   nRows++;
}

void BufferBuilder::Add( bbstring &s )
{
   if( colCount >= nCols )
      throw BufferException( _TEXT("max columns exceeded") );

   if( !colCount )
   {
      curRow = new bbstring[nCols];
      rows.push_back( curRow );
   }

   curRow[colCount++] = s;
}

void BufferBuilder::Add( LPTSTR cval )
{
   if( cval == NULL )
      cval = _TEXT(" ");

   Add( *(new bbstring(cval)) );
}

void BufferBuilder::Add( int ival )
{
   TCHAR buf[64];
   _stprintf( buf, _TEXT("%d"), ival );
   Add( buf );
}

void BufferBuilder::Add( long lval )
{
   TCHAR buf[64];
   _stprintf( buf, _TEXT("%ld"), lval );
   Add( buf );
}

void BufferBuilder::Add( void *pval )
{
   TCHAR buf[10];
   _stprintf( buf, _TEXT("%p"), pval );
   Add( buf );
}

void *BufferBuilder::Construct( int *length, BOOL unicode )
{
   if( colCount ) EndRow();

   int len = sizeof(size_t) * 2;
   len += nRows * nCols * sizeof(LPTSTR *);

   int hdrLen = len;

   for( int i = 0; i < nRows; i++ )
   {
      bbstring *row = rows.at( i );

      for( int j = 0; j < nCols; j++ )
         len += ( row[j].size() + 1 ) * sizeof(TCHAR);
   }

   PVOID buf = malloc( len );
   *((size_t *)buf) = nRows;
   *((size_t *)buf + 1) = nCols;

   int *rowptrs = (int *)( (pd_byte_t *)buf + sizeof(size_t) * 2 );
   LPTSTR ptr = (LPTSTR)( (pd_byte_t *)buf + hdrLen );

   for( i = 0; i < nRows; i++ )
   {
      bbstring *row = rows.at( i );

      for( int j = 0; j < nCols; j++ )
      {
         rowptrs[ i * nCols + j ] = (int)( (char *)ptr - (char *)buf );
         if( unicode )
            memcpy( ptr, row[j].c_str(),
                    ( row[j].size() + 1 ) * sizeof(TCHAR) );
         else
            WideCharToMultiByte( CP_ACP, 0, row[j].c_str(),
                                 row[j].size() + 1, (char *)ptr,
                                 ( row[j].size() + 1 ) * sizeof(TCHAR),
                                 NULL, NULL );

         ptr += row[j].size() + 1;
      }
   }

   *length = len;

   return buf;
}

