/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: bufferbuilder.h,v 1.11 2001/02/23 19:02:02 stevek Exp $
 */

#ifndef __BUFFERBUILDER_H
#define __BUFFERBUILDER_H 1

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786) 

#include <string>
#include <vector>

typedef std::basic_string<TCHAR> bbstring;

class BufferException
{
   protected:
      LPTSTR reason;

   public:
      BufferException( const LPTSTR reason )
      {
         if( reason == NULL )
            this->reason = _TEXT("a buffer exception occurred");
         else
            this->reason = reason;
      }

      ~BufferException() {};

      const LPTSTR ShowReason() const { return (const LPTSTR)reason; }
};

class BufferBuilder
{
   protected:
      int nRows;
      int nCols;
      int colCount;

      std::vector<bbstring *> rows;

      bbstring *curRow;

   private:
      void Add( bbstring &s );

   public:
      BufferBuilder( int cols ) { this->nCols = cols; nRows = colCount = 0; };
      ~BufferBuilder() {};

      void EndRow();

      void Add( LPTSTR cval );
      void Add( int ival );
      void Add( long lval );
      void Add( void *pval );

      int GetNumColumns() { return nCols; };
      int GetNumRows() { return nRows; };

      void Dump();

      void *Construct( int *len, BOOL unicode );
};

#endif /* __BUFFERBUILDER_H */

