/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: clientcomm.cxx,v 1.76 2001/08/06 20:00:42 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Routines to communicate with WSS management clients
 */

#include <wr_conf.h>

#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <errno.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_module.h>
#include <wss_mutex.h>
#include <wss_process.h>
#include <wss_db.h>
#include <wss_vm.h>
#include <wss_library.h>
#include <wss_linker.h>
#include <wss_wrapper.h>
#include <wss_wrv.h>
#include <wss_wrvevent.h>

#include <wsi_time.h>

#include <wrerrno.h>

#pragma warning(disable: 4786)

#include "service.h"
#include "gwm.h"
#include "gwm_comm.h"
#include "bufferbuilder.h"


extern "C" {
void    IMPORT wss_db_set_row_value_from_string __P(( wqcd_t, wqrow_t, int,
                                                      pd_char_t * ));
int     IMPORT wss_db_insert_nulls              __P(( u_long ));
int     IMPORT wss_db_delete_by_handle          __P(( u_long, u_long ));
}

static void _GWMClientEventThread __P(( LPVOID ));

typedef struct _GWM_CLIENT_PACKET_DISPATCH GWM_CLIENT_PACKET_DISPATCH,
                                           *LPGWM_CLIENT_PACKET_DISPATCH;
typedef struct _GWM_CLIENT_EVENT           GWM_CLIENT_EVENT,
                                           *LPGWM_CLIENT_EVENT;

struct _GWM_CLIENT_PACKET_DISPATCH
{
   PipeSrvCommDispatchFunc fDispatch;
   LPTSTR                  tcDescription;
};

struct _GWM_CLIENT_EVENT
{
   wss_proc_t         *pWRP;
   long                lLength;
   PVOID               pEvent;
   LPGWM_CLIENT_EVENT  pNext;
};


GWM_CLIENT_PACKET_DISPATCH g_packetDispatch[] =
{
   { NULL,                            _TEXT("Invalid command zero") },
   { ClientComm_LoadModule,           _TEXT("Load a module") },
   { ClientComm_UnloadModule,         _TEXT("Unload a module") },
   { ClientComm_WrapperEnabled,       _TEXT("Set/unset a wrapper enabled") },
   { ClientComm_StatLinkers,          _TEXT("Display available linkers") },
   { ClientComm_StatRefs,             _TEXT("Display reference counts for WSS components") },
   { ClientComm_StatMemory,           _TEXT("Display memory statistics") },
   { ClientComm_GetTableHandles,      _TEXT("Get table handles") },
   { ClientComm_GetTableColumnDef,    _TEXT("Get column definitions for table") },
   { ClientComm_GetTableRows,         _TEXT("Get rows for table") },
   { ClientComm_GetTableHandleByName, _TEXT("Get handle for named table") },
   { ClientComm_GetModuleById,        _TEXT("Get module from id") },
   { ClientComm_GetWrapperById,       _TEXT("Get wrapper from id") },
   { ClientComm_GetProcsByWrapperId,  _TEXT("Get processes wrapped by wrapper") },
   { ClientComm_GetActCritById,       _TEXT("Get activation criteria from id")},
   { ClientComm_GetLibraryById,       _TEXT("Get wrapper library from id") },
   { ClientComm_GetDeactCritById,     _TEXT("Get deactivation criteria from id") },
   { ClientComm_GetModuleIdByName,    _TEXT("Get module name from id") },
   { ClientComm_GetProcByPID,         _TEXT("Get process associated with PID")},
   { ClientComm_GetActiveWrappers,    _TEXT("Get list of active wrappers") },
   { ClientComm_SetTableRowField,     _TEXT("Set the field value of a row") },
   { ClientComm_AddNullTableRow,      _TEXT("Add a new null-value row to a table") },
   { ClientComm_AddTableRow,          _TEXT("Add a new row to a table") },
   { ClientComm_DelTableRow,          _TEXT("Delete a row from a table") },
   { ClientComm_GetVersion,           _TEXT("Get client communication version") },
   { ClientComm_AdminDone,            _TEXT("Signal that criteria should now be activated") }
};

#define CLIENTCOMM_DISPATCH_TABLE_SIZE ( sizeof(g_packetDispatch) / \
                                      sizeof(GWM_CLIENT_PACKET_DISPATCH) )

extern BOOL        g_bDebugging;
extern HANDLE      g_hServerStopEvent;


static int         _eventThread;

HANDLE             g_hPostReadyEvent = NULL;

LPGWM_CLIENT_EVENT g_pProcessingQueue = NULL;
LPGWM_CLIENT_EVENT g_pPendingQueue    = NULL;
WSS_MUTEX_PROTECT( g_pPendingQueue );

extern "C" {
extern wss_linker_t      WSSLINKAGE *wss_linker_list;
extern wss_linker_file_t WSSLINKAGE  wss_linker_file_list;
WSS_MUTEX_EXPORT                    (wss_linker_list);
WSS_MUTEX_EXPORT                    (wss_linker_file_list);
}


/////////////////////////////////////////////////////////////////////////////
// private functions
//

static void
_GWMClientEventThread( LPVOID args )
{
   HANDLE     hEvents[2];

   hEvents[0] = g_hServerStopEvent;
   hEvents[1] = g_hPostReadyEvent;

   if( g_bDebugging )
      _tprintf( _TEXT("ClientEventThread: Initialized\n") );

   while( 1 )
   {
      WSS_MUTEX_ENTER( g_pPendingQueue );
      if( g_pPendingQueue )
      {
         if( g_bDebugging )
            _tprintf( _TEXT("ClientEventThread: Grabbing pending queue\n") );

         // Make the pending queue the processing queue
         g_pProcessingQueue = g_pPendingQueue;
         g_pPendingQueue = NULL;
      }
      WSS_MUTEX_EXIT( g_pPendingQueue );

      if( g_pProcessingQueue )
      {
         if( g_bDebugging )
            _tprintf( _TEXT("ClientEventThread: Working on processing queue\n") );

         // Work through the processing queue
         while( g_pProcessingQueue )
         {
            LPGWM_CLIENT_EVENT pEvent = g_pProcessingQueue;
            g_pProcessingQueue = g_pProcessingQueue->pNext;

            PIPESRV_COMM_RESP_HEADER commHdr;
            DWORD dwID = GetTickCount() << GWM_COMM_ID_SHIFT;
            commHdr.commandId = WRV_EVENT_POST | ( dwID & GWM_COMM_ID_MASK );
            commHdr.pid = pEvent->pWRP->wrp_p->p_pid;
            commHdr.flags = 0;
            commHdr.pktlen = sizeof(commHdr) + pEvent->lLength;
            commHdr.errorCode = 0;

            GWMClientCommWritePacket(
               (LPPIPESRV_PIPE_INFO)WSI_PROC_OPAQUE(pEvent->pWRP)->hClientPipe, &commHdr,
               pEvent->pEvent );

            wss_proc_rele( pEvent->pWRP );

            wss_free( pEvent->pEvent );
            wss_free( pEvent );
         }
      }

      if( g_bDebugging )
         _tprintf( _TEXT("ClientEventThread: Sleeping for event\n") );

      DWORD dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
      if( dwWait != ( WAIT_OBJECT_0 + 1 ) )
         goto eventend;

      if( g_bDebugging )
         _tprintf( _TEXT("ClientEventThread: Waking from event\n") );
   }

eventend:
   _endthread();
}


/////////////////////////////////////////////////////////////////////////////
// public functions
//

void
GWMClientCommSetup()
{
   // Create event for post thread to wait on
   g_hPostReadyEvent = CreateEvent( NULL, FALSE, FALSE, NULL );

   // Create mutex which protects the pending queue
   WSS_MUTEX_INIT( g_pPendingQueue );

   // Construct event thread (posts events to viewer)
   _eventThread = _beginthread( _GWMClientEventThread, 0, NULL );
}

//
// GWMClientCommProcessClient
//
// Process client communication packets from passed pipe
//
// Peeks the pipe to see if any data is available, if so, reads a packet
// If there is no data available, this pipe will be returned to the
// active queue.
//
BOOL
GWMClientCommProcessPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   BOOL bRet = GWMClientCommReadPacket( pInfo );
   if( !bRet )
      return TRUE;

   // process packet
   //
   if( GWM_COMM_CMD(pInfo->commHdr) >= CLIENTCOMM_DISPATCH_TABLE_SIZE )
   {
      if( g_bDebugging )
         _tprintf( _TEXT("GWMClientCommProcessPacket: invalid packet encountered\n") );

      PIPESRV_COMM_RESP_HEADER commHdr;
      commHdr.commandId = pInfo->commHdr.commandId;
      commHdr.pid = pInfo->commHdr.pid;
      commHdr.flags = 0;
      commHdr.pktlen = sizeof(commHdr);
      commHdr.errorCode = -ENOSYS;

      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }
   else
   {
      if( g_bDebugging )
         _tprintf( _TEXT("Processing: %s\n"),
                   g_packetDispatch[GWM_COMM_CMD(pInfo->commHdr)].tcDescription );

      if( g_packetDispatch[GWM_COMM_CMD(pInfo->commHdr)].fDispatch )
      {
         bRet = (*g_packetDispatch[GWM_COMM_CMD(pInfo->commHdr)].fDispatch)( pInfo );
         if( !bRet )
            return TRUE;
      }
      else
         return TRUE;
   }

   return FALSE;
}

BOOL GWMClientCommReadPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   OVERLAPPED os;
   DWORD      dwWait;
   HANDLE     hEvents[2];
   DWORD      cbRead;
   BOOL       bRet;

   if( pInfo == NULL )
      return FALSE;
   
   hEvents[0] = pInfo->hEvents[0];
   hEvents[1] = pInfo->hEvents[PIPESRV_OVERLAPPED_IO_READ_EVENT];

   WSS_MUTEX_ENTER( pInfo->readPipe );

   // init the overlapped structure
   //
   memset( &os, 0, sizeof(OVERLAPPED) );
   os.hEvent = hEvents[1];
   ResetEvent( hEvents[1] );

   // grab packet header
   //
   bRet = ReadFile( pInfo->hPipe, &(pInfo->commHdr), sizeof(PIPESRV_COMM_HEADER),
                    &cbRead, &os );

   if( !bRet )
   {
      switch( GetLastError() )
      {
         case ERROR_IO_PENDING:
#ifdef DEBUG_IO
            if( g_bDebugging )
               _tprintf( _TEXT("Waiting(1)...\n") );
#endif /* DEBUG_IO */
            dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
            if( dwWait != WAIT_OBJECT_0 + 1 )
            {
               WSS_MUTEX_EXIT( pInfo->readPipe );
               return FALSE;
            }
            break;

         case ERROR_HANDLE_EOF:
#ifdef DEBUG_IO
            if( g_bDebugging )
               _tprintf( _TEXT("EOF reached\n") );
#endif /* DEBUG_IO */

            WSS_MUTEX_EXIT( pInfo->readPipe );
            return FALSE;

         default:
#ifdef DEBUG_IO
            if( g_bDebugging )
               _tprintf( _TEXT("Error: %ld\n"), GetLastError() );
#endif /* DEBUG_IO */

            WSS_MUTEX_EXIT( pInfo->readPipe );
            return FALSE;
      }
   }

   // grab data
   //
   if( pInfo->commHdr.pktlen > sizeof(PIPESRV_COMM_HEADER) )
   {
      DWORD buflen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

      if( pInfo->pBuf )
         pInfo->pBuf = realloc( pInfo->pBuf, buflen );
      else
         pInfo->pBuf = malloc( buflen );
      
      // init the overlapped structure
      //
      memset( &os, 0, sizeof(OVERLAPPED) );
      os.hEvent = hEvents[1];
      ResetEvent( hEvents[1] );

      bRet = ReadFile( pInfo->hPipe, pInfo->pBuf, buflen, &cbRead, &os );

      if( !bRet )
      {
         switch( GetLastError() )
         {
            case ERROR_IO_PENDING:
#ifdef DEBUG_IO
               if( g_bDebugging )
                  _tprintf( _TEXT("Waiting(2)...\n") );
#endif /* DEBUG_IO */
               dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
               if( dwWait != WAIT_OBJECT_0 + 1 )
               {
                  // not overlapped i/o event
                  // - error occured or server stop signaled
                  WSS_MUTEX_EXIT( pInfo->readPipe );
                  return FALSE;                   
               }
               break;

            case ERROR_HANDLE_EOF:
#ifdef DEBUG_IO
               if( g_bDebugging )
                  _tprintf( _TEXT("EOF reached\n") );
#endif /* DEBUG_IO */

               WSS_MUTEX_EXIT( pInfo->readPipe );
               return FALSE;

            default:
#ifdef DEBUG_IO
               if( g_bDebugging )
                  _tprintf( _TEXT("Error: %ld\n"), GetLastError() );
#endif /* DEBUG_IO */

               WSS_MUTEX_EXIT( pInfo->readPipe );
               return FALSE;
         }
      }
   }

   WSS_MUTEX_EXIT( pInfo->readPipe );
   return TRUE;
}

BOOL
GWMClientCommWritePacket( LPPIPESRV_PIPE_INFO pInfo,
                          LPPIPESRV_COMM_RESP_HEADER pCommHdr,
                          PVOID pData )
{
   OVERLAPPED os;
   DWORD      dwWait;
   HANDLE     hEvents[2];
   DWORD      dwDatalen;
   DWORD      cbWritten;
   BOOL       bRet;
   
   dwDatalen = pCommHdr->pktlen - sizeof(*pCommHdr);

   if( pInfo == NULL )
      return FALSE;

   hEvents[0] = pInfo->hEvents[0];
   hEvents[1] = pInfo->hEvents[PIPESRV_OVERLAPPED_IO_WRITE_EVENT];

   WSS_MUTEX_ENTER( pInfo->writePipe );

   // init the overlapped structure
   //
   memset( &os, 0, sizeof(OVERLAPPED) );
   os.hEvent = hEvents[1];
   ResetEvent( hEvents[1] );

   // send out the header...
   //
   bRet = WriteFile( pInfo->hPipe, pCommHdr, sizeof(*pCommHdr), &cbWritten, &os );

   if( !bRet )
   {
      switch( GetLastError() )
      {
         case ERROR_IO_PENDING:
#ifdef DEBUG_IO
            if( g_bDebugging )
               _tprintf( _TEXT("Waiting on WriteFile(1)...\n") );
#endif /* DEBUG_IO */
            dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
            switch( dwWait )
            {
               case WAIT_OBJECT_0 + 1:
                  break;

               case WAIT_FAILED:
#ifdef DEBUG_IO
                  if( g_bDebugging )
                     _tprintf( _TEXT("Wait on WriteFile(1) failed.\n") );
#endif /* DEBUG_IO */

                  WSS_MUTEX_EXIT( pInfo->writePipe );
                  return FALSE;

               default:
                  // not overlapped i/o event
                  // - error occured or server stop signaled
#ifdef DEBUG_IO
                  if( g_bDebugging )
                     _tprintf( _TEXT("WriteFile(1) dwWait = %ld\n"), dwWait );
#endif /* DEBUG_IO */

                  WSS_MUTEX_EXIT( pInfo->writePipe );
                  return FALSE;
            }
            break;

         default:
#ifdef DEBUG_IO
            if( g_bDebugging )
               _tprintf( _TEXT("WriteFile(1) Error: %ld\n"), GetLastError() );
#endif /* DEBUG_IO */

            WSS_MUTEX_EXIT( pInfo->writePipe );
            return FALSE;
      }
   }
   else
   {
#ifdef DEBUG_IO
      if( g_bDebugging )
         _tprintf( _TEXT("WriteFile(1) no waiting necessary.\n") );
#endif /* DEBUG_IO */
   }

#ifdef DEBUG_IO
   if( g_bDebugging )
      _tprintf( _TEXT("WriteFile(1): wrote %ld bytes\n"), sizeof(*pCommHdr) );
#endif /* DEBUG_IO */

   // send out the data...
   //
   if( dwDatalen > 0 )
   {
      // init the overlapped structure
      //
      memset( &os, 0, sizeof(OVERLAPPED) );
      os.hEvent = hEvents[1];
      ResetEvent( hEvents[1] );

      bRet = WriteFile( pInfo->hPipe, pData, dwDatalen, &cbWritten, &os );

      if( !bRet )
      {
         switch( GetLastError() )
         {
            case ERROR_IO_PENDING:
#ifdef DEBUG_IO
               if( g_bDebugging )
                  _tprintf( _TEXT("Waiting on WriteFile(2)...\n") );
#endif /* DEBUG_IO */
               dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
               if( dwWait != WAIT_OBJECT_0 + 1 )
               {
                  // not overlapped i/o event
                  // - error occured or server stop signaled
#ifdef DEBUG_IO
                  if( g_bDebugging )
                     _tprintf( _TEXT("WriteFile(2) dwWait = %ld\n"), dwWait );
#endif /* DEBUG_IO */

                  WSS_MUTEX_EXIT( pInfo->writePipe );
                  return FALSE;
               }
               break;

            default:
#ifdef DEBUG_IO
               if( g_bDebugging )
                  _tprintf( _TEXT("WriteFile(2) Error: %ld\n"), GetLastError() );
#endif /* DEBUG_IO */

               WSS_MUTEX_EXIT( pInfo->writePipe );
               return FALSE;
         }
      }
      else
      {
#ifdef DEBUG_IO
         if( g_bDebugging )
            _tprintf( _TEXT("WriteFile(2) no waiting necessary.\n") );
#endif /* DEBUG_IO */
      }

#ifdef DEBUG_IO
      if( g_bDebugging )
         _tprintf( _TEXT("WriteFile(2): wrote %ld bytes\n"), dwDatalen );
#endif /* DEBUG_IO */
   }
   else
   {
#ifdef DEBUG_IO
      if( g_bDebugging )
         _tprintf( _TEXT("WriteFile(2) no data, no write.\n") );
#endif /* DEBUG_IO */
   }

   // process cached response
   if( ( pCommHdr->flags & GWM_COMM_RESENT ) == 0 )
   {
      LPPIPESRV_LAST_PACKET_DATA pPacket;
      WSS_MUTEX_ENTER( pInfo->pLastPackets );
      PIPESRV_LAST_PACKETS::iterator it = pInfo->pLastPackets->find( pCommHdr->tid );
      if( it == pInfo->pLastPackets->end() )
         pPacket = (LPPIPESRV_LAST_PACKET_DATA) malloc( sizeof(*pPacket) );
      else
      {
         // Use old packet cache data structure
         pPacket = (*it).second;
         pInfo->pLastPackets->erase( it );

         if( pPacket->pCommHdr )
         {
            free( pPacket->pCommHdr );
            pPacket->pCommHdr = NULL;
         }
         if( pPacket->pData )
         {
            free( pPacket->pData );
            pPacket->pData = NULL;
         }
      }

      if( pPacket )
      {
         pPacket->pCommHdr =
            (LPPIPESRV_COMM_RESP_HEADER) malloc( sizeof(*pCommHdr) );
         if( pPacket->pCommHdr )
         {
            memcpy( pPacket->pCommHdr, pCommHdr, sizeof(pCommHdr) );
            if( pData )
            {
               pPacket->pData = malloc( dwDatalen );
               if( pPacket->pData )
               {
                  memcpy( pPacket->pData, pData, dwDatalen );
                  pInfo->pLastPackets->insert( PIPESRV_LAST_PACKETS::value_type(pCommHdr->tid, pPacket) );
               }
               else
               {
                  free( pPacket->pCommHdr );
                  free( pPacket );
               }
            }
            else
               pPacket->pData = NULL;
         }
         else
            free( pPacket );
      }
      WSS_MUTEX_EXIT( pInfo->pLastPackets );
   }

   WSS_MUTEX_EXIT( pInfo->writePipe );
   return TRUE;
}

BOOL
GWMClientCommOpenCallback( LPPIPESRV_PIPE_INFO pInfo )
{
   return TRUE;
}

void
GWMClientCommCloseCallback( LPPIPESRV_PIPE_INFO pInfo )
{
   wss_proc_t *wrp;

   WSS_MUTEX_ENTER( wss_proc_list );
   for( wrp = wss_proc_list.lh_first; wrp; wrp = wrp->wrp_p_list.le_next )
   {
      if( WSI_PROC_OPAQUE(wrp)->hClientPipe == (HANDLE)pInfo )
      {
         WSI_PROC_OPAQUE(wrp)->hClientPipe = NULL;

         BOOL killproc = FALSE;
         if( WSI_PROC_OPAQUE(wrp)->hLWMPipe == NULL )
            killproc = TRUE;

         WSS_MUTEX_EXIT( wss_proc_list );

         if( killproc == TRUE )
            wss_proc_exit( wrp->wrp_p );

         // clean up cached responses
         WSS_MUTEX_ENTER( pInfo->pLastPackets );
         for( PIPESRV_LAST_PACKETS::iterator it = pInfo->pLastPackets->begin();
              it != pInfo->pLastPackets->end();
              it++ )
         {
            LPPIPESRV_LAST_PACKET_DATA pPacket = (*it).second;

            pInfo->pLastPackets->erase( it );
            if( pPacket->pCommHdr )
            {
               free( pPacket->pCommHdr );
               pPacket->pCommHdr = NULL;
            }
            if( pPacket->pData )
            {
               free( pPacket->pData );
               pPacket->pData = NULL;
            }
            free( pPacket );
         }
         WSS_MUTEX_EXIT( pInfo->pLastPackets );

         return;
      }
   }
   WSS_MUTEX_EXIT( wss_proc_list );
}

void
GWMClientCommPostEvent( const pd_char_t *group, u_long types,
                        size_t length, const void *buffer)
{
#if 0
   wss_wrv_event_t *event, *next;

   /* Process events pending */
   WSS_MUTEX_ENTER( wrp->wrp_viewer );
   for( event = wrp->wrp_viewer->wrv_ev_queue; event; event = next )
   {
      void *buffer;
      long  len;

      next = wrv_event_next( event, &wrp->wrp_viewer->wrv_ev_queue );

      len = sizeof(event->event) + event->event.buffer_length;
      buffer = wss_malloc( len );

      memcpy( buffer, &event->event, sizeof( event->event ) );
      if( event->event.buffer_length )
         memcpy( (pd_byte_t *)buffer + sizeof( event->event ),
                 event->event.buffer, event->event.buffer_length );

      /* Unlink event */
      wrp->wrp_viewer->wrv_ev_queue = next;
      if( next )
      {
         next->prev = event->prev;
         event->prev->next = next;
      }

      LPGWM_CLIENT_EVENT pEvent =
         (LPGWM_CLIENT_EVENT)wss_malloc( sizeof(GWM_CLIENT_EVENT) );

      wss_proc_ref( wrp );
      pEvent->pWRP = wrp;
      pEvent->lLength = len;
      pEvent->pEvent = buffer;

      WSS_MUTEX_ENTER( g_pPendingQueue );
      pEvent->pNext = g_pPendingQueue;
      g_pPendingQueue = pEvent;
      WSS_MUTEX_EXIT( g_pPendingQueue );

      SetEvent( g_hPostReadyEvent );

      if( event->event.buffer > 0 )
         wss_free( event->event.buffer );
      wss_free( event );
   }

   /* Flush event queue */
   wss_wrvevent_release_all( &((wss_wrv_t *)wrp->wrp_viewer)->wrv_ev_queue );

   WSS_MUTEX_EXIT( wrp->wrp_viewer );

   WSS_MUTEX_ENTER( wrp->wrp );
   wrp->wrp_flags &= ~WRPF_POSTED;
   WSS_MUTEX_EXIT( wrp->wrp );
   return FALSE;
#endif
}

BOOL
ClientCommStringIsValid( PVOID buffer, DWORD length, BOOL unicode )
{
   if( unicode )
   {
      pd_char_t *last = (pd_char_t *)buffer +
                         ( length / sizeof(pd_char_t) ) - 1;

      // Check for misalignment and non-null-terminated string
      if( ( length % sizeof(pd_char_t) ) || *last )
         return FALSE;
   }
   else if( *((PCHAR)buffer + length - 1) )
      return FALSE;

   return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// dispatch functions
//
BOOL
ClientComm_LoadModule( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int         namelen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);
   pd_char_t *modname = (pd_char_t *)pInfo->pBuf;

   if( !ClientCommStringIsValid( pInfo->pBuf, namelen,
                                 (pInfo->commHdr.flags & GWM_COMM_UNICODE) ?
                                    TRUE : FALSE ) )
   {
      commHdr.errorCode = -EINVAL;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
   {
      // Convert ASCII string to Unicode
      pInfo->pBuf = malloc( namelen * sizeof(pd_char_t) );
      MultiByteToWideChar( CP_ACP, 0, (LPCSTR)modname, namelen,
                           (pd_char_t *)pInfo->pBuf,
                           namelen * sizeof(pd_char_t) );
      free( modname );
      modname = (pd_char_t *)pInfo->pBuf;
   }

   wss_mod_t *wrm;
   int        error;
   if( ( wrm = wss_mod_new( modname, &error ) ) == NULL )
   {
      commHdr.errorCode = -error;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   commHdr.errorCode = wrm->wrm_id;
   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_UnloadModule( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wss_mod_t *wrm;
   int        error;
   int        type  = 0;
   int        id    = *((int *)pInfo->pBuf);

   /* Find the module */
   if( ( wrm = wss_mod_find( id ) ) != NULL )
   {
      type = wrm->wrm_type;
      wss_mod_rele( wrm );
   }

   /* Delete the module */
   error = wss_mod_delete( id );

   if( error )
      commHdr.errorCode = -error;
   else
   {
      /*
       * Post module unloaded event
       */
      wss_wrvevent_post_modunloaded( id, (enum wss_mod_type)type );

      commHdr.errorCode = id;
   }

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_WrapperEnabled( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = -ENOSYS;

   wss_crit_t *act;
   wss_mod_t  *wrm;
   int         id      = *((int *)pInfo->pBuf);
   int         enabled = *((int *)pInfo->pBuf + 1);

   if( ( wrm = wss_mod_find( id ) ) == NULL )
      commHdr.errorCode = -ENOENT;
   else
   {
      /* Make sure it's a criteria module */
      if( ( wrm->wrm_type != WR_MOD_ACTIVATION ) &&
          ( wrm->wrm_type != WR_MOD_DEACTIVATION ) )
      {
         wsi_printf( PD_MAKESTR("TBD: wrappers don't have an enabled bit to set (yet)\n") );
         commHdr.errorCode = -EINVAL;
      }

      /* Make sure the criteria exists and toggle the status accordinly */
      if( wrm->wrm_type == WR_MOD_ACTIVATION )
      {
         WSS_MUTEX_ENTER( wss_act_crit_list );
         for( act = wss_act_crit_list; act; act = act->crit_next )
            if( act->crit_module->wrm_id == id )
            {
               wss_crit_ref( act );
               act->crit_status = enabled?WRACT_ENABLE:WRACT_DISABLE;
               break;
            }
         WSS_MUTEX_EXIT( wss_act_crit_list );

         if( act == NULL )
         {
            WSS_MUTEX_ENTER( wss_deact_crit_list );
            for( act = wss_deact_crit_list; act; act = act->crit_next )
               if( act->crit_module->wrm_id == id )
               {
                  wss_crit_ref( act );
                  act->crit_status = enabled?WRACT_ENABLE:WRACT_DISABLE;
               }
            WSS_MUTEX_EXIT( wss_deact_crit_list );
         }
      
         if( act == NULL )
            commHdr.errorCode = -ENOENT;
         else
            wss_crit_rele( act );
      }

      wss_mod_rele( wrm );
   }

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_GetTableHandles( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int dataSize = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

   DWORD *handles  = NULL;
   int    nHandles = 0;

   for( handles = (DWORD *)pInfo->pBuf, nHandles = 0;
        ( (pd_byte_t *)handles < ( (pd_byte_t *)pInfo->pBuf + dataSize ) ) &&
           *handles;
        handles++, nHandles++ )
   ;

   if( nHandles > 0 )
   {
      handles = (DWORD *)wss_malloc( nHandles * sizeof(DWORD) );
      memset( handles, 0, nHandles * sizeof(DWORD) );
   }
   else
      handles = NULL;

   int     record_count = 0;
   int     size         = 0;
   wqtab_t tab;

   for( tab = (wqtab_t)( (DBHEAD->db_tables)?DB_REL2ABS(DBHEAD->db_tables)
                                            :NULL );
        tab;
        tab = (wqtab_t)( (tab->next)?DB_REL2ABS(tab->next):NULL ) )
   {
      if( nHandles > 0 )
      {
         DWORD rel = (DWORD)DB_ABS2REL( tab );
         for( int i = 0;
              ( i < nHandles ) && ( *( (DWORD *)pInfo->pBuf + i ) != rel );
              i++ )
         ;

         if( i >= nHandles )
            continue;

         handles[i] = *( (DWORD *)pInfo->pBuf + i );
      }

      pd_char_t *name = DB_REL2ABS( tab->name );
      size += sizeof(wss_wrv_table_id_t) + ( wsi_strlen( name ) + 1 ) *
                ( (commHdr.flags & GWM_COMM_UNICODE) ? sizeof(pd_char_t) : 1 );
      record_count++;
   }

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetTableHandles: %d record(s)\n"),
                record_count );

   DWORD rel;
   int   hindex = 0;

   if( nHandles > 0 )
   {
      if( record_count > 0 )
      {
         for( hindex = 0; handles[hindex] == 0; hindex++ )
         ;

         tab = (wqtab_t)DB_REL2ABS( handles[hindex] );
         rel = handles[hindex];
      }
      else
         tab = NULL;
   }
   else
   {
      tab = (wqtab_t)( (DBHEAD->db_tables)?DB_REL2ABS(DBHEAD->db_tables)
                                          :NULL );
      rel = (DWORD)(DBHEAD->db_tables);
   }

   wss_wrv_table_id_t *data   = (wss_wrv_table_id_t *)wss_malloc( size );
   wss_wrv_table_id_t *prev   = NULL;
   DWORD               offset = sizeof(wss_wrv_table_id_t) * record_count;

   commHdr.pktlen += size;
   memset( (PVOID)data, 0, size );

   for( int i = 0; tab && ( i < record_count ); i++ )
   {
      wss_wrv_table_id_t *nid = data + i;
      if( prev )
         prev->next = (wss_wrv_table_id_t *)( sizeof(wss_wrv_table_id_t) * i );
      prev = nid;

      nid->handle = (DWORD)DB_ABS2REL( tab );

      pd_char_t *name      = DB_REL2ABS( tab->name );
      int         name_size = wsi_strlen(name) + 1;

      nid->name = (pd_char_t *)offset;

      if( pInfo->commHdr.flags & GWM_COMM_UNICODE )
         memcpy( (pd_byte_t *)data + offset, name, name_size *
                 sizeof(pd_char_t) );
      else
      {
         if( !WideCharToMultiByte( CP_ACP, 0, name, name_size,
                                   (char *)data + offset,
                                   name_size * sizeof(pd_char_t), NULL,
                                   NULL ) )
         {
            if( g_bDebugging )
            {
               LPVOID lpMsgBuf;

               FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                              FORMAT_MESSAGE_FROM_SYSTEM |
                              FORMAT_MESSAGE_IGNORE_INSERTS,
                              NULL, GetLastError(),
                              MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
                              (LPTSTR) &lpMsgBuf, 0, NULL );
               _tprintf( _TEXT("ClientComm_GetTableHandles: %s\n"),
                         (LPCTSTR)lpMsgBuf );
               LocalFree( lpMsgBuf );
            }
         }
      }

      offset += name_size *
                ( (commHdr.flags & GWM_COMM_UNICODE) ? sizeof(pd_char_t) : 1 );

      if( nHandles > 0 )
      {
         if( ( i + 1 ) < record_count )
         {
            for( ; handles[hindex] == 0; hindex++ )
            ;

            tab = (wqtab_t)DB_REL2ABS( handles[hindex] );
            rel = handles[hindex];
         }
         else
            tab = NULL;
      }
      else
      {
         rel = (DWORD)tab->next;
         tab = (wqtab_t)( (tab->next)?DB_REL2ABS(tab->next):NULL );
      }
   }

   if( prev )
      prev->next = NULL;

   if( ( nHandles > 0 ) && handles )
      wss_free( (void *)handles );

   if( commHdr.errorCode == 0 )
      commHdr.errorCode = record_count;

   BOOL bRet = GWMClientCommWritePacket( pInfo, &commHdr, (PVOID)data );

   if( data )
      wss_free( (void *)data );

   return bRet;
}

BOOL
ClientComm_GetTableColumnDef( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *indata= (wrv_h_t *)pInfo->pBuf;
   DWORD handle = indata->handle;

   wqtab_t tab = wss_db_find_table_by_handle( handle );
   if( tab == NULL )
   {
      commHdr.errorCode = -ENOENT;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   int    record_count = tab->col_count;
   DWORD  offset       = sizeof(wqcd) * record_count;
   int    size         = offset;
   wqcd_t col_defs     = (wqcd_t)( (tab->col_defs)?DB_REL2ABS(tab->col_defs):NULL );

   for( int i = 0; i < record_count; i++ )
   {
      wqcd_t cd = col_defs + i;
      size += ( wsi_strlen( DB_REL2ABS(cd->name) ) + 1 ) * sizeof(pd_char_t);
   }

   commHdr.pktlen += size;
   commHdr.errorCode = record_count;

   PVOID data = wss_malloc( size );

   for( i = 0; i < record_count; i++ )
   {
      wqcd_t      cd   = col_defs + i;
      wqcd_t      ncd  = (wqcd_t)data + i;
      pd_char_t *name = DB_REL2ABS( cd->name );

      memcpy( ncd, cd, sizeof(wqcd) );

      int str_len = wsi_strlen( name ) + 1;

      if( commHdr.flags & GWM_COMM_UNICODE )
         memcpy( (pd_byte_t *)data + offset, name,
                 str_len * sizeof(pd_char_t) );
      else
         WideCharToMultiByte( CP_ACP, 0, name, str_len,
                              (char *)data + offset,
                              str_len * sizeof(pd_char_t), NULL, NULL );

      ncd->name = (pd_char_t *)offset;
      offset += str_len *
                ( (commHdr.flags & GWM_COMM_UNICODE) ? sizeof(pd_char_t) : 1 );
   }

   wss_db_unlock_table( tab );

   BOOL bRet = GWMClientCommWritePacket( pInfo, &commHdr, data );

   if( data )
      wss_free( data );

   return bRet;
}

BOOL
ClientComm_GetTableRows( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int datalen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

   wrv_h_t *indata = (wrv_h_t *)pInfo->pBuf;
   int handle = indata->handle;

   wqtab_t tab = wss_db_find_table_by_handle( handle );

   if( tab == NULL )
   {
      commHdr.errorCode = -ENOENT;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   wqtabopaque_t opaque = (wqtabopaque_t)( (tab->opaque) ? DB_REL2ABS(tab->opaque)
                                                         : NULL );

   for( DWORD *handles = (DWORD *)(indata + 1), nHandles = 0;
        ( (pd_byte_t *)handles < ( (pd_byte_t *)pInfo->pBuf + datalen ) ) &&
           *handles;
        handles++, nHandles++ )
   ;

   if( nHandles > 0 )
   {
      handles = (DWORD *)wss_malloc( sizeof(DWORD) * nHandles );
      if( handles == NULL )
      {
         commHdr.errorCode = -ENOMEM;
         wss_db_unlock_table( tab );
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }

      memset( handles, 0, sizeof(DWORD) * nHandles );
   }

   int record_count = 0;

   for( wqrow_t row = (wqrow_t)( (tab->rows)?DB_REL2ABS(tab->rows):NULL );
        row;
        row = (wqrow_t)( (row->next)?DB_REL2ABS(row->next):NULL) )
   {
      DWORD rel = (DWORD)DB_ABS2REL( row );

      if( nHandles > 0 )
      {
         for( int i = 0;
              ( i < nHandles ) && ( *( (DWORD *)(indata + 1) + i ) != rel );
              i++ )
         ;

         if( i >= nHandles )
            continue;

         handles[i] = *( (DWORD *)(indata + 1) + i );
      }

      record_count++;
   }

   DWORD offset        = sizeof(wss_wrv_db_row_t) * record_count;
   int   string_offset = offset + sizeof(wqval) * tab->col_count * record_count;
   int   size          = string_offset;
   int   hindex        = 0;
   DWORD rel           = 0;

   if( nHandles > 0 )
   {
      if( record_count > 0 )
      {
         for( hindex = 0; handles[hindex] == 0; hindex++ )
         ;

         row = (wqrow_t)DB_REL2ABS( handles[hindex] );
         rel = handles[hindex];
      }
      else
         row = NULL;
   }
   else
   {
      rel = (DWORD)tab->rows;
      row = (wqrow_t)( (tab->rows)?DB_REL2ABS(tab->rows):NULL );
   }

   for( int i = 0; row && ( i < record_count ); i++ )
   {
      wqcd_t  cd       = (wqcd_t)( (tab->col_defs)?DB_REL2ABS(tab->col_defs):NULL );
      wqval_t val_list = (wqval_t)( (row->vals)?DB_REL2ABS(row->vals):NULL );

      for( int k = 0; k < tab->col_count; k++ )
      {
         wqval_t val = val_list + k;

         if( cd[k].dtype == pd_char_t_t )
         {
            pd_char_t *value = DB_REL2ABS(val->v.cval);
            size += ( wsi_strlen( value ) + 1 ) * sizeof(pd_char_t);
         }
      }

      if( nHandles > 0 )
      {
         if( ( i + 1 ) < record_count )
         {
            for( ; handles[hindex] == 0; hindex++ )
            ;

            row = (wqrow_t)DB_REL2ABS( handles[hindex] );
         }
         else
            row = NULL;
      }
      else
      {
         row = (wqrow_t)( (row->next)?DB_REL2ABS(row->next):NULL );
      }
   }

   if( nHandles > 0 )
   {
      if( record_count > 0 )
      {
         for( hindex = 0; handles[hindex] == 0; hindex++ )
         ;

         row = (wqrow_t)DB_REL2ABS( handles[hindex] );
         rel = handles[hindex];
      }
      else
         row = NULL;
   }
   else
   {
      rel = (DWORD)tab->rows;
      row = (wqrow_t)( (tab->rows)?DB_REL2ABS(tab->rows):NULL );
   }

   pd_byte_t *data = (pd_byte_t *)wss_malloc( size );
   if( data == NULL )
   {
      wss_db_unlock_table( tab );
      commHdr.errorCode = -ENOMEM;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   commHdr.errorCode = record_count;
   commHdr.pktlen += size;

   for( i = 0; row && ( i < record_count ); i++ )
   {
      wss_wrv_db_row_t *nrow = (wss_wrv_db_row_t *)data + i;

      nrow->handle = rel;
      nrow->numvals = tab->col_count;
      ROW( nrow )->vals = (wqval_t)offset;

      wqval_t nvalues = (wqval_t)( data + offset );
      offset += sizeof(wqval) * tab->col_count;

      wqcd_t  cd       = (wqcd_t)( (tab->col_defs)?DB_REL2ABS(tab->col_defs):NULL );
      wqval_t val_list = (wqval_t)( (row->vals)?DB_REL2ABS(row->vals):NULL );

      for( int k = 0; k < tab->col_count; k++ )
      {
         wqval_t nval = nvalues + k;
         wqval_t val  = val_list + k;

         memcpy( nval, val, sizeof(wqval) );

         if( cd[k].dtype == pd_char_t_t )
         {
            pd_char_t *value   = DB_REL2ABS(val->v.cval);
            int         val_len = wsi_strlen(value) + 1;

            if( commHdr.flags & GWM_COMM_UNICODE )
               memcpy( data + string_offset, value,
                       val_len * sizeof(pd_char_t) );
            else
               WideCharToMultiByte( CP_ACP, 0, value, val_len,
                                    (char *)data + string_offset,
                                    val_len * sizeof(pd_char_t), NULL, NULL );

            nval->v.cval = (pd_char_t *)string_offset;
            string_offset += val_len *
               ( (commHdr.flags & GWM_COMM_UNICODE) ? sizeof(pd_char_t) : 1 );
         }
      }

      if( nHandles > 0 )
      {
         if( ( i + 1 ) < record_count )
         {
            for( ; handles[hindex] == 0; hindex++ )
            ;

            row = (wqrow_t)DB_REL2ABS( handles[hindex] );
            rel = handles[hindex];
         }
         else
            row = NULL;
      }
      else
      {
         rel = (DWORD)row->next;
         row = (wqrow_t)( (row->next)?DB_REL2ABS(row->next):NULL );
      }

      wqrow_t next = NULL;
      if( row )
         next = (wqrow_t)( ( i + 1 ) * sizeof(wss_wrv_db_row_t) );

      ROW( nrow )->next = next;
   }

   wss_db_unlock_table( tab );

   if( ( nHandles > 0 ) && handles )
      wss_free( (void *)handles );

   BOOL bRet = GWMClientCommWritePacket( pInfo, &commHdr, data );

   wss_free( data );

   return bRet;
}

BOOL
ClientComm_GetTableHandleByName( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int         namelen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);
   pd_char_t *tabname = (pd_char_t *)pInfo->pBuf;

   if( !ClientCommStringIsValid( pInfo->pBuf, namelen,
                                 (pInfo->commHdr.flags & GWM_COMM_UNICODE) ?
                                    TRUE : FALSE ) )
   {
      commHdr.errorCode = -EINVAL;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
   {
      // Convert ASCII string to Unicode
      pInfo->pBuf = malloc( namelen * sizeof(pd_char_t) );
      MultiByteToWideChar( CP_ACP, 0, (LPCSTR)tabname, namelen,
                           (pd_char_t *)pInfo->pBuf,
                           namelen * sizeof(pd_char_t) );
      free( tabname );
      tabname = (pd_char_t *)pInfo->pBuf;
   }

   wqtab_t tab = wss_db_find_table_by_name( tabname );
   if( tab == NULL )
   {
      commHdr.errorCode = -ENOENT;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   pd_char_t *name = DB_REL2ABS( tab->name );
   namelen = wsi_strlen( name ) + 1;
   void *data = wss_malloc( namelen * sizeof(pd_char_t *) + sizeof(wss_wrv_table_id_t) );

   wss_wrv_table_id_t *id = (wss_wrv_table_id_t *)data;
   id->handle = (DWORD)DB_ABS2REL( tab );
   id->name = (pd_char_t *)sizeof(*id);
   if( commHdr.flags & GWM_COMM_UNICODE )
      memcpy( (pd_byte_t *)(id + 1), (pd_byte_t *)name,
              namelen * sizeof(pd_char_t) );
   else
      WideCharToMultiByte( CP_ACP, 0, name, namelen,
                           (char *)(id + 1), namelen * sizeof(pd_char_t *), NULL, NULL );

   commHdr.pktlen += namelen + sizeof(wss_wrv_table_id_t);

   wss_db_unlock_table( tab );

   commHdr.errorCode = 1;

   BOOL bRet = GWMClientCommWritePacket( pInfo, &commHdr, data );

   wss_free( data );

   return bRet;
}

BOOL
ClientComm_GetModuleById( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   wss_mod_t *wrm;

   WSS_MUTEX_ENTER( wss_module_list );

   if( id ) 
   {
      for( wrm = wss_module_list;
           wrm && ( wrm->wrm_id != id );
           wrm = wrm->wrm_next )
      ;

      if( wrm == NULL )
      {
         WSS_MUTEX_EXIT( wss_module_list );

         commHdr.errorCode = -EINVAL;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(5);

   for( wrm = wss_module_list; wrm; wrm = wrm->wrm_next )
   {
      if( ( id == 0 ) || ( id == wrm->wrm_id ) )
      {
         bb.Add( wrm->wrm_id );
         bb.Add( (LPTSTR)wrm->wrm_linked->linker_file_name );

         switch( wrm->wrm_type )
         {
            case WR_MOD_ACTIVATION:
               bb.Add( _TEXT("activation") );
               break;
            case WR_MOD_DEACTIVATION:
               bb.Add( _TEXT("deactivation") );
               break;
            case WR_MOD_WRAPPER:
               bb.Add( _TEXT("wrapper") );
               break;
            case WR_MOD_LIBRARY:
               bb.Add( _TEXT("library") );
               break;
            default:
               bb.Add( _TEXT("unknown") );
               break;
         }

#ifdef WR_SHOW_ADDR
         bb.Add( (PVOID)wrm->wrm_linked->linker_file_addr.addr );
#else
         bb.Add( _TEXT("N/A") );
#endif /* WR_SHOW_ADDR */
         bb.Add( (long)wrm->wrm_linked->linker_file_size );
         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_module_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetModuleById: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetWrapperById( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   wss_wrapper_t *w;

   WSS_MUTEX_ENTER( wss_wrapper_list );

   if( id ) 
   {
      for( w = wss_wrapper_list;
           w && ( w->w_module->wrm_id != id );
           w = w->w_next )
      ;

      if( w == NULL )
      {
         WSS_MUTEX_EXIT( wss_wrapper_list );
         commHdr.errorCode = -EINVAL;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(7);

   for( w = wss_wrapper_list; w; w = w->w_next )
   {
      if( ( id == 0 ) || ( id = w->w_module->wrm_id ) )
      {
         bb.Add( w->w_module->wrm_id );
         bb.Add( (LPTSTR)w->w_module->wrm_linked->linker_file_name );
         bb.Add( WSS_REF_COUNT(w->w_module->wrm_refcount) - 2 );
         bb.Add( w->w_module->wrm_uid );
         bb.Add( w->w_module->wrm_loadtime );
#ifdef WR_SHOW_ADDR
         bb.Add( (PVOID)wrm->wrm_linked->linker_file_addr.addr );
#else
         bb.Add( _TEXT("N/A") );
#endif /* WR_SHOW_ADDR */
         bb.Add( (long)w->w_module->wrm_linked->linker_file_size );
         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_wrapper_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetWrapperById: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetProcsByWrapperId( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   wss_proc_t *wrp;

   WSS_MUTEX_ENTER( wss_proc_list );

   if( id ) 
   {
      BOOL found = FALSE;
      for( wrp = wss_proc_list.lh_first; wrp; wrp = wrp->wrp_p_list.le_next )
      {
         WSS_MUTEX_ENTER( wrp->wrp_wi_list );
         for( wss_wrapper_instance_t *wi =
                 (wss_wrapper_instance_t *)(wrp->wrp_wi_list);
              wi;
              wi = wi->wi_next )
            if( id && wi->wi_wrapper &&
                ( wi->wi_wrapper->w_module->wrm_id == id ) )
               found = TRUE;
         WSS_MUTEX_EXIT( wrp->wrp_wi_list );
      }

      if( !found )
      {
         WSS_MUTEX_EXIT( wss_proc_list );
         commHdr.errorCode = -EINVAL;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(6);

   for( wrp = wss_proc_list.lh_first; wrp; wrp = wrp->wrp_p_list.le_next )
   {
      BOOL found = FALSE;
      int size = 0;
      WSS_MUTEX_ENTER( wrp->wrp_wi_list );
      for( wss_wrapper_instance_t *wi =
              (wss_wrapper_instance_t *)(wrp->wrp_wi_list);
           wi;
           wi = wi->wi_next )
         if( wi->wi_wrapper )
         {
            size += ( _tcslen( (LPTSTR)wi->wi_wrapper->w_module->wrm_linked->linker_file_name) +
                      1 ) * sizeof(TCHAR);
            if( id && ( wi->wi_wrapper->w_module->wrm_id == id ) )
               found = TRUE;
         }
      WSS_MUTEX_EXIT( wrp->wrp_wi_list );

      if( id && !found )
         continue;

      bb.Add( (int)wrp->wrp_pid );
      bb.Add( (LPTSTR)wrp->wrp_progname );
      bb.Add( 0 );                       // credentials uid
      bb.Add( (LPTSTR)wrp->wrp_path );
      bb.Add( (LPTSTR)wrp->wrp_cwd );

      if( size > 0 )
      {
         LPTSTR buf = (LPTSTR)malloc( size );
         if( buf == NULL )
         {
            WSS_MUTEX_EXIT( wss_proc_list );
            commHdr.errorCode = -ENOMEM;
            return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
         }
         memset( buf, 0, size );
         WSS_MUTEX_ENTER( wrp->wrp_wi_list );
         for( wss_wrapper_instance_t *wi =
                 (wss_wrapper_instance_t *)(wrp->wrp_wi_list);
              wi;
              wi = wi->wi_next )
            if( wi->wi_wrapper )
            {
               if( *buf )
                  _tcscat( buf, TEXT(",") );
               _tcscat( buf, (LPTSTR)wi->wi_wrapper->w_module->wrm_linked->linker_file_name );
            }
         WSS_MUTEX_EXIT( wrp->wrp_wi_list );

         bb.Add( buf );
         free( buf );
      }
      else
         bb.Add( PD_MAKESTR("") );

      bb.EndRow();
   }

   WSS_MUTEX_EXIT( wss_proc_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetProcsByWrapperId: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetActCritById( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   wss_crit_t *crit;

   WSS_MUTEX_ENTER( wss_act_crit_list );

   if( id ) 
   {
      for( crit = wss_act_crit_list;
           crit && ( crit->crit_module->wrm_id != id );
           crit = crit->crit_next )
      ;

      if( crit == NULL )
      {
         WSS_MUTEX_EXIT( wss_act_crit_list );
         commHdr.errorCode = -EINVAL;

         if( g_bDebugging )
            _tprintf( _TEXT("ClientComm_GetActCritById: sending response [ EINVAL ]\n") );
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(7);

   for( crit = wss_act_crit_list; crit; crit = crit->crit_next )
   {
      if( !id || ( id == crit->crit_module->wrm_id ) )
      {
         bb.Add( crit->crit_module->wrm_id );
         bb.Add( (LPTSTR)crit->crit_module->wrm_linked->linker_file_name );
         bb.Add( (crit->crit_status==WRACT_ENABLE)? PD_MAKESTR("enabled")
                                                  : PD_MAKESTR("disabled") );
         bb.Add( crit->crit_module->wrm_uid );
         bb.Add( crit->crit_module->wrm_loadtime );
#ifdef WR_SHOW_ADDR
         bb.Add( (PVOID)wrm->wrm_linked->linker_file_addr.addr );
#else
         bb.Add( _TEXT("N/A") );
#endif /* WR_SHOW_ADDR */
         bb.Add( (int)crit->crit_module->wrm_linked->linker_file_size );
         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_act_crit_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetActCritById: sending response [ %d ]\n"),
                commHdr.errorCode );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetLibraryById( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   int count = 0;

   wss_lib_t *lib = NULL;

   if( id )
   {
      lib = wss_lib_find( id );
      if( lib == NULL )
      {
         commHdr.errorCode = -ENOENT;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }

      count = id + 1;
   }
   else
   {
      WSS_MUTEX_ENTER( wss_library_list );
      for( lib = wss_library_list, count = 0;
           lib;
           lib = lib->lib_next, count++ )
      ;
      WSS_MUTEX_EXIT( wss_library_list );
   }

   BufferBuilder bb(8);

   WSS_MUTEX_ENTER( wss_library_list );

   for( lib = wss_library_list; lib; lib = lib->lib_next )
   {
      if( ( id == 0 ) || ( id == lib->lib_module->wrm_id ) )
      {
         bb.Add( lib->lib_module->wrm_id );
         bb.Add( (LPTSTR)lib->lib_module->wrm_linked->linker_file_name );
         bb.Add( WSS_REF_COUNT(lib->lib_module->wrm_refcount) - 2 );
         bb.Add( lib->lib_module->wrm_uid );
         bb.Add( lib->lib_module->wrm_loadtime );
#ifdef WR_SHOW_ADDR
         bb.Add( (PVOID)wrm->wrm_linked->linker_file_addr.addr );
#else
         bb.Add( _TEXT("N/A") );
#endif /* WR_SHOW_ADDR */
         bb.Add( (int)lib->lib_module->wrm_linked->linker_file_size );

         wss_symbol_t *symtab;
         int           sym;
         int           symlen = 0;
 
         for( symtab = lib->lib_symtab, sym = 0;
              symtab[sym].name != NULL;
              sym++ )
            symlen += ( wsi_strlen( symtab[sym].name ) + 1 ) *
                      sizeof(pd_char_t);

         if( symlen > 0 )
         {
            pd_char_t *symbuf    = (pd_char_t *)wss_malloc( symlen );
            int         symbuflen = 0;

            memset( symbuf, 0, symlen );

            for( symtab = lib->lib_symtab, sym = 0;
                 symtab[sym].name != NULL;
                 sym++ )
            {
               symlen = wsi_strlen( symtab[sym].name );

               if( symbuflen > 0 )
               {
                  memcpy( (pd_byte_t *)(symbuf + symbuflen),
                          PD_MAKESTR(","),
                          sizeof(pd_char_t) * 2 );
                  symbuflen++;
               }

               memcpy( (pd_byte_t *)(symbuf + symbuflen),
                       (pd_byte_t *)symtab[sym].name,
                       symlen * sizeof(pd_char_t) );

               symbuflen += symlen;
            }
            bb.Add( symbuf );
            wss_free( symbuf );
         }
         else
            bb.Add( PD_MAKESTR("") );

         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_library_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetDeactCritById: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetDeactCritById( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;
   int id = data->handle;

   wss_crit_t *crit;

   WSS_MUTEX_ENTER( wss_deact_crit_list );

   if( id ) 
   {
      for( crit = wss_deact_crit_list;
           crit && ( crit->crit_module->wrm_id != id );
           crit = crit->crit_next )
      ;

      if( crit == NULL )
      {
         WSS_MUTEX_EXIT( wss_deact_crit_list );

         commHdr.errorCode = -EINVAL;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(7);

   for( crit = wss_deact_crit_list; crit; crit = crit->crit_next )
   {
      if( !id || ( id == crit->crit_module->wrm_id ) )
      {
         bb.Add( crit->crit_module->wrm_id );
         bb.Add( (LPTSTR)crit->crit_module->wrm_linked->linker_file_name );
         bb.Add( (crit->crit_status==WRACT_ENABLE)? PD_MAKESTR("enabled")
                                                  : PD_MAKESTR("disabled") );
         bb.Add( crit->crit_module->wrm_uid );
         bb.Add( crit->crit_module->wrm_loadtime );
#ifdef WR_SHOW_ADDR
         bb.Add( (PVOID)wrm->wrm_linked->linker_file_addr.addr );
#else
         bb.Add( _TEXT("N/A") );
#endif /* WR_SHOW_ADDR */
         bb.Add( (int)crit->crit_module->wrm_linked->linker_file_size );
         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_deact_crit_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen += dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetDeactCritById: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetModuleIdByName( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int namelen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);
   int id = 0;

   if( namelen > 0 )
   {
      pd_char_t *modname = (pd_char_t *)pInfo->pBuf;

      if( !ClientCommStringIsValid( pInfo->pBuf, namelen,
                                    (pInfo->commHdr.flags & GWM_COMM_UNICODE) ?
                                       TRUE : FALSE ) )
      {
         commHdr.errorCode = -EINVAL;
         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }

      if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
      {
         // Convert ASCII string to Unicode
         pInfo->pBuf = malloc( namelen * sizeof(pd_char_t) );
         MultiByteToWideChar( CP_ACP, 0, (LPCSTR)modname, namelen,
                              (pd_char_t *)pInfo->pBuf,
                              namelen * sizeof(pd_char_t) );
         free( modname );
         modname = (pd_char_t *)pInfo->pBuf;
      }

      namelen /= sizeof(TCHAR);

      wss_mod_t *mod;

      WSS_MUTEX_ENTER( wss_module_list );

      for( mod = wss_module_list;
           mod && mod->wrm_linked->linker_file_name &&
           _tcsncmp( (LPTSTR)mod->wrm_linked->linker_file_name, (LPTSTR)pInfo->pBuf, namelen );
           mod = mod->wrm_next )
      ;

      if( mod )
         id = mod->wrm_id;

      WSS_MUTEX_EXIT( wss_module_list );

      commHdr.pktlen += sizeof(int);
   }
   else
      commHdr.errorCode = -EINVAL;

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetModuleIdByName: sending response\n") );

   return GWMClientCommWritePacket( pInfo, &commHdr, &id );
}

BOOL
ClientComm_GetProcByPID( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;

   wss_proc_t *wrp = NULL;

   WSS_MUTEX_ENTER( wss_proc_list );

   if( data && ( data->handle != 0 ) )
   {
      for( wrp = wss_proc_list.lh_first;
           wrp && ( wrp->wrp_pid != data->handle );
           wrp = wrp->wrp_p_list.le_next )
      ;

      if( !wrp )
      {
         WSS_MUTEX_EXIT( wss_proc_list );

         commHdr.errorCode = -ENOENT;
         if( g_bDebugging )
            _tprintf( _TEXT("ClientComm_GetProcByPID: sending response [ ENOENT ]\n") );

         return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
      }
   }

   BufferBuilder bb(7);

   for( wrp = wss_proc_list.lh_first; wrp; wrp = wrp->wrp_p_list.le_next )
   {
      if( data || ( data->handle == 0 ) || ( wrp->wrp_pid == data->handle ) )
      {
         wss_wrapper_instance_t *wi;
         int size = 0;

         WSS_MUTEX_ENTER( wrp->wrp_wi_list );
         for( wi = (wss_wrapper_instance_t *)(wrp->wrp_wi_list);
              wi;
              wi = wi->wi_next )
            if( wi->wi_wrapper )
               size += _tcslen( (LPTSTR)wi->wi_wrapper->w_module->wrm_linked->linker_file_name ) +
                       sizeof(TCHAR);

         bb.Add( (int)wrp->wrp_pid );
         bb.Add( (LPTSTR)wrp->wrp_progname );
         bb.Add( 0 );                        // TBD: uid
         bb.Add( 0L );                       // TBD: loadtime
         bb.Add( (LPTSTR)wrp->wrp_path );
         WSI_RW_ENTER( wrp->wrp_cwd, RW_READER );
         bb.Add( (LPTSTR)wrp->wrp_cwd );
         WSI_RW_EXIT( wrp->wrp_cwd );

         if( size )
         {
            LPTSTR buf;
            buf = (LPTSTR)malloc( size );
            memset( buf, 0, size );

            for( wi = (wss_wrapper_instance_t *)(wrp->wrp_wi_list);
                 wi;
                 wi = wi->wi_next )
               if( wi->wi_wrapper )
               {
                  if( *buf )
                     _tcscat( buf, _TEXT(",") );
                  _tcscat( buf, (LPTSTR)wi->wi_wrapper->w_module->wrm_linked->linker_file_name );
               }

            bb.Add( buf );
            free( buf );
         }
         else
            bb.Add( PD_MAKESTR("") );
         WSS_MUTEX_EXIT( wrp->wrp_wi_list );

         bb.EndRow();
      }
   }

   WSS_MUTEX_EXIT( wss_proc_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen = sizeof(commHdr) + dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetProcByPID: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_GetActiveWrappers( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = pInfo->commHdr.flags & GWM_COMM_UNICODE;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wss_wrapper_t *w;

   BufferBuilder bb(2);

   WSS_MUTEX_ENTER( wss_wrapper_list );
   for( w = wss_wrapper_list; w; w = w->w_next )
   {
      if( w->w_procs )
      {
         bb.Add( (LPTSTR)w->w_module->wrm_linked->linker_file_name );
         bb.Add( w->w_procs );
         bb.EndRow();
      }
   }
   WSS_MUTEX_EXIT( wss_wrapper_list );

   int dataLen;
   PVOID outdata = bb.Construct( &dataLen,
                                 (commHdr.flags & GWM_COMM_UNICODE) ? TRUE
                                                                    : FALSE );
   commHdr.pktlen = sizeof(commHdr) + dataLen;
   commHdr.errorCode = bb.GetNumRows();

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetProcByPID: sending response\n") );

   BOOL rval = GWMClientCommWritePacket( pInfo, &commHdr, outdata );
   free( outdata );
   return rval;
}

BOOL
ClientComm_SetTableRowField( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int datalen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);
   int wanted = sizeof(wrv_trnd_t);
   if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
      wanted -= ( sizeof(wrv_trnd_t) - (int) &((wrv_trnd_t *)0)->name ) /
                sizeof(TCHAR);

   if( datalen <= wanted )
   {
      commHdr.errorCode = -EINVAL;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   wrv_trnd_t *raw  = (wrv_trnd_t *)pInfo->pBuf;
   wrv_trnd_t  data;
   pd_char_t *setdata = (pd_char_t *)(raw + 1);

   memcpy( &data, pInfo->pBuf, sizeof(data) );
   if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
   {
      // Convert ASCII string to Unicode
      MultiByteToWideChar( CP_ACP, 0, (LPCSTR)raw->name, MAXNAMELEN,
                           (pd_char_t *)data.name,
                           MAXNAMELEN * sizeof(pd_char_t) );

      // Convert ASCII string to Unicode
      int numChars = datalen - wanted;
      char *olddata = (char *)( (pd_byte_t *)raw + wanted );
      setdata = (pd_char_t *)wss_malloc( numChars * sizeof(pd_char_t) );
      MultiByteToWideChar( CP_ACP, 0, (LPCSTR)olddata, numChars, setdata,
                           numChars * 2 );
   }

   wqtab_t tab = wss_db_find_table_by_handle( data.thandle );
   if( tab == NULL )
   {
      if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
         wss_free( setdata );

      commHdr.errorCode = -ENOENT;
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   wqcd_t cd = (wqcd_t)( (tab->col_defs)?DB_REL2ABS(tab->col_defs):NULL );
   if( cd == NULL )
   {
      if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
         wss_free( setdata );

      commHdr.errorCode = -ENOENT;
      wss_db_unlock_table( tab );
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   for( wqrow_t row = (wqrow_t)( (tab->rows)?DB_REL2ABS(tab->rows):NULL );
        row && ( (DWORD)DB_ABS2REL( row ) != data.rhandle );
        row = (wqrow_t)( (row->next)?DB_REL2ABS(row->next):NULL ) )
   ;

   if( row == NULL )
   {
      if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
         wss_free( setdata );

      commHdr.errorCode = -ENOENT;
      wss_db_unlock_table( tab );
      return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
   }

   for( int i = 0;
        ( i < tab->col_count ) &&
           wsi_strcmp( (pd_char_t *)DB_REL2ABS(cd[i].name), data.name );
        i++ )
   ;

   if( i < tab->col_count )
   {
      // tell WSS to set the value based on the type
      wss_db_set_row_value_from_string( cd, row, i, setdata );
   }

   if( !(pInfo->commHdr.flags & GWM_COMM_UNICODE) )
      wss_free( setdata );

   wss_db_unlock_table( tab );

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_AddNullTableRow( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int datalen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

   if( datalen >= sizeof(wrv_h_t) )
   {
      wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;

      int error = wss_db_insert_nulls( data->handle );
      switch( error )
      {
         case -WRE_NOTFOUND:
            commHdr.errorCode = -ENOENT;
            break;
         default:
            break;

      }
   }
   else
      commHdr.errorCode = -EINVAL;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_AddTableRow( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int datalen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

   if( datalen >= sizeof(wrv_h_t) )
   {
      wrv_h_t *data = (wrv_h_t *)pInfo->pBuf;

//      int error = wss_db_insert_nulls( data->handle );
//      switch( error )
//      {
//         case -WRE_NOTFOUND:
//            commHdr.errorCode = -ENOENT;
//            break;
//         default:
//            break;
//
//      }
      commHdr.errorCode = -ENOSYS;
   }
   else
      commHdr.errorCode = -EINVAL;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_DelTableRow( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   int datalen = pInfo->commHdr.pktlen - sizeof(PIPESRV_COMM_HEADER);

   if( datalen >= sizeof(wrv_tr_t) )
   {
      wrv_tr_t *data = (wrv_tr_t *)pInfo->pBuf;

      commHdr.errorCode = wss_db_delete_by_handle( data->thandle,
                                                   data->rhandle );
   }
   else
      commHdr.errorCode = -EINVAL;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_GetVersion( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   int                  data[2]  = { GWM_CLIENT_VERSION_MAJOR,
                                     GWM_CLIENT_VERSION_MINOR };

   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr) + sizeof(int) * 2;
   commHdr.errorCode = 0;

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_GetVersion: sending response [ %d, %d ]\n"),
                data[0], data[1] );

   return GWMClientCommWritePacket( pInfo, &commHdr, (PVOID)data );
}

BOOL
ClientComm_StatLinkers( LPPIPESRV_PIPE_INFO pInfo )
{
   if( g_bDebugging )
   {
      wss_linker_t *linker;
      int           num;

      WSS_MUTEX_ENTER( wss_linker_list );
      for( linker = wss_linker_list, num = 0;
           linker;
           linker = linker->linker_next, num++ )
         continue;

      _tprintf( _TEXT("[ %d Linker%s Registered ]\n"), num, 
                (num==1)?_TEXT(""):_TEXT("s") );
      for( linker = wss_linker_list;
           linker;
           linker = linker->linker_next )
         _tprintf( _TEXT("%s @ 0x%x\n"), linker->linker_description,
                   (u_int32_t)linker );
      WSS_MUTEX_EXIT( wss_linker_list );
   }

   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = -ENOSYS;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_StatRefs( LPPIPESRV_PIPE_INFO pInfo )
{
   if( g_bDebugging )
   {
      int id = *(int *)pInfo->pBuf;
   
      if( id & ( WRCTL_STAT_REFS_PROCESSES |
                 WRCTL_STAT_REFS_WRAPPER_INSTANCES ) )
      {
         wss_proc_t *wrp;
   
         WSS_MUTEX_ENTER( wss_proc_list );
         _tprintf( _TEXT("\n[ Process References ]\n") );
         for( wrp = wss_proc_list.lh_first;
              wrp;
              wrp = wrp->wrp_p_list.le_next )
         {
            _tprintf( _TEXT("%-50s = %d reference%s\n"), wrp->wrp_progname,
                      WSS_REF_COUNT(wrp->wrp_refcount),
                      (WSS_REF_COUNT(wrp->wrp_refcount)==1)?"":"s" );
#ifdef DEBUG_REFS
            if( id & WRCTL_STAT_REFS_PROCESSES_DUMP_CHAIN )
            {
               wss_ref_t *ref;
   
               for( ref = wrp->wrp_refcount.ref_next;
                    ref;
                    ref = ref->ref_next )
                  _tprintf( _TEXT("   %s:%d\n"), ref->un.st.ref_file,
                            ref->un.st.ref_line );
            }
#endif /* DEBUG_REFS */
   
            if( id & WRCTL_STAT_REFS_WRAPPER_INSTANCES )
            {
               wss_wrapper_instance_t *wi;
   
               if( wrp->wrp_wi_list )
               {
                  _tprintf( _TEXT("   === Wrapper Instances ===\n") );
                  for( wi = (wss_wrapper_instance_t *)wrp->wrp_wi_list;
                       wi;
                       wi = wi->wi_next )
                  {
                     _tprintf( _TEXT("   %-47s = %d reference%s\n"),
                              wi->wi_wrapper->w_module->wrm_linked->linker_file_name,
                              WSS_REF_COUNT(wi->wi_refcount),
                              (WSS_REF_COUNT(wi->wi_refcount)==1)?"":"s" );
#ifdef DEBUG_REFS
                     if( id & WRCTL_STAT_REFS_WRAPPER_INSTANCES_DUMP_CHAIN )
                     {
                        wss_ref_t *ref;
   
                        for( ref = wi->wi_refcount.ref_next;
                             ref;
                             ref = ref->ref_next )
                           _tprintf( _TEXT("   %s:%d\n"),
                                     ref->un.st.ref_file, ref->un.st.ref_line );
                     }
#endif /* DEBUG_REFS */
                  }
               }
            }
         }
         WSS_MUTEX_EXIT( wss_proc_list );
      }
   
      if( id & WRCTL_STAT_REFS_MODULES )
      {
         wss_mod_t *wrm;
   
         WSS_MUTEX_ENTER( wss_module_list );
         _tprintf( _TEXT("\n[ Module References ]\n") );
         for( wrm = wss_module_list; wrm; wrm = wrm->wrm_next )
         {
            _tprintf( _TEXT("%-50s = %d reference%s\n"),
                      wrm->wrm_linked->linker_file_name,
                      WSS_REF_COUNT(wrm->wrm_refcount),
                      (WSS_REF_COUNT(wrm->wrm_refcount)==1)?"":"s" );
#ifdef DEBUG_REFS
            if( id & WRCTL_STAT_REFS_MODULES_DUMP_CHAIN )
            {
               wss_ref_t *ref;
   
               for( ref = wrm->wrm_refcount.ref_next;
                    ref;
                    ref = ref->ref_next )
                  _tprintf( _TEXT("   %s:%d\n"), ref->un.st.ref_file,
                            ref->un.st.ref_line );
            }
#endif /* DEBUG_REFS */
         }
         WSS_MUTEX_EXIT( wss_module_list );
      }
   
      if( id & WRCTL_STAT_REFS_WRAPPERS )
      {
         wss_wrapper_t *w;
   
         WSS_MUTEX_ENTER( wss_wrapper_list );
         _tprintf( _TEXT("\n[ Wrapper References ]\n") );
         for( w = wss_wrapper_list; w; w = w->w_next )
         {
            _tprintf( _TEXT("%-50s = %d reference%s\n"),
                      w->w_module->wrm_linked->linker_file_name,
                      WSS_REF_COUNT(w->w_refcount),
                      (WSS_REF_COUNT(w->w_refcount)==1)?"":"s" );
#ifdef DEBUG_REFS
            if( id & WRCTL_STAT_REFS_WRAPPERS_DUMP_CHAIN )
            {
               wss_ref_t *ref;
   
               for( ref = w->w_refcount.ref_next; ref; ref = ref->ref_next )
                  _tprintf( _TEXT("   %s:%d\n"), ref->un.st.ref_file,
                            ref->un.st.ref_line );
            }
#endif /* DEBUG_REFS */
         }
         WSS_MUTEX_EXIT( wss_wrapper_list );
      }
   
      if( id & WRCTL_STAT_REFS_LINKER_FILES )
      {
         wss_linker_file_t *lf;
   
         WSS_MUTEX_ENTER( wss_linker_file_list );
         _tprintf( _TEXT("\n[ Linker File References ]\n") );
         for( lf = &wss_linker_file_list; lf; lf = lf->linker_file_next )
         {
            _tprintf( _TEXT("%-50s = %d reference%s\n"),
                      lf->linker_file_name,
                      WSS_REF_COUNT(lf->linker_file_refcount),
                      (WSS_REF_COUNT(lf->linker_file_refcount)==1)?"":"s" );
         }
         WSS_MUTEX_EXIT( wss_linker_file_list );
      }
   }

   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = -ENOSYS;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_StatMemory( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = -ENOSYS;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}

BOOL
ClientComm_AdminDone( LPPIPESRV_PIPE_INFO pInfo )
{
   PIPESRV_COMM_RESP_HEADER commHdr;
   commHdr.commandId = pInfo->commHdr.commandId;
   commHdr.pid = pInfo->commHdr.pid;
   commHdr.flags = 0;
   commHdr.pktlen = sizeof(commHdr);
   commHdr.errorCode = 0;

   wss_crit_admin_done = TRUE;

   return GWMClientCommWritePacket( pInfo, &commHdr, NULL );
}


// resync by sending last-cached-response
BOOL
ClientComm_SyncResend( LPPIPESRV_PIPE_INFO pInfo )
{
   LPPIPESRV_COMM_RESP_HEADER pCommHdr = &(pInfo->commHdr);

   if( g_bDebugging )
      _tprintf( _TEXT("ClientComm_SyncResend: !!! RESYNC !!!\n") );

   pCommHdr->flags |= GWM_COMM_RESENT;

   WSS_MUTEX_ENTER( pInfo->pLastPackets );
   PVOID pData = NULL;
   PIPESRV_LAST_PACKETS::iterator it = pInfo->pLastPackets->find( pCommHdr->tid );
   if( it == pInfo->pLastPackets->end() )
   {
      pCommHdr->commandId = *(DWORD *)pInfo->pBuf;
      pCommHdr->errorCode = -ENOENT;
   }
   else
   {
      LPPIPESRV_LAST_PACKET_DATA pPacket = (*it).second;

      memcpy( pCommHdr, pPacket->pCommHdr, sizeof(*pCommHdr) );
      if( pPacket->pData )
         pData = pPacket->pData;
   }
   WSS_MUTEX_EXIT( pInfo->pLastPackets );

   return GWMLWMCommWritePacket( pInfo, pCommHdr, pData );
}

