/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: debugwin.cxx,v 1.21 2001/04/05 18:40:22 stevek Exp $
 */


#include <wr_conf.h>
#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_process.h>
#include <wss_mutex.h>
#include <wss_cv.h>

#include <eventfs.h>

#include "debugwin.h"
#include "layout.h"
#include "resource.h"
#include "gwm_comm.h"

static volatile DWORD nextIcon = TRAYICONS;

extern g_bConsole;
extern g_bDebugging;

BOOL CALLBACK
DebugWin::_PropWinProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_COMMAND:
      {
         DebugWin *pDebugWin = (DebugWin *)GetWindowLong( hWnd, GWL_USERDATA );

         switch( LOWORD(wParam) )
         {
            case IDC_CHKTRACE:
            case IDC_CHKCOUNT:
            case IDC_CHKTHR:
            case IDC_CHKPROC:
            case IDC_CHKGLOBAL:
            case IDC_CHKPOPUP:
            case IDC_CHKCONSOLEDBG:
               EnableWindow( GetDlgItem( hWnd, IDC_APPLY ), TRUE );
               return TRUE;

            case IDOK:
            case IDC_APPLY:
            {
               if( IsWindowEnabled( GetDlgItem( hWnd, IDC_APPLY ) ) )
               {
                  DWORD flags = 0;

                  if( IsDlgButtonChecked( hWnd, IDC_CHKTRACE ) )
                     flags |= PDNF_TRACECALLS;
                  if( IsDlgButtonChecked( hWnd, IDC_CHKCOUNT ) )
                     flags |= PDNF_COUNTCALLS;
                  if( IsDlgButtonChecked( hWnd, IDC_CHKTHR ) )
                     flags |= PDNF_NOTHRMSGS;
                  if( IsDlgButtonChecked( hWnd, IDC_CHKPROC ) )
                     flags |= PDNF_NOPROCMSGS;

                  pDebugWin->SetProcFlags( flags, PDNF_MASK );
                  pDebugWin->AlwaysUseGlobal( IsDlgButtonChecked( hWnd, IDC_CHKGLOBAL ) );
                  pDebugWin->PopupOnActivity( IsDlgButtonChecked( hWnd, IDC_CHKPOPUP ) );
                  if( g_bConsole )
                     g_bDebugging = IsDlgButtonChecked( hWnd, IDC_CHKCONSOLEDBG );

                  if( LOWORD(wParam) == IDC_APPLY )
                  {
                     EnableWindow( GetDlgItem( hWnd, IDC_APPLY ), FALSE );
                     return TRUE;
                  }
               }
            }

            case IDCANCEL:
               DestroyWindow( hWnd );
               return TRUE;
         }
      }
   }

   return FALSE;
}

BOOL CALLBACK
DebugWin::_WinProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_DRAWITEM:
      {
         LayoutManager *pLMan =
            (LayoutManager *)GetWindowLong( hWnd, GWL_USERDATA );
         DebugWin *pDebugWin = (DebugWin *)pLMan->FetchUserData();

         return pDebugWin->_IconDrawItem( (LPDRAWITEMSTRUCT)lParam );
      }

      case WM_DBG_NOTIFYICON:
      {
         switch( lParam )
         {
            case WM_RBUTTONDOWN:
               ShowWindow( hWnd, SW_SHOW );
               SetForegroundWindow( hWnd );
               break;

            default:
               break;
         }

         return TRUE;
      }

      case WM_CLOSE:
         // Hide the dialog
         ShowWindow( hWnd, SW_HIDE );
         return TRUE;

      case WM_NOTIFY:
      {
         LPNMHDR pnmh = (LPNMHDR) lParam;

         switch( (int) wParam )
         {
            case IDC_TREE:
               switch( pnmh->code )
               {
                  case TVN_SELCHANGED:
                     EnableWindow( GetDlgItem( hWnd, IDC_WATCH ), TRUE );
                     return TRUE;

                  default:
                     break;
               }
               break;
         }
         break;
      }

      case WM_COMMAND:
      {
         LayoutManager *pLMan =
            (LayoutManager *)GetWindowLong( hWnd, GWL_USERDATA );
         DebugWin *pDebugWin = (DebugWin *)pLMan->FetchUserData();

         switch( LOWORD(wParam) )
         { 
            case ID_GWM_PROPERTIES:
            {
               HWND hDialog = pDebugWin->GetPropDialog();

               if( hDialog )
               {
                  ShowWindow( hDialog, SW_SHOW );
                  SetForegroundWindow( hDialog );
               }

               return TRUE;
            }

            case ID_VIEW_PROCLIST:
            {
               HMENU hMenu = GetMenu( hWnd );

               MENUITEMINFO mii;
               mii.cbSize = sizeof(mii);
               mii.fMask = MIIM_STATE;
               GetMenuItemInfo( hMenu, ID_VIEW_PROCLIST, FALSE, &mii );

               if( !(mii.fState & MFS_CHECKED) )
               {
                  HWND hControl = GetDlgItem( hWnd, IDC_TREE );
                  ShowWindow( hControl, SW_HIDE );
                  hControl = GetDlgItem( hWnd, IDC_LIST );
                  ShowWindow( hControl, SW_SHOW );

                  HWND hButton = GetDlgItem( hWnd, IDC_WATCH ); 

                  int index = SendDlgItemMessage( hWnd, IDC_LIST, LB_GETCURSEL,
                                                  0, 0 );
                  EnableWindow( hButton, (index != LB_ERR) ? TRUE : FALSE );
                  if( index != LB_ERR )
                     SetFocus( hControl );

                  mii.fMask = MIIM_STATE;
                  mii.fState = MFS_CHECKED;
                  SetMenuItemInfo( hMenu, ID_VIEW_PROCLIST, FALSE, &mii );

                  mii.fMask = MIIM_STATE;
                  mii.fState = MFS_UNCHECKED;
                  SetMenuItemInfo( hMenu, ID_VIEW_PROCTREE, FALSE, &mii );

                  DrawMenuBar( hWnd );
               }

               return TRUE;
            }

            case ID_VIEW_PROCTREE:
            {
               HMENU hMenu = GetMenu( hWnd );

               MENUITEMINFO mii;
               mii.cbSize = sizeof(mii);
               mii.fMask = MIIM_STATE;
               GetMenuItemInfo( hMenu, ID_VIEW_PROCTREE, FALSE, &mii );

               if( !(mii.fState & MFS_CHECKED) )
               {
                  HWND hControl = GetDlgItem( hWnd, IDC_LIST );
                  ShowWindow( hControl, SW_HIDE );
                  hControl = GetDlgItem( hWnd, IDC_TREE );
                  ShowWindow( hControl, SW_SHOW );

                  HTREEITEM hTreeItem = TreeView_GetSelection( hControl );
                  if( hTreeItem )
                  {
                     TreeView_Select( hControl, hTreeItem, TVGN_CARET );
                     SetFocus( hControl );
                  }

                  HWND hButton = GetDlgItem( hWnd, IDC_WATCH ); 
                  EnableWindow( hButton, hTreeItem ? TRUE : FALSE );

                  mii.fMask = MIIM_STATE;
                  mii.fState = MFS_CHECKED;
                  SetMenuItemInfo( hMenu, ID_VIEW_PROCTREE, FALSE, &mii );

                  mii.fMask = MIIM_STATE;
                  mii.fState = MFS_UNCHECKED;
                  SetMenuItemInfo( hMenu, ID_VIEW_PROCLIST, FALSE, &mii );

                  DrawMenuBar( hWnd );
               }

               return TRUE;
            }

            case IDC_WATCH:
            {
               HWND hTree = GetDlgItem( hWnd, IDC_TREE );
               if( IsWindowVisible( hTree ) )
               {
                  HTREEITEM hTreeItem = TreeView_GetSelection( hTree );

                  TVITEM tvi;
                  tvi.hItem = hTreeItem;
                  tvi.mask = TVIF_HANDLE;
                  TreeView_GetItem( hTree, &tvi );

                  int pid = (int) tvi.lParam;

                  ProcDebugNode *pProcNode = pDebugWin->FindProc( pid );
                  if( pProcNode )
                  {
                     HWND hProcDialog = pProcNode->GetHandle();
                     if( !IsWindowVisible( hProcDialog ) )
                        ShowWindow( hProcDialog, SW_SHOW );
                  }

                  return TRUE;
               }
               else
               {
                  wParam = MAKEWPARAM( IDC_LIST, LBN_DBLCLK );
                  lParam = (LPARAM) GetDlgItem( hWnd, IDC_LIST );
               }

               // List falls through to below
            }

            case IDC_LIST:
               switch( HIWORD(wParam) )
               {
                  case LBN_SELCHANGE:
                  {
                     EnableWindow( GetDlgItem( hWnd, IDC_WATCH ), TRUE );
                     return TRUE;
                  }

                  case LBN_SELCANCEL:
                  {
                     EnableWindow( GetDlgItem( hWnd, IDC_WATCH ), FALSE );
                     return TRUE;
                  }

                  case LBN_DBLCLK:
                  {
                     HWND hList = (HWND) lParam;
                     int id = GetDlgCtrlID( hList );

                     int index = SendDlgItemMessage( hWnd, id, LB_GETCURSEL,
                                                     0, 0 );
                     int len = SendDlgItemMessage( hWnd, id, LB_GETTEXTLEN,
                                                   (WPARAM) index, 0 );
                     pd_char_t *pBuf =
                        (pd_char_t *)wss_malloc( ( len + 1 ) *
                                                  sizeof(pd_char_t) );
                     SendDlgItemMessage( hWnd, id, LB_GETTEXT, (WPARAM) index,
                                         (LPARAM) pBuf );

                     for( pd_char_t *start = pBuf;
                          *start && (*start == PD_MAKESTR(' '));
                          start++ )
                     ;

                     for( pd_char_t *end = start;
                          *end && (*end != PD_MAKESTR(' '));
                          end++ )
                     ;
                     *end = PD_MAKESTR('\0');

                     int pid = _ttoi( start );
                     wss_free( pBuf );

                     ProcDebugNode *pProcNode = pDebugWin->FindProc( pid );
                     if( pProcNode )
                     {
                        HWND hProcDialog = pProcNode->GetHandle();
                        if( !IsWindowVisible( hProcDialog ) )
                           ShowWindow( hProcDialog, SW_SHOW );
                     }

                     return TRUE;
                  }

                  default:
                     break;

               }
               break;

            default:
               break;
         }
         break;
      }

      default:
         break;
   }

   return FALSE;
}

void
DebugWin::_WinMsgThread( LPVOID lpParam )
{
   DebugWin *pDWin = (DebugWin *)lpParam;
   HWND      hWnd;
   MSG       msg;

   pDWin->LockHandle();
   hWnd = CreateDialog( NULL, MAKEINTRESOURCE(IDD_MAINDIALOG),
                        NULL, _WinProc );
   if( hWnd )
   {
      SetWindowLong( hWnd, GWL_USERDATA, (LONG) pDWin );
      pDWin->hWnd = hWnd;
      pDWin->layoutManager =
         new LayoutManager( hWnd, MAKEINTRESOURCE(IDD_MAINDIALOG) );
   }

   pDWin->bTried = TRUE;
   pDWin->UnlockHandle();
   WSS_CV_BROADCAST( pDWin->hWnd );

   if( !hWnd )
   {
      wsi_printf( PD_MAKESTR("Unable to create window.\n") );
      _endthread();
   }

   while( GetMessage( &msg, (HWND)NULL, 0, 0 ) )
   {
      if( !IsWindow( hWnd ) || !IsDialogMessage( hWnd, &msg ) )
      {
         TranslateMessage( &msg );
         DispatchMessage( &msg );
      }
   }

   _endthread();
}

BOOL
DebugWin::_TrayMessage( DWORD dwMessage, UINT id, HICON hIcon, LPCTSTR pszTip )
{
   NOTIFYICONDATA tnd;
   tnd.cbSize           = sizeof(tnd);
   tnd.hWnd             = hWnd;
   tnd.uID              = id;
   tnd.uFlags           = NIF_MESSAGE | NIF_ICON | NIF_TIP;
   tnd.uCallbackMessage = WM_DBG_NOTIFYICON;
   tnd.hIcon            = hIcon;

   if( pszTip )
      wsi_strncpy( tnd.szTip, pszTip, sizeof(tnd.szTip) / sizeof(pd_char_t) );
   else
      tnd.szTip[0] = PD_MAKESTR('\0');

   BOOL bRet = Shell_NotifyIcon( dwMessage, &tnd );

   if( hIcon )
      DestroyIcon( hIcon );

   return bRet;
}

LRESULT
DebugWin::_IconDrawItem( LPDRAWITEMSTRUCT lpdi )
{
   HICON hIcon = (HICON)LoadImage( GetModuleHandle( NULL ),
                                   MAKEINTRESOURCE(lpdi->CtlID),
                                   IMAGE_ICON, 16, 16, 0);
   if( !hIcon ) return FALSE;

   DrawIconEx( lpdi->hDC, lpdi->rcItem.left, lpdi->rcItem.top, hIcon,
               16, 16, 0, NULL, DI_NORMAL);
   return TRUE;
}

HWND
DebugWin::GetPropDialog()
{
   LockPropDialog();
   if( ( hPropWnd == NULL ) || !IsWindow( hPropWnd ) )
   {
      hPropWnd = CreateDialog( NULL, MAKEINTRESOURCE(IDD_PROPDIALOG),
                               NULL, _PropWinProc );

      if( hPropWnd )
      {
         SetWindowLong( hPropWnd, GWL_USERDATA, (LONG) this );

         DWORD flags = GetProcFlags();

         if( flags & PDNF_TRACECALLS )
            CheckDlgButton( hPropWnd, IDC_CHKTRACE, BST_CHECKED );
         if( flags & PDNF_COUNTCALLS )
            CheckDlgButton( hPropWnd, IDC_CHKCOUNT, BST_CHECKED );
         if( flags & PDNF_NOTHRMSGS )
            CheckDlgButton( hPropWnd, IDC_CHKTHR, BST_CHECKED );
         if( flags & PDNF_NOPROCMSGS )
            CheckDlgButton( hPropWnd, IDC_CHKPROC, BST_CHECKED );

         if( bAlwaysGlobal )
            CheckDlgButton( hPropWnd, IDC_CHKGLOBAL, BST_CHECKED );
         if( bPopupOnActivity )
            CheckDlgButton( hPropWnd, IDC_CHKPOPUP, BST_CHECKED );
         if( g_bConsole )
         {
            HWND hControl = GetDlgItem( hPropWnd, IDC_CHKCONSOLEDBG );
            EnableWindow( hControl, TRUE );

            if( g_bDebugging )
               CheckDlgButton( hPropWnd, IDC_CHKCONSOLEDBG, BST_CHECKED );
         }
      }
   }

   HWND hDialog = hPropWnd;
   UnlockPropDialog();

   return hDialog;
}

void
DebugWin::SetProcFlags( DWORD dwFlags, DWORD dwMask )
{
   DWORD dwOldFlags = dwProcFlags;

   dwProcFlags &= ~dwMask;
   dwProcFlags |= (dwFlags & dwMask);

   if( dwOldFlags != dwProcFlags )
   {
      HKEY  hKey;
      if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, HKEY_WRAPPERS PD_MAKESTR("\\GWM"),
                        0, KEY_SET_VALUE, &hKey ) == ERROR_SUCCESS )
      {
         RegSetValueEx( hKey, PD_MAKESTR("ProcessFlags"), NULL, REG_DWORD,
                        (CONST BYTE *)&dwProcFlags, sizeof(dwProcFlags) );
         RegCloseKey( hKey );
      }
   }
}

void
DebugWin::AlwaysUseGlobal( BOOL flag )
{
   if( flag != bAlwaysGlobal )
   {
      bAlwaysGlobal = flag;

      HKEY  hKey;
      if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, HKEY_WRAPPERS PD_MAKESTR("\\GWM"),
                        0, KEY_SET_VALUE, &hKey ) == ERROR_SUCCESS )
      {
         DWORD dwVal = (DWORD)( flag ? 1 : 0 );
         RegSetValueEx( hKey, PD_MAKESTR("AlwaysUseGlobal"), NULL, REG_DWORD,
                        (CONST BYTE *)&dwVal, sizeof(dwVal) );
         RegCloseKey( hKey );
      }
   }
}

void
DebugWin::PopupOnActivity( BOOL flag )
{
   if( flag != bPopupOnActivity )
   {
      bPopupOnActivity = flag;

      HKEY  hKey;
      if( RegOpenKeyEx( HKEY_LOCAL_MACHINE, HKEY_WRAPPERS PD_MAKESTR("\\GWM"),
                        0, KEY_SET_VALUE, &hKey ) == ERROR_SUCCESS )
      {
         DWORD dwVal = (DWORD)( flag ? 1 : 0 );
         RegSetValueEx( hKey, PD_MAKESTR("PopupOnActivity"), NULL, REG_DWORD,
                        (CONST BYTE *)&dwVal, sizeof(dwVal) );
         RegCloseKey( hKey );
      }
   }
}

void
DebugWin::Flush( int pid )
{
   LockHandle();
   if( ( hWnd == NULL ) || !bTried )
      WSS_CV_WAIT( hWnd );

   if( !hWnd )
   {
      UnlockHandle();
      return;
   }
   UnlockHandle();

   ProcDebugNode *pProcNode = FindProc( pid );
   if( !pProcNode ) return;

   pProcNode->Flush();
}

ProcDebugNode *
DebugWin::FindProc( int pid )
{
   ProcDebugNode *pNode = NULL;

   LockMap();
   PROCDEBUGMAP::iterator it = debugMap.find( pid );
   if( it != debugMap.end() )
      pNode = (*it).second->pProcNode;
   UnlockMap();

   return pNode;
}

ProcDebugNode *
DebugWin::NewProcess( int pid )
{
   ProcDebugNode *pProcNode = new ProcDebugNode( dwTabStop, pid );
   LPPROCMAPNODE  pMapNode  = new PROCMAPNODE;
   pMapNode->pProcNode = pProcNode;

   LockMap();
   debugMap.insert( PROCDEBUGMAP::value_type( pid, pMapNode ) );
   UnlockMap();

   pd_char_t procTitle[MAXPATHLEN];
   wss_proc_t *wrp = wss_pfind( pid );
   if( wrp && wrp->wrp_progname )
      wsi_snprintf( procTitle, MAXPATHLEN, PD_MAKESTR("%-7d  %s"),
                    pid, wrp->wrp_progname );
   else
      wsi_snprintf( procTitle, MAXPATHLEN, PD_MAKESTR("%-7d"), pid );

   int ppid = wrp->wrp_p->p_ppid;

   if( wrp )
      wss_proc_rele( wrp );

   LockMap();
   HTREEITEM hTreeItem = NULL;
   PROCDEBUGMAP::iterator it = debugMap.find( ppid );
   if( it != debugMap.end() )
   {
      LPPROCMAPNODE pParentMapNode = (*it).second;
      hTreeItem = pParentMapNode->hTreeItem;

      DWORD dwFlags = pParentMapNode->pProcNode->GetFlags();
      if( bAlwaysGlobal )
         pProcNode->SetFlags( dwProcFlags, dwProcFlags );
      else
         pProcNode->SetFlags( dwFlags, dwFlags );
   }
   else
      pProcNode->SetFlags( dwProcFlags, dwProcFlags );
   UnlockMap();

   LockHandle();
   HWND hTree = GetDlgItem( hWnd, IDC_TREE );

   TVINSERTSTRUCT tvis;
   tvis.hParent = hTreeItem;
   tvis.hInsertAfter = TVI_FIRST;
   tvis.item.mask = TVIF_TEXT | TVIF_PARAM;
   tvis.item.lParam = (LPARAM) pid;
   tvis.item.pszText = procTitle;

   pMapNode->hTreeItem = TreeView_InsertItem( hTree, &tvis );

   if( hTreeItem )
      TreeView_Expand( hTree, hTreeItem, TVE_EXPAND );

   SendDlgItemMessage( hWnd, IDC_LIST, LB_ADDSTRING, 0, (LPARAM) procTitle );

   UnlockHandle();

   PostMessage( pid, 0, PD_MAKESTR("Initialized debugging information.\n") );

   return pProcNode;
}

BOOL
DebugWin::_PostMessageInternal( int pid, DWORD tid, const pd_char_t *msg,
                                DWORD len )
{
   LockHandle();
   if( ( hWnd == NULL ) && !bTried )
      dwTID = _beginthread( _WinMsgThread, 0, this );

   if( ( hWnd == NULL ) || !bTried )
      WSS_CV_WAIT( hWnd );

   if( !hWnd )
   {
      UnlockHandle();
      return FALSE;
   }
   UnlockHandle();

   ProcDebugNode *pProcNode = FindProc( pid );
   if( !pProcNode )
      pProcNode = NewProcess( pid );

   LockHandle();
   if( bPopupOnActivity && !IsWindowVisible( hWnd ) )
      ShowWindow( hWnd, SW_SHOW );
   UnlockHandle();

   pProcNode->PostMessage( tid, msg, len );

   return TRUE;
}


BOOL
DebugWin::PostMessage( int pid, DWORD tid, const pd_char_t *msg )
{
   pd_char_t *buf, *ptr, *cptr;
   
   if( msg == NULL )
      return _PostMessageInternal( pid, tid, NULL, 0 );

   buf = ptr = wss_strdup( msg );
   
   BOOL bRet = TRUE;
   while( bRet && ( cptr = _tcschr( ptr, PD_MAKESTR('\n') ) ) )
   {
      DWORD len = ( (DWORD)cptr - (DWORD)ptr ) / sizeof(pd_char_t) + 1;

      bRet = _PostMessageInternal( pid, tid, ptr, len );

      ptr = cptr + 1;
   }

   if( bRet && *ptr )
      bRet = _PostMessageInternal( pid, tid, ptr, wsi_strlen(ptr) );

   wss_free( buf );

   return bRet;
}

BOOL
DebugWin::TrayIcon( HICON hIcon, const pd_char_t *pszTip )
{
   BOOL bRet = FALSE;

   if( iconId == -1 )
   {
      LockHandle();
      if( ( hWnd == NULL ) && !bTried )
         dwTID = _beginthread( _WinMsgThread, 0, this );

      if( ( hWnd == NULL ) || !bTried )
         WSS_CV_WAIT( hWnd );

      if( !hWnd )
      {
         UnlockHandle();
         return FALSE;
      }
      UnlockHandle();

      iconId = nextIcon++;
      bRet = _TrayMessage( NIM_ADD, iconId, NULL, NULL );
   }

   if( hIcon == NULL )
      bRet = _TrayMessage( NIM_DELETE, iconId, hIcon, pszTip );
   else
      bRet = _TrayMessage( NIM_MODIFY, iconId, hIcon, pszTip );

   return bRet;
}

DebugWin::DebugWin( DWORD tabStop )
{
   hWnd = NULL;
   WSS_MUTEX_INIT( hWnd );
   WSS_CV_INIT( hWnd );
   WSS_MUTEX_INIT( debugMap );
   WSS_MUTEX_INIT( hPropWnd );
   bTried = FALSE;
   bAlwaysGlobal = FALSE;
   bPopupOnActivity = FALSE;
   layoutManager = NULL;
   iconId = -1;
   dwTabStop = tabStop;

   HKEY  hKey;
   DWORD dwDisposition;
   if( RegCreateKeyEx( HKEY_LOCAL_MACHINE, HKEY_WRAPPERS PD_MAKESTR("\\GWM"),
                       0, PD_MAKESTR(""), REG_OPTION_NON_VOLATILE,
                       KEY_ALL_ACCESS, NULL, &hKey,
                       &dwDisposition ) == ERROR_SUCCESS )
   {
      DWORD type;
      DWORD flags;
      DWORD size = sizeof(flags);
      if( RegQueryValueEx( hKey, PD_MAKESTR("ProcessFlags"), NULL, &type,
                           (LPBYTE)&flags, &size ) == ERROR_SUCCESS )
      {
         dwProcFlags = flags;
      }

      if( RegQueryValueEx( hKey, PD_MAKESTR("AlwaysUseGlobal"), NULL, &type,
                           (LPBYTE)&flags, &size ) == ERROR_SUCCESS )
      {
         if( flags )
            bAlwaysGlobal = TRUE;
      }

      if( RegQueryValueEx( hKey, PD_MAKESTR("PopupOnActivity"), NULL, &type,
                           (LPBYTE)&flags, &size ) == ERROR_SUCCESS )
      {
         if( flags )
            bPopupOnActivity = TRUE;
      }

      RegCloseKey( hKey );
   }
}

DebugWin::~DebugWin()
{
   if( iconId != -1 )
      _TrayMessage( NIM_DELETE, iconId, NULL, NULL );

   WSS_CV_DESTROY( hWnd );
   WSS_MUTEX_DESTROY( hWnd );
   if( hWnd )
      DestroyWindow( hWnd );
   if( hPropWnd )
      DestroyWindow( hPropWnd );
   if( layoutManager )
      delete layoutManager;
}


//
// ProcDebugNode methods
//
ProcDebugNode::ProcDebugNode( DWORD tabStop, int id ) : DebugNode( tabStop )
{
   WSS_CV_INIT(hWnd);
   WSS_MUTEX_INIT(map);
   layoutManager = NULL;
   bTried = FALSE;
   pid = id;
   flags = 0;
}

ProcDebugNode::~ProcDebugNode()
{
   WSS_MUTEX_DESTROY(map);
   WSS_CV_DESTROY(hWnd);

   if( hWnd )
      DestroyWindow( hWnd );

   if( layoutManager )
      delete layoutManager;
}

void
ProcDebugNode::SetFlags( DWORD dwFlags, DWORD dwMask )
{
   DWORD oflags = flags;
   flags &= ~dwMask;
   flags |= (dwFlags & dwMask);

   if( dwMask & PDNF_COUNTCALLS )
   {
      if( ( oflags & PDNF_COUNTCALLS ) != ( flags & PDNF_COUNTCALLS ) )
         GWMLWMCommSendCountCallsEvent( pid,
                                        ( dwFlags & PDNF_COUNTCALLS )
                                        ? TRUE : FALSE );
      LockHandle();
      if( hWnd )
         CheckDlgButton( hWnd, IDC_CHKCOUNT,
                         ( dwFlags & PDNF_COUNTCALLS )
                         ? BST_CHECKED : BST_UNCHECKED );
      UnlockHandle();
   }
   if( dwMask & PDNF_TRACECALLS )
   {
      if( ( oflags & PDNF_TRACECALLS ) != ( flags & PDNF_TRACECALLS ) )
         GWMLWMCommSendTraceCallsEvent( pid,
                                        ( dwFlags & PDNF_TRACECALLS )
                                        ? TRUE : FALSE );
      LockHandle();
      if( hWnd )
         CheckDlgButton( hWnd, IDC_CHKTRACE,
                         ( dwFlags & PDNF_TRACECALLS )
                         ? BST_CHECKED : BST_UNCHECKED );
      UnlockHandle();
   }
   if( dwMask & PDNF_NOTHRMSGS )
   {
      if( ( oflags & PDNF_NOTHRMSGS ) != ( flags & PDNF_NOTHRMSGS ) )
         GWMLWMCommSendThreadMessagesEvent( pid,
                                            ( dwFlags & PDNF_NOTHRMSGS )
                                            ? TRUE : FALSE );
      LockHandle();
      if( hWnd )
         CheckDlgButton( hWnd, IDC_CHKTHR,
                         ( dwFlags & PDNF_NOTHRMSGS )
                         ? BST_CHECKED : BST_UNCHECKED );
      UnlockHandle();
   }
   if( dwMask & PDNF_NOPROCMSGS )
   {
      if( ( oflags & PDNF_NOPROCMSGS ) != ( flags & PDNF_NOPROCMSGS ) )
         GWMLWMCommSendProcessMessagesEvent( pid,
                                             ( dwFlags & PDNF_NOPROCMSGS )
                                             ? TRUE : FALSE );
      LockHandle();
      if( hWnd )
         CheckDlgButton( hWnd, IDC_CHKPROC,
                         ( dwFlags & PDNF_NOPROCMSGS )
                         ? BST_CHECKED : BST_UNCHECKED );
      UnlockHandle();
   }
}

BOOL CALLBACK
ProcDebugNode::_WinProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   switch( msg )
   {
      case WM_COMMAND:
      {
         LayoutManager *pLMan =
            (LayoutManager *)GetWindowLong( hWnd, GWL_USERDATA );
         ProcDebugNode *pNode = (ProcDebugNode *)pLMan->FetchUserData();
         int pid = pNode->GetPID();

         switch( LOWORD(wParam) )
         { 
            case IDOK:
               // Hide the dialog
               ShowWindow( hWnd, SW_HIDE );
               return TRUE;

            case IDC_CHKTRACE:
               pNode->SetFlags( IsDlgButtonChecked( hWnd, IDC_CHKTRACE )
                                ? PDNF_TRACECALLS : 0, PDNF_TRACECALLS );
               return TRUE;

            case IDC_CHKCOUNT:
               pNode->SetFlags( IsDlgButtonChecked( hWnd, IDC_CHKCOUNT )
                                ? PDNF_COUNTCALLS : 0, PDNF_COUNTCALLS );
               return TRUE;

            case IDC_CHKPROC:
               pNode->SetFlags( IsDlgButtonChecked( hWnd, IDC_CHKPROC )
                                ? PDNF_NOPROCMSGS : 0, PDNF_NOPROCMSGS );
               return TRUE;

            case IDC_CHKTHR:
               pNode->SetFlags( IsDlgButtonChecked( hWnd, IDC_CHKTHR )
                                ? PDNF_NOTHRMSGS : 0, PDNF_NOTHRMSGS );
               return TRUE;

            case ID_FILE_SAVEALL:
            case ID_FILE_SAVETHREAD:
            {
               TCHAR        szFile[260];
               wss_proc_t  *wrp = wss_pfind( pid );

               if( LOWORD(wParam) == ID_FILE_SAVEALL )
               {
                  if( wrp && wrp->wrp_progname )
                     _sntprintf( szFile, 260, _TEXT("%s-pid%d-trace.TXT"),
                                 wrp->wrp_progname, pid );
                  else
                     _sntprintf( szFile, 260, _TEXT("pid%d-trace.TXT"), pid );
               }
               else
               {
                  HWND hTab = GetDlgItem( pNode->GetHandle(), IDC_TAB );

                  TCITEM tie;
                  tie.mask = TCIF_PARAM;
                  TabCtrl_GetItem( hTab, TabCtrl_GetCurSel( hTab ), &tie );

                  DWORD tid = (DWORD) tie.lParam;

                  if( wrp && wrp->wrp_progname )
                     _sntprintf( szFile, 260, _TEXT("%s-pid%d-tid%d-trace.TXT"),
                                 wrp->wrp_progname, pid, tid );
                  else
                     _sntprintf( szFile, 260, _TEXT("pid%d-tid%d-trace.TXT"),
                                 pid, tid );
               }

               OPENFILENAME ofn;
               memset( &ofn, 0, sizeof(OPENFILENAME) );
               ofn.lStructSize = sizeof(ofn);
               ofn.hwndOwner = hWnd;
               ofn.lpstrFile = szFile;
               ofn.nMaxFile = 260;
               ofn.lpstrFilter = _TEXT("Text File\0*.TXT\0All\0*.*\0");
               ofn.nFilterIndex = 1;
               ofn.lpstrFileTitle = NULL;
               ofn.nMaxFileTitle = 0;
               ofn.lpstrInitialDir = NULL;
               if( LOWORD(wParam) == ID_FILE_SAVEALL )
                  ofn.lpstrTitle = _TEXT("Save Trace Messages As");
               else
                  ofn.lpstrTitle = _TEXT("Save Selected Thread Trace Messages As");
               ofn.Flags = OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;

               if( GetOpenFileName( &ofn ) == TRUE )
               {
                  HANDLE hFile = CreateFile( ofn.lpstrFile, GENERIC_WRITE, 0,
                                             NULL, CREATE_ALWAYS,
                                             FILE_ATTRIBUTE_NORMAL, NULL );
                  if( hFile == INVALID_HANDLE_VALUE )
                  {
                     LPVOID lpMsgBuf;
                     FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                                    FORMAT_MESSAGE_FROM_SYSTEM |
                                    FORMAT_MESSAGE_IGNORE_INSERTS,
                                    NULL, GetLastError(),
                                    MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
                                    (LPTSTR) &lpMsgBuf, 0, NULL );
                     MessageBox( 0, (LPTSTR) lpMsgBuf, _TEXT("Save Error"),
                                 ( MB_OK | MB_ICONEXCLAMATION |
                                   MB_DEFAULT_DESKTOP_ONLY ) );
                     LocalFree( lpMsgBuf );
                  }
                  else
                  {
                     if( LOWORD(wParam) == ID_FILE_SAVEALL )
                        pNode->DumpMessages( hFile );
                     else
                     {
                        HWND hTab = GetDlgItem( pNode->GetHandle(), IDC_TAB );

                        TCITEM tie;
                        tie.mask = TCIF_PARAM;
                        TabCtrl_GetItem( hTab, TabCtrl_GetCurSel( hTab ),
                                         &tie );

                        DWORD tid = (DWORD) tie.lParam;
                        ThrDebugNode *pThrNode = pNode->FindThread( tid );

                        if( pThrNode )
                           pThrNode->DumpMessages( hFile );
                     }
                     CloseHandle( hFile );
                  }
               }
               break;
            }

            default:
               break;
         }
         break;
      }

      case WM_NEWTHREAD:
      {
         ProcDebugNode *pNode    = (ProcDebugNode *)lParam;
         int            tid      = LOWORD(wParam);
         ThrDebugNode  *pThrNode = new ThrDebugNode( pNode->GetTabStop(),
                                                     pNode->GetHandle(), tid );
         HWND           hTab     = GetDlgItem( pNode->GetHandle(), IDC_TAB );

         int index = pNode->AddThread( tid, pThrNode );

         WINDOWPLACEMENT wp;
         wp.length = sizeof(wp);
         GetWindowPlacement( hTab, &wp );

         LPRECT r = &wp.rcNormalPosition;
         TabCtrl_AdjustRect( hTab, FALSE, r );

         HWND hTabArea = pThrNode->GetHandle();
         SetWindowPos( hTabArea, HWND_TOP, r->left, r->top,
                       r->right - r->left, r->bottom - r->top, 0 );

         if( index == 0 )
            ShowWindow( hTabArea, SW_SHOW );

         RECT rect;
         GetClientRect( hTabArea, &rect );

         HWND hList = GetDlgItem( hTabArea, IDC_LIST );
         SetWindowPos( hList, 0, 0, 0, rect.right, rect.bottom, 0 );

         HMENU hMenu = GetMenu( hWnd );

         MENUITEMINFO mii;
         mii.cbSize = sizeof(mii);
         mii.fMask = MIIM_STATE;
         GetMenuItemInfo( hMenu, ID_FILE_SAVETHREAD, FALSE, &mii );

         if( mii.fState & MFS_GRAYED )
         {
            mii.fMask = MIIM_STATE;
            mii.fState = MFS_ENABLED;
            SetMenuItemInfo( hMenu, ID_FILE_SAVETHREAD, FALSE, &mii );

            DrawMenuBar( hWnd );
         }

         return TRUE;
      }

      case WM_NOTIFY:
      {
         switch( (int) wParam )
         {
            case IDC_TAB:
            {
               LayoutManager *pLMan =
                  (LayoutManager *)GetWindowLong( hWnd, GWL_USERDATA );
               ProcDebugNode *pNode = (ProcDebugNode *)pLMan->FetchUserData();
               HWND           hTab  = GetDlgItem( hWnd, IDC_TAB );
               LPNMHDR        pnmh  = (LPNMHDR) lParam;

               switch( pnmh->code )
               {
                  case TCN_SELCHANGING:
                  case TCN_SELCHANGE:
                  {
                     TCITEM tie;
                     tie.mask = TCIF_PARAM;
                     TabCtrl_GetItem( hTab, TabCtrl_GetCurSel( hTab ), &tie );

                     DWORD tid = (DWORD) tie.lParam;
                     ThrDebugNode *pThrNode = pNode->FindThread( tid );

                     if( pnmh->code == TCN_SELCHANGE )
                     {
                        WINDOWPLACEMENT wp;
                        wp.length = sizeof(wp);
                        GetWindowPlacement( hTab, &wp );

                        LPRECT r = &wp.rcNormalPosition;
                        TabCtrl_AdjustRect( hTab, FALSE, r );

                        HWND hTabArea = pThrNode->GetHandle();
                        SetWindowPos( hTabArea, HWND_TOP, r->left, r->top,
                                      r->right - r->left, r->bottom - r->top,
                                      0 );
                        ShowWindow( hTabArea, SW_SHOW );

                        RECT rect;
                        GetClientRect( hTabArea, &rect );

                        HWND hList = GetDlgItem( hTabArea, IDC_LIST );
                        SetWindowPos( hList, 0, 0, 0, rect.right, rect.bottom,
                                      0 );
                     }
                     else
                        ShowWindow( pThrNode->GetHandle(), SW_HIDE );

                     return TRUE;
                  }

                  default:
                     break;
               }

               break;
            }

            default:
               break;
         }

         break;
      }

      case WM_SIZE:
      {
         LayoutManager *pLMan = (LayoutManager *)GetWindowLong( hWnd,
                                                                GWL_USERDATA );
         ProcDebugNode *pNode = (ProcDebugNode *)pLMan->FetchUserData();
         HWND           hTab  = GetDlgItem( hWnd, IDC_TAB );

         TCITEM tie;
         tie.mask = TCIF_PARAM;
         TabCtrl_GetItem( hTab, TabCtrl_GetCurSel( hTab ), &tie );

         DWORD tid = (DWORD) tie.lParam;
         ThrDebugNode *pThrNode = NULL;

         if( tid )
            pThrNode = pNode->FindThread( tid );

         WINDOWPLACEMENT wp;
         wp.length = sizeof(wp);
         GetWindowPlacement( hTab, &wp );

         LPRECT r = &wp.rcNormalPosition;
         TabCtrl_AdjustRect( hTab, FALSE, r );

         if( tid )
         {
            HWND hTabArea = pThrNode->GetHandle();
            SetWindowPos( hTabArea, HWND_TOP, r->left, r->top,
                          r->right - r->left, r->bottom - r->top, 0 );
            ShowWindow( hTabArea, SW_SHOW );

            RECT rect;
            GetClientRect( hTabArea, &rect );

            HWND hList = GetDlgItem( hTabArea, IDC_LIST );
            SetWindowPos( hList, 0, 0, 0, rect.right, rect.bottom, 0 );
         }

         break;
      }

      default:
         break;
   }

   return FALSE;
}

void
ProcDebugNode::_WinMsgThread( LPVOID lpParam )
{
   ProcDebugNode *pPDNode = (ProcDebugNode *)lpParam;
   HWND           hWnd;
   MSG            msg;

   InitCommonControls();
   hWnd = CreateDialog( NULL, MAKEINTRESOURCE(IDD_PROCDIALOG),
                        NULL, _WinProc );
   if( hWnd )
   {
      pd_char_t pBuf[MAXPATHLEN];

      GetWindowText( hWnd, pBuf, MAXPATHLEN );
      _sntprintf( pBuf, MAXPATHLEN, PD_MAKESTR("%s [PID %d]"), pBuf,
                  pPDNode->GetPID() );
      SetWindowText( hWnd, pBuf );

      SetWindowLong( hWnd, GWL_USERDATA, (LONG) pPDNode );
      pPDNode->SetHandle( hWnd );
      pPDNode->layoutManager =
         new LayoutManager( hWnd, MAKEINTRESOURCE(IDD_PROCDIALOG) );

      // Force flags/checkboxes to be set based on parent
      pPDNode->SetFlags( pPDNode->GetFlags(), pPDNode->GetFlags() );

   }

   pPDNode->bTried = TRUE;
   WSS_CV_BROADCAST( pPDNode->hWnd );

   if( !hWnd )
   {
      wsi_printf( PD_MAKESTR("Unable to create window.\n") );
      _endthread();
   }

   while( GetMessage( &msg, (HWND)NULL, 0, 0 ) )
   {
      if( !IsWindow( hWnd ) || !IsDialogMessage( hWnd, &msg ) )
      {
         TranslateMessage( &msg );
         DispatchMessage( &msg );
      }
   }

   _endthread();
}

void
ProcDebugNode::DumpMessages( HANDLE hFile )
{
   char pBuf[260];

   DWORD nBytes = _snprintf( pBuf, 260, "[Process #%d]\r\n", pid );

   WriteFile( hFile, pBuf, nBytes, &nBytes, NULL );

   DebugNode::DumpMessages( hFile );

   WriteFile( hFile, "\r\n", 2, &nBytes, NULL );

   // Dump thread messages
   LockMap();
   THRDEBUGMAP::iterator it = map.begin();
   while( it != map.end() )
   {
      ThrDebugNode *pNode = (*it).second;
      pNode->DumpMessages( hFile );
      it++;
   }
   UnlockMap();
}

BOOL
ProcDebugNode::PostMessage( DWORD tid, const pd_char_t *msg, DWORD len )
{
   if( !bTried )
   {
      LockHandle();
      if( ( hWnd == NULL ) && !bTried )
         dwTID = _beginthread( _WinMsgThread, 0, this );

      if( ( hWnd == NULL ) || !bTried )
         WSS_CV_WAIT( hWnd );

      if( !hWnd )
      {
         WSS_MUTEX_EXIT( hWnd );
         return FALSE;
      }
      UnlockHandle();
   }

   if( !tid )
   {
      DebugNode::PostMessage( msg, len );
   }
   else
   {
      ThrDebugNode *pThrNode = FindThread( tid );
      if( !pThrNode )
      {
         SendMessage( hWnd, WM_NEWTHREAD, MAKEWPARAM(tid, 0), (LPARAM) this );
         pThrNode = FindThread( tid );
      }

      if( pThrNode )
         pThrNode->PostMessage( msg, len );
   }

   return TRUE;
}

ThrDebugNode *
ProcDebugNode::FindThread( DWORD tid )
{
   ThrDebugNode *pNode = NULL;

   LockMap();

   THRDEBUGMAP::iterator it = map.find( tid );
   if( it != map.end() )
      pNode = (*it).second;

   UnlockMap();

   return pNode;
}

int
ProcDebugNode::AddThread( DWORD tid, ThrDebugNode *pNode )
{
   LockMap();
   int index = map.size();
   map.insert( THRDEBUGMAP::value_type( tid, pNode ) );
   UnlockMap();

   pd_char_t pBuf[16];
   _sntprintf( pBuf, 16, PD_MAKESTR("%d"), tid );

   TCITEM tie;
   tie.mask = TCIF_TEXT | TCIF_IMAGE | TCIF_PARAM;
   tie.iImage = -1;
   tie.pszText = pBuf;
   tie.lParam = (LPARAM) tid;

   LockHandle();
   HWND hTab = GetDlgItem( hWnd, IDC_TAB );
   UnlockHandle();

   return TabCtrl_InsertItem( hTab, index, &tie );
}

void
ProcDebugNode::Flush()
{
   PostMessage( 0, NULL, 0 );
   for( THRDEBUGMAP::iterator it = map.begin(); it != map.end(); it++ )
      PostMessage( (*it).first, NULL, 0 );

   LockHandle();
   if( hWnd != NULL )
   {
      pd_char_t pBuf[MAXPATHLEN];

      GetWindowText( hWnd, pBuf, MAXPATHLEN );
      _sntprintf( pBuf, MAXPATHLEN, PD_MAKESTR("%s -- Exited"), pBuf );
      SetWindowText( hWnd, pBuf );

      HWND hControl = GetDlgItem( hWnd, IDC_CHKTHR );
      EnableWindow( hControl, FALSE );

      hControl = GetDlgItem( hWnd, IDC_CHKPROC );
      EnableWindow( hControl, FALSE );

      hControl = GetDlgItem( hWnd, IDC_CHKTRACE );
      EnableWindow( hControl, FALSE );

      hControl = GetDlgItem( hWnd, IDC_CHKCOUNT );
      EnableWindow( hControl, FALSE );
   }
   UnlockHandle();
}


//
// ThrDebugNode methods
//
ThrDebugNode::ThrDebugNode( DWORD tabStop, HWND hParent, DWORD id )
   : DebugNode( tabStop )
{
   tid = id;
   hWnd = CreateDialog( NULL, MAKEINTRESOURCE(IDD_PAGE), hParent, _WinProc );
}

ThrDebugNode::~ThrDebugNode()
{
   if( hWnd )
      DestroyWindow( hWnd );
}

void
ThrDebugNode::DumpMessages( HANDLE hFile )
{
   char pBuf[260];

   DWORD nBytes = _snprintf( pBuf, 260, "[Thread #%d]\r\n", tid );

   WriteFile( hFile, pBuf, nBytes, &nBytes, NULL );

   DebugNode::DumpMessages( hFile );

   WriteFile( hFile, "\r\n", 2, &nBytes, NULL );
}

BOOL CALLBACK
ThrDebugNode::_WinProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
   return FALSE;
}


//
// DebugNode methods
//
DebugNode::DebugNode( DWORD tabStop )
{
   WSS_MUTEX_INIT(str);
   WSS_MUTEX_INIT(hWnd);
   str = NULL;
   hWnd = NULL;

   dwTabStop = tabStop;
   pd_char_t *ts = new pd_char_t[tabStop];
   for( int i = 0; i < tabStop; i++ )
      ts[i] = PD_MAKESTR(' ');
   wsTabStop = new wss_string(ts);
   delete ts;
}

DebugNode::~DebugNode()
{
   delete wsTabStop;
   WSS_MUTEX_DESTROY(str);
   WSS_MUTEX_DESTROY(hWnd);
}

wss_string *
DebugNode::GetString( void )
{
   wss_string *oldstr;

   LockString();
   oldstr = str;
   str = NULL;
   UnlockString();

   return oldstr;
}

BOOL
DebugNode::SetString( wss_string *newstr )
{
   if( str )
      return FALSE;

   LockString();
   str = newstr;
   UnlockString();

   return TRUE;
}

HWND
DebugNode::GetHandle()
{
   HWND handle;
   LockHandle();
   handle = hWnd;
   UnlockHandle();
   return handle;
}

void
DebugNode::SetHandle( HWND handle )
{
   LockHandle();
   hWnd = handle;
   UnlockHandle();
}

void
DebugNode::DumpMessages( HANDLE hFile )
{
   LockHandle();
   LONG nItems = SendDlgItemMessage( hWnd, IDC_LIST, LB_GETCOUNT, 0, 0 );

   if( nItems != LB_ERR )
   {
      pd_char_t *pBuf  = NULL;
#ifdef UNICODE
      char       *pABuf = NULL;
#endif
      LONG        len   = 0;

      for( LONG i = 0; i < nItems; i++ )
      {
         LONG nLen = SendDlgItemMessage( hWnd, IDC_LIST, LB_GETTEXTLEN,
                                         (WPARAM) i, 0 );

         if( len < nLen )
         {
            if( pBuf )
            {
               pBuf = (pd_char_t *)realloc( pBuf, ( nLen + 1 ) * sizeof(pd_char_t) );
#ifdef UNICODE
               pABuf = (char *)realloc( pABuf, ( nLen + 1 ) * sizeof(pd_char_t) );
#endif
            }
            else
            {
               pBuf = (pd_char_t *)malloc( ( nLen + 1 ) * sizeof(pd_char_t) );
#ifdef UNICODE
               pABuf = (char *)malloc( ( nLen + 1 ) * sizeof(pd_char_t) );
#endif
            }

            SendDlgItemMessage( hWnd, IDC_LIST, LB_GETTEXT, (WPARAM) i,
                                (LPARAM) pBuf );
            DWORD nWrite;
#ifdef UNICODE
            nWrite = WideCharToMultiByte( CP_ACP, 0, pBuf, nLen + 1, pABuf,
                                          ( nLen + 1 ) * sizeof(pd_char_t),
                                          NULL, NULL );
            WriteFile( hFile, pABuf, nWrite, &nWrite, NULL );
#else
            WriteFile( hFile, pBuf, nLen, &nWrite, NULL );
#endif
            WriteFile( hFile, "\r\n", 2, &nWrite, NULL );
         }
      }

      if( pBuf )
         free( pBuf );
   }
   UnlockHandle();
}

BOOL
DebugNode::PostMessage( const pd_char_t *msg, DWORD len )
{
   wss_string *msgLine = GetString();
   if( !msgLine )
   {
      if( msg )
         msgLine = new wss_string();
      else
         return TRUE;
   }

   if( msg )
   {
      wss_string *newMsg    = new wss_string(msg);
      int         newMsgLen = newMsg->length();
      int         msgLen    = msgLine->length();
      for( DWORD i = 0; ( i < len ) && ( i < newMsg->length() ); i++ )
      {
         pd_char_t c = newMsg->at(i);

         switch( c )
         {
            case PD_MAKESTR('\t'):
            {
               DWORD tabLen = dwTabStop - ( msgLen % dwTabStop );
               if( tabLen != dwTabStop )
                  msgLine->append( wsTabStop->substr( 0, tabLen ) );
               break;
            }

            default:
               msgLine->append( newMsg->substr(i, 1) );
               msgLen++;
         }
      }
      delete newMsg;
   }

   if( msgLine->length() )
   {
      if( msgLine->at( msgLine->length() - 1 ) != PD_MAKESTR('\n') )
      {
         SetString( msgLine );
      }
      else
      {
         wss_string &msgStr = msgLine->substr( 0, msgLine->length() - 1 );
         int len = 10;

         if( msgLine->length() > 1 )
            msg = msgStr.c_str();
         else
            msg = PD_MAKESTR("");

         SendDlgItemMessage( hWnd, IDC_LIST, LB_ADDSTRING, 0, (LPARAM) msg );

         delete msgLine;
      }
   }

   return TRUE;
}

