/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: debugwin.h,v 1.15 2001/02/23 19:02:02 stevek Exp $
 *
 */

#ifndef __DEBUGWIN_H
#define __DEBUGWIN_H 1

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786)

#include <wr_conf.h>
#include <wsi_stdlib.h>

#include <wss_mutex.h>
#include <wss_cv.h>

#include <commctrl.h>

#include "layout.h"

#include <string>
#include <map>

#define WM_DEBUGWIN       (WM_APP + 0x800)
#define WM_NEWTHREAD      WM_DEBUGWIN + 0x00
#define WM_DBG_NOTIFYICON WM_DEBUGWIN + 0x01

typedef struct _ProcMapNode               PROCMAPNODE, * LPPROCMAPNODE;
typedef class ThrDebugNode                ThrDebugNode;
typedef std::map < DWORD, ThrDebugNode *> THRDEBUGMAP;
typedef class ProcDebugNode               ProcDebugNode;
typedef std::map < int, LPPROCMAPNODE >   PROCDEBUGMAP;
typedef class DebugWin                    DebugWin;

struct _ProcMapNode
{
   ProcDebugNode *pProcNode;
   HTREEITEM      hTreeItem;
};

class DebugNode
{
   protected:
      WSS_MUTEX_PROTECT(str);
      wss_string       *str;
      WSS_MUTEX_PROTECT(hWnd);
      HWND              hWnd;
      DWORD             dwTabStop;
      wss_string       *wsTabStop;

      void        LockString  ( void ) { WSS_MUTEX_ENTER(str); }
      void        UnlockString( void ) { WSS_MUTEX_EXIT(str); }
      void        LockHandle  ( void ) { WSS_MUTEX_ENTER(hWnd); }
      void        UnlockHandle( void ) { WSS_MUTEX_EXIT(hWnd); }

   public:
      void        DumpMessages( HANDLE );
      DWORD       GetTabStop  ( void ) { return dwTabStop; }
      HWND        GetHandle   ( void );
      void        SetHandle   ( HWND handle );
      wss_string *GetString   ( void );
      BOOL        SetString   ( wss_string *);
      BOOL        PostMessage ( const pd_char_t *, DWORD );

      DebugNode( DWORD );
     ~DebugNode();
};

class ThrDebugNode : public DebugNode
{
   protected:
      DWORD             tid;

   public:
      static BOOL CALLBACK _WinProc( HWND, UINT, WPARAM, LPARAM );

      void  DumpMessages           ( HANDLE );
      DWORD GetThreadId            ( void ) { return tid; }

      ThrDebugNode( DWORD, HWND, DWORD );
     ~ThrDebugNode();
};

class ProcDebugNode : public DebugNode
{
   protected:
      int               pid;
      DWORD             dwTID;
      WSS_CV_PROTECT   (hWnd);
      BOOL              bTried;
      THRDEBUGMAP       map;
      WSS_MUTEX_PROTECT(map);
      LayoutManager    *layoutManager;
      DWORD             flags;

      void                 LockMap      ( void ) { WSS_MUTEX_ENTER(map); }
      void                 UnlockMap    ( void ) { WSS_MUTEX_EXIT(map); }

   private:
      static BOOL CALLBACK _WinProc     ( HWND, UINT, WPARAM, LPARAM );
      static void          _WinMsgThread( LPVOID );

   public:
      void                 DumpMessages ( HANDLE );
      int                  GetPID       ( void ) { return pid; }
      ThrDebugNode        *FindThread   ( DWORD );
      int                  AddThread    ( DWORD , ThrDebugNode * );
      void                 Flush        ( void );
      BOOL                 PostMessage  ( DWORD, const pd_char_t *, DWORD );
      void                 SetFlags     ( DWORD, DWORD );
      DWORD                GetFlags     ( void ) { return flags; }

      ProcDebugNode( DWORD, int );
     ~ProcDebugNode();
};

class DebugWin
{
   protected:
      DWORD             dwTID;
      DWORD             dwTabStop;
      HWND              hWnd;
      WSS_MUTEX_PROTECT(hWnd);
      WSS_CV_PROTECT   (hWnd);
      BOOL              bTried;
      WSS_MUTEX_PROTECT(debugMap);
      PROCDEBUGMAP      debugMap;
      HWND              hPropWnd;
      WSS_MUTEX_PROTECT(hPropWnd);
      LayoutManager    *layoutManager;
      UINT              iconId;
      DWORD             dwProcFlags;
      BOOL              bAlwaysGlobal;
      BOOL              bPopupOnActivity;

      LRESULT              _IconDrawItem       ( LPDRAWITEMSTRUCT );
      BOOL                 _TrayMessage        ( DWORD, UINT, HICON, LPCTSTR );
      BOOL                 _PostMessageInternal( int, DWORD, const pd_char_t *,
                                                 DWORD );

   private:
      void                 LockMap             ( void ) { WSS_MUTEX_ENTER(debugMap); }
      void                 UnlockMap           ( void ) { WSS_MUTEX_EXIT(debugMap); }
      void                 LockHandle          ( void ) { WSS_MUTEX_ENTER(hWnd); }
      void                 UnlockHandle        ( void ) { WSS_MUTEX_EXIT(hWnd); }
      void                 LockPropDialog      ( void ) { WSS_MUTEX_ENTER(hPropWnd); }
      void                 UnlockPropDialog    ( void ) { WSS_MUTEX_EXIT(hPropWnd); }

      static BOOL CALLBACK _PropWinProc        ( HWND, UINT, WPARAM, LPARAM );
      static BOOL CALLBACK _WinProc            ( HWND, UINT, WPARAM, LPARAM );
      static void          _WinMsgThread       ( LPVOID );

   public:
      void                 AlwaysUseGlobal     ( BOOL );
      ProcDebugNode       *FindProc            ( int );
      void                 Flush               ( int );
      DWORD                GetProcFlags        ( void ) { return dwProcFlags; }
      HWND                 GetPropDialog       ( void );
      ProcDebugNode       *NewProcess          ( int );
      void                 PopupOnActivity     ( BOOL );
      BOOL                 PostMessage         ( int, DWORD,
                                                 const pd_char_t * );
      void                 SetProcFlags        ( DWORD, DWORD );
      BOOL                 TrayIcon            ( HICON, const pd_char_t * );

      DebugWin( DWORD tabStop = 8 );
     ~DebugWin();
};


#endif /* __DEBUGWIN_H */

