/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: gwm.h,v 1.47 2001/08/06 19:55:54 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Header for the Global Wrapper Manager service
 */

#ifndef __GWM_H
#define __GWM_H 1

#include <wr_conf.h>
#include <pdpi/pi_defs.h>

#include <wss_db.h>
#include <wss_mutex.h>



// macros
//

// registry key (under HKEY_LOCAL_MACHINE) for wrappers entries
#define HKEY_WRAPPERS _TEXT("SOFTWARE\\TIS Labs\\Wrappers")

// pipe for Local Wrapper Manager communication
#define GWM_LWM_PIPE _TEXT("\\\\.\\pipe\\gwm\\proc")

// pipe for management client communication
#define GWM_CLIENT_PIPE _TEXT("\\\\.\\pipe\\gwm\\client")

// masks for command and id parts of packet
#define GWM_COMM_CMD_MASK (DWORD)0xff
#define GWM_COMM_ID_MASK  (DWORD)(~0xff)

#define GWM_COMM_ID_SHIFT 8

#define GWM_COMM_CMD(ph) ((ph).commandId & (GWM_COMM_CMD_MASK))
#define GWM_COMM_ID(ph)  ((ph).commandId & (GWM_COMM_ID_MASK))


#define GWM_CLIENT_VERSION_MAJOR 0
#define GWM_CLIENT_VERSION_MINOR 1

#define GWM_WAIT_TIMEOUT 1000L


// LWM communication commands
//
#define LWM_COMM_REQUEST_WRAPPERS      0
#define LWM_COMM_MESSAGE               1
#define LWM_COMM_CREATE_TABLE          2
#define LWM_COMM_DROP_TABLE            3
#define LWM_COMM_INSERT_TABLE_ROW      4
#define LWM_COMM_SELECT_TABLE_ROWS     5
#define LWM_COMM_DELETE_TABLE_ROWS     6
#define LWM_COMM_UPDATE_TABLE_ROWS     7
#define LWM_COMM_REQUEST_TABLE         8
#define LWM_COMM_CREATING_CHILD        9
#define LWM_COMM_CREATED_CHILD        10
#define LWM_COMM_FIND_TABLE           11
#define LWM_COMM_SYNC_RESEND          12
#define LWM_COMM_AUDIT_POST           13

// static command responses (events)
#define LWM_COMM_EVENT_DROP_TABLE    100

// debug command events
#define LWM_COMM_DEBUG_THR_MESSAGES  200
#define LWM_COMM_DEBUG_PROC_MESSAGES 201
#define LWM_COMM_DEBUG_TRACE_CALLS   202
#define LWM_COMM_DEBUG_COUNT_CALLS   203



// Client communication commands
//

// wrctl commands
#define WRCTL_LOAD                            1
#define WRCTL_UNLOAD                          2
#define WRCTL_ENABLED                         3
#define WRCTL_STAT_LINKERS                    4
#define WRCTL_STAT_REFS                       5
#define WRCTL_STAT_MEMORY                     6

// Flags for WRCTL_STAT_REFS
#define WRCTL_STAT_REFS_PROCESSES         0x001
#define WRCTL_STAT_REFS_MODULES           0x002
#define WRCTL_STAT_REFS_LINKER_FILES      0x004
#define WRCTL_STAT_REFS_CRITERIA          0x010
#define WRCTL_STAT_REFS_LIBRARIES         0x020
#define WRCTL_STAT_REFS_WRAPPERS          0x040
#define WRCTL_STAT_REFS_WRAPPER_INSTANCES 0x080

#define WRCTL_STAT_REFS_ALL               0x0ff 


// wrv commands
#define WRV_GET_TABLE_HANDLES                 7
#define WRV_GET_TABLE_COLUMN_DEF              8
#define WRV_GET_TABLE_ROWS                    9
#define WRV_GET_TABLE_HANDLE_BY_NAME         10
#define WRV_GET_MODULE_BY_ID                 11
#define WRV_GET_WRAPPER_BY_ID                12
#define WRV_GET_PROCS_BY_WRAPPER_ID          13
#define WRV_GET_ACT_CRIT_BY_ID               14
#define WRV_GET_LIBRARY_BY_ID                15
#define WRV_GET_DEACT_CRIT_BY_ID             16
#define WRV_GET_MODULE_ID_BY_NAME            17
#define WRV_GET_PROC_BY_PID                  18
#define WRV_GET_ACTIVE_WRAPPERS              19
#define WRV_SET_TABLE_ROW_FIELD              20
#define WRV_ADD_NULL_TABLE_ROW               21
#define WRV_ADD_TABLE_ROW                    22
#define WRV_DEL_TABLE_ROW                    23
#define WRV_GET_VERSION                      24
#define WRV_ADMIN_DONE                       25
#define WRV_COMM_SYNC_RESEND                 26

// static command responses (events)
#define WRV_EVENT_POST                      100


// GWM communication packet flags
//
#define GWM_COMM_UNICODE 0x0001         // packets without this flag are ASCII
#define GWM_COMM_RESENT  0x0002         // this is a resent packet

// Proc debug node flags
//
#define PDNF_NOTHRMSGS  0x01
#define PDNF_NOPROCMSGS 0x02
#define PDNF_TRACECALLS 0x04
#define PDNF_COUNTCALLS 0x08

#define PDNF_MASK       0x0f


// structures and types
//
typedef struct _DB_HANDLERS DB_HANDLERS, * LPDB_HANDLERS;
typedef struct _DB_CACHE DB_CACHE, *LPDB_CACHE;

struct _DB_HANDLERS
{
   int    (*CreateTable)     ( pd_char_t *, int, pd_char_t *, int, wqcd_t,
                               int, u_long * );
   int    (*DropTable)       ( u_long );
   int    (*InsertTableRow)  ( u_long, int, wqrow_t );
   int    (*SelectTableRows) ( u_long, int, wqsx_t, int, int, int, wqtab_t );
   int    (*DeleteTableRows) ( u_long, int, wqsx_t, int );
   int    (*UpdateTableRows) ( u_long, int, wqsx_t, int, wqusl_t );
   u_long (*FindTableByNames)( pd_char_t *, int, pd_char_t * );
   
   void   (*AuditPost)       ( u_long, const void*, size_t);
};

struct _DB_CACHE
{
   WSS_MUTEX_PROTECT(id);
   u_long            id;
   pd_char_t        *name;
   int               refs;
   int               colCount;
   wqcd_t            colDefs;
   LPDB_CACHE       *prev;
   LPDB_CACHE        next;
};

struct create_block
{
   pd_char_t *wname;
   int        instance;
   pd_char_t *tname;
   int        count;
   wqcd_t     cd;
};

struct insert_block
{
   u_long  id;
   int     rowCount;
};

struct delete_block
{
   u_long id;
   int    takeFirst;
};

struct select_block
{
   u_long id;
   int    takeFirst;
   int    orderBy;
   int    desc;
};

struct update_block
{
   u_long  id;
   int     takeFirst;
   wqusl_t setScript;
};

struct event_block
{
    u_long      types;
    size_t      length;
    char        buffer[1];
};

// public function prototypes
//

#ifdef __cplusplus
extern "C" {
#endif

#ifndef _WSS
BOOL LINKAGE WRImpRegisterDBHandlers __P(( LPDB_HANDLERS pHandlers ));
#endif /* _WSS */

#ifdef __cplusplus
}
#endif

#endif /* !__GWM_H */

