/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: gwm_comm.h,v 1.38 2001/08/06 19:59:43 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Header for the Global Wrapper Manager service communication routines
 */

#ifndef __GWM_COMM_H
#define __GWM_COMM_H 1

#ifndef UNICODE
#error The GWM must be compiled with UNICODE enabled
#endif

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786)

#include <wr_conf.h>

#include "gwm.h"

#include <wss_mutex.h>
#include <wss_process.h>

#include <pipesrv.h>

#include <map>
#include <queue>


// structures and types
//
typedef struct _GWM_PROC_PENDING_DATA GWM_PROC_PENDING_DATA, * LPGWM_PROC_PENDING_DATA;
typedef std::map < int, LPGWM_PROC_PENDING_DATA > GWM_PROC_PENDING;

struct _GWM_PROC_PENDING_DATA
{
   LPPIPESRV_PIPE_INFO  pPipe;
   proc_t              *p;
   int                  ppid;
   pd_char_t           *cwd;
   pd_char_t           *path;
   BOOL                 sleep;
};


// public functions
//

#ifdef __cplusplus
extern "C" {
#endif
void GWMClientCommPostEvent( const pd_char_t *, u_long, size_t, const void *);
#ifdef __cplusplus
}
#endif

BOOL GWMClientCommOpenCallback( LPPIPESRV_PIPE_INFO );
void GWMClientCommCloseCallback( LPPIPESRV_PIPE_INFO );
void GWMLWMCommCloseCallback( LPPIPESRV_PIPE_INFO );
BOOL GWMClientCommProcessPacket( LPPIPESRV_PIPE_INFO );
BOOL GWMLWMCommProcessPacket( LPPIPESRV_PIPE_INFO );
BOOL GWMClientCommReadPacket( LPPIPESRV_PIPE_INFO );
BOOL GWMLWMCommReadPacket( LPPIPESRV_PIPE_INFO );
void GWMClientCommSetup( void );
void GWMLWMCommSetup( void );
BOOL GWMClientCommWritePacket( LPPIPESRV_PIPE_INFO, LPPIPESRV_COMM_RESP_HEADER,
                               PVOID );
BOOL GWMLWMCommWritePacket( LPPIPESRV_PIPE_INFO, LPPIPESRV_COMM_RESP_HEADER,
                            PVOID );

void GWMLWMCommSendThreadMessagesEvent( int pid, BOOL state );
void GWMLWMCommSendProcessMessagesEvent( int pid, BOOL state );
void GWMLWMCommSendCountCallsEvent( int pid, BOOL state );
void GWMLWMCommSendTraceCallsEvent( int pid, BOOL state );


// dispatch functions
//
BOOL LWMComm_RequestWrappers( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_Message( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_CreateTable( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_DropTable( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_InsertTableRow( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_SelectTableRows( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_DeleteTableRows( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_UpdateTableRows( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_RequestTable( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_CreatingChild( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_CreatedChild( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_FindTable( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_SyncResend( LPPIPESRV_PIPE_INFO );
BOOL LWMComm_AuditPost( LPPIPESRV_PIPE_INFO );

BOOL ClientComm_AddEventMask( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_AddNullTableRow( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_AddTableRow( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_AdminDone( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_DelEventMask( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_DelTableRow( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetActCritById( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetActiveWrappers( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetDeactCritById( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetEventMask( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetEvents( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetLibraryById( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetTableColumnDef( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetTableHandleByName( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetTableHandles( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetTableRows( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetModuleById( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetModuleIdByName( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetProcByPID( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetProcsByWrapperId( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetVersion( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_GetWrapperById( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_LoadModule( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_Register( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_SetEventMask( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_SetTableRowField( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_StatLinkers( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_StatMemory( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_StatRefs( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_UnloadModule( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_Unregister( LPPIPESRV_PIPE_INFO );
BOOL ClientComm_WrapperEnabled( LPPIPESRV_PIPE_INFO );

#endif /* !__GWM_COMM_H */

