/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: layout.h,v 1.6 2001/02/23 19:02:02 stevek Exp $
 */

#ifndef __LAYOUT_H
#define __LAYOUT_H 1

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786)

#include <wr_conf.h>

#include "layout.rh"

#include <string>
#include <map>


typedef std::map < DWORD, LPVOID >        USERDATA_MAP;
typedef struct Layout                     Layout;
typedef std::map < HWND, Layout * >       LAYOUT_MAP;
typedef std::basic_string < pd_char_t >   wss_string;
typedef std::map < wss_string, Layout * > LAYOUT_NAME_MAP;
typedef class LayoutManager               LayoutManager;


struct Layout
{
   HWND hControl;      // Handle to control
   UINT style;         // Style flags
   HWND hRelative;     // Handle of relative control
   RECT rect;          // Control position
   int  rx;            // Relative X position
   int  ry;            // Relative Y position
   BOOL bit;
};

class LayoutManager
{
   protected:
      HWND            hWnd;
      BOOL          (*dlgProc)( HWND, UINT, WPARAM, LPARAM );
      USERDATA_MAP    map;
      LAYOUT_MAP      components;
      LAYOUT_NAME_MAP compNames;
      RECT            rect;
      RECT            orect;
      BOOL            bit;
      BOOL            initialized;

   private:
      static BOOL CALLBACK _DlgProc     ( HWND, UINT, WPARAM, LPARAM );
      static BOOL CALLBACK _EnumProc    ( HWND, LPARAM );
      BOOL                 StoreUserData( LPVOID );
      void                 DoLayout     ( Layout * );

   public:
      BOOL                 StoreUserData( DWORD, LPVOID );
      LPVOID               FetchUserData( DWORD );
      LPVOID               FetchUserData( void );
      void                 DoLayout     ( WORD, WORD );
      void                 DoLayoutInit ( void );
      void                 AddComponent ( HWND, Layout * );

      LayoutManager( HWND window, LPCTSTR resname );
     ~LayoutManager();
};

#endif /* __LAYOUT_H */

