/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 */

#define WR_SUPPRESS_WQL 1

#include <tchar.h>

/*
 * Generated wrapper's C source.  Do not edit.
 */

#define WR_KERNEL_MODULE

#if 0
#include <sys/param.h>                                          
#include <sys/sysproto.h>                                          
                                                                
#include <sys/types.h>
#include <sys/proc.h>
#ifndef _SYS_QUEUE_H_
#include "queue.h"
#endif
#include <sys/ipc.h>
#include <sys/fcntl.h>
#include <sys/msg.h>
#include <stdarg.h>
#endif

struct wr_module;

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Chris Vance and Lee Badger
 *
 * This file contains definitions for errors returned by the WSS
 */ 

#define WRE_OK		0		/* Everything is OK; success	*/

#define WRE_ERROR	1		/* General, nondescript error */
#define WRE_PERM	2		/* Operation not permitted */
#define WRE_NOMEM	3		/* Cannot allocate memory */
#define WRE_NOTFOUND	4		/* Object not found */
#define WRE_REF		5		/* Object still has references */
#define WRE_EXIST	6		/* Object already exists	*/
#define	WRE_INVALID_PARAMETER	7	/* Any of a variety of problems */
#define	WRE_BAD_FIELD	8		/* Field name doesn't exist */
#define	WRE_DB_SPACE_EXHAUSTED	9	/* wss.db exhausted */
#define	WRE_TYPE_MISMATCH	10	/* Comparison of incomparable types. */
#define WRE_ALREADY	11		/* Operation already done */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 * 
 * Author: Douglas Kilpatrick
 * 
 * Purpose:
 * To place random platform specific typedef's and defines and header-fixes
 * in one place.
 * 
 * This file should be included by all source files in the project.
 * 
 * If you need to have two different bits of code for different
 * compiliers, create a define to represent the issue
 * (ie, use "WR_BROKEN_STL" instead of "GCC_2_7") and set the define
 * conditionally here.
 */

#ifndef WR_DEFS_INCLUDED
#define WR_DEFS_INCLUDED

#if defined(__sun__) && defined(__svr4__)
#define WR_SOLARIS
/* Solaris specific issues. */
/* No vm_offset_t typedef.  I expect this is global to everything not BSD */
typedef unsigned int vm_offset_t;
#ifndef __P
#define __P(foo) foo
#endif
#define WR_NOAOUTSUPPORT
#define WR_SIGINFO_HEADER <siginfo.h>

#define WR_THREADED		1
#define WR_HAS_ELF_MODULES	1
#define WR_BROKEN_SYSTM_H	1
#define WR_NO_QSORT_DECL        1
#define WR_NO_SNPRINTF_DECL     1
#define WR_BROKEN_SPRINTF_RETV  1
#define WRV_USE_THREADS         1

#define FALSE 0
#define TRUE 1

#define PICFLAG -fPIC

/* If in kernel */
#ifdef WR_KERNEL_MODULE
#define _KERNEL 1

#include <sys/types.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/uio.h>
#include <sys/systm.h>
#include <sys/cmn_err.h>
#else
#define _REENTRANT

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#endif

#endif


#if defined(__FreeBSD__)
#define WR_FREEBSD
/* FreeBSD specific issues. */
#define WR_SIGINFO_HEADER <signal.h>
#define WR_SIGNAL SIGINFO   1
#define WR_HAS_AOUT_MODULES 1
#define WR_BZERO_IS_FN_PTR  1
#define WRV_USE_PIPE        1

#define PICFLAG -fpic

/* If in kernel */
#ifdef WR_KERNEL_MODULE
#define KERNEL 1

#define COMPAT_43	1
#define NFS		1
#define LFS		1
#define ACTUALLY_LKM_NOT_KERNEL	1
#define WR_NO_SNPRINTF_DECL     1

#include <sys/cdefs.h>
#include <sys/time.h>
#include <sys/proc.h>
#include <sys/uio.h>
#include <sys/systm.h>
#include <machine/stdarg.h>
#include <sys/socket.h>
#include <sys/msg.h>
#else
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#endif

typedef struct proc proc_t;

#ifndef MAXNAMELEN
#define MAXNAMELEN 256
#endif

#endif


#if defined(_WIN32)
#define WR_WINNT
/* Windows NT specific issues. */
#include <windows.h>
#include <tlhelp32.h>

#ifndef __P
#define __P(foo) foo
#endif

typedef unsigned long       u_long;
typedef DWORD               pid_t;

typedef struct proc proc_t;
struct proc {
  DWORD p_pid;
  DWORD p_ppid;
  TCHAR *user_name; 
};

#define EXPORT __declspec( dllexport )
#define IMPORT __declspec( dllimport )

#define MAXPATHLEN _MAX_PATH
#endif


/*
 * WR_BROKEN_STL:
 * gcc-2.7's default STL is broken.  Use the "template/" directory
 * for STLplus on that platform.  On other platforms, use the native
 * STL support.
 */
#if ( __GNUC__ == 2 ) && ( __GNUC_MINOR__ <= 7 )
#define WR_BROKEN_STL                                           1
#endif /* GCC-2.7 */


/*
 * WR_NO_FOO_DECL
 * 
 * A common function does not exist in the kernel.
 * The lkm/wsi/platform/wsi_utilities.c file contains an implimentation,
 * so we just have to give a prototype here.
 */
#if defined(WR_KERNEL_MODULE)
#ifdef WR_NO_SNPRINTF_DECL
int snprintf(char *str, size_t size, const char *format, ...);
#endif
#ifdef WR_NO_QSORT_DECL
void qsort(void *base, size_t bel, size_t width,
        int (*compar)(const void*, const void*));
#endif
#endif

#endif /* WR_DEFS_INCLUDED */
/*
 * Copyright (c) 1991, 1993
 *	The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *


 */

#ifndef _SYS_QUEUE_H_
#define	_SYS_QUEUE_H_

/*
 * This file defines five types of data structures: singly-linked lists,
 * slingly-linked tail queues, lists, tail queues, and circular queues.
 *
 * A singly-linked list is headed by a single forward pointer. The elements
 * are singly linked for minimum space and pointer manipulation overhead at
 * the expense of O(n) removal for arbitrary elements. New elements can be
 * added to the list after an existing element or at the head of the list.
 * Elements being removed from the head of the list should use the explicit
 * macro for this purpose for optimum efficiency. A singly-linked list may
 * only be traversed in the forward direction.  Singly-linked lists are ideal
 * for applications with large datasets and few or no removals or for
 * implementing a LIFO queue.
 *
 * A singly-linked tail queue is headed by a pair of pointers, one to the
 * head of the list and the other to the tail of the list. The elements are
 * singly linked for minimum space and pointer manipulation overhead at the
 * expense of O(n) removal for arbitrary elements. New elements can be added
 * to the list after an existing element, at the head of the list, or at the
 * end of the list. Elements being removed from the head of the tail queue
 * should use the explicit macro for this purpose for optimum efficiency.
 * A singly-linked tail queue may only be traversed in the forward direction.
 * Singly-linked tail queues are ideal for applications with large datasets
 * and few or no removals or for implementing a FIFO queue.
 *
 * A list is headed by a single forward pointer (or an array of forward
 * pointers for a hash table header). The elements are doubly linked
 * so that an arbitrary element can be removed without a need to
 * traverse the list. New elements can be added to the list before
 * or after an existing element or at the head of the list. A list
 * may only be traversed in the forward direction.
 *
 * A tail queue is headed by a pair of pointers, one to the head of the
 * list and the other to the tail of the list. The elements are doubly
 * linked so that an arbitrary element can be removed without a need to
 * traverse the list. New elements can be added to the list before or
 * after an existing element, at the head of the list, or at the end of
 * the list. A tail queue may only be traversed in the forward direction.
 *
 * A circle queue is headed by a pair of pointers, one to the head of the
 * list and the other to the tail of the list. The elements are doubly
 * linked so that an arbitrary element can be removed without a need to
 * traverse the list. New elements can be added to the list before or after
 * an existing element, at the head of the list, or at the end of the list.
 * A circle queue may be traversed in either direction, but has a more
 * complex end of list detection.
 *
 * For details on the use of these macros, see the queue(3) manual page.
 */

/*
 * Singly-linked List definitions.
 */
#define SLIST_HEAD(name, type)						\
struct name {								\
	struct type *slh_first;	/* first element */			\
}
 
#define SLIST_ENTRY(type)						\
struct {								\
	struct type *sle_next;	/* next element */			\
}
 
/*
 * Singly-linked List functions.
 */
#define SLIST_INIT(head) {						\
	(head)->slh_first = NULL;					\
}

#define SLIST_INSERT_AFTER(slistelm, elm, field) {			\
	(elm)->field.sle_next = (slistelm)->field.sle_next;		\
	(slistelm)->field.sle_next = (elm);				\
}

#define SLIST_INSERT_HEAD(head, elm, field) {				\
	(elm)->field.sle_next = (head)->slh_first;			\
	(head)->slh_first = (elm);					\
}

#define SLIST_REMOVE_HEAD(head, field) {				\
	(head)->slh_first = (head)->slh_first->field.sle_next;		\
}

#define SLIST_REMOVE(head, elm, type, field) {				\
	if ((head)->slh_first == (elm)) {				\
		SLIST_REMOVE_HEAD((head), field);			\
	}								\
	else {								\
		struct type *curelm = (head)->slh_first;		\
		while( curelm->field.sle_next != (elm) )		\
			curelm = curelm->field.sle_next;		\
		curelm->field.sle_next =				\
		    curelm->field.sle_next->field.sle_next;		\
	}								\
}

/*
 * Singly-linked Tail queue definitions.
 */
#define STAILQ_HEAD(name, type)						\
struct name {								\
	struct type *stqh_first;/* first element */			\
	struct type **stqh_last;/* addr of last next element */		\
}

#define STAILQ_ENTRY(type)						\
struct {								\
	struct type *stqe_next;	/* next element */			\
}

/*
 * Singly-linked Tail queue functions.
 */
#define	STAILQ_INIT(head) {						\
	(head)->stqh_first = NULL;					\
	(head)->stqh_last = &(head)->stqh_first;			\
}

#define STAILQ_INSERT_HEAD(head, elm, field) {				\
	if (((elm)->field.stqe_next = (head)->stqh_first) == NULL)	\
		(head)->stqh_last = &(elm)->field.stqe_next;		\
	(head)->stqh_first = (elm);					\
}

#define STAILQ_INSERT_TAIL(head, elm, field) {				\
	(elm)->field.stqe_next = NULL;					\
	*(head)->stqh_last = (elm);					\
	(head)->stqh_last = &(elm)->field.stqe_next;			\
}

#define STAILQ_INSERT_AFTER(head, tqelm, elm, field) {			\
	if (((elm)->field.stqe_next = (tqelm)->field.stqe_next) == NULL)\
		(head)->stqh_last = &(elm)->field.stqe_next;		\
	(tqelm)->field.stqe_next = (elm);				\
}

#define STAILQ_REMOVE_HEAD(head, field) {				\
	if (((head)->stqh_first =					\
	     (head)->stqh_first->field.stqe_next) == NULL)		\
		(head)->stqh_last = &(head)->stqh_first;		\
}

#define STAILQ_REMOVE(head, elm, type, field) {				\
	if ((head)->stqh_first == (elm)) {				\
		STAILQ_REMOVE_HEAD(head, field);			\
	}								\
	else {								\
		struct type *curelm = (head)->stqh_first;		\
		while( curelm->field.stqe_next != (elm) )		\
			curelm = curelm->field.stqe_next;		\
		if((curelm->field.stqe_next =				\
		    curelm->field.stqe_next->field.stqe_next) == NULL)	\
			(head)->stqh_last = &(curelm)->field.stqe_next;	\
	}								\
}

/*
 * List definitions.
 */
#define LIST_HEAD(name, type)						\
struct name {								\
	struct type *lh_first;	/* first element */			\
}

#define LIST_ENTRY(type)						\
struct {								\
	struct type *le_next;	/* next element */			\
	struct type **le_prev;	/* address of previous next element */	\
}

/*
 * List functions.
 */
#define	LIST_INIT(head) {						\
	(head)->lh_first = NULL;					\
}

#define LIST_INSERT_AFTER(listelm, elm, field) {			\
	if (((elm)->field.le_next = (listelm)->field.le_next) != NULL)	\
		(listelm)->field.le_next->field.le_prev =		\
		    &(elm)->field.le_next;				\
	(listelm)->field.le_next = (elm);				\
	(elm)->field.le_prev = &(listelm)->field.le_next;		\
}

#define LIST_INSERT_BEFORE(listelm, elm, field) {			\
	(elm)->field.le_prev = (listelm)->field.le_prev;		\
	(elm)->field.le_next = (listelm);				\
	*(listelm)->field.le_prev = (elm);				\
	(listelm)->field.le_prev = &(elm)->field.le_next;		\
}

#define LIST_INSERT_HEAD(head, elm, field) {				\
	if (((elm)->field.le_next = (head)->lh_first) != NULL)		\
		(head)->lh_first->field.le_prev = &(elm)->field.le_next;\
	(head)->lh_first = (elm);					\
	(elm)->field.le_prev = &(head)->lh_first;			\
}

#define LIST_REMOVE(elm, field) {					\
	if ((elm)->field.le_next != NULL)				\
		(elm)->field.le_next->field.le_prev = 			\
		    (elm)->field.le_prev;				\
	*(elm)->field.le_prev = (elm)->field.le_next;			\
}

/*
 * Tail queue definitions.
 */
#define TAILQ_HEAD(name, type)						\
struct name {								\
	struct type *tqh_first;	/* first element */			\
	struct type **tqh_last;	/* addr of last next element */		\
}

#define TAILQ_HEAD_INITIALIZER(head)					\
	{ NULL, &(head).tqh_first }

#define TAILQ_ENTRY(type)						\
struct {								\
	struct type *tqe_next;	/* next element */			\
	struct type **tqe_prev;	/* address of previous next element */	\
}

/*
 * Tail queue functions.
 */
#define	TAILQ_EMPTY(head) ((head)->tqh_first == NULL)

#define	TAILQ_FIRST(head) ((head)->tqh_first)

#define	TAILQ_LAST(head) ((head)->tqh_last)

#define	TAILQ_NEXT(elm, field) ((elm)->field.tqe_next)

#define TAILQ_PREV(elm, field) ((elm)->field.tqe_prev)

#define	TAILQ_INIT(head) {						\
	(head)->tqh_first = NULL;					\
	(head)->tqh_last = &(head)->tqh_first;				\
}

#define TAILQ_INSERT_HEAD(head, elm, field) {				\
	if (((elm)->field.tqe_next = (head)->tqh_first) != NULL)	\
		(head)->tqh_first->field.tqe_prev =			\
		    &(elm)->field.tqe_next;				\
	else								\
		(head)->tqh_last = &(elm)->field.tqe_next;		\
	(head)->tqh_first = (elm);					\
	(elm)->field.tqe_prev = &(head)->tqh_first;			\
}

#define TAILQ_INSERT_TAIL(head, elm, field) {				\
	(elm)->field.tqe_next = NULL;					\
	(elm)->field.tqe_prev = (head)->tqh_last;			\
	*(head)->tqh_last = (elm);					\
	(head)->tqh_last = &(elm)->field.tqe_next;			\
}

#define TAILQ_INSERT_AFTER(head, listelm, elm, field) {			\
	if (((elm)->field.tqe_next = (listelm)->field.tqe_next) != NULL)\
		(elm)->field.tqe_next->field.tqe_prev = 		\
		    &(elm)->field.tqe_next;				\
	else								\
		(head)->tqh_last = &(elm)->field.tqe_next;		\
	(listelm)->field.tqe_next = (elm);				\
	(elm)->field.tqe_prev = &(listelm)->field.tqe_next;		\
}

#define TAILQ_INSERT_BEFORE(listelm, elm, field) {			\
	(elm)->field.tqe_prev = (listelm)->field.tqe_prev;		\
	(elm)->field.tqe_next = (listelm);				\
	*(listelm)->field.tqe_prev = (elm);				\
	(listelm)->field.tqe_prev = &(elm)->field.tqe_next;		\
}

#define TAILQ_REMOVE(head, elm, field) {				\
	if (((elm)->field.tqe_next) != NULL)				\
		(elm)->field.tqe_next->field.tqe_prev = 		\
		    (elm)->field.tqe_prev;				\
	else								\
		(head)->tqh_last = (elm)->field.tqe_prev;		\
	*(elm)->field.tqe_prev = (elm)->field.tqe_next;			\
}

/*
 * Circular queue definitions.
 */
#define CIRCLEQ_HEAD(name, type)					\
struct name {								\
	struct type *cqh_first;		/* first element */		\
	struct type *cqh_last;		/* last element */		\
}

#define CIRCLEQ_ENTRY(type)						\
struct {								\
	struct type *cqe_next;		/* next element */		\
	struct type *cqe_prev;		/* previous element */		\
}

/*
 * Circular queue functions.
 */
#define	CIRCLEQ_INIT(head) {						\
	(head)->cqh_first = (void *)(head);				\
	(head)->cqh_last = (void *)(head);				\
}

#define CIRCLEQ_INSERT_AFTER(head, listelm, elm, field) {		\
	(elm)->field.cqe_next = (listelm)->field.cqe_next;		\
	(elm)->field.cqe_prev = (listelm);				\
	if ((listelm)->field.cqe_next == (void *)(head))		\
		(head)->cqh_last = (elm);				\
	else								\
		(listelm)->field.cqe_next->field.cqe_prev = (elm);	\
	(listelm)->field.cqe_next = (elm);				\
}

#define CIRCLEQ_INSERT_BEFORE(head, listelm, elm, field) {		\
	(elm)->field.cqe_next = (listelm);				\
	(elm)->field.cqe_prev = (listelm)->field.cqe_prev;		\
	if ((listelm)->field.cqe_prev == (void *)(head))		\
		(head)->cqh_first = (elm);				\
	else								\
		(listelm)->field.cqe_prev->field.cqe_next = (elm);	\
	(listelm)->field.cqe_prev = (elm);				\
}

#define CIRCLEQ_INSERT_HEAD(head, elm, field) {				\
	(elm)->field.cqe_next = (head)->cqh_first;			\
	(elm)->field.cqe_prev = (void *)(head);				\
	if ((head)->cqh_last == (void *)(head))				\
		(head)->cqh_last = (elm);				\
	else								\
		(head)->cqh_first->field.cqe_prev = (elm);		\
	(head)->cqh_first = (elm);					\
}

#define CIRCLEQ_INSERT_TAIL(head, elm, field) {				\
	(elm)->field.cqe_next = (void *)(head);				\
	(elm)->field.cqe_prev = (head)->cqh_last;			\
	if ((head)->cqh_first == (void *)(head))			\
		(head)->cqh_first = (elm);				\
	else								\
		(head)->cqh_last->field.cqe_next = (elm);		\
	(head)->cqh_last = (elm);					\
}

#define	CIRCLEQ_REMOVE(head, elm, field) {				\
	if ((elm)->field.cqe_next == (void *)(head))			\
		(head)->cqh_last = (elm)->field.cqe_prev;		\
	else								\
		(elm)->field.cqe_next->field.cqe_prev =			\
		    (elm)->field.cqe_prev;				\
	if ((elm)->field.cqe_prev == (void *)(head))			\
		(head)->cqh_first = (elm)->field.cqe_next;		\
	else								\
		(elm)->field.cqe_prev->field.cqe_next =			\
		    (elm)->field.cqe_next;				\
}

#ifdef KERNEL

/*
 * XXX insque() and remque() are an old way of handling certain queues.
 * They bogusly assumes that all queue heads look alike.
 */

struct quehead {
	struct quehead *qh_link;
	struct quehead *qh_rlink;
};

#ifdef	__GNUC__

static __inline void
insque(void *a, void *b)
{
	struct quehead *element = a, *head = b;

	element->qh_link = head->qh_link;
	element->qh_rlink = head;
	head->qh_link = element;
	element->qh_link->qh_rlink = element;
}

static __inline void
remque(void *a)
{
	struct quehead *element = a;

	element->qh_link->qh_rlink = element->qh_rlink;
	element->qh_rlink->qh_link = element->qh_link;
	element->qh_rlink = 0;
}

#else /* !__GNUC__ */

void	insque __P((void *a, void *b));
void	remque __P((void *a));

#endif /* __GNUC__ */

#endif /* KERNEL */

#endif /* !_SYS_QUEUE_H_ */
/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: 
 *
 */

#ifndef __WSI_MUTEX_H
#define __WSI_MUTEX_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

/*
 * TBD: If we ever thread-safe the FreeBSD port these need to be set
 */
#define WSI_MUTEX_PROTECT(var) int mutex_ ## var
#define WSI_MUTEX_EXPORT(var)
#define WSI_MUTEX_INIT(var) 
#define WSI_MUTEX_DESTROY(var)
#define WSI_MUTEX_ENTER(var)
#define WSI_MUTEX_ENTER_DEBUG(var)
#define WSI_MUTEX_EXIT(var)
#define WSI_MUTEX_EXIT_DEBUG(var)

#define WSI_MUTEX_INIT_NOMNG(var,name)
#define WSI_MUTEX_DESTROY_NOMNG(var)
#define WSI_MUTEX_ENTER_NOMNG(var)
#define WSI_MUTEX_EXIT_NOMNG(var)


/************************************************************************\
 * Public types                                                         *
\************************************************************************/

typedef unsigned int wsi_mutex_t;


#endif /* !__WSS_MUTEX_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent condition variables
 *
 */

#ifndef __WSI_CV_H
#define __WSI_CV_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_CV_PROTECT(var)
#define WSI_CV_EXPORT(var)
#define WSI_CV_INIT(var)
#define WSI_CV_DESTROY(var)
#define WSI_CV_WAIT(var)
#define WSI_CV_WAIT_DEBUG(var)
#define WSI_CV_SIGNAL(var)
#define WSI_CV_SIGNAL_DEBUG(var)
#define WSI_CV_BROADCAST(var)
#define WSI_CV_BROADCAST_DEBUG(var)

#endif /* !__WSS_CV_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures used by the
 *          system-dependant sections of the module loading code.
 */

#ifndef __WSI_MODULE_H
#define __WSI_MODULE_H  1


/************************************************************************\
 * External structures                                                  *
\************************************************************************/

#ifndef __WSS_MODULE_H
struct wss_mod;
#endif


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

void wsi_mod_delete __P(( struct wss_mod *wrm ));

#endif /* _WSI_MODULE_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Chris Vance and Lee Badger
 *
 * Purpose: 
 */

#ifndef  __WSS_IFACE_H
#define __WSS_IFACE_H   1

/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

/*
 * Return codes for event interception.
 */
#define WR_ALLOW   0
#define WR_SUCCESS 0
#define WR_DENY    1
#define WR_NONE    2
#define WR_MASK1   0x3
#define WR_MASK2   0xfffffffc


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_symbol wss_symbol_t;
typedef int               wss_id_t;

struct wss_symbol
{
   char   *name;
   u_long  addr;
   int     count;
};


/*
 * Types for generated wrappers.
 * 
 * A string is a string of course.  Char* representation.
 * A databuf_t is a structure that holds data.  buf+len+max size
 * 
 * But the actual buffer first so that "char *foo = buf" type code
 * won't trash anything important
 */
typedef char *string;

#define WR_BUFSIZE      128
struct wr_databuf
{
   char *buf;
   int max, size;
};

typedef struct wr_databuf *databuf_t;

#endif /* __WSS_IFACE_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the module
 * loading code.
 */

#ifndef __WSS_PROCESS_H
#define __WSS_PROCESS_H 1

#include <sys/types.h>
#include <errno.h>



#ifndef __WSS_IFACE_H
#include <wss_iface.h>
#endif

#ifndef __WSI_MUTEX_H
#include <wsi_mutex.h>
#endif

#ifndef __WSI_CV_H
#include <wsi_cv.h>
#endif

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_arg  wss_arg_t;
typedef struct wss_proc wss_proc_t;

/*
 * Arg-replacement structure.
 */
struct wss_arg
{
   int        index;
   size_t     size;
   void      *old;
   wss_arg_t *next;
};

/* 
 * This referred to as the "wrappers-specific process information."
 * 
 * After the WSS has been installed, each process created is
 * intercepted during the fork1() syscall.  At this time a wss_proc
 * structure is created and added to the global list.  This structure
 * stores process-related information.
 */
struct wss_proc
{
   WSI_MUTEX_PROTECT(wss_proc);            /* Structure lock                  */
   LIST_ENTRY(wss_proc)         p_list;    /* List of all processes.          */
   pid_t                        pid;       /* Process identifier.             */
   int                          flags;     /* Flags                           */
   char                        *pathbuf;   /* Stores full path of executable
                                              MALLOC'd memory NULL-terminated */
   char                        *opathbuf;  /* Stores full path of old program
                                              MALLOC'd memory NULL-terminated */
   char                        *progname;  /* Stores program name (no path info)
                                              MALLOC'd memory NULL-terminated */
   char                        *cwd;       /* Stores current working directory
                                              MALLOC'd memory NULL-terminated */
   int                          setuid_to; /* Indicates new uid, if current
                                              exec is setuid; -1 otherwise.   */
   int                          setgid_to; /* Indicates new gid, if current
                                              exec is setgid; -1 otherwise.   */
   proc_t                      *p;         /* Ptr to process (may be unused)  */
   struct sysentvec            *p_sysent;  /* The original p_sysent vector    */
   struct wss_wrapper_instance *wrappers;  /* List of wrappers used           */
   wss_arg_t                   *arglist;   /* List of replaced arguments      */
   struct wss_wrv              *viewer;    /* Stores viewer structure when
                                              process is a wrapper viewer     */
   int                          insyscall; /* Reference counter for syscalls  */
   void                        *opaque;    /* System-dependent data           */
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define WRPF_GMASK   0x00000fff        /* Global flags mask                */
#define WRPF_WRAPPED 0x00000001        /* Process wrapped flag             */
#define WRPF_POSTED  0x00000002        /* Event posted flag                */

/* System-dependent parts can use flags within the WRPF_LMASK */
#define WRPF_LMASK   0xfffff000        /* Local flags mask                 */
#define WRPF_LOCAL   0x00001000        /* Starting value for local flags   */


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

LIST_HEAD(wss_proclist, wss_proc);
extern struct wss_proclist wss_allproc;
WSI_MUTEX_EXPORT( wss_allproc );

extern int wss_proc_num_in_syscall;
WSI_MUTEX_EXPORT( wss_proc_num_in_syscall );
WSI_CV_EXPORT( wss_proc_num_in_syscall );


#ifndef WR_SUPPRESS_FUNCTION_DECL
/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

wss_proc_t *wss_pfind                 __P(( register pid_t pid ));
wss_proc_t *wss_pfind_r               __P(( register pid_t pid ));
void        wss_proc_init             __P(( void ));
void        wss_proc_shutdown         __P(( void ));
void        wss_proc_exit             __P(( proc_t *p ));
void        wss_proc_exit_stage1      __P(( proc_t *p ));
void        wss_proc_exit_stage2      __P(( pid_t pid ));
wss_proc_t *wss_proc_new              __P(( proc_t *p ));
void        wss_proc_fork             __P(( proc_t *parent, proc_t *child,
                                            int flags ));
int         wss_proc_add_wrapper      __P(( wss_proc_t *wrp, char *name,
                                            wss_id_t id ));
int         wss_proc_del_wrapper      __P(( wss_proc_t *wrp, char *name,
                                            wss_id_t id ));
void        wss_proc_del_all_wrappers __P(( wss_proc_t *wrp ));
#endif /* WR_SUPPRESS_FUNCTION_DECL */

#endif /* __WSS_PROCESS_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: This file contains prototypes and structures use by the module
 * loading code.
 */

#ifndef __WSS_WRAPPER_H
#define __WSS_WRAPPER_H 1

#ifndef __WSS_PROCESS_H
#include <wss_process.h>
#endif


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_wrapper          wss_wrapper_t;
typedef struct wss_wrapper_data     wss_wrapper_data_t;
typedef struct wss_wrapper_instance wss_wrapper_instance_t;

typedef int(*wss_wrapper_data_func_t) __P(( proc_t *, wss_proc_t * ));
typedef int(*wss_wrapper_func_t)      __P(( proc_t *, wss_proc_t *,
                                            wss_wrapper_data_t * ));
typedef int(*wss_wrapper_intercept_t) __P(( proc_t *p, void *uap, int *retval,
                                            int code, wss_proc_t *wrp,
                                            wss_wrapper_data_t *data ));


/*
 *  Structure for holding addition data in wrappers
 */
struct wss_wrapper_data
{
   int  *errnum;
   void *wiv;
   /* add addition data here */
};

/*
 * This is the structure used by wrappers when they are loaded into
 * the WSS.  Each wrapper has a one-to-one correspondence with a
 * wr_module structure, representing loaded module.  Since the unload
 * function was specific to the unloading of the module, and not the
 * unload of a wrapper, it's pointer is stored in the wr_module not
 * the wrapper.
 */
struct wss_wrapper
{
   wss_id_t                  id;         /* Unique identifier; == module id */
   char                     *name;       /* Name of the wrapper - ALLOCATED */
   struct wss_mod           *module;     /* Pointer to module               */
   wss_wrapper_t            *next;       /* Link List Pointer               */
   int                       refcount;   /* Reference Count                 */
   int                       procs;      /* Active Count of wrapped procs   */
   void                    **pre;        /* Array of function pointers      */
   void                    **post;       /* Array of function pointers      */
   wss_wrapper_func_t        activate;   /* Pointer to activate function    */
   wss_wrapper_func_t        deactivate; /* Pointer to deactivate function  */
   wss_wrapper_func_t        duplicate;  /* Pointer to duplicate function   */
   wss_wrapper_data_func_t   new_data;   /* Pointer to data create function */
};

/*
 * Structure for an activated wrapper.  This struct has a one-to-one
 * relationship with a wrapped process.
 */
struct wss_wrapper_instance
{
   wss_wrapper_t          *wrapper;  /* Pointer to the actual wrapper    */
   int                     active:1; /* Bool: has activate() been called */
   wss_id_t                actid;    /* ID of the activation criteria    */
   void                   *data;     /* Local data.                      */
   wss_wrapper_instance_t *next;     /* Link to next wrapper instance    */
   wss_wrapper_instance_t *prev;     /* Link to prev wrapper instance    */
};



/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

extern wss_wrapper_t *wrapper_list;
WSI_MUTEX_EXPORT( wrapper_list );


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void           wss_wrapper_init              __P(( void ));
void           wss_wrapper_shutdown          __P(( void ));
int            wss_wrapper_loaded            __P(( char *name ));
int            wss_wrapper_ref               __P(( char *name ));
int            wss_wrapper_deref             __P(( char *name ));
int            wss_wrapper_register          __P(( struct wss_mod *wrm,
                                                   void **pre, void **post,
                                                   void *activate,
                                                   void *deactivate,
                                                   void *duplicate ));
int            wss_wrapper_remove            __P(( wss_id_t id ));
wss_wrapper_t *wss_wrapper_find              __P(( char *name ));
int            wss_wrapper_call_activation   __P(( proc_t *p,
                                                   wss_proc_t *wrp ));
int            wss_wrapper_call_deactivation __P(( proc_t *p,
                                                   wss_proc_t *wrp ));
int            wss_wrapper_call_duplication  __P(( proc_t *p,
                                                   wss_proc_t *wrp ));


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#define WSS_GET_WIV_VAR(var) (((struct wr_wiv*)(data->wiv))->var)
#define WSS_SAVE_WIV(newwiv) (data->wiv = (newwiv))
#define WSS_GET_WIV()        ((struct wr_wiv*)(data->wiv))

#endif /* __WSS_WRAPPER_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures used by the
 *          activation and deactivation critera code.
 */

#ifndef __WSS_CRITERIA_H
#define __WSS_CRITERIA_H   1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSS_PROCESS_H
#include <wss_process.h>
#endif


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_crit wss_crit_t;

/* Represents a loaded activation or deactivation criterion.  */
struct wss_crit
{
   char           *name;
   void           *df;
   wss_id_t        id;
#define WRACT_ENABLE  1
#define WRACT_DISABLE 0
   int             status;
   struct wss_mod *module;
   wss_crit_t     *next;
};


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

extern wss_crit_t *wss_act_crit_list;
WSI_MUTEX_EXPORT( wss_act_crit_list );
extern wss_crit_t *wss_deact_crit_list;
WSI_MUTEX_EXPORT( wss_deact_crit_list );


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void wss_crit_init               __P(( void ));
void wss_crit_shutdown           __P(( void ));
int  wss_crit_check_activation   __P(( wss_proc_t *wrp ));
int  wss_crit_check_deactivation __P(( wss_proc_t *wrp ));
int  wss_crit_add                __P(( struct wss_mod *wrm, void *df ));
int  wss_crit_remove             __P(( wss_id_t id ));

#endif /* __WSS_CRITERIA_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: This file contains prototypes and structures use by the module
 * loading code.
 */

#ifndef __WSS_LIBRARY_H
#define __WSS_LIBRARY_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_MUTEX_H
#include <wsi_mutex.h>
#endif

#ifndef __WSS_IFACE_H
#include <wss_iface.h>
#endif


/***************************************************************************\
 * Forward references                                                      *
\***************************************************************************/

struct wss_mod;


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_lib wss_lib_t;

struct wss_lib
{
   char             *name;
   struct wss_mod   *module;
   wss_id_t          id;
   int               status;
   int               major;
   int               minor;
   void            (*fini)();
   WSI_MUTEX_PROTECT(symtab );
   wss_symbol_t     *symtab;
   wss_lib_t        *next;
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

wss_lib_t *wss_lib_find     __P(( wss_id_t ));
void       wss_lib_init     __P(( void ));
int        wss_lib_loaded   __P(( const char * ));
int        wss_lib_register __P(( struct wss_mod *, wss_symbol_t *, void * ));
void       wss_lib_remove   __P(( wss_id_t ));
void       wss_lib_shutdown __P(( void ));

#endif /* __WSS_LIBRARY_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Mark Feldman
 *
 * Purpose: This file contains prototypes and structures used to maintain
 *          a list of ids to keep track of inter-module references.
 */

#ifndef __WSS_IDLIST_H
#define __WSS_IDLIST_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif /* WR_DEFS_INCLUDED */

#include <sys/types.h>

#ifndef __WSS_IFACE_H
#include <wss_iface.h>
#endif


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_idlist wss_idlist_t;

struct wss_idlist
{
   wss_id_t       id;
   wss_idlist_t *next;
};

typedef wss_idlist_t *wss_idlist_iterator_t;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int  wss_idlist_has_id        __P(( wss_idlist_t *head, wss_id_t id ));
void wss_idlist_add_id        __P(( wss_idlist_t **head, wss_id_t id ));
void wss_idlist_del_id        __P(( wss_idlist_t **head, wss_id_t id ));
void wss_idlist_del_all_ids   __P(( wss_idlist_t **head ));
int  wss_idlist_count_ids     __P(( wss_idlist_t *head ));
void wss_idlist_iterate       __P(( wss_idlist_iterator_t *, wss_idlist_t * ));
int  wss_idlist_iterator_next __P(( wss_idlist_iterator_t * ));


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#define wss_idlist_iterate(i,l)     ((void)(*(i) = (l)))

#endif /* !__WSS_IDLIST_H */

/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Chris Vance, Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the module
 * loading code.
 */

#ifndef __WSS_MODULE_H
#define __WSS_MODULE_H  1

#include <sys/types.h>

#ifndef __WSS_PROCESS_H
#include <wss_process.h>
#endif
#ifndef __WSS_WRAPPER_H
#include <wss_wrapper.h>
#endif
#ifndef __WSS_CRITERIA_H
#include <wss_criteria.h>
#endif
#ifndef __WSS_LIBRARY_H
#include <wss_library.h>
#endif
#ifndef __WSS_IDLIST_H
#include <wss_idlist.h>
#endif


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define WRLINK_MOD_NAME             __wr_myname
#define WRLINKNAME_MOD_NAME         "__wr_myname"
#define WRLINK_MOD_TYPE             __wr_mytype
#define WRLINKNAME_MOD_TYPE         "__wr_mytype"
#define WRLINK_MOD_DEPENDENCIES     __wr_mydependencies
#define WRLINKNAME_MOD_DEPENDENCIES "__wr_mydependencies"


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef enum wss_mod_type wss_mod_type_t;
typedef struct wss_mod    wss_mod_t;

/*
 * The WR_MOD_* values are stored in the wr_module structure.  They
 * are used to specify what type of module this is.  It should be set
 * during the module's registration routine.
 */

enum wss_mod_type
{
   WR_MOD_WRAPPER      = 1,
   WR_MOD_ACTIVATION   = 2,
   WR_MOD_LIBRARY      = 4,
   WR_MOD_DEACTIVATION = 8
};


/*
 * The wrappers module structure.  It stores information about modules
 * that have been loaded into the WSS.  Currently modules are either
 * wrappers, activation criteria, or libraries.
 */
struct wss_mod
{
   WSI_MUTEX_PROTECT(mod);      /* Mutex to protect structure            */
   wss_id_t          id;        /* Unique identifier                     */
   char             *name;      /* name of the module                    */
   wss_mod_type_t    type;      /* WR_MOD_WRAPPER,_ACTIVATION,_LIBRARY   */
   u_long            addr;      /* memory location where module resides  */
   u_long            size;      /* allocated size                        */
   int               uid;       /* Person that loaded this module        */
   long              loadtime;  /* Time the module was loaded into WSS   */
   WSI_MUTEX_PROTECT(used);     /* Mutex to protect used list            */
   wss_idlist_t     *used;      /* Link list of modules using this one   */
   WSI_MUTEX_PROTECT(uses);     /* Mutex to protect uses list            */
   wss_idlist_t     *uses;      /* Link list of modules this module uses */
   wss_mod_t        *next;      /* link list ptr                         */
   void            (*unload)(); /* Pointer to module unload funtion      */
};


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

extern wss_mod_t *wss_module_list;
WSI_MUTEX_EXPORT( wss_module_list );


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
void       wss_mod_init              __P(( void ));
void       wss_mod_shutdown          __P(( void ));
wss_mod_t *wss_mod_find              __P(( wss_id_t id ));
wss_mod_t *wss_mod_create            __P(( int addr, size_t size ));
void       wss_mod_destroy           __P(( wss_mod_t *wrm ));
int        wss_mod_delete            __P(( wss_id_t id ));
void       wss_mod_unload_all        __P(( void ));
int        wss_mod_at_unload         __P(( void *uf, wss_id_t id ));
int        wss_mod_remove_at_unload  __P(( wss_id_t id ));
void       wss_mod_add_dependency    __P(( wss_id_t uses_id, wss_id_t used_id ));
void       wss_mod_remove_dependency __P(( wss_id_t uses_id, wss_id_t used_id ));

#ifndef __WSI_MODULE_H
#include <wsi_module.h>
#endif

#endif /* WR_SUPPRESS_FUNCTION_DECL */

#endif /* _WSS_MODULE_H */

/* EPERM, 1,  Operation not permitted */
#define WR_BADPERM  		(1  * 4)
#define WR_A_BADPERM		(1  * 4 | WR_ALLOW)
#define WR_D_BADPERM		(1  * 4 | WR_DENY)
/* ENOENT, 2,  No such file or directory */
#define WR_NOFILE  		(2  * 4)
#define WR_A_NOFILE		(2  * 4 | WR_ALLOW)
#define WR_D_NOFILE		(2  * 4 | WR_DENY)
/* ESRCH, 3,  No such process */
#define WR_NOPROC  		(3  * 4)
#define WR_A_NOPROC		(3  * 4 | WR_ALLOW)
#define WR_D_NOPROC		(3  * 4 | WR_DENY)
/* EINTR, 4,  Interrupted system call */
#define WR_EINTR  		(4  * 4)
#define WR_A_EINTR		(4  * 4 | WR_ALLOW)
#define WR_D_EINTR		(4  * 4 | WR_DENY)
/* EIO, 5,  Input/output error */
#define WR_IOERR  		(5  * 4)
#define WR_A_IOERR		(5  * 4 | WR_ALLOW)
#define WR_D_IOERR		(5  * 4 | WR_DENY)
/* ENXIO, 6,  Device not configured */
#define WR_ENXIO  		(6  * 4)
#define WR_A_ENXIO		(6  * 4 | WR_ALLOW)
#define WR_D_ENXIO		(6  * 4 | WR_DENY)
/* E2BIG, 7,  Argument list too long */
#define WR_ARG2LONG  		(7  * 4)
#define WR_A_ARG2LONG		(7  * 4 | WR_ALLOW)
#define WR_D_ARG2LONG		(7  * 4 | WR_DENY)
/* ENOEXEC, 8,  Exec format error */
#define WR_ENOEXEC  		(8  * 4)
#define WR_A_ENOEXEC		(8  * 4 | WR_ALLOW)
#define WR_D_ENOEXEC		(8  * 4 | WR_DENY)
/* EBADF, 9,  Bad file descriptor */
#define WR_BADFD  		(9  * 4)
#define WR_A_BADFD		(9  * 4 | WR_ALLOW)
#define WR_D_BADFD		(9  * 4 | WR_DENY)
/* ECHILD, 10,  No child processes */
#define WR_NOCHILD  		(10  * 4)
#define WR_A_NOCHILD		(10  * 4 | WR_ALLOW)
#define WR_D_NOCHILD		(10  * 4 | WR_DENY)
/* EDEADLK, 11,  Resource deadlock avoided */
#define WR_EDEADLK  		(11  * 4)
#define WR_A_EDEADLK		(11  * 4 | WR_ALLOW)
#define WR_D_EDEADLK		(11  * 4 | WR_DENY)
/* ENOMEM, 12,  Cannot allocate memory */
#define WR_ENOMEM  		(12  * 4)
#define WR_A_ENOMEM		(12  * 4 | WR_ALLOW)
#define WR_D_ENOMEM		(12  * 4 | WR_DENY)
/* EACCES, 13,  Permission denied */
#define WR_BADACCES  		(13  * 4)
#define WR_A_BADACCES		(13  * 4 | WR_ALLOW)
#define WR_D_BADACCES		(13  * 4 | WR_DENY)
/* EFAULT, 14,  Bad address */
#define WR_EFAULT  		(14  * 4)
#define WR_A_EFAULT		(14  * 4 | WR_ALLOW)
#define WR_D_EFAULT		(14  * 4 | WR_DENY)
/* ENOTBLK, 15,  Block device required */
#define WR_ENOTBLK  		(15  * 4)
#define WR_A_ENOTBLK		(15  * 4 | WR_ALLOW)
#define WR_D_ENOTBLK		(15  * 4 | WR_DENY)
/* EBUSY, 16,  Device busy */
#define WR_EBUSY  		(16  * 4)
#define WR_A_EBUSY		(16  * 4 | WR_ALLOW)
#define WR_D_EBUSY		(16  * 4 | WR_DENY)
/* EEXIST, 17,  File exists */
#define WR_FILEEXIST  		(17  * 4)
#define WR_A_FILEEXIST		(17  * 4 | WR_ALLOW)
#define WR_D_FILEEXIST		(17  * 4 | WR_DENY)
/* EXDEV, 18,  Cross-device link */
#define WR_EXDEV  		(18  * 4)
#define WR_A_EXDEV		(18  * 4 | WR_ALLOW)
#define WR_D_EXDEV		(18  * 4 | WR_DENY)
/* ENODEV, 19,  Operation not supported by device */
#define WR_ENODEV  		(19  * 4)
#define WR_A_ENODEV		(19  * 4 | WR_ALLOW)
#define WR_D_ENODEV		(19  * 4 | WR_DENY)
/* ENOTDIR, 20,  Not a directory */
#define WR_ENOTDIR  		(20  * 4)
#define WR_A_ENOTDIR		(20  * 4 | WR_ALLOW)
#define WR_D_ENOTDIR		(20  * 4 | WR_DENY)
/* EISDIR, 21,  Is a directory */
#define WR_EISDIR  		(21  * 4)
#define WR_A_EISDIR		(21  * 4 | WR_ALLOW)
#define WR_D_EISDIR		(21  * 4 | WR_DENY)
/* EINVAL, 22,  Invalid argument */
#define WR_BADARG  		(22  * 4)
#define WR_A_BADARG		(22  * 4 | WR_ALLOW)
#define WR_D_BADARG		(22  * 4 | WR_DENY)
/* ENFILE, 23,  Too many open files in system */
#define WR_ENFILE  		(23  * 4)
#define WR_A_ENFILE		(23  * 4 | WR_ALLOW)
#define WR_D_ENFILE		(23  * 4 | WR_DENY)
/* EMFILE, 24,  Too many open files */
#define WR_EMFILE  		(24  * 4)
#define WR_A_EMFILE		(24  * 4 | WR_ALLOW)
#define WR_D_EMFILE		(24  * 4 | WR_DENY)
/* ENOTTY, 25,  Inappropriate ioctl for device */
#define WR_ENOTTY  		(25  * 4)
#define WR_A_ENOTTY		(25  * 4 | WR_ALLOW)
#define WR_D_ENOTTY		(25  * 4 | WR_DENY)
/* ETXTBSY, 26,  Text file busy */
#define WR_FILEBSY  		(26  * 4)
#define WR_A_FILEBSY		(26  * 4 | WR_ALLOW)
#define WR_D_FILEBSY		(26  * 4 | WR_DENY)
/* EFBIG, 27,  File too large */
#define WR_FILE2BIG  		(27  * 4)
#define WR_A_FILE2BIG		(27  * 4 | WR_ALLOW)
#define WR_D_FILE2BIG		(27  * 4 | WR_DENY)
/* ENOSPC, 28,  No space left on device */
#define WR_NOSPC  		(28  * 4)
#define WR_A_NOSPC		(28  * 4 | WR_ALLOW)
#define WR_D_NOSPC		(28  * 4 | WR_DENY)
/* ESPIPE, 29,  Illegal seek */
#define WR_ESPIPE  		(29  * 4)
#define WR_A_ESPIPE		(29  * 4 | WR_ALLOW)
#define WR_D_ESPIPE		(29  * 4 | WR_DENY)
/* EROFS, 30,  Read-only file system */
#define WR_EROFS  		(30  * 4)
#define WR_A_EROFS		(30  * 4 | WR_ALLOW)
#define WR_D_EROFS		(30  * 4 | WR_DENY)
/* EMLINK, 31,  Too many links */
#define WR_EMLINK  		(31  * 4)
#define WR_A_EMLINK		(31  * 4 | WR_ALLOW)
#define WR_D_EMLINK		(31  * 4 | WR_DENY)
/* EPIPE, 32,  Broken pipe */
#define WR_EPIPE  		(32  * 4)
#define WR_A_EPIPE		(32  * 4 | WR_ALLOW)
#define WR_D_EPIPE		(32  * 4 | WR_DENY)
/* EDOM, 33,  Numerical argument out of domain */
#define WR_EDOM  		(33  * 4)
#define WR_A_EDOM		(33  * 4 | WR_ALLOW)
#define WR_D_EDOM		(33  * 4 | WR_DENY)
/* ERANGE, 34,  Result too large */
#define WR_ERANGE  		(34  * 4)
#define WR_A_ERANGE		(34  * 4 | WR_ALLOW)
#define WR_D_ERANGE		(34  * 4 | WR_DENY)
/* EAGAIN, 35,  Resource temporarily unavailable */
#define WR_EAGAIN  		(35  * 4)
#define WR_A_EAGAIN		(35  * 4 | WR_ALLOW)
#define WR_D_EAGAIN		(35  * 4 | WR_DENY)
/* EINPROGRESS, 36,  Operation now in progress */
#define WR_EINPROGRESS  	(36  * 4)
#define WR_A_EINPROGRESS	(36  * 4 | WR_ALLOW)
#define WR_D_EINPROGRESS	(36  * 4 | WR_DENY)
/* EALREADY, 37,  Operation already in progress */
#define WR_EALREADY  		(37  * 4)
#define WR_A_EALREADY		(37  * 4 | WR_ALLOW)
#define WR_D_EALREADY		(37  * 4 | WR_DENY)
/* ENOTSOCK, 38,  Socket operation on non-socket */
#define WR_ENOTSOCK  		(38  * 4)
#define WR_A_ENOTSOCK		(38  * 4 | WR_ALLOW)
#define WR_D_ENOTSOCK		(38  * 4 | WR_DENY)
/* EDESTADDRREQ, 39,  Destination address required */
#define WR_EDESTADDRREQ  	(39  * 4)
#define WR_A_EDESTADDRREQ	(39  * 4 | WR_ALLOW)
#define WR_D_EDESTADDRREQ	(39  * 4 | WR_DENY)
/* EMSGSIZE, 40,  Message too long */
#define WR_EMSGSIZE  		(40  * 4)
#define WR_A_EMSGSIZE		(40  * 4 | WR_ALLOW)
#define WR_D_EMSGSIZE		(40  * 4 | WR_DENY)
/* EPROTOTYPE, 41,  Protocol wrong type for socket */
#define WR_EPROTOTYPE  		(41  * 4)
#define WR_A_EPROTOTYPE		(41  * 4 | WR_ALLOW)
#define WR_D_EPROTOTYPE		(41  * 4 | WR_DENY)
/* ENOPROTOOPT, 42,  Protocol not available */
#define WR_ENOPROTOOPT  	(42  * 4)
#define WR_A_ENOPROTOOPT	(42  * 4 | WR_ALLOW)
#define WR_D_ENOPROTOOPT	(42  * 4 | WR_DENY)
/* EPROTONOSUPPORT, 43,  Protocol not supported */
#define WR_EPROTONOSUPPORT  	(43  * 4)
#define WR_A_EPROTONOSUPPORT	(43  * 4 | WR_ALLOW)
#define WR_D_EPROTONOSUPPORT	(43  * 4 | WR_DENY)
/* ESOCKTNOSUPPORT, 44,  Socket type not supported */
#define WR_ESOCKTNOSUPPORT  	(44  * 4)
#define WR_A_ESOCKTNOSUPPORT	(44  * 4 | WR_ALLOW)
#define WR_D_ESOCKTNOSUPPORT	(44  * 4 | WR_DENY)
/* EOPNOTSUPP, 45,  Operation not supported */
#define WR_EOPNOTSUPP  		(45  * 4)
#define WR_A_EOPNOTSUPP		(45  * 4 | WR_ALLOW)
#define WR_D_EOPNOTSUPP		(45  * 4 | WR_DENY)
/* EPFNOSUPPORT, 46,  Protocol family not supported */
#define WR_EPFNOSUPPORT  	(46  * 4)
#define WR_A_EPFNOSUPPORT	(46  * 4 | WR_ALLOW)
#define WR_D_EPFNOSUPPORT	(46  * 4 | WR_DENY)
/* EAFNOSUPPORT, 47,  Address family not supported by protocol family */
#define WR_EAFNOSUPPORT  	(47  * 4)
#define WR_A_EAFNOSUPPORT	(47  * 4 | WR_ALLOW)
#define WR_D_EAFNOSUPPORT	(47  * 4 | WR_DENY)
/* EADDRINUSE, 48,  Address already in use */
#define WR_EADDRINUSE  		(48  * 4)
#define WR_A_EADDRINUSE		(48  * 4 | WR_ALLOW)
#define WR_D_EADDRINUSE		(48  * 4 | WR_DENY)
/* EADDRNOTAVAIL, 49,  Can't assign requested address */
#define WR_EADDRNOTAVAIL  	(49  * 4)
#define WR_A_EADDRNOTAVAIL	(49  * 4 | WR_ALLOW)
#define WR_D_EADDRNOTAVAIL	(49  * 4 | WR_DENY)
/* ENETDOWN, 50,  Network is down */
#define WR_ENETDOWN  		(50  * 4)
#define WR_A_ENETDOWN		(50  * 4 | WR_ALLOW)
#define WR_D_ENETDOWN		(50  * 4 | WR_DENY)
/* ENETUNREACH, 51,  Network is unreachable */
#define WR_ENETUNREACH  	(51  * 4)
#define WR_A_ENETUNREACH	(51  * 4 | WR_ALLOW)
#define WR_D_ENETUNREACH	(51  * 4 | WR_DENY)
/* ENETRESET, 52,  Network dropped connection on reset */
#define WR_ENETRESET  		(52  * 4)
#define WR_A_ENETRESET		(52  * 4 | WR_ALLOW)
#define WR_D_ENETRESET		(52  * 4 | WR_DENY)
/* ECONNABORTED, 53,  Software caused connection abort */
#define WR_ECONNABORTED  	(53  * 4)
#define WR_A_ECONNABORTED	(53  * 4 | WR_ALLOW)
#define WR_D_ECONNABORTED	(53  * 4 | WR_DENY)
/* ECONNRESET, 54,  Connection reset by peer */
#define WR_ECONNRESET  		(54  * 4)
#define WR_A_ECONNRESET		(54  * 4 | WR_ALLOW)
#define WR_D_ECONNRESET		(54  * 4 | WR_DENY)
/* ENOBUFS, 55,  No buffer space available */
#define WR_ENOBUFS  		(55  * 4)
#define WR_A_ENOBUFS		(55  * 4 | WR_ALLOW)
#define WR_D_ENOBUFS		(55  * 4 | WR_DENY)
/* EISCONN, 56,  Socket is already connected */
#define WR_EISCONN  		(56  * 4)
#define WR_A_EISCONN		(56  * 4 | WR_ALLOW)
#define WR_D_EISCONN		(56  * 4 | WR_DENY)
/* ENOTCONN, 57,  Socket is not connected */
#define WR_ENOTCONN  		(57  * 4)
#define WR_A_ENOTCONN		(57  * 4 | WR_ALLOW)
#define WR_D_ENOTCONN		(57  * 4 | WR_DENY)
/* ESHUTDOWN, 58,  Can't send after socket shutdown */
#define WR_ESHUTDOWN  		(58  * 4)
#define WR_A_ESHUTDOWN		(58  * 4 | WR_ALLOW)
#define WR_D_ESHUTDOWN		(58  * 4 | WR_DENY)
/* ETOOMANYREFS, 59,  Too many references: can't splice */
#define WR_ETOOMANYREFS  	(59  * 4)
#define WR_A_ETOOMANYREFS	(59  * 4 | WR_ALLOW)
#define WR_D_ETOOMANYREFS	(59  * 4 | WR_DENY)
/* ETIMEDOUT, 60,  Operation timed out */
#define WR_ETIMEDOUT  		(60  * 4)
#define WR_A_ETIMEDOUT		(60  * 4 | WR_ALLOW)
#define WR_D_ETIMEDOUT		(60  * 4 | WR_DENY)
/* ECONNREFUSED, 61,  Connection refused */
#define WR_ECONNREFUSED  	(61  * 4)
#define WR_A_ECONNREFUSED	(61  * 4 | WR_ALLOW)
#define WR_D_ECONNREFUSED	(61  * 4 | WR_DENY)
/* ELOOP, 62,  Too many levels of symbolic links */
#define WR_ELOOP  		(62  * 4)
#define WR_A_ELOOP		(62  * 4 | WR_ALLOW)
#define WR_D_ELOOP		(62  * 4 | WR_DENY)
/* ENAMETOOLONG, 63,  File name too long */
#define WR_FNAME2LONG  		(63  * 4)
#define WR_A_FNAME2LONG		(63  * 4 | WR_ALLOW)
#define WR_D_FNAME2LONG		(63  * 4 | WR_DENY)
/* EHOSTDOWN, 64,  Host is down */
#define WR_EHOSTDOWN  		(64  * 4)
#define WR_A_EHOSTDOWN		(64  * 4 | WR_ALLOW)
#define WR_D_EHOSTDOWN		(64  * 4 | WR_DENY)
/* EHOSTUNREACH, 65,  No route to host */
#define WR_EHOSTUNREACH  	(65  * 4)
#define WR_A_EHOSTUNREACH	(65  * 4 | WR_ALLOW)
#define WR_D_EHOSTUNREACH	(65  * 4 | WR_DENY)
/* ENOTEMPTY, 66,  Directory not empty */
#define WR_ENOTEMPTY  		(66  * 4)
#define WR_A_ENOTEMPTY		(66  * 4 | WR_ALLOW)
#define WR_D_ENOTEMPTY		(66  * 4 | WR_DENY)
/* EPROCLIM, 67,  Too many processes */
#define WR_2MANYPROC  		(67  * 4)
#define WR_A_2MANYPROC		(67  * 4 | WR_ALLOW)
#define WR_D_2MANYPROC		(67  * 4 | WR_DENY)
/* EUSERS, 68,  Too many users */
#define WR_2MANYUSER  		(68  * 4)
#define WR_A_2MANYUSER		(68  * 4 | WR_ALLOW)
#define WR_D_2MANYUSER		(68  * 4 | WR_DENY)
/* EDQUOT, 69,  Disc quota exceeded */
#define WR_EDQUOT  		(69  * 4)
#define WR_A_EDQUOT		(69  * 4 | WR_ALLOW)
#define WR_D_EDQUOT		(69  * 4 | WR_DENY)
/* ESTALE, 70,  Stale NFS file handle */
#define WR_ESTALE  		(70  * 4)
#define WR_A_ESTALE		(70  * 4 | WR_ALLOW)
#define WR_D_ESTALE		(70  * 4 | WR_DENY)
/* EREMOTE, 71,  Too many levels of remote in path */
#define WR_EREMOTE  		(71  * 4)
#define WR_A_EREMOTE		(71  * 4 | WR_ALLOW)
#define WR_D_EREMOTE		(71  * 4 | WR_DENY)
/* EBADRPC, 72,  RPC struct is bad */
#define WR_EBADRPC  		(72  * 4)
#define WR_A_EBADRPC		(72  * 4 | WR_ALLOW)
#define WR_D_EBADRPC		(72  * 4 | WR_DENY)
/* ERPCMISMATCH, 73,  RPC version wrong */
#define WR_ERPCMISMATCH  	(73  * 4)
#define WR_A_ERPCMISMATCH	(73  * 4 | WR_ALLOW)
#define WR_D_ERPCMISMATCH	(73  * 4 | WR_DENY)
/* EPROGUNAVAIL, 74,  RPC prog. not avail */
#define WR_EPROGUNAVAIL  	(74  * 4)
#define WR_A_EPROGUNAVAIL	(74  * 4 | WR_ALLOW)
#define WR_D_EPROGUNAVAIL	(74  * 4 | WR_DENY)
/* EPROGMISMATCH, 75,  Program version wrong */
#define WR_EPROGMISMATCH  	(75  * 4)
#define WR_A_EPROGMISMATCH	(75  * 4 | WR_ALLOW)
#define WR_D_EPROGMISMATCH	(75  * 4 | WR_DENY)
/* EPROCUNAVAIL, 76,  Bad procedure for program */
#define WR_EPROCUNAVAIL  	(76  * 4)
#define WR_A_EPROCUNAVAIL	(76  * 4 | WR_ALLOW)
#define WR_D_EPROCUNAVAIL	(76  * 4 | WR_DENY)
/* ENOLCK, 77,  No locks available */
#define WR_ENOLCK  		(77  * 4)
#define WR_A_ENOLCK		(77  * 4 | WR_ALLOW)
#define WR_D_ENOLCK		(77  * 4 | WR_DENY)
/* ENOSYS, 78,  Function not implemented */
#define WR_NOSYSCALL  		(78  * 4)
#define WR_A_NOSYSCALL		(78  * 4 | WR_ALLOW)
#define WR_D_NOSYSCALL		(78  * 4 | WR_DENY)
/* EFTYPE, 79,  Inappropriate file type or format */
#define WR_EFTYPE  		(79  * 4)
#define WR_A_EFTYPE		(79  * 4 | WR_ALLOW)
#define WR_D_EFTYPE		(79  * 4 | WR_DENY)
/* EAUTH, 80,  Authentication error */
#define WR_EAUTH  		(80  * 4)
#define WR_A_EAUTH		(80  * 4 | WR_ALLOW)
#define WR_D_EAUTH		(80  * 4 | WR_DENY)
/* ENEEDAUTH, 81,  Need authenticator */
#define WR_ENEEDAUTH  		(81  * 4)
#define WR_A_ENEEDAUTH		(81  * 4 | WR_ALLOW)
#define WR_D_ENEEDAUTH		(81  * 4 | WR_DENY)
/* ELAST, 81,  Must be equal largest errno */
#define WR_ELAST  		(81  * 4)
#define WR_A_ELAST		(81  * 4 | WR_ALLOW)
#define WR_D_ELAST		(81  * 4 | WR_DENY)
#define	WR_SUCC			(100 * 4)	/* SUCC, 100, Successful */
#define	WR_FAIL			(101 * 4)	/* FAIL, 101, Fail */
/*
 * $Id: simple.c,v 1.6 2001/02/23 19:02:02 stevek Exp $
 *
 * Author: Doug Kilpatrick and Lee Badger
 *
 * Purpose: contains the database representation structures.
 */
/*
 * Preliminary WQL data structures and accessing functions.
 *
 * In one contiguous region of memory, the DB is represented in the
 * most simple way: a list of tables; for each table, a list of rows;
 * for each row, a list of values.  All lists are NULL-terminated;
 * order within a list matters.  Each row of a table is an ordered
 * list of wql_val structures, which _correspond_ to the table's
 * ordered list of wql_col structures.  The entire DB can be searched
 * by starting at the first (well-known) table pointer; optimization
 * for speed will come later.  Of course, it's slightly more
 * complicated than that, but the simplest view is:
 */

#ifndef WR_SUPPRESS_WQL

#ifndef __WSS_DB_H
#define __WSS_DB_H 1

#ifdef WR_KERNEL_MODULE
#ifndef __WSI_MUTEX_H
#include <wsi_mutex.h>
#endif
#endif /* WR_KERNEL_MODULE */


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef u_long wss_db_handle_t;

#ifdef WR_KERNEL_MODULE
typedef struct wql_table_opaque
{
   /* Protects everything except "next".  Next is protected by
    * a mutex internal to wss_db.
    */
   WSI_MUTEX_PROTECT(table);
}
wqtabopaque, *wqtabopaque_t;
#endif /* WR_KERNEL_MODULE */

typedef struct wql_table
{
   /* In order to not polute user-space with kernel-space constructs, we
    * use an opaque pointer which contains the appropriate lock for the
    * table (see struct wql_table_opaque above)
    */
   void *opaque;
   char *name;
   int col_count;
   int max_rows;                 /* TBD */
   int row_count;                /* TBD */
   struct wql_col_def *col_defs; /* defines this table's format */
   struct wql_row *rows;
   struct wql_row *tail;         /* TBD */
   struct wql_table *next;
}
wqtab, *wqtab_t;

typedef enum wql_dtype { none_t, char_t, int_t, float_t } wql_dtype;
typedef struct wql_col_def
{
   char           *name;
   enum wql_dtype  dtype;
   int             max_len;
}
wqcd, *wqcd_t;

typedef struct wql_row
{
   struct wql_val *vals;
   struct wql_row *next;
}
wqrow, *wqrow_t;

typedef struct wql_val
{
   union
   {
      char *cval;
      int   ival;
      float fval;
   }
   v;
}
wqval, *wqval_t;

/* holds data returned from select */
/* This is a structure instead of a pointer to a structure so that it does
 * not need an extra creation step.
 */
struct wql_dbrow
{
   struct wql_table  results;
   struct wql_row   *cur;
};

/*
 * Definitions to support select/update/delete expressions.  An
 * expression is essentially a tree of structures.  Code in
 * select/update/delete traverses the tree to evaluate rows to act
 * upon.  It's a bit messy; will think about how to pretty it up
 * later.
 */

enum sxop { gt_op, gte_op, lt_op, lte_op, ne_op, eq_op, true_op };
enum sxarg { literal, field, variable };
typedef struct sxval
{
   union
   {
      union literal
      {
         float       fval;
         int         ival;
         const char *cval;
      }
      l;
      struct field
      {
         int         column;  /* By index, starting at 0 */
      }
      f;
      struct variable
      {
         const void *ptr;
      }
      v;
   }
   v;
   enum wql_dtype dtype;
   enum sxarg which;
   char owned;                /* boolean */
}
sxval, *sxval_t;

/*
 * Represents a relational expression such as ".count = 5" in a select
 * expression.
 */
typedef struct term_exp
{
   sxval_t left;
   sxval_t right;
   enum sxop op;
}
tx, *tx_t;

/* Hack.  not_op negates the left side, and ignores the right */
enum logop { and_op, or_op, not_op };
typedef struct nterm_exp
{
   struct sx_node *left;
   struct sx_node *right;
   enum logop      logop;
}
ntx, *ntx_t;


/*
 * A select (or delete or update) expression is a tree of these.  The tree
 * is evaluated at runtime to identify rows of a table that should be
 * returned/deleted/updated.
 */
enum nodekind { term, nonterm };
typedef struct sx_node
{
   union
   {
      tx_t  exp;
      ntx_t children;
   }
   un;                  /* Hideous hack to get around hideous brokenness. */
   enum nodekind which;
}
wqsx, *wqsx_t;

/*
 * Represents an update_set_list: a list of columns to set in an update
 * expression.
 */
typedef struct update_set_list
{
   struct sxval             *left;
   struct update_expression *right;
   struct update_set_list   *next;
}
wqusl, *wqusl_t;

/*
 * Represents a update_expression: a numeric value calculated from this
 * tree structure.
 */
enum numop {multiplication, division, mod, addition, subtraction, negation};
typedef struct update_expression
{
   union
   {
      sxval_t terminal;
      struct
      {
         struct update_expression *left;
         struct update_expression *right;
         enum numop                numop;
      }
      e;
   }
   w;
   enum nodekind which;
}
wquexp, *wquexp_t;


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#ifdef WR_KERNEL_MODULE
#define WQL_TABLE_OPAQUE(t) ((wqtabopaque_t)rel2abs((t)->opaque))
#endif /* WR_KERNEL_MODULE */


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*
 * WR_SUPPRESS_FUNCTION_DECL is turned on by libraries.  This prevents
 * wrappers from attempting to reference symbols like wss_create, which
 * doesn't exist as far as the linker is concerned.
 */
#ifndef WR_SUPPRESS_FUNCTION_DECL

#if 0
int wr_dbrow_free                        __P(( struct wql_dbrow *rows ));
#endif

void wss_dbck                            __P(( void ));
void wss_db_init                         __P(( void ));

#if USER_MODE_TEST
wqtab_t wss_tables                       __P(( void ));
#endif
unsigned long wss_db_find_table_by_names __P(( char *wrapper_name, int id,
                                               char *table_name ));
wqtab_t wss_db_find_table_by_handle_r    __P(( u_long handle ));

int wss_create                           __P(( char *wrapper_name,
                                               int instance_id,
                                               char *table_name,
                                               int free_col_defs,
                                               struct wql_col_def *col_defs,
                                               int col_count,
                                               u_long *table_id ));
int wss_drop                             __P(( u_long table_id ));
int wss_insert                           __P(( u_long table_id, int free_vals,
                                               wqrow_t vals ));
int wss_select                           __P(( u_long table_id, int free_scrit,
                                               wqsx_t scrit, int take_first,
                                               int order_by, int desc,
                                               wqtab_t out_table ));
int wss_delete                           __P(( u_long table_id, int free_scrit,
                                               wqsx_t scrit, int take_first ));
int wss_update                           __P(( u_long table_id,
                                               int free_scripts, wqsx_t scrit,
                                               int take_first,
                                               wqusl_t set_script ));

/*
 * Support functions.
 * Used by the compilier
 */

wqcd_t   wss_db_mcol_def                 __P(( int count, ... ));
wqrow_t  wss_db_mrow                     __P(( wqval_t vals, wqrow_t next ));
wqsx_t   wss_db_mwqsx_and                __P(( wqsx_t left, wqsx_t right ));
wqsx_t   wss_db_mwqsx_or                 __P(( wqsx_t left, wqsx_t right ));
wqsx_t   wss_db_mwqsx_terminal           __P(( tx_t terminal ));
wqsx_t   wss_db_mwqsx_not                __P(( wqsx_t expression ));
tx_t     wss_db_mtx                      __P(( sxval_t left, sxval_t right,
                                               enum sxop op ));
sxval_t  wss_db_msxval_field             __P(( int column ));
sxval_t  wss_db_msxval_lit_int           __P(( int i ));
sxval_t  wss_db_msxval_lit_float         __P(( double d ));
sxval_t  wss_db_msxval_lit_string        __P(( const char *string ));
#if 0
int      wss_db_dbrow_start              __P(( struct wql_dbrow *dbrp ));
int      wss_db_dbrow_next               __P(( struct wql_dbrow *dbrp ));
int      wss_db_dbrow_isempty            __P(( struct wql_dbrow *dbrp ));
#endif
void     wss_db_dbrow_finish             __P(( struct wql_dbrow *dbrp ));
wqusl_t  wss_db_musl                     __P(( sxval_t left, wquexp_t right,
                                               wqusl_t next ));
wquexp_t wss_db_muexp                    __P(( wquexp_t left, wquexp_t right,
                                               enum numop numop ));
wquexp_t wss_db_muexp_term               __P(( sxval_t value ));
wqval_t  wss_db_mvals                    __P(( int count, ... ));
void     wss_db_free_criteria            __P(( wqsx_t expr ));
void     wss_db_free_wqusl               __P(( wqusl_t list ));

/*
 * Table locking functions
 */
#ifdef WR_KERNEL_MODULE
void     wss_db_lock_table               __P(( wqtab_t ));
void     wss_db_unlock_table             __P(( wqtab_t ));
#endif /* WR_KERNEL_MODULE */


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/
#ifdef WR_KERNEL_MODULE
#define  wss_db_lock_table(t) WSI_MUTEX_ENTER(WQL_TABLE_OPAQUE(t)->table)
#define  wss_db_unlock_table(t) WSI_MUTEX_EXIT(WQL_TABLE_OPAQUE(t)->table)
#endif /* WR_KERNEL_MODULE */

#endif /* WR_SUPPRESS_FUNCTION_DECL */
#endif /* __WSS_DB_H */
#endif /* WR_SUPPRESS_WQL */



/* global variables used in string matching and replacement */
char *gstr_beg, *gstr_end, *gstr;

int myversion = 1;
int myhandle = 0;

/*
 * Function declarations.
 */
int wr_wrapper_loaded(char *name);
int wr_register_wrapper(int handle, char *name, void *pre[],
                           void *post[], void *activate,
			   void *deactivate, void *duplicate,
			   void *newprivdata);
int wr_rm_wrapper(int handle);
int wr_at_unload(void *uf, int handle);
int wr_rm_at_unload(int handle);
#if 0 /* move to libwr.h, Calvin */
int wr_strcmp(const char *s1, const char *s2);
char *wr_strcpy(const char *dst, const char *src);
int wr_strncmp(const char *s1, const char *s2, int len);
int wr_strlen(const char *str);
char *wr_abspath(char *path);
char *wr_strchr(const char *str, int c);
char *wr_strrchr(const char *str, int c);
#if defined(DEBUG)
char *wr_strdup_debug(const char *str, char *file, int line);
#define wr_strdup(str) (wr_strdup_debug((str), __FILE__, __LINE__))
#else
char *wr_strdup(const char *str);
#endif
#if defined(DEBUG)
void *wr_malloc_debug(size_t size, char *file, int line);
#define wr_malloc(size) (wr_malloc_debug((size), __FILE__, __LINE__))
#else
void *wr_malloc(size_t size);
#endif
void wr_free(void *ptr);
void *wr_curproc_malloc(size_t size);
void wr_curproc_free(void *ptr, size_t size);
char *wr_curproc_strdup(const char *str);
#endif
int wr_arg_replace(struct wss_proc *wrp, void *uap[], void *new, int index, size_t size);
int wr_arg_copy(void *uap[], void *new, int index, size_t size);

#if 0 /* move to libwr.h, Calvin */
void wr_bcopy(const void *src, void *dst, size_t len);
void wr_bcopy(const void *src, void *dst, size_t len);
int wr_printf(const char *, ...);
#endif
long wr_get_curtime(void);
char *wr_get_osname(void);
int wr_get_osvers(void);
int wr_get_wrvers(void);
const char *wr_syscall_name(int call);
int wr_match(char *src, char *pat, char **beg, char **end);
int wr_substitute(char **src, char *txt, char *sub);
int wr_copyin(void* uaddr, void *kaddr, size_t len);
int wr_copyout(void* kaddr, void *uaddr, size_t len);
int IMPORT wr_copyinstr(void* uaddr, void *kaddr, size_t len, unsigned int *done);
void *wr_dupin_ntermlist(void *uaddr);
void *wr_dupout_ntermlist(void *kaddr);
void *wr_dupin(void *uaddr, size_t size);
void *wr_dupout(void *kaddr, size_t size);
int wr_replace_ntermlist(struct wss_proc *wrp, void *uap[], void *new, int index);
void wr_free_ntermlist(void *kaddr);

void wr_mutex_create(wsi_mutex_t *mutex, const char *name);
void wr_mutex_destroy(wsi_mutex_t *mutex);
void wr_mutex_enter(wsi_mutex_t *mutex);
void wr_mutex_exit(wsi_mutex_t *mutex);

#if 0
string wr_str_new(const char *s);
string wr_str_add(string s1, string s2);
int    wr_str_equ(string s1, string s2);
string wr_str_assign (string s, string rhs);
#endif

/* Define some of the wdl environment variables to data in the proc
structure.  Other environment variables can't be created this way. */

#if defined(WR_SOLARIS)
#define _descriptors	p->p_fd->fd_nfiles
#define _niceval	p->p_nice
#define _priority	p->p_priority
#define _pid		p->p_pid
#define _pgid		(p->p_parent?p->parent->p_cred->cr_rgid:-1)
#define _pegid		(p->p_parent?p->parent->p_cred->cr_gid:-1)
#define _puid           (p->p_parent?p->parent->p_cred->cr_ruid:-1)
#define _peuid           (p->p_parent?p->parent->p_cred->cr_uid:-1)
#define _ppid  		(p->p_ppid)
#define _progname  	wrp->progname
#define _uid  		p->p_cred->cr_ruid
#define _euid  		p->p_cred->cr_uid
#define _gid  		p->p_cred->cr_rgid
#define _egid  		p->p_cred->cr_gid
#elif defined(WR_FREEBSD)
#define _descriptors	p->p_fd->fd_nfiles
#define _niceval	p->p_nice
#define _priority	p->p_priority
#define _pid		p->p_pid
#define _pgid		(p->p_pptr?p->p_pptr->p_cred->p_rgid:-1)
#define _pegid  	(p->p_pptr?p->p_pptr->p_cred->p_svgid:-1)
#define _puid		(p->p_pptr?p->p_pptr->p_cred->p_ruid:-1)
#define _peuid  	(p->p_pptr?p->p_pptr->p_cred->p_svuid:-1)
#define _ppid  		(p->p_pptr?p->p_pptr->p_pid:0)
#define _progname  	wrp->progname
#define _uid  		p->p_cred->p_ruid
#define _euid  		p->p_cred->p_svuid
#define _gid  		p->p_cred->p_rgid
#define _egid  		p->p_cred->p_svgid
#elif defined(WR_WINNT)
#define _pid		wrp->p->p_pid
#define _ppid		0
#else
#error unsupported target.
#endif

/* Using default library */
int  wr_strcmp (const  char  * s1,const  char  * s2);
int  wr_strncmp (const  char  * s1,const  char  * s2,int  len);
int  wr_strlen (const  char  * str);
char  * wr_abspath (char  * path);
char  * wr_strcpy (const  char  * dst,const  char  * src);
char  * wr_strchr (const  char  * str,int  c);
char  * wr_strrchr (const  char  * str,int  c);
char  * wr_strcat (char  * s,const  char  * append);
char  * wr_strncat (char  * s,const  char  * append,size_t  count);
void  * wr_bcopy (const  void  * src,void  * dst,size_t  len);
void  * wr_bzero (const  void  * src,size_t  len);
int  IMPORT wr_printf (const  char *, ...);
int  wr_sprintf (char  * s,const  char *, ...);
char  * wr_strdup (const  char  * str);
void  IMPORT * wr_malloc (size_t  size);
void  IMPORT wr_free (void  * ptr);
void  * wr_curproc_malloc (size_t  size);
void  wr_curproc_free (void  * ptr,size_t  size);
char  * wr_curproc_strdup (const  char  * str);
const char *wr_syscall_name(int call);
int  wrq_newq ( );
int  wrq_insert (int  qid,char  * buf,int  size);
int  wr_fdtype (unsigned  int );
int  wr_fdfflag (unsigned  int );
char  * wr_get_family (const  struct sockaddr  * saddr,int  len);
char  * wr_get_addr (const  struct sockaddr  * saddr,int  len);
int  wr_getsockname (int ,struct sockaddr  * saddr,int  * len);
int  wr_getpeername (int ,struct sockaddr  * saddr,int  * len);
/* end library */


#if 0
static string wr_str_new(const char *s);
static string wr_str_add(string s1, string s2);
static int   wr_str_equ(string s1, string s2);
static string wr_str_assign(string s, string rhs);


static string wr_str_assign(string s, string rhs)

{
     int len = wr_strlen(rhs) + 1;
     s = (string) wr_malloc(len);
     wr_strcpy(s, rhs);
     return s;
}

static string wr_str_add(string s1, string s2)

{
     int len = wr_strlen(s1) + wr_strlen(s2) + 1;  
     int bufsize = len >> 5 == 0 ? len : ((len >> 5) + 1) << 5;
     string ret;
     ret = (string) wr_malloc(bufsize);
     wr_strcpy(ret, s1);
     wr_strcat(ret, s2);
     return ret;
}

static int wr_str_equ(string s1, string s2)
{
     return !wr_strcmp(s1, s2);
}
#endif /* 0 */

static databuf_t buf_new()
{
     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));
     buf->size = 0;
     buf->max = 0;
     buf->buf = NULL;
     return buf;
}

#if 0
static databuf_t buf_newstr(string s)
{
     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));
     buf->size = wr_strlen(s) + 1;
     buf->max = ((buf->size >> 5) + 1) << 5;
     buf->buf = (char *) wr_malloc(buf->max);
     wr_strncpy(buf->buf, s, buf->max);
     // if (buf->buf == NULL) handle_error : TBD
     return buf;
}
#endif

static databuf_t buf_newdata(char *data, int size)
{
     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));
     buf->size = size;
     buf->max = ((buf->size >> 5) + 1) << 5;
     buf->buf = (char *) wr_malloc(buf->max);
     wr_copyin(data, buf->buf, size);
     return buf;
}

static void buf_del(databuf_t buf)

{
     if (buf->buf != NULL) wr_free(buf->buf);
     wr_free(buf);
}

static int buf_replace_data(databuf_t buf, const char *data, int len)

{
     if (len > buf->max) {
          wr_free(buf->buf);
          buf->max = ((buf->size >> 5) + 1) << 5;
          buf->buf = (char *) wr_malloc(buf->max);
     }
     buf->size = len;
     wr_bcopy(data, buf->buf, len);
     return 1;
}

static int buf_replace_str(databuf_t buf, string s)

{
     int len = wr_strlen(s) + 1;
     if (len > buf->max) {
          wr_free(buf->buf);
          buf->max = ((buf->size >> 5) + 1) << 5;
          buf->buf = (char *) wr_malloc(buf->max);
     }
     buf->size = len;
     wr_strncpy(buf->buf, s, len);
     return 1;
}

static int buf_replace(databuf_t buf, databuf_t buf1)

{
     return buf_replace_data(buf, buf1->buf, buf1->size);
}

static int buf_size(databuf_t buf) 
{
     return buf->size;
}

static char *buf_getdataptr(databuf_t buf, int i) 
{
     if (buf->size > i) 
          return &(buf->buf[i]);
     else
          return NULL;   // TBD handle error
}

#ifndef WR_SUPPRESS_WQL

typedef struct wql_dbrow dbrow;

wqcd_t wr_mcol_def(int count, ...);
wqrow_t wr_mrow(wqval_t vals, wqrow_t next);
wqsx_t wr_mwqsx_and(wqsx_t left, wqsx_t right);
wqsx_t wr_mwqsx_or(wqsx_t left, wqsx_t right);
wqsx_t wr_mwqsx_terminal(tx_t terminal);
wqsx_t wr_mwqsx_not(wqsx_t expression);
tx_t wr_mtx(sxval_t left, sxval_t right, enum sxop op);
sxval_t wr_msxval_field(int column);
sxval_t wr_msxval_lit_int(int i);
sxval_t wr_msxval_lit_float(double d);
sxval_t wr_msxval_lit_string(const char *string);
void wr_dbrow_finish(struct wql_dbrow *dbrp);
wqusl_t wr_musl(sxval_t left, wquexp_t right, wqusl_t next);
wquexp_t wr_muexp(wquexp_t left, wquexp_t right, enum numop numop);
wquexp_t wr_muexp_term(sxval_t value);
wqval_t wr_mvals(int count, ...);
void wr_free_criteria(wqsx_t expr);
void wr_free_wqusl(wqusl_t list);

unsigned long
wql_find_table_by_names(char *wrapper_name, int id, char *table_name);
wqtab_t wss_db_find_table_by_handle_r( unsigned long handle );

int wql_create( char *wrapper_name,
		int instance_id,
		char *table_name,
		int free_col_defs,
		struct wql_col_def *col_defs,
                int col_count,
                unsigned long *table_id);
int wql_drop(	unsigned long table_id );
int wql_insert(	unsigned long table_id,
		int free_vals,
		wqrow_t vals);
int wql_select(	unsigned long table_id,
		int free_scrit,
		wqsx_t scrit,
                int take_first,
		int order_by,
		int desc,
                wqtab_t out_table);
int wql_delete(	unsigned long table_id,
		int free_scrit,
		wqsx_t scrit,
                int take_first);
int wql_update(	unsigned long table_id,
		int free_scripts,
                wqsx_t scrit,
                int take_first,
                wqusl_t set_script);

/*
 * See wss_db.c for the reference implimentations.
 */
#define dbrow_start(dbrp) ((((dbrp)->cur) = (dbrp)->results.rows),((dbrp) != NULL && (dbrp)->cur != NULL))

#define dbrow_next(dbrp) (((dbrp)->cur = (dbrp)->cur->next) != NULL)

#define dbrow_isempty(dbrp) ((dbrp)->results.rows == NULL)

#endif /* WR_SUPPRESS_WQL */

    
/* Mutex stuff */

#ifdef WR_THREADED
#define WR_MUTEX_PROTECT(var) WSI_MUTEX_PROTECT(var)

#define WR_MUTEX_INIT(var) wr_mutex_init( &(var ## _lock), #var ## "lock")
#define WR_MUTEX_DESTROY(var) wr_mutex_destroy( &(var ## _lock) )
#define WR_MUTEX_ENTER(var) wr_mutex_enter( &(var ## _lock) )
#define WR_MUTEX_EXIT(var) wr_mutex_exit( &(var ## _lock) )

#define WR_WIV_MUTEX_INIT(var)     wr_mutex_init(&WSS_GET_WIV_VAR(var ## _lock), #var ## "lock")
#define WR_WIV_MUTEX_DESTROY(var)     wr_mutex_destroy(&WSS_GET_WIV_VAR(var ## _lock) )
#define WR_WIV_MUTEX_ENTER(var)     wr_mutex_enter(&WSS_GET_WIV_VAR(var ## _lock) )
#define WR_WIV_MUTEX_EXIT(var)     wr_mutex_exit(&WSS_GET_WIV_VAR(var ## _lock) )
#else /* WR_THREADED */
#define WR_MUTEX_PROTECT(var)
#define WR_MUTEX_INIT(var)
#define WR_MUTEX_DESTROY(var)
#define WR_MUTEX_ENTER(var)
#define WR_MUTEX_EXIT(var)

#define WR_WIV_MUTEX_INIT(var)
#define WR_WIV_MUTEX_DESTROY(var)
#define WR_WIV_MUTEX_ENTER(var)
#define WR_WIV_MUTEX_EXIT(var)
#endif /* WR_THREADED */

/* Wrappers referenced by this activation criteron. */                 
char *imp_list[] = {
	"life",
	NULL
};

char EXPORT **get_imp_list()
{
    return imp_list;
}

/*									
 * Decision function, called by WSS at exec.  Assumes list is		
 * initialized; adds elements to it if wrappers should be		
 * activated								
 */									
void EXPORT activate(struct wss_proc *wrproc)					
{
  if ( (!_tcsicmp(wrproc->p->user_name, _TEXT(ACTUSER) ) && 
        wss_proc_add_wrapper(wrproc,"foo", myhandle) || 1) )
		/* do nothing */ ;
}

static char *myname = "(user==" ACTUSER ") ==> life";


static char *WRLINK_MOD_NAME = "(user==" ACTUSER ") ==> life";

char EXPORT *get_mod_name()
{
  return WRLINK_MOD_NAME;
}

static char *WRLINK_MOD_TYPE = "activation";

char EXPORT *get_mod_type()
{
  return WRLINK_MOD_TYPE;
}

static char *WRLINK_MOD_DEPENDENCIES = "";

char EXPORT *get_mod_dependencies()
{
  return WRLINK_MOD_DEPENDENCIES;
}

