/* definition of constant in AKS */

#define AKS_HEADERTK_ID  3
#define AKS_PROCTK_ID    4
#define AKS_OBJTK_ID     5
#define AKS_SOCKTK_ID    6
#define AKS_PARAMTK_ID   7


/* 
 * This file contains the definitions of the tokens of the Agile Audit Module 
 */

typedef struct {
	int16_t 		tid;			/* token ID */
	int16_t		evid;		/* event ID */
	int32_t		len;			/* len of the audit record */
     int16_t     	errno;         /* error number: 0 means successful */
     int16_t     	flag;          /* pre:0 or post:1 */
     int32_t        time;          /* event timestamp, no of sec since Jan 1,1970 */
     int32_t        usec;          /* macrosecond */
} aks_headertk;

typedef struct {
	int16_t		tid;		/* token ID 				*/
     int16_t       	thid;     /* thread Id                  */
     int32_t       	pid;      /* process Id                 */
     int32_t       	ppid;     /* parent process Id          */
     int32_t       	ruid;     /* real user Id               */
     int32_t       	euid;     /* effective user Id          */
     int32_t       	rgid;     /* real group Id              */
     int32_t       	egid;     /* effective group Id         */
	char			prog[0];	/* name of the program in execution */
} aks_proctk;

typedef struct {
	int16_t		tid;			/* token ID					*/
     int16_t       	nlink;         /* number of hard links            */
	int32_t       	mode;          /* inode protection mode           */
     int32_t       	ouid;          /* user ID of the file's owner     */
     int32_t       	ogid;          /* group ID of the file's group    */
     int32_t      	ino;           /* inode's number                  */
     int32_t      	atime;         /* time of last access             */
     int32_t      	size;          /* file size, in bytes             */
     int32_t      	gno;           /* file generation number          */
     int32_t      	rdev;          /* device ID (if file is character */
     char      	path[0];       /* absolute path                   */
} aks_objtk;

typedef struct {
	int16_t 	tid;			/* token ID 			*/
	int16_t	len;			/* length of the str 	*/
} aks_pathtk;

typedef struct {
	int16_t 		tid;			/* token ID 			*/
     int16_t       	type;          /* type, TCP/UCP/UNIX    */
     int32_t       	afamily;       /* address family        */
     int32_t       	lport;         /* local port            */
     int32_t       	rport;         /* remote port           */
     int32_t		laddr;         /* local inet address    */
     int32_t		raddr;         /* remote inet address   */
	char 		name[0];		/* name				*/
} aks_socktk;

typedef void *ostrbuf_t;
extern ostrbuf_t *gbufp;

int write_headertk(ostrbuf_t *, short, short, short, int, int);
int write_proctk(ostrbuf_t *, int, int, int, int, int, int, int, char *);
int write_objtk(ostrbuf_t *, int, int, int, int, int, int, int, int, 
				int, char *);
int write_socktk(ostrbuf_t *, int, int, int, int, int, int);
int write_paramtk();
int send_arecord(int, ostrbuf_t *buf);



