/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: bsd.mod,v 1.16 2001/04/05 18:40:24 stevek Exp $
 */
/*
 * The xxx.mod file definse the format of audit records for the recorded
 * events and provides linkage information for the event generator to
 * generate appropriate audit record from the intercepted system calls or
 * raw audit trails.
 */



#include  <sys/types.h>
#include  <sys/stat.h>
#include  <fcntl.h>



typedef struct timespec timespec_t;


typedef struct {
	int 	time;		/* event timestamp, no of sec since Jan1,1970 */
	int 	msec; 		/* millisecond */
	int	errno;		/* error number: 0 means successful */
	char *  event; 		/* event name */
	int  	evid; 		/* event Id */
} Header;

typedef struct {
        int     tid;     	/* thread Id  		*/
     	int	pid;		/* process Id 		*/
     	int     ppid;		/* parent process Id 	*/
     	int	ruid;		/* real user Id 	*/
     	int	euid;		/* effective user Id 	*/
     	int	rgid;		/* real group Id	*/
     	int	egid;		/* effective group Id 	*/
	char *  pgname;		/* name of the executing program */
	char *  pgpath; 	/* fullpath name of the program */
	char *  cdir;		/* current directory 	*/
} SubjTk;

typedef struct {
	int		error;		/* content valid ? 0 - valid*/
 	int		st_mode;	/* inode protection mode */
	int		st_uid;		/* user ID of the file's owner */ 
	int		st_gid;		/* group ID of the file's group */
	long 		st_ino; 	/* inode's number */
	int		st_nlink;	/* number of hard links */
        timespec_t 	st_atimespec; 	/* time of last access */ 
        timespec_t 	st_mtimespec;	/* time of last modification */ 
        long		st_size;	/* file size, in bytes */	
        long  		st_gen;		/* file generation number */
	long		st_rdev;	/* device ID (if file is character */
} FileTk, ObjTk;

typedef struct {
   	int		error;		/* content valid? 0 - valid */
	int		type;		/* type, TCP/UCP/UNIX 	*/	
	int		afamily;	/* address family 	*/	
	int		lport;		/* local port 		*/
	int		rport;		/* remote port 		*/
	unsigned int	laddr;		/* local inet address	*/
	unsigned int	raddr;		/* remote inet address	*/
	char		name[128];	
} SockTk;

/* An audit event consists of attributes describing the event. The attributes
 * describe the process that performed the event, the objects on which it
 * was performed, time of the operation, and other information.
 * The attributes are grouped into tokens.
 * In generate, an audit event consists of at least a Header token and 
 * a Subject Token.
 * They can be accessed using $variable in the specification
 */

















	


module WSS {

   sysop generic {
        /* Header */
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno}; 
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};  
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};

	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		/* Process ID 		*/
	int	ppid	{_ppid};	/* Parent process ID	*/
	int	ruid    {_uid};		/* Real User ID		*/
	int	euid    {_euid};	/* Effective User ID 	*/
	int	rgid    {_gid};		/* Real Group ID 	*/
	int	egid    {_egid}; 	/* Effective Group ID 	*/
	int	sid 	{an->sid}; 	/* Subject ID 		*/
   };

   sysop open {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fdret,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fdret,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fdret,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fdret,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fdret,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fdret,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fdret,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fdret,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fdret,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fdret,&env->s)}, env->s.st_mode}; 
	int     a_mode	{init{env->int2 = $mode}, env->int2};
        int     flag    {init{env->int1 = $flags}, env->int1};
        int     fd	{init{env->fd = $fdret}, env->fd};
        int     cflag   {env->cflag};   /* has a file been created */
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
   };

   sysop open^ {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s}; 
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s}; 
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int     a_mode	{init{env->int2 = $mode}, env->int2};
        int     flag    {init{env->int1 = $flags}, env->int1};
        int     fd	{init{env->fd = 0}, env->fd};
        int     cflag   {env->cflag};   /* has a file been created */
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
   };

   sysop old_creat {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fdret,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fdret,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fdret,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fdret,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fdret,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fdret,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fdret,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fdret,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fdret,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fdret,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	int     a_mode	{init{env->int2 = $mode}, env->int2};
        int     fd	{init{env->fd = $fdret}, env->fd};
        int     cflag   {env->cflag};
   };

   sysop old_creat^ {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{ZeroFile(env)}, env->s};    
	int     validf  {init{ZeroFile(env)}, env->s.error};
	int     ouid    {init{ZeroFile(env)}, env->s.st_uid};
	int	ogid    {init{ZeroFile(env)}, env->s.st_gid};
	int	nodeid  {init{ZeroFile(env)}, env->s.st_ino};
	int	fsid    {init{ZeroFile(env)}, env->s.st_dev};
	int	gno     {init{ZeroFile(env)}, env->s.st_gen};
	int	pmode   {init{ZeroFile(env)}, env->s.st_mode};
	int	mode    {init{Zerofile(env)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	int     a_mode	{init{env->int2 = $mode}, env->int2};
        int     fd	{init{env->fd = 0}, env->fd};
        int     cflag   {env->cflag};
   };

   sysop close {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fd,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fd,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fd,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fd,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fd,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fd,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fd,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fd,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fd,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fd,&env->s)}, env->s.st_mode}; 
        char *  path 	{init{env->path = getpath($fd)}, env->path}; 
	char *  abspath {init{env->abspath = getpath($fd)}, env->abspath}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fd}, env->fd};
   }; 

   sysop bind {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fd}, env->fd};
   };

   sysop accept {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fdret}, env->fd};
        int	bdsock	{init{env->int1 = $fd}, env->int1};
   };

   sysop connect {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fd}, env->fd};
   };

   sysop read {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fd,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fd,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fd,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fd,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fd,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fd,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fd,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fd,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fd,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fd,&env->s)}, env->s.st_mode}; 
        char *  path 	{init{env->path = getpath($fd)}, env->path}; 
	char *  abspath {init{env->abspath = getpath($fd)}, env->abspath}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fd}, env->fd};
        int     nbytes	{init{env->au_nbytes = $nbytes}, env->au_nbytes};
	char *  iobuf	{init{env->iobuf = $iobuf}, env->iobuf};
        int     sizeret {init{env->ret = $sizeret}, env->ret};
   };

   sysop execve {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *path    	{init{env->path = _progname}, env->path}; 
	char *abspath 	{init{env->abspath = _path}, env->abspath}; 
        char **argv	{init{env->argv = $argv}, env->argv};
        char **envp	{init{env->envp = $envp}, env->envp};
   };

   sysop exit {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int 	ecode	{init{env->au_status = $code}, env->au_status};
   };

   /* we handle fork case specially in wrappers because when a process
      forks, the process and its child will receive a post fork event,
      and the post fork event of the children will occur first.
      At a high level, what we want to see is a post fork event for 
      the parent occur first, and then events of the children start to
      appear.  Therefore, in the generated code, the post fork event
      of the children is treated as the post fork event of the parent,
      and the parent will ignore all the successful post fork event.
    */
   sysop fork {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int 	chpid	{init{env->au_chpid = _pid}, env->au_chpid};
   };

   sysop vfork {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int 	chpid	{init{env->au_chpid = _pid}, env->au_chpid};
   };

   sysop kill {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
        int	a_pid	{init{env->int1 = $pid}, env->int1};
        int	signal	{init{env->au_signal = $signum}, env->au_signal};
   };

   sysop write {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fd,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fd,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fd,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fd,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fd,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fd,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fd,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fd,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fd,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fd,&env->s)}, env->s.st_mode}; 
        char *  path 	{init{env->path = getpath($fd)}, env->path}; 
	char *  abspath {init{env->abspath = getpath($fd)}, env->abspath}; 
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
        int     fd	{init{env->fd = $fd}, env->fd};
        int     nbytes	{init{env->au_nbytes = $nbytes}, env->au_nbytes};
	char *  iobuf	{init{env->iobuf = $iobuf}, env->iobuf};
        int     sizeret {init{env->ret = $sizeret}, env->ret};
   };

   sysop rename {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk	file	{preinit{wr_stat($path,&env->s)}, env->s};   
		/* file attributes of the "fromfile" before rename operation */
	FileTk 	tofile	{init{wr_stat($destpath,&env->s1)}, env->s1};  
		/* file attributes of the "tofilepath" before the operation. 
                 * Note that the path "tofilepath" may not exist, in this case
                 * tofile is not valid, otherwise, "tofilepath" will be unlinked
                 * and tofile will contain the attribute of the "tofilepath" 
                 * before the rename operation.
                 * ouid... refers to the 
		 */
	int     ouid    {init{wr_stat($destpath,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($destpath,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($destpath,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($destpath,&env->s)}, env->s.st_dev};
	int	pmode   {init{wr_stat($destpath,&env->s)}, env->s.st_mode};
	int	gno     {init{wr_stat($destpath,&env->s)}, env->s.st_gen};
	int	mode    {init{wr_stat($destpath,&env->s)}, env->s.st_mode};
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	char * 	dstpath {init{env->topath = $destpath}, env->topath};
	char *  absdstpath {init{env->abstopath = wr_abspath($destpath)}, env->abstopath};
   };

   sysop rename^ {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk	file	{preinit{wr_stat($path,&env->s)}, env->s};   
		/* file attributes of the "fromfilepath" */
	FileTk 	tofile	{init{wr_stat($destpath,&env->s1)}, env->s1};  
		/* file attributes of the "tofilepath". */
	int     ouid    {init{wr_stat($destpath,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($destpath,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($destpath,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($destpath,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($destpath,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($destpath,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($destpath,&env->s)}, env->s.st_mode};
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	char * 	dstpath {init{env->topath = $destpath}, env->topath};
	char *  absdstpath {init{env->abstopath = wr_abspath($destpath)}, env->abstopath};
   };

   sysop unlink {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	FileTk	file	{preinit{wr_stat($path,&env->s)}, env->s};   
		/* attributes of "path" before the unlink operation */
	int     ouid    {preinit{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {preinit{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {preinit{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {preinit{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {preinit{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {preinit{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {preinit{wr_stat($path,&env->s)}, env->s.st_mode};
   };

   sysop chmod {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	int	oldmode {preinit{wr_stat($path, &env->s)}, preinit{env->au_oldmode = env->s.st_mode}, env->au_oldmode};
	int	newmode {init{env->au_newmode = $newmode}, env->au_newmode};
		/* the new mode given in the argument of the syscall.
                 * note that it may not equal to the mode of the file
                 * ($mode) after the call
                 */
   };

   sysop chmod^ {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	int	oldmode {init{wr_stat($path, &env->s)}, init{env->au_oldmode = env->s.st_mode}, env->au_oldmode};
	int	newmode {init{env->au_newmode = $newmode}, env->au_newmode};
   };

   sysop fchmod {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fd,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fd,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fd,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fd,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fd,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fd,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fd,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fd,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fd,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fd,&env->s)}, env->s.st_mode}; 
        char *  path 	{init{env->path = getpath($fd)}, env->path}; 
	char *  abspath {init{env->abspath = getpath($fd)}, env->abspath}; 
        int     fd	{init{env->fd = $fd}, env->fd};
	int	oldmode {preinit{wr_fstat($fd, &env->s)}, preinit{env->au_oldmode = env->s.st_mode}, env->au_oldmode};
	int	newmode {init{env->au_newmode = $newmode}, env->au_newmode};
   };

   sysop chown {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	int     oldouid {preinit{wr_stat($path, &env->s)}, preinit{env->au_oldouid = env->s.st_uid}, env->au_oldouid};
	int	newouid {init{env->au_newouid = $uid}, env->au_newouid};
		/* the new owner uid given in the argument of the syscall.
                 * note that it may not equal to the owner of the file
                 * ($ouid) after the call
                 */
	int     oldogid {preinit{wr_stat($path, &env->s)}, preinit{env->au_oldogid = env->s.st_gid}, env->au_oldogid};
	int	newogid {init{env->au_newouid = $gid}, env->au_newogid};
   };

   sysop fchown {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_fstat($fd,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_fstat($fd,&env->s)}, env->s};    
	int     validf	{init{wr_fstat($fd,&env->s)}, env->s.error};
	int     ouid    {init{wr_fstat($fd,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_fstat($fd,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_fstat($fd,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_fstat($fd,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_fstat($fd,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_fstat($fd,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_fstat($fd,&env->s)}, env->s.st_mode}; 
        char *  path 	{init{env->path = getpath($fd)}, env->path}; 
	char *  abspath {init{env->abspath = getpath($fd)}, env->abspath}; 
        int     fd	{init{env->fd = $fd}, env->fd};
	int     oldouid {preinit{wr_fstat($fd, &env->s)}, preinit{env->au_oldouid = env->s.st_uid}, env->au_oldouid};
	int	newouid {init{env->au_newouid = $uid}, env->au_newouid};
	int     oldogid {preinit{wr_fstat($fd, &env->s)}, preinit{env->au_oldogid = env->s.st_gid}, env->au_oldogid};
	int	newogid {init{env->au_newouid = $gid}, env->au_newogid};
   };

   sysop link {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	char *	dstpath		{init{env->topath = destpath}, env->topath};
	char *	absdstpath	{init{env->toabspath = wr_abspath(destpath)}, env->toabspath};
   };

   sysop symlink {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
	char *	dstpath		{init{env->topath = $destpath}, env->topath};
   };

   sysop mkdir {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
   };

   sysop mknod {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s};    
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s};    
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	gno     {init{wr_stat($path,&env->s)}, env->s.st_gen};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode}; 
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
   };

   sysop setuid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	olduid	{preinit{env->au_olduid = _uid}, env->au_olduid};
	int	newruid	{init{env->au_uid = $uid}, env->au_uid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop seteuid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int     oldeuid {preinit{env->au_oldeuid = _euid}, env->au_oldeuid};
	int	neweuid	{init{env->au_euid = $uid}, env->au_euid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop setreuid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	newruid	{init{env->au_uid = $ruid}, env->au_uid};
	int	neweuid	{init{env->au_euid = $euid}, env->au_euid};
	int	olduid	{preinit{env->au_olduid = _uid}, env->au_olduid};
	int     oldeuid {preinit{env->au_oldeuid = _euid}, env->au_oldeuid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop setgid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	oldgid	{preinit{env->au_oldgid = _gid}, env->au_oldgid};
	int	newrgid	{init{env->au_gid = $gid}, env->au_gid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop setegid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int     oldegid {preinit{env->au_oldegid = _egid}, env->au_oldegid};
	int	newegid	{init{env->au_egid = $gid}, env->au_egid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop setregid {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	newrgid	{init{env->au_gid = $rgid}, env->au_gid};
	int	newegid	{init{env->au_egid = $egid}, env->au_egid};
	int	oldgid	{preinit{env->au_oldgid = _gid}, env->au_oldgid};
	int     oldegid {preinit{env->au_oldegid = _egid}, env->au_oldegid};
	int	ret	{init{env->ret = $erret}, env->ret};
   };

   sysop socket {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	int	domain	{init{env->au_domain = $domain}, env->au_domain};
	int	type	{init{env->au_socktype = $socktype}, env->au_socktype};
	int	proto	{init{env->au_proto = $proto}, env->au_proto};
        int	fd      {init{env->fd = $fdret}, env->fd};
   };

   sysop sendto {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
        int	fd      {init{env->fd = $fd}, env->fd};
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 

   };

   sysop recvfrom {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
        int	fd      {init{env->fd = $fd}, env->fd};
	int	type 	{init{getSock($fd, &env->sock)}, env->sock.type};  
	int	afamily {init{getSock($fd, &env->sock)}, env->sock.afamily};
	int	lport 	{init{getSock($fd, &env->sock)}, env->sock.lport}; 
	int	rport   {init{getSock($fd, &env->sock)}, env->sock.rport}; 
	u_int	laddr 	{init{getSock($fd, &env->sock)}, env->sock.laddr}; 
	u_int	raddr 	{init{getSock($fd, &env->sock)}, env->sock.raddr}; 
	char *  sname	{init{getSock($fd, &env->sock)}, env->sock.name}; 
   };

   sysop truncate {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s}; 
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s}; 
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode};
        char *  path    {init{env->path = $path}, env->path}; 
	char *  abspath {init{env->abspath = wr_abspath($path)}, env->abspath}; 
   };

   sysop ftruncate {
	Header	hd	{env->hdr}; 
	int 	time	{init{env->hdr.time = get_curtime()}, env->hdr.time};
	int 	msec	{env->hdr.msec}; 
	int 	errno	{init{env->errno = $errno}, env->errno};  
	char *	event	{init{env->hdr.event = $$}, env->hdr.event}; 
	int	evid    {init{env->hdr.evid = $#}, env->hdr.evid};   
	char *	pgname  {init{env->hdr.pgname = _progname}, env->hdr.pgname};
	char *	pgpath  {init{env->hgr.pgpath = _path}, env->hdr.pgpath};
 
	SubjTk 	proc	{init{InitProc(env->proc)}, env->proc}; 
	SubjTk 	subj	{init{InitProc(env->proc)}, env->proc}; 
	int	pid	{_pid};		
	int	ppid	{_ppid};	
	int	ruid    {_uid};		
	int	euid    {_euid};	
	int	rgid    {_gid};		
	int	egid    {_egid}; 	
	int	sid 	{an->sid}; 
	FileTk 	file	{init{wr_stat($path,&env->s)}, env->s}; 
	FileTk 	obj	{init{wr_stat($path,&env->s)}, env->s}; 
	int     validf  {init{wr_stat($path,&env->s)}, env->s.error};
	int     ouid    {init{wr_stat($path,&env->s)}, env->s.st_uid};
	int	ogid    {init{wr_stat($path,&env->s)}, env->s.st_gid};
	int	nodeid  {init{wr_stat($path,&env->s)}, env->s.st_ino};
	int	fsid    {init{wr_stat($path,&env->s)}, env->s.st_dev};
	int	pmode   {init{wr_stat($path,&env->s)}, env->s.st_mode};
	int	mode    {init{wr_stat($path,&env->s)}, env->s.st_mode};
        int	fd      {init{env->fd = $fd}, env->fd};
   };
};
