/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  code-cpp.cc:  Source for generating C++ analyzer code
 *
 *  Author: Calvin Ko
 *  Email : calvin_ko@nai.com
 *
 *  $Id: code-cpp.cc,v 1.2 2001/04/05 18:40:24 stevek Exp $
 */

#include <stdio.h>
#include <string>
#include "pegc.h"
#include "code-cpp.h" 

source *CppCodeGen::gencode() 

{
	return gen_tokenfuncs();
}

/** Generate all the functions for acceptance a token
 *
 *  Return a source object that contain the function code
 */
source *CppCodeGen::gen_tokenfuncs()

{
	int i,j;
	source *src = new source();
	for (i = nvt; i < nterm + nvt; i++) {
         	TermDef *td = (*(tdtbl.find(idtbl[i]->name))).second;
		*src << gen_tokenfunc(td);
    	};
    	src->indent();
    	return src;
}

/** Generate a audit token function that take an audit record and
 *  determine whether the audit record can be recognized as the token
 *
 */
source CppCodeGen::gen_tokenfunc(TermDef *td)

{
	source src;
	if (td->texpr->etype() != Expr::TermDef) {
		cerr << "Internal Error #11 (TermDefExpr Expected)\n";
		return source();
	}

	TermDefExpr *te = (TermDefExpr *) td->texpr;	
     char strbuf[512];
     bzero(strbuf,512);
     ostrstream sstr(strbuf, 512);
     sstr << *te;
     src.gen_comment("Generated Code for ", sstr.str());
	
	Expr *l;	
     if (te->type == 1) { /* positive */
          for (l = te->elist; l != NULL; l = l->next) {            
			EvSelExpr *se = (EvSelExpr *) l;
			src << "if (arec.evid == " << se->aevent.getSysOpName() << ") {\n";
			src << "     if (" << *se->cond->code << ") {\n";
			src << "          recognize = 1\n"; 
			src << "     }\n"; 
			src << "}\n";
          }
     } else {  /* negative termdef expression */
	}
	return src;
};
	
