/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  code-wdl.cc:  Source for generating WDL wrapper code 
 *
 *  Author: Calvin Ko
 *  Email : calvin_ko@nai.com
 *
 *  $Id: code-wdl.cc,v 1.10 2001/04/05 18:40:24 stevek Exp $
 */

#include <stdio.h>
#include <string>
#include "pegc.h"
#include "code-wdl.h"


static char *defstr = 
"#define  initP(p) 		{ p.pid = _pid; p.ppid = _ppid; p.ruid = _uid; \\\n"
"                          p.euid = _euid; p.rgid = _gid; p.egid = _egid; p.cdir = _path;} \n"
"\n"
"\n"
"#define  getProgName()   wr_strdup(_progname);\n"
"\n"
"#define  au_status	     int1	/* exit() status */ \n"
"#define  au_nbytes 	int1	/* read/write nbytes */ \n"
"#define  au_chpid 	     int1	/* fork() chpid */ \n"
"#define  au_signal 	int2	/* kill() signal */ \n"
"#define  au_newmode	int1	/* chmod/fchmod() mode */ \n"
"#define  au_oldmode	int2	/* chmod/fchmod() mode */ \n"
"#define  au_newouid	int1	/* chown/fchown() owner */ \n"
"#define  au_oldouid	int2	/* chown/fchown() owner */ \n"
"#define  au_newogid	int3	/* chown/fchown() group */ \n"
"#define  au_oldogid	int4	/* chown/fchown() group */ \n"
"#define  au_uid	     int1	/* setuid/setreuid owner */ \n"
"#define  au_euid	     int2	/* setduid/setreuid owner */ \n"
"#define  au_olduid	     int3	/* setuid/setreuid owner */ \n"
"#define  au_oldeuid	int4	/* setduid/setreuid owner */ \n"
"#define  au_gid	     int1	/* setgid/setregid group */ \n"
"#define  au_egid	     int2	/* setdgid/setregid group */ \n"
"#define  au_oldgid	     int3	/* setgid/setregid group */ \n"
"#define  au_oldguid	int4	/* setdgid/setregid group */ \n"
"#define  au_domain	     int1	/* socket() domain */ \n"
"#define  au_socktype	int2	/* socket() type */ \n"
"#define  au_proto	     int3	/* socket() proto */ \n"
"\n"
"#define  detach_proc()	{  wrap = 0;  } \n"
"\n";

static char *topdecl = 
"    typedef struct { \n"
"        int     time;           /* event timestamp, no of sec since Jan1,1970 */\n"
"        int     usec;           /* macrosecond */\n"
"        int     errno;          /* error number: 0 means successful */\n"
"        int     flag;		   /* pre:0 or post:1	*/\n"
"        const char *  event;    /* event name */\n"
"        int     evid;           /* event Id */\n"
"    } Header;\n"
"\n"
"    typedef struct {\n"
"        int     uid; 		   /* user id of the invoker */\n"
"        int     gid;		   /* group id of the invoker */\n"
"        char    pgname[64];     /* name of the executing program */\n"
"        char    pgpath[128];    /* fullpath name of the program */\n"
"    } SubjTk;\n"
"\n"
"    typedef struct {\n"
"        int     pid;\n"
"        int     ppid;\n"
"        int     ruid;\n"
"        int     euid;\n"
"        int     rgid;\n"
"        int     egid;\n"
"        char *  cdir;\n"
"    } ProcTk;\n"
"\n"
"    typedef struct {\n"
"        int             error;         /* content valid ? 0 - valid       */   \n"
"        int             mode;           /* inode protection mode */\n"
"        int             ouid;           /* user ID of the file's owner */\n"
"        int             ogid;           /* group ID of the file's group */\n"
"        long            ino;            /* inode's number */\n"
"        u_int           nlink;          /* number of hard links */\n"
"        long			atime;          /* time of last access */\n"
"        long			mtime;          /* time of last modification */\n"
"        long            size;           /* file size, in bytes */\n"
"        long            gno;            /* file generation number */\n"
"        long            dev;            /* ID of device containing file */\n"
"        long            rdev;           /* device ID (if file is character */\n"
"	    char *          path;           /* absolute path */\n"
"    } FileTk;\n"
"\n"
"    typedef struct {\n"
"        int             error;          /* SockTk Valid 0 - valid */ \n"
"        int             type;           /* type, TCP/UCP/UNIX   */ \n"
"        int             afamily;        /* address family       */ \n"
"        int             lport;          /* local port           */ \n"
"        int             rport;          /* remote port          */ \n"
"        unsigned int    laddr;          /* local inet address   */ \n"
"        unsigned int    raddr;          /* remote inet address  */ \n"
"        char            name[128];	 /* name of the socket   */ \n"
"    } SockTk;\n"
"\n"
"    static char g_pathbuf[MAXPATHLEN];   // temporary buffer for hold pathnames \n"
"    static int  g_creatflag;             // flag for create event\n"
"           \n"
"           \n"
"#ifdef TRACK_FD \n"
"    /*  Definition of the file descriptor table for keeping track of the\n"
"     *  fd to filename mapping\n"
"     */\n"
"\n"
"     DBTABLE fd_table {\n"
"         int key    fd;\n"
"         char(128)  name;\n"
"     };\n"
"     local fd_table fdtbl;\n"
"    \n"
"     DBTABLE gfd_table {\n"
"         int        pid;\n"
"         int        fd;\n"
"         char(128)  name;\n"
"     };\n"
"    \n"
"     gfd_table gfdtbl;\n"
"#endif /* TRACK_FD */ \n"
"    \n"
"    /* Definition of file table used to keep track of files created by\n"
"     * the wrapped process.\n"
"     */\n"
"     DBTABLE file_table {\n"
"         int key          ino;           // inode of the file\n"
"         int              gen;           // generation number of the file\n"
"         int              mode;          // permission mode of the file\n"
"         int              cpid;          // process id of the creator\n"
"         char(256)        name;          // name of the file created\n"
"     };\n"
"     file_table filetbl;\n"
"    \n"
"\n";

static char *envdecl_1 = 
"     typedef struct _env {\n"
"           int    deny;        	/* whether to deny the op */\n"
"           Header H;              /* header */ \n"
"           ProcTk P;              /* process info */\n"
"           SubjTk S;              /* subject info */ \n"
"           FileTk F;              /* File */\n"
"           FileTk F1;					\n"
"           SockTk K;              /* Socket */\n"
"           int    fd;\n" 
"           int    ret;\n" 
"           int    int1;\n"
"           int    int2;\n"
"           int    int3;\n"
"           int    int4;\n"
"           int    cflag;\n"
"           char   *str1;\n" 
"           char   *str2;\n" 
"           char   *iobuf;\n"
"           /* user defined environment variables */\n";

static char *envdecl_2 = 
"     } Env;\n"
"     struct _ana;\n"
"     typedef struct _tp {\n"
"           int    	top;\n"
"           int    	sid;		/* Subject id */ \n"
"           int    	refcount;\n"
"           short  	stack[STACKSIZE];\n"
"           LocalEnv  	lenv;\n"
"           Env    	*env;		\n"
"           struct _tp  *next;\n"
"           struct _ana *a;\n"
"     } ThParser;\n" 
"\n"
"     typedef struct _ana {\n"
"           int    	type;		/* Analyzer Type */ \n"
"           int    	sid;		 	/* Subject id */ \n"
"           Env    	env;			/* Environment */ \n"
"           ThParser	*tparsers;	/* List of Thread Parser */ \n"
"     } Analyzer;\n" 
"\n"
"     static Analyzer 	*an;\n"
"     static ThParser 	*tp;\n"
"     static Env      	*env;\n"
"     static LocalEnv	*lenv;\n"
"     static int	gotFork; /* has the process got the first fork op? \n"
"                          gotFork is used to fix the problem that once a \n"
"                          process forks, the child process will run first \n"
"                          and the wrapper instance for the child will get a \n"
"                          fork post event, before the wrapper instance for \n"
"                          the parent. */ \n"
"     static int 	wrap;    /* wrap that process, will be init to 1 */\n"
"\n"
"     Analyzer *Analyzer_new(int sid) {\n"
"          Analyzer *a;\n"
"#ifdef VERBOSE\n"
"          wr_printf(\"New Analyzer %s %d \\n\", AZ_NAME, sid); \n"
"#endif\n"
"          a = (Analyzer *) wr_malloc(sizeof(Analyzer));\n"
"          a->sid = sid;\n"
"          a->tparsers = NULL;\n"
"          return a;\n"
"     }\n"
"\n"
"     void Analyzer_del(Analyzer *a) {\n"
"          wr_free(a);\n"
"     }\n"
"\n"
"     ThParser *ThParser_new(Analyzer *a, int start, int sid) {\n"
"          ThParser *p;\n"
"#ifdef VERBOSE\n"
"          wr_printf(\"New Thread Parser for Ana %s %d: sym: %d sid: %d\\n\", AZ_NAME, a->sid, start, sid); \n"
"#endif\n"
"          p = (ThParser *) wr_malloc(sizeof(ThParser));\n"
"          p->sid = sid;\n"
"          p->refcount = 1;\n"
"          p->top = 0;\n"
"          p->stack[p->top] = start;\n"
"          p->env = &a->env;\n"
"          p->a = a;\n"
"          p->next = a->tparsers; \n"
"          a->tparsers = p; \n"
"          return p;\n"
"     }\n"
"\n"
"     void ThParser_del(ThParser *t) {\n"
"          ThParser *t1 = t->a->tparsers;\n"
"          if (t1 == NULL) return;\n"
"          if (t1 == t) { \n"
"               t->a->tparsers = t1->next;\n"
"#ifdef VERBOSE\n"
"               wr_printf(\"Delete Thread Parser %s %d\\n\", AZ_NAME, t->sid); \n"
"#endif\n"
"               wr_free(t);\n"
"               return;\n"
"          }\n"
"          while (t1->next != NULL) {\n"
"               if (t1->next == t) { \n"
"                    t1->next = t->next;\n"
"                    wr_printf(\"Delete Thread Parser %s %d\\n\", AZ_NAME, t->sid); \n"
"                    wr_free(t);\n"
"                    return;\n"
"               }\n"
"               t1 = t1->next;\n"
"          }\n"
"     }\n"
"\n"
"     int FUNC_Error(ThParser *tp, int token, int *accept, Env *env, LocalEnv *lenv)\n"
"\n"
"     {\n"
"           wr_printf(\"Top = %d: Wrong Token:\\n\", tp->top); \n"
"     }\n"
"\n"
"     char eventmsg[256];\n"
"\n"
"     static int ParseError(char *msg)\n"
"     {\n"
"           wr_printf(\"Top = %d: Wrong Token:%s\\n\", tp->stack[tp->top], msg); \n"
"           wr_audit_printf(0, \"Top = %d: Wrong Token:%s\\n\", tp->stack[tp->top], msg); \n"
"     }\n"
"\n";

static char *parsefunc = 
"static int parse_token(ThParser *tp, int token, Env *env, LocalEnv *lenv)\n"
"\n"
"{\n"
"     int accept = 0;\n"
"     int error  = 0;\n"
"     while (!error && (!accept || tp->stack[tp->top] > NSYMS)) {\n"
"          if (tp->stack[tp->top] <= NTERMS) {\n"
"               if (tp->stack[tp->top] == token) {\n"
"                    tp->top--;\n"
"                    accept == 1;\n"
"                } else {\n"
"                    error = 1;\n"
"                    FUNC_Error(tp, token, &accept, env, lenv);\n"
"                }\n"
"          } else if (tp->stack[tp->top] <= NITEMS ) {\n"
"                error = (*subFuncTbl[tp->stack[tp->top] - NTERMS - 2])\n"
"                              (_wrp, tp, token, &accept, env, lenv, _data);\n"
"          } else {\n"
"                error = P_STACKVALUE_OVERFLOW;\n"
"                tp->top--;\n"
"                if (tp->top < 0) tp->top = 0;\n"
"	   }\n"
"     }\n"
"}\n";

static char *localfuncs= 
"   /*********************************************************************\n"
"    *                          LOCAL FUNCTIONS                          *\n"
"    *********************************************************************/\n"
"\n"
"#include \"otherfuncs.c\"\n"
"\n"
"   /* get the pathname from fd, used with fd_table  */\n"
"    static char *getpath(int fd)\n"
"    {    int i;\n"
"         DBROW::fd_table dbrow;\n"
"         i = wql { select into dbrow from fdtbl where .fd = fd; };\n"
"         if (i == 1) {\n"
"              dbrow.start();\n"
"              wr_strcpy(g_pathbuf, dbrow.name);\n"
"         } else\n"
"              wr_strcpy(g_pathbuf, \"NotFound\");\n"
"         return g_pathbuf;\n"
"    }    \n"
"\n"
"   /* get the socket info, if any, from fd */\n"
"    void getSock(int fd, SockTk *sock)\n"
"    {    int error, error1, len, len1; \n"
"         char buf[32], buf1[32], *n;\n"
"         struct sockaddr s, s1;\n"
"         wr_bzero(sock, sizeof(SockTk));\n"
"         error = wr_getsockname(fd, &s, &len);\n"
"         error1 = wr_getpeername(fd, &s1, &len1);\n"
"         sock->error = error != 0 ? error :\n"
"                          (error1 != 0 || error1 != ENOTCONN ? error1 : 0);\n"
"         sock->afamily = !error && len >= 2 ? s.sa_family :\n"
"                            (!error1 && len1 >= 2 ? s1.sa_family : (u_char) 0);\n"
"         switch(sock->afamily) {\n"
"         case AF_UNIX :\n"
"              wr_strcpy(sock->name, getSockAddr(&s1, len1,\n"
"                                      &sock->raddr, &sock->rport));\n"
"              return; \n"
"         case AF_INET :\n"
"              getSockAddr(&s, len, &sock->laddr, &sock->lport);\n"
"              getSockAddr(&s1, len1, &sock->raddr, &sock->rport);\n"
"              wr_sprintf(sock->name, \"%s:%d<->%s:%d\",\n"
"                           ntoa(sock->laddr, buf), sock->lport,\n"
"                           ntoa(sock->raddr, buf1), sock->rport);\n"
"              return;\n"
"         default      :\n"
"              wr_sprintf(sock->name, \"AF_USPEC\");\n"
"              return;\n"
"         }\n"
"     }   \n"
"\n"
"    char sflag[16];\n"
"    char *symflag(int flag) \n"
"    {    sflag[0] = '\\0';   \n"
"         if ((flag & O_ACCMODE) == O_RDONLY) {   \n"
"              wr_strcat(sflag, \"r\");   \n"
"         } else if ((flag & O_ACCMODE) == O_WRONLY) {   \n"
"              wr_strcat(sflag, \"w\");   \n"
"         } else {   \n"
"              wr_strcat(sflag, \"rw\");   \n"
"         }   \n"
"         if (flag & O_TRUNC)   \n"
"              wr_strcat(sflag, \"t\");   \n"
"         if (flag & O_CREAT)   \n"
"              wr_strcat(sflag, \"c\");   \n"
"         if (flag & O_EXCL)   \n"
"              wr_strcat(sflag, \"x\");   \n"
"         if (flag & O_APPEND)   \n"
"              wr_strcat(sflag, \"a\");   \n"
"         return sflag;   \n"
"    }   \n"
"\n"
"    int created(int pid, ObjTk *f)\n"
"    {    \n"
"         DBROW::file_table row; \n"
"         int i;\n"
"         int inode = f->st_ino;\n"
"         i = wql { select into row from filetbl\n"
"                        where .ino = inode and .cpid = pid;    };\n"
"         return i == 0 ? 0 : 1;\n"
"    }\n"
"    \n"
"    int createdBySubj(int sid, ObjTk *f)\n"
"    {    \n"
"         DBROW::file_table row; \n"
"         int i;\n"
"         int inode = f->st_ino;\n"
"         int pid = sid + 100000;\n"
"         i = wql { select into row from filetbl\n"
"                        where .ino = inode and .cpid = pid;    };\n"
"         return i == 0 ? 0 : 1;\n"
"    }\n"
"    int inside(char *p, char *d)\n"
"    {\n"
"         if (p =~ m|d|)\n"
"              return 0;\n"
"         else\n"
"              return 1;\n"
"    } \n"
"\n"
;

static char *wrdecl = 
"    wr_uninstall() {\n"
"         wr_printf(\"specids uninstall\\n\");\n"
"#ifdef TRACK_FD\n"
"         wql { drop table filetbl; \n"
"               drop table gfdtbl; \n"
"             };\n"
"#endif\n"
"    }\n"
"       \n"
"    wr_deactivate() {\n"
"#ifdef VERBOSE\n"
"         wr_printf(\"Deactivate wrapper %s for process %d\\n\", AZ_NAME, _pid);\n"
"#endif\n"
"#ifdef TRACK_FD\n"
"         wql { drop table fdtbl; };\n"
"#endif\n"
"         tp->refcount--;\n"
"         if (tp->refcount == 0)\n"
"              ThParser_del(tp);\n"
"    }\n"
"    \n"
"\n"
"\n"
"\n"
;

static char *wr_install_begin = 
"    wr_install() {\n"
"         int i;\n"
"         wr_printf(\"specids install\\n\");\n"
"#ifdef TRACK_FD\n"
"         i = wql {\n"
"              create table gfdtbl;\n"
"              create table filetbl;\n"
"         };\n"
"         if (i < 0 && i != -WRE_EXIST) {\n"
"             wr_printf(\"Error creating tables filetbl.\\n\");\n"
"         }\n"
"#endif\n"
;

static char *wr_install_end = 
"    }   \n"
"\n"
;

static char *wr_activate_begin = 
"    wr_activate() {\n"
"         int i,j;\n"
"         an = Analyzer_new(_pid);\n"
"         tp = ThParser_new(an, START_NTERM, _pid);\n"
"         env = &an->env;\n"
"         lenv = &tp->lenv;\n"
"         gotFork = 1;\n"
"         wrap = 1;\n"
"         env->H.errno = 0;\n"
"         wr_strncpy(env->S.pgname, _progname, 64);\n"
"         env->S.uid = _uid;\n"
"         env->S.gid = _gid;\n" 
"         wr_strncpy(env->S.pgpath, _path, 128);\n"
"#ifdef VERBOSE\n"
"         wr_printf(\"Activate wrapper %s for process %d\\n\", AZ_NAME,  _pid);\n"
"#endif\n"
"#ifdef TRACK_FD\n"
"         i = wql {\n"
"             create table fdtbl;\n"
"         };\n"
"         if (i < 0) {\n"
"             wr_printf(\"Error creating table fdtbl.\\n\");\n"
"         }\n"
"         /* Identify the file associated with each opened descriptor */\n"
"         for (j=0; j<20; j++) {\n"
"              struct wr_stat s;\n"
"		     int e = wr_fstat(j, &s);\n"
"              char sbuf[256]; \n"
"              if (e == 0) {\n"
"                   if (IsCharDev(s)) {\n"
"                        // st.st_rdev contain major and minor device number\n"
"                        wr_sprintf(sbuf, \"CharDev-%x\", s.st_rdev);\n"
"                   } else if (IsBlkDev(s)) {\n"
"                        wr_sprintf(sbuf, \"BlkDev-%x\", s.st_rdev);\n"
"                   } else if (IsRegFile(s)) {\n"
"                        wr_sprintf(sbuf, \"RegFile-%d-%x\", j, s.st_ino, s.st_mode);\n"
"                   } else if (IsSocket(s)) {\n"
"                        SockTk sock; \n"
"                        getSock(j, &sock); \n"
"                        wr_strcpy(sbuf, sock.name); \n"
"                   } else {\n"
"                        wr_strcpy(sbuf, \"Unknown\");\n"
"                   }\n"          
"                   wql { insert into fdtbl values (j, sbuf); };\n"
"              }\n"
"         }\n"
"#endif\n"
;

static char *wr_activate_end = 
"    }\n"
"\n"
;

static char *wr_duplicate_begin = 
"    wr_duplicate() {\n"
"         int i;\n"
"#ifdef TRACK_FD\n"
"         DBROW::gfd_table myrow;\n"
"#endif\n"
"#ifdef VERBOSE\n"
"         wr_printf(\"Duplicate wrapper %s for proc %d: parent: %d\\n\", AZ_NAME, _pid, _ppid);\n"
"#endif\n"
"         tp->refcount++;\n"
"         gotFork = 0;\n"
"         wrap = 1;\n"
"#ifdef TRACK_FD\n"
"         i = wql {\n"
"              create table fdtbl;\n"
"         };\n"
"         if (i < 0) {\n"
"              wr_printf(\"Error creating table fdtbl.\\n\");\n"
"         }\n"
"         i = wql { select into myrow from gfdtbl where .pid = _ppid ; };\n"
"         if (i > 0) {\n"
"             myrow.start();\n"
"             do {  \n"
"                  wql { insert into fdtbl\n"
"                           values (myrow.fd, myrow.name); \n"
"                      };\n"
"             } while (myrow.next() != NULL);\n"
"         } \n"
"         i = wql { delete from gfdtbl where .pid = _ppid ; };\n"
"#endif\n"
"\n"
;

static char *wr_duplicate_end = 
"    }\n"
"\n"
;

static char *wr_debugclause =
"    /* For debuging */\n"
"    WR_LOCAL::ALL_AUDIT_OPS pre {\n"
"         wr_printf(\"syscall is %s\\n\", $$);\n"
"    };\n"
"\n"  
;     

static char *wr_parseerror_generic =
"    WR_LOCAL::ALL_AUDIT_OPS ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s \\t\", $$);\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n";
static char *wr_parseerror_open =
"    /* Come to here if syscall is not accept by any token funcs */\n"
"    WR_LOCAL::op{open} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s \\t\", $$, $path);\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n";
static char *wr_parseerror_read =
"    WR_LOCAL::op{read} ($errno == 0) post {\n"
"         struct wr_stat s;\n"
"         wr_fstat($fd, &s);\n"
"         wr_sprintf(eventmsg, \"%s %s %o\\t\", $$, getpath($fd), s.st_mode);\n"
"         ParseError(eventmsg);\n" 
"    };\n" 
"\n";
static char *wr_parseerror_write =
"    WR_LOCAL::op{write} ($errno == 0) post {\n"
"         struct wr_stat s;\n"
"         wr_fstat($fd, &s);\n"
"         wr_sprintf(eventmsg, \"%s %s %o\\t\", $$, getpath($fd), s.st_mode);\n"
"         ParseError(eventmsg);\n" 
"    };\n" 
"\n";

static char *wr_parseerror_close =
"    WR_LOCAL::op{close} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s \\t\", $$, getpath($fd));\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n"
;
static char *wr_parseerror_execve =
"    WR_LOCAL::op{execve} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s \\t\", $$, _progname);\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n"
;
static char *wr_parseerror_chmod =
"    WR_LOCAL::op{chmod} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s \\t\", $$, $path);\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n";
static char *wr_parseerror_fchmod =
"    WR_LOCAL::op{fchmod} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s \\t\", $$, getpath($fd));\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n";
static char *wr_parseerror_rename =
"    WR_LOCAL::op{rename} ($errno == 0) post {\n"
"         wr_sprintf(eventmsg, \"%s %s to %s\\t\", $$, $path, $destpath);\n"
"         ParseError(eventmsg);\n" 
"    };\n"
"\n";

static char *wr_mustwrapfunc_post = "#include \"postclause.c\" \n";


static char *wr_mustwrapfunc_pre =
"#ifdef TRACK_FD\n"
"    WR_LOCAL::op{fork||vfork} pre { \n"
"        int i; \n"
"        DBROW::fd_table myrow;\n"
"        i = wql { select into myrow from fdtbl; };\n"
"        if (i > 0) {\n"
"             myrow.start();\n"
"             do {   \n"
"                  wql { insert into gfdtbl \n"
"                          values (_pid, myrow.fd, myrow.name); \n"
"                      };\n"
"             } while (myrow.next() != NULL);\n"
"        }               \n"
"    };         \n"
"#endif\n"
"\n"
"    WR_LOCAL::op{open} pre { \n"
"         struct wr_stat s; \n"
"         env->cflag = 0; \n"
"         if ($flags & O_CREAT && wr_stat($path, &s) == ENOENT /*!exist*/) { \n"
"              env->cflag = 1;\n"
"         }\n"
"    };         \n"
"\n"
"    WR_LOCAL::op{creat} pre { \n"
"         struct wr_stat s; \n"
"         env->cflag = 0; \n"
"         if (wr_stat($path, &s) == ENOENT /*!exist*/) { \n"
"              env->cflag = 1;\n"
"         }\n"
"    };         \n"
"\n"
;

source PEGram_WDL::gen_define_auditops()

{
	source src, s1;
	int i = 0;
	SelList::iterator siter;
	for (siter = selector->begin(); siter != selector->end(); siter++) {
		AEvSet *eset = (*siter)->evset;
		AEvSet::iterator iter;
		for (iter = eset->begin(); iter != eset->end(); iter++) {
			if (!s1.is_empty()) s1 << " || ";
			s1 << (*iter).sevent->name;
		}
		src << "#define ALL_AUDIT_OPS_S" << i++ << "\top{" << s1 << "}\n";
      }
      src << "#define ALL_AUDIT_OPS\tALL_AUDIT_OPS_S0\n";
      return src;
};

source PEGram_WDL::gen_decl()

{
      source src; 
      src << topdecl;
      src << "     typedef struct _lenv {\n";
      src << lenvdecl;
      src << "     } LocalEnv;\n";
      src << envdecl_1;
      /* add declaration of environment variables */
      src << envdecl; 
      src << envdecl_2;
      src << getsrc_fromfile("localfuncs.c");
      return src;
}


source PEGram_WDL::gen_subfunctbl()

{
      int i,j;
      source src;
      src.gen_comment("Sub Function Table"); 
      src << "int (*subFuncTbl[NO_OF_SUBFUNCS])();\n";
      return src;
}

source PEGram_WDL::gen_subfunc()

{
      source src; 
      source atsrc;
      int i,j;

      /*
       * Generate Action functions
       */
      
      GSymList::iterator giter;
      for (giter = alist.begin(); giter != alist.end(); giter++) {
           atsrc << "static int " << "FUNC_" << (*giter)->cname;
           atsrc << "(ThParser *tp, int token, int *accept, "
		 	  << "Env *env, LocalEnv *lenv)\n";
           atsrc << *((Action *)(*giter))->src << "\n";
      }; 
      atsrc.indent();
      src << atsrc;
      /*
       * Generate NTerm functions
       * Right now only generate simple case <nterm>, need to implement
       * <nterm, 1, 3,..>
       */
      for (i=0; i<nvars; i++) {
           NTerm *n = (NTerm *) idtbl[i+nterm+nvt];
           source body, sbody; 
           for (j=0; j<nterm+nvt; j++) {
                if (opt_ptable[i][j] != NULL) {
                     source sbody1; 
                     source sbody2; 
                     sbody << "case " << idtbl[j]->cname << ":\n";
				 // if (!sbody.is_empty()) sbody << "} else ";
                     // sbody << "if (token == " << idtbl[j]->cname << ") {\n";
                     GSymList *rhs = opt_ptable[i][j]->rhs;
                     GSymList::iterator iter = rhs->begin();
                     sbody1 << "tp->top--;\n";
                     while (iter != rhs->end()) {
                          if ((*iter)->objtype() == GSym::TERM) { 
                               if ((*iter)->id != j) {
                                    cerr << "Internal Error #3\n";
                                    exit(0);
                               };
                               sbody1 << "*accept = 1;\n";
                               iter++;
                               break;
                          } else if ((*iter)->objtype() == GSym::NTERM) { 
                               break;
                          } else {  /* action */
                               sbody1 << "FUNC_" << (*iter)->cname << "();\n";
                               iter++;
                          };
                     }
                     while (iter != rhs->end() && (*iter)->objtype() == GSym::ACTION) {
                          sbody1 << "FUNC_" << (*iter)->cname
                                 <<  "(tp, token, accept, env, lenv);\n";
                          iter++;
                     }
                     while (iter != rhs->end()) {
                          source s;
                          s << "tp->stack[++tp->top] = " << (*iter)->cname << ";\n";
                          sbody2.prepend(s.all());
                          iter++;
                     }
                     sbody2 << "return P_SUCC;\n";
                     sbody1.indent();   
                     sbody2.indent();   
                     sbody << sbody1 << sbody2; 
                }
           }
           if (!sbody.is_empty()) {
           	sbody << "default:\n";
               sbody << "     return P_FAIL;\n";
               body << "switch(token) {\n" << sbody << "}\n";
			/*  if code
                sbody << "} else \n";
			 sbody << "    return P_FAIL;\n";
		      body << sbody; */
           }
           src.gen_nterm_funcdecl("FUNC_" + n->cname);
           body.indent();
           src << "{\n" << body << "}\n";
      }
      return src;
}

source PEGram_WDL::gen_parsefunc()

{
      return source(parsefunc);
};

/** Generate all the functions for acceptance a token
 *
 *  Return a source object that contain the function code
 */
source *PEGram_WDL::gen_tokenfuncs() 

{
	int i,j;
    source *src; 
    for (i = nvt; i < nterm + nvt; i++) {
         TermDef *td = (*(tdtbl.find(idtbl[i]->name))).second;
         source cond; 
         /* generate the condition in which the terminal is viable */
         cond << "ts == " << i; 
         for (j = 0; j < nvars; j++) {
              if (!ptable[j][i].empty()) {
                   cond << " || ts == " << j+nterm+nvt; 
              }
         }
         *src << gen_tokenfunc(td, &cond);
    };
    src->indent();
    return src; 
       
}


source *PEGram_WDL::gencode()

{
     int i;
     source *src = new source;
     *src << gen_define() << "\n";
	*src << defstr << "\n";
	*src << gen_define_auditops() << "\n";
     *src << "wrapper " << name << " {\n";

     *src << gen_decl() 		<< "\n";
     *src << gen_subfunc() 	<< "\n";
     *src << gen_subfunctbl() << "\n";
     *src << gen_parsefunc() 	<< "\n";
     *src << wrdecl;

     /* generate wr_install */
     *src << wr_install_begin;
     for (i=0; i<nvars; i++) {
          *src << "\t subFuncTbl[" << i << "] = &FUNC_" 
               << idtbl[i+nterm+nvt]->cname << ";\n";
     }; 
     i = nvars;
     GSymList::iterator giter;
     for (giter = alist.begin(); giter != alist.end(); giter++) {
          *src << "\t subFuncTbl[" << i++ << "] = &FUNC_" 
               << (*giter)->cname << ";\n";
     }; 
     *src << "\t subFuncTbl[" << i++ << "] = &FUNC_Error;\n";
     *src << wr_install_end;

     // TBD about init and fini
     init.indent("       ");
     *src << wr_activate_begin 
		<< envinit 
		<< init 
		<< wr_activate_end;
     *src << wr_duplicate_begin << wr_duplicate_end;


     if (debugCode) 
         *src << wr_debugclause;  

     *src << wr_mustwrapfunc_pre;

     *src << wr_parseerror_generic;

     src->append(gen_errorclause()); 

     src->append(gen_tokenfuncs());

     *src << getsrc_fromfile("postclause.c");
     src->append("}\n");
     return src;
}

source *PEGram_WDL::gen_errorclause()

{
     source *src = new source;
     SelList::iterator siter;
     for (siter = selector->begin(); siter != selector->end(); siter++) {
          AEvSet *eset = (*siter)->evset;
          AEvSet::iterator iter;
          for (iter = eset->begin(); iter != eset->end(); iter++) {
              string ename = (*iter).sevent->name[0] != '^' ?
                                 (*iter).sevent->name : 
                                 &(*iter).sevent->name[1];
              if (ename == "read") {
                  *src << wr_parseerror_read;
              } else if (ename == "write") {
                  *src << wr_parseerror_write;
              } else if (ename == "open") {
                  *src << wr_parseerror_open;
              } else if (ename == "close") {
                  *src << wr_parseerror_close;
              } else if (ename == "execve") {
                  *src << wr_parseerror_execve;
              } else if (ename == "chmod") {
                  *src << wr_parseerror_chmod;
              } else if (ename == "fchmod") {
                  *src << wr_parseerror_fchmod;
              } else if (ename == "rename") {
                  *src << wr_parseerror_rename;
              }
          }
     }
     return src;
}



source PEGram_WDL::gen_tokenfunc(TermDef *td, source *cond)

{
	source src;
	if (td->texpr->etype() != Expr::TermDef) {
		cerr << "Internal Error #11 (TermDefExpr Expected)\n";
		return source();
	}
	
	TermDefExpr *te = (TermDefExpr *) td->texpr;	
	source ifbody;

	char strbuf[512];
	bzero(strbuf,512);
	ostrstream sstr(strbuf, 512);
	sstr << *te;
	src.gen_comment("Generated Code for ", sstr.str());
	Expr *l;
	if (te->type == 1) { /* positive */
		for (l = te->elist; l != NULL; l = l->next) {
			// TBD: add checking code
			src << gen_wdlclause(td, (EvSelExpr *) l, cond);
		}
	} else { 	/* negative termdef expression */
		/* the way we generate the wrapper action to recognize
		 * a negative tk expression ~(open, $path == "/etc/passwd")
		 * is to generate two wrapper actions. The first intercept
		 * the event (open) and recognize the token in the condition
		 * fail, the second recognize the rest of the events.
		 */
		
          /* generate the first wrapper action, in the case of
           * ~(event), don't generate this action */
		for (l = te->elist; l != NULL; l = l->next) {
			source init;
			EvSelExpr *se = (EvSelExpr *) l;
		}
/*			if (*se->cond != ConstExpr(1)) {
				init << "int ctok = TERM_" << name << ";\n";
				init.indent(); 

				source s, s1;
				s1 << "if (!" << *se->cond->code << ") {\n" << ifbody << "}\n";
				s1.indent(); 
				s  << "int ts = tp->stack[tp->top];\n";
				s  << "FileTk file;\n";
				s  << "ProcTk proc;\n";
				s  << "struct wr_stat s;\n";
				s  << "if (" << cond->all() << ") {\n" << init << s1 << "}\n";
				s.indent();
				out  << "WR_LOCAL::op{" << se->aevent.getSysOpName() 
				     << "} post {\n" << s <<  "};\n";
			};
              
			
			source s2;
			// generate the second wrapper action
			s2 << "int ts = tp->stack[tp->top];\n";
			s2 << "if (" << cond->all() << ") {\n" 
 			  << "     int ctok = TERM_" << name << ";\n" << ifbody << "}\n";
			s2.indent(); 
			out << "WR_LOCAL::oattr{auditop} && !op{" << se->aevent.getSysOpName()
                   << "} post {\n" << s2 << "};\n";
        		out << "\n";
		}
		*/
     }
     return src;
};

/** Generate WDL clause for the given EvSelExpr */
source PEGram_WDL::gen_wdlclause(TermDef *td, EvSelExpr *se, source *cond)
{
	source 	src;
	source 	ifbody;
  	AttrList	attlist = td->getAttrRefList(se);
  	
  	StrSet::iterator 	iter;
  	AttrList::iterator 	aiter;

  	// ifbody << "wr_printf(\"%s: Token is TERM_" << td->name << "\\n\",$$);\n";
	ifbody << "parse_token(tp, ctok, tp->env, &tp->lenv);\n"
		  << "wr_skip();\n"
		  << "if (tp->env->H.errno != 0) $errno = tp->env->H.errno;\n";
	ifbody.indent();
		  			
  	/* if the audit event includes a pre event */
  	if (se->aevent.isPreEvent())	{
  		StrSet initcode;
  		for (aiter = attlist.begin(); aiter != attlist.end(); aiter++) {
  			initcode.insert((*aiter)->getPreCode().begin(), (*aiter)->getPreCode().end());	
  		}
  		if (!initcode.empty()) {
  			source isrc;
			isrc <<  "env->H.errno = $errno;\n"
				<<  "env->H.evid = $#;\n" 
				<<  "env->H.event = $$;\n"; 
  			for (iter = initcode.begin(); iter != initcode.end(); iter++) {
  				isrc << (*iter) << ";\n";
  			}
              	isrc.indent();
              	src  << "WR_LOCAL::op{" << se->aevent.getSysOpName() << "} pre {\n"
				<< "     int ts = tp->stack[tp->top];\n"
  				<< "     int ctok = TERM_" << td->name << ";\n"
                    << "     if (" << cond->all() << ") {\n"
                    <<             isrc
                    <<             ifbody.dup()
                    << "           if (tp->env.deny)\n"
				<< "                return WR_DENY;\n"
				<< "           else\n"
				<< "                return WR_ALLOW;\n"
                    << "     };\n"
                    << "};\n"
                    << "\n";
  		}
    	}

    	/* if the event includes a post event */
  	if (se->aevent.isPostEvent()) {
  		StrSet postAcode, postBcode;
  		for (aiter = attlist.begin(); aiter != attlist.end(); aiter++) {
  			postAcode.insert((*aiter)->getPostACode().begin(), (*aiter)->getPostACode().end());	
  			postBcode.insert((*aiter)->getPostBCode().begin(), (*aiter)->getPostBCode().end());	
  		}
  		if (!postAcode.empty()) {
  			source isrc;
  			for (iter = postAcode.begin(); iter != postAcode.end(); iter++) {
  				isrc << (*iter) << ";\n";
  			}
               isrc.indent();
               src  << "WR_LOCAL::op{" << se->aevent.getSysOpName() << "} pre {\n"
                	<< "     int ts = tp->stack[tp->top];\n"
                    << "     if (" << cond->all() << ") {\n" << isrc << "     };\n"
                    << "};\n"
                    << "\n";
		}

  		source isrc;
		isrc <<  "env->H.errno = $errno;\n"
			<<  "env->H.evid = $#;\n" 
			<<  "env->H.event = $$;\n" 
			<<  "env->H.time = wr_getcurtime(&env->H.usec);\n"; 
		for (iter = postBcode.begin(); iter != postBcode.end(); iter++) {
			isrc << (*iter) << ";\n";
		}
		isrc.indent();
		source s, s1;
		s1 << "if (" << *se->cond->code << ") {\n" << ifbody << "}\n";
		s1.indent();
		s  << "int ts = tp->stack[tp->top];\n"
		   << "     int ctok = TERM_" << td->name << ";\n"
		   << "if (" << cond->all() << ") {\n" << isrc << s1 << "}\n";
		s.indent();
		source esrc;
		if (se->aevent.getSysOpName() != "fork" &&
				se->aevent.getSysOpName() != "vfork" &&
				se->aevent.getSysOpName() != "fork1")
			esrc << "($errno == 0)";
		else
			esrc << "($errno == 0 && !gotFork++)";
		src  << "WR_LOCAL::op{" << se->aevent.getSysOpName() << "} " << esrc << " post {\n"
			<< s
			<<  "};\n";
	}
	return src;
};
