/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: linux.mod,v 1.7 2001/04/05 18:40:24 stevek Exp $
 *
 * The xxx.mod file definse the format of audit records for the recorded
 * events and provides linkage information for the event generator to
 * generate appropriate audit record from the intercepted system calls or
 * raw audit trails.
 */

#include  <sys/types.h>
#include  <sys/stat.h>
#include  <fcntl.h>

void sprintf();

#define WR_UNIX 1

#define STD_OPS \\
          {    exit, fork, kill, signal, \\
               read, write, pread, pwrite, readv, writev, \\
               open, close, creat, execve, \\
			lock, ioctl, fcntl, \\
               link, unlink, symlink, \\
               chmod, fchmod, chown, fchown, lchown, \\
               setuid, setgid, setreuid, setregid, \\
               setresuid, setresgid, setfsuid, \\
               rename, mkdir, rmdir, setgid, signal, \\
               select, bind, accept, connect, listen, \\
               recvfrom, sendto, recvmsg, sendmsg, send, recv \\
            }


/* Header Token */
typedef struct {
     int       time;          /* event timestamp, no of sec since Jan 1,1970 */
     int       usec;          /* millisecond */
     int       errno;         /* error number: 0 means successful */
	int		flag;		/* pre:0 or post:1 */
     char *    event;         /* event name */
     int       evid;          /* event Id */
} HeaderTk;

/* Subj Token */
typedef struct {
	int		uid;			/* user id of the invoker 		*/
	int		gid;			/* group id of the invoker		*/
     char *    pgname;        /* name of the executing program   */
     char *    pgpath;        /* fullpath name of the program    */
} SubjTk;

/* Process Token */
typedef struct {
     int       tid;           /* thread Id                  */
     int       pid;           /* process Id                 */
     int       ppid;          /* parent process Id          */
     int       ruid;          /* real user Id               */
     int       euid;          /* effective user Id          */
     int       rgid;          /* real group Id              */
     int       egid;          /* effective group Id         */
     char *    cdir;          /* current directory          */
} ProcTk;

/* Object Token */
typedef struct {
     int       error;         /* content valid ? 0 - valid		*/
     int       mode;       	/* inode protection mode 		*/
     int       ouid;        	/* user ID of the file's owner 	*/
     int       ogid;        	/* group ID of the file's group 	*/
     long      ino;        	/* inode's number 				*/
     int       nlink;      	/* number of hard links 			*/
     long		atime; 		/* time of last access 			*/
     long		mtime; 		/* time of last modification 		*/
     long      size;       	/* file size, in bytes 			*/
     long      gno;        	/* file generation number 		*/
     long      rdev;       	/* device ID (if file is character */
     char *  	path;		/* absolute path				*/

} FileTk, ObjTk;

/* Socket Token */
typedef struct {
     int       error;         /* content valid? 0 - valid */
     int       type;          /* type, TCP/UCP/UNIX    */
     int       afamily;       /* address family        */
     int       lport;         /* local port            */
     int       rport;         /* remote port           */
     unsigned  laddr;         /* local inet address    */
     unsigned  raddr;         /* remote inet address   */
     char      name[128];     /* Whole connection name */
} SockTk;

#ifdef PEG_WDL
#include "linux-wdl.ich"
#endif

#ifdef PEG_CPP
#include "linux-cpp.ich"
#endif

#ifdef PEG_WMO
#include "linux-wmo.ich"
#endif

#ifdef PEG_LKM
#include "linux-lkm.ich"
#endif

#ifdef PEG_GENERIC

module Generic {

/* definition of a system call and the attributes */
/* In general, F, P, S contain the attributes of the file 
 * 2) before the operation   if the event is a pre system-call event
 * 1) after the operation    if the event is a post system-call event
 */


/* correspond to open and creat system call 		*/
sysop open {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
     FileTk    F;			/* the object being accessed 	*/

	int		a_path;		/* 1st argument, path		*/
	int		path;		/* 1st argument, path		*/
     int  	a_mode;		/* 3rd argument, mode 		*/
     int 		a_flag;		/* 2nd argument, flag 		*/
     int 		fd;			/* return value, fd			*/
     int 		cflag;		/* creation flag			*/

	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Open %s", $F.path);
	};
};

sysop creat {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
     FileTk    F;			/* the object being accessed 	*/

	int		a_path;		/* 1st argument, path		*/
	int		path;		/* 1st argument, path		*/
     int  	a_mode;		/* 3rd argument, mode 		*/
     int 		a_flag;		/* 2nd argument, flag 		*/
     int 		fd;			/* return value, fd			*/
     int 		cflag;		/* creation flag			*/

	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Open %s", $F.path);
	};
};

sysop close {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Close %s", $F.path);
	};
};

sysop bind {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	int sprintf(char *buf, int size) {
		sprintf(buf, "Bind %s", $K.name);
	};
};

sysop accept {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	int sprintf(char *buf, int size) {
		sprintf(buf, "Bind %s", $K.name);
	};
};

sysop connect {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	int sprintf(char *buf, int size) {
		sprintf(buf, "Connect %s", $K.name);
	};
};

sysop read {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Read %s", $F.path);
	};	
};

sysop write {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/
	SockTk	K;			/* Socket info				*/

	int		fd;			/* file descriptor			*/
	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Write %s", $F.path);
	};	
};

sysop truncate {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/

	char *	a_path;		/* 1st argument: path 		*/
	int		a_len;		/* 2nd argument: length 		*/
};

sysop ftruncate {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/

	int		fd;			/* 1st argument: fd			*/
	int		a_len;		/* 2nd argument: length 		*/
};

sysop execve {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 				*/
     ProcTk    P;			/* process 				*/
     FileTk    F;			/* the object being accessed 	*/

	int		fd;			/* file descriptor			*/
	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Execve %s", $F.path);
	};	
};

sysop rename {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the from-object */
     FileTk    F1;			/* the attribute of the to-object before the op */
	char	*	oldpath;		/* 1st Argument 				*/
	char	*	newpath;		/* 2nd Argument 				*/
};

sysop link {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object 	*/
	char	*	oldpath;		/* 1st Argument 				*/
	char	*	newpath;		/* 2nd Argument 				*/
};

sysop unlink {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object 	*/
	char	*	path;		/* 1st Argument 				*/
};

sysop symlink {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object 	*/
	char	*	oldpath;		/* 1st Argument 				*/
	char	*	newpath;		/* 2nd Argument 				*/
};

sysop chmod {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object  	*/
	int		oldmode;		/* permission mode of the object before the op */
	int		pathname;		/* 1st argument 				*/
	int		newmode;		/* 2nd argument 				*/
};

sysop fchmod {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object 	*/
	int		oldmode;		/* permission mode of the object before the op */
	int		newmode;		/* 2nd argument 				*/
	int		fd;			/* file descriptor 				*/
};

sysop chown {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object  	*/
	int		oldouid;		/* ouid of the object before the op */
	int		pathname;		/* 1st argument 				*/
	int		newouid;		/* 2nd argument 				*/
};

sysop fchown {
     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject 					*/
     ProcTk    P;			/* process 					*/
     FileTk    F;			/* the attribute of the object 	*/
	int		oldouid;		/* ouid of the object before the op */
	int		fd;			/* file descriptor, 1st argument 	*/
	int		newouid;		/* 2nd argument 				*/
};

sysop mkdir {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
     FileTk    F;			/* the object being accessed 	*/

	int		a_path;		/* 1st argument, path		*/
     int  	a_mode;		/* 2nd argument, mode 		*/
	/* member function */
	int sprint(char *buf, int size) {
		sprintf(buf, "Mkdir %s", $F.path);
	};
};

sysop mknod {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
     FileTk    F;			/* the object being accessed 	*/

	int		a_path;		/* 1st argument, path		*/
     int  	a_mode;		/* 2nd argument, mode 		*/
	int		a_dev;		/* 3rd argument, dev 		*/
	/* member function */
	int sprintf(char *buf, int size) {
		sprintf(buf, "Mknod %s", $F.path);
	};
};

sysop setuid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_uid;		/* 1st argument, uid		*/
	int		olduid;		/* old effective uid of the process */
};

sysop seteuid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_euid;		/* 1st argument, euid			*/
	int		oldeuid;		/* old effective uid of the process */
};

sysop setreuid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_ruid;		/* 1st argument, ruid			*/
	int		a_euid;		/* 2nd argument, euid			*/
	int		oldruid;		/* old real uid of the proc		*/
	int		oldeuid;		/* old effective uid of the proc	*/
};

sysop setresuid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_ruid;		/* 1st argument, ruid			*/
	int		a_euid;		/* 2nd argument, euid			*/
	int		a_suid;		/* 3rd argument, suid			*/

	int		oldruid;		/* old real gid of the proc		*/
	int		oldeuid;		/* old effective uid of the proc 	*/
	int		oldsuid;		/* old save uid of the process 	*/
};

sysop setgid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_gid;		/* 1st argument, gid		*/
	int		oldgid;		/* old effective gid of the process */
};

sysop setegid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_egid;		/* 1st argument, egid			*/
	int		oldegid;		/* old effective gid of the process */
};

sysop setregid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_rgid;		/* 1st argument, rgid			*/
	int		a_egid;		/* 2nd argument, egid			*/
	int		oldegid;		/* old effective gid of the proc	*/
	int		oldrgid;		/* old real gid of the process 	*/
};

sysop setresgid {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */

	int		a_rgid;		/* 1st argument, rgid			*/
	int		a_egid;		/* 2nd argument, egid			*/
	int		a_sgid;		/* 3rd argument, sgid			*/
	int		oldegid;		/* old effective gid of the proc	*/
	int		oldrgid;		/* old real gid of the process 	*/
	int		oldsgid;		/* old save gid of the process 	*/
};

sysop socket {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
	SockTk	K;			/* Socket info				*/

	int		a_domain;		/* 1st argument, domain			*/
	int		a_type;		/* 2nd argument, type			*/
	int		a_proto;		/* 3rd argument, protocol 		*/

};

sysop connect {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
	SockTk	K;			/* Socket info				*/
	int		fd;			/* 1st argument, fd			*/
						/* 2nd argument, sockaddr	*/
	int		addrlen;		/* address len				*/	
};

sysop bind {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
	SockTk	K;			/* Socket info				*/
	int		fd;			/* 1st argument, fd			*/
						/* 2nd argument, sockaddr	*/
	int		addrlen;		/* address len				*/	
};

sysop accept {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
	SockTk	K;			/* Socket info				*/
	int		fd;			/* 1st argument, fd			*/
						/* 2nd argument, sockaddr	*/
	int		addrlen;		/* address len				*/	
};

sysop listen {

     HeaderTk  H;			/* generic system call information */
	SubjTk	S;			/* subject */
     ProcTk    P;			/* process */
	SockTk	K;			/* Socket info				*/
	int		fd;			/* 1st argument, fd			*/
	int		backlog; 		/* 2nd argument, backlog		*/
};

}

#endif
