   /*********************************************************************
    *                          LOCAL FUNCTIONS                          *
    *********************************************************************/


	static void initF_Path(FileTk *f, const char *p) {
		struct wr_stat s;
		f->error = wr_stat(p, &s); 
		f->mode = s.st_mode;
		f->ouid = s.st_uid;
		f->ogid = s.st_gid;
		f->ino  = s.st_ino;
		f->nlink = s.st_nlink;	 
		f->atime = s.st_atimespec.tv_sec;
		f->mtime = s.st_mtimespec.tv_sec;
		f->size = s.st_size;
		f->gno = s.st_gen;
		f->rdev = s.st_rdev;
		f->path = wr_abspath(p);
	};


	char sflag[16];
	char *symflag(int flag)
	{    sflag[0] = '\0';
		if ((flag & O_ACCMODE) == O_RDONLY) {
			wr_strcat(sflag, "r");
		} else if ((flag & O_ACCMODE) == O_WRONLY) {
			wr_strcat(sflag, "w");
		} else {
			wr_strcat(sflag, "rw");
		}
		if (flag & O_TRUNC)
			wr_strcat(sflag, "t");
		if (flag & O_CREAT)
			wr_strcat(sflag, "c");
		if (flag & O_EXCL)
			wr_strcat(sflag, "x");
		if (flag & O_APPEND)
			wr_strcat(sflag, "a");
		return sflag;
	}

	int created(int pid, FileTk *f)
	{
		DBROW::file_table row;
		int i;
		int inode = f->ino;
		i = wql { select into row from filetbl
                        where .ino = inode and .cpid = pid;    };
		return i == 0 ? 0 : 1;
	}

    int createdBySubj(int sid, FileTk *f)
    {
         DBROW::file_table row;
         int i;
         int inode = f->ino;
         int pid = sid + 100000;
         i = wql { select into row from filetbl
                        where .ino = inode and .cpid = pid;    };
         return i == 0 ? 0 : 1;
    }
    /* return 1 if path p is inside dir d */
    int inside(char *p, char *d)
    {
         if (p =~ m|d|)
              return 1;
         else
              return 0;
    }

     /* name have the form 124.10.1.1::port */
     int extractPort(char *name) 
     {
          char *c = name;
          while (c != NULL && *c != '\0' && *c != ':') c++;
          if (c != NULL && *c == ':') {
               return wr_atoi(c+2);
          }
          else 
               return 0;
     }

     char iaddrbuf[128];
     char *getSockAddr(struct sockaddr *saddr, int len, u_int *addr, int *port)
     {
          if (len <= 2) {
               wr_strcpy(iaddrbuf, "");
               *port = 0;
               *addr = 0;
               return iaddrbuf;
          };
          switch(saddr->sa_family) {
          case AF_UNIX:
              wr_strncpy(iaddrbuf, (char *) &saddr->sa_data, len-2);
              iaddrbuf[len-2] = '\0';
              *port = 0;
              *addr = 0;
              return iaddrbuf;
          case AF_INET: {
              struct sockaddr_in *iaddr = (struct sockaddr_in *) saddr;
              unsigned char *ucp = (unsigned char *) &(iaddr->sin_addr.s_addr);
              wr_sprintf(iaddrbuf, "%d.%d.%d.%d", ucp[0] & 0xff, ucp[1] & 0xff,
                                              ucp[2] & 0xff, ucp[3] & 0xff);
              *port = wr_ntohs(iaddr->sin_port);
              *addr =  iaddr->sin_addr.s_addr;
              return iaddrbuf;
            }
          };
     }

     char *ntoa(unsigned int in, char *buf) 
     {
          unsigned char *c = (unsigned char *) &in;
          wr_sprintf(buf, "%d.%d.%d.%d", c[0], c[1], c[2], c[3]);
          return buf;
     } 

     char *otname[8] = { "DIR", "CHR", "BLK", "FIFO", 
                         "FILE", "SLNK", "SOCK", "NIL"};

     char *objType(int mode) 
     {
          int m = mode & S_IFMT;
          switch(m) {
              case S_IFDIR:  return otname[0];
              case S_IFCHR:  return otname[1];
              case S_IFBLK:  return otname[2];
              case S_IFIFO:  return otname[3];
              case S_IFREG:  return otname[4];
              case S_IFLNK:  return otname[5];
              case S_IFSOCK: return otname[6];
	      default:       return otname[7];
          }
     }

     static int new_tparser(int start, int pid)
	{
          tp->refcount--;
          tp = ThParser_new(an, start, pid);
     }

#ifdef TRACK_FD
   /* get the pathname from fd, used with fd_table  */
    static char *getpath(int fd)
    {    int i;
         DBROW::fd_table dbrow;
         i = wql { select into dbrow from fdtbl where .fd = fd; };
         if (i == 1) {
              dbrow.start();
              wr_strcpy(g_pathbuf, dbrow.name);
         } else
              wr_strcpy(g_pathbuf, "NotFound");
         return g_pathbuf;
    }

   /* get the socket info, if any, from fd */
    void getSock(int fd, SockTk *sock)
    {   int error, error1, len, len1;
        char buf[32], buf1[32], *n;
        struct sockaddr s, s1;
        wr_bzero(sock, sizeof(SockTk));
        error = wr_getsockname(fd, &s, &len);
        error1 = wr_getpeername(fd, &s1, &len1);
        sock->error = error != 0 ? error :
                          (error1 != 0 || error1 != ENOTCONN ? error1 : 0);
        sock->afamily = !error && len >= 2 ? s.sa_family :
                            (!error1 && len1 >= 2 ? s1.sa_family : (u_char) 0);
        switch(sock->afamily) {
        case AF_UNIX :
             wr_strcpy(sock->name, getSockAddr(&s1, len1,
                                      &sock->raddr, &sock->rport));
             return;
        case AF_INET :
             getSockAddr(&s, len, &sock->laddr, &sock->lport);
             getSockAddr(&s1, len1, &sock->raddr, &sock->rport);
             wr_sprintf(sock->name, "%s:%d<->%s:%d",
                          ntoa(sock->laddr, buf), sock->lport,
                          ntoa(sock->raddr, buf1), sock->rport);
             return;
        default      :
             wr_sprintf(sock->name, "AF_USPEC %d %d", sock->afamily, sock->error);
             return;
        }
    }
#endif /* TRACK_FD */

	void initK_Fd(SockTk *sock, int fd) 
     {	int error, error1, len, len1;
          char buf[32], buf1[32], *n;
          struct sockaddr s, s1;
          wr_bzero(sock, sizeof(SockTk));
          error = wr_getsockname(fd, &s, &len);
          error1 = wr_getpeername(fd, &s1, &len1);
          sock->error = error != 0 ? error :
                          (error1 != 0 || error1 != ENOTCONN ? error1 : 0);
          sock->afamily = !error && len >= 2 ? s.sa_family :
                            (!error1 && len1 >= 2 ? s1.sa_family : (u_char) 0);
          switch(sock->afamily) {
          case AF_UNIX :
             wr_strcpy(sock->name, getSockAddr(&s1, len1,
                                      &sock->raddr, &sock->rport));
             return;
          case AF_INET :
             getSockAddr(&s, len, &sock->laddr, &sock->lport);
             getSockAddr(&s1, len1, &sock->raddr, &sock->rport);
             wr_sprintf(sock->name, "%s:%d<->%s:%d",
                          ntoa(sock->laddr, buf), sock->lport,
                          ntoa(sock->raddr, buf1), sock->rport);
             return;
          default      :
             wr_sprintf(sock->name, "AF_USPEC");
             return;
          }
	};

	static void initF_Rename(FileTk *f, int errno, char *oldpath, char *newpath) {
		if (errno == 0)
			initF_Path(f, newpath);
		else 
			initF_Path(f, oldpath);
	};

	static void initF_Fd(FileTk *f, int fd) {
		struct wr_stat s;
		f->error = wr_fstat(fd, &s);
		f->mode = s.st_mode;
		f->ouid = s.st_uid;
		f->ogid = s.st_gid;
		f->ino  = s.st_ino;
		f->nlink = s.st_nlink;	 
		f->atime = s.st_atimespec.tv_sec;
		f->mtime = s.st_mtimespec.tv_sec;
		f->size = s.st_size;
		f->gno = s.st_gen;
		f->rdev = s.st_rdev;
		f->path = getpath(fd);
	};

	static void initF(FileTk *f, int errno, int fd, char *p) {
		if (errno == 0)
			initF_Fd(f, fd);
		else 
			initF_Path(f, p);

	};
