%{
/* Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: pegram.l,v 1.13 2001/04/05 18:40:24 stevek Exp $
 *
 *  Authors: Calvin Ko
 *
 *  Purpose: Scanner for PEgrammar compiler.
 * 
 */

#include <string.h>
#include "type.h"

/*
 *		STATE VARIABLES
 */
extern source *symbols; 
void count();

int comment_caller, wql_caller, tbldecl_caller, module_caller;
int attribute_caller, attribute_paren_count = 0;
int win_asm_caller;

int lineno=1;
int wql_braces = 0, module_braces = 0;
char inputfile[255];

int check_type();

#define MAX_INCLUDE_DEPTH 15
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

%}

%x	COMMENT
%x      WQL
%x	INCLUDE
%s      MODULE
%s	ATTRIBUTE

ID      [_a-zA-Z][a-zA-Z0-9_]*
DEC	[0-9]
PUNC	[.,;:()\\\'#%*+\-<>!/{}\[\]&|=\$?^~]
HEX	[0-9a-fA-F]
EXP	[Ee][+-]?{DEC}+
FS	(f|F|l|L)
IS	(u|U|l|L)*

LINENUM	^#\ {DEC}+\ \".+\".*
PRAGMA  ^#pragma\ .*
%%

    { }

<INITIAL,MODULE>{PRAGMA}   {} /* Ignore #pragma */

<INITIAL,MODULE>{LINENUM}	{
			char buff[256];
			sscanf(pegtext, "# %d %255s", &lineno, buff);

			/* the directive is immediately followed by a 
			 * newline, don't include this in the counter */
			lineno--;	

			/* don't copy the quotes around buff */
			strncpy(inputfile, &buff[1], strlen(buff)-2);
			inputfile[strlen(buff)-2] = 0;
		}

<INITIAL,MODULE>"/*" {
			comment_caller = YY_START;
			BEGIN COMMENT;
		}
                
<COMMENT>"*/"	{ BEGIN comment_caller; }

<COMMENT>.	{ /* copy into file */ }

<COMMENT><<EOF>>	{
			pegerror("Comment not terminated");
			yyterminate();
		}

module 		{   module_caller = YY_START;
		    BEGIN(MODULE);
		    count(); return K_MODULE;
 		}

<MODULE>\{	{   /* should not match in a quoted tring */
		    module_braces++;
		    count(); return '{';
		}

<MODULE>\}	{   module_braces--;
		    if (module_braces == 0) {
		        BEGIN module_caller;
		    }
		    count();
		    return '}';
		}

<INITIAL,MODULE>init	{   return K_INIT;	}
<MODULE>init_pre	{   	return K_INITPRE;	}
<MODULE>init_postA	{   	return K_INITPOSTA;	}
<MODULE>init_postB	{   	return K_INITPOSTB;	}
<MODULE>ref		{ 	return K_REF;		}
<INITIAL>fini	{   return K_FINI;	}
<INITIAL>err	{   return K_ERROR;	}

<INITIAL>"__attribute__" {
    		    attribute_caller = YY_START;
    		    BEGIN ATTRIBUTE;
		}

<ATTRIBUTE>"("  {   ++attribute_paren_count; }

<ATTRIBUTE>")"  {
                    --attribute_paren_count;
                    if (!attribute_paren_count) BEGIN attribute_caller;
                }
  
<ATTRIBUTE>ID  { }
<ATTRIBUTE>[^()\n]  { }

<ATTRIBUTE><<EOF>> { pegerror("attribute not terminated"); yyterminate(); }





SPEC|spec		{ count(); return K_SPEC; }
END			{ count(); return K_END;  }
ENV			{ count(); return K_ENV;  }
uses			{ count(); return K_USES; }
<INITIAL>includes	{ count(); return K_INCLUDES; }
<INITIAL,MODULE>float	{ count(); return K_FLOAT; }

opset		{ count(); return K_OPSET;  }
sysop		{ count(); return K_SYSOP;  }
double		{ count(); return K_DOUBLE; }
goto		{ count(); return K_GOTO; }
set		{ count(); return K_SET;  }
switch		{ count(); return K_SWITCH; }
sizeof		{ count(); return K_SIZEOF; }
case		{ count(); return K_CASE; }
default		{ count(); return K_DEFAULT; }
struct		{ count(); return K_STRUCT; }
union		{ count(); return K_UNION; }
enum		{ count(); return K_ENUM; }
if		{ count(); return K_IF; }
else		{ count(); return K_ELSE; }
do		{ count(); return K_DO; }
<INITIAL>for		{ count(); return K_FOR; }
while		{ count(); return K_WHILE; }
void		{ count(); return K_VOID; }
<INITIAL>return		{ count(); return K_RETURN; }
break		{ count(); return K_BREAK; }
continue	{ count(); return K_CONTINUE; }
__inline__	{ count(); return K___INLINE; }
__extension__	{ count(); }
__inline	{ count(); return K___INLINE; }
_inline	        { count(); return K___INLINE; }
__cdecl 	{ }
_cdecl 	        { }
__declspec\([^)]*\) 	{ }
__stdcall 	{ }
"->"		{ count(); return K_INDIRECT; }
signed		{ count(); return K_SIGNED; }
__signed	{ count(); return K_SIGNED; }   // for parsing stdio.h (Calvin)
short		{ count(); return K_SHORT; }
unsigned	{ count(); return K_UNSIGNED; }
<INITIAL,MODULE>int		{ count(); return K_INT; }
long		{ count(); return K_LONG; }
__int64		{ count(); return K_LONGLONG; }
%string		{ count(); return K_STR;  }
char	        { count(); return K_CHAR; }
auto		{ count(); return K_AUTO; }
extern		{ count(); return K_EXTERN; }
register	{ count(); return K_REGISTER; }
static		{ count(); return K_STATIC; }
local		{ count(); return K_LOCAL; }
typedef		{ count(); return K_TYPEDEF; }
const		{ count(); return K_CONST; }
__const		{ count(); }
volatile	{ count(); return K_VOLATILE; }
delete		{ count(); return K_DELETE; }
SE		{ count(); return K_SE; }

":-"		{ count(); return K_COND;     }
"=~"		{ count(); return K_STRMATCH; }
"@@"		{ count(); return K_STRMATCH; }
"+="		{ count(); return K_PLUSEQ; }
"-="		{ count(); return K_MINUSEQ; }
"*="		{ count(); return K_MULTEQ; }
"/="		{ count(); return K_DIVEQ; }
"%="		{ count(); return K_MODEQ; }
"<<="		{ count(); return K_SHFTLEQ; }
">>="		{ count(); return K_SHFTREQ; }
"&="		{ count(); return K_ANDEQ; }
"|="		{ count(); return K_OREQ; }
"^="		{ count(); return K_XOREQ; }
"||"		{ count(); return K_OR; }
"&&"		{ count(); return K_AND; }
"=="		{ count(); return K_EQEQ; }
"!="		{ count(); return K_NOTEQ; }
"<="		{ count(); return K_LTEQ; }
">="		{ count(); return K_GTEQ; }
">>"		{ count(); return K_SHFTR; }
"<<"		{ count(); return K_SHFTL; }
"..."		{ count(); return K_ELIPSIS; }
"++"		{ count(); return K_PLUSPLUS; }
"--"		{ count(); return K_MINUSMINUS; }

Term|TERM	{ count(); return K_TERM; }

<INITIAL,WQL>DBROW      { count(); return K_DBROW; }
<INITIAL,WQL>DBTABLE    {
                            count();
                            tbldecl_caller = YY_START;
                            return K_DBTABLE;
                        }

"#include"      { BEGIN INCLUDE; /* TBD: remove this? */}

wql|WQL		{
			wql_caller = YY_START;
			BEGIN(WQL);
			count(); return K_WQL;
		}

<WQL>create|CREATE	{ count(); return K_CREATE; }
<WQL>drop|DROP		{ count(); return K_DROP; }
<WQL>insert|INSERT	{ count(); return K_INSERT; }
<WQL>select|SELECT	{ count(); return K_SELECT; }
<WQL>delete|DELETE	{ count(); return K_DELETE; }
<WQL>update|UPDATE	{ count(); return K_UPDATE; }

<WQL>set|SET		{ count(); return K_SET; }
<WQL>table|TABLE	{ count(); return K_TABLE; }
<WQL>values|VALUES	{ count(); return K_VALUES; }
<WQL>into|INTO		{ count(); return K_INTO; }
<WQL>first|FIRST	{ count(); return K_FIRST; }
<WQL>from|FROM		{ count(); return K_FROM; }
<WQL>where|WHERE	{ count(); return K_WHERE; }
<WQL>order|ORDER	{ count(); return K_ORDER; }
<WQL>by|BY		{ count(); return K_BY; }
<WQL>desc|DESC		{ count(); return K_DESC; }
<WQL>like|LIKE		{ count(); return K_LIKE; }
<WQL>and|AND		{ count(); return K_WQL_AND; }
<WQL>or|OR		{ count(); return K_WQL_OR; }
<WQL>\{			{ /* should not match in a quoted string */
				wql_braces++;
				count(); return '{';
			}
<WQL>\}			{
				wql_braces--;
				if (wql_braces == 0) {
					BEGIN wql_caller;
				}
				count();
				return '}';
			}

<INCLUDE>[ \t]*		/* eat the whitespace */
<INCLUDE>(\"|<)[^ \t\n]+(\"|<) {
		   char *filename = strdup(pegtext);
		   filename++; 
		   filename[strlen(filename)-1] = 0;
                   if ( include_stack_ptr >= MAX_INCLUDE_DEPTH )
                       {
                       fprintf( stderr, "Includes nested too deeply" );
                       exit( 1 );
                       }

                   include_stack[include_stack_ptr++] =
                       YY_CURRENT_BUFFER;

                   pegin = fopen( filename, "r" );

                   if ( ! pegin ) {
                       fprintf(stderr, "Couldn't open include file '%s'\n", 
				filename );
		       exit(2);
		   }


                   yy_switch_to_buffer(
                       yy_create_buffer( pegin, YY_BUF_SIZE ) );

                   BEGIN(INITIAL);
                   }


<INITIAL,MODULE,WQL>L?\"(\\.|[^\\"])*\" {
                                /* peglval.string = strdup(pegtext); */
				int nchars=strlen(pegtext)+sizeof("PD_MAKESTR(")+2;
                                peglval.cstr = (char *)malloc(nchars);
                                snprintf(peglval.cstr, nchars, "PD_MAKESTR(%s)", pegtext);
                                count(); return(S_QSTR); 
                        }

<INITIAL,MODULE,WQL>{PUNC}	{ count(); return pegtext[0]; }

0[xX]{HEX}+{IS}?	{ 
				peglval.cstr = strdup(pegtext);
                                count(); return(S_NUMCONST);
 			}
0{DEC}+{IS}?		{
				peglval.cstr = strdup(pegtext);
                                count(); return(S_NUMCONST);
			}
<MODULE,INITIAL,WQL>{DEC}+{IS}?	{
				peglval.cstr = strdup(pegtext);
                                count(); return(S_NUMCONST);
			}

<MODULE,INITIAL,WQL>'(\\.|[^\\'])+'	{
                                peglval.cstr = strdup(pegtext);
                                count(); return(S_CHARCONST);
                        }

<MODULE,INITIAL,WQL>{DEC}+{EXP}{FS}?	{
			peglval.cstr = strdup(pegtext);
                        count(); return S_FLOATCONST; }
<INITIAL,MODULE,WQL>{DEC}*"."{DEC}+({EXP})?{FS}?	{
			peglval.cstr = strdup(pegtext); 
			count(); return S_FLOATCONST;  }
<INITIAL,MODULE,WQL>{DEC}+"."{DEC}*({EXP})?{FS}?	{
			peglval.cstr = strdup(pegtext);
			count(); return S_FLOATCONST;  }

<INITIAL,MODULE,WQL>{ID}	{ count(); return (check_type()); }

<<EOF>>		{ if ( --include_stack_ptr < 0) 
			yyterminate();
		  else {
			yy_delete_buffer( YY_CURRENT_BUFFER );
			yy_switch_to_buffer(include_stack[include_stack_ptr]);
		  }
		}

<*>[\n]		{ count(); lineno++; }

<*>[ \t]	{ count(); /* do something? */ }

.		{ count(); /* pegwarning("Character not recognized"); */ }

%%
int column = 0;
char lastline[2048] = {0}, thisline[2048]; /* TBD: evil hardcoded array */
int lastlinec = 0;

/* The following function should be called for every match above.  A
 * convenient location is just prior to the token return.  It counts
 * the characters in the buffer and stores the characters in two line
 * buffers. It stores the previous input line and the current line (up
 * to the last token received).  These two buffer lines are only used
 * in the error routine; they provide context for the error.
 * Also stores the location of the current input column in the global
 * variable column. */
void
count()
{
	int i;
	for (i = 0; pegtext[i] != '\0'; i++) {
		/* At the end of a line: bump the current line to the
		 * previous line, clear and reset the current line */
		if (pegtext[i] == '\n') {
			thisline[column] = pegtext[i];
			thisline[column+1] = 0;
/* 			printf("LINE: %s", thisline); */
			strcpy(lastline, thisline);
			/* TBD: remove line length constraint? */
// Not on NT...		bzero(thisline, 2048*sizeof(char)); 
			memset(thisline, 0, 2048*sizeof(char)); 
			column = 0;
		}
		/* tabs are assumed to be 8 spaces */
		else if (pegtext[i] == '\t') {
			strcpy(&thisline[column], "        ");
			column += 8 - (column % 8);
		}
		/* All other characters are copied and the column
		 * counter is incremented */
		else {
			thisline[column] = pegtext[i];
			column++;
		}
	}
}

/* Prints the previous input line, the current input line (up to the
 * last token received) and a message from the parameter s.  Uses the
 * column global variable to "point" at the location at which the
 * error occurred. */
void 
pegerror(const char *s, const char *s1="")
{
	thisline[column+1] = 0;
	fprintf(stderr, "%s%s",lastline, thisline);
        fprintf(stderr, "\n%*s\n%*s %s at line %d of %s\n", column, "^", 
			column, s, s1, lineno, inputfile?inputfile:"stdin");

        exit(2);
}

/* Same as pegerror, except it doesn't abort.
 */
int
pegwarning(const char *s, const char *s1="")
{
	thisline[column+1] = 0;
	fprintf(stderr, "%s%s",lastline, thisline);
        fprintf(stderr, "\n%*s\nWarning: %*s %s at line %d of %s\n",
                column, "^", column-9, s, s1,
		lineno, inputfile?inputfile:"stdin");
        return 1;
}

int
pegwrap()
{
        return (1);
}

/* Looks up the symbol in the symbol table.  Symbols are added to the
 * table in typedef statements.  This scanner needs to differentiate
 * between identifiers and typdef'd names in order for C to be
 * parseable by YACC. */
int check_type()
{
	peglval.cstr = strdup(pegtext);

        if (vstore.get_type(pegtext)) {
	   return(S_TYPENAME);
        }

        return(S_IDENT);
}
