%{
/* Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: pegram.y,v 1.27 2001/04/05 18:40:24 stevek Exp $
 *
 * Authors: Calvin Ko
 *
 * Purpose: Parser for Parallel Environment Grammars
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <string>

#include "conf.h"

#ifdef WR_UNIX
#include <iterator.h>
#include <pair.h>
#endif

#ifdef WR_WINNT
#include <iterator>
#include <utility>
#endif

#include <set>
#include <list>

#ifdef WR_UNIX
#include <algo.h>
#endif
#ifdef WR_WINNT
#include <algorithm>
#include <vector>
#endif

/* #include "wr_string.h" */

#include "type.h"
#include "expr.h"
#include "gram.h"
#include "smodule.h"
#include "pegc.h"
#include "code-wdl.h"
#include "code-cpp.h"

/* global for holding the current section */
int	in_termdef = 0;
int	in_progspec = 0;
int	in_attrdecl = 0;

/* global for Variable Declaration */
VarStore vstore;

/* global for module definition */
Attr   	*curr_attr = NULL;		// hold the current attribute
SEvent 	*curr_sevent = NULL;	// hold the current sevent

/* global for pegram definition */
GSym 	*curr_gsym = NULL;		// hold the current grammer symbol
PEGram	*curr_pegram = NULL;	// hold the current PEgrammar
Rule 	*curr_rule = NULL;		// hold the current rule
SModule 	*curr_module = NULL;	// hold the current module
NTerm 	*crule_lhs = NULL;		// hold the lhs symbol in current rule
Type    	*curr_enumtype = NULL;	// hold the enumeration type

VarRefTbl *svar_tbl = NULL;		// hold the set of special variable
                                   // use in the action block or termdef

ModTbl  modtbl;
AEvSet  *curr_evset;			// hold the current set of selected events

extern void pegerror(const char *s, const char *s1="");
extern int pegwarning(const char *s, const char *s1="");
extern int peglex();
extern int lineno;

%}

%union {
	int	      num;
	char      *cstr;
	string    *str;
	source    *src;
	Type      *type;
	CStrList  *cstrl;
	Expr      *expr;
	Attr      *attr;
	GSym      *gsym;
	GSymList  *gsyml;
	NTerm     *nterm;
	AEvSet    *evset;
	SelList   *slist;
	Selector  *sel;
	AEvent    *ev;
	VarRefTbl *vrtbl;
}

%nonassoc LOWER_THAN_ELSE
%nonassoc K_ELSE    
                    
%token  <cstr>	S_IDENT S_TYPENAME
%token  <cstr>	S_QSTR S_NUMCONST S_FLOATCONST S_CHARCONST

%token  	K_INCLUDES K_INIT K_FINI K_INITPRE K_INITPOSTA K_INITPOSTB
%token	K_USES K_ERROR
%token	K_MODULE
%token  	K_SYSOP K_REF

/* Used for variable declarations */

%token  K_AUTO K_EXTERN K_REGISTER K_STATIC K_TYPEDEF K_GLOBAL K_LOCAL
%token  K_SHORT K_SHORTINT K_SIGNEDSHORT K_SIGNEDSHORTINT
%token  K_INT K_SIGNEDINT K_SIGNED K_UNSIGNEDINT
%token  K_LONG K_LONGLONG K_SIGNEDLONG
%token  K_UNSIGNED K_UNSIGNEDINT K_UNSIGNEDSHORT K_UNSIGNEDSHORTINT
%token  K_UNSIGNEDLONG 
%token  K_CHAR K_SIGNEDCHAR K_UNSIGNEDCHAR K_STR K_WSS_CHAR_T
%token  K_FLOAT K_DOUBLE K_LONGDOUBLE K_OPSET
%token  K_VOID K_STRUCT K_UNION
%token  K_CONST K_VOLATILE K___INLINE

/* For expressions */
%token  K_OR K_AND K_NOTEQ K_EQEQ K_LTEQ K_GTEQ K_SHFTL K_SHFTR
%token  K_PLUSEQ K_MINUSEQ K_MULTEQ K_DIVEQ K_MODEQ K_SHFTLEQ
%token  K_SHFTREQ K_ANDEQ K_OREQ K_XOREQ
%token  K_ELIPSIS K_COND
%token  K_STRMATCH
%token  K_PLUSPLUS K_MINUSMINUS

/* for statements */
%token  K_GOTO K_BREAK K_CONTINUE K_RETURN K_SWITCH K_CASE K_DEFAULT
%token  K_SIZEOF K_STRUCT K_UNION K_VOID K_ENUM K_IF K_ELSE K_DO K_WHILE
%token  K_FOR K_INDIRECT K_DELETE

/* WQL tokens */
%token  K_WQL K_CREATE K_DROP K_INSERT K_SELECT K_INTO K_VALUES
%token  K_DELETE K_UPDATE K_TABLE K_FROM K_WHERE K_LIKE K_ORDER K_BY
%token  K_WQL_OR K_WQL_AND K_SET K_DESC K_LOCAL K_TRY K_CATCH K_FIRST
%token  K_DBROW K_DBTABLE K_KEY
 

/* For PEGRAM */
%token	K_SPEC K_END K_ENV K_SE K_TERM

%type 	<cstr> 	assignment_op enumeration_tag equality_op 
%type 	<cstr> 	relational_op shift_op
%type 	<cstr> 	unary_op type_qualifier 

%type	<num>	attribute attributes
%type	<num>	markupstmt markupstmts modifier
%type	<str>	refname refnames primary_refname op_start
%type	<type>	floating_type integer_type string_type void_type

%type	<expr>	constant 
%type 	<ev>	event 

%type	<expr>	struct_declarator struct_declarator_list
%type	<expr>	initializer initializers
%type	<expr>	init_declarator init_declarators

%type	<src>   	abstract_declarator attr_follow
%type	<src>	compound_stmt compound_stmt_1 conditional_stmt 
%type	<src>   	declaration declaration_list declaration_spec
%type	<src>   	declaration_specifier declaration_specifiers
%type	<src>   	declarator direct_abstract_declarator direct_declarator 
%type	<src>	enumeration_constant
%type	<src>	enumeration_definition_list enumeration_type_specifier
%type	<src>	enumeration_type_definition enumeration_type_reference
%type	<src>	expression_stmt function_specifier
%type	<src>	function_definition function_declarator
%type 	<src> 	iterative_stmt 
%type 	<src> 	parameter_list parameter_type_list
%type 	<src> 	parameter_declaration pointer return_stmt
%type	<src>   	simple_type_specifier
%type	<src>  	statement statement_list
%type	<src>	struct_or_union 
%type	<src>	struct_or_union_definition
%type	<src>	struct_or_union_reference 
%type	<src>	struct_or_union_specifier 
%type	<src>	struct_declaration_body
%type	<src>	struct_declaration_list
%type	<src>	struct_declaration 
%type	<src>	type_definition type_name type_reference	
%type	<src>   	type_qualifiers
%type	<src>	var_storage_class	


%type <expr> additive_expression assignment_expression 
%type <expr> bitwise_and_expression bitwise_or_expression
%type <expr> bitwise_xor_expression conditional_expression
%type <expr> cast_expression condition_list constant_expression
%type <expr> equality_expression expression 
%type <expr> expression_list function_call
%type <expr> logical_and_expression
%type <expr> logical_or_expression multiplicative_expression
%type <expr> constant string_list 
%type <expr> postfix_expression primary_expression 
%type <expr> termdef_expr ptermdef_expr eventsel_expr
%type <expr> relational_expression shift_expression
%type <expr> unary_expression


/* PEGram Specification */
%type 	<str>	snotion
%type 	<gsym>	term rhs_sym action
%type	<gsyml> 	rhs_syms
%type	<expr>	hparam hparams
%type	<nterm> 	nonterm
%type   	<cstrl> 	metavars sidents
%type   	<evset> 	op_evsel event_selector event_list
%type   	<slist> 	select_expr selectors 
%type    	<sel>   	selector
%type	<evset>	sysop_selector

%%

specfile: block
	  ;
	
block  :  gstmts
	  ;
gstmts :  gstmt
	  |  gstmt gstmts
	  ;	
gstmt  :  module_spec {}
       |  declaration {}
	  |  function_definition {}
       |  includes {}
	  |  progspec {}
       |  error {
               pegerror("Error in attempting type-unification recovery.");
		}
	  |  ';' {}
	  ;

includes:  K_INCLUDES '{' '}'
	  ;

/* Module Specification */
module_spec
	  :	K_MODULE S_IDENT {
               curr_module = new SModule($2);
		}
		'{' module_body '}' {
			if (modtbl[$2] != NULL)
				pegerror("Duplicated Definitions of Module");
               else
                   	modtbl[$2] = curr_module;
	  	}
	  ;

module_body:
	   sysopdefs
	;

sysopdefs
	  :  sysopdef
	  |  sysopdef ';' sysopdefs
	  ;
	
sysopdef: K_SYSOP S_IDENT modifier
          {  	string s = $2;
           	if ($3 == 1) s = "^" + s;
               curr_sevent = new SEvent(s.c_str());
          }
	   	'{' attributes '}'
	   	{
               if (curr_module->insert(curr_sevent) == -1) {
               	pegerror("Duplicated Definitions of System Event", $2);
               };
        	}
	  |  {}
	  ;

modifier
	  :	'^' {   $$ = 1;  }
	  |  '+' {   $$ = 2;  }
	  |  '-' {   $$ = 3;  }
	  |  {   $$ = 0;  }
	  ;
	
attributes
	  :	attribute ';' attributes
	  |	attribute ';'
	  ;

attribute
	  :  type_reference declarator
		{	$2->type->fillin_type(*$1->type);
               curr_attr = new Attr($2->vname, $2->type);

               /* handle member function attribute 	*/
               if ($2->type->is_functype()) {
                   FuncTypeInfo *fti = (FuncTypeInfo *)$2->type->info;
                   if (fti->vtbl != NULL) {
                       vstore.pushtbl(fti->vtbl);
                   }
               }
               in_attrdecl = 1;
			vstore.newtbl();
		}
		attr_follow
		{	
			/* handle member function attribute 	*/
			if (curr_attr->type->is_functype()) {
                   FuncTypeInfo *fti = (FuncTypeInfo *) curr_attr->type->info;
                   if (fti->vtbl != NULL) {
                       vstore.poptbl();
                   }
              	}
              		
			if (curr_sevent->insert(curr_attr) == -1) {
                  	pegerror("Redefinition of Attribute", $2->vname);
               };
               in_attrdecl = 0;
               vstore.poptbl();

		}
	  ;

attr_follow
	  :	'{' markupstmts '}' {
	   	}
	  |  '{' declaration_list statement_list '}' {
			if (!curr_attr->type->is_functype()) {
				pegerror("Function type expected");
			}
               $2->append($3);
			curr_attr->fcode = $2;
          }
	  |  '{' statement_list '}' {
			curr_attr->fcode = $2;
          }
	  |  {}
	  ;

markupstmts:
	   markupstmt
	|  markupstmt ',' markupstmts
	;
	
markupstmt
	  : 	K_REF '{' refname '}' {
			curr_attr->tname = *$3;
			delete $3;
			$$ = 1;
	   	}
	  |  K_INIT  '{' refname '}' {
			curr_attr->init_code.insert(*$3);
               delete $3;
               $$ = 1;
	   	}
	  |  K_INITPRE  '{' refname '}' {
			curr_attr->initpre_code.insert(*$3);
			delete $3;
			$$ = 1;
		}
	  |  K_INITPOSTA  '{' refname '}' {
			curr_attr->initpostA_code.insert(*$3);
			delete $3;
			$$ = 1;
		}
	  |  K_INITPOSTB  '{' refname '}' {
			curr_attr->initpostB_code.insert(*$3);
			delete $3;
			$$ = 1;
		}
	  ;

refname
	  : '&' primary_refname {
			$$ = new string("&");
			$$->append(*$2);
			delete $2;
		}
	  |  primary_refname
	  |  primary_refname '=' refname {
               $$ = $1;
               $1->append(" = ");
               $1->append(*$3);
               delete $3
           }
	;

primary_refname	
	:  constant {
	        $$ = new string($1->str());
           }
	|  S_IDENT {
                $$ = new string($1);
           }
	|  primary_refname '.' S_IDENT {
                $$ = $1;
                $$->append(".");
                $$->append($3);
                delete $3; 
	   }
	|  primary_refname K_INDIRECT S_IDENT {
                $$ = $1;
                $$->append("->");
                $$->append($3);
                delete $3; 
	   }
	|  S_IDENT '(' refnames ')' {
                $$ = new string($1);
                $$->append("(");
                $$->append($3->c_str());
                $$->append(")");
                delete $3; 
           }
	|  S_IDENT '(' ')' {
                $$ = new string($1);
                $$->append("()");
           }
	|  S_IDENT '[' primary_refname ']' {
		$$ = new string($1);
                $$->append("[");
                $$->append($3->c_str());
                $$->append("]");
                delete $3; 
           }
	|  '$' '$' {
                $$ = new string("$$");
           }
	|  '$' '#' {
                $$ = new string("$#");
           }
	|  '$' S_IDENT {
                $$ = new string("$");
                $$->append($2);
                delete $2;
           }
	;

refnames:  refnames ',' refname {
                $$ = $1;
                $1->append(", ");
                $1->append($3->c_str());
                delete $3;
           }
	|  refname 
	;

/* End Module Specification */

/*********************************************************************
 *                     Declaration                                   *
 *********************************************************************/

declaration_list
	:  declaration 
	|  declaration declaration_list {  
               source *sp = new source;
               SourceAppend2(sp, $1, $2); 
               delete $1; delete $2; 
               $$=sp;
           } 
	;   

declaration
	:  declaration_spec init_declarators ';' {
               source *sp = new source;
               Expr *e;
               for (e = $2; e != NULL; e = e->next) {
                    DeclExpr *de = (DeclExpr *) e;
                    de->type.fillin_type(*$1->type);
		    VarNode *vn = new VarNode (de->vname, de->type, $1->num); 
                    if (!vstore.insVarInTopTbl(de->vname, vn))
                         pegerror("Duplicated definition of identifier");

	       }
               source tsrc(*$1);
               sp->append($1);
               for (e = $2; e != NULL; e = e->next) {
                    if (e != $2) sp->append(", ");
                    sp->append(e->code);
               }
               sp->append(";\n");
               /* Environment variable declaration */
               if ($1->num & VC_ENV) {
                    if (curr_pegram == NULL || vstore.get_scopelevel() != 1) {
                         pegerror("Environment Variable can only declare in",
                                  "the 1st local scope of a SPEC");
                    } 
                    source dsrc, isrc;
                    string eref;
                    if ($1->num & VC_LOCAL) 
                        eref = "lenv->";
                    else
                        eref = "env->";
                    for (e = $2; e != NULL; e = e->next) {
                         source mysrc(tsrc);
                         DeclExpr *de = (DeclExpr *) e;
                         dsrc << mysrc << " " << *de->d << ";\n";
                         if (de->init)
                             isrc << eref << de->vname << " = " 
                                  << *de->init->code << ";\n";
                    }
                    dsrc.indent("           ");
                    isrc.indent("         ");
                    if ($1->num & VC_LOCAL) {
                        curr_pegram->lenvdecl << dsrc;
                        curr_pegram->lenvinit << isrc;
                    } else {
                        curr_pegram->envdecl << dsrc;
                        curr_pegram->envinit << isrc;
                    }
               }
               $$ = sp;
	   }
	|  type_definition
	;

declaration_spec
	:  var_storage_class declaration_specifiers {
                source *sp = new source;
                SourceAppend2(sp, $1, $2);
                sp->num = $1->num; 
                sp->type = $2->type;
		$$ = sp;
           }
	|  declaration_specifiers
	|  K_ENV var_storage_class declaration_specifiers {
                source *sp = new source;
                SourceAppend2(sp, $2, $3);
                sp->num = $2->num | VC_ENV; 
                sp->type = $3->type;
                $$ = sp;
           }
	|  K_ENV declaration_specifiers {
                $2->num = VC_ENV; 
                $$ = $2;
           }
	;

var_storage_class 
	:  K_AUTO     {  $$ = new source("auto ", VC_AUTO);         }
        |  K_EXTERN   {  $$ = new source("extern ", VC_EXTERN);     }
        |  K_REGISTER {  $$ = new source("register ", VC_REGISTER); }
        |  K_STATIC   {  $$ = new source("static ", VC_STATIC);     }
        |  K_GLOBAL   {  $$ = new source("global ", VC_GLOBAL);     }
        |  K_LOCAL    {  $$ = new source("local ", VC_LOCAL);       }
	;

declaration_specifiers  
	:  declaration_specifier declaration_specifiers  
	   {   source *sp = new source;
               Type *t1, *t2;
               t1 = $1->type;
               t2 = $2->type;
               sp->type = new Type(concat(*t1,*t2));
               if (*sp->type == errType) {
                    pegerror("Errors in type specification");
               }
               SourceAppend3(sp, $1, " ", $2);
               $$ = sp;
           }
	|  declaration_specifier 
	;

declaration_specifier
	:  simple_type_specifier
	|  enumeration_type_specifier
	|  struct_or_union_specifier
	|  type_qualifiers simple_type_specifier {
               $$ = $2
           }
	|  type_qualifiers struct_or_union_specifier {
               $$ = $2
           }
	;

init_declarators
	:  init_declarator
	|  init_declarator  ',' init_declarators {
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
	   }
	;

init_declarator
	:  declarator '=' initializer {
               Expr *e = new DeclExpr($1, $3, $1->vname);
               SourceAppend3(e->code, $1->all(), " = ", $3->code->all());
               e->type = *$1->type;
               $$ = e;
	   }
	|  declarator {
               Expr *e = new DeclExpr($1, NULL, $1->vname);
               e->code->append($1->all());
               e->type = *$1->type;
               $$ = e;
	   }
	;

declarator:
	   pointer direct_declarator {
               source *sp = new source;
               SourceAppend3(sp, $1, " ", $2);
               $2->type->fillin_type(*$1->type);
               sp->type = $2->type;
               sp->vname = $2->vname;
               $$ = sp;
	   }
	|  direct_declarator
	;

pointer	:  '*' {  
               $$ = new source("*", new Type(TC_PTR, new PtrTypeInfo(tmpType)));
	   }
	|  '*' pointer {
               source *sp = new source;
               SourceAppend2(sp, "*", $2);
               sp->type = new Type(TC_PTR, new PtrTypeInfo(*$2->type));
               $$ = sp;
           }
	|  '*' type_qualifiers {
               source *sp = new source;
               SourceAppend2(sp, "*", $2);
               sp->type = new Type(TC_PTR, new PtrTypeInfo(tmpType));
               $$ = sp;
           }
	|  '*' type_qualifiers pointer {
               source *sp = new source; 
               SourceAppend4(sp, "*", $2, " ", $3);
               sp->type = new Type(TC_PTR, new PtrTypeInfo(*$3->type));
               $$ = sp;
	}
	;

direct_declarator:
	   S_IDENT {
               $$ = new source($1, new Type(TC_TMP));
               $$->vname = strdup($1);
	   }
	|  '(' declarator ')' {
               source *sp = new source;
               SourceAppend3(sp, "(", $2, ")");
               sp->type = $2->type;
               sp->vname = $2->vname;
               $$ = sp;
	   }
	|  direct_declarator '[' ']' {
               source *sp = new source;
               SourceAppend2(sp, $1, " []");
               $1->type->fillin_type(Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
               sp->type = $1->type;
               sp->vname = $1->vname;
               $$ = sp;
	   }
	|  direct_declarator '[' constant_expression ']' {
               source *sp = new source;
               SourceAppend4(sp, $1, " [", $3, "]");
               $1->type->fillin_type(Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
               sp->type = $1->type;
               sp->vname = $1->vname;
               $$ = sp;
	   }
	|  function_declarator
	;

abstract_declarator
	:  pointer
	|  pointer direct_abstract_declarator {
               source *sp = new source;
               SourceAppend3(sp, $1, " ", $2);
               $2->type->fillin_type(*$1->type);
               sp->type = $2->type;
               $$ = sp;
	   }
	|  direct_abstract_declarator
	;

direct_abstract_declarator
	:  '(' abstract_declarator ')'  {
               source *sp = new source;
               SourceAppend3(sp, "(", $2, ")");
               sp->type = $2->type;
               $$ = sp;
	   }
	|  direct_abstract_declarator '[' constant_expression ']' {
               source *sp = new source;
               SourceAppend4(sp, $1, " [", $3, "]");
               $1->type->fillin_type(Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
               sp->type = $1->type;
               $$ = sp;
	   }
	|  direct_abstract_declarator '[' ']' {
               source *sp = new source;
               SourceAppend2(sp, $1, " []");
               $1->type->fillin_type(Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
               sp->type = $1->type;
               $$ = sp;
	   }
	|  direct_abstract_declarator '(' parameter_type_list ')' {
	       source *sp = new source;
               SourceAppend4(sp, $1, " (", $3, ")");
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(*$1->type));
               $$ = sp;
           }
	|  direct_abstract_declarator '(' ')' {
               source *sp = new source;
               SourceAppend2(sp, $1, " ()");
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(*$1->type));
               $$=sp;
           }
	|  '[' constant_expression ']' {
               source *sp = new source;
               SourceAppend3(sp, "[", $2, "]");
               sp->type = new Type(TC_ARRAY, new 
                                  ArrayTypeInfo(tmpType, $2->str()));
               $$ = sp;
           }
	|  '[' ']' {
               source *sp = new source;
               sp->append("[]");
               sp->type = new Type(TC_ARRAY,new ArrayTypeInfo(tmpType));
               $$ = sp;
           }
	|  '(' parameter_type_list ')' {
               source *sp = new source;
               SourceAppend3(sp, "(", $2, ")");
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(intType));
               $$=sp;
           }
	|  '(' ')' {
               source *sp = new source;
               sp->append("()");
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(intType));
               $$ = sp;
           }
	;



initializer
	:  assignment_expression
	|  '{' initializers '}' {
               $$ = new ListExpr($2);
               SourceAppend3($$->code, "{", $2, "}\n");
           }
	;
	
initializers
	:  initializer 
	|  initializer ',' initializers {
               SourceAppend2($1->code, ", ", $3->code);
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
           }
	;



/*********************************************************************
 *  Type Specification                                               *
 *********************************************************************/

type_definition
	:  K_TYPEDEF declaration_spec init_declarators ';' {
               source *sp = new source;
               Expr *e;
               for (e = $3; e != NULL; e = e->next) {
                    DeclExpr *de = (DeclExpr *) e;
                    de->type.fillin_type(*$2->type);
                    if (!vstore.add_type(de->vname, new Type(de->type)))
                         pegerror("Duplicated definition of type", de->vname);
               }
               /*
               for (e = $2; e != NULL; e = e->next) {
                    if (e != $2) sp->append(", ");
                    sp->append(e->code);
               }
               */
               $$ = sp;                                                       
	   }
	|  declaration_spec ';' {
               if (!$1->type->is_rectype() &&  $1->type->tc != TC_NAME) {
                    pegwarning("Abstract declarator used as declaration");
               }
           }
	;

type_reference
	:  simple_type_specifier
	|  struct_or_union_reference
	;

type_qualifiers
	:  type_qualifiers type_qualifier {
               source *sp = new source;
               SourceAppend3(sp, $1, " ", $2);
               $$ = sp;
           }
	|  type_qualifier  {
               source *sp = new source;
               sp->append($1);
               $$ = sp;
           }
	;

type_qualifier
	:  K_CONST    {$$="const ";}
	|  K_VOLATILE {$$="volatile ";}
	|  K___INLINE {$$="__inline__ ";}
	;


simple_type_specifier
	:  integer_type      { $$ = new source($1->str(), $1);   }
	|  floating_type     { $$ = new source($1->str(), $1);   }
        |  string_type       { $$ = new source($1->str(), $1);   }
	|  void_type         { $$ = new source($1->str(), $1);   }
	|  K_OPSET 	     { $$ = new source("Opset", new Type(TC_OPSET)); }
	|  S_TYPENAME {
               Type *t = vstore.get_type($1);
               if (t == NULL)
                    pegerror("Type reference not found",$1);
               $$ = new source(t->str(), t);
           }
	;

integer_type    
	:  K_SHORT          { $$ = new Type(TC_SHORT);           }
        |  K_INT            { $$ = new Type(TC_INT);             }
        |  K_LONG           { $$ = new Type(TC_LONG);            }
        |  K_LONGLONG       { $$ = new Type(TC_LONGLONG);        }
        |  K_SIGNED         { $$ = new Type(TC_SIGNED);          }
        |  K_UNSIGNED       { $$ = new Type(TC_UNSIGNED);        }
        |  K_CHAR           { $$ = new Type(TC_CHAR);            }
        ;

floating_type:    
	   K_DOUBLE         { $$ = new Type(TC_DOUBLE);          }
        |  K_FLOAT          { $$ = new Type(TC_FLOAT);           }
        ;

void_type:    
	   K_VOID           { $$ = &voidType;                    }
        ;

string_type:    
	   K_STR            { $$ = new Type(TC_STR);             }
        ;

/*********************************************************************
 *  Struct or Union Specification                                    *
 *********************************************************************/

struct_or_union_specifier:
                  struct_or_union_definition
                | struct_or_union_reference
                ;

struct_or_union_reference:
	   struct_or_union S_IDENT {
               char buf[272];
               source *sp = new source;
               strcpy(buf, $1->all());
               strcat(buf, " ");
               strncat(buf, $2, 256);
               sp->type = vstore.get_type(buf) != NULL 
		           ? vstore.get_type(buf)
                           : new Type(TC_NAME, new NameTypeInfo(buf));
               SourceAppend4(sp, $1, " ", $2, " ");
               $$ = sp;
           }
        |  struct_or_union S_TYPENAME {
               char buf[272];
               source *sp = new source;
               strcpy(buf, $1->all());
               strcat(buf, " ");
               strncat(buf, $2, 256);
               sp->type = vstore.get_type(buf) != NULL
                              ? vstore.get_type(buf)
                              : new Type(TC_NAME, new NameTypeInfo(buf));
               SourceAppend4(sp, $1, " ", $2, " ");
               $$ = sp;
           }
	;

struct_or_union_definition:
	   struct_or_union S_IDENT {
               Type *t;
               vstore.newtbl();
               char buf[272];
               strcpy(buf, $1->all());
               strcat(buf, " ");
               strncat(buf, $2, 256);
               t = new Type($1->num, new RecTypeInfo(vstore.toptbl(), buf));
               if (!vstore.add_type(buf, t))
                    pegerror("Duplicated definition of struct/union");
           }
	   '{' struct_declaration_body '}' {
               source *sp = new source;
               char buf[272];
               vstore.poptbl();
               strcpy(buf, $1->all());
               strcat(buf, " ");
               strncat(buf, $2, 256);
               sp->type = vstore.get_type(buf);
               SourceAppend6(sp, $1, " ", $2, " {\n", $5, "}");
               $$ = sp;
           }
	|  struct_or_union
           {   vstore.newtbl();   }
           '{' struct_declaration_body '}' {
               source *sp = new source;
               sp->type = new Type($1->num, new RecTypeInfo(vstore.poptbl()));
               SourceAppend4(sp, $1->all(), " {\n", $4, "}");
               $$ = sp;
           } 
	;

struct_or_union:  
	    K_UNION  {$$ = new source("union", TC_UNION);   }
       	|   K_STRUCT {$$ = new source("struct", TC_STRUCT); }
       	;

struct_declaration_body:
          struct_declaration_list
          {   $$ = $1;   }
       |
          {   $$ = new source; }
       ;

struct_declaration_list:
           struct_declaration
	|  struct_declaration_list struct_declaration {
              source *sp = new source;
              SourceAppend2(sp, $1, $2);
              $$=sp;
          }
       ;

struct_declaration: 
	   declaration_spec struct_declarator_list ';'  {
	       Expr *e;
               for (e = $2; e != NULL; e = e->next) {
                   DeclExpr *de = (DeclExpr *) e;
                   de->type.fillin_type(*$1->type);
                   VarNode *vn = new VarNode (de->vname, de->type);
                   if (!vstore.toptbl()->insert(make_pair(de->vname,vn)).second)
                      pegerror("Duplicated definition of identifier in struct");
               }
               source *sp = new source;
               for (e = $2; e != NULL; e = e->next) {
                   if (e != $2) sp->append(", ");
                   sp->append(e->code);
               }
               sp->append(";\n");
               $$ = sp;
           }
	|  declaration_spec ';' {
               $$ = new source;
               SourceAppend2($$, $1, ";\n");
               delete $1;
           }
        |  ';' {   $$ = new source;  }
        ;

struct_declarator_list:
	   struct_declarator
	|  struct_declarator_list ',' struct_declarator   {
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
           }
	;

struct_declarator:
	   declarator {
               Expr *e = new DeclExpr($1, NULL, $1->vname);
               e->code->append($1->all());
               e->type = *$1->type;
               $$ = e;
           }
	|  declarator ':' constant_expression {
               Expr *e = new DeclExpr($1, NULL, $1->vname);
               e->code->append($1->all());
               e->code->append(":");
               e->code->append($3);
               e->type = *$1->type;
               $$ = e;
           }
        ;


/*********************************************************************
 *  Enumeration Type                                                 *
 *********************************************************************/


enumeration_type_specifier
	:  enumeration_type_definition
       	|  enumeration_type_reference
	;

enumeration_type_definition
	:  K_ENUM '{' 
           {  curr_enumtype = new Type(TC_ENUM, new EnumTypeInfo);  }
           enumeration_definition_list '}' {
               source *sp = new source;
               *sp << "enum {" << *$4 << "}\n";
               sp->type = curr_enumtype;
               $$ = sp;
           }
	|  K_ENUM enumeration_tag '{' 
           {  curr_enumtype = new Type(TC_ENUM, new EnumTypeInfo);  }
           enumeration_definition_list '}' {
               string tname = "enum ";
               source *sp = new source;
               *sp << "enum " << $2 << " {" << *$5 << "}\n";
               sp->type = curr_enumtype;
               tname += $2;
               vstore.add_type(tname.c_str(), sp->type);
               $$ = sp;
	   }
	;

enumeration_type_reference
	:  K_ENUM enumeration_tag {
               source *sp = new source;
               *sp << "enum " << $2;
               sp->type = vstore.get_type(sp->all());
               if (sp->type == NULL)
                    pegerror("Type reference not found for ",$2);
               $$ = sp;
           }
	;

enumeration_tag
	:  S_IDENT
	;

enumeration_definition_list
	:  enumeration_constant
	|  enumeration_definition_list ',' enumeration_constant {
               source *sp = new source;
               if ($3->vname != NULL)
                    *sp << *$1 << ", " << *$3;
               else
                    *sp << *$1;
               $$ = sp;
               delete $1; delete $3;
	}
	;

enumeration_constant
	:  S_IDENT {
               VarNode *vn = new VarNode($1, *curr_enumtype);
               ((EnumTypeInfo *)curr_enumtype->info)->addEnum($1);
               if (vstore.insVarInTopTbl($1, vn) == 0) 
                     pegerror("Duplicated definition of identifier", $1);

               source *sp = new source($1);
               sp->append($1);
               sp->vname = strdup($1);
               sp->type = curr_enumtype;
               $$=sp;
           }
	|  S_IDENT '=' constant_expression {
               VarNode *vn = new VarNode($1, *curr_enumtype);
               ((EnumTypeInfo *)curr_enumtype->info)->addEnum($1, $3);
               if (vstore.insVarInTopTbl($1, vn) == 0) 
                     pegerror("Duplicated definition of identifier", $1);

               source *sp = new source;
               *sp << $1 << " = " << *$3->code;
               sp->vname = strdup($1);
               sp->type = curr_enumtype;
               $$ = sp;
	   }
	|  {    $$ = new source;    }
	;


/*********************************************************************
 *  Expressions						  	     *
 *********************************************************************/

expression
	:  assignment_expression
	|  expression ',' assignment_expression  {
               SourceAppend2($1->code, ", ", $3->code);
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
           }
	;

assignment_expression
	:  unary_expression assignment_op assignment_expression
           {   Expr *e = new AssmtExpr($2, $1, $3);
               SourceAppend3(e->code, $1->code, $2, $3->code);
               e->type = $1->type;
               $$ = e;
           }
	|  conditional_expression
	;

assignment_op
	:  '='       {$$="="; }
	|  K_PLUSEQ  {$$="+=";}
	|  K_MINUSEQ {$$="-=";}
	|  K_MULTEQ  {$$="*=";}
	|  K_DIVEQ   {$$="/=";}
	|  K_MODEQ   {$$="%=";}
	|  K_SHFTLEQ {$$="<<=";}
	|  K_SHFTREQ {$$=">>=";}
	|  K_ANDEQ   {$$="&=";}
	|  K_OREQ    {$$="|=";}
	|  K_XOREQ   {$$="^=";}
	;

unary_expression
	:  postfix_expression
        |  '*' cast_expression {
               Expr *e = new UnaryExpr("*", $2);
               SourceAppend2(e->code, "*", $2->code);
               if ($2->type.tc != TC_PTR)
                    pegerror("Pointer type expected for ", $2->str());
               e->type = $2->type.get_elmtype();
               e->lvalue = 1;
               $$ = e;
           }
	|  '&' cast_expression {
               Expr *e = new UnaryExpr("&", $2);
               SourceAppend2(e->code, "&", $2->code);
               e->type = Type(TC_PTR, new PtrTypeInfo($2->type));
               e->lvalue = 0;
               $$ = e;
           }
	|  unary_op cast_expression {
               Expr *e = new UnaryExpr($1, $2);
               SourceAppend2(e->code, $1, $2->code);
               e->type = $2->type;
               e->lvalue = 0;
               $$ = e;
	   }
	|  K_PLUSPLUS unary_expression {
               Expr *e = new UnaryExpr("++", $2);
               SourceAppend2(e->code, "++", $2->code);
               e->type = $2->type;
               e->lvalue = $2->lvalue;
               $$ = e;
	   }
	|  K_MINUSMINUS unary_expression {
               Expr *e = new UnaryExpr("--", $2);
               SourceAppend2(e->code, "--", $2->code);
               e->type = $2->type;
               e->lvalue = $2->lvalue;
               $$ = e;
	   }
	|  K_SIZEOF '(' type_name ')' {
	       Expr *e = new UnaryExpr("sizeof", new IdentExpr($3->all()));
               SourceAppend3(e->code, "sizeof (", $3, ")");
               e->type = intType;
               e->lvalue = 0;
               $$ = e;
           }
	|  K_SIZEOF unary_expression {
               Expr *e = new UnaryExpr("sizeof", $2);
               SourceAppend2(e->code, "sizeof ", $2->code);
               e->type = intType;
               e->lvalue = 0;
               $$ = e;
	   }
	;

unary_op:  '+' {$$="+";}
	|  '-' {$$="-";}
	|  '~' {$$="~";}
	|  '!' {$$="!";}
	;

postfix_expression
	:  primary_expression
        |  postfix_expression '[' expression ']' {
               Expr *e = new ArrayElmExpr($1, $3);
               if ($1->type.tc == TC_ARRAY || $1->type.tc == TC_PTR
                   || $1->type.tc == TC_STR) {
                   e->type = $1->type.get_elmtype();
                   SourceAppend4(e->code, $1, " [", $3,"]");
                   $$ = e;
               } else
                   pegerror($1->str(), "is not an array, ptr");
           }       
	|  function_call
	|  postfix_expression '.' S_IDENT {
               Expr *e = new RecElmExpr($1, $3);
               if ($1->type.is_rectype()) {
                    e->type = $1->type.get_elmtype($3);
                    if (e->type.tc == TC_ERR)
                          pegerror($3, "is not a member of struct/union");
                    SourceAppend3(e->code, $1->code, ".", $3);
                    $$ = e;
	       } else 
                    pegerror($1->str(), "is not a struct or union");
	   }
	|  postfix_expression K_INDIRECT S_IDENT {
               Expr *e = new PtrToElmExpr($1, $3);
               if ($1->type.is_ptrtorec()) {
                    e->type = $1->type.get_elmtype().get_elmtype($3);
                    if (e->type.tc == TC_ERR)
                         pegerror($3, "is not a member of struct/union");
                    else
                          pegerror($1->str(), "is not a ptr to struct/union");
                    SourceAppend3(e->code, $1->code, "->", $3);
                    $$ = e;
               } else
                    pegerror($1->str(), "is not a ptr to struct/union");
	   }
	|  postfix_expression K_PLUSPLUS {
               Expr *e = new PostFixExpr("++", $1);
               SourceAppend2(e->code, $1->code, "++");
               e->type = $1->type;
               $$ = e;
           }
	|  postfix_expression K_MINUSMINUS {
               Expr *e = new PostFixExpr("--", $1);
               SourceAppend2(e->code, $1->code, "--");
               e->type = $1->type;
               $$ = e;
	   }
	;

expression_list
	:  assignment_expression
	|  expression_list ',' assignment_expression {
               SourceAppend2($1->code, ", ", $3->code);
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
           }
	;

primary_expression
	:  S_IDENT {
	       VarNode *vn = vstore[$1];
               if (vn == NULL)
                    pegerror($1, ": variable is not declared");
               $$ = new IdentExpr($1);
               if (vn->sflag & VC_ENV) {
                    $$->code = new source;
                    $$->code->type = &vn->type;
                    *($$->code) << "env->" << $1;
               } else
                    $$->code = new source($1, &vn->type);
               $$->type = vn->type;
	   }
        |  '$' S_IDENT {
               $$ = new SpecIdentExpr($2);
               if (in_termdef) {
                    if (svar_tbl == NULL) 
					pegerror("Fatal: svartbl is NULL");

                    svar_tbl->insert(make_pair($2, lineno));

                    if (curr_sevent == NULL)
                         pegerror("Fatal: curr_sevent is NULL");

                    Attr *at = curr_sevent->find($2);
                    if (at == NULL)
                         pegerror("Attribute is not defined", $2);

                    $$ = new SpecIdentExpr($2);
                    $$->code = new source(at->tname.c_str(), at->type);
                    $$->type = *at->type;
               } else if (in_progspec) {
                    if (svar_tbl != NULL) { /* in action */
                         svar_tbl->insert(make_pair($2, lineno));
                    }
                    Attr *at = curr_module->findAttr($2);
                    if (at == NULL)
                         pegerror("Attribute is not defined", $2);
                    $$ = new SpecIdentExpr($2);
                    $$->code = new source(at->tname.c_str(), at->type);
                    $$->type = *at->type;
               } else if (in_attrdecl) {
               	Attr *at = curr_sevent->find($2);
                    if (at == NULL)
                         pegerror("Attribute is not defined", $2);
				$$ = new SpecIdentExpr($2);
                    $$->code = new source(at->tname.c_str(), at->type);
                    $$->type = *at->type;
               } else {
                    pegerror("$var appears in invalid scope \n");
               };
           }
	|  '(' expression ')' { 
	       source *sp = new source;
               *sp << "(" << *$2->code << ")";
               // delete ($2->code);
               $2->code = sp;
               $$ = $2;    
           }     
	|  constant  
	;


conditional_expression
	:  logical_or_expression
        |  logical_or_expression '?' expression ':' conditional_expression {
               Expr *e = new TrinaryExpr("?", $1, $3, $5);
               SourceAppend5(e->code, $1, " ? ", $3, " : ", $5);
               if (!$1->type.is_inttype())
                   pegerror("int type expected for", $1->str());
               if ($3->type != $5->type)
                   pegerror("equal type expected for 2nd &3rd operand");
               e->type = $3->type;
               $$ = e;
           }
	;

logical_or_expression
	:  logical_and_expression
        |  logical_or_expression K_OR logical_and_expression {
               Expr *e = new BinaryExpr("||", $1, $3);
               if (!$1->type.logop_chk($3->type))
                    pegerror("Invalid operands to binary operator ||");
               e->type = intType;
               SourceAppend3(e->code, $1->code, " || ", $3->code);
               $$ = e;
  	   }
	;

logical_and_expression
	:  bitwise_or_expression
        |  logical_and_expression K_AND bitwise_or_expression {
               Expr *e = new BinaryExpr("&&", $1, $3);
               if (!$1->type.logop_chk($3->type))
                   pegerror("Invalid operands to binary operator &&");
               e->type = intType;
               SourceAppend3(e->code, $1->code, " && ", $3->code);
               $$ = e;
           }
	;

bitwise_or_expression
	:  bitwise_xor_expression
        |  bitwise_or_expression '|' bitwise_xor_expression  {
               Expr *e = new BinaryExpr("|", $1, $3);
               if (!$1->type.bitop_chk($3->type))
                    pegerror("Invalid operands to binary operator |");
               e->type = $1->type;
               SourceAppend3(e->code, $1->code, " | ", $3->code);
               $$ = e;
           }
	;

bitwise_xor_expression
	:  bitwise_and_expression
        |  bitwise_xor_expression '^' bitwise_and_expression  {
               Expr *e = new BinaryExpr("^", $1, $3);
               if (!$1->type.bitop_chk($3->type))
                    pegerror("Invalid operands to binary operator ^");
               e->type = $1->type;
               SourceAppend3(e->code, $1->code, " ^ ", $3->code);
               $$ = e;
           }
	;

bitwise_and_expression
	:  equality_expression
        |  bitwise_and_expression '&' equality_expression  {
               Expr *e = new BinaryExpr("&", $1, $3);
               if (!$1->type.bitop_chk($3->type))
                   pegerror("Invalid operands to binary operator &");
               e->type = $1->type;
               SourceAppend3(e->code, $1->code, " & ", $3->code);
               $$ = e;
           }
	;

equality_expression
	:  relational_expression
        |  equality_expression equality_op relational_expression  {
               Expr *e = new BinaryExpr($2, $1, $3);
               if (!$1->type.equop_chk($3->type))
                    pegerror("Invalid operands to binary operator", $2);
               *e->code << *$1->code << " " << $2 << " " << *$3->code;
               e->type = intType;
               $$ = e;
	   }

equality_op
	:  K_EQEQ  {$$="==";}
        |  K_NOTEQ {$$="!=";}
        ;

relational_expression
	:  shift_expression
        |  relational_expression relational_op shift_expression  {
               Expr *e = new BinaryExpr($2, $1, $3);
               if (!$1->type.relop_chk($3->type))
                    pegerror("Invalid operands to binary operator", $2);
               SourceAppend3(e->code, $1, $2, $3);
               e->type = intType;
               $$ = e;
           }
	;

relational_op
	:  '<'     {$$="<";}
        |  K_LTEQ  {$$="<=";}
        |  '>'     {$$=">";}
        |  K_GTEQ  {$$=">=";}
	;

shift_expression
	:  additive_expression
        |  shift_expression shift_op additive_expression  {
               Expr *e = new BinaryExpr($2, $1, $3);
               if (!$1->type.shfop_chk($3->type))
                    pegerror("Invalid operands to binary operator", $2);
               e->type = $1->type;
               SourceAppend3(e->code, $1, $2, $3);
               $$ = e;
           }
	;

shift_op:  K_SHFTL {$$="<<";}
	|  K_SHFTR {$$=">>";}
	;

additive_expression
	:  multiplicative_expression
        |  additive_expression '+' multiplicative_expression  {
               Expr *e = new BinaryExpr("+", $1, $3);
               e->type = $1->type.addop_chk($3->type);
               if (e->type == errType)
                    pegerror("Invalid operands to binary operator +");
               if ($1->type.is_strtype() && $3->type.is_strtype()) {
                    SourceAppend5(e->code, "wr_str_add(", $1, ", ", $3, ")");
               } else {
                    SourceAppend3(e->code, $1, " + ", $3);
               }
               $$ = e;
	   }
	|  additive_expression '-' multiplicative_expression  {
               Expr *e = new BinaryExpr("-", $1, $3);
               SourceAppend3(e->code, $1->code, " - ", $3->code);
               e->type = $1->type.subop_chk($3->type);
               if (e->type == errType)
                    pegerror("Invalid operands to binary operator -");
               $$ = e;
           }
	;

multiplicative_expression
	:  cast_expression
	|  multiplicative_expression '*' cast_expression  {
               Expr *e = new BinaryExpr("*", $1, $3);
               SourceAppend3(e->code, $1->code, " * ", $3->code);
               e->type= $1->type.mulop_chk($3->type);
               if (e->type == errType)
                    pegerror("Invalid operands to binary operator *");
               $$ = e;
           }
	|  multiplicative_expression '/' cast_expression {
               Expr *e = new BinaryExpr("/", $1, $3);
               SourceAppend3(e->code, $1->code, " / ", $3->code);
               e->type= $1->type.mulop_chk($3->type);
               if (e->type == errType)
                   pegerror("Invalid operands to binary operator /");
               $$ = e;
	   }
	|  multiplicative_expression '%' cast_expression {
               Expr *e = new BinaryExpr("%", $1, $3);
               SourceAppend3(e->code, $1->code, " % ", $3->code);
               if (!$1->type.modop_chk($3->type))
                    pegerror("Invalid operands to binary operator %");
               e->type = $1->type;
               $$ = e;
           }
	;

cast_expression
	:  unary_expression 
	|  '(' type_name ')' cast_expression {
               source *sp = new source;
               $4->type = *$2->type;
               SourceAppend4(sp, "(", $2, ")", $4);
               $4->code = sp;
               $$ = $4;
           }
	;

type_name
	:  declaration_spec abstract_declarator {  
               source *sp = new source;
               SourceAppend3(sp, $1, " ", $2);
               $2->type->fillin_type(*$1->type);
               sp->type = $2->type;
               $$ = sp;
           }
	|  declaration_spec
	; 

constant_expression
	:  conditional_expression
	;


/*********************************************************************
 *  Functions							     *
 *********************************************************************/

function_call
	:  postfix_expression '(' expression_list ')' {
                Expr *e = new FuncCallExpr($1, $3);
                if ($1->type.is_functype())
                    e->type = $1->type.get_elmtype();
                else
                    pegerror("Function expected for ", $1->str());
                FuncTypeInfo *fti = (FuncTypeInfo *) $1->type.info;
                SourceAppend4(e->code, $1, " (", $3->code, ")");
                $$ = e;  
           }
	|  postfix_expression '(' ')' {
                Expr *e = new FuncCallExpr($1, NULL);
                if ($1->type.is_functype())
                    e->type = $1->type.get_elmtype();
                else
                    pegerror("Function expected for ", $1->str());
                FuncTypeInfo *fti = (FuncTypeInfo *) $1->type.info;
                SourceAppend2(e->code, $1, " ()");
                $$ = e;
           }
	;

function_definition
	:  function_specifier {
               if ($1->type->is_functype()) { 
                   FuncTypeInfo *fti = (FuncTypeInfo *)$1->type->info;
                   if (fti->vtbl != NULL) {
                       vstore.pushtbl(fti->vtbl);
                   }
               } else 
                   pegerror("Function declaration expected");
           }
           compound_stmt 
	   {   
               if ($1->type->is_functype()) { 
                    FuncTypeInfo *fti = (FuncTypeInfo *) $1->type->info;  
                    if (fti->vtbl != NULL)
                         vstore.poptbl();  
               } else
                   pegerror("Function declaration expected");
               source *sp = new source;
               SourceAppend2(sp, $1, $3);
               $$ = sp;
           } 
	;       

function_specifier
	:  declaration_spec declarator {
               $2->type->fillin_type(*$1->type);
               source *sp = new source;
               SourceAppend2(sp, $1, $2);
               sp->type = $2->type;
               sp->vname = $2->vname;
               $$=sp
           }
	|  declarator {
               source *sp = new source;
               sp->append($1);
               sp->type = $1->type;
               sp->vname = $1->vname;
               $$=sp
           } 
	;

function_declarator
	:  direct_declarator '('
           {  vstore.newtbl();  }
           parameter_type_list ')' 
	   {
               VarTbl *vtbl = vstore.poptbl();
               source *sp = new source;
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(tmpType,vtbl));
               sp->vname = $1->vname;
               $$ = sp;
           }
	|  direct_declarator '(' ')' {
               source *sp = new source;
               sp->type = new Type(TC_FUNC, new FuncTypeInfo(tmpType, NULL));
               sp->vname = $1->vname;
               $$ = sp;
	   }
	;

parameter_type_list
	:  parameter_list
	|  parameter_list ',' K_ELIPSIS  {
               source *sp = new source;
               SourceAppend2(sp, $1, ", ...");
               $$=sp;
           }
	;

parameter_list  
	:  parameter_list ',' parameter_declaration   {
               source *sp = new source;
               SourceAppend3(sp, $1, ",", $3);
               $$=sp;
           }
	|  parameter_declaration
	;

parameter_declaration
	:  declaration_spec declarator   {
               $2->type->fillin_type(*$1->type);
               VarNode *vn = new VarNode($2->vname, *$2->type);
               if (!vstore.toptbl()->
                     insert(make_pair($2->vname, vn)).second) {
                    pegerror("Duplicated definition of identifier");
               }
               source *sp = new source;
               SourceAppend3(sp, $1, " ", $2);
               $$=sp;
           }
	|  declaration_spec abstract_declarator   {
               source *sp = new source;
               SourceAppend2(sp, $1, $2);
               $$=sp;
           } 
	|  declaration_spec {
               source *sp = new source;
               sp->append($1);
               $$=sp;
           }
	;


/*********************************************************************
 *  Statements							     *
 *********************************************************************/

statement_list
	:  statement_list statement {
               source *sp = new source;
               SourceAppend2(sp, $1, $2);
               $$=sp;
           } 
	| statement
	;       

statement
	:  expression_stmt 
	|  compound_stmt
	|  iterative_stmt
	|  conditional_stmt
	|  return_stmt
        /*	|  switch_stmt
		|  memory_management_stmt
	 */
	; 

return_stmt
	:  K_RETURN expression ';' {
           $$ = new source;
           SourceAppend3($$, "return ", $2, ";\n");
	}
	;

expression_stmt
	:  ';' {
               source *sp = new source;
               sp->append(";\n");
               $$=sp; 
           } 
	|  expression ';' {
               source *sp = new source;
               SourceAppend2(sp, $1->code, ";\n");
               $$=sp;
           }
	;

compound_stmt
	:  	{   	vstore.newtbl();    }
          compound_stmt_1 {
               source *sp = new source;
               sp->append("{\n");
               $2->indent("    ");
               sp->append($2);
               sp->append("}\n");
               delete $2; 
               vstore.deltop();
               $$ = sp;
           }     
	;
                            
compound_stmt_1
	  :  '{' declaration_list statement_list '}' {
               $2->append($3);
               $$ = $2;
               delete $3;
           }
	  |  '{' '}' {
               source *sp = new source;
               sp->append("/*Empty block*/");
               $$=sp;
           }
	  |  '{' statement_list '}' {
               $$=$2;
           }
	  |  '{' declaration_list '}' {
               $$=$2;
           }
	  ;

iterative_stmt
	  : 	K_DO statement K_WHILE '(' expression ')' ';' {
        		source *sp = new source;
               SourceAppend6(sp, "do\n", $2, "\n", "while (", $5->code, ");\n");
               $$=sp;
		}
	  |  K_WHILE '(' expression ')' statement {
               source *sp = new source;
               $5->indent("   ");
               SourceAppend4(sp, "while (", $3->code, ")\n", $5);
               $$=sp;
		}
       |  K_FOR '(' expression_stmt expression_stmt ')' statement {
          	source *sp = new source;
               $6->indent("   ");
               SourceAppend5(sp, "for (", $3, $4, ")\n", $6);
               $$=sp;
          } 
	  | 	K_FOR '(' expression_stmt expression_stmt expression ')' statement {
               source *sp = new source; 
               $7->indent("   ");
               SourceAppend6(sp, "for (", $3, $4, $5->code, ")\n", $7);
               $$=sp;
          } 
	  ;                 

conditional_stmt
	  :  K_IF '(' expression ')' statement    %prec LOWER_THAN_ELSE {
               source *sp = new source;
               $5->indent("    ");
               SourceAppend4(sp, "if (", $3->code, ")\n", $5);  
               $$ = sp;
       	} 
	  |  K_IF '(' expression ')' statement K_ELSE statement {
               source *sp = new source;
               $5->indent("    ");
               SourceAppend6(sp, "if (", $3->code, ")\n", $5, "else ", $7);
               $$=sp;
       	}
	  ;
                    
/* Constant */
constant:  S_NUMCONST  {
               Expr *e = new ConstExpr($1, intType);
               e->code->append($1);
               $$ = e;
           }
	|  S_FLOATCONST {
               Expr *e = new ConstExpr($1, floatType);
               e->code->append($1);
               $$ = e;
           }
	|  S_CHARCONST  {
               Expr *e = new ConstExpr($1, charType);
               e->code->append($1);
               $$ = e;
           }
	|  string_list
	;

string_list
	:  S_QSTR string_list {
               char *buffer;
               int len1, len2, len;
               len1 = strlen($1) - 1 ; /*Trailing ""*/
               len2 = strlen(((ConstExpr*)$2)->val) - 1; /*leading "*/
               len  = len1 + len2 + 1 /* '\0' */;
               buffer = (char*)malloc(len);
               strncpy(buffer,      $1,                      len1);
               strncpy(buffer+len1, ((ConstExpr*)$2)->val+1, len2);
               buffer[len] = '\0';

               Expr *e = new ConstExpr(buffer, carrType);
               e->code->append(buffer);
               $$ = e;

               free(buffer);
               delete $2;
           }
	|  S_QSTR {
               Expr *e = new ConstExpr($1, carrType);
               // SourceAppend3(e->code, "WSS_MAKESTR(", $1, ")");
               e->code->append($1);
               $$ = e;
           }
	;



/* Program Specification */

progspec:  K_SPEC S_IDENT 
           {	if (target == "wdl")
           		curr_pegram = new PEGram_WDL($2);
           	else if (target == "cpp")
           		curr_pegram = new CppCodeGen($2);
			else
           		pegerror("Unknown target platform", target.c_str());
               if (curr_module == NULL) {
                    pegerror("Need to specify which module to use");
               }
               vstore.newtbl();
               in_progspec = 1;
           }
           select_expr specbody K_END 
           {
               if (dflag & DF_GRAM_RULE)
                   curr_pegram->printrule(cerr);
               curr_pegram->selector = $4;
               curr_pegram->process();
               vstore.poptbl();
               in_progspec = 0;
           }
	;

select_expr
	:  '(' selectors ')'
	   {   $$ = $2;           }
	;

selectors
	:  selector {
               $$ = new SelList;
               $$->push_back($1);
	   }
	|  selectors ','  selector {
               $1->push_back($3);
               $$ = $1;
	   }
	;

selector:  '<' snotion ',' snotion  ',' snotion  ',' snotion op_evsel '>' 
           op_start
           {
               $$ = new Selector(*$2, *$4, *$6, *$8, $9);
               delete $2;
               delete $4;
               delete $6;
               delete $8;
	   }
	;

snotion	:  '*' 		{    $$ = new string("*");	}
	|  '?'		{    $$ = new string("?");	}
	|  S_IDENT 	{    $$ = new string($1); 	}
	;

op_start:
	   ':' S_IDENT
           {   $$ = new string($2);   }
	|  {   $$ = new string(" ");  }
	;

op_evsel
	  :  ',' event_selector 
          {    $$ = $2;                }
	  |  {	$$ = new AEvSet;	
           	EvTbl::iterator iter;
            	for (iter = curr_module->evtbl.begin();
              		iter != curr_module->evtbl.end();
              		iter++) 
			{
				if ((*iter).first != "*")
                 		$$->insert((*iter).second);
               }     
         	}
	  ;

event_selector	
	  :  '*' {    
                $$ = new AEvSet;  
                EvTbl::iterator iter;
                for (iter = curr_module->evtbl.begin();
                     iter != curr_module->evtbl.end();
                     iter++) 
			{
				if ((*iter).first != "*")
                   		$$->insert((*iter).second);
           	}     
           }
	  |  '{' event_list '}' {
           	$$ = $2;
           }
	  ;

event_list
	:   event_list ',' event {
                 $1->insert(*$3);
                 delete $3;
                 $$ = $1;
            }
	|   event {
                 $$ = new AEvSet;
                 $$->insert(*$1);
            }
	;

specbody:  op_declarations addon start_expr_def 
           production_rules op_termdefs
	   {   }
	;

op_termdefs
	:  termdefs 
           {   }
	|
           {   }
	;
	

termdefs:  termdef ';' 
           {   }
	|  termdef ';' termdefs
           {   }
	;  

op_declarations
	:  declaration_list {
           }
	|  {   }
	;
	
addon	:  K_INIT compound_stmt
           {   curr_pegram->init <<  *$2;  } 
	|  K_FINI compound_stmt 
           {   curr_pegram->fini << *$2;   } 
	|  K_ERROR compound_stmt
           {   curr_pegram->error << *$2;   } 
	|  {}
	;

start_expr_def
	:  K_SE ':' start_expr {
           }
	;
start_expr
	:  start_expr_elm {
           }
	|  start_expr_elm K_OR start_expr {
           }
	;

start_expr_elm
	:  nonterm {
               /* Ad hoc TBD: fixe it */
               curr_pegram->start = $1;
           }
	|  nonterm '*' {
           }
	;

production_rules
	:  production_rule {
	   }
	|  production_rule prod_delim production_rules {
	   }
	;

prod_delim
	:  ';'
	|  '.'
	;


production_rule	
	:  nonterm {
                crule_lhs = $1;
           }
           delim alternatives {
                crule_lhs = NULL;
	   }
	|  {       }
	;

delim	:  K_INDIRECT 	{ }
	|  ':' 		{ }
	;

alternatives
	:  alternative { }
	|  alternative '|' alternatives { }
	;

alternative
	:  rhs_syms 
	   {   curr_pegram->addrule(new Rule(crule_lhs, $1));		}
	|  {   curr_pegram->addrule(new Rule(crule_lhs, new GSymList));	}
	;

nonterm :  '<' hparams '>' {
               if ($2->etype() != Expr::Ident) {
                   pegerror("Identifiers expected in the beginning of ",
                            "hypernotion\n"); 
               }
	       $$ = curr_pegram->add_nterm($2);
               if ($$ == NULL)
                    pegerror("Hypernotion overlaps with Terminal", 
                            ((IdentExpr *)$2)->name);
           }
	;

hparams :  hparam ',' hparams {
               $1->next = $3;
               $$ = $1;
	   }
	|  hparam
	;
	
hparam  :  constant 
	|  S_IDENT  {  $$ = new IdentExpr($1);  }
	;
	
rhs_syms:  rhs_syms rhs_sym {
               $1->push_back($2);
               $$ = $1;
           }
	|  rhs_sym {
               $$ = new GSymList;
               $$->push_back($1); 
           }
	;


rhs_sym :  nonterm {
               $$ = (GSym *) $1;
           }
	|  term 
	|  action 
	;

term   :  S_IDENT metavars {
			$$ = curr_pegram->add_term($1);
               if ($$ == NULL) 
                   pegerror("Terminal name overlaps with hypernotions", $1);
		}
	  |  {	in_termdef = 1;	}
          termdef_expr 
		{	TermDefExpr *e = (TermDefExpr *) $2; 
               TermDef *t = new TermDef(curr_pegram->gen_term_name(e->getName()));
               t->texpr = $2;
	       	in_termdef = 0;  
               if (!curr_pegram->add_termdef(t)) {
                    pegerror("Duplicated definition of Terminal", t->name.c_str());
               }
               $$ = curr_pegram->add_term(t->name);
		}
	  ;

/* for meta variables, not really being used right now */

metavars:  '-' S_IDENT metavars 
           {    $$ = $3; 
                $$->push_front(strdup($2))
           }
	|
	   '[' sidents ']'
           {    $$ = $2;               }
	|  {    $$ = new CStrList;     }
	;
	  
sidents	:  sidents ',' S_IDENT 
           {    $$ = $1; 
                $$->push_back(strdup($3));
           }
	|  S_IDENT
	   {    $$ = new CStrList;     
                $$->push_back(strdup($1));
           }
	;

action  : {   	svar_tbl = new VarRefTbl;    }
          compound_stmt
	   	{	$$ = curr_pegram->add_action($2, svar_tbl, lineno);   
              	svar_tbl = NULL;
          }	
	   ;

termdef : K_TERM S_IDENT {  
	     	in_termdef = 1;  
          }
	     '=' termdef_expr
	   	{   	TermDef *t = new TermDef($2);
               t->texpr = $5;
               in_termdef = 0;
               if (!curr_pegram->add_termdef(t)) {
                    pegerror("Duplicated definition of Terminal", $2);
               }
           }
	  ;

termdef_expr	
	  :  ptermdef_expr
	   	{  $$ = new TermDefExpr($1);	}
	  |  '~' ptermdef_expr
	   	{   $$ = new TermDefExpr($2, 0);	}
	  ;

ptermdef_expr
	  :  ptermdef_expr K_OR eventsel_expr 
		{	
			Expr *e = $1;
			while (e->next != NULL) e = e->next;
			e->next = $3;
               $$ = $1;  
		}
	  |  eventsel_expr {
			$$ = $1;
		}
	  ;	

eventsel_expr
	  :	'(' sysop_selector 
		{	svar_tbl = new VarRefTbl;	} 
		',' condition_list ')'
          {	AEvSet::iterator iter;	
			ListExpr *l = new ListExpr;	
			for (iter = $2->begin(); iter != $2->end(); iter++) {
				l->append(new EvSelExpr((*iter), svar_tbl, $5));
			}
			svar_tbl = NULL;
			$$ = l->dumpList();
			delete l;
               delete $2;
          }
	  |  '(' sysop_selector ')'
           {   ConstExpr *ce = new ConstExpr(1);  
               *ce->code << "(1)";
          	AEvSet::iterator iter;	
          	ListExpr *l = new ListExpr;	
			for (iter = $2->begin(); iter != $2->end(); iter++) {
               	l->append(new EvSelExpr(*iter, new VarRefTbl, ce));
			}
               $$->code = new source;
			$$ = l->dumpList();
			delete l;
               delete $2;
	   	}
	  |  '(' '*' ')'
          {	Expr *ce = new ConstExpr("1", intType); 
               *ce->code << "(1)";
               $$ = new EvSelExpr(curr_module->find("*"), new VarRefTbl, ce);
               $$->code = new source;
		}
	  ;

condition_list
	  :  assignment_expression
	  |  condition_list ',' assignment_expression {
               SourceAppend2($1->code, " && ", $3->code);
               Expr *e = $1;
               while (e->next != NULL) e = e->next;
               e->next = $3;
               $$ = $1;
		}
	  ;

sysop_selector
	  :  event 
		{	$$ = new AEvSet;		
		  	$$->insert(*$1);
		  	delete $1;
		}
	  |  sysop_selector K_OR event
		{ 	$1->insert(*$3);
			delete $3;
			$$ = $1;
		}
	  ;
	
event  :  S_IDENT
		{	curr_sevent = curr_module->find($1);
               if (curr_sevent == NULL) {
                   pegerror("Event is not defined");
               }
               $$ = new AEvent(curr_sevent, AEvent::Succ);
		}
	  |  S_IDENT '+'
          {	curr_sevent = curr_module->find($1);
               if (curr_sevent == NULL) {
                   pegerror("Event is not defined");
               }
               $$ = new AEvent(curr_sevent, AEvent::Succ);
		}
	  |  S_IDENT '-'
		{	curr_sevent = curr_module->find($1);
               if (curr_sevent == NULL) {
                   pegerror("Event is not defined");
               }
               $$ = new AEvent(curr_sevent, AEvent::Fail);
		}
	  |  S_IDENT '^'
		{   curr_sevent = curr_module->find($1);
               if (curr_sevent == NULL) {
                   pegerror("Event is not defined");
               }
               $$ = new AEvent(curr_sevent, AEvent::Pre);
		}
	  ;

%%
#include "lex.peg.c"
extern FILE *pegin;
extern int lineno;
#if YYDEBUG != 0
extern int pegdebug;
#endif
int
wr_parse_prototypes(FILE *fp)
{
#if YYDEBUG != 0
        pegdebug = 0;
#endif
        pegin = fp;
        return(pegparse());
}


