#ifndef _SOURCE_H
#define _SOURCE_H
/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: source.h,v 1.9 2001/02/23 19:02:03 stevek Exp $
 *
 * Authors: Calvin Ko
 *
 * Purpose: Object to hold generated C-code.
 * 
 */

#include "type.h"
#include <string.h>

struct Param;
struct mlabel;
class  Expr;

struct srcrec {
	char *txt;
	srcrec *next;
	srcrec(const char *s=NULL, srcrec *n=NULL) {
		txt = (s != NULL) ? strdup(s) : NULL;
		next = n;
	}
	~srcrec() {
		if (txt != NULL) free(txt);
	}
};

enum tabsc { local_table, global_table };

class source {
private:
	srcrec *head, *tail, *cur;
	int    indent_space;		/* number of space characters  */

public:
	Type *type;				/* used to store type info     */
	char *vname;				/* used in declaration parsing */
	int  num;					/* used to store integer       */

public:
	source();
	source(const source&);
	source(const char *s, int n=0);
	source(const char *s, Type *t);
	~source();

	int  is_empty();
	
	void make_header(source *);

	void prepend(const char *txt);

	source& append(char c);
	source& append(int n);
	source& append(unsigned int n);
	source& append(long n);
	source& append(unsigned long n);
	source& append(const char *s);
	source& append(source *src);
	source& append(Expr *e);
	source& append(const string& s);

	source& operator=(const source&); 
	source& operator<<(char c) 	    		{    return append(c);	     } 
	source& operator<<(unsigned char c) 	{    return append((char) c);    }
    	source& operator<<(signed char c)   	{    return append((char) c);    }
    	source& operator<<(int n)	    		{    return append(n);	     }
    	source& operator<<(unsigned int n)  	{    return append(n);           }
    	source& operator<<(long n)	    		{    return append(n);	     }
    	source& operator<<(unsigned long n) 	{    return append(n);	     }

    	source& operator<<(const char *s)   	{    return append(s);	     }
    	source& operator<<(const unsigned char *s) { 
		return append((const char*) s); 
	}
    	source& operator<<(const signed char *s) { 
		return append((const char*) s); 
	}
	source& operator<<(const string& s) {    return append(s);           }

	source& operator<<(const source& s);

	source dup();

	void gen_tbd();
	void gen_tbd(char *txt);
	void gen_debug_code();
	void gen_ascii(char *str);

	void gen_comment(char *str, char *str1 = NULL);
	void gen_nterm_funcdecl(const string& name);
 
	int  operator==(char *s); 
	int  rec_count();

	void rewind();
	void indent(const char *space="     ");
	bool more_text();
	char *text();
	char *next();

	char *all();

	bool find(char *txt);
        void insert(char *txt);
	void remove();

};

#define SourceAppend1(sp,one) {sp->append(one); }
#define SourceAppend2(sp,one,two) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	}
#define SourceAppend3(sp,one,two,three) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	}
#define SourceAppend4(sp,one,two,three,four) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	}
#define SourceAppend5(sp,one,two,three,four,five) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	}
#define SourceAppend6(sp,one,two,three,four,five,six) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	}
#define SourceAppend7(sp,one,two,three,four,five,six,seven) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	}
#define SourceAppend8(sp,one,two,three,four,five,six,seven,eight) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	}
#define SourceAppend9(sp,one,two,three,four,five,six,seven,eight,nine) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	}
#define SourceAppend10(sp,one,two,three,four,five,six,seven,eight,nine,ten) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	}
#define SourceAppend11(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	}
#define SourceAppend12(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	}
#define SourceAppend13(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven, twelve, thirteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	}
#define SourceAppend14(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	}
#define SourceAppend15(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	}
#define SourceAppend17(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	  (sp)->append(sixteen); \
	  (sp)->append(seventeen); \
	}  
#define SourceAppend19(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen,eightteen,nineteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	  (sp)->append(sixteen); \
	  (sp)->append(seventeen); \
	  (sp)->append(eightteen); \
	  (sp)->append(nineteen); \
	}  

#endif /* _SOURCE_H */
