/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: type.cc,v 1.12 2001/04/05 18:40:24 stevek Exp $
 */

#include <stdio.h> 
#include <stdlib.h> 

#define WR_UNIX
#ifdef WR_UNIX
#include <libio.h>
#include <iostream.h> 
#include <string.h>
#include <string>
#endif
#ifdef WR_WINNT
#include <iostream>
#include <string>
#endif


#include "type.h"
#include "expr.h"
#include "source.h"

void pegerror(const char *s, const char *s1 = "");

extern VarStore vstore;
#define MAXNAME 31
struct btinfo {
    char *name;
    int  size;
};

btinfo tntbl[33] = {
        {"error", 		4},
        {"unknown ", 		4},
        {"tmp ", 		4},
        {"void ", 		4},
        {"char ", 		1},
        {"pd_char_t ", 	1},
        {"signed char ", 	1},
        {"unsigned char ", 	1},
        {"short ",		2},
        {"signed short",  	2},
        {"int ",		4},
        {"signed ",		4},
        {"signed int ",		4},
        {"long ",		4},
        {"long long ",		8},
        {"signed long ",	4},
        {"signed long long ",	8},
        {"unsigned ",		4},
        {"unsigned int ",	4},
        {"unsigned short ",	2},
        {"unsigned long ",	4},
        {"unsigned long long ",	8},
        {"float ",		4},
        {"double ",		8},
        {"long double ",	12},
        {"string ",		4},
        {"databuf_t ",		4},
        {"complex ",		4}
};

btinfo  *typeinfo_tbl = &tntbl[1];
               
Type tmpType(TC_TMP);
Type errType(TC_ERR);
Type voidType(TC_VOID);
Type intType(TC_INT);
Type floatType(TC_FLOAT);
Type charType(TC_CHAR);
Type strType(TC_STR);
Type cstrType(TC_PTR, new PtrTypeInfo(charType));
Type cptrType(TC_PTR, new PtrTypeInfo(charType));
Type carrType(TC_PTR, new ArrayTypeInfo(charType));
Type vptrType(TC_PTR, new PtrTypeInfo(voidType));
 
char * Type::str()

{
     if (tc < TC_ENUM) {
          return typeinfo_tbl[(int) tc].name;
     } else 
          return typeinfo_tbl[MAXNAME - 2].name;

};

ostream& Type::print(ostream& out) const

{
     if (tc < TC_ENUM) {
          out << typeinfo_tbl[(int) tc].name;
          return out;
     } else {
          switch(tc) {
          case TC_ENUM:	  
            out << "enum ";
            return out;
          case TC_ARRAY:  
            out << "array of " << ((ArrayTypeInfo *)info)->etype;
            return out;
	  case TC_STRUCT: 
            out << ((RecTypeInfo *)info)->name;    
            info->print(out);
            return out;
	  case TC_UNION:
            out << ((RecTypeInfo *)info)->name;  
            return out;
	  case TC_FUNC: 
            out << "func " << ((FuncTypeInfo *)info)->rtype;
            return out;
	  case TC_PTR:     
            out << "ptr to " << ((PtrTypeInfo *)info)->etype; 
            return out;
	  case TC_NAME:	
            out << ((NameTypeInfo *)info)->name;
            return out;
	  case TC_STR:   	
            out << "str";
            return out;
	  case TC_DBUF:
            out << "DBUF";
            return out;
          default:	
            out << "default ";
            return out;
          }
     }
}

char *Type::get_name()

{
     char *c;
     if (tc < TC_ENUM) {
          return typeinfo_tbl[(int) tc].name;
     } else {
          switch(tc) {
	  case TC_NAME:	
            return ((NameTypeInfo *)info)->name;
          case TC_ARRAY:
            c = (char *) malloc (256);
            strcpy(c, ((ArrayTypeInfo *)info)->etype.get_name());
            strcat(c, " *");
            return c;
	  case TC_STRUCT:	
            return ((RecTypeInfo *)info)->name;  
	  case TC_UNION:
            return ((RecTypeInfo *)info)->name;  
	  case TC_FUNC: 
            return strdup("func");
	  case TC_PTR: 
            c = (char *) malloc (256);
            strcpy(c, ((ArrayTypeInfo *)info)->etype.get_name());
            strcat(c, " *"); 
            return c;
          default:	
            return strdup("default ");
          }
     }
}

int Type::fillin_type(Type t) 

{
     if (tc == TC_TMP) { 
          tc = t.tc;
          info = t.info;     
          return 1;
     } else if (tc == TC_ARRAY) {
          return ((ArrayTypeInfo *)info)->etype.fillin_type(t);
     } else if (tc == TC_PTR) {
          return ((PtrTypeInfo *)info)->etype.fillin_type(t);
     } else if (tc == TC_FUNC) {
          return ((FuncTypeInfo *)info)->rtype.fillin_type(t);
     }
     cerr << "Warning in fillin_type \n";
     return 0; 
}

Type Type::type()

{
     if (tc == TC_NAME) {
          Type *t = vstore.get_type(((NameTypeInfo *)info)->name);
          if (t == NULL) {
               pegerror("Type is not defined: ",((NameTypeInfo*)info)->name);
               exit(1);
          } else
               return *t;
     } else 
          return *this;
}

Type Type::get_elmtype(char *s) const

{
     if (s == NULL) {
         	switch(tc) {
         	case TC_ARRAY:   	return ((ArrayTypeInfo *)info)->etype.type();
         	case TC_PTR  :   	return ((PtrTypeInfo *)info)->etype.type();
         	case TC_FUNC :   	return ((FuncTypeInfo *)info)->rtype.type();
         	case TC_NAME :   	cerr << ((NameTypeInfo *)info)->name
                            		<<  ": Type ref not define\n";
                       		return errType;
	 	case TC_DBUF:   	return charType;
	 	case TC_STR :   	return charType;
         	case TC_WSS_CHAR_T:	cerr << "Hit get_elmtype- WSS_CHAR_T\n";
         					return charType;
         	default  	:   		return errType;
         	}
     } else {
         if (tc == TC_STRUCT || tc == TC_UNION) {
              RecTypeInfo *rti = (RecTypeInfo *) info; 
              VarNode *vn = (*rti->vtbl)[s];
              if (vn != NULL) 
                  return vn->type.type();
         } else if (tc == TC_NAME) {
             cerr << ((NameTypeInfo *)info)->name <<  ": Type ref not define\n";
             return Type(TC_ERR);
         }; 
         return Type(TC_ERR);
     }
}

int Type::is_ptrtorec() const

{
     return tc == TC_PTR && get_elmtype().is_rectype() ;
}

int Type::is_strtype() const

{
  return tc == TC_STR ||
    ((tc == TC_PTR || tc == TC_ARRAY) && get_elmtype().is_chartype());
}

int Type::is_inttype() const

{
     return tc == TC_CHAR || 
            tc == TC_SIGNEDCHAR || 
            tc == TC_UNSIGNEDCHAR ||
            tc == TC_SHORT || 
            tc == TC_SIGNEDSHORT ||
            tc == TC_INT ||
            tc == TC_SIGNED ||
            tc == TC_SIGNEDINT ||
            tc == TC_LONG ||
            tc == TC_LONGLONG ||
            tc == TC_SIGNEDLONG ||
            tc == TC_SIGNEDLONGLONG ||
            tc == TC_UNSIGNED ||
            tc == TC_UNSIGNEDINT ||
            tc == TC_UNSIGNEDSHORT || 
            tc == TC_UNSIGNEDLONG ||
            tc == TC_UNSIGNEDLONGLONG ||
            tc == TC_WSS_CHAR_T;
}

int Type::is_floattype() const

{
     return tc == TC_FLOAT ||
	    tc == TC_DOUBLE ||
	    tc == TC_LONGDOUBLE;
}

int Type::equiv(const Type& t) const

{
     switch (tc) {
     case TC_FUNC  :
     case TC_ARRAY	:
     case TC_PTR	:  return tc == t.tc && get_elmtype().equiv(t.get_elmtype());
     case TC_STRUCT:
     case TC_UNION :
     case TC_ENUM  :  return tc == t.tc && info == t.info;
     case TC_NAME  :
     case TC_DEF   :  cerr << "Fatal error in type.cc\n";
                   exit(0);
     default	:  
         return typeinfo_tbl[(int)tc].size == typeinfo_tbl[(int) tc].size;
     }
}

// return 1 if Ok, otherwise 0
int Type::assmt_chk(const Type& t, char* ebuf)

{
     if (is_inttype() || is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return 1;
          else {
              strcpy(ebuf, "Assignment of non-int type to int type");
              return 0;
          }
     } else if (is_rectype() || tc == TC_ENUM || tc == TC_FUNC){
          if (tc == t.tc && info == t.info) 
              return 1; 
          else {
              strcpy(ebuf, "Incompatible struct types in assignment");
              return 0;
          }
     } else if (tc == TC_PTR) {
          if (t.is_inttype()) {
              return 1; 
          } else if ((t.tc == TC_PTR || t.tc == TC_ARRAY)) {
              if( get_elmtype().equiv(t.get_elmtype()))
                      return 1; 
              if (get_elmtype().tc == TC_VOID || t.get_elmtype().tc == TC_VOID)
                  return 1; 
          }
          strcpy(ebuf, "Incompatible pointer types in assignment");
          return 0;
     } else if (tc == TC_STR) {
          if (t.is_strtype())
              return 1; 
          else {
              strcpy(ebuf, "Assignment of non-str type to string");
              return 0; 
          }
     } else if (tc == TC_DBUF) {
          if (t.tc == TC_DBUF || t.is_strtype())
              return 1;
          else {
              strcpy(ebuf, "Assignment of non-dbuf or non-str type to dbuf");
              return 0; 
          }
     } else {
          strcpy(ebuf, "Invalid lhs type in assignment");
          return 0;
     }
};

/* 
 * check type for binary == operators 
 * return 1 if valid, otherwise 0
 */
int Type::equop_chk(const Type& t)

{
     if (is_inttype()) {
         /* Integers can get promoted to pointers in pointer comparisons. */
         if (t.is_inttype() || t.is_floattype())
             return 1;
         if (t.tc == TC_PTR || t.tc == TC_ARRAY)
             return 1;
         return 0;
     } else if (is_floattype()) {
          return t.is_inttype() || t.is_floattype() ;
     } else if (t.is_strtype()) {
          return t.is_strtype();
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
              if (t.is_inttype()) {
                  /* Integers can get promoted to pointers in pointer
                   * comparisons.
                   */
                  return 1;
              } else if ((t.tc == TC_PTR || t.tc == TC_ARRAY)) {
                  if (get_elmtype().equiv(t.get_elmtype()))
                   return 1; 
                  if (get_elmtype().tc == TC_VOID || t.get_elmtype().tc == TC_VOID)
                   return 1; 
              }
              return 0;
          case TC_ENUM:
              return t.tc == TC_ENUM;
          default:
              return 0;
          }
     }
}

/* 
 * check type for binary + operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::addop_chk(const Type& t)

{
     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype()
                  || (t.tc == TC_PTR || t.tc == TC_ARRAY))
              return t;
          else
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
               if (t.is_inttype())
                    /* Pointer math */
                    return *this;
               else if (is_strtype())
                    return t.is_strtype() ? strType : errType;
               else
                    return errType;
          case TC_STR   :
 	       return t.is_strtype() ? strType : errType;
               
          default:
               return errType;
          }
     }
}

/* 
 * check type for binary - operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::subop_chk(const Type& t)

{
     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype()
                  || (t.tc == TC_PTR || t.tc == TC_ARRAY))
              return t;
          else 
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
     } else if (tc == TC_PTR || tc == TC_ARRAY) {
          if (t.is_inttype())
              return *this;
          else
              return errType;
     }
     // How the compilier can get here, I don't know
     return errType;
}

/* 
 * check type for binary *, / operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::mulop_chk(const Type& t)

{
     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype())
              return t;
          else 
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
     } else
          return errType;
}

/* 
 * check type for >, >=, <, <= operators 
 * return 1 if valid, otherwise 0
 */
int Type::relop_chk(const Type& t)

{
     //  char/int/float > char/int/float  is allowed
     //  Technically (pointer <relop> int) shoudl be allowed, but that's too
     //  ugly to exist.
     if (is_inttype() || is_floattype()) {
          return t.is_inttype() || t.is_floattype();
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
              if ((t.tc == TC_PTR || t.tc == TC_ARRAY) && 
                      get_elmtype().equiv(t.get_elmtype()))
                   return 1; 
              else 
                   return 0;
          case TC_ENUM:
              return t.tc == TC_ENUM;
          default:
              return 0;
          }
     }
}

/* 
 * check type for >> and <<  operators 
 * return 1 if valid, otherwise 0
 */
int Type::shfop_chk(const Type& t)

{
     return is_inttype() && t.is_inttype();
}

/* 
 * check type for % operators 
 * return 1 if valid, otherwise 0
 */
int Type::modop_chk(const Type& t)

{
     return shfop_chk(t);
}

/* 
 * check type for &, |, ^, << operators 
 * return 1 if valid, otherwise 0
 */
int Type::bitop_chk(const Type& t)

{
     return shfop_chk(t);
}

/* 
 * check type for && and || operators 
 * return 1 if valid, otherwise 0
 */
int Type::logop_chk(const Type& t)

{
     return (is_inttype() || is_floattype() 
             || tc == TC_PTR || tc == TC_ARRAY  || tc == TC_ENUM) &&
             (t.is_inttype() || t.is_floattype() 
             || t.tc == TC_PTR || t.tc == TC_ARRAY  || t.tc == TC_ENUM);
}


Type funcType(const Type& t)

{  return Type(TC_FUNC, new FuncTypeInfo(t));  }


VarStore::VarStore() 

{
     top = 0; 
     typetbl.insert("databuf_t", new Type(TC_DBUF));
     vtbl_tbl[0] = new VarTbl;
     (*vtbl_tbl[0])["_children"] = new VarNode("_children", intType);
     (*vtbl_tbl[0])["_cputime"] = new VarNode("_cputime", intType);
     (*vtbl_tbl[0])["_cwd"] = new VarNode("_cwd", cstrType);
     (*vtbl_tbl[0])["_descriptor"] = new VarNode("_descriptor", intType);
     (*vtbl_tbl[0])["_euid"] = new VarNode("_euid", intType);
     (*vtbl_tbl[0])["_egid"] = new VarNode("_egid", intType);
     (*vtbl_tbl[0])["_gid"] = new VarNode("_gid", intType);
     (*vtbl_tbl[0])["_memoryuse"] = new VarNode("_memoryuse", intType);
     (*vtbl_tbl[0])["_niceval"] = new VarNode("_niceval", intType);
     (*vtbl_tbl[0])["_pgid"] = new VarNode("_pgid", intType);
     (*vtbl_tbl[0])["_pid"] = new VarNode("_pid", intType);
     (*vtbl_tbl[0])["_ppid"] = new VarNode("_ppid", intType);
     (*vtbl_tbl[0])["_progname"] = new VarNode("_progname", cstrType);
     (*vtbl_tbl[0])["_uid"] = new VarNode("_uid", intType);
     (*vtbl_tbl[0])["_user"] = new VarNode("_user", cstrType);
     (*vtbl_tbl[0])["_curtime"] = new VarNode("_curtime", cstrType);
     (*vtbl_tbl[0])["_loadave"] = new VarNode("_loadave", intType);
     (*vtbl_tbl[0])["_numusers"] = new VarNode("_numusers", intType);
     (*vtbl_tbl[0])["_osname"] = new VarNode("_osname", cstrType);
     (*vtbl_tbl[0])["_osvers"] = new VarNode("_osvers", intType);
     (*vtbl_tbl[0])["_wrvers"] = new VarNode("_wrvers", intType);
     (*vtbl_tbl[0])["WR_SUCCESS"] = new VarNode("WR_SUCCESS", intType);
     (*vtbl_tbl[0])["WR_DENY"] = new VarNode("WR_DENY", intType);
     (*vtbl_tbl[0])["WR_ALLOW"] = new VarNode("WR_ALLOW", intType);
     (*vtbl_tbl[0])["WR_NONE"] = new VarNode("WR_NONE", intType);
     (*vtbl_tbl[0])["wr_copyin"] = new VarNode("wr_copyin", funcType(intType));
     (*vtbl_tbl[0])["wr_copyout"] = 
                   new VarNode("wr_copyout", funcType(intType));
     (*vtbl_tbl[0])["wr_copyinstr"] = 
                   new VarNode("wr_copyinstr", funcType(intType));
     (*vtbl_tbl[0])["wr_substitute"] = new VarNode("wr_substitute", intType);
     
     (*vtbl_tbl[0])["wr_mutex_init"] = 
             new VarNode("wr_mutex_init", funcType(voidType));
     (*vtbl_tbl[0])["wr_mutex_destroy"] =
             new VarNode("wr_mutex_destroy", funcType(voidType));
     (*vtbl_tbl[0])["wr_mutex_enter"] =
             new VarNode("wr_mutex_enter", funcType(voidType));
     (*vtbl_tbl[0])["wr_mutex_exit"] =
             new VarNode("wr_mutex_exit", funcType(voidType));
}

/* return 1 if successful, otherwise 0 */
int VarStore::insVarInTopTbl(const char *n, VarNode *vn) 

{
     /* return 0 if there is already a variable in the table and has
        a different type from the current one */
     if (!toptbl()->insert(make_pair(n, vn)).second && 
            !(*toptbl())[n]->type.equiv(vn->type)) 
     {
          return 0;
     } else {
          return 1;
     }
}

ostream& RecTypeInfo::print(ostream& out) 

{
     VarTbl::iterator iter;
     out << "{ ";
     for (iter = vtbl->begin(); iter != vtbl->end(); iter++) {
          VarNode *vn = (*iter).second;
          if (vn != NULL)
              out << vn->type.get_name() << "   " << vn->name << ";  "; 
          else
              cerr << "Null vn\n";
     }
     out << "  }";
     return out;
};

Type concat(const Type& t1, const Type& t2)

{
     if (t1.tc == TC_TMP)
	return Type(t2.tc);
     if (t2.tc == TC_TMP)
	return Type(t1.tc);

     switch (t1.tc) {
     case TC_UNSIGNED:   
          switch (t2.tc) {
          case TC_SHORT    :  return Type(TC_UNSIGNEDSHORT);
          case TC_LONG     :  return Type(TC_UNSIGNEDLONG);
          case TC_LONGLONG :  return Type(TC_UNSIGNEDLONGLONG);
          case TC_INT      :  return Type(TC_UNSIGNEDINT); 
          case TC_CHAR     :  return Type(TC_UNSIGNEDCHAR); 
          default       :  return Type(TC_ERR);
          }
     case TC_SIGNED:
          switch (t2.tc) {
          case TC_SHORT    :  return Type(TC_SIGNEDSHORT);
          case TC_LONG     :  return Type(TC_SIGNEDLONG);
          case TC_LONGLONG :  return Type(TC_SIGNEDLONGLONG);
          case TC_INT      :  return Type(TC_SIGNEDINT); 
          case TC_CHAR     :  return Type(TC_SIGNEDCHAR); 
          default       :  return Type(TC_ERR);
          }
     case TC_SHORT:
          if (t2.tc == TC_INT) 
              return Type(TC_SHORT);
          else 
              return Type(TC_ERR);
     case TC_LONG:
          switch (t2.tc) {
          case TC_INT     :  return Type(TC_LONG);
          case TC_LONG    :  return Type(TC_LONGLONG);
          case TC_DOUBLE  :  return Type(TC_LONGDOUBLE);
          default      :  return Type(TC_ERR);
          }
     case TC_UNSIGNEDLONG:
          switch (t2.tc) {
          case TC_INT     :  return Type(TC_UNSIGNEDLONG);
          case TC_LONG    :  return Type(TC_UNSIGNEDLONGLONG);
          default      :  return Type(TC_ERR);
          }

     default:   
	  return Type(TC_ERR);
     }
}

/* generate code to clean up for addon type, e.g., string, buf */
source *Type::gen_final(char *vname)

{
     source *src = new source;  
     if (tc == TC_STR) {
          SourceAppend3(src, "wr_obj_del(", vname, ");");
     } else if (tc == TC_ARRAY) {
          ArrayTypeInfo *ati = (ArrayTypeInfo *) info;
          source *prefix = new source;
          source *body; 
          SourceAppend2(prefix, vname, "[i]");
          body = ati->etype.gen_final(prefix->all());
          if (!body->is_empty()) {
              body->indent("        ");
              SourceAppend6(src, "{   int i;\n",
 	                         "    for (i = 0; i < ", ati->size, "; i++)\n",
                                 body, "}\n");
          }
          delete prefix; delete body;
     } else if (tc == TC_DBUF) {
     } 
     return src;
}


int EnumTypeInfo::addEnum(string s, Expr *c) 

{
     extbl.insert(make_pair(s, c));
}

int EnumTypeInfo::addEnum(string s) 

{
     extbl.insert(make_pair(s, new ConstExpr(1)));
}

