/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 *  $Id: type.h,v 1.11 2001/02/23 19:02:03 stevek Exp $
 */
#ifndef _TYPE_H_0699
#define _TYPE_H_0699

#ifdef WR_WINNT
// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786) 
#endif

#include <stdio.h>
#include <string>

#include "conf.h"

#ifdef WR_UNIX
#include <libio.h>
#include <iostream.h>
#include <string.h>
#endif

#include <map>
#include <set>
#include <list>

#ifdef WR_WINNT
#include <iostream>
#include <string>
#include <utility>
#endif

#include "smap.h"

#ifndef WR_BROKEN_NAMESPACES
using namespace std;
#endif

typedef set<string> StrSet;

#define VC_EMPTY 		0
#define VC_AUTO  		1
#define VC_REGISTER 	2
#define VC_STATIC		4
#define VC_EXTERN		8
#define VC_GLOBAL		16  /* Global */
#define VC_LOCAL     	32  /* Local */
#define VC_ENV			64  /* variable is an environment var */

class source;

enum TypeCode { TC_ERR = -1, TC_UNKNOWN = 0, 
                TC_TMP, TC_VOID, TC_CHAR, TC_WSS_CHAR_T,
			 TC_SIGNEDCHAR, TC_UNSIGNEDCHAR,
                TC_SHORT, TC_SIGNEDSHORT, 
                TC_INT, TC_SIGNED, TC_SIGNEDINT, 
                TC_LONG, TC_LONGLONG, 
                TC_SIGNEDLONG, TC_SIGNEDLONGLONG,
                TC_UNSIGNED, TC_UNSIGNEDINT, TC_UNSIGNEDSHORT,
                TC_UNSIGNEDLONG, TC_UNSIGNEDLONGLONG, 
                TC_FLOAT, TC_DOUBLE, TC_LONGDOUBLE, TC_STR, TC_DBUF,
                TC_ENUM, TC_ARRAY, TC_STRUCT, TC_UNION,
                TC_FUNC, TC_PTR, TC_NAME,
                TC_DEF, TC_DBROW, TC_DBTABLE, TC_OPSET
              };


struct VarNode;
struct Expr;
typedef map<string, VarNode *, less<string> > VarTbl;
typedef map<string, Expr *, less<string> > ExprTbl;

class TypeInfo {
public:
     TypeInfo()  {};     
     virtual ostream& print (ostream& out) { 
          return out << "TypeInfo" << endl; 
     };
};

class Type {
public:
    TypeCode tc;
    TypeInfo *info;
    Type(TypeCode t, TypeInfo *i=NULL)   {  tc = t;     info = i; 	}
    Type(int t, TypeInfo *i=NULL)   {  
         tc = (TypeCode) t;     
         info = i; 	
    }
    Type (const Type& t)                 {  tc = t.tc;  info = t.info;	}
    Type ()                              {  tc = TC_UNKNOWN; info = NULL;	}
    Type& operator=(const Type& t) {  
         tc = t.tc;
         info = t.info;
         return *this;
    }
    Type type();
    int  fillin_type(Type t);
    char *str();

    source *gen_final(char *vname);
    ostream& print(ostream& out) const;
    friend ostream& operator<<(ostream& out, const Type& t);
    Type get_elmtype(char *s = NULL) const;
    int  assmt_chk(const Type&, char *);
    int  equop_chk(const Type&);
    Type addop_chk(const Type&);
    Type subop_chk(const Type&);
    Type mulop_chk(const Type&);
    int  relop_chk(const Type&);
    int  shfop_chk(const Type&);
    int  modop_chk(const Type&);
    int  bitop_chk(const Type&);
    int  logop_chk(const Type&);
    int  equiv(const Type&) const;
    char *get_name();
    int  is_rectype() const  {  return tc == TC_STRUCT
                                    || tc == TC_UNION
                                    || tc == TC_DBTABLE;  }
    int  is_functype() const {  return tc == TC_FUNC;                   }
    int  is_chartype() const {  return tc == TC_CHAR 
                                    || tc == TC_UNSIGNEDCHAR
                                    || tc == TC_WSS_CHAR_T
                                    || tc == TC_SIGNEDCHAR;             }
    int  is_ptrtype()  const {  return tc == TC_PTR
                                    || tc == TC_STR
                                    || tc == TC_DBUF;                   }
    int  is_ptrtorec() const;
    int  is_strtype() const;     
    int  is_inttype() const;
    int  is_floattype() const;
    
};

inline int operator==(const Type& t1, const Type& t2)
{
     switch (t1.tc) {
     case TC_ENUM    :
     case TC_STRUCT  :
     case TC_UNION   :   return t1.tc == t2.tc && t1.info == t2.info;
     case TC_PTR     :
     case TC_ARRAY   :   return t1.tc == t2.tc && 
                             (t1.get_elmtype() == t2.get_elmtype() 
				|| t1.get_elmtype() == TC_VOID || t2.get_elmtype() == TC_VOID);
     case TC_FUNC    :   return t1.tc == t2.tc && 
                              t1.get_elmtype() == t2.get_elmtype();
                             
     case TC_DBROW   :   return t1.tc == t2.tc;
     default      :   return t1.tc == t2.tc;
     }
}

#ifndef __SGI_STL_INTERNAL_RELOPS

inline int operator!=(const Type& t1, const Type& t2)
{
  return !(t1 == t2); 
}

#endif

inline ostream& operator<<(ostream& out, const Type& t) {
     return t.print(out);
}

class EnumTypeInfo : public TypeInfo {
public:
     int cval;
     ExprTbl extbl;
     EnumTypeInfo()     {   cval = 0;  };
     int addEnum(string s, Expr *c);
     int addEnum(string s);
};

class RecTypeInfo : public TypeInfo {
public:
     char *name;
     int count;
     VarTbl *vtbl;
     RecTypeInfo (VarTbl *v = NULL, char *s = "") {   
         vtbl = v;      
         name = strdup(s);
     }
     ostream& print (ostream& out);
};

class ArrayTypeInfo : public TypeInfo {
public:
     Type etype;
     char *size;
     ArrayTypeInfo(const Type& t, const char *s="") {   
         etype = t;  size = strdup(s);  
     }
//     void print (ostream& out)             {   out << etype << "[]";     }
};

class PtrTypeInfo : public TypeInfo {
public:
     Type etype;
     PtrTypeInfo(const Type& t)          {   etype = t;                }
//     void print (ostream& out)           {   out << etype << "*";      }
};

#define IS_GLBFUNC 0x0
#define IS_DBFUNC 0x1
#define IS_LIBFUNC 0x2
#define IS_LOCALFUNC 0x4
#define IS_WI_LOCALFUNC 0x8

class FuncTypeInfo : public TypeInfo {
public:
     Type rtype;
     int  flag;            /* IS_DBFUNC, IS_WRFUNC, IS_LOCALFUNC */;  
     VarTbl *vtbl;         /* contains the parameters  */
     FuncTypeInfo(const Type& t, VarTbl *tbl = NULL, int f=0) {
         vtbl = tbl;
         rtype = t;
         flag = f;
     }
     ostream& print (ostream& out)       {   return out << "func ";   }
};

class NameTypeInfo : public TypeInfo {
public:
     char *name;
     NameTypeInfo(char *c = "")          {   name = strdup(c);         }
     ostream& print (ostream& out)       {   return out << name;       }
};

typedef smap<Type *> TypeTbl;

class VarNode {
public:
     char    *name;		/* name of the variable */
     int     sflag;		/* storage flag		*/
     Type    type;		/* type of the variable */
     short   ref; 		/* number of time it has been reference */
     short   def;		/* number of time it has been defined   */
                                /* in an assignment 	*/

     VarNode (const char *n, const Type& t, int sf = VC_EMPTY)  {
         name = strdup(n);
         type = t;
         sflag = sf;
         ref = def = 0;
     };
};

typedef list<VarNode *> VarList;
typedef multimap<string, int>   VarRefTbl;

extern Type concat(const Type&, const Type&);
extern Type tmpType;
extern Type errType;
extern Type voidType;
extern Type intType;
extern Type floatType;
extern Type charType; 
extern Type cptrType;
extern Type cstrType;
extern Type carrType;
extern Type strType;
extern Type vptrType;

class Scope {
	VarTbl 	vtbl;
	VarList 	deflist;
	VarList	uselist;

	/** insert variable that has been assigned */
	void insDef(VarNode *vn) {
		deflist.push_back(vn);
		return;
	}

	/** insert variable that has been used/referenced */
	void insUse(VarNode *vn) {
		uselist.push_back(vn);
		return;
	}
};

class VarStore {
     int 		top;
     VarTbl 	*vtbl_tbl[10];
     TypeTbl 	typetbl;
public:
     typedef VarTbl::iterator iterator;
     VarStore();
     void newtbl(int s=10) {   
         if (top < 9) 
              vtbl_tbl[++top] = new VarTbl; 
         else {         
              cerr << "Too many var table\n";
              exit(0);
         }
     };
     void deltop()     	    {   if (top > 0) delete vtbl_tbl[top--];      };
     void pushtbl(VarTbl *vtbl)  {   
         if (top < 9) 
             vtbl_tbl[++top] = vtbl;       
         else {         
              cerr << "Too many var table\n";
              exit(0);
         }
     };
     VarTbl& gtbl()         {   return *vtbl_tbl[0];          };
     VarTbl *toptbl()       {   return vtbl_tbl[top];         };
     VarTbl *poptbl()       {   return vtbl_tbl[top--];       };


     VarNode *operator [] (const char *s) {
          int i;
          for (i=top; i>=0 ; i--) {
               VarTbl::iterator iter = vtbl_tbl[i]->find(s);
               if (iter != vtbl_tbl[i]->end())
                    return (*iter).second;
          }
          if (!strncmp(s, "NTERM_", 6)) {
              VarNode *vn = new VarNode(s, intType);
              (*vtbl_tbl[0])[string(s)] = vn;
              return vn;
          }
          return NULL;
     }

     int add_type(const char *s, Type *t) {
          return typetbl.insert(s, t);
     }

     Type *get_type(const char *s) {
          return typetbl[s];
     }

     /* 
      * return the level of nested scope in which the variable is found
      * 0 is global, 1 is the next and so on ...
      * -1 mean variable not found
      */
     int get_scopelevel(const char *s) {
          int i;
          for (i=top; i>=0 ; i--) {
               VarTbl::iterator iter = vtbl_tbl[i]->find(s);
               if (iter != vtbl_tbl[i]->end())
                    return i;
          }
          return -1;
     }     
     
     int get_scopelevel(void) {
         return top;
     }

	/**Perform the necessary step when enter a new scope
      * 1) create a new Scope object (symbtl table, varuse/ref tables)
	 * 2) push the Scope object onto the top of the scope stack
	 */
	void enter_scope(void) {
	}

	/**Perform the necessary step when exit a scope
      * 1) create a new Scope object (symbtl table, varuse/ref tables)
	 * 2) push the Scope object onto the top of the scope stack
	 */
	void leave_scope(void) {
	}

     int insVarInTopTbl(const char *, VarNode *);

     iterator begin()       {   return vtbl_tbl[0]->begin();  }
     iterator end()         {   return vtbl_tbl[0]->end();    }

};


#endif /* _TYPE_H_O699 */
