%:- unset(cover)?
:- set(c,10)?
:- set(h,100)?
:- set(posonly)?
:- set(r, 1000)?
%:- unset(condition)?            

:- modeh(*, writeop(+subj, +proc, +path, +fattr, +pmode))?
%:- modeh(*, writeop(s(+id, +id, +id), +path, +fattr, +pmode))?

:- modeh(*, readop(+subj, +proc, +path, +fattr, +pmode))?
%:- modeh(*, readop(s(+id, +id, +id), +path, +fattr, +pmode))?

:- modeh(*, creatop(+subj, +proc, +path, +fattr, +pmode))?

:- modeh(*, chownop(+subj, +proc, +path, +fattr, +pmode, id))?
:- modeh(*, chmodop(+subj, +proc, +path, +fattr, +pmode, pmode))?

:- modeb(1, own(+subj, +fattr))?
:- modeb(1, group(+subj, +fattr))?
:- modeb(1, createdby(+proc, +fattr))?
:- modeb(1, createdby(+subj, +fattr))?
:- modeb(1, worldreadable(+pmode))?
:- modeb(1, groupreadable(+pmode))?
:- modeb(1, ownerreadable(+pmode))?
:- modeb(1, worldwritable(+pmode))?
:- modeb(1, groupwritable(+pmode))?
:- modeb(1, ownerwritable(+pmode))?
:- modeb(1, isfile(+path, #path))?
:- modeb(5, indir(+path, #path))?
:- modeb(1, own1(#id, +fattr))?

% Definitions of subject

% Definitions of user ID
id(U) :- int(U), U =< 65535, U >= 0.
pid(X) :- int(X).

subj(s(Sid, U, G)) :- id(U), id(G), pid(Sid).
proc(p(Pid, Euid, Egid)) :- pid(Pid), id(Euid), id(Egid).

createdby(p(Pid, Eu, Eg), f(Ouid, Ogid, I)) :- create(Pid, I).
createdby(S, f(Ouid, Ogid, I)) :- create(S, I).

% Definitions of path   /etc/passwd is represented as [etc, passwd]
path([]).
path([A]).
path([A|B]) :- path(B).

% Definition of indir
indir([X|D], D).
indir([X|Y], D) :- indir(Y,D).

% Definitions of permission mode
pmode(m(W, G, O)) :- int(W), int(G), int(O), W =< 7, W >= 0, G =< 7, G >= 0, O =< 7, O >= 0.


isfile(A, B) :- A = B.

% Definitions of fattr, file attributes

fattr(f(Ouid, Ogid, I)) :- id(Ouid), id(Ogid), int(I).

% Definitions of own
own(s(Sid, U, G), f(U, OG, I)).
own1(U, f(U, G, I)). 
group(s(U, G, Pid), f(OU, G, I)).
group1(G, f(U, G, I)).

worldreadable(m(O, G, W)) :- W >= 4.
groupreadable(m(O, G, W)) :- G >= 4.
ownerreadable(m(O, G, W)) :- O >= 4.

worldwritable(m(O, G, W)) :- Y is W / 2, Z is floor(Y), odd(Z).
groupwritable(m(O, G, W)) :- Y is G / 2, Z is floor(Y), odd(Z).
ownerwritable(m(O, G, W)) :- Y is O / 2, Z is floor(Y), odd(Z).

:- hypothesis(readop(A,B,C,D,E), groupreadable(E), _).
:- hypothesis(readop(A,B,C,D,E), ownerreadable(E), _).

:- hypothesis(writeop(A,B,C,D,E), groupreadable(E), _).
:- hypothesis(writeop(A,B,C,D,E), own1(X,D), _).

:- hypothesis(creatop(A,B,C,D,E), groupreadable(E), _).

% define the privileged and generality

gval(10) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, 0).
gval(9) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, 1).
gval(8) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, 2).
gval(6) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, 3).
gval(5) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, 4).
gval(4) :- hypothesis(Head, Body, _), in(indir(X, D), Body), length(D, E), E >= 5.
gval(5) :- hypothesis(Head, Body, _), in(worldreadable(X), Body).
gval(5) :- hypothesis(Head, Body, _), in(groupreadable(X), Body).
gval(5) :- hypothesis(Head, Body, _), in(ownerreadable(X), Body).
gval(5) :- hypothesis(Head, Body, _), in(own1(X,Y), Body).
gval(4) :- hypothesis(Head, Body, _), in(own(X,Y), Body).
gval(1) :- hypothesis(Head, Body, _), in(isfile(X,Y), Body).
gval(1) :- hypothesis(Head, Body, _), in(createdby(X,Y), Body).
                                                                                                          
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%								   %
%                            Pruning 				   %
% 								   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% allow only one indir clause in the body
prune(Head, Body) :- in(indir(N, A), Body), in(indir(N, B), Body), not(A==B).

% allow either indir or isfile in the body
prune(Head, Body) :- in(isfile(N, A), Body), in(indir(N, B), Body).

% disallow indir(X, []) in the body, too general
prune(Head, Body) :- in(indir(X,[]), Body).

prune(Head, Body) :- in(ownerreadable(W), Body), not(in(own(X,Y), body)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%								   %
%                            Privileges 		 	   %
% 								   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%priv(0) :- readop(S, P, N, F, M), worldreadable(M).
%priv(1) :- readop(S, P, N, F, M), not(worldreadable(M)).
%priv(1) :- readop(S, P, N, F, M), not(worldreadable(M)), indir(N, [etc]).
%priv(2) :- readop(S, P, N, F, M), not(worldreadable(M)), own(root, F).
%priv(5) :- readop(S, P, N, [spwd.db, etc], M).

%priv(1) :- writeop(S, P, N, F, M), not(worldwritable(M)).
%priv(1) :- writeop(S, P, N, F, M), not(worldreadable(M)), own(root, F).
%priv(1) :- writeop(S, P, N, F, M), not(worldreadable(M)), indir(N, [etc]).

priv(10) :- readop(s(352,215,10), p(353, 0, 20), ['spwd.db', 'etc'],f(X,7,184332), m(6, 0, 0)).
priv(8) :- readop(s(352,215,10), p(353, 0, 20), X, f(3,7,184332), m(6, 0, 0)).
priv(1) :- readop(s(352,215,10), p(353, 0, 20), X, f(3,7,184332), m(Y, Z, Q)), Q >= 4.       
priv(8) :- readop(s(352,215,10), p(353, 0, 20), X, f(3,7,184332), m(Y, Z, Q)), Q < 4.       
priv(9) :- writeop(s(352,215,10), p(353, 0, 20), ['spwd.db', 'etc'],f(3,7,184332), m(6, 0, 0)).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%								   %
%                            Generality 			   %
% 								   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%:-writeop(s(352,215,10), p(354, 0, 20), ['spwd.db', 'etc'],f(3,7,184332), m(6, 0, 0)).
%:-writeop(s(352,215,10), p(354, 0, 20), ['passwd', 'etc'],f(3,7,184332), m(6, 0, 0)).
%:-writeop(s(352,215,10), p(354, 0, 20), ['at.deny', 'etc'],f(0,7,132108), m(6, 0, 0)).

%:-readop(s(352,215,10), p(353, 0, 20), ['spwd.db', 'etc'],f(3,7,184332), m(6, 0, 0)).
%:-readop(s(352,215,10), p(353, 0, 20), ['at.deny', 'etc'],f(0,7,132108), m(6, 0, 0)).                                



