#! /usr/local/bin/perl 
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: tran.pl,v 1.8 2001/04/05 18:40:26 stevek Exp $

use Getopt::Std;

sub usage {
     print "tran.pl -c list|pl|raw [-e event|all] [-s prog,pid]\n";
     print "        [-p prog] [-o outfile] infile\n";
     exit(0);
}

if ($#ARGV < 0) {
     usage();
}
getopts('c:s:e:p:o:i');
if (defined($opt_i)) {
     usage();
}

my $cmd;
if (defined($opt_c)) {
     $cmd = $opt_c;
} else {
     $cmd = "list";
}

if (defined($opt_e)) {
     if ($opt_e eq "all") {
         $event = "\\w+";
     } else {
         $event = $opt_e; 
     }
} else {
     $event = "\\w+";
} 
print "Event = $event\n";

if (defined($opt_s)) {
     if (($sprog, $sid) = $opt_s =~ /(\w+),(\d+|all)/) {
         if ($sprog eq "all") {
             $sprog = "\\w+";
         } 
         if ($sid eq "all") {
             $sid = "\\d+";
         }
     } else {     
         usage();
     }
} else {
     $sid = "\\d+";
     $prog = "\\w+";
}

if (defined($opt_p)) {
     if ($opt_p eq "all") {
         $sprog = "\\w+";
     } else {
         $sprog = $opt_p;
     }
}

print "Subj ($sprog, $sid)\n";

@mpty = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 
         'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
         'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
         'u', 'v', 'w', 'x', 'y', 'z');

sub convertpath {
     if ($_[0] =~ /^\/(.*)/)  {
         my @items1;
         my @items = split('/',$1);
         @items = reverse(@items);
         @items = grep(push(@items1, "'$_'"), @items);
         my $s1 = join(", ", @items1);
         my $plist = "[" . $s1 . "]"; 
         return $plist;
     } else {
         return "[]";
     }
}

sub convertmode {
     my $mode = "$_[0]";
     my @digits = split('', $mode); 
     @digits = reverse(@digits);
     return "m($digits[2], $digits[1], $digits[0])";
}

sub processline {
     my $print = 0;
     my $line = $_;
     my @fields = split;
     my $s_pid, $s_pname, $pid, $pname, $ruid, $euid, $egid, $fd, $path;
     ($s_pname, $s_pid, $pname, $pid, $ruid, $euid, $egid) = /\((\w+),(\d+)\):(\w+),(\d+),(\d+),(\d+),(\d+)/;
     #print "s_pname = $s_pname, s_pid = $s_pid\n";
     if (($s_pname =~ /$sprog/) && $s_pid =~ /$sid/) {
          if ($fields[1] eq "open") {
               $fd = $fields[3]; 
               $path = $fields[4]; 
               $fdtable[$fd]{'path'} = $path;
               #print "$field[4] HHH\n";
               $obj = $field[5];
               $fields[5] =~ /<(\d+):(\d+),(\d+),(\d+)>/;
               $fdtable[$fd]{'ouid'} = $ouid = $1;
               $fdtable[$fd]{'ogid'} = $2;
               $fdtable[$fd]{'nodeid'} = $3;
               $fdtable[$fd]{'mode'} = $4;
               #print "($pname,$pid)\t", $fdtable[$fd]{'mode'}, "\n";
          } elsif ($fields[1] eq "creat") {
               $fd = $fields[2]; 
               $path = $fields[3]; 
               $fdtable[$fd]{'path'} = $path;
               $fields[4] =~ /<(\d+):(\d+),(\d+),(\d+)>/;
               $ouid = $1;
               $ogid = $2;
               $nodeid = $3;
               $mode = $4;
               $print = 1;
          } elsif ($fields[1] eq "connect") {
               $fd = $fields[2]; 
               $path = $fields[3]; 
          } elsif ($fields[1] =~ /read|write/) {
               if ($fields[2] eq "FILE") {
                    $path = $fields[3]; 
                    if ($path =~ /^\//) {
                        $print = 1;
                    }
                    $fd = $fields[4]; 
               } elsif ($fields[2] eq "SOCK") {
                    $path = $fields[3];
                    $print = 1;
                    if ($path =~ /^\//) {
                        $fields[1] = $fields[1] . "usock";
                    } else {
                        $fields[1] = $fields[1] . "isock";
                    }
               } else {
                    $path = $fields[3];
                    #print "path = $path\n";
                    if ($path =~ /^\//) {
                        $print = 1;
                    } else {
                        my @dinfo = split('-', $path);
                        if ($dinfo[0] eq "CharDev") {
                             my $major = hex $dinfo[1] & 0xff00;
                             my $minor = hex $dinfo[1] & 0xff;
                             #print "major = $major, minor = $minor\n";
                             if ($major == 0x500) {
                                  $print = 1;
                                  $path = "/dev/ttyp" . $mpty[$minor];
                             }
                        } elsif ($dinfo[0] eq "BlkDev") {
                        } else {
                        } 
                    };
                    $fd = $fields[4]; 
               } 
               if ($fields[5] =~ /<(\d+):(\d+),(\d+),(\d+)>/) {
                    $ouid = $1;
                    $ogid = $2;
                    $nodeid = $3;
                    $mode = $4;
               } else { 
                    $ouid = $fdtable[$fd]{'ouid'};     
                    $ogid = $fdtable[$fd]{'ogid'};     
                    $nodeid = $fdtable[$fd]{'nodeid'};     
                    $mode = $fdtable[$fd]{'mode'};     
               }
          };
          if ($fields[1] =~ /$event/) {
               my $newpath = convertpath($path);
               if ($print == 1) {
                    print DST "$fields[1]op(s($s_pid,$ruid,$egid), p($pid,$euid,$egid),", 
                        convertpath($path), ", f($ouid,$ogid,$nodeid), ", 
                        convertmode($mode), ").\n";
               } else {
                    print STDERR "$line";
                    print STDERR "$fields[1]op(s($_pid,$ruid,$egid), p($pid,$euid,$egid),", 
                        convertpath($path), ", f($ouid,$ogid,$nodeid), ", 
                        convertmode($mode), ").\n";
               }
          };
          if ($fields[1] eq "creat") {
               print DST "create($pid, $nodeid).\n";
               print DST "create(s($s_pid, $ruid, $egid), $nodeid).\n";
          }
     }
};

my $source = $ARGV[0];

open( SRC, "<$source" ) || die "$source: $!";

if (defined($opt_o)) {
    open( DST, ">$opt_o" )   || die "$opt_o: $!";
} else {
    open( DST, ">-");
};

if ($cmd eq "list") {

} elsif ($cmd eq "pl") {
    while ( <SRC> ) {
         processline();
    };
    print DST ":- generalise($event", "op/5)?\n";
    close (DST);
    $progolname = $event . "op.pl";
    if (defined($opt_o)) {
#        if (stat($progolname)) {
#            system("cat $progolname $opt_o > $opt_o.pl");
#        } else {
            system("cat allop.pl $opt_o > $opt_o.pl");
#        }
    }
} elsif ($cmd eq "raw") {

};

close (SRC);
