/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: comm.c,v 1.5 2001/02/23 19:02:04 stevek Exp $
 */
/***************************************************************************
                          comm.c  -  description
                             -------------------
    begin                : Fri May 26 2000
    copyright            : (C) 2000 by Calvin Ko
    email                : ko@tislabs.com
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "comm.h"

char errmsg[256];
char netbuf[256];

SockLink *new_SockLink()

{
	SockLink *s;
	s = (SockLink *) malloc(sizeof(SockLink));
	s->rhost[0] = '\0';
	s->rhost[HOST_STRLEN] = '\0';
	return s;
}




/*-------------------------------------------------------------------------------
 * connectsock - allocate & connect a socket using TCP or UDP			
 * return    	- socket descriptor on success, -1 on fail and set errmsg			
 *-------------------------------------------------------------------------------
 */
int connectsock(
	char *host,		/* name of host */
	int  port,		/* port */
	char *protocol		/* protocol */
)

{

     int s;
	int type;					/* socket type 						*/
	struct hostent     *phe;		/* pointer to host information entry 		*/
 	struct protoent    *ppe;		/* pointer to prototol information entry 	*/
	struct sockaddr_in	sin;		/* an Internet endpt address 				*/

	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;

 	/* Map host name to IP address, allowing for dotted decimal */
	if ((phe = gethostbyname(host)) != NULL)
		bcopy(phe->h_addr, (char*)&sin.sin_addr, phe->h_length);
	else if ( (sin.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
		sprintf(errmsg, "Can't resolve host name %s\n", host);
		return -1;
	}

     sin.sin_port = htons(port);

     /* Map protocol name to protocol number */
	if ( (ppe = getprotobyname(protocol)) == 0) {
		sprintf(errmsg, "Can't get protocol tcp\n");
		return -1;
	}

	/* User protocol to choose a socket type */
	if (strcmp(protocol, "udp") == 0)
		type = SOCK_DGRAM;
	else	
		type = SOCK_STREAM;

	s = socket(PF_INET, type, ppe->p_proto);
	if (s < 0) {
		sprintf(errmsg, "Can't create socket\n");
		return -1;
	}

     if (connect(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		sprintf(errmsg, "Can't connect to %s:%d\n", host, port);
		return -1;
     }
     return s;
}

int connectTCP(char *host, int port)		

{
 	return connectsock(host, port, "tcp");
}

int connectUDP(char *host, int port)		

{
 	return connectsock(host, port, "ucp");
}


/*------------------------------------------------------------------------------
 * passivesock - allocate and bind a server socket using TCP or UDP
 *
 * return: 	the socket descriptor (>=0) if successful
 * 			otherwise return -1 and set errmsg;
 *------------------------------------------------------------------------------
 */
int passivesock(
	int	port,		/* port number */
	char *protocol, 	/* name to protocol, tcp or udp */
	int  qlen			/* maximum length of the request queue */
)

{
	struct protoent    	*ppe;	/* pointer to protocol info entry */
	struct sockaddr_in	sin;		/* an Internet endpt address */
	int	s, type;				/* socket descriptor and type */

	bzero((char *)&sin, sizeof(sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;

	sin.sin_port = htons(port);

     /* Map protocol name to protocol number */
	if ( (ppe = getprotobyname("tcp")) == 0) {
		sprintf(errmsg, "Can't get protocol tcp\n");
		return -1;
	}

     /* Use protocol to choose a socket type */
	if (strcmp(protocol, "udp") == 0)
		type = SOCK_DGRAM;
	else
		type = SOCK_STREAM;
	
	/* Allocate a socket */	
	s = socket(PF_INET, type, ppe->p_proto);
	if (s < 0) {
		sprintf(errmsg, "Can't create socket\n");
		return -1;
	};

	/* Bind the socket */
	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
		sprintf(errmsg, "Can't bind to port %d\n", port);
		return -1;
	};

	if (type == SOCK_STREAM && listen(s, qlen) < 0) {
		sprintf(errmsg, "Can't listen on port %d\n", port);
		return -1;
	};

	return s;
}


/*
 * passivesock - allocate and bind a server socket using TCP
 *
 * return: 	the socket descriptor (>=0) if successful
 * 			otherwise return -1 and set errmsg;
 */
int passiveTCP(int port, int qlen)

{
	return passivesock(port, "tcp", qlen);
}

int passiveUDP(int port, int qlen)

{
	return passivesock(port, "udp", qlen);
}

int sendstr(int s, char *line)

{
	write(s, line, strlen(line));
	return 1;
}






