/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: netcmd.c,v 1.8 2001/02/23 19:02:04 stevek Exp $
 */

#include <wr_conf.h>
#include <stdarg.h>
#include <wsi_stdlib.h>

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "libwrv.h"
#include "comm.h"
#include "netcmd.h"

/* fill in the command header
 * buf contain the WQL command without the first 8 byte header
 * which is the WQL command name "WQL0" and the total size of the
 * command (int)
 */
int fillWQLCmdHdr(char *buf, int size)

{
	size += 8;
	strncpy(buf, "WQL0", 4);
	bcopy(&size, buf + 4, sizeof(size));
	return size;
}

/** Pack an Insert Row Command for transferring to remote host
 *  cbuf 		- the command buffer containing the packed command
 *  tablename 	- the name of the table (cstr)
 *  fields 	- the content of the row, NULL terminated array of cstr
 */
int packInsRowCmd(char *cbuf, char *tablename, char **fields)

{
	short i;
	short cmd = WQL_INSERTROW;
	int len = 2;
	bcopy(&cmd, cbuf, len);
	len += 2;
	strncpy(cbuf+len, tablename, WQL_NAME_MAX);
	len += strlen(tablename) + 1;	
	for (i = 0; fields[i] != NULL; i++) {
		strncpy(cbuf+len, fields[i], WQL_NAME_MAX);
		len += strlen(fields[i]) + 1;
	};
	bcopy(&i, cbuf + 2, 2);
	return len;	
}

int packUpdateRowCmd1(char *cbuf, char *tablename, char *cond, char **assmts)

{
	short cmd = WQL_UPDATEROW;
	int len = 4;
	int i;
	bcopy(&cmd, cbuf, 2);
	
	strncpy(cbuf+len, tablename, WQL_NAME_MAX);
	len += strlen(tablename) + 1;
		
	strncpy(cbuf+len, cond, WQL_NAME_MAX);
	len += strlen(cond) + 1;
	
	for (i = 0; assmts[i] != NULL && assmts[i + 1] != NULL; i = i + 2) {
		strncpy(cbuf+len, assmts[i], WQL_NAME_MAX);
		len += strlen(assmts[i]) + 1;
		strncpy(cbuf+len, assmts[i + 1], WQL_NAME_MAX);
		len += strlen(assmts[i + 1]) + 1;
	};
	bcopy(&i, cbuf + 2, 2);
	return len;
};


int packUpdateRowCmd(char *cbuf, char *tablename, char *fieldassmt, char *cond)

{
	short cmd = WQL_UPDATEROW;
	int len = 2;
	bcopy(&cmd, cbuf, 2);
	strncpy(cbuf+len, tablename, WQL_NAME_MAX);
	len += strlen(tablename) + 1;
		
	strncpy(cbuf+len, fieldassmt, WQL_NAME_MAX);
	len += strlen(fieldassmt) + 1;
	
	strncpy(cbuf+len, cond, WQL_NAME_MAX);
	len += strlen(cond) + 1;
	return len;
};

/** UnPack an Insert Row Command
 *  cbuf 		- the command buffer containing the packed command
 *  len 		- the length of the data in cbuf
 *  wc (out)	- the struct which will contain the command on return
 */
int unpackInsRowCmd(char *cbuf, int len, WQLCmd *wc)
{
	int 	i;
	int	index = 2;
	bcopy(cbuf + index, &wc->nitems, sizeof(wc->nitems));
	index += sizeof(wc->nitems);

	wc->tblname = cbuf + index;
	/* move to next item */
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	wc->rowitem = (char **) malloc( (wc->nitems + 1) * sizeof(char *) );
	for (i = 0; i < wc->nitems; i++) {
		wc->rowitem[i] = cbuf + index;
		while (index < len && *(cbuf + index) != '\0')
			index ++;
		if (index >= len)
			return -1;
		index++;
	};
	wc->rowitem[wc->nitems] = NULL;
	return 1;
};
int unpackUpdateRowCmd1(char *cbuf, int len, WQLCmd *wc)

{
	int	index = 2;
	
	/* unpack tablename */
	wc->tblname = cbuf + index;
	/* move index to next item */
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	/* unpack cond */
	wc->cond = cbuf + index;
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	/* unpack assmt items */
	return 1;
};
int unpackUpdateRowCmd(char *cbuf, int len, WQLCmd *wc)

{
	int	index = 2;
	
	/* unpack tablename */
	wc->tblname = cbuf + index;
	/* move index to next item */
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	/* unpack field assignment */
	wc->assmt = cbuf + index;
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	/* unpack cond */
	wc->cond = cbuf + index;
	while (index < len && *(cbuf + index) != '\0')
		index++;
	if (index >= len)
		return -1;
	index++;
	
	return 1;
};

/* unpack a WQL Command
 *  cmd : buffer holding the command str
 *  size: len of the command string
 *
 *  InsertRow NumofItem:int TableName:cstr ItemLists:cstrlist
 *  DeleteRow TableName:cstr condition:cstr
 *  UpdateRow NumofItem:int TableName condition:cstr (e.g., a=b) AssmtItemList:cstrlist
 *  CreatTbl TableName:cstr
 *  DropTbl TableName:cstr
*/
int unpackWQLCmd(char *cbuf, int size, WQLCmd *wc)

{
	bcopy(cbuf, &wc->cmd, sizeof(&wc->cmd));
	switch (wc->cmd) {
	case	WQL_INSERTROW:
			return unpackInsRowCmd(cbuf, size, wc);
	case	WQL_UPDATEROW:
			return unpackUpdateRowCmd(cbuf, size, wc);
	case WQL_DELETEROW:
	case WQL_CREATTBL:
	case WQL_DROPTBL:
	default:
		return -1;
	}
}

int sendCmd(char *ip, char *cmd, int len)

{
	int s;
	s = connectTCP(ip, WRD_PORT);
     if (s < 0) {
          perror(errmsg);
          return -1;
     }
	write(s, cmd, len);
	return 1;
}


/**
 * return the first row in the list rows that satisfies the condition
 * if (cond == NULL)
 *    return the next row in the list that satisfies the condition given in the previous call
 * return NULL if not found
 */
wrdb_row_t selectRow(wrdb_row_t rows, wrdb_coldef_t coldef, char *cond) {
	return NULL;
};


/** Perform an InsertRow Command
 *  return 0 on success, -errnum on error
 */
int doInsRowCmd(WQLCmd *wc)

{
	int err;
	wrv_table_id_t *tid;
	tid = wrvDbGetTableIdByName(wc->tblname);
	if (tid == NULL) {
		// handle error
		return -1;
	}	
	err = wrvDbAddTableRow(tid->handle, wc->rowitem);
	return err;
}

/** Perform an InsertRow Command
 *  return 0 on success, -errnum on error
 *  Not fully implemented. To be done
 */
int doUpdateRowCmd(WQLCmd *wc)

{
	wrv_table_id_t *tid;
	wrdb_row_t rows;
	wrdb_coldef_t coldef;
	tid = wrvDbGetTableIdByName(wc->tblname);
	if (tid == NULL) {
		// handle error
		return -1;
	}
	rows = wrvDbGetRows(tid->handle);
	coldef = wrvDbGetColumnDefinition(tid->handle, NULL);
	// row = wrvDbUpdateTableRow(rows...)
	return 1;
};


/** perform a WQL command
 *  cmd : buffer holding the command str
 *  size: len of the command string
 *
 *  InsertRow NumofItem:int TableName:cstr ItemLists:cstrlist
 *  DeleteRow TableName:cstr condition:cstr
 *  UpdateRow TableName fieldassmt:cstr condition:cstr (e.g., a=b)
 *  CreatTbl TableName:cstr
 *  DropTbl TableName:cstr
 */
int doWQLCmd(char *cbuf, int size)

{
	WQLCmd wc;
	int err = unpackWQLCmd(cbuf, size, &wc);
	if (err < 0) return err;
	switch (wc.cmd) {
	case	WQL_INSERTROW:
			return doInsRowCmd(&wc);				
	case	WQL_UPDATEROW:
			return doUpdateRowCmd(&wc);
	case WQL_DELETEROW:	
	case WQL_CREATTBL:
	case WQL_DROPTBL:
	default:
		return -1;
	}
}


int printWQLCmd(char *cbuf, int size)

{
	int i;
	int err;
	WQLCmd wc;
	err = unpackWQLCmd(cbuf, size, &wc);
	if (err < 0) {
		fprintf(stderr, "unpackWQLCmd: %s\n", wc.errmsg);
		return err;
	}
	switch (wc.cmd) {
	case	WQL_INSERTROW:
			printf("insert %s (", wc.tblname);
			for (i=0; i < wc.nitems; i++) {
				printf("%s ", wc.rowitem[i]);
			}
			printf(")\n");
			return 1;	
	case WQL_DELETEROW:
			printf("Delete Row\n");
			return 1;
	case	WQL_UPDATEROW:
			printf("update %s set %s \n", wc.tblname, wc.assmt);
			printf("       where %s\n", wc.cond);
			return 1;
	case WQL_CREATTBL:
			printf("Create Table\n");
			return 1;
	case WQL_DROPTBL:
			printf("Drop Table\n");
			return 1;
	default:
		return 0;
	}
}
