/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: server.c,v 1.10 2001/02/23 19:02:04 stevek Exp $
 */
/***************************************************************************
                          server.c  -  description
                             -------------------
    begin                : Fri May 26 2000
    copyright            : (C) 2000 by Calvin Ko
    email                : ko@tislabs.com

    $Id: server.c,v 1.10 2001/02/23 19:02:04 stevek Exp $

 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "comm.h"
#include "wdaemon.h"
#include "netcmd.h"

extern int 		h_errno;	

	
/*
 *  process_cmd - process a command
 *  cmd (in)    - contain the command
 *  result (out)- contain the reply
 *  return 	 - 1 if success, 0 if fail and set errmsg
 *
 */
int processCmd(struct WrNetCmd *cmd)

{
	if (!strncmp(cmd->name, "WQL0", 4)) {
		printf("WQL command\n");
		printWQLCmd(cmd->buf, cmd->size - WRNETCMD_HEADER_SIZE);
		doWQLCmd(cmd->buf, cmd->size - WRNETCMD_HEADER_SIZE);
	} else if (!strncmp(cmd->name, "WDB0", 4)) {
		printf("WDB command\n");
		
	} else {
		char cname[8];
		strncpy(cmd->name, cname, 4);
		cname[4] = '\0';
		printf("Invalid Cmd %s\n", cname);
	}
	return 1;
}

/* return 1 if a command is read, otherwise -errno */
int readCmd(int fd, struct WrNetCmd *cmd)

{
	int n, size;
	n = read(fd, cmd, WRNETCMD_HEADER_SIZE);
	/* When we turn a socket fd to non-blocking mode
	 *  a read will return
	 *  1) the no of byte read, if data is available for reading,
	 *  2) -1 if no data is available
	 *  3) 0 if the socket is close at the other end.
	 *  Also, select will detect when the fd changes state, i.e., either
	 *  new data is available or the socket is close at the other end.
	 */
	if (n <= 0)
		return n;
	if (n != WRNETCMD_HEADER_SIZE || cmd->size > WRNETCMD_MAX_SIZE) {

		printf("We have a problem n = %d, size = %d\n", n, cmd->size);

		/* now read all the data from the socket and discard it */
		while ((n = read(fd, cmd->buf, WRNETCMD_MAX_SIZE)) > 0);
		return n;
	}
	size = read(fd, cmd->buf, cmd->size - 8);
	return 1;
}


int doWrapperCmd(char *buf)

{
	char *cmd = strtok(buf, " \t\n");
	char *rows[20];
	int i;
	int len;
	char cbuf[1024];
	if (cmd != NULL && !strcmp(cmd, "WQLinsert")) {
		char *rhost 	= strtok(NULL, " \t\n");
		char *tbl		= strtok(NULL, " \t\n");
		if (rhost == NULL || tbl == NULL)
			return -1;
		for (i=0; i<20; i++) {
			rows[i] = strtok(NULL, " \t\n");
			if (rows[i] == NULL) break;
		};
		len = packInsRowCmd(cbuf+8, tbl, rows);
		len = fillWQLCmdHdr(cbuf, len);
		sendCmd(rhost, cbuf, len);
	}
	return -1;
}

int mainloop()

{
     struct sockaddr_in csin;      /* address of client 		*/
     int  	msock, ssock;       /* master and slave socket 	*/
     int  	alen;               /* client address len 		*/
	int		auditfd;			/* audit file descriptor 	*/
	fd_set	rfds;			/* read file descriptor set 	*/
	fd_set	afds;			/* active file descriptor set */
	

	char		buf[1024];
     struct hostent *hent;
     int		getauditfd();
     int		getauditdata(char *);

	/*
	 * get the audit file descriptor, WSS needed to be loaded
	 * 
	 */
     if ((dflag & DFLAG_NOWSS) == 0) {
		auditfd = getauditfd();
		if (auditfd < 0) {
			exit(1);
		}	
	} else {
		auditfd = -1;
	}

	msock = passiveTCP(WRD_PORT, 20);
     if (msock < 0) {
          perror(errmsg);
          exit(1);
     }

	FD_ZERO(&afds);
	FD_SET(msock, &afds);
	if ((dflag & DFLAG_NOWSS) == 0)
		FD_SET(auditfd, &afds);
	
     while (1) {
          char 	*caddr;
          char		nbuf[256];
		int		fd;

		bcopy((char *) &afds, (char *) &rfds, sizeof(rfds));
		if (dflag & DFLAG_VERBOSE1)
			fprintf(stderr, "Waiting ... \n");
		if (select(nfds, &rfds, (fd_set *)0, (fd_set *)0, (struct timeval *)0) <0) {
			perror("select");
			if (errno == EINTR)
				continue;
			else
				exit(1);
		}
		
		if (FD_ISSET(msock, &rfds)) {	/* A new socket init come in */
			alen = sizeof(struct sockaddr_in);
          	ssock = accept(msock, (struct sockaddr *)&csin, &alen);
          	if (ssock < 0) {
               	perror("Accept fail");
          	}
          	FD_SET(ssock, &afds);
          	caddr = inet_ntoa(csin.sin_addr);

          	hent = gethostbyaddr((char *) &csin.sin_addr, 4, AF_INET);
          	if (hent == NULL) {
          		strcpy(nbuf, "Unresolved hostname");
          		fprintf(stderr, "Can't get hostname for addr %s, h_err = %d\n", caddr, h_errno);
          		FD_CLR(ssock, &afds);
          		close(ssock);
          	} else {
          		strncpy(nbuf, hent->h_name, 255);
				if (dflag & DFLAG_VERBOSE1)
          			printf("Accept request from %s:%s, fd = %d\n", caddr, nbuf, ssock);
          		
          		/* add authentication & encryption here ? */
          		
          		
          		if (linktbl[ssock] == NULL)
          			linktbl[ssock] = new_SockLink();
          		strncpy(linktbl[ssock]->rhost, nbuf, HOST_STRLEN);
          		
          		
          		fcntl(ssock, F_SETFL, O_NONBLOCK);
          	}
          	
     	} else if (auditfd > 0 && FD_ISSET(auditfd, &rfds)) { /* audit data from ID wrappers */
     		getauditdata(buf);
     		doWrapperCmd(buf);
     	}

		for (fd=0; fd<nfds; ++fd)
			if (fd != msock && fd != auditfd && FD_ISSET(fd, &rfds)) {
				struct WrNetCmd cmd;
				int n;
				
				/* read and process command */
				while ((n = readCmd(fd, &cmd)) > 0) {
					processCmd(&cmd);
				}

				if (n == 0) {	
					/* a real error occur, most likely
					 * the socket is closed by the other end */
					if (dflag & DFLAG_VERBOSE1)
						fprintf(stderr, "Close %d\n", fd);
					close(fd);
					FD_CLR(fd, &afds);
				}
			}
	}
}







