/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: wdaemon.c,v 1.5 2001/02/23 19:02:04 stevek Exp $
 */
/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Tue May 30 2000
    copyright            : (C) 2000 by Calvin Ko
    email                : 
 ***************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "comm.h"
#include "wdaemon.h"

#define FNAMELEN	256



/* Global variables */

int		nfds;				/* number of file descriptors 	*/
SockLink 	**linktbl;			/* Active socket link table 		*/
int		dflag = 0;			/* debug flag 0 - no debugging 	*/
char		logfilename[FNAMELEN+2];	/* log file name 				*/

/* End Global variables */

/* signal handler */
void dumpstatHandler(int sig)
{
	int i;
	for (i = 0; i < nfds; i++) {
		if (linktbl[i] != NULL) {
			printf("%d:\t%s\n", i, linktbl[i]->rhost);
		}	
	}
}

void usage()

{
     fprintf(stderr, "wserver [-l logfile] [-d dflag] \n");
     exit(0);
}


int main(int argc, char *argv[])

{

	
	struct sigaction 	sa;		
	int	i, ch;
	int 	mainloop();
	int	doWrapperCmd();

	
	while ((ch = getopt(argc, argv, "d:l:t:")) != EOF)
		switch(ch) {
		case 'd':		/* debug flag */
					dflag = atoi(optarg);
					continue;
		case 'l':		/* log file */
					strncpy(logfilename, optarg, FNAMELEN);
					continue;
		case 't':		/* test */
					doWrapperCmd(optarg);
					exit(0);
		default :		usage();
		};
	
		
     /*
      * initialize nfds and linktbl
      */
     nfds = getdtablesize();
     linktbl = (SockLink **) malloc(nfds * sizeof (SockLink *));
	for (i = 0; i < nfds; i++) {
		linktbl[i] = NULL;	
	}
	
	/*
      * Set up handler dumping stats data
      */
   	sa.sa_handler = dumpstatHandler;
   	sigemptyset( &sa.sa_mask );
   	sa.sa_flags = 0;
   	sigaction(SIGUSR1, &sa, NULL );

   	
   	/*
   	 * Call the mainloop
   	 */
   	mainloop();
   	return 1;	
}






