/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#include "wr_conf.h"

#ifdef WR_BROKEN_STL
#define __PUT_STATIC_DATA_MEMBERS_HERE
#endif

#ifndef DATA_H
#include "Data.h"
#endif

#ifndef MESSAGE_H
#include "Message.h"
#endif

#include "unistd.h"

void process(
        DB *db,
        Source *source)
{
    pid_to_pdi_t pid_to_pdi;
    pid_to_seq_t pid_to_seq;
    bool error;

    while (1) {
        Message *mp = source->next_message();
        error = mp->process(db, &pid_to_pdi, &pid_to_seq);
        delete mp;
        if (error)
            break;
    }
}

void usage(char *progname)
{
    cerr << progname << endl << endl <<

        "Option: "<< endl <<
        "[-r] puts " << progname << "into record mode.  Make sure that the "
        "the processes" << endl << "not being attacked during the recording"
        " session." << endl << endl <<
        
        "[-p] toggles the \"process\" step.  This is really only usefull"
        " in combination" << endl << "with the '-e' option." << endl << endl <<

        "[-f filename] specifies the name of the database file.  The default "
        "is" << endl << "\"depth-<depth>.db\"" << endl << endl <<
        
        "[-d #] sets the depth of the sequence tree.  The default is 8" <<
        endl << endl <<

        "[-i inputfile] specifies a file to use for input.  If this option is "
        << endl << "not supplied, " << progname << " will listen to the "
        "audit queue specified instead." << endl << endl <<

        "[-e progname] exports the database for the specified program"
        " in a \"fast\"" << endl << "format suitable for a in-kernel "
        "wrapper.  There is no default." << endl << 
        "The sequence length must be 8 for this option." << endl << endl <<
        
        "[-E progfile] specifies the file for -e to export.  The default " <<
        "is" << endl << "\"fast-<depth>.db\"" << endl << endl

        << "-q specifies the audit queue to use.  The default is 0." << endl
        ;
    exit(1);
}

int main(int argc, char *argv[])
{
    char buf[1024];
    char *prog_name = rindex(argv[0], '/');
    
    if (!prog_name)
        prog_name = argv[0];
    else
        ++prog_name;

    Name_To_PDI name_map;
    bool fRecord = false;
    bool fProcess = true;

    char *db_name = NULL;
    char *input_file = NULL;
    char *fast_file = NULL;
    char *fast_program = NULL;
    int depth = 8;
    int audit_number = 0;

    int ch;
    while ((ch = getopt(argc, argv, "hprf:d:i:e:E:q:")) != -1) {
        switch(ch) {
        case 'p':
            fProcess = !fProcess;
            break;
        case 'r':
            fRecord = true;
            break;
        case 'f':
            db_name = strdup(optarg);
            break;
        case 'd':
            depth = atoi(optarg);
            break;
        case 'i':
            input_file = strdup(optarg);
            break;
        case 'e':
            fast_program = strdup(optarg);
            break;
        case 'E':
            fast_file = strdup(optarg);
            break;
        case 'q':
            audit_number = atoi(optarg);
            break;
        default:
            usage(prog_name);
        }
    }
    
    if ((fast_file != NULL && fast_program == NULL)
            || (fast_program != NULL && depth != 8)
            || (!fProcess && fRecord)
            || (!fProcess && fast_program == NULL ))
        usage(prog_name);

    if (db_name == NULL) {
        snprintf(buf, 1024, "depth-%d.db", depth);
        db_name = strdup(buf);
    }

    if (fast_file == NULL) {
        snprintf(buf, 1024, "fast-%d.db", depth);
        fast_file = strdup(buf);
    }


    Source *source;

    DB db(depth);

    if (input_file != NULL && fProcess) {
        source = new FileSource(input_file);
    } else if (fProcess) {
        source = new EventSource(audit_number);
    } else {
        source = NULL;
    }

    if (fRecord) {
        if (fProcess)
            process(&db, source);
        db.export_to_stream(db_name);
        sprintf(buf, ".depth-%d.db",depth);
        unlink(buf);
    } else {
        db.import(db_name);
        if (fProcess)
            process(&db, source);
    }
    
    if (fast_file != NULL && fast_program != NULL) {
        db.export_fast(fast_file, fast_program);
    }
    
    free(db_name);
    return 0;
}
