/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: Message.h,v 1.7 2001/02/23 19:02:05 stevek Exp $
 * 
 * File: Message.h
 * 
 * Author: Douglas Kilpatrick
 * 
 * Purpose: Message classes to encapsulate talking to the wrappers
 * event stream.
 * 
 * Classes:
 * 
 *      Source: data source
 *      EventSource: Source subclass that talks to the event manager.
 *      
 *      Message: root message class.
 *      execMessage: subclass for process creation
 *      exitMessage: subclass for process termination
 *      syscallMessage: subclass for syscall execution
 *      quitMessage: subclass for wrapper shutdown
 *
 */

#ifndef MESSAGE_H
#define MESSAGE_H

#ifndef DATA_H
#include "Data.h"
#endif

#ifdef WR_BROKEN_STL
#include "template/list"
#include "template/map"
#else
#include <list>
#include <map>
#endif

#ifndef __LIBEVENTFS_H
extern "C" {
#include "libeventfs.h"
}
#endif

#if 0
map<pid,pdi>
map<name,pdi>
map<pdi,List<sequence> >
window_size;
#endif

typedef map<int,int,less<int>,alloc> pid_to_pdi_t;
typedef map<int, list<Sequence*> *,less<int>,alloc> pid_to_seq_t;

class Message {
    public:
        virtual int process(
                DB *,
                pid_to_pdi_t *,
                pid_to_seq_t *) = 0;
        virtual ~Message() { };
        Message() {};
};

class execMessage : public Message {
    private:
        int pid;
        char *name;
    public:
        virtual int process(
                DB *,
                pid_to_pdi_t *,
                pid_to_seq_t *);
        virtual ~execMessage() { free(name); };
        execMessage(int process_id, const char *process_name);
};

class exitMessage : public Message {
    private:
        int pid;
    public:
        virtual int process(
                DB *,
                pid_to_pdi_t *,
                pid_to_seq_t *);
        virtual ~exitMessage() { };
        exitMessage(int process_id) { pid = process_id; };
};

class syscallMessage : public Message {
    private:
        int pid;
        int syscall_num;
    public:
        virtual int process(
                DB *,
                pid_to_pdi_t *,
                pid_to_seq_t *);
        virtual ~syscallMessage() { };
        syscallMessage(int process_id, int syscall);
};

class quitMessage : public Message {
    private:
    public:
        virtual int process(
                DB *,
                pid_to_pdi_t *,
                pid_to_seq_t *);
        virtual ~quitMessage() { };
        quitMessage() { };
};


class Source {
    public:
        virtual Message *next_message() = 0;
        Source() { };
        virtual ~Source() { };
};

class EventSource : public Source {
    private:
        eventfs_event_stream_t      stream;
    public:
        virtual Message *next_message();
        EventSource(int num);
        virtual ~EventSource();
};

class FileSource: public Source {
    private:
        istream *input_stream;
    public:
        virtual Message *next_message();
        FileSource(char *);
        virtual ~FileSource();
};

#endif /* MESSAGE_H */
