/***************************************************************
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * testarp
 *
 * Author:  Tim Fraser
 *
 * This is a quick-and-dirty program for testing the arp wrapper.
 * It attempts to cause at least one example from each class of
 * event in the ARI abstract interface described in
 * sta/include/ari.ch.
 *
 * USAGE: 
 *
 * Run this program wrapped by arp and arc (in that order).  This
 * program will print messages to stdout describing the events it
 * creates.  The arp wrapper should produce an equivalent report of
 * these events.
 * 
 ***************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/param.h>

int main( void ) {

  char tempfile_s[ MAXPATHLEN ];
  char targetfile_s[ MAXPATHLEN ];
  FILE *p_file;
  int fd;

  /* Get a name for a temporary file, put it in tempfile_s */
  tmpnam( tempfile_s );
  tmpnam( targetfile_s );  /* a second name for rename, link */

  /* introduce ourselves... */
  printf( "test program pid %u file %s, target file %s\n", getpid(), 
	  tempfile_s, targetfile_s );


  /* --- Test all "fileop" file operations with no "target" path --- */

  /* test - truncate */
  printf( "WRITE(truncate)\n" );
  truncate( tempfile_s, 16 );

  /* test - unlink */
  printf( "DELETE(unlink)\n" );
  unlink( tempfile_s );

  /* test - mkdir */
  printf( "CREATE(mkdir)\n" );
  mkdir( tempfile_s, ( S_IWGRP | S_IWOTH ) );

  /* test - rmdir */
  printf( "DELETE(rmdir)\n" );
  rmdir( tempfile_s );

  /* test - creat */
  printf( "CREATE(creat)\n" );
  printf( "WRITE(creat)\n" );
  creat( tempfile_s, S_IRWXU );

  /* test - chown */
  printf( "MODIFYOWNER(chown)\n" );
  chown( tempfile_s, getuid(), getgid() );

  /* test - chmod */
  printf( "MODIFYPERM(chmod)\n" );
  chmod( tempfile_s, S_IRWXU );

  /* test - open, no create, read-only */
  printf( "READ(open)\n" );
  p_file = fopen( tempfile_s, "r" );
  fclose( p_file );

  /* test - open, no create, read-write */
  printf( "READ(open)\n" );
  printf( "WRITE(open)\n" );
  p_file = fopen( tempfile_s, "r+" );
  fclose( p_file );

  /* test - another unlink */
  printf( "DELETE(unlink)\n" );
  unlink( tempfile_s );

  /* test - open, create, write-only */
  printf( "CREATE(open)\n" );
  printf( "WRITE(open)\n" );
  p_file = fopen( tempfile_s, "a" );
  fclose( p_file );

  /* test - yet another unlink */
  printf( "DELETE(unlink)\n" );
  unlink( tempfile_s );

  /* test - open, create, read-write */
  printf( "CREATE(open)\n" );
  printf( "READ(open)\n" );
  printf( "WRITE(open)\n" );
  p_file = fopen( tempfile_s, "a+" );
  fclose( p_file );

  /* test - open, no create, read-write */
  printf( "READ(open)\n" );
  printf( "WRITE(open)\n" );
  fd = open( tempfile_s, ( O_RDWR ) );
  

  /* --- Test all "fdop" file operations with no "target" path --- */

  /* test - ftruncate */
  printf( "WRITE(ftruncate)\n" );
  ftruncate( fd, 16 );

  /* test - fchown */
  printf( "MODIFYOWNER(fchown)\n" );
  fchown( fd, getuid(), getgid() );
  
  /* test - fchmod */
  printf( "MODIFYPERM(fchmod)\n" );
  fchmod( fd, S_IRWXU );
  close( fd );

  /* -------- Test all file operations with "target" paths -------- */

  /* test - rename */
  printf( "RENAME(rename)\n" );
  rename( tempfile_s, targetfile_s );

  /* test - symlink */
  printf( "LINK(symlink)\n" );
  symlink( targetfile_s, tempfile_s );

  /* pause for tester */
  printf( "hit ctrl-D to continue...\n" );
  scanf( "%s\n", &tempfile_s );

  /* test - exit */
  printf( "EXIT\n" );
  return( 0 );

} /* main() */


