
/*
 * $Id: basiclist.h,v 1.8 2001/03/27 21:11:41 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Basic Linked Lists
 * 
 */

#ifndef _GSWTK_KERNEL_BASICLIST_H
#define _GSWTK_KERNEL_BASICLIST_H 1

#ifndef GSWTK_INCL_ASSEMBLY

GSWTK_COMPONENT(basiclist);

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct gswk_basiclist gswk_basiclist_t;
typedef struct gswk_bl_node   gswk_bl_node_t;

struct gswk_bl_node
{
   gswk_bl_node_t *node_next;
};

struct gswk_basiclist
{
   gswk_bl_node_t  *bl_front;
   gswk_bl_node_t **bl_back;
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSW_BL_NODE_INIT() { (gswk_bl_node_t *)NULL }

#define GSW_BASICLIST_STATIC(label) \
   static gswk_basiclist_t label; \
   const gswk_basiclist_t *_gswtk_bl_ ## label = &(label);

#define GSW_BASICLIST(label) \
   gswk_basiclist_t label; \
   const gswk_basiclist_t *_gswtk_bl_ ## label = &(label);


/***************************************************************************\
 * Private macros used by component set                                    *
\***************************************************************************/

#define GSWTK_BASICLIST_INITIALIZE(n,retc) \
   do { \
      extern const gswk_basiclist_t *_gswtk_bl_ ## n; \
      (retc) = gswk_basiclist_init( (gswk_basiclist_t *)_gswtk_bl_ ## n ); \
   } while(0)

#define GSWTK_BASICLIST_TEARDOWN(n,retc) \
   do { \
      extern const gswk_basiclist_t *_gswtk_bl_ ## n; \
      (retc) = gswk_basiclist_destroy( (gswk_basiclist_t *)_gswtk_bl_ ## n ); \
   } while(0)



/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int            GSWKERNLINKAGE  gswk_basiclist_init   ( gswk_basiclist_t * );
int            GSWKERNLINKAGE  gswk_basiclist_destroy( gswk_basiclist_t * );
void           GSWKERNLINKAGE  gswk_basiclist_enqueue( gswk_basiclist_t *,
                                                       gswk_bl_node_t * );
gswk_bl_node_t GSWKERNLINKAGE *gswk_basiclist_dequeue( gswk_basiclist_t * );
gswk_bl_node_t GSWKERNLINKAGE *gswk_basiclist_peek   ( gswk_basiclist_t * );
gswk_bl_node_t GSWKERNLINKAGE *gswk_basiclist_next   ( gswk_bl_node_t * );

#else /* GSWTK_INCL_ASSEMBLY */
#include GSWTK_KERNEL_OS_HEADER(basiclist_asm.h)
#endif /* GSWTK_INCL_ASSEMBLY */

#endif /* _GSWTK_KERNEL_BASICLIST_H */

