/*
 * $Id: component.h,v 1.14 2001/05/14 20:41:39 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Components
 * 
 */

#ifndef _GSWTK_KERNEL_COMPONENT_H
#define _GSWTK_KERNEL_COMPONENT_H 1

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct gswk_component             gswk_component_t;

typedef int (*gswk_start_func_t)   (void);
typedef int (*gswk_stop_func_t)    (void);
typedef int (*gswk_init_func_t)    (const void *);
typedef int (*gswk_teardown_func_t)(const void *);


struct gswk_component
{
   unsigned int         comp_entid;
   union
   {
      struct { const void           *d1;   const void *d2;   } dummy;
      struct { const pd_char_t      *name;                   } tag;
      struct { gswk_start_func_t     func;                   } start;
      struct { gswk_stop_func_t      func;                   } stop;
      struct { gswk_component_t     *ptr;                    } dependency;
      struct { gswk_init_func_t      func; const void *data; } initializer;
      struct { gswk_teardown_func_t  func; const void *data; } teardown;
   }
   comp_arg;
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSWTK_COMPONENT(name)              extern gswk_component_t \
                                              _gswtk_comp_ ## name[]
#define GSWTK_COMPONENT_BEGIN(name)        gswk_component_t \
                                              _gswtk_comp_ ## name[] = {
#define GSWTK_COMPONENT_END                {GSW_COMP_END, {{NULL}}} };
#define GSWTK_COMPONENT_DEPENDS_ON(name)   {GSW_COMP_DEP, \
                                              {{_gswtk_comp_ ## name}}},
#define GSWTK_COMPONENT_START(start)       {GSW_COMP_START, {{start}}},
#define GSWTK_COMPONENT_STOP(stop)         {GSW_COMP_STOP, {{stop}}},
#define GSWTK_COMPONENT_TAG(tag)           {GSW_COMP_TAG, {{PD_MAKESTR(tag)}}},
#define GSWTK_COMPONENT_INIT(fnc,data)     {GSW_COMP_INIT, {{fnc,data}}},
#define GSWTK_COMPONENT_TEARDOWN(fnc,data) {GSW_COMP_TEARDOWN, {{fnc,data}}},

#define GSWTK_REGISTER_COMPONENT(group,name,retc) \
   do { extern struct gswk_component _gswtk_comp_ ## name[] ; \
        (retc) = gswk_component_register( (group), _gswtk_comp_ ## name ); \
   } while(0)
#define GSWTK_UNREGISTER_COMPONENT(name,retc) \
   do { extern struct gswk_component _gswtk_comp_ ## name[] ; \
        (retc) = gswk_component_unregister( _gswtk_comp_ ## name ); \
   } while(0)
#define GSWTK_TEARDOWN_COMPONENT(group,retc) \
   do { (retc) = gswk_component_teardown( (group) ); } while(0)


/* Markers for component set entries */
#define GSW_COMP_END      0              /* End component entry     */
#define GSW_COMP_DEP      1              /* Depedency entry         */
#define GSW_COMP_START    2              /* Startup function entry  */
#define GSW_COMP_STOP     3              /* Shutdown function entry */
#define GSW_COMP_TAG      4              /* Component tag entry     */
#define GSW_COMP_INIT     5              /* Initialization entry    */
#define GSW_COMP_TEARDOWN 6              /* Teardown entry          */
#define GSW_COMP_ENTIDS   7              /* Number of entries       */

/* Flags for gswk_component_find */
#define GSW_COMP_FIND_REF    0x0001
#define GSW_COMP_FIND_PARENT 0x0002
#define GSW_COMP_FIND_INSERT 0x0004


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int GSWKERNLINKAGE gswk_component_initialize        ( void );
int GSWKERNLINKAGE gswk_component_initialize_by_name( const pd_char_t * );
int GSWKERNLINKAGE gswk_component_register          ( const pd_char_t *,
                                                      gswk_component_t * );
int GSWKERNLINKAGE gswk_component_teardown          ( const pd_char_t * );
int GSWKERNLINKAGE gswk_component_unregiser         ( gswk_component_t * );

#endif /* _GSWTK_KERNEL_COMPONENT_H */

