/*
 * $Id: hash.h,v 1.20 2001/03/21 23:08:41 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Hash Table
 * 
 */

#ifndef _GSWTK_KERNEL_HASH_H_SEEN
#define _GSWTK_KERNEL_HASH_H_SEEN 1

#define GSWTK_INCL_BASICLIST
#define GSWTK_INCL_COND
#define GSWTK_INCL_MUTEX
#define GSWTK_INCL_THREAD
#include <gswtk/gswtk.h>

#endif /* _GSWTK_KERNEL_HASH_H_SEEN */

#ifndef _GSWTK_KERNEL_HASH_H
#define _GSWTK_KERNEL_HASH_H 1

GSWTK_COMPONENT(hash);

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef u_int32_t               gswk_hashcode_t;
typedef struct gswk_hash        gswk_hash_t;
typedef struct gswk_hash_bucket gswk_hash_bucket_t;
typedef struct gswk_hash_waiter gswk_hash_waiter_t;

typedef gswk_hashcode_t (*gswk_hash_func_t)       ( gswk_hash_t *, void * );
typedef int             (*gswk_hash_cmp_func_t)   ( gswk_hash_t *, void *,
                                                    void *);
typedef void            (*gswk_hash_remove_func_t)( gswk_hash_t *, void * );

struct gswk_hash_bucket
{
   void                     *bucket_data;
   gswk_hashcode_t           bucket_key;
   u_int16_t                 bucket_flags;
   gswk_thread_id_t          bucket_holder;
   u_int16_t                 bucket_waiters;
   gswk_basiclist_t          bucket_waiter_list;
   gswk_hash_bucket_t       *bucket_next;
   gswk_hash_bucket_t       *bucket_right;
   gswk_hash_bucket_t       *bucket_left;
};

struct gswk_hash_waiter
{
   gswk_bl_node_t            waiter_listnode;
   pd_bool_t                 waiter_wakeup;
   gswk_hash_bucket_t       *waiter_bucket;
   gswk_cond_os_t            waiter_cv;
};

struct gswk_hash
{
   gswk_mutex_os_t           hash_mtx;
   const pd_char_t          *hash_name;
   gswk_hash_func_t          hash_func;
   gswk_hash_cmp_func_t      hash_cmpfunc;
   gswk_hash_remove_func_t   hash_removefunc;
   size_t                    hash_buckets;
   int                       hash_lockstatus;
   int                       hash_wait_op;
   gswk_cond_os_t            hash_wait_op_cv;
   int                       hash_wait_get;
   gswk_cond_os_t            hash_wait_get_cv;
   gswk_hash_bucket_t      **hash_table;
   gswk_hash_bucket_t       *hash_start;
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSW_HASH_LOCK_UNUSED 0

#define GSW_HASH_BUCKET_LOCKED 1

#define GSW_HASH_DEFAULT_BUCKETS 27

#define GSW_HASH_INIT(ht,n)         gswk_hash_init((ht),PD_MAKESTR(n), \
                                                   GSW_HASH_DEFAULT_BUCKETS)
#define GSW_HASH_INIT_SIZE(ht,n,sz) gswk_hash_init((ht),PD_MAKESTR(n),(sz))


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

pd_bool_t       GSWKERNLINKAGE   gswk_hash_add             ( gswk_hash_t *,
                                                             void * );
int             GSWKERNLINKAGE   gswk_hash_clean           ( gswk_hash_t * );
gswk_hashcode_t GSWKERNLINKAGE   gswk_hash_compute         ( gswk_hash_t *,
                                                             void * );
void            GSWKERNLINKAGE **gswk_hash_find            ( gswk_hash_t *,
                                                             void * );
void            GSWKERNLINKAGE **gswk_hash_get             ( gswk_hash_t *,
                                                             void * );
void            GSWKERNLINKAGE   gswk_hash_init            ( gswk_hash_t *,
                                                             const pd_char_t *,
                                                             size_t );
void            GSWKERNLINKAGE **gswk_hash_interate        ( gswk_hash_t * );
void            GSWKERNLINKAGE   gswk_hash_lock            ( gswk_hash_t * );
void            GSWKERNLINKAGE **gswk_hash_next            ( gswk_hash_t *,
                                                             void ** );
int             GSWKERNLINKAGE   gswk_hash_release         ( gswk_hash_t *,
                                                             void ** );
int             GSWKERNLINKAGE   gswk_hash_remove          ( gswk_hash_t *,
                                                             void ** );
void            GSWKERNLINKAGE   gswk_hash_set_compare_func( gswk_hash_t *,
                                                             gswk_hash_cmp_func_t );
void            GSWKERNLINKAGE   gswk_hash_set_hashing_func( gswk_hash_t *,
                                                             gswk_hash_func_t );
void            GSWKERNLINKAGE   gswk_hash_set_remove_func ( gswk_hash_t *,
                                                             gswk_hash_remove_func_t );
void            GSWKERNLINKAGE   gswk_hash_unlock          ( gswk_hash_t * );

#endif /* _GSWTK_KERNEL_HASH_H */

