/*
 * $Id: memory.h,v 1.13 2001/03/27 21:12:15 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Memory Management
 * 
 */

#ifndef _GSWTK_KERNEL_MEMORY_H_SEEN
#define _GSWTK_KERNEL_MEMORY_H_SEEN 1

#define GSWTK_INCL_BASICLIST
#define GSWTK_INCL_HASH
#define GSWTK_INCL_MUTEX
#include <gswtk/gswtk.h>

#endif /* _GSWTK_KERNEL_MEMORY_H_SEEN */

#ifndef _GSWTK_KERNEL_MEMORY_H
#define _GSWTK_KERNEL_MEMORY_H 1

GSWTK_COMPONENT(memory);


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct gswk_mem_zone gswk_mem_zone_t;
#ifdef GSWTK_OPTION_MEMORY_SENTINELS
typedef struct gswk_mem_head gswk_mem_head_t;
typedef struct gswk_mem_tail gswk_mem_tail_t;
#endif /* GSWTK_OPTION_MEMORY_SENTINELS */

struct gswk_mem_zone
{
   gswk_bl_node_t   memzone_node;
   gswk_mutex_os_t  memzone_mtx;
   const pd_char_t *memzone_group;
   const pd_char_t *memzone_tag;
   const pd_bool_t  memzone_initialized;
   unsigned long    memzone_allocs;
   unsigned long    memzone_frees;
#ifdef GSWTK_OPTION_MEMORY_SENTINELS
   unsigned long    memzone_active;
#endif /* GSWTK_OPTION_MEMORY_SENTINELS */
#ifdef GSWTK_OPTION_MEMORY_TRACKING
   /* Add tracking tree here */
#endif /* GSWTK_OPTION_MEMORY_TRACKING */
};

#ifdef GSWTK_OPTION_MEMORY_SENTINELS
struct gswk_mem_head
{
   gswk_magic_t     magic1;
   gswk_mem_zone_t *zone;
   size_t           size;
   gswk_magic_t     magic2;
};

struct gswk_mem_tail
{
   gswk_magic_t     magic3;
};
#endif /* GSWTK_OPTION_MEMORY_SENTINELS */


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSWTK_MEM_ZONE_TOP(zone,tag) \
   gswk_mem_zone_t _gswtk_mz_ ## zone = { \
      GSW_BL_NODE_INIT(), GSW_MUTEX_OS_INIT(), NULL, PD_MAKESTR(tag), FALSE, 0, 0
#define GSWTK_MEM_ZONE_BOTTOM(zone,tag) \
   }

#ifdef GSWTK_OPTION_MEMORY_SENTINELS
#  define GSWTK_MEM_ZONE_DEFINE(zone, tag) \
      GSWTK_MEM_ZONE_TOP(zone,tag) \
         , 0 \
      GSWTK_MEM_ZONE_BOTTOM(zone,tag)
#elif defined(GSWTK_OPTION_MEMORY_TRACKING)
#  define GSWTK_MEM_ZONE_DEFINE(zone, tag) \
      GSWTK_MEM_ZONE_TOP(zone,tag) \
         /* Add memory tracking init here */
      GSWTK_MEM_ZONE_BOTTOM(zone,tag)
#else
#  define GSWTK_MEM_ZONE_DEFINE(zone, tag) \
      GSWTK_MEM_ZONE_TOP(zone,tag) \
      GSWTK_MEM_ZONE_BOTTOM(zone,tag)
#endif
#define GSWTK_MEM_ZONE(zone) extern gswk_mem_zone_t _gswtk_mz_ ## zone

#define GSWTK_MEM_ZONE_INITIALIZE(group, zone, retc) \
   do { extern struct gswk_mem_zone _gswtk_mz_ ## zone; \
        (retc) = gswk_mem_zone_initialize( (group), &(_gswtk_mz_ ## zone)); \
   } while(0)
#define GSWTK_MEM_ZONE_TEARDOWN(group, retc) \
   do { (retc) = gswk_mem_zone_teardown( (group) ); } while(0)


/***************************************************************************\
 * Public functions implemented as macros                                  *
\***************************************************************************/

#define gswk_malloc(s,zone) gswk_mem_alloc(&_gswtk_mz_ ## zone, (s))
#define gswk_free(s,zone)   gswk_mem_free(&_gswtk_mz_ ## zone, (s))


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

GSWTK_MEM_ZONE(kernel);


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void GSWKERNLINKAGE *gswk_mem_alloc( gswk_mem_zone_t *, size_t );
void GSWKERNLINKAGE  gswk_mem_free ( gswk_mem_zone_t *, void * );

int  GSWKERNLINKAGE  gswk_mem_zone_initialize( const pd_char_t *,
                                               gswk_mem_zone_t * );
int  GSWKERNLINKAGE  gswk_mem_zone_teardown  ( const pd_char_t * );
void GSWKERNLINKAGE  gswk_mem_zone_lock      ( gswk_mem_zone_t * );
void GSWKERNLINKAGE  gswk_mem_zone_unlock    ( gswk_mem_zone_t * );


/***************************************************************************\
 * Include OS-specific header                                              *
\***************************************************************************/

#include <gswtk/kernel/os/memory.h>


#endif /* _GSWTK_KERNEL_MEMORY_H */

