/*
 * $Id: cond.h,v 1.6 2001/03/20 20:46:08 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: OS-specific Condition Variables
 * 
 */

#ifndef _GSWTK_KERNEL_OS_COND_H
#define _GSWTK_KERNEL_OS_COND_H 1


struct gswk_cond_os;
struct gswk_mutex_os;

/***************************************************************************\
 * Public strutures and types                                              *
\***************************************************************************/

struct _gswk_cond_os_meta
{
   struct gswk_cond_os *cond;
   const pd_char_t     *name;
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSW_OS_COND_STATIC(label, name) \
   static gswk_cond_os_t label; \
   const struct _gswk_cond_os_meta _gswtk_cond_ ## label = { &(label), name }

#define GSW_OS_COND(label, name) \
   gswk_cond_os_t label; \
   const struct _gswk_cond_os_meta _gswtk_cond_ ## label = { &(label), name }


/***************************************************************************\
 * Private macros used by component set                                    *
\***************************************************************************/

#define GSWTK_OS_COND_INITIALIZE(n,retc) \
   do { \
      extern const struct _gswk_cond_os_meta _gswtk_cond_ ## n; \
      (retc) = gswk_cond_os_init( _gswtk_cond_ ## n ## .cond, \
                                  _gswtk_cond_ ## n ## .name ); \
   } while(0)

#define GSWTK_OS_COND_TEARDOWN(n,retc) \
   do { \
      extern const struct _gswk_cond_os_meta _gswtk_cond_ ## n; \
      (retc) = gswk_cond_os_destroy( _gswtk_cond_ ## n ## .cond ); \
   } while(0)



/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int  GSWKERNLINKAGE gswk_cond_os_init           ( struct gswk_cond_os *,
                                                  const pd_char_t * );
int  GSWKERNLINKAGE gswk_cond_os_destroy        ( struct gswk_cond_os * );
int  GSWKERNLINKAGE gswk_cond_os_wait           ( struct gswk_cond_os *,
                                                  struct gswk_mutex_os * );
void GSWKERNLINKAGE gswk_cond_os_broadcast      ( struct gswk_cond_os * );
void GSWKERNLINKAGE gswk_cond_os_signal         ( struct gswk_cond_os * );
#ifdef GSWTK_OPTION_DEBUG_COND
void GSWKERNLINKAGE gswk_cond_os_wait_debug     ( struct gswk_cond_os *,
                                                  const pd_char_t *, int );
void GSWKERNLINKAGE gswk_cond_os_broadcast_debug( struct gswk_cond_os *,
                                                  const pd_char_t *, int );
void GSWKERNLINKAGE gswk_cond_os_signal_debug   ( struct gswk_cond_os *,
                                                  const pd_char_t *, int );
#endif /* GSWTK_OPTION_DEBUG_COND */


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#ifdef GSWTK_OPTION_DEBUG_COND
#define gswk_cond_os_wait(c,m) \
           gswk_cond_os_wait_debug((c),(m),__FILE__,__LINE__)
#define gswk_cond_os_broadcast(c) \
           gswk_cond_os_broadcast_debug((c),__FILE__,__LINE__)
#define gswk_cond_os_signal(c)    \
           gswk_cond_os_signal_debug((c),__FILE__,__LINE__)
#endif /* GSWTK_OPTION_DEBUG_COND */


#include GSWTK_KERNEL_OS_HEADER(cond.h)

#endif /* _GSWTK_KERNEL_OS_COND_H */

