/*
 * $Id: mutex.h,v 1.8 2001/03/20 20:46:08 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: OS-specific Mutexes
 * 
 */

#ifndef _GSWTK_KERNEL_OS_MUTEX_H
#define _GSWTK_KERNEL_OS_MUTEX_H 1

#ifndef GSWTK_INCL_ASSEMBLY

struct gswk_mutex_os;

/***************************************************************************\
 * Public strutures and types                                              *
\***************************************************************************/

struct _gswk_mutex_os_meta
{
   struct gswk_mutex_os *mtx;
   const pd_char_t     *name;
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSW_OS_MUTEX_STATIC(label, name) \
   static gswk_mutex_os_t label; \
   const struct _gswk_mutex_os_meta _gswtk_mtx_ ## label = { &(label), name }

#define GSW_OS_MUTEX(label, name) \
   gswk_mutex_os_t label; \
   const struct _gswk_mutex_os_meta _gswtk_mtx_ ## label = { &(label), name }


/***************************************************************************\
 * Private macros used by component set                                    *
\***************************************************************************/

#define GSWTK_OS_MUTEX_INITIALIZE(n,retc) \
   do { \
      extern const struct _gswk_mutex_os_meta _gswtk_mtx_ ## n; \
      (retc) = gswk_mutex_os_init( _gswtk_mtx_ ## n ## .mtx, \
                                  _gswtk_mtx_ ## n ## .name ); \
   } while(0)

#define GSWTK_OS_MUTEX_TEARDOWN(n,retc) \
   do { \
      extern const struct _gswk_mutex_os_meta _gswtk_mtx_ ## n; \
      (retc) = gswk_mutex_os_destroy( _gswtk_mtx_ ## n ## .mtx ); \
   } while(0)


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int  GSWKERNLINKAGE gswk_mutex_os_init        ( struct gswk_mutex_os *,
                                                const pd_char_t * );
int  GSWKERNLINKAGE gswk_mutex_os_destroy     ( struct gswk_mutex_os * );
void GSWKERNLINKAGE gswk_mutex_os_lock        ( struct gswk_mutex_os * );
void GSWKERNLINKAGE gswk_mutex_os_unlock      ( struct gswk_mutex_os * );
#ifdef GSWTK_OPTION_DEBUG_MUTEX
void GSWKERNLINKAGE gswk_mutex_os_lock_debug  ( struct gswk_mutex_os *,
                                                const pd_char_t *, int );
void GSWKERNLINKAGE gswk_mutex_os_unlock_debug( struct gswk_mutex_os *,
                                                const pd_char_t *, int );
#endif /* GSWTK_OPTION_DEBUG_MUTEX */


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#ifdef GSWTK_OPTION_DEBUG_MUTEX
#define gswk_mutex_os_lock(m)   gswk_mutex_os_lock_debug((m),__FILE__,__LINE__)
#define gswk_mutex_os_unlock(m) gswk_mutex_os_unlock_debug((m),__FILE__,__LINE__)
#endif /* GSWTK_OPTION_DEBUG_MUTEX */


#include GSWTK_KERNEL_OS_HEADER(mutex.h)

#else /* GSWTK_INCL_ASSEMBLY */
#include GSWTK_KERNEL_OS_HEADER(mutex_asm.h)
#endif /* GSWTK_INCL_ASSEMBLY */

#endif /* _GSWTK_KERNEL_OS_MUTEX_H */

