/*
 * $Id: ref.h,v 1.8 2001/03/20 20:46:08 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Reference Counting
 * 
 */

#ifndef _GSWTK_KERNEL_REF_H
#define _GSWTK_KERNEL_REF_H 1

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

#ifdef DEBUG_REFS
typedef struct gswk_ref gswk_ref_t;

struct gswk_ref
{
   u_int32_t ref_count;
};
#else
typedef u_int32_t gswk_ref_t;
#endif


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void      gswk_ref_init   ( gswk_ref_t );
int       gswk_ref_count  ( gswk_ref_t );
pd_bool_t gswk_ref_is_used( gswk_ref_t );
void      gswk_ref_inc    ( gswk_ref_t );
void      gswk_ref_dec    ( gswk_ref_t );


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/
#ifdef DEBUG_REFS
#define gswk_ref_init(r) \
   do { \
      (r).ref_count = 1; \
   } while(0)
#define gswk_ref_inc(r) (++(r).ref_count)
#define gswk_ref_dec(r) (--(r).ref_count)
#define gswk_ref_count(r) ((r).ref_count)
#else
#define gswk_ref_init(r) \
   do { \
      r = 1; \
   } while(0)
#define gswk_ref_inc(r) (++(r))
#define gswk_ref_dec(r) (--(r))
#define gswk_ref_count(r) (r)
#endif
#define gswk_ref_is_used(r) ((pd_bool_t)(gswk_ref_count(r)>1)?TRUE:FALSE)

#endif /* _GSWTK_KERNEL_REF_H */

