
/*
 * $Id: string.h,v 1.12 2001/03/22 23:20:00 stevek Exp $
 *
 *   Section: GSWTK Kernel
 * Component: Strings
 * 
 */

#ifndef _GSWTK_KERNEL_STRING_H_SEEN
#define _GSWTK_KERNEL_STRING_H_SEEN 1

#define GSWTK_INCL_REF
#define GSWTK_INCL_STRING
#include <gswtk/gswtk.h>

#endif /* _GSWTK_KERNEL_STRING_H_SEEN */

#ifndef _GSWTK_KERNEL_STRING_H
#define _GSWTK_KERNEL_STRING_H 1


GSWTK_COMPONENT(string);

/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define GSW_STR_INTERNED 0x0001
#define GSW_STR_NULL     0x0002


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct gswk_string gswk_string_t;

struct gswk_string
{
   gswk_hashcode_t  str_hash;
   pd_char_t       *str_chars;
   u_int32_t        str_length;
   u_int32_t        str_size;
   u_int16_t        str_flags;
   gswk_ref_t       str_ref;
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int             GSWKERNLINKAGE  gswk_string_init           ( gswk_string_t * );
gswk_string_t   GSWKERNLINKAGE *gswk_string_create         ( const pd_char_t * );
gswk_string_t                  *gswk_string_create_interned( const pd_char_t * );
int             GSWKERNLINKAGE  gswk_string_destroy        ( gswk_string_t * );
gswk_hashcode_t GSWKERNLINKAGE  gswk_string_hash           ( gswk_hash_t *,
                                                             gswk_string_t * );
gswk_hashcode_t GSWKERNLINKAGE  gswk_string_hash_chars     ( gswk_hash_t *,
                                                             const pd_char_t * );
int             GSWKERNLINKAGE  gswk_string_hash_compare   ( gswk_hash_t *,
                                                             gswk_string_t *,
                                                             gswk_string_t * );
gswk_string_t   GSWKERNLINKAGE *gswk_string_intern         ( gswk_string_t * );
pd_bool_t                       gswk_string_is_interned    ( gswk_string_t * );
pd_bool_t                       gswk_string_is_null        ( gswk_string_t * );
gswk_string_t   GSWKERNLINKAGE *gswk_string_set            ( gswk_string_t *,
                                                             const pd_char_t * );


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#define gswk_string_create_interned(s) gswk_string_intern(gswk_string_create(s))
#define gswk_string_is_null(s)         (((s)->str_flags & GSW_STR_NULL) \
                                          ? TRUE : FALSE)
#define gswk_string_is_interned(s)     (((s)->str_flags & GSW_STR_INTERNED) \
                                          ? TRUE : FALSE)

#endif /* _GSWTK_KERNEL_STRING_H */

