/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_defs.h,v 1.11 2001/05/09 21:16:14 stevek Exp $
 * 
 * Authors: Douglas Kilpatrick, Steve Kiernan
 */

#ifndef __PD_DEFS_H
#define __PD_DEFS_H

#ifdef __FreeBSD__

#ifndef PD_ASSEMBLY
#ifndef __PD_TYPES_H
#include <pdpi/freebsd/pd_types.h>
#endif /* __PD_TYPES_H */
#endif /* !PD_ASSEMBLY */

#if __FreeBSD_version < 220000
#  error Only 2.2.x and above are supported
#elif __FreeBSD_version < 300000
#  define PD_FREEBSD 2
#elif __FreeBSD_version < 400000
#  define PD_FREEBSD 3
#elif __FreeBSD_version < 500000
#  define PD_FREEBSD 4
#else
#  define PD_FREEBSD 5
#endif
#define PD_UNIX
/* FreeBSD specific issues. */
#define PD_BZERO_IS_FN_PTR  1

#define FILESEP '/'

#define PICFLAG -fpic


/* If in kernel */
#ifdef PD_IN_KERNEL

#define COMPAT_43	1
#define NFS		1
#define LFS		1
#if __FreeBSD_version < 310000
#define ACTUALLY_LKM_NOT_KERNEL 1
#else
#define KLD_MODULE 1
#endif /* PD_FREEBSD < 3 */
#define PD_NO_SNPRINTF_DECL     1

#ifndef PD_ASSEMBLY
#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/time.h>
#ifndef _SYS_QUEUE_H_
#include <pdpi/queue.h>
#endif
#include <sys/proc.h>
#include <sys/systm.h>
#include <machine/stdarg.h>

typedef struct proc proc_t;

#endif /* !PD_ASSEMBLY */

#else

#ifndef PD_ASSEMBLY
#include <string.h>

#ifdef USER_MODE_TEST
#ifndef _SYS_QUEUE_H_
#include <pdpi/queue.h>
#endif
#endif /* USER_MODE_TEST */

typedef struct proc proc_t;

#endif /* !PD_ASSEMBLY */

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif
#endif

#define PD_PAGESIZE PAGE_SIZE

#ifndef MAXNAMELEN
#define MAXNAMELEN 256
#endif

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#ifndef LINKAGE
#define LINKAGE
#endif

#define IMPORT
#define EXPORT
#endif

#if !defined(PD_SMP_SUPPORT)
#if defined(PD_USE_MUTEX)
#define PD_SMP_SUPPORT          1
#else
#define PD_SMP_SUPPORT          0
#endif
#endif

#ifndef PD_KTHREAD_SUPPORT
#define PD_KTHREAD_SUPPORT      0
#endif

#define pd_printf printf

#endif /* __PD_DEFS_H */

