/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_mutex.h,v 1.3 2001/02/23 19:02:06 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: 
 *
 */

#ifndef __PD_MUTEX_H
#define __PD_MUTEX_H  1


#ifndef __PD_TYPES_H
#include <pdpi/freebsd/pd_types.h>
#endif /* __PD_TYPES_H */


/************************************************************************\
 * Public types                                                         *
\************************************************************************/

typedef pid_t pd_mutex_t;


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#if !defined(USER_MODE_TEST) && defined(PD_USE_MUTEX)

#define PD_MUTEX_PROTECT(linkage,var) pd_mutex_t var ## _lock
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var) pd_mutex_t var ## _lock
#define PD_MUTEX_EXPORT(linkage,var) extern pd_mutex_t var ## _lock
#define PD_MUTEX_POINTER(linkage,var) pd_mutex_t * var ## _lock
#define PD_MUTEX_ADDR(linkage,var) (pd_mutex_t *)(&(var ## _lock))

#ifndef PD_SUPPRESS_FUNCTION_DECL
#define PD_MUTEX_INIT(linkage,var) var ## _lock = 0
#define PD_MUTEX_DESTROY(linkage,var)
#ifdef DEBUG_REFS
#define PD_MUTEX_ENTER_POINTER(linkage,var) \
   do { \
      if( var == curproc->p_pid ) \
         panic( "PD_MUTEX_ENTER(" ## #var ## ") recursize enter at %s:%d (owner thread 0x%x)", \
                __FILE__, __LINE__, var ## _lock ); \
      else { \
         while( *(var) { \
            tsleep( (caddr_t) (var), curproc->p_priority, "mtxent", 0 ); \
         } \
         *(var) = curproc->p_pid; \
      } \
   } while(0)
#else /* DEBUG_REFS */
#define PD_MUTEX_ENTER_POINTER(linkage,var) \
   do { \
      while( *(var) ) { \
         tsleep( (caddr_t) (var), curproc->p_priority, "mtxent", 0 ); \
      } \
      *(var) = curproc->p_pid; \
   } while(0)
#endif /* DEBUG_REFS */
#define PD_MUTEX_ENTER(linkage,var) PD_MUTEX_ENTER_POINTER( linkage,&(var ## _lock) )
#define PD_MUTEX_ENTER_POINTER_DEBUG(linkage,var) \
   do { \
      printf( "PD_MUTEX_ENTER(" ## #var ## "=0x%x)/pid=%d @ %s:%d\n", \
              (caddr_t) (var), (int) curproc->p_pid, __FILE__, __LINE__ ); \
      while( *(var) ) { \
         tsleep( (caddr_t) (var), curproc->p_priority, "mtxent", 0 ); \
      } \
      *(var) = curproc->p_pid; \
   } while(0)
#define PD_MUTEX_ENTER_DEBUG(linkage,var) \
   PD_MUTEX_ENTER_POINTER_DEBUG( linkage,&(var ## _lock) )
#define PD_MUTEX_EXIT_POINTER(linkage,var) \
   do { \
      if( *(var) != curproc->p_pid ) \
         panic( "PD_MUTEX_EXIT(" ## #var ## "=0x%x) not owned by process", \
                (u_int) (var) ); \
      *(var) = 0; \
      wakeup( (caddr_t) (var) ); \
   } while(0)
#define PD_MUTEX_EXIT(linkage,var) PD_MUTEX_EXIT_POINTER( linkage,&(var ## _lock) )
#define PD_MUTEX_EXIT_POINTER_DEBUG(linkage,var) \
   do { \
      printf( "PD_MUTEX_EXIT(" ## #var ## "=0x%x)/pid=%d @ %s:%d\n", \
              (caddr_t) (var), (int) curproc->p_pid, __FILE__, __LINE__ ); \
      if( var ## _lock != curproc->p_pid ) \
         panic( "PD_MUTEX_EXIT(" ## #var ## "=0x%x) not owned by process", \
                (u_int) (var) ); \
      var ## _lock = 0; \
      wakeup( (caddr_t) (var) ); \
   } while(0)
#define PD_MUTEX_EXIT_DEBUG(linkage,var) \
   PD_MUTEX_EXIT_POINTER_DEBUG( linkage,&(var ## _lock) )

#define PD_MUTEX_INIT_NOMNG(linkage,var,name) (var) = 0
#define PD_MUTEX_DESTROY_NOMNG(linkage,var)
#define PD_MUTEX_ENTER_NOMNG(linkage,var) \
   do { \
      while( (var) ) { \
         tsleep( (caddr_t) &(var), curproc->p_priority, "mtxent", 0 ); \
      } \
      (var) = curproc->p_pid; \
   } while(0)
#define PD_MUTEX_EXIT_NOMNG(linkage,var) \
   do { \
      if( (var) != curproc->p_pid ) \
         panic( "PD_MUTEX_EXIT(" ## #var ## "=0x%x) not owned by process", \
                (u_int)&(var) ); \
      (var) = 0; \
      wakeup( (caddr_t) &(var) ); \
   } while(0)

#endif /* PD_SUPPRESS_FUNCTION_DECL */

#else /* !USER_MODE_TEST || PD_USE_MUTEX */

#define PD_MUTEX_PROTECT(linkage,var)
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var)
#define PD_MUTEX_EXPORT(linkage,var)
#define PD_MUTEX_POINTER(linkage,var)
#define PD_MUTEX_ADDR(linkage,var) NULL
#define PD_MUTEX_INIT(linkage,var) do{} while(0)
#define PD_MUTEX_DESTROY(linkage,var) do{} while(0)
#define PD_MUTEX_ENTER(linkage,var) do{} while(0)
#define PD_MUTEX_ENTER_POINTER(linkage,var) do {} while(0)
#define PD_MUTEX_ENTER_DEBUG(linkage,var) do{} while(0)
#define PD_MUTEX_ENTER_POINTER_DEBUG(linkage,var) do{} while(0)
#define PD_MUTEX_EXIT(linkage,var) do{} while(0)
#define PD_MUTEX_EXIT_POINTER(linkage,var) do {} while(0)
#define PD_MUTEX_EXIT_DEBUG(linkage,var) do{} while(0)
#define PD_MUTEX_EXIT_POINTER_DEBUG(linkage,var) do{} while(0)
#define PD_MUTEX_INIT_NOMNG(linkage,var,name) do{} while(0)
#define PD_MUTEX_DESTROY_NOMNG(linkage,var) do{} while(0)
#define PD_MUTEX_ENTER_NOMNG(linkage,var) do{} while(0)
#define PD_MUTEX_EXIT_NOMNG(linkage,var) do{} while(0)

#endif /* !USER_MODE_TEST || PD_USE_MUTEX */

#endif /* !__PD_MUTEX_H */

