/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_mutex.h,v 1.4 2001/04/20 22:03:17 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: 
 *
 */

#ifndef __PD_MUTEX_H
#define __PD_MUTEX_H  1

#ifndef __PD_TYPES_H
#include <pdpi/linux/pd_types.h>
#endif /* __PD_TYPES_H */

#if PD_LINUX == 0x02004
#  ifndef __LINUX_SPINLOCK_H
#  include <linux/spinlock.h>
#  endif
#else
#  ifndef __ASM_SPINLOCK_H
#  include <asm/spinlock.h>
#  endif
#endif


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define PD_MUTEX_PROTECT(linkage,var) spinlock_t var ## _lock
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var) spinlock_t var ## _lock
#define PD_MUTEX_EXPORT(linkage,var) extern spinlock_t var ## _lock
#define PD_MUTEX_POINTER(linkage,var) spinlock_t * var ## _lock
#define PD_MUTEX_ADDR(linkage,var) (spinlock_t *)(&(var ## _lock))
#define PD_MUTEX_INIT(linkage,var) spin_lock_init(var ## _lock )
#define PD_MUTEX_DESTROY(linkage,var) do {} while (0)
#define PD_MUTEX_ENTER(linkage,var) spin_lock( var ## _lock )
#define PD_MUTEX_ENTER_DEBUG(linkage,var) \
    do { \
        printf("mutex_enter(" ## #var ## "_lock=0x%x @ %s:%d\n", \
                (unsigned int)&(var ## _lock), wsi_proc_pid(current), \
                __FILE__, __LINE__ ); \
        PD_MUTEX_ENTER(linkage,var); \
    } while (0)

#define PD_MUTEX_EXIT(linkage,var) spin_unlock( var ## _lock )
#define PD_MUTEX_EXIT_DEBUG(linkage,var) \
    do { \
        printf("mutex_exit(" ## #var ## "_lock=0x%x @ %s:%d\n", \
                (unsigned int)&(var ## _lock), wsi_proc_pid(current), \
                __FILE__, __LINE__ ); \
        PD_MUTEX_EXIT(var); \
    } while (0)

#define PD_MUTEX_INIT_NOMNG(linkage,var,name) spin_lock_init( var )
#define PD_MUTEX_DESTROY_NOMNG(linkage,var) do {} while (0)
#define PD_MUTEX_ENTER_NOMNG(linkage,var) spin_lock( var )
#define PD_MUTEX_EXIT_NOMNG(linkage,var) spin_unlock( var )


/************************************************************************\
 * Public types                                                         *
\************************************************************************/

typedef spinlock_t pd_mutex_t;


#endif /* !__PD_MUTEX_H */

