/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pi_defs.h,v 1.4 2001/03/29 22:36:08 stevek Exp $
 * 
 * Author: Douglas Kilpatrick, Steve Kiernan
 * 
 * Purpose:
 *    To place random platform specific typedef's and defines and header-fixes
 *    in one place.
 */

#ifndef __PI_DEFS_H
#define __PI_DEFS_H

#if defined(__sun__) && defined(__svr4__)
#ifndef __PD_DEFS_H
#include <pdpi/solaris/pd_defs.h>
#endif /* __PD_DEFS_H */

#elif defined(__linux__)
#ifndef __PD_DEFS_H
#include <pdpi/linux/pd_defs.h>
#endif /* __PD_DEFS_H */

#elif defined(__FreeBSD__)
#ifndef __PD_DEFS_H
#include <pdpi/freebsd/pd_defs.h>
#endif /* __PD_DEFS_H */

#elif defined(_WIN32)
#ifndef __PD_DEFS_H
#include <pdpi/winnt/pd_defs.h>
#endif /* __PD_DEFS_H */

#else
#error "Platform not supported by the common includes"
#endif

#ifdef PD_UNIX
#define _tcslen(a)      strlen(a)
#define _tcscmp(a,b)    strcmp(a,b)
#define _tcsncmp(a,b,n) strncmp(a,b,n)
#define _tcsrchr(a,b)   strrchr(a,b)
#define _tcscpy(a,b)    strcpy(a,b)
#define _tcscat(a,b)    strcat(a,b)
#define _tcsncat(a,b,n) strncat(a,b,n)
#define _stprintf       sprintf
#define _sntprintf      snprintf
#define _tprintf        printf
#define _vstprintf      vsprintf
#define _vtprintf       vprintf
#endif

#ifdef PD_IN_KERNEL
#ifndef assert
#  ifndef NDEBUG
#define assert(s) \
   ((s) ? (void)0 : panic("assertion %s failed: %s:%d\n",#s,__FILE__,__LINE__))
#  else
#define assert(s)
#  endif
#endif
#else /* ! PD_IN_KERNEL */
#include <assert.h>
#endif


/*
 * PD_BROKEN_STL:
 * gcc-2.7's default STL is broken.  Use the "template/" directory
 * for STLplus on that platform.  On other platforms, use the native
 * STL support.
 */
#if ( __GNUC__ == 2 ) && ( __GNUC_MINOR__ <= 7 )
#define PD_BROKEN_STL                                           1
#define PD_BROKEN_NAMESPACES                                    1
#endif /* GCC-2.7 */


#ifndef PD_ASSEMBLY

/*
 * PD_NO_FOO_DECL
 * 
 * A common function does not exist in the kernel.
 */
#ifdef PD_IN_KERNEL
#  ifdef PD_NO_SNPRINTF_DECL
int snprintf(char *str, size_t size, const char *format, ...);
#  endif

#  ifdef PD_NO_QSORT_DECL
void qsort(void *base, size_t bel, size_t width,
           int (*compar)(const void*, const void*));
#  endif
#endif /* PD_IN_KERNEL */

#endif /* !PD_ASSEMBLY */

#endif /* __PI_DEFS_H */

