/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_cv.h,v 1.3 2001/02/23 19:02:07 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent condition variables
 *
 */

#ifndef __PD_CV_H
#define __PD_CV_H  1

#include <sys/condvar.h>

#ifndef __PD_TYPES_H
#include <pdpi/solaris/pd_types.h>
#endif /* __PD_TYPES_H */

#ifndef __PD_MUTEX_H
#include <pd_mutex.h>
#endif


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define PD_CV_PROTECT(linkage,var) kcondvar_t var ## _cv
#define PD_CV_PROTECT_EXPORTED(linkage,var) kcondvar_t var ## _cv
#define PD_CV_EXPORT(linkage,var) extern kcondvar_t var ## _cv

#ifndef PD_SUPPRESS_FUNCTION_DECL
#define PD_CV_INIT(linkage,var) cv_init( &(var ## _cv), NULL, CV_DEFAULT, NULL )
#define PD_CV_DESTROY(linkage,var) cv_destroy( &(var ## _cv) )
#define PD_CV_WAIT(linkage,var) cv_wait( &(var ## _cv), &(var ## _lock) )
#define PD_CV_WAIT_DEBUG(linkage,var) \
   do { printf( "cv_wait(" ## #var ## "_cv=0x%x, " ## #var ## "_lock=0x%x)" \
                "/thread=0x%x @ %s:%d\n", (uint32_t)&(var ## _cv), \
                (uint32_t)&(var ## _lock), (uint32_t)curthread, __FILE__, \
                __LINE__ ); \
        cv_wait( &(var ## _cv), &(var ## _lock) ); } while(0)
#define PD_CV_SIGNAL(linkage,var) cv_signal( &(var ## _cv) )
#define PD_CV_SIGNAL_DEBUG(linkage,var) \
   do { printf( "cv_signal(" ## #var ## "_cv=0x%x)/thread=0x%x @ %s:%d\n", \
                (uint32_t)&(var ## _cv), (uint32_t)curthread, __FILE__, \
                __LINE__ ); \
        cv_signal( &(var ## _cv) ); } while(0)
#define PD_CV_BROADCAST(linkage,var) cv_broadcast( &(var ## _cv) )
#define PD_CV_BROADCAST_DEBUG(linkage,var) \
   do { printf( "cv_broadcast(" ## #var ## "_cv=0x%x)/thread=0x%x @ %s:%d\n", \
                (uint32_t)&(var ## _cv), (uint32_t)curthread, __FILE__, \
                __LINE__ ); \
        cv_signal( &(var ## _cv) ); } while(0)
#endif /* PD_SUPPRESS_FUNCTION_DECL */

#endif /* !__WSS_CV_H */

