/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_mutex.h,v 1.3 2001/02/23 19:02:07 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: System-dependent mutex macros and types
 *
 */

#ifndef __PD_MUTEX_H
#define __PD_MUTEX_H  1

#ifndef __PD_TYPES_H
#include <pdpi/solaris/pd_types.h>
#endif /* __PD_TYPES_H */

#ifndef USER_MODE_TEST
#include <sys/cmn_err.h>
#include <sys/mutex.h>
#endif /* USER_MODE_TEST */


/************************************************************************\
 * Public types                                                         *
\************************************************************************/

#ifndef USER_MODE_TEST
typedef kmutex_t pd_mutex_t;
#else
typedef void pd_mutex_t;
#endif /* USER_MODE_TEST */


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#ifndef USER_MODE_TEST

#define PD_MUTEX_PROTECT(linkage,var) pd_mutex_t var ## _lock
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var) pd_mutex_t var ## _lock
#define PD_MUTEX_EXPORT(linkage,var) extern pd_mutex_t var ## _lock
#define PD_MUTEX_POINTER(linkage,var) pd_mutex_t * var ## _lock
#define PD_MUTEX_ADDR(linkage,var) (pd_mutex_t *)(&(var ## _lock))

#ifndef PD_SUPPRESS_FUNCTION_DECL
#define PD_MUTEX_INIT(linkage,var) \
   mutex_init( &(var ## _lock), #var ## " lock", MUTEX_DEFAULT, NULL )
#define PD_MUTEX_DESTROY(linkage,var) mutex_destroy( &(var ## _lock) )
#ifdef DEBUG_REFS
#define PD_MUTEX_ENTER(linkage,var) \
   if( MUTEX_HELD( &(var ## _lock) ) ) \
      panic( "PD_MUTEX_ENTER(" ## #var ## "_lock) recursive enter at %s:%d (owner thread 0x%x)", \
             __FILE__, __LINE__, mutex_owner( &(var ## _lock) ) ); \
   else \
      mutex_enter( &(var ## _lock) )

#else /* DEBUG_REFS */
#define PD_MUTEX_ENTER(linkage,var) mutex_enter( &(var ## _lock) )
#endif /* DEBUG_REFS */
#define PD_MUTEX_ENTER_DEBUG(linkage,var) \
   do { printf( "mutex_enter(" ## #var ## "_lock=0x%x)/thread=0x%x @ %s:%d\n", \
                 (uint32_t)&(var ## _lock), (uint32_t)curthread, __FILE__, \
                 __LINE__ ); \
        mutex_enter( &(var ## _lock ) ); } while(0)
#define PD_MUTEX_EXIT(linkage,var) mutex_exit( &(var ## _lock) )
#define PD_MUTEX_EXIT_DEBUG(linkage,var) \
   do { printf( "mutex_exit(" ## #var ## "_lock=0x%x)/thread=0x%x @ %s:%d\n", \
                 (uint32_t)&(var ## _lock), (uint32_t)curthread, __FILE__, \
                 __LINE__ ); \
        mutex_exit( &(var ## _lock ) ); } while(0)

#define PD_MUTEX_INIT_NOMNG(linkage,var, name) \
   mutex_init( &(var), (name), MUTEX_DEFAULT, NULL )
#define PD_MUTEX_DESTROY_NOMNG(linkage,var) mutex_destroy( &(var) )
#define PD_MUTEX_ENTER_NOMNG(linkage,var) mutex_enter( &(var) )
#define PD_MUTEX_EXIT_NOMNG(linkage,var) mutex_exit( &(var) )

#endif /* PD_SUPPRESS_FUNCTION_DECL */

#else /* USER_MODE_TEST */

#define PD_MUTEX_PROTECT(linkage,var)
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var)
#define PD_MUTEX_EXPORT(linkage,var)
#define PD_MUTEX_POINTER(linkage,var)
#define PD_MUTEX_ADDR(linkage,var) NULL
#define PD_MUTEX_INIT(linkage,var)
#define PD_MUTEX_DESTROY(linkage,var)
#define PD_MUTEX_ENTER(linkage,var)
#define PD_MUTEX_ENTER_DEBUG(linkage,var)
#define PD_MUTEX_EXIT(linkage,var)
#define PD_MUTEX_EXIT_DEBUG(linkage,var)
#define PD_MUTEX_INIT_NOMNG(linkage,var,name)
#define PD_MUTEX_DESTROY_NOMNG(linkage,var)
#define PD_MUTEX_ENTER_NOMNG(linkage,var)
#define PD_MUTEX_EXIT_NOMNG(linkage,var)

#endif /* USER_MODE_TEST */

#endif /* !__WSS_MUTEX_H */

