/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_cv.h,v 1.3 2001/02/23 19:02:07 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent condition variables
 *
 */

#ifndef __PD_CV_H
#define __PD_CV_H  1

#ifndef __PD_TYPES_H
#include <pdpi/winnt/pd_types.h>
#endif /* __PD_TYPES_H */

#ifndef __PD_MUTEX_H
#include <pd_mutex.h>
#endif /* __PD_MUTEX_H */


#ifdef __cplusplus
extern "C" {
#endif


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define PD_CV_PROTECT(linkage,var) pd_cv_t var ## _cv
#define PD_CV_PROTECT_EXPORTED(linkage,var) pd_cv_t linkage var ## _cv
#define PD_CV_EXPORT(linkage,var) extern pd_cv_t linkage var ## _cv
#define PD_CV_INIT(linkage,var) pd_cv_init( &(var ## _cv) )
#define PD_CV_DESTROY(linkage,var) pd_cv_destroy( var ## _cv )
#ifdef DEBUG_MUTEX
#define PD_CV_WAIT(linkage,var) pd_cv_wait( var ## _cv, &(var ## _lock), \
                                            PD_MAKESTR(__FILE__), __LINE__ )
#else
#define PD_CV_WAIT(linkage,var) pd_cv_wait( var ## _cv, var ## _lock )
#endif /* DEBUG_MUTEX */
#define PD_CV_WAIT_DEBUG(linkage,var) PD_CV_WAIT(var)
#define PD_CV_SIGNAL(linkage,var) pd_cv_signal( var ## _cv )
#define PD_CV_SIGNAL_DEBUG(linkage,var) PD_CV_SIGNAL(var)
#define PD_CV_BROADCAST(linkage,var) pd_cv_broadcast( var ## _cv )
#define PD_CV_BROADCAST_DEBUG(linkage,var) PD_CV_BROADCAST(var)

/************************************************************************\
 * Public types                                                         *
\************************************************************************/

typedef HANDLE pd_cv_t;


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

void PDLINKAGE pd_cv_init      __P(( pd_cv_t * ));
void PDLINKAGE pd_cv_destroy   __P(( pd_cv_t ));
#ifdef DEBUG_MUTEX
void PDLINKAGE pd_cv_wait      __P(( pd_cv_t, pd_mutex_t *,
                                     const pd_char_t *, int ));
#else
void PDLINKAGE pd_cv_wait      __P(( pd_cv_t, pd_mutex_t )); 
#endif /* DEBUG_MUTEX */
void PDLINKAGE pd_cv_signal    __P(( pd_cv_t ));
void PDLINKAGE pd_cv_broadcast __P(( pd_cv_t ));

#ifdef __cplusplus
}
#endif

#endif /* !__PD_CV_H */

