/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pd_mutex.h,v 1.3 2001/02/23 19:02:07 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: 
 *
 */

#ifndef __PD_MUTEX_H
#define __PD_MUTEX_H  1

#ifndef __PD_TYPES_H
#include <pdpi/winnt/pd_types.h>
#endif /* __PD_TYPES_H */

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define PD_MUTEX_PROTECT(linkage,var) pd_mutex_t var ## _lock
#define PD_MUTEX_PROTECT_EXPORTED(linkage,var) pd_mutex_t linkage var ## _lock
#define PD_MUTEX_EXPORT(linkage,var) extern pd_mutex_t linkage var ## _lock

#ifdef DEBUG_MUTEX
#  define PD_MUTEX_INIT(linkage,var) \
      pd_mutex_init( &(var ## _lock), WSS_MAKESTR(__FILE__), __LINE__ )
#else
#  define PD_MUTEX_INIT(linkage,var) \
      pd_mutex_init( &(var ## _lock) )
#endif /* DEBUG_MUTEX */

#ifdef DEBUG_MUTEX
#  define PD_MUTEX_DESTROY(linkage,var) \
      pd_mutex_destroy( &(var ## _lock), WSS_MAKESTR(__FILE__), __LINE__ )
#else
#  define PD_MUTEX_DESTROY(linkage,var) \
      pd_mutex_destroy( var ## _lock )
#endif /* DEBUG_MUTEX */

#ifdef DEBUG_MUTEX
#  define PD_MUTEX_ENTER(linkage,var) \
      pd_mutex_enter( &(var ## _lock), WSS_MAKESTR(__FILE__), __LINE__ )
#else
#  define PD_MUTEX_ENTER(linkage,var) \
      pd_mutex_enter( var ## _lock )
#endif /* DEBUG_MUTEX */

#ifdef DEBUG_MUTEX
#  define PD_MUTEX_EXIT(linkage,var) \
      pd_mutex_exit( &(var ## _lock), WSS_MAKESTR(__FILE__), __LINE__ )
#else
#  define PD_MUTEX_EXIT(linkage,var) \
      pd_mutex_exit( var ## _lock )
#endif /* DEBUG_MUTEX */

#ifdef DEBUG_MUTEX
#  define PD_MUTEX_INIT_NOMNG(linkage,var,name) \
      pd_mutex_init( &(var), WSS_MAKESTR(__FILE__), __LINE__ )
#  define PD_MUTEX_DESTROY_NOMNG(linkage,var) \
      pd_mutex_destroy( &(var), WSS_MAKESTR(__FILE__), __LINE__ )
#  define PD_MUTEX_ENTER_NOMNG(linkage,var) \
      pd_mutex_enter( &(var), WSS_MAKESTR(__FILE__), __LINE__ )
#  define PD_MUTEX_EXIT_NOMNG(linkage,var) \
      pd_mutex_exit( &(var), WSS_MAKESTR(__FILE__), __LINE__ )
#else
#  define PD_MUTEX_INIT_NOMNG(linkage,var,name) pd_mutex_init( &(var) )
#  define PD_MUTEX_DESTROY_NOMNG(linkage,var)   pd_mutex_destroy( (var) )
#  define PD_MUTEX_ENTER_NOMNG(linkage,var)     pd_mutex_enter( (var) )
#  define PD_MUTEX_EXIT_NOMNG(linkage,var)      pd_mutex_exit( (var) )
#endif


/************************************************************************\
 * Public types and structures                                          *
\************************************************************************/

#ifdef DEBUG_MUTEX
typedef struct _pd_mutex pd_mutex_t;

struct _pd_mutex
{
   HANDLE hMutex;
   DWORD  refCount;
};
#else
typedef HANDLE pd_mutex_t;
#endif /* DEBUG_MUTEX */


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifdef DEBUG_MUTEX
void PDLINKAGE pd_mutex_init    __P(( pd_mutex_t *, const pd_char_t *,
                                      int ));
void PDLINKAGE pd_mutex_destroy __P(( pd_mutex_t *, const pd_char_t *,
                                      int ));
void PDLINKAGE pd_mutex_enter   __P(( pd_mutex_t *, const pd_char_t *,
                                      int ));
void PDLINKAGE pd_mutex_exit    __P(( pd_mutex_t *, const pd_char_t *,
                                      int ));
#else
void PDLINKAGE pd_mutex_init    __P(( pd_mutex_t * ));
void PDLINKAGE pd_mutex_destroy __P(( pd_mutex_t ));
void PDLINKAGE pd_mutex_enter   __P(( pd_mutex_t ));
void PDLINKAGE pd_mutex_exit    __P(( pd_mutex_t ));
#endif /* DEBUG_MUTEX */

#ifdef __cplusplus
}
#endif

#endif /* !__PD_MUTEX_H */

