#!/usr/local/bin/perl
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: genkmod.pl,v 1.4 2001/04/05 18:40:32 stevek Exp $

print "OS Release: $ARGV[0]\n";

my $osrel = $ARGV[0];
my $num = $#ARGV if @ARGV;
print "Processing $num linker(s)...\n";

shift @ARGV;

$| = 1;

open( OUT, '>kmod_freebsd.c' ) || die "kmod_freebsd.c: $!";

print OUT <<EOF;
/*
 * Automatically generated by genkmod.pl
 *
 * Do not modify.
 */
#include <wr_conf.h>

#include <wsi_stdlib.h>

EOF

map { print OUT 'extern int linker_'.$_.'_init __P((void));'."\n" } @ARGV;

if( $osrel ge 300000 ) {
   # FreeBSD 3.x and 4.x
   print OUT <<EOF;

#include <sys/module.h>
#include <sys/linker.h>
#include <sys/kernel.h>

EOF
}
else {
   # FreeBSD 2.2.x
   print OUT <<EOF;

#include <sys/exec.h>
#include <sys/sysent.h>
#include <sys/lkm.h>
#include <sys/kernel.h>

MOD_MISC( wss );

extern int wsi_load   __P(( struct lkm_table *, int ));
extern int wsi_unload __P(( struct lkm_table *, int ));
static int wss_stat   __P(( struct lkm_table *, int ));

static int
wss_stat( struct lkm_table *lkmtp, int cmd )
{
   return(0);
}

static int
wss_unload( struct lkm_table *lkmtp, int cmd )
{
   return wsi_unload( lkmtp, cmd );
}

static int
wss_load( struct lkm_table *lkmtp, int cmd )
{
   int error = wsi_load( lkmtp, cmd );

EOF
}

if( $osrel ge 500000 ) {
   print OUT <<EOF;
MODULE_VERSION(wss,1);
EOF

   foreach $dep ( split(' ',$ENV{'KMODDEPENDS'}) ) {
      print OUT <<EOF;
MODULE_DEPEND(wss,$dep,1,1,1);
EOF
   }
}


#
# Call version-specific initialization
#
foreach $format ( @ARGV ) {
   print "  $format ";

   if( $osrel ge 300000 ) {
      print OUT <<EOF;
static int linker_${format}_mod_event __P(( module_t, int, void * ));

static moduledata_t linker_${format}_mod =
{
   "WSS - ${format} module linker",
   linker_${format}_mod_event,
   NULL,
};
DECLARE_MODULE( ${format}, linker_${format}_mod, SI_SUB_KTHREAD_IDLE,
                SI_ORDER_MIDDLE );

static int
linker_${format}_mod_event( module_t mod, int type, void *data )
{
   switch( type )
   {
      case MOD_LOAD:
         return linker_${format}_init();
      case MOD_UNLOAD:
      default:
         break;
   }
   return 0;
}

EOF
   }
   else {
      print OUT <<EOF;
   if( error == 0 )
      error = linker_${format}_init();
EOF
   }

   print "- done\n";
}

#
# Close out the file, doing final code writes
#
if( $osrel lt 300000 ) {
   print OUT <<EOF;

   return error;
}

int
wss( struct lkm_table *lkmtp, int cmd, int ver )
{
   DISPATCH( lkmtp, cmd, ver, wss_load, wss_unload, wss_stat );
}

EOF
}

close( OUT );

