/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 */

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/filedesc.h>
#include <sys/fcntl.h>
#include <sys/proc.h>
#include <sys/uio.h>
#include <sys/namei.h>
#include <stdarg.h>

#include "vnode.h"


static struct vnode *wr_audit_file = NULL;
static u_long wr_audit_offset = 0;
static char wr_audit_buffer[1024];

struct vnode *wr_file_open(int fmode, char *path, int cmode);
int  wr_write_file_buffer(char *exp_write_buf,  
        		  unsigned long file_offset, 
        		  int buf_size,
        		  struct vnode *vp);
void wr_file_close(struct vnode *vp, int mode);

extern int wr_audit(const char *format, ...);
extern void wr_audit_close(struct vnode *vp, int mode);
extern void wr_audit_init();


int wr_audit(const char *format, ...)

{
        int len;
        va_list ap;

        va_start(ap, format);
        len = kvprintf(format, NULL, (void *)wr_audit_buffer, 10, ap);
        wr_audit_buffer[len] = '\0';
        va_end(ap);

        VOP_LOCK(wr_audit_file);
        wr_write_file_buffer(wr_audit_buffer,wr_audit_offset,len,wr_audit_file);
        VOP_UNLOCK(wr_audit_file);
        wr_audit_offset += len;

        return(len);
}

void
wr_audit_close(struct vnode *vp, int mode)
{
        wr_file_close(wr_audit_file, FREAD|FWRITE);
}

void wr_audit_init() 
{
     wr_audit_file = 
           wr_file_open(O_RDWR|O_CREAT|O_TRUNC, "/tmp/audit.dat", 0744);
     if (!wr_audit_file) {     
          printf("wr_audit_init:: Can't audit without a valid file!\n"); 
          return;
     }       
     VOP_UNLOCK(wr_audit_file);
}
