/*! \file linker_aout.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: linker_aout.h,v 1.5 2001/02/23 19:02:08 stevek Exp $
 *
 * \brief   Skeleton header of prototypes and structures used by a module linker
 * \author  Steve Kiernan
 * \version $Revision: 1.5 $
 * \date    $Date: 2001/02/23 19:02:08 $
 */

#ifndef __LINKER_AOUT_H
#define __LINKER_AOUT_H	1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSS_LINKER_H
#include <wss_linker.h>
#endif

#include "linker_aout_machdep.h"


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

/*
 * Execution header macros
 */
#define N_GETMAGIC(ex) \
        ( (ex)->a_midmag & 0xffff )
#define N_GETMID(ex) \
        ( (N_GETMAGIC_NET(ex) == ZMAGIC) ? N_GETMID_NET(ex) : \
        ((ex)->a_midmag >> 16) & 0x03ff )
#define N_GETFLAG(ex) \
        ( (N_GETMAGIC_NET(ex) == ZMAGIC) ? N_GETFLAG_NET(ex) : \
        ((ex)->a_midmag >> 26) & 0x3f )
#define N_SETMAGIC(ex,mag,mid,flag) \
        ( (ex)->a_midmag = (((flag) & 0x3f) <<26) | (((mid) & 0x03ff) << 16) | \
        ((mag) & 0xffff) )

#define N_GETMAGIC_NET(ex) \
        (ntohl((ex)->a_midmag) & 0xffff)
#define N_GETMID_NET(ex) \
        ((ntohl((ex)->a_midmag) >> 16) & 0x3ff)
#define N_GETFLAG_NET(ex) \
        ((ntohl((ex)->a_midmag) >> 26) & 0x3f)
#define N_SETMAGIC_NET(ex,mag,mid,flag) \
        ( (ex)->a_midmag = htonl( (((flag)&0x3f)<<26) | (((mid)&0x03ff)<<16) | \
        (((mag)&0xffff)) ) )

#define N_ALIGN(ex,x) \
        (N_GETMAGIC(ex) == ZMAGIC || N_GETMAGIC(ex) == QMAGIC || \
         N_GETMAGIC_NET(ex) == ZMAGIC || N_GETMAGIC_NET(ex) == QMAGIC ? \
         ((x) + __LDPGSZ - 1) & ~(unsigned long)(__LDPGSZ - 1) : (x))

/* Valid magic number check */
#define N_BADMAG(ex) \
        (N_GETMAGIC(ex) != OMAGIC && N_GETMAGIC(ex) != NMAGIC && \
         N_GETMAGIC(ex) != ZMAGIC && N_GETMAGIC(ex) != QMAGIC && \
         N_GETMAGIC_NET(ex) != OMAGIC && N_GETMAGIC_NET(ex) != NMAGIC && \
         N_GETMAGIC_NET(ex) != ZMAGIC && N_GETMAGIC_NET(ex) != QMAGIC)


/* Address of the bottom of the text segment */
#define N_TXTADDR(ex) \
        ((N_GETMAGIC(ex) == OMAGIC || N_GETMAGIC(ex) == NMAGIC || \
        N_GETMAGIC(ex) == ZMAGIC) ? \
        ((ex)->a_entry < (ex)->a_text ? 0 : (ex)->a_entry & ~__LDPGSZ) : __LDPGSZ)

/* Address of the bottom of the data segment */
#define N_DATADDR(ex) \
        N_ALIGN(ex, N_TXTADDR(ex) + (ex)->a_text)

/* Text segment offset */
#define N_TXTOFF(ex) \
        (N_GETMAGIC(ex) == ZMAGIC ? __LDPGSZ : (N_GETMAGIC(ex) == QMAGIC || \
        N_GETMAGIC_NET(ex) == ZMAGIC) ? 0 : sizeof(AOut_Exec))

/* Data segment offset */
#define N_DATOFF(ex) \
        N_ALIGN(ex, N_TXTOFF(ex) + (ex)->a_text)

/* Uninitialized data segment offset */
#define N_BSSOFF(ex) \
        N_ALIGN(ex, N_DATOFF(ex) + (ex)->a_data)

/* Relocation table offset */
#define N_RELOFF(ex) \
        N_ALIGN(ex, N_DATOFF(ex) + (ex)->a_data)

/* Symbol table offset */
#define N_SYMOFF(ex) \
        (N_RELOFF(ex) + (ex)->a_trsize + (ex)->a_drsize)

/* String table offset */
#define N_STROFF(ex)    (N_SYMOFF(ex) + (ex)->a_syms)

/* a_magic */
#define OMAGIC          0407    /* old impure format */
#define NMAGIC          0410    /* read-only text */
#define ZMAGIC          0413    /* demand load format */
#define QMAGIC          0314    /* "compact" demand load format */

/* a_mid */
#define MID_ZERO        0       /* unknown - implementation dependent */
#define MID_SUN010      1       /* sun 68010/68020 binary */
#define MID_SUN020      2       /* sun 68020-only binary */
#define MID_I386        134     /* i386 BSD binary */
#define MID_SPARC       138     /* sparc */
#define MID_HP200       200     /* hp200 (68010) BSD binary */
#define MID_HP300       300     /* hp300 (68020+68881) BSD binary */
#define MID_HPUX        0x20C   /* hp200/300 HP-UX binary */
#define MID_HPUX800     0x20B   /* hp800 HP-UX binary */

/* a_flags */
#define EX_PIC          0x10    /* contains position independent code */
#define EX_DYNAMIC      0x20    /* contains run-time link-edit info */
#define EX_DPMASK       0x30    /* mask for the above */


/*
 * Symbol macros
 */
#define IS_DEBUGGER_SYMBOL(x) ((x) & N_STAB)
#define SYMBOL_TYPE(x)     ((x) & N_TYPE)
#define SYMBOL_BIND(x)     (((x) >> 4) & 0xf)

/* n_type */
#define N_UNDF  0x00            /* undefined */
#define N_ABS   0x02            /* absolute address */
#define N_TEXT  0x04            /* text segment */
#define N_DATA  0x06            /* data segment */
#define N_BSS   0x08            /* bss segment */
#define N_INDR  0x0a            /* alias definition */
#define N_SIZE  0x0c            /* pseudo type, defines a symbol's size */
#define N_COMM  0x12            /* common reference */
/* GNU extensions */
#define N_SETA  0x14            /* Absolute set element symbol */
#define N_SETT  0x16            /* Text set element symbol */
#define N_SETD  0x18            /* Data set element symbol */
#define N_SETB  0x1a            /* Bss set element symbol */
#define N_SETV  0x1c            /* Pointer to set vector in data area */
/* end GNU extensions */
#define N_FN    0x1e            /* file name (N_EXT on) */
#define N_WARN  0x1e            /* warning message (N_EXT off) */

#define N_EXT   0x01            /* external (global) bit, OR'ed in */
#define N_TYPE  0x1e            /* mask for all the type bits */
#define N_STAB  0xe0            /* mask for debugger symbols -- stab(5) */


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct _AOut_Exec       AOut_Exec;
typedef struct _AOut_Rel        AOut_Rel;
typedef struct _AOut_Sym        AOut_Sym;
typedef enum _linker_aout_reloc linker_aout_reloc_t;
typedef struct linker_aout_data linker_aout_data_t;


/*
 * Header for a.out file
 */
struct _AOut_Exec
{
   unsigned long a_midmag;       /* flags, mid, and magic */
   unsigned long a_text;         /* text segment size */
   unsigned long a_data;         /* initialized data size */
   unsigned long a_bss;          /* uninitialized data size */
   unsigned long a_syms;         /* symbol table size */
   unsigned long a_entry;        /* entry point */
   unsigned long a_trsize;       /* text relocation size */
   unsigned long a_drsize;       /* data relocation size */
};


/*
 * Symbol table entry structure
 */
struct _AOut_Sym
{
   long          n_strx;    /* string table offset */
   unsigned char n_type;    /* type defines */
   char          n_other;   /* ".type" and binding information */
   short         n_desc;    /* used by stab entries */
   unsigned long n_value;   /* address/value of the symbol */
};


/*
 * Linker structures
 */
struct linker_aout_data
{
   AOut_Exec  *ex;
   pd_byte_t *text;
   pd_byte_t *data;
   AOut_Sym   *symtab;
   pd_char_t *strtab;
   AOut_Rel   *reltext;
   AOut_Rel   *reldata;
   int         flags;
};

enum _linker_aout_reloc
{
   AOUT_RELOC_UNKNOWN,
   AOUT_RELOC_TEXT,
   AOUT_RELOC_DATA,
};


/***************************************************************************\
 * Public externals                                                        *
\***************************************************************************/

extern u_int32_t __LDPGSZ;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int WSSLINKAGE _linker_aout_machdep_relocate  __P(( wss_linker_file_t *,
                                                    pd_byte_t *, int,
                                                    AOut_Rel * ));
int WSSLINKAGE _linker_aout_machdep_supported __P(( wss_linker_file_t * ));

#endif /* __LINKER_AOUT_H */

