/*! \file linker_elf_common.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: linker_elf_common.h,v 1.4 2001/02/23 19:02:10 stevek Exp $
 *
 * \brief   Macros used by the Elf module linkers
 * \author  Steve Kiernan
 * \version $Revision: 1.4 $
 * \date    $Date: 2001/02/23 19:02:10 $
 */

#ifndef __LINKER_ELF_COMMON_H
#define __LINKER_ELF_COMMON_H	1

/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

/* Indexes into the e_ident array */
#define EI_MAG0         0       /* Magic number, byte 0 */
#define EI_MAG1         1       /* Magic number, byte 1 */
#define EI_MAG2         2       /* Magic number, byte 2 */
#define EI_MAG3         3       /* Magic number, byte 3 */
#define EI_CLASS        4       /* Class of machine */
#define EI_DATA         5       /* Data format */
#define EI_VERSION      6       /* ELF format version */
#define EI_PAD          7       /* Start of padding */
#define EI_BRAND        8       /* Start of architecture identification */
#define EI_NIDENT       16      /* Size of e_ident array */


/* Values for the magic number bytes */
#define ELFMAG0         0x7f
#define ELFMAG1         'E'
#define ELFMAG2         'L'
#define ELFMAG3         'F'
#define ELFMAG          "\177ELF"
#define SELFMAG         4


/* Values for e_ident[EI_VERSION] and e_version. */
#define EV_NONE         0
#define EV_CURRENT      1


/* Values for e_ident[EI_CLASS]. */
#define ELFCLASSNONE    0       /* Unknown class */
#define ELFCLASS32      1       /* 32-bit architecture */
#define ELFCLASS64      2       /* 64-bit architecture */
#define ELFCLASSNUM     3


/* Values for e_ident[EI_DATA]. */
#define ELFDATANONE     0       /* Unknown data format */
#define ELFDATA2LSB     1       /* 2's complement little-endian */
#define ELFDATA2MSB     2       /* 2's complement big-endian */
#define ELFDATANUM      3


/* Values for e_type */
#define	ET_NONE		0
#define	ET_REL		1
#define	ET_EXEC		2
#define	ET_DYN		3
#define	ET_CORE		4
#define	ET_NUM		5


/* Values for e_machine */
#define	EM_NONE		0		/* Unknown machine */
#define	EM_M32		1		/* AT&T WE 32100 */
#define	EM_SPARC	2		/* Sun SPARC */
#define	EM_386		3		/* Intel 80386 */
#define	EM_68K		4		/* Motorola 68000 */
#define	EM_88K		5		/* Motorola 88000 */
#define	EM_486		6		/* Intel 80486 */
#define	EM_860		7		/* Intel i860 */
#define	EM_MIPS		8		/* MIPS RS3000 Big-Endian */
#define	EM_UNKNOWN9	9
#define	EM_MIPS_RS3_LE	10		/* MIPS RS3000 Little-Endian */
#define	EM_RS6000	11		/* RS6000 */
#define EM_SPARC64      11              /* SPARC v9 64-bit unofficial */
#define	EM_UNKNOWN12	12
#define	EM_UNKNOWN13	13
#define	EM_UNKNOWN14	14
#define	EM_PA_RISC	15		/* PA-RISC */
#define	EM_nCUBE	16		/* nCUBE */
#define	EM_VPP500	17		/* Fujitsu VPP500 */
#define	EM_SPARC32PLUS	18		/* Sun SPARC 32+ */
#define	EM_UNKNOWN19	19
#define	EM_PPC		20		/* PowerPC */
#define	EM_NUM		21

#define EM_ALPHA        0x9026          /* Alpha */


/* Special section indexes */
#define SHN_UNDEF            0          /* Undefined, missing, irrelevant */
#define SHN_LORESERVE   0xff00          /* First of reserved range */
#define SHN_LOPROC      0xff00          /* First processor-specific */
#define SHN_HIPROC      0xff1f          /* Last processor-specific */
#define SHN_ABS         0xfff1          /* Absolute values */
#define SHN_COMMON      0xfff2          /* Common data */
#define SHN_HIRESERVE   0xffff          /* Last of reserved range */


/* Values for sh_type */
#define SHT_NULL        0               /* inactive */
#define SHT_PROGBITS    1               /* program defined information */
#define SHT_SYMTAB      2               /* symbol table section */
#define SHT_STRTAB      3               /* string table section */
#define SHT_RELA        4               /* relocation section with addends*/
#define SHT_HASH        5               /* symbol hash table section */
#define SHT_DYNAMIC     6               /* dynamic section */ 
#define SHT_NOTE        7               /* note section */
#define SHT_NOBITS      8               /* no space section */
#define SHT_REL         9               /* relation section without addends */
#define SHT_SHLIB       10              /* reserved - purpose unknown */
#define SHT_DYNSYM      11              /* dynamic symbol table section */ 
#define SHT_NUM         12

#define SHT_LOPROC      0x70000000      /* reserved range for processor */
#define SHT_HIPROC      0x7fffffff      /* specific section header types */
#define SHT_LOUSER      0x80000000      /* reserved range for application */
#define SHT_HIUSER      0xffffffff      /* specific indexes */


/* Flags for sh_flags */
#define SHF_WRITE       0x1             /* Section contains writable data */
#define SHF_ALLOC       0x2             /* Section occupies memory */
#define SHF_EXECINSTR   0x4             /* Section contains instructions */
#define SHF_MASKPROC    0xf0000000      /* Reserved for processor-specific */


/* Values for p_type */
#define PT_NULL         0               /* Unused entry */
#define PT_LOAD         1               /* Loadable segment */
#define PT_DYNAMIC      2               /* Dynamic link information segment */
#define PT_INTERP       3               /* Pathname of interpreter */
#define PT_NOTE         4               /* Auxiliary information */
#define PT_SHLIB        5               /* Reserved */
#define PT_PHDR         6               /* Location of program header */
#define PT_NUM          7

#define PT_LOPROC       0x70000000      /* First processor-specific type */
#define PT_HIPROC       0x7fffffff      /* Last processor-specific type */


/* Values for p_flags. */
#define PF_X            0x1             /* Executable */
#define PF_W            0x2             /* Writable */
#define PF_R            0x4             /* Readable */


/* Symbol Binding - ELFNN_ST_BIND - st_info */
#define STB_LOCAL       0               /* Local symbol */
#define STB_GLOBAL      1               /* Global symbol */
#define STB_WEAK        2               /* like global - lower precedence */
#define STB_NUM         3

#define STB_LOPROC      13              /* reserved range for processor */
#define STB_HIPROC      15              /*  specific symbol bindings */


/* Symbol type - ELFNN_ST_TYPE - st_info */
#define STT_NOTYPE      0               /* Unspecified type */
#define STT_OBJECT      1               /* Data object */
#define STT_FUNC        2               /* Function */
#define STT_SECTION     3               /* Section */
#define STT_FILE        4               /* Source file */
#define STT_NUM         5

#define STT_LOPROC      13              /* reserved range for processor */
#define STT_HIPROC      15              /*  specific symbol types */


/* Special symbol table indexes. */
#define STN_UNDEF       0               /* Undefined symbol index */

#endif /* __LINKER_ELF_COMMON_H */

