/*! \file linker_skel.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: linker_skel.c,v 1.6 2001/02/23 19:02:10 stevek Exp $
 *
 * \brief   Skeleton source file to be used as a template for adding support
 *          for other module formats
 * \author  Steve Kiernan
 * \version $Revision: 1.6 $
 * \date    $Date: 2001/02/23 19:02:10 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <sys/types.h>

#include <wss_file.h>
#include <wss_malloc.h>
#include <wss_vm.h>

#include "linker_skel.h"


/***************************************************************************\
 * Private macros                                                          *
\***************************************************************************/

/***************************************************************************\
 * Forward declarations                                                    *
\***************************************************************************/

static int _linker_skel_load                    __P(( const pd_char_t *,
                                                      wss_linker_file_t * ));
static int _linker_skel_lookup_symbol           __P(( wss_linker_file_t *,
                                                      const pd_char_t *,
                                                      wss_linker_addr_t * ));
static int _linker_skel_export_symbols          __P(( wss_linker_file_t * ));
static int _linker_skel_finalize                __P(( wss_linker_file_t * ));
static int _linker_skel_unload                  __P(( wss_linker_file_t * ));


/***************************************************************************\
 * Private globals                                                         *
\***************************************************************************/

static wss_linker_ops_t _linker_skel_ops =
{
   _linker_skel_load
};

static wss_linker_file_ops_t _linker_skel_file_ops =
{
   _linker_skel_lookup_symbol,
   _linker_skel_export_symbols,
   _linker_skel_finalize,
   _linker_skel_unload
};

static const pd_char_t *_linker_skel_description = PD_MAKESTR("skeleton");


/***************************************************************************\
 * Private functions                                                       *
\***************************************************************************/

/*
 * Lookup a single symbol in the binary's symbol table and return the
 * symbol structure
 */
static int
_linker_skel_lookup_symbol( wss_linker_file_t *lf, const pd_char_t *symbol,
                             wss_linker_addr_t *addr )
{
   return EINVAL;
}


/*
 * Export library symbols for other modules to use
 */
static int
_linker_skel_export_symbols( wss_linker_file_t *lf )
{
   return EINVAL;
}


/*  _linker_skel_load */
/*! \brief Load an skeleton format module
 *
 *  Resolves and relocates symbols within the .text, .rodata, .data, and .bss
 *  sections.
 *
 *  The module does not depend on the file image in the filesystem once it is
 *  finalized.
 *
 *  \param filename Pointer to string containing the name of the file to load
 *  \param result   Pointer to linker file structure to fill
 *
 *  \return 0 if successfully loaded,
 *  \return ENOENT if unable to resolve symbols,
 *  \return EEXIST if the module is already loaded,
 *  \return EINVAL if the module is of an unknown type
 */                                                               
int EXPORT
_linker_skel_load( const pd_char_t *filename, wss_linker_file_t *result )
{
   return ENOEXEC;
}

static int
_linker_skel_finalize( wss_linker_file_t *lf )
{
   return EINVAL;
}

static int
_linker_skel_unload( wss_linker_file_t *lf )
{
   return EINVAL;
}


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void
linker_skel_init()
{
   wss_linker_register( _linker_skel_description, &_linker_skel_ops );
}

