/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: hook.cxx,v 1.25 2001/04/05 18:40:36 stevek Exp $ 
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include "hook.h"
#include "MSJDBG.h"

/*//////////////////////////////////////////////////////////////////////
                         File Specific Defines
//////////////////////////////////////////////////////////////////////*/
#define MakePtr( cast , ptr , AddValue ) \
                                 (cast)( (DWORD)(ptr)+(DWORD)(AddValue))

/*//////////////////////////////////////////////////////////////////////
                        File Specific Prototypes
//////////////////////////////////////////////////////////////////////*/
static PIMAGE_IMPORT_DESCRIPTOR
                     GetNamedImportDescriptor ( HMODULE hModule     ,
                                                LPCSTR  szImportMod  ) ;

static PIMAGE_EXPORT_DIRECTORY
                     GetNamedExportDirectory ( HMODULE hModule     ,
                                                LPCSTR  szExportMod  );

/*//////////////////////////////////////////////////////////////////////
                             Implementation
//////////////////////////////////////////////////////////////////////*/

LPCSTR HOOKFUNCDESC:: getDllName( void )
{
   if( hmodDll == NULL )
      hmodDll = GetModuleHandleA( szDll );
   return szDll;
}

HMODULE HOOKFUNCDESC:: getDllHandle( void )
{
   if( hmodDll == NULL )
      hmodDll = GetModuleHandleA( szDll );
   return hmodDll;
}

int HOOKFUNCDESC:: callDispatchPre( int space, pd_syscall_id_t code,
                                    void* uap[], void *retval,
                                    struct wss_proc *wrp,
                                    struct wss_wrapper_data *data,
                                    int args_in_userspace)
{
  if (preFuncs.size()>0) 
    usePre = TRUE;
  
  int retVal=WR_DENY;
  if (usePre) {
    PREPOSTCODEMAPLIST::iterator i;

    for (i = preFuncs.begin(); i!= preFuncs.end(); i++) {
      HOOKCODEMAP *codemap = *i;
      wss_wrapper_dispatch_t pre_func = codemap->func;
      data->wiv = codemap->wi->wi_data;
      retVal=pre_func(space, codemap->mapcode, uap, retval, wrp,
                      data, args_in_userspace);
      codemap->wi->wi_data = data->wiv;
    }
  } 
  else {
    // could be one of the "LWM_Always" hooks
    // return WR_ALLOW so that the API is allowed
    // to be called even though it's not really wrapped
    retVal = WR_ALLOW;
  }

  return retVal;
}

int HOOKFUNCDESC:: callDispatchPost( int space, pd_syscall_id_t code,
                                     void* uap[], void *retval,
                                     struct wss_proc *wrp,
                                     struct wss_wrapper_data *data,
                                     int args_in_userspace)
{
  if (postFuncs.size()>0) 
    usePost = TRUE;
  
  int retVal=WR_DENY;
  if (usePost) {
    PREPOSTCODEMAPLIST::reverse_iterator i;

    for (i = postFuncs.rbegin(); i!= postFuncs.rend(); i++) {
      HOOKCODEMAP *codemap = *i;
      wss_wrapper_dispatch_t post_func = codemap->func;
      data->wiv = codemap->wi->wi_data;
      retVal=post_func(space, codemap->mapcode, uap, retval, wrp,
                       data, args_in_userspace);
      codemap->wi->wi_data = data->wiv;
    }
  }
  else {
    // could be one of the "LWM_Always" hooks
    // return WR_ALLOW so that the API is allowed
    // to be called even though it's not really wrapped
    retVal = WR_ALLOW;
  }

  return retVal;
}

BOOL HookImportedFunctionsByName ( HMODULE         hModule     ,
                                   const char**    szImportModules ,
                                   HOOKLIST        hooklist,
                                   LPUINT          puiHooked    )
{
  UINT            uiCount = hooklist.size();

  // Double check the parameters.
  ASSERT ( NULL != szImportModules ) ;
  ASSERT ( 0 != uiCount ) ;

  ASSERT ( FALSE == IsBadWritePtr ( puiHooked , sizeof ( UINT ) ) ) ;
#ifdef _DEBUG
  // Check each function name in the hook array.
  {
    for ( HOOKLIST::iterator i=hooklist.begin( ); 
          i !=hooklist.end() ; i++ )
    {
      ASSERT ( NULL != (*i)->szFunc  ) ;
      ASSERT ( '\0' != *((*i)->szFunc) ) ;
      // If the proc is not NULL, then it is checked.
      if ( NULL != (*i)->pProc )
      {
        ASSERT ( FALSE == IsBadCodePtr ( (*i)->pProc));
      }
    }
  }
#endif
    // Do the parameter validation for real.
    if ( ( 0    == uiCount      )                        ||
         ( NULL == szImportModules  )                                  ||
         ( TRUE == IsBadWritePtr ( puiHooked , sizeof ( UINT ) ) ))
        
    {
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( FALSE ) ;
    }

    // TODO TODO
    //  Should each item in the hook array be checked in release builds?

    // Set the number of functions hooked to zero.
    *puiHooked = 0 ;

    const CHAR *szImportMod;

    for (int i=0; *szImportModules[i]!=0; i++) {
      szImportMod=szImportModules[i];

    // Get the specific import descriptor.
      PIMAGE_IMPORT_DESCRIPTOR pImportDesc =
        GetNamedImportDescriptor ( hModule , szImportMod );

    // Get the specific export descriptor.
      PIMAGE_EXPORT_DIRECTORY pExportDesc =
        GetNamedExportDirectory ( hModule , szImportMod );

      if ( NULL == pImportDesc )
      {
        // The requested module was not imported.
        continue; 
      }

      // Get the original thunk information for this DLL.  I cannot use
      //  the thunk information stored in the pImportDesc->FirstThunk
      //  because the that is the array that the loader has already
      //  bashed to fix up all the imports.  This pointer gives us acess
      //  to the function names.
      PIMAGE_THUNK_DATA pOrigThunk =
        MakePtr ( PIMAGE_THUNK_DATA       ,
                  hModule                 ,
                  pImportDesc->OriginalFirstThunk  ) ;
      // Get the array pointed to by the pImportDesc->FirstThunk.  This is
      //  where I will do the actual bash.
      PIMAGE_THUNK_DATA pRealThunk = MakePtr ( PIMAGE_THUNK_DATA       ,
                                               hModule                 ,
                                               pImportDesc->FirstThunk  );


      // Loop through and look for the one that matches the name.
      while ( NULL != pOrigThunk->u1.Function )
      {
        // Determines if we do the hook.
        BOOL bDoHook = FALSE ;

        // The hooklist iterator
        HOOKLIST::iterator i;

        // Only look at those that are imported by name, not ordinal.
        if (  IMAGE_ORDINAL_FLAG !=
              ( pOrigThunk->u1.Ordinal & IMAGE_ORDINAL_FLAG ))
        {
          // Look get the name of this imported function.
          PIMAGE_IMPORT_BY_NAME pByName ;

          pByName = MakePtr ( PIMAGE_IMPORT_BY_NAME    ,
                              hModule                  ,
                              pOrigThunk->u1.AddressOfData  ) ;

          // If the name starts with NULL, then just skip out now.
          if ( '\0' == pByName->Name[ 0 ] )
          {
            continue ;
          }

          // TODO TODO
          //  Might want to consider bsearch here.

          // See if the particular function name is in the import
          //  list.  It might be good to consider requiring the
          //  paHookArray to be in sorted order so bsearch could be
          //  used so the lookup will be faster.  However, the size of
          //  uiCount coming into this function should be rather
          //  small but it is called for each function imported by
          //  szImportMod.

          for ( i=hooklist.begin( ); 
                i !=hooklist.end() ; i++ )
          {
            if ( ( (*i)->szFunc[0] == pByName->Name[0] ) &&
                 ( 0 == strcmpi ( (*i)->szFunc, (char*)pByName->Name ) ) &&
                 ( 0 == strcmpi ( (*i)->szDll, (char*)szImportMod ) ) )
            {
              // If the proc is NULL, kick out, otherwise go
              //  ahead and hook it.
              if ( NULL != (*i)->pProc )
              {
                bDoHook = TRUE ;
              }
              break ;
            }
          }
        } 
        else if (  IMAGE_ORDINAL_FLAG ==
                   ( pOrigThunk->u1.Ordinal & IMAGE_ORDINAL_FLAG ))
        {
          DWORD ordinal = pOrigThunk->u1.Ordinal ^ IMAGE_ORDINAL_FLAG;

          //  See if the particular function ordinal name is in the 
          //  import list.

          for ( i=hooklist.begin( ); 
                i !=hooklist.end() ; i++ )
          {
            HOOKFUNCDESC *desc = *i;

            if ( ( (*i)->ordinal == ordinal ))
            {
              // The ordinal is to be hooked directly in this module
              if ( 0 == strcmpi ( (*i)->szDll, (char*)szImportMod ) )
              {
                // If the proc is NULL, kick out, otherwise go
                //  ahead and hook it.
                if ( NULL != (*i)->pProc )
                {
                  bDoHook = TRUE ;
                }
                break ;
              }
            } 
#ifdef NO_WAY
            else {
              // The ordinal might be forwarded somewhere else,
              // so search for forwards.
              HINSTANCE forwardModule;
              forwardModule = LoadLibraryEx(szImportMod, NULL, 
                                            DONT_RESOLVE_DLL_REFERENCES);
              SearchForForwardedFunction(forwardModule, ordinal);
            }
#endif
          }
        }
        if ( TRUE == bDoHook )
        {
          // I found it.  Now I need to change the protection to
          //  writable before I do the blast.  Note that I am now
          //  blasting into the real thunk area!
          MEMORY_BASIC_INFORMATION mbi_thunk ;

          VirtualQuery ( pRealThunk                          ,
                         &mbi_thunk                          ,
                         sizeof ( MEMORY_BASIC_INFORMATION )  ) ;

          VERIFY ( VirtualProtect ( mbi_thunk.BaseAddress ,
                                    mbi_thunk.RegionSize  ,
                                    PAGE_READWRITE        ,
                                    &mbi_thunk.Protect     ) ) ;

          // Do the actual hook saving the original address to
          //  return it.
          (*i)->origProc = (PROC)pRealThunk->u1.Function ;

          // We use the Ordinal member, because the Function member type
          //   changes from a DWORD to a PDWORD between SDKs and there's no
          //   easy way to determine which we should be using
          pRealThunk->u1.Ordinal = (DWORD)(*i)->pProc;

          DWORD dwOldProtect ;

                // Change the protection back to what it was before I
                //  blasted.
          VERIFY ( VirtualProtect ( mbi_thunk.BaseAddress ,
                                    mbi_thunk.RegionSize  ,
                                    mbi_thunk.Protect     ,
                                    &dwOldProtect          ) ) ;

          // Increment the total number hooked.
          *puiHooked += 1 ;
        }
      
        // Increment both tables.
        pOrigThunk++ ;
        pRealThunk++ ;
      }
    }
    // All OK, JumpMaster!
    SetLastError ( ERROR_SUCCESS ) ;
    return ( TRUE ) ;
}


/*----------------------------------------------------------------------
FUNCTION        :   GetNamedImportDescriptor
DISCUSSION      :
    Gets the import descriptor for the requested module.  If the module
is not imported in hModule, NULL is returned.
    This is a potential useful function in the future.
PARAMETERS      :
    hModule      - The module to hook in.
    szImportMod  - The module name to get the import descriptor for.
RETURNS         :
    NULL  - The module was not imported or hModule is invalid.
    !NULL - The import descriptor.
----------------------------------------------------------------------*/
static PIMAGE_IMPORT_DESCRIPTOR
                     GetNamedImportDescriptor ( HMODULE hModule     ,
                                                LPCSTR  szImportMod  )
{
    // Always check parameters.
    ASSERT ( NULL != szImportMod ) ;
    ASSERT ( NULL != hModule     ) ;
    if ( ( NULL == szImportMod ) || ( NULL == hModule ) )
    {
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    PIMAGE_DOS_HEADER pDOSHeader = (PIMAGE_DOS_HEADER)hModule ;

    // Is this the MZ header?
    if ( ( TRUE == IsBadReadPtr ( pDOSHeader                  ,
                                 sizeof ( IMAGE_DOS_HEADER )  ) ) ||
         ( IMAGE_DOS_SIGNATURE != pDOSHeader->e_magic           )   )
    {
        ASSERT ( FALSE ) ;
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    // Get the PE header.
    PIMAGE_NT_HEADERS pNTHeader = MakePtr ( PIMAGE_NT_HEADERS       ,
                                            pDOSHeader              ,
                                            pDOSHeader->e_lfanew     ) ;

    // Is this a real PE image?
    if ( ( TRUE == IsBadReadPtr ( pNTHeader ,
                                  sizeof ( IMAGE_NT_HEADERS ) ) ) ||
         ( IMAGE_NT_SIGNATURE != pNTHeader->Signature           )   )
    {
        ASSERT ( FALSE ) ;
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    // Get the pointer to the imports section.
    PIMAGE_IMPORT_DESCRIPTOR pImportDesc
     = MakePtr ( PIMAGE_IMPORT_DESCRIPTOR ,
                 pDOSHeader               ,
                 pNTHeader->OptionalHeader.
                         DataDirectory[ IMAGE_DIRECTORY_ENTRY_IMPORT ].
                                                      VirtualAddress ) ;

    // Does it have an imports section?
    // The pImportDesc == pDOSHeader check should never happen,
    // but it does- when explorer loads cfgmgr32.dll (Win2k RC2)
    if (( 0 == pImportDesc ) || 
	( (void *)pImportDesc == (void *)pDOSHeader ))
    {
        return ( NULL ) ;
    }
    // Loop through the import module descriptors looking for the
    //  module whose name matches szImportMod.
    while ( NULL != pImportDesc->Name )
    {
        PSTR szCurrMod = MakePtr ( PSTR              ,
                                   pDOSHeader        ,
                                   pImportDesc->Name  ) ;
        if ( 0 == stricmp ( szCurrMod , szImportMod ) )
        {
            // Found it.
            break ;
        }
        // Look at the next one.
        pImportDesc++ ;
    }

    // If the name is NULL, then the module is not imported.
    if ( NULL == pImportDesc->Name )
    {
        return ( NULL ) ;
    }

    // All OK, Jumpmaster!
    return ( pImportDesc ) ;

}


/*
 * Utility function to get the export section of the PE image
 */
static PIMAGE_EXPORT_DIRECTORY
                     GetNamedExportDirectory ( HMODULE hModule     ,
                                                LPCSTR  szExportMod  )
{
    // Always check parameters.
    ASSERT ( NULL != szExportMod ) ;
    ASSERT ( NULL != hModule     ) ;
    if ( ( NULL == szExportMod ) || ( NULL == hModule ) )
    {
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    PIMAGE_DOS_HEADER pDOSHeader = (PIMAGE_DOS_HEADER)hModule ;

    // Is this the MZ header?
    if ( ( TRUE == IsBadReadPtr ( pDOSHeader                  ,
                                 sizeof ( IMAGE_DOS_HEADER )  ) ) ||
         ( IMAGE_DOS_SIGNATURE != pDOSHeader->e_magic           )   )
    {
        ASSERT ( FALSE ) ;
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    // Get the PE header.
    PIMAGE_NT_HEADERS pNTHeader = MakePtr ( PIMAGE_NT_HEADERS       ,
                                            pDOSHeader              ,
                                            pDOSHeader->e_lfanew     ) ;

    // Is this a real PE image?
    if ( ( TRUE == IsBadReadPtr ( pNTHeader ,
                                  sizeof ( IMAGE_NT_HEADERS ) ) ) ||
         ( IMAGE_NT_SIGNATURE != pNTHeader->Signature           )   )
    {
        ASSERT ( FALSE ) ;
        SetLastErrorEx ( ERROR_INVALID_PARAMETER , SLE_ERROR ) ;
        return ( NULL ) ;
    }

    // Get the pointer to the exports section.
    PIMAGE_EXPORT_DIRECTORY pExportDesc
     = MakePtr ( PIMAGE_EXPORT_DIRECTORY ,
                 pDOSHeader               ,
                 pNTHeader->OptionalHeader.
                         DataDirectory[ IMAGE_DIRECTORY_ENTRY_EXPORT ].
                                                      VirtualAddress ) ;

    // Does it have an exports section?
    if ( (void *)hModule == (void *)pExportDesc )
    {
        return ( NULL ) ;
    }

    PSTR moduleName= MakePtr (PSTR, hModule, pExportDesc->Name);

    // If the name is NULL, then the module is not exported.
    if (( NULL == moduleName ) || (!strcmpi(moduleName, "ntdll.dll")))
    {
        return ( NULL ) ;
    }

    DWORD *name_table=MakePtr(DWORD*, hModule, pExportDesc->AddressOfNames);
    WORD *ordinal_table=MakePtr(WORD*, hModule, pExportDesc->AddressOfNameOrdinals);
    DWORD *function_table=MakePtr(DWORD*, hModule, pExportDesc->AddressOfFunctions);
    PSTR funcName=NULL;
    void *function;
    DWORD ordinal; 
    PSTR forwardName;
    DWORD exportMax=(DWORD)pExportDesc+(pNTHeader->OptionalHeader.
                                 DataDirectory[ IMAGE_DIRECTORY_ENTRY_EXPORT ].Size);
    DWORD baseOfCode=(DWORD)hModule+(pNTHeader->OptionalHeader.BaseOfCode);
    DWORD endOfCode=(DWORD)baseOfCode+(pNTHeader->OptionalHeader.SizeOfCode);

    for (int i = 0; i<min(pExportDesc->NumberOfFunctions, pExportDesc->NumberOfNames);
         i++)
    {
      funcName=MakePtr( PSTR, hModule, name_table[i]);
      ordinal=ordinal_table[i];

      function=MakePtr( void *, hModule, function_table[ordinal]);
      if (((function <= (void *)exportMax) && (function >= pExportDesc))||
          (function_table[ordinal] == NULL))
      {
        if (funcName)
          forwardName=(PSTR)function;
      } else
      {
        forwardName=NULL;
      }
    }
    return ( pExportDesc ) ;

}

