/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: hook.h,v 1.24 2001/02/23 19:02:11 stevek Exp $ 
 */

#ifndef __HOOK_H
#define __HOOK_H 1

#include <wr_conf.h>

#include <wss_wrapper.h>

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786) 

#include <map>
#include <string>
#include <vector>
#include <list>

#ifdef __cplusplus
extern "C" {
#endif
void EXPORT panic( const char *, ... );
#ifdef __cplusplus
}
#endif


typedef struct tag_HOOKCODEMAP
{
  tag_HOOKCODEMAP(wss_wrapper_instance_t *_wi,
                  wss_wrapper_dispatch_t _func=NULL,
                  pd_syscall_id_t _mapcode=0)
  {
    wi      = _wi;
    func    = _func;
    mapcode = _mapcode;
  }

  wss_wrapper_instance_t *wi;
  wss_wrapper_dispatch_t  func;
  pd_syscall_id_t         mapcode;
} HOOKCODEMAP, * LPHOOKCODEMAP ;

typedef std::list < LPHOOKCODEMAP > PREPOSTCODEMAPLIST;

typedef struct tag_HOOKFUNCDESC
{
  // The constructor
  tag_HOOKFUNCDESC(LPCSTR _szFunc=NULL, 
                   PROC _pProc=NULL, 
                   LPCSTR _szDll=NULL,
                   DWORD _ordinal=0,
                   PROC _origProc=NULL, 
                   HMODULE _hmodDll=NULL,
                   PREPOSTCODEMAPLIST _preFuncs=PREPOSTCODEMAPLIST(),
                   PREPOSTCODEMAPLIST _postFuncs=PREPOSTCODEMAPLIST(),
                   BOOL _usePre=NULL,
                   BOOL _usePost=NULL)
    {
      szFunc=_szFunc;
      pProc=_pProc;
      origProc=_origProc;
      szDll=_szDll;
      hmodDll=_hmodDll;
      ordinal=_ordinal;
      preFuncs=_preFuncs;
      postFuncs=_postFuncs;
      usePre=_usePre;
      usePost=_usePost;
    }
  
  // The name of the function to hook.
  LPCSTR szFunc   ;
  // The procedure to blast in.
  PROC   pProc    ;
  // The original procedure
  PROC   origProc ;
  // The name of the DLL where the function resides.
  LPCSTR szDll    ;
  // The handle to the module where the function resides.
  HMODULE hmodDll ;
  // The ordinal vaule of the function.
  DWORD ordinal;
  // The list of wrapper pre functions and code maps
  PREPOSTCODEMAPLIST preFuncs;
  // The list of wrapper pre functions and code maps
  PREPOSTCODEMAPLIST postFuncs;
  // should we bother calling the pre functions? 
  BOOL usePre;
  // should we bother calling the post functions? 
  BOOL usePost;

  void addPreFunc(wss_wrapper_instance_t *wi, wss_wrapper_dispatch_t pre_func,
                  pd_syscall_id_t mapcode)
    {wss_wrapper_instance_ref(wi);
     preFuncs.push_back(new HOOKCODEMAP(wi,pre_func,mapcode));}
  void addPostFunc(wss_wrapper_instance_t *wi, wss_wrapper_dispatch_t post_func,
                   pd_syscall_id_t mapcode)
    {wss_wrapper_instance_ref(wi);
     postFuncs.push_back(new HOOKCODEMAP(wi,post_func,mapcode));}

  PREPOSTCODEMAPLIST *getPreFuncList(void) {return &preFuncs;}
  PREPOSTCODEMAPLIST *getPostFuncList(void) {return &postFuncs;}
  LPCSTR              getDllName(void);
  HMODULE             getDllHandle(void);
  PROC                getProcAddr(void) {return pProc;}

  int callDispatchPre( int space, pd_syscall_id_t code,
                       void* uap[], void *retval,
                       struct wss_proc *wrp,
                       struct wss_wrapper_data *data,
                       int args_in_userspace);

  int callDispatchPost( int space, pd_syscall_id_t code,
                        void* uap[], void *retval,
                        struct wss_proc *wrp,
                        struct wss_wrapper_data *data,
                        int args_in_userspace);

} HOOKFUNCDESC , * LPHOOKFUNCDESC ;


/*
 * Here's a map type for holding a series of HOOKFUNCDESC structures
 */
typedef std::map < std::string, HOOKFUNCDESC > HOOKMAP;

typedef std::list < HOOKFUNCDESC *> HOOKLIST;


/*----------------------------------------------------------------------
FUNCTION        :   HookImportedFunctionsByName
DISCUSSION      :
    Hooks the specified functions imported into hModule by the module
indicated by szImportMod.  This function can be used to hook from one
to 'n' of the functions imported.
    The techniques used in the function are slightly different than
that shown by Matt Pietrek in his book, "Windows 95 System Programming
Secrets."  He uses the address of the function to hook as returned by
GetProcAddress.  Unfortunately, while this works in almost all cases, it
does not work when the program being hooked is running under a debugger
on Windows95 (an presumably, Windows98).  The problem is that
GetProcAddress under a debugger returns a "debug thunk," not the address
that is stored in the Import Address Table (IAT).
    This function gets around that by using the real thunk list in the
PE file, the one not bashed by the loader when the module is loaded and
fixed up, to find where the named import is located.  Once the named
import is found, then the original table is blasted to make the hook.
As the name implies, this function will only hook functions imported by
name.
PARAMETERS      :
    hModule     - The module where the imports will be hooked.
    szImportMod - The names of the modules whose functions will be
                  imported.
    uiCount     - The number of functions to hook.  This is the size of
                  the paHookArray and paOrigFuncs arrays.
    paHookArray - The array of function descriptors that list which
                  functions to hook.  At this point, the array does not
                  have to be in szFunc name order.  Also, if a
                  particular pProc is NULL, then that item will just be
                  skipped.  This makes it much easier for debugging.
    paOrigFuncs - The array of original addresses that were hooked.  If
                  a function was not hooked, then that item will be
                  NULL.
    puiHooked   - Returns the number of functions hooked out of
                  paHookArray.
RETURNS         :
    FALSE - There was a problem, check GetLastError.
    TRUE  - The function succeeded.  See the parameter discussion for
            the output parameters.
----------------------------------------------------------------------*/
BOOL HookImportedFunctionsByName ( HMODULE         hModule     ,
                                   const char**    szImportMod ,
                                   HOOKLIST        hooklist,
                                   LPUINT          puiHooked    ) ;


#endif /* !__HOOK_H */

