# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Kernel component makefile defaults
#
# $Id: component.mak,v 1.9 2001/04/30 20:17:52 stevek Exp $
#
# Makefile variables:
#
# [Required]
#	OBJS			: List of object files to generate from source
#
# [Optional]
#	CFLAGS			: Additional flags to pass to the C compiler
#				  on the command line
#	CXXFLAGS		: Additional flags to pass to the C++ compiler
#				  on the command line
#	
#	SUBDIR			: Sub-directories to process before building
#				  object files and libraries
#

include ${TOPDIR}/mak/common.mak

OBJMAK = pre
include ${TOPDIR}/mak/obj.mak.${MAKEINC}

LIBMAK = pre
include ${TOPDIR}/mak/component.mak.${MAKEINC}

CFLAGS		+= -DPD_IN_KERNEL -I ${TOPDIR}/include ${DEFINES}
CXXFLAGS	+= -DPD_IN_KERNEL -I ${TOPDIR}/include ${DEFINES}
CLEANFILES	+= .component.mak
DEPINC		+= -I ${TOPDIR}/include 

.PHONY: all doall clean install release component
.PHONY: postcompall postcompclean postcompinstall postcomprelease
.PHONY: makecompmak

all: doall

include ${TOPDIR}/mak/depend.mak

doall: ${PRETARGET} .component.mak ${OBJS} postcompall ${POSTTARGET}

makecompmak: .component.mak

.component.mak: Makefile
	@echo "--[ Generating component object list ]--"
	@${RM} .component.mak
	@touch .component.mak
	@for a in ${OBJS}; do \
	   echo "COMPOBJS += ${BASEDIR}/$$a" >> .component.mak; \
	done

#
# Clean target
#
clean: ${PRETARGETCLEAN} postcompclean
	${RM} ${OBJS} ${CLEANFILES}


#
# Suffix rules
#
.SUFFIXES: .c .i .s .cxx .cc ${OBJ} ${SHOBJ}

.c.i:
	${RM} $@
	${COMPILE.c} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.cxx.i:
	${RM} $@
	${COMPILE.cxx} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.cc.i:
	${RM} $@
	${COMPILE.cc} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.c.s:
	${RM} $@
	${COMPILE.c} ${ASMFLAGS} $< > $@ 

.cxx${OBJ}: .component.mak
	${COMPILE.cxx} ${STATIC_CXXFLAGS} ${INCLUDES} $<

.cc${OBJ}: .component.mak
	${COMPILE.cc} ${STATIC_CXXFLAGS} ${INCLUDES} $<

.c${OBJ}: .component.mak
	${COMPILE.c} ${STATIC_CXXFLAGS} ${INCLUDES} $<


SUBDIRPRFX	= postcomp
include ${TOPDIR}/mak/subdir.mak

#
# Installation targets
#
install: ${INSTALLTARGETS} postcompinstall

#
# Release targets
#
release: ${RELEASETARGETS} postcomprelease

LIBMAK = post
include ${TOPDIR}/mak/component.mak.${MAKEINC}

OBJMAK = post
include ${TOPDIR}/mak/obj.mak.${MAKEINC}

