#
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: java.mak,v 1.45 2001/04/09 22:30:07 stevek Exp $
#
# Makefile variables:
#
# [Required]
#	JAVASRCS	: List of java files to compile
#
# [Optional]
#
#	JAVACFLAGS	: Additional flags to pass to javac on the command
#			  line
#
#	CLASSPATH	: List of paths in which to search for imported
#			  classfiles (using the appropriate platform-specific
#			  path delimiter)
#
#	JAVA1SRCS	: Java(tm) sources which should only be compiled on
#			  with a Java 1 environment
#	JAVA2SRCS	: Java(tm) sources which should only be compiled on
#			  with a Java 2 environment
#
#	SUBDIR		: Sub-directories to process before Java(tm) sources
#
#	INSTALLCLASSES	: If set to 'yes', invocation of the 'install' target
#			  will copy all classfiles to INSTALLDIR
#	INSTALLDIR	: Path to directory where classfiles should be
#			  installed (defaults to CLASSDIR set by configure
#			  script)
#
#	PRETARGET	: Targets to invoke before compiling classfiles
#
#	PRETARGETCLEAN	: Targets to invoke before cleaning classfiles and
#			  other files set in CLEANFILES
#	CLEANFILES	: List of files to clean on invocation of 'clean'
#			  target
#
#	PRETARGETTEST	: Targets to invoke before test target
#
#	INSTALL_TARGET	: Dependency targets to invoke on install
#

include ${TOPDIR}/mak/common.mak

JAVAMAK = pre
include ${TOPDIR}/mak/java.mak.${MAKEINC}

JAVAP_INTERNAL_FLAGS += -s -p
JAVAH_JNI_FLAGS += -jni
JCLASSPATH = ${JAVACLASSESZIP}${CLASSESCFG}${PATHSEP}.${PATHSEP}${CLASSPATH}

JAVACFLAGS += -classpath "${JCLASSPATH}"
JAVAP_INTERNAL_FLAGS += -classpath "${JCLASSPATH}"
JAVAH_JNI_FLAGS += -classpath "${JCLASSPATH}"

JAVA_CLASSNAMES = ${JAVASRCS:.java=}
JAVA_CLASSES = ${JAVASRCS:.java=.class}

.PHONY: all clean native_code
.PHONY: javasuball javasubclean javasubinstall javasubrelease

all: javasuball ${PRETARGET} ${JAVA_CLASSES} ${OBJS} native_code

.SUFFIXES: .class .java

.java.class:
	${JAVAC} ${JAVACFLAGS} $<
	@if [ ! -z "${JAVAP}" ]; then \
		package=`awk '/^[ \t]*package[ \t]+[^ \t]+;/ { print substr($$2,0,length($$2)-1) "."; }' $<` ;\
		if ${JAVAP} ${JAVAP_INTERNAL_FLAGS} $${package}$* | \
			egrep "(public|private) native" >/dev/null; then \
			echo Constructing JNI header file for class $*; \
			if [ ! -d $*_jni ]; then \
				mkdir $*_jni; \
			fi; \
			${JAVAH} ${JAVAH_JNI_FLAGS} -o \
				$*_jni/$*_jni.h \
				$${package}$*; \
		fi; \
	fi

native_code:
	@for i in ${JAVA_CLASSNAMES}; do \
		if [ -d "$${i}_jni" ]; then \
			if ( \
				echo "===> ${DIRPRFX}$${i} [JNI Code]"; \
				cd $${i}_jni; \
				${MAKE} all DIRPRFX="${DIRPRFX}$${i} [JNI Code]:"); \
			then \
				echo "<=== ${DIRPRFX}$${i} [JNI Code]"; \
			else exit 1; fi; \
		fi; \
	done

clean: javasubclean ${PRETARGETCLEAN}
	${RM} ${JAVA_CLASSES}
	${RM} ${JAVA_INNER_CLASSES}
	${RM} a.out Errs errs mklog ${CLEANFILES} ${OBJS}
	@for i in ${JAVA_CLASSNAMES}; do \
		if [ -d "$${i}_jni" ]; then \
			if ( \
				echo "===> ${DIRPRFX}$${i} [JNI Code]"; \
				cd $${i}_jni; \
				${MAKE} clean DIRPRFX="${DIRPRFX}$${i} [JNI Code]:"); \
			then \
				echo "<=== ${DIRPRFX}$${i} [JNI Code]"; \
			else exit 1; fi; \
		fi \
	done

test: javasubtest ${PRETARGETTEST}
posttest: javasubposttest

install: ${INSTALL_TARGET} javasubinstall
	@if [ "${INSTALLCLASSES}" = "yes" ]; then \
	   INSTALL_CLASSES=`echo "${JAVASRCS}" | \
	                    sed -e 's/\([^ 	]*\)\.java /\1$$*.class \1.class /g' \
	                        -e 's/\([^ 	]*\)\.java$$/\1$$*.class \1.class/'`; \
	fi; \
	if [ ! -z "$${INSTALL_CLASSES}" ]; then \
	   if [ -z "${INSTALLDIR}" ]; then \
	      INSTALLDIR="${CLASSDIR}"; export INSTALLDIR; \
	   else \
	      INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	   fi; \
	   if [ ! -z "${INSTALLROOT}" ]; then \
	      INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	   fi; \
	   if [ ! -d "$${INSTALLDIR}" ]; then \
	      ${SHELL} -c "${TOPDIR}/mak/mkdirhier \"$${INSTALLDIR}\""; \
	   fi; \
	   for a in ""$${INSTALL_CLASSES}; do \
	      if [ -f $$a ]; then \
	         echo "${CP} $$a \"$${INSTALLDIR}\""; \
	         ${CP} $$a "$${INSTALLDIR}"; \
	      fi; \
	   done; \
	fi

release: javasubrelease

SUBDIRPRFX=javasub

include ${TOPDIR}/mak/subdir.mak

JAVAMAK = post
include ${TOPDIR}/mak/java.mak.${MAKEINC}

