# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Java(tm) package makefile rules
#
# $Id: javapkg.mak,v 1.40 2001/04/09 22:30:07 stevek Exp $
#
# Makefile variables:
#
# [REQUIRED]
#	PACKAGES	: List of package names which should be included
#			  in the JAR archive (Ex. com.foo com.foo.bar)
#	ARCHIVE		: Name of the JAR archive
#
# [OPTIONAL]
#	MANIFEST	: Name of the manifest file (if one exists)
#
#	JAVA1_MANIFEST	: Name of the manifest file to use only in a Java 1
#                         environment
#	JAVA2_MANIFEST	: Name of the manifest file to use only in a Java 2
#                         environment
#
#	SUBDIR		: Sub-directories to process before packages
#	SUBDIR_POST	: Sub-directories to process after packages
#
#	JARCONTENTS	: Additional files to include in the JAR archive
#	JARFLAGS	: Additional flags to pass on jar command line
#
#	JAVADOCDESTDIR	: Destination directory of JavaDoc documentation
#			  (defaults to 'javadoc')
#	JAVADOCFLAGS	: Additional flags to pass on javadoc command line
#
#	INSTALL_TARGET	: Dependency targets to invoke on install
#

include ${TOPDIR}/mak/common.mak

JAR_CONTENTS	+= ${PACKAGES:=/*class}

CLEANDIRS	+= META-INF

.PHONY: all clean install release javadoc SUBDIRS

all: ${ARCHIVE}

clean: ${PRETARGETCLEAN} Makefile
	@${MAKE} SUBDIRTARGET=$@ SUBDIRS
	${RM} ${ARCHIVE} ${CLEANFILES}

install: ${INSTALL_TARGET}
	@${MAKE} SUBDIRTARGET=$@ SUBDIRS
	@if [ -z "${INSTALLDIR}" ]; then \
	   INSTALLDIR="${CLASSDIR}"; export INSTALLDIR; \
	else \
	   INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -z "${INSTALLROOT}" ]; then \
	   INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -d "$${INSTALLDIR}" ]; then \
	   ${SHELL} -c "${TOPDIR}/mak/mkdirhier \"$${INSTALLDIR}\""; \
	fi; \
	echo "${CP} ${ARCHIVE} \"$${INSTALLDIR}\""; \
	${CP} ${ARCHIVE} "$${INSTALLDIR}"

release:
	@${MAKE} SUBDIRTARGET=$@ SUBDIRS

javadoc:
	@if [ -z "${JAVADOCDESTDIR}" ]; then \
	   JAVADOCDESTDIR="./javadoc"; \
	else \
	   JAVADOCDESTDIR="${JAVADOCDESTDIR}"; \
	fi; \
	if [ ! -d $${JAVADOCDESTDIR} ]; then \
	   ${TOPDIR}/mak/mkdirhier "$${JAVADOCDESTDIR}"; \
	fi; \
	echo "${JAVADOC} ${JAVADOCFLAGS} -sourcepath . -splitIndex -J-Xmx180m -d $${JAVADOCDESTDIR} ${PACKAGES}"; \
	${JAVADOC} ${JAVADOCFLAGS} -sourcepath . -J-Xmx180m \
	   -d $${JAVADOCDESTDIR} ${PACKAGES}

SUBDIRS:
	@if [ -z "${SUBDIRTARGET}" ]; then \
	   SUBDIRTAG=; \
	else \
	   SUBDIRTAG=" [${SUBDIRTARGET}]"; \
	fi; \
	export SUBDIRTAG;
	@for i in ""${SUBDIR}; do \
	   if [ ! -z "$$i" ]; then \
	      if ( \
	         if [ -z "${SUBDIRTARGET}" ]; then \
	            echo "===> ${DIRPRFX}$$i"; \
	         else \
	            echo "===> ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	         fi; \
	         cd $$i ; \
	         ${MAKE} DIRPRFX="${DIRPRFX}$${i}." ${SUBDIRTARGET}); \
	      then \
	         if [ -z "${SUBDIRTARGET}" ]; then \
	            echo "<=== ${DIRPRFX}$$i"; \
	         else \
	            echo "<=== ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	         fi; \
	      else exit 1; \
	      fi; \
	   fi; \
	done
	@for i in ${PACKAGES}; do \
	   if ( \
	      if [ -z "${SUBDIRTARGET}" ]; then \
	         echo "===> ${DIRPRFX}$$i"; \
	      else \
	         echo "===> ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	      fi; \
	      cd `echo $$i | sed 'y/./\//'` ; \
	      ${MAKE} DIRPRFX="${DIRPRFX}$${i}." ${SUBDIRTARGET}); \
	   then \
	      if [ -z "${SUBDIRTARGET}" ]; then \
	         echo "<=== ${DIRPRFX}$$i"; \
	      else \
	         echo "<=== ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	      fi; \
	   else exit 1; \
	   fi; \
	done
	@for i in ""${SUBDIR_POST}; do \
	   if [ ! -z "$$i" ]; then \
	      if ( \
	         if [ -z "${SUBDIRTARGET}" ]; then \
	            echo "===> ${DIRPRFX}$$i"; \
	         else \
	            echo "===> ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	         fi; \
	         cd $$i ; \
	         ${MAKE} DIRPRFX="${DIRPRFX}$${i}." ${SUBDIRTARGET}); \
	      then \
	         if [ -z "${SUBDIRTARGET}" ]; then \
	            echo "<=== ${DIRPRFX}$$i"; \
	         else \
	            echo "<=== ${DIRPRFX}$$i [${SUBDIRTARGET}]"; \
	         fi; \
	      else exit 1; \
	      fi; \
	   fi; \
	done

include ${TOPDIR}/mak/javapkg.mak.${MAKEINC}

