# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Library makefile defaults
#
# $Id: library.mak,v 1.66 2001/04/30 20:15:52 stevek Exp $
#
# Makefile variables:
#
# [Required]
#	LIBBASE			: Base name for the output library
#	OBJS			: List of object files to generate from source
#
# [Required for Shared Libraries]
#	SHLIB_MAJOR		: Major version number of shared library
#	SHLIB_MINOR		: Minor version number of shared library
#
# [Required for yes_copy_to_bin With Shared Libraries]
#	COPYBINDIR		: Path where to copy resultant shared library
#				  (Typically set by the configure script)
#
# [Optional]
#	CFLAGS			: Additional flags to pass to the C compiler
#				  on the command line
#	CXXFLAGS		: Additional flags to pass to the C++ compiler
#				  on the command line
#	
#	SUBDIR			: Sub-directories to process before building
#				  object files and libraries
#
#	LDFLAGS			: Additional flags to pass to the linker on
#				  the command line
#	LDADD			: Additional files to link against when
#				  constructing the library
#	LDLIBS			: Additional libraries to link against when
#				  constructing the library
#
#	NOLIBPREFIX		: If defined, the library prefix is not
#				  applied to output library name
#
#	BUILD_SHARED		: Flag to indicate whether or not to build the
#				  shared library (.so.<major>.<minor> on Unix,
#				  .dll on Windows NT/2000)
#
#				  Must be one of the following values:
#				     yes		- build the library
#				     no			- do not build the
#				                          library
#				     yes_copy_to_bin	- build the library and
#				                          copy it to COPYBINDIR
#	BUILD_SHARED_OBJS_ONLY	: If set to 'yes', only build the shared
#				  object files, do not generate a library
#
#	BUILD_STATIC		: Flag to indicate whether or not to build the
#				  static library
#
#				  Must be one of the following values:
#				     yes		- build the library
#				     no			- do not build the
#				                          library
#	BUILD_STATIC_OBJS_ONLY	: If set to 'yes', only build the static
#				  object files, do not generate a library
#
#	BUILD_PIC		: Flag to indicate whether or not to build the
#				  Position Independent Code (PIC) library
#				  (Note: Only has meaning on Unix platforms)
#
#				  Must be one of the following values:
#				     yes		- build the library
#				     no			- do not build the
#				                          library
#	BUILD_PIC_OBJS_ONLY	: If set to 'yes', only build the PIC object
#				  files, do not generate a library
#
# [Note: All the optional variables listed above also have platform-specific
#        versions available.  Check the default.mak file generated by the
#        bootstrap process of the build tree for specific namings.]
#
#	INSTALL_TARGET		: Dependency targets to be invoked on install
#	INSTALL_STATIC_TARGET	: Dependency targets to be invoked on static
#				  library installation
#	INSTALL_SHARED_TARGET	: Dependency targets to be invoked on shared
#				  library installation
#	INSTALL_PIC_TARGET	: Dependency targets to be invoked on PIC
#				  library installation
#

include ${TOPDIR}/mak/common.mak

OBJMAK = pre
include ${TOPDIR}/mak/obj.mak.${MAKEINC}

LIBMAK = pre
include ${TOPDIR}/mak/library.mak.${MAKEINC}

CFLAGS += -I ${TOPDIR}/include
CXXFLAGS += -I ${TOPDIR}/include

.PHONY: all clean

.PHONY: build_static_objs build_static_lib build_shared_objs build_shared_lib
.PHONY: build_pic_objs build_pic_lib
.PHONY: postliball postlibclean postlibinstall postlibrelease


all: ${PRETARGET} ${BUILDTARGETS} postliball ${POSTTARGET}

build_static_objs: ${OBJS}

build_static_lib: ${LIBRARY}

${LIBRARY}: ${OBJS} ${LIBDEPENDS}
	@echo "Building standard ${LIBBASE} library "
	@${RM} ${LIBRARY}
	${LIB_LINK} ${LIB_OUT} ${LIB_OBJS}
	${RANLIB} ${LIBRARY}


build_shared_objs: ${SHOBJS}

build_shared_lib: ${SHLIBRARY}

${SHLIBRARY}: ${SHOBJS} ${LIBDEPENDS}
	@echo "Building shared ${LIBBASE} library (version ${SHLIB_MAJOR}.${SHLIB_MINOR})"
	@${RM} ${SHLIBRARY}
	${SHLIB_LINK} ${SHLIB_OUT} ${SHLIB_OBJS}

copy_shared_lib_to_bin: build_shared_lib
	@echo Copying library to shared bin directory
	@if [ ! -d ${COPYBINDIR} ]; then \
		${SHELL} -c "${TOPDIR}/mak/mkdirhier \"${COPYBINDIR}\""; \
	fi 
	@${CP} ${SHLIBRARY} ${COPYBINDIR} 


build_pic_objs: ${SHOBJS}

build_pic_lib: ${PICLIBRARY}

${PICLIBRARY}: ${SHOBJS} ${LIBDEPENDS}
	@echo "Building PIC ${LIBBASE} library"
	@${RM} ${PICLIBRARY}
	${PICLIB_LINK} ${PICLIB_OUT} ${PICLIB_OBJS}
	${RANLIB} ${PICLIBRARY}


#
# Clean target
#
clean: ${PRETARGETCLEAN} postlibclean
	${RM} ${OBJS} ${CLEANFILES} ${LIBRARY}
	${RM} ${SHOBJS} ${SHLIBRARY}
	${RM} ${PICLIBRARY}
	${RM} ${COPYBINDIR}/${SHLIBRARY}

#
# Suffix rules
#
.SUFFIXES: .s .rc .res .cxx .c .cc .i

.rc.res:
	${RC} ${INCLUDES} $<

.c.i:
	${RM} $@
	${COMPILE.c} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.cxx.i:
	${RM} $@
	${COMPILE.cxx} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.cc.i:
	${RM} $@
	${COMPILE.cc} ${INCLUDES} ${CPPFLAGS} $< > $@ 

.c.s:
	${RM} $@
	${COMPILE.c} ${ASMFLAGS} $< > $@ 


#
# Subdir rules
#
SUBDIRPRFX	= postlib
include ${TOPDIR}/mak/subdir.mak

.PHONY: install install_headers
.PHONY: install_static_lib install_shared_lib install_pic_lib

#
# Installation targets
#
install: install_headers ${INSTALLTARGETS} ${INSTALL_TARGET} postlibinstall

install_headers:
	@if [ ! -z "${HEADERS}" ]; then \
	   if [ -z "${INSTALLDIR}" ]; then \
	      INSTALLDIR="${INCDIR}"; export INSTALLDIR; \
	   else \
	      INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	   fi; \
	   if [ ! -z "${INSTALLROOT}" ]; then \
	      INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	   fi; \
	   if [ ! -d "$${INSTALLDIR}" ]; then \
	      ${TOPDIR}/mak/mkdirhier "$${INSTALLDIR}"; \
	   fi; \
	   echo "${CP} ${HEADERS} \"$${INSTALLDIR}\""; \
	   ${CP} ${HEADERS} "$${INSTALLDIR}"; \
	fi

#   Static libraries
install_static_lib: ${INSTALL_STATIC_TARGET}
	@if [ -z "${INSTALLDIR}" ]; then \
	   INSTALLDIR="${LIBDIR}"; export INSTALLDIR; \
	else \
	   INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -z "${INSTALLROOT}" ]; then \
	   INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -d "$${INSTALLDIR}" ]; then \
	   ${TOPDIR}/mak/mkdirhier "$${INSTALLDIR}"; \
	fi; \
	echo "${CP} ${LIBRARY} \"$${INSTALLDIR}\""; \
	${CP} ${LIBRARY} "$${INSTALLDIR}"


#   Shared libraries
install_shared_lib: ${INSTALL_SHARED_TARGET}
	@if [ -z "${INSTALLDIR}" ]; then \
	   INSTALLDIR="${LIBDIR}"; export INSTALLDIR; \
	else \
	   INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -z "${INSTALLROOT}" ]; then \
	   INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -d "$${INSTALLDIR}" ]; then \
	   ${TOPDIR}/mak/mkdirhier "$${INSTALLDIR}"; \
	fi; \
	echo "${CP} ${SHLIBRARY} \"$${INSTALLDIR}\""; \
	${CP} ${SHLIBRARY} "$${INSTALLDIR}"


#    PIC libraries
install_pic_lib: ${INSTALL_PIC_TARGET}
	@if [ -z "${INSTALLDIR}" ]; then \
	   INSTALLDIR="${LIBDIR}"; export INSTALLDIR; \
	else \
	   INSTALLDIR="${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -z "${INSTALLROOT}" ]; then \
	   INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -d "$${INSTALLDIR}" ]; then \
	   ${TOPDIR}/mak/mkdirhier "$${INSTALLDIR}"; \
	fi; \
	echo "${CP} ${PICLIBRARY} \"$${INSTALLDIR}\""; \
	${CP} ${PICLIBRARY} "$${INSTALLDIR}"


#
# Release targets
#
release: postlibrelease

LIBMAK = post
include ${TOPDIR}/mak/library.mak.${MAKEINC}

OBJMAK = post
include ${TOPDIR}/mak/obj.mak.${MAKEINC}

