# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Library makefile defaults for GNU make
#
# $Id: library.mak.GNU,v 1.17 2001/04/05 18:40:50 stevek Exp $
#
# Makefile variables:
#
# [Optional]
#	DEFAULT_BUILD_STATIC	: Defaults for building static libraries
#	DEFAULT_BUILD_SHARED	: Defaults for building shared libraries
#	DEFAULT_BUILD_PIC	: Defaults for building PIC libraries
#

ifeq "${LIBMAK}" "pre"

ifeq (${strip ${LIBBASE}},)
${error LIBBASE must be set to the base name for the output library}
endif

ifeq (${strip ${OBJS}},)
${error OBJS must be set to the objects which should be included in the library}
endif

ifeq (${strip ${SHLIB_MAJOR}},)
SHLIB_MAJOR = 1
endif
ifeq (${strip ${SHLIB_MINOR}},)
SHLIB_MINOR = 0
endif


LIBRARY         = ${LIBPRFX}${LIBBASE}${LIBEXT}
PICLIBRARY      = ${LIBPRFX}${LIBBASE}${PICLIBEXT}

ifeq (${strip ${NOLIBPREFIX}},)
SHLIBRARY       = ${LIBPRFX}${LIBBASE}${SHLIBEXT}
else
SHLIBRARY       = ${LIBBASE}${SHLIBEXT}
endif


#
# Check OS-specific build parameters
#
ifdef OSTARGET

# Static libraries
ifneq (${${OSTARGET}_BUILD_STATIC},)
ifeq "${${OSTARGET}_BUILD_STATIC}" "yes"
DO_BUILD_STATIC = yes
else
ifeq "${${OSTARGET}_BUILD_STATIC}" "no"
DO_BUILD_STATIC = no
else
${error ${OSTARGET}_BUILD_STATIC must be set to "yes", "no", or left undefined}
endif
endif
endif
ifneq (${DO_BUILD_STATIC},)
ifeq "${DO_BUILD_STATIC}" "yes"
ifneq (${${OSTARGET}_BUILD_STATIC_OBJS_ONLY},)
ifeq "${${OSTARGET}_BUILD_STATIC_OBJS_ONLY}" "yes"
DO_BUILD_STATIC_OBJS_ONLY = yes
else
ifeq "${${OSTARGET}_BUILD_STATIC_OBJS_ONLY}" "no"
DO_BUILD_STATIC_OBJS_ONLY = no
else
${error ${OSTARGET}_BUILD_STATIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif

# Shared libraries
ifneq (${${OSTARGET}_BUILD_SHARED},)
ifeq "${${OSTARGET}_BUILD_SHARED}" "yes"
DO_BUILD_SHARED = yes
else
ifeq "${${OSTARGET}_BUILD_SHARED}" "no"
DO_BUILD_SHARED = no
else
${error ${OSTARGET}_BUILD_SHARED must be set to "yes", "no", or left undefined}
endif
endif
endif
ifneq (${DO_BUILD_SHARED},)
ifeq "${DO_BUILD_SHARED}" "yes"
ifneq (${${OSTARGET}_BUILD_SHARED_OBJS_ONLY},)
ifeq "${${OSTARGET}_BUILD_SHARED_OBJS_ONLY}" "yes"
DO_BUILD_SHARED_OBJS_ONLY = yes
else
ifeq "${${OSTARGET}_BUILD_SHARED_OBJS_ONLY}" "no"
DO_BUILD_SHARED_OBJS_ONLY = no
else
${error ${OSTARGET}_BUILD_SHARED_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif

# PIC libraries
ifneq (${${ostarget}_build_shared},)
ifeq "${${OSTARGET}_BUILD_PIC}" "yes"
DO_BUILD_PIC = yes
else
ifeq "${${OSTARGET}_BUILD_PIC}" "no"
DO_BUILD_PIC = no
else
${error ${OSTARGET}_BUILD_PIC must be set to "yes", "no", or left undefined}
endif
endif
endif
ifneq (${DO_BUILD_PIC},)
ifeq "${DO_BUILD_PIC}" "yes"
ifneq (${${OSTARGET}_BUILD_PIC_OBJS_ONLY},)
ifeq "${${OSTARGET}_BUILD_PIC_OBJS_ONLY}" "yes"
DO_BUILD_PIC_OBJS_ONLY = yes
else
ifeq "${${OSTARGET}_BUILD_PIC_OBJS_ONLY}" "no"
DO_BUILD_PIC_OBJS_ONLY = no
else
${error ${OSTARGET}_BUILD_PIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif

endif


#
# Check platform-specific build parameters
#
ifdef PLATFORMTARGET

# Static libraries
ifndef DO_BUILD_STATIC
ifneq (${${PLATFORMTARGET}_BUILD_STATIC},)
ifeq "${${PLATFORMTARGET}_BUILD_STATIC}" "yes"
DO_BUILD_STATIC = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_STATIC}" "no"
DO_BUILD_STATIC = no
else
${error ${PLATFORMTARGET}_BUILD_STATIC must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_STATIC
ifeq "${DO_BUILD_STATIC" "yes"
ifeq (${DO_BUILD_STATIC_OBJS_ONLY},)
ifneq (${${PLATFORMTARGET}_BUILD_STATIC_OBJS_ONLY},)
ifeq "${${PLATFORMTARGET}_BUILD_STATIC_OBJS_ONLY}" "yes"
DO_BUILD_STATIC_OBJS_ONLY = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_STATIC_OBJS_ONLY}" "no"
DO_BUILD_STATIC_OBJS_ONLY = no
else
${error ${PLATFORMTARGET}_BUILD_STATIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif

# Shared libraries
ifndef DO_BUILD_SHARED
ifneq (${${PLATFORMTARGET}_BUILD_SHARED},)
ifeq "${${PLATFORMTARGET}_BUILD_SHARED}" "yes"
DO_BUILD_SHARED = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_SHARED}" "no"
DO_BUILD_SHARED = no
else
${error ${PLATFORMTARGET}_BUILD_SHARED must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_SHARED
ifeq "${DO_BUILD_SHARED}" "yes"
ifeq (${DO_BUILD_SHARED_OBJS_ONLY},)
ifneq (${${PLATFORMTARGET}_BUILD_SHARED_OBJS_ONLY},)
ifeq "${${PLATFORMTARGET}_BUILD_SHARED_OBJS_ONLY}" "yes"
DO_BUILD_SHARED_OBJS_ONLY = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_SHARED_OBJS_ONLY}" "no"
DO_BUILD_SHARED_OBJS_ONLY = no
else
${error ${PLATFORMTARGET}_BUILD_SHARED_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif

# PIC libraries
ifndef DO_BUILD_PIC
ifneq (${${PLATFORMTARGET}_BUILD_PIC},)
ifeq "${${PLATFORMTARGET}_BUILD_PIC}" "yes"
DO_BUILD_PIC = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_PIC}" "no"
DO_BUILD_PIC = no
else
${error ${PLATFORMTARGET}_BUILD_PIC must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_PIC
ifeq "${DO_BUILD_PIC}" "yes"
ifeq (${DO_BUILD_PIC_OBJS_ONLY},)
ifneq (${${PLATFORMTARGET}_BUILD_PIC_OBJS_ONLY},)
ifeq "${${PLATFORMTARGET}_BUILD_PIC_OBJS_ONLY}" "yes"
DO_BUILD_PIC_OBJS_ONLY = yes
else
ifeq "${${PLATFORMTARGET}_BUILD_PIC_OBJS_ONLY}" "no"
DO_BUILD_PIC_OBJS_ONLY = no
else
${error ${PLATFORMTARGET}_BUILD_PIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif

endif


#
# Check generic build parameters
#

# Static libraries
ifndef DO_BUILD_STATIC
ifneq (${BUILD_STATIC},)
ifeq "${BUILD_STATIC}" "yes"
DO_BUILD_STATIC = yes
else
ifeq "${BUILD_STATIC}" "no"
DO_BUILD_STATIC = no
else
${error BUILD_STATIC must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_STATIC
ifeq "${DO_BUILD_STATIC}" "yes"
ifeq (${DO_BUILD_STATIC_OBJS_ONLY},)
ifneq (${BUILD_STATIC_OBJS_ONLY},)
ifeq "${BUILD_STATIC_OBJS_ONLY}" "yes"
DO_BUILD_STATIC_OBJS_ONLY = yes
else
ifeq "${BUILD_STATIC_OBJS_ONLY}" "no"
DO_BUILD_STATIC_OBJS_ONLY = no
else
${error BUILD_STATIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif

# Shared libraries
ifndef DO_BUILD_SHARED
ifneq (${BUILD_SHARED},)
ifeq "${BUILD_SHARED}" "yes"
DO_BUILD_SHARED = yes
else
ifeq "${BUILD_SHARED}" "no"
DO_BUILD_SHARED = no
else
${error BUILD_SHARED must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_SHARED
ifeq "${DO_BUILD_SHARED}" "yes"
ifeq (${DO_BUILD_SHARED_OBJS_ONLY},)
ifneq (${BUILD_SHARED_OBJS_ONLY},)
ifeq "${BUILD_SHARED_OBJS_ONLY}" "yes"
DO_BUILD_SHARED_OBJS_ONLY = yes
else
ifeq "${BUILD_SHARED_OBJS_ONLY}" "no"
DO_BUILD_SHARED_OBJS_ONLY = no
else
${error BUILD_SHARED_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif

# PIC libraries
ifndef DO_BUILD_PIC
ifneq (${BUILD_PIC},)
ifeq "${BUILD_PIC}" "yes"
DO_BUILD_PIC = yes
else
ifeq "${BUILD_PIC}" "no"
DO_BUILD_PIC = no
else
${error BUILD_PIC must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
ifdef DO_BUILD_PIC
ifeq "${DO_BUILD_PIC}" "yes"
ifeq (${DO_BUILD_PIC_OBJS_ONLY},)
ifneq (${BUILD_PIC_OBJS_ONLY},)
ifeq "${BUILD_PIC_OBJS_ONLY}" "yes"
DO_BUILD_PIC_OBJS_ONLY = yes
else
ifeq "${BUILD_PIC_OBJS_ONLY}" "no"
DO_BUILD_PIC_OBJS_ONLY = no
else
${error BUILD_PIC_OBJS_ONLY must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif
endif


#
# Add default parameters as necessary
#
ifeq (${DO_BUILD_STATIC},)
ifdef DEFAULT_BUILD_STATIC
DO_BUILD_STATIC = ${DEFAULT_BUILD_STATIC}
else
DO_BUILD_STATIC = yes
endif
endif
ifeq (${DO_BUILD_SHARED},)
ifdef DEFAULT_BUILD_SHARED
DO_BUILD_SHARED = ${DEFAULT_BUILD_SHARED}
else
DO_BUILD_SHARED = no
endif
endif
ifeq (${DO_BUILD_PIC},)
ifdef DEFAULT_BUILD_PIC
DO_BUILD_PIC = ${DEFAULT_BUILD_PIC}
else
DO_BUILD_PIC = no
endif
endif

ifndef DO_BUILD_STATIC_OBJS_ONLY
DO_BUILD_STATIC_OBJS_ONLY=no
endif
ifndef DO_BUILD_SHARED_OBJS_ONLY
DO_BUILD_SHARED_OBJS_ONLY=no
endif
ifndef DO_BUILD_PIC_OBJS_ONLY
DO_BUILD_PIC_OBJS_ONLY=no
endif


#
# Determine copy shared to bin
#
ifeq "${DO_BUILD_SHARED}" "yes"

ifeq "${DO_BUILD_SHARED_OBJS_ONLY}" "no"

ifneq (${${OSTARGET}_COPY_SHARED_TO_BIN},)
ifeq "${${OSTARGET}_COPY_SHARED_TO_BIN}" "yes"
DO_COPY_SHARED_TO_BIN = yes
else
ifeq "${${OSTARGET}_COPY_SHARED_TO_BIN}" "no"
DO_COPY_SHARED_TO_BIN = no
else
${error ${OSTARGET}_COPY_SHARED_TO_BIN must be set to "yes", "no", or left undefined}
endif
endif

else

ifneq (${${PLATFORMTARGET}_COPY_SHARED_TO_BIN},)
ifeq "${${PLATFORMTARGET}_COPY_SHARED_TO_BIN}" "yes"
DO_COPY_SHARED_TO_BIN = yes
else
ifeq "${${PLATFORMTARGET}_COPY_SHARED_TO_BIN}" "no"
DO_COPY_SHARED_TO_BIN = no
else
${error ${PLATFORMTARGET}_COPY_SHARED_TO_BIN must be set to "yes", "no", or left undefined}
endif
endif

else

ifneq (${COPY_SHARED_TO_BIN},)
ifeq "${COPY_SHARED_TO_BIN}" "yes"
DO_COPY_SHARED_TO_BIN = yes
else
ifeq "${COPY_SHARED_TO_BIN}" "no"
DO_COPY_SHARED_TO_BIN = no
else
${error COPY_SHARED_TO_BIN must be set to "yes", "no", or left undefined}
endif
endif
endif
endif
endif

endif
endif



#
# Add targets based upon determined parameters
#
ifeq "${DO_BUILD_STATIC}" "yes"
BUILDTARGETS += build_static_objs
ifeq "${DO_BUILD_STATIC_OBJS_ONLY}" "no"
BUILDTARGETS += build_static_lib
INSTALLTARGETS += install_static_lib
endif
endif

ifeq "${DO_BUILD_SHARED}" "yes"
BUILDTARGETS += build_shared_objs
ifeq "${DO_BUILD_SHARED_OBJS_ONLY}" "no"
BUILDTARGETS += build_shared_lib
INSTALLTARGETS += install_shared_lib
ifeq "${DO_COPY_SHARED_TO_BIN}" "yes"
BUILDTARGETS += copy_shared_lib_to_bin
endif
endif
endif

ifeq "${DO_BUILD_PIC}" "yes"
BUILDTARGETS += build_pic_objs
ifeq "${DO_BUILD_PIC_OBJS_ONLY}" "no"
BUILDTARGETS += build_pic_lib
INSTALLTARGETS += install_pic_lib
endif
endif

endif

ifeq "${LIBMAK}" "post"

ifneq (${COPYTARGET},)
ifneq (${COPYTARGETPRFX},)
DO_COPYTARGET=${COPYTARGET}/${COPYTARGETPRFX}
else
DO_COPYTARGET=${COPYTARGET}/
endif
else
DO_COPYTARGET=
endif

.SUFFIXES: .cxx .c .cc .s .S ${OBJ} ${SHOBJ}

.cxx${OBJ}:
	${COMPILE.cxx} ${STATIC_CXXFLAGS} ${INCLUDES} $<
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.cc${OBJ}:
	${COMPILE.cc} ${STATIC_CXXFLAGS} ${INCLUDES} $<
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.s${OBJ}:
	${COMPILE.s} ${STATIC_CFLAGS} ${INCLUDES} $< -o $@
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.S${OBJ}:
	${COMPILE.S} ${STATIC_CFLAGS} ${INCLUDES} $< -o $@
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.c${OBJ}:
	${COMPILE.c} ${STATIC_CFLAGS} ${INCLUDES} $<
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.cxx${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.cxx} ${SHLIB_CXXFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.cxx} ${SHLIB_CXXFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.cc${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.cc} ${SHLIB_CXXFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.cc} ${SHLIB_CXXFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.cpp${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.cpp} ${SHLIB_CXXFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.cpp} ${SHLIB_CXXFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.s${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.s} ${SHLIB_CFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.s} ${SHLIB_CFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.S${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.S} ${SHLIB_CFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.S} ${SHLIB_CFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

.c${SHOBJ}:
ifeq "${BUILDPLATFORM}" "Windows_NT"
	${COMPILE.c} ${SHLIB_CFLAGS} ${INCLUDES} $< /Fo$@
else
	@${RM} $@
	${COMPILE.c} ${SHLIB_CFLAGS} ${PICFLAG} -DPIC -c $< -o tmp${SHOBJ}
	@${LD} -o $@ -r tmp${SHOBJ}
	@${RM} tmp${SHOBJ}
endif
ifneq (${DO_COPYTARGET},)
	${CP} $@ "${DO_COPYTARGET}$@"
endif

endif

