# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Kernel module makefile rules 
#
# $Id: module.mak,v 1.52 2001/04/09 22:30:08 stevek Exp $
#
# Makefile variables:
#
# [Required]
#	KMOD		: Name of the kernel module (minus any applicable
#			  extensions required by the platform, they will be
#			  added where necessary)
#	OBJS		: Object files to build from source and link into
#			  a kernel module
#
# [Optional]
#	KMODDEPENDS	: Names of modules depended on by module being built
#
#	SUBDIR		: Sub-directories to process before building
#			  kernel module
#
#	CFLAGS		: Additional flags to pass to the C compiler on the
#			  command line
#
#	KMOD_CATEGORY	: Category to file the kernel module under
#

include ${TOPDIR}/mak/common.mak

MODULEMAK = pre
include ${TOPDIR}/mak/module.mak.${MAKEINC}

KMOD_TARGET=${KMOD}${KMOD_EXT}

CFLAGS	+= -I ${TOPDIR}/include
CFLAGS	+= -DPD_IN_KERNEL=1
CFLAGS	+= -DKMOD_DEPENDS="\"${KMODDEPENDS}\""

FREEBSD_CFLAGS	+= -fformat-extensions

.PHONY: all clean
.PHONY: sysdepall sysdepclean sysdepinstall sysdeprelease

all: sysdepall ${PRETARGET} ${KMOD_TARGET} ${POSTTARGET}

clean: sysdepclean ${PRETARGETCLEAN}
	rm -f ${CLEANFILES}
	rm -f ${KMOD_TARGET}


${KMOD_TARGET}: ${KMODOBJS} ${KMODBUILDDEPS}
	$(PREGENERATE) $(KMODOBJS) $(KMOD_PREGENOBJS)
	$(BUILDSETS)
	$(KMOD_LD) $(KMOD_LDFLAGS) $(LDFLAGS) -o tmp.o $(KMOD_LDLIBS_PRE) $(KMODOBJS) ${KMODDEPLINKS} $(KMOD_LDLIBS_POST) ${KMODDEPLINKSOLD} ${KMOD_LD_POST}
	@if [ ! -z "${EXPORT_SYMS}" ]; then \
		rm -f symb.tmp; \
		for i in "" ${EXPORT_SYMS}; do echo $$i >> symb.tmp; done; \
		symorder -c symb.tmp tmp.o; \
		rm -f symb.tmp; \
	fi
	$(KMOD_MV) tmp.o $@ ${KMOD_MV_POST}


include ${TOPDIR}/mak/obj.mak


SUBDIRPRFX	= sysdep

depend: sysdepdepend

install: sysdepinstall
	@if [ ! "${BUILDPLATFORM}" = "FreeBSD" ]; then \
	   KMOD_CATEGORY=${KMOD_CATEGORY}; \
	fi; \
	if [ -z "${INSTALLDIR}" ]; then \
	   INSTALLDIR="${KMODDIR}/$${KMOD_CATEGORY}"; export INSTALLDIR; \
	else \
	   INSTALLDIR="${INSTALLDIR}/$${KMOD_CATEGORY}"; export INSTALLDIR; \
	fi; \
	if [ ! -z "${INSTALLROOT}" ]; then \
	   INSTALLDIR="${INSTALLROOT}/$${INSTALLDIR}"; export INSTALLDIR; \
	fi; \
	if [ ! -d "$${INSTALLDIR}" ]; then \
	   ${SHELL} -c "${TOPDIR}/mak/mkdirhier \"$${INSTALLDIR}\""; \
	fi; \
	echo "${CP} ${KMOD_TARGET} \"$${INSTALLDIR}\""; \
	${CP} ${KMOD_TARGET} "$${INSTALLDIR}"

release: sysdeprelease

include ${TOPDIR}/mak/subdir.mak

MODULEMAK = post
include ${TOPDIR}/mak/module.mak.${MAKEINC}

