# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Kernel module makefile rules for GNU Make
#
# $Id: module.mak.GNU,v 1.22 2001/05/02 22:41:00 stevek Exp $
#
# Makefile variables:
#

ifeq "${MODULEMAK}" "pre"

BROKEN_MAKE=$(shell echo ${MAKE_VERSION} | awk '{ if($$1<3.79) print "yes" }')

ifeq (${shell test -r module.def && echo "yes"},)

# Hand-written module

ifeq (${strip ${KMOD}},)
${error KMOD must be set to the base name for the kernel module}
endif

else

# Automatically generated module

ifneq (${strip ${KMOD}},)
${error Usage of both KMOD and module.def is prohibited}
endif

ifeq (${strip ${COMPONENTDIRS}},)
${error COMPONENTDIRS must be set to the list of directories containing component object to be linked into the kernel module}
endif

MAKECOMPOBJ := ${foreach dir,${COMPONENTDIRS},${shell if [ -d ${TOPDIR}/${dir} ]; then cd ${TOPDIR}/${dir}; make .component.mak; fi}}

INCCOMPMAK := ${foreach dir,${COMPONENTDIRS},${wildcard ${TOPDIR}/${dir}/.component.mak}}
ifneq (${strip ${INCCOMPMAK}},)
include ${INCCOMPMAK}
endif

ifneq (${COMPOBJS},)
BUILDOBJS	+= ${COMPOBJS:%=${TOPDIR}/%}
endif

ifeq (${shell test -r .module.mak && echo "yes"},yes)
include .module.mak
else
KMOD_MV_POST	+=; ${MV} $@ `cat .module.mak | cut -d= -f2`${KMOD_EXT}

KMOD		= _dummy
endif

BUILDOBJS	+= _gsw_module${OBJ} _gsw_compset${OBJ}
CLEANFILES	+= _gsw_compset.c _gsw_compset${OBJ}
CLEANFILES	+= _gsw_module.c _gsw_module${OBJ} .module.mak

ifeq "${PLATFORMTARGET}" "UNIX"
CFLAGS		+= -O2
endif
endif

BUILDOBJS	+= ${OBJS}

KMODOBJS	= ${BUILDOBJS}
CLEANFILES	+= ${OBJS}

endif

ifeq "${MODULEMAK}" "post"

ifneq (${COMPOBJS},)
${COMPOBJS:%=${TOPDIR}/%}:
	(cd $(dir $@); make $(notdir $@))

endif

ifeq (${shell test -r module.def && echo "yes"},yes)
_gsw_compset.c: ${INCCOMPMAK}
	@echo "--[ Constructing component set source ]--"
	@env PERLLIB=${PERLLIB} ${PERL} -- ${TOPDIR}/mak/makecompset `cat .module.mak | cut -d= -f2` ${TOPDIR} ${COMPONENTDIRS}

_gsw_module.c .module.mak: module.def ${INCCOMPMAK}
	@echo "--[ Constructing module source from module.def ]--"
	@env PERLLIB=${PERLLIB} ${PERL} -- ${TOPDIR}/mak/makemod ${TOPDIR}/mak/${OSDIRECTORY}.modskel
endif

endif

