# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# Build Files for Toolkit
#
# Subdirectory traversal makefile rules
#
# $Id: subdir.mak,v 1.40 2001/04/05 23:49:45 stevek Exp $
#
# Makefile variables:
#
# [Required]
#	SUBDIR		: Sub-directories to traverse first
#
# [Optional]
#	SUBDIR_POST	: Sub-directories to traverse last
#
#	SUBDIRPRFX	: Prefix for subdirectory rules (allows this set of
#			  makefile rules to be included and used by other
#			  makefile rules without namespace collisions)
#

include ${TOPDIR}/mak/common.mak

SUBDIR += ${CONFIG_SUBDIR} ${SUBDIR_POST}

.PHONY: ${SUBDIRPRFX}all ${SUBDIRPRFX}clean
.PHONY: ${SUBDIRPRFX}test ${SUBDIRPRFX}posttest
.PHONY: ${SUBDIRPRFX}install ${SUBDIRPRFX}release

${SUBDIRPRFX}all: ${PRETARGET} Makefile
	@if (echo "${SUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${SUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i"; \
	        if [ -d $$i ]; then \
	          cd $$i && ${MAKE} all DIRPRFX=${DIRPRFX}$$i/; \
	        else \
	          echo "Missing or invalid directory $$i"; \
	          exit 1; \
	        fi); \
	    then \
	      echo "<=== ${DIRPRFX}$$i"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}clean: ${PRETARGETCLEAN} Makefile
	${RM} ${CLEANFILES}
	@if (echo "${SUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${SUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [clean]"; \
	        cd $$i && ${MAKE} clean DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [clean]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}depend: ${PRETARGETCLEAN} Makefile
	@if (echo "${SUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${SUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [depend]"; \
	        cd $$i && ${MAKE} depend DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [depend]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}test: ${PRETARGETTEST} Makefile
	@if (echo "${TESTSUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${TESTSUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [test]"; \
	        cd $$i && ${MAKE} test DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [test]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}posttest: ${PRETARGETTEST} Makefile
	@if (echo "${TESTSUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${TESTSUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [post test]"; \
	        cd $$i && ${MAKE} posttest DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [post test]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}install: ${PRETARGETINSTALL}
	@if (echo "${INSTALLSUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${INSTALLSUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [install]"; \
	        cd $$i && ${MAKE} install DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [install]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

${SUBDIRPRFX}release: ${PRETARGETRELEASE}
	@if (echo "${RELEASESUBDIR}" | egrep -v "^[ 	]*$$" > ${NULLDEV}); then \
	  for i in `echo ""${RELEASESUBDIR} | awk '{print $$0}'`; do \
	    if (echo "===> ${DIRPRFX}$$i [release]"; \
	        cd $$i && ${MAKE} release DIRPRFX=${DIRPRFX}$$i/); \
	    then \
	      echo "<=== ${DIRPRFX}$$i [release]"; \
            else \
              exit 1; \
            fi; \
	  done ; \
	fi

