/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: hook1.c,v 1.8 2001/02/23 19:02:13 stevek Exp $
 */

#define DRIVER_SOURCE
#include <ntddk.h>
#include <undocnt.h>
#include <ntdllconst.h>

#include "hook1.h"

typedef NTSTATUS (*NTPROTECTVIRTUALMEMORY)(
	HANDLE hProcess,
	PVOID *BaseAddress,
	PULONG RegionSize,
	ULONG Protect,
	PULONG OldProtect
);

typedef NTSTATUS (*NTALLOCATEVIRTUALMEMORY)(
	HANDLE hProcess,
	PVOID *PreferredBaseAddress,
	ULONG nLowerZeroBits,
	PULONG SizeRequestedAllocated,
	ULONG AllocationType,
	ULONG ProtectionAttributes
);

typedef NTSTATUS (*NTFREEVIRTUALMEMORY)(
	HANDLE hProcess,
	PVOID StartingAddress,
	PULONG SizeRequestedReleased,
	ULONG ReleaseType
);


#define SYSTEMSERVICE(index) \
	KeServiceDescriptorTable.ServiceTableBase[index]

NTPROTECTVIRTUALMEMORY OldNtProtectVirtualMemory;
NTALLOCATEVIRTUALMEMORY OldNtAllocateVirtualMemory;
NTFREEVIRTUALMEMORY OldNtFreeVirtualMemory;

/* Replacement system calls */
NTSTATUS
MyNtProtectVirtualMemory(
	HANDLE hProcess,
	PVOID *BaseAddress,
	PULONG RegionSize,
	ULONG Protect,
	PULONG OldProtect)
{
	int rc;

	rc = ((NTPROTECTVIRTUALMEMORY)(OldNtProtectVirtualMemory))(
		hProcess,
		BaseAddress,
		RegionSize,
		Protect,
		OldProtect);

	return rc;
}

NTSTATUS
MyNtAllocateVirtualMemory(
	HANDLE hProcess,
	PVOID *PreferredBaseAddress,
	ULONG nLowerZeroBits,
	PULONG SizeRequestedAllocated,
	ULONG AllocationType,
	ULONG ProtectionAttributes)
{
	int rc;

	rc = ((NTALLOCATEVIRTUALMEMORY)(OldNtAllocateVirtualMemory))(
		hProcess,
		PreferredBaseAddress,
		nLowerZeroBits,
		SizeRequestedAllocated,
		AllocationType,
		ProtectionAttributes);

	return rc;
}

NTSTATUS
MyNtFreeVirtualMemory(
	HANDLE hProcess,
	PVOID StartingAddress,
	PULONG SizeRequestedReleased,
	ULONG ReleaseType)
{
	int rc;

	rc = ((NTFREEVIRTUALMEMORY)(OldNtFreeVirtualMemory))(
		hProcess,
		StartingAddress,
		SizeRequestedReleased,
		ReleaseType);

	return rc;
}

VOID
MyCreateProcessNotifyRoutine( HANDLE hParent, HANDLE hChild, BOOLEAN bCreate )
{
	DbgPrint("CreateProcess: Parent 0x%x, Child 0x%x, %s\n",
	         hParent, hChild, (bCreate?"Create":"Exit"));
}


/* Driver routines */
NTSTATUS
HookServices()
{
	OldNtProtectVirtualMemory =
		(NTPROTECTVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwProtectVirtualMemory));
	OldNtAllocateVirtualMemory =
		(NTALLOCATEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwAllocateVirtualMemory));
	OldNtFreeVirtualMemory =
		(NTFREEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwFreeVirtualMemory));

	_asm cli

	(NTPROTECTVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwProtectVirtualMemory)) = MyNtProtectVirtualMemory;
	(NTALLOCATEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwAllocateVirtualMemory)) = MyNtAllocateVirtualMemory;
	(NTFREEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwFreeVirtualMemory)) = MyNtFreeVirtualMemory;

	_asm sti

	PsSetCreateProcessNotifyRoutine( MyCreateProcessNotifyRoutine, FALSE );

	return STATUS_SUCCESS;
}

void
UnHookServices()
{
	PsSetCreateProcessNotifyRoutine( MyCreateProcessNotifyRoutine, TRUE );

	_asm cli

	(NTPROTECTVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwProtectVirtualMemory)) =
		OldNtProtectVirtualMemory;
	(NTALLOCATEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwAllocateVirtualMemory)) =
		OldNtAllocateVirtualMemory;
	(NTFREEVIRTUALMEMORY)(SYSTEMSERVICE(SERVICEID_ZwFreeVirtualMemory)) =
		OldNtFreeVirtualMemory;

	_asm sti

	return;
}

NTSTATUS
DriverEntry(
	IN PDRIVER_OBJECT DriverObject,
	IN PUNICODE_STRING RegistryPath)
{
	MYDRIVERENTRY(DRIVER_DEVICE_NAME,
		FILE_DEVICE_HOOK1,
		HookServices());
	return ntStatus;
}

NTSTATUS
DriverDispatch(
	IN PDEVICE_OBJECT DeviceObject,
	IN PIRP Irp)
{
	Irp->IoStatus.Status = STATUS_SUCCESS;
	IoCompleteRequest( Irp, IO_NO_INCREMENT );
	return Irp->IoStatus.Status;
}

VOID
DriverUnload(
	IN PDRIVER_OBJECT DriverObject)
{
	WCHAR deviceLinkBuffer[] = L"\\DosDevices\\"DRIVER_DEVICE_NAME;
	UNICODE_STRING deviceLinkUnicodeString;

	UnHookServices();

	RtlInitUnicodeString(
		&deviceLinkUnicodeString,
		deviceLinkBuffer);
	IoDeleteSymbolicLink( &deviceLinkUnicodeString );
	IoDeleteDevice( DriverObject->DeviceObject );
}

