#!/bin/sh -v
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: test.sh,v 1.4 2001/04/05 18:40:53 stevek Exp $
#
# test plan:
#	1) load wss
#	2) load each wrapper individually
#	   set activation criteria
#	   run test script
#	   unload activation criteria and wrapper
#	3) load all wrappers
#	   set activation criteria for all wrappers
#	   run test scripts for all wrappers
#	   unload all wrappers and activation criteria
#	4) load multiple instances of each wrapper
#	   set different activation criteria for each instance
#	   run multiple test scripts
#	   unload all
#	5) unload wss
#	6) repeat ?
#
#################################################################


## 1) load wss ##
modload wss_mod.o
modstat

## 2) load individual wrappers
### intercept wrapper
ID=`wrload -d intercept.o | grep "registered with" | awk '{print $NF}'`
wra "user==joe==>intercept"
su joe <<- END
	source ./intercept_test
	END
wrunload -d $ID
ID=`wrstat -C | grep intercept | awk '{print $1}'`
wrunload -d $ID

### wqltest wrapper
ID=`wrload -d wqltest.o | grep "registered with" | awk '{print $NF}'`
wra "user==joe==>wqltest"
su joe <<- END
	END
wrunload -d $ID
ID=`wrstat -C | grep wqltest | awk '{print $1}'`
wrunload -d $ID

### noadmin wrapper
ID=`wrload -d noadmin.o | grep "registered with" | awk '{print $NF}'`
## Setting user==joe seems dumb. The noadmin syscalls can only be 
## executed by root. But, if we set the act. crit. for user==root, 
## then the rest of the script fails because root can no longer 
## execute wrload/wrunload/wrstat calls. Once the WR_DENY checks are in
## place in the dispatch code, however, this should test it. 
wra "user==joe==>noadmin"
su joe <<- END
	source ./noadmin_test
	END
wrunload -d $ID
ID=`wrstat -C | grep noadmin | awk '{print $1}'`
wrunload -d $ID


## 3) load all wrappers ##
wrload -d noadmin.o
wrload -d intercept.o
wrload -d wqltest.o
wra "user==joe==>intercept"
wra "user==joe==>wqltest"
wra "user==joe==>noadmin"
su joe <<- END
	source ./intercept_test
	source ./noadmin_test
	END
for ID in `wrstat -M | grep -v "id    module" | grep -v "\-\-\-\-" | awk '{print $1}'`
do
    wrunload -d $ID
done


## 5) unload wss ##
wrstat -M
modunload -n wss_mod
modstat


###########################
##
##  load up the modules and then unload the wss
##
###########################

modload wss_mod.o
modstat

x=1
while [ $x -lt 5 ]
do
     wrload noadmin.o
     wrload intercept.o
##     wrload wqltest.o
     wrload null.o
     wrload nonexistent.o
     x=`expr $x + 1`
done

wra "user==joe==>noadmin"
wra "user==joe==>intercept"
wra "user==joe==>null"
wra "prog==cat==>intercept"

wrstat -M
modunload -n wss_mod
modstat

