/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: rtld.h,v 1.3 2001/02/23 19:02:14 stevek Exp $
 */

#ifndef __RTLD_H
#define __RTLD_H	1

#ifdef __FreeBSD__
#  if __FreeBSD_version >= 300000
#     ifdef __ALPHA__
#        include <sys/elf64.h>

typedef Elf64_Addr Elf_Addr;
typedef Elf64_Dyn  Elf_Dyn;
typedef Elf64_Phdr Elf_Phdr;
typedef Elf64_Rela Elf_Rela;
typedef Elf64_Rel  Elf_Rel;
typedef Elf64_Sym  Elf_Sym;
typedef Elf64_Word Elf_Word;
#     else
#        include <sys/elf32.h>

typedef Elf32_Addr Elf_Addr;
typedef Elf32_Dyn  Elf_Dyn;
typedef Elf32_Phdr Elf_Phdr;
typedef Elf32_Rela Elf_Rela;
typedef Elf32_Rel  Elf_Rel;
typedef Elf32_Sym  Elf_Sym;
typedef Elf32_Word Elf_Word;
#     endif

#     ifdef FALSE
#       undef FALSE
#     endif
#     ifdef TRUE
#       undef TRUE
#    endif

typedef enum { FALSE, TRUE } bool;

typedef struct Struct_Obj_Entry Obj_Entry;

struct Struct_Obj_Entry
{
    Elf_Word magic;		/* Magic number (sanity check) */
    Elf_Word version;		/* Version number of struct format */

    struct Struct_Obj_Entry *next;
    char *path;			/* Pathname of underlying file (%) */
    int refcount;
    int dl_refcount;		/* Number of times loaded by dlopen */

    /* These items are computed by map_object() or by digest_phdr(). */
    caddr_t mapbase;		/* Base address of mapped region */
    size_t mapsize;		/* Size of mapped region in bytes */
    size_t textsize;		/* Size of text segment in bytes */
    Elf_Addr vaddrbase;		/* Base address in shared object file */
    caddr_t relocbase;		/* Relocation constant = mapbase - vaddrbase */
    const Elf_Dyn *dynamic;	/* Dynamic section */
    caddr_t entry;		/* Entry point */
    const Elf_Phdr *phdr;	/* Program header if it is mapped, else NULL */
    size_t phsize;		/* Size of program header in bytes */

    /* Items from the dynamic section. */
    Elf_Addr *pltgot;		/* PLT or GOT, depending on architecture */
    const Elf_Rel *rel;		/* Relocation entries */
    unsigned long relsize;	/* Size in bytes of relocation info */
    const Elf_Rela *rela;	/* Relocation entries with addend */
    unsigned long relasize;	/* Size in bytes of addend relocation info */
    const Elf_Rel *pltrel;	/* PLT relocation entries */
    unsigned long pltrelsize;	/* Size in bytes of PLT relocation info */
    const Elf_Rela *pltrela;	/* PLT relocation entries with addend */
    unsigned long pltrelasize;	/* Size in bytes of PLT addend reloc info */
    const Elf_Sym *symtab;	/* Symbol table */
    const char *strtab;		/* String table */
    unsigned long strsize;	/* Size in bytes of string table */

    const Elf_Addr *buckets;	/* Hash table buckets array */
    unsigned long nbuckets;	/* Number of buckets */
    const Elf_Addr *chains;	/* Hash table chain array */
    unsigned long nchains;	/* Number of chains */

    const char *rpath;		/* Search path specified in object */
    void *needed;		/* Shared objects needed by this one (%) */

    void (*init)(void);		/* Initialization function to call */
    void (*fini)(void);		/* Termination function to call */

    bool mainprog;		/* True if this is the main program */
    bool rtld;			/* True if this is the dynamic linker */
    bool textrel;		/* True if there are relocations to text seg */
    bool symbolic;		/* True if generated with "-Bsymbolic" */
    bool traced;		/* Already printed in ldd trace output */
};
#  endif /* __FreeBSD_version >= 300000 */
#endif /* __FreeBSD__ */

#endif /* __RTLD_H */

