/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: linker_misc.c,v 1.12 2001/02/23 19:02:14 stevek Exp $
 */

#include <wr_conf.h>

#include <stdarg.h>
#include <ctype.h>

#include <wss_malloc.h>
#include <wss_evspace.h>
#include <wss_process.h>
#include <wss_iface.h>
#include <wsi_iface.h>

#ifndef WR_WINNT
size_t     WSSLINKAGE  wsi_strlen       __P(( const pd_char_t * ));
int        WSSLINKAGE  wsi_strcmp       __P(( const pd_char_t *,
                                              const pd_char_t * ));
#if defined(WR_SOLARIS) || defined(WR_LINUX)
pd_char_t WSSLINKAGE *wsi_strcpy       __P(( pd_char_t *,
                                              const pd_char_t * ));
#endif /* WR_SOLARIS || WR_LINUX */
pd_char_t WSSLINKAGE *wsi_strncpy      __P(( pd_char_t *, const pd_char_t *,
                                              size_t ));
#endif /* !WR_WINNT */
void       WSSLINKAGE *wsi_malloc       __P(( size_t ));
void       WSSLINKAGE  wsi_free         __P(( void * ));
int        WSSLINKAGE  wsi_printf       __P(( const pd_char_t *, ... ));
pd_char_t WSSLINKAGE  wsi_tolower      __P(( const pd_char_t ));
pd_char_t WSSLINKAGE  wsi_toupper      __P(( const pd_char_t ));
pd_char_t WSSLINKAGE *wsi_syscall_name     __P(( int ));

int wsi_unloading = 0;

wss_symbol_t wss_export_table[] =
{
   WSS_EXPORT_ENTRY("wr_malloc",              (u_long)wsi_malloc),
   WSS_EXPORT_ENTRY("wr_free",                (u_long)wsi_free),
   WSS_EXPORT_ENTRY("wr_bzero",               (u_long)bzero),
   WSS_EXPORT_ENTRY("wr_bcopy",               (u_long)bcopy),
   WSS_EXPORT_ENTRY("wr_copyin",              (u_long)bcopy),
   WSS_EXPORT_ENTRY("wr_strlen",              (u_long)wsi_strlen),
   WSS_EXPORT_ENTRY("wr_strcmp",              (u_long)wsi_strcmp),
#ifdef MALLOC_DEBUG
   WSS_EXPORT_ENTRY("wr_strdup_debug",        (u_long)wss_strdup_debug),
#else
   WSS_EXPORT_ENTRY("wr_strdup",              (u_long)wss_strdup),
#endif
#if defined(WR_SOLARIS) || defined(WR_LINUX)
   WSS_EXPORT_ENTRY("wr_strcpy",              (u_long)wsi_strcpy),
#endif /* WR_SOLARIS */
   WSS_EXPORT_ENTRY("wr_strncpy",             (u_long)wsi_strncpy),
   WSS_EXPORT_ENTRY("wr_printf",              (u_long)wsi_printf),
   WSS_EXPORT_ENTRY("wr_resolve_event_space", (u_long)wss_evspace_resolve),
   WSS_EXPORT_ENTRY("wr_syscall_name",        (u_long)wsi_syscall_name),
   { NULL, 0, 0 }
};

#ifndef WR_WINNT
size_t WSSLINKAGE
wsi_strlen( const pd_char_t *str )
{
   return _tcslen( str );
}

int WSSLINKAGE
wsi_strcmp( const pd_char_t *s1, const pd_char_t *s2 )
{
   return _tcscmp( s1, s2 );
}

pd_char_t WSSLINKAGE *
wsi_strncpy( pd_char_t *dst, const pd_char_t *src, size_t len )
{
   return strncpy( dst, src, len );
}

#if defined(WR_SOLARIS) || defined(WR_LINUX)
pd_char_t WSSLINKAGE *
wsi_strcpy( pd_char_t *dst, const pd_char_t *src )
{
   return strcpy( dst, src );
}
#endif /* WR_SOLARIS */

#endif /* !WR_WINNT */

int WSSLINKAGE
wsi_printf( const pd_char_t *fmt, ... )
{
   va_list va;
   int     retc;

   va_start( va, fmt );
   retc = _vtprintf( fmt, va );
   va_end( va );

   return( retc );
}

void WSSLINKAGE *
wsi_malloc( size_t size )
{
   return malloc( size );
}

void WSSLINKAGE
wsi_free( void *ptr )
{
   free( ptr );
}

void WSSLINKAGE
panic( pd_char_t *format, ... )
{
   va_list va;

   va_start( va, format );
   _vtprintf( format, va );
   va_end( va );
}

pd_char_t WSSLINKAGE *
wsi_syscall_name( int code )
{
   return PD_MAKESTR("syscall");
}

pd_char_t WSSLINKAGE
wsi_tolower( const pd_char_t c )
{
   return tolower( c );
}

pd_char_t WSSLINKAGE
wsi_toupper( const pd_char_t c )
{
   return toupper( c );
}

#ifdef WR_WINNT
void
bzero( void *buf, size_t size )
{
   memset( buf, 0, size );
}
#endif /* WR_WINNT */

int WSSLINKAGE
wss_proc_add_wrapper_blind( wss_proc_t *wrp, const pd_char_t *name )
{
   return 0;
}

int WSSLINKAGE
wss_proc_del_wrapper( wss_proc_t *wrp, pd_char_t *name )
{
   return 0;
}

