/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_group.cxx,v 1.14 2001/04/05 18:40:57 stevek Exp $
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <eventfs.h>
#include "eventfs_windows_nt.h"
#include "eventfs_pipe.h"

#include <pipesrv.h>


BOOL
EventFSProcessGroupStatusPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   struct eventfs_group *eg =
      (struct eventfs_group *) pInfo->pPipeSrvQueueInfo->pData;

   pd_char_t sbuf[PAGE_SIZE];
   int index = _sntprintf( sbuf, PAGE_SIZE, _TEXT("[%s]\n"), eg->efgroup_name );

   u_long mask = eg->efgroup_eventmask;
   for( int i = 0; ( i < EFGROUP_NTYPES ) && ( index < (PAGE_SIZE - 1) ); i++ )
   {
      if( eg->efgroup_types[i] && ( mask & (((u_long)1)<<i) ) )
         index += _sntprintf( sbuf + index, PAGE_SIZE - index - 1,
                              _TEXT("%lu\t%s\n"), ((u_long)1<<i),
                              eg->efgroup_types[i] );
   }
   sbuf[PAGE_SIZE-1] = _TEXT('\0');

   EventFSWritePacket( pInfo, (PVOID) sbuf, index * sizeof(pd_char_t) );

   return TRUE;
}

BOOL
EventFSProcessGroupIoctlPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   struct eventfs_request req;
   OVERLAPPED             os;
   BOOL                   bRet;
   DWORD                  dwWait;

   memset( &os, 0, sizeof(os) );
   os.hEvent = pInfo->hEvents[PIPESRV_OVERLAPPED_IO_READ_EVENT];
   ResetEvent( os.hEvent );

   bRet = ReadFile( pInfo->hPipe, (PVOID) &req.mask, sizeof(req.mask), NULL,
                    &os );
   if( !bRet )
   {
      DWORD error = GetLastError();
      switch( error )
      {
         case ERROR_IO_PENDING:
            dwWait = WaitForSingleObject( os.hEvent, INFINITE );
            switch( dwWait )
            {
               case WAIT_OBJECT_0:
                  bRet = TRUE;
                  break;

               default:
                  break;
            }
      }
   }

   if( bRet )
   {
      struct eventfs_group *eg =
         (struct eventfs_group *) pInfo->pPipeSrvQueueInfo->pData;
      pd_char_t *name = NULL;

      int error = EFSPURENAME(eventfs_construct_stream)( eg->efgroup_name,
                                                         req.mask, &name );
      if( !error )
         EventFSWritePacket( pInfo, (PVOID) name,
                             _tcslen(name) * sizeof(pd_char_t) );
      if( name )
         EFSPURENAME(eventfs_free_stream_name)( name );
   }

   return TRUE;
}

