/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_machdep.cxx,v 1.24 2001/04/05 18:40:57 stevek Exp $
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <eventfs.h>
#include <eventfs_machine.h>
#include "eventfs_windows_nt.h"
#include "eventfs_pipe.h"

HANDLE g_hTimer;

int
eventfs_machine_init()
{
   g_hTimer = CreateWaitableTimer( NULL, FALSE, NULL );
   if( g_hTimer )
   {
      const LARGE_INTEGER li = {0,0};
      SetWaitableTimer( g_hTimer, &li, HEARTBEAT_TIMEOUT, NULL, NULL, FALSE );
   }
   return 0;
}

int
eventfs_machine_uninit()
{
   CloseHandle( g_hTimer );
   return 0;
}

void
eventfs_machine_flush( struct eventfs_stream *es )
{
}

void
eventfs_machine_destroy_group( struct eventfs_group *eg )
{
   LPEVENTFS_GROUP_DATA efgData = (LPEVENTFS_GROUP_DATA)eg->efgroup_data;

   LPPIPESRV_QUEUE_INFO pqi = efgData->pqiStats;
   while( pqi->thread )
      WaitForSingleObject( pqi->hEvents[PIPESRV_ACCEPTOR_DIED], INFINITE );
   delete pqi->lpszPipeName;
   delete pqi;

   pqi = efgData->pqiIoctl;
   while( pqi->thread )
      WaitForSingleObject( pqi->hEvents[PIPESRV_ACCEPTOR_DIED], INFINITE );
   delete pqi->lpszPipeName;
   delete pqi;

   delete efgData;
}

void
eventfs_machine_destroy_stream( struct eventfs_stream *es )
{
   LPEVENTFS_STREAM_DATA efsData = (LPEVENTFS_STREAM_DATA)es->efstream_data;

   LPPIPESRV_QUEUE_INFO pqi = efsData->pqiStats;
   while( pqi->thread )
      WaitForSingleObject( pqi->hEvents[PIPESRV_ACCEPTOR_DIED], INFINITE );
   delete pqi->lpszPipeName;
   delete pqi;

   pqi = efsData->pqiData;
   while( pqi->thread )
      WaitForSingleObject( pqi->hEvents[PIPESRV_ACCEPTOR_DIED], INFINITE );
   delete pqi->lpszPipeName;
   LPEVENTFS_DATA_STREAM pEDS = (LPEVENTFS_DATA_STREAM)pqi->pData;
   CloseHandle( pEDS->hSema );
   delete pEDS;
   delete pqi;

   delete efsData;
}

void
eventfs_machine_register_group( struct eventfs_group *eg )
{
#ifdef EVENTFS_DIAGNOSTICS
   _tprintf( _TEXT("eventfs_machine_register_group(0x%x): %s\n"),
             (UINT) eg, eg->efgroup_name );
#endif /* EVENTFS_DIAGNOSTICS */

   // Construct the group status pipe
   LPPIPESRV_QUEUE_INFO pqi = new PIPESRV_QUEUE_INFO();

   pd_char_t *pipeName = new pd_char_t[256];
   _stprintf( pipeName, _TEXT("%s%c%s%c%s"), _TEXT(_PATH_EVENTFS),
              _TEXT(FILESEP), eg->efgroup_name, _TEXT(FILESEP),
              _TEXT("stats") );

   pqi->pData = (LPVOID) eg;
   pqi->bReadOnly = TRUE;
   pqi->lpszPipeName = pipeName;
   pqi->nInstances = 0;
   pqi->nThreads = 0;
   pqi->nActive = 0;
   pqi->fOpen = NULL;
   pqi->fClose = NULL;
   pqi->fProcessing = EventFSProcessGroupStatusPacket;
   pqi->hEvents[0] = g_hPipeSrvStopEvent;
   pqi->hEvents[PIPESRV_PIPE_AVAILABLE_EVENT] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqi->hEvents[PIPESRV_PIPE_ACTIVE_EVENT] = 
      CreateSemaphore( NULL, 0, 10000, NULL );
   pqi->hEvents[PIPESRV_WORKER_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqi->hEvents[PIPESRV_ACCEPTOR_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   InitializeCriticalSection( &pqi->critSec );
   pqi->thread = _beginthread( PipeSrvAcceptThread, 0, pqi );

   // Construct the stream ioctl pipe
   LPPIPESRV_QUEUE_INFO pqiIoctl = new PIPESRV_QUEUE_INFO();

   pipeName = new pd_char_t[256];
   _stprintf( pipeName, _TEXT("%s%c%s%c%s"), _TEXT(_PATH_EVENTFS),
              _TEXT(FILESEP), eg->efgroup_name, _TEXT(FILESEP),
              _TEXT("ioctl") );

   pqiIoctl->pData = (LPVOID) eg;
   pqiIoctl->bReadOnly = FALSE;
   pqiIoctl->lpszPipeName = pipeName;
   pqiIoctl->nInstances = 0;
   pqiIoctl->nThreads = 0;
   pqiIoctl->nActive = 0;
   pqiIoctl->fOpen = NULL;
   pqiIoctl->fClose = NULL;
   pqiIoctl->fProcessing = EventFSProcessGroupIoctlPacket;
   pqiIoctl->hEvents[0] = g_hPipeSrvStopEvent;
   pqiIoctl->hEvents[PIPESRV_PIPE_AVAILABLE_EVENT] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiIoctl->hEvents[PIPESRV_PIPE_ACTIVE_EVENT] = 
      CreateSemaphore( NULL, 0, 10000, NULL );
   pqiIoctl->hEvents[PIPESRV_WORKER_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiIoctl->hEvents[PIPESRV_ACCEPTOR_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   InitializeCriticalSection( &pqiIoctl->critSec );
   pqiIoctl->thread = _beginthread( PipeSrvAcceptThread, 0, pqiIoctl );

   // Construct group data
   LPEVENTFS_GROUP_DATA efgData = new EVENTFS_GROUP_DATA;
   efgData->pqiStats = pqi;
   efgData->pqiIoctl = pqiIoctl;
   eg->efgroup_data = (LPVOID) efgData;
}

void
eventfs_machine_register_stream( struct eventfs_stream *es )
{
#ifdef EVENTFS_DIAGNOSTICS
   _tprintf( _TEXT("eventfs_machine_register_stream(0x%x): %s\n"),
             (UINT) es, es->efstream_name );
#endif /* EVENTFS_DIAGNOSTICS */

   // Construct the stream status pipe
   LPPIPESRV_QUEUE_INFO pqiStatus = new PIPESRV_QUEUE_INFO();

   pd_char_t *pipeName = new pd_char_t[256];
   _stprintf( pipeName, _TEXT("%s%c%s%c%s%c%s"), _TEXT(_PATH_EVENTFS),
              _TEXT(FILESEP), es->efstream_group->efgroup_name, _TEXT(FILESEP),
              es->efstream_name, _TEXT(FILESEP), _TEXT("stats") );

   pqiStatus->pData = (LPVOID) es;
   pqiStatus->bReadOnly = TRUE;
   pqiStatus->lpszPipeName = pipeName;
   pqiStatus->nInstances = 0;
   pqiStatus->nThreads = 0;
   pqiStatus->nActive = 0;
   pqiStatus->fOpen = NULL;
   pqiStatus->fClose = NULL;
   pqiStatus->fProcessing = EventFSProcessStreamStatusPacket;
   pqiStatus->hEvents[0] = g_hPipeSrvStopEvent;
   pqiStatus->hEvents[PIPESRV_PIPE_AVAILABLE_EVENT] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiStatus->hEvents[PIPESRV_PIPE_ACTIVE_EVENT] = 
      CreateSemaphore( NULL, 0, 10000, NULL );
   pqiStatus->hEvents[PIPESRV_WORKER_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiStatus->hEvents[PIPESRV_ACCEPTOR_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   InitializeCriticalSection( &pqiStatus->critSec );
   pqiStatus->thread = _beginthread( PipeSrvAcceptThread, 0, pqiStatus );

   // Construct the stream data pipe
   LPPIPESRV_QUEUE_INFO pqiData = new PIPESRV_QUEUE_INFO();

   pipeName = new pd_char_t[256];
   _stprintf( pipeName, _TEXT("%s%c%s%c%s%c%s"), _TEXT(_PATH_EVENTFS),
              _TEXT(FILESEP), es->efstream_group->efgroup_name, _TEXT(FILESEP),
              es->efstream_name, _TEXT(FILESEP), _TEXT("data") );

   LPEVENTFS_DATA_STREAM pEDS = new EVENTFS_DATA_STREAM;
   pEDS->pES = es;
   pEDS->hSema = CreateSemaphore( NULL, 0, 10000, NULL );
   pqiData->pData = (LPVOID) pEDS;
   pqiData->bReadOnly = TRUE;
   pqiData->lpszPipeName = pipeName;
   pqiData->nInstances = 0;
   pqiData->nThreads = 0;
   pqiData->nActive = 0;
   pqiData->fOpen = EventFSOpenStreamData;
   pqiData->fClose = EventFSCloseStreamData;
   pqiData->fProcessing = EventFSProcessStreamDataPacket;
   pqiData->hEvents[0] = g_hPipeSrvStopEvent;
   pqiData->hEvents[PIPESRV_PIPE_AVAILABLE_EVENT] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiData->hEvents[PIPESRV_PIPE_ACTIVE_EVENT] = 
      CreateSemaphore( NULL, 0, 10000, NULL );
   pqiData->hEvents[PIPESRV_WORKER_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   pqiData->hEvents[PIPESRV_ACCEPTOR_DIED] =
      CreateEvent( NULL, TRUE, FALSE, NULL );
   InitializeCriticalSection( &pqiData->critSec );
   pqiData->thread = _beginthread( PipeSrvAcceptThread, 0, pqiData );

   // Construct group data
   LPEVENTFS_STREAM_DATA efsData = new EVENTFS_STREAM_DATA;
   efsData->pqiStats = pqiStatus;
   efsData->pqiData  = pqiData;
   es->efstream_data = (LPVOID) efsData;
}

void
eventfs_machine_post_event_to_stream( struct eventfs_stream *stream,
                                      struct eventfs_entry *entry )
{
#ifdef EVENTFS_DIAGNOSTICS
   _tprintf( _TEXT("eventfs_machine_post_event_to_stream\n") );
#endif /* EVENTFS_DIAGNOSTICS */

   LPEVENTFS_STREAM_DATA efsData = (LPEVENTFS_STREAM_DATA)stream->efstream_data;
   if( efsData == NULL ) return;

   LPPIPESRV_QUEUE_INFO pqi = efsData->pqiData;
   if( pqi == NULL ) return;

   EnterCriticalSection( &pqi->critSec );
   LPEVENTFS_DATA_STREAM pEDS = (LPEVENTFS_DATA_STREAM) pqi->pData;
   if( pqi->nActive )
      ReleaseSemaphore( pEDS->hSema, pqi->nActive, NULL );
   LeaveCriticalSection( &pqi->critSec );
}

