/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_pipe.cxx,v 1.8 2001/04/05 18:40:57 stevek Exp $
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <eventfs.h>
#include "eventfs_windows_nt.h"
#include "eventfs_pipe.h"

#include <pipesrv.h>


BOOL
EventFSWritePacket( LPPIPESRV_PIPE_INFO pInfo, PVOID pData, size_t size )
{
   OVERLAPPED os;
   HANDLE     hEvents[2];

   if( pInfo == NULL )
      return FALSE;

   hEvents[0] = pInfo->hEvents[0];
   hEvents[1] = pInfo->hEvents[PIPESRV_OVERLAPPED_IO_WRITE_EVENT];

   PD_MUTEX_ENTER( EFSLINKAGE, pInfo->writePipe );

   // init the overlapped structure
   //
   memset( &os, 0, sizeof(OVERLAPPED) );
   os.hEvent = hEvents[1];
   ResetEvent( hEvents[1] );

   // send out the packet
   DWORD cbWritten, dwWait;
   BOOL bRet = WriteFile( pInfo->hPipe, pData, size, &cbWritten, &os );

   if( !bRet )
   {
      switch( GetLastError() )
      {
         case ERROR_IO_PENDING:
            dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
            switch( dwWait )
            {
               case WAIT_OBJECT_0 + 1:
                  break;

               case WAIT_FAILED:
                  PD_MUTEX_EXIT( EFSLINKAGE, pInfo->writePipe );
                  return FALSE;

               default:
                  // not overlapped i/o event
                  // - error occurred or server stop signaled
                  PD_MUTEX_EXIT( EFSLINKAGE, pInfo->writePipe );
                  return FALSE;
            }
            break;

         default:
            PD_MUTEX_EXIT( EFSLINKAGE, pInfo->writePipe );
            return FALSE;
      }
   }

   PD_MUTEX_EXIT( EFSLINKAGE, pInfo->writePipe );
   return TRUE;
}

