/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_stream.cxx,v 1.27 2001/04/05 18:40:57 stevek Exp $
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <eventfs.h>
#include "eventfs_windows_nt.h"
#include "eventfs_pipe.h"

#include <pipesrv.h>

extern HANDLE g_hTimer;

extern "C" {
   void _eventfs_close_stream_unsafe( struct eventfs_stream *,
                                      struct eventfs_stream_entry * );
}

BOOL
EventFSProcessStreamStatusPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   LPEVENTFS_DATA_STREAM pEDS =
      (LPEVENTFS_DATA_STREAM) pInfo->pPipeSrvQueueInfo->pData;
   struct eventfs_stream *es = (struct eventfs_stream *) pEDS->pES;

   if( !es || ( es->efstream_flags & EFS_FLG_GOING ) )
      return TRUE;

   struct eventfs_group  *eg = es->efstream_group;
   pd_char_t sbuf[PAGE_SIZE];
   int index = _sntprintf( sbuf, PAGE_SIZE, _TEXT("[%s::%s]\n"),
                           eg->efgroup_name, es->efstream_name );

   u_long mask = es->efstream_mask;
   for( int i = 0; ( i < EFGROUP_NTYPES ) && ( index < (PAGE_SIZE - 1) ); i++ )
   {
      if( eg->efgroup_types[i] && ( mask & (((u_long)1)<<i) ) )
         index += _sntprintf( sbuf + index, PAGE_SIZE - index - 1,
                              _TEXT("%lu\t%s\n"), ((u_long)1<<i),
                              eg->efgroup_types[i] );
   }
   sbuf[PAGE_SIZE-1] = _TEXT('\0');

   EventFSWritePacket( pInfo, (PVOID) sbuf, index * sizeof(pd_char_t) );

   return TRUE;
}

BOOL
EventFSProcessStreamDataPacket( LPPIPESRV_PIPE_INFO pInfo )
{
   if( g_bPipeSrvDebugging )
      _tprintf( _TEXT("EventFSProcessStreamDataPacket: Processing...\n" ) );

   LPEVENTFS_DATA_STREAM pEDS =
      (LPEVENTFS_DATA_STREAM) pInfo->pPipeSrvQueueInfo->pData;
   struct eventfs_stream *es = (struct eventfs_stream *) pEDS->pES;

   if( !es || ( es->efstream_flags & EFS_FLG_GOING ) )
   {
      if( g_bPipeSrvDebugging )
         _tprintf( _TEXT("EventFSProcessStreamDataPacket: Invalid or dead ")
                   _TEXT(" stream encountered\n") );
      return TRUE;
   }

   struct eventfs_stream_entry *estrent =
      (struct eventfs_stream_entry *) pInfo->pData;
   struct eventfs_entry *efsent =
      (struct eventfs_entry *) estrent->efstrent_entry;

   if( efsent == NULL )
   {
      if( g_bPipeSrvDebugging )
         _tprintf( _TEXT("EventFSProcessStreamDataPacket: No events pending\n") );

#ifdef EFS_HEARTBEAT
      HANDLE hEvents[3];
#else
      HANDLE hEvents[2];
#endif /* EFS_HEARTBEAT */

      hEvents[0] = g_hPipeSrvStopEvent;
#ifdef EFS_HEARTBEAT
      hEvents[2] = g_hTimer;
#endif /* EFS_HEARTBEAT */
rewait:
      hEvents[1] = pEDS->hSema;

#ifdef EFS_HEARTBEAT
      DWORD dwWait = WaitForMultipleObjects( 3, hEvents, FALSE, INFINITE );
#else
      DWORD dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
#endif /* EFS_HEARTBEAT */
      if( dwWait != (WAIT_OBJECT_0 + 1) )
      {
#ifdef EFS_HEARTBEAT
         if( dwWait == (WAIT_OBJECT_0 + 2) )
         {
            hEvents[1] = pInfo->hEvents[GWM_OVERLAPPED_IO_WRITE_EVENT];

            // init the overlapped structure
            OVERLAPPED os;
            memset( &os, 0, sizeof(OVERLAPPED) );
            os.hEvent = hEvents[1];
            ResetEvent( hEvents[1] );

            // write out zero-length
            BOOL bRet = WriteFile( pInfo->hPipe, NULL, 0, NULL, &os );
            if( !bRet )
            {
               switch( GetLastError() )
               {
                  case ERROR_IO_PENDING:
                     dwWait = WaitForMultipleObjects( 2, hEvents, FALSE, INFINITE );
                     if( dwWait == (WAIT_OBJECT_0 + 1) )
                        goto rewait;
                     break;

                  default:
                     break;
               }
            }
            else
               goto rewait;
         }
#endif /* EFS_HEARTBEAT */
         return TRUE;
      }

      // regrab event entry
      efsent = (struct eventfs_entry *) estrent->efstrent_entry;

      // check to see if we're at the sentinel
      if( efsent == NULL )
         goto rewait;
   }

   if( g_bPipeSrvDebugging )
      _tprintf( _TEXT("EventFSProcessStreamDataPacket: Writing event\n") );

   if( !EventFSWritePacket( pInfo, (PVOID) &efsent->efsent_event,
                            sizeof(efsent->efsent_event) ) )
   {
      LPVOID lpMsgBuf;

      FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                     FORMAT_MESSAGE_FROM_SYSTEM |
                     FORMAT_MESSAGE_IGNORE_INSERTS,
                     NULL, GetLastError(),
                     MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
                     (LPTSTR) &lpMsgBuf, 0, NULL );

      _tprintf( _TEXT("EventFSProcessStreamDataPacket: Error writing header: %s\n"),
                (LPCTSTR)lpMsgBuf );
      LocalFree( lpMsgBuf );
      return TRUE;
   }
   if( ( efsent->efsent_event.efsevent_size > 0 ) &&
       !EventFSWritePacket( pInfo, (PVOID) (efsent + 1),
                            efsent->efsent_event.efsevent_size ) )
   {
      LPVOID lpMsgBuf;

      FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER |
                     FORMAT_MESSAGE_FROM_SYSTEM |
                     FORMAT_MESSAGE_IGNORE_INSERTS,
                     NULL, GetLastError(),
                     MAKELANGID( LANG_NEUTRAL, SUBLANG_DEFAULT ),
                     (LPTSTR) &lpMsgBuf, 0, NULL );

      _tprintf( _TEXT("EventFSProcessStreamDataPacket: Error writing data: %s\n"),
                (LPCTSTR)lpMsgBuf );
      LocalFree( lpMsgBuf );
      return TRUE;
   }

   pInfo->pData = (PVOID) estrent->efstrent_link.le_next;

    if( g_bPipeSrvDebugging )
       _tprintf( _TEXT("EventFSProcessStreamDataPacket: Releasing event\n") );
   eventfs_strent_rele( estrent );

   return FALSE;
}

BOOL
EventFSOpenStreamData( LPPIPESRV_PIPE_INFO pInfo )
{
   LPEVENTFS_DATA_STREAM pEDS =
      (LPEVENTFS_DATA_STREAM) pInfo->pPipeSrvQueueInfo->pData;
   struct eventfs_stream *es = (struct eventfs_stream *) pEDS->pES;

   if( !es || ( es->efstream_flags & EFS_FLG_GOING ) )
      return FALSE;

   struct eventfs_stream_entry *estrent = es->efstream_tail;
   eventfs_strent_ref( estrent );
   pInfo->pData = (PVOID) estrent;
   eventfs_stream_ref( es );

   return TRUE;
}

void
EventFSCloseStreamData( LPPIPESRV_PIPE_INFO pInfo )
{
   LPEVENTFS_DATA_STREAM pEDS =
      (LPEVENTFS_DATA_STREAM) pInfo->pPipeSrvQueueInfo->pData;
   struct eventfs_stream *es = (struct eventfs_stream *) pEDS->pES;
   struct eventfs_stream_entry *estrent =
      (struct eventfs_stream_entry *) pInfo->pData;

   PD_MUTEX_ENTER( EFSLINKAGE, es->efstream );
   _eventfs_close_stream_unsafe( es, estrent );
   PD_MUTEX_EXIT( EFSLINKAGE, es->efstream );
   pInfo->pData = NULL;
}

