/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_windows_nt.h,v 1.17 2001/02/23 19:02:16 stevek Exp $
 */

#ifndef __EVENTFS_WINDOWS_NT_H
#define __EVENTFS_WINDOWS_NT_H 1

#define   _PATH_EVENTFS   "\\\\.\\pipe\\eventfs"      /* Default mountpoint */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#ifdef PD_IN_KERNEL
#define EFS_MALLOC(v,t,s) ((v) = (t)malloc((s)))
#define EFS_FREE(v) free(v)

#ifndef _EFS
#ifdef EFSNAME
#undef EFSNAME
#endif /* EFSNAME */
#ifdef EFSPURENAME
#undef EFSPURENAME
#endif /* EFSPURENAME */
#define EFSPURENAME(a) EFSKERNEL_ ## a
#define EFSNAME(a) EFSLINKAGE EFSPURENAME(a)
#endif

#ifdef __cplusplus
extern "C" {
#endif

void EFSLINKAGE panic __P(( char *format, ... ));

int eventfs_sprintf __P(( pd_char_t *buf, const pd_char_t *format, ... ));
#define eventfs_sprintf _stprintf

#ifndef bcopy
#define bcopy(s,d,c) memcpy(d,s,c)
#endif
#ifndef bzero
#define bzero(a,l)   memset(a,0,l)
#endif

#ifdef __cplusplus
}
#endif

#endif /* PD_IN_KERNEL */


struct eventfs_request
{
    unsigned long       mask;
    char                name[MAXNAMELEN];
};

enum eventfs_ioctl_cmds {
    EFSIOCREQ   = 1
};


#ifdef __cplusplus
extern "C" {
#endif

typedef struct eventfs_interface eventfs_interface_t;
struct eventfs_interface
{
   int (*ConstructStream)  ( const pd_char_t *, u_long, pd_char_t ** );
   int (*DestroyGroup)     ( const pd_char_t * );
   int (*DestroyStream)    ( const pd_char_t *, const pd_char_t * );
   int (*FlushStream)      ( const pd_char_t *, const pd_char_t * );
   int (*GetEventMask)     ( const pd_char_t *, const pd_char_t *, u_long * );
   int (*Init)             ( void );
   int (*PostEvent)        ( const pd_char_t *, const pd_char_t *, size_t, void * );
   int (*PostEventByTypes) ( const pd_char_t *, u_long, size_t, const void * );
   int (*RegisterEventType)( const pd_char_t *, const pd_char_t *, u_long * );
   int (*RegisterGroup)    ( const pd_char_t * );
   int (*RegisterStream)   ( const pd_char_t *, const pd_char_t *, u_long, pd_bool_t );
   int (*Uninit)           ( void );
};


/*
 * Public functions
 */
void EFSLINKAGE eventfs_interface_init    ( void );
void EFSLINKAGE eventfs_interface_uninit  ( void );
BOOL EFSLINKAGE eventfs_interface_register( eventfs_interface_t * );
#ifdef __cplusplus
}
#endif

#endif /* __EVENTFS_WINDOWS_NT_H */

