/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: pipesrv.h,v 1.12 2001/02/23 19:02:17 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Header for the Global Wrapper Manager service communication routines
 */

#ifndef __PIPESRV_H
#define __PIPESRV_H 1

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786)

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <pdpi/pi_mutex.h>

#ifdef __cplusplus
#include <map>
#include <queue>
#endif /* __cplusplus */


// convienence macros for accept thread event arrays
#define PIPESRV_PIPE_AVAILABLE_EVENT 1
#define PIPESRV_PIPE_ACTIVE_EVENT    2
#define PIPESRV_WORKER_DIED          3
#define PIPESRV_ACCEPTOR_DIED        4

// convienence macros for worker thread event arrays
//
#define PIPESRV_OVERLAPPED_IO_READ_EVENT  1
#define PIPESRV_OVERLAPPED_IO_WRITE_EVENT 2


// structures and types
//
#ifdef __cplusplus
typedef struct _PIPESRV_QUEUE_INFO PIPESRV_QUEUE_INFO, * LPPIPESRV_QUEUE_INFO;
typedef struct _PIPESRV_PIPE_INFO PIPESRV_PIPE_INFO, * LPPIPESRV_PIPE_INFO;
#endif /* __cplusplus */
typedef BOOL (*PipeSrvPipeOpenFunc)( LPPIPESRV_PIPE_INFO );
typedef void (*PipeSrvPipeCloseFunc)( LPPIPESRV_PIPE_INFO );
typedef BOOL (*PipeSrvPipeProcessingFunc)( LPPIPESRV_PIPE_INFO );
typedef BOOL (*PipeSrvCommDispatchFunc)( LPPIPESRV_PIPE_INFO );
typedef struct _PIPESRV_LAST_PACKET_DATA PIPESRV_LAST_PACKET_DATA, * LPPIPESRV_LAST_PACKET_DATA;
#ifdef __cplusplus
typedef std::queue < LPPIPESRV_PIPE_INFO > PIPESRV_QUEUE;
typedef std::map < int, LPPIPESRV_LAST_PACKET_DATA > PIPESRV_LAST_PACKETS, * LPPIPESRV_LAST_PACKETS;
#endif /* __cplusplus */
typedef struct _PIPESRV_COMM_HEADER PIPESRV_COMM_HEADER, * LPPIPESRV_COMM_HEADER;
typedef struct _PIPESRV_COMM_RESP_HEADER PIPESRV_COMM_RESP_HEADER, * LPPIPESRV_COMM_RESP_HEADER;


struct _PIPESRV_COMM_HEADER
{
   DWORD commandId;
   DWORD pid;
   DWORD tid;
   WORD  flags;
   DWORD pktlen;
};

struct _PIPESRV_COMM_RESP_HEADER
{
   DWORD commandId;
   DWORD pid;
   DWORD tid;
   WORD  flags;
   DWORD pktlen;
   DWORD errorCode;
};

#ifdef __cplusplus
struct _PIPESRV_QUEUE_INFO
{
   unsigned long             thread;
   LPTSTR                    lpszPipeName;
   BOOL                      bReadOnly;
   CRITICAL_SECTION          critSec;
   HANDLE                    hEvents[5];
   DWORD                     nInstances;
   DWORD                     nThreads;
   DWORD                     nActive;
   PipeSrvPipeOpenFunc       fOpen;
   PipeSrvPipeProcessingFunc fProcessing;
   PipeSrvPipeCloseFunc      fClose;
   PIPESRV_QUEUE             qActive;
   PIPESRV_QUEUE             qAvailable;
   PVOID                     pData;

   // constructor
   PDLINKAGE _PIPESRV_QUEUE_INFO();

   // destructor
   PDLINKAGE ~_PIPESRV_QUEUE_INFO();
};

struct _PIPESRV_PIPE_INFO
{
   unsigned long             thread;
   HANDLE                    hPipe;
   HANDLE                    hEvents[3];
   DWORD                     nPipe;
   PD_MUTEX_PROTECT         (PDLINKAGE,readPipe);
   PD_MUTEX_PROTECT         (PDLINKAGE,writePipe);
   PSECURITY_DESCRIPTOR      pSD;
   SECURITY_ATTRIBUTES       sa;
   PIPESRV_COMM_RESP_HEADER  commHdr;
   PVOID                     pBuf;
   LPPIPESRV_QUEUE_INFO      pPipeSrvQueueInfo;
   PVOID                     pData;
   LPPIPESRV_LAST_PACKETS    pLastPackets;
   PD_MUTEX_PROTECT         (PDLINKAGE,pLastPackets);

   // destructor
   PDLINKAGE ~_PIPESRV_PIPE_INFO();
};
#endif /* __cplusplus */

struct _PIPESRV_LAST_PACKET_DATA
{
   LPPIPESRV_COMM_RESP_HEADER pCommHdr;
   PVOID                      pData;
};

#ifdef __cplusplus
extern "C" {
#endif

// publicly exported globals
//
extern HANDLE PDLINKAGE g_hPipeSrvStopEvent;
extern BOOL   PDLINKAGE g_bPipeSrvDebugging;


// public functions
//
void PDLINKAGE PipeSrvAcceptThread( void * );

#ifdef __cplusplus
}
#endif

#endif /* !__PIPESRV_H */

