/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: EndianDataInputStream.java,v 1.8 2001/04/05 18:40:58 stevek Exp $
 */

package com.endian;

import java.io.*;

public class EndianDataInputStream extends InputStream
{
	private DataInputStream dis;
	private int             endian;

	// Constructors
	public EndianDataInputStream( InputStream is )
	{
		dis = new DataInputStream(is);
		endian = Endian.getEndian();
	}

	// Public methods
	public final int available() throws IOException
	{ return dis.available(); }

	public final void close() throws IOException
	{ dis.close(); }

	public final synchronized void mark(int off)
	{ dis.mark(off); }

	public final boolean markSupported()
	{ return dis.markSupported(); }

	public final int read() throws IOException
	{ return dis.read(); }

	public final int read(byte[] ba) throws IOException
	{ return dis.read(ba); }

	public final int read(byte[] ba, int off, int size) throws IOException
	{ return dis.read(ba, off, size); }

	public final int readInt() throws IOException
	{
		if( endian == Endian.BIG_ENDIAN )
			return dis.readInt();
		else if( endian == Endian.LITTLE_ENDIAN )
		{
			long hi = (long)dis.readUnsignedShort();
			long lo = (long)dis.readUnsignedShort();
			return (int)((((hi & 0xff00) >> 8) |
			              ((hi & 0x00ff) << 8) |
			              ((lo & 0xff00) << 8) |
			              ((lo & 0x00ff) << 24)) & 0xffffffff);
		}
		else
			return 0;
	}

	public final synchronized void reset() throws IOException
	{ dis.reset(); }

	public final long skip(long off) throws IOException
	{ return dis.skip(off); }
}

