/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: EndianDataOutputStream.java,v 1.5 2001/04/05 18:40:58 stevek Exp $
 */

package com.endian;

import java.io.*;

public class EndianDataOutputStream extends OutputStream
{
	private DataOutputStream dos;
	private int              endian;

	// Constructors
	public EndianDataOutputStream( OutputStream is )
	{
		dos = new DataOutputStream(is);
		endian = Endian.getEndian();
	}

	// Public methods
	public final void close() throws IOException
	{ dos.close(); }

	public final void flush() throws IOException
	{ dos.flush(); }

	public final void write(int i) throws IOException
	{ dos.write(i); }

	public final void write(byte[] ba) throws IOException
	{ dos.write(ba); }

	public final void write(byte[] ba, int off, int size) throws IOException
	{ dos.write(ba, off, size); }

	public final void writeInt(int val) throws IOException
	{
		if( endian == Endian.BIG_ENDIAN )
			dos.writeInt(val);
		else if( endian == Endian.LITTLE_ENDIAN )
		{
			int hi = (int)(((val << 8) & 0xff00) |
			               ((val >> 8) & 0x00ff));
			int lo = (int)(((val >> 8)  & 0xff00) |
                                       ((val >> 24) & 0x00ff));
			dos.writeShort(hi);
			dos.writeShort(lo);
		}
	}
}

