/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: TestEndianStream.java,v 1.3 2001/04/05 18:40:59 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.io.*;
import com.endian.EndianDataOutputStream;
import com.endian.EndianDataInputStream;

public class TestEndianStream
{
	public TestEndianStream() throws IOException
	{
		ByteArrayOutputStream baos = new ByteArrayOutputStream(1024);
		EndianDataOutputStream edos = new EndianDataOutputStream(baos);
		for( int i = 1; i > 0; i <<= 1 )
		{
			System.out.println("Writing " + Integer.toBinaryString(i));
			edos.writeInt( i );
		}
		for( int i = 0x11111111; i != 0; i <<= 1 )
		{
			System.out.println("Writing " + Integer.toBinaryString(i));
			edos.writeInt( i );
		}

		ByteArrayInputStream bais =
			new ByteArrayInputStream(baos.toByteArray());
		EndianDataInputStream edis = new EndianDataInputStream(bais);
		for( int i = 1; i > 0; i <<= 1 )
		{
			System.out.println("Reading " + Integer.toBinaryString(i));
			int val = edis.readInt();
			System.out.println("   Read " + Integer.toBinaryString(val));
			if( val != i )
				throw new RuntimeException( "Mismatched values: " + i + " != " + val );
		}
		for( int i = 0x11111111; i != 0; i <<= 1 )
		{
			System.out.println("Reading " + Integer.toBinaryString(i));
			int val = edis.readInt();
			System.out.println("   Read " + Integer.toBinaryString(val));
			if( val != i )
				throw new RuntimeException( "Mismatched values: " + i + " != " + val );
		}
		System.out.println("<<<PASSED>>>");
	}


	// Public methods
	public static void main( String args[] )
	{
		try
		{
			new TestEndianStream();
		}
		catch( Exception e )
		{
			e.printStackTrace(System.out);
		}
	}
}

