/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: EventGroup.java,v 1.21 2001/04/05 18:40:59 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.eventfs;

import java.util.*;
import java.lang.reflect.*;

public class EventGroup
{
	private String         name;
	private EventManager   em;
	private EventTypeTable eventTypes;
	private EventMap       eventMap;

	// Constructors
	public EventGroup() throws Exception
	{
		em = EventManager.getEventManager();
	}

	public EventGroup( EventGroup eg ) throws Exception
	{
		this();
		eventTypes = new EventTypeTable( eg.eventTypes );
	}

	public EventGroup( String groupName ) throws Exception
	{
		this();
		accessGroup( groupName );
	}


	// Public methods
	public void accessGroup( String groupName ) throws Exception
	{
		name = new String(groupName);
		eventTypes = new EventTypeTable(em.openGroupStats(this));
	}

	public EventGroupEvent createEvent( int mask ) throws Exception
	{
		// Find the name of the class
		String className = (String)eventMap.get( new Integer(mask) );

		if( className == null )
			throw new IllegalArgumentException("Unable to find mapping for mask '" + mask + "'");

		// Get a reference to the class object
		Class classRef = Class.forName(className);

		// Obtain the proper constructor
		Constructor constrRef =
		   classRef.getConstructor( new Class[] { EventGroup.class,
		                                          int.class } );
		// Create a new instance of the event
		Object o =
		   constrRef.newInstance( new Object[] { this,
		                                         new Integer(mask) } );
		return (EventGroupEvent)o;
	}

	public void registerEventMap( EventMap em ) throws InvalidEventMapException
	{
		eventMap = em;
		if( em.size() != eventTypes.size() )
			throw new InvalidEventMapException(em, "Size of event map (" + em.size() + " entries) does not match type table (" + eventTypes.size() + " entries)");

		// validate the entries
		Vector v = new Vector();
		for( Enumeration e = em.keys(); e.hasMoreElements(); )
		{
			String key = (String)e.nextElement();
			if( eventTypes.lookupMask(key) == null )
				throw new InvalidEventMapException(em, "No event '" + key + "' in type table");

			// Attempt to get class object
			String className = (String)em.get(key);
			try
			{
				Class c = Class.forName(className);
				if( !EventGroupEvent.class.isAssignableFrom(c) )
					throw new InvalidEventMapException(em, "Mapped events must be extended from com.eventfs.EventGroupEvent class");
			}
			catch( ClassNotFoundException cnfe )
			{
				throw new InvalidEventMapException(em, "Unable to find class '" + className + "'");
			}

			v.addElement(key);
		}

		// remap the entries to the mask
		for( Enumeration e = v.elements(); e.hasMoreElements(); )
		{
			String key = (String)e.nextElement();
			em.remap( key, eventTypes.lookupMask(key) );
		}
	}

	public boolean manages( EventTypeTable ett )
	{
		return eventTypes.contains( ett );
	}

	public String getName() { return name; }

	public int lookupMask( String tag )
	{
		return eventTypes.lookupMask(tag).intValue();
	}

	public String lookupTag( int mask )
	{
		return eventTypes.lookupTag(new Integer(mask));
	}

	public String toString() { return "EventGroup: " + name; }
}

