/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: EventManager.java,v 1.14 2001/04/05 18:40:59 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.eventfs;

import java.io.*;

public class EventManager
{
	protected String base;

	// Public methods
	public static EventManager getEventManager() throws ClassNotFoundException
	{
		String osName = System.getProperty("os.name");
		String className;
		if( osName.startsWith("Windows") )
			className = "com.eventfs.Win32EventManager";
		else if( osName.startsWith("Linux") )
			className = "com.eventfs.LinuxEventManager";
		else
			className = "com.eventfs.NativeEventManager";
		try
		{
			Class c = Class.forName(className);
			return (EventManager)c.newInstance();
		}
		catch( IllegalAccessException iae )
		{
			throw new ClassNotFoundException(className + ": IllegalAccessException" );
		}
		catch( InstantiationException ie )
		{
			throw new ClassNotFoundException(className + ": InstantiationException" );
		}
	}


	// Public methods
	public String constructPath( String groupName )
	{
		StringBuffer sb = new StringBuffer();
		return sb.append(base)
		         .append(System.getProperty("file.separator"))
		         .append(groupName).toString();
	}

	public String constructPath( String groupName, String streamName )
	{
		StringBuffer sb = new StringBuffer();
		return sb.append(constructPath(groupName))
		         .append(System.getProperty("file.separator"))
		         .append(streamName).toString();
	}

	public String constructPath( EventGroup group )
	{
		return constructPath( group.getName() );
	}

	public String constructPath( EventGroup group, String streamName )
	{
		return constructPath( group.getName(), streamName );
	}

	public String constructPath( EventGroup group, EventStream stream )
	{
		return constructPath( group.getName(), stream.getName() );
	}

	public InputStream openGroupStats( EventGroup group ) throws FileNotFoundException
	{
		StringBuffer sb = new StringBuffer();
		String path = sb.append(constructPath(group.getName()))
		                .append(System.getProperty("file.separator"))
		                .append("stats").toString();
		FileInputStream fis = new FileInputStream(path);
		return fis;
	}

	public InputStream openStream( EventStream stream ) throws FileNotFoundException
	{
		StringBuffer sb = new StringBuffer();
		String path =
		   sb.append(constructPath(stream.getEventGroup().getName()))
		     .append(System.getProperty("file.separator"))
		     .append(stream.getName())
		     .append(System.getProperty("file.separator"))
		     .append("data").toString();
		FileInputStream fis = new FileInputStream(path);
		return fis;
	}

	public InputStream openStreamStats( EventStream stream ) throws FileNotFoundException
	{
		EventGroup group = stream.getEventGroup();
		StringBuffer sb = new StringBuffer();
		String path = sb.append(constructPath(group.getName(),
		                                      stream.getName()))
		                .append(System.getProperty("file.separator"))
		                .append("stats").toString();
		FileInputStream fis = new FileInputStream(path);
		return fis;
	}
}

