/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: NativeEventManager_jni.c,v 1.10 2001/02/23 19:02:17 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#ifndef LINKAGE
#define LINKAGE IMPORT
#endif /* LINKAGE */

#include <errno.h>

#include "NativeEventManager_jni.h"

#ifdef PD_FREEBSD
#include <sys/param.h>
#include <sys/mount.h>
#endif

#ifdef PD_SOLARIS
#include <sys/fstyp.h>
#include <sys/fsid.h>
#include <sys/mnttab.h>
#endif


/*
 * JNI methods
 */
JNIEXPORT jstring JNICALL
Java_com_eventfs_NativeEventManager_getBasePath( JNIEnv *env, jobject obj )
{
   jstring path = NULL;

#ifdef PD_FREEBSD
   struct vfsconf vfc;
   if( !getvfsbyname( "eventfs", &vfc ) )
   {
      struct statfs *mntbuf;
      int            cnt;
      if( ( cnt = getmntinfo( &mntbuf, MNT_WAIT ) ) > 0 )
      {
         int i;
         for( i = 0; i < cnt; i++, mntbuf++ )
            if( mntbuf->f_type == vfc.vfc_typenum )
            {
               path = (*env)->NewStringUTF( env, mntbuf->f_mntonname );
               break;
            }
      }
   }
#elif defined(PD_SOLARIS)
   FILE *f;
   int   fsid = sysfs( GETFSIND, "eventfs" );
   if( ( fsid >= 0 ) &&
       ( ( f = fopen( "/etc/mnttab", "r" ) ) != NULL ) )
   {
      struct mnttab m;
      while( getmntent( f, &m ) == 0 )
         if( !strcmp( m.mnt_fstype, "eventfs" ) )
         {
            path = (*env)->NewStringUTF( env, m.mnt_mountp );
            break;
         }
      fclose( f );
   }
#else
#  error Missing implementation for com.eventfs.NativeEventManager.getBasePath()
#endif

   return path;
}

