/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: TestEventCreation.java,v 1.10 2001/04/05 18:41:01 stevek Exp $
 */

import java.io.*;
import java.util.*;
import com.eventfs.EventGroupEvent;
import com.eventfs.EventStream;
import com.eventfs.UnhandledEventException;

public class TestEventCreation extends TestEventFakeSetup
{
	// Constructors
	public TestEventCreation() throws Exception
	{
		super();

		// Open streams
		EventStream rawStream = openStream(TestConstants.RAW_STREAM);
		EventStream timedStream = openStream(TestConstants.TIMED_STREAM);
		EventStream randomStream = openStream(TestConstants.RANDOM_STREAM);

		// Construct a 'fake' event stream
		byte[] streamData = constructFakeStream();

		// Pass stream data to each stream, catching the
		//    appropriate number of exceptions for each stream

		// First, the 'raw' stream
		System.out.println("Passing stream data to 'raw' stream, expecting no exceptions");
		ByteArrayInputStream bais = new ByteArrayInputStream(streamData);
		int count = 0;
		try
		{
			while( bais.available() > 0 )
			{
				EventGroupEvent ege =
				   EventGroupEvent.readEvent(group, bais);

				if( !rawStream.manages(ege) )
					throw new UnhandledEventException(ege);
				System.out.print(++count);
				System.out.print("\t");
				System.out.println(ege.toString());
			}
			System.out.println("Read " + count + " events");
		}
		catch( EOFException e )
		{
			System.out.println("Read " + count + " events");
		}

		// Next, the 'timed' stream
		System.out.println("Passing stream data to 'timed' stream, expecting 4 exceptions");
		bais.reset();
		count = 0;
		int ecount = 0;
		try
		{
			while( bais.available() > 0 )
			{
				EventGroupEvent ege = null;
				try
				{
					ege = EventGroupEvent.readEvent(group, bais);

					if( !timedStream.manages(ege) )
						throw new UnhandledEventException(ege);
					System.out.print(++count);
					System.out.println("\t" + ege.toString());
				}
				catch( UnhandledEventException uee )
				{
					System.out.print("UnhandledEventException #" + ++ecount);
					System.out.println(": " + ege.toString());
				}
			}
			if( ecount != 4 )
				throw new RuntimeException("Expected 4 exceptions, encountered " + ecount);
			System.out.println("Read " + count + " events");
		}
		catch( EOFException e )
		{
			System.out.println("Read " + count + " events");
		}

		// Finally, the 'random' stream
		System.out.println("Passing stream data to 'random' stream, expecting 11 exceptions");
		bais.reset();
		count = 0;
		ecount = 0;
		try
		{
			while( bais.available() > 0 )
			{
				EventGroupEvent ege = null;
				try
				{
					ege = EventGroupEvent.readEvent(group, bais);

					if( !randomStream.manages(ege) )
						throw new UnhandledEventException(ege);
					System.out.print(++count);
					System.out.println("\t" + ege.toString());
				}
				catch( UnhandledEventException uee )
				{
					System.out.print("UnhandledEventException #" + ++ecount);
					System.out.println(": " + ege.toString());
				}
			}
			if( ecount != 11 )
				throw new RuntimeException("Expected 4 exceptions, encountered " + ecount);
			System.out.println("Read " + count + " events");
		}
		catch( EOFException e )
		{
			System.out.println("Read " + count + " events");
		}

		System.out.println("<<<PASSED>>>");
	}


	// Public methods
	public static void main( String args[] )
	{
		try
		{
			new TestEventCreation();
		}
		catch( Exception e )
		{
			e.printStackTrace(System.out);
		}
	}
}


