/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: TestEventDistribution.java,v 1.5 2001/04/05 18:41:01 stevek Exp $
 */

import java.io.*;
import com.eventfs.EventGroupEvent;
import com.eventfs.EventGroupEventListener;
import com.eventfs.EventStream;
import com.eventfs.UnhandledEventException;

public class TestEventDistribution extends TestEventFakeSetup
{
	protected int rawCount = 0;
	protected int timedCount = 0;
	protected int exceptions = 0;

	public TestEventDistribution() throws Exception
	{
		super();

		// Open streams
		EventStream rawStream = openStream(TestConstants.RAW_STREAM);
		EventStream timedStream = openStream(TestConstants.TIMED_STREAM);

		// Construct a 'fake' event stream
		byte[] streamData = constructFakeStream();

		// Add event listeners
		rawStream.addEventListener(
			new EventGroupEventListener() {
				public void eventGroupEventOccurred( EventGroupEvent e )
				{ rawCount++; }
			} );
		timedStream.addEventListener(
			new EventGroupEventListener() {
				public void eventGroupEventOccurred( EventGroupEvent e )
				{ timedCount++; }
			} );

		// Pass stream data to streams
		ByteArrayInputStream bais = new ByteArrayInputStream(streamData);
		while( bais.available() > 0 )
		{
			EventGroupEvent ege =
			   EventGroupEvent.readEvent(group, bais);

			// First, the 'raw' stream
			rawStream.eventGroupEventOccurred( ege );

			// Next the 'timed' stream
			try { timedStream.eventGroupEventOccurred( ege ); }
			catch( UnhandledEventException uee ) { exceptions++; }
		}

		// Check results
		System.out.println("Events on streams:");
		System.out.println("\t  raw: " + rawCount);
		System.out.println("\ttimed: " + timedCount);
		if( rawCount != 19 )
			throw new RuntimeException("Expected 19 events on 'raw' stream, received " + rawCount);
		if( timedCount != 15 )
			throw new RuntimeException("Expected 15 events on 'timed' stream, received " + timedCount);
		if( exceptions != 4 )
			throw new RuntimeException("Expected 4 exceptions, encountered " + exceptions);

		System.out.println("<<<PASSED>>>");
	}

	public static void main( String args[] )
	{
		try
		{
			new TestEventDistribution();
		}
		catch( Exception e )
		{
			e.printStackTrace(System.out);
		}
	}
}


