/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: TestEventFakeSetup.java,v 1.8 2001/04/05 18:41:01 stevek Exp $
 */

import java.io.*;
import java.util.*;
import java.util.zip.Adler32;
import com.endian.EndianDataOutputStream;
import com.eventfs.EventGroup;
import com.eventfs.EventMap;
import com.eventfs.EventStream;
import com.eventfs.EventStreamException;

public class TestEventFakeSetup
{
	// Protected data
	protected int        periodicCount = 0;
	protected Random     rand          = new Random();
	protected EventGroup group;


	// Constructors
	public TestEventFakeSetup() throws Exception
	{
		group = new EventGroup( TestConstants.GROUP );

		// Register event map
		EventMap em = new EventMap();
		em.put(TestConstants.NULL_TYPE, "TestNullEvent");
		em.put(TestConstants.PERIODIC_TYPE, "TestPeriodicEvent");
		em.put(TestConstants.RANDOM_TYPE, "TestRandomEvent");
		group.registerEventMap( em );
	}


	// Public methods
	public byte[] constructFakeStream() throws IOException
	{
		// Construct a 'fake' event stream
		ByteArrayOutputStream baos = new ByteArrayOutputStream(2048);
		writeNullEvent(baos);
		writeRandomEvent(baos);
		writeRandomEvent(baos);
		writePeriodicEvent(baos);
		writeNullEvent(baos);
		writeRandomEvent(baos);
		writeRandomEvent(baos);
		writeRandomEvent(baos);
		writeNullEvent(baos);
		writePeriodicEvent(baos);
		writePeriodicEvent(baos);
		writePeriodicEvent(baos);
		writeNullEvent(baos);
		writePeriodicEvent(baos);
		writeRandomEvent(baos);
		writePeriodicEvent(baos);
		writeRandomEvent(baos);
		writeRandomEvent(baos);
		writePeriodicEvent(baos);
		return baos.toByteArray();
	}

	public EventStream openStream( String name ) throws EventStreamException
	{
		EventStream es = new EventStream( group, name );
		return es;
	}

	public synchronized void writeEventHeader( OutputStream os, String eventName, int size ) throws IOException
	{
		EndianDataOutputStream edos = new EndianDataOutputStream(os);
		edos.writeInt(group.lookupMask(eventName));
		edos.writeInt(size);
	}

	public synchronized void writeChecksum( OutputStream os, byte[] data ) throws IOException
	{
		EndianDataOutputStream edos = new EndianDataOutputStream(os);
		Adler32 adler = new Adler32();
		adler.update(data);
		int chksum = (int)adler.getValue();
		edos.writeInt(chksum);
	}

	public synchronized void writeNullEvent( OutputStream os ) throws IOException
	{
		EndianDataOutputStream edos = new EndianDataOutputStream(os);
		writeEventHeader(edos, TestConstants.NULL_TYPE, 0);
		edos.writeInt(0);
	}

	public synchronized void writePeriodicEvent( OutputStream os ) throws IOException
	{
		writeEventHeader(os, TestConstants.PERIODIC_TYPE, 4);

		ByteArrayOutputStream baos = new ByteArrayOutputStream(16);
		EndianDataOutputStream edos = new EndianDataOutputStream(baos);
		edos.writeInt(++periodicCount);
		byte[] buf = baos.toByteArray();
		writeChecksum(os, buf);
		os.write(buf, 0, buf.length);
	}

	public synchronized void writeRandomEvent( OutputStream os ) throws IOException
	{
		int nbytes = (Math.abs(rand.nextInt()) % 50) + 1;
		writeEventHeader(os, TestConstants.RANDOM_TYPE, nbytes);
		byte[] buf = new byte[nbytes];
		for( int i = 0; i < nbytes; i++ )
			buf[i] = new Integer(rand.nextInt()).byteValue();
		writeChecksum( os, buf );
		os.write(buf, 0, nbytes);
	}
}


