/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: TestEventStreamThread.java,v 1.5 2001/04/05 18:41:01 stevek Exp $
 */

import java.io.*;
import com.eventfs.EventGroup;
import com.eventfs.EventGroupEvent;
import com.eventfs.EventGroupEventListener;
import com.eventfs.EventMap;
import com.eventfs.EventStream;

public class TestEventStreamThread
{
	protected int rawCount = 0;
	protected int timedCount = 0;
	protected int exceptions = 0;

	public TestEventStreamThread() throws Exception
	{
		EventGroup eg = new EventGroup(TestConstants.GROUP);

		// Register event map
		EventMap em = new EventMap();
		em.put(TestConstants.NULL_TYPE, "TestNullEvent");
		em.put(TestConstants.PERIODIC_TYPE, "TestPeriodicEvent");
		em.put(TestConstants.RANDOM_TYPE, "TestRandomEvent");
		eg.registerEventMap( em );

		// Open streams
		EventStream rawStream = new EventStream(eg, TestConstants.RAW_STREAM);
		EventStream timedStream = new EventStream(eg, TestConstants.TIMED_STREAM);

		// Register listeners
		rawStream.addEventListener( new EventGroupEventListener() {
			public void eventGroupEventOccurred( EventGroupEvent e )
			{ System.out.println("Raw Event: " + e.toString()); } } );
		timedStream.addEventListener( new EventGroupEventListener() {
			public void eventGroupEventOccurred( EventGroupEvent e )
			{ System.out.println("Timed Event: " + e.toString()); } } );

		// Start threads
		rawStream.listen();
		timedStream.listen();

		// Wait for conditions or timeout
		Thread.currentThread().sleep(60000);

		// Stop threads
		rawStream.stop();
		timedStream.stop();

		System.out.println("<<<PASSED>>>");
	}

	public static void main( String args[] )
	{
		try
		{
			new TestEventStreamThread();
		}
		catch( Exception e )
		{
			e.printStackTrace(System.out);
		}
	}
}


