/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: efsevents.cxx,v 1.9 2001/04/05 18:41:03 stevek Exp $
 */

#define _EFS 1
#include <configuration.h>
#include <pi_defs.h>

#include <time.h>

#include <eventfs.h>

#define EFSGROUP PD_MAKESTR("java event test")

#define PERIODIC_TIMEOUT     20000
#define MAX_RANDOM_TIMEOUT   120
#define MAX_RANDOM_PAYLOADSZ 8000


HANDLE g_hPeriodic;
HANDLE g_hRandom;

LONG g_lRandomTimeout = 1000L * MAX_RANDOM_TIMEOUT;
INT  g_iPeriodicCount = 0;


VOID CALLBACK EFSServerPeriodicTimer( LPVOID lpArg, DWORD dwLow, DWORD dwHigh );
VOID CALLBACK EFSServerRandomTimer  ( LPVOID lpArg, DWORD dwLow, DWORD dwHigh );


typedef struct _TIMER_ARG TIMER_ARG, * LPTIMER_ARG;
struct _TIMER_ARG
{
   u_long mask;
   HANDLE hTimer;
};


int
EFSServerRegisterEvents()
{
   int    error;
   u_long nulltype;
   u_long periodictype;
   u_long randomtype;

   error = eventfs_register_group( EFSGROUP );
   if( !error )
      error = eventfs_register_event_type( EFSGROUP,
                                           PD_MAKESTR("null"), &nulltype );
   if( !error )
      error = eventfs_register_event_type( EFSGROUP, PD_MAKESTR("periodic"),
                                           &periodictype );
   if( !error )
      error = eventfs_register_event_type( EFSGROUP, PD_MAKESTR("random"),
                                           &randomtype );
   if( !error )
      error = eventfs_register_stream( EFSGROUP, PD_MAKESTR("raw"),
                                       nulltype | periodictype | randomtype,
                                       FALSE );
   if( !error )
      error = eventfs_register_stream( EFSGROUP, PD_MAKESTR("timed"),
                                       periodictype | randomtype,
                                       FALSE );
   if( !error )
      error = eventfs_register_stream( EFSGROUP, PD_MAKESTR("random"),
                                       randomtype, FALSE );
   if( !error )
      error = eventfs_register_stream( EFSGROUP, PD_MAKESTR("periodic"),
                                       randomtype, FALSE );
   if( !error )
   {
      g_hPeriodic = CreateWaitableTimer( NULL, FALSE, NULL );
      if( g_hPeriodic )
      {
         static TIMER_ARG periodicArg;
	 periodicArg.mask = periodictype;
	 periodicArg.hTimer = g_hPeriodic;
         const LARGE_INTEGER li = {0,0};
         SetWaitableTimer( g_hPeriodic, &li, PERIODIC_TIMEOUT,
                           EFSServerPeriodicTimer, &periodicArg, FALSE );
      }
      srand( (unsigned)time(NULL) );
      g_hRandom = CreateWaitableTimer( NULL, TRUE, NULL );
      if( g_hRandom )
      {
         static TIMER_ARG randomArg;
	 randomArg.mask = randomtype;
	 randomArg.hTimer = g_hRandom;
         g_lRandomTimeout = 1000L * ( rand() % MAX_RANDOM_TIMEOUT );
         const LARGE_INTEGER li = {0,0};
         SetWaitableTimer( g_hRandom, &li, g_lRandomTimeout,
                           EFSServerRandomTimer, &randomArg, FALSE );
      }
   }

   if( error )
      (void) eventfs_destroy_group( EFSGROUP );
   return error;
}

int
EFSServerDestroyEvents()
{
   CancelWaitableTimer( g_hPeriodic );
   CloseHandle( g_hPeriodic );
   CancelWaitableTimer( g_hRandom );
   CloseHandle( g_hRandom );

   return eventfs_destroy_group( EFSGROUP );
}

VOID CALLBACK
EFSServerPeriodicTimer( LPVOID lpArg, DWORD dwLow, DWORD dwHigh )
{
   LPTIMER_ARG lpTimer = (LPTIMER_ARG)lpArg;

   g_iPeriodicCount++;
   int error = eventfs_post_event_by_types( EFSGROUP, lpTimer->mask,
                                            sizeof(g_iPeriodicCount),
                                            &g_iPeriodicCount );
   if( error )
   {
      _tprintf( _TEXT("EFSServerPeriodicTimer: event posting failed ")
                _TEXT("with error code %d"), error );
      CancelWaitableTimer( lpTimer->hTimer );
   }
}

VOID CALLBACK
EFSServerRandomTimer  ( LPVOID lpArg, DWORD dwLow, DWORD dwHigh )
{
   extern BOOL g_bRunning;
   LPTIMER_ARG lpTimer = (LPTIMER_ARG)lpArg;

   int size = (rand() % MAX_RANDOM_PAYLOADSZ) + 1;
   PUCHAR payload = (PUCHAR)LocalAlloc(LMEM_FIXED, size);
   if( payload == NULL )
   {
      _tprintf( _TEXT("EFSServerPeriodicTimer: LocalAlloc failed\n") );
      CancelWaitableTimer( lpTimer->hTimer );
   }
   else
   {
      UCHAR randChar = (UCHAR)(rand() % 256);
      for( int i = 0; i < size; i++ )
         payload[i] = randChar;

      int error = eventfs_post_event_by_types( EFSGROUP, lpTimer->mask, size,
                                               payload );
      LocalFree(payload);

      if( error )
      {
         _tprintf( _TEXT("EFSServerPeriodicTimer: event posting failed ")
                   _TEXT("with error code %d\n"), error );
         CancelWaitableTimer( lpTimer->hTimer );
      }
      else if( g_bRunning )
      {
         g_lRandomTimeout = 1000L * ( rand() % MAX_RANDOM_TIMEOUT );
         const LARGE_INTEGER li = {0,0};
         SetWaitableTimer( g_hRandom, &li, g_lRandomTimeout,
                           EFSServerRandomTimer, lpArg, FALSE );
      }
   }
}

