/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: efsserver.cxx,v 1.9 2001/04/05 18:41:03 stevek Exp $
 */

#include <configuration.h>
#include <pi_defs.h>

#include <eventfs.h>
#include <pipesrv.h>


// External functions
//
extern "C" {
int EFSLINKAGE eventfs_init( void );
int EFSLINKAGE eventfs_uninit( void );
}

extern int EFSServerRegisterEvents( void );
extern int EFSServerDestroyEvents( void );


// Prototypes
//
BOOL WINAPI ControlHandler   ( DWORD dwCtrlType );
int         EFSServerShutdown( LPPIPESRV_PIPE_INFO pInfo );


// This event is signaled when the service should end
//
BOOL g_bRunning = TRUE;

BOOL g_bDebugging = FALSE;



eventfs_interface_t g_efsIFace =
{
   EFSPURENAME(eventfs_construct_stream),
   EFSPURENAME(eventfs_destroy_group),
   EFSPURENAME(eventfs_destroy_stream),
   EFSPURENAME(eventfs_flush_stream),
   EFSPURENAME(eventfs_get_event_mask),
   EFSPURENAME(eventfs_init),
   EFSPURENAME(eventfs_post_event),
   EFSPURENAME(eventfs_post_event_by_types),
   EFSPURENAME(eventfs_register_event_type),
   EFSPURENAME(eventfs_register_group),
   EFSPURENAME(eventfs_register_stream),
   EFSPURENAME(eventfs_uninit)
};


int
main( int argc, char *argv[] )
{
   // Setup a console control handler
   //
   SetConsoleCtrlHandler( ControlHandler, TRUE );

   // Pass debugging flag to pipe service library
   //
   g_bPipeSrvDebugging = g_bDebugging;

   // Create the server stop event
   //
   g_hPipeSrvStopEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
   if( g_hPipeSrvStopEvent  == NULL )
   {
      _tprintf( _TEXT("CreateEvent failed.\n") );
      return 0;
   }

   // Initialize the EventFS interface
   //
   _tprintf( _TEXT("EventFS Server: Initializing interface...\n") );
   eventfs_interface_init();

   // Register eventfs interface handlers
   //
   _tprintf( _TEXT("EventFS Server: Registering handlers...\n") );
   if( !eventfs_interface_register( &g_efsIFace ) )
   {
      _tprintf( _TEXT("<<< FAILED >>>\n") );
      goto cleanup;
   }

   // Initialize EventFS
   //
   eventfs_init();


   // Construct group and streams
   //
   _tprintf( _TEXT("EventFS Server: Registering group and streams...\n") );
   if( EFSServerRegisterEvents() )
      goto cleanup;


   // Construct a pipe for signalling shutdown
   //
   {
      LPPIPESRV_QUEUE_INFO pqi = new PIPESRV_QUEUE_INFO();
      pqi->bReadOnly = TRUE;
      pqi->lpszPipeName = _TEXT("\\\\.\\pipe\\efstest\\shutdown");
      pqi->nInstances = 0;
      pqi->nThreads = 0;
      pqi->nActive = 0;
      pqi->fOpen = EFSServerShutdown;
      pqi->fClose = NULL;
      pqi->fProcessing = NULL;
      pqi->hEvents[0] = g_hPipeSrvStopEvent ;
      pqi->hEvents[PIPESRV_PIPE_AVAILABLE_EVENT] =
         CreateEvent( NULL, TRUE, FALSE, NULL );
      pqi->hEvents[PIPESRV_PIPE_ACTIVE_EVENT] =
         CreateSemaphore( NULL, 0, 10000, NULL );
      pqi->hEvents[PIPESRV_WORKER_DIED] =
         CreateEvent( NULL, TRUE, FALSE, NULL );
      pqi->hEvents[PIPESRV_ACCEPTOR_DIED] =
         CreateEvent( NULL, TRUE, FALSE, NULL );
      InitializeCriticalSection( &pqi->critSec );
      pqi->thread = _beginthread( PipeSrvAcceptThread, 0, pqi );


      // Wait for shutdown
      //
      _tprintf( _TEXT("EventFS Server: Started.\n") );
      while ( 1 )
      {
         DWORD dwWait = WaitForSingleObjectEx( g_hPipeSrvStopEvent , INFINITE,
                                               TRUE );

         if( dwWait == WAIT_OBJECT_0 ) // server stop signaled
            break;
      }

      EFSServerDestroyEvents();

      // Wait for thread to stop
      //
      while( pqi->thread )
         WaitForSingleObject( pqi->hEvents[PIPESRV_ACCEPTOR_DIED], INFINITE );
      delete pqi;
   }

cleanup:

   if( g_hPipeSrvStopEvent  )
      CloseHandle( g_hPipeSrvStopEvent  );

   eventfs_interface_uninit();
   eventfs_uninit();

   return 0;
}

int
EFSServerShutdown( LPPIPESRV_PIPE_INFO pInfo )
{
   _tprintf( _TEXT("EventFS Server: Shutting down...\n") );
   g_bRunning = FALSE;
   if( g_hPipeSrvStopEvent  )
      SetEvent( g_hPipeSrvStopEvent  );

   return FALSE;
}


BOOL WINAPI ControlHandler ( DWORD dwCtrlType )
{
   switch( dwCtrlType )
   {
      case CTRL_BREAK_EVENT:  // use Ctrl+C or Ctrl+Break to simulate
      case CTRL_C_EVENT:      // SERVICE_CONTROL_STOP in debug mode
         _tprintf( _TEXT("EventFS Server: Shutting down...\n") );
         if( g_hPipeSrvStopEvent  )
            SetEvent( g_hPipeSrvStopEvent  );
         return TRUE;

      default:
         break;
   }
   return FALSE;
}

