/*! \file eventfs.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: eventfs_machine.h,v 1.9 2001/02/23 19:02:16 stevek Exp $
 *
 * \brief   Header file for the machine-dependent parts of eventfs extension
 * \author  Steve Kiernan
 * \version $Revision: 1.9 $
 * \date    $Date: 2001/02/23 19:02:16 $
 */

#ifndef __EVENTFS_MACHINE_H
#define __EVENTFS_MACHINE_H 1

#include <pdpi/pi_defs.h>

#ifdef PD_IN_KERNEL

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

extern u_int eventfs_debug_mask;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/* Functions which should be implemented by system-dependent part */
int  eventfs_machine_init                ( void );
int  eventfs_machine_uninit              ( void );
void eventfs_machine_flush               ( struct eventfs_stream * );

/*
 * Functions which could be implemented by system-dependant part if
 * it makes life easier.  (or macroed away in the header file)
 */
void eventfs_machine_destroy_group       (struct eventfs_group  *);
void eventfs_machine_destroy_stream      (struct eventfs_stream *);
void eventfs_machine_register_group      (struct eventfs_group  *);
void eventfs_machine_register_stream     (struct eventfs_stream *);
void eventfs_machine_post_event_to_stream(struct eventfs_stream *,
                                            struct eventfs_entry * );

#ifdef __cplusplus
}
#endif

#endif /* PD_IN_KERNEL */

#endif /* __EVENTFS_MACHINE_H */

