/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_freebsd.h,v 1.23 2001/02/23 19:02:18 stevek Exp $
 */

#ifndef __EVENTFS_FREEBSD_H
#define __EVENTFS_FREEBSD_H 1

#define   _PATH_EVENTFS   "/event"      /* Default mountpoint */

#include "eventfs_ioctl.h"

#if defined(KERNEL) || defined(_KERNEL)

#include <sys/malloc.h>
#include <sys/mount.h>
#include <sys/vnode.h>

struct eventfs_opennode
{
   LIST_ENTRY(eventfs_opennode)             efsopen_hash;
   int                                      efsopen_refcount;
   struct eventfs_node                     *efsopen_node;
   struct eventfs_stream_entry             *efsopen_curentry;
   int                                      efsopen_state;
   struct sigio                            *efsopen_sigio;
   off_t                                    efsopen_offset;
};

#define EFS_NODE_ASYNC 0x0001

typedef enum
{
   Eroot, Egroup, Estream, Esdata, Esstats, Egstats
}
efstype;

struct eventfs_node
{
   struct eventfs_node                     *efs_next;
   efstype                                  efs_type;
   u_int32_t                                efs_fileno;
   struct vnode                            *efs_node;
   pid_t                                    efs_lockowner;
   union
   {
      struct eventfs_group                 *efs_group;
      struct eventfs_stream                *efs_stream;
   }
   un;
};

#define UIO_MX sizeof(struct dirent)
#define EVENTFS_FILENO(gid,sid,sfno,type) \
	((type) == Eroot) ?	1 : \
	((type) == Egroup) ?	(((int) (gid)+1) << 1) : \
	((type) == Estream) ? (((int) (sid)+1) << 9) + (((int) (gid)+1) << 1) : \
      (((int) (sfno)+1) << 25) + (((int) (sid)+1) << 9) + (((int) (gid)+1) << 1)

#define VT_EVENTFS (VT_NFS + 1)

#define VTOEVENTFS(vp)  ((struct eventfs_node *)(vp)->v_data)
#define EVENTFSTOV(efs) ((efs)->efs_node)

#define EFS_MALLOC(v,t,s) MALLOC(v, t, s, M_TEMP, M_WAITOK)
#define EFS_FREE(v) FREE(v, M_TEMP)

#define EFS_DEBUG_MD_VNOPS	0x0010
#define EFS_DEBUG_MD_VFS	0x0020
#define EFS_DEBUG_MD_READ	0x0040
#define EFS_DEBUG_MD_POLL	0x0080
#define EFS_DEBUG_MD_OPEN	0x0100
#define EFS_DEBUG_MD_DATA	0x0200
#define EFS_DEBUG_MD_ASYNC	0x0400


int eventfs_allocvp      __P(( struct mount *, struct vnode **, void *,
                               efstype ));
int eventfs_check_stream __P(( struct eventfs_node * ));


struct eventfs_opennode *eventfs_lookup_opennode __P(( struct proc *p,
                                                       struct eventfs_node *efs ));


#if PD_FREEBSD < 4
#define eventfs_quotactl ((int (*) __P((struct mount *, int, uid_t, caddr_t, \
       struct proc *)))eopnotsupp)
#define eventfs_sync ((int (*) __P((struct mount *, int, struct ucred *, \
       struct proc *)))nullop)
#define eventfs_fhtovp ((int (*) __P((struct mount *, struct fid *, \
       struct vnode **)))eopnotsupp)
#define eventfs_vptofh ((int (*) __P((struct vnode *, struct fid *)))eopnotsupp)
#define eventfs_vget ((int (*) __P((struct mount *, ino_t, struct vnode **))) \
       eopnotsupp)
#else
#define eventfs_quotactl   vfs_stdquotactl
#define eventfs_sync       vfs_stdsync
#define eventfs_vget       vfs_stdvget
#define eventfs_fhtovp     vfs_stdfhtovp
#define eventfs_checkexp   vfs_stdcheckexp
#define eventfs_vptofh     vfs_stdvptofh
#define eventfs_extattrctl vfs_stdextattrctl
#endif
extern vop_t **eventfs_vnodeop_p;

int  eventfs_root     __P(( struct mount *, struct vnode ** ));
void eventfs_sigasync __P(( struct eventfs_stream * ));

int eventfs_sprintf __P(( pd_char_t *buf, const pd_char_t *format, ... ));
#define eventfs_sprintf  sprintf

#endif /* KERNEL */

#endif /* __EVENTFS_H */

