/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_vfsops.c,v 1.13 2001/02/23 19:02:18 stevek Exp $
 *
 * Event Filesystem
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <eventfs.h>
#include <eventfs_machine.h>
#include "eventfs_freebsd.h"

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/conf.h>
#include <sys/proc.h>
#include <sys/vnode.h>
#include <sys/mount.h>
#include <sys/malloc.h>

dev_t rrootdev = NODEV;

static int eventfs_mount   __P(( struct mount *mp, char *path, caddr_t data,
                                 struct nameidata *ndp, struct proc *p ));
static int eventfs_start   __P(( struct mount *mp, int flags, struct proc *p ));
static int eventfs_unmount __P(( struct mount *mp, int mntflags,
                                 struct proc *p ));
static int eventfs_statfs  __P(( struct mount *mp, struct statfs *sbp,
                                 struct proc *p ));

/*
 * Mount the Event filesystem
 */
static int
eventfs_mount( mp, path, data, ndp, p )
   struct mount *mp;
   char *path;
   caddr_t data;
   struct nameidata *ndp;
   struct proc *p;
{
   u_int size;

   /*
    * Update is a no-op
    */
   if( mp->mnt_flag & MNT_UPDATE )
      return (EOPNOTSUPP);

   mp->mnt_flag |= MNT_LOCAL;
   mp->mnt_data = 0;
   vfs_getnewfsid( mp );

   (void) copyinstr( path, mp->mnt_stat.f_mntonname, MNAMELEN, &size );

   bzero( mp->mnt_stat.f_mntonname + size, MNAMELEN - size );

   size = sizeof("eventfs") - 1;
   bcopy("eventfs", mp->mnt_stat.f_mntfromname, size );
   bzero( mp->mnt_stat.f_mntfromname + size, MNAMELEN - size );

   (void)eventfs_statfs( mp, &mp->mnt_stat, p );

   return (0);
}

static int
eventfs_start( mp, flags, p )
   struct mount *mp;
   int flags;
   struct proc *p;
{
   return (0);
}

static int
eventfs_unmount( mp, mntflags, p )
   struct mount *mp;
   int mntflags;
   struct proc *p;
{
   int error;
   int flags = 0;

   if (mntflags & MNT_FORCE)
      flags |= FORCECLOSE;

#ifdef EVENTFS_DIAGNOSTICS
   if( eventfs_debug_mask & EFS_DEBUG_MD_VFS )
      printf( "eventfs_unmount: flushing\n" );
#endif /* EVENTFS_DIAGNOSTICS */

   error = vflush(mp, rootvp, flags);
   if (error)
      return (error);

   return (0);
}

int
eventfs_root( mp, vpp )
   struct mount *mp;
   struct vnode **vpp;
{
#ifdef EVENTFS_DIAGNOSTICS
   if( eventfs_debug_mask & EFS_DEBUG_MD_VFS )
      printf( "eventfs_root\n" );
#endif /* EVENTFS_DIAGNOSTICS */
   return (eventfs_allocvp( mp, vpp, 0, Eroot ));
}

static int
eventfs_statfs( mp, sbp, p )
   struct mount *mp;
   struct statfs *sbp;
   struct proc *p;
{
#ifdef EVENTFS_DIAGNOSTICS
   if( eventfs_debug_mask & EFS_DEBUG_MD_VFS )
      printf( "eventfs_statfs\n" );
#endif /* EVENTFS_DIAGNOSTICS */

   sbp->f_bsize = PAGE_SIZE;
   sbp->f_iosize = PAGE_SIZE;
   sbp->f_blocks = 1;
   sbp->f_bfree = 0;
   sbp->f_bavail = 0;
   sbp->f_files = 0;
   sbp->f_ffree = 0;

   if( sbp != &mp->mnt_stat )
   {
      sbp->f_type = mp->mnt_vfc->vfc_typenum;
      bcopy( &mp->mnt_stat.f_fsid, &sbp->f_fsid, sizeof(sbp->f_fsid) );
      bcopy( mp->mnt_stat.f_mntonname, sbp->f_mntonname, MNAMELEN );
      bcopy( mp->mnt_stat.f_mntfromname, sbp->f_mntfromname, MNAMELEN );
   }

   return (0);
}


static struct vfsops eventfs_vfsops =
{
   eventfs_mount,
   eventfs_start,
   eventfs_unmount,
   eventfs_root,
   eventfs_quotactl,
   eventfs_statfs,
   eventfs_sync,
   eventfs_vget,
   eventfs_fhtovp,
#if PD_FREEBSD >= 4
   eventfs_checkexp,
#endif
   eventfs_vptofh,
   (int(*)(struct vfsconf *)) eventfs_init,
#if PD_FREEBSD >= 4
   (int(*)(struct vfsconf *)) eventfs_uninit,
   eventfs_extattrctl
#endif
};

VFS_SET(eventfs_vfsops, eventfs, VFCF_SYNTHETIC);

