/*! \file libeventfs.h 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: libeventfs.h,v 1.7 2001/02/23 19:02:19 stevek Exp $
 *
 * \brief   Header file for EventFS interface library
 *          
 * \author  Steve Kiernan, Douglas Kilpatrick
 * \version $Revision: 1.7 $
 * \date    $Date: 2001/02/23 19:02:19 $
 */

#ifndef  __LIBEVENTFS_H
#define  __LIBEVENTFS_H  1

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __EVENTFS_H
#include <eventfs.h>
#endif


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct eventfs_event eventfs_event, *eventfs_event_t;

struct typelist_data {
    u_long                        mask;
    LPTSTR                        name;
};
typedef struct eventfs_event_group_typelist {
    size_t                        count;
    struct typelist_data          list[0];
} eventfs_event_group_typelist;

typedef struct eventfs_event_group {
    LPTSTR                        name;
    unsigned long                 mask;
    eventfs_event_group_typelist *list;
} eventfs_event_group, *eventfs_event_group_t;

typedef struct eventfs_event_stream {
    eventfs_event_group_t         group;
    pd_fd_obj                     stream;
    u_long                        mask;
} eventfs_event_stream, *eventfs_event_stream_t;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/* eventfsEvent */
eventfs_event_group_t  LINKAGE eventfsEventGetGroup       ( const LPTSTR name );
eventfs_event_stream_t LINKAGE eventfsEventGetStreamByFile( const LPTSTR path );



/* eventfsEventGroup */
eventfs_event_stream_t LINKAGE
   eventfsEventGroupGetStream      ( eventfs_event_group_t group, u_long mask );

eventfs_event_stream_t LINKAGE
   eventfsEventGroupGetStreamByName( eventfs_event_group_t group,
                                     LPTSTR stream_name);

const LPTSTR LINKAGE
   eventfsEventGroupGetTypeName    ( eventfs_event_group_t group, u_long mask );

u_long  LINKAGE
   eventfsEventGroupGetType        ( eventfs_event_group_t group,
                                     LPTSTR type_name);

u_long  LINKAGE
   eventfsEventGroupGetTypeList    ( eventfs_event_group_t group);

const LPTSTR LINKAGE
   eventfsEventGroupGetName        ( eventfs_event_group_t group);

void LINKAGE
   eventfsEventGroupPostEvent      ( eventfs_event_group_t group,
                                     eventfs_event_t event );

void LINKAGE
   eventfsEventGroupRelease        ( eventfs_event_group_t group );



/* eventfsEventStream */
eventfs_event_group_t LINKAGE
   eventfsEventStreamGetGroup      ( eventfs_event_stream_t stream);

u_long  LINKAGE
   eventfsEventStreamGetTypeList   ( eventfs_event_stream_t stream);

pd_fd_obj LINKAGE
   eventfsEventStreamGetWaitObject ( eventfs_event_stream_t stream);

#if EVENTFS_EVENT_PROCESS
void LINKAGE
   eventfsEventStreamProcess       ( eventfs_event_stream_t stream,
                                     volatile int *quit_var);
#endif

eventfs_event_t LINKAGE
   eventfsEventStreamGetNextEvent  ( eventfs_event_stream_t stream);

void LINKAGE
   eventfsEventStreamRelease       ( eventfs_event_stream_t stream);



/* eventfsEvent */
unsigned long   LINKAGE  eventfsEventGetTypes   ( eventfs_event_t event );
eventfs_event_t LINKAGE  eventfsEventClone      ( eventfs_event_t event );
void            LINKAGE *eventfsEventGetData    ( eventfs_event_t event );
unsigned long   LINKAGE  eventfsEventGetSize    ( eventfs_event_t event );
void            LINKAGE  eventfsEventRelease    ( eventfs_event_t event );


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

/* Event group accessors */
#define eventfsEventGroupGetTypeList(group) ((group)->mask)
#define eventfsEventGroupGetName(group)     ((group)->name)
/* Event stream accessors */
#define eventfsEventStreamGetGroup(stream)      ((stream)->group)
#define eventfsEventStreamGetTypeList(stream)   ((stream)->mask)
#define eventfsEventStreamGetWaitObject(stream) ((stream)->stream)
/* Event accessors */
#define eventfsEventGetTypes(event) (((event)==NULL)?0:(event)->efsevent_type)
#define eventfsEventGetSize(event)  (((event)==NULL)?0:(event)->efsevent_size)
#define eventfsEventGetData(event)  (((event)==NULL)?0: \
                                    ((void*)((char*)(event)+sizeof(*(event)))))

#define eventfsEventHasBuffer(event) (((event)==NULL)?0: \
                                    ((event)->efsevent_size > sizeof(*(event))))

#ifdef __cplusplus
}
#endif

#endif /* !__LIBEVENTFS_H */

