/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: test.c,v 1.3 2001/02/23 19:02:19 stevek Exp $
 */

#define _WSS 1
#define WR_KERNEL_MODULE 1
#define WR_HAS_EVENTFS 1
#define MODULE 1

#include <linux/module.h>

#include "../../eventfs.h"

#include "../eventfs_linux.h"

#include <linux/proc_fs.h>

static volatile int gohome = 0;

static struct timer_list ep_timer;

static struct wait_queue *sleepq;

void ep_post(unsigned long arg)
{
    eventfs_post_event("testing","t1",0,0);
    wake_up(&sleepq);
}

static int spin_and_post(void *arg)
{
    MOD_INC_USE_COUNT;

    while (!gohome) {
        init_timer(&ep_timer);
        ep_timer.function = ep_post;
        ep_timer.data = 0;
        ep_timer.expires = jiffies+HZ;

        add_timer(&ep_timer);
        interruptible_sleep_on(&sleepq);
    }
    MOD_DEC_USE_COUNT;
    return 0;
}

int killfile_read( char *b, char **bl, off_t o, int l, int *eof, void *d)
{
    /* procfile contract stuff.
     */
    *bl = b;
    memcpy(b, "asdf\n", 6);
    *eof = 1;

    if (gohome == 0) {
        gohome = 1;
        wake_up(&sleepq);
    }

    if (o > 0)
        return 0;
    return 6;
}

struct proc_dir_entry killfile = {
    0, /* low_ino */
    11, /* length of filename */
    "killtestmod", /* name */
    S_IFREG|S_IRUGO, /* mode */
    1, 0, 0, 6, /* nlink, uid, gid, size */
    0, /* inode_ops */
    0, /* get_info: older style of proc. */
    0, /* fill_inode */
    0, 0, 0, /* next, parent, subdir */
    0, /* data */
    killfile_read /* read_proc */
        /* rest 0: write, readlink, use count, deleted */
};

int init_module(void)
{
    u_long                      type;
    int                         pid;

    eventfs_register_group("testing");

    eventfs_register_event_type("testing","t1", &type);

    eventfs_register_stream("testing","stream",type);

    proc_register(&proc_root, &killfile);

    printf(KERN_INFO"starting kernel thread.\n");

    pid = kernel_thread(spin_and_post, 0, CLONE_FS);
    return 0;
}

void cleanup_module(void)
{
    proc_unregister(&proc_root, killfile.low_ino);
    return;
}


