#!/usr/local/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: efsdebug.pl,v 1.6 2001/04/05 18:41:08 stevek Exp $

use strict;

my $EFS_DEBUG_ALL       = 0xffff;
my $EFS_DEBUG_GROUP     = 0x0001;
my $EFS_DEBUG_STREAM    = 0x0002;
my $EFS_DEBUG_EVENT     = 0x0004;
my $EFS_DEBUG_MD_MASK   = 0xfff0;
my $EFS_DEBUG_MD_VNOPS  = 0x0010;
my $EFS_DEBUG_MD_VFS    = 0x0020;
my $EFS_DEBUG_MD_READ   = 0x0040;
my $EFS_DEBUG_MD_DATA   = 0x0080;
my $EFS_DEBUG_MD_BADOPS = 0x0100;
my $EFS_DEBUG_MD_OPEN   = 0x0200;

sub queryFlags {
   my $flags = `sysctl -n vfs.eventfs.debugmask`;
}

sub setFlags {
   my( $flags ) = @_;

   `sysctl -w vfs.eventfs.debugmask=$flags`;
}

sub printFlags {
   my( $msg, $flags ) = @_;

   print $msg . "Flags:\n";
   print "\tGROUP  - group manipulation\n" if( $flags & $EFS_DEBUG_GROUP );
   print "\tSTREAM - stream manipulation\n" if( $flags & $EFS_DEBUG_STREAM );
   print "\tEVENT  - event manipulation\n" if( $flags & $EFS_DEBUG_EVENT );
   print "\tVNOPS  - vnode operations\n" if( $flags & $EFS_DEBUG_MD_VNOPS );
   print "\tVFS    - VFS operations\n" if( $flags & $EFS_DEBUG_MD_VFS );
   print "\tREAD   - read syscall\n" if( $flags & $EFS_DEBUG_MD_READ );
   print "\tDATA   - reading from stream\n" if( $flags & $EFS_DEBUG_MD_DATA );
   print "\tBADOPS - bad vnode operations\n" if( $flags & $EFS_DEBUG_MD_BADOPS );
   print "\tOPEN   - open syscall\n" if( $flags & $EFS_DEBUG_MD_OPEN );
}

my $flags = queryFlags;

if( !@ARGV ) {
   printFlags "Set ", $flags;
   exit 0;
}

foreach( @ARGV ) {
   my $negate = 0;

   $_ = uc($_);

   /^-H$/ && do { printFlags "Available ", $EFS_DEBUG_ALL; exit 0; };

   (s/^-//, $negate = 1) if /^-/;

   my $bitmask;
   /^ALL$/    && do { $bitmask = $EFS_DEBUG_ALL; };
   /^GROUP$/  && do { $bitmask = $EFS_DEBUG_GROUP; };
   /^STREAM$/ && do { $bitmask = $EFS_DEBUG_STREAM; };
   /^EVENT$/  && do { $bitmask = $EFS_DEBUG_EVENT; };
   /^VNOPS$/  && do { $bitmask = $EFS_DEBUG_MD_VNOPS; };
   /^VFS$/    && do { $bitmask = $EFS_DEBUG_MD_VFS; };
   /^READ$/   && do { $bitmask = $EFS_DEBUG_MD_READ; };
   /^DATA$/   && do { $bitmask = $EFS_DEBUG_MD_DATA; };
   /^BADOPS$/ && do { $bitmask = $EFS_DEBUG_MD_BADOPS; };
   /^OPEN$/   && do { $bitmask = $EFS_DEBUG_MD_OPEN; };
   /^MD$/     && do { $bitmask = $EFS_DEBUG_MD_MASK; };

   die "Unknown flag: $_" if !$bitmask;

   if( $negate ) {
      $flags &= ~$bitmask;
   }
   else {
      $flags |= $bitmask;
   }
}

setFlags $flags;
printFlags "Written ", $flags;

0;

