/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_mod.c,v 1.10 2001/02/23 19:02:20 stevek Exp $
 */

#include <configuration.h>
#include <pdpi/pi_defs.h>

#include <sys/systm.h>
#include <sys/modctl.h>
#include <sys/vfs.h>
#include <sys/time.h>
#include <sys/errno.h>

#include <eventfs.h>
#include "eventfs_solaris.h"

extern struct mod_ops mod_fsops;

struct vfsops eventfs_vfsops =
{
   eventfs_mount,
   eventfs_unmount,
   eventfs_root,
   eventfs_statvfs,
   eventfs_sync,
   eventfs_vget,
   eventfs_mountroot,
   eventfs_swapvp
};

static struct vfssw eventfs_vfssw =
{
   "eventfs",
   eventfs_solaris_init,
   &eventfs_vfsops,
   0
};

static struct modlfs modlfs =
{
   &mod_fsops,
   "Event File System v0.01",
   &eventfs_vfssw
};

static struct modlinkage modlinkage =
{
   MODREV_1,
   {
      &modlfs,
      NULL
   }
};

static int module_keepcnt = 0;

int
_init()
{
   int error = mod_install( &modlinkage );

#ifdef EVENTFS_DIAGNOSTICS
   printf( "_init(): %d\n", error );
#endif /* EVENTFS_DIAGNOSTICS */

   return (error);
}

int
_fini()
{
   if( module_keepcnt != 0 )
      return (EBUSY);

   return( mod_remove( &modlinkage ) );
}

int
_info( struct modinfo *modinfop )
{
   int error = mod_info( &modlinkage, modinfop );

   printf( "eventfs: _info: module loaded at address 0x%x, size 0x%x\n",
           (unsigned int) modinfop->mi_base, modinfop->mi_size );

   return error;
}

int
eventfs_keep()
{
   return( ++module_keepcnt );
}

int
eventfs_release()
{
   return( --module_keepcnt );
}

