/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: eventfs_solaris.h,v 1.18 2001/02/23 19:02:20 stevek Exp $
 */

#ifndef __EVENTFS_SOLARIS_H
#define __EVENTFS_SOLARIS_H 1

#define   _PATH_EVENTFS   "/event"      /* Default mountpoint */

#include "eventfs_ioctl.h"

#ifdef _KERNEL
#include <sys/dirent.h>
#include <sys/param.h>
#include <sys/kmem.h>
#include <sys/mount.h>
#include <sys/mutex.h>
#include <sys/vfs.h>
#include <sys/vnode.h>

#include <pdpi/queue.h>

struct eventfs_opennode
{
   LIST_ENTRY(eventfs_opennode)             efsopen_hash;
   int                                      efsopen_refcount;
   int                                      efsopen_flags;
   int                                      efsopen_offset;
   struct eventfs_node                     *efsopen_node;
   struct eventfs_stream_entry             *efsopen_curentry;
};

typedef enum
{
   Eroot, Egroup, Estream, Esdata, Esstats, Egstats
}
efstype;

struct eventfs_node
{
   struct eventfs_node                     *efs_next;
   efstype                                  efs_type;
   uint32_t                                 efs_nodeid;
   struct vnode                            *efs_node;
   kmutex_t                                 efs_lock;
   kcondvar_t                               efs_cv;
   union
   {
      struct eventfs_group                 *efs_group;
      struct eventfs_stream                *efs_stream;
   }
   un;
};

#define NAMELEN 240

#define EVENTFS_FILENO(gid,sid,sfno,type) \
   ((type) == Eroot) ?   2 : \
   ((type) == Egroup) ?   (((int) (gid)+1)<<2) : \
   ((type) == Estream) ? (((int) (sid)+1)<<10) + (((int) (gid)+1)<<2) : \
        (((int) (sfno)+1)<<26) + (((int) (sid)+1)<<10) + (((int) (gid)+1)<<2)

#define VTOEVENTFS(vp)  ((struct eventfs_node *)(vp)->v_data)
#define EVENTFSTOV(efs) ((efs)->efs_node)

#define EFS_MALLOC(v,t,s) \
   do { \
      (v) = (t) kmem_zalloc( (s) + sizeof(u_long), KM_SLEEP ); \
      *((u_long *)(v)) = (s) + sizeof(u_long); \
      (v) = (t) ((u_long *)(v) + 1); \
   } while(0)

#define EFS_FREE(v) \
   kmem_free( (void *)((u_long *)(v) - 1), *((u_long *)(v) - 1) )

#define EFS_DEBUG_MD_VNOPS      0x0010
#define EFS_DEBUG_MD_VFS        0x0020
#define EFS_DEBUG_MD_READ       0x0040
#define EFS_DEBUG_MD_DATA       0x0080
#define EFS_DEBUG_MD_BADOPS     0x0100
#define EFS_DEBUG_MD_OPEN       0x0200


int  eventfs_allocvp      __P(( struct vfs *, struct vnode **, void *,
                                efstype ));
int  eventfs_check_stream __P(( struct eventfs_node * ));
void eventfs_setflags     __P(( struct eventfs_node *, int ));
int  eventfs_getflags     __P(( struct eventfs_node * ));

int eventfs_mount     __P(( struct vfs *, struct vnode *, struct mounta *,
                            struct cred * ));
int eventfs_unmount   __P(( struct vfs *, struct cred * ));
int eventfs_root      __P(( struct vfs *, struct vnode ** ));
int eventfs_statvfs   __P(( struct vfs *, struct statvfs64 * ));
int eventfs_sync      __P(( struct vfs *, short, struct cred * ));
int eventfs_vget      __P(( struct vfs *, struct vnode **, struct fid * ));
int eventfs_mountroot __P(( struct vfs *, enum whymountroot ));
int eventfs_swapvp    __P(( struct vfs *, struct vnode **, char * ));

int eventfs_solaris_init __P(( struct vfssw *, int ));

struct eventfs_opennode *eventfs_lookup_opennode __P(( struct proc *,
                                                       struct eventfs_node * ));

int eventfs_sprintf __P(( pd_char_t *buf, const pd_char_t *format, ... ));
#define eventfs_sprintf  sprintf

#endif /* _KERNEL */

#endif /* __EVENTFS_SOLARIS_H */

