/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: mount_eventfs.c,v 1.6 2001/02/23 19:02:21 stevek Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/mnttab.h>

static void
usage( char *progname )
{
   fprintf( stderr, "Usage: %s <special> <path>\n", progname );
   exit( 1 );
}

int
updatemnttab( char *mountp )
{
   int ret;
   int mnttablock;
   FILE *mnttabfd;
   struct mnttab mp;
   struct timeval tp;
   char timebuf[15];

   if( ( mnttablock = open( "/etc/.mnttab.lock", O_WRONLY|O_CREAT, 0 ) ) < 0 )
   {
      perror( "open: /etc/.mnttab.lock" );
      exit( 1 );
   }

   if( ( ret = lockf( mnttablock, F_LOCK, 0 ) ) )
   {
      perror( "lockf" );
      exit( 1 );
   }

   if( ( mnttabfd = fopen( "/etc/mnttab", "a" ) ) == NULL )
   {
      perror( "fopen: /etc/mnttab");
      exit( 1 );
   }

   if( ( ret = lockf( fileno( mnttabfd ), F_LOCK, 0 ) ) )
   {
      perror( "lockf" );
      exit( 1 );
   }

   gettimeofday( &tp, NULL );
   snprintf( timebuf, 15, "%d", tp.tv_sec );

   mp.mnt_special = "eventfs";
   mp.mnt_mountp = mountp;
   mp.mnt_fstype = "eventfs";
   mp.mnt_mntopts = "ro";
   mp.mnt_time = timebuf;

   if( ( ret = putmntent( mnttabfd, &mp ) ) == EOF )
   {
      perror( "putmntent" );
      exit( 1 );
   }

   lockf( fileno( mnttabfd ), F_ULOCK, 0 );
   fclose( mnttabfd );

   lockf( mnttablock, F_ULOCK, 0 );
   close( mnttablock );

   return (0);
}

int
main( int argc, char *argv[] )
{
   int ret;

   if( argc != 3 )
      usage( argv[0] );

   if( strncmp( argv[1], "eventfs", 8 ) )
      usage( argv[0] );

   if( ( ret = mount( "event", argv[2], MS_FSS, "eventfs", NULL, 0 ) ) ) {
      perror( "mount" );
      return ret;
   }

   updatemnttab( argv[2] );

   return 0;
}

