/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: Grid.java,v 1.16 2001/04/05 18:41:10 stevek Exp $
 */
package com.nailabs.ui;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
 
public class Grid extends Panel implements ItemSelectable
{
	//initialize the variables
	private boolean       marginFlag         = true;
	private boolean       headerFlag         = true;
	private boolean       editFlag           = true;
	private boolean       selectEditFlag     = true;
	private boolean       livedragFlag       = false;
	private boolean       initialized        = false;
	private boolean       columnWidthsSet    = false;

	private int           borderFlag         = GridRow.BOTH_BORDER;

	private boolean       haveFocus          = false;
	private boolean       tabFocus           = false;

	private	boolean       mouseDown          = false;
	private	boolean       draggable          = false;
	private	boolean       highLight          = false;
	private	boolean       redraw             = false;
	private	long          last;

	private	int           currentRow;
	private	int           currentColumn;
	private	int           prevRow;
	private	int           prevColumn;
	private	int           selRow             = 0;
	private int           selColumn          = 0;
	private	int           startX;
	private	int           startY;
	private int           mouseX;
	private int           mouseY;
	private	int           left               = 0;
	private	int           top                = 0;
	private	int           leftShift;
	private	int           topShift;
	private	int           imgWidth;
	private	int           imgHeight;
	private	int           curImgWidth;
	private	int           curImgHeight;
	private	int           screenWidth;
	private	int           screenHeight;
	private	int           dragGridWidth;
	private	int           gridWidth;
	private	int           gridHeight;
	private	int           marginWidth        = 20;
	private	int           headerHeight       = 20;
	private	int           rowHeight          = 25;
	private	int           rowCount;
	private	int           columnCount;
	private	int           maxRows            = 20;
	private	int           maxCols            = 50;
	private	int           defColumnWidth     = -1;
	private	int           horScrollUnit;
	private	int           verticalScrollUnit;
	private	int           hSbHeight;
	private	int           vSbWidth;
	private	int           draggableColumn;

	private	int[]         colWidths;
	private	int[]         yPos;
	private	int[]         xPos;

	private Vector        gridEventListeners = new Vector();
	private Vector        itemEventListeners = new Vector();

	private	Vector        headers;
	private	Vector	      rowObjects;

	private	Component     component;
	private	Scrollbar     hSb;
	private	Scrollbar     vSb;
	private	Graphics      bg;
	private Graphics      headerBg;
	private	Graphics      marginBg;
	private	FontMetrics   fntMetrics;
	private Font          font               = new Font( "Dialog", Font.PLAIN, 12 );
	private	Image         bim;
	private Image         headerIm;
	private	Image         marginIm;
	private	Insets        in;

	private	Color         textColor          = Color.black;
	private	Color         bgColor            = Color.white;

	private	Color         hlTextColor        = Color.white;
	private	Color         hlBgColor          = Color.blue;

	private	Color         headerColor1       = Color.black;
	private	Color         headerColor2       = Color.gray;
	private	Color         headerColor3       = Color.white;
	private	Color         headerColor4       = Color.lightGray;

	public static final int HEADER_COLOR1    = 6;
	public static final int HEADER_COLOR2    = 7;
	public static final int HEADER_COLOR3    = 8;
	
	public static final int GRID             = 0;
	public static final int ROW              = 1;
	public static final int COLUMN           = 2;
	public static final int CELL             = 3;

	public static final int	BG_COLOR         = 0;
	public static final int	TEXT_COLOR       = 1;
	public static final int	HL_BG_COLOR      = 2;
	public static final int	HL_TEXT_COLOR    = 3;
	public static final int	BORDER_COLOR     = 4;	


	/**
	 * Grid constructor.
	 * @param title[] java.lang.String is the array of grid headers.
	 */
	public Grid( String[] title )
	{
		super();
		setLayout( null );

		//create the scrollbars
		hSb = new Scrollbar( Scrollbar.HORIZONTAL );
		vSb = new Scrollbar( Scrollbar.VERTICAL );

		add( hSb );
		add( vSb );

		rowObjects = new Vector();
		headers = new Vector();

		//create the header 
		if( title != null )
		{
			int len;
			int index;
			columnCount = title.length;
			colWidths = new int[maxCols];
			for( int i = 0; i < maxCols; i++ )
				colWidths[i] = 0;
			yPos = new int[maxRows];
			xPos = new int[maxCols];
			GridCell cell = new GridCell();
			headers.addElement( cell );
			for( index = 1; index <= columnCount; index++ )
			{
				cell = new GridCell();
				cell.bgColor = Color.lightGray;
				cell.textColor = Color.black;
				cell.setValue( title[index -1] );
				headers.addElement( cell );
			}
			cell = new GridCell();
			cell.bgColor = Color.lightGray;
			headers.addElement( cell );
			GridHeaderRow headerRow = new GridHeaderRow( headers );
			rowObjects.addElement( headerRow );
		}	

		final Grid myself = this;
		addFocusListener( new FocusListener()
		{
			public void focusGained( FocusEvent e )
			{
				focusCell();
				haveFocus = true;
				tabFocus = false;
			}

			public void focusLost( FocusEvent e )
			{
				haveFocus = false;
				if( tabFocus )
				{
					GridRow erow = (GridRow)rowObjects.elementAt( selRow );
					erow.highlight( false );		
					erow.invalidate( true );
					redraw = true;
					repaint();
					tabFocus = false;
				}
			}
		} );
		addKeyListener( new KeyAdapter()
		{
			public void keyPressed( KeyEvent evt )
			{
				switch( evt.getKeyCode() )
				{
					case KeyEvent.VK_UP:
						//prev row
						prevRow = currentRow;
						prevColumn = currentColumn;
						do
						{
							if( currentRow == 1 )
							{
								currentRow = prevRow;
								return;
							}

							currentRow--;
						}
						while( isEditable( currentRow, currentColumn ) == false );
						break;
					case KeyEvent.VK_DOWN:
						//prev row
						prevRow = currentRow;
						prevColumn = currentColumn;
						do
						{
							if( currentRow == rowCount )
							{
								currentRow = prevRow;
								return;
							}

							currentRow++;
						}
						while( isEditable( currentRow, currentColumn ) == false );
						break;
					case KeyEvent.VK_LEFT:
						//prev cell
						if( editFlag && !selectEditFlag )
						{
							prevRow = currentRow;
							prevColumn = currentColumn;
							do
							{
								if( currentColumn == 1 )
								{
									currentColumn = prevColumn;
									return;
								}

								//if first column in a row, go to previous row
								if( currentColumn == 1 )
								{
									currentRow--;
									currentColumn = columnCount;
								}
								else
									currentColumn--;
							}
							while( isEditable( currentRow, currentColumn ) == false );
						}
						else
							return;
						break;
					case KeyEvent.VK_RIGHT:
						//last cell
						if( editFlag && !selectEditFlag )
						{
							prevRow = currentRow;
							prevColumn = currentColumn;
							do
							{
								if( currentColumn == columnCount )
								{
									currentColumn = prevColumn;
									return;
								}

								//if last column in a row, go to next row
								if( currentColumn == columnCount )
								{
									currentRow++;
									currentColumn = 1;
								}	
								else 
									currentColumn++;
							}
							while( isEditable( currentRow, currentColumn ) == false );
						}
						else
							return;
						break;
					case KeyEvent.VK_TAB:
						tabFocus = true;
						break;
					case KeyEvent.VK_ENTER:
						break;
					default:
						return;
				}

				focusCell();
			}
		} );
		addMouseListener( new MouseAdapter()
		{
			public void mouseClicked( MouseEvent evt )
			{
				if( evt.getClickCount() == 2 )
				{
					int x = evt.getX();
					int y = evt.getY();
					x += leftShift;
					y += topShift;
					Point p = getCellPosition( x, y );

					if( y <= headerHeight )
						processGridEvent( new GridEvent( myself, GridEvent.COLUMN_DOUBLE_CLICKED, p.x, 0 ) );
					else if( marginFlag && ( x <= marginWidth ) )
						processGridEvent( new GridEvent( myself, GridEvent.ROW_DOUBLE_CLICKED, 0, p.y ) );
					else
						processGridEvent( new GridEvent( myself, GridEvent.CELL_DOUBLE_CLICKED, p.x, p.y ) );
				}
			}

			public void mousePressed( MouseEvent evt )
			{
				int x = evt.getX();
				int y = evt.getY();
				x += leftShift;
				y += topShift;
				mouseDown = true;

				last = evt.getWhen();
			}

			public void mouseReleased( MouseEvent evt )
			{
				int x = evt.getX();
				int y = evt.getY();

				if( draggable || ( y <= headerHeight ) )
				{
					//clicked on header
					if( draggable )
					{
						computeHScroll();
						bim = null;
						headerIm = null;
						repaint();

						processGridEvent( new GridEvent( myself, GridEvent.COLUMN_RESIZED, draggableColumn, 0 ) );
					}
					else
					{
						x += leftShift;
						y += topShift;
						Point p = getCellPosition( x, y );

						processGridEvent( new GridEvent( myself, GridEvent.COLUMN_CLICKED, p.x, p.y ) );
					}
				}
				else if( marginFlag && ( x <= marginWidth ) )
				{
					//clicked on left margin ( highlight shifting )
					x += leftShift;
					y += topShift;
					Point p = getCellPosition( x, y );
					currentRow = p.y;
					currentColumn = p.x;
					GridRow row = (GridRow)rowObjects.elementAt( currentRow );
					if( haveFocus )
						row.highlight( true );
					else
						row.highlight( false );
					row.invalidate( true );
					if( selRow != currentRow )
					{
						row = (GridRow)rowObjects.elementAt( selRow );
						row.highlight( false );
						row.invalidate( true );
						selRow = currentRow;
						processItemEvent( new ItemEvent( myself, ItemEvent.ITEM_STATE_CHANGED, myself, ItemEvent.SELECTED ) );
					}
					redraw = true;
					repaint();

					processGridEvent( new GridEvent( myself, GridEvent.ROW_CLICKED, 0, currentRow ) );
				}
				else
				{
					//clicked in a cell
					x += leftShift;
					y += topShift;
					Point p = getCellPosition( x, y );
		
					prevRow       = currentRow;
					prevColumn    = currentColumn;
					currentRow    = p.y;
					currentColumn = p.x;

					requestFocus();

					focusCell();

					processItemEvent( new ItemEvent( myself, ItemEvent.ITEM_STATE_CHANGED, myself, ItemEvent.SELECTED ) );

					if( editFlag )
						processGridEvent( new GridEvent( myself, GridEvent.CELL_CLICKED, currentColumn, currentRow ) );
					else
						processGridEvent( new GridEvent( myself, GridEvent.ROW_CLICKED, 0, currentRow ) );
		
				}
				mouseDown = false;
				draggable = false;
			}
		} );
		addMouseMotionListener( new MouseMotionListener()
		{
			public void mouseDragged( MouseEvent evt )
			{
				int x = evt.getX();
				int y = evt.getY();

				x += leftShift;
				y += topShift;

				mouseX = x;
				mouseY = y;
	
				if( mouseDown && draggable )
				{
					if( x > xPos[draggableColumn] )
					{
						colWidths[draggableColumn] += x - ( xPos[draggableColumn] +
						                                    colWidths[draggableColumn] );
						respace();
					}	
					if( livedragFlag )
					{
						computeHScroll();
						bim = null;
						headerIm = null;
					}
					else if( ( component != null ) && component.isVisible() )
						component.setVisible( false );

					repaint();
				}
			}

			public void mouseMoved( MouseEvent evt )
			{
				int x = evt.getX();
				int y = evt.getY();
				boolean cursorChanged = false;

				x += leftShift;
				if( !mouseDown )
					draggable = false;
				if( y <= headerHeight )
				{
					GridRow row = (GridRow)rowObjects.firstElement();

					for( int i=1;i <= columnCount; i++ )
					{
						if( java.lang.Math.abs( x - ( colWidths[i] + xPos[i] ) ) <= 2 )
						{
							if( row.getCell( i ).isResizable() )
							{
								dragGridWidth = gridWidth;
								cursorChanged = true;
								draggable = true;
								draggableColumn = i;
							}
						}
					}
					if( cursorChanged )
						changeCursor( Cursor.E_RESIZE_CURSOR );
					else
						changeCursor( Cursor.DEFAULT_CURSOR );
					return;
				}
				else
					changeCursor( Cursor.DEFAULT_CURSOR );

				draggable = false;
			}
		} );
		hSb.addAdjustmentListener( new AdjustmentListener()
		{
			public void adjustmentValueChanged( AdjustmentEvent evt )
			{
				left = evt.getValue();
				computeHScroll();
				repaint();
			}
		} );
		vSb.addAdjustmentListener( new AdjustmentListener()
		{
			public void adjustmentValueChanged( AdjustmentEvent evt )
			{
				top = evt.getValue();
				computeVScroll();
				repaint();
			}
		} );
	}

	/**
	 * Grid constructor comment.
	 * @param title 	java.lang.String[] is the array of headers.
	 * @param margin 	boolean if true, display margin else do not display
	 * margin
	 * @param edit 	boolean if true, sets the grid editable.
	 */
	public Grid( String[] title, boolean margin, boolean edit )
	{
		this( title );
		marginFlag = margin;
		editFlag = edit;
	}

	/**
	 * Appends a new column to the grid.
	 * @return boolean true if new column is added. else false.
	 * @param title java.lang.String is the header of the new column
	 * @param column java.lang.String[] is the contents of the new
	 * column
	 */
	public boolean addColumn( String title, String[] column )
	{
		GridCell    cell;
		GridRow	    row;
		TextField comp;

		if( title != null )
		{
			//add a new header cell
			cell = new GridCell();
			cell.setValue( title );
			headers.addElement( cell );

			//append a new cell to each row
			for( int i= 1; i <= rowCount; i++ )
			{
				row = (GridRow)rowObjects.elementAt( i );
				cell = new GridCell();
				comp = new TextField();
				//comp.setInsets( new Insets( 0, 0, 0, 0 ) );
				final Grid myself = this;
				comp.addComponentListener( new ComponentAdapter()
				{
					public void componentHidden( ComponentEvent e )
					{
						myself.requestFocus();
					}
				} );
				comp.addKeyListener( new KeyAdapter()
				{
					public void keyPressed( KeyEvent e )
					{
						GridRow erow = (GridRow)rowObjects.elementAt( currentRow );
						GridCell ecell = erow.getCell( currentColumn );

						switch( e.getKeyCode() )
						{
							case KeyEvent.VK_ENTER:
								ecell.setValue( getValue( currentRow, currentColumn ) );
								processGridEvent( new GridEvent( myself, GridEvent.CELL_CHANGED, currentColumn, currentRow ) );
							case KeyEvent.VK_ESCAPE:
								e.getComponent().setVisible( false );
								if( selectEditFlag )
								{
									ecell.highlight( true );
									ecell.invalidate( true );
								}

								selRow = 0;
								selColumn = 0;

								erow.invalidate( true );
								redraw = true;
								repaint();
								break;
							default:
								return;
						}

						haveFocus = false;
						focusCell();
					}
				} );
				cell.setComponent( comp );
				if( column != null )
				{
					cell.setValue( column[i-1] );
					comp.setText( column[i-1] );
				}	
				row.addCell( cell );
			}	
			columnCount++;
			computeHScroll();
			bim = null;
			headerIm = null;
			repaint();
			return true;
		}
		else	
			return false;
	}

	/**
	 * add a grid event listener
	 */
	public synchronized void addGridListener( GridListener l )
	{
		synchronized( gridEventListeners )
		{
			if( !gridEventListeners.contains( l ) )
				gridEventListeners.addElement( l );
		}
	}

	/**
	 * add an item event listener
	 */
	public synchronized void addItemListener( ItemListener l )
	{
		synchronized( itemEventListeners )
		{
			if( !itemEventListeners.contains( l ) )
				itemEventListeners.addElement( l );
		}
	}

	/**
	 * gets selected objects
	 */
	public Object[] getSelectedObjects()
	{
		Object[] selected = new Object[1];

		selected[0] = getSelectedRow();
		if( selected[0] == null )
			selected[0] = getSelectedCell();

		return selected;
	}

	/**
	 * addes a new row to the grid. 
	 * @return GridRow row object if addRow is successful.
	 * @param row java.lang.String[] is the contents of new row. set
	 * this value as null if you want to add an empty row.
	 */
	public GridRow addRow( String[] row )
	{
		GridCell    cell;
		GridRow     rowObj;
		Vector      newRow = new Vector();
		TextField comp; 

		//create the margin
		cell = new GridCell();
		newRow.addElement( cell );

		//create cells
		for( int i=1; i <= columnCount; i++ )
		{
			cell = new GridCell();
			setProperties( cell );
			if ( row != null )
				comp = new TextField( row[i-1] );
			else
				comp = new TextField( "" );
			//comp.setInsets( new Insets( 0, 0, 0, 0 ) );
			final Grid myself = this;
			comp.addComponentListener( new ComponentAdapter()
			{
				public void componentHidden( ComponentEvent e )
				{
					myself.requestFocus();
				}
			} );
			comp.addKeyListener( new KeyAdapter()
			{
				public void keyPressed( KeyEvent e )
				{
					GridRow erow = (GridRow)rowObjects.elementAt( currentRow );
					GridCell ecell = erow.getCell( currentColumn );

					switch( e.getKeyCode() )
					{
						case KeyEvent.VK_ENTER:
							ecell.setValue( getValue( currentRow, currentColumn ) );
							processGridEvent( new GridEvent( myself, GridEvent.CELL_CHANGED, currentColumn, currentRow ) );
						case KeyEvent.VK_ESCAPE:
							e.getComponent().setVisible( false );
							if( selectEditFlag )
							{
								ecell.highlight( true );
								ecell.invalidate( true );
							}

							selRow = 0;
							selColumn = 0;

							erow.invalidate( true );
							redraw = true;
							repaint();
							break;
						default:
							return;
					}

					haveFocus = false;
					focusCell();
				}
			} );
			cell.setComponent( comp );
			if ( row != null )
				cell.setValue( row[i-1] );
			else
				cell.setValue( "" );
			newRow.addElement( cell );
		}	

		cell = new GridCell();
		newRow.addElement( cell );

		//add this row
		rowObj = new GridRow( newRow, borderFlag, marginFlag );
		rowObjects.addElement( rowObj );
		
		rowCount++;

		//rearrange the vertical scrollbar
		computeVScroll();

		//set to render the image
		bim = null;

		//set to render margin
		marginIm = null;
		repaint();
		return rowObj;
	}

	/**
	 * addes more than on row to the grid.
	 * @return boolean if successfull
	 * @param rows java.lang.String[][] the array of rows.
	 */
	public boolean addRows( String[][] rows )
	{
		int rowLength;
		boolean flag = true;
		if( rows != null )
		{
			rowLength = rows.length;
			for( int i=0; i < rowLength && flag; i++ )
				if( addRow( rows[i] ) == null )
					flag = false;
		}	
		return flag;
	}

	/**
	 * called to adjust component of a cell. This method is only called
	 * for editable grid.
	 */
	private void adjustCell()
	{
		//check whether new position is not over header. 
		if( ( yPos[currentRow]-topShift ) >= headerHeight )
		{
			//make sure component will fit into the display area
			int compWidth, compHeight;
			Dimension cdim = component.getPreferredSize();
			int wantHeight = Math.max( cdim.height, rowHeight );
			int yshift = 0;
			if( cdim.height > rowHeight )
				yshift = ( int )( ( ( cdim.height - rowHeight )+0.5 )/2 );
			compWidth = ( screenWidth > ( xPos[currentColumn]-leftShift + colWidths[currentColumn-( marginFlag?1:0 )] ) ) ? colWidths[currentColumn-( marginFlag?1:0 )] : ( screenWidth - ( xPos[currentColumn]-leftShift ) );
			compHeight = ( screenHeight > ( yPos[currentRow]-topShift + wantHeight ) ) ? wantHeight : ( screenHeight - ( yPos[currentRow]-topShift )+yshift );
			if( !marginFlag || ( ( xPos[currentColumn] - leftShift ) >= marginWidth ) )
				component.setBounds( xPos[currentColumn]-leftShift, yPos[currentRow]-topShift-yshift, compWidth, compHeight );
			else
				component.setBounds( marginWidth, yPos[currentRow]-topShift-yshift, colWidths[currentColumn-( marginFlag?1:0 )]-leftShift, compHeight );
		}
		else
		{
			//hide the component
			component.setVisible( false );
		}	
	}

	/**
	 * changes the cursor.
	 * @param cursorType int
	 */
	private void changeCursor( int cursorType ) { setCursor( Cursor.getPredefinedCursor( cursorType ) ); }

	/**
	 * calculates the column width of each column. if defColumnWidth is
	 * not set, width is calculated according to best fit.
	 */
	private void computeColumnWidths()
	{
		// set the marginWidth
		colWidths[0] = (marginFlag?marginWidth:0);

		if( columnWidthsSet )
		{
			gridWidth = colWidths[0];
			for( int i = 1; i <= columnCount; i++ )
				gridWidth += colWidths[i];
		}
		else if( defColumnWidth != -1 )
		{
			// if there is a default column width
			gridWidth = colWidths[0];
			for( int i = 1; i <= columnCount; i++ )
			{
				colWidths[i] = defColumnWidth;
				gridWidth += defColumnWidth;
			}	
		}	
		else
		{
			// column width is best fit
			if( !initialized )
			{
				gridWidth = -1;
				return;
			}	
			Vector column;
			gridWidth = colWidths[0];
			for( int i=1; i < columnCount; i++ )
			{
				column = getColumn( i );
				colWidths[i] = getMaxWidth( column );
				gridWidth += colWidths[i];
			}	
		}	
		colWidths[columnCount+1] = screenWidth - gridWidth;
		if( colWidths[columnCount+1] < 0 )
			colWidths[columnCount+1] = 0;
	}

	/**
	 * rearranges the position and value of horizontal scroll bar.
	 */
	protected void computeHScroll()
	{
		if( !initialized ) return;

		//size of the screen
		int val = java.lang.Math.round( screenWidth / horScrollUnit );

		//max size of the grid
		int max = java.lang.Math.round( imgWidth / horScrollUnit );

		hSb.setValues( left, val, 0, max );
		val = ( int )( val * 0.9 );
		if( val < 1 ) val = 1;
		hSb.setBlockIncrement( val );
	}

	/**
	 * rearranges the position and value of vertical scroll bar.
	 */
	protected void computeVScroll()
	{
		if( !initialized ) return;
		
		int val = java.lang.Math.round( ( screenHeight-headerHeight ) / verticalScrollUnit );
		int max = java.lang.Math.round( ( imgHeight-headerHeight ) / verticalScrollUnit );
		vSb.setValues( top, val, 0, max );
		val = ( int )( val * 0.9 );
		if( val < 1 ) val = 1;
		vSb.setBlockIncrement( val );
	}

	/**
	 * deletes column identified by index. Note that index starts from 1.
	 * @return boolean
	 * @param index int column index to be deleted
	 */
	public boolean deleteColumn( int index )
	{
		if( ( index <= 0 ) || ( index > columnCount ) )
			return false;

		GridRow row;
		for( int i=0; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			row.deleteCell( index+1 );
		}	
		columnCount--;
		if( selColumn == index )
		{
			selRow = selColumn = 0;
			processItemEvent( new ItemEvent( this, ItemEvent.ITEM_STATE_CHANGED, this, ItemEvent.DESELECTED ) );
		}
		if( currentColumn == index )
			currentRow = currentColumn = 0;
		else if( currentColumn > index )
			currentColumn--;
		computeHScroll();
		bim = null;
		repaint();
		return true;
	}

	/**
	 * deletes row identified by index from grid. 
	 * @return boolean
	 * @param index int is the index of the row to be deleted. This
	 * starts from 1.
	 */
	public boolean deleteRow( int index )
	{
		if ( ( index <= 0 )||( index > rowCount ) )
			return false;

		rowObjects.removeElementAt( index );
		rowCount--;
		if( selRow == index )
		{
			selRow = selColumn = 0;
			processItemEvent( new ItemEvent( this, ItemEvent.ITEM_STATE_CHANGED, this, ItemEvent.DESELECTED ) );
		}
		if( currentRow == index )
			currentRow = currentColumn = 0;
		else if( currentRow > index )
			currentRow--;
		computeVScroll();
		bim = null;
		repaint();
		return true;
	}

	/**
	 * draws the grid margin.
	 * @param g java.awt.Graphics
	 */
	private void drawMargin( Graphics g )
	{
		int y;
		int y1;

		if( marginFlag == false ) return;

		//draw left vertical line
		g.setColor( headerColor1 );
		g.drawLine( startX, startY, startX, screenHeight );
		if( marginIm == null )
		{
			marginIm = createImage( marginWidth, gridHeight );
			marginBg = marginIm.getGraphics();
			
			y1 = headerHeight;	
			for( int i=0; i <= rowCount; i++ )
			{
				y  = yPos[i]+1;
				marginBg.setColor( Color.lightGray );
				marginBg.fillRect( 0, y, marginWidth, ( y1-y ) );
				marginBg.setColor( headerColor3 );
				marginBg.drawLine( 0, y, marginWidth, y );
				marginBg.drawLine( 0, y, 0, y1 );
				marginBg.setColor( headerColor1 );
				marginBg.drawLine( marginWidth-1, y, marginWidth-1, y1 );
				marginBg.drawLine( 0, y1, marginWidth-1, y1 );
				y1 += rowHeight;
			}
		}
		topShift = top*verticalScrollUnit;
		topShift = ( screenHeight-imgHeight ) > -topShift ? ( imgHeight-screenHeight ) : topShift;
		int actualY = startY-topShift;
		g.drawImage( marginIm, startX+1, actualY, this );

		//draw the cell at 0, 0
		g.setColor( Color.lightGray );
		g.fillRect( startX+1, 0, marginWidth, headerHeight );
	
		g.setColor( headerColor3 );
		g.drawLine( startX+1, headerHeight, marginWidth, headerHeight );
		g.drawLine( startX+1, 0, startX+1, headerHeight );
	
		g.setColor( headerColor1 );
		g.drawLine( marginWidth, 0, marginWidth, headerHeight );
		g.drawLine( startX+1, 0, marginWidth, 0 );
		g.drawLine( startX+1, headerHeight, marginWidth, headerHeight );

		//draw bottom line
		int ht = ( imgHeight > gridHeight ) ? gridHeight : imgHeight;
		g.setColor( headerColor1 );
		g.drawLine( startX, ht, imgWidth-1, ht);
	}

	public void focusCell()
	{
		GridRow row = (GridRow)rowObjects.elementAt( selRow );
		row.highlight( false );		
		row.invalidate( true );
		redraw = true;
		repaint();

		if( editFlag )
		{
			if( ( component != null ) && component.isVisible() )
			{
				row = (GridRow)rowObjects.elementAt( selRow );
				GridCell cell = row.getCell( selColumn );
				cell.resetValue();
				component.setVisible( false );
				remove( component );
			}

			component = getComponent( currentRow, currentColumn );
			if( component != null )
				component.setVisible( false );

			row = (GridRow)rowObjects.elementAt( currentRow );
			GridCell cell = row.getCell( currentColumn );
			if( ( ( ( selRow == currentRow ) && ( selColumn == currentColumn ) &&
			      !selectEditFlag && haveFocus ) || selectEditFlag ) && !tabFocus &&
			    isEditable( currentRow, currentColumn ) )
			{
				if( ( component != null ) && !component.isVisible() )
				{
					add( component );
					adjustCell();
					component.setVisible( true );
					component.requestFocus();
				}
			}
			else if( !selectEditFlag )
			{
				row.highlight( false );
				cell.highlight( true );
				cell.invalidate( true );
				row.invalidate( true );
			}
			else
			{
				row.highlight( true );
				row.invalidate( true );
			}
			selRow = currentRow;
			selColumn = currentColumn;
			redraw = true;
			repaint();
		}
		else
		{
			row = (GridRow)rowObjects.elementAt( currentRow );
			row.highlight( true );
			row.invalidate( true );
			selRow = currentRow;
			redraw = true;
			repaint();
			requestFocus();
		}
	}

	/**
	 * returns the content of the cell at the specified column and
	 * row as a String.
	 * @return java.lang.String is the content of the cell
	 * @param column int is the index of the column. Note that 1<= index <= columnCount.
	 * @param row int is the index of the row. Note that 1<= index <= rowCount.
	 */
	public String getCell( int column, int row )
	{
		if( ( row<0 ) || ( row> rowCount ) ||
		    ( column<1 ) || ( column > columnCount ) )
			return null;
			
		GridRow gridRow = (GridRow)rowObjects.elementAt( row );
		return new String( gridRow.getCell( column ).getValue() );
	}

	/**
	 * returns the row and column of the cell at point ( x, y ).
	 * @return Point is the row, column value of the cell.
	 * @param x int is the x cordinate
	 * @param y int is the y cordinate
	 */
	private Point getCellPosition( int x, int y )
	{
		Point posn = new Point( 0, 0 );
		
		for( int i=columnCount; i > 0; i-- )
		{
			if( x >= xPos[i] )
			{
				posn.x = i;
				break;
			}	
		}	
		
		for( int i=rowCount; i > 0; i-- )
		{
			if( y >= yPos[i] )
			{
				posn.y = i;
				break;
			}	
		}	
	
		return posn;
	}

	/**
	 * get the elements in column as a cell.
	 * @return java.util.Vector
	 * @param index int
	/**/
	private Vector getColumn( int index )
	{
		if( ( index < 0 ) || ( index > columnCount ) )
			return null;

		Vector v = new Vector();
		GridRow   row;
		GridCell  cell;
		for( int i=1; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			cell = row.getCell( index );
			v.addElement( cell );
		}		
		return v;
	}

	/**
	 * returns the component for the cell at ( row, column ).
	 * @return java.awt.Component is the component at ( row, column )
	 * @param row int is the row 
	 * @param column int is the column
	 */
	public Component getComponent( int row, int column )
	{
		GridRow currRow;
		GridCell cell;
		
		if( ( row <= 0 ) || ( row > rowCount ) ||
		    ( column <= 0 ) || ( column > columnCount ) )
			return null;
		currRow = (GridRow)rowObjects.elementAt( row );
		cell = currRow.getCell( column );
		return cell.getComponent();
	}

	/**
	 * returns value of the component at ( row, column ).
	 * @return java.lang.String
	 */
	private String getComponentValue( int row, int column )
	{
		Component cmp = getComponent( row, column );
		if( cmp instanceof TextField )
			return ( ( TextField )cmp ).getText();
		else if( cmp instanceof Choice )
			return ( ( Choice )cmp ).getSelectedItem();
		else
			return null;
	}

	/**
	 * returns maximum width of the column.
	 */
	private int getMaxWidth( Vector column )
	{
		if( ( column == null ) || ( bg == null ) )
			return 0;

		GridCell cell;
		int      cellWidth;
		int      len       = column.size();
		int      width     = 0;

		for( int i= 0; i < len; i++ )
		{
			cell = ( GridCell )column.elementAt( i );
			cellWidth = cell.getWidth( bg );
			if( cellWidth > width )
				width = cellWidth + 4;
		}	
		return width;	
	}

	/**
	 * override getMinimumSize() of component.
	 * @return java.awt.Dimension
	 */
	public Dimension getMinimumSize()
	{
		Dimension dim = new Dimension();
		dim.height = headerHeight + ( rowCount+2 )*rowHeight + vSbWidth;
		dim.width  = Math.abs( defColumnWidth*columnCount ) + marginWidth;
		
		return dim;
	}

	/**
	 * overrides getPreferredSize() method of component.
	 * @return java.awt.Dimension
	 */
	public Dimension getPreferredSize()
	{
		Dimension dim = new Dimension();
		dim.height = 1024;
		dim.width = 768;

		return dim;
	}

	/**
	 * returns the content of the row at index as an array of String.
	 * @return java.lang.String[] is the content of the row
	 * @param index int is the index of the row. Note that 1<= index <= rowCount.
	 */
	public String[] getRow( int index )
	{
		if( ( index <0 ) || ( index > rowCount ) )
			return null;
			
		GridRow  row;
		String[] rowStr;

		row = (GridRow)rowObjects.elementAt( index );
		rowStr = new String[columnCount];

		for( int i= 1; i <=columnCount; i++ )
			rowStr[i-1] = ( row.getCell( i ) ).getValue();

		if( currentRow == index )
			rowStr[currentColumn-1] = getComponentValue( currentRow, currentColumn );

		return rowStr;
	}

	/**
	 * returns the content of the selected cell as a String.
	 * @return java.lang.String is the content of the cell
	 */
	public String getSelectedCell()
	{
		if( !editFlag ) return null;

		return getCell( selColumn, selRow );
	}

	/**
	 * returns the position of the selected cell
	 * @return java.awt.Point is the position of the cell
	 */
	public Point getSelectedCellPosition()
	{
		if( !editFlag ) return null;

		return new Point( selColumn, selRow );
	}

	/**
	 * returns the content of the selected row as an array of String.
	 * @return java.lang.String[] is the content of the row
	 */
	public String[] getSelectedRow()
	{
		if( ( editFlag && ( !marginFlag || ( selColumn == 0 ) ) ) || ( selRow == 0 ) ) return null;

		GridRow  row;
		String[] rowStr;

		row = (GridRow)rowObjects.elementAt( selRow );
		rowStr = new String[columnCount];

		for( int i= 1; i <=columnCount; i++ )
			rowStr[i-1] = ( row.getCell( i ) ).getValue();

		if( currentRow == selRow )
			rowStr[currentColumn-1] = getComponentValue( currentRow, currentColumn );

		return rowStr;
	}

	/**
	 * returns headers of the grid as an array of String
	 * @return java.lang.String[] is the headers
	 */
	public String[] getTitle()
	{
		GridRow     row;
		String[] rowStr;

		row = (GridRow)rowObjects.elementAt( 0 );
		rowStr = new String[columnCount];
		for( int i= 0; i < columnCount; i++ )
			rowStr[i] = ( row.getCell( i+1 ) ).getValue();

		return rowStr;
	}

	/**
	 * returns number of columns in this grid.
	 * @return int
	 */
	public int getTotalColumns() { return columnCount; }

	/**
	 * returns number of rows in this grid.
	 * @return int is the no. of rows
	 */
	public int getTotalRows() { return rowCount; }

	/**
	 * returns value of the cell at ( row, column ).
	 * @return java.lang.String
	 * @param row int
	 * @param column int
	 */
	public String getValue( int row, int column )
	{
		GridRow currRow;
		GridCell cell;
		
		if( ( row <= 0 ) || ( row > rowCount ) ||
		    ( column <= 0 ) || ( column > columnCount ) )
			return null;

		if( ( currentRow == row ) || ( currentColumn == column ) )
			return getComponentValue( row, column );


		currRow = (GridRow)rowObjects.elementAt( row );
		cell = currRow.getCell( column );
		return cell.getValue();
	}

	/**
	 * Inserts a new column at position index. 
	 * @return boolean returns true if success.
	 * @param index int is the position at which new column should be
	 * inserted
	 * @param title java.lang.String is the header of the new column
	 * @param column java.lang.String[] is the content of the new
	 * column
	 */
	public boolean insertColumn( int index, String title, String[] column )
	{
		GridCell 	cell;
		GridRow		row;
		TextField	comp;
		if( ( index > columnCount ) || ( index < 0 ) )
			return false;

		if( title != null )
		{
			//add a new header cell
			cell = new GridCell();
			cell.setValue( title );
			headers.insertElementAt( cell, index+1 );
			//append a new cell to each row
			for( int i= 1; i <= rowCount; i++ )
			{
				row = (GridRow)rowObjects.elementAt( i );
				cell = new GridCell();
				comp = new TextField();
				//comp.setInsets( new Insets( 0, 0, 0, 0 ) );
				final Grid myself = this;
				comp.addComponentListener( new ComponentAdapter()
				{
					public void componentHidden( ComponentEvent e )
					{
						myself.requestFocus();
					}
				} );
				comp.addKeyListener( new KeyAdapter()
				{
					public void keyPressed( KeyEvent e )
					{
						GridRow erow = (GridRow)rowObjects.elementAt( currentRow );
						GridCell ecell = erow.getCell( currentColumn );

						switch( e.getKeyCode() )
						{
							case KeyEvent.VK_ENTER:
								ecell.setValue( getValue( currentRow, currentColumn ) );
								processGridEvent( new GridEvent( myself, GridEvent.CELL_CHANGED, currentColumn, currentRow ) );
							case KeyEvent.VK_ESCAPE:
								e.getComponent().setVisible( false );
								if( selectEditFlag )
								{
									ecell.highlight( true );
									ecell.invalidate( true );
								}

								selRow = 0;
								selColumn = 0;

								erow.invalidate( true );
								redraw = true;
								repaint();
								break;
							default:
								return;
						}

						haveFocus = false;
						focusCell();
					}
				} );
				cell.setComponent( comp );
				if( column != null )
				{
					cell.setValue( column[i-1] );
					comp.setText( column[i-1] );
				}	
				row.insertCell( index+1, cell );
			}	
			columnCount++;
			computeColumnWidths();
			computeHScroll();
			bim = null;
			repaint();
			return true;
		}
		else	
			return false;
	}

	/**
	 * inserts a new row at position index.
	 * @return boolean returns true if successfull.
	 * @param index int is the position at which new row be inserted.
	 * @param row java.lang.String[] is the content of this row.
	 */
	public boolean insertRow( int index, String[] row )
	{
		if( index > rowCount )
			return false;

		if( row != null )
		{
			GridCell    cell;
			GridRow     rowObj;
			Vector      newRow = new Vector();
			TextField comp;

			//create the margin
			cell = new GridCell();
			newRow.addElement( cell );

			//create cells
			for( int i=1; i <= columnCount; i++ )
			{
				cell = new GridCell();
				comp = new TextField( row[i-1] );
				//comp.setInsets( new Insets( 0, 0, 0, 0 ) );
				final Grid myself = this;
				comp.addComponentListener( new ComponentAdapter()
				{
					public void componentHidden( ComponentEvent e )
					{
						myself.requestFocus();
					}
				} );
				comp.addKeyListener( new KeyAdapter()
				{
					public void keyPressed( KeyEvent e )
					{
						GridRow erow = (GridRow)rowObjects.elementAt( currentRow );
						GridCell ecell = erow.getCell( currentColumn );

						switch( e.getKeyCode() )
						{
							case KeyEvent.VK_ENTER:
								ecell.setValue( getValue( currentRow, currentColumn ) );
								processGridEvent( new GridEvent( myself, GridEvent.CELL_CHANGED, currentColumn, currentRow ) );
							case KeyEvent.VK_ESCAPE:
								e.getComponent().setVisible( false );
								if( selectEditFlag )
								{
									ecell.highlight( true );
									ecell.invalidate( true );
								}

								selRow = 0;
								selColumn = 0;

								erow.invalidate( true );
								redraw = true;
								repaint();
								break;
							default:
								return;
						}

						haveFocus = false;
						focusCell();
					}
				} );
				cell.setValue( row[i-1] );
				cell.setComponent( comp );
				newRow.addElement( cell );
			}	

			//add this row
			rowObj = new GridRow( newRow, borderFlag, marginFlag );
			rowObjects.insertElementAt( rowObj, index+1 );
		}	
		rowCount++;
		computeVScroll();
		bim = null;
		repaint();
		return true;
	}

	/**
	 * marks content of this grid to be repainted.
	 * @param flag boolean
	 */
	private void invalidate( boolean flag )
	{
		GridRow row;
		if( rowObjects != null )
		{
			for( int i=0; i < rowObjects.size(); i++ )
			{
				row = (GridRow)rowObjects.elementAt( i );
				row.invalidate( flag );	
			}	
		}	
	}

	/**
	 * returns whether cell at ( row, column ) of this grid is editable.
	 * @return boolean returns true if editable. false otherwise.
	 * @param row int
	 * @param column int
	 */
	public boolean isEditable( int row, int column )
	{
		if( ( row < 0 ) || ( row > rowCount ) ||
		    ( column < 0 ) || ( column > columnCount ) )
			return false;

		GridRow  rowObj;
		GridCell cell;
	
		rowObj = (GridRow)rowObjects.elementAt( row );
		cell = rowObj.getCell( column );
		return cell.isEditable();
	}

	/**
	 * @returns boolean returns true if focus traversable, false otherwise
	 */
	public boolean isFocusTraversable()
	{
		boolean isFocusable = true;

		if( ( component != null ) && component.isVisible() )
			isFocusable = false;

		return isFocusable;
	}

	/**
	 * .
	 * @param g java.awt.Graphics
	 */
	public void paint( Graphics g )
	{
		Graphics hg = null;

		super.paint( g );

		g.setFont( font );

		if( !initialized )
		{
			// do all first time painting initializations here
			fntMetrics = g.getFontMetrics();
			horScrollUnit = fntMetrics.charWidth( ' ' );
			verticalScrollUnit = fntMetrics.getHeight() + 4;
			rowHeight = verticalScrollUnit;
			initialized = true;
			computeColumnWidths();
			respace();
		}	
		if( bim == null )
		{
			// content of the grid has changed. render it.
			respace();
			bim = createImage( imgWidth, imgHeight );
			bg = bim.getGraphics();
			invalidate( true );
			render( bg );
			curImgWidth = imgWidth;
			curImgHeight = imgHeight;
			computeHScroll();
			computeVScroll();
		}	
		else if( redraw )
		{
			respace();
			render( bg );
			redraw = false;
		}	

		//draw border
		g.setColor( headerColor1 );
		g.drawRect( 0, 0, screenWidth, screenHeight );

		if( mouseDown && draggable && !livedragFlag )
			hg = g.create();

		if( bim != null )
		{
			//calculate new positions
			leftShift = left*horScrollUnit;
			leftShift = ( screenWidth-imgWidth ) > -leftShift ? ( imgWidth-screenWidth ) : leftShift;
			topShift = top*verticalScrollUnit;
			topShift = ( screenHeight-imgHeight ) > -topShift ? ( imgHeight-screenHeight ) : topShift;

			//draw margin
			drawMargin( g );

			//now draw the header
			//clip the rectangle so that margin will not be over written
			int x = 1;
			int y = headerHeight + 1;
			int actualX = startX-leftShift;
			int actualY = startY-topShift;

			if( marginFlag )
				x += marginWidth;

			if( headerFlag )
			{
				g.clipRect( x, 0, screenWidth, screenHeight );

				if( headerIm == null )
				{
					//create header image
					headerIm = createImage( imgWidth, y );
					headerBg = headerIm.getGraphics();

					//get header row
					GridHeaderRow row = (GridHeaderRow)rowObjects.elementAt( 0 );
					row.invalidate( true );
					row.setColWidths( colWidths, columnCount );

					//redraw image
					row.draw( headerBg, new Rectangle( x-1, 0, imgWidth, y ) );
				}	
				g.drawImage( headerIm, actualX, startY, this );
			}

			//clip the rectangle so that header will not be over written
			g.clipRect( x, y, screenWidth, screenHeight );
			g.drawImage( bim, actualX, actualY, this );

			if( screenHeight > curImgHeight )
			{
				g.setColor( bgColor );
				g.fillRect( startX, startY + curImgHeight, screenWidth, screenHeight - ( startY + curImgHeight ) + 1 );
			}
		}	

		if( ( vSb.isVisible() ) && ( hSb.isVisible() ) )
		{
			g.setColor( Color.lightGray );
			g.fillRect( screenWidth, screenHeight, vSbWidth, hSbHeight );	
		}	

		// now, show the visible portion of component cell.
		if( ( !mouseDown || !draggable || livedragFlag ) &&
		    editFlag && isEditable( currentRow, currentColumn ) &&
		    ( component != null ) && component.isVisible() )
			adjustCell();

		if( mouseDown && draggable && !livedragFlag && ( hg != null ) )
		{
			int x = 1 + ( marginFlag ? marginWidth : 0 );
			if( mouseX < x )
				mouseX = x;
			else if( mouseX > screenWidth )
				mouseX = screenWidth;
			hg.clipRect( x, startY, screenWidth, screenHeight );
			hg.setColor( headerColor1 );
			hg.drawLine( mouseX - 1, startY, mouseX - 1, screenHeight );
			hg.drawLine( mouseX, startY, mouseX, screenHeight );

			hg.dispose();
		}
	}

	/**
	 * post grid event to listeners
         */
	protected void processGridEvent( GridEvent event )
	{
		synchronized( gridEventListeners )
		{
			for( int i = 0; i < gridEventListeners.size(); i++ )
			{
				GridListener l = (GridListener)gridEventListeners.elementAt( i );
				if( l != null )
				{
					switch( event.getType() )
					{
						case GridEvent.COLUMN_RESIZED:
							l.columnResized( event );
							break;
						case GridEvent.COLUMN_CLICKED:
							l.columnClicked( event );
							break;
						case GridEvent.COLUMN_DOUBLE_CLICKED:
							l.columnDoubleClicked( event );
							break;
						case GridEvent.CELL_CLICKED:
							l.cellClicked( event );
							break;
						case GridEvent.CELL_DOUBLE_CLICKED:
							l.cellDoubleClicked( event );
							break;
						case GridEvent.CELL_CHANGED:
							l.cellChanged( event );
							break;
						case GridEvent.ROW_CLICKED:
							l.rowClicked( event );
							break;
						case GridEvent.ROW_DOUBLE_CLICKED:
							l.rowDoubleClicked( event );
							break;
					}
				}
			}
		}
	}

	/**
	 * post item event to listeners
         */
	protected void processItemEvent( ItemEvent event )
	{
		synchronized( itemEventListeners )
		{
			for( int i = 0; i < itemEventListeners.size(); i++ )
			{
				ItemListener l = (ItemListener)itemEventListeners.elementAt( i );
				if( l != null )
					l.itemStateChanged( event );
			}
		}
	}

	/**
	 * removes all contents fron the grid.
	 */
	public void removeAll()
	{
		rowCount = 0;
		rowObjects.setSize( 1 );
		bim = null;
		currentRow = 0;
		currentColumn = 0;
		prevRow = 0;
		prevColumn = 0;
		selRow = 0;
		selColumn = 0;
		repaint();
	}

	/**
	 * remove a grid event listener
	 */
	public synchronized void removeGridListener( GridListener l )
	{
		synchronized( gridEventListeners )
		{
			if( gridEventListeners.contains( l ) )
				gridEventListeners.removeElement( l );
		}
	}

	/**
	 * remove an item event listener
	 */
	public synchronized void removeItemListener( ItemListener l )
	{
		synchronized( itemEventListeners )
		{
			if( itemEventListeners.contains( l ) )
				itemEventListeners.removeElement( l );
		}
	}

	/**
	 * renders the grid image.
	 * @param g java.awt.Graphics
	 */
	protected void render( Graphics g )
	{
		Graphics tg;
		GridRow row;

		//create a new graphics object for row. and paint.
		for( int i=1; i <= rowCount; i++ )
		{
			tg = g.create( xPos[0], yPos[i], imgWidth, rowHeight );
			row = (GridRow)rowObjects.elementAt( i );
			row.setColWidths( colWidths, columnCount );
			row.draw( tg );
			tg.dispose();
		}	

		//draw the bottom line.
		int ht, height;
		int wd;
		
		ht = ( imgHeight > gridHeight ) ? gridHeight : imgHeight;
		wd = ( imgWidth > gridWidth ) ? gridWidth : imgWidth;
		g.setColor( headerColor1 );
		g.drawLine( startX, ht, imgWidth-1, ht );
		g.drawLine( startX + wd, headerHeight, startX + wd, ht );

		if( imgHeight > gridHeight )
		{
			g.setColor( bgColor );
			g.fillRect( startX, ht + 1, imgWidth, imgHeight - ht );
		}
	}

	/**
	 * recomputes row and column widths.
	 */
	private void respace()
	{
		if( !initialized )
			return;
		while( columnCount >= maxCols )
		{
			maxCols *= 2;
			colWidths = new int[maxCols];
			yPos = new int[maxCols];
		}

		while( rowCount >= maxRows )
		{
			maxRows *= 2;
			yPos = new int[maxRows];
		}	
			
		int xpos = startX;
		int ypos = startY;
		for( int i = 0; i <= columnCount; i++ )
		{
			xPos[i] = xpos;
			if( marginFlag || ( i > 0 ) )
				xpos += colWidths[i];
		}	
		gridWidth = xpos;

		yPos[0]	= ypos;
		ypos += headerHeight;
		for( int i = 1; i <= rowCount; i++ )
		{
			yPos[i] = ypos;
			ypos += rowHeight;
		}	
		gridHeight = ypos;

		colWidths[columnCount+1] = screenWidth - gridWidth;
		if( colWidths[columnCount+1] < 0 )
			colWidths[columnCount+1] = 0;

		imgWidth = ( gridWidth > screenWidth ) ? gridWidth : screenWidth;
		imgHeight = ( gridHeight > screenHeight ) ? gridHeight : screenHeight;
	}

	/**
	 *
	 */
	public void setBorder( int borderFlag )
	{
		GridRow row;

		this.borderFlag = borderFlag;

		for( int i = 1; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			row.setBorder( borderFlag );
		}
	}

	/**
	 * overrides the setBounds method of component.
	 * @param x int
	 * @param y int
	 * @param width int
	 * @param height int
	 */
	public void setBounds( int x, int y, int width, int height )
	{
		int oldwidth = getBounds().width;
		super.setBounds( x, y, width, height );
		in = getInsets();
		startX = in.left;
		startY = in.top;

		//compute the width of hor & vertical scrollbar
		hSbHeight = hSb.getMinimumSize().height;
		vSbWidth = vSb.getMinimumSize().width;

		//compute the width and height of the area where we can draw the grid
		screenWidth = width - ( in.left + in.right ) - vSbWidth;
		screenHeight = height - ( in.top + in.bottom ) - hSbHeight;

		//place scrollbars at propert places
		vSb.setBounds( screenWidth+in.left, startY, vSbWidth, screenHeight );
		hSb.setBounds( startX, screenHeight + in.top, screenWidth, hSbHeight );

		//recompute the positions of the scrollbars and render again
		if( width > oldwidth )
		{
			bim = null;				//force to render again
			headerIm = null;
		}
		GridRow row = (GridRow)rowObjects.firstElement();
		if( row != null )
			row.invalidate( true );
		respace();
		computeHScroll();
		computeVScroll();
		repaint();
	}

	public void setCellColor( int colorType, Color color, int row, int column )
	{
		GridRow  rowObj;
		GridCell cell;
		try
		{
			rowObj = (GridRow)rowObjects.elementAt( row );
			cell	= rowObj.getCell( column );
			cell.setColor( colorType, color );
		}
		catch( Exception e )
		{}	
			
	}

	/**
	 * sets the cell font.
	 * @param rowIndex int
	 * @param columnIndex int
	 * @param font java.awt.Font
	 */
	private void setCellFont( int rowIndex, int columnIndex, Font font )
	{
		GridRow  row;
		GridCell cell;
		try
		{
			row = (GridRow)rowObjects.elementAt( rowIndex );
			cell	= row.getCell( columnIndex );
			cell.setFont( font );
		}
		catch( Exception e )
		{}
			
	}

	/**
	 *
	 */
	public void setColumnResizable( int index, boolean flag )
	{
		try
		{
			GridRow row = (GridRow)rowObjects.firstElement();
			GridCell cell = row.getCell( index );
			cell.setResizable( flag );
		}
		catch( Exception e )
		{}
	}

	/**
	 * Set color of the grid according to parameters.
	 * @param colorType int is the type of the color.
	 * @param color java.awt.Color
	 */
	public void setColor( int colorType, Color color )
	{
		switch( colorType )
		{
			case TEXT_COLOR:
				textColor = color;
				break;
			case HL_TEXT_COLOR:
				hlTextColor = color;
				break;
			case BG_COLOR:
				bgColor = color;
				break;
			case HL_BG_COLOR:
				hlBgColor = color;
				break;
		}
	}

	/**
	 * Setsl color of a particular column.
	 * @param colorType int is the type of the color
	 * @param color java.awt.Color is the Color
	 * @param index int is the index of the Column
	 */
	public void setColumnColor( int colorType, Color color, int index )
	{
		if( ( index < 0 ) || ( index > columnCount ) )
			return ;

		GridRow  row;
		GridCell cell;
		for( int i=1; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			cell	= row.getCell( index );
			cell.setColor( colorType, color );
		}
	}

	/**
	 * sets font of a particular column.
	 * @param index int
	 * @param font java.awt.Font
	 */
	private void setColumnFont( int index, Font font )
	{
		if( ( index < 0 ) || ( index > columnCount ) )
			return ;

		GridRow  row;
		GridCell cell;
		for( int i=1; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			cell = row.getCell( index );
			cell.setFont( font );
		}
	}

	/**
	 * sets the column widths
	 * @param widths int[] array of widths for columns
	 */
	public void setColumnWidths( int[] widths )
	{
		if( widths.length == columnCount )
		{
			for( int i = 0; i < columnCount; i++ )
				colWidths[i+1] = widths[i];

			columnWidthsSet = true;
		}
	}

	/**
	 * sets component at a cell position.
	 * @return boolean returns true if successfull.
	 * @param row int is the row position
	 * @param column int is the column
	 * @param component java.awt.Component is the new component
	 */
	public boolean setComponent( int row, int column, Component component )
	{
		GridRow currRow;
		GridCell cell;
		
		if( ( row < 0 ) || ( row > rowCount ) ||
		    ( column < 0 ) || ( column > columnCount ) )
			return false;

		currRow = (GridRow)rowObjects.elementAt( row );
		cell = currRow.getCell( column );
		cell.setComponent( component );
		return true;
	}

	/**
	 * sets the width of the column as width.
	 * @param width int is the width of the column
	 */
	public void setDefaultColumnWidth( int width ) { if( width != 0 ) defColumnWidth = width; }

	/**
	 * @param flag boolean if true, selection focus is enabled. else not
	 */
	public void setSelectEdit( boolean flag ) { selectEditFlag = flag; }

	/**
	 * sets the cell at ( row, column ) as editable/non editable.
	 * @param row int 
	 * @param column int
	 * @param flag boolean if true, sets cell as editable else non edita
	 * ble.
	 */
	public void setEditable( int row, int column, boolean flag )
	{
		if( ( row < 0 ) || ( row > rowCount ) ||
		    ( column < 0 ) || ( column > columnCount ) )
			return;

		GridRow  rowObj;
		GridCell cell;
	
		rowObj = (GridRow)rowObjects.elementAt( row );
		cell = rowObj.getCell( column );
		cell.setEditable( flag );
	}

	/**
	 * sets the column editable/non editable.
	 * @param column int is the index of the grid column
	 * @param flag boolean if true, sets cells in this flag as editable
	 * if false, non editable
	 */
	public void setEditable( int column, boolean flag )
	{
		if( ( column < 0 ) || ( column > columnCount ) )
			return;

		GridRow  row;
		GridCell cell;
		for( int i=1; i <= rowCount; i++ )
		{
			row = (GridRow)rowObjects.elementAt( i );
			cell = row.getCell( column );
			cell.setEditable( flag );
		}
		return ;
	}
	/**
	 * sets contents of this grid editable/non editable.
	 * @param flag boolean
	 */
	public void setEditable( boolean flag ) { editFlag = flag; }

	/**
	 * sets the font of the grid/row/column/cell 
	 * @param type int  indicates whether font to be set for grid/row/column/cell
	 * @param row int set this value for row/cell
	 * @param column int set this value for column/cell
	 * @param font java.awt.Font is the new font
	 */
	public void setFont( int type, Font font, int row, int column )
	{
		switch( type )
		{
			case GRID:
				this.font = font;
				break;
			case ROW:
				setRowFont( row, font );
				break;
			case COLUMN:
				setColumnFont( column, font );
				break;
			case CELL:
				setCellFont( row, column, font );
				break;
		}
	}

	/**
	 * sets different color type of the grid.
	 * @param colorType int is the type of color like textcolor etc.
	 * @param color java.awt.Color is the new Color
	 */
	public void setGridColor( int colorType, Color color )
	{
		switch( colorType )
		{
			case TEXT_COLOR:
				textColor = color;
				break;
			case HL_TEXT_COLOR:
				hlTextColor = color;
				break;
			case BG_COLOR:
				bgColor = color;
				break;
			case HL_BG_COLOR:
				hlBgColor = color;
				break;
		}
	}

	/**
	 * changes columns headings and clears the grid
	 */
	public void setHeadings( String[] headings )
	{
		this.removeAll();
		rowObjects = new Vector();
		headers = new Vector();
		columnCount = 0;
		columnWidthsSet = false;

		//create the header 
		if( headings != null )
		{
			columnCount = headings.length;
			for( int i = 0; i < maxCols; i++ )
				colWidths[i] = 0;
			GridCell cell = new GridCell();
			headers.addElement( cell );
			for( int index = 1; index <= columnCount; index++ )
			{
				cell = new GridCell();
				cell.bgColor = Color.lightGray;
				cell.textColor = Color.black;
				cell.setValue( headings[index -1] );
				headers.addElement( cell );
			}
			cell = new GridCell();
			headers.addElement( cell );
		}	
		GridHeaderRow headerRow = new GridHeaderRow( headers );
		rowObjects.addElement( headerRow );
		computeColumnWidths();
		computeHScroll();
		computeVScroll();
		respace();
		headerIm = null;
		bim = null;
		repaint();
	}

	/**
	 * enables highlighting.
	 * @param flag boolean if true, enable highlighting.
	 */
	public void setHighLight( boolean flag ) { highLight = flag; }

	/**
	 * @param flag boolean if true, live drag is enabled. else not
	 */
	public void setLiveDrag( boolean flag ) { livedragFlag = flag; }

	/**
	 * sets properties of the grid.
	 */
	private void setProperties( GridCell cell )
	{
		cell.setColor( TEXT_COLOR, textColor );
		cell.setColor( HL_TEXT_COLOR, hlTextColor );
		cell.setColor( BG_COLOR, bgColor );
		cell.setColor( HL_BG_COLOR, hlBgColor );
	
		cell.setFont( font );
	}

	/**
	 * replaces content of an existing row with new values.
	 * @param row int is the index of the grid row
	 * @param values java.lang.String[] new contents.
	 */
	public boolean setRow( int row, String[] values )
	{
		GridRow currRow;
		GridCell cell;
		
		if( ( row <= 0 ) || ( row > rowCount ) ||
		    ( values.length > columnCount ) )
			return false;

		currRow = (GridRow)rowObjects.elementAt( row );
		for( int i=1; i <= columnCount; i++ )
		{
			cell = currRow.getCell( i );
			cell.setValue( values[i-1] );
		}
		bim = null;
		repaint();
		return true;
	}

	/**
	 * sets color of the row
	 * @param colorType int is the type of color.
	 * @param color java.awt.Color is the new color
	 */
	public void setRowColor( int colorType, Color color, int index ) {
		if( ( index <0 ) || ( index > rowCount ) )
			return;

		GridRow row;
		row = (GridRow)rowObjects.elementAt( index );
		row.setColor( colorType, color );
	}

	/**
	 * sets font for this row.
	 * @param index int is the index of grid row
	 * @param font java.awt.Font is the new font
	 */
	private void setRowFont( int index, Font font )
	{
		if( ( index < 0 ) || ( index > rowCount ) )
			return;

		GridRow row;
		row = (GridRow)rowObjects.elementAt( index );
		row.setFont( font );
	}

	/**
	 * sets the header values of the grid.
	 * @return java.lang.String[] is the new header values
	 */
	public void setTitle( String[] title )
	{
		GridRow row;
		row = (GridRow)rowObjects.elementAt( 0 );
		for( int i= 1; i <=columnCount; i++ )
			( row.getCell( i ) ).setValue( title[i-1] );

		computeHScroll();
		bim = null;
		repaint();
	}

	/**
	 * sets value of the cell at ( row, column ).
	 * @return boolean returns true if success
	 * @param row int 
	 * @param column int
	 * @param value java.lang.String is the new value of the cell
	 */
	public boolean setValue( int row, int column, String value )
	{
		GridRow currRow;
		GridCell cell;
		
		if( ( row <= 0 ) || ( row > rowCount ) ||
		    ( column <= 0 ) || ( column > columnCount ) )
			return false;

		currRow = (GridRow)rowObjects.elementAt( row );
		cell = currRow.getCell( column );
		cell.setValue( value );
		bim = null;
		repaint();
		return true;
	}

	/**
	 * makes header visible/invisible according to value of flag.
	 * @param flag boolean if true, header is displayed. else not
	 */
	public void showHeader( boolean flag )
	{
		headerFlag = flag;
		computeHScroll();
		computeVScroll();
		bim = null;
		repaint();
	}

	/**
	 * makes margin visible/invisible according to value of flag.
	 * @param flag boolean if true, margin is displayed. else not
	 */
	public void showMargin( boolean flag )
	{
		marginFlag = flag;
		for( int i = 1; i <= rowCount; i++ )
		{
			GridRow row = (GridRow)rowObjects.elementAt( i );
			row.setMargin( flag );
		}
		computeHScroll();
		computeVScroll();
		bim = null;
		repaint();
	}

	/**
	 * overrides the update method of component
	 * @param g java.awt.Graphics
	 */
	public void update( Graphics g ) { paint( g ); }
}

