/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * $Id: GridHeaderRow.java,v 1.7 2001/04/05 18:41:10 stevek Exp $
 */
package com.nailabs.ui;

/**
 * This class is the implementation of the Header Row
 *
 */
import java.awt.*;
import java.util.Vector;

public class GridHeaderRow extends GridRow
{
	/**
	 * GridHeaderRow constructor comment.
	 * @param cells java.util.Vector
	 */
	public GridHeaderRow( java.util.Vector cells ) { super( cells ); }

	/**
	 * GridHeaderRow constructor comment.
	 * @param cells java.util.Vector
	 * @param borderFlag int - display  border or not
	 * @param marginFlag boolean - display margin or not
	 */
	public GridHeaderRow( java.util.Vector cells, int borderFlag, boolean marginFlag )
	{
		super( cells, borderFlag, marginFlag );
	}

	/**
	 * computes x position of each cell in this row
	 */
	protected void computeXpos()
	{
		int startX = x;
		for( int i = 1; i <= columnCount + 1; i++ )
		{
			xPos[i] = startX;
			startX += colWidths[i];
		}
	}

	/**
	 * draw the contents of this row
	 * @param g Graphics
	 */
	public void draw( Graphics g )
	{
		if( redraw && ( g != null ) && widthFlag )
		{
			clipRect = g.getClipBounds( );
			draw( g, clipRect );
		}
		redraw = false;
	}

	/**
	 * draw the contents of this row in the rectangle rect
	 * @param g java.awt.Graphics
	 * @param rect java.awt.Rectangle
	 */
	public void draw( Graphics g, Rectangle rect )
	{
		if( redraw && ( g != null ) && widthFlag )
		{
			int startX, startY;
			int cellWidth, cellHeight;

			rowWidth = rect.width;
			rowHeight = rect.height;
			x = rect.x;
			y = rect.y;

			GridCell cell;
			computeXpos();
			for( int index = 1; index <= columnCount + 1; index++ )
			{
				if( colWidths[index] == 0 ) continue;

				cellWidth = colWidths[index] - 2;
				cellHeight = rowHeight - 2;
				startX = xPos[index] + 2;
				startY = y + 2;

				// do rest of the things here
				drawBackground( g, index );
				cell = (GridCell)contents.elementAt( index );
				cell.draw( new Rectangle( startX, startY, cellWidth, cellHeight ), g );
				drawMargin( g, index );
			}
			drawBorder( g );
		}
		redraw = false;
	}

	/**
	 * draw the border for this row
	 * @param g java.awt.Graphics
	 */
	protected void drawBorder( Graphics g )
	{
		int x1 = x + rowWidth + 1;
		int y1 = y + rowHeight + 1;

		//draw the outer border
		g.setColor( headerColor1 );		//black
		g.drawRect( x, y, x1, y1 );
	}

	/**
	 * draw background for this row
	 * @param g java.awt.Graphics
	 */
	protected void drawBackground( Graphics g, int i )
	{
		int x0;
		int y0;
		int x1;
		int y1;

		if( colWidths[i] != 0 )
		{
			x0 = xPos[i]+1;
			y0 = y + 1;

			g.setColor( headerColor4 );
			g.fillRect( x0, y0, colWidths[i], rowHeight );
		}
	}

	/**
	 * draw margin for this row
	 * @param g java.awt.Graphics
	 */
	protected void drawMargin( Graphics g, int i )
	{
		int x0;
		int y0;
		int x1;
		int y1;

		if( colWidths[i] != 0 )
		{
			x0 = xPos[i]+1;
			y0 = y + 1;
			x1 = x0+colWidths[i]-1;
			y1 = y + rowHeight-1;

			g.setColor( headerColor3 );
			g.drawLine( x0, y0, x1, y0 );
			g.drawLine( x0, y0, x0, y1 );

			g.setColor( headerColor1 );
			g.drawLine( x1, y0, x1, y1 );
			g.drawLine( x0, y1, x1, y1 );
		}
	}
}

