package BinFile::COFF;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: COFF.pm,v 1.12 2001/04/05 18:41:11 stevek Exp $

require 5.004;
use strict;
use vars qw($VERSION @ISA @EXPORT);
use Carp;
use BinFile;
use BinFile::COFF::Header;
use BinFile::COFF::Section;
use IO::Seekable;

$VERSION = 0.001;
@ISA     = qw(BinFile);
@EXPORT  = qw(findSectionByRVA);

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $file = shift;
   my $options = @_?shift:0;
   my $self = {};
   bless $self, $class;
   $self->_initialize($file,$options);
}

sub _initialize {
   my $self = shift;
   my $file = shift;
   my $options = shift;

   # Initialize BinFile
   $self->SUPER::_initialize( $file, $options );

   my $fd;
   *fd = $self->{'Descriptor'};

   # read COFF header
   my $coffHeader   = loadCoffHeader( \*fd );

   # skip over optional header
   sysseek(*fd, $coffHeader->optsize, SEEK_CUR) if( $coffHeader->optsize > 0 );

   # read COFF sections
   my @coffSections;
   if( $coffHeader->nsections > 0 )
   {
      foreach( 1 .. $coffHeader->nsections ) {
         push( @coffSections, loadCoffSection( \*fd ) );
      }
   }
   $self->{'COFF Header'}   = $coffHeader;
   $self->{'COFF Sections'} = \@coffSections;
   $self;
}

sub format ($) { "COFF"; }

sub findSectionByRVA($, $) {
   my $this = shift;
   my $rva = shift;
   my $offset;

   foreach( $this->coffsections ) {
      $offset = ( $rva - $_->virtaddr ) + $_->rawoffset, last
         if( ( $_->virtaddr <= $rva ) &&
             ( ( $_->virtaddr + $_->rawsize ) > $rva ) );
   }
   $offset;
}

sub coffheader ($) {
   my $this = shift;
   $this->{'COFF Header'};
}

sub coffsections ($) {
   my $this = shift;
   my $sections = $this->{'COFF Sections'};
   ref($sections)?@$sections:$sections;
}

sub dump ($) {
   my $this = shift;
}

sub dumpheader ($) {
   my $this = shift;
   $this->coffheader->dump();
}

sub dumpsections ($) {
   my $this = shift;
   foreach( $this->coffsections ) {
      $_->dump();
   }
}

1;
