package BinFile::COFF::Header;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Header.pm,v 1.8 2001/04/05 18:41:12 stevek Exp $
#
require 5.004;
use strict;
use Carp;
use IO::Seekable;

BEGIN {
   use Exporter   ();
   use vars       qw(@EXPORT $VERSION $HEADER_SIZE);
   @EXPORT      = qw(loadCoffHeader);
   $VERSION     = 0.001;
   $HEADER_SIZE = 20;
}

sub import { goto &Exporter::import }

use Class::Struct qw(struct);
struct 'BinFile::COFF::Header' => [
   machine    => '$', # 2 bytes (unsigned short)  offset 0
   nsections  => '$', # 2 bytes (unsigned short)  offset 2
   timestamp  => '$', # 4 bytes (unsigned long)   offset 4
   symtab     => '$', # 4 bytes (unsigned long)   offset 8
   nsyms      => '$', # 4 bytes (unsigned long)   offset 12
   optsize    => '$', # 2 bytes (unsigned short)  offset 16
   charistics => '$'  # 2 bytes (unsigned short)  offset 18
];

sub loadCoffHeader ($) {
   my $fd = shift;

   croak "Invalid argument" unless ref($fd);
   my $header;
   my $total = 0;
   while( $total < $HEADER_SIZE ) {
      my $nread = sysread( $fd, $header, $HEADER_SIZE, $total );
      croak "Unable to read COFF header" unless $nread;
      $total += $nread;
   }
   croak "Unable to read COFF header" if( $total < $HEADER_SIZE );

   my @parts = unpack('vv VVV vv', $header);
   my $ch = new();
   ( $ch->[0], $ch->[1] ) = @parts[ (0..1) ];
   ( $ch->[2], $ch->[3],
     $ch->[4] )           = @parts[ (2..4) ];
   ( $ch->[5], $ch->[6] ) = @parts[ (5..6) ];
   $ch;
}

sub size { return $HEADER_SIZE }

sub dump ($) {
   my $this = shift;

   printf("[COFF Header]\n");
   printf("   machine:     %4.4x\n", $this->machine);
   printf(" nsections:     %4.4x\n", $this->nsections);
   printf(" timestamp: %8.8x\n", $this->timestamp);
   printf("    symtab: %8.8x\n", $this->symtab);
   printf("     nsyms: %8.8x\n", $this->nsyms);
   printf("   optsize:     %4.4x\n", $this->optsize);
   printf("charistics:     %4.4x\n", $this->charistics);
};

1;
