package BinFile::DOS::EXE::Header;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Header.pm,v 1.4 2001/04/05 18:41:13 stevek Exp $

require 5.004;
use strict;
use Carp;
use IO::Seekable;

BEGIN {
   use Exporter   ();
   use vars       qw(@EXPORT $VERSION $HEADER_SIZE);
   @EXPORT      = qw(loadDosExeHeader);
   $VERSION     = 0.001;
   $HEADER_SIZE = 64;
}

sub import { goto &Exporter::import }

use Class::Struct qw(struct);
struct 'BinFile::DOS::EXE::Header' => [
   signature => '$', # 2 bytes  (char [2])        offset 0
   lastsize  => '$', # 2 bytes  (short)           offset 2
   nblocks   => '$', # 2 bytes  (short)           offset 4
   nreloc    => '$', # 2 bytes  (short)           offset 6
   hdrsize   => '$', # 2 bytes  (short)           offset 8
   minalloc  => '$', # 2 bytes  (short)           offset 10
   maxalloc  => '$', # 2 bytes  (short)           offset 12
   sp        => '$', # 4 bytes  (void far *)      offset 14
   checksum  => '$', # 2 bytes  (short)           offset 18
   ip        => '$', # 4 bytes  (void far *)      offset 20
   relocpos  => '$', # 2 bytes  (short)           offset 24
   noverlay  => '$', # 2 bytes  (short)           offset 26
   reserved  => '@', # 8 bytes  (short [4])       offset 28
   oemid     => '$', # 2 bytes  (short)           offset 36
   oeminfo   => '$', # 2 bytes  (short)           offset 38
   reserved2 => '@', # 20 bytes (short [10])      offset 40
   lfanew    => '$', # 4 bytes  (long)            offset 60
];

sub loadDosExeHeader ($) {
   my $fd = shift;

   croak "Invalid argument" unless ref($fd);
   sysseek( $fd, 0, SEEK_SET );
   my $header;
   my $total = 0;
   while( $total < $HEADER_SIZE ) {
      my $nread = sysread( $fd, $header, $HEADER_SIZE, $total );
      croak "Unable to read DOS EXE header" unless $nread;
      $total += $nread;
   }
   croak "Unable to read DOS EXE header" if( $total < $HEADER_SIZE );

   my @parts = unpack('C2 vvvvvv V v V vv v4 vv v10 V', $header);
   my $deh = new();
   $deh->[0]                  = pack('C2',@parts[ (0..1) ]);
   ( $deh->[1], $deh->[2],
     $deh->[3], $deh->[4],
     $deh->[5], $deh->[6],
     $deh->[7], $deh->[8],
     $deh->[9], $deh->[10],
     $deh->[11] )             = @parts[ (2..12) ]; 
   $deh->[12]                 = @parts[ (13..16) ];
   ( $deh->[13], $deh->[14] ) = @parts[ (17..18) ];
   $deh->[ 15 ]               = @parts[ (19..28) ];
   $deh->[ 16 ]               = $parts[ 29 ];
   $deh;
}

sub size { return $HEADER_SIZE; }

1;
