package BinFile::ELF::ELF32::Header;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Header.pm,v 1.7 2001/01/16 23:34:38 stevek Exp $
#
require 5.004;
use strict;
use Exporter;
use vars qw(@EXPORT @ISA $VERSION $HEADER_SIZE);
use Carp;
use IO::Seekable;

@ISA         = qw(BinFile::ELF::Header);
@EXPORT      = qw(loadElf32Header);
$VERSION     = 0.001;
$HEADER_SIZE = 52;

use Class::OOStruct qw(substruct);

substruct [];

sub loadElf32Header ($) {
   my $fd = shift;

   croak "Invalid argument" unless ref($fd);
   sysseek( $fd, 0, SEEK_SET );

   my $header;
   my $total = 0;
   while( $total < $HEADER_SIZE ) {
      my $nread = sysread( $fd, $header, $HEADER_SIZE, $total );
      croak "Unable to read ELF header" unless $nread;
      $total += $nread;
   }
   croak "Unable to read ELF header" if( $total < $HEADER_SIZE );

   my @bytes = unpack('C'.$HEADER_SIZE, $header);
   my @parts;
   @parts = unpack('C4 CCCC C8 vv VVVVV vvvvvv', $header)
      if $bytes[5] == 0x01; # Little Endian
   @parts = unpack('C4 CCCC C8 nn NNNNN nnnnnn', $header)
      if $bytes[5] == 0x02; # Big Endian
   my $eh = new BinFile::ELF::ELF32::Header;
   foreach( 0 .. 3 ) { $eh->ident($_, $parts[$_]); }
   $eh->class( $parts[4] );
   $eh->data( $parts[5] );
   $eh->fmtversion( $parts[6] );
   $eh->osabi( $parts[7] );
   foreach( 0 .. 7 ) { $eh->brand($_, $parts[8+$_]); }
   $eh->type( $parts[16] );
   $eh->machine( $parts[17] );
   $eh->version( $parts[18] );
   $eh->entry( $parts[19] );
   $eh->phoff( $parts[20] );
   $eh->shoff( $parts[21] );
   $eh->flags( $parts[22] );
   $eh->ehsize( $parts[23] );
   $eh->phentsize( $parts[24] );
   $eh->phnum( $parts[25] );
   $eh->shentsize( $parts[26] );
   $eh->shnum( $parts[27] );
   $eh->shstrndx( $parts[28] );

   croak "Mismatched size for ELF header" if( $eh->ehsize != $HEADER_SIZE );
   $eh;
}

sub size { return $HEADER_SIZE }

1;
