package BinFile::ELF::ELF32::Section;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Section.pm,v 1.9 2001/01/18 00:05:30 stevek Exp $

require 5.004;
use strict;

use Exporter    ();
use vars        qw(@EXPORT @ISA $VERSION $SECTION_SIZE);
use BinFile::ELF::Section;
use Carp;

@ISA          = qw(BinFile::ELF::Section);
@EXPORT       = qw();
$VERSION      = 0.002;
$SECTION_SIZE = 40;

use Class::OOStruct qw(substruct);

substruct [];

sub load ($$@) {
   my $this = shift;
   my $index = shift;
   my($elf,$fd) = @_;

   croak 'Invalid index' if ref($index);

   if( ref($elf) eq 'GLOB' ) {
      $fd = $elf;
      undef $elf;
   }
   elsif( !$fd ) {
      $fd = $elf->descriptor;
   }
   croak 'Invalid descriptor' if( $fd && ref($fd) ne 'GLOB' );

   my $section;
   my $total = 0;
   while( $total < $SECTION_SIZE ) {
      my $nread = sysread( $fd, $section, $SECTION_SIZE, $total );
      croak "Unable to read ELF section: $!" unless $nread;
      $total += $nread;
   }
   croak "Unable to read ELF section: $!" if( $total < $SECTION_SIZE );

   my @parts;
   @parts = unpack('VVVVVVVVVV', $section)
      if $elf->elfheader->data == 0x01; # Little Endian
   @parts = unpack('nnnnnnnnnn', $section)
      if $elf->elfheader->data == 0x02; # Big Endian

   ( $this->[0], $this->[1],
     $this->[2], $this->[3],
     $this->[4], $this->[5],
     $this->[6], $this->[7],
     $this->[8], $this->[9] ) = @parts[ (0..9) ];
   $this->index($index);
   $this->elf($elf) if ref($elf);
   $elf->addsection($this) if ref($elf);
   $this;
}

1;
