package BinFile::ELF::ELF32::Symbol;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Symbol.pm,v 1.3 2001/01/18 16:36:25 stevek Exp $
#
require 5.004;
use strict;
use vars qw(@ISA @EXPORT $VERSION $SYM_SIZE);
use Exporter;
use Class::OOStruct qw(substruct);
use BinFile::ELF::Symbol;
use Carp;

@ISA      = qw(BinFile::ELF::Symbol);
@EXPORT   = qw();

$VERSION  = 0.001;
$SYM_SIZE = 16;

substruct [];

sub load ($@) {
   my $this = shift;
   my($elf,$symtab,$fd);
   croak 'Invalid number of arguments' unless 3 >= @_;
   while( @_ ) {
      my $ref = shift;
      croak 'Not a reference argument' unless ref($ref);
      $elf = $ref, next if $ref->isa('BinFile::ELF');
      $symtab = $ref, next if $ref->isa('BinFile::SymbolTable');
      $fd = $ref, next if ref($ref) eq 'GLOB';
      croak 'Unknown argument';
   }

   croak 'Missing ELF reference argument' unless ref($elf);
   $fd = $elf->descriptor if !$fd;

   my $symbol;
   my $total = 0;
   while( $total < $SYM_SIZE ) {
      my $nread = sysread( $fd, $symbol, $SYM_SIZE, $total );
      croak "Unable to read ELF symbol: $!" unless $nread;
      $total += $nread;
   }
   croak "Unable to read ELF symbol: $!" if( $total < $SYM_SIZE );

   my @parts;
   @parts = unpack('VVV CC v', $symbol)
      if $elf->elfheader->data == 0x01; # Little Endian
   @parts = unpack('NNN CC n', $symbol)
      if $elf->elfheader->data == 0x02; # Big Endian

   $this->symtab($symtab) if ref($symtab);
   $this->name($parts[0]);
   $this->addr($parts[1]);
   $this->size($parts[2]);
   $this->info($parts[3]);
   $this->other($parts[4]);
   $this->shndx($parts[5]);
   $symtab->addsymbol($this) if ref($symtab);
   $this;
}

1;
