package BinFile::ELF::Section;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Section.pm,v 1.8 2001/01/18 00:05:00 stevek Exp $

require 5.004;
use strict;
use Carp;
use BinFile::ELF::Header;
use IO::Seekable;
use Data::Dumper;

BEGIN {
   use Exporter ();
   use vars     qw(@EXPORT $VERSION);

   @EXPORT    = qw(SHT_NULL SHT_PROGBITS SHT_SYMTAB SHT_STRTAB SHT_RELA SHT_HASH
                   SHT_DYNAMIC SHT_NOTE SHT_NOBITS SHT_REL SHT_SHLIB SHT_DYNSYM
                   SHF_WRITE SHF_ALLOC SHF_EXECINSTR);
   $VERSION   = 0.001;
}

# Types for sh_type
sub SHT_NULL { 0x00; }     # Inactive
sub SHT_PROGBITS { 0x01; } # Program defined information
sub SHT_SYMTAB { 0x02; }   # Symbol table section
sub SHT_STRTAB { 0x03; }   # String table section
sub SHT_RELA { 0x04; }     # Relocation section with addends
sub SHT_HASH { 0x05; }     # Symbol hash table section
sub SHT_DYNAMIC { 0x06; }  # Dynamic section
sub SHT_NOTE { 0x07; }     # Note section
sub SHT_NOBITS { 0x08; }   # No space section
sub SHT_REL { 0x09; }      # Relocation section without addends
sub SHT_SHLIB { 0x0a; }    # Reserved
sub SHT_DYNSYM { 0x0b; }   # Dynamic symbol table section

# Flags for sh_flags
sub SHF_WRITE { 0x1; }     # Section contains writable data
sub SHF_ALLOC { 0x2; }     # Section occupies memory
sub SHF_EXECINSTR { 0x4; } # Section contains instructions


sub import { goto &Exporter::import }

use Class::OOStruct qw(struct);
struct 'BinFile::ELF::Section' => [
   name      => '$',
   type      => '$',
   flags     => '$',
   addr      => '$',
   offset    => '$',
   size      => '$',
   link      => '$',
   info      => '$',
   addralign => '$',
   entsize   => '$',
   elf       => '$',
   index     => '$'
];

sub dump ($) {
   my $this = shift;
   my $hdrstr = shift if @_;

   printf("     name: %8.8x", $this->name);
   printf(" [%s]", $hdrstr->string($this->name) || '') if $hdrstr;
   print("\n");
   printf("     type: %8.8x", $this->type);
   print(" [Inactive]\n")               if $this->type == SHT_NULL;
   print(" [Program Bits]\n")           if $this->type == SHT_PROGBITS;
   print(" [Symbol Table]\n")           if $this->type == SHT_SYMTAB;
   print(" [String Table]\n")           if $this->type == SHT_STRTAB;
   print(" [Relocations w/Addends]\n")  if $this->type == SHT_RELA;
   print(" [Symbol Hash Table]\n")      if $this->type == SHT_HASH;
   print(" [Dynamic]\n")                if $this->type == SHT_DYNAMIC;
   print(" [Note]\n")                   if $this->type == SHT_NOTE;
   print(" [No Space]\n")               if $this->type == SHT_NOBITS;
   print(" [Relocation w/o Addends]\n") if $this->type == SHT_REL;
   print(" [Reserved]\n")               if $this->type == SHT_SHLIB;
   print(" [Dynamic Symbol Table]\n")   if $this->type == SHT_DYNSYM;
   print(" [UNKNOWN]\n")                if $this->type > SHT_DYNSYM;
   printf("    flags: %8.8x\n", $this->flags);
   printf("     addr: %8.8x\n", $this->addr);
   printf("   offset: %8.8x\n", $this->offset);
   printf("     size: %8.8x\n", $this->size);
   printf("     link: %8.8x\n", $this->link);
   printf("     info: %8.8x\n", $this->info);
   printf("addralign: %8.8x\n", $this->addralign);
   printf("  entsize: %8.8x\n", $this->entsize);
}

1;
