package BinFile::ELF::StringTable;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: StringTable.pm,v 1.10 2001/01/17 23:07:54 stevek Exp $
#
require 5.004;
use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Exporter;
use BinFile::StringTable;
use BinFile::ELF::Section;
use Carp;
use IO::Seekable;

@ISA     = qw(BinFile::StringTable);
@EXPORT  = qw();

$VERSION = 0.001;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {};
   bless $self, $class;
   $self->_initialize(@_);
}

sub _initialize ($@) {
   my $self = shift;
   $self->SUPER::_initialize(@_);
}

sub load ($$@) {
   my $this = shift;
   my $index = shift;
   my($elf,$fd) = @_;

   if( ref($elf) eq 'GLOB' ) {
      $fd = $elf;
      undef $elf;
   }
   elsif( !$fd ) {
      $fd = $elf->descriptor;
   }
   croak 'Invalid descriptor' if( $fd && ref($fd) ne 'GLOB' );
   croak 'Not an ELF instance' unless !$elf || $elf->isa('BinFile::ELF');

   my $strtab;
   if( ref($index) && $index->isa('BinFile::ELF::Section') ) {
      $strtab = $index;
   }
   else {
      croak 'Missing ELF instance argument' unless $elf;
      my @sections = $elf->elfsections;
      $strtab = $sections[$index];
      croak "Not an ELF section" unless $strtab->isa('BinFile::ELF::Section');
      croak "Not a String Table section" unless $strtab->type == SHT_STRTAB;
   }

   sysseek( $fd, $strtab->offset, SEEK_SET );
   my $size = $strtab->size;

   my $chars;
   my $total = 0;
   while( $total < $size ) {
      my $nread = sysread( $fd, $chars, $size, $total );
      croak "Unable to read ELF strings" unless $nread;
      $total += $nread;
   }
   croak "Unable to read ELF strings" if( $total < $size );
   my @bytes = unpack('C'.$size, $chars);
   $this->strings( \@bytes );
   $this->section($strtab);
   $elf->addstrtab($this) if ref($elf);
   $this;
}

sub section ($@) {
   my $this = shift;
   $this->{'Section'} = shift if @_;
   $this->{'Section'};
}

1;
