package BinFile::ELF::Symbol;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Symbol.pm,v 1.7 2001/01/19 19:06:42 stevek Exp $
#
require 5.004;
use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Exporter;
use Class::OOStruct qw(substruct);
use BinFile::Symbol;
use Carp;
use IO::Seekable;

@ISA     = qw(BinFile::Symbol);
@EXPORT  = qw();

$VERSION = 0.001;

substruct [
   namendx => '$',
   size    => '$',
   info    => '$',
   other   => '$',
   shndx   => '$'
];

sub name ($@) {
   my $this = shift;

   if( @_ ) {
      my $ndx = shift;

      my $symtab = $this->symtab;
      my $section = $symtab->section if $symtab;
      my $elf = $section->elf if $section;
      my $strtab = $elf->elfstrtabs->{$section->link} if $elf;

      if( $strtab ) {
         $this->SUPER::name($strtab->string($ndx));
         $this->namendx($ndx);
      }
      else {
         $this->SUPER::name($ndx);
         $this->namendx($ndx);
      }
   }
   else {
      $this->SUPER::name;
   }
}

sub value ($@) {
   my $this = shift;
   $this->addr(@_);
}

sub isUnknown ($) {
   my $this = shift;
   ($this->shndx)?0:1;
}

sub dump ($) {
   my $this = shift;

   my $symtab = $this->symtab;
   my $section = $symtab->section if $symtab;
   my $elf = $section->elf if $section;

   printf("\t[%s Symbol]\n", ($elf?$elf->format:'ELF'));
   printf("\t name: %8.8x", $this->namendx);
   my $name = $this->name;
   my $namendx = $this->namendx;
   printf(" [%s]", $this->name || '') if $name && $namendx && $name ne $namendx;
   print("\n");
   printf("\t addr: %8.8x\n", $this->addr);
   printf("\t size: %8.8x\n", $this->size);
   printf("\t info: %8.8x\n", $this->info);
   printf("\tother: %8.8x\n", $this->other);
   printf("\tshndx: %8.8x\n", $this->shndx);
}

1;
