package BinFile::ELF::SymbolTable;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: SymbolTable.pm,v 1.6 2001/01/18 16:36:52 stevek Exp $
#
require 5.004;
use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Exporter;
use BinFile::SymbolTable;
use BinFile::ELF::Section;
use Carp;
use IO::Seekable;

@ISA     = qw(BinFile::SymbolTable);
@EXPORT  = qw();

$VERSION = 0.001;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $self = {};
   bless $self, $class;
   $self->_initialize(@_);
}

sub _initialize ($@) {
   my $self = shift;
   $self->SUPER::_initialize(@_);
}

sub load ($$@) {
   my $this = shift;
   my $index = shift;
   my($elf,$fd) = @_;

   if( ref($elf) eq 'GLOB' ) {
      $fd = $elf;
      undef $elf;
   }
   elsif( !$fd ) {
      $fd = $elf->descriptor;
   }
   croak 'Invalid descriptor' if( $fd && ref($fd) ne 'GLOB' );
   croak 'Not an ELF instance' unless !$elf || $elf->isa('BinFile::ELF');

   my $symtab;
   if( ref($index) && $index->isa('BinFile::ELF::Section') ) {
      $symtab = $index;
   }
   else {
      croak 'Missing ELF instance argument' unless $elf;
      my @sections = $elf->elfsections;
      $symtab = $sections[$index];
      croak "Not an ELF section" unless $symtab->isa('BinFile::ELF::Section');
      croak "Not a Symbol Table section" unless $symtab->type == SHT_SYMTAB;
   }

   my @sections = $elf->elfsections;
   my $strtab = $sections[$symtab->link];
   croak 'Invalid linkage reference for symbol table'
      unless $strtab && $strtab->type == SHT_STRTAB;

   $strtab = $elf->elfstrtabs->{$symtab->link};
   if( !$strtab ) {
      $strtab = $elf->strtabclass->new;
      $strtab->load($symtab->link,$elf);
   }
   croak 'Unable to locate linkage reference section for symbol table'
      unless ref($strtab) && $strtab->isa($elf->strtabclass);

   sysseek( $fd, $symtab->offset, SEEK_SET );
   my $nentries = $symtab->size / $symtab->entsize;

   $this->section($symtab);
   $elf->addsymtab($this) if ref($elf);

   my $symclass = $elf->formatclass.'::Symbol';
   my $result = eval 'use '.$symclass;
   carp $@ if $@;
   foreach( 0 .. ($nentries - 1) ) {
      my $sym = $symclass->new;
      $sym->load($elf,$this);
   }
   $this;
}

sub section ($@) {
   my $this = shift;
   $this->{'Section'} = shift if @_;
   $this->{'Section'};
}

1;
