package BinFile::PE;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: PE.pm,v 1.16 2001/04/05 18:41:11 stevek Exp $

require 5.004;
use strict;
use vars qw($VERSION @ISA @EXPORT);
use Carp;
use BinFile::DOS::EXE;
use BinFile::PE::OptionalHeader;
use BinFile::COFF;
use IO::Seekable;

$VERSION = 0.001;
@ISA     = qw(BinFile::DOS::EXE);
@EXPORT  = qw();

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my $file = shift;
   my $options = @_?shift:0;
   my $self = {};
   bless $self, $class;
   $self->_initialize($file,$options);
}

sub _initialize {
   my $self = shift;
   my $file = shift;
   my $options = shift;

   # Read DOS EXE stub
   $self->SUPER::_initialize( $file, $options );
   my $fd;
   *fd = $self->{'Descriptor'};

   sysseek( *fd, $self->dosheader->lfanew, SEEK_SET );

   # Read signature
   my $sig;
   my $total = 0;
   while( $total < 4 ) {
      my $nread = sysread( *fd, $sig, 4, $total );
      croak "Unable to read PE signature" unless $nread;
      $total += $nread;
   }
   croak "Unabled to read PE signature" if( $total < 4 );
   croak "Not a PE file" unless $sig eq "PE\0\0";

   # Read COFF parts
   my $coff = new BinFile::COFF( \*fd );

   # Seek to Optional Header and read
   sysseek( *fd, $self->dosheader->lfanew + 4 + $coff->coffheader->size,
            SEEK_SET );
   my $optHeader = loadPEOptionalHeader( \*fd, $coff );

   $self->{'PE Signature'}    = $sig;
   $self->{'COFF'}            = $coff;
   $self->{'Optional Header'} = $optHeader;
   $self;
}

sub format ($) { "PE"; }

sub sig {
   my $this = shift;
   $this->{'PE Signature'};
}

sub coff {
   my $this = shift;
   $this->{'COFF'};
}

sub optheader {
   my $this = shift;
   $this->{'Optional Header'};
}

sub findDirectoryByName ($, $) {
   my $this = shift;
   my $name = shift;
   my $offset;

   my $directory = $this->optheader->directory;
   $offset = $this->coff->findSectionByRVA( $directory->{$name}->{'RVA'} )
      if( exists $directory->{$name} );
   $offset;
}

sub dump {
   my $this = shift;
   $this->coff->dumpheader();
   print("\n");
   $this->optheader->dump();
   print("\n");
   $this->coff->dumpsections();
}

1;
