package BinFile::PE::Directory::Import;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Import.pm,v 1.4 2001/04/05 18:41:14 stevek Exp $

require 5.004;
use strict;
use Carp;
use IO::Seekable;

BEGIN {
   use Exporter   ();
   use vars       qw(@EXPORT $VERSION $DIR_SIZE);
   @EXPORT      = qw(loadPEImportDirectory);
   $VERSION     = 0.001;
   $DIR_SIZE    = 20;
}

sub import { goto &Exporter::import }

use Class::Struct qw(struct);
struct 'BinFile::PE::Directory::Import' => [
   charistics   => '$', # 4 bytes  (unsigned long)  offset 0
   timestamp    => '$', # 4 bytes  (unsigned long)  offset 4
   forwardchain => '$', # 4 bytes  (unsigned long)  offset 8
   name         => '$', # 4 bytes  (unsigned long)  offset 12
   firstthunk   => '$'  # 4 bytes  (unsigned long)  offset 16
];

sub loadPEImportDirectory($) {
   my $fd = shift;

   croak "Invalid argument" unless ref($fd);

   my $dir;
   my $total = 0;
   while( $total < $DIR_SIZE ) {
      my $nread = sysread( $fd, $dir, $DIR_SIZE, $total );
      croak "Unable to read PE Import directory" unless $nread;
      $total += $nread;
   }
   croak "Unable to read PE Import directory" if( $total < $DIR_SIZE );
   my @parts = unpack('VVVVV', $dir);
   my $ped;
   if( $parts[0] ) {
      $ped = new();
      ($ped->[0], $ped->[1],
       $ped->[2], $ped->[3],
       $ped->[4])            = @parts[ (0..4) ];
   }
   $ped;
}

sub size { return $DIR_SIZE; }

sub dump ($) {
   my $this = shift;
   printf("[PE Import Directory]\n");
   printf("  charistics: %8.8x\n", $this->charistics);
   printf("   timestamp: %8.8x\n", $this->timestamp);
   printf("forwardchain: %8.8x\n", $this->forwardchain);
   printf("        name: %8.8x\n", $this->name);
   printf("  firstthunk: %8.8x\n", $this->firstthunk);
}

1;
