package BinFile::PE::Directory::Import::Name;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Name.pm,v 1.3 2001/04/05 18:41:14 stevek Exp $

require 5.004;
use strict;
use Carp;
use IO::Seekable;

BEGIN {
   use Exporter   ();
   use vars       qw(@EXPORT $VERSION $NAME_SIZE);
   @EXPORT      = qw(loadPEImportName);
   $VERSION     = 0.001;
   $NAME_SIZE   = 3;
}

sub import { goto &Exporter::import }

use Class::Struct qw(struct);
struct 'BinFile::PE::Directory::Import::Name' => [
   hint => '$', # 2 bytes  (unsigned short)   offset 0
   name => '$', # n bytes  (unsigned char[])  offset 2
];

sub loadPEImportName ($) {
   my $fd = shift;

   croak "Invalid argument" unless ref($fd);

   my $name;
   my $total = 0;
   while( $total < $NAME_SIZE ) {
      my $nread = sysread( $fd, $name, $NAME_SIZE, $total );
      croak "Unable to read PE Import name" unless $nread;
      $total += $nread;
   }
   croak "Unable to read PE Import name" if( $total < $NAME_SIZE );

   my @parts = unpack('v C', $name);
   return undef if $parts[ $#parts ] == 0;

   while( $parts[ $#parts ] ) {
      my $c;
      my $nread = sysread( $fd, $c, 1 );
      croak "Unable to read PE Import name" unless $nread;
      push( @parts, unpack('C',$c) );
   }
   my $ped;
   if( $parts[0] ) {
      $ped = new();
      $ped->[0] = $parts[0];
      $ped->[1] = pack('C'.($#parts-1), @parts[ (1..($#parts)) ]);
   }
   $ped;
}

sub size { return $NAME_SIZE; }

sub dump ($) {
   my $this = shift;
   printf("[PE Import Name]\n");
   printf("hint: %4.4x\n", $this->hint);
   printf("name: %s\n", $this->name);
}

1;
