package BinFile::StringTable;
# 
# Copyright (C) 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: StringTable.pm,v 1.3 2001/01/19 21:47:15 stevek Exp $
#
require 5.004;
use strict;
use vars qw(@ISA $VERSION);
use Exporter;
use Carp;
use IO::Seekable;

@ISA     = qw(Exporter);

$VERSION = 0.001;

sub new {
   my $this = shift;
   my $class = ref($this) || $this;
   my @data = @_;
   my $self = {};
   bless $self, $class;
   $self->_initialize(\@data);
}

sub _initialize ($@) {
   my $self = shift;
   $self->strings(@_);
   $self;
}

sub strings ($@) {
   my $this = shift;
   croak 'Argument must be an array reference' if @_ && ref($_[0]) ne 'ARRAY';
   $this->{'Strings'} = shift if @_;
   $this->{'Strings'};
}

sub string ($$) {
   my $self = shift;
   my $offset = shift;
   my @strings = @{$self->{'Strings'}};
   my $nstrings = @strings;
   (warn "Offset out of range ($offset >= $nstrings)"), return undef
      if ($offset < 0) || ($offset >= $nstrings);

   my $end = $offset;
   foreach( $offset .. ($nstrings - 1) ) {
      $end = $_, last if $strings[$_] == 0x00;
   }
   return undef if $end == $offset;
   pack('C'.($end-$offset), @strings[$offset..$end-1]);
}

sub dump ($) {
   my $self = shift;
   printf("[String Table]\n");
   my @strings = @{$self->{'Strings'}};
   my $nstrings = @strings;
   foreach( 0 .. ($nstrings - 1) ) {
      print "\n" if ($_ % 16) == 0 && $_ > 0;
      print " " if ($_ % 16) == 8;
      printf("   %4.4x:", $_) if ($_ % 16) == 0;
      printf(" %2.2x", $strings[$_]);
   }
   print "\n" if $nstrings > 0;
}

1;
