#!/usr/local/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: dumper.pl,v 1.6 2001/04/05 18:41:15 stevek Exp $

require 5.004;
use strict;
use CPU::x86::Opcodes qw(:FIELDS);
use BinFile::PE;
use IO::Seekable;
use Fcntl;
use BinFile::PE::Directory::Export;
use BinFile::PE::Directory::Import;
use BinFile::PE::Directory::Import::Name;

die "Usage: dumper.pl <module>" if !@ARGV;

my @searchpath = split(';', $ENV{'PATH'});

sub openFile {
   my( $filename ) = @_;
   my $fp;

   foreach( @searchpath ) {
      my $name = $_ . '\\' . $filename;
      return \*fp if( sysopen( *fp, $name, O_RDONLY|eval('O_BINARY') ) );
   }
}

my $fp = openFile( $ARGV[0] );
die $ARGV[0].": $!" if !$fp;
my $exe = new BinFile::PE( \*fp );
$exe->dump();

# Read Export Directory
if( exists $exe->optheader->directory->{'Export'}->{'Instance'} ) {
   my $inst = $exe->optheader->directory->{'Export'}->{'Instance'};
   (print "\n"), $inst->dump() if defined($inst);
}

# Read Import Directory
my $offset = $exe->findDirectoryByName('Import');
if( $offset ) {
   sysseek( *fp, $offset, SEEK_SET ) || die;

   my $dirSize = $exe->optheader->directory->{'Import'}->[1];
   my $dirOff = 0;
   my $importDir;
   while( $dirOff < $dirSize ) {
      $importDir = loadPEImportDirectory( \*fp ) || last;
      print "\n";
      $importDir->dump();
      $dirOff += BinFile::PE::Directory::Import::size();

      # Read DLL name
      my $nameOff = $exe->findDirectoryByRVA($importDir->name);
      sysseek( *fp, $nameOff, SEEK_SET ) || die;
      my @parts;
      do {
         my $c;
         sysread( *fp, $c, 1 ) || die;
         push( @parts, unpack('C', $c) );
      }
      while( $parts[ $#parts ] );
      print "\n   [" . pack('C'.($#parts), @parts) . "]\n"
         if( @parts );

      # Read names off of characteristic pointer
      my $charOff = $exe->findDirectoryByRVA($importDir->charistics);
      sysseek( *fp, $charOff, SEEK_SET ) || die;
      my $symNum = 0;
      while( 1 ) {
         my $rva;
         sysread( *fp, $rva, 4 ) || die;
         ( $rva ) = unpack('V', $rva);
         last if !$rva;

         $nameOff = $exe->findDirectoryByRVA($rva);
         sysseek( *fp, $nameOff, SEEK_SET ) || die;

         my $impName = loadPEImportName( \*fp ) || die;
         print("     Hint  Name\n") if( !$symNum++ );
         printf("     %4d   %s\n", $impName->hint, $impName->name);

         $charOff += 4;
         sysseek( *fp, $charOff, SEEK_SET ) || die;
      }
      sysseek( *fp, $offset + $dirOff, SEEK_SET ) || die;
   }
}

