#!/usr/local/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: loader.pl,v 1.13 2001/04/05 18:41:15 stevek Exp $

require 5.004;
use strict;
use CPU::x86::Opcodes qw(:FIELDS);
use BinFile::PE;
use IO::Seekable;
use Fcntl;
use BinFile::PE::Directory::Import;
use BinFile::PE::Directory::Import::Name;
use Data::Dumper;

my %runtime;
my @searchpath = split(';', $ENV{'PATH'});
push( @searchpath, '.' );

sub openFile {
   my( $filename ) = @_;
   my $fp;

   foreach( @searchpath ) {
      my $name = $_ . '\\' . $filename;
      return \*fp if( sysopen( *fp, $name, O_RDONLY|eval('O_BINARY') ) );
   }
   undef;
}

sub loadModule {
   my( $prfx, $name, $filename ) = @_;

   print $prfx."[Module $filename]\n";
   return if( exists $runtime{$name} );

   my $fp = openFile( $filename );
   die "Unable to find or load module '$filename': $!" unless defined($fp);
   my $exe = new BinFile::PE( \*fp );

   $runtime{$name} = {
      'File'     => $fp,
      'Filename' => $filename,
   };

   # Read Import Directory
   my $offset = $exe->findDirectoryByName('Import');
   if( $offset ) {
      print '*'.$prfx.'Loading imports'."\n";

      sysseek( *fp, $offset, SEEK_SET ) || die;
      $runtime{$name}->{'Imports'} = {};

      my $dirSize = $exe->optheader->directory->{'Import'}->{'Size'};
      my $dirOff = 0;
      my $importDir;
      while( $dirOff < $dirSize ) {
         $importDir = loadPEImportDirectory( \*fp ) || last;
         $dirOff += BinFile::PE::Directory::Import::size();

         # Read DLL name
         my $nameOff = $exe->coff->findSectionByRVA($importDir->name);
         sysseek( *fp, $nameOff, SEEK_SET ) || die;
         my @parts;
         do {
            my $c;
            sysread( *fp, $c, 1 ) || die;
            push( @parts, unpack('C', $c) );
         }
         while( $parts[ $#parts ] );
         next if( !@parts );

         my $newDll = pack('C'.($#parts), @parts);
         $runtime{$name}->{'Imports'}->{$newDll} = $importDir;
         sysseek( *fp, $offset + $dirOff, SEEK_SET ) || die;
      }

      my $imports = $runtime{$name}->{'Imports'};
      foreach( keys %$imports ) {
         loadModule('*'.$prfx, $_, $_);
      }
   }
}

my $module = 'test\\hello.exe';
$module = $ARGV[0] if @ARGV;
loadModule(' ', '%%Main%%', $module );

print Dumper(%runtime);

