#!/usr/local/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: opcode.pl,v 1.7 2001/04/05 18:41:15 stevek Exp $

use CPU::x86::Opcodes qw(:FIELDS);
# use Data::Dumper;

# Generate mask and required shift
sub makeMask {
   my( $bchar, $bits ) = @_;

   my @bits = map(chr, unpack("C*", $bits));

   my( $mask, $shift, $bit ) = (0,0,0);
   foreach $bit ( @bits ) {
      $shift++;
      $mask <<= 1;
      $shift = 0, $mask |= 1 if $bit =~ /[$bchar]/x;
   }

   my $bytes = @bits;
   $bytes /= 8;

   if( $mask ) {
      foreach( $bytes ) {
         /^1$/ && do { $mask = unpack("B*", pack("C", $mask)); last; };
         /^2$/ && do { $mask = unpack("B*", pack("n", $mask)); last; };
         /^3$/ && do { my @bytes = unpack("C4", pack("N", $mask));
                       $mask = unpack("B*", pack("C3", @bytes[1..3])); last; };
         /^4$/ && do { $mask = unpack("B*", pack("N", $mask)); last; };
      }
   }

   return undef if !$mask;
   [ $bytes, $shift, $mask ];
}

my @newcodes;

foreach( @$x86_opcodes ) {
   my $entry = [ $_->[0], $_->[1], 0, undef, undef, {} ];

   my $r = makeMask( '10', $_->[2] );
   $entry->[2] = ($r ? $r->[0] : 0);
   $entry->[3] = $r->[2];
   $r = makeMask( '1', $_->[2] );
   $entry->[4] = $r->[2];
   $r = makeMask( 'w', $_->[2] );
   $entry->[5]->{'RegType'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   $r = makeMask( 's', $_->[2] );
   $entry->[5]->{'SegmentReg'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   $r = makeMask( 'o', $_->[2] );
   $entry->[5]->{'OpcodeFunc'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   $r = makeMask( 'm', $_->[2] );
   $entry->[5]->{'IndexFunc'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   $r = makeMask( 'r', $_->[2] );
   if( $_->[2] =~ /w/ ) {
      $entry->[5]->{'IndexReg'} = undef;
      $entry->[5]->{'Reg'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   }
   else {
      $entry->[5]->{'Reg'} = undef;
      $entry->[5]->{'IndexReg'} = ($r ? [ $r->[1], $r->[2] ] : undef);
   }

   push( @newcodes, $entry );
}

my @x86_opcodes = sort { $a->[0] cmp $b->[0] } @newcodes;

open( OUTPM, ">opcodes.pm" ) || die;
my $ref = select( OUTPM );

print <<EOF;
package CPU::x86::Opcodes;
use strict;

BEGIN {
   use Exporter;
   use vars qw(\$VERSION \@ISA \@EXPORT_OK \%EXPORT_TAGS);

   \$VERSION = 0.001;
   \@EXPORT_OK = qw( \@x86_opcodes \%x86_index_registers \%x86_registers
                    \$x86_segments );
   \%EXPORT_TAGS = ( FIELDS => [ \@EXPORT_OK ] );
}

use vars \@EXPORT_OK;

sub import { goto \&Exporter::import }

\@x86_opcodes = (
EOF

my $index = 0;
foreach( @x86_opcodes ) {
   print ",\n" if( $index++ );
   print '   [ "' . $_->[0] . '", "' . $_->[1] . '", ' . $_->[2] . ",\n";
   print '     ' . ($_->[3] ? '"'.$_->[3].'"' : 'undef') . ', ';
   print '' . ($_->[4] ? '"'.$_->[4].'"' : 'undef') . ",\n";
   print "     {\n";
   my $index2 = 0;
   while( my($key, $value) = each %{$_->[5]} ) {
      print ",\n" if( $index2++ );
      print '       "' . sprintf("%-11s", $key . '"') . ' => ';
      if( defined($value) ) {
         print '[ ' . $value->[0] . ', "' . $value->[1] . '" ]';
      }
      else {
         print 'undef';
      }
   }
   print "\n     }\n   ]";
}
print <<EOF;

);

1;
EOF

select( $ref );
close( OUTPM );

