/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: dbck.c,v 1.26 2001/02/23 19:02:22 stevek Exp $
 */

#include <wr_conf.h>
#include <stdarg.h>
#include <wsi_stdlib.h>

#include <stdio.h>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef WR_WINNT
#include <sys/mman.h>
#include <unistd.h>
#else
#include <io.h>
#endif

#include <wr_conf.h>

#include <pdpi/queue.h>

#include <wss_db.h>
#include <wss_vm.h>

#define TRUE  1
#define FALSE 0

u_long wss_config_db_handle;

extern pd_char_t *wss_db_filename;
extern size_t     wss_db_size;

#ifdef panic
#undef panic
#endif
void panic( const char *, ... );

#ifdef WR_WINNT
#define stat  _stat

pd_char_t *
wsi_strdup( const pd_char_t *str )
{
   return _tcsdup( str );
}

pd_char_t *
wss_strdup( const pd_char_t *str )
{
   return _tcsdup( str );
}
#endif /* WR_WINNT */

void
wss_db_lock()
{
   /* noop */
}

void
wss_db_unlock()
{
   /* noop */
}

void
wsi_file_close( wss_file_t file )
{
/*
   _tprintf( PD_MAKESTR("VM Shutting down...\n") );
 wss_file_t file #ifdef WR_WINNT
   if( !UnmapViewOfFile( wss_vm_data ) )
#else
   if( munmap( (caddr_t)wss_vm_data, size_in_bytes ) < 0 )
#endif
      perror( "munmap" );
*/
}

vm_offset_t
_wss_vm_map_file( const pd_char_t *filename, size_t size_in_bytes,
                  wss_file_t *pfile, pd_bool_t *isnew )
{
#ifdef WR_WINNT
   HANDLE       hFile;
   DWORD        nBytes;
#else
   int          fd;
#endif /* WR_WINNT */
   struct stat  sb;
   off_t        offset;
   vm_offset_t  vm_offset;
   pd_bool_t    newfile = FALSE;

   if( stat( filename, &sb ) )
   {
#ifdef WR_WINNT
      if( ( hFile = CreateFile( filename, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                                CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL ) )
          == INVALID_HANDLE_VALUE )
#else
      if( ( fd = open( filename, O_RDWR | O_CREAT, 0644 ) ) < 0 )
#endif /* WR_WINNT */
      {
         perror( "open" );
         return 0;
      }

#ifdef WR_WINNT
      offset = SetFilePointer( hFile, size_in_bytes - 1, NULL, FILE_BEGIN );
#else
      offset = lseek( fd, size_in_bytes - 1, SEEK_SET );
#endif /* WR_WINNT */

#ifdef WR_WINNT
      WriteFile( hFile, "x", 1, &nBytes, NULL );
#else
      write( fd, "x", 1 );
#endif /* WR_WINNT */

      newfile = TRUE;
   }
   else
   {
#ifdef WR_WINNT
      if( ( hFile = CreateFile( filename, GENERIC_READ | GENERIC_WRITE, 0, NULL,
                                OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL ) )
          == INVALID_HANDLE_VALUE )
#else
      if( ( fd = open( filename, O_RDWR ) ) < 0 )
#endif /* WR_WINNT */
      {
         perror( "open" );
         return 0;
      }
      if( sb.st_size != size_in_bytes )
      {
         _tprintf( PD_MAKESTR("existing wss.db file wrong size!\n") );
         return 0;
      }
   }

#ifdef WR_WINNT
   vm_offset = (vm_offset_t)MapViewOfFile( CreateFileMapping( hFile, NULL,
                                                              PAGE_READWRITE,
                                                              0, size_in_bytes,
                                                              NULL ),
                                           FILE_MAP_READ | FILE_MAP_WRITE, 0, 0,
                                           size_in_bytes );
#else
   vm_offset = (vm_offset_t)mmap( (caddr_t) 0, size_in_bytes,
                                  (PROT_READ|PROT_WRITE), MAP_SHARED, fd, 0 );
#endif /* WR_WINNT */

#ifdef WR_WINNT
   if( vm_offset == 0 )
#else
   if( (void *)vm_offset == MAP_FAILED )
#endif /* WR_WINNT */
   {
      _tprintf( PD_MAKESTR("map failed!\n") );
      return 0;
   }

#ifdef WR_WINNT
   *pfile = (wss_file_t)hFile;
#else
   *pfile = (wss_file_t)fd;
#endif /* WR_WINNT */

   if( newfile )
   {
      wss_vmspace_header_t *header = (wss_vmspace_header_t *)vm_offset;
      bzero( header, sizeof(*header) );
      header->vms_head_version = WSS_VMS_VERSION;
   }

   if( isnew ) *isnew = newfile;

   return( vm_offset );
}

void
panic( const char *msg, ... )
{
   va_list ap;
   va_start( ap, msg );

   fprintf( stderr, "panic: " );
   vfprintf( stderr, msg, ap );
   fprintf( stderr, "\n" );

   abort();
}

void *
wsi_malloc( size_t size )
{
   return( malloc( size ) );
}

void
wsi_free( void *ptr )
{
   free( ptr );
}


#ifdef MALLOC_DEBUG
void *
wss_malloc_debug( size_t s, pd_char_t *file, int line )
{
   return wsi_malloc(s);
}

void
wss_free_debug( void *ptr )
{
   return wsi_free(ptr);
}
#endif /* MALLOC_DEBUG */


void
dbck_shutdown( void )
{
   u_long id = wss_db_find_table_by_names( "dbsynthetic", 0, "alerts" );
   if( id )
      wss_drop( id );
   wss_vm_shutdown();
}

int
main( int argc, char *argv[] )
{
   int retc;

   wss_db_filename = PD_MAKESTR("wss.db");
   if( argc == 2 )
      wss_db_size = atoi(argv[1]);
   if( wss_db_size <= 0 )
      wss_db_size = 16;

   wss_vm_init();
   retc = wss_db_init( TRUE );
   if( retc ) return retc;

   atexit( dbck_shutdown );

   if( !wss_db_find_table_by_names( "", 0, "CONFIG" ) )
   {
      wqtab_t tab;
      wqcd_t cd = wss_db_mcol_def( 2, "Parameter", pd_char_t_t, 64, "Value",
                                   pd_char_t_t, 1024 );
      wss_create( "", 0, "CONFIG", FALSE, cd, 2, &wss_config_db_handle );

      tab = wss_db_find_table_by_handle( wss_config_db_handle );
      tab->flags |= DB_FLAG_NONPERSISTENT;
   }

   if( !wss_db_find_table_by_names( "dbsynthetic", 0, "alerts" ) )
   {
      wqcd_t cd = wss_db_mcol_def( 2, "Parameter", pd_char_t_t, 64, "Value",
                                   pd_char_t_t, 1024 );
      wss_create( "dbsynthetic", 0, "alerts", FALSE, cd, 2,
                  &wss_config_db_handle );
   }

   return 0;
}

