%{
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: dbi.l,v 1.11 2001/04/05 18:41:16 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: Scanner for activation criteria
 * 
 */

#define token(c)   { return(c); }
#include <string.h>

char *dbi_strin = NULL;

int first_tok = 0;
%}

%{
/* redefine macro in order to optionally read from a string a provided file */

#define YY_INPUT(buf,result,max_size) \
	{ \
		int c; \
		c = *(dbi_strin++); \
		result = (c == '\0') ? YY_NULL : (buf[0] = c, 1); \
	}
%}

%x reg_ex_state

ID      [_a-zA-Z][a-zA-Z0-9_\-\.]*
NUM     [0-9\.]*
REGEX      [/a-zA-Z0-9_\-\.\^[\]$]*

%%

%{
	if (first_tok) {
		int tok = first_tok;
		first_tok = 0;
		return tok;
	}
%}

set		{       return(K_SET); }
where		{	return(K_WHERE); }
char		{	return(K_CHAR_T); }
float		{	return(K_FLOAT_T); }
int		{	return(K_INT_T); }
and		{	return(K_AND); }
or		{	return(K_OR); }
setuid		{	return(K_SETUID); }

setgid		{	return(K_SETGID); }

order		{
			/* "order" and "desc" are used as keywords by the
			 * select command.  When this gets moved over into
			 * wrapc I'll find some way of not using reserved-
			 * words.
			 */
			return(K_ORDER); }

desc		{	return(K_DESC); }

"\."		{	return('.');}

{ID}		{
                        dbilval.name = strdup(dbitext);
                        return IDENTIFIER;
		}

{NUM}		{
                        dbilval.name = strdup(dbitext);
                        return NUM;
		}

[ \t\n]		{}


"==>"		{	return(K_ARROW); }
"=~"		{	return(K_STRCMP); }
">="		{	return(K_GTE); }
">"		{	return(K_GT); }
"<="		{	return(K_LTE); }
"<"		{	return(K_LT); }
"<>"		{	return(K_NE); }
"="		{	return(K_EQ); }
"m|"		{	BEGIN(reg_ex_state); return(K_REGEX); }

<reg_ex_state>{REGEX}	{
                        dbilval.name = strdup(dbitext);
                        BEGIN(INITIAL);
                        return REGEX;
		}

"|"		{	token('|') }
"!"		{	token('!') }

"\{"		{	token('{') }
"\}"		{	token('}') }
"("		{	token('(') }
")"		{	token(')') }
";"		{	token(';') }
","		{	token(',') }
"*"		{	token('*') }
"+"		{	token('+') }
"/"		{	token('/') }
"-"		{	token('-') }

<<EOF>>		{	yyterminate(); }

.		{	dbierror("Character not recognized");
		}

%%

int
dbierror(char *s)
{
        printf("\n'%s' at '%s'  (%d)\n", s, dbitext, YY_START);
        return 0;
/*        exit(2);*/
}

void set_grammar(int start_token)
{
	first_tok = start_token;
}
