%{
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: dbi.y,v 1.19 2001/04/05 18:41:16 stevek Exp $
 *
 * author: Lee Badger
 *
 * Purpose: Parser for database utility.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <errno.h>
#include "queue.h"
#include <sys/types.h>
#include <sys/stat.h>

#include "wr_conf.h"
#ifdef WR_BROKEN_STL
#define __PUT_STATIC_DATA_MEMBERS_HERE
#include "template/pair.h"
#include "template/set"
#else /* WR_BROKEN_STL */
#include <pair.h>
#include <set>
#endif /* WR_BROKEN_STL */

#include <iostream.h>
#include "source.h"

extern "C" {
#include "../../wr.include/wr_iface.h"
#include "wss_db.h"
}
#include "dbi.h"
#include "smap.h"

int dbierror(char *s);
int dbilex();
int dbiparse();

extern char *dbi_strin;
char *saved_strin;
source *src_obj;
struct imp_head imp_head;
int debug_flag = 0;

int col_count;
smap<int> *name2num;
extern int wrapper_id;
extern char * table_name;
extern char * wrapper_name;

struct parse_result *parse_results;

extern "C" {
    void print_sxval(sxval_t val);
}

typedef struct col_def_list {
    wqcd cd;
    struct col_def_list *next;
} cdl, *cdl_t;

typedef struct val_list {
    wqval val;
    struct val_list *next;
} vall, *vall_t;

void fill_in_map(void) {
    if (name2num)
        delete name2num;
    name2num = new smap<int>;
    
    wqtab_t tab;
    tab = (wqtab_t)wss_db_find_table_by_names(wrapper_name, wrapper_id, table_name);
    
    int i=((wqtab_t)rel2abs(tab))->col_count - 1;
    wqcd_t col_defs = (wqcd_t)rel2abs(((wqtab_t)rel2abs(tab))->col_defs);
    for ( ; i >= 0 ; --i) {
        name2num->insert((char *)rel2abs(col_defs[i].name), i);
    }
}
%}

%union {
	char *name;
	operators_t op;
	source *src;
	wqcd_t coldef;
	wqrow_t rowdef;
	wqval_t valdef;
	wqsx_t sel_exp;
	wqoc_t order;
	enum sxop sxop;
	sxval_t val;
	tx_t term_exp;
        wqusl_t update_set_list;
        wquexp_t update_expression;
        cdl_t coldef_list;
        vall_t val_list;
}

%token	<name>		IDENTIFIER
%token	<name>		NUM
%token	<name>		REGEX
%token			K_AND K_OR K_ARROW K_STRCMP K_REGEX
%token			K_GTE K_GT K_LTE K_LT K_NE K_EQ K_SETUID K_SETGID

%token	CREATE_CMD INSERT_CMD SELECT_CMD DELETE_CMD UPDATE_CMD DROP_CMD

%token	K_CHAR_T K_INT_T K_FLOAT_T K_WHERE K_ORDER K_DESC K_SET

%left K_OR
%left K_AND

/*%type	<op> bin_op*/
/*%type	<name> value*/

/*%type	<src> rel_expr*/
/*%type	<src> bool_expr*/
%type	<sel_exp> select_expr select_stmt
%type	<order> order_clause
%type   <update_set_list> update_set_list
%type   <update_set_list> update_statement
%type   <update_expression> update_expression
%type   <update_expression> update_additive_expression
%type   <update_expression> update_multiplicative_expression
%type   <update_expression> update_primary_expression
/*%type	<src> setid_expr*/

%type	<coldef_list> cd;
%type	<coldef_list> cd_list;
%type	<coldef> create_grammar;
%type	<rowdef> insert_grammar;
%type	<rowdef> row_list;
%type	<rowdef> row;
%type	<val_list> val_list;
%type	<val_list> val;
%type	<sxop> rel_op;
%type	<val> vref;
%type	<val> column_name;
%type	<term_exp> term_exp;

/*%left K_AND K_OR '!'*/

%%
dbi:	CREATE_CMD create_grammar
	{
		parse_results->r.c.create = $2;
                parse_results->r.c.col_count = col_count;
		parse_results->cmd = create_c;
	}
	| INSERT_CMD insert_grammar
	{
		parse_results->r.insert = $2;
		parse_results->cmd = insert_c;
	}
	| SELECT_CMD {fill_in_map();} select_expr order_clause
	{
		parse_results->r.s.select = $3;
		parse_results->r.s.order = $4;
		parse_results->cmd = select_c;
	}
	| DELETE_CMD
	| UPDATE_CMD {fill_in_map();} K_SET update_set_list select_stmt
	{
		parse_results->r.u.select = $5;
		parse_results->r.u.update = $4;
		parse_results->cmd = update_c;
	}

/*
 * Grammar for the create command.
 */
create_grammar: '(' cd_list  ')'
		{
                    int count = 0;
                    wqcd_t retval;
                    cdl_t cd = $2;
                    while (cd) {
                        cd = cd->next;
                        count++;
                    }
                    col_count = count;
                    retval = (wqcd_t)malloc(count * sizeof(*retval));
                    cd = $2;
                    count = 0;
                    while (cd) {
                        retval[count] = cd->cd;
                        count++;
                        cd = cd->next;
                    }
                    $$ = retval;
		}

cd_list:	cd_list ',' cd
		{
			cdl_t cd = $1;
			while (cd->next) {
				cd = cd->next;
			}
			cd->next = $3;
		}
		| cd
		;

cd:	IDENTIFIER K_CHAR_T '(' NUM ')'
	{
		cdl_t cd = (cdl_t)malloc(sizeof(*cd));
		cd->cd.name = strdup($1);
		cd->cd.dtype = char_t;
		cd->cd.max_len = atoi($4);
		cd->next = NULL;
		$$ = cd;
	}
	| IDENTIFIER K_INT_T
	{
		cdl_t cd = (cdl_t)malloc(sizeof(*cd));
		cd->cd.name = strdup($1);
		cd->cd.dtype = int_t;
		cd->cd.max_len = 0;
		cd->next = NULL;
		$$ = cd;
	}
	| IDENTIFIER K_FLOAT_T
	{
		cdl_t cd = (cdl_t)malloc(sizeof(*cd));
		cd->cd.name = strdup($1);
		cd->cd.dtype = float_t;
		cd->cd.max_len = 0;
		cd->next = NULL;
		$$ = cd;
	}

/*
 * Grammar for the insert command.
 */
insert_grammar: row_list

row_list:	row_list ',' row
		{
			wqrow_t row = $1;
			while (row->next) {
				row = row->next;
			}
			row->next = $3;
		}
		| row
		;

row:		'(' val_list ')'
		{
                    int count = 0;
                    wqval_t retval;
                    vall_t vals = $2;
                    while (vals) {
                        vals = vals->next;
                        count++;
                    }
                    retval = (wqval_t)malloc(count * sizeof(*retval));
                    vals = $2;
                    count = 0;
                    while (vals) {
                        retval[count] = vals->val;
                        count++;
                        vals = vals->next;
                    }
                    wqrow_t row = (wqrow_t)malloc(sizeof(wqrow));
                    row->vals = retval;
                    row->next = NULL;
                    $$ = row;
		}

val_list:	val_list ',' val
		{
			vall_t val = $1;
			while (val->next) {
				val = val->next;
			}
			val->next = $3;
		}
		| val
		;

val:		IDENTIFIER
		{
			vall_t val = (vall_t)malloc(sizeof(*val));
			val->val.v.cval = strdup($1);
			val->next = NULL;
			$$ = val;
		}
		| NUM
		{
			vall_t val = (vall_t)malloc(sizeof(*val));
			if (strchr($1, '.')) {
				/* convert to float */
				val->val.v.fval= (float)atof($1);
			}
			else {
				val->val.v.ival = atoi($1);
			}
			val->next = NULL;
			$$ = val;
		}

/*
 * Grammar for the select command.
 */
select_stmt: K_WHERE select_expr
            {
                $$ = $2;
            }
            |
            {
                $$ = NULL;
            }
            ;

select_expr:	'(' select_expr ')'
		{
			$$ = $2
		}
		| select_expr K_AND select_expr
		{
			wqsx_t node = (wqsx_t)malloc(sizeof(wqsx));
			node->which = nonterm;
			node->un.children = (ntx_t)malloc(sizeof(ntx));
			node->un.children->left = $1;
			node->un.children->right = $3;
			node->un.children->logop = and_op;
			$$ = node;
		}
		| select_expr K_OR select_expr
		{
			wqsx_t node = (wqsx_t)malloc(sizeof(wqsx));
			node->which = nonterm;
			node->un.children = (ntx_t)malloc(sizeof(ntx));
			node->un.children->left = $1;
			node->un.children->right = $3;
			node->un.children->logop = or_op;
			$$ = node;
		}
		| term_exp
		{
			wqsx_t node = (wqsx_t)malloc(sizeof(wqsx));
			node->which = term;
			node->un.exp = $1;
			$$ = node;
		}

order_clause:
		/* Empty */
		{
			wqoc_t oc = (wqoc_t)malloc(sizeof(*oc));
			oc->order_field = -1;
			oc->decending = 0;
			$$ = oc;
		}
		| K_ORDER '.' IDENTIFIER
		{
			wqoc_t oc = (wqoc_t)malloc(sizeof(*oc));
			oc->order_field = (*name2num)[$3];
			oc->decending = 0;
			$$ = oc;
		}
		| K_ORDER '.'IDENTIFIER K_DESC
		{
			wqoc_t oc = (wqoc_t)malloc(sizeof(*oc));
			oc->order_field = (*name2num)[$3];
			oc->decending = 1;
			$$ = oc;
		}

term_exp:	vref rel_op vref
		{
			tx_t term = (tx_t)malloc(sizeof(tx));
			term->left = $1;
			term->op = $2;
			term->right = $3;
			$$ = term;
		}

rel_op:		K_GTE
		{ $$ = gte_op; }
		| K_GT
		{ $$ = gt_op; }
		| K_LTE
		{ $$ = lte_op; }
		| K_LT
		{ $$ = lt_op; }
		| K_NE
		{ $$ = ne_op; }
		| K_EQ 
		{ $$ = eq_op; }

vref:	'.' IDENTIFIER
	{
		/* a field reference, so we don't know the dtype yet */
		sxval_t val = (sxval_t)malloc(sizeof(sxval));
		bzero((char*)val, sizeof(sxval));
		val->which = field;
		val->v.f.column = (*name2num)[$2];
		$$ = val;
	}
	| IDENTIFIER
	{
		sxval_t val = (sxval_t)malloc(sizeof(sxval));
		bzero((char*)val, sizeof(sxval));
		val->which = literal;
		val->v.l.cval = strdup($1);
		val->dtype = char_t;
		$$ = val;
	}
	| NUM
	{
		sxval_t val = (sxval_t)malloc(sizeof(sxval));
		bzero((char*)val, sizeof(sxval));
		val->which = literal;
		if (strchr($1, '.')) {
			/* convert to float */
			val->v.l.fval= (float)atof($1);
			val->dtype = float_t;
		}
		else {
			/* convert to int */
			val->v.l.ival = atoi($1);
			val->dtype = int_t;
		}
		/* print_sxval(val); */
		$$ = val;
	}


update_set_list: update_statement
	{
	    $1->next = NULL;
	    $$ = $1;
	}
	| update_statement ',' update_set_list
	{
	    $1->next = $3;
	    $$ = $1;
	}
	
update_statement: '.' column_name K_EQ update_expression
	{
	    wqusl_t list = (wqusl_t)malloc(sizeof(wqusl));
	    list->left = $2;
	    list->right = $4;
	    $$ = list;
	}

update_expression: update_additive_expression
	
update_additive_expression: update_multiplicative_expression
	| update_additive_expression '+' update_multiplicative_expression
	{
	    wquexp_t expr = (wquexp_t)malloc(sizeof(wquexp));
	    expr->which = nonterm;
	    expr->w.e.left = $1;
	    expr->w.e.right = $3;
	    expr->w.e.numop = addition;
	    $$ = expr;
	}
	| update_additive_expression '-' update_multiplicative_expression
	{
	    wquexp_t expr = (wquexp_t)malloc(sizeof(wquexp));
	    expr->which = nonterm;
	    expr->w.e.left = $1;
	    expr->w.e.right = $3;
	    expr->w.e.numop = subtraction;
	    $$ = expr;
	}

update_multiplicative_expression: update_primary_expression
	| update_multiplicative_expression '*' update_primary_expression
	{
	    wquexp_t expr = (wquexp_t)malloc(sizeof(wquexp));
	    expr->which = nonterm;
	    expr->w.e.left = $1;
	    expr->w.e.right = $3;
	    expr->w.e.numop = multiplication;
	    $$ = expr;
	}
	| update_multiplicative_expression '/' update_primary_expression
	{
	    wquexp_t expr = (wquexp_t)malloc(sizeof(wquexp));
	    expr->which = nonterm;
	    expr->w.e.left = $1;
	    expr->w.e.right = $3;
	    expr->w.e.numop = division;
	    $$ = expr;
	}
	
update_primary_expression: vref
	{
	    wquexp_t expr = (wquexp_t)malloc(sizeof(wquexp));
	    expr->which = term;
	    expr->w.terminal = $1;
	    $$ = expr;
	}
	| '(' update_expression ')'
	{
	    $$ = $2;
	}



	
column_name: IDENTIFIER
	{
	    sxval_t terminal = (sxval_t)malloc(sizeof(sxval));
	    terminal->which = field;
	    terminal->v.f.column = (*name2num)[$1];
	    $$ = terminal;
	}

%%

#include "lex.dbi.c"

int
dbiwrap()
{
        return (1);
}

/*
 * Parse for the the desired command.  Read the characters from the
 * provided string.  Resets the lexer, so this function can be called
 * multiple times, perhaps with different input strings, during the
 * same program run.
 *
 * Returns a structure holding results in the res parameter.
 *
 * Returns NULL on success, !NULL if the parse failed.
 */
int wr_parse_dbi_command(struct parse_result *res, char *str, enum cmds cmd)
{
	void set_grammar(int start_token);
	dbi_strin = str;

	bzero(res, sizeof(struct parse_result));
	parse_results = res;
	switch(cmd) {
	case create_c:
		set_grammar(CREATE_CMD);
		break;
	case insert_c:
		set_grammar(INSERT_CMD);
		break;
	case select_c:
		set_grammar(SELECT_CMD);
		break;
	case delete_c:
		set_grammar(DELETE_CMD);
		break;
	case update_c:
		set_grammar(UPDATE_CMD);
		break;
        case drop_c:
                set_grammar(DROP_CMD);
                break;
	}
printf("parse(%s)\n", str);
	int err = dbiparse();
	/* yyrestart(NULL); */
	/* returns NULL on success */
	return(err);
}
