/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: source.h,v 1.7 2001/02/23 19:02:22 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: Object to hold generated C-code.
 * 
 */

#include "queue.h"

/* list of referenced wrapper names */
SLIST_HEAD(imp_head, imp_entry);
struct imp_entry {
	char *name;
	SLIST_ENTRY(imp_entry) entries;
};

enum operators_t { not, gt, lt, gte, lte, ne, eq};

struct srcrec;

class source {
private:
	struct srcrec *head, *tail, *cur;
public:
	source();
	~source();
	void append(char *txt);
	void append(char *txt, char *txt);
	void append(char *txt, char *txt2, char *txt3);
	void append(char *txt, char *txt2, char *txt3, char *txt4);
	void append(source *src);

	void gen_import_list(struct imp_head *list);
	void gen_decision_function(source *expr);
	void gen_debug_code();

	void rewind();
	int more_text();
	char *next();
};

source *gen_binary_comparison(char *one, enum operators_t op,
					char *two);
source * gen_arrow(source *expr, char *name, struct imp_head *imp_list);
source * gen_setuid(char *name);
source * gen_setgid(char *name);
source * gen_regex(char *str, char *regex);


/*
 * The following functions are intended to be exported from this library.
 */
