/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: debug.c,v 1.8 2001/02/23 19:02:23 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: TMP, test driver code.  Optionally included by activation
 *	criteria.
 */

#include <stdio.h>
#include <string.h>

/* simulated */
int sim_wr_wrapper_loaded(char *name)
{
	printf("\tsim_wr_wrapper_loaded(%s)\n", name);
	return(! NULL);
}

/* simulated */
int sim_wr_ref_wrapper(char *name)
{
	printf("\tsim_wr_ref_wrapper(%s)\n", name);
	return(! NULL);
}

/* simulated */
int sim_wr_deref_wrapper(char *name)
{
	printf("\tsim_wr_deref_wrapper(%s)\n", name);
	return(! NULL);
}

/* simulated */
int sim_wr_strcmp(const char *s1, const char *s2)
{
	return(strcmp(s1, s2));
}

/* simulated */
char *sim_wr_strdup(const char *str)
{
	return(strdup(str));
}

/* simulated */
void *sim_wr_malloc(size_t size)
{
	return((void*)malloc(size));
}

/* simulated */
int (*my_at_exec)(struct act_wrap *list, struct wr_proc *wrproc);
int sim_wr_add_criterion(char *str, void *df, int handle)
{
	printf("\tsim_wr_add_criterion(%s)\n", str);
	(void*)my_at_exec = (void*)df;
	return(! NULL);
}

/* simulated */
int sim_wr_rm_criterion(char *str)
{
	printf("\tsim_wr_rm_criterion(%s)\n", str);
	return(! NULL);
}

int (*my_at_unload)(void *uf);
/* simulated */
int sim_wr_at_unload(void *uf)
{
  	printf("\tsim_wr_at_unload()\n");
	(void*)my_at_unload = (void*)uf;
	return(! NULL);
}

/* simulated */
int sim_wr_rm_at_unload(void *uf)
{
	printf("\tsim_wr_rm_at_unload()\n");
	return(! NULL);
}

/* simulated */
int sim_wr_printf(const char *fm, ...)
{
	return(! NULL);
}

/* pointers to simulated functions */
struct wr_iface ifaces[] = {
    { "wr_wrapper_loaded", 1, (u_long)sim_wr_wrapper_loaded, NULL },
    { "wr_ref_wrapper", 1, (u_long)sim_wr_ref_wrapper, NULL },
    { "wr_deref_wrapper", 1, (u_long)sim_wr_deref_wrapper, NULL },
    { "wr_strcmp", 1, (u_long)sim_wr_strcmp, NULL },
    { "wr_strdup", 1, (u_long)sim_wr_strdup, NULL },
    { "wr_malloc", 1, (u_long)sim_wr_malloc, NULL },
    { "wr_add_criterion", 1, (u_long)sim_wr_add_criterion, NULL },
    { "wr_rm_criterion", 1, (u_long)sim_wr_rm_criterion, NULL },
    { "wr_at_unload", 1, (u_long)sim_wr_at_unload, NULL },
    { "wr_rm_at_unload", 1, (u_long)sim_wr_rm_at_unload, NULL },
    { "wr_printf", 1, (u_long)sim_wr_printf, NULL }
};
struct wr_export_table ftable = { ifaces, 11};

struct pcred newpcred = { NULL, 42, 42, 42, 42, 42 }; 
struct proc newproc; 

struct wr_proc wrproc = {  
	{ NULL, NULL },			
	42,				
	"/usr/local/bin/vi",		
	"/usr/local/obin/csh",	
	"vi",		                
	"/usr/home/badger",		
	NULL,				
	-1,				
	&newproc,			
	0,				
	(struct sysentvec*) NULL	
}; 

main() 
{ 
	struct act_wrap *list; 
	int i;
	newproc.p_cred = &newpcred; 
	newproc.p_cpticks = 42000;	
	newproc.p_cred->p_svuid = 80;	
	newproc.p_cred->p_rgid = 80;	
	newproc.p_cred->p_ruid = 0;	
	newproc.p_cred->p_rgid = 10;	
	newproc.p_nice = 20;	
	newproc.p_cred->p_svuid = 80;	
	newproc.p_pid = 42;	
	SLIST_INIT(&act_wrap);	
	list = &act_wrap;

	/* link table elements */
	for (i=0; i< ftable.count - 1; i++) {
		ftable.iface[i].next = &ftable.iface[i+1];
	}
	ftable.iface[ftable.count-1].next = NULL;

	printf("%s\n", myname); 
	printf("call init()\n");
	init(&ftable, 1);

	printf("call decision function\n");
	{
		struct act_wrap_entry *ep;

		(*my_at_exec)(list, &wrproc);
		ep = list->slh_first; 
		for (; ep != NULL; ep = ep->entries.sle_next) { 
			printf("	active(%s)\n", ep->name); 
		}
	}

	printf("call unload()\n");
	unload();
} 
