/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrapc.cc,v 1.28 2001/04/06 21:41:54 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: Source for the main routine of the wrapper compiler.
 * 
 *
 */

#define LINKAGE IMPORT

#include <wr_conf.h>

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

#ifdef WR_UNIX
#include <unistd.h>
#endif

#ifdef WR_FREEBSD
#include <osreldate.h>
#endif

#ifdef WR_WINNT
#include <io.h>
#include <getopt.h>
#endif

#ifdef WR_UNIX
#  ifdef WR_FREEBSD
#      define FREEBSDFLAGS "-DWR_IN_WRAPC=1 -DPD_IN_KERNEL=1 -x c -E"
#    if (__FreeBSD_version >= 500000)
#      define PREPROCESSFLAGS "-D_FREEBSD=5 " FREEBSDFLAGS
#    elif (__FreeBSD_version >= 400000)
#      define PREPROCESSFLAGS "-D_FREEBSD=4 " FREEBSDFLAGS
#    elif (__FreeBSD_version >= 300000)
#      define PREPROCESSFLAGS "-D_FREEBSD=3 " FREEBSDFLAGS
#    else
#      error "Unsupported version of FreeBSD"
#    endif
#  else
#    define PREPROCESSFLAGS "-DWR_IN_WRAPC=1 -DPD_IN_KERNEL=1 -x c -E"
#  endif /* WR_FREEBSD */
#endif
#ifdef WR_WINNT
#if (WR_WINNT >= 0x0500) 
#define PREPROCESSFLAGS "-nologo -DWR_IN_WRAPC=1 -DPD_IN_KERNEL=1 -DWSSLINKAGE=IMPORT -D_WIN32_WINNT=0x0500 -D_INC_EXCPT -D_UNICODE -DUNICODE /MT /TC /EP"
#elif (WR_WINNT >= 0x0400) 
#define PREPROCESSFLAGS "-nologo -DWR_IN_WRAPC=1 -DPD_IN_KERNEL=1 -DWSSLINKAGE=IMPORT -D_WIN32_WINNT=0x0400 -D_INC_EXCPT -D_UNICODE -DUNICODE /MT /TC /EP"
#endif
#endif

#ifdef WR_UNIX
#include "libwrinit.h"
#endif /* WR_UNIX */

extern "C" {
const char *pathfind( const char *path, const char *name, const char *mode );
}

int wr_parse_prototypes(FILE *fp);

int dflag = 0;			// debug flag
int typechk = 1; 		// perform variable type checking

char *outfilename = NULL; /* Leave as NULL for wrapc.out.c */

void usage() 

{
     fprintf(stderr, "wrapc [-I incdir] [-v] [-d dflag] [-t typechkflag] "
#ifdef WR_UNIX
                     "[-l] "
#endif /* WR_UNIX */
                     "wrspec-file\n");
     exit(1);
}

int
main(int argc, char **argv)
{
	int error = 0;
	FILE *fp;
	char buff[1024];
	char *out;
	char *suf;
        char *include_dir = strdup("");
        char *define_list= strdup("");
        char *old_incdir;
        char *old_deflist;
	const char *cc;
        extern char *optarg;
        extern int optind;
        int ch, len;
        pd_bool_t fPreprocess = FALSE;
        pd_bool_t fNoIncludes = FALSE;

        while ((ch = getopt(argc, argv, "lEd:t:vI:D:")) != EOF)
             switch(ch) {
             case 'd':
                     dflag = atoi(optarg);
                     break;
             case 't':
                     if (!strcmp(optarg, "NONE"))
                          typechk = 0;
                     else if (!strcmp(optarg, "ALL"))
                          typechk = 1;
                     break;
             case 'I':
                     old_incdir = include_dir;
                     len = strlen(old_incdir) + strlen(optarg)+strlen("-I ")
                         + 2;
                     include_dir = (char*)malloc(len);
                     snprintf(include_dir, len, "%s -I %s",old_incdir,optarg);
                     free(old_incdir);
                     break;
             case 'D':
                     old_deflist = define_list;
                     len = strlen(old_deflist) + strlen(optarg)+strlen("-I ")
                         + 2;
                     define_list = (char*)malloc(len);
                     snprintf(define_list, len, "%s -D %s",old_deflist,optarg);
                     free(old_deflist);
                     break;
             case 'E':
                     fPreprocess = TRUE;
                     break;
             case 'l':
                     fNoIncludes = TRUE;
                     break;

             case 'v':
                     fprintf(stderr, "NAI Labs Wrapper Compiler version 1.1\n");
                     usage();
		
             case '?':
             default:
                     usage();
        }
	argc -= optind;
	argv += optind;

	if (argc < 1) {
		usage();
                /* Calvin commented this out because we can 
                   invoked the preprocessor

                   error = wr_parse_prototypes(stdin);
		   exit(error);
                 */
	}

#ifdef WR_UNIX
	/* Read configuration information */
	if( !fNoIncludes )
	{
		(void) wr_init_read_conf();
		if( wss_wrap_inc_path && *wss_wrap_inc_path )
		{
			/* Append wrapper include directory */
			old_incdir = include_dir;
			len = strlen(old_incdir) + strlen(wss_wrap_inc_path) +
			      strlen("-I ") + 2;
			include_dir = (char*)malloc(len);
			snprintf(include_dir, len, "%s -I %s", old_incdir,
			         wss_wrap_inc_path);
			free(old_incdir);
		}
		if( wss_inc_path && *wss_inc_path )
		{
			/* Append include directory */
			old_incdir = include_dir;
			len = strlen(old_incdir) + strlen(wss_inc_path) +
			      strlen("-I ") + 2;
			include_dir = (char*)malloc(len);
			snprintf(include_dir, len, "%s -I %s", old_incdir,
			         wss_inc_path);
			free(old_incdir);
		}
	}
#endif /* WR_UNIX */

	if( ( ( cc = getenv( "CC" ) ) == NULL ) &&
	    ( ( cc = pathfind( getenv( "PATH" ), "gcc", "rx" ) ) == NULL ) &&
	    ( ( cc = pathfind( getenv( "PATH" ), "cc", "rx" ) ) == NULL ) &&
            ( ( cc = _CC ) == NULL ) )
	{
		fprintf( stderr, "ERROR unable to determine compiler\n" );
                free(include_dir);
                free(define_list);
		exit( ENOENT );
	}

	while (argc) {
		snprintf(buff, 1024, "%s %s %s %s %s", cc,
                        define_list, include_dir, PREPROCESSFLAGS, *argv);

		if (!(fp = popen(buff,"r"))) {
                        fprintf(stderr, "Couldn't open file %s\n",
                                *argv);
                        exit(1);
		}
                if (fPreprocess) {
                    size_t count;
                    char bigbuf[2048];
                    /* Dump it all to standard out and quit */
                    while ((count = read(fileno(fp),bigbuf,sizeof bigbuf)))
                        write(fileno(stdout),bigbuf,count);

                    exit(0);
                }
		out = strdup(*argv);
		suf = strrchr(out, '.');
		if (suf != NULL && (strcmp(suf, ".wr") == 0)) {
		   strcpy(suf, ".c");
                   outfilename = strdup(out);
		} 
		else {
		  fprintf(stderr, 
			  "Warning: File name %s does not end in .wr\n",
			  out);
		}

		free(out);
		error |= wr_parse_prototypes(fp);
		argv++, argc--;
		pclose(fp);
	}

        free(include_dir);
        free(define_list);
	return(error);
}
