/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: main.cc,v 1.12.2.2 2001/07/11 20:35:19 stevek Exp $
 *
 * Author: Lee Badger, Steve Kiernan
 *
 * Purpose: Program for testing and loading deactivation criteria.
 * 
 */

#define LINKAGE IMPORT

#include <stdio.h>

#include <libre.h>
#include <libcrit.h>
#include <libwrmod.h>

#include <string.h>
#include <stdlib.h>

#include <errno.h>

#ifndef WR_WINNT
#include <unistd.h>
#endif

#ifdef WR_WINNT
#include "getopt.h"
#endif

char *usage_str = "\
wra [-voh] [-c <filename>] 'deactivation crit'\n\
     with no flags, compile and load into  the WSS\n\
     -v don't invoke the kernel; just parse/validate\n\
     -c don't invoke the kernel; just generate & compile to <filename>\n\
     -h this help message";

void usage()
{
   fprintf(stderr, "%s\n", usage_str);
   exit(2);
}

int
main(int argc, char **argv)
{
   int error = 0;
   char *instr = NULL, *output = NULL;
   char c;
   char *argstr = "vohc:";
   int vflag = 0;
   Processor *proc;

   while ((c = getopt(argc, argv, argstr)) != -1)
   {
      switch(c)
      {
         case 'v':
            vflag++;
            break;

         case 'h':
            usage();
            break;

         case 'c':
            output = optarg;
            break;

         case '?':
            usage();
      }
   }
   argc -= optind;
   argv += optind;

   if (argc == 1)
      instr = argv[argc-1];
   else
      usage();

   wrmodSetup();

   error = critParse(instr, &proc);
   if( error != 0 )
   {
      if( vflag )
         printf("deactivation criteria NOT valid\n");
      else
         printf("%s\n", critParserError());
      return(error);
   }
   if( vflag )
      printf("deactivation criteria are valid\n");
   else
   {
      char *tmp_name;

      if( output )
         tmp_name = critMakeTempFile(proc, output, FALSE);
      else
      {
         tmp_name = critMakeTempFile(proc, NULL, FALSE);

         if( tmp_name )
            error = wrmodLoad(tmp_name);
         else
            error = -errno;

         if( error < 0 )
         {
            fprintf( stderr, "wra: %s\n", strerror( errno ) );
            free(tmp_name);
            return(errno);
         }
         printf( "Loaded deactivation criteria as module %d\n", error );
         unlink(tmp_name);
      }
   }
   return 0;
}

