/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrenable.c,v 1.12 2001/06/22 04:42:28 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: User-space program to enable/disable activation criteria
 */

#define LINKAGE IMPORT

#include <wr_conf.h>

#include <stdio.h>
#include <stdlib.h>

#include <libwrmod.h>
#include <libwrv.h>

void
usage()
{
   fprintf( stderr, "Usage: wrenable (gswtk|<id> <on/off>)\n" );
}

/*
 * Very tiny program with a simple set of arguments.  Turns on or off
 * the Criteria with the given id.  If you turn `on' a criteria that
 * is already on, it will return success and do nothing.
 */
int
main( int argc, char **argv )
{
   int id, val, error;
   
   if( ( argc == 2 ) && !strncmp( argv[1], "gswtk", 5 ) )
   {
      wrvAdminDone();
      return 0;
   }

   if( argc != 3 )
   {
      usage();
      return 1;
   }


   id = atoi(argv[1]);
   if( id <= 0 )
   {
      fprintf( stderr, "Bad id, must be > 0\n" );
      return 1;
   }
   
   if( strcmp( argv[2], "on" ) == 0 )
      val = 1;
   else if( strcmp(argv[2], "off" ) == 0 )
      val = 0;
   else
   {
      fprintf( stderr, "Bad value, must be `on' or `off'\n" );
      return 1;
   }

   /*
    * All the "real" work, call the WSS syscall.
    */
   error = wrmodEnable( id, val );
   if( error != 0 )
      perror( "wrenable" );

   return error;
}

