/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrload.c,v 1.21 2001/02/23 19:02:24 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: User-space program to load a wrapper module into the WSS.
 */

#define LINKAGE IMPORT

#include <wr_conf.h>

#include <stdio.h>

#include <errno.h>
#include <string.h>

#include <sys/stat.h>

#ifdef WR_LINUX
#include <unistd.h>
#endif
#ifdef WR_WINNT
#include "getopt.h"
#define stat(a,b) _stat(a,b)
typedef struct _stat stat_t;
#endif
#ifdef WR_UNIX
typedef struct stat stat_t;
#endif

#include <libwrmod.h>

/*
 * Print the usage information.
 */
void
usage()
{
   fprintf( stderr, "Usage: wrload [-h] <input file> [<input file> ...] \n" );
   exit( 1 );
}

/*
 * The main() routine calls a library to do all the work.
 */ 
int
main( int argc, char **argv )
{
   int c, error;
   char *modobj = NULL;
   
   while( ( c = getopt( argc, argv, "h" ) ) != EOF )
   {
      switch (c)
      {
         case 'h':
         case '?':
            usage();
            break;
         default:
            break;
      }
   }

   /*
    * skip the parameters
    */
   argc -= optind;
   argv += optind;

   if( argc < 1 ) usage();

   wrmodSetup();

   for( c = 0; c < argc; c++ )
   {
      pd_bool_t alloced = FALSE;

      modobj = argv[c];
      if( !_tcsrchr( modobj, PD_MAKESTR(FILESEP) ) )
      {
         stat_t st;
         int    step;

         for( step = 0; step < 2; step++ )
         {
            switch( step )
            {
               case 0:
               {
                  int size = ( _tcslen(argv[c]) + 1 ) * sizeof(TCHAR) +
                             sizeof(PD_MAKESTR(CONF_WRAPDIR));
                  modobj = (LPTSTR)malloc(size);
                  _sntprintf( modobj, size, PD_MAKESTR("%s%c%s"),
                              PD_MAKESTR(CONF_WRAPDIR), PD_MAKESTR(FILESEP),
                              argv[c] );
                  alloced = TRUE;
                  break;
               }

               default:
                  if( alloced ) free( modobj );
                  modobj = argv[c];
                  alloced = FALSE;
                  break;
            }

            if( !stat( modobj, &st ) ) break;
         }
      }
      error = wrmodLoad( modobj );
      if( error < 0 )
      {
         fprintf( stderr, "wrload: Error loading '%s': %s\n",
             modobj, strerror( errno ) );
         if( alloced )
            free( modobj );
         return( errno );
      }
      printf( "Loaded %s; registered with the WSS as module %d\n",
              modobj, error );
      if( alloced )
         free( modobj );
   }
   return( 0 );
}

