/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrstat.c,v 1.51 2001/02/23 19:02:24 stevek Exp $
 *
 * Authors: Chris Vance, Steve Kiernan, Linda Thomas
 *
 * Purpose: Program to return information on the status of the WSS.
 * 
 */

#define LINKAGE IMPORT

#include <wr_conf.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <wsi_types.h>

#include <time.h>
#ifndef WR_WINNT
#include <pwd.h>
#include <unistd.h>
#endif

#include <wss_db.h>
#include <libwrv.h>
#include <libwrstat.h>
#include <wrappers.h>

#ifdef WR_WINNT
#include "getopt.h"
#endif

#define DEFAULT_BUFSIZE 1024
#define PAD_SIZE 512
#define DEFAULT_ROWSIZE 16

#define FORMAT_STRING_BLOCK(str,size) (str)+=(strlen(str)>(size))?(size):(strlen(str))




/*
 * Prints information about wrappers
 */
void
DisplayWrappers()
{
   wrv_buffer *wrapinfo;
   size_t      i;

   wrapinfo = wrvGetWrappers();

   if( wrapinfo )
   {
      printf( "Id    Wrapper Name                                 References\n" );
      printf( "----  -------------------------------------------  ----------\n" );
      for( i = 0; i < wrvBufferGetRowCount( wrapinfo ); i++ )
      {
         printf( "%-6s%-45s%s\n", 
                 wrvBufferGetString( wrapinfo, i, 0 ), 
                 wrvBufferGetString( wrapinfo, i, 1 ),
                 wrvBufferGetString( wrapinfo, i, 2 ) );
      }
      wrvReleaseWrappers( wrapinfo );
   }
   else
   {
      perror( "wrstat" );
   }
}

/*
 * Prints information about libraries
 */
void
DisplayLibraries()
{
  wrv_buffer *libinfo;
  size_t      i;

  libinfo = wrvGetLibraries();
  if (libinfo) {
    printf("Id    Library Name                                 References\n");
    printf("----  -------------------------------------------  ----------\n");
    for (i = 0; i < wrvBufferGetRowCount (libinfo); i++) {
      printf ("%-6s%-45s%s\n", 
              wrvBufferGetString (libinfo, i, 0),
              wrvBufferGetString (libinfo, i, 1),
              wrvBufferGetString (libinfo, i, 2));
    }
    wrvReleaseLibraries( libinfo );
  }
  else
    perror ("wrstat");

}

/*
 * Prints information about activation criteria
 */
void
DisplayActCriteria()
{
  wrv_buffer  *crit;
  size_t i;

  crit = wrvGetActivationCriteria();

  if (crit != NULL) {
    printf("Id    Activation Criteria                          Status\n");
    printf("----  -------------------------------------------  ------\n");
    for (i = 0; i < wrvBufferGetRowCount (crit); i++) {
      printf( "%-6.6s%-45.45s%-8.8s\n",
              wrvBufferGetString (crit, i, 0),
              wrvBufferGetString (crit, i, 1),
              wrvBufferGetString (crit, i, 2));
        }
        wrvReleaseActivationCriteria( crit );
  }
  else
    perror ("wrstat");

}

/*
 * Prints information about deactivation criteria
 */
void
DisplayDeactCriteria()
{
  wrv_buffer  *crit;
  size_t i;
        
  crit = wrvGetDeactivationCriteria();
  if (crit != NULL) {
    printf("Id    Deactivation Criteria                        Status\n");
    printf("----  -------------------------------------------  ------\n");
    for (i = 0; i < wrvBufferGetRowCount (crit); i++) {
      printf( "%-6.6s%-45.45s%-8.8s\n",
              wrvBufferGetString (crit, i, 0),
              wrvBufferGetString (crit, i, 1),
              wrvBufferGetString (crit, i, 2));
    }
    wrvReleaseDeactivationCriteria( crit );
  }
  else
    perror ("wrstat");

}

/*
 * Prints information about processes
 */
void
DisplayProcesses()
{
  wrv_buffer  *procinfo;
  size_t i;

  procinfo = wrvGetProcesses();

  if (procinfo) {
    printf("PID    Program       UID    Path                CWD                 Wrappers\n");
    printf("-----  ------------  -----  ------------------  ------------------  ---------\n");
    for (i = 0; i < wrvBufferGetRowCount (procinfo); i++) {
        printf( "%-5s  %-12s  %-5s  %-18.18s  %-18.18s %s\n",
                wrvBufferGetString (procinfo, i, 0),
                wrvBufferGetString (procinfo, i, 1),
                wrvBufferGetString (procinfo, i, 2),
                wrvBufferGetString (procinfo, i, 3),
                wrvBufferGetString (procinfo, i, 4),
                wrvBufferGetString (procinfo, i, 5));
    }
    wrvReleaseProcesses( procinfo );
  }
  else
    perror ("wrstat");

}

/*
 * Prints information about modules
 */
void
DisplayModules()
{
  wrv_buffer *mods;
  size_t i;

  mods = wrvGetModules();

  if (mods != NULL) {
    printf("Id    Module                              Type        Loadaddr    Size\n");
    printf("----  ----------------------------------  ----------  ----------  --------\n");
    for( i = 0; i < wrvBufferGetRowCount(mods); i++ ) {
      printf( "%-4s  %-34.34s  %-10.10s  %-10.10s  %8.8lx\n",
              wrvBufferGetString(mods, i, 0), wrvBufferGetString(mods, i, 1),
              wrvBufferGetString(mods, i, 2), wrvBufferGetString(mods, i, 3),
              atol(wrvBufferGetString(mods, i, 4)));
    }
    wrvReleaseModules( mods );
  }
  else {
    perror("wrstat");
  }
}

/*
 * Prints more detailed information about a single wrapper;
 */
void
DisplayWrapperInfo( int id )
{

  time_t loadtime, curtime, timediff;
  int days, hours, minutes, seconds;
#ifndef WR_WINNT
  struct passwd *pw;
#endif /* !WR_WINNT */
  wrv_buffer  *mod;

  if (id == 0) {
    printf( "Wrapper not found!\n" );
    return;
  }
               
  mod = wrvGetWrapperByHandle( id );
  if( !mod ) {
    printf( "Wrapper not found!\n" );
    return;
  }

#ifndef WR_WINNT
  pw = getpwuid( atoi (wrvBufferGetString (mod, 0, 3)));
#endif /* !WR_WINNT */
  curtime = time (0);
  loadtime = atol (wrvBufferGetString (mod, 0, 4));
  timediff = curtime - loadtime;
  days = timediff / 86400;
  hours = ( timediff % 86400 ) / 3600;
  minutes = ( timediff % 3600 ) / 60;
  seconds = timediff % 60;

  printf ("%11s:  %s (id %s)\n", "Wrapper", 
          wrvBufferGetString (mod, 0, 1), 
          wrvBufferGetString (mod, 0, 0));
  printf ("%11s:  %s\n", "Loaded by", 
#ifndef WR_WINNT
          pw ? pw->pw_name :
#endif /* !WR_WINNT */
          wrvBufferGetString (mod, 0, 3));
  printf ("%11s:  %s", "Load time", ctime (&loadtime));
  printf ("%11s:  %d days, %d hours, %d minutes, %d seconds\n",
          "Loaded for", days, hours, minutes, seconds );
  printf ("%11s:  %s bytes\n", "Loaded size", 
          wrvBufferGetString (mod, 0, 5));

  wrvReleaseWrappers(mod);

}

/*
 * Prints more detailed information about a single library
 */
void
DisplayLibraryInfo( int id )
{
  time_t loadtime, curtime, timediff;
  int days, hours, minutes, seconds;
#ifndef WR_WINNT
  struct passwd *pw;
#endif /* !WR_WINNT */
  wrv_buffer  *lib;
  char  *sym;

  lib = wrvGetLibraryByHandle( id );
  if (!lib) {
    printf( "Library not found!\n" );
    return;
  }

#ifndef WR_WINNT
  pw = getpwuid( atoi( wrvBufferGetString (lib, 0, 3)));
#endif /* !WR_WINNT */
  curtime = time( 0 );
  loadtime = atol( wrvBufferGetString (lib, 0, 4));
  timediff = curtime - loadtime;
  days = timediff / 86400;
  hours = ( timediff % 86400 ) / 3600;
  minutes = ( timediff % 3600 ) / 60;
  seconds = timediff % 60;
  
  printf ("%11s:  %s (id %s)\n", "Library", 
          wrvBufferGetString (lib, 0, 1), wrvBufferGetString (lib, 0, 0));
  printf ("%11s:  %s\n", "Loaded by", 
#ifndef WR_WINNT
          pw ? pw->pw_name :
#endif /* !WR_WINNT */
     wrvBufferGetString (lib, 0, 3));
  printf ("%11s:  %s", "Load time", ctime( &loadtime ) );
  printf ("%11s:  %d days, %d hours, %d minutes, %d seconds\n",
          "Loaded for", days, hours, minutes, seconds );
  printf ("%11s:  %s bytes\n", "Loaded size", 
          wrvBufferGetString (lib, 0, 5));
  
  printf ("%11s:  ", "Functions" );
  
  sym = wrvBufferGetString (lib, 0, 6);
  
  if (sym) {
    while (*sym) {
      char *sname = sym;
      while (*sym && *sym != ',')
        sym++;
      *sym++ = '\0';
      if (sname == wrvBufferGetString (lib, 0, 6))
        printf( "%s\n", sname );
      else
        printf( "%11s:  %s\n", "", sname );
    }
  }
  else
    printf( "\n" );

  wrvReleaseLibraries( lib );

}

/*
 * Prints more detailed information about a single process
 */
void
DisplayProcessInfo( int pid )
{
  time_t loadtime, curtime, timediff;
  int days, hours, minutes, seconds;
#ifndef WR_WINNT
  struct passwd *pw;
#endif /* !WR_WINNT */
  wrv_buffer  *proc;
  char  *wrap;

  proc = wrvGetProcessByPID( pid );
  if (!proc) {
    printf( "Process not found!\n" );
    return;
  }

#ifndef WR_WINNT
  pw = getpwuid (atoi (wrvBufferGetString (proc, 0, 2)));
#endif /* !WR_WINNT */
  curtime = time( 0 );
  loadtime = atol (wrvBufferGetString (proc, 0, 3));
  timediff = curtime - loadtime;
  days = timediff / 86400;
  hours = ( timediff % 86400 ) / 3600;
  minutes = ( timediff % 3600 ) / 60;
  seconds = timediff % 60;

  printf ("%13s:  %s (pid %s)\n", "Program", 
          wrvBufferGetString (proc, 0, 1), wrvBufferGetString (proc, 0, 0));
  printf ("%13s:  %s\n", "User", 
#ifndef WR_WINNT
          pw ? pw->pw_name :
#endif /* !WR_WINNT */
     wrvBufferGetString (proc, 0, 2));
  printf ("%13s:  %s", "Started at", ctime( &loadtime ) );
  printf ("%13s:  %d days, %d hours, %d minutes, %d seconds\n",
          "Running for", days, hours, minutes, seconds );
  printf ("%13s:  %s\n", "Program path", 
          wrvBufferGetString (proc, 0, 4));
  printf ("%13s:  %s\n", "Working dir", 
          wrvBufferGetString (proc, 0, 5));
  printf ("%13s:  ", "Wrappers");
  
  wrap = wrvBufferGetString (proc, 0, 6);

  if (*wrap) {
      
    while (*wrap) {
      char *wname = wrap;
      while (*wrap && *wrap != ',' )
        wrap++;
      *wrap++ = '\0';
      if (wname == wrvBufferGetString (proc, 0, 6))
        printf( "%s\n", wname );
      else
        printf( "%13s:  %s\n", "", wname );
    }
  }
  else
    printf( "\n" );                

  wrvReleaseProcesses (proc);

}

/*
 * Prints more detailed information about a single activation criteria
 */
void
DisplayActCriteriaInfo( int id )
{
  time_t loadtime, curtime, timediff;
  int days, hours, minutes, seconds;
#ifndef WR_WINNT
  struct passwd *pw;
#endif /* !WR_WINNT */
  wrv_buffer  *crit;
                
  crit = wrvGetActivationCriteriaByHandle( id );

  if( !crit ) {
    printf( "Activation criteria not found!\n" );
    return;
  }

#ifndef WR_WINNT
  pw = getpwuid (atoi (wrvBufferGetString (crit, 0, 3)));
#endif /* !WR_WINNT */
  curtime = time( 0 );
  loadtime = atol (wrvBufferGetString (crit, 0, 4));
  timediff = curtime - loadtime;
  days = timediff / 86400;
  hours = ( timediff % 86400 ) / 3600;
  minutes = ( timediff % 3600 ) / 60;
  seconds = timediff % 60;
  
  printf ("%11s:  %s (id %s)\n", "Activation", 
          wrvBufferGetString (crit, 0, 1), wrvBufferGetString (crit, 0, 0));
  printf ("%11s:  %s\n", "Loaded by", 
#ifndef WR_WINNT
          pw ? pw->pw_name :
#endif /* !WR_WINNT */
     wrvBufferGetString (crit, 0, 3));
  printf ("%11s:  %s", "Load time", ctime( &loadtime ) );
  printf ("%11s:  %d days, %d hours, %d minutes, %d seconds\n",
          "Loaded for", days, hours, minutes, seconds );
  printf ("%11s:  %s bytes\n", "Loaded size", 
          wrvBufferGetString (crit, 0, 5));
  printf ("%11s:  %s \n", "Status",
          wrvBufferGetString (crit, 0, 2));
  
  wrvReleaseActivationCriteria( crit );

}


/*
 * Prints more detailed information about a single activation criteria
 */
void
DisplayDeactCriteriaInfo( int id )
{
  time_t loadtime, curtime, timediff;
  int days, hours, minutes, seconds;
#ifndef WR_WINNT
  struct passwd *pw;
#endif /* !WR_WINNT */
  wrv_buffer  *crit;
                
  crit = wrvGetDeactivationCriteriaByHandle( id );

  if( !crit ) {
    printf( "Deactivation criteria not found!\n" );
    return;
  }

#ifndef WR_WINNT
  pw = getpwuid (atoi (wrvBufferGetString (crit, 0, 3)));
#endif /* !WR_WINNT */
  curtime = time( 0 );
  loadtime = atol (wrvBufferGetString (crit, 0, 4));
  timediff = curtime - loadtime;
  days = timediff / 86400;
  hours = ( timediff % 86400 ) / 3600;
  minutes = ( timediff % 3600 ) / 60;
  seconds = timediff % 60;
  
  printf ("%11s:  %s (id %s)\n", "Deactivation", 
          wrvBufferGetString (crit, 0, 1), wrvBufferGetString (crit, 0, 0));
  printf ("%11s:  %s\n", "Loaded by", 
#ifndef WR_WINNT
          pw ? pw->pw_name :
#endif /* !WR_WINNT */
     wrvBufferGetString (crit, 0, 3));
  printf ("%11s:  %s", "Load time", ctime( &loadtime ) );
  printf ("%11s:  %d days, %d hours, %d minutes, %d seconds\n",
          "Loaded for", days, hours, minutes, seconds );
  printf ("%11s:  %s bytes\n", "Loaded size", 
          wrvBufferGetString (crit, 0, 5));
  printf ("%11s:  %s \n", "Status", 
          wrvBufferGetString (crit, 0, 2));
  
  wrvReleaseDeactivationCriteria( crit );

}




/*
 * Prints active wrappers and number of processes wrapped by each
 */
void
DisplayActiveWrappers()
{
  wrv_buffer   *wrap;
  size_t i;

  wrap = wrvGetActiveWrappers();
  if (!wrap)
    printf( "No active wrappers!\n" );
  else {
    printf( "Active Wrapper                      Wrapped Processes\n" );
    printf( "----------------------------------  -----------------\n" );
    for (i = 0; i < wrvBufferGetRowCount(wrap); i++ )
      printf( "%-34.34s  %s\n", 
              wrvBufferGetString (wrap, i, 0), 
              wrvBufferGetString (wrap, i, 1));
    wrvReleaseActiveWrappers( wrap );
  }

}

/*
 * Displays processes running under wrapper
 */
void
DisplayProcessesUnder( int id )
{
  wrv_buffer  *procinfo;
  size_t i;

  if (id == 0) {
    printf( "Wrapper not found!\n" );
    return;
  }

  procinfo = wrvGetWrappedProcesses( id );
  if (!procinfo) {
    printf( "No processes found wrapped by module %d!\n", id );
    return;
  }

  printf( "Processes wrapped by %d\n\n", id );
  printf("PID    Program       UID    Path                CWD                 Wrappers\n");
  printf("-----  ------------  -----  ------------------  ------------------  ---------\n");
  for( i = 0; i < wrvBufferGetRowCount (procinfo); i++ )
  {
    printf( "%-5s  %-12s  %-5s  %-18.18s  %-18.18s  %s\n",
            wrvBufferGetString (procinfo, i, 0),
            wrvBufferGetString (procinfo, i, 1),
            wrvBufferGetString (procinfo, i, 2),
            wrvBufferGetString (procinfo, i, 3),
            wrvBufferGetString (procinfo, i, 4),
            wrvBufferGetString (procinfo, i, 5) );
  }
  wrvReleaseWrappedProcesses (procinfo);

}


/*
 * Prints the usage information and exits the program with an error.
 */
void
usage()
{
   printf( "wrstat" );
   printf( "\t-P Show all process information\n" );
   printf( "\t-M Show all module information\n" );
   printf( "\t-A Show all loaded activation criteria information\n" );
   printf( "\t-D Show all loaded deactivation criteria information\n" );
   printf( "\t-W Show all loaded wrapper information\n" );
   printf( "\t-L Show all loaded library information\n" );
   printf( "\t-a <id> Show all information on activation criteria <id>\n" );
   printf( "\t-d <id> Show all information on deactivation criteria <id>\n" );
   printf( "\t-w <id> Show all information on wrapper <id>\n" );
   printf( "\t-l <id> Show all information on library <id>\n" );
   printf( "\t-p <pid> Show all information on process <pid>\n" );
   printf( "\t-t List active wrappers and number of processes wrapped\n" );
   printf( "\t-u <name> List processes running under wrapper <name>\n" );
   printf( "\t-h Print help (this message)\n" );
   exit( -1 );
}

/*
 * The main routine doesn't do much except parse the arguments and
 * call the correct display routine.
 */
int
main( int argc, char **argv )
{
   char c;
   int id;

   if( argc < 2 )
      usage();

   if( !wrvIsLoaded() )
   {
      fprintf( stderr, "wrstat: WSS not loaded\n" );
      exit( 1 );
   }

   while( ( c = getopt( argc, argv, "PMADWLhtl:p:a:d:w:u:s:" ) ) != EOF )
   {
      switch(c)
      {
         case 'P':
            DisplayProcesses();
            break;
         case 'M':
            DisplayModules();
            break;
         case 'a':
            id = atoi( optarg );
            if( id == 0 && *optarg != '0' )
               id = wrvGetModuleIdByName( optarg );
            DisplayActCriteriaInfo( id );
            break;
         case 'd':
            id = atoi( optarg );
            if( id == 0 && *optarg != '0' )
               id = wrvGetModuleIdByName( optarg );
            DisplayDeactCriteriaInfo( id );
            break;
         case 'w':
            id = atoi( optarg );
            if( id == 0 && *optarg != '0' )
               id = wrvGetModuleIdByName( optarg );
            DisplayWrapperInfo( id );
            break;
         case 'p':
            id = atoi( optarg );
            DisplayProcessInfo( id );
            break;
         case 'A':
            DisplayActCriteria();
            break;
         case 'D':
            DisplayDeactCriteria();
            break;
         case 'W':
            DisplayWrappers();
            break;
         case 'L':
            DisplayLibraries();
            break;
         case 'l':
            id = atoi(optarg);
            if( id == 0 && *optarg != '0' )
               id = wrvGetModuleIdByName( optarg );
            DisplayLibraryInfo( id );
            break;
         case 'h':
            usage();
            break;
         case 't':
            DisplayActiveWrappers();
            break;
         case 'u':
            id = atoi( optarg );
            if( id == 0 && *optarg != '0' )
               id = wrvGetModuleIdByName( optarg );
            DisplayProcessesUnder( id );
            break;
         case 's':
            while( *optarg )
            {
               switch( *optarg++ )
               {
                  case 'r':
                     wrstatRefs();
                     break;
                  case 'm':
                     wrstatMemory();
                     break;
                  case 'l':
                     wrstatLinkers();
                     break;
                  default:
                     printf("default???\n");
                     usage();
               }
            }
            break;
         case '?':
         default:
            printf("default???\n");
            usage();
      }
   }

   return( 0 );
}

