/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrunload.c,v 1.18 2001/02/23 19:02:24 stevek Exp $
 *
 * Author: Chris Vance
 *
 * Purpose: User-space program to unload a wrapper module from the WSS.
 */


#define LINKAGE IMPORT

#include <wr_conf.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>

#ifdef WR_LINUX
#include <unistd.h>
#endif
#ifdef WR_WINNT
#include "getopt.h"
#endif

#include <libwrmod.h>


/*
 * Print the usage information.
 */
void
usage()
{
   fprintf( stderr, "Usage: wrunload [-h] <module-id>\n" );
   exit( 1 );
}

/*
 * The main() routine does all the work.  It parses parameters and
 * calls the module unload syscall.
 */
int
main( int argc, char *argv[] )
{
   int c, id, error;
   int last, first;

   while( ( c = getopt( argc, argv, "h" ) ) != EOF )
   {
      switch( c )
      {
         case 'h':
         case '?':
            usage();
            break;
         default:
            break;
      }
   }

   /*
    * skip the parameters
    */
   argc -= optind;
   argv += optind;

   if( argc != 1 ) usage();

   wrmodSetup();

   if (strchr(argv[0], '-') == NULL) {
       last = first = atoi( argv[0] );
   } else {
       char *p = strchr(argv[0], '-');
       *p++ = '\0'; 
       first = atoi(argv[0]);
       last = atoi(p);
       fprintf(stderr, "wrunload %d - %d \n", first, last);
   }

   for (id = last; id >= first;  id--) {
        error = wrmodUnload( id );
        if( error < 0 ) {
           fprintf( stderr, "wrunload: Error unloading module %d: %s\n",
               id, strerror( errno ) );
           return( error );
        }
   } 
   return error;
}

