/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: AboutPanel.java,v 1.13 2001/04/05 18:41:20 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * This notbook page is used for displaying credits for wrappers
 *
 * It is provided as non-dockable placeholder in the notebook to
 * work around deficiencies in the JClass implementation.  If this
 * page was not provided and all other tabs were undocked, the
 * notebook draws the last-undocked tab, which is wrong.
 *
 */

import java.awt.*;
import jclass.bwt.*;
import jclass.util.*;
import com.gswtk.ui.*;

public class AboutPanel extends WRVPanel
{
	protected String[] logoPixels =
	{
		"                      BBBBBB              ",
	        "                BBBBBBccccccBBBB          ",
		"           BBBBBccccccccccBBccccB         ",
		"        BBBcccccBccBBBBccBcccccBcB        ",
		"      BBcccBBBBccBBbbYbbbbbBBBBcccBBw     ",
		"     BccBBB bBcBBbb..bYYYYYbbbbBBBccB     ",
		"    BcBB   bBcBbYYbbbbrrbbbYYbbbbbBBcB    ",
		"   BcB    bBcBbbbbYYYbbbbbYYbYYYYwbbYcB   ",
		"  BcB    YYBcBYbbbYbYbYYYYbbYbbbwYbYbBcB  ",
		"  BcB   bbBcB.bYbYbYbYb..bbbbYYwbbYbbBcB  ",
		" BcB   bbbBcBbYbYbYbYbbbbbbbbbwbYb..b BcB ",
		" BcB  wwwBcBwwwwwwwwwwwwwwwwwwbbYbbbb BcB ",
		"BcB   bbbBcBbYbYbYbYbbbbbb..bwbYbbbbY  BcB",
		"BcB   YYbBcBYbbYbYbYbbrrbbbbbwYbbbbYb  BcB",
		"BcB   bbBcBbbbbYbYbYbbbbbbbYYwbbrrYbb  BcB",
		"BcB   YYBcBYYYYbbYbbYYYYYYYbbwb.bYbbb  BcB",
		"BcB   bbBcBbbbbbbYbbbbbbbbbYbwbbbYbbb  BcB",
		"BcB   bbBcBbrrbbbbYYYYYYYbbYbwbbYYbb   BcB",
		" BcB  b.BcBbbbb..bbbbbbbbYbYbwbYbYb   BcB ",
		" BcB  bbBcBbbbbbbbbbbbbbbYbYbwbYbY    BcB ",
		"  BcB YYYBcBYYYYYYYYYYYbbYbYbwrYb    BcB  ",
		"  BcB bbbBcBbbbbbbbbbbbYbYbYbwbY     BcB  ",
		"   BcBYYbbBcBbbbrrbbbbbYbYbYbwb     BcB   ",
		"    BcbbYbBcBbbbbbbbbbbYbYbYbw    BBcB    ",
		"     BccBBBBcBB                BBBccB     ",
		"      BBcccBBccBBb        BBBBBcccBB      ",
		"        BBBcccBccBBBBBBBBBcccccBBB        ",
		"           BBBccccccccccccBBBBB           ",
		"              BBBBBBBBBBBB                ",
	};

	protected String[] credNames =
	{
		"Lee Badger", "Mark Feldman",
		"Kelly Djahandari", "Steve Kiernan",
		"Doug Kilpatrick", "Calvin Ko",
		"Larry Spector", "Linda Thomas",
		"Tim Fraser", "Karen Oostendorp", 
		"Ben Uecker", "Chris Vance"
	};

	// Constructors
	public AboutPanel( WRVPanelNotebook owner )
	{
		super( owner, "About Wrappers" );
		setTabColor( Color.black );
		setTabForeground( Color.white );

		// Add title to about page
		JCImageCreator im = new JCImageCreator( this, logoPixels[0].length(), logoPixels.length );
		im.setColor( 'b', Color.green.darker() );
		im.setColor( 'w', Color.green );
		im.setColor( 'c', Color.cyan );
		im.setColor( 'B', Color.blue );
		im.setColor( 'Y', Color.yellow );
		im.setColor( 'r', Color.red );
		im.setColor( '.', Color.black );
		Image logoImage = im.create( logoPixels );

		JCString s = new JCString();
		s.add( logoImage );
		s.add( JCString.HORIZ_SPACE.intValue(), new Integer( 10 ) );
		s.add( new Font( "TimesRoman", Font.ITALIC, 24 ) );
		s.add( Color.white );
		s.add( "WSS Wrapper Viewer" );
		JCLabel label = new JCLabel( s );
		label.setBackground( Color.black );

		WRVContainer lc = new WRVContainer();
		lc.setLayout( new GridLayout( 1, 1 ) );
		lc.setInsets( new Insets( 1, 1, 1, 1 ) );
		lc.add( label );

		Panel panel = new Panel();
		panel.add( lc );

		// Add credits to about page
		Panel names = new Panel();
		names.setLayout( new GridLayout( (credNames.length + 1) / 2, 2 ) );
		for( int i = 0; i < credNames.length; i++ )
		{
			Label l = new Label( credNames[i] );
			l.setFont( new Font( "Helvetica", Font.PLAIN, 14 ) );
			names.add( l );
		}

		Label tl = new Label( "Wrappers Development Team" );
		tl.setFont( new Font( "Helvetica", Font.BOLD, 18 ) );
		tl.setForeground( Color.yellow );

		Panel holder = new Panel();
		holder.setLayout( new BorderLayout() );
		holder.add( "North", tl );
		holder.add( "Center", names );

		WRVContainer credcon = new WRVContainer();
		credcon.setInsets( new Insets( 2, 2, 2, 2 ) );
		credcon.setPreferredSize( 680, 260 );
		credcon.add( holder );

		// Add title and credits to container
		WRVContainer container = new WRVContainer( WRVContainer.DRAW_SUNKEN );
		container.setLayout( new BorderLayout() );
		container.setPreferredSize( 700, 300 );
		container.setInsets( new Insets( 2, 2, 2, 2 ) );
		container.add( "North", panel );
		container.add( "Center", credcon );

		// Add container to page
		JCContainer page = (JCContainer)getPage();
		page.setLayout( new BorderLayout() );
		page.setBackground( Color.lightGray );
		page.setInsets( new Insets( 1, 1, 2, 2 ) );
		page.add( container, "Center" );
	}
}

