/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: CriteriaBox.java,v 1.6 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.text.*;
import jclass.bwt.*;
import jclass.util.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class CriteriaBox extends Frame
{
	protected JCButton	cancelButton = new JCButton( "Cancel" );
	protected JCButton	okButton     = new JCButton( "Ok" );
	protected TextField	textField;
	protected MainFrame	mainFrame;
	protected CheckboxGroup	checks	     = new CheckboxGroup();

	// Constructors
	public CriteriaBox( MainFrame mf )
	{
		super( "Add Criteria" );
		mainFrame = mf;

		setLayout( new GridLayout( 1, 1 ) );
		setBackground( Color.lightGray );

		JCContainer drawArea = new JCContainer();
		drawArea.setLayout( new JCGridLayout( 1, 2 ) );
		drawArea.add( new Label( "Criteria:" ) );
		textField = new TextField( 40 );
		textField.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { compileActivation(); }
		} );
		drawArea.add( textField );

		JCContainer checkArea = new JCContainer();
		checkArea.add( new Checkbox( "Activation", checks, true ) );
		checkArea.add( new Checkbox( "Deactivation", checks, false ) );

		JCContainer buttonArea = new JCContainer();
		okButton.addActionListener( new JCActionListener() {
			public void actionPerformed( JCActionEvent e ) { compileActivation(); }
		} );
		cancelButton.addActionListener( new JCActionListener() {
			public void actionPerformed( JCActionEvent e ) { dispose(); }
		} );
		buttonArea.add( okButton );
		buttonArea.add( cancelButton );

		WRVContainer container = new WRVContainer();
		container.setLayout( new JCGridLayout( 3, 1 ) );
		container.setInsets( new Insets( 2, 2, 2, 2 ) );
		container.setBackground( Color.lightGray );
		container.add( drawArea );
		container.add( checkArea );
		container.add( buttonArea );

		add( container );
		pack();

		setResizable( false );
	}

	// Public methods
	public void compileActivation()
	{
		if( textField.getText().length() > 0 )
		{
			String act = textField.getText();
			WrapperViewer wv = WrapperViewer.getGlobal();

			try
			{
				if( checks.getSelectedCheckbox().getLabel().equals("Activation") )
					wv.addActivationCriteria( act );
				else
					wv.addDeactivationCriteria( act );
				dispose();
			}
			catch( ParseException ex )
			{
				final Frame frame = new Frame( "Parse Error" );
				frame.addWindowListener( new WindowAdapter() {
					public void windowClosed( WindowEvent we ) { frame.setVisible( false ); }
				} );
				frame.setBackground( Color.lightGray );
				frame.setLayout( new BorderLayout() );
				String[] pstring = JCStringTokenizer.parse( ex.getMessage(), '\n' );
				String[] error = JCStringTokenizer.parse( pstring[2], '\'' );
				int index = pstring[0].indexOf( error[3] );
				JCString slabel = new JCString();
				slabel.add( new Font( "Courier", Font.PLAIN, 12 ) );
				slabel.add( Color.blue );
				if( index < 0 )
					slabel.add( pstring[0] );
				else
				{
					if( index > 0 ) slabel.add( pstring[0].substring( 0, index ) );
					slabel.add( Color.red );
					slabel.add( new Font( "Courier", Font.BOLD, 12 ) );
					slabel.add( pstring[0].substring( index, index + error[3].length() ) );
					if( index + error[3].length() < pstring[0].length() )
					{
						slabel.add( Color.blue );
						slabel.add( new Font( "Courier", Font.PLAIN, 12 ) );
						slabel.add( pstring[0].substring( index + error[3].length() ) );
					}
					slabel.add( JCString.NEWLINE );
					slabel.add( new Font( "Courier", Font.PLAIN, 12 ) );
					for( int c = 0; c < index; c++ )
						slabel.add(" ");
					slabel.add( Color.black );
					slabel.add( new Font( "Courier", Font.BOLD, 10 ) );
					slabel.add( "^" );
				}
				JCLabel l = new JCLabel( slabel );
				l.setAlignment( BWTEnum.MIDDLELEFT );
				WRVContainer sc = new WRVContainer( WRVContainer.DRAW_SUNKEN );
				sc.setLayout( new GridLayout( 1, 1 ) );
				sc.setInsets( new Insets( 1, 1, 1, 1 ) );
				sc.add( l );
				JCContainer c = new JCContainer();
				c.setLayout( new GridLayout( 1, 1 ) );
				c.setInsets( new Insets( 1, 3, 1, 3 ) );
				c.add( sc );
				l = new JCLabel( pstring[2] );

				JCButton closeButton = new JCButton( "Ok" );
				closeButton.addActionListener( new JCActionListener() {
					public void actionPerformed( JCActionEvent ae ) { frame.setVisible( false ); }
				} );
				closeButton.setForeground( Color.black );
				frame.add( "North", c );
				frame.add( "Center", l );
				frame.add( "South", closeButton );
				frame.pack();

				Point loc = getLocationOnScreen();
				Rectangle bounds = this.getBounds();
				Rectangle fbounds = frame.getBounds();
				frame.setLocation( loc.x + ( bounds.width - fbounds.width ) / 2,
						   loc.y + ( bounds.height - fbounds.height ) / 2 );
				frame.setResizable( false );
				frame.show();
			}
		}
		else
		{
			dispose();
		}
	}
}

