/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: CriteriaViewer.java,v 1.6 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jclass.bwt.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class CriteriaViewer extends Frame
{
	public static final int ACTIVATION = 1;
	public static final int DEACTIVATION = 2;

	protected String title = "";

	protected String[] dotw = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
	protected String[] months = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
				      "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

	// Constructors
	public CriteriaViewer( int type, int id )
	{
		super();

		String[] info;
		
		if( type == ACTIVATION )
			info = WrapperViewer.getGlobal().requestActivationCriteriaInfo( id );
		else
			info = WrapperViewer.getGlobal().requestDeactivationCriteriaInfo( id );

		if( info != null )
		{
			if( type == ACTIVATION )
				title = "Activation";
			else
				title = "Deactivation";

			title += " Criteria Information: " + info[1];
			setTitle( title );
		}

		if( info != null )
		{
			WRVContainer c = new WRVContainer( WRVContainer.DRAW_SUNKEN );
			c.setLayout( new GridLayout( 1, 1 ) );
			c.setInsets( new Insets( 1, 1, 1, 1 ) );

			Panel cpanel = new Panel();
			cpanel.setLayout( new JCAlignerLayout() );
			cpanel.setBackground( Color.white );
			JCLabel l = new JCLabel( "Id:" );
			l.setForeground( Color.blue );
			cpanel.add( l );
			cpanel.add( new Label( info[0] ) );
			l = new JCLabel( "Name:" );
			l.setForeground( Color.blue );
			cpanel.add( l );
			cpanel.add( new Label( info[1] ) );
			l = new JCLabel( "Status:" );
			l.setForeground( Color.blue );
			cpanel.add( l );
			cpanel.add( new Label( info[2] ) );
			l = new JCLabel( "User Id:" );
			l.setForeground( Color.blue );
			cpanel.add( l );
			cpanel.add( new Label( info[3] ) );
			l = new JCLabel( "Load Time:" );
			l.setForeground( Color.blue );
			cpanel.add( l );

			long loaded = Integer.valueOf( info[4] ).intValue();
			loaded *= 1000;
		        String[] ids = TimeZone.getAvailableIDs( -5 * 60 * 60 * 1000 );
		        SimpleTimeZone edt = new SimpleTimeZone( -5 * 60 * 60 * 1000, ids[0] );
        		edt.setStartRule( Calendar.APRIL, 1, Calendar.SUNDAY, 2 * 60 * 60 * 1000 );
		        edt.setEndRule( Calendar.OCTOBER, -1, Calendar.SUNDAY, 2 * 60 * 60 * 1000 );
		        Calendar calendar = new GregorianCalendar( edt );
		        Date d = new Date( loaded );
		        calendar.setTime( d );
			String s = dotw[calendar.get( Calendar.DAY_OF_WEEK )] + " " +
				   months[calendar.get( Calendar.MONTH )] + " " +
				   calendar.get( Calendar.DATE ) + " " +
				   calendar.get( Calendar.HOUR_OF_DAY ) + ":" +
				   calendar.get( Calendar.MINUTE ) + ":" +
				   calendar.get( Calendar.SECOND ) + " " +
				   (edt.inDaylightTime( d )?"EDT":"EST") + " " +
				   calendar.get( Calendar.YEAR );
			cpanel.add( new Label( s ) );
			l = new JCLabel( "Size:" );
			l.setForeground( Color.blue );
			cpanel.add( l );
			cpanel.add( new Label( info[5] ) );

			c.add( cpanel );

			Panel buttons = new Panel();
			JCButton closeButton = new JCButton( "Close" );
			closeButton.addActionListener( new JCActionListener() {
				public void actionPerformed( JCActionEvent e ) { setVisible( false ); }
			} );
			buttons.add( closeButton );

			setLayout( new JCGridLayout( 2, 1 ) );
			add( c );
			add( buttons );

			pack();

			setResizable( false );
			addWindowListener( new WindowAdapter() {
				public void windowClosed( WindowEvent e ) { setVisible( false ); }
			} );
		}
	}

	// Public methods
	public void setVisible( boolean visible )
	{
		if( visible == true && title.length() == 0 )
			throw new RuntimeException( "Unable to get information for selected module" );

		super.setVisible( visible );
	}
}

