/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: DatabasePanel.java,v 1.15 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * This notebook page is used for browsing WSS database tables
 *
 * The contents are layed out in a split window with a tree view of
 * available tables on the left and a grid of the currently-
 * browsing table on the right.
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jclass.bwt.*;
import jclass.util.*;
import com.nailabs.ui.*;
import com.gswtk.db.*;
import com.gswtk.wrapper.event.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class DatabasePanel extends WRVPanel implements TableEventListener
{
	protected JCOutliner	       outliner		 = null;
	protected JCOutlinerFolderNode localDbFolder	 = null;
	protected JCOutlinerFolderNode globalDbFolder	 = null;
	protected JCOutlinerFolderNode wssGlobalDbFolder = null;
	protected Grid		       grid		 = null;
	protected Table		       browsing		 = null;
	protected JCSplitterWindow     splitter		 = null;
	protected JCLabel	       barLabel		 = null;
	protected Image		       wrapImage	 = null;
	protected Image		       wrapClosedImage	 = null;
	protected Image		       tableImage	 = null;
	protected Image		       tableClosedImage	 = null;
	protected Menu		       tableMenu	 = null;
	protected MenuItem	       tableNewRow	 = null;
	protected MenuItem	       tableDeleteRow	 = null;

	private   String[]	       rootPixels	 =
	{
		"bbbbbbbbbbbbbb    ",
		"bwwwwwwwwwwwwb    ",
		"bwwwwwwwwwwwwbbb  ",
		"bwwwwwwwwwwwwbgb  ",
		"bwwwwwwwwwwwwbgbbb",
		"bwwwwwwwwwwwwbgbgb",
		"bwwwwwwwwwwwwbgbgb",
		"bwwwwwwwwwwwwbgbgb",
		"bwwwwwwwwwwwwbgbgb",
		"bwwwwwwwwwwwwbgbgb",
		"bwwwwwwwwwwwwbgbgb",
		"bbbbbbbbbbbbbbgbgb",
		"  bggggggggggggbgb",
		"  bbbbbbbbbbbbbbgb",
		"    bggggggggggggb",
		"    bbbbbbbbbbbbbb"
	};

	private   String[]	       wrapPixels	 =
	{
		" bbb          bbb ",
		"bYYYb        bYYYb",
		"bYYYb   bb   bYYYb",
		" bYYYb bYYb bYYYb ",
		" bYYYb bYYb bYYYb ",
		"  bYYYbYYYYbYYYb  ",
		"  bYYYYYYYYYYYYb  ",
		"   bYYYYbbYYYYb   ",
		"   bYYYb  bYYYb   ",
		"    bbb    bbb    "
	};

	private   String[]	       tablePixels	 =
	{
		"bbbbbbbbbbbbbbbbb",
		"bgggggggbgggggggb",
		"bgggggggbgggggggb",
		"bbbbbbbbbbbbbbbbb",
		"bwwwwwwwwwwwwwwwb",
		"bwBBBBBwwBBBBBBwb",
		"bwwwwwwwwwwwwwwwb",
		"bwBBBBwwwBBwBwBwb",
		"bwwwwwwwwwwwwwwwb",
		"bwBBwwBwwBwBBBwwb",
		"bwwwwwwwwwwwwwwwb",
		"bwBBwBBwwBBwBBBwb",
		"bwwwwwwwwwwwwwwwb",
		"bbbbbbbbbbbbbbbbb"
	};

	// Constructors
	public DatabasePanel( WRVPanelNotebook notebook, MenuBar menubar )
	{
		super( notebook, "WSS Databases", true );
		setMenuBar( menubar );
		setTabColor( Color.red );
                 
		tableMenu = new Menu( "Table" );
		tableMenu.setFont( menubar.getFont() );

		tableNewRow = new MenuItem( "New Row", new MenuShortcut( 'N' ) );
		tableNewRow.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { newRow(); }
		} );
		tableNewRow.setFont( menubar.getFont() );
		tableNewRow.setEnabled( false );
		tableMenu.add( tableNewRow );

		tableDeleteRow = new MenuItem( "Delete Row", new MenuShortcut( 'D' ) );
		tableDeleteRow.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { deleteSelectedRow(); }
		} );
		tableDeleteRow.setFont( menubar.getFont() );
		tableDeleteRow.setEnabled( false );
		tableMenu.add( tableDeleteRow );

		// Construct tree view for databases
		JCOutlinerFolderNode root = new JCOutlinerFolderNode( null, "Database Tables" );
		JCImageCreator im = new JCImageCreator( this, rootPixels[0].length(), rootPixels.length );
		im.setColor( 'w', Color.white );
		im.setColor( 'b', Color.black );
		im.setColor( 'g', Color.lightGray );
		Image rootImage = im.create( rootPixels );
		JCOutlinerNodeStyle style = new JCOutlinerNodeStyle();
		style.setFolderOpenIcon( rootImage );
		style.setFolderClosedIcon( rootImage );
		style.setShortcut( true );
		root.setStyle( style );

		localDbFolder = new JCOutlinerFolderNode( "Instance", root );
		globalDbFolder = new JCOutlinerFolderNode( "Wrapper", root );
		wssGlobalDbFolder = new JCOutlinerFolderNode( "WSS Global", root );

		im = new JCImageCreator( this, wrapPixels[0].length(), wrapPixels.length );
		im.setColor( 'Y', Color.yellow );
		im.setColor( 'b', Color.black );
		im.setColor( 'g', Color.lightGray );
		wrapClosedImage = im.create( wrapPixels );
		im.setColor( 'Y', Color.cyan );
		wrapImage = im.create( wrapPixels );

		im = new JCImageCreator( this, tablePixels[0].length(), tablePixels.length );
		im.setColor( 'b', Color.black );
		im.setColor( 'g', Color.lightGray );
		im.setColor( 'B', Color.blue );
		im.setColor( 'w', Color.white );
		tableImage = im.create( tablePixels );
		im.setColor( 'B', Color.white );
		tableClosedImage = im.create( tablePixels );

		outliner = new JCOutliner( root );
		outliner.getOutliner().setBackground( Color.white );
		outliner.setPreferredSize( 200, 250 );
        	outliner.getDefaultNodeStyle().setShortcut( true );

		outliner.addItemListener( new JCItemListener()
		{
			public void itemStateChanged( JCItemEvent e )
			{
				if( e.getStateChange() == JCItemEvent.SELECTED )
				{
					JCOutlinerNode n = outliner.getSelectedNode();

					if( n.getUserData() != null )
					{
						DatabaseCache dbc = WrapperViewer.getGlobal().getDatabaseCache();
						Table t = dbc.getTable( ((Integer)n.getUserData()).intValue() );
						browseTable( t );
						tableNewRow.setEnabled( true );
					}
				}
			}
		} );

		// Set up grid
		Panel gridPanel = new Panel();
		gridPanel.setLayout( new BorderLayout() );

		grid = new Grid( new String[] { "No Table Selected" } );
		grid.setSelectEdit( false );
		grid.setDefaultColumnWidth( 200 );
		grid.showMargin( false );
		grid.addItemListener( new ItemListener()
		{
			public void itemStateChanged( ItemEvent e )
			{
				if( e.getStateChange() == ItemEvent.SELECTED )
					tableDeleteRow.setEnabled( true );
				else
					tableDeleteRow.setEnabled( false );
			}
		} );
		grid.addGridListener( new GridAdapter()
		{
			public void cellChanged( GridEvent e )
			{
				int row = e.getRow();
				int col = e.getColumn();

				String rowHandle = grid.getCell( grid.getTotalColumns(), row );
				String cellData = grid.getCell( col, row );

				Row cellRow = browsing.get( Integer.valueOf( rowHandle ).intValue() );
				Column column = browsing.getColumnDefinition().columnAt( col - 1 );
				
				WrapperViewer.getGlobal().updateTableRowColumnData( browsing, cellRow, column, cellData );
			}
		} );
		gridPanel.add( grid, "Center" );

		// Add splitter between tree view and notebook
		splitter = new JCSplitterWindow( BWTEnum.HORIZONTAL );
		splitter.setPreferredSize( 700, 275 );
		splitter.add( outliner );
		splitter.add( gridPanel );

		Panel page = getPage();
		page.setLayout( new BorderLayout() );
		page.setBackground( Color.lightGray );
		page.add( splitter, "Center" );

		addComponentListener( new ComponentAdapter()
		{
			public void componentHidden( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.remove( tableMenu );
			}

			public void componentShown( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.add( tableMenu );

				if( browsing != null )
				{
					tableNewRow.setEnabled( true );
					if( grid.getSelectedCell() != null )
						tableDeleteRow.setEnabled( true );
				}
			}
		} );

		// Set up label in button bar
		barLabel = new JCLabel();
		barLabel.setFont( new Font( "Helvetica", Font.BOLD, 12 ) );
		barLabel.setAlignment( BWTEnum.MIDDLELEFT );
		JCContainer bar = (JCContainer)getButtonBar();
		bar.setLayout( new GridLayout( 1, 1 ) );
		bar.setBackground( Color.lightGray );
		bar.setPreferredSize( 660, 20 );
		bar.add( barLabel );
		bar.setVisible( true );

		// Add us as EventFS table event listener
		DatabaseCache dbCache = WrapperViewer.getGlobal().getDatabaseCache();
		dbCache.addEventListener( this );

		// Add tables to tree view
		for( Enumeration e = dbCache.getTables(); e.hasMoreElements(); )
		{
			Table t = (Table)e.nextElement();
			addTable( t );
		}
	}

	// Public methods
	public void addTable( Table t )
	{
		JCOutlinerNode n;
		JCVector v;

		// Get the children for the local/global table node
		if( t.isLocal() )
			v = localDbFolder.getChildren();
		else if( t.getWrapperName().length() > 0 )
			v = globalDbFolder.getChildren();
		else
			v = wssGlobalDbFolder.getChildren();

		JCOutlinerFolderNode wrapNode = null;
		
		// Find the wrapper node
		if( ( t.getWrapperName().length() > 0 ) &&
		    ( v != null ) )
		{
			for( Enumeration enum = v.elements(); enum.hasMoreElements(); )
			{
				JCOutlinerFolderNode child = (JCOutlinerFolderNode)enum.nextElement();
				if( child.getLabel().equals( t.getWrapperName() ) )
				{
					wrapNode = child;
					break;
				}
			}
		}

		if( t.isLocal() )
		{
			// If we could not find the wrapper node, create one
			if( wrapNode == null )
			{
				wrapNode = new JCOutlinerFolderNode( t.getWrapperName(), localDbFolder );
				JCOutlinerNodeStyle style = new JCOutlinerNodeStyle();
				style.setFolderOpenIcon( wrapImage );
				style.setFolderClosedIcon( wrapClosedImage );
				style.setShortcut( true );
				wrapNode.setStyle( style );
			}

			v = wrapNode.getChildren();
			JCOutlinerFolderNode tableNode = null;

			// Find the table node
			if( v != null )
			{
				for( Enumeration enum = v.elements(); enum.hasMoreElements(); )
				{
					JCOutlinerFolderNode child = (JCOutlinerFolderNode)enum.nextElement();
					if( child.getLabel().equals( t.getTableName() ) )
					{
						tableNode = child;
						break;
					}
				}
			}

			// If we could not find the table node, create one
			if( tableNode == null )
			{
				tableNode = new JCOutlinerFolderNode( t.getTableName(), wrapNode );
				JCOutlinerNodeStyle style = new JCOutlinerNodeStyle();
				style.setFolderOpenIcon( tableImage );
				style.setFolderClosedIcon( tableClosedImage );
				style.setShortcut( true );
				tableNode.setStyle( style );
			}

			// Create the process id (PID) node
			n = new JCOutlinerNode( String.valueOf( t.getPID() ), tableNode );
		}
		else
		{
			// If we could not find the wrapper node, create one
			if( t.getWrapperName().length() > 0 )
			{
				if( wrapNode == null )
				{
					wrapNode = new JCOutlinerFolderNode( t.getWrapperName(), globalDbFolder );
					JCOutlinerNodeStyle style = new JCOutlinerNodeStyle();
					style.setFolderOpenIcon( wrapImage );
					style.setFolderClosedIcon( wrapClosedImage );
					style.setShortcut( true );
					wrapNode.setStyle( style );
				}

				// Create the table node
				n = new JCOutlinerNode( t.getTableName(), wrapNode );
			}
			else
			{
				// Create the table node
				n = new JCOutlinerNode( t.getTableName(), wssGlobalDbFolder );
			}
		}

		// Set the table handle on the node
		n.setUserData( new Integer( t.getHandle() ) );
	}

	public void browseTable( Table t )
	{
		// Request table if it has not been loaded
		if( t.isLoaded() == false )
			WrapperViewer.getGlobal().requestTable( t );

		// If the data is newer than last viewing or we were browsing another table,
		//    clear the grid and refresh
		if( t.lastViewed() == null ||
		    t.lastViewed().getTime() < t.lastUpdated().getTime() ||
		    browsing != t )
		{
			grid.removeAll();

			if( browsing != t )
			{
				// Set up the column headings
				ColumnDefinition cd = t.getColumnDefinition();
				String[] headings = new String[cd.size() + 1];
				int cols = 0;
				for( Enumeration e = cd.columns(); e.hasMoreElements(); cols++ )
				{
					Column c = (Column)e.nextElement();
					headings[cols] = c.getName();
				}
				headings[cols] = "Handle";
				grid.setHeadings( headings );

				Rectangle rect = grid.getBounds();
				int[] widths = new int[headings.length];
				for( int i = 0; i < headings.length - 1; i++ )
					widths[i] = ( rect.width - 20 ) / (headings.length - 1);
				widths[headings.length-1] = 0;
				grid.setColumnWidths( widths );
			}

			// Fill the items in the grid
			for( Enumeration e = t.rows(); e.hasMoreElements(); )
			{
				String items[] = new String[t.getColumnDefinition().size() + 1];
				Row r = (Row)e.nextElement();
				int i = 0;

				for( Enumeration enum = r.values(); enum.hasMoreElements(); i++ )
					items[i] = ((Column)enum.nextElement()).getData().toString();
				items[i] = Integer.toString( r.getHandle() );
				grid.addRow( items );
			}

			// Keep track of the table we're browsing so we won't
			//    have to go through extra work if the user browses
			//    the same table
			browsing = t;
		}

		// Set status message in button bar
		JCString s = new JCString();
		s.add( "Browsing '" );
		s.add( Color.blue );
		s.add( t.getName() );
		s.add( Color.black );
		s.add( "'" );
		barLabel.setLabel( s );
	}

	public void deleteSelectedRow()
	{
		if( browsing == null ) return;

		Point    p         = grid.getSelectedCellPosition();
		String[] rowValues = grid.getRow( p.y );
		Row      row       = browsing.get( Integer.valueOf( rowValues[rowValues.length-1] ).intValue() );

		WrapperViewer.getGlobal().delTableRow( browsing, row );
	}

	public void newRow()
	{
		if( browsing == null ) return;

		WrapperViewer.getGlobal().addTableRow( browsing );
	}

	public void removeTable( Table t )
	{
		JCVector v;

		// Get the children for the local/global table node
		if( t.isLocal() )
			v = localDbFolder.getChildren();
		else
			v = globalDbFolder.getChildren();

		// Find the wrapper node (node with the wrapper's name)
		JCOutlinerFolderNode wrapNode = null;
		if( v != null )
		{
			for( Enumeration enum = v.elements(); enum.hasMoreElements(); )
			{
				JCOutlinerFolderNode child = (JCOutlinerFolderNode)enum.nextElement();
				if( child.getLabel().equals( t.getWrapperName() ) )
				{
					wrapNode = child;
					break;
				}
			}
		}

		// Once we've found the wrapper node, we need to find the table node
		if( wrapNode != null )
		{
			v = wrapNode.getChildren();

			// Find the table node (node with the table's name)
			JCOutlinerNode tableNode = null;
			for( Enumeration enum = v.elements(); enum.hasMoreElements(); )
			{
				JCOutlinerNode child = (JCOutlinerNode)enum.nextElement();
				if( child.getLabel().equals( t.getTableName() ) )
				{
					tableNode = child;
					break;
				}
			}

			// Once we've found the table node, we remove it if it's a global table.
			// If the table is local, we need to find the process id (PID) node.
			if( tableNode != null )
			{
				if( t.isLocal() )
				{
					// Find the PID node and remove it
					v = tableNode.getChildren();

					for( Enumeration enum = v.elements(); enum.hasMoreElements(); )
					{
						JCOutlinerNode child = (JCOutlinerNode)enum.nextElement();
						if( child.getLabel().equals( String.valueOf( t.getPID() ) ) )
						{
							// Remove the PID node
							((JCOutlinerFolderNode)tableNode).removeChild( child );

							// If there are no more children for the table node,
							//   remove it
							if( ((JCOutlinerFolderNode)tableNode).getChildren().size() <= 0 )
							{
								wrapNode.removeChild( tableNode );

								// If there are no more children for the wrapper
								//    node, remove it
								if( wrapNode.getChildren().size() <= 0 )
									localDbFolder.removeChild( wrapNode );
							}
						}
					}
				}
				else
				{
					// Remove the table node
					wrapNode.removeChild( tableNode );

					// If there are no more children for the wrapper node, remove it
					if( wrapNode.getChildren().size() <= 0 )
						globalDbFolder.removeChild( wrapNode );
				}
			}
		}
	}

	public void tableEventOccurred( TableEvent e )
	{
		Table t = (Table)e.getNote( "table" );

		// Add table to tree view
		if( e instanceof CreateTableEvent )
		{
			addTable( t );

			outliner.repaint();
		}
		// Remove table from tree view
		else if( e instanceof DropTableEvent )
		{
			removeTable( t );

			if( browsing == t )
			{
				grid.setHeadings( new String[] { "No Table Selected" } );
				barLabel.setLabel( "" );
				tableNewRow.setEnabled( false );
				tableDeleteRow.setEnabled( false );
			}

			outliner.repaint();
		}
		// Add row to browsing table if same as event table
		else if( e instanceof InsertRowEvent )
		{
			if( browsing == t )
			{
				Row r = t.get( ((UpdateTableEvent)e).getRowID() );
				if( r != null )
				{
					String[] items = new String[r.size() + 1];
					int i = 0;
					for( Enumeration enum = r.values(); enum.hasMoreElements(); i++ )
						items[i] = ((Column)enum.nextElement()).getData().toString();
					items[i] = String.valueOf( r.getHandle() );
					grid.addRow( items );
				}
			}
		}
		// Update row in browsing table if same as event table
		else if( e instanceof UpdateRowEvent )
		{
			if( browsing == t )
			{
				Row r = t.get( ((UpdateTableEvent)e).getRowID() );
				if( r != null )
				{
					for( int i = 1; i <= grid.getTotalRows(); i++ )
					{
						String[] items = grid.getRow( i );
						if( items[items.length-1].equals( Integer.toString( r.getHandle() ) ) )
						{
							int j = 0;
							for( Enumeration enum = r.values(); enum.hasMoreElements(); j++ )
								items[j] = ((Column)enum.nextElement()).getData().toString();
							items[j] = Integer.toString( r.getHandle() );
							grid.setRow( i, items );
							return;
						}
					}
				}
			}
		}
		// Remove row from browsing table if same as event table
		else if( e instanceof DeleteRowEvent )
		{
			if( browsing == t )
			{
				String handle = Integer.toString( ((UpdateTableEvent)e).getRowID() );
				for( int i = 1; i <= grid.getTotalRows(); i++ )
				{
					String[] items = grid.getRow( i );
					if( items[items.length-1].equals( handle ) )
					{
						grid.deleteRow( i );
						return;
					}
				}
			}
		}
	}
}

