/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: DeactivationPanel.java,v 1.13 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import jclass.bwt.*;
import jclass.util.*;
import com.eventfs.*;
import com.nailabs.ui.*;
import com.gswtk.wrapper.event.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class DeactivationPanel extends WRVPanel implements ModuleEventListener
{
	public final static String[] columnNames  =
	{
		"ID", "Criteria", "Status"
	};

	public final static int[]    columnSizes  =
	{
		70, 300, 100
	};

	protected Grid		     grid;
	protected Menu		     criteriaMenu;
	protected ModuleCache        modCache;


	// Constructors
	public DeactivationPanel( WRVPanelNotebook notebook, MenuBar menubar, ModuleCache cache )
	{
		super( notebook, "Deactivation Critera", true );
		setMenuBar( menubar );
		setTabColor( new Color( 0, 192, 178 ) );
		modCache = cache;

		criteriaMenu = new Menu( "Deactivation" );
		criteriaMenu.setFont( menubar.getFont() );

		final MenuItem criteriaUnload = new MenuItem( "Unload", new MenuShortcut( 'U' ) );
		criteriaUnload.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { unloadSelected(); }
		} );
		criteriaUnload.setFont( menubar.getFont() );
		criteriaUnload.setEnabled( false );

		final MenuItem criteriaInfo = new MenuItem( "View Information", new MenuShortcut( 'I' ) );
		criteriaInfo.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { viewSelected(); }
		} );
		criteriaInfo.setFont( menubar.getFont() );
		criteriaInfo.setEnabled( false );

		criteriaMenu.add( criteriaUnload );
		criteriaMenu.add( criteriaInfo );

		grid = new Grid( columnNames );
		grid.setDefaultColumnWidth( 100 );
		grid.setColumnWidths( columnSizes );
		grid.showMargin( false );
		grid.setEditable( false );
		grid.addItemListener( new ItemListener()
		{
			public void itemStateChanged( ItemEvent e )
			{
				if( e.getStateChange() == ItemEvent.SELECTED )
				{
					criteriaUnload.setEnabled( true );
					criteriaInfo.setEnabled( true );
				}
				else
				{
					criteriaUnload.setEnabled( false );
					criteriaInfo.setEnabled( false );
				}
			}
		} );
		grid.addGridListener( new GridAdapter()
		{
			public void rowDoubleClicked( GridEvent e )
			{
				viewSelected();
			}
		} );

		Panel page = getPage();
		page.setLayout( new BorderLayout() );
		page.setBackground( Color.lightGray );
		page.add( grid, "Center" );

		addComponentListener( new ComponentAdapter()
		{
			public void componentHidden( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.remove( criteriaMenu );
			}

			public void componentShown( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.add( criteriaMenu );

				if( grid.getSelectedRow() != null )
				{
					criteriaUnload.setEnabled( true );
					criteriaInfo.setEnabled( true );
				}
			}
		} );

		modCache.addModuleEventListener( ModuleCache.CACHE_DEACTIVATION,
		                                 this );

		updateList();
	}

	// Protected methods
	protected void updateList()
	{
		Enumeration e = modCache.getModules( ModuleCache.CACHE_DEACTIVATION );

		if( e != null )
		{
			grid.removeAll();

			while( e.hasMoreElements() )
				addRow( (DeactivationCriteria)e.nextElement() );
		}
	}

	// Public methods
	public void finalize()
	{
		modCache.removeModuleEventListener( ModuleCache.CACHE_DEACTIVATION, this );
	}

	public void addRow( DeactivationCriteria dc )
	{
		String[] values = new String[3];

		values[0] = Integer.toString( dc.getHandle() );
		values[1] = dc.getName();
		values[2] = dc.getStatus();

		grid.addRow( values );
	}

	public void addRow( int id )
	{
		DeactivationCriteria dc =
		   (DeactivationCriteria)modCache.get( new Integer(id) );

		addRow( dc );
	}

	public void removeRow( int id )
	{
		String strID = Integer.toString(id);

		for( int i = 1; i <= grid.getTotalRows(); i++ )
		{
			String[] items = grid.getRow( i );

			if( items[0].equals( strID ) )
			{
				grid.deleteRow( i );
				break;
			}
		}
	}

	public void moduleEventOccurred( ModuleEvent e )
	{
		DeactivationCriteriaEvent dce = (DeactivationCriteriaEvent)e;
		int id = dce.getModuleID();

		if( e instanceof ModuleLoadedEvent )
			addRow( id );
		else if( e instanceof ModuleUnloadedEvent )
			removeRow( id );
	}

	public void viewSelected()
	{
		String[] item = grid.getSelectedRow();

		if( item != null )
		{
			int modId = Integer.valueOf( item[0] ).intValue();
			CriteriaViewer cv = new CriteriaViewer( CriteriaViewer.ACTIVATION, modId );
			try { cv.setVisible( true ); } catch( RuntimeException re ) {}
		}
	}

	public void unloadSelected()
	{
		String[] item = grid.getSelectedRow();

		if( item != null )
		try
		{
			WrapperViewer.getGlobal().unloadModule(
				Integer.valueOf( item[0] ).intValue() );
		}
		catch( IOException ioe )
		{}
	}
}

