/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: ProcessesPanel.java,v 1.11 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import jclass.bwt.*;
import jclass.util.*;
import com.nailabs.ui.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class ProcessesPanel extends WRVPanel
{
	public final static String[] columnNames =
	{
		"PID", "Program", "UID", "Path", "CWD", "Wrappers"
	};

	public final static int[]    columnSizes =
	{
		70, 130, 70, 130, 130, 130
	};

	protected Grid		     grid	 = null;

	// Constructors
	public ProcessesPanel( WRVPanelNotebook notebook )
	{
		super( notebook, "Process Information", true );
		setTabColor( Color.green );

		grid = new Grid( columnNames );
		grid.setDefaultColumnWidth( 100 );
		grid.setColumnWidths( columnSizes );
		grid.showMargin( false );
		grid.setEditable( false );

		Panel page = getPage();
		page.setLayout( new BorderLayout() );
		page.setBackground( Color.lightGray );
		page.add( grid, "Center" );

		JCButton updateButton = new JCButton( "Update" );
		updateButton.addActionListener( new JCActionListener()
		{
			public void actionPerformed( JCActionEvent e ) { updateList(); }
		} );

		Panel bar = getButtonBar();
		bar.setLayout( new GridLayout( 1, 1 ) );
		bar.add( updateButton );
		bar.setVisible( true );

		updateList();
	}

	// Protected methods
	protected void updateList()
	{
		String[] processes = WrapperViewer.getGlobal().requestProcesses();

		if( processes != null )
		{
			grid.removeAll();

			for( int i = 0; i < processes.length; i++ )
			{
				Vector vals = new Vector();
				StringTokenizer st = new StringTokenizer( processes[i], "|" );

				for( int j = 0; st.hasMoreTokens() && j < 6; j++ )
					vals.addElement( st.nextToken() );

				String[] values = new String[6];
				for( int j = 0; j < vals.size(); j++ )
					values[j] = (String)vals.elementAt( j );
				if( ( vals.size() > 0 ) && processes[i].endsWith( "|" ) )
					values[5] = "<not wrapped>";
				grid.addRow( values );
				if( ( vals.size() > 0 ) && processes[i].endsWith( "|" ) )
					grid.setCellColor( Grid.TEXT_COLOR, Color.red, grid.getTotalRows(), 6 );
			}
		}
	}
}

