/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: WrappersPanel.java,v 1.17 2001/04/05 18:41:21 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import jclass.bwt.*;
import jclass.util.*;
import com.nailabs.ui.*;
import com.gswtk.wrapper.event.*;
import com.gswtk.ui.*;
import com.gswtk.viewer.*;

public class WrappersPanel extends WRVPanel implements ModuleEventListener
{
	public final static String[] columnNames =
	{
		"ID", "Wrapper", "Reference Count"
	};

	public final static int[]    columnSizes =
	{
		70, 245, 140
	};

	protected Grid		     grid;
	protected Menu		     wrapperMenu;
	protected ModuleCache        modCache;

	// Constructors
	public WrappersPanel( WRVPanelNotebook notebook, MenuBar menubar, ModuleCache cache )
	{
		super( notebook, "Loaded Wrappers", true );
		setMenuBar( menubar );
		setTabColor( Color.cyan );
		modCache = cache;

		wrapperMenu = new Menu( "Wrapper" );
		wrapperMenu.setFont( menubar.getFont() );

		final MenuItem wrapperUnload = new MenuItem( "Unload", new MenuShortcut( 'U' ) );
		wrapperUnload.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { unloadSelected(); }
		} );
		wrapperUnload.setFont( menubar.getFont() );
		wrapperUnload.setEnabled( false );
		wrapperMenu.add( wrapperUnload );

		grid = new Grid( columnNames );
		grid.setDefaultColumnWidth( 100 );
		grid.setColumnWidths( columnSizes );
		grid.showMargin( false );
		grid.setEditable( false );
		grid.addItemListener( new ItemListener()
		{
			public void itemStateChanged( ItemEvent e )
			{
				if( e.getStateChange() == ItemEvent.SELECTED )
					wrapperUnload.setEnabled( true );
				else
					wrapperUnload.setEnabled( false );
			}
		} );

		Panel page = getPage();
		page.setLayout( new BorderLayout() );
		page.setBackground( Color.lightGray );
		page.add( grid, "Center" );

		addComponentListener( new ComponentAdapter()
		{
			public void componentHidden( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.remove( wrapperMenu );
			}

			public void componentShown( ComponentEvent e )
			{
				MenuBar mbar = getMenuBar();
				mbar.add( wrapperMenu );

				if( grid.getSelectedRow() != null )
					wrapperUnload.setEnabled( true );
			}
		} );

		modCache.addModuleEventListener( ModuleCache.CACHE_WRAPPER,
		                                 this );

		updateList();
	}


	// Protected methods
	protected void updateList()
	{
		Enumeration e = modCache.getModules( ModuleCache.CACHE_WRAPPER );

		if( e != null )
		{
			grid.removeAll();

			while( e.hasMoreElements() )
				addRow( (Wrapper)e.nextElement() );
		}
	}


	// Public methods
	public void finalize()
	{
		modCache.removeModuleEventListener( ModuleCache.CACHE_WRAPPER, this );
	}

	public void unloadSelected()
	{
		String[] item = grid.getSelectedRow();

		if( item != null )
		{
			int modId = Integer.valueOf( item[0] ).intValue();
			try { WrapperViewer.getGlobal().unloadModule( modId ); } catch( IOException ioe ) {}
		}
	}

	public void addRow( Wrapper wrap )
	{
		String[] values = new String[3];

		values[0] = Integer.toString( wrap.getHandle() );
		values[1] = wrap.getName();
		values[2] = Integer.toString( wrap.getRefCount() );

		grid.addRow( values );
	}

	public void addRow( int id )
	{
		Wrapper wrap = (Wrapper)modCache.get( new Integer(id) );

		addRow( wrap );
	}

	public void removeRow( int id )
	{
		String strID = Integer.toString(id);

		for( int i = 1; i <= grid.getTotalRows(); i++ )
		{
			String[] items = grid.getRow( i );

			if( items[0].equals( strID ) )
			{
				grid.deleteRow( i );
				break;
			}
		}
	}

	public void moduleEventOccurred( ModuleEvent e )
	{
		WrapperEvent we = (WrapperEvent)e;
		int id = we.getModuleID();

		if( e instanceof ModuleLoadedEvent )
			addRow( id );
		else if( e instanceof ModuleUnloadedEvent )
			removeRow( id );
	}
}

