/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 *
 * $Id: Column.java,v 1.5 2001/04/05 18:41:23 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.db;

import java.lang.*;

public class Column implements Cloneable
{
	public static final int TYPE_STRING = 0;
	public static final int TYPE_INT    = 1;
	public static final int TYPE_FLOAT  = 2;

	protected String name;
	protected Object data;
	protected int    maxlen;

	// Constructors
	public Column( Column col )
	{
		name = col.getName();
		data = col.getData();
		maxlen = col.getMaxLength();
	}

	public Column( String newname, Object newdata ) { this( newname, newdata, 0 ); }

	public Column( String newname, Object newdata, int newmaxlen )
	{
		name = newname.intern();
		data = newdata;
		maxlen = newmaxlen;
	}

	// Public methods
	public synchronized Object clone()
	{
		return new Column( this );
	}

	public Object getData() { return data; }
	public synchronized void setData( Object obj ) { data = obj; }
	public int getMaxLength() { return maxlen; }
	public synchronized void setMaxLength( int newmaxlen ) { maxlen = newmaxlen; }
	public String getName() { return name; }
	public int getType() throws Exception
	{
		if( data instanceof String )
			return TYPE_STRING;
		else if( data instanceof Integer )
			return TYPE_INT;
		else if( data instanceof Float )
			return TYPE_FLOAT;
		else
			throw new UnknownError();
	}
}

