/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: ColumnDefinition.java,v 1.5 2001/04/05 18:41:23 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.db;

import java.lang.*;
import java.util.*;

public class ColumnDefinition implements Cloneable
{
	protected Hashtable colHash = new Hashtable();
	protected Vector columns = new Vector();

	// Constructors
	public ColumnDefinition() { super(); }
	public ColumnDefinition( ColumnDefinition cd )
	{
		for( int i = 0; i < cd.size(); i++ )
		{
			Column col = cd.columnAt(i);
			Object newcol = col.clone();

			addColumn( (Column)newcol );
		}
	}

	// Public methods
	public final synchronized void addColumn( Column col )
	{
		if( contains( col ) == false )
		{
			colHash.put( col.getName(), col );
			columns.addElement( col );
		}
	}

	public final synchronized void addColumn( String name, int type, int maxlen ) throws Exception
	{
		Object obj;

		switch( type )
		{
			case Column.TYPE_STRING:
				obj = new String();
				break;
			case Column.TYPE_INT:
				obj = new Integer(0);
				break;
			case Column.TYPE_FLOAT:
				obj = new Float(0f);
				break;
			default:
				throw new IllegalArgumentException();
		}

		addColumn( new Column( name, obj, maxlen ) );
	}

	public final int capacity() { return columns.capacity(); }
	public synchronized Object clone() { return new ColumnDefinition( this ); }
	public final boolean contains( Column col ) { return columns.contains( col ); }
	public final synchronized void copyInto( Column cols[] )
	{
		columns.copyInto( cols );

		colHash.clear();
		for( int i = 0; i < size(); i++ )
		{
			Column col = columnAt( i );
			colHash.put( col.getName(), col );
		}
	}

	public final synchronized Column columnAt( int index ) { return (Column)columns.elementAt( index ); }
	public final synchronized Enumeration columns() { return columns.elements(); }
	public final synchronized void ensureCapacity( int size ) { columns.ensureCapacity( size ); }
	public final synchronized Column firstElement() { return (Column)columns.firstElement(); }
	public final synchronized Column get( String name ) { return (Column)colHash.get( name ); }
	public final int indexOf( Column col ) { return columns.indexOf( col ); }
	public final synchronized int indexOf( Column col, int index ) { return columns.indexOf( col, index ); }
	public final synchronized void insertColumnAt( Column col, int index ) throws Exception
	{
		if( contains( col ) )
			throw new ColumnAlreadyExistsException();
		columns.insertElementAt( col, index );
		colHash.put( col.getName(), col );
	}

	public final boolean isEmpty() { return columns.isEmpty(); }
	public final synchronized Column lastColumn() { return (Column)columns.lastElement(); }
	public final int lastIndexOf( Column col ) { return columns.lastIndexOf( col ); }
	public final synchronized int lastIndexOf( Column col, int index ) { return columns.lastIndexOf( col, index ); }
	public final synchronized void put( Column col ) { addColumn( col ); }
	public final synchronized void removeAllColumns()
	{
		colHash.clear();
		columns.removeAllElements();
	}

	public final synchronized boolean removeColumn( Column col )
	{
		colHash.remove( col.getName() );
		return columns.removeElement( col );
	}

	public final synchronized void removeColumnAt( int index )
	{
		Column col = columnAt( index );
		colHash.remove( col.getName() );
		columns.removeElementAt( index );
	}

	public final synchronized void setColumnAt( Column col, int index ) throws Exception
	{
		Column oldcol = columnAt( index );
		if( oldcol != null )
			colHash.remove( oldcol.getName() );

		if( contains( col ) )
			throw new ColumnAlreadyExistsException();
		columns.setElementAt( col, index );
		colHash.put( col.getName(), col );
	}

	public final synchronized void setSize( int size ) { columns.setSize( size ); }
	public final int size() { return columns.size(); }
	public final synchronized void trimToSize() { columns.trimToSize(); }
}

