/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: Row.java,v 1.7 2001/04/05 18:41:23 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.db;

import java.lang.*;
import java.util.*;

public class Row
{
	protected ColumnDefinition cd;
	protected ColumnDefinition rowdata = new ColumnDefinition();
	protected int              id;

	// Constructors
	public Row( ColumnDefinition refcd, int newid )
	{
		cd = refcd;
		id = newid;
	}

	public Row( int newid )
	{
		// This constructor is provided to construct a
		// hash value for the Table class to search
		// for rows by id (which is their hash code)
		cd = rowdata;
		id = newid;
	}

	// Private methods
	private synchronized void set( String name, Object data )
	{
		Column col = get( name );
		if( col == null )
			throw new IllegalArgumentException();

		col.setData( data );
	}

	protected synchronized void add( Column col, Object data )
	{
		if( ( col == null ) ||
		    ( col.getData().getClass().isInstance( data ) == false ) )
			throw new IllegalArgumentException();

		rowdata.addColumn( new Column( col.getName(), data, col.getMaxLength() ) );
	}


	// Public methods
	public synchronized void add( String name, String value ) { add( name, (Object)value ); }
	public synchronized void add( String name, int value ) { add( name, new Integer( value ) ); }
	public synchronized void add( String name, float value ) { add( name, new Float( value ) ); }
	public synchronized void add( String name, Object data ) { add( cd.get( name ), data ); }
	public synchronized void add( int index, String value ) { add( index, (Object)value ); }
	public synchronized void add( int index, int value ) { add( index, new Integer(value) ); }
	public synchronized void add( int index, float value ) { add( index, new Float(value) ); }
	public synchronized void add( int index, Object data ) { add( cd.columnAt( index ), data ); }

	public boolean equals( Object obj )
	{
		if( obj instanceof Row )
			return( ((Row)obj).hashCode() == this.hashCode() );
		else
			return false;
	}

	public synchronized Column get( String name ) { return rowdata.get( name ); }
	public synchronized Column getAt( int index ) { return rowdata.columnAt( index ); }
	public int getHandle() { return this.id; }
	public int hashCode() { return this.id; }
	public void set( String name, String value ) { set( name, (Object)value ); }
	public void set( String name, int value ) { set( name, new Integer( value ) ); }
	public void set( String name, float value ) { set( name, new Float( value ) ); }
	public synchronized int size() { return rowdata.size(); }
	public synchronized Enumeration values() { return rowdata.columns(); }
	public String toString()
	{
		String s = "Row #" + this.id + "(" + this.size() + ") [ ";
		for( int i = 0; i < this.size(); i++ )
		{
			Column c = (Column)this.getAt( i );

			if( i > 0 )
				s += ", ";
			s += c.getData().toString();
		}
		s += " ]";

		return s;
	}
}

