/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: Table.java,v 1.15 2001/04/05 18:41:23 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.db;

import java.lang.*;
import java.util.*;
import com.gswtk.viewer.*;

public class Table
{
	protected String	   name;
	protected int		   handle;
	protected boolean	   localTable  = false;
	protected Calendar	   updated     = null;
	protected Calendar	   viewed      = null;
	protected ColumnDefinition cd	       = null;
	protected Hashtable	   rowHash     = new Hashtable();
	protected Vector           rows	       = new Vector();
	protected String	   wrapperName = "";
	protected String	   tableName   = "";
	protected int		   pid	       = -1;

	// Constructors
	public Table( String newname, int newhandle, ColumnDefinition newcd )
	{
		name = newname.intern();
		handle = newhandle;

		if( newcd != null )
			cd = (ColumnDefinition)newcd.clone();

		if( name.length() > 0 )
		{
			// Determine if this is a local table
			// * We look for <string>_<number>_<string> formatted name
			String parse = new String( name );
			int index = parse.indexOf( "." );
			if( index > 0 ) wrapperName = parse.substring( 0, index );

			int nextIndex = parse.substring( index + 1 ).indexOf( "." );
			if( nextIndex >= 0 )
			{
				try
				{
					Integer i = Integer.valueOf( parse.substring( index + 1,
										      index + 1 + nextIndex ) );
					localTable = true;
					pid = i.intValue();
					tableName = parse.substring( index + 1 + nextIndex + 1 );
					return;
				}
				catch( NumberFormatException nfe )
				{ }
			}
			tableName = parse.substring( index + 1 );
		}
	}

	public Table( String newname, int newhandle ) { this( newname, newhandle, null ); }

	// Public methods
	public boolean equals( Object obj )
	{
		if( obj instanceof Table )
			return( ((Table)obj).hashCode() == this.hashCode() );
		else
			return false;
	}

	public synchronized Row get( int rowhandle )
	{
		viewed = Calendar.getInstance();
		viewed.setTime( new Date() );

		return (Row)rowHash.get( new Row( rowhandle ) );
	}

	public synchronized ColumnDefinition getColumnDefinition()
	{
		return (cd != null)?(ColumnDefinition)cd.clone():null;
	}
	public int getHandle() { return this.handle; }
	public String getName() { return this.name; }
	public String getTableName() { return this.tableName; }
	public String getWrapperName() { return this.wrapperName; }
	public int getPID() { return this.pid; }
	public int hashCode() { return getHandle(); }
	public boolean isLoaded() { return( lastUpdated() != null ); }
	public boolean isLocal() { return this.localTable; };
	public Date lastUpdated() { return( (updated!=null)?updated.getTime():null ); }
	public Date lastViewed() { return( (viewed!=null)?viewed.getTime():null ); }
	public synchronized void put( Row row )
	{
		Row oldrow = (Row)rowHash.get( row );

		if( oldrow != null )
			rows.removeElement( oldrow );
		rows.insertElementAt( row, 0 );
		rowHash.put( row, row );

		updated = Calendar.getInstance();
		updated.setTime( new Date() );
	}

	public synchronized void remove( Row row )
	{
		rowHash.remove( row );
		rows.removeElement( row );
	}

	public synchronized Enumeration rows()
	{
		viewed = Calendar.getInstance();
		viewed.setTime( new Date() );

		return rows.elements();
	}

	public synchronized void setColumnDefinition( ColumnDefinition newcd ) throws IllegalArgumentException
	{
		if( cd == null )
			cd = newcd;
		else
			throw new IllegalArgumentException();

		updated = Calendar.getInstance();
		updated.setTime( new Date() );
	}

	public String toString() { return "Table: '" + name + "', Handle = " + handle; }
}

