/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: WRVContainer.java,v 1.7 2001/04/05 18:41:24 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.ui;

import java.awt.*;
import jclass.bwt.JCContainer;

public class WRVContainer extends JCContainer
{
	public static final int DRAW_RAISED = 0;
	public static final int DRAW_SUNKEN = 1;

	protected int	drawStyle = DRAW_RAISED;
	protected Color highlightColor = Color.white;
	protected Color shadowColor = Color.darkGray.brighter();

	// Constructors
	public WRVContainer() { super(); }
	public WRVContainer( int style )
	{
		this();
		drawStyle = style;
	}

	// Public methods
	public Dimension getMinimumSize() { return this.getPreferredSize(); }

	public void paint( Graphics g )
	{
		super.paint( g );

		int w = getSize().width - 1;
		int h = getSize().height - 1;

		if( drawStyle == DRAW_RAISED )
			g.setColor( highlightColor );
		else if( drawStyle == DRAW_SUNKEN )
			g.setColor( shadowColor );
		g.drawLine( 0, 0, w, 0 );
		g.drawLine( 0, 0, 0, h );
		if( drawStyle == DRAW_RAISED )
			g.setColor( shadowColor );
		else if( drawStyle == DRAW_SUNKEN )
			g.setColor( highlightColor );
		g.drawLine( w, h, w, 1 );
		g.drawLine( w, h, 1, h );
	}

	public void useSystemColors()
	{
		highlightColor = SystemColor.controlLtHighlight;
		shadowColor = SystemColor.controlDkShadow;
	}

	public int getDrawStyle() { return drawStyle; }
	public Color getHighlightColor() { return highlightColor; }
	public Color getShadowColor() { return shadowColor; }
	public void setHighlightColor( Color c )
	{
		if( c == null ) throw new IllegalArgumentException( "Color values must not be null" );

		highlightColor = c;

		if( isVisible() ) repaint();
	}

	public void setDrawStyle( int style )
	{
		if( style == DRAW_RAISED || style == DRAW_SUNKEN )
			drawStyle = style;
		else
			throw new IllegalArgumentException( "Draw style must be DRAW_RAISED or DRAW_SUNKEN" );
	}

	public void setInsets( Insets i )
	{
		i.left += 1;
		i.right += 1;
		i.top += 1;
		i.bottom += 1;

		super.setInsets( i );
	}

	public void setShadowColor( Color c )
	{
		if( c == null ) throw new IllegalArgumentException( "Color values must not be null" );

		shadowColor = c;

		if( isVisible() ) repaint();
	}
}

