/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: WRVPanel.java,v 1.10 2001/04/05 18:41:24 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.ui;

import java.awt.*;
import java.awt.event.*;
import jclass.bwt.*;
import jclass.util.*;

public class WRVPanel extends JCContainer
{
	public static final int DOCK_BUTTON_LEFT = 1;
	public static final int DOCK_BUTTON_RIGHT = 2;

	protected boolean	   dockable;
	protected boolean	   docked;
	protected int		   dockButtonLocation = DOCK_BUTTON_LEFT;
	protected JCContainer	   pagePanel;
	protected WRVContainer	   buttonBar;
	protected JCContainer	   userButtonBar;
	protected JCButton	   dockButton;
	protected WRVPanelNotebook notebook;
	protected JCTabManager	   tabContainer;
	protected Frame		   frame;
	protected String	   name;
	protected JCTabButton	   dockedTab;
	protected JCTabButton	   undockedTab;
	protected Color		   tabColor = null;
	protected Color		   tabFgColor = null;
	protected int		   position;
	protected MenuBar	   frameBar = null;
	protected MenuBar	   dockedBar = null;

	// These are used to hide the tab when it is undocked
	protected Color		   tabHighlight = null;
	protected int		   tabHighlightThickness = 0;
	protected int		   tabShadowThickness = 0;

	// Constructors
	public WRVPanel( WRVPanelNotebook owner, String myname )
	{
		this( owner, myname, false );
	}

	public WRVPanel( WRVPanelNotebook owner, String myname, boolean isDockable )
	{
		// Construct superclass
		super();

		// Save parameters
		notebook = owner;
		name = myname.intern();
		dockable = isDockable;

		// Construct container for undocked frame
		tabContainer = new JCTabManager();
		tabContainer.setTabShape( notebook.getTabShape() );
		tabContainer.setTabResize( notebook.getTabResize() );
		tabContainer.setTabStretch( notebook.getTabStretch() );
		int pos = tabContainer.addPage( name, null );
		undockedTab = tabContainer.getTab( pos );

		// Set up undocked frame
		Font menuFont = new Font( "Helvetica", Font.PLAIN, 12 );
		Menu windowMenu = new Menu( "Window" );
		windowMenu.setFont( menuFont );
		MenuItem windowDock = new MenuItem( "Dock", new MenuShortcut( 'D' ) );
		windowDock.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) { dock(); }
		} );
		windowDock.setFont( menuFont );
		windowMenu.add( windowDock );

		frameBar = new MenuBar();
		frameBar.add( windowMenu );

		frame = new Frame( "WRV: " + myname );
		frame.setLayout( new GridLayout( 1, 1 ) );
		frame.addWindowListener( new WindowAdapter()
		{
			public void windowClosing( WindowEvent e )
			{
				if( !docked ) dock();
			}
		} );
		frame.add( tabContainer );

		// Add page to notebook
		pos = notebook.addPage( name, this );
		dockedTab = notebook.getTab( pos );
		dockedTab.addComponentListener( new ComponentAdapter()
		{
			public void componentShown( ComponentEvent e )
			{
				if( !docked ) dockedTab.setVisible( false );
			}
		} );

		// Construct dock button
		dockButton = new JCButton( "Undock" );
		dockButton.addActionListener( new JCActionListener()
		{
			public void actionPerformed( JCActionEvent e )
			{
				if( !docked )
					dock();
				else
					undock();
			}
		} );

		// Set up the button bar
		buttonBar = new WRVContainer();
		buttonBar.setDrawStyle( WRVContainer.DRAW_SUNKEN );
		buttonBar.setInsets( new Insets( 1, 0, 1, 0 ) );
		if( dockable )
		{
			buttonBar.setLayout( new JCGridLayout( 1, 2, 1, 1 ) );
			buttonBar.add( "West", dockButton );
		}
		else
		{
			buttonBar.setLayout( new GridLayout( 1, 1 ) );
			buttonBar.setVisible( false );
		}

		// Set up the user-modifiable section of the button bar
		userButtonBar = new JCContainer();
		userButtonBar.setVisible( false );
		userButtonBar.setInsets( new Insets( 0, 0, 0, 0 ) );
		userButtonBar.addComponentListener( new ComponentAdapter()
		{
			public void componentHidden( ComponentEvent e )
			{
				if( !dockable )
				{
					buttonBar.setVisible( false );
					remove( buttonBar );
				}
				else
					buttonBar.doLayout();
			}

			public void componentShown( ComponentEvent e )
			{
				if( !dockable )
				{
					add( buttonBar );
					buttonBar.setVisible( true );
				}
				else
					buttonBar.doLayout();
			}
		} );
		buttonBar.add( userButtonBar );

		// Set up the page panel
		pagePanel = new JCContainer();


		// Set up the notebook page
		super.setLayout( new BorderLayout() );
//		if( dockable )
//			super.setLayout( new JCGridLayout( 2, 1, 1, 2 ) );
//		else
//			super.setLayout( new JCGridLayout( 1, 1, 1, 2 ) );
		setBackground( Color.lightGray );
		add( pagePanel, "Center" );
		if( dockable )
			add( buttonBar, "South" );

		docked = true;
	}

	// Protected methods
	protected void hideTab()
	{
		if( docked )
		{
			notebook.hideTab( dockedTab );
			tabHighlight = dockedTab.getHighlightColor();
			tabHighlightThickness = dockedTab.getHighlightThickness();
			tabShadowThickness = dockedTab.getShadowThickness();
			dockedTab.setForeground( getBackground() );
			dockedTab.setBackground( getBackground() );
			dockedTab.setHighlightColor( getBackground() );
			dockedTab.setHighlightThickness( 0 );
			dockedTab.setShadowThickness( 0 );
			dockedTab.setPage( null );
		}
	}

	protected void showTab()
	{
		if( docked )
		{
			dockedTab.setBackground( tabColor );
			dockedTab.setForeground( tabFgColor );
			dockedTab.setHighlightColor( tabHighlight );
			dockedTab.setHighlightThickness( tabHighlightThickness );
			dockedTab.setShadowThickness( tabShadowThickness );
			dockedTab.setPage( this );
			notebook.showTab( dockedTab );
		}
	}

	protected void handleDock()
	{
		frame.dispose();

		docked = true;

		dockButton.setLabel( "Undock" );

		undockedTab.setVisible( false );
		undockedTab.setPage( null );
		showTab();

		if( tabColor != null )
		{
			getTab().setBackground( tabColor );
			getTab().setForeground( tabFgColor );
		}

		if( notebook.isVisible() == false )
			notebook.setVisible( true );
	}

	protected void handleUndock()
	{
		hideTab();
		undockedTab.setPage( this );
		undockedTab.setVisible( true );
		tabContainer.makeTabVisible( 0 );
		tabContainer.setCurrentTab( 0 );

		dockButton.setLabel( "Dock" );

		frame.setMenuBar( frameBar );
		frame.pack();
		frame.show();

		docked = false;

		if( tabColor != null )
		{
			getTab().setBackground( tabColor );
			getTab().setForeground( tabFgColor );
		}
	}

	// Public methods
	public void dock() { if( !docked ) handleDock(); }
	public Panel getButtonBar() { return userButtonBar; }
	public MenuBar getMenuBar() { return( docked?dockedBar:frameBar ); }
	public Dimension getMinimumSize() { return getPreferredSize(); }
	public JCTabButton getTab()
	{
		JCTabButton tab = null;
		if( !docked )
			tab = undockedTab;
		else
			tab = dockedTab;
		return tab;
	}

	public Panel getPage() { return pagePanel; }
	public boolean isDockable() { return dockable; }
	public boolean isDocked() { return docked; }
	public void setDockable( boolean isDockable )
	{
		if( isDockable != dockable )
		{
			if( dockable )
			{
				buttonBar.remove( dockButton );
				buttonBar.setLayout( new GridLayout( 1, 1 ) );
				buttonBar.doLayout();

				if( userButtonBar.isVisible() == false )
				{
					buttonBar.setVisible( false );
					remove( buttonBar );
					super.setLayout( new JCGridLayout( 1, 1, 1, 2 ) );
					doLayout();
				}
			}
			else
			{
				buttonBar.removeAll();
				buttonBar.setLayout( new JCGridLayout( 1, 2 ) );

				if( dockButtonLocation == DOCK_BUTTON_LEFT )
					buttonBar.add( "West", dockButton );
				buttonBar.add( userButtonBar );
				if( dockButtonLocation == DOCK_BUTTON_RIGHT )
					buttonBar.add( "East", dockButton );

				if( buttonBar.isVisible() == false )
				{
					super.setLayout( new JCGridLayout( 2, 1, 1, 2 ) );
					add( buttonBar );
					doLayout();
					buttonBar.setVisible( true );
				}
			}

			dockable = isDockable;
		}
	}

	public void setDockButtonLocation( int location )
	{
		if( location == DOCK_BUTTON_LEFT || location == DOCK_BUTTON_RIGHT )
		{
			if( dockable &&
			    location != dockButtonLocation )
			{
				buttonBar.removeAll();
				dockButtonLocation = location;

				if( dockButtonLocation == DOCK_BUTTON_LEFT )
					buttonBar.add( dockButton );
				buttonBar.add( userButtonBar );
				if( dockButtonLocation == DOCK_BUTTON_RIGHT )
					buttonBar.add( dockButton );

				if( buttonBar.isVisible() == false )
					buttonBar.setVisible( true );
			}

			dockButtonLocation = location;
		}
		else
			throw new IllegalArgumentException( "Location must be DOCK_BUTTON_LEFT " +
							    "or DOCK_BUTTON_RIGHT" );
	}

	public void setLayout( LayoutManager manager ) {}
	public void setMenuBar( MenuBar menubar ) { dockedBar = menubar; }
	public void setTabColor( Color mytabColor )
	{
		tabColor = mytabColor;
		getTab().setBackground( tabColor );
	}

	public void setTabForeground( Color mycolor )
	{
		tabFgColor = mycolor;
		getTab().setForeground( tabFgColor );
	}

	public void undock() { if( docked ) handleUndock(); }
}

