/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: Criteria.java,v 1.11 2001/04/05 18:41:25 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.viewer;

public abstract class Criteria extends Module
{
	// Public finals
	public static final int STATUS_ENABLED  = 0;
	public static final int STATUS_DISABLED = 1;


	// Private data
	private int status = STATUS_ENABLED;


	// Constructors
	public Criteria( int handle )
	{ super(handle); }
	public Criteria( String name, int handle )
	{ super(name, handle); }
	public Criteria( String name, int handle, int newStatus )
	{ this(name, handle); status = newStatus; }

	public Criteria( String name, int handle, int newStatus, String user, String loadtime, long loadaddr, int size )
	{
		super(name, handle, user, loadtime, loadaddr, size);
		status = newStatus;
		markClean();
	}


	// Public methods
	public String getStatus()
	{ return( (status==STATUS_ENABLED)?"enabled":"disabled" ); }

	public abstract void refresh();

	public void refresh( String[] data )
	{
		int id = Integer.parseInt(data[0]);
		if( id != getHandle() )
			throw new RuntimeException("Refresh encountered mismatch in handle");
		name = data[1].intern();
		if( data[2].equals("disabled") )
			status = Criteria.STATUS_DISABLED;
		else
			status = Criteria.STATUS_ENABLED;
		user = data[3];
		loadtime = data[4];
		String hexAddr = data[5];
		if( hexAddr.startsWith("0x") )
			loadaddr = Long.valueOf(hexAddr.substring(2),16).longValue();
		else
			loadaddr = 0;
		size = Integer.parseInt(data[6]);
		markClean();
	}
}

