/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: Library.java,v 1.11 2001/04/05 18:41:25 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.viewer;

public class Library extends Module
{
	// Public data
	int refCount = 0;


	// Constructors
	public Library( int handle )
	{ super( handle ); }
	public Library( String name, int handle )
	{ super( name, handle ); }
	public Library( String name, int handle, int refs )
	{ this( name, handle ); refCount = refs; }

	public Library( String name, int handle, int refs, String user, String loadtime, long loadaddr, int size )
	{
		super( name, handle, user, loadtime, loadaddr, size );
		refCount = refs;
		markClean();
	}


	// Public methods
	public int getRefCount() { return refCount; }

	public void refresh()
	{
		if( isDirty() )
		{
			WrapperViewer wv = WrapperViewer.getGlobal();

			String[] data = wv.requestLibraryInfo( getHandle() );

			int id = Integer.parseInt(data[0]);
			if( id != getHandle() )
				throw new RuntimeException("Refresh encountered mismatch in handle");

			name = data[1].intern();
			refCount = Integer.parseInt(data[2]);
			user = data[3];
			loadtime = data[4];
			String hexAddr = data[5];
			if( hexAddr.startsWith("0x") )
				loadaddr = Long.valueOf(hexAddr.substring(2),16).longValue();
			else
				loadaddr = 0;
			size = Integer.parseInt(data[6]);
		}
	}
}

