/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: LibraryCache.java,v 1.11 2001/04/05 18:41:25 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

package com.gswtk.viewer;

import java.util.*;
import com.eventfs.*;
import com.gswtk.wrapper.event.*;

public class LibraryCache extends ModuleObjectCache
{
	// Constructors
	public LibraryCache() { this(false); }
	public LibraryCache( boolean fill ) { super(); if( fill ) fill(); }


	// Public methods
	public void fill()
	{
		String[] libdata = WrapperViewer.getGlobal().requestLibraries();

		if( libdata != null )
		{
			clear();

			for( int i = libdata.length - 1; i >= 0; i-- )
			{
				StringTokenizer st = new StringTokenizer( libdata[i], "|" );

				Integer id = new Integer(st.nextToken());
				if( !st.hasMoreTokens() ) continue;
				String name = st.nextToken();
				if( !st.hasMoreTokens() ) continue;
				int refs = Integer.parseInt(st.nextToken());
				if( !st.hasMoreTokens() ) continue;
				String user = st.nextToken();
				if( !st.hasMoreTokens() ) continue;
				String loadtime = st.nextToken();
				if( !st.hasMoreTokens() ) continue;
				long loadaddr = 0;
				String hexAddr = st.nextToken();
				if( hexAddr.startsWith("0x") )
					loadaddr = Long.valueOf(hexAddr.substring(2),16).longValue();
				if( !st.hasMoreTokens() ) continue;
				int size = Integer.parseInt(st.nextToken());

 				Library lib = new Library( name, id.intValue(),
				                           refs, user, loadtime,
				                           loadaddr, size );

				// Store it in the cache
				put( id, lib );
			}
		}
	}

	public Module fill( int id )
	{
		if( get(id) == null )
		{
			Library lib = new Library(id);

			// Refresh the library object, since it is
			//    currently dirty
			lib.refresh();

			// Store it in the cache
			put(new Integer(id), lib);

			// Return the Library
			return lib;
		}

		return null;
	}

	public void moduleEventOccurred( ModuleEvent me ) throws EventException
	{
		if( me instanceof LibraryEvent )
			super.moduleEventOccurred( me );
	}
}

