/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: ModuleObjectCache.java,v 1.9 2001/04/05 18:41:25 stevek Exp $
 *
 * Author: Steve Kiernan
 */

package com.gswtk.viewer;

import java.util.*;
import com.eventfs.*;
import com.gswtk.wrapper.event.*;

public abstract class ModuleObjectCache extends ObjectCache implements ModuleEventListener
{
	// Private data
	private Vector modEventListeners = new Vector();


	// Protected data
	protected ModuleEventStream mes;


	// Constructors
	public ModuleObjectCache() { this(false); }
	public ModuleObjectCache( boolean fill ) { super(fill); }

	
	// Public methods
	public void addModuleEventListener( ModuleEventListener mel )
	{ modEventListeners.addElement(mel); }

	public abstract void fill();
	public abstract Module fill( int id );
	public Object get( int id ) { return get(new Integer(id)); }

	public void listen( ModuleEventStream stream ) throws Exception
	{
		stop();
		mes = stream;
		mes.addModuleEventListener(this);
	}

	public void listen() throws Exception
	{
		ModuleEventStream stream = new ModuleEventStream();
		stream.setOwner( this );
		listen( stream );
		stream.listen();
	}

	public void moduleEventOccurred( ModuleEvent me ) throws EventException
	{
		// add a note denoting in which cache this is stored
		me.addNote( "cache", this );

		// fill in the module, if it is not flagged for no caching
		if( me.getNote( "nocache" ) == null )
		{
			if( me instanceof ModuleLoadedEvent )
				fill( me.getModuleID() );
			else if( me instanceof ModuleUnloadedEvent )
				remove( new Integer(me.getModuleID()) );
		}

		// notify all listeners
		for( Enumeration e = modEventListeners.elements();
		     e.hasMoreElements(); )
			((ModuleEventListener)e.nextElement()).moduleEventOccurred(me);
	}

	public void removeModuleEventListener( ModuleEventListener mel )
	{ modEventListeners.removeElement(mel); }

	public void stop()
	{
		if( mes != null )
		{
			mes.removeModuleEventListener(this);
			if( mes.getOwner() == this )
				mes.stop();
			mes = null;
		}
	}
}

