/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: WrapperViewer.java,v 1.24 2001/04/05 18:41:25 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Wrapper viewer interface class
 */

package com.gswtk.viewer;

import java.io.*;
import java.text.*;
import java.util.*;
import com.eventfs.*;
import com.gswtk.db.*;
import com.gswtk.wrapper.event.*;

public class WrapperViewer
{
	static {
		try { System.loadLibrary("gswtknative"); }
		catch( UnsatisfiedLinkError ule ) {
			System.loadLibrary("WrapperViewer");
		}
	}


	// Protected data
	protected static WrapperViewer     globalViewer = new WrapperViewer();
	protected	 DatabaseCache     db;


	// Private data
	private Vector modEventListeners         = new Vector();
	private Vector modLoadedEventListeners   = new Vector();
	private Vector modUnloadedEventListeners = new Vector();


	// Constructors
	protected WrapperViewer()
	{
		try {
			_register();

			db = new DatabaseCache( this );
		}
		catch( Exception e ) {
			e.printStackTrace(System.err);
			System.exit(0);
		}
	}

	// Finalization
	public void finalize()
	{
		_unregister();
	}

	// Private methods
	private native void  _register();
	private native Table _requestTableByHandle( int handle );
	private native void  _unregister();

	// Public methods
	public native synchronized void addActivationCriteria( String act ) throws ParseException;
	public native synchronized void addDeactivationCriteria( String act ) throws ParseException;
	public native synchronized void addTableRow( Table tab );
	public native synchronized void delTableRow( Table tab, Row row );
	public DatabaseCache getDatabaseCache() { return db; }
	public static WrapperViewer getGlobal() { return globalViewer; }
	public native synchronized void loadModule( String file ) throws IOException;
	public native String[] requestActivationCriteria();
	public native String[] requestActivationCriteriaInfo( int id );
	public native String[] requestDeactivationCriteria();
	public native String[] requestDeactivationCriteriaInfo( int id );
	public native void requestColumnDefinition( Table tab );
	public native String[] requestLibraries();
	public native String[] requestLibraryInfo( int id );
	public native String[] requestModules();
	public native String[] requestProcesses();
	public void requestRow( Table tab, int rowid ) throws InvalidHandleException
	{
		int rowids[] = { rowid };
		requestRows( tab, rowids );
	}

	public native void requestRows( Table tab, int rowids[] );
	public native void requestTable( Table tab );
	public Table requestTableByHandle( int handle, boolean fill )
	{
		Table tab = _requestTableByHandle( handle );

		if( fill )
			requestTable( tab );

		return tab;
	}

	public native Enumeration requestTables();
	public native String[] requestWrappers();
	public native String[] requestWrapperInfo( int id );

	public native synchronized void unloadModule( int handle ) throws IOException;
	public native synchronized void updateTableRowColumnData( Table tab, Row row, Column col, String data );
}

