/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: WrapperViewer_jni.c,v 1.55 2001/02/23 19:02:25 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

#include <wr_conf.h>

#ifndef LINKAGE
#define LINKAGE IMPORT
#endif /* LINKAGE */

#include <stdarg.h>
#ifdef WR_UNIX
#include <sys/param.h>
#endif /* WR_UNIX */

#include <wsi_stdlib.h>

#include <errno.h>

#ifndef WR_WINNT
#include <sys/time.h>
#include <unistd.h>
#endif /* !WR_WINNT */

#include "wss_db.h"
#include "libwrv.h"

#include "WrapperViewer_jni.h"

#include <limits.h>


/*
 * Private functions
 */
static void
FillTableWithRows( JNIEnv *env, jobject obj, jobject tab, wrdb_row_t rows )
{
   wrdb_row_t ptr;
   jclass     tabClass, cdClass, rowClass, colClass, viewerClass;
   jobject    cdObj;
   jint       stringType, intType, floatType;
   int        i;

   tabClass = (*env)->GetObjectClass( env, tab );
   cdClass = (*env)->FindClass( env, "com/gswtk/db/ColumnDefinition" );
   cdObj = (*env)->CallObjectMethod( env, tab,
              (*env)->GetMethodID( env, tabClass, "getColumnDefinition",
                                   "()Lcom/gswtk/db/ColumnDefinition;" ) );
   if( cdObj == NULL )
   {
      viewerClass = (*env)->GetObjectClass( env, obj );
      (*env)->CallVoidMethod( env, obj,
         (*env)->GetMethodID( env, viewerClass, "requestColumnDefinition",
                              "(Lcom/gswtk/db/Table;)V" ), tab );
      cdObj = (*env)->CallObjectMethod( env, tab,
                 (*env)->GetMethodID( env, tabClass, "getColumnDefinition",
                                      "()Lcom/gswtk/db/ColumnDefinition;" ) );

      if( cdObj == NULL )
      {
         jclass     ec = (*env)->FindClass( env, "java/lang/UnknownError" );
         jthrowable ex = (jthrowable)(*env)->NewObject( env, ec,
                               (*env)->GetMethodID( env, ec, "<init>",
                                     "()V" ) );
         (*env)->Throw( env, ex );
         return;
      }
   }

   if( rows )
   {
      rowClass = (*env)->FindClass( env, "com/gswtk/db/Row" );
      colClass = (*env)->FindClass( env, "com/gswtk/db/Column" );
      stringType = (*env)->GetStaticIntField( env, colClass,
                     (*env)->GetStaticFieldID( env, colClass, "TYPE_STRING", "I" ) );
      intType = (*env)->GetStaticIntField( env, colClass,
                    (*env)->GetStaticFieldID( env, colClass, "TYPE_INT", "I" ) );
      floatType = (*env)->GetStaticIntField( env, colClass,
                         (*env)->GetStaticFieldID( env, colClass, "TYPE_FLOAT", "I" ) );

      for( ptr = rows; ptr; ptr = ROW_NEXT(ptr) )
      {
         jobject rowObj = (*env)->NewObject( env, rowClass,
                             (*env)->GetMethodID( env, rowClass, "<init>",
                                "(Lcom/gswtk/db/ColumnDefinition;I)V" ),
                             cdObj, ptr->handle );

         for( i = 0; i < ROW_NUMVALS(ptr); i++ )
         {
            wrdb_value_t val;

            jobject      colObj;
            jint         valType;
            jobject      valObj;

#ifdef WR_WINNT
            val = (wrdb_value_t)( (pd_byte_t *)rows +
                                  (DWORD)(ROW(ptr)->vals) ) + i;
#else
            val = ROW(ptr)->vals + i;
#endif /* WR_WINNT */

            colObj = (*env)->CallObjectMethod( env, cdObj,
                        (*env)->GetMethodID( env, cdClass, "columnAt",
                                             "(I)Lcom/gswtk/db/Column;" ),
                        i );
            valType = (*env)->CallIntMethod( env, colObj,
                         (*env)->GetMethodID( env, colClass, "getType",
                                              "()I" ) );

            if( valType == stringType )
            {
               pd_char_t *cval;

#ifdef WR_WINNT
               cval = (pd_char_t *)( (pd_byte_t *)rows +
                                      (DWORD)( val->v.cval ) );
#else
               cval = val->v.cval;
#endif /* WR_WINNT */
               valObj = (*env)->NewStringUTF( env, cval );
            }
            else if( valType == intType )
            {
               jclass c = (*env)->FindClass( env, "java/lang/Integer" );
               valObj = (*env)->NewObject( env, c,
                            (*env)->GetMethodID( env, c, "<init>", "(I)V" ),
                            val->v.ival );
            }
            else if( valType == floatType )
            {
               jclass c = (*env)->FindClass( env, "java/lang/Float" );
               valObj = (*env)->NewObject( env, c,
                            (*env)->GetMethodID( env, c, "<init>", "(F)V" ),
                            val->v.fval );
            }
            else
            {
               jclass      ec = (*env)->FindClass( env, "java/lang/UnknownError" );
               jthrowable ex =
                  (jthrowable)(*env)->NewObject( env, ec,
                                  (*env)->GetMethodID( env, ec, "<init>",
                                        "()V" ) );
               (*env)->Throw( env, ex );
               return;
            }

            (*env)->CallVoidMethod( env, rowObj,
                     (*env)->GetMethodID( env, rowClass, "add",
                                "(ILjava/lang/Object;)V" ), i, valObj );
         }

         (*env)->CallVoidMethod( env, tab, (*env)->GetMethodID( env, tabClass, "put",
                                  "(Lcom/gswtk/db/Row;)V" ), rowObj );
#ifdef WR_WINNT
         if( ROW_NEXT(ptr) != NULL )
            ROW_NEXT(ptr) = (wrdb_row_t)( (pd_byte_t *)rows + (DWORD)ROW_NEXT(ptr) );
#endif /* WR_WINNT */
      }
   }
}


/*
 * JNI methods
 */
JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer__1register( JNIEnv *env, jobject obj )
{
    /*
     * Initialize Wrapper Viewer interface and register with WSS
     */
    jclass c;

    if( !wrvIsLoaded() )
    {
        c = (*env)->FindClass( env, "java/lang/RuntimeException" );
        (*env)->ThrowNew( env, c, "WSS not loaded" );
        return;
    }

    wrvInit();
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer__1unregister( JNIEnv *env, jobject obj )
{
   /*
    * Shutdown the Wrapper Viewer interface
    */
   wrvShutdown();
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_loadModule( JNIEnv *env, jobject obj, jstring file )
{
   const char *wrapperName;
   jboolean isCopy;

   wrapperName = (*env)->GetStringUTFChars( env, file, &isCopy );
   if( wrvLoadModule( wrapperName ) < 0 )
   {
      jclass  c;
      char    errorBuffer[MAXPATHLEN];
      char   *errorStr = (char *)wrapperName;

      if( isCopy == JNI_TRUE )
         (*env)->ReleaseStringUTFChars( env, file, wrapperName );

      switch( errno )
      {
         case ENOENT:
            c = (*env)->FindClass( env, "java/io/FileNotFoundException" );
            break;
         default:
         {
            c = (*env)->FindClass( env, "java/io/IOException" );
            snprintf( errorBuffer, MAXPATHLEN, "%s: %s", wrapperName,
                      strerror( errno ) );
            errorStr = errorBuffer;
            break;
         }
      }
      (*env)->ThrowNew( env, c, errorStr );
      return;
   }

   if( isCopy == JNI_TRUE )
      (*env)->ReleaseStringUTFChars( env, file, wrapperName );
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_addActivationCriteria( JNIEnv *env, jobject obj, jstring act )
{
   const char *criteria;
   jboolean    isCopy;

   criteria = (*env)->GetStringUTFChars( env, act, &isCopy );
   if( wrvAddActivationCriteria( criteria ) < 0 )
   {
      jclass     exc, sbc;
      jthrowable ex, sb;

      sbc = (*env)->FindClass( env, "java/lang/StringBuffer" );
      sb = (*env)->NewObject( env, sbc,
                              (*env)->GetMethodID( env, sbc, "<init>",
                                                   "(Ljava/lang/String;)V" ),
                              act );
      sb = (*env)->CallObjectMethod( env, sb,
                      (*env)->GetMethodID( env, sbc, "append",
                            "(Ljava/lang/String;)Ljava/lang/StringBuffer;" ),
                      (*env)->NewStringUTF( env, "\n" ) );
      sb = (*env)->CallObjectMethod( env, sb,
                      (*env)->GetMethodID( env, sbc, "append",
                            "(Ljava/lang/String;)Ljava/lang/StringBuffer;" ),
                      (*env)->NewStringUTF( env, wrvGetParserError() ) );
      exc = (*env)->FindClass( env, "java/text/ParseException" );
      ex = (jthrowable)(*env)->NewObject( env, exc,
                      (*env)->GetMethodID( env, exc, "<init>",
                            "(Ljava/lang/String;I)V" ),
                      (*env)->CallObjectMethod( env, sb,
                                    (*env)->GetMethodID( env, sbc,
                                       "toString",
                                       "()Ljava/lang/String;" ) ),
                      0 );
      (*env)->Throw( env, ex );
   }

   if( isCopy == JNI_TRUE )
      (*env)->ReleaseStringUTFChars( env, act, criteria );
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_addDeactivationCriteria( JNIEnv *env, jobject obj, jstring act )
{
   const char *criteria;
   jboolean    isCopy;

   criteria = (*env)->GetStringUTFChars( env, act, &isCopy );
   if( wrvAddDeactivationCriteria( criteria ) < 0 )
   {
      jclass      exc, sbc;
      jthrowable ex, sb;

      sbc = (*env)->FindClass( env, "java/lang/StringBuffer" );
      sb = (*env)->NewObject( env, sbc,
                              (*env)->GetMethodID( env, sbc, "<init>",
                                                   "(Ljava/lang/String;)V" ),
                              act );
      sb = (*env)->CallObjectMethod( env, sb,
                      (*env)->GetMethodID( env, sbc, "append",
                            "(Ljava/lang/String;)Ljava/lang/StringBuffer;" ),
                      (*env)->NewStringUTF( env, "\n" ) );
      sb = (*env)->CallObjectMethod( env, sb,
                      (*env)->GetMethodID( env, sbc, "append",
                            "(Ljava/lang/String;)Ljava/lang/StringBuffer;" ),
                      (*env)->NewStringUTF( env, wrvGetParserError() ) );
      exc = (*env)->FindClass( env, "java/text/ParseException" );
      ex = (jthrowable)(*env)->NewObject( env, exc,
                      (*env)->GetMethodID( env, exc, "<init>",
                            "(Ljava/lang/String;I)V" ),
                      (*env)->CallObjectMethod( env, sb,
                                    (*env)->GetMethodID( env, sbc,
                                       "toString",
                                       "()Ljava/lang/String;" ) ),
                      0 );
      (*env)->Throw( env, ex );
   }

   if( isCopy == JNI_TRUE )
      (*env)->ReleaseStringUTFChars( env, act, criteria );
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestActivationCriteria( JNIEnv *env, jobject obj )
{
   jobjectArray actcrit = NULL;
   wrv_buffer *actcritStrings = wrvGetActivationCriteria();

   if( actcritStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      size_t  j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( actcritStrings );

      actcrit = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( actcritStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( actcritStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, actcrit, i, s );
      }

      wrvReleaseActivationCriteria( actcritStrings );
   }
   else
      perror( "wrvGetActivationCriteria" );

   return actcrit;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestDeactivationCriteria( JNIEnv *env, jobject obj )
{
   jobjectArray actcrit = NULL;
   wrv_buffer *actcritStrings = wrvGetDeactivationCriteria();

   if( actcritStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      size_t  j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( actcritStrings );

      actcrit = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( actcritStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( actcritStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, actcrit, i, s );
      }

      wrvReleaseDeactivationCriteria( actcritStrings );
   }
   else
      perror( "wrvGetDeactivationCriteria" );

   return actcrit;
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestColumnDefinition( JNIEnv *env, jobject obj, jobject tab )
{
   wrdb_coldef_t cd;
   int           size, i;
   jclass        tabClass, cdClass;
   jint          id;
   jobject       cdObj; /* com/gswtk/db/ColumnDefinition */

   tabClass = (*env)->GetObjectClass( env, tab );
   id = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, tabClass, "hashCode", "()I" ) );
   cd = wrvDbGetColumnDefinition( id, &size);
   if( cd == NULL )
      return;

   cdClass = (*env)->FindClass( env, "com/gswtk/db/ColumnDefinition" );
   cdObj = (*env)->NewObject( env, cdClass, (*env)->GetMethodID( env, cdClass, "<init>", "()V" ) );

   for (i = 0 ; i < size ; i++)
   {
      wrdb_coldef_t ptr = cd + i;
      jstring       name;
      jint          type;
      jclass        colClass;

#ifdef WR_WINNT
      ptr->name = (pd_char_t *)( (pd_byte_t *)cd + (DWORD)ptr->name );
#endif /* WR_WINNT */

      name = (*env)->NewStringUTF( env, ptr->name );


      colClass = (*env)->FindClass( env, "com/gswtk/db/Column" );

      switch( ptr->dtype )
      {
         case pd_char_t_t:
            type = (*env)->GetStaticIntField( env, colClass,
                          (*env)->GetStaticFieldID( env, colClass,
                                     "TYPE_STRING", "I" ) );
            break;
         case int_t:
            type = (*env)->GetStaticIntField( env, colClass,
                          (*env)->GetStaticFieldID( env, colClass,
                                     "TYPE_INT", "I" ) );
            break;
         case float_t:
            type = (*env)->GetStaticIntField( env, colClass,
                          (*env)->GetStaticFieldID( env, colClass,
                                     "TYPE_FLOAT", "I" ) );
            break;
         default:
         {
            jclass      ec = (*env)->FindClass( env, "java/lang/UnknownError" );
            jthrowable ex =
               (jthrowable)(*env)->NewObject( env, ec,
                               (*env)->GetMethodID( env, ec, "<init>",
                                     "()V" ) );
            (*env)->Throw( env, ex );
            return;
         }
      }

      (*env)->CallVoidMethod( env, cdObj, (*env)->GetMethodID( env, cdClass, "addColumn",
                            "(Ljava/lang/String;II)V" ),
               name, type, ptr->max_len );
   }

   wrvDbReleaseColumnDefinition( cd );

   (*env)->CallVoidMethod( env, tab, (*env)->GetMethodID( env, tabClass, "setColumnDefinition",
                            "(Lcom/gswtk/db/ColumnDefinition;)V" ),
            cdObj );
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestLibraryInfo( JNIEnv *env, jobject obj, jint id )
{
   jobjectArray  info  = NULL;
   wrv_buffer   *modStrings = wrvGetLibraryByHandle( id );

   if( modStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      size_t  i;
      jstring s;

      info = (*env)->NewObjectArray( env, wrvBufferGetColCount( modStrings ),
                                     c, NULL );

      for( i = 0; i < wrvBufferGetColCount( modStrings ); i++ )
      {
         s = (*env)->NewStringUTF( env,
                                   wrvBufferGetString( modStrings, 0, i ) );
         (*env)->SetObjectArrayElement( env, info, i, s );
      }

      wrvReleaseModules( modStrings );
   }
   else
      perror( "wrvGetLibraryByHandle" );

   return info;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestLibraries( JNIEnv *env, jobject obj )
{
   jobjectArray  libraries = NULL;
   wrv_buffer   *libStrings = wrvGetLibraries();

   if( libStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      size_t  j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( libStrings );

      libraries = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( libStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( libStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, libraries, i, s );
      }

      wrvReleaseLibraries( libStrings );
   }
   else
      perror( "wrvGetLibraries" );

   return libraries;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestModules( JNIEnv *env, jobject obj )
{
   jobjectArray  modules = NULL;
   wrv_buffer   *modStrings   = wrvGetModules();

   if( modStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      size_t  j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( modStrings );

      modules = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( modStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( modStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, modules, i, s );
      }

      wrvReleaseModules( modStrings );
   }
   else
      perror( "wrvGetModules" );

   return modules;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestActivationCriteriaInfo( JNIEnv *env, jobject obj, jint id )
{
   jobjectArray  info  = NULL;
   wrv_buffer   *modStrings = wrvGetActivationCriteriaByHandle( id );

   if( modStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      size_t  i;
      jstring s;

      info = (*env)->NewObjectArray( env, wrvBufferGetColCount( modStrings ),
                                     c, NULL );

      for( i = 0; i < wrvBufferGetColCount( modStrings ); i++ )
      {
         s = (*env)->NewStringUTF( env,
                                   wrvBufferGetString( modStrings, 0, i ) );
         (*env)->SetObjectArrayElement( env, info, i, s );
      }

      wrvReleaseModules( modStrings );
   }
   else
      perror( "wrvGetActivationCriteriaByHandle" );

   return info;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestDeactivationCriteriaInfo( JNIEnv *env, jobject obj, jint id )
{
   jobjectArray  info  = NULL;
   wrv_buffer   *modStrings = wrvGetDeactivationCriteriaByHandle( id );

   if( modStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      size_t  i;
      jstring s;

      info = (*env)->NewObjectArray( env, wrvBufferGetColCount( modStrings ),
                                     c, NULL );

      for( i = 0; i < wrvBufferGetColCount( modStrings ); i++ )
      {
         s = (*env)->NewStringUTF( env,
                                   wrvBufferGetString( modStrings, 0, i ) );
         (*env)->SetObjectArrayElement( env, info, i, s );
      }

      wrvReleaseModules( modStrings );
   }
   else
      perror( "wrvGetDeactivationCriteriaByHandle" );

   return info;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestProcesses( JNIEnv *env, jobject obj )
{
   jobjectArray  processes = NULL;
   wrv_buffer   *procStrings   = wrvGetProcesses();

   if( procStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      size_t  j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( procStrings );
      processes = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( procStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( procStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, processes, i, s );
      }

      wrvReleaseProcesses( procStrings );
   }
   else
      perror( "wrvGetProcesses" );

   return processes;
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestRows( JNIEnv *env, jobject obj, jobject tab, jintArray rowHandles )
{
   wrdb_row_t  rows, ptr;
   int         size, i;
   jint       *intArray;
   jboolean    isCopy;
   jclass      tabClass;
   jint        thandle;

   tabClass = (*env)->GetObjectClass( env, tab );
   thandle = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, tabClass, "hashCode", "()I" ) );
   size = (*env)->GetArrayLength( env, rowHandles );
   intArray = (*env)->GetIntArrayElements( env, rowHandles, &isCopy );
   rows = wrvDbGetRowsByHandle( thandle, (u_long *)intArray, size );
   if( isCopy == JNI_TRUE )
      (*env)->ReleaseIntArrayElements( env, rowHandles, intArray, 0 );

   for( ptr = rows, i = 0;
        ptr;
#ifdef WR_WINNT
        ptr = (wrdb_row_t)( (ROW_NEXT(ptr))? ( (pd_byte_t *)rows +
                                               (DWORD)ROW_NEXT(ptr) )
                                           : NULL ),
#else
        ptr = ROW_NEXT(ptr),
#endif /* WR_WINNT */
        i++ )
   ;

   if( i < size )
   {
      jclass     ec = (*env)->FindClass( env, "com/gswtk/db/InvalidHandleException" );
      jthrowable ex = (jthrowable)(*env)->NewObject( env, ec, (*env)->GetMethodID( env, ec, "<init>",
                                      "()V" ) );
      (*env)->Throw( env, ex );
      return;
   }

   FillTableWithRows( env, obj, tab, rows );

   if( rows )
      wrvDbReleaseRows( rows );
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestTable( JNIEnv *env, jobject obj, jobject tab )
{
   wrdb_row_t rows;
   jint       handle;

   handle = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, tab ),
                               "getHandle", "()I" ) );

   rows = wrvDbGetRowsByHandle( handle, NULL, 0 );

   FillTableWithRows( env, obj, tab, rows );

   if( rows )
      wrvDbReleaseRows( rows );
}

JNIEXPORT jobject JNICALL
Java_com_gswtk_viewer_WrapperViewer__1requestTableByHandle( JNIEnv *env, jobject obj, jint handle )
{
   jobject         tab = NULL;  /* com/gswtk/db/Table */
   unsigned long   handle_arg = handle;
   wrv_table_id_t *table = wrvDbGetTableIdByHandle( handle_arg );

   if( table )
   {
      jclass  c = (*env)->FindClass( env, "com/gswtk/db/Table" );
      jstring name = (*env)->NewStringUTF( env, table->name );

      tab = (*env)->NewObject( env, c, (*env)->GetMethodID( env, c, "<init>",
                              "(Ljava/lang/String;I)V" ),
                name, table->handle );
      wrvDbReleaseTableIds( table );
   }

   return tab;
}

JNIEXPORT jobject JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestTables( JNIEnv *env, jobject obj )
{
   wrv_table_id_t *tables;
   wrv_table_id_t *id;

   jclass  c = (*env)->FindClass( env, "java/util/Vector" );
   jobject v = (*env)->NewObject( env, c, (*env)->GetMethodID( env, c, "<init>", "()V" ) );
   jobject throwObj;
   jobject elements;

   tables = wrvDbGetTableIds();
   if( tables )
   {
      jclass tc = (*env)->FindClass( env, "com/gswtk/db/Table" );

      for( id = tables; id; id = id->next )
      {
         jstring name;
         jobject tab;

#ifdef WR_WINNT
         id->name = (pd_char_t *)( (pd_byte_t *)tables + (DWORD)id->name );
#endif /* WR_WINNT */
         
         name = (*env)->NewStringUTF( env, id->name );
         tab =
            (*env)->NewObject( env, tc,
                               (*env)->GetMethodID( env, tc, "<init>",
                                                    "(Ljava/lang/String;I)V" ),
                               name, id->handle );

         if( ( throwObj = (*env)->ExceptionOccurred( env ) ) != NULL )
         {
            (*env)->CallVoidMethod( env, throwObj,
                     (*env)->GetMethodID( env,
                                (*env)->GetObjectClass( env, throwObj ),
                                "printStackTrace", "()V" ) );
            (*env)->ExceptionClear( env );
         }
         else
         {
            (*env)->CallVoidMethod( env, v, 
                     (*env)->GetMethodID( env, c, "addElement",
                                "(Ljava/lang/Object;)V" ),
                     tab );
         }
#ifdef WR_WINNT
         if( id->next )
            id->next = (wrv_table_id_t *)( (pd_byte_t *)tables + (DWORD)id->next );
#endif /* WR_WINNT */
      }

      wrvDbReleaseTableIds( tables );
   }

   elements = (*env)->CallObjectMethod( env, v,
                    (*env)->GetMethodID( env, c, "elements",
                          "()Ljava/util/Enumeration;" ) );
   return elements;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestWrapperInfo( JNIEnv *env, jobject obj, jint id )
{
   jobjectArray  info  = NULL;
   wrv_buffer   *modStrings = wrvGetWrapperByHandle( id );

   if( modStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      size_t  i;
      jstring s;

      info = (*env)->NewObjectArray( env, wrvBufferGetColCount( modStrings ),
                                     c, NULL );

      for( i = 0; i < wrvBufferGetColCount( modStrings ); i++ )
      {
         s = (*env)->NewStringUTF( env,
                                   wrvBufferGetString( modStrings, 0, i ) );
         (*env)->SetObjectArrayElement( env, info, i, s );
      }

      wrvReleaseModules( modStrings );
   }
   else
      perror( "wrvGetWrapperByHandle" );

   return info;
}

JNIEXPORT jobjectArray JNICALL
Java_com_gswtk_viewer_WrapperViewer_requestWrappers( JNIEnv *env, jobject obj )
{
   jobjectArray  wrappers = NULL;
   wrv_buffer   *wrapStrings = wrvGetWrappers();

   if( wrapStrings )
   {
      jclass  c = (*env)->FindClass( env, "java/lang/String" );
      int     i;
      int     j;
      int     rows;
      jstring s;

      rows = wrvBufferGetRowCount( wrapStrings );

      wrappers = (*env)->NewObjectArray( env, rows, c, NULL );

      for( i = 0; i < rows; i++ )
      {
         s = (*env)->NewStringUTF( env, "" );

         for( j = 0; j < wrvBufferGetColCount( wrapStrings ); j++ )
         {
            if( j > 0 )
               s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                              "(Ljava/lang/String;)Ljava/lang/String;" ),
                              (*env)->NewStringUTF( env, "|" ) );
            s = (*env)->CallObjectMethod( env, s, (*env)->GetMethodID( env, c, "concat",
                           "(Ljava/lang/String;)Ljava/lang/String;" ),
                           (*env)->NewStringUTF( env,
                              wrvBufferGetString( wrapStrings, i, j ) ) );
         }

         (*env)->SetObjectArrayElement( env, wrappers, i, s );
      }

      wrvReleaseWrappers( wrapStrings );
   }
   else
      perror( "wrvGetWrappers" );

   return wrappers;
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_unloadModule( JNIEnv *env, jobject obj, jint handle )
{
   if( wrvUnloadModule( handle ) < 0 )
   {
      jclass c = (*env)->FindClass( env, "java/io/IOException" );
      (*env)->ThrowNew( env, c, "Unable to unload module" );
   }
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_updateTableRowColumnData( JNIEnv *env, jobject obj, jobject tab, jobject row, jobject col, jstring data )
{
   jint        thandle;
   jint        rhandle;
   jstring     column;
   const char *columnName;
   jboolean    cnIsCopy;
   const char *dataValue;
   jboolean    dvIsCopy;

   thandle = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, tab ),
                                    "getHandle", "()I" ) );
   rhandle = (*env)->CallIntMethod( env, row, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, row ),
                                    "getHandle", "()I" ) );
   column = (jstring)(*env)->CallObjectMethod( env, col, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, col ),
                                               "getName", "()Ljava/lang/String;" ) );
   columnName = (*env)->GetStringUTFChars( env, column, &cnIsCopy );
   dataValue = (*env)->GetStringUTFChars( env, data, &dvIsCopy );

   if( wrvDbSetTableRowField( thandle, rhandle, columnName, dataValue ) < 0 )
   {
      jclass c;

      if( cnIsCopy == JNI_TRUE )
         (*env)->ReleaseStringUTFChars( env, column, columnName );
      if( dvIsCopy == JNI_TRUE )
         (*env)->ReleaseStringUTFChars( env, data, dataValue );

      switch( errno )
      {
         case EINVAL:
            c = (*env)->FindClass( env, "java/lang/InvalidArgumentException" );
            (*env)->ThrowNew( env, c, columnName );
            break;

         case ENOENT:
            c = (*env)->FindClass( env, "java/io/InvalidObjectException" );
            (*env)->ThrowNew( env, c, columnName );
            break;

         default:
            c = (*env)->FindClass( env, "java/lang/RuntimeException" );
            (*env)->ThrowNew( env, c, strerror( errno ) );
            break;
      }
      return;
   }

   if( cnIsCopy == JNI_TRUE )
      (*env)->ReleaseStringUTFChars( env, column, columnName );
   if( dvIsCopy == JNI_TRUE )
      (*env)->ReleaseStringUTFChars( env, data, dataValue );
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_addTableRow( JNIEnv *env, jobject obj, jobject tab )
{
   jint thandle;

   thandle = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, tab ),
                                    "getHandle", "()I" ) );

   if( wrvDbAddNullTableRow( thandle ) < 0 )
   {
      jclass c = (*env)->FindClass( env, "java/lang/RuntimeException" );
      (*env)->ThrowNew( env, c, strerror( errno ) );
      return;
   }
}

JNIEXPORT void JNICALL
Java_com_gswtk_viewer_WrapperViewer_delTableRow( JNIEnv *env, jobject obj, jobject tab, jobject row )
{
   jint thandle;
   jint rhandle;

   thandle = (*env)->CallIntMethod( env, tab, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, tab ),
                                    "getHandle", "()I" ) );
   rhandle = (*env)->CallIntMethod( env, row, (*env)->GetMethodID( env, (*env)->GetObjectClass( env, row ),
                                    "getHandle", "()I" ) );

   if( wrvDbDelTableRow( thandle, rhandle ) < 0 )
   {
      jclass c = (*env)->FindClass( env, "java/lang/RuntimeException" );
      (*env)->ThrowNew( env, c, strerror( errno ) );
      return;
   }
}

