/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: freebsd4.ch,v 1.14 2001/06/23 00:09:52 stevek Exp $
 */

includes {
#ifndef _WR_DEFS_INCLUDED
#include "wr_conf.h"
#endif

#ifndef __WSI_TYPES_H
#include "wsi_types.h"
#endif

#include <sys/unistd.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/socket.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/utsname.h>
#include <sys/uio.h>
#include <sys/fcntl.h>
#include <sys/acl.h>
#include <errno.h>
};

#include <sys/poll.h>

#include "wr_errname.h"

#define WR_LOCAL freebsd

characterize WR_LOCAL {

int errnum{
       errno{
          vmap(WR_SUCC,         _),
          vmap(WR_FAIL,         _),
          vmap(WR_BADPERM,      EPERM),
          vmap(WR_NOFILE,       ENOENT),
          vmap(WR_NOPROC,       ESRCH),
          vmap(WR_IOERR,        EIO),
          vmap(WR_ARG2LONG,     E2BIG),
          vmap(WR_BADFD,        EBADF),
          vmap(WR_NOCHILD,      ECHILD),
          vmap(WR_BADACCES,     EACCES),
          vmap(WR_FILEEXIST,    EEXIST),
          vmap(WR_BADARG,       EINVAL),
          vmap(WR_FILEBSY,      ETXTBSY),
          vmap(WR_FILE2BIG,     EFBIG),
          vmap(WR_NOSPC,        ENOSPC),
          vmap(WR_FNAME2LONG,   ENAMETOOLONG)
       }
   };

/* 
 * syscalls that return 0 on success and -1 on failure
 */
#define STD_RET wrret{vmap(WR_SUCC,0), vmap(*,-1)}
/* 
 * syscalls that return some values on success and -1 on failure
 */
#define VAL_RET wrret{vmap(WR_SUCC,_), vmap(*,-1)}
/* 
 * syscalls that are always successful, and no return value is 
 * reserved to indicate an error. 
 */
#define NOERR_RET wrret     

int{erret,STD_RET} __acl_aclcheck_fd{aclop,fdop}(int filedes{fd},
	acl_type_t acltype, struct acl *aclp);

int{erret,STD_RET} __acl_aclcheck_file{aclop,fileop}(const char *path{path,nterm},
	acl_type_t acltype, struct acl *aclp);

int{erret,STD_RET} __acl_delete_fd{aclop,fdop}(int filedes{fd},
	acl_type_t acltype);

int{erret,STD_RET} __acl_delete_file{aclop,fileop}(const char *path{path,nterm},
	acl_type_t acltype);

int{erret,STD_RET} __acl_get_fd{aclop,fdop}(int filedes{fd},
	acl_type_t acltype, struct acl *aclp{out});

int{erret,STD_RET} __acl_get_file{aclop,fileop}(const char *path{path,nterm},
	acl_type_t acltype, struct acl *aclp{out});

int{erret,STD_RET} __acl_set_fd{aclop,fdop}(int filedes{fd},
	acl_type_t acltype, struct acl *aclp);

int{erret,STD_RET} __acl_set_file{aclop,fileop}(const char *path{path,nterm},
	acl_type_t acltype, struct acl *aclp);

int{erret,STD_RET} __getcwd(u_char *buf{nterm,out},
	u_int buflen{nbytes});

int{STD_RET} __semctl(int semid, int semnum, int cmd, union semun *arg); 

int{erret,STD_RET} __syscall(void);

int{erret,STD_RET} __sysctl(int *name, u_int namelen,
	void *old, size_t *oldlenp, 
	void *new, size_t newlen);

int{erret,STD_RET} acct{adminop,rootop,fileop}(char *path{path,nterm});

int{fdret,VAL_RET} accept(int s{fd, socket}, 
	caddr_t name{addr, copy(name, *anamelen), out}, 
	int *anamelen{nbytes});

int{erret,VAL_RET} old_accept(int s{fd, socket}, 
	caddr_t name{addr, copy(name, *anamelen), out}, 
	int *anamelen{nbytes});

int{erret,STD_RET} access{fileop}(char *path{path,nterm}, 
	int flags{flags,mode});

int{erret,STD_RET} adjtime{adminop,rootop}(struct timeval
		*delta,
		struct timeval *olddelta{out});

int{erret,STD_RET} aio_return{aioop}(struct aiocb *aiocbp);

int{erret,STD_RET} aio_suspend{aioop}(struct aiocb * const * aiocbp,
	int nent, const struct timespec *timeout);

int{erret,STD_RET} aio_cancel{aioop,fdop}(int fd{fd}, struct aiocb *aiocbp);

int{erret,STD_RET} aio_error{aioop}(struct aiocb *aiocbp);

int{erret,STD_RET} aio_read{aioop}(struct aiocb *aiocbp);

int{erret,STD_RET} aio_waitcomplete{aioop}(struct aiocb **aiocbp,
	struct timespec *timeout);

int{erret,STD_RET} aio_write{aioop}(struct aiocb *aiocbp);

int{erret,STD_RET} bind(int s{fd,socket},
	const struct sockaddr *name{addr,copy(name, namelen)},
	int namelen{nbytes});

int{erret,STD_RET} old_break{memop}(char *nsize); 

int{erret,STD_RET} chdir{fileop}(char *path{path,nterm});

int{erret,STD_RET} chflags{fileop}(char *path{path, nterm}, 
	int flags);

int{erret,STD_RET} chmod{fileop, auditop}(char *path{path,nterm},
		int mode{mode, newmode});

int{erret,STD_RET} chown{fileop, auditop, if(uid > 0) rootop}(
	char *path{path,nterm},
	int uid{uid}, 		
	int gid{gid}
);

int{erret,STD_RET} chroot{adminop, rootop, fileop, auditop}(char *path{path, nterm});

int{erret,STD_RET} clock_getres(clockid_t clock_id,
	struct timespec *tp{out});

int{erret,STD_RET} clock_gettime(clockid_t clock_id,
	struct timespec *tp{out});

int{erret,STD_RET} clock_settime(clockid_t clock_id,
	const struct timespec *tp);

int{erret,STD_RET} close{fdop, auditop}(int fd{fd});

int{erret,STD_RET} connect{fdop}(int s{fd,socket},
	const struct sockaddr *name{addr, copy(name,namelen)},
	int namelen{nbytes});

int{fdret,VAL_RET} old_creat{fileop,fdretop,creatop}(char *path{path, nterm}, 
	int mode{mode});

int{fdret,VAL_RET} dup{fdop,fdretop}(u_int fd{fd});

int{fdret,VAL_RET} dup2{fdop,fdretop}(u_int from{fd},
	u_int to{ofd, newfd});

int{erret,STD_RET} execve{fileop, execop, auditop} (
	char *fname{path,nterm},
	char **argv{argv, *nterm}, 
	char **envv{envp, *nterm}
);

void exit(int rval{code});

int{erret,STD_RET} extattrctl{extattrop,fileop}(const char *path{path,nterm},
	int cmd, const char *attrname{attrname,nterm}, char *arg);

int{erret,STD_RET} extattr_set_file{extattrop,fileop}(const char *path{path,nterm},
	const char *attrname{attrname,nterm},
	struct iovec *iovp{copy(iovp,iovcnt),iovcnt},
	unsigned iovcnt);

int{erret,STD_RET} extattr_get_file{extattrop,fileop}(const char *path{path,nterm},
	const char *attrname{attrname,nterm},
	struct iovec *iovp{copy(iovp,iovcnt),iovcnt,out},
	unsigned iovcnt);

int{erret,STD_RET} extattr_delete_file{extattrop,fileop}(const char *path{path,nterm},
	const char *attrname{attrname,nterm});

int{erret,STD_RET} fchdir{fdop}(int fd{fd});

int{erret,STD_RET} fchflags{fdop}(int fd{fd}, int flags);

int{erret,STD_RET} fchmod{fdop, auditop}(int fd{fd}, int mode{mode, newmode});

int{erret,STD_RET} fchown{fdop, auditop, if (uid > 0) rootop} (	
	int fd{fd}, 
	int uid{uid}, 
	int gid{gid}
);

int{if(cmd == F_DUPFD) fdret, if(cmd == F_GETOWN) pidret, erret,VAL_RET} 
fcntl{fdop, if(cmd==F_DUPFD)fdretop}(
	int fd{fd}, 
	int cmd{cmd}, 
	int arg
);

int{erret,STD_RET} fhopen{handleop}(const struct fhandle *u_fhp{handle,fh},
	int flags);

int{erret,STD_RET} fhstat{handleop}(const struct fhandle *u_fhp{handle,fh},
	struct stat *sb{out});

int{erret,STD_RET} fhstatfs{handleop}(const struct fhandle *u_fhp{handle,fh},
	struct statfs *buf{out});

int{erret,STD_RET} flock{fdop}(int fd{fd}, 
	int how);

pid_t{pidret,STD_RET} fork{forkop, auditop}(void);

long{limret,VAL_RET} fpathconf{fdop}(int fd{fd}, 
	int name);

int{erret,STD_RET} fstat{fdop}(int fd{fd}, 
	struct stat *sb{out});

int{erret,STD_RET} old_fstat{fdop}(int fd{fd}, 
	struct ostat *sb{out});

/*
 * was: "struct statfs *buf{out,copy(buf->f_mntoname,MNAMELEN),...}"
 * f_mntonname and f_mntfromname are part of the structure, and do not
 * need to be copied seperately.
 */
int{erret,STD_RET} fstatfs{fdop}(int fd{fd}, 
	struct statfs *buf{out});

int{erret,STD_RET} fsync{fdop}(int fd{fd});

int{erret,STD_RET} ftruncate{fdop}(int fd{fd},
	int pad, off_t length{nbytes});

int{erret,STD_RET} old_ftruncate{fdop}(int fd{fd},
	long length{nbytes});

int{erret,STD_RET} futimes{fdop}(int fd{fd},
	struct timeval *tptr);

int{erret,STD_RET} getdirentries{direntop, getop, fdop}(int fd{fd}, 
	char *buf{copy(buf,count),buf,out}, u_int count{nbytes}, 
	long *basep{out});

int{erret,STD_RET} old_getdirentries{direntop, getop, fdop}(int fd{fd}, 
	char *buf{copy(buf,count),buf,out}, u_int count{nbytes}, 
	long *basep{out});

int{erret,STD_RET} getdomainname{getop}(char *domainname{nterm,out}, 
	int len);

int{sizeret,NOERR_RET} getdtablesize{getop}(void);

gid_t{gidret,NOERR_RET} getegid{getop}(void);

uid_t{uidret,NOERR_RET} geteuid{getop}(void);

int{erret,STD_RET} getfh{getop,adminop,rootop,fileop}(char *fname{nterm, path}, 
	fhandle_t *fhp{out});

/*
 * was: "struct statfs *buf{out,copy(buf->f_mntoname,MNAMELEN),...}"
 * f_mntonname and f_mntfromname are part of the structure, and do not
 * need to be copied seperately.
 */
int{erret,VAL_RET} getfsstat{getop}(struct statfs 
	*buf{ out },
	long bufsize, int flags);

gid_t{gidret,NOERR_RET} getgid{getop}(void);

int{erret,VAL_RET} getgroups{getop}(u_int gidsetsize, 
	gid_t *gidset{out});

long{nidret,NOERR_RET} old_gethostid{getop}(void);

int{erret,STD_RET} old_gethostname{getop}(char *hostname{nterm,out}, 
	u_int len{nbytes});

int{erret,STD_RET} getitimer{getop}(u_int which, 
	struct itimerval *itv{out});

int{erret,STD_RET} old_getkerninfo{getop}(int oper, 
	char *where{nterm}, int *size, int arg);

int{erret,VAL_RET} getlogin{getop}(char *namebuf{nterm, out}, 
	u_int namelen{nbytes});

int{sizeret,STD_RET} old_getpagesize{getop}(void);

int{erret,STD_RET} getpeername{getop, fdop}(int fdes{fd,socket}, 
	caddr_t asa{addr, buf,out}, int *alen{nbytes});

int{erret,STD_RET} old_getpeername{getop, fdop}(int fdes{fd,socket}, 
	caddr_t asa{addr, buf,out}, int *alen{nbytes});

int{pidret,STD_RET} getpgid{getop, pidop}(pid_t pid);

int{pidret,VAL_RET} getpgrp{getop, pidop}(void);

pid_t{pidret,VAL_RET} getpid{getop, pidop}(void);

pid_t{pidret,VAL_RET} getppid{getop, pidop}(void);

int{prioid,NOERR_RET} getpriority{getop, prioop}(int which, int who);

int{erret,STD_RET} getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);

int{erret,STD_RET} getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);

int{erret,STD_RET} getrlimit{getop}(u_int which, 
	struct orlimit *rlp{out});

int{erret,STD_RET} old_getrlimit{getop}(u_int which, 
	struct ogetrlimit *rlp{out});

int{erret,STD_RET} getrusage{getop}(int who, 
	struct rusage *rusage{out});

int{sidret,VAL_RET} getsid{pidop}(pid_t pid);

int{erret,STD_RET} getsockname{getop, fdop}(int fdes{fd,socket}, 
	caddr_t asa{addr, buf,out}, int *alen{nbytes});

int{erret,STD_RET} old_getsockname{getop, fdop}(int fdec{fd,socket}, 
	caddr_t asa{addr, buf,out}, int *alen{nbytes});

int{erret,STD_RET} getsockopt{getop, fdop}(int s{fd,socket}, 
	int level, int name, 
	caddr_t val{addr, buf,out}, int *avalsize{nbytes});

int{erret,STD_RET} gettimeofday{getop}(struct timeval *tp{out}, 
	struct timezone *tzp);

uid_t{uidret,NOERR_RET} getuid{getop}(void);

int{erret,STD_RET} ioctl{fdop}(int fd{fd}, 
	u_long com, caddr_t data);

int{boolret,VAL_RET} issetugid(void);

int{erret,STD_RET} jail{adminop,rootop,jailop}(struct jail *jail);

int{erret,STD_RET} kevent{keventop,fdop}(int fd{fd},
	const struct kevent *changelist{copy(changelist,nchanges),changelist},
	int nchanges,
	struct kevent *eventlist{out,copy(eventlist,nevents),eventlist},
	int nevents,
	const struct timespec *timeout);

int{erret,STD_RET} kill{pidop}(int pid{pid}, 
	int signum{signum});

int{erret,STD_RET} old_killpg{pidop}(int pgid{gid}, 
	int signum);

int{erret,STD_RET} kldfind{modop}(const char *file{path,nterm});

int{erret,STD_RET} kldfirstmod{modop}(int fileid);

int{erret,STD_RET} kldload{modop,adminop,rootop}(const char *file{path,nterm});

int{erret,STD_RET} kldnext{modop}(int fileid);

int{erret,STD_RET} kldstat{modop}(int fileid,
	struct kld_file_stat* stat{out});

int{erret,STD_RET} kldsym{modop}(int fileid, int cmd, void *data);

int{erret,STD_RET} kldunload{modop,adminop,rootop}(int fileid);

int{erret,STD_RET} kqueue{keventop}(void);

int{erret,STD_RET} ktrace{adminop,rootop,fileop}(const char *fname{path,nterm},
	int ops, int facs, int pid{pid});

int{erret,STD_RET} lchmod{fileop, auditop}(char *path{path,nterm},
	int mode{mode, newmode});

int{erret,STD_RET} lchown{fileop, auditop, if(uid > 0) rootop} (
	char *path{path,nterm}, 
	int uid{uid}, 
	int gid{gid}
);

int{erret,STD_RET} lfs_bmapv(fsid_t *fsidp, struct block_info *blkiov,
	int blkcnt);

int{erret,STD_RET} lfs_markv(fsid_t *fsidp, struct block_info *blkiov, 
	int blkcnt);

int{erret,STD_RET} lfs_segclean(fsid_t *fsidp, u_long segment);

int{erret,STD_RET} lfs_segwait(fsid_t *fsidp, struct timeval *tv);

int{erret,STD_RET} link{fileop}(char *path{path,nterm}, 
	char *link{destpath,nterm});

int{erret,STD_RET} lio_listio(int mode,
	struct aiocb * const *acb_list, int nent, struct sigevent *sig);

int{erret,STD_RET} listen{fdop}(int s{fd,socket}, int backlog);

off_t{locret,VAL_RET} lseek{fdop}(int fd{fd}, 
	int pad, off_t offset{offset}, int whence{whence});

long{locret,VAL_RET} old_lseek{fdop}(int fd{fd}, 
	long offset{offset}, int whence{whence});

int{erret,STD_RET} lstat{fileop}(char *path{path,nterm}, struct stat *ub);

int{erret,STD_RET} old_lstat{fileop}(char *path{path,nterm}, struct ostat *ub);

int{erret,STD_RET} lutimes{fileop}(char *path{path,nterm},
	struct timeval *tptr); 

int{erret,STD_RET} madvise{memop}(caddr_t addr, 
	size_t len{nbytes}, int behav); 

int{erret,STD_RET} mincore{memop}(caddr_t addr, 
	size_t len{nbytes}, char *vec{nterm,out}); 

int{erret,STD_RET} minherit{memop}(caddr_t addr,
	size_t len, int inherit);

int{erret,STD_RET} mkdir{fileop, auditop} (
	char *path{path,nterm}, 
	int mode{mode}
);

int{erret,STD_RET} mkfifo{fileop, auditop} (
	char *path{path,nterm}, 
	int mode{mode}
);

int{erret,STD_RET} mknod{adminop,rootop,fileop, auditop} (
	char *path{path,nterm}, 
	int mode{mode},
	int dev{dev}
);

int{erret,STD_RET} mlock{memop}(caddr_t addr, size_t len); 

int{erret,STD_RET} mlockall{memop}(int how);

caddr_t{locret,VAL_RET} mmap{memop,fdop}(caddr_t addr, 
	size_t len{nbytes}, int prot{prot}, int flags, 
	int fd{fd}, long pad, off_t pos{offset}); 

int{locret,VAL_RET} old_mmap{memop,fdop}(caddr_t addr, 
	int len{nbytes}, int prot{prot}, int flags, 
	int fd{fd}, long pos{offset}); 

int{erret,STD_RET} modfind{modop}(const char *name{name,nterm});

int{erret,STD_RET} modfnext{modop}(int modid);

int{erret,STD_RET} modnext{modop}(int modid);

int{erret,STD_RET} modstat{modop}(int modid,
	struct module_stat* stat{out});

int{erret,STD_RET} mount{adminop,rootop,fileop}(int mttype, char
	*path{path,nterm}, 
	int flags, caddr_t data);

int{erret,STD_RET} mprotect{memop}(caddr_t addr, 
	size_t len{nbytes}, int prot); 

/*
 * Fortunately for my sanity, msgctl never uses buf->msg_first or
 * buf->msg_last... So the values they point to do not need to be copied
 * into system space (Or copyied out of system space.).
 * 
 * Also, AFAIK (Calvin, can you check this?) buf->msg_qnum has no relationship
 * to the size of buf.
 */
#if 0
/* Was: */
int{erret,STD_RET} msgctl{msgop}(int msqid{msgq}, 
	int cmd, 
	struct msqid_ds *buf{buf,out,
		rcopy(buf, buf->msg_qnum),
		copy(buf->msg_first, buf->msg_first->msg_ts),
		copy(buf->msg_last, buf->msg_last->msg_ts)} );
#else
/* Is: */
int{erret,STD_RET} msgctl{msgop}(int msqid{msgq}, 
	int cmd, 
	struct msqid_ds *buf{buf,out} );
#endif

int{msgret,VAL_RET} msgget{msgop}(key_t key, int msgflg); 

int{sizeret,VAL_RET} msgrcv{msgop}(int msqid, void *msgp{out}, 
	size_t msgsz, long msgtyp, int msgflg); 

int{erret,STD_RET} msgsnd{msgop}(int msqid, void *msgp, 
	size_t msgsz, int msgflg); 

int{erret,STD_RET} msgsys{msgop}(int which, 
	int a2, int a3, int a4, int a5, int a6); 

int{erret,STD_RET} msync{memop}(caddr_t addr, 
	size_t len{nbytes}, int flags); 

int{erret,STD_RET} munlock{memop}(caddr_t addr, size_t len{nbytes}); 

int{erret,STD_RET} munlockall{memop}(void);

int{erret,STD_RET} munmap{memop}(caddr_t addr, size_t len{nbytes}); 

int{erret,STD_RET} nanosleep(const struct timespec *rqtp,
	struct timespec *rmtp{out});

int{erret,STD_RET} nfstat{fdop}(int fd{fd}, struct nstat *sb{out});

int{erret,STD_RET} nfssvc(int flags, caddr_t argp);

int{erret,STD_RET} nlstat{fileop}(char *path{path,nterm},
	struct nstat *ub{out});

int{erret,STD_RET} nstat{fileop}(char *path{path,nterm},
	struct nstat *ub{out});

int{erret,STD_RET} ntp_adjtime(struct timex *tp); 

int{fdret,VAL_RET} open{fileop, fdretop, auditop, if (flags & O_CREAT)creatop} (
	char *path{path,nterm}, 
	int flags{flags},
	int mode{mode}
);

int{fdret,STD_RET} openbsd_poll{fdvecop}(
        struct pollfd *fds{fds,rcopy(fds,nfds)}, u_int nfds{nfds}, int timeout);

int{erret,STD_RET} ovadvise(int anom); 
 
int{limret,VAL_RET} pathconf{fileop}(char *path{nterm,path}, 
	int name); 

int *{erret, fdret, STD_RET} pipe(void);

int{erret,STD_RET} poll{fdvecop}(
	struct pollfd *fds{fds,rcopy(fds,nfds)}, u_int nfds{nfds}, int timeout);

int{erret,STD_RET} profil(caddr_t samples{buf}, 
	u_int size{nbytes}, u_int offset{offset}, u_int scale); 

int{erret,STD_RET} ptrace{adminop,rootop}(int req, pid_t pid{pid}, caddr_t addr, int data);

int{erret,STD_RET} old_quota(void);

int{erret,STD_RET} quotactl{adminop,rootop,fileop}(char
	*path{path,nterm}, int cmd,
	int uid, caddr_t arg);

#if WR_USE_DATABUF
int{sizeret,VAL_RET} read{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out,type(databuf_t)}, u_int nbyte{nbytes});
#else
int{sizeret,VAL_RET} read{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out}, u_int nbyte{nbytes});
#endif

int{erret,STD_RET} readlink{fileop}(char *path{path,nterm},
	char *buf{copy(buf,count),buf,out}, int count{nbytes});

int{sizeret,VAL_RET} readv{fdop}(int fd{fd}, struct iovec *iovp{
	      iovp,
	      copy(iovp->iov_base, iovp->iov_len),  
	      rcopy(iovp, iovcnt), out},
	      u_int iovcnt{count});

int{STD_RET} reboot{adminop,rootop}(int opt);

int{sizeret,VAL_RET} old_ecv{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,out}, int len{nbytes}, int flags); 

#if WR_USE_DATABUF
int{sizeret,VAL_RET} recvfrom{fdop}(
	int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf,len),out}, 
	size_t len{nbytes}, 
	int flags, 
	caddr_t from{sockadr, addr, copy(from, *fromlenaddr), out}, 
	int *fromlenaddr{adrlen}); 
#else
int{sizeret,VAL_RET} recvfrom{fdop}(
	int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf,len),out,type(databuf_t)}, 
	size_t len{nbytes}, 
	int flags, 
	caddr_t from{sockadr, addr, copy(from, *fromlenaddr), out}, 
	int *fromlenaddr{adrlen}); 
#endif

int{sizeret,VAL_RET} old_recvfrom{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,out}, size_t len{nbytes}, 
	int flags, caddr_t from, int *fromlenaddr); 

int{sizeret,VAL_RET} recvmsg{fdop}(int s{fd, socket}, 
	struct msghdr *msg
      	{
        msghdr,
	copy(msg->msg_name, msg->msg_namelen),
	rcopy(msg->msg_iov, msg->msg_iovlen),
	copy(msg->msg_iov->iov_base, msg->msg_iov->iov_len),
	copy(msg->msg_control, msg->msg_controllen),
	out
      	},
      	int flags);

int{sizeret,VAL_RET} old_recvmsg{fdop}(int s{fd, socket}, 
	struct omsghdr *msg
      	{
	copy(msg->msg_name, msg->msg_namelen),
	rcopy(msg->msg_iov, msg->msg_iovlen),
	copy(msg->msg_iov->iov_base, msg->msg_iov->iov_len),
	copy(msg->msg_accrights, msg->msg_accrightslen),
	out
      	},
      	int flags);

int{erret,STD_RET} rename{fileop, auditop} (
	char
	*from{path,nterm},
	char *to{destpath,nterm}
);

int{erret,STD_RET} revoke{fileop}(char *path{nterm, path}); 

int{pidret,VAL_RET} rfork{forkop}(int flags);

int{erret,STD_RET} rmdir{fileop}(char *path{path,nterm});

int{erret,STD_RET} rtprio(int function, pid_t pid{pid}, struct rtprio *rtp);

int{locret,VAL_RET} sbrk{memop}(int incr); 

int{erret,STD_RET} sched_get_priority_max{schedop}(int policy);

int{erret,STD_RET} sched_get_priority_min{schedop}(int policy);

int{erret,STD_RET} sched_getparam{schedop}(pid_t pid,
	struct sched_param *param{out});

int{erret,STD_RET} sched_getscheduler{schedop}(pid_t pid);

int{erret,STD_RET} sched_rr_get_interval{schedop}(pid_t pid,
	struct timespec *interval);

int{erret,STD_RET} sched_setparam{schedop}(pid_t pid,
	const struct sched_param *param);

int{erret,STD_RET} sched_setscheduler{schedop}(pid_t pid,
	int policy, const struct sched_param *param);

int{erret,STD_RET} sched_yield{schedop}(void);

int{sizeret,VAL_RET} select{fdvecop}(int nd{nd}, 
	fd_set *input{input,rcopy(input,1)},
	fd_set *output{output,rcopy(output,1)},
	fd_set *except{except,rcopy(except,1)},
	struct timeval *tv{tv,rcopy(tv,1)}); 

int{erret,STD_RET} semconfig{semop}(int flag); 

int{semidret,VAL_RET} semget{semop}(key_t key, int nsems, int semflg); 

int{erret,STD_RET} semop{semop}(int semid, struct sembuf *sops, u_int nsops); 

int{erret,STD_RET} semsys{semop}(int which, int a2, int a3, int a4, int a5);

int{sizeret,VAL_RET} old_send{fdop}(int s{fd, socket}, 
	caddr_t buf{buf}, int len{nbytes}, int flags); 

int{erret,STD_RET} sendfile{fdop}(int fd{fd},
	int s, off_t offset, size_t nbytes, struct sf_hdtr *hdtr,
	off_t *sbytes, int flags);

int{sizeret,VAL_RET} sendmsg{fdop}(int s{fd, socket}, 
	caddr_t msg{buf}, int flags); 

int{sizeret,VAL_RET} old_sendmsg{fdop}(int s{fd, socket}, 
	caddr_t msg{buf}, int flags); 

#if WR_USE_DATABUF
int{sizeret,VAL_RET} sendto{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf, len),type(databuf_t},
        size_t len{nbytes}, 
	int flags, caddr_t to{sockadr, addr, copy(to, tolen)}, int tolen{adrlen}); 
#else
int{sizeret,VAL_RET} sendto{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf, len)}, size_t len{nbytes}, 
	int flags, caddr_t to{sockadr, addr, copy(to, tolen)}, int tolen{adrlen}); 
#endif

int{erret,STD_RET} setdomainname{adminop,rootop, setop}(char *domainname{nterm}, 
	int len{nbytes}); 

int{erret,STD_RET} setegid{setop}(gid_t egid{egid}); 

int{erret,STD_RET} seteuid{setop}(uid_t euid{uid}); 

int{erret,STD_RET} setgid{setop}(gid_t gid{gid}); 

int{erret,STD_RET} setgroups{setop}(u_int gidsetsize, gid_t *gidset); 

int{erret,STD_RET} old_sethostid{adminop,rootop, setop}(long hostid); 

int{erret,STD_RET} old_sethostname{adminop,rootop, setop}(char *hostname{nterm}, 
	u_int len{nbytes}); 

int{erret,STD_RET} setitimer{setop}(u_int which, 
	struct itimerval *itv, struct itimerval *oitv); 

int{erret,STD_RET} setlogin{rootop, setop}(char *namebuf{nterm}); 

int{erret,STD_RET} setpgid{setop}(int pid{pid}, 
	int pgid{gid}); 

int{erret,STD_RET} setpriority{setop, prioop}(int which, 
	int who, int prio);  

int{erret,STD_RET} setregid{setop, if(rgid != egid) adminop, if (rgid != egid) rootop}(int rgid{rgid}, 
	int egid{egid}); 

int{erret,STD_RET} setresgid{setop}(gid_t rgid, gid_t egid, gid_t sgid);

int{erret,STD_RET} setresuid{setop}(uid_t ruid, uid_t euid, uid_t suid);

int{erret,STD_RET} setreuid{setop, if(ruid != euid) adminop, if(ruid != euid) rootop}(int ruid{ruid}, 
	int euid{euid}); 

int{erret,STD_RET} setrlimit{setop}(u_int which, 
	struct orlimit *rlp); 

int{erret,STD_RET} old_setrlimit{setop}(u_int which, 
	struct ogetrlimit *rlp); 

int{gidret,VAL_RET} setsid{setop}(void); 

int{erret,STD_RET} setsockopt{fdop, setop}(int s{fd, socket}, 
	int level, int name, caddr_t val, int valsize);

int{erret,STD_RET} settimeofday{adminop,rootop}(struct timeval *tv, 
	struct timezone *tzp);

int{erret,STD_RET} setuid{setop}(uid_t uid{uid}); 

int{locret,VAL_RET} shmat{shmop}(int shmid, void *shmaddr, int shmflg); 

int{erret,STD_RET} shmctl{shmop}(int shmid, int cmd, 
	struct shmid_ds *buf{buf}); 

int{erret,STD_RET} shmdt{shmop}(void *shmaddr); 

int{shmret,VAL_RET} shmget{shmop}(key_t key, int size, int shmflg); 

int{shmret,VAL_RET} shmsys{shmop}(int which, int a2, int a3, int a4);

int{erret,STD_RET} shutdown{fdop}(int s{fd, socket}, int how); 

int{erret,STD_RET} old_sigaction{sigop}(int signum, 
	struct osigaction *nsa, struct osigaction *osa{out}); 

int{erret,STD_RET} sigaction{sigop}(int sig, 
	struct sigaction *act, struct sigaction *oact{out}); 

int{erret,STD_RET} sigaltstack{sigop}(stack_t *ss{copy(ss->ss_sp, ss->ss_size)},
	stack_t *oss{copy(oss->ss_sp, oss->ss_size),out});

int{erret,STD_RET} old_sigblock{sigop}(int mask);

int{erret,STD_RET} old_sigpending{sigop}(void); 

int{erret,STD_RET} sigpending{sigop}(sigset_t *sigset); 

int{erret,STD_RET} old_sigprocmask{sigop}(int how, osigset_t mask); 

int{erret,STD_RET} sigprocmask{sigop}(int how,
	const sigset_t *sigset, sigset_t *osigset{out}); 

int{erret,STD_RET} old_sigreturn{sigop}(struct osigcontext *sigcntxp); 

int{erret,STD_RET} sigreturn{sigop}(ucontext_t *sigcntxp); 

int{erret,STD_RET} old_sigsetmask{sigop}(int mask);

int{erret,STD_RET} old_sigstack{sigop}(struct sigstack *nss,
	struct sigstack *oss{out});

int{erret,STD_RET} old_sigsuspend{sigop}(osigset_t mask); 

int{erret,STD_RET} sigsuspend{sigop}(const sigset_t *sigmask);

int{erret,STD_RET} old_sigvec{sigop}(int signum,
	struct sigvec *nsv, struct sigvec *osv{out});

int{fdret,VAL_RET} socket{fdretop, if (domain == PF_INET) isockop, 
	if (domain == PF_LOCAL) usockop} 
	(int domain{domain}, int socktype{socktype}, int protocol{proto});

int{erret,STD_RET} socketpair{if (domain == PF_INET) isockop,
	if (domain == PF_LOCAL) usockop}
	(int domain{domain}, int socktype, int protocol, int *rsv); 

int{erret,STD_RET} sstk(int incr); 

int{erret,STD_RET} stat{fileop}(char *path{path,nterm}, struct stat *ub);

int{erret,STD_RET} old_stat{fileop}(char *path{path,nterm}, struct ostat *ub);

int{erret,STD_RET} statfs{fileop}(char
	*path{path,nterm},
	struct statfs *buf{buf, out, 
	     copy(buf->f_mntonname, MNAMELEN),
	     copy(buf->f_mntfromname, MNAMELEN)});

int{erret,STD_RET} swapon{adminop,rootop,fileop}(char *name);

int{erret,STD_RET} symlink{fileop}(char *path{path,nterm},
	char *link{destpath,nterm});

int{erret,STD_RET} sync{fileop}(void); 

int{erret,STD_RET} sysarch(int oper, char *parms); 

int{erret,STD_RET} syscall(int dummy);

int{erret,STD_RET} thr_sleep{threadop}(const struct timespec *timeout);

int{erret,STD_RET} thr_wakeup{threadop}(pid_t pid);

int{erret,STD_RET} truncate{fileop}(char *path{path, nterm}, 
	int pad, off_t length); 

int{erret,STD_RET} old_truncate{fileop}(char *path{path, nterm}, 
	long length); 

int{erret,STD_RET} umask(int newmask{modemask}); 

int{erret,STD_RET} uname(struct utsname *name
	{copy(name->sysname, SYS_NMLN),
	 copy(name->nodename, SYS_NMLN),
	 copy(name->release, SYS_NMLN),
	 copy(name->version, SYS_NMLN),
	 copy(name->machine, SYS_NMLN)} ); 

int{erret,STD_RET} undelete{fileop}(char *path{path,nterm});

int{erret,STD_RET} unlink{fileop}(char *path{path,nterm});

int{erret,STD_RET} unmount{adminop,rootop,fileop}(char *path{path,nterm}, int flags);

int{erret,STD_RET} utimes{fileop}(char *path{path,nterm}, struct timeval *tptr); 

int{erret,STD_RET} utrace(const void *addr, size_t len); 

int{pidret,VAL_RET} vfork{forkop}(void);

int{pidret,VAL_RET} old_wait(void); 

int{pidret,VAL_RET} wait4(int pid, int *status, int options, 
	struct rusage *rusage{out}); 

#if WR_USE_DATABUF
int{sizeret,VAL_RET} write{fdop}(int fd{fd},
	char *buf{copy(buf,nbyte), iobuf,type(databuf_t)}, 
        u_int nbyte{nbytes});
#else
int{sizeret,VAL_RET} write{fdop}(int fd{fd},
	char *buf{copy(buf,nbyte), iobuf}, 
        u_int nbyte{nbytes});
#endif

int{sizeret,VAL_RET} writev{fdop}(int fd{fd},
	struct iovec *iovp{iovp, copy(iovp->iov_base,iovp->iov_len),
			   rcopy(iovp, iovcnt)}, 
	u_int iovcnt);

int{erret,STD_RET} yield(void);

}

