/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: libwr.h,v 1.65 2001/06/23 00:09:27 stevek Exp $
 */


#define WR_WRAPC

#include "wrerrno.h"

#if 0
struct auq_ctrl {
        int bufsize;
        int flag;               /* block on full, return error on full */
        int auth;
};
typedef struct auq_ctrl auq_ctrl_t;
#endif

#if !WR_SUPPRESS_WDL_STUFF
includes {
#endif

#ifndef _WR_DEFS_INCLUDED
#include "wr_conf.h"
#endif

#ifndef __WSI_TYPES_H
#include "wsi_types.h"
#endif

#ifndef __WSS_FILE_H
#include "wss_file.h"
#endif

#ifndef __WSI_FILEUTIL_H
#include "wsi_fileutil.h"
#endif
 
#ifndef __WSS_PROCESS_H
#include "wss_process.h"
#endif
    
#ifndef __WSS_MUTEX_H
#include "wss_mutex.h"
#endif

#if !WR_SUPPRESS_WDL_STUFF

};
#endif

#if !WR_SUPPRESS_WDL_STUFF
uses {
#endif

#ifdef __cplusplus
extern "C" {
#endif

    int  LINKAGE  wr_mycontext(void);

#define WR_CONTEXT_ARG int __context
#define WR_CONTEXT     (wr_mycontext())

    int  LINKAGE  wr_strcmp    (const pd_char_t *s1, const pd_char_t *s2);
    int  LINKAGE  wr_strncmp   (const pd_char_t *s1, const pd_char_t *s2, int len);
    int  LINKAGE  wr_strlen    (const pd_char_t *str);
    pd_char_t LINKAGE *wr_abspath   (const pd_char_t *path);
    pd_char_t LINKAGE *wr_strcpy    (pd_char_t *dst, const pd_char_t *src);
    pd_char_t LINKAGE *wr_strncpy   (pd_char_t *dst, const pd_char_t *src, size_t n);
    pd_char_t LINKAGE *wr_strchr    (const pd_char_t *str, int c); 
    pd_char_t LINKAGE *wr_strrchr   (const pd_char_t *str, int c);
    pd_char_t LINKAGE *wr_strcat    (pd_char_t *s, const pd_char_t *append);
    pd_char_t LINKAGE *wr_strncat   (pd_char_t *s, const pd_char_t *append, size_t count);
    pd_char_t LINKAGE *wr_downcase  (const pd_char_t *str);
    pd_char_t LINKAGE *wr_upcase    (const pd_char_t *str);
#ifdef WR_WINNT
    pd_char_t LINKAGE *wr_strunicode(const char *str, int *size);
    char       LINKAGE *wr_strascii  (const pd_char_t *str, int *size);
    void LINKAGE  wr_seterror  (int code);
    pd_char_t LINKAGE *wr_time_get_current_string ( void );
#endif /* WR_WINNT */

    int  LINKAGE  wr_atoi      (const pd_char_t *s);
    void LINKAGE  wr_bcopy     (const void *src, void *dst, size_t len);
    void LINKAGE  wr_bzero     (void *src, size_t len);
    int  LINKAGE  wr_printf    (const pd_char_t*, ...);
    int  LINKAGE  wr_sprintf   (pd_char_t *s, const pd_char_t*, ...);
    int  LINKAGE  wr_log       (const pd_char_t *format, ...);
    void LINKAGE  wr_audit     (unsigned long types,
                                   const void* buffer, unsigned long length);
    int  LINKAGE  wr_audit_printf (unsigned long types,
                                   const pd_char_t *, ...);

    int  LINKAGE  wr_copyinstr (void *uaddr, void *kaddr, size_t len, unsigned int *done);
    int  LINKAGE  wr_copyin    (void *uaddr, void *kaddr, size_t len);
    int  LINKAGE  wr_copyout   (void *kaddr, void *uaddr, size_t len);
    int  LINKAGE  wr_resolve_event_space(pd_char_t *name);
    int  LINKAGE  wr_dispatch( int space, int code, wss_proc_t *wrp,
                               int flags, int arg_count, ... );

    long  LINKAGE  wr_get_curtime (long *usec);
    pd_char_t  LINKAGE *wr_get_hostname ();


#if defined(MALLOC_DEBUG)
    pd_char_t LINKAGE *wr_strdup_debug(const pd_char_t *str, pd_char_t *file, int line);
    #define wr_strdup(str) (wr_strdup_debug((str), __FILE__, __LINE__))
#else
    pd_char_t LINKAGE *wr_strdup(const pd_char_t *str);
#endif
#if defined(MALLOC_DEBUG)
    void LINKAGE *wr_malloc_debug(size_t size, pd_char_t *file, int line);
    #define wr_malloc(size) (wr_malloc_debug((size), __FILE__, __LINE__))
#else
    void LINKAGE *wr_malloc(size_t size);
#endif
    int LINKAGE wr_arg_replace(struct wss_proc *wrp,
                               int handle, int space, int code,
                               void *uap[], 
                               void *new_arg, int index, size_t size);

    void LINKAGE wr_free(void *ptr);
    void LINKAGE *wr_curproc_malloc(size_t size);
    void LINKAGE wr_curproc_free(void *ptr, size_t size);
    pd_char_t LINKAGE *wr_curproc_strdup(const pd_char_t *str);

    const pd_char_t *wr_syscall_name(int call);
    int LINKAGE wr_match(pd_char_t *src, pd_char_t *pat, 
                         pd_char_t **beg, pd_char_t **end);
    int LINKAGE wr_substitute(pd_char_t **src, pd_char_t *txt, pd_char_t *sub);
    int LINKAGE  wr_proc_add_wrapper ( wss_proc_t *wrp, pd_char_t *name );
    int LINKAGE  wr_proc_del_wrapper ( wss_proc_t *wrp, pd_char_t *name );

    pd_char_t LINKAGE *wr_proc_get_path( pid_t pid );
    pd_char_t LINKAGE *wr_proc_get_cwd( pid_t pid );

/*
    databuf_t LINKAGE buf_new();
    databuf_t LINKAGE buf_newstr(char *);
    databuf_t LINKAGE buf_newbuf(databuf_t);
    int LINKAGE buf_replace_data(databuf_t, const char *, int);
    int LINKAGE buf_replace_str(databuf_t, string);
    int LINKAGE buf_replace(databuf_t, databuf_t);
*/

    /* function for string manipulation */
    /* to be added later
    string LINKAGE str_add(string s1, string s2); 
    string LINKAGE str_cmp(string s1, string s2); 
    string LINKAGE str_assign(string s1, string s2);
     */

    /* function for databuf_t manipulation */
    /* to be added later 
       int LINKAGE buf_replace_data(databuf_t, const char *, int);
       int LINKAGE buf_replace_str (databuf_t, string);
       int LINKAGE buf_replace_cstr (databuf_t, string);
       int LINKAGE buf_append(databuf_t, databuf_t);
       int LINKAGE buf_append_data(databuf_t, const char *, int);
       int LINKAGE buf_append_str (databuf_t, string);

       Internal functions:
       int LINKAGE buf_size (databuf_t);
       char *LINKAGE buf_getdataptr(databuf_t, int);
     */

    int LINKAGE wrq_newq();
    int LINKAGE wrq_insert(int qid, pd_char_t *buf, int size);

#if !WR_SUPPRESS_STAT_STUFF
    int   LINKAGE wr_stat(const pd_char_t *, struct wr_stat *);
    int   LINKAGE wr_lstat(const pd_char_t *, struct wr_stat *);
    int   LINKAGE wr_fstat(unsigned int, struct wr_stat *);
    int   LINKAGE wr_fdtype(unsigned int);
    int   LINKAGE wr_fdfflag(unsigned int);
    pd_char_t *LINKAGE wr_get_family(const struct sockaddr *saddr, int len);  
    int  LINKAGE wr_get_addr(const struct sockaddr *saddr, int len,
                             char *buffer, int buffer_length);
    int  LINKAGE wr_getsockname(int, struct sockaddr *saddr, int *len);
    int  LINKAGE wr_getpeername(int, struct sockaddr *saddr, int *len);
    int  LINKAGE wr_ntohs(short);
#endif
    
#if !WR_SUPPRESS_FILE_STUFF
    void       LINKAGE  wr_close     ( wss_file_t file );
    int        LINKAGE  wr_mmap      ( wss_file_t file, size_t s, int mapmode,
                                       pd_offset_t pos );
    void       LINKAGE *wr_mmap_addr ( wss_file_t handle );
    wss_file_t LINKAGE  wr_open      ( const pd_char_t *filename,
                                       wss_file_mode_t mode);
    int        LINKAGE  wr_read      ( wss_file_t file,
                                       pd_byte_t *buffer, size_t len );
    pd_byte_t  LINKAGE *wr_read_alloc( wss_file_t file, size_t *len );
    int        LINKAGE  wr_seek      ( wss_file_t file, pd_offset_t offset,
                                       int whence );
    size_t     LINKAGE  wr_size      ( wss_file_t file );
    int        LINKAGE  wr_write     ( wss_file_t file, pd_byte_t *buffer,
                                       size_t len );

#ifdef WR_LIBRARY_SUPPRESS_WDL_STUFF
    void       LINKAGE  wr_file_close     ( WR_CONTEXT_ARG, wss_file_t file );
    int        LINKAGE  wr_file_mmap      ( WR_CONTEXT_ARG, wss_file_t file,
                                            size_t s, int mapmode,
                                            pd_offset_t pos );
    void       LINKAGE *wr_file_mmap_addr ( WR_CONTEXT_ARG, wss_file_t handle );
    wss_file_t LINKAGE  wr_file_open      ( WR_CONTEXT_ARG,
                                            const pd_char_t *filename,
                                            wss_file_mode_t mode);
    int        LINKAGE  wr_file_read      ( WR_CONTEXT_ARG, wss_file_t file,
                                            pd_byte_t *buffer, size_t len );
    pd_byte_t  LINKAGE *wr_file_read_alloc( WR_CONTEXT_ARG, wss_file_t file,
                                            size_t *len );
    int        LINKAGE  wr_file_seek      ( WR_CONTEXT_ARG, wss_file_t file,
                                            pd_offset_t offset, int whence );
    size_t     LINKAGE  wr_file_size      ( WR_CONTEXT_ARG, wss_file_t file );
    int        LINKAGE  wr_file_write     ( WR_CONTEXT_ARG, wss_file_t file,
                                            pd_byte_t *buffer, size_t len );

#define wr_close(f)        wr_file_close(WR_CONTEXT,(f))
#define wr_mmap(f,s,m,p)   wr_file_mmap(WR_CONTEXT,(f),(s),(m),(p))
#define wr_mmap_addr(f)    wr_file_mmap_addr(WR_CONTEXT,(f))
#define wr_open(f,m)       wr_file_open(WR_CONTEXT,(f),(m))
#define wr_read(f,b,l)     wr_file_read(WR_CONTEXT,(f),(b),(l))
#define wr_read_alloc(f,l) wr_file_read_alloc(WR_CONTEXT,(f),(l))
#define wr_seek(f,o,w)     wr_file_seek(WR_CONTEXT,(f),(o),(w))
#define wr_size(f)         wr_file_size(WR_CONTEXT,(f))
#define wr_write(f,b,l)    wr_file_write(WR_CONTEXT,(f),(b),(l))
#endif /* WR_LIBRARY_SUPPRESS_WDL_STUFF */
#endif

/* WR_MUTEX_FOO macros. */
#ifdef PD_THREADED
#define WR_MUTEX_PROTECT(var) WSS_MUTEX_PROTECT(var)

    void LINKAGE wr_mutex_init(pd_mutex_t *mutex, const pd_char_t *name);
    void LINKAGE wr_mutex_destroy(pd_mutex_t *mutex);
    void LINKAGE wr_mutex_enter(pd_mutex_t *mutex);
    void LINKAGE wr_mutex_exit(pd_mutex_t *mutex);

#define WR_MUTEX_INIT(var) \
    wr_mutex_init( &(var ## _lock), #var "_lock")
#define WR_MUTEX_DESTROY(var) wr_mutex_destroy( &(var ## _lock) )
#define WR_MUTEX_ENTER(var) wr_mutex_enter( &(var ## _lock) )
#define WR_MUTEX_EXIT(var) wr_mutex_exit( &(var ## _lock) )

#define WR_WIV_MUTEX_INIT(var) \
    wr_mutex_init(&WSS_GET_WIV_VAR(var ## _lock), #var "_lock")
#define WR_WIV_MUTEX_DESTROY(var) \
    wr_mutex_destroy(&WSS_GET_WIV_VAR(var ## _lock) )
#define WR_WIV_MUTEX_ENTER(var) \
    wr_mutex_enter(&WSS_GET_WIV_VAR(var ## _lock) )
#define WR_WIV_MUTEX_EXIT(var) \
    wr_mutex_exit(&WSS_GET_WIV_VAR(var ## _lock) )
#else /* PD_THREADED */
#define WR_MUTEX_PROTECT(var)
#define WR_MUTEX_INIT(var)
#define WR_MUTEX_DESTROY(var)
#define WR_MUTEX_ENTER(var)
#define WR_MUTEX_EXIT(var)

#define WR_WIV_MUTEX_INIT(var)
#define WR_WIV_MUTEX_DESTROY(var)
#define WR_WIV_MUTEX_ENTER(var)
#define WR_WIV_MUTEX_EXIT(var)

    int  LINKAGE wr_sleep(void *ident);
    void LINKAGE wr_wakeup(void *ident);
#endif /* PD_THREADED */

#ifdef __cplusplus
}
#endif

#if !WR_SUPPRESS_WDL_STUFF
};
#endif

