/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: linux.ch,v 1.26 2001/06/14 14:38:48 stevek Exp $
 */

includes {
#ifndef _WR_DEFS_INCLUDED
#include "wr_conf.h"
#endif

#ifndef __WSI_TYPES_H
#include "wsi_types.h"
#endif

#include "wr_errname.h"

#include <linux/kdev_t.h>
#include <linux/fcntl.h>
#include <linux/quota.h>
#include <linux/uio.h>
#include <linux/signal.h>
#include <linux/vfs.h>
#include <linux/socket.h>
#include <linux/utime.h>
    
    /* Stuff we need is defined in UTSname... but including
     * it includes all sorts of userspace cruft... so I reproduce large
     * parts.  ug.
     */
#define _UTSNAME_LENGTH 65
#define _UTSNAME_DOMAIN_LENGTH _UTSNAME_LENGTH
#define _UTSNAME_NODENAME_LENGTH _UTSNAME_LENGTH

    struct utsname {
        char sysname[_UTSNAME_LENGTH];
        char nodename[_UTSNAME_NODENAME_LENGTH];
        char release[_UTSNAME_LENGTH];
        char version[_UTSNAME_LENGTH];
        char machine[_UTSNAME_LENGTH];
        char domainname[_UTSNAME_DOMAIN_LENGTH];
    };

    struct mmap_arg_struct {
        unsigned long addr;
        unsigned long len;
        unsigned long prot;
        unsigned long flags;
        unsigned long fd;
        unsigned long offset;
    };

typedef int (*clone_func)(void *);

};

#include "wr_errname.h"

#define WR_LOCAL Linux

characterize WR_LOCAL {

int errnum{
       errno{
          vmap(WR_SUCC,         _),
          vmap(WR_FAIL,         _),
          vmap(WR_BADPERM,      EPERM),
          vmap(WR_NOFILE,       ENOENT),
          vmap(WR_NOPROC,       ESRCH),
          vmap(WR_IOERR,        EIO),
          vmap(WR_ARG2LONG,     E2BIG),
          vmap(WR_BADFD,        EBADF),
          vmap(WR_NOCHILD,      ECHILD),
          vmap(WR_BADACCES,     EACCES),
          vmap(WR_FILEEXIST,    EEXIST),
          vmap(WR_BADARG,       EINVAL),
          vmap(WR_FILEBSY,      ETXTBSY),
          vmap(WR_FILE2BIG,     EFBIG),
          vmap(WR_NOSPC,        ENOSPC),
          vmap(WR_FNAME2LONG,   ENAMETOOLONG)
       }
   };

/* 
 * syscalls that return 0 on success and -1 on failure
 */
#define STD_RET wrret{vmap(WR_SUCC,0), vmap(*,-1)}
/* 
 * syscalls that return some values on success and -1 on failure
 */
#define VAL_RET wrret{vmap(WR_SUCC,_), vmap(*,-1)}
/* 
 * syscalls that are always successful, and no return value is 
 * reserved to indicate an error. 
 */
#define NOERR_RET wrret     
/* 
 * syscalls that return some value on success and some other magic value
 * on failure.
 */
#define SIG_RET wrret{vmap(WR_SUCC,_), vmap(*,-1)}


int{fdret,erret,VAL_RET} accept{fdop,socketop}(int s{fd, socket},
                struct sockaddr *upeer_sockaddr
                    {addr, copy(upeer_sockaddr, *upeer_addrlen), out}, 
                int *upeer_addrlen{nbytes});

int{erret,STD_RET} access{fileop}(char *path{path,nterm}, 
	int flags{mode});

int{erret,STD_RET} acct{rootop,fileop}(char *path{path,nterm});

int{erret,STD_RET} adjtime{rootop}(struct timeval
		*delta,
		struct timeval *olddelta{out});

unsigned int{timeret,NOERR_RET} alarm{timeop}(unsigned int seconds{seconds});

int {erret,STD_RET} bdflush{rootop}(int func, long data);

int{erret,STD_RET} bind{fdop,socketop}(int s{fd,socket},
	const struct sockaddr *name{addr,copy(name, namelen)},
	int namelen{nbytes});

int {erret,STD_RET} brk(void *end_data_segment);

/* capget */

/* capset */

int{erret,STD_RET} chdir{fileop}(char *path{path,nterm});

int{erret,STD_RET} chmod{fileop, auditop}(char *path{path,nterm},
		int mode{mode, newmode});

int{erret,STD_RET} chown{fileop, auditop, if(uid > 0) rootop}(
	char *path{path,nterm},
	int uid{uid,ouid}, 		
	int gid{gid,ogid}
);

int{erret,STD_RET} chroot{rootop, fileop, auditop}(char *path{path, nterm});

int{pidret,VAL_RET} clone{forkop}(
        int     flags,
        void   *child_stack,
        void   *processor_registers);

int{erret,STD_RET} close{fdop, auditop}(int fd{fd});

int{erret,STD_RET} connect{fdop,socketop}(int fd{fd,socket},
	struct sockaddr *serv_addr{addr,copy(serv_addr,addrlen)},
	int addrlen{len});

int{fdret,VAL_RET} creat{fileop,fdretop,creatop}(
	const char *path{path,nterm},mode_t mode {mode});

caddr_t{addr_ret,VAL_RET} create_module{rootop,modop}(
	const char *name{name, nterm}, size_t size{size});

int{erret,STD_RET} delete_module{rootop,modop}(const char*name{name, nterm});

int{fdret,VAL_RET} dup{fdop,fdretop}(u_int fd{fd});

int{fdret,VAL_RET} dup2{fdop,fdretop}(u_int from{fd},
	u_int to{ofd,newfd});

int{erret,STD_RET} execve{fileop, execop, auditop} (
	char *fname{path,nterm},
	char **argv{argv, *nterm}, 
	char **envv{envp, *nterm},
        void *processor_registers
);

void exit(int rval{code});

int{erret,STD_RET} fchdir{fdop}(int fd{fd});

int{erret,STD_RET} fchmod{fdop, auditop}(int fd{fd}, int mode{mode, newmode});

int{erret,STD_RET} fchown{fdop, auditop, if (uid > 0) rootop} (	
	int fd{fd}, 
	int uid{uid,ouid}, 
	int gid{gid,ogid}
);

int{if(cmd == F_DUPFD) fdret, if(cmd == F_GETOWN) pidret, erret,VAL_RET} 
fcntl{fdop, if(cmd==F_DUPFD)fdretop}(
	int fd{fd}, 
	int cmd{cmd}, 
	int arg
);

int{erret,STD_RET} fdatasync{fdop}(int fd{fd});

int{erret,STD_RET} flock{fdop}(int fd{fd}, 
	int how);

pid_t{pidret,STD_RET} fork{forkop, auditop}(void *processor_registers);

int{erret,STD_RET} fstat{fdop}(int fd{fd}, 
	struct stat *sb{out,statbuf});

int{erret,STD_RET} fstatfs{fdop}(int fd{fd}, 
	struct statfs *buf{out});

int{erret,STD_RET} fsync{fdop}(int fd{fd});

int{erret,STD_RET} ftruncate{fdop,truncateop}(int fd{fd},
	int pad, off_t length{nbytes});

int{countret,NOERR_RET} get_kernel_syms(struct kernel_sym *table{table,out});

/* getcwd */

int{erret,STD_RET} getdents{fdop}(
	unsigned int fd{fd},
	struct dirent *dirp{dirp},
	unsigned int count{count});

gid_t{gidret,NOERR_RET} getegid{getop}(void);

uid_t{uidret,NOERR_RET} geteuid{getop}(void);

gid_t{gidret,NOERR_RET} getgid{getop}(void);

int{erret,VAL_RET} getgroups{getop}(u_int gidsetsize, 
	gid_t *gidset{out});

int{erret,STD_RET} getitimer{getop}(u_int which, 
	struct itimerval *itv{out});

gid_t{gidret,NOERR_RET} getpgid{getop}(void);

int{pidret,VAL_RET} getpgrp{getop, pidop}(void);

pid_t{pidret,VAL_RET} getpid{getop, pidop}(void);

pid_t{pidret,VAL_RET} getppid{getop, pidop}(void);

int{prioid,NOERR_RET} getpriority{getop, prioop}(int which, int who);

int{erret,STD_RET} getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid);

int{erret,STD_RET} getresuid(uid_t *ruid, uid_t *euid, uid_t *suid);

int{erret,STD_RET} getrlimit{getop}(int resource, struct rlimit *rlim{out});

int{erret,STD_RET} getrusage{getop}(int who, struct rusage *rusage{out});

int{pidret,VAL_RET} getsid{getop}(pid_t pid);

int{erret,STD_RET} getsockopt{fdop,socketop}(
        int             fd{fd},
        int             level{level},
        int             optname{optname},
        void      *     optval{value,out,copy(optval,*optlen)},
        int *           optlen{optlen,inout});

int{erret,STD_RET} gettimeofday{getop}(struct timeval *tv{out}, 
	struct timezone *tzp);

uid_t{uidret,NOERR_RET} getuid{getop}(void);

/* idle.  Reverse system call can only be called by idle process */

int{erret,STD_RET} init_module{rootop,modop}(
	const char *name{name,nterm},
	struct module *image{image});

int{erret,STD_RET} ioctl{fdop}(int fd{fd}, u_long com, char *argp);

int{erret,STD_RET} ioperm{rootop}(
	u_long from{from}, u_long num{num}, int turn_on{state});

int{erret,STD_RET} iopl{rootop}(int level{level});

int{erret,STD_RET} kill{pidop}(int pid{pid}, 
	int signum{signum});

int{erret,STD_RET} lchown{fileop, auditop, if(uid > 0) rootop} (
	char *path{path,nterm}, 
	int uid{uid,ouid}, 
	int gid{gid,ogid}
);

int{erret,STD_RET} link{fileop}(char *path{path,nterm}, 
	 char *link{destpath,nterm});

int{erret,STD_RET} listen{fdop,socketop}(int s{fd,socket},
        int backlog{backlog});

int{erret,STD_RET} llseek{fdop}(
	int fd{fd},
	unsigned long offset_high{high},
	unsigned long offset_low{high},
	loff_t *result{result,out}, unsigned int whence{whence});

off_t{locret,VAL_RET} lseek{fdop}(int fd{fd}, 
	off_t offset{offset}, int whence{whence});

int{erret,STD_RET} lstat{fileop}(
        char *path{path,nterm},
        struct stat *ub{out});

int{erret,STD_RET} mkdir{fileop, auditop} (
	char *path{path,nterm}, 
	int mode{mode}
);

int{erret,STD_RET} mknod{rootop,fileop, auditop} (
	char *path{path,nterm}, 
	mode_t mode{mode},
	dev_t dev
);

int{erret,STD_RET} mlock{memop,rootop}(const void *addr, size_t len); 

int{erret,STD_RET} mlockall{memop,rootop}(int flags{flags}); 

int{count,VAL_RET} modify_ldt{x86op}(
	int func,
	void *ptr{out},
	u_long bytecount);

int mount{fsop, rootop}(const char *specialfile{node, path, nterm},
	const char *dir{mountpoint, destpath, nterm},
	const char *filesystemtype{fstype, nterm},
	unsigned long rwflag{flag},
	const void *data);

int{erret,STD_RET} mprotect{memop}(const void *addr, 
	size_t len{nbytes}, int prot); 

void *{retptr,VAL_RET} mremap{memop}(
	void *old_address{copy(old_address,old_size)},
	size_t old_size,
	size_t new_size{nbytes},
	u_long flags{flags});

int{erret,STD_RET} msync{memop}(caddr_t addr, 
	size_t len{nbytes}, int flags); 

int{erret,STD_RET} munlock{memop}(const void *addr, size_t len{nbytes}); 

int{erret,STD_RET} munlockall{memop}(void); 

int{erret,STD_RET} munmap{memop}(void *start, size_t len{nbytes}); 

int{erret,STD_RET} nanosleep(
	const struct timespec *req,
	struct timespec *rem{out});

int{erret,STD_RET} newfstat(unsigned int fd{fd},
        struct stat *statbuf{out,statbuf});

int{erret,STD_RET} newlstat(char *filename{path,nterm},
        struct stat *statbuf{out,statbuf});

int{erret,STD_RET} newstat(char *filename{path,nterm},
        struct stat *statbuf{out,statbuf});

int{erret,STD_RET} newuname(struct new_utsname *name{out,uname});

int{erret,STD_RET} nfsservctl(
	int cmd,
	struct nfsctl_arg *argp,
	union nfsctl_res *resp);

int{erret,STD_RET} nice{if (inc < 0) rootop}(int inc{inc});

int{erret,STD_RET} old_mmap{memop,fdop}(struct mmap_arg_struct *argp
        {argp,
         copy(argp->addr, 4),
         copy(argp->len, 4),
         copy(argp->prot, 4),
         copy(argp->flags, 4),
         copy(argp->fd, 4),
         copy(argp->offset, 4)});

/* oldumount */

/* olduname */

int{erret,fdret,VAL_RET} open{fileop, fdretop, auditop,
			if (flags & O_CREAT)creatop,
			if (flags & O_TRUNC)truncateop} (
	char *path{path,nterm}, 
	int flags{flags},
	int mode{mode}
);

int{erret,NOERRET} pause(void);

int{prevpersona,VAL_RET} personality{confusionop}(u_long persona{persona});

int{erret,STD_RET} pipe(int filedes[2]{pipeends});

int{count,VAL_RET} poll{fdop}(
	struct pollfd *ufds{ufds},
	unsigned int nfds{nfds},
	int timeout{timeout});

int{erret,STD_RET} prctl(
	int option, u_long arg2, u_long arg3, u_long arg4, u_long arg5);

/* pread */

int{erret,STD_RET} ptrace(int request, int pid, int addr, int data);

/* pwrite */

int{ret,NOERR_RET} query_module{modop}(
	const char *name{name, nterm},
	int which{which},
	void *buf{buf, out, copy(buf,bufsize)},
	size_t bufsize{size},
	size_t *ret{buf,out});

int{erret,STD_RET} quotactl{rootop,fileop,
		if(cmd == Q_QUOTAON || cmd == Q_QUOTAOFF
		|| cmd == Q_SETQUOTA || cmd == Q_SETQLIM) rootop}(
	int cmd{cmd},
	char *special{nterm, path},
	int uid{uid},
	const void *addr);

size_t{sizeret,VAL_RET} read{fdop}(int fd{fd}, void *buf{copy(buf,
	nbyte), iobuf, out}, size_t nbyte{nbytes});

int{erret,STD_RET} readlink{fileop}(const char *path{path,nterm},
	char *buf{copy(buf,count),buf,out}, int count{nbytes});

int{sizeret,VAL_RET} readv{fdop}(
	int fd{fd},
	struct iovec *iovp{
	      iovp,
	      copy(iovp->iov_base, iovp->iov_len),  
	      rcopy(iovp, iovcnt), out},
	u_int iovcnt{count});

int{STD_RET} reboot{rootop}(int magic, int magic2, int flag, void *arg);

int{erret,VAL_RET} recvfrom{fdop,socketop}(
        int                     fd{fd,socket},
        void *                  buf{buf,out,copy(buf,len)},
        size_t                  len{len},
        int                     flags{flags,inout},
        struct sockaddr *       from{from,out,copy(from,*fromlen)},
        int *                   fromlen{fromlen,out});

int{erret,STD_RET} rename{fileop, auditop} (
	const char *from{path,nterm},
	const char *to{destpath,nterm}
);

int{erret,STD_RET} rmdir{fileop}(const char *path{path,nterm});

/* rt_sigaction */

/* rt_sigpending */

/* rt_sigprocmask */

/* rt_sigqueueinfo */

/* rt_sigreturn */

/* rt_sigsuspend*/

/* rt_sigtimedwait*/

int{maxret,VAL_RET} sched_get_priority_max(int policy{policy});

int{minret,VAL_RET} sched_get_priority_min(int policy{policy});

int{erret,STD_RET} sched_getparam(
	pid_t pid{pid},
	struct sched_param *p{proc,out});

int{policyret,VAL_RET} sched_getscheduler(pid_t pid{pid});

int{erret,STD_RET} sched_rr_get_interval(pid_t pid{pid}, struct timespec *tp);

int{erret,STD_RET} sched_setparam{rootop}(
	pid_t pid{pid},
	const struct sched_param *p{proc});

int{erret,STD_RET} sched_setscheduler(
	pid_t pid{pid},
	int policy{policy},
	const struct sched_param *p);

int{erret,STD_RET} sched_yield(void);

int{sizeret,VAL_RET} select{fdop}(int n, 
	fd_set *readfds,
	fd_set *writefds,
	fd_set *exceptfds,
	struct timeval *timeout); 

int{sizeret, VAL_RET} send{fdop}(
	int s{fd},
	void *mesg{iobuf, out, copy(mesg, len)},
	size_t len{len},
	int flags{flags}
);

int{countret,VAL_RET} sendfile{fdop}(
	int out_fd{destfd},
	int in_fd{fd},
	off_t *offset{offset,inout}, size_t count{count});

int{erret,STD_RET} setdomainname{rootop}(
	const char *name{name,nterm},
	size_t len{count});

int{erret,STD_RET} setegid{setop}(gid_t egid{egid}); 

int{erret,STD_RET} seteuid{setop}(uid_t euid{euid,uid}); 

int{erret,STD_RET} setgid{setop}(gid_t gid{gid}); 

int{prevfsgidret,VAL_RET} setfsgid{rootop}(uid_t fsgid{fsgid});

int{prevfsuidret,VAL_RET} setfsuid{rootop}(uid_t fsuid{fsuid});

int{erret,STD_RET} setgid(gid_t gid{gid});

int{erret,STD_RET} setgroups{rootop, setop}(
	size_t size{size},
	const gid_t *list{rcopy(list,size)}); 


int{erret,STD_RET} sethostname{rootop, setop}(
	const char *hostname{nterm}, 
	size_t len{nbytes}); 

int{erret,STD_RET} setitimer{setop}(
	int which, 
	const struct itimerval *itv,
	struct itimerval *ovalue{out}); 

int{erret,STD_RET} setpgid{setop}(
	int pid{pid}, 
	int pgid{gid}); 

int{erret,STD_RET} setpriority{setop, prioop}(int which, 
	int who, int prio);  

int{erret,STD_RET} setregid{setop, if (rgid != egid) rootop}(int rgid{rgid}, 
	int egid{egid}); 

int{erret,STD_RET} setresgid{setop}(gid_t rgid, gid_t egid, gid_t sgid);

int{erret,STD_RET} setresuid{setop, if (ruid != euid) rootop}(uid_t ruid{ruid},
        uid_t euid{euid}, uid_t suid{suid});

int{erret,STD_RET} setreuid{setop, if(ruid != euid) rootop}(int ruid{ruid}, 
	int euid{euid}); 

int{erret,STD_RET} setrlimit{setop}(int resource, struct rlimit *rlim); 

int{gidret,VAL_RET} setsid{setop}(void); 

int{erret,STD_RET} setsockopt{fdop,socketop}(
        int             fd{fd,socket},
        int             level{level},
        int             optname{optname},
        void      *     optval{value,copy(optval,optlen)},
        int             optlen{optlen});

int{erret,STD_RET} settimeofday{setop, rootop}(
	const struct timeval *tv, 
	const struct timezone *tz);

int{erret,STD_RET} setuid{setop}(uid_t uid{ruid,uid}); 

int{erret,STD_RET} sigaction{sigop}(int signum, 
	const struct sigaction *act, struct sigaction *oldact{out}); 

/* sigaltstack
int{erret,STD_RET} sigaltstack{sigop}(struct sigaltstack *nss
	{ copy(nss->ss_sp, nss->ss_size) },
	struct sigaltstack *oss { copy(nss->ss_sp, nss->ss_size) });
*/

__sighandler_t{sigret,SIG_RET} signal(int signum, __sighandler_t handler);

int{erret,STD_RET} sigpending{sigop}(sigset_t *sset); 

int{erret,STD_RET} sigprocmask{sigop}(
	int how,
	const sigset_t *sset,
	sigset_t *oldset{out});

int{noret,NOERR_RET} sigreturn(u_long not_unused);

int{erret,STD_RET} sigsuspend{sigop}(const sigset_t *mask); 

int{fdret,VAL_RET} socket{fdretop, if (domain == AF_INET) isockop, 
	if (domain == AF_LOCAL) usockop,socketop} 
	(int domain{domain}, int socktype{socktype}, int protocol{proto});

/* All socket calls come through this one syscall */
/* Someone else can actually figure out how to characterize it usefully. */
int socketcall(int call{call}, unsigned long *args {args} ); 

/* ssetmask */

int{erret,STD_RET} stat{fileop}(char *path{path,nterm}, struct stat *buf{out});

/* statfs */

int{erret,STD_RET} stime{setop, rootop}(time_t *t);

int{erret,STD_RET} swapoff{rootop,fileop}(const char *path{path,nterm});

int{erret,STD_RET} swapon{rootop,fileop}(
	const char *name{path,nterm},
	int swapflags);

int{erret,STD_RET} symlink{fileop}(const char *path{path,nterm},
	const char *link{destpath,nterm});

int{erret,STD_RET} sync{fileop}(void); 

int{erret,STD_RET} sysctl(struct __cysctl_args *args);

int{intret,VAL_RET} sysfs(int option, void *arg1, void *arg2);

int{erret,STD_RET} sysinfo(struct sysinfo *info{out});

int{nbytesret,VAL_RET} syslog{if (command != 3) rootop}(
	int command{command},
	char *bufp{out,copy(bufp,len)},
	int len{nbytes});

time_t{timeret,VAL_RET} time(time_t *t{out});

clock_t{tickret,VAL_RET} times(struct tms *buf{out});

int{erret,STD_RET} truncate{fileop,truncateop}(char *path{path, nterm}, 
	int pad, off_t length); 

int{erret,STD_RET} umask{fdop}(int newmask); 

int{erret,STD_RET} uname(struct utsname *name
	{name,
         copy(name->sysname, _UTSNAME_LENGTH),
	 copy(name->nodename, _UTSNAME_NODENAME_LENGTH),
	 copy(name->release, _UTSNAME_LENGTH),
	 copy(name->version, _UTSNAME_LENGTH),
	 copy(name->machine, _UTSNAME_LENGTH),
         copy(name->domainname, _UTSNAME_DOMAIN_LENGTH)} ); 

int{erret,STD_RET} unlink{fileop}(char *path{path,nterm});

int{erret,STD_RET} uselib{fileop}(const char *library{path,nterm});

int{erret,STD_RET} ustat(dev_t dev{dev}, struct ustat *ubuf{out});

int{erret,STD_RET} utime{fileop}(char *path{path,nterm},
        struct utimbuf *tvp{tvp}); 

int{pidret,VAL_RET} vfork{forkop}(void *processor_registers);

int{erret,STD_RET} vm86{x86op}(unsigned long fn, struct vm86plus_struct *v86);

int{erret,STD_RET} vm86old{x86op}(struct vm86_struct *info);

int{pidret,VAL_RET} wait4{procop}(int pid, int *status, int options, 
	struct rusage *rusage{out}); 

pid_t{pidret,VAL_RET} waitpid{procop}(pid_t pid, int *status, int options);

int{sizeret,VAL_RET} write{fdop}(
	int fd{fd},
	char *buf{copy(buf,nbyte), iobuf}, 
        u_int nbyte{nbytes});

int{sizeret,VAL_RET} writev{fdop}(
	int fd{fd},
	struct iovec *iovp{iovp, copy(iovp->iov_base,iovp->iov_len),
			   rcopy(iovp, iovcnt)}, 
	u_int iovcnt);

}

/* Evil-ness: There is currently no way to look up the code for various
 * events, so where the events are triggered, the codes are hard-wired.
 * coredump == 1, update == 2
 * Don't switch order, or event-receivers will get confused.
 */
characterize system {
    /* Core dumped to $path. */
    int coredump(const char *path{path});
    /* $table changed by userspace program*/
    int update(const char *tablename{tablename});
    int insert(const char *tablename{tablename});
}
