/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: solaris.ch,v 1.8 2001/04/05 18:41:30 stevek Exp $
 */

includes {
#ifndef _WR_DEFS_INCLUDED
#include "wr_conf.h"
#endif

#ifndef __WSI_TYPES_H
#include "wsi_types.h"
#endif

#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <sys/signal.h>
#include <sys/statfs.h>
#include <sys/utsname.h>
#include <sys/acl.h>
#include <sys/signal.h>
#include <sys/stropts.h>
#include <sys/processor.h>
#include <sys/poll.h>
#include <sys/ucontext.h>
#include <sys/fcntl.h>
};

#include "wr_errname.h"

#define WR_LOCAL solaris

characterize WR_LOCAL {

int errno{
       errno{
          vmap(WR_SUCC,         _),
          vmap(WR_FAIL,         _),
          vmap(WR_BADPERM,      EPERM),
          vmap(WR_NOFILE,       ENOENT),
          vmap(WR_NOPROC,       ESRCH),
          vmap(WR_IOERR,        EIO),
          vmap(WR_ARG2LONG,     E2BIG),
          vmap(WR_BADFD,        EBADF),
          vmap(WR_NOCHILD,      ECHILD),
          vmap(WR_BADACCES,     EACCES),
          vmap(WR_FILEEXIST,    EEXIST),
          vmap(WR_BADARG,       EINVAL),
          vmap(WR_FILEBSY,      ETXTBSY),
          vmap(WR_FILE2BIG,     EFBIG),
          vmap(WR_NOSPC,        ENOSPC),
          vmap(WR_FNAME2LONG,   ENAMETOOLONG)
       }
   };

/* 
 * syscalls that return 0 on success and -1 on failure
 */
#define STD_RET wrret{vmap(WR_SUCC,0), vmap(*,-1)}
/* 
 * syscalls that return some values on success and -1 on failure
 */
#define VAL_RET wrret{vmap(WR_SUCC,_), vmap(*,-1)}
/* 
 * syscalls that are always successful, and no return value is 
 * reserved to indicate an error. 
 */
#define NOERR_RET wrret     

int{fdret,VAL_RET} accept(int s{fd, socket}, 
	caddr_t name{copy(name, *anamelen), out}, 
	int *anamelen{nbytes});

int{erret,STD_RET} access{fileop}(char *path{path,nterm}, 
	int flags{mode});

int{erret,VAL_RET} acl{fileop}(char *path{path,nterm},
	int cmd,
	int nentries,
	aclent_t *aclbufp{out});

int{erret,STD_RET} adjtime{rootop}(struct timeval *delta,
	struct timeval *olddelta{out});

u_int{timeret,VAL_RET} alarm(u_int sec);

int{erret,STD_RET} bind(int s{fd,socket},
	caddr_t name{addr, copy(name, namelen)},
	int namelen{nbytes});

int{erret,STD_RET} brk(void *endds);

/* c2audit */

int{erret,STD_RET} chdir{fileop}(char *path{path,nterm});

int{erret,STD_RET} chmod{fileop}(char *path{path,nterm},
		int mode{mode});

int{erret,STD_RET} chown{fileop, if(uid > 0) rootop}(char *path{path,nterm},
		int uid{uid}, int gid{gid});

int{erret,STD_RET} chroot{rootop,fileop}(char *path{path, nterm});

int{erret,STD_RET} clock_getres{getop}(clockid_t clock_id,
	struct timespec *res{out});

int{erret,STD_RET} clock_gettime{getop}(clockid_t clock_id,
	struct timespec *tp{out});

int{erret,STD_RET} clock_settime{setop}(clockid_t clockid,
	struct timespec *tp);

int{erret,STD_RET} close{fdop}(int fd{fd});

int{erret,STD_RET} connect{fdop}(int s{fd,socket},
	caddr_t name{addr, copy(name, namelen)},
	int namelen{namelen, nbytes});

int{fdret,VAL_RET} creat{fileop,fdretop}(char *path{path, nterm}, 
	int mode{mode});

int{fdret,VAL_RET} creat64{fileop,fdretop}(char *path{path, nterm}, 
	int mode{mode});

/* doorfs */

int{fdret,VAL_RET} dup{fdop,fdretop}(u_int fd{fd});

int{erret,STD_RET} exec{execop}(
	struct execa *a{arglist/*,copy(a->fname,wr_strlen(a->fname))*/},
	union rval* r{rval});

int{erret,STD_RET} exece{fileop,execop}(char *fname{path,nterm},
	char **argv{argv, *nterm}, char **envv{envp, *nterm});

int{if(cmd == GETACL ) aclret,
    if(cmd == GETACLCNT) aclret,
    erret,VAL_RET} facl{fdop}(int fd{fd}, int cmd, int nentries,
	aclent_t *aclbufp);

int{erret,STD_RET} fchdir{fdop}(int fd{fd});

int{erret,STD_RET} fchmod{fdop}(int fd{fd}, 
	int mode{mode});

int{erret,STD_RET} fchown{fdop, if (uid > 0) rootop}(int fd{fd}, 
	int uid{uid}, int gid{gid});

int{erret,STD_RET} fchroot{fdop}(int fd{fd});

int{if(cmd == F_DUPFD || cmd == F_DUP2FD) fdret,
    if(cmd == F_GETOWN) pidret,
    erret,VAL_RET} fcntl
    {fdop, if(cmd==F_DUPFD||cmd==F_DUP2FD) fdretop}(int fd{fd}, 
	int cmd{cmd}, int arg);

/* fdsync */

pid_t{pidret,STD_RET} fork{forkop}(void);

pid_t{pidret,STD_RET} fork1{forkop}(void);

long{limret,VAL_RET} fpathconf{fdop}(int fd{fd}, 
	int name);

int{erret,STD_RET} fstat{fdop}(int fd{fd}, 
	struct stat *sb{out});

int{erret,STD_RET} fstat{fdop}(int fd{fd}, 
	struct stat64 *sb{out});

int{erret,STD_RET} fstatfs{fdop}(int fd{fd}, 
	struct statfs *buf{out});

int{erret,STD_RET} fstatvfs{fdop}(int fd{fd}, 
	struct statvfs *buf{out});

int{erret,STD_RET} fstatvfs64{fdop}(int fd{fd}, 
	struct statvfs64 *buf{out});

/* fxstat */

int{erret,VAL_RET} getdents{getop,fdop}(int fd{fd},
	struct dirent *buf{copy(buf,nbyte),buf,out}, size_t nbyte{nbytes});

int{erret,VAL_RET} getdents64{getop,fdop}(int fd{fd},
	struct dirent64 *buf{copy(buf,nbyte),buf,out}, size_t nbyte{nbytes});

gid_t{gidret,NOERR_RET} getgid{getop}(void);

int{erret,VAL_RET} getgroups{getop}(u_int gidsetsize, 
	gid_t *gidset{out});

int{erret,STD_RET} getitimer{getop}(u_int which, 
	struct itimerval *itv{out});

int{erret,VAL_RET} getmsg{getop,fdop}(int fd{fd},
	struct strbuf *ctl{
		ctl,
		copy(ctl->buf, ctl->len),
		out},
	struct strbuf *data{
		data,
		copy(data->buf, data->len),
		out},
	int *flags{out});

int{erret,STD_RET} getpeername{getop, fdop}(int fdes{fd,socket}, 
	caddr_t asa{buf,out}, int *alen{nbytes});

pid_t{pidret,VAL_RET} getpid{getop, pidop}(void);

int{erret,VAL_RET} getpmsg{getop,fdop}(int fd{fd},
	struct strbuf *ctl{
		ctl,
		copy(ctl->buf, ctl->len),
		out},
	struct strbuf *data{
		data,
		copy(data->buf, data->len),
		out},
	int *band{out},
	int *flags{out});

int{erret,STD_RET} getrlimit{getop}(int which, 
	struct rlimit *rlp{out});

int{erret,STD_RET} getrlimit{getop}(int which, 
	struct rlimit64 *rlp{out});

int{erret,STD_RET} getsockname{getop, fdop}(int fdes{fd,socket}, 
	caddr_t asa{buf,out}, int *alen{nbytes});

int{erret,STD_RET} getsockopt{getop, fdop}(int s{fd,socket}, 
	int level, int name, 
	caddr_t val{buf,out}, int *avalsize{nbytes});

int{erret,STD_RET} gettimeofday{getop}(struct timeval *tp{out}, 
	struct timezone *tzp);

uid_t{uidret,NOERR_RET} getuid{getop}(void);

/* gtime */

/* gtty */

/* inst_sync */

/* install_utrap */

int{erret,STD_RET} ioctl{fdop}(int fd{fd}, 
	u_long com, caddr_t data);

/* kaio */

int{erret,STD_RET} kill{pidop}(int pid{pid}, 
	int signum);

int{erret,STD_RET} lchown{fileop, if(uid > 0) rootop}(char *path{path,nterm}, 
	int uid{uid}, int gid{gid});

int{erret,STD_RET} link{fileop}(char *path{path,nterm}, 
	 char *link{destpath,nterm});

int{erret,STD_RET} listen{fdop}(int s{fd,socket}, int backlog);

offset_t{locret,VAL_RET} llseek{fdop}(int fd{fd}, offset_t offset{offset},
	int whence{whence});

off_t{locret,VAL_RET} lseek{fdop}(int fd{fd}, 
	off_t offset{offset}, int whence{whence});

int{erret,STD_RET} lstat{fileop}(char *path{path,nterm}, struct stat *ub);

int{erret,STD_RET} lstat64{fileop}(char *path{path,nterm}, struct stat64 *ub);

/* lwp_alarm */

/* lwp_continue */

/* lwp_create */

/* lwp_cv_broadcast */

/* lwp_cv_signal */

/* lwp_cv_wait */

/* lwp_exit */

/* lwp_getprivate */

/* lwp_info */

/* lwp_kill */

/* lwp_mutex_enter */

/* lwp_mutex_exit */

/* lwp_self */

/* lwp_sema_post */

/* lwp_sema_trywait */

/* lwp_sema_wait */

/* lwp_setprivate */

/* lwp_sigredirect */

/* lwp_suspend */

/* lwp_wait */

/* lxstat */

int{erret,STD_RET} memcntl{memop}(caddr_t addr, size_t len{nbytes},
	int cmd, caddr_t arg, int attr, int mask);

int{erret,STD_RET} mincore{memop}(caddr_t addr, 
	size_t len{nbytes}, char *vec{nterm,out});

int{erret,STD_RET} mkdir{fileop}(char *path{path,nterm}, int mode{mode});

int{erret,STD_RET} mknod{rootop,fileop}(char
	*path{path,nterm}, int mode{mode},
		int dev);

caddr_t{locret,VAL_RET} mmap{memop,fdop}(caddr_t addr, 
	size_t len{nbytes}, int prot, int flags, 
	int fd{fd}, long pad, off_t pos{offset});

caddr_t{locret,VAL_RET} mmap64{memop,fdop}(caddr_t addr, 
	size_t len{nbytes}, int prot, int flags, 
	int fd{fd}, long pad, off64_t pos{offset});

/*
 * Note: This might not be entirely correct, as the modctl syscall
 *       has 14 different commands
 */
int{erret,STD_RET} modctl(int cmd, struct modlsys *modlsys,
	struct modlinkage *modlinkage, struct modinfo *modinfo{out});

int{erret,STD_RET} mount{rootop,fileop}(int mttype, char
	*path{path,nterm}, 
	int flags, caddr_t data);

int{erret,STD_RET} mprotect{memop}(caddr_t addr, 
	size_t len{nbytes}, int prot);

int{erret,STD_RET} msgsys{msgop}(int which, 
	int a2, int a3, int a4, int a5, int a6);

int{erret,STD_RET} munmap{memop}(caddr_t addr, size_t len{nbytes});

int{erret,STD_RET} nanosleep(struct timespec *rqt, struct timespec *rmt{out});

/* nfs */

int{erret,VAL_RET} nice(int incr);

/* nosys */

int{erret,STD_RET} ntp_adjtime(struct timex *tp);

int{erret,STD_RET} ntp_gettime(struct ntptimeval *tp);

int{fdret,VAL_RET} open{fileop, fdretop}(char
	*path{path,nterm}, int flags{flags},
	int mode{mode});

int{fdret,VAL_RET} open64{fileop, fdretop}(char
	*path{path,nterm}, int flags{flags},
	int mode{mode});

int{psret,VAL_RET} p_online(processorid_t processorid, int flag);

int{limret,VAL_RET} pathconf{fileop}(char *path{nterm,path}, 
	int name);

int{erret,STD_RET} pause(void);

int{erret,STD_RET} pipe(void);

int{nfdret,VAL_RET} poll{fdsop}(struct pollfd *fds{out}, nfds_t nfds,
	int timeout);

#if WR_USE_DATABUF
ssize_t{sizeret,VAL_RET} pread{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, type(databuf_t), out}, size_t nbyte{nbytes},
	off_t offset{offset});
#else
ssize_t{sizeret,VAL_RET} pread{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out}, size_t nbyte{nbytes},
	off_t offset{offset});
#endif


#if WR_USE_DATABUF
ssize_t{sizeret,VAL_RET} pread64{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, type(databuf_t), out}, size_t nbyte{nbytes},
	off64_t offset{offset});
#else
ssize_t{sizeret,VAL_RET} pread64{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out}, size_t nbyte{nbytes},
	off64_t offset{offset});
#endif

/* priocntlsys */

int{erret,STD_RET} processor_bind(idtype_t idtype, id_t id,
	processorid_t processorid, processorid_t *obind{out});

int{erret,STD_RET} process_info(processorid_t processorid,
	processor_info_t *infop{out});

int{erret,STD_RET} profil(caddr_t samples{buf}, 
	u_int size{nbytes}, u_int offset{offset}, u_int scale);

/* pset */

int{erret,STD_RET} putmsg{fdop}(int fd{fd},
	struct strbuf *ctl{
		ctl,
		copy(ctl->buf, ctl->len)},
	struct strbuf *data{
		data,
		copy(data->buf, data->len)},
	int flags);

int{erret,STD_RET} putpmsg{fdop}(int fd{fd},
	struct strbuf *ctl{
		ctl,
		copy(ctl->buf, ctl->len)},
	struct strbuf *data{
		data,
		copy(data->buf, data->len)},
	int band,
	int flags);

#if WR_USE_DATABUF
ssize_t{sizeret,VAL_RET} pwrite{fdop}(int fd{fd},
	char *buf{copy(buf,nbytes), iobuf, type(databuf_t)}, 
	size_t nbytes{nbytes},
	off_t offset{offset});
#else
ssize_t{sizeret,VAL_RET} pwrite{fdop}(int fd{fd},
	char *buf{copy(buf,nbytes), iobuf}, 
	size_t nbytes{nbytes},
	off_t offset{offset});
#endif

#if WR_USE_DATABUF
ssize_t{sizeret,VAL_RET} pwrite64{fdop}(int fd{fd},
	char *buf{copy(buf,nbytes), iobuf, type(databuf_t)}, 
	size_t nbytes{nbytes},
	off64_t offset{offset});
#else
ssize_t{sizeret,VAL_RET} pwrite64{fdop}(int fd{fd},
	char *buf{copy(buf,nbytes), iobuf}, 
	size_t nbytes{nbytes},
	off64_t offset{offset});
#endif

#if WR_USE_DATABUF
int{sizeret,VAL_RET} read{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out,type(databuf_t)}, u_int nbyte{nbytes});
#else
int{sizeret,VAL_RET} read{fdop}(int fd{fd}, char *buf{copy(buf,
	nbyte), iobuf, out}, u_int nbyte{nbytes});
#endif

int{erret,STD_RET} readlink{fileop}(char *path{path,nterm},
	char *buf{copy(buf,count),buf,out}, int count{nbytes});

int{sizeret,VAL_RET} readv{fdop}(int fd{fd}, struct iovec *iovp{
	      iovp,
	      copy(iovp->iov_base, iovp->iov_len),  
	      rcopy(iovp, iovcnt), out},
	      u_int iovcnt{count});

#if WR_USE_DATABUF
int{sizeret,VAL_RET} recv{fdop}(int s{fd, socket},
	caddr_t buf{buf,iobuf,copy(buf,len),out,type(databuf_t)},
	size_t len{nbytes},
	int flags);
#else
int{sizeret,VAL_RET} recv{fdop}(int s{fd, socket},
	caddr_t buf{buf,iobuf,copy(buf,len),out},
	size_t len{nbytes},
	int flags);
#endif

#if WR_USE_DATABUF
int{sizeret,VAL_RET} recvfrom{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf,len),out,type(databuf_t)}, 
	size_t len{nbytes}, 
	int flags, 
	caddr_t from{addr,sockadr, copy(from, *fromlenaddr), out}, 
	int *fromlenaddr);
#else
int{sizeret,VAL_RET} recvfrom{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf,len),out}, 
	size_t len{nbytes}, 
	int flags, 
	caddr_t from{addr,sockadr, copy(from, *fromlenaddr), out}, 
	int *fromlenaddr);
#endif

int{sizeret,VAL_RET} recvmsg{fdop}(int s{fd, socket}, 
	struct msghdr *msg
      	{
        msghdr,
	copy(msg->msg_name, msg->msg_namelen),
	rcopy(msg->msg_iov, msg->msg_iovlen),
	copy(msg->msg_iov->iov_base, msg->msg_iov->iov_len),
	copy(msg->msg_accrights, msg->msg_accrightslen),
	out
      	},
      	int flags);

int{erret,STD_RET} rename{fileop}(char
	*from{path,nterm},
		char *to{destpath,nterm});

int{sizeret,VAL_RET} resolvepath{fileop}( char *path{path,nterm},
	char *buf{copy(buf,bufsiz),buf,out}, size_t bufsiz{nbytes});

void rexit(int rval);

int{erret,STD_RET} rmdir{fileop}(char *path{path,nterm});

/* rpcmod */

/* schedctl */

int{erret,STD_RET} semsys{semop}(int which, int a2, int a3, int a4, int a5);

#if WR_USE_DATABUF
int{sizeret,VAL_RET} send{fdop}(int s{fd, socket},
	caddr_t msg{buf,iobuf,copy(msg, len),type(databuf_t)},
        size_t len{nbytes}, int flags);
#else
int{sizeret,VAL_RET} send{fdop}(int s{fd, socket},
	caddr_t msg{buf,iobuf,copy(msg, len)}, size_t len{nbytes}, int flags);
#endif

int{sizeret,VAL_RET} sendmsg{fdop}(int s{fd, socket}, 
	caddr_t msg{buf}, int flags);

#if WR_USE_DATABUF
int{sizeret,VAL_RET} sendto{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf, len), type(databuf_t)},
        size_t len{nbytes}, int flags,
        caddr_t to{addr,sockadr, copy(to, tolen)}, int tolen{adrlen});
#else
int{sizeret,VAL_RET} sendto{fdop}(int s{fd, socket}, 
	caddr_t buf{buf,iobuf,copy(buf, len)}, size_t len{nbytes}, 
	int flags, caddr_t to{addr,sockadr, copy(to, tolen)}, int tolen{adrlen});
#endif

int{erret,STD_RET} setcontext{setop}(ucontext_t *ucp);

int{erret,STD_RET} setegid{setop}(gid_t egid{egid});

int{erret,STD_RET} seteuid{setop}(uid_t euid{uid});

int{erret,STD_RET} setgid{setop}(gid_t gid{gid});

int{erret,STD_RET} setgroups{rootop, setop}(u_int gidsetsize, gid_t *gidset);

int{erret,STD_RET} setitimer{setop}(u_int which, 
	struct itimerval *itv, struct itimerval *oitv);

pid_t{pidret,VAL_RET} setpgrp{setop}(void);

int{erret,STD_RET} setregid{setop, if (rgid != egid) rootop}(int rgid{rgid}, 
	int egid{egid});

int{erret,STD_RET} setreuid{setop, if(ruid != euid) rootop}(int ruid{ruid}, 
	int euid{euid});

int{erret,STD_RET} setrlimit{setop}(u_int which, 
	struct rlimit *rlp);

int{erret,STD_RET} setrlimit64{setop}(u_int which, 
	struct rlimit64 *rlp);

int{erret,STD_RET} setsockopt{fdop, setop}(int s{fd, socket}, 
	int level, int name, caddr_t val, int valsize);

int{erret,STD_RET} setuid{setop}(uid_t uid{uid});

int{shmret,VAL_RET} shmsys{shmop}(int which, int a2, int a3, int a4);

int{erret,STD_RET} shutdown{fdop}(int s{fd, socket}, int how);

int{erret,STD_RET} sigaction{sigop}(int signum, 
	struct sigaction *nsa, struct sigaction *osa{out});

int{erret,STD_RET} sigaltstack{sigop}(struct sigaltstack *nss
	{ copy(nss->ss_sp, nss->ss_size) },
	struct sigaltstack *oss { copy(nss->ss_sp, nss->ss_size) });

/* signotify */

/* signotifywait */

int{erret,STD_RET} sigpending{sigop}(void);

int{erret,STD_RET} sigprocmask{sigop}(int how,
	sigset_t *iset, sigset_t *oset{out});

int{erret,STD_RET} sigqueue{sigop}(pid_t pid, int signo, union sigval value);

/* sigsendsys */

int{erret,STD_RET} sigsuspend{sigop}(int mask);

int{erret,STD_RET} sigwait{sigop}(sigset_t *iset, int *sig{out});

int{fdret,VAL_RET} so_socket{fdretop, if (domain == PF_INET) isockop} 
	(int domain{domain}, int socktype{socktype}, int protocol,
	char *path{path,nterm});

/* so_socketpair */

/* sockconfig */

/* ssig */

int{erret,STD_RET} stat{fileop}(char *path{path,nterm}, struct stat *ub);

int{erret,STD_RET} stat64{fileop}(char *path{path,nterm}, struct stat64 *ub);

int{erret,STD_RET} statfs{fileop}(char
	*path{path,nterm},
	struct statfs *buf{buf, out}, int, int);

int{erret,STD_RET} statvfs{fileop}(char
	*path{path,nterm},
	struct statvfs *buf{buf, out});

int{erret,STD_RET} statvfs64{fileop}(char
	*path{path,nterm},
	struct statvfs64 *buf{buf, out});

int{erret,STD_RET} stime(time_t *tp);

/* stty */

int{erret,STD_RET} symlink{fileop}(char *path{path,nterm},
	char *link{destpath,nterm});

/* sysacct */

/* sysconfig */

/*
 * This is a wierd one, the parameters can be one of three different sets
 * depending on the opcode:
 * opcode = 1   sysfs(int opcode, char *fsname)
 * opcode = 2   sysfs(int opcode, int fs_index, char *buf)
 * opcode = 3   sysfs(int opcode)
 */
int{erret,VAL_RET} sysfs(int opcode);

/* syssync */

/* systeminfo */

int{erret,STD_RET} timer_create(clockid_t clockid,
	struct sigevent *evp,
	timer_t *timerid{out});

int{erret,STD_RET} timer_delete(timer_t timerid);

int{erret,VAL_RET} timer_getoverrun(timer_t timerid);

int{erret,STD_RET} timer_gettime(timer_t timerid,
	struct itimerspec *value{out});

int{erret,STD_RET} timer_settime(timer_t timerid, int flags,
	struct itimerspec *value,
	struct itimerspec *ovalue{out});

clock_t{clockret,VAL_RET} times(struct tms *buf{out});

/* tsolsys */

/*
 * Note: This is the shutdown/reboot/remount/freeze syscall
 */
int{erret,STD_RET} uadmin(int cmd, int fcn, int mdep);

/* uexch */

long{limret,VAL_RET} ulimit(int cmd, long newlimit);

int{erret,STD_RET} umask{fdretop}(int newmask{modemask});

int{erret,STD_RET} umount{fileop}(char *path{path,nterm});

int{erret,STD_RET} uname(struct utsname *name
	{copy(name->sysname, SYS_NMLN),
	 copy(name->nodename, SYS_NMLN),
	 copy(name->release, SYS_NMLN),
	 copy(name->version, SYS_NMLN),
	 copy(name->machine, SYS_NMLN)} );

int{erret,STD_RET} unlink{fileop}(char *path{path,nterm});

int{erret,STD_RET} utime{fileop}(char *path{path,nterm}, struct utimbuf *times);

int{erret,STD_RET} utimes{fileop}(char *path{path,nterm}, struct timeval *tptr);

/* utssys */

int{pidret,VAL_RET} vfork{forkop}(void);

void vhangup(void);

/* vtrace */

pid_t{pidret,VAL_RET} wait(int *statloc{out});

/* waitsys */

#if WR_USE_DATABUF
int{sizeret,VAL_RET} write{fdop}(int fd{fd},
	char *buf{copy(buf,nbyte), iobuf,type(databuf_t)}, 
        u_int nbyte{nbytes});
#else
int{sizeret,VAL_RET} write{fdop}(int fd{fd},
	char *buf{copy(buf,nbyte), iobuf}, 
        u_int nbyte{nbytes});
#endif

int{sizeret,VAL_RET} writev{fdop}(int fd{fd},
	struct iovec *iovp{iovp, copy(iovp->iov_base,iovp->iov_len),
			   rcopy(iovp, iovcnt)}, 
	u_int iovcnt);

/* xmknod */

/* xstat */

void yield(void);

}

