/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wr_conf.h,v 1.103 2001/06/21 17:42:24 stevek Exp $
 * 
 * Author: Douglas Kilpatrick
 * 
 * Purpose:
 * To place random platform specific typedef's and defines and header-fixes
 * in one place.
 * 
 * This file should be included by all source files in the project.
 * 
 * If you need to have two different bits of code for different
 * compiliers, create a define to represent the issue
 * (ie, use "WR_BROKEN_STL" instead of "GCC_2_7") and set the define
 * conditionally here.
 */

#ifndef WR_DEFS_INCLUDED
#define WR_DEFS_INCLUDED

/* Include header generated by configure script */
#ifndef __WR_CONFIGURATION_H
#include <wr_configuration.h>
#endif /* !__WR_CONFIGURATION_H */

/* Special case to fix some bogosity with bzero */
#if defined(__FreeBSD__)
#  ifdef PD_IN_KERNEL
#    define KERNEL 1
#    define _KERNEL 1
#    include <sys/param.h>
#  else
#    include <osreldate.h>
#  endif
#  if ( ( (__FreeBSD_version >= 420000) && (__FreeBSD_version < 500000) ) || \
        (__FreeBSD_version >= 500012) ) && defined(WR_IN_WRAPC)
#    define _SYS_LIBKERN_H_
#  endif
#endif /* -*- FreeBSD -*- */

/* Include PI/PD common includes */
#ifndef __PI_DEFS_H
#include <pdpi/pi_defs.h>
#endif /* __PI_DEFS_H */


/* +++ Solaris +++ */
#if defined(__sun__) && defined(__svr4__)
#define WR_SOLARIS PD_SOLARIS
#define WR_UNIX PD_UNIX

/* If in kernel */
#ifdef PD_IN_KERNEL
#include <netinet/in.h>
#include <sys/socket.h>
#include <stdarg.h>

#define WR_NCALLS NSYSCALL
#else
#define _REENTRANT

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <stdarg.h>
#endif

#ifndef WSSLINKAGE
#define WSSLINKAGE
#endif /* WSSLINKAGE */

#endif
/* --- Solaris --- */


/* +++ Linux +++ */
#if defined(__linux__)
#define WR_LINUX PD_LINUX
#define WR_UNIX PD_UNIX

#ifdef PD_IN_KERNEL
#include <linux/linkage.h>
#include <linux/version.h>
#include <linux/module.h>
/* Kernel includes stdarg.h */
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/utime.h>
#include <linux/sys.h>

#define WR_NCALLS NR_syscalls

#else /* PD_IN_KERNEL */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/param.h>
#include <stdarg.h>
#endif /* PD_IN_KERNEL */

#ifndef WSSLINKAGE
#define WSSLINKAGE
#endif

#endif
/* --- Linux --- */


/* +++ FreeBSD +++ */
#if defined(__FreeBSD__)
#define WR_FREEBSD PD_FREEBSD
#define WR_UNIX PD_FREEBSD

/* If in kernel */
#ifdef PD_IN_KERNEL
#include <sys/proc.h>
#include <sys/systm.h>

#if ( ( (__FreeBSD_version >= 420000) && (__FreeBSD_version < 500000) ) || \
      (__FreeBSD_version >= 500012) ) && defined(WR_IN_WRAPC)
#undef _SYS_LIBKERN_H_
#define bzero wr_bzero
extern void wr_bzero( void *, size_t );
#include <sys/libkern.h>
#endif

#include <sys/uio.h>
#include <machine/stdarg.h>
#include <sys/socket.h>
#include <sys/msg.h>
#include <netinet/in.h>
#include <sys/syscall.h>
#include <sys/conf.h>
#include <sys/tty.h>
#include <sys/dirent.h>

#define WR_NCALLS SYS_MAXSYSCALL

#else /* PD_IN_KERNEL */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <stdarg.h>
#endif /* PD_IN_KERNEL */

#ifndef WSSLINKAGE
#define WSSLINKAGE
#endif /* WSSLINKAGE */

#endif
/* --- FreeBSD --- */


/* +++ Windows NT/2000 +++ */
#if defined(_WIN32)
#define WR_WINNT PD_WINNT

#define WSS_REG_KEY      "Software\\NAI Labs\\Wrappers"
#define WSS_LOG_FILENAME "wss.log"
#define WSS_DB_FILENAME	 "wss.db"
#define WSS_HOME_PATH    ""
#define WSS_INIT_FILE	 "wss.ini"

#if defined(PD_IN_KERNEL) && !defined(WSSLINKAGE)
#define WSSLINKAGE EXPORT
#endif

#define WR_NOSELECT1ST

#endif
/* --- Windows NT/2000 --- */


#ifdef WR_UNIX
#define WSS_LOG_FILENAME  CONF_LOCALSTATEDIR ## "/wss.log"
#define WSS_DB_FILENAME	  CONF_LOCALSTATEDIR ## "/wss.db"
#define WSS_HOME_PATH     CONF_LOCALSTATEDIR
#define WSS_INIT_FILE	  CONF_SYSCONFDIR ## "/wss.conf"
#define WSS_INC_PATH      CONF_INCDIR ## "/gswtk"
#define WSS_WRAP_INC_PATH CONF_DATADIR ## "/gswtk"
#endif /* WR_UNIX */

#endif /* WR_DEFS_INCLUDED */
