/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrapper_library.h,v 1.11 2001/06/13 13:32:25 stevek Exp $
 * 
 * This header file is required to be included by the C files that make up a 
 * wrapper library.
 * 
 * Wrapper libraries are done in C since ther WDL language does not currently
 * lend itself to libraries... That and the implimentation gets a lot easier.
 * 
 * 
 * To create a library:
 * 
 * file: library.c:
 * +----------------------------
 * |
 * | /* My library */                                                   /*
 * | #include <wrapper_library.h>
 * |
 * | WR_LIBRARY_DEF("mylibrary", 1, 0, "otherlib1\0otherlib2\0");
 * |
 * | #include "mylibrary.wr"
 * |
 * | void foo(...)
 * | {
 * | }
 * | 
 * +----------------------------
 * 
 * file: mylibrary.wr:
 * +----------------------------
 * |
 * | /* Header files for the wrappers that use this library. */         /*
 * | #ifndef WR_LIBRARY_SUPPRESS_WDL_STUFF
 * | library mylibrary {
 * | #endif
 * |
 * | void foo(...);
 * | 
 * | #ifndef WR_LIBRARY_SUPPRESS_WDL_STUFF
 * | }
 * | #endif
 * +----------------------------
 * 
 * The "WR_LIBRARY_DEF" macro sets up a few variables that the wrapper
 * linker requires.  The first argument is the name of the library.  The
 * second argument is the major version number.  The third argument is the
 * minor version number.  The fourth argument is a list of libraries your
 * library depends on.
 * 
 * In wrappers the user{} clause adds dependency information.  Since we
 * don't process your library you have to provide that information for
 * us.  The argument to the macro is a character string that contains
 * the names of the libraries you depend on terminated by '\0's.
 * The nul terminates the library name, it is not a seperator.  If you
 * depend on no libraries (a common situation) use the empty string. ("")
 * 
 * A wrapper that uses the library will provide an expected major and minor
 * number.  If the major number does not match, the load will fail.  If the
 * minor number expected is higher than the minor number the library
 * impliments, a warning will be issued.
 */

#define PD_IN_KERNEL 1
#define WR_LIBRARY_SUPPRESS_WDL_STUFF 1
#define WR_SUPPRESS_WDL_STUFF 1
#define PD_SUPPRESS_FUNCTION_DECL 1

#define WR_LIBRARY_DEF(name,major,minor,depends)		\
static char *WRLINK_MOD_NAME=PD_MAKESTR(name);			\
static int   WRLINK_MOD_MAJOR=major;				\
static int   WRLINK_MOD_MINOR=minor;				\
static char *WRLINK_MOD_TYPE=PD_MAKESTR("library");		\
static char *WRLINK_MOD_DEPENDENCIES=PD_MAKESTR(depends);	\
static int   WRLINK_MOD_INTERCEPT_LIST[] = { 0 }

/* No trailing semicolon since the invocation of the macro will have one */

#include <wr_conf.h>
#include <wss_process.h>
#include <wss_db.h>
#include <wss_module.h>

/*
 * Function declarations.
 */
#include <libwr.h>

#define wr_mycontext() (__context)

