%{
/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: criteria.l,v 1.11 2001/04/05 18:41:31 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: Scanner for criteria
 * 
 */

#define token(c)   { return(c); }
#include <string.h>

#ifdef _WIN32
#include <io.h>
#define snprintf _snprintf
#define isatty   _isatty
#endif

char error_code[256];
char *criteria_strin = NULL;

extern Processor *proc;

%}

%{
/* redefine macro in order to optionally read from a string a provided file */

#define YY_INPUT(buf,result,max_size) \
	{ \
		int c; \
		c = *(criteria_strin++); \
		result = (c == 0) ? YY_NULL : (buf[0] = c, 1); \
	}
%}

%x reg_ex_state

ID      [/_a-zA-Z][a-zA-Z0-9_\-/\.]*
NUM     [0-9]*
REGEX      [/a-zA-Z0-9_\-\.\*\+\\\^[\]\(\)$]*

%%

setuid		{	return(K_SETUID); }

setgid		{	return(K_SETGID); }

{ID}		{
                        criterialval.lit = new Literal( proc, criteriatext );
                        return IDENTIFIER;
		}

{NUM}		{
                        criterialval.lit = new Literal( proc, atoi(criteriatext) );
                        return NUM;
		}

[ \t\n]		{}

"&&"		{	return(K_AND); }
"||"		{	return(K_OR); }
"==>"		{	return(K_ARROW); }
"=~"		{	BEGIN(reg_ex_state);
			return(K_STRCMP);
		}
">="		{	return(K_GTE); }
"<="		{	return(K_LTE); }
"!="		{	return(K_NE); }
"=="		{	return(K_EQ); }

<reg_ex_state>{REGEX}	{
                        criterialval.regex = new Regex( proc, criteriatext );
                        BEGIN(INITIAL);
                        return REGEX;
		}

<reg_ex_state,INITIAL>"|" {
			token('|')
                }
"!"		{	token('!') }
">"		{	token('>') }
"<"		{	token('<') }

"\{"		{	token('{') }
"\}"		{	token('}') }
"("		{	token('(') }
")"		{	token(')') }
";"		{	token(';') }
","		{	token(',') }
"*"		{	token('*') }

<<EOF>>		{	yyterminate(); }

.		{	/*criteriaerror("Character not recognized");*/
		}

%%

int
criteriaerror(char *s)
{
        snprintf( error_code, 256, "\n'%s' at '%s'  (%d)\n", s, criteriatext, YY_START );
/*        exit(2); */
	return 0;
}

