/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: libcrit.h,v 1.25 2001/06/18 15:28:14 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Header for criteria processing and file generation library
 *
 */

#ifndef __LIBCRIT_H
#define __LIBCRIT_H 1

#include <wr_conf.h>
#include <wsi_types.h>

typedef struct crit_literal    crit_literal_t;
typedef struct crit_header     crit_header_t;
typedef struct crit_expression crit_expression_t;
typedef struct crit_regex      crit_regex_t;
typedef struct crit_regex_node crit_regex_node_t;
typedef re_token_type          crit_re_t;
typedef u_int32_t              crit_id_t;


enum crit_type
{
   CT_NONE = 0,
   CT_INT,
   CT_STRING,
   CT_LABEL,
   CT_BUILTIN
};

enum crit_op
{
   CO_NOP = 0,
   CO_AND,
   CO_OR,
   CO_NOT,
   CO_LT,
   CO_GT,
   CO_LTE,
   CO_GTE,
   CO_NE,
   CO_EQ,
   CO_RE,
   CO_SUID,
   CO_SGID,
   CO_ARROW
};

typedef enum crit_type         crit_type_t;
typedef enum crit_op           crit_op_t;

struct crit_literal
{
   crit_id_t   id;
   crit_type_t type;
   crit_id_t   soff;
   int         ival;
};

struct crit_header
{
   u_int32_t  magic;
   u_int16_t  version;
   pd_bool_t activation;
   crit_id_t  literals;
   size_t     litsize;
   crit_id_t  strings;
   size_t     strsize;
   crit_id_t  expression;
   size_t     expsize;
   crit_id_t  regextab;
   size_t     regextabsize;
   crit_id_t  start;
};

struct crit_expression
{
   crit_op_t    op;
   crit_id_t    arg1;
   crit_id_t    arg2;
   pd_bool_t   seen;
};

struct crit_regex
{
   crit_id_t    regex;
   size_t       regexsize;
};

struct crit_regex_node
{
   crit_re_t    op;
   crit_id_t    next;
   crit_id_t    oneofnext;
   crit_id_t    data;
};


#define CRIT_MAGIC   0x2ac1c001

#define CRIT_VERSION 0x0007

#define CRIT_OP_MASK      0x00008000
#define CRIT_BUILTIN_MASK 0x00004000
#define CRIT_RE_MASK      0x00002000
#define CRIT_ARG_MASK     0x000000ff
#define CRIT_RE_UNCLE     0x00008000
#define CRIT_RE_NEGATE    0x00008000
#define CRIT_RE_INSENS    0x00004000
#define CRIT_RE_STINGY    0x00002000

#define CTOK_PROG  CRIT_BUILTIN_MASK
#define CTOK_PPROG (CTOK_PROG + 1)
#define CTOK_PID   (CTOK_PPROG + 1)
#define CTOK_PPID  (CTOK_PID + 1)
#define CTOK_CWD   (CTOK_PPID + 1)
#define CTOK_USER  (CTOK_CWD + 1)
#define CTOK_UID   (CTOK_USER + 1)
#define CTOK_EUID  (CTOK_UID + 1)
#define CTOK_GID   (CTOK_EUID + 1)
#define CTOK_EGID  (CTOK_GID + 1)
#define CTOK_TTY   (CTOK_EGID + 1)

#if !defined(PD_IN_KERNEL) && !defined(USER_MODE_TEST)

#ifdef __cplusplus
#include "processor.h"

extern "C" {
int        LINKAGE  critParse( char *, Processor ** );
char       LINKAGE *critMakeTempFile( Processor *, char *, pd_bool_t );
const char LINKAGE *critParserError( void );
int        LINKAGE  critCompile( char * );
}
#endif

#endif /* !PD_IN_KERNEL && !USER_MODE_TEST */

#endif /* __LIBCRIT_H */

