/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef __RE_H
#define __RE_H 1

#include <wr_conf.h>

#ifdef __cplusplus
#include <stdio.h>
#include <stdlib.h>
#include <vector>
#endif  /* __cplusplus */

/*
 * Syntax options
 */
#define RE_BACKSLASH_ESCAPE_IN_LISTS  0x0000001
#define RE_BK_PLUS_QM                 0x0000002
#define RE_CHAR_CLASSES               0x0000004
#define RE_CONTEXT_INDEP_ANCHORS      0x0000008
#define RE_CONTEXT_INDEP_OPS          0x0000010
#define RE_CONTEXT_INVALID_OPS        0x0000020
#define RE_DOT_NEWLINE                0x0000040
#define RE_DOT_NOT_NULL               0x0000080
#define RE_INTERVALS                  0x0000100
#define RE_LIMITED_OPS                0x0000200
#define RE_NEWLINE_ALT                0x0000400
#define RE_NO_BK_BRACES               0x0000800
#define RE_NO_BK_PARENS               0x0001000
#define RE_NO_BK_REFS                 0x0002000
#define RE_NO_BK_VBAR                 0x0004000
#define RE_NO_EMPTY_RANGES            0x0008000
#define RE_UNMATCHED_RIGHT_PAREN_ORD  0x0010000
#define RE_HAT_LISTS_NOT_NEWLINE      0x0020000
#define RE_STINGY_OPS                 0x0040000
#define RE_CHAR_CLASS_ESCAPES         0x0080000
#define RE_PURE_GROUPING              0x0100000
#define RE_LOOKAHEAD                  0x0200000
#define RE_STRING_ANCHORS             0x0400000
#define RE_COMMENTS                   0x0800000
#define RE_CHAR_CLASS_ESC_IN_LISTS    0x1000000
#define RE_TOTAL                      0x2000000

#define RE_SYNTAX_AWK                    ( RE_TOTAL | \
                                           RE_BACKSLASH_ESCAPE_IN_LISTS | \
                                           RE_DOT_NOT_NULL | \
                                           RE_NO_BK_PARENS | \
                                           RE_NO_BK_REFS | \
                                           RE_NO_BK_VBAR | \
                                           RE_NO_EMPTY_RANGES | \
                                           RE_UNMATCHED_RIGHT_PAREN_ORD )
#define RE_SYNTAX_ED                     ( RE_SYNTAX_POSIX_BASIC )
#define RE_SYNTAX_EGREP                  ( RE_TOTAL | \
                                           RE_CHAR_CLASSES | \
                                           RE_CONTEXT_INDEP_ANCHORS | \
                                           RE_CONTEXT_INDEP_OPS | \
                                           RE_HAT_LISTS_NOT_NEWLINE | \
                                           RE_NEWLINE_ALT | \
                                           RE_NO_BK_PARENS | \
                                           RE_NO_BK_VBAR )
#define RE_SYNTAX_EMACS                  ( RE_TOTAL )
#define RE_SYNTAX_GREP                   ( RE_TOTAL | \
                                           RE_BK_PLUS_QM | \
                                           RE_CHAR_CLASSES | \
                                           RE_HAT_LISTS_NOT_NEWLINE | \
                                           RE_INTERVALS | \
                                           RE_NEWLINE_ALT )
#define RE_SYNTAX_POSIX_AWK              ( RE_POSIX_EXTENDED | \
                                           RE_BACKSLASH_ESCAPE_IN_LISTS )
#define RE_SYNTAX_POSIX_BASIC            ( RE_SYNTAX_POSIX_COMMON | \
                                           RE_BK_PLUS_QM )
#define RE_SYNTAX_POSIX_COMMON           ( RE_CHAR_CLASSES | \
                                           RE_DOT_NEWLINE | \
                                           RE_DOT_NOT_NULL | \
                                           RE_INTERVALS | \
                                           RE_NO_EMPTY_RANGES )
#define RE_SYNTAX_POSIX_EGREP            ( RE_SYNTAX_EGREP | \
                                           RE_INTERVALS | \
                                           RE_NO_BK_BRACES )
#define RE_SYNTAX_POSIX_EXTENDED         ( RE_SYNTAX_POSIX_COMMON | \
                                           RE_CONTEXT_INDEP_ANCHORS | \
                                           RE_CONTEXT_INDEP_OPS | \
                                           RE_NO_BK_BRACES | \
                                           RE_NO_BK_PARENS | \
                                           RE_NO_BK_VBAR | \
                                           RE_UNMATCHED_RIGHT_PAREN_ORD )
#define RE_SYNTAX_POSIX_MINIMAL_BASIC    ( RE_SYNTAX_POSIX_COMMON | \
                                           RE_LIMITED_OPS )
#define RE_SYNTAX_POSIX_MINIMAL_EXTENDED ( RE_SYNTAX_POSIX_COMMON | \
                                           RE_CONTEXT_INDEP_ANCHORS | \
                                           RE_CONTEXT_INVALID_OPS | \
                                           RE_NO_BK_BRACES | \
                                           RE_NO_BK_PARENS | \
                                           RE_NO_BK_REFS | \
                                           RE_NO_BK_VBAR | \
                                           RE_UNMATCHED_RIGHT_PAREN_ORD )
#define RE_SYNTAX_SED                    ( RE_SYNTAX_POSIX_BASIC )
#define RE_SYNTAX_PERL4                  ( RE_TOTAL | \
                                           RE_BACKSLASH_ESCAPE_IN_LISTS | \
                                           RE_CONTEXT_INDEP_ANCHORS | \
                                           RE_CONTEXT_INDEP_OPS | \
                                           RE_INTERVALS | \
                                           RE_NO_BK_BRACES | \
                                           RE_NO_BK_PARENS | \
                                           RE_NO_BK_VBAR | \
                                           RE_NO_EMPTY_RANGES | \
                                           RE_CHAR_CLASS_ESCAPES )
#define RE_SYNTAX_PERL4_S                ( RE_SYNTAX_PERL4 | \
                                           RE_DOT_NEWLINE )
#define RE_SYNTAX_PERL5                  ( RE_SYNTAX_PERL4 | \
                                           RE_PURE_GROUPING | \
                                           RE_STINGY_OPS | \
                                           RE_LOOKAHEAD | \
                                           RE_STRING_ANCHORS | \
                                           RE_CHAR_CLASS_ESC_IN_LISTS | \
                                           RE_COMMENTS )
#define RE_SYNTAX_PERL5_S                ( RE_SYNTAX_PERL5 | \
                                           RE_DOT_NEWLINE )
/* End of syntax options */

#if defined(WR_SOLARIS) && defined(PD_IN_KERNEL)
#define RE_FALSE FALSE
#define RE_TRUE  TRUE
#else
typedef enum { RE_FALSE = 0, RE_TRUE = 1 } bool_t;
#endif

#define REG_ICASE       0x0002
#define REG_DOT_NEWLINE 0x0004
#define REG_MULTILINE   0x0008
#define REG_NOTBOL      0x0010
#define REG_NOTEOL      0x0020
#define REG_ANCHORINDEX 0x0040

enum re_token_type
{
   RE_NOP = 0,
   RE_EXP,
   RE_CHAR,
   RE_ANY,
   RE_END,
   RE_ONEOF,
   RE_POSIX,
   RE_RANGE,
   RE_REPEAT,
   RE_START,
   RE_TOKENTYPES
};
typedef enum re_token_type re_token_type;

enum re_posix_type
{
   RE_INVALID = -1,
   RE_ALNUM, RE_ALPHA, RE_BLANK, RE_CNTRL, RE_DIGIT, RE_GRAPH, RE_LOWER,
   RE_PRINT, RE_PUNCT, RE_SPACE, RE_UPPER, RE_XDIGIT
};
typedef enum re_posix_type re_posix_type;

#ifdef __cplusplus
typedef std::vector <int>                 i_vec;
typedef std::vector <class regex_token *> tok_vec;

typedef struct regex_charindex regex_charindex;
struct regex_charindex
{
   char *re_str;
   int   re_index;
};

typedef class regex_token regex_token;
class LINKAGE regex_token
{
   protected:
      bool_t        re_stingy;
      int           re_subindex;

   private:
      regex_token *re_uncletok;
      regex_token *re_nexttok;
      bool_t       re_canbestingy;

   public:
                            regex_token( int subindex = 0,
                                         bool_t stingy = RE_FALSE );
      virtual              ~regex_token();
      virtual int           re_minlen();
      virtual regex_token * operator[]( int index );
      virtual int           re_children();
      virtual re_token_type re_gettype() = 0;
      i_vec *               re_next( regex_charindex &input, int index,
                                     int eflags );
      virtual i_vec *       re_match( regex_charindex &input, int index,
                                      int eflags ) = 0;
      regex_token *         re_walk();
      virtual bool_t        re_chain( regex_token *next );
      void                  re_setuncle( regex_token *uncle );
      regex_token *         re_getuncle();
      regex_token *         re_getnext();
      virtual void          re_dump( FILE *f );
      void                  re_dumpall( FILE *f );
      bool_t                re_canstingy();
      int                   re_getsubindex();
      void                  re_setstingy();
};

typedef class regex_posix_token regex_posix_token;
class LINKAGE regex_posix_token : public regex_token
{
   private:
      int          re_type;
      bool_t       re_insens;
      bool_t       re_negated;

   public:
                    regex_posix_token( int subindex, int type, bool_t insens,
                                       bool_t negated );
                   ~regex_posix_token();
      re_token_type re_gettype();
      int           re_getposixtype();
      bool_t        re_isinsens();
      bool_t        re_isnegated();
      int           re_minlen();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};
int re_posix_charclass( char *ccname );

typedef class regex_any_token regex_any_token;
class LINKAGE regex_any_token : public regex_token
{
   private:
      bool_t        re_newline;
      bool_t        re_null;

   public:
                    regex_any_token( int subindex, bool_t newline,
                                     bool_t null );
                   ~regex_any_token();
      re_token_type re_gettype();
      bool_t        re_canbenull();
      bool_t        re_canbenewline();
      int           re_minlen();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};

typedef class regex_char_token regex_char_token;
class LINKAGE regex_char_token : public regex_token
{
   private:
      bool_t       re_insens;
      char *       re_str;
      unsigned int re_len;

   public:
                    regex_char_token( int subindex, char ch, bool_t insens );
                   ~regex_char_token();
      re_token_type re_gettype();
      bool_t        re_isinsens();
      int           re_getlen();
      const char *  re_getstr();
      int           re_minlen();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      bool_t        re_chain( regex_token *next );
      void          re_dump( FILE *f );
};

typedef class regex_oneof_token regex_oneof_token;
class LINKAGE regex_oneof_token : public regex_token
{
   private:
      tok_vec     *re_options;
      bool_t       re_negative;

   public:
                    regex_oneof_token( int subindex, char *options,
                                       bool_t negative, bool_t insens );
                    regex_oneof_token( int subindex, tok_vec *options,
                                       bool_t negative );
                   ~regex_oneof_token();
      re_token_type re_gettype();
      int           re_minlen();
      bool_t        re_isnegated();
      regex_token * operator[]( int index );
      int           re_children();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      bool_t        re_chain( regex_token *next );
      void          re_dump( FILE *f );
};

typedef class regex_range_token regex_range_token;
class LINKAGE regex_range_token : public regex_token
{
   private:
      char         re_lo;
      char         re_hi;
      bool_t       re_insens;

   public:
                    regex_range_token( int subindex, char lo, char hi,
                                       bool_t insens );
                   ~regex_range_token();
      re_token_type re_gettype();
      bool_t        re_isinsens();
      char          re_getlo();
      char          re_gethi();
      int           re_minlen();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};

typedef class regex_repeated_token regex_repeated_token;
class LINKAGE regex_repeated_token : public regex_token
{
   private:
      regex_token *re_tok;
      int          re_min;
      int          re_max;

   public:
                    regex_repeated_token( int subindex, regex_token *tok,
                                          int min, int max );
                   ~regex_repeated_token();
      regex_token * operator[]( int index );
      int           re_children();
      re_token_type re_gettype();
      int           re_getmin();
      int           re_getmax();
      int           re_minlen();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};

typedef class regex_start_token regex_start_token;
class LINKAGE regex_start_token : public regex_token
{
   private:
      bool_t       re_newline;

   public:
                    regex_start_token( int subindex, bool_t newline );
                   ~regex_start_token();
      re_token_type re_gettype();
      bool_t        re_canbenewline();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};

typedef class regex_end_token regex_end_token;
class LINKAGE regex_end_token : public regex_token
{
   private:
      bool_t       re_newline;

   public:
                    regex_end_token( int subindex, bool_t newline );
                   ~regex_end_token();
      re_token_type re_gettype();
      bool_t        re_canbenewline();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      void          re_dump( FILE *f );
};

typedef class regex regex;
class LINKAGE regex : public regex_token
{
   private:
      regex_token *re_firsttok;
      regex_token *re_lasttok;
      int          re_nsubs;

             regex( regex_token *first, regex_token *last, int subs,
                    int subindex );
      void   re_addtoken( regex_token *next );

   public:
                    regex();
                   ~regex();
      re_token_type re_gettype();
      regex_token * operator[]( int index );
      int           re_children();
      i_vec *       re_match( regex_charindex &input, int index, int eflags );
      int           re_construct( char *pattern, int cflags = 0,
                                  int syntax = RE_SYNTAX_PERL5, int index = 0,
                                  int nextsub = 0 );
      bool_t        re_ismatch( regex_charindex &input, int index = 0,
                                int eflags = 0 );
      void          re_setuncle( regex_token *uncle );
      bool_t        re_chain( regex_token *next );
      void          re_dump( FILE *f );
};
#endif /* __cplusplus */

#endif /* __RE_H */

