/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re.cc,v 1.18 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <errno.h>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

#include <vector>

#ifdef __CYGWIN__
#include <windows.h>

extern "C" {
int WINAPI redll_init( HANDLE, DWORD, void * );
}

int WINAPI
redll_init( HANDLE h, DWORD reason, void *foo )
{
   return 1;
}
#endif

typedef struct charunit charunit;
struct charunit
{
   char   ch;
   bool_t bk;
};

regex::regex() : regex_token( 0 ) {}

regex::regex( regex_token *first, regex_token *last, int subs, int subindex )
   : regex_token( subindex )
{
   re_firsttok = first;
   re_lasttok  = last;
   re_nsubs    = subs;
}

regex::~regex()
{
   if( re_firsttok )
      delete re_firsttok;
}

re_token_type
regex::re_gettype()
{
   return RE_EXP;
}

regex_token *
regex::operator[]( int index )
{
   return( ( re_firsttok && ( index == 0 ) )?re_firsttok:NULL );
}

int
regex::re_children()
{
   return( re_firsttok?1:0 );
}

static int
getcharunit( char *pattern, int plen, int index, charunit &unit )
{
   unit.ch = pattern[index++];
   if( ( unit.bk = (unit.ch == '\\')?RE_TRUE:RE_FALSE ) )
      if( index < plen )
         unit.ch = pattern[index++];
      else
         return -EINVAL;
   return index;
}

static int
getposixset( char *pattern, int plen, int index, char *buf )
{
   int j = 0;
   for( int i = index; i < (plen-1); i++ )
   {
      if( ( pattern[i] == ':' ) && ( pattern[i+1] == ']' ) )
         return i+2;
      buf[j++] = pattern[i];
   }
   return index;
}

static int
getminmax( char *pattern, int plen, int index, int *minmax, int syntax )
{
   // Precondition: pattern[index-1] == '{', minmax != NULL
   if( index == plen )
   {
      fprintf( stderr, "no matching brace\n" );
      return -EINVAL;
   }

   int min = 0;
   int max = 0;

   charunit unit;
   char *buf = new char[plen];
   int bindex = 0;

   while( ( ( index = getcharunit(pattern,plen,index,unit) ) != plen ) &&
          isdigit(unit.ch) )
      buf[bindex++] = unit.ch;

   if( bindex == 0 )
   {
      delete buf;
      fprintf( stderr, "bad brace construct\n" );
      return -EINVAL;
   }

   buf[bindex] = '\0';
   min = atoi(buf);

   if( ( unit.ch == '}' ) &&
       ((( syntax & RE_NO_BK_BRACES )?RE_TRUE:RE_FALSE) ^ unit.bk) )
      max = min;
   else if( ( unit.ch == ',' ) && !unit.bk )
   {
      bindex = 0;
      while( ( ( index = getcharunit(pattern,plen,index,unit) ) != plen ) &&
             isdigit(unit.ch) )
         buf[bindex++] = unit.ch;

      if( bindex == 0 )
         max = INT_MAX;
      else
      {
         buf[bindex] = '\0';
         max = atoi(buf);
      }
   }
   else
   {
      delete buf;
      fprintf( stderr, "expected end of interval\n" );
      return -EINVAL;
   }

   delete buf;
   minmax[0] = min;
   minmax[1] = max;

   return index;
}

int
regex::re_construct( char *pattern, int cflags, int syntax, int index,
                     int nextsub )
{
   if( pattern == NULL )
      return ENOENT;

   int plen = strlen(pattern);
   re_nsubs = 0;

   re_firsttok = re_lasttok = NULL;
   bool_t insens = (cflags & REG_ICASE)?RE_TRUE:RE_FALSE;

   int minmax[2];

   regex_token *curtok = NULL;
   char         ch;
   charunit     unit;
   tok_vec     *branches = NULL;

   while( index < plen )
   {
      index = getcharunit( pattern, plen, index, unit );
      if( index < 0 )
         return -index;

      // ALTERNATION OPERATOR
      //  \| or | (if RE_NO_BK_VBAR) or newline (if RE_NEWLINE_ALT)
      //  not available if RE_LIMITED_OPS is set
      if( ( unit.ch == '|' ) &&
          ((( syntax & RE_NO_BK_VBAR )?RE_TRUE:RE_FALSE) ^ unit.bk) &&
          !( syntax & RE_LIMITED_OPS ) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: ALTERNATION OPERATOR]\n" );
#endif DEBUG_RE

         // Make everything up to here be a branch.
         // Create vector if necessary.
         if( branches == NULL )
            branches = new tok_vec;
         re_addtoken( curtok );
         branches->push_back( new regex(re_firsttok, re_lasttok, re_nsubs, re_subindex));
         re_firsttok = re_lasttok = curtok = NULL;
      }

      // INTERVAL OPERATOR
      //  {x} | {x,} | {x,y}   (RE_INTERVALS && RE_NO_BK_BRACES)
      //  \{x\} | \{x,\} | \{x,y\}  (RE_INTERVALS && !RE_NO_BK_BRACES)
      else if( ( unit.ch == '{' ) && ( syntax & RE_INTERVALS ) &&
               ((( syntax & RE_NO_BK_BRACES )?RE_TRUE:RE_FALSE) ^ unit.bk) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: INTERVAL OPERATOR]\n" );
#endif DEBUG_RE

         // { without preceding token
         if( curtok == NULL )
         {
            fprintf( stderr, "{ without preceding token\n" );
            return EINVAL;
         }
         index = getminmax(pattern,plen,index,minmax,syntax);
         if( index < 0 )
            return -index;

         if( ( curtok->re_minlen() == 0 ) && ( minmax[1] == INT_MAX ) )
         {
            fprintf( stderr, "repeated argument may be empty" );
            return EINVAL;
         }

         curtok = new regex_repeated_token( curtok->re_getsubindex(), curtok,
                                            minmax[0], minmax[1] );
      }

      // LIST OPERATOR
      //  [...] | [^...]
      else if( ( unit.ch == '[') && !unit.bk )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: LIST OPERATOR]\n" );
#endif DEBUG_RE

         tok_vec *options  = new tok_vec;
         bool_t   negative = RE_FALSE;
         char     lastchar = '\0';

         if( index == plen )
         {
            fprintf( stderr, "unmatched [\n" );
            return EINVAL;
         }

         // check for initial caret, negation
         if( ( ch = pattern[index] ) == '^' )
         {
            negative = RE_TRUE;
            if( ++index == plen )
            {
               fprintf( stderr, "no end of list\n" );
               return EINVAL;
            }
            ch = pattern[index];
         }

         // check for leading right bracket literal
         if( ch == ']' )
         {
            lastchar = ch;
            if( ++index == plen )
            {
               fprintf( stderr, "no end of list\n" );
               return EINVAL;
            }
         }

         while( ( ch = pattern[index++] ) != ']' )
         {
            if( ( ch == '-' ) && ( lastchar != '\0' ) )
            {
               if( index == plen )
               {
                  fprintf( stderr, "no end of list\n" );
                  return EINVAL;
               }

               if( ( ch = pattern[index] ) == ']' )
               {
                  options->push_back(new regex_char_token(re_subindex,
                                                          lastchar,insens));
                  lastchar = '-';
               }
               else
               {
                  options->push_back(new regex_range_token(re_subindex,
                                                           lastchar,ch,insens));
                  lastchar = '\0';
                  index++;
               }
            }
            else if( ( ch == '\\' ) &&
                     ( syntax & RE_BACKSLASH_ESCAPE_IN_LISTS ) )
            {
               if( index == plen )
               {
                  fprintf( stderr, "no end of list\n" );
                  return EINVAL;
               }

               re_posix_type posixid = RE_INVALID;
               bool_t negate = RE_FALSE;
               if( syntax & RE_CHAR_CLASS_ESC_IN_LISTS )
               {
                  switch( pattern[index] )
                  {
                     case 'D':
                        negate = RE_TRUE;
                     case 'd':
                        posixid = RE_DIGIT;
                        break;
                     case 'S':
                        negate = RE_TRUE;
                     case 's':
                        posixid = RE_SPACE;
                        break;
                     case 'W':
                        negate = RE_TRUE;
                     case 'w':
                        posixid = RE_ALNUM;
                        break;
                  }
               }
               if( lastchar != '\0' )
                  options->push_back(new regex_char_token(re_subindex,lastchar,
                                                          insens));

               if( posixid != RE_INVALID )
                  options->push_back(new regex_posix_token(re_subindex,posixid,
                                                           insens,negate));
               else
                  lastchar = pattern[index];
               ++index;
            }
            else if( ( ch == '[' ) && ( syntax & RE_CHAR_CLASSES ) &&
                     ( pattern[index] == ':' ) )
            {
               char *posixset = new char[plen];
               index = getposixset(pattern, plen, index+1, posixset);
               int posixid = re_posix_charclass( posixset );
               if( posixid != -1 )
                  options->push_back(new regex_posix_token(re_subindex,posixid,
                                                           insens,RE_FALSE));
               delete posixset;
            }
            else
            {
               if( lastchar != '\0' )
                  options->push_back(new regex_char_token(re_subindex,lastchar,
                                                          insens));
               lastchar = ch;
            }

            if( index == plen )
            {
               fprintf( stderr, "no end of list\n" );
               for( unsigned int i = 0; i < options->size(); i++ )
                  delete (*options)[i];
               delete options;
               return EINVAL;
            }
         }

         // Out of list, index is one past ']'
         if( lastchar != '\0' )
            options->push_back(new regex_char_token(re_subindex,lastchar,
                                                    insens));

         // Create a new regex_oneof_token
         re_addtoken( curtok );
         curtok = new regex_oneof_token(re_subindex,options,negative);
      }

      // SUBEXPRESSIONS
      //  (...) | \(...\) depending on RE_NO_BK_PARENS
      else if( ( unit.ch == '(' ) &&
               (((syntax & RE_NO_BK_PARENS)?RE_TRUE:RE_FALSE) ^ unit.bk) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: SUBEXPRESSIONS]\n" );
#endif DEBUG_RE

         bool_t pure    = RE_FALSE;
         bool_t comment = RE_FALSE;

         if( ( ( index + 1 ) < plen ) && ( pattern[index] == '?' ) )
         {
            switch( pattern[index+1] )
            {
               case ':':
                  if( syntax & RE_PURE_GROUPING )
                  {
                     pure = RE_TRUE;
                     index += 2;
                  }
                  break;

               case '#':
                  if( syntax & RE_COMMENTS )
                     comment = RE_TRUE;
                  break;
            }
         }

         // find end of subexpression
         int endindex  = index;
         int nextindex = index;
         int nested    = 0;

         while( ( (nextindex = getcharunit(pattern,plen,endindex,unit)) > 0 ) &&
                !( ( nested == 0 ) && ( unit.ch == ')' ) &&
                   (((syntax & RE_NO_BK_PARENS)?RE_TRUE:RE_FALSE) ^ unit.bk) ) )
            if( ( endindex = nextindex ) >= plen )
            {
               fprintf( stderr, "no end of subexpression\n" );
               return EINVAL;
            }
            else if( ( unit.ch == '(' ) &&
                     (((syntax & RE_NO_BK_PARENS)?RE_TRUE:RE_FALSE) ^ unit.bk) )
               nested++;
            else if( ( unit.ch == ')' ) &&
                     (((syntax & RE_NO_BK_PARENS)?RE_TRUE:RE_FALSE) ^ unit.bk) )
               nested--;

         // endindex is now position at a ')','\)'
         // nextindex is end of string or position after ')' or '\)'

         if( comment )
            index = nextindex;
         else
         {
            // not a comment
            re_addtoken( curtok );
            if( !pure )
            {
               nextsub++;
               re_nsubs++;
            }

            int useindex = pure ? 0 : nextsub;

            char *subexpression = new char[endindex-index+1];
            memcpy( subexpression, pattern + index, endindex - index );
            subexpression[endindex-index] = '\0';
            regex *tok = new regex();
            tok->re_construct( subexpression, cflags, syntax, useindex,
                               nextsub );
            delete subexpression;
            curtok = tok;
            nextsub += tok->re_nsubs;
            re_nsubs += tok->re_nsubs;
            index = nextindex;
         }
      }

      // UNMATCHED RIGHT PAREN
      else if( !( syntax & RE_UNMATCHED_RIGHT_PAREN_ORD ) &&
               ( unit.ch == ')' ) &&
               (((syntax & RE_NO_BK_PARENS)?RE_TRUE:RE_FALSE) ^ unit.bk) )
      {
         fprintf( stderr, "unmatched right paren\n" );
         return EINVAL;
      }

      // START OF LINE OPERATOR
      //   ^
      else if( ( unit.ch == '^' ) && !unit.bk )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: START OF LINE OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = NULL;
         re_addtoken(new regex_start_token(re_subindex,
                                           (cflags & REG_MULTILINE)?RE_TRUE
                                                                   :RE_FALSE));
      }

      // END OF LINE OPERATOR
      //  $
      else if( ( unit.ch == '$' ) && !unit.bk )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: END OF LINE OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = NULL;
         re_addtoken(new regex_end_token(re_subindex,
                                         (cflags & REG_MULTILINE)?RE_TRUE
                                                                 :RE_FALSE));
      }

      // MATCH-ANY-CHARACTER OPERATOR (except possibly newline and null)
      //  .
      else if( ( unit.ch == '.' ) && !unit.bk )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: MATCH-ANY-CHARACTER OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_any_token(re_subindex,
                                      ((syntax & RE_DOT_NEWLINE) ||
                                       (syntax & REG_DOT_NEWLINE))?RE_TRUE
                                                                  :RE_FALSE,
                                      (syntax & RE_DOT_NOT_NULL)?RE_TRUE
                                                                :RE_FALSE);
      }

      // ZERO-OR-MORE REPEAT OPERATOR
      //  *
      else if( ( unit.ch == '*' ) && !unit.bk )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: ZERO-OR-MORE REPEAT OPERATOR]\n" );
#endif DEBUG_RE

         if( ( curtok == NULL ) || ( curtok->re_minlen() == 0 ) )
         {
            fprintf( stderr, "repeated argument may be empty\n" );
            return EINVAL;
         }

         curtok = new regex_repeated_token( re_subindex, curtok, 0, INT_MAX );
      }

      // ONE-OR-MORE REPEAT OPERATOR
      //  + | \+ depending on RE_BK_PLUS_QM
      //  not available if RE_LIMITED_OPS is set
      else if( ( unit.ch == '+' ) && !(syntax & RE_LIMITED_OPS) &&
               ((!(syntax & RE_BK_PLUS_QM)?RE_TRUE:RE_FALSE) ^ unit.bk) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: ONE-OR-MORE REPEAT OPERATOR]\n" );
#endif DEBUG_RE

         if( ( curtok == NULL ) || ( curtok->re_minlen() == 0 ) )
         {
            fprintf( stderr, "repeated argument may be empty\n" );
            return EINVAL;
         }

         curtok = new regex_repeated_token( re_subindex, curtok, 1, INT_MAX );
      }

      // ZERO-OR-ONE REPEAT OPERATOR / STINGY MATCHING OPERATOR
      //  ? | \? depending on RE_BK_PLUS_QM
      //  not available if RE_LIMITED_OPS is set
      //  stingy matching if RE_STINGY_OPS is set and it follows a quantifier
      else if( ( unit.ch == '?') && !(syntax & RE_LIMITED_OPS) &&
               ((!(syntax & RE_BK_PLUS_QM)?RE_TRUE:RE_FALSE) ^ unit.bk) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: ZERO-OR-ONE REPEAT OPERATOR]\n" );
#endif DEBUG_RE

         if( ( curtok == NULL ) || ( curtok->re_minlen() == 0 ) )
         {
            fprintf( stderr, "repeated argument may be empty\n" );
            return EINVAL;
         }

         // Check for stingy matching on regex_repeated_token
         if( curtok->re_canstingy() && ( syntax & RE_STINGY_OPS ) )
            curtok->re_setstingy();
         else
            curtok = new regex_repeated_token( re_subindex, curtok, 0, 1 );
      }

      // START OF STRING OPERATOR
      //  \A if RE_STRING_ANCHORS is set
      else if( unit.bk && ( unit.ch == 'A' ) && (syntax & RE_STRING_ANCHORS) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: START OF STRING OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_start_token( re_subindex, RE_FALSE );
      }

      // DIGIT OPERATOR
      //  \d if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 'd' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: DIGIT OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_DIGIT,insens,RE_FALSE);
      }

      // NON-DIGIT OPERATOR
      //  \D if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 'D' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: NON-DIGIT OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_DIGIT,insens,RE_TRUE);
      }

      // NEWLINE ESCAPE
      //  \n
      else if( unit.bk && ( unit.ch == 'n' ) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: NEWLINE ESCAPE]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_char_token(re_subindex,'\n',RE_FALSE);
      }

      // RETURN ESCAPE
      //  \r
      else if( unit.bk && ( unit.ch == 'r' ) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: RETURN ESCAPE]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_char_token(re_subindex,'\r',RE_FALSE);
      }

      // WHITESPACE OPERATOR
      //  \s if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 's' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: WHITESPACE OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_SPACE,insens,RE_FALSE);
      }

      // NON-WHITESPACE OPERATOR
      //  \S if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 'S' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: NON-WHITESPACE OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_SPACE,insens,RE_TRUE);
      }

      // TAB ESCAPE
      //  \t
      else if( unit.bk && ( unit.ch == 't' ) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: TAB ESCAPE]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_char_token(re_subindex,'\t',RE_FALSE);
      }

      // ALPHANUMERIC OPERATOR
      //  \w if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 'w' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: ALPHANUMERIC OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_ALNUM,insens,RE_FALSE);
      }

      // NON-ALPHANUMERIC OPERATOR
      //  \W if RE_CHAR_CLASS_ESCAPES is set
      else if( unit.bk && ( unit.ch == 'W' ) &&
               (syntax & RE_CHAR_CLASS_ESCAPES) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: NON-ALPHANUMERIC OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_posix_token(re_subindex,RE_ALNUM,insens,RE_TRUE);
      }

      // END OF STRING OPERATOR
      //  \Z if RE_STRING_ANCHORS is set
      else if( unit.bk && ( unit.ch == 'Z' ) &&
               (syntax & RE_STRING_ANCHORS) )
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: END OF STRING OPERATOR]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_end_token( re_subindex, RE_FALSE );
      }

      // NON-SPECIAL CHARACTER (or escape to make literal)
      //  c | \* for example
      else
      {
#ifdef DEBUG_RE
         fprintf( stderr, "[DEBUG: NON-SPECIAL CHARACTER]\n" );
#endif DEBUG_RE

         re_addtoken( curtok );
         curtok = new regex_char_token( re_subindex, unit.ch, insens );
      }
   }

   // Add final buffered token if applicable
   re_addtoken( curtok );

   if( branches != NULL )
   {
      branches->push_back(new regex(re_firsttok,re_lasttok,re_nsubs,
                                    re_subindex));
      re_firsttok = re_lasttok = new regex_oneof_token( re_subindex, branches,
                                                        RE_FALSE );
   }

#ifdef DEBUG_RE
   fprintf( stderr, "[DEBUG: DONE SUCCESSFUL]\n" );
#endif DEBUG_RE
   return 0;
}

void
regex::re_addtoken( regex_token *next )
{
   if( next == NULL ) return;

   if( re_firsttok == NULL )
      re_lasttok = re_firsttok = next;
   else
      if( re_lasttok->re_chain(next) ) re_lasttok = next;
}

i_vec *
regex::re_match( regex_charindex &input, int index, int eflags )
{
   if( re_firsttok == NULL )
   {
      i_vec *ivec = new i_vec;
      ivec->push_back( index );
      return ivec;
   }

   i_vec *newindex = re_firsttok->re_match( input, index, eflags );
   if( newindex != NULL )
   {
      i_vec *doables = new i_vec;
      for( unsigned int i = 0; i < newindex->size(); i++ )
      {
         i_vec *thisresult = re_next( input, (*newindex)[i], eflags );
         if( thisresult != NULL )
         {
            for( unsigned int j = 0; j < thisresult->size(); j++ )
               doables->push_back( (*thisresult)[j] );
            delete thisresult;
         }
      }
      if( doables->size() == 0 )
         delete doables;
      else
         return doables;
   }
   return NULL;
}

bool_t
regex::re_ismatch( regex_charindex &input, int index, int eflags )
{
   if( re_firsttok == NULL )
      return ((input.re_str == NULL) ||
              (input.re_str[0] == '\0'))?RE_TRUE:RE_FALSE;

   for( int i = 0; i < input.re_index; i++ )
   {
      i_vec *ivec = re_firsttok->re_match( input, i, eflags );
      if( ivec != NULL )
      {
         delete ivec;
         return RE_TRUE;
      }
   }
   return RE_FALSE;
}

void
regex::re_setuncle( regex_token *uncle )
{
   re_lasttok->re_setuncle( uncle );
}

bool_t
regex::re_chain( regex_token *next )
{
   regex_token::re_chain( next );
   if( re_lasttok != NULL )
      re_lasttok->re_setuncle( next );
   return RE_TRUE;
}

void
regex::re_dump( FILE *f )
{
   fprintf( f, "(" );
   if( re_subindex == 0 )
      fprintf( f, "?:" );
   if( re_firsttok != NULL )
      re_firsttok->re_dumpall( f );
   fprintf( f, ")" );
}

