/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re_char.cc,v 1.13 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <stdlib.h>
#include <ctype.h>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

regex_char_token::regex_char_token( int subindex, char ch, bool_t insens )
   : regex_token( subindex )
{
   re_insens = insens;
   re_str    = new char[2];
   re_str[0] = ch;
   re_str[1] = '\0';
   re_len    = 1;
}

regex_char_token::~regex_char_token()
{
   if( re_str != NULL )
      delete re_str;
}

re_token_type
regex_char_token::re_gettype()
{
   return RE_CHAR;
}

int
regex_char_token::re_minlen()
{
   return re_len;
}

bool_t
regex_char_token::re_isinsens()
{
   return re_insens;
}

int
regex_char_token::re_getlen()
{
   return re_len;
}

const char *
regex_char_token::re_getstr()
{
   return (const char *)re_str;
}

i_vec *
regex_char_token::re_match( regex_charindex &input, int index, int eflags )
{
   for( unsigned int i = 0; i < re_len; i++ )
   {
      char ch = input.re_str[index+i];
      if( (re_insens?tolower(ch):ch) != re_str[i] )
         return NULL;
   }
   return re_next(input,index+re_len,eflags);
}

bool_t
regex_char_token::re_chain( regex_token *next )
{
   if( next->re_gettype() == RE_CHAR )
   {
      int newsize = re_len + next->re_minlen();

      char *newstr = new char [newsize+1];

      memcpy( newstr, re_str, re_len );

      regex_char_token *nexttok = (regex_char_token *)next;
      memcpy( newstr + re_len, nexttok->re_str, nexttok->re_minlen() );
      newstr[newsize] = '\0';

      delete re_str;
      re_str = newstr;
      re_len = newsize;

      return RE_FALSE;
   }
   else
      return regex_token::re_chain( next );
}

void
regex_char_token::re_dump( FILE *f )
{
   fprintf( f, re_str );
}

