/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re_oneof.cc,v 1.15 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <stdlib.h>
#include <limits.h>

#include <vector>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

typedef std::vector <int> i_vec;
typedef std::vector <i_vec *> iv_vec;

regex_oneof_token::regex_oneof_token( int subindex, char *options,
                                      bool_t negative, bool_t insens )
   : regex_token( subindex )
{
   re_negative = negative;

   re_options = new tok_vec();

   int optlen = strlen(options);
   for( int i = 0; i < optlen; i++ )
      re_options->push_back( new regex_char_token(subindex,options[i],insens) );
}

regex_oneof_token::regex_oneof_token( int subindex, tok_vec *options,
                                      bool_t negative )
   : regex_token( subindex ), re_options( options )
{
   re_negative = negative;
}

regex_oneof_token::~regex_oneof_token()
{
   if( re_options != NULL )
   {
      if( re_options->size() > 0 )
         for( unsigned int i = 0; i < re_options->size(); i++ )
             delete (*re_options)[i];
      delete re_options;
   }
}

re_token_type
regex_oneof_token::re_gettype()
{
   return RE_ONEOF;
}

bool_t
regex_oneof_token::re_isnegated()
{
   return re_negative;
}

regex_token *
regex_oneof_token::operator[]( int index )
{
   return (*re_options)[index];
}

int
regex_oneof_token::re_children()
{
   return re_options->size();
}

int
regex_oneof_token::re_minlen()
{
   int min = INT_MAX;

   for( unsigned int i = 0; i < re_options->size(); i++ )
   {
      int x = (*re_options)[i]->re_minlen();
      if( x < min )
         min = x;
   }
   return min;
}

i_vec *
regex_oneof_token::re_match( regex_charindex &input, int index, int eflags )
{
   i_vec *newindex;
   i_vec *possibles = new i_vec;

   regex_token *tk;
   unsigned int i;
   for( i = 0; i < re_options->size(); i++ )
   {
      tk = (*re_options)[i];
      newindex = tk->re_match(input,index,eflags);

      if( newindex != NULL )
      {
         if( re_negative )
         {
            delete newindex;
            delete possibles;
            return NULL;
         }

         for( unsigned int j = 0; j < newindex->size(); j++ )
            possibles->push_back( (*newindex)[j] );
         delete newindex;
      }
   }

   i_vec *doables = new i_vec;
   for( i = 0; i < possibles->size(); i++ )
   {
      newindex = re_next(input,(*possibles)[i],eflags);
      if( newindex != NULL )
      {
         for( unsigned int j = 0; j < newindex->size(); j++ )
            doables->push_back( (*newindex)[j] );
         delete newindex;
      }
   }
   delete possibles;

   if( doables->size() > 0 )
   {
      if( re_negative )
      {
         delete doables;
         return NULL;
      }
      else
         return doables;
   }
   else
   {
      delete doables;
      return re_negative?re_next(input,index+1,eflags):NULL;
   }
}

bool_t
regex_oneof_token::re_chain( regex_token *next )
{
   regex_token::re_chain( next );
   for( unsigned int i = 0; i < re_options->size(); i++ )
      (*re_options)[i]->re_setuncle( next );
   return RE_TRUE;
}

void
regex_oneof_token::re_dump( FILE *f )
{
   fprintf( f, re_negative?"[^":"(?:" );
   for( unsigned int i = 0; i < re_options->size(); i++ )
   {
      if( !re_negative && ( i > 0 ) )
         fprintf( f, "|" );
      (*re_options)[i]->re_dumpall( f );
   }
   fprintf( f, re_negative?"]":")" );
}

