/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re_posix.cc,v 1.15 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <wr_conf.h>
#include <stdlib.h>
#include <ctype.h>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

static char *_posix_charclass[] =
{
   "alnum", "alpha", "blank", "cntrl", "digit", "graph", "lower",
   "print", "punct", "space", "upper", "xdigit"
};

static int _posix_ncharclass = sizeof(_posix_charclass) / sizeof(char *);

#if !defined(WR_FREEBSD)
int
isblank( const char c )
{
   return( ( c == ' ' ) || ( c == '\t' ) );
}
#endif

int
re_posix_charclass( char *ccname )
{
   int i;
   for( i = 0; i < _posix_ncharclass; i++ )
      if( !strcmp( _posix_charclass[i], ccname ) )
         return i;
   return -1;
}

regex_posix_token::regex_posix_token( int subindex, int type, bool_t insens,
                                      bool_t negated )
   : regex_token( subindex )
{
   re_type    = type;
   re_insens  = insens;
   re_negated = negated;
}

regex_posix_token::~regex_posix_token() {}

int
regex_posix_token::re_minlen()
{
   return 1;
}

re_token_type
regex_posix_token::re_gettype()
{
   return RE_POSIX;
}

int
regex_posix_token::re_getposixtype()
{
   return re_type;
}

bool_t
regex_posix_token::re_isinsens()
{
   return re_insens;
}

bool_t
regex_posix_token::re_isnegated()
{
   return re_negated;
}

i_vec *
regex_posix_token::re_match( regex_charindex &input, int index, int eflags )
{
   char ch = input.re_str[index];
   if( ch == '\0' ) 
      return NULL;

   bool_t retval = RE_FALSE;
   switch( re_type )
   {
      case RE_ALNUM:
         retval = isalnum(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_ALPHA:
         retval = isalpha(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_BLANK:
         retval = isblank(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_CNTRL:
         retval = iscntrl(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_DIGIT:
         retval = isdigit(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_GRAPH:
         retval = isgraph(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_LOWER:
         retval = islower(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_PRINT:
         retval = isprint(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_PUNCT:
         retval = ispunct(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_SPACE:
         retval = isspace(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_UPPER:
         retval = isupper(ch)?RE_TRUE:RE_FALSE;
         break;
      case RE_XDIGIT:
         retval = isxdigit(ch)?RE_TRUE:RE_FALSE;
         break;
   }
   if( re_negated ) retval = retval?RE_FALSE:RE_TRUE;
   if( retval ) return re_next(input, index+1, eflags);
   return NULL;
}

void
regex_posix_token::re_dump( FILE *f )
{
   if( re_negated )
      fprintf( f, "^" );
   fprintf( f, "[:%s:]", _posix_charclass[re_type] );
}

