/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re_range.cc,v 1.11 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <stdlib.h>
#include <ctype.h>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

regex_range_token::regex_range_token( int subindex, char lo, char hi,
                                      bool_t insens )
   : regex_token( subindex )
{
   re_lo     = lo;
   re_hi     = hi;
   re_insens = insens;
}

regex_range_token::~regex_range_token() {}

re_token_type
regex_range_token::re_gettype()
{
   return RE_RANGE;
}

int
regex_range_token::re_minlen()
{
   return 1;
}

bool_t
regex_range_token::re_isinsens()
{
   return re_insens;
}

char
regex_range_token::re_getlo()
{
   return re_lo;
}

char
regex_range_token::re_gethi()
{
   return re_hi;
}

i_vec *
regex_range_token::re_match( regex_charindex &input, int index, int eflags )
{
   char ch = input.re_str[index];
   if( ch == '\0' ) return NULL;
   if( re_insens ) ch = tolower(ch);
   return ((ch >= re_lo) && (ch <= re_hi))?re_next(input,index+1,eflags):NULL;
}

void
regex_range_token::re_dump( FILE *f )
{
   fprintf( f, "%c-%c", re_lo, re_hi );
}

