/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 * 
 * $Id: re_repeated.cc,v 1.13 2001/04/05 18:41:32 stevek Exp $
 */
#include <wr_conf.h>

#include <stdlib.h>
#include <ctype.h>

#include <vector>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libre.h"

typedef std::vector <int> i_vec;
typedef std::vector <i_vec *> iv_vec;

regex_repeated_token::regex_repeated_token( int subindex, regex_token *tok,
                                            int min, int max )
   : regex_token( subindex, RE_TRUE )
{
   re_tok = tok;
   re_min = min;
   re_max = max;
}

regex_repeated_token::~regex_repeated_token()
{
   if( re_tok != NULL )
      delete re_tok;
}

re_token_type
regex_repeated_token::re_gettype()
{
   return RE_REPEAT;
}

int
regex_repeated_token::re_minlen()
{
   return (re_min * re_tok->re_minlen());
}

int
regex_repeated_token::re_getmin()
{
   return re_min;
}

int
regex_repeated_token::re_getmax()
{
   return re_max;
}

regex_token *
regex_repeated_token::operator[]( int index )
{
   return( (index==0)?re_tok:NULL );
}

int
regex_repeated_token::re_children()
{
   return( (re_tok==NULL)?0:1 );
}

i_vec *
regex_repeated_token::re_match( regex_charindex &input, int index, int eflags )
{
   int nrepeats = 0;
   i_vec *newindex = new i_vec;
   newindex->push_back( index );

   iv_vec pos;
   do
   {
      pos.push_back( newindex );
      if( re_stingy && ( nrepeats >= re_min ) )
         for( unsigned int i = 0; i < newindex->size(); i++ )
         {
            i_vec *s = re_next(input,(*newindex)[i],eflags);
            if( s != NULL )
            {
               for( unsigned int j = 0; j < pos.size(); j++ )
                  delete( pos[j] );
               return s;
            }
         }

      i_vec *doables = new i_vec;
      i_vec *thisresult;
      for( unsigned int i = 0; i < newindex->size(); i++ )
      {
         thisresult = re_tok->re_match(input,(*newindex)[i],eflags);
         if( thisresult != NULL )
         {
            for( unsigned int j = 0; j < thisresult->size(); j++ )
               doables->push_back( (*thisresult)[j] );
            delete thisresult;
         }
      }
      if( doables->size() == 0 )
      {
         delete doables;
         break;
      }

      newindex = doables;
   }
   while( nrepeats++ < re_max );

   if( nrepeats < re_min )
   {
      for( unsigned int i = 0; i < pos.size(); i++ )
         delete pos[i];
      return NULL;
   }

   int posindex = pos.size();

   i_vec *doneindex;
   while( --posindex >= re_min )
   {
      newindex = pos[posindex];
      for( unsigned int i = 0; i < newindex->size(); i++ )
      {
         doneindex = re_next(input,(*newindex)[i],eflags);
         if( doneindex != NULL )
         {
            for( unsigned int i = 0; i < pos.size(); i++ )
               delete pos[i];
            return doneindex;
         }
      }
   }

   for( unsigned int i = 0; i < pos.size(); i++ )
      delete pos[i];
   return NULL;
}

void
regex_repeated_token::re_dump( FILE *f )
{
   fprintf( f, "(" );
   if( re_tok->re_getsubindex() == 0 )
      fprintf( f, "?:" );
   re_tok->re_dumpall( f );
   fprintf( f, ")" );
   if( ( re_max == INT_MAX ) && ( re_min <= 1 ) )
      fprintf( f, (re_min == 0)?"*":"+" );
   else if( ( re_min == 0 ) && ( re_max == 1 ) )
      fprintf( f, "?" );
   else
   {
      fprintf( f, "{%d", re_min );
      if( re_max > re_min )
      {
         fprintf( f, "," );
         if( re_max != INT_MAX )
            fprintf( f, "%d", re_max );
      }
      fprintf( f, "}" );
   }
   if( re_stingy )
      fprintf( f, "?" );
}

