/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: main.cc,v 1.6 2001/04/05 18:41:33 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Driver program for testing the regular expression library.
 *
 */

#define LINKAGE IMPORT

#include <libre.h>

static const char *usage_str = "\
testre <regex> [<match1>]\n\
\n\
   Processes <regex> and performs a match run against <match string>, if\n\
   provided.  Otherwise it starts in interactive mode where you can enter\n\
   match strings.";

static void
Usage()
{
   fprintf( stderr, "%s\n", usage_str );
}

void
Match( regex *re, const char *str )
{
   regex_charindex rci;
   rci.re_str = (char *)str;
   rci.re_index = strlen(str);
   if( re->re_ismatch( rci ) )
      printf("%s: Matched\n", str);
   else
      printf("%s: Did not match\n", str);
}

int
main( int argc, char *argv[] )
{
   regex *re;

   if( argc <= 1 )
   {
      Usage();
      return 0;
   }

   re = new regex();
   re->re_construct( argv[1] );

   if( argc > 2 )
      Match( re, argv[2] );
   else
   {
      char buf[1024];

      printf( "testre> " );
      fflush( stdin );

      while( fgets( buf, 1024, stdin ) )
      {
         int i = strlen(buf);
         do { i--; } while( (i >= 0) && (buf[i] == '\n' || buf[i] == '\r') );
         if( i >= 0 )
         {
            buf[i+1] = '\0';

            if( !strcmp( buf, "quit" ) ) break;

            Match( re, buf );
         }

         printf( "testre> " );
         fflush( stdin );
      }
   }

   return 0;
}

