/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: codefrags.h,v 1.130.2.3 2001/08/07 00:01:38 stevek Exp $
 *
 * Authors: Lee Badger, Mark Feldman, Ben Uecker
 *
 * Purpose: Static code fragments used by the source object in codegen.cc.
 *          Keeping them in here keeps things cleaner.
 */


/* standard preamble for a wrapper */
static char *std_includes = "\n"
"\n"
"/*\n"
" * Generated wrapper's C source.  Do not edit.\n"
" */\n"
"\n"
"#ifndef PD_IN_KERNEL\n"
"#define PD_IN_KERNEL\n"
"#endif\n"
"#define WSSLINKAGE IMPORT\n"
"\n"
"\n"
"struct wr_module;\n"
"\n"
"#ifdef WSSLINKAGE\n"
"#undef WSSLINKAGE\n"
"#endif\n"
"#define WSSLINKAGE IMPORT\n"
"\n"
#if defined(WR_SOLARIS)
"#include <sys/socket.h>\n"
#endif

#if PD_KTHREAD_SUPPORT
"#define CUR_THREAD_ID wr_curthreadid()\n"
#else
"#define CUR_THREAD_ID 0\n"
#endif

;

/*=========================================================================*/

/*  This is a copy of wr_iface.h and wr_proc.h,
 *  generated in the Makefile.
 */

 
/*=========================================================================*/

static char *src_header = "\n"
"\n"
"/* global variables used in string matching and replacement */\n"
"pd_char_t *gstr_beg, *gstr_end, *gstr;\n"
"\n"
"int myversion = 1;\n"
"int myhandle = 0;\n"
"\n"
"/*\n"
" * Function declarations.\n"
" */\n"
"int IMPORT wr_wrapper_loaded(char *name);\n"
"int IMPORT wr_register_wrapper(int handle, char *name, void *pre[],\n"
"                           void *post[], void *activate,\n"
"			   void *deactivate, void *duplicate,\n"
"			   void *newprivdata);\n"
"int IMPORT wr_rm_wrapper(int handle);\n"
"int IMPORT wr_at_unload(void *uf, int handle);\n"
"int IMPORT wr_rm_at_unload(int handle);\n"
"#if defined(MALLOC_DEBUG)\n"
"#define wr_strdup(str) (wr_strdup_debug((str), __FILE__, __LINE__))\n"
"#define wr_malloc(size) (wr_malloc_debug((size), __FILE__, __LINE__))\n"
"#endif\n"
"\n"
#if 0 /* move to libwr.h */
"int IMPORT wr_arg_replace(struct wss_proc *wrp, int handle, int space, int code, void *uap[], void *new, int index, size_t size);\n"
#endif
"int IMPORT wr_arg_copy(void *uap[], void *new, int index, size_t size);\n"
"\n"
#if 0 /* move to libwr.h, Calvin */
"void IMPORT wr_bcopy(const void *src, void *dst, size_t len);\n"
"void IMPORT wr_bcopy(const void *src, void *dst, size_t len);\n"
"int IMPORT wr_printf(const char *, ...);\n"
"long IMPORT wr_get_curtime(int *);\n"
#endif /* 0 libwr.h */
"char IMPORT *wr_get_osname(void);\n"
"int IMPORT wr_get_osvers(void);\n"
"int IMPORT wr_get_wrvers(void);\n"
#if 0 /* Moved to libwr.h */
"const pd_char_t *wr_syscall_name(int call);\n"

"int IMPORT wr_match(char *src, char *pat, char **beg, char **end);\n"
"int IMPORT wr_substitute(char **src, char *txt, char *sub);\n"

"int IMPORT wr_copyin(void* uaddr, void *kaddr, size_t len);\n"
"int IMPORT wr_copyout(void* kaddr, void *uaddr, size_t len);\n"
"int IMPORT wr_copyinstr(void* uaddr, void *kaddr, size_t len, unsigned int *done);\n"
#endif /* 0 libwr.h */
"void IMPORT *wr_dupin_ntermlist(void *uaddr);\n"
"void IMPORT *wr_dupout_ntermlist(void *kaddr);\n"
"void IMPORT *wr_dupin(void *uaddr, size_t size);\n"
"void IMPORT *wr_dupout(void *kaddr, size_t size);\n"
"int IMPORT wr_replace_ntermlist(struct wss_proc *wrp, void *uap[], void *new, int index);\n"
"void IMPORT wr_free_ntermlist(void *kaddr);\n"
"\n"
#if 0 /* Moved to libwr.h */
"void IMPORT wr_mutex_init(pd_mutex_t *mutex, const char *name);\n"
"void IMPORT wr_mutex_destroy(pd_mutex_t *mutex);\n"
"void IMPORT wr_mutex_enter(pd_mutex_t *mutex);\n"
"void IMPORT wr_mutex_exit(pd_mutex_t *mutex);\n"
"\n"
"string IMPORT wr_str_new(const char *s);\n"
"string IMPORT wr_str_add(string s1, string s2);\n"
"int    IMPORT wr_str_equ(string s1, string s2);\n"
"string IMPORT wr_str_assign (string s, string rhs);\n"
#endif /* 0 libwr.h */
"void       IMPORT wr_file_close( int id, wss_file_t file );\n"
"size_t     IMPORT wr_file_get_size(int id, wss_file_t file );\n"
"void       IMPORT *wr_file_get_map_addr(int id, wss_file_t handle );\n"
"wss_file_t IMPORT wr_file_open(int id, const pd_char_t*filename,\n"
"                               wss_file_mode_t mode);\n"
"int        IMPORT wr_file_mmap(int id, wss_file_t file,size_t s,\n"
"                               int mapmode,pd_offset_t pos);\n"
"int        IMPORT wr_file_read(int id, wss_file_t file,pd_byte_t *buffer,\n"
"                               size_t len);\n"
"int        IMPORT wr_file_seek(int id, wss_file_t file,pd_offset_t offset,\n"
"                               int whence);\n"
"pd_byte_t IMPORT *wr_file_read_alloc(int id,wss_file_t file, size_t *len );\n"
"int        IMPORT wr_file_write(int id, wss_file_t file,pd_byte_t *buffer,\n"
"                                size_t len);\n"
"unsigned int IMPORT wr_curthreadid(void);\n"
"\n"
"/* Define some of the wdl environment variables to data in the proc\n"
"structure.  Other environment variables can't be created this way. */\n"
"\n"
"#if defined(WR_SOLARIS)\n"
"#define _descriptors   wrp->wrp_p->p_fd->fd_nfiles\n"
"#define _niceval       wrp->wrp_p->p_nice\n"
"#define _priority      wrp->wrp_p->p_priority\n"
"#define _sid           (wrp->wrp_p->p_sessp?wrp->wrp_p->p_sessp->s_sid:0)\n"
"#define _pid           wrp->wrp_p->p_pid\n"
"#define _pgid          (wrp->wrp_p->p_parent?wrp->p->parent->p_cred->cr_rgid:-1)\n"
"#define _pegid         (wrp->wrp_p->p_parent?wrp->p->parent->p_cred->cr_gid:-1)\n"
"#define _puid          (wrp->wrp_p->p_parent?wrp->p->parent->p_cred->cr_ruid:-1)\n"
"#define _peuid         (wrp->wrp_p->p_parent?wrp->p->parent->p_cred->cr_uid:-1)\n"
"#define _ppid          (wrp->wrp_p->p_ppid)\n"
"#define _progname      wrp->wrp_progname\n"
"#define _path          wrp->wrp_path\n"
"#define _uid           wrp->wrp_p->p_cred->cr_ruid\n"
"#define _euid          wrp->wrp_p->p_cred->cr_uid\n"
"#define _svuid         wrp->wrp_p->p_cred->cr_suid\n"
"#define _gid           wrp->wrp_p->p_cred->cr_rgid\n"
"#define _egid          wrp->wrp_p->p_cred->cr_gid\n"
"#define _svgid         wrp->wrp_p->p_cred->cr_sgid\n"
"#elif defined(WR_FREEBSD)\n"
"#define _descriptors   wrp->wrp_p->p_fd->fd_nfiles\n"
"#define _niceval       wrp->wrp_p->p_nice\n"
"#define _priority      wrp->wrp_p->p_priority\n"
"#define _sid           (wrp->wrp_p->p_pgrp?wrp->wrp_p->p_pgrp->pg_session->s_sid:0)\n"
"#define _tty           ((wrp->wrp_p->p_pgrp&&wrp->wrp_p->p_pgrp->pg_session->s_ttyp)?wrp->wrp_p->p_pgrp->pg_session->s_ttyp->t_dev->si_udev:0)\n"
"#define _pid           wrp->wrp_p->p_pid\n"
"#define _pgid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->p_cred->p_rgid:-1)\n"
"#define _pegid         (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->p_cred->p_svgid:-1)\n"
"#define _puid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->p_cred->p_ruid:-1)\n"
"#define _peuid         (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->p_cred->p_svuid:-1)\n"
"#define _ppid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->p_pid:0)\n"
"#define _progname      wrp->wrp_progname\n"
"#define _path          wrp->wrp_path\n"
"#define _uid           wrp->wrp_p->p_cred->p_ruid\n" 
"#define _euid          wrp->wrp_p->p_ucred->cr_uid\n"
"#define _svuid         wrp->wrp_p->p_cred->p_svuid\n"
"#define _gid           wrp->wrp_p->p_cred->p_rgid\n"
"#define _egid          wrp->wrp_p->p_ucred->cr_groups[0]\n" 
"#define _svgid         wrp->wrp_p->p_cred->p_svgid\n" 
"#elif defined(WR_WINNT)\n"
"#define _pid           wrp->wrp_p->p_pid\n"
"#define _ppid          wrp->wrp_p->p_ppid\n"
"#define _uid           wrp->wrp_p->p_user_name\n"
"#define _progname      wrp->wrp_progname\n"
"#define _path          wrp->wrp_path\n"
"#elif defined(WR_LINUX)\n"
"#define _descriptors   wrp->wrp_p->files->fd\n"
"#define _niceval       wrp->wrp_p->p_nice\n"
"#define _priority      wrp->wrp_p->priority\n"
"#define _pid           wrp->wrp_p->pid\n"
"#define _pgid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->gid:-1)\n"
"#define _pegid         (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->egid:-1)\n"
"#define _puid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->uid:-1)\n"
"#define _peuid         (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->euid:-1)\n"
"#define _ppid          (wrp->wrp_p->p_pptr?wrp->wrp_p->p_pptr->pid:0)\n"
"#define _tty           (wrp->wrp_p->tty?kdev_t_to_nr(wrp->wrp_p->tty->device):0)\n"
"#define _sid           wrp->wrp_p->session\n"
"#define _progname      wrp->wrp_progname\n"
"#define _path          wrp->wrp_path\n"
"#define _uid           wrp->wrp_p->uid\n" 
"#define _euid          wrp->wrp_p->euid\n"
"#define _gid           wrp->wrp_p->gid\n"
"#define _egid          wrp->wrp_p->egid\n" 
"#else\n"
"#error unsupported target.\n"
"#endif\n"
"\n"
"\n";         

static char *src_header1 = "\n"
"\n"
"#define wr_mycontext()          myhandle\n"
"#define wr_close(f)             wr_file_close(myhandle, (f))\n"
"#define wr_size(f)              wr_file_get_size(myhandle, (f))\n"
"#define wr_mmap_addr(f)         wr_file_get_map_addr(myhandle, (f))\n"
"#define wr_open(n,m)            wr_file_open(myhandle, (n), (m))\n"
"#define wr_mmap(f,s,m,p)        wr_file_mmap(myhandle, (f), (s), (m), (p))\n"
"#define wr_read(f,b,l)          wr_file_read(myhandle, (f), (b), (l))\n"
"#define wr_read_alloc(f,l)      wr_file_read_alloc(myhandle, (f), (l))\n"
"#define wr_seek(f,o,w)          wr_file_seek(myhandle, (f), (o), (w))\n"
"#define wr_write(f,b,l)         wr_file_write(myhandle, (f), (b), (l))\n"
"\n"
"#define wr_skip()		 data->skip = 1;\n"
"#define _wrp			 wrp\n"
"#define _data			 data\n"
"\n"
#if 0
"static string wr_str_new(const char *s);\n"
"static string wr_str_add(string s1, string s2);\n"
"static int   wr_str_equ(string s1, string s2);\n"
"static string wr_str_assign(string s, string rhs);\n"
"\n"
"\n"
"static string wr_str_assign(string s, string rhs)\n"
"\n"
"{\n"
"     int len = wr_strlen(rhs) + 1;\n"
"     s = (string) wr_malloc(len);\n"
"     wr_strcpy(s, rhs);\n"
"     return s;\n"
"}\n"
"\n"
"static string wr_str_add(string s1, string s2)\n"
"\n"
"{\n"
"     int len = wr_strlen(s1) + wr_strlen(s2) + 1;  \n"
"     int bufsize = len >> 5 == 0 ? len : ((len >> 5) + 1) << 5;\n"
"     string ret;\n"
"     ret = (string) wr_malloc(bufsize);\n"
"     wr_strcpy(ret, s1);\n"
"     wr_strcat(ret, s2);\n"
"     return ret;\n"
"}\n"
"\n"
"static int wr_str_equ(string s1, string s2)\n"
"{\n"
"     return !wr_strcmp(s1, s2);\n"
"}\n"
#endif /* 0  string stuff */
"\n"
"static databuf_t buf_new()\n"
"{\n"
"     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));\n"
"     buf->size = 0;\n"
"     buf->max = 0;\n"
"     buf->buf = NULL;\n"
"     return buf;\n"
"}\n"
"\n"
#if 0
"static databuf_t buf_newstr(string s)\n"
"{\n"
"     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));\n"
"     buf->size = wr_strlen(s) + 1;\n"
"     buf->max = ((buf->size >> 5) + 1) << 5;\n"
"     buf->buf = (char *) wr_malloc(buf->max);\n"
"     wr_strncpy(buf->buf, s, buf->max);\n"
"     // if (buf->buf == NULL) handle_error : TBD\n"
"     return buf;\n"
"}\n"
#endif
"\n"
"static databuf_t buf_newdata(char *data, int size)\n"
"{\n"
"     databuf_t buf = (databuf_t) wr_malloc(sizeof(*buf));\n"
"     buf->size = size;\n"
"     buf->max = ((buf->size >> 5) + 1) << 5;\n"
"     buf->buf = (char *) wr_malloc(buf->max);\n"
"     wr_copyin(data, buf->buf, size);\n"
"     return buf;\n"
"}\n"
"\n"
"static void buf_del(databuf_t buf)\n"
"\n"
"{\n"
"     if (buf->buf != NULL) wr_free(buf->buf);\n"
"     wr_free(buf);\n"
"}\n"
"\n"
"static int buf_replace_data(databuf_t buf, const char *data, int len)\n"
"\n"
"{\n"
"     if (len > buf->max) {\n"
"          wr_free(buf->buf);\n"
"          buf->max = ((buf->size >> 5) + 1) << 5;\n"
"          buf->buf = (char *) wr_malloc(buf->max);\n"
"     }\n"
"     buf->size = len;\n"
"     wr_bcopy(data, buf->buf, len);\n"
"     return 1;\n"
"}\n"
"\n"
"static int buf_replace_str(databuf_t buf, pd_char_t *s /*string*/)\n"
"\n"
"{\n"
"     size_t len = (wr_strlen(s) + 1)*sizeof(pd_char_t);\n"
"     if (len > (size_t)buf->max) {\n"
"          wr_free(buf->buf);\n"
"          buf->max = ((buf->size >> 5) + 1) << 5;\n"
"          buf->buf = (char *) wr_malloc(buf->max);\n"
"     }\n"
"     buf->size = len;\n"
"     wr_strncpy((pd_char_t*)buf->buf, s, (len/sizeof(pd_char_t)) );\n"
"     return 1;\n"
"}\n"
"\n"
"static int buf_replace(databuf_t buf, databuf_t buf1)\n"
"\n"
"{\n"
"     return buf_replace_data(buf, buf1->buf, buf1->size);\n"
"}\n"
"\n"
"static int buf_size(databuf_t buf) \n"
"{\n"
"     return buf->size;\n"
"}\n"
"\n"
"static char *buf_getdataptr(databuf_t buf, int i) \n"
"{\n"
"     if (buf->size > i) \n"
"          return &(buf->buf[i]);\n"
"     else\n"
"          return NULL;   // TBD handle error\n"
"}\n"
"\n"
"#ifndef WR_SUPPRESS_WQL\n"
"\n"
"typedef struct wql_dbrow dbrow;\n"
"\n"
"void IMPORT wr_free_criteria(wqsx_t expr);\n"
"void IMPORT wr_free_wqusl(wqusl_t list);\n"
"\n"
"wqtab_t IMPORT wss_db_find_table_by_handle_r( unsigned long handle );\n"
"\n"
"int IMPORT wql_create( pd_char_t *wrapper_name,\n"
"		        int instance_id,\n"
"		        pd_char_t *table_name,\n"
"		        int free_col_defs,\n"
"		        struct wql_col_def *col_defs,\n"
"                       int col_count,\n"
"                       unsigned long *table_id);\n"
"int IMPORT wql_drop(	unsigned long table_id );\n"
"int IMPORT wql_insert(	unsigned long table_id,\n"
"		        int free_vals,\n"
"		        wqrow_t vals);\n"
"int IMPORT wql_select(	unsigned long table_id,\n"
"		        int free_scrit,\n"
"		        wqsx_t scrit,\n"
"                       int take_first,\n"
"		        int order_by,\n"
"		        int desc,\n"
"                       wqtab_t out_table);\n"
"int IMPORT wql_delete(	unsigned long table_id,\n"
"		        int free_scrit,\n"
"		        wqsx_t scrit,\n"
"                       int take_first);\n"
"int IMPORT wql_update(	unsigned long table_id,\n"
"		        int free_scripts,\n"
"                       wqsx_t scrit,\n"
"                       int take_first,\n"
"                       wqusl_t set_script);\n"
"\n"
"void IMPORT wql_free_update_expr( wqusl_t list );\n"
"void IMPORT wql_free_criteria( wqsx_t expr );\n"
"u_long IMPORT wql_find_table_by_names( pd_char_t *wrapper_name, int id,\n"
"                                       pd_char_t *table_name );\n"
"wqcd_t IMPORT wr_mcol_def( int count, ... );\n"
"wqval_t IMPORT wr_mvals( int count, ... );\n"
"wqrow_t IMPORT wr_mrow( wqval_t vals, wqrow_t next );\n"
"wqsx_t IMPORT wr_mwqsx_and( wqsx_t left, wqsx_t right );\n"
"wqsx_t IMPORT wr_mwqsx_or( wqsx_t left, wqsx_t right );\n"
"wqsx_t IMPORT wr_mwqsx_terminal( tx_t terminal );\n"
"wqsx_t IMPORT wr_mwqsx_not( wqsx_t expression );\n"
"tx_t IMPORT wr_mtx( sxval_t left, sxval_t right, enum sxop sxop );\n"
"sxval_t IMPORT wr_msxval_field( int column );\n"
"sxval_t IMPORT wr_msxval_lit_int( int i );\n"
"sxval_t IMPORT wr_msxval_lit_float( double d );\n"
"sxval_t IMPORT wr_msxval_lit_string( const pd_char_t *str );\n"
"wqusl_t IMPORT wr_musl( sxval_t left, wquexp_t right, wqusl_t next );\n"
"wquexp_t IMPORT wr_muexp( wquexp_t left, wquexp_t right, enum numop numop );\n"
"wquexp_t IMPORT wr_muexp_term( sxval_t value );\n"
"void IMPORT dbrow_finish( struct wql_dbrow *dbrp );\n"
"\n"
"/*\n"
" * See wss_db.c for the reference implimentations.\n"
" */\n"
"#define dbrow_start(dbrp) \\\n"
"((((dbrp)->cur) = (dbrp)->results.rows),((dbrp) != NULL && (dbrp)->cur != NULL))\n"
"\n"
"#define dbrow_next(dbrp) (((dbrp)->cur = (dbrp)->cur->next) != NULL)\n"
"\n"
"#define dbrow_isempty(dbrp) ((dbrp)->results.rows == NULL)\n"
"\n"
"#endif /* WR_SUPPRESS_WQL */\n"
"\n"
"    \n"
"/* Mutex stuff */\n"
"\n"
#if PD_SMP_SUPPORT
"#define WR_MUTEX_PROTECT(var) WSS_MUTEX_PROTECT(var)\n"
"\n"
"#define WR_MUTEX_INIT(var) \\\n"
"    wr_mutex_init( &(var ## _lock), PD_MAKESTR(#var) ## PD_MAKESTR(\"lock\"))\n"
"#define WR_MUTEX_DESTROY(var) wr_mutex_destroy( &(var ## _lock) )\n"
"#define WR_MUTEX_ENTER(var) wr_mutex_enter( &(var ## _lock) )\n"
"#define WR_MUTEX_EXIT(var) wr_mutex_exit( &(var ## _lock) )\n"
"\n"
"#define WR_WIV_MUTEX_INIT(var) \\\n"
"    wr_mutex_init(&WSS_GET_WIV_VAR(var ## _lock), \\\n"
"                  PD_MAKESTR(#var) ## PD_MAKESTR(\"lock\"))\n"
"#define WR_WIV_MUTEX_DESTROY(var) \\\n"
"    wr_mutex_destroy(&WSS_GET_WIV_VAR(var ## _lock) )\n"
"#define WR_WIV_MUTEX_ENTER(var) \\\n"
"    wr_mutex_enter(&WSS_GET_WIV_VAR(var ## _lock) )\n"
"#define WR_WIV_MUTEX_EXIT(var) \\\n"
"    wr_mutex_exit(&WSS_GET_WIV_VAR(var ## _lock) )\n"
#else
"#define WR_MUTEX_PROTECT(var)\n"
"#define WR_MUTEX_INIT(var)\n"
"#define WR_MUTEX_DESTROY(var)\n"
"#define WR_MUTEX_ENTER(var)\n"
"#define WR_MUTEX_EXIT(var)\n"
"\n"
"#define WR_WIV_MUTEX_INIT(var)\n"
"#define WR_WIV_MUTEX_DESTROY(var)\n"
"#define WR_WIV_MUTEX_ENTER(var)\n"
"#define WR_WIV_MUTEX_EXIT(var)\n"
#endif
"\n";         

/*=========================================================================*/

/*=========================================================================*/

static char *seq_step_funcs_str = "\n"
"\n"
"/* \n"
" * Code to deal with sequence step instance structures.\n"
" */\n"
"\n"
"/* Structure used to keep track of sequence instances */\n"
"\n"
"struct seq_inst {\n"
"  unsigned int step_number;   /* First step at which to try this instance   */\n"
"  unsigned int try_next;      /* Try successive steps?                      */\n"
"  unsigned int has_run;       /* Has this step run?                         */\n"
"  unsigned int done;          /* Finished with this instance for this call? */\n"
"  void *inst_vars;            /* Pointer to local variables                 */\n"
"  struct seq_inst *prev;  \n"
"  struct seq_inst *next;  \n"
"};\n"
"\n"
"\n"
"/* Allocate a new sequence instance and place it at the head of the\n"
" * list */\n"
"\n"
"static struct seq_inst *new_seq_instance(struct seq_inst **head, int space)\n"
"{\n"
"  struct seq_inst *tmp;\n"
"\n"
"  tmp = (struct seq_inst *)wr_malloc(sizeof(struct seq_inst));\n"
"  tmp->step_number =  tmp->has_run = tmp->done =  tmp->try_next = 0;\n"
"  if (space)\n"
"    tmp->inst_vars = wr_malloc(space);\n"
"  else\n"
"    tmp->inst_vars = NULL;\n"
"  tmp->prev = NULL;\n"
"  tmp->next = *head;\n"
"  if (tmp->next != NULL)\n"
"    tmp->next->prev = tmp;\n"
"  *head = tmp;\n"
"  return tmp;\n"
"}\n"
"\n"
"\n"
"/* remove a sequence instance from the list and free it */\n"
"\n"
"static void free_seq_instance(struct seq_inst **head, \n"
"		       struct seq_inst **instance)\n"
"{\n"
"  if (instance != NULL && *instance != NULL) {  \n"
"    if (*head == *instance)\n"
"      *head = (*instance)->next;\n"
"    if ((*instance)->next != NULL)\n"
"      (*instance)->next->prev = (*instance)->prev;\n"
"    if ((*instance)->prev != NULL)\n"
"      (*instance)->prev->next = (*instance)->next;\n"
"    if ((*instance)->inst_vars != NULL)\n"
"      wr_free((*instance)->inst_vars);\n"
"    wr_free(*instance);\n"
"    *instance = NULL;\n"
"  }\n"
"}\n"
"\n"
"\n"
"/* Ensure that there is an unrun sequence at the head of the list by\n"
" * adding one if one does not exist.  */\n"
"\n"
"static void start_sequence (struct seq_inst **head, int space)\n"
"\n"
"{ \n"
"    if (*head == NULL || (*head)->step_number || (*head)->has_run)\n"
"      (void) new_seq_instance(head, space);\n"
"}\n"
"\n"
"\n"
"/* Remove completed sequences -- those with step_number == max_step */\n"
"\n"
"static void prune_sequences (struct seq_inst **head, unsigned int max_step)\n"
"\n"
"{ \n"
"  struct seq_inst *tmp, *next;\n"
"\n"
"  for (tmp = *head; tmp != NULL; tmp = next) {\n"
"    next = tmp->next;\n"
"    if (tmp->step_number >= max_step) \n"
"      free_seq_instance(head, &tmp);\n"
"  }\n"
"}\n"
"\n"
"\n"
"\n"; 

