/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: codegen.h,v 1.53 2001/06/13 13:22:55 stevek Exp $
 *
 * Authors: Lee Badger, Mark Feldman, Ben Uecker
 *
 * Purpose: Object to hold generated C-code.
 * 
 */

#ifndef _CODEGEN_H
#define _CODEGEN_H

#include "type.h"

struct Param;
struct mlabel;
class  Expr;
struct srcrec;

enum tabsc { instance_table, wrapper_table, global_table };

class source {
private:
	struct srcrec *head, *tail, *cur;
	int flags;

public:
	source();
	int  is_empty();
	
	void make_header(source *);

	void append(const char *txt);
	void prepend(const char *txt);
#ifdef	notyet
	void append(const char *txt1, const char *txt2);
	void append(const char *txt1, const char *txt2, const char *txt3);
	void append(const char *txt1, const char *txt2, const char *txt3,
		    const char *txt4);
#endif

	void append(int n);
	void append(source *src);
	void append(Expr *e);

	void gen_tbd();
	void gen_tbd(char *txt);
	void gen_debug_code();
	void gen_ascii(char *str);
        void gen_comment(char *str, char *str1=NULL);
        void gen_wql_find(enum tabsc scope, char *tabname);
	void gen_wql_create(enum tabsc scope, char *tabname, source *cols,
                int count);
	void gen_wql_insert(enum tabsc scope, char *tabname, source *rows);
	void gen_wql_value(source *exp);
	void gen_seq_step_funcs();
	void gen_libfunc_declaration(source *, char *);
	void gen_argrepl_code(source *lhs, source *rhs);
	void gen_argcopyin(mlabel *l, char *offset);
	void gen_argcopyout(mlabel *l, char *offset);
	void gen_argfree(mlabel *l, char *offset);
	void gen_argrepl(mlabel *l, char *offset, source *rhs=NULL);
        void gen_wr_uninstall(source *supplied_body, source *dbcleanup);
        void gen_wr_install(source *init, source *supplied_body);
        void gen_wr_lifecycle(const char *name, char *init, source *supplied);
        void gen_dispatchers(source *prelist, source *postlist);

        int  operator==(char *s); 
        int  rec_count();

	void rewind();
        void indent(const char *);
	bool more_text();
	char *text();
	char *next();

	char *all();

	bool find(char *txt);
        void insert(char *txt);
	void remove();

	void set_flags(int fl);
	int get_flags();

	int paren_count;	/* used by WQL subsystem */
        // int  lvalue;		/* used in type checking */
        Type *d_type;		/* used to store type info     */
        char *vname;		/* used in declaration parsing */
        mlabel *label;		/* used to store $var info     */
        char *offset;		/* used to store offset of a $variable */
};

#define SourceAppend1(sp,one) {sp->append(one); }
#define SourceAppend2(sp,one,two) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	}
#define SourceAppend3(sp,one,two,three) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	}
#define SourceAppend4(sp,one,two,three,four) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	}
#define SourceAppend5(sp,one,two,three,four,five) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	}
#define SourceAppend6(sp,one,two,three,four,five,six) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	}
#define SourceAppend7(sp,one,two,three,four,five,six,seven) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	}
#define SourceAppend8(sp,one,two,three,four,five,six,seven,eight) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	}
#define SourceAppend9(sp,one,two,three,four,five,six,seven,eight,nine) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	}
#define SourceAppend10(sp,one,two,three,four,five,six,seven,eight,nine,ten) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	}
#define SourceAppend11(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	}
#define SourceAppend12(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	}
#define SourceAppend13(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven, twelve, thirteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	}
#define SourceAppend14(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	}
#define SourceAppend15(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	}
#define SourceAppend17(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	  (sp)->append(sixteen); \
	  (sp)->append(seventeen); \
	}  
#define SourceAppend19(sp,one,two,three,four,five,six,seven,eight,nine,ten,eleven,twelve,thirteen,fourteen,fifteen,sixteen,seventeen,eightteen,nineteen) \
        { \
          (sp)->append(one); \
	  (sp)->append(two); \
	  (sp)->append(three); \
	  (sp)->append(four); \
	  (sp)->append(five); \
	  (sp)->append(six); \
	  (sp)->append(seven); \
	  (sp)->append(eight); \
	  (sp)->append(nine); \
	  (sp)->append(ten); \
	  (sp)->append(eleven); \
	  (sp)->append(twelve); \
	  (sp)->append(thirteen); \
	  (sp)->append(fourteen); \
	  (sp)->append(fifteen); \
	  (sp)->append(sixteen); \
	  (sp)->append(seventeen); \
	  (sp)->append(eightteen); \
	  (sp)->append(nineteen); \
	}  

#endif /* _CODEGEN_H */
