/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: event.cc,v 1.47 2001/04/05 18:41:33 stevek Exp $
 *
 * Author: Calvin Ko
 *
 * Purpose: Contains member functions of the characterzation class
 *
 */

#include "wr_conf.h"
#include <stdio.h>
#ifdef WR_UNIX
#include <string.h>
#include <iostream.h>
#endif
#ifdef WR_WINNT
// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#pragma warning(disable: 4786) 

#include <iostream>
#include <string>
#include <vector>
#endif

#include "libwdl.h"
#include "codegen.h"
#include "expr.h"
#include "event.h"

#define ExitErrorCond(c,s)  if (!(c)) ; else wdlerror(s)
void wdlerror(const char *s, const char *s1 = "");

extern EvSubSysTbl esstbl;


/*
void characterization::print(ostream& out) 

{
     evmap::iterator iter;
     for (iter = evtbl.begin(); iter != evtbl.end(); iter++ ) {
          WrEvent *e = *iter;
          Param *p = e->plist; 
          out << e->name << " (";
          while (p) {
              out << p->type->all() << " " << p->name << ", ";
              p = p->next;
          }
          out << ");\n";
     }
}
*/

IntTbl *syscalltbl = NULL;

#if   defined(WR_UNIX)

IntTbl *codetbl = NULL;

// return 0 if not found
int get_syscallcode(const char *s)
{    
   if( syscalltbl == NULL )
   {
      syscalltbl = new IntTbl;
      codetbl = new IntTbl;
      for( int i = 1; syscallnames[i] != NULL; i++ )
         syscalltbl->insert( syscallnames[i], i );
   }

   if( !(*codetbl)[s] && (*syscalltbl)[s] ) {
      codetbl->insert( s, (*syscalltbl)[s] );
   }
   return (*syscalltbl)[s];
}

#elif defined(WR_WINNT)

IntTbl *codetbl = NULL;
int     codendx = 0;
typedef vector<string *> CodeVector;
CodeVector *codeVect = NULL;

// return 0 if not found
int get_syscallcode(const char *s)
{    
   if( codetbl == NULL )
   {
      syscalltbl = new IntTbl;
      codetbl = new IntTbl;
      codeVect = new CodeVector;

      for( int i = 1; syscallnames[i] != NULL; i++ )
          syscalltbl->insert( syscallnames[i], i );
      
      codeVect->reserve(i);
   }


   if( !(*codetbl)[s] && (*syscalltbl)[s] ) {
     codetbl->insert( s, ++codendx );
     string *codeStr = new string(s);
     codeVect->push_back( codeStr );
   }
   return (*codetbl)[s];
}

string *get_syscallname(int code)
{
  if( codetbl == NULL )
  {
    syscalltbl = new IntTbl;
    codetbl = new IntTbl;
    codeVect = new CodeVector;
    
    for( int i = 1; syscallnames[i] != NULL; i++ )
      syscalltbl->insert( syscallnames[i], i );
    
    codeVect->reserve(i);
    
    for( i = 0; syscallnames[i] != NULL; i++ ) {
      codetbl->insert( syscallnames[i], ++codendx );
      string *codeStr = new string(syscallnames[i]);
      codeVect->push_back( codeStr );
    }
  }
  
  try {
    return codeVect->at(code);
  }
  catch (out_of_range) {
    return NULL;
  }
}

#endif


void characterization::print(ostream& out) 

{
     node *n;
     for (n = evlist; n != NULL; n = n->next ) {
          Param *p = n->e->plist; 
          out << n->e->name << " (";
          while (p) {
              out << p->type->all() << " " << p->name << ", ";
              p = p->next;
          }
          out << ");\n";
     }
}

void characterization::print_oattr(ostream& out, char *n) 

{
     iterator iter(OATTR, n);
     out << n << ":\t";
     for (iter = begin(OATTR, n); iter != end(); iter++) {
          out << (*iter)->name << " ### ";
     } 
     out << "\n";
}

void characterization::print_pattr(ostream& out, char *n) 

{
     iterator iter(PATTR, n);
     out << n << ":\t";
     for (iter = begin(PATTR, n); iter != end(); iter++) {
          out << (*iter)->name << " ### ";
     } 
     out << "\n";
}

void mlabel::compute_type(WrEvent *ev)

{
     if (type == mlabel::MLT_COPY) {
           expr1->compute_type(ev);
           expr2->compute_type(ev);
     } else if (type == mlabel::MLT_RCOPY) {
           expr1->compute_type(ev);
           expr2->compute_type(ev);
     }
     /* TBD:
     else if (type == mlabel::MLT_TYPE) {
     } else if (type == mlabel::IF) {
     }
      */
     return;
}


/* return NULL if not find */
mlabel* Param::find_label(const char *n) 

{
     mlabel *ml = labels;
     while (ml != NULL) {
          if ((ml->type == mlabel::MLT_LABEL ||
	       ml->type == mlabel::MLT_HANDLE) && !strcmp(ml->data->all(), n))
               return ml;
          ml = ml->next;
     }
     return NULL;
}

void Param::set_labels(mlabel *l) 

{
     int ntype = 0;
     labels = l;
     for (l = labels; l != NULL; l = l->next) {
           if (l->type == mlabel::MLT_TYPE) ntype++;
           l->para = this;
     }
     if (ntype > 1)
          wdlerror(this->name, ": only 1 type() specification is allowed");
}

int  Param::is_nterm()

{
     mlabel *ml;
     for (ml = labels; ml != NULL; ml = ml->next)
         if (ml->type == mlabel::MLT_NTERM)
              return 1;
     return 0;        
}

int  Param::is_ntermlist()

{
     mlabel *ml;
     for (ml = labels; ml != NULL; ml = ml->next)
         if (ml->type == mlabel::MLT_NTERMLIST)
              return 1;
     return 0;        
}

int  Param::is_out()

{
     mlabel *ml;
     for (ml = labels; ml != NULL; ml = ml->next)
         if (ml->type == mlabel::MLT_OUT)
              return 1;
     return 0;        
}

int  Param::is_handle()
{
     mlabel *ml;
     for (ml = labels; ml != NULL; ml = ml->next)
         if (ml->type == mlabel::MLT_HANDLE)
              return 1;
     return 0;
}

Type *Param::has_type()

{
     mlabel *ml;
     for (ml = labels; ml != NULL; ml = ml->next)
         if (ml->type == mlabel::MLT_TYPE)
              return ml->data->d_type;
     return NULL;        
}

mlabel *Param::get_copy(Expr *e)

{
     mlabel *ml = labels;
     while (ml != NULL) {
          if (ml->type == mlabel::MLT_COPY && *ml->expr1 == *e) 
               return ml;
          ml = ml->next;
     }
     return NULL;
}

mlabel *Param::get_rcopy(Expr *e)

{
     mlabel *ml = labels;
     while (ml != NULL) {
          if (ml->type == mlabel::MLT_RCOPY && *ml->expr1 == *e) 
               return ml;
          ml = ml->next;
     }
     return NULL;
}

void WrEvent::set_params(Param *p)

{    int i = 0;
     plist = p;
     for (p = plist; p != NULL; p = p->next) {
          p->offset = i++;
          // find type for all expr in markup stmts
          mlabel *ml = p->labels;
          while (ml != NULL) {
               ml->compute_type(this); 
               ml = ml->next;
          }
     }
}

Param *WrEvent::find_param(const char *n)

{
     Param *p = plist;
     while (p != NULL) {
         if (!strcmp(p->name, n))
              return p; 
         p = p->next;
     } 
     return NULL;
}

mlabel *WrEvent::find_elabel(const char *label)

{
     mlabel *ml = elabels;
     while (ml != NULL) {
          if ((ml->type == mlabel::MLT_LABEL || ml->type == mlabel::MLT_HANDLE)
                  && !strcmp(ml->data->all(), label))
               return ml;
          ml = ml->next;
     }
     return NULL;
}

mlabel *WrEvent::find_rlabel(const char *label)

{
     mlabel *ml = ret->labels;
     while (ml != NULL) {
          if ((ml->type == mlabel::MLT_LABEL || ml->type == mlabel::MLT_HANDLE)
                  && !strcmp(ml->data->all(), label))
               return ml;
          ml = ml->next;
     }
     return NULL;
}

mlabel *WrEvent::find_plabel(const char *label)

{
     Param *pl = plist;
     while (pl != NULL) {
          mlabel *l = pl->find_label(label);
          if (l != NULL) return l;
          pl = pl->next;
     }
     return NULL;
}

/* Get the type of the plabel 
   return the type specified in the type() clause if any,
   otherwise, return the type declared in the system call
 */
Type *WrEvent::find_plabeltype(const char *label)

{
     mlabel *l = find_plabel(label); 
     if (l != NULL) {
         mlabel *l1 = l->para->labels;
         while (l1) {
              if (l1->type == mlabel::MLT_TYPE) 
                    return l1->data->d_type;
              l1 = l1->next;
         } 
#if 0
         if (l->para->is_nterm()) 
              return &strType;
         else
#endif
              return l->para->d_type;
     } else
          return &errType;
}


// return NULL if valid
// otherwise return the first WrEvent that s is an invalid attr
// on return sp stores the attribute type 
WrEvent *EvSubSysTbl::is_valid_attr(EvSet *aSet, char *s, mlabel **ml)

{
      EvSet::iterator iter;
      mlabel *label = NULL;
      for (iter = aSet->begin() ; iter != aSet->end(); iter++) {
           WrEvent *ev = get_event(*iter);
           if (ev == NULL)
               wdlerror("Internal Error #51");
           label = ev->find_plabel(s);
           if (label == NULL) return ev;
      }
      if (label == NULL)
          wdlerror("Internal Error #51");
      *ml = label;
      return NULL;
}

WrEvent *EvSubSysTbl::is_valid_rattr(EvSet *aSet, char *s, mlabel **ml)

{
      EvSet::iterator iter;
      mlabel *label = NULL;
      for (iter = aSet->begin() ; iter != aSet->end(); iter++) {
           WrEvent *ev = get_event(*iter);
           if (ev == NULL)
               wdlerror("Internal Error #51");
           label = ev->find_rlabel(s);
           if (label == NULL) return ev;
      }
      if (label == NULL)
          wdlerror("Internal Error #51");
      *ml = label;
      return NULL;
}

Param *EvSubSysTbl::is_valid_var(EvSet *, char *s, mlabel **ml)
{
      CharTbl::iterator iter;
      for (iter = tbl.begin(); iter != tbl.end(); iter++) {
            if (!(*iter)) continue;
            mlabel *l = (*iter)->find_vattr(s);
            if (l != NULL) {
                *ml = l;
                return l->para;
            }
      }       
      return NULL;
}

/*     EvSet::iterator iter;
      mlabel *label;
      for (iter = set->begin() ; iter != set->end(); iter++) {
           WrEvent *ev = get_event(*iter);
           if (ev == NULL) wdlerror("Internal Error #51");
           label = ev->find_rlabel(s);
           if (label == NULL) return ev;
      }
      if (label == NULL) wdlerror("Internal Error #51");
      *ml = label;
      return NULL;
*/

source *EvSubSysTbl::gen_attr_offset(EvSet *aSet, char *s)

{
     int i, max, os_tbl[10];
     source srctbl[10];
     for (i=0;i<10;i++)
          os_tbl[i] = 0; 
     EvSet::iterator iter; 
     for (iter = aSet->begin(); iter != aSet->end(); iter++) {
           WrEvent *e = get_event(*iter);
           int code = esstbl[e->space]->get_code(*iter);
           mlabel *l = e->find_plabel(s);
           if (l != NULL && code > 0) {
                int index = l->para->offset;
                os_tbl[index]++;
                if (!srctbl[index].is_empty()) 
                     srctbl[index].append(" || ");
                srctbl[index].append("code == ");
                srctbl[index].append(code);
           }
     }
     // find offset number associated with the max number of events
     max = 0; 
     for (i = 1; i < 10; i++) 
         if (os_tbl[i] > os_tbl[max]) max = i;
     source *src = new source; 
     for (i = 0; i < 10; i++) 
         if (i != max && !srctbl[i].is_empty()) {
              src->append(&srctbl[i]);
              src->append(" ? ");
              src->append(i);
              src->append(" : ");
         } 
     src->append(max); 
     return src;
};

struct lsnode {
    mlabel *l;
    source *s;
    int    count;
    struct lsnode *next;
    lsnode(mlabel *label, source *src) {
        count = 1;
        l = label;
        s = src;
        next = NULL;
    }
};


int vmap_equ(mlabel *l1, mlabel *l2)

{
    ExitErrorCond(l1 == NULL || l2 == NULL, "Internal Error #13(event.cc)");
    mlabel *m1 = l1->son, *m2 = l2->son;
    while (m1 != NULL || m2 != NULL) {
        while (m1 != NULL && m1->type != mlabel::MLT_VMAP) m1 = m1->next;
        while (m2 != NULL && m2->type != mlabel::MLT_VMAP) m2 = m2->next;
        if (m1 != NULL && m2 != NULL) {
            if (!(*m1->expr1 == *m2->expr1 && *m1->expr2 == *m2->expr2))
                return 0;
            m1 = m1->next;
            m2 = m2->next;
        } else if (m1 == NULL && m2 != NULL)
            return 0;
        else if (m1 != NULL && m2 == NULL)
            return 0;
    }
    return 1;
}

void insert_vmap(lsnode **lslist, mlabel *l, int code) 

{
    lsnode *ls;
    if (*lslist == NULL) {
        source *sp = new source;
        SourceAppend2(sp, "code == ", code);
        *lslist = new lsnode(l, sp);
        return;
    } else {
        for (ls = *lslist; ls != NULL; ls = ls->next) {
            if (vmap_equ(ls->l, l)) {
                SourceAppend3(ls->s, " || ", "code == ", code);
                ls->count++;
                return;
            }
            if (ls->next == NULL) {
                source *sp = new source;
                SourceAppend2(sp, "code == ", code);
                ls->next = new lsnode(l, sp);
                return;
            }
         }
    }
    
}

// given the wrapper error name, return the corresponding system error name
// return if no assignment
char *get_vmap(mlabel *l, char *val) 

{
    mlabel *sl;
    for (sl = l->son; sl != NULL; sl = sl->next) {
        if (sl->type == mlabel::MLT_VMAP) {
            source *s1 = sl->expr1->get_src();
            source *s2 = sl->expr2->get_src();
            if (!strcmp(s1->all(), val) || !strcmp(s1->all(), "*")) {
                if (!strcmp(s2->all(), "_"))
                    return NULL;
                else
                    return s2->all();
            }
        }
    }
    return val;
}

source *gen_vmap_rhs(mlabel *l, char *lhs, source *rhs)

{
    source *sp = new source;;
    mlabel *sl;
    char *rhstext = rhs->all();
    for (sl = l->son; sl != NULL; sl = sl->next) {
        if (sl->type == mlabel::MLT_VMAP) {
            source *s1 = sl->expr1->get_src();
            // cerr << "XXX :" << s1->all() << "\n";
            source *s2 = sl->expr2->get_src();
            // cerr << "XXX :" << s2->all() << "\n";
            if (!strcmp(s2->all(), "_")) {
                 // free s2
                 s2 = new source;  
                 s2->append(lhs);
            } 
            if (*s1 == "*") {
                 sp->append(s2);
                 free(rhstext);
                 return sp;
            } else {
                 SourceAppend6(sp, rhstext, " == ", s1, " ? ", s2, " : ");
            }
        }
    }
    sp->append(rhstext);
    free(rhstext);
    return sp;
}      

int valid_ename(char *)
{
     return 1;  /* TBD */
}


int EvSubSysTbl::special_case(EvSet *aSet, source *in, source *retsp,
			      source *rvsp, source *errsp)

{
     mlabel *l;
     char *ret = in->all(); 
     if (in->rec_count() == 1) {
          if (!strcmp(ret, "WR_ALLOW") || !strcmp(ret, "WR_DENY")
              || !strcmp(ret, "WR_SUCCESS") || !strcmp(ret, "0") 
              || !strcmp(ret, "1") ) {
               retsp->append(ret);
               return 1;
          } else {
               char rval[64], ename[64];
               if (!strncmp(ret, "WR_A_", 5)) {
                    strcpy(ename, "WR_");
                    strcpy(ename+3, ret+5);
                    strcpy(rval, "WR_ALLOW");
               } else if (!strncmp(ret, "WR_D_", 5)) {
                    strcpy(ename, "WR_");
                    strcpy(ename+3, ret+5);
                    strcpy(rval, "WR_DENY");
               } else if (!strncmp(ret, "WR_", 3)) {
                    strcpy(ename, ret);
                    strcpy(rval, "WR_ALLOW");
               } else 
                    return 0; 
               if (!valid_ename(ename)) {
                    wdlerror("Definition started with WR_ not defined");
               }    
               retsp->append(rval);
               source *r1 = new source;
               SourceAppend3(r1, " (", ret, ") & WR_MASK2 ");
               rvsp->append(gen_rhs(aSet, "wrret", r1));
               if (is_valid_var(aSet, "errno", &l) != NULL) {  
                    char *out = get_vmap(l, ename);
                    if (out != NULL) {
                         SourceAppend3(errsp, "*data->errnum = ", out, ";\n");
                    }
               }
               return 1;
          }
     } else if (in->rec_count() == 3) {
          in->rewind();
          char *e1 = in->next();
          char *e2 = in->next();
          char *e3 = in->next();
          if (!strncmp(e2, " | ", 10)) {
              if (!strcmp(e1, "WR_ALLOW") || !strcmp(e1, "WR_DENY") || 
                  !strcmp(e1, "WR_SUCCESS")) 
              {
                  if (!strncmp(e3, "WR_", 3) && e3[4] != '_') {
                       retsp->append(e1);
                       source *r1 = new source;
                       SourceAppend3(r1, " (", ret, ") & WR_MASK2 ");
                       rvsp->append(gen_rhs(aSet, "wrret", r1));
                       if (is_valid_var(aSet, "errno", &l) != NULL) {  
                           char *out = get_vmap(l, e3);
                           if (out != NULL) {
                               SourceAppend3(errsp, "*data->errnum = ", out, 
                                                    ";\n");
                           }
                       }
                       return 1;
                  }
              }
          } 
     }
     return 0;
}    


source *EvSubSysTbl::gen_svar_assmt(mlabel *l, source *rhs)

{
     char lhs[256];
     static smap<char*> *translation_table = NULL;

     if (translation_table == NULL) {
         /* Hidious hack.  Errno can not be used as an identifier name under
          * solaris since they #define it, but errno is hard coded into parts
          * of the compilier to have characterization information on error
          * codes.  So any time this function see's errno, it will output
          * errnum (what the data-> element was renamed)
          */
         translation_table = new smap<char*>;
         translation_table->insert("errno", "errnum");
         translation_table->insert("errnum", "errnum");
         translation_table->insert("wiv", "wiv");
     }
     strcpy(lhs, "*data->");
     strcat(lhs, (*translation_table)[l->para->name]);
     source *sp = new source;;
     char *rstr = rhs->all();
     if (!strncmp(rstr, "WR_", 3) && rstr[4] != '_') {
          char *out = get_vmap(l, rstr);
          if (out != NULL) {
               SourceAppend5(sp, "*data->",
                       (*translation_table)[l->para->name], " = ", out, ";\n");
          }
     }
     SourceAppend5(sp, "*data->",
                       (*translation_table)[l->para->name], " = ", 
                       gen_vmap_rhs(l, lhs, rhs), ";\n") 
     return sp;
}

source *EvSubSysTbl::gen_rhs(EvSet *aSet, char *s, source *rhs)

{
     lsnode *lslist = NULL;
     lsnode *l1, *max; 
     source *sp = new source;
     EvSet::iterator iter; 
     for (iter = aSet->begin(); iter != aSet->end(); iter++) {
         WrEvent *e = get_event(*iter);
         int code = esstbl[e->space]->get_code(*iter);
         mlabel *l = e->find_rlabel(s);
         /* The syscall may not exist on this platform. */
         if (code <= 0) {
             /* printf("Skipping event %s\n", *iter); */
             continue;
         }
         if (l != NULL)
              insert_vmap(&lslist, l, code);
     }
     ExitErrorCond(lslist == NULL || lslist->count == 0,
             "No valid event codes: Internal Error #12");
     for (l1 = lslist, max = lslist; l1 != NULL; l1 = l1->next) {
         if (l1->count > max->count) 
             max = l1;
     }
     for (l1 = lslist, max = lslist; l1 != NULL; l1 = l1->next) {
         if (l1 != max) {
             SourceAppend7(sp, 
                  "if (", l1->s, ") \n", 
                  "    *retval = ", gen_vmap_rhs(l1->l, "*retval", rhs), ";\n",
                  "else ");
         }
     } 
     if (sp->is_empty()) {
          SourceAppend3(sp,"*retval = ", 
                               gen_vmap_rhs(max->l, "*retval", rhs), ";\n");
     } else {
          SourceAppend4(sp,"\n", "   *retval = ", 
                            gen_vmap_rhs(max->l, "*retval", rhs), ";\n");
     }
     return sp;
}

/* generate code for argument replacement  e.g.,  $path = strcat(str1, "y");
 * rhs   - source of the right hand side of the assignment
 * pname - name of the $variable
 */
source *WrEvent::gen_argrepl_code(char *pname, source *rhs)

{
     source *sp = new source;
     mlabel *l = find_plabel(pname);
     ExitErrorCond(l == NULL, "Fatal Error");
     IdentExpr iexpr(l->para->name);
     mlabel *cl  = l->para->get_copy(&iexpr);
     mlabel *rcl = l->para->get_rcopy(&iexpr);
     int offset = l->para->offset;
     char *ptype = l->para->type->all();
     char *fnhead = "(void)wr_arg_replace(wrp, CUR_THREAD_ID,space,code, uap, ";
     if (rcl != NULL) {   /* has RCOPY */
         source *num = gen_param(&iexpr, rcl->expr2);
         source *size = new source;
         SourceAppend4(size, "sizeof(",
                             rcl->expr1->type.get_elmtype().get_name(),
                             ") * ", num->all());
         sp->gen_comment("Code for ", rcl->data->all());
         source *elmdup = gen_elmdup_code(rcl->para, &iexpr, 
                               "params0", D_OUT);
         if (!elmdup->is_empty()) {
              source *decl = new source;
              SourceAppend6(decl, ptype, "params0 = ((", ptype, ") uapk [",
                                  offset, "]) + i;\n")
              decl->indent("       ");
              elmdup->indent("       ");
              SourceAppend8(sp, "{  int i;\n", "   for (i=0;i<", num, 
                                ";i++) {\n", decl, elmdup, 
                                "   }\n", "}\n");
         }
         SourceAppend7(sp,fnhead, rhs->all(), ", ", offset, ", ", size, ");\n");
     } else if (cl != NULL) {  
         source *size = gen_param(&iexpr, cl->expr2);
         sp->gen_comment("Code for ", cl->data->all());
         SourceAppend7(sp,fnhead, rhs->all(), ", ", offset, ", ", size, ");\n");
     } else { /* does not have COPY(pname, _) or RCOPY(pname, _) */
         source *size = new source;
         SourceAppend3(size, "sizeof(", 
                             l->para->d_type->get_elmtype().get_name(), ")");
         /* generate code for subsequent copying of the elements
            if pname is a ptr to structure                         */
         source *elmdup = gen_elmdup_code(l->para, &iexpr, "params0", D_OUT);
         if (!elmdup->is_empty()) {
              source *decl = new source;
              SourceAppend6(decl, ptype, "params0 = (", ptype, ") uapk[", 
                                         offset, "];\n")
              elmdup->indent("    ");
              decl->indent("    ");
              SourceAppend4(sp, "{\n", decl, elmdup, "}\n");
         }
         SourceAppend7(sp,fnhead, rhs->all(), ", ", offset, ", ", size, ");\n");
     } 
     return sp;
}

source *WrEvent::gen_elmdup_code(Param *p, Expr *e, char *pname, int direction)

{
    source *sp = new source;
    Expr *bexpr = new PtrToElmExpr(e, "*"); 
    Expr *subexpr;
    char func[64];
    mlabel *ml = p->labels;
    if (direction == D_IN)
         strcpy(func, "wr_dupin(");
    else
         strcpy(func, "wr_dupout(wrp, ");
    while (ml != NULL) {
         if (ml->type == mlabel::MLT_COPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, pname, "->", ename);
              source *size = gen_param(e, ml->expr2, pname);
              sp->gen_comment("Code for ", ml->data->all());
              SourceAppend9(sp, src->all(), " = (", 
                                ml->expr1->type.get_name(), ") ",
                                func, src, ", ", size->all(), ");\n");
         } else if (ml->type == mlabel::MLT_RCOPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, pname, "->", ename);
              subexpr = new PtrToElmExpr(e, ename);
              source *num = gen_param(e, ml->expr2, pname);
              source *size = new source;
              SourceAppend4(size, "sizeof(", 
                                  ml->expr1->type.get_elmtype().get_name(),
                                  ") * ", num->all());
              source *sp1 = new source;
              sp1->gen_comment("Code for ", ml->data->all());
              if (direction == D_IN) {
                  SourceAppend9(sp, src->all(), " = (", 
                                ml->expr1->type.get_name(), ") ",
                                func, src, ", ", size->all(), ");\n");
              }
              char pname1[64];
              strcpy(pname1, pname);
              pname1[6]++; 
              source *elmdup = 
                   gen_elmdup_code(ml->para, subexpr, pname1, direction);
              if (!elmdup->is_empty()) {
                   source *decl = new source;
                   SourceAppend8(decl, ml->expr1->type.get_name(), " ", pname1, 
                                 " = ", pname, "->", ml->expr1->tail(), 
                                 " + i;\n");
                   elmdup->indent("       ");
                   decl->indent("       ");
                   SourceAppend8(sp, "{  int i;\n",  "   for (i=0;i<", num, 
                                     ";i++) {\n", decl, elmdup, "   }\n",
                                     "}\n");
              }
              if (direction == D_OUT) {
                  SourceAppend9(sp, src->all(), " = (", 
                                ml->expr1->type.get_name(), ") ",
                                func, src, ", ", size->all(), ");\n");
              }
         }
         ml = ml->next;
    }
    return sp;
}


source *WrEvent::gen_argcopyout_code(char *pname)

{
     source *sp = new source;
     mlabel *l = find_plabel(pname);
     IdentExpr iexpr(l->para->name);
     mlabel *cl  = l->para->get_copy(&iexpr);
     mlabel *rcl = l->para->get_rcopy(&iexpr);
     char *ptype = l->para->type->all();
     int offset = l->para->offset;
     if (rcl != NULL) {   /* has RCOPY */
          source *num = gen_param(&iexpr, rcl->expr2);
          source *size = new source;
          SourceAppend4(size, "sizeof(",
                             rcl->expr1->type.get_elmtype().get_name(),
                             ") * ", num->all());
          sp->gen_comment("Code for ", rcl->data->all());
          source *elmcopy = gen_elmcopy_code(rcl->para, &iexpr, 
                                                "params0", "paramd0");
          if (!elmcopy->is_empty()) {
              source *decl1 = new source;
              SourceAppend8(decl1, ptype, "tmp = (", ptype, ") wr_dupin(uap[", 
                               offset, "], ", size->all(), ");\n"); 
              source *cleanup = new source;
              SourceAppend1(cleanup, "wr_free(tmp);\n");
              source *decl = new source;
              SourceAppend6(decl, ptype, "params0 = ((", ptype, ") uapk [",
                                  offset, "]) + i;\n");
              SourceAppend2(decl, ptype, "paramd0 = tmp + i;\n");
              decl1->indent("    ");
              cleanup->indent("    ");
              decl->indent("        ");
              elmcopy->indent("        ");
              SourceAppend11(sp, "{\n","    int i;\n", decl1,
                                "    for (i=0;i<", num, ";i++) {\n",
                                    decl, elmcopy, 
                                "   }\n", cleanup, "}\n");
          }
          SourceAppend7(sp, "wr_copyout(uapk[", offset, "], uap[", offset, 
                             "], ", size, ");\n");
          SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
     } else if (l->para->has_type() && l->para->has_type()->tc == TC_DBUF) {
          source *size = new source;
          SourceAppend3(size, "((databuf_t) uapk[", offset, "])->size");
          SourceAppend8(sp, "wr_copyout(((databuf_t) uapk[",offset,"])->buf, ", 
                            "uap[", offset, "], ", size, ");\n");
          SourceAppend3(sp, "buf_del((databuf_t) uapk[", offset, "]);\n");
     } else if (cl != NULL) {   /* has COPY */
          sp->gen_comment("Code for ", cl->data->all());
          source *size = gen_param(&iexpr, cl->expr2);
          SourceAppend7(sp, "wr_copyout(uapk[", offset, "], uap[", offset, 
                             "], ", size, ");\n");
          SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
     } else if (!l->para->is_handle()) {
       // If it is a handle, it's a special case
          source *size = new source;
          SourceAppend3(size, "sizeof(", ptype, ")");
          sp->gen_comment("Code for ", pname);
          source *elmcopy = gen_elmcopy_code(l->para, &iexpr, 
                                                "params0", "paramd0");
          if (!elmcopy->is_empty()) {
              source *decl = new source;
              source *cleanup = new source;
              SourceAppend6(decl, ptype, "params0 = (", ptype, ") uapk[", 
                                         offset, "];\n")
              SourceAppend8(decl, ptype, "paramd0 = (", ptype, 
                            ") wr_dupin(uap[", offset, "], ", 
                             size->all(), ");\n"); 
              cleanup->append("wr_free(paramd0);\n");
              elmcopy->indent("    ");
              decl->indent("    ");
              cleanup->indent("    ");
              SourceAppend5(sp, "{\n", decl, elmcopy, cleanup, "}\n");
          } 
          SourceAppend7(sp, "wr_copyout(uapk[", offset, "], uap[", offset, 
                            "], ", size, ");\n");
          SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
     }
     return sp;
}

source *WrEvent::gen_argfree_code(char *pname)
{
    source *sp = new source;
    mlabel *l = find_plabel(pname);
    IdentExpr iexpr(l->para->name);
    mlabel *cl  = l->para->get_copy(&iexpr);
    mlabel *rcl = l->para->get_rcopy(&iexpr);
    char *ptype = l->para->type->all();
    int offset = l->para->offset;
    if (rcl != NULL) {   /* has RCOPY */
        source *num = gen_param(&iexpr, rcl->expr2);
        source *size = new source;
        SourceAppend4(size, "sizeof(",
                rcl->expr1->type.get_elmtype().get_name(),
                ") * ", num->all());
        sp->gen_comment("Code for ", rcl->data->all());
        source *elmcopy = gen_elmfree_code(rcl->para, &iexpr, "params0");
        if (!elmcopy->is_empty()) {
            source *decl = new source;
            SourceAppend6(decl, ptype, "params0 = ((", ptype, ") uapk [",
                    offset, "]) + i;\n");
            decl->indent("        ");
            elmcopy->indent("        ");
            SourceAppend9(sp, "{\n","    int i;\n",
                    "    for (i=0;i<", num, ";i++) {\n",
                    decl, elmcopy, 
                    "   }\n", "}\n");
        }
        SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
    } else if (l->para->has_type() && l->para->has_type()->tc == TC_DBUF) {
        SourceAppend3(sp, "buf_del((databuf_t) uapk[", offset, "]);\n");
    } else if (cl != NULL) {   /* has COPY */
        sp->gen_comment("Code for ", cl->data->all());
        SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
    } else if (!l->para->is_handle()) {
       // If it is a handle, it's a special case
         source *size = new source;
         SourceAppend3(size, "sizeof(", ptype, ")");
         sp->gen_comment("Code for ", pname);
         source *elmfree = gen_elmfree_code(l->para, &iexpr, 
                                            "params0");
         if (!elmfree->is_empty()) {
           source *decl = new source;
           SourceAppend6(decl, ptype, "params0 = (", ptype, ") uapk[", 
                         offset, "];\n");
           elmfree->indent("    ");
           decl->indent("    ");
           SourceAppend4(sp, "{\n", decl, elmfree, "}\n");
         } 
         SourceAppend7(sp, "wr_copyout(uapk[", offset, "], uap[", offset, 
                       "], ", size, ");\n");
         SourceAppend3(sp, "wr_free(uapk[",offset,"]);\n");
    }
    return sp;
}

/* 
 * Generate code to copy out all the elements in a structure contained
 * in param p and referred to by e.
 */  
source *WrEvent::gen_elmcopy_code(Param *p, Expr *e, 
			          char *sname, char *dname)

{
    source *sp = new source;
    Expr *bexpr = new PtrToElmExpr(e, "*"); 
    Expr *subexpr;
    mlabel *ml = p->labels;
    while (ml != NULL) {
         if (ml->type == mlabel::MLT_COPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, sname, "->", ename);
              source *dst = new source;
              SourceAppend3(dst, dname, "->", ename);
              source *size = gen_param(e, ml->expr2, sname);
              sp->gen_comment("Code for ", ml->data->all());
              SourceAppend7(sp, "wr_copyout(", src->all(), ", ", dst->all(), 
                                ", ", size, ");\n");
              SourceAppend3(sp, "wr_free(", src, ");\n");
         } else if (ml->type == mlabel::MLT_RCOPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, sname, "->", ename);
              source *dst = new source;
              SourceAppend3(dst, dname, "->", ename);
              subexpr = new PtrToElmExpr(e, ename);
              source *num = gen_param(e, ml->expr2, sname);
              source *size = new source;
              SourceAppend4(size, "sizeof(", 
                                  ml->expr1->type.get_elmtype().get_name(),
                                  ") * ", num->all());
              char sname1[64];
              strcpy(sname1, sname);
              sname1[6]++; 
              char dname1[64];
              strcpy(dname1, dname);
              dname1[6]++; 
              source *elmcopy = gen_elmcopy_code(ml->para, subexpr, 
                                                   sname1, dname1);
              if (!elmcopy->is_empty()) {
                  source *decl1 = new source;
                  SourceAppend8(decl1, ml->expr1->type.get_name(), 
                                " tmp = wr_dupin(", dname, "->", 
                                ml->expr1->tail(), ", ", size->all(), ");\n");
                  source *cleanup = new source;
                  cleanup->append("wr_free(tmp);\n");
                  source *decl = new source;
                  SourceAppend8(decl, ml->expr1->type.get_name(), " ", sname1, 
                                 " = ", sname, "->", ml->expr1->tail(), 
                                 " + i;\n");
                  SourceAppend4(decl, ml->expr1->type.get_name(), " ", dname1, 
                                 " = tmp + i;\n");
                  decl1->indent("    ");
                  decl->indent("        ");
                  elmcopy->indent("        ");
                  cleanup->indent("    ");
                  SourceAppend11(sp, "{\n","    int i;\n", decl1,
                                    "    for (i=0;i<", num, ";i++) {\n",
                                        decl, elmcopy, 
                                "    }\n", cleanup, "}\n");
              }
              SourceAppend7(sp, "wr_copyout(", src->all(), ", ", dst->all(), 
                                ", ", size, ");\n");
              SourceAppend3(sp, "wr_free(", src, ");\n");
         }
         ml = ml->next;
    }
    return sp;
}

/* 
 * Generate code to free all the elements in a structure contained
 * in param p and referred to by e.
 */  
source *WrEvent::gen_elmfree_code(Param *p, Expr *e, char *sname )
{
    source *sp = new source;
    Expr *bexpr = new PtrToElmExpr(e, "*"); 
    Expr *subexpr;
    mlabel *ml = p->labels;
    while (ml != NULL) {
         if (ml->type == mlabel::MLT_COPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, sname, "->", ename);
              sp->gen_comment("Code for ", ml->data->all());
              SourceAppend3(sp, "wr_free(", src, ");\n");
         } else if (ml->type == mlabel::MLT_RCOPY && *ml->expr1 == *bexpr) {
              char *ename = ((PtrToElmExpr *)ml->expr1)->name;
              source *src = new source;
              SourceAppend3(src, sname, "->", ename);
              subexpr = new PtrToElmExpr(e, ename);
              source *num = gen_param(e, ml->expr2, sname);
              char sname1[64];
              strcpy(sname1, sname);
              sname1[6]++; 
              source *elmcopy = gen_elmfree_code(ml->para, subexpr, 
                                                   sname1);
              if (!elmcopy->is_empty()) {
                  source *decl = new source;
                  SourceAppend8(decl, ml->expr1->type.get_name(), " ", sname1, 
                                 " = ", sname, "->", ml->expr1->tail(), 
                                 " + i;\n");
                  decl->indent("        ");
                  elmcopy->indent("        ");
                  SourceAppend9(sp, "{\n",
                                     "    int i;\n",
                                     "    for (i=0;i<", num, ";i++) {\n",
                                              decl, elmcopy, 
                                     "    }\n",
                                     "}\n");
              }
              SourceAppend3(sp, "wr_free(", src, ");\n");
         }
         ml = ml->next;
    }
    return sp;
}

source *WrEvent::gen_argcopyin_code(char *pname)

{
     source *sp = new source;
     mlabel *l = find_plabel(pname);
     ExitErrorCond(l == NULL, "Fatal Error");
     IdentExpr iexpr(l->para->name);
     mlabel *cl  = l->para->get_copy(&iexpr);
     mlabel *rcl = l->para->get_rcopy(&iexpr);
     char *ptype = l->para->type->all();
     int offset = l->para->offset;
     char src[32], dst[32], func[32];
     strcpy(src, "uap[");
     strcpy(dst, "uapk[");
     strcpy(func, "wr_dupin(");
     if (rcl != NULL) {   /* has RCOPY */
           source *num = gen_param(&iexpr, rcl->expr2);
           source *size = new source;
           SourceAppend4(size, "sizeof(",
                               rcl->expr1->type.get_elmtype().get_name(),
                               ") * ", num->all());
           sp->gen_comment("Code for ", rcl->data->all());
           SourceAppend9(sp, dst, offset, "] = ", func, src, offset,
                             "], ", size, ");\n");
           source *elmdup = gen_elmdup_code(rcl->para, &iexpr, "params0", 
                                              D_IN);
           if (!elmdup->is_empty()) {
                source *decl = new source;
                SourceAppend6(decl, ptype, "params0 = ((", ptype, ") uapk[",
                                    offset, "]) + i;\n")
                decl->indent("       ");
                elmdup->indent("       ");
                SourceAppend8(sp, "{  int i;\n", "   for (i=0;i<", num, 
                                  ";i++) {\n", decl, elmdup, 
                                  "   }\n", "}\n");
           }
     } else if (l->para->has_type() && l->para->has_type()->tc == TC_DBUF) {
           if (cl == NULL) 
                wdlerror("Copy method missing for parameter ", l->para->name);
           source *size = gen_param(&iexpr, cl->expr2);
           SourceAppend7(sp, "(databuf_t) uapk[", offset ,
                             "] = buf_newdata((char *) uap[", offset, 
                             "], ", size, ");\n");
     } else if (cl != NULL) {   /* has COPY */
           source *size = gen_param(&iexpr, cl->expr2);
           sp->gen_comment("Code for ", cl->data->all());
           SourceAppend9(sp, dst, offset, "] = ", func, src, offset,
                             "], ", size, ");\n");
           source *elmdup = gen_elmdup_code(cl->para, &iexpr, "params0", 
                                              D_IN);
           if (!elmdup->is_empty()) {
                source *decl = new source;
                SourceAppend6(decl, ptype, "params0 = (", ptype, ") uapk[",
                                    offset, "];\n")
                decl->indent("    ");
                elmdup->indent("    ");
                SourceAppend4(sp, "{\n", decl, elmdup, "}\n");
           }
     } else if (l->para->is_handle()) {
         SourceAppend6(sp, dst, offset, "] = ", src, offset, "];\n");
     } else { /* does not have RCOPY or COPY */
         source *size = new source;
         SourceAppend3(size, "sizeof(", ptype, ")");
         SourceAppend9(sp, dst, offset, "] = ", func, src, offset,
                       "], ", size, ");\n");
         source *elmdup = gen_elmdup_code(l->para, &iexpr, 
                                          "params0", D_IN);
         if (!elmdup->is_empty()) {
           source *decl = new source;
           SourceAppend6(decl, ptype, "params0 = (", ptype, ") uapk[",
                         offset, "];\n")
             elmdup->indent("    ");
           decl->indent("    ");
           SourceAppend4(sp, "{\n", decl, elmdup, "}\n");
         }
     }
     return sp;
}

source *WrEvent::gen_param(Expr *pe, Expr *e, char *sname)

{
     source *sp = new source;
     if (e->etype() == Expr::Const) {
          sp->append(((ConstExpr *)e)->val);
     } else if (e->etype() == Expr::Ident) {
          Param *p1 = find_param(((IdentExpr *)e)->name);
          ExitErrorCond(p1 == NULL, "Error: gen_param()");
          SourceAppend3(sp, "(int) uap[", p1->offset, "]");
     } else if (e->etype() == Expr::PtrRef) {
          Param *p1 = find_param(e->head());
          ExitErrorCond(p1==NULL, "Fatal Error: gen_argcopy_code 3");
          SourceAppend5(sp, "(*(", ((PtrRefExpr *)e)->expr->type.get_name(),
                            ") uap[", p1->offset, "])");
     } else if (e->etype() == Expr::PtrToElm) {
          source *tail = e->subexpr(*pe);
          if (sname != NULL && tail != NULL) {
               SourceAppend2(sp, sname, tail);
          } else {
               Param *p1 = find_param(e->head());
               ExitErrorCond(p1 == NULL, "Error: gen_param()");
               SourceAppend7(sp, "((", p1->d_type->get_name(), ") uap[", 
                                 p1->offset, "])", "->", e->tail());
          }
     }
     return sp;
}
