/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: event.h,v 1.39 2001/02/23 19:02:28 stevek Exp $
 *
 * Author: Calvin Ko
 *
 * Purpose: Structures for holding info in an interface characterization 
 *
 */

#ifndef _WR_EVENT_H
#define _WR_EVENT_H

#include <stdio.h>
#include <stdlib.h>
#ifdef WR_UNIX
#include <string.h>
#include <iostream.h>
#endif
#ifdef WR_WINNT
#include <iostream>
#include <string>
#include <utility>
#endif


#include "smap.h"
#include "type.h"

#ifdef WR_UNIX
#include "template/pair.h"
#endif

#include "libwdl.h"

#define K_LABEL 0

#define D_IN	0
#define D_OUT	1


class Expr;
struct Param;
struct WrEvent;


#ifdef WR_WINNT
string *get_syscallname(int code);
#endif
/* 
 * structure for holding a str-str map with wildcard matching
 */
struct StrMap {
    StrTbl *stbl;
public:
    StrMap(int size = 5) {
        stbl = new StrTbl(size);
    };    
    int insert(char *key, char *val) {
        return stbl->insert(key, val);
    }
    char * operator[](const char *key) {
        char *r = (*stbl)[key];
        return r ? r : (*stbl)["*"];
    }
    int size() {
         int i = 0;
         StrTbl::iterator iter;
         for (iter = stbl->begin(); iter != stbl->end(); iter++) {
             i++;
         }     
         return i;
    }
    int operator ==(const StrMap& sm) {
         StrTbl::iterator iter;
         for (iter = stbl->begin(); iter != stbl->end(); iter++) {
              char *s = (*sm.stbl)[iter.get_key()];
              if (*iter != NULL)
                   if (s == NULL || strcmp(*iter, s))
                        return 0;
         }
         for (iter = sm.stbl->begin(); iter != sm.stbl->end(); iter++) {
              char *s = (*stbl)[iter.get_key()];
              if (*iter != NULL)
                   if (s == NULL || strcmp(*iter, s))
                        return 0;
         }
         return 1;
    }
    source *gen_assmt(source *rhs);
};

typedef smap<int> IntTbl;

/* 
 * structure for holding a markup label 
 */
struct mlabel {
     enum mltype { MLT_LABEL, MLT_NTERM, MLT_NTERMLIST, MLT_IN, MLT_OUT,
                   MLT_VMAP, MLT_DO, MLT_COPY, MLT_RCOPY, MLT_HANDLE, MLT_TYPE,
                   MLT_IF, MLT_REF, MLT_SET };
     mltype type;
     Param  *para;
     source *data;
     Expr   *expr1, *expr2;
     Expr   *if_clause;
     struct mlabel *next, *son;
     mlabel(mltype t, source *d=NULL, Expr *e1=NULL, Expr *e2=NULL)	{  
         type = t; 
         data = d;  
         expr1 = e1;
         expr2 = e2;
         next = NULL;
         son = NULL;
         if_clause = NULL;
     }
     void compute_type(WrEvent *ev);
};

/* 
 * structure for holding a parameter or a variable
 */
struct Param {
     char     *name;
     source   *type;
     mlabel   *labels;
     WrEvent  *event;
     int       offset;
     Type     *d_type;
     struct Param *next;
     Param () {  
         name = NULL;  
         type = new source;   
         labels = NULL;
         event = NULL;
         d_type = NULL;
         offset = 0;
         next = NULL;
     }
     Param (char *n, WrEvent *e, source *t) {
         name = strdup(n);
         type = t;
         event = e;
         d_type = NULL; 
         labels = NULL;
         offset = 0;
         next = NULL;
     }
     Param(char *n, Type *t) {
         name = strdup(n);
         type = NULL;
         d_type = t; 
         event = NULL;
         labels = NULL;
         offset = 0;
         next = NULL;
     }
     mlabel *find_label(const char *name);
     void   set_labels(mlabel *l);
     int    is_nterm();
     int    is_ntermlist();
     int    is_out();
     int    is_handle();
     Type   *Param::has_type();
     mlabel *get_copy(Expr *e);
     mlabel *get_rcopy(Expr *e);
     // is_in()
     // is_out()
};

struct WrEvent {
     char *name;			/*  event name 	   */
     char *space;                       /*  event space    */
     int  evid; 			/*  event ID	   */    
     mlabel *elabels; 			/*  event labels   */
     // mlabel *rlabel;			/*  return label   */
     Param  *plist;			/*  parameter list */
     Param  *ret;			/*  return param   */
     // source *rtype;			/*  return type str*/
     WrEvent(char *n, int id = -1) {
          name = strdup(n);
          evid = id;
     };
     void set_params(Param *p);
     Param  *find_param(const char *n);
     mlabel *find_plabel(const char *label);
     mlabel *find_elabel(const char *label);
     mlabel *find_rlabel(const char *label);

     Type   *find_plabeltype(const char *label);

     source *gen_argrepl_code(char *pname, source *rhs);
     // source *gen_argcopy_code(char *pname);
     source *gen_argcopyin_code(char *pname);
     source *gen_argcopyout_code(char *pname);
     source *gen_argfree_code(char *pname);
     source *gen_elmdup_code(Param *p, Expr *e, char *, int);
     source *gen_elmcopy_code(Param *p, Expr *e, char *, char *);
     source *gen_elmfree_code(Param *p, Expr *e, char *);
     source *gen_param(Expr *, Expr *, char * =NULL);

};

/* 
 * structure for holding information of attribute being referenced or used
 * in a pre or post block
 */
struct AttrNode {
    int  write;
    mlabel *m;
    char *ofstr;
    AttrNode (mlabel *ml = NULL, char *s = NULL) {
	 write = 0;
         m = ml;
         ofstr = s;
    }
};
typedef smap<AttrNode *> AttrTbl;

typedef smap<WrEvent *> evmap;
typedef pair<int, int> IntPair;
typedef lnode<IntPair> IntsList;

/*
 * EvSet:  A class representing a set of WrEvents
 */
class EvSet {
     IntTbl tbl; 
public:
     EvSet() {   }
     EvSet(char *s) {
          tbl.set_value(s, 1); 
     }
     EvSet(EvSet& set) {
          IntTbl::iterator iter;
          for (iter = set.tbl.begin(); iter != set.tbl.end(); iter++) {
                if (*iter == 1) 
                     tbl.insert(iter.get_key(), 1);
          };
     }
     ~EvSet() {   }
     void insert(char *s)   {    tbl.set_value(s, 1);  }
     int  has_elm(char *s)  {    return tbl[s];        }
     int  empty ()          {    
          IntTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++)
               if (*iter == 1)
                     return 0;
          return 1;
     }
     int  size()            {
          int i = 0;
          IntTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++)
               if (*iter == 1)
                     i++;
          return i;
     }
     void remove(char *s)   {    
          if (tbl[s] != 0)  
               tbl.mod_value(s, 0);  
     }
     void merge(EvSet& set) {
          if (set.empty())
              return;
          IntTbl::iterator iter;
          for (iter = set.tbl.begin(); iter != set.tbl.end(); iter++) {
               if (*iter == 1)
                    insert(iter.get_key());
          };
     }
     void intersect(EvSet &set) { 
          IntTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if ((*iter) == 1) {
                    if (set.tbl[iter.get_key()] == 0) 
                         tbl.mod_value(iter.get_key(), 0);
               }
          }
     }
     void minus(EvSet &set) {
          IntTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if ((*iter) == 1) {
                    if (set.tbl[iter.get_key()] == 1) 
                         tbl.mod_value(iter.get_key(), 0);
               }
          }
     }
     class iterator {
          IntTbl::iterator iter, end; 
     public:
          iterator() {}
          iterator(const IntTbl::iterator& i, const IntTbl::iterator& e) {   
              iter = i;              
              end = e;
          }
          friend bool operator==(const iterator& A, const iterator& B) {
              return A.iter == B.iter;
          }
          friend bool operator!=(const iterator& A, const iterator& B) {
              return A.iter != B.iter;
          }
          bool operator==(const iterator& A) {
              return A.iter == iter;
          }
          bool operator!=(const iterator& A) {
              return A.iter != iter;
          }
          char *operator*()                    {   return iter.get_key(); }
          iterator& operator=(iterator x) {   
              iter = x.iter; 
              end = x.end;
              return *this;  
          }
          iterator  operator++(int)       {   
              iterator tmp = *this;
              iter++;
              while (iter != end && *iter != 1)
                  iter++;
              return tmp;
          }
     };
     iterator begin() {
          IntTbl::iterator it = tbl.begin();
          while (it != tbl.end() && *it != 1) 
               it++;
          return iterator(it, tbl.end());
     }
     iterator end()   {     return iterator(tbl.end(), tbl.end());   }
     void print(ostream& out) {
         iterator iter;
          out << "Set:  ";
          for (iter = begin(); iter != end(); iter++) {
               out << *iter << "  ";
          }
          out << "\n";
     }
};

/*
 * characterization:  A class for holding info in an interface characterization
 */
class characterization {
     struct node {
          WrEvent *e;
          struct node *next; 
          node(WrEvent *ev) { e = ev; next = NULL; }
     };
     Param *vlist; 		// variable
     evmap evtbl;		// wr_events
     IntTbl codetbl;
     node  *evlist, *evlist_tail;
     
     int count;
public:
     char  *name;
     EvSet evset;
     characterization(char *n) {  
          name = strdup(n);  
          evlist = evlist_tail = NULL;
          vlist = NULL;
          count = 0;
     }
     int insert (char *key, WrEvent *val)  
     {
          evset.insert(key);
          int i = evtbl.insert(key, val);
          if (i == 1) {
               if (evlist == NULL)
                    evlist = evlist_tail = new node(val);
               else {
                    evlist_tail->next = new node(val);
                    evlist_tail = evlist_tail->next;
               }
               codetbl.insert(key, ++count);
          }
          return i;
     }
     int insert_var(char *key, Param *v) {
          if (vlist == NULL) {
               vlist = v;
               v->next = NULL;
          } else {
               if (find_var(key))
                   return 0;
               else {
                   Param *vp = vlist;
                   for (vp = vlist; vp->next != NULL; vp = vp->next);
                   vp->next = v;
                   v->next = NULL;
               };
          }
          return 1;
     }
     EvSet *get_evset_from_oattr(char *s) {
          node *p; 
          EvSet *set = new EvSet;
          for (p = evlist; p != NULL; p = p->next)
               if (p->e->find_elabel(s))
                    set->insert(p->e->name);
          return set;
     }
     EvSet *get_evset_from_pattr(char *s) {
          node *p; 
          EvSet *set = new EvSet;
          for (p = evlist; p != NULL; p = p->next)
               if (p->e->find_plabel(s))
                    set->insert(p->e->name);
          return set;
     }
     WrEvent *operator [] (const char *key) {    
          return evtbl[key];                    
     }
     int get_code(const char *key) {
         // Hack hack hack hack
         if (    strcmp(name, "freebsd")     && strcmp(name, "sunos") &&
                 strcmp(name, "solaris") && strcmp(name, "win32") &&
                 strcmp(name, "Linux"))
             return codetbl[key];
         return get_syscallcode(key);
     }
     Param *find_var (const char *key) {
          Param *v;
          for (v = vlist; v != NULL; v = v->next) {
               if (!strcmp(v->name, key))
                    return v;
          }
          return NULL;
     }
     mlabel *find_vattr (const char *key) {
          Param *v;
          for (v = vlist; v != NULL; v = v->next) {
               mlabel *l = v->find_label(key);
                   if (l != NULL) 
                        return l;
          }
          return NULL;
     }
     int is_valid_op(const char *key) {    
          return evtbl[key] != NULL ? 1 : 0;    
     }
     void print(ostream& out); 
     void print_oattr(ostream& out, char *n);
     void print_pattr(ostream& out, char *n);
     enum ntype { END, OP, OATTR, PATTR, ATEXPR};
     class iterator {
          ntype type;
          node *n;
          char *name;
     public:
          iterator(ntype t = OP, node *p=NULL) {
               type = t;
               name = NULL;
               n = p;
               // cerr << this << ":  Creat 1 " << "\n";
          }
          iterator(const iterator& x) {
               type = x.type;
               name = strdup(x.name);
               n = x.n;
               // cerr << this << ":  Creat 2 " << name << "\n";
          } 
          iterator(ntype t, char *s, node *p=NULL) {
               type = t;
               name = strdup(s);
               n = p;
               // cerr << this << ":  Creat 3 " << name << "\n";
          }
          ~iterator() {
               if (name != NULL) free(name);
          }
          friend bool operator==(const iterator& A, const iterator& B) {
              return A.n == B.n;
          }
          friend bool operator!=(const iterator& A, const iterator& B) {
              return A.n != B.n;
          }
          bool operator==(const iterator& A) {
              return A.n == n;
          }
          bool operator!=(const iterator& A) {
              return A.n != n;
          }
          iterator& operator=(const iterator& x) {
               type = x.type;
               n = x.n;
               if (name != NULL) free(name);
               name = strdup(x.name);
               return *this;
          }
          WrEvent *operator*() {
               return n->e;
          } 
          iterator operator++(int) {
               iterator tmp = *this;
               if (n != NULL) n = n->next;
               if (type == OATTR) {
                    while (n != NULL && !n->e->find_elabel(name))
                          n = n->next;
               } else if (type == PATTR) {
                    while (n != NULL && !n->e->find_plabel(name))
                          n = n->next;
               } else if (type == ATEXPR) {
               };
               return tmp;
          }
     };
     iterator begin(ntype t = OP) {
          return iterator(t, evlist);
     }
     iterator begin(ntype t, char *s) {
          node *n = evlist;     
          if (t == OATTR) {
               while (n != NULL && !n->e->find_elabel(s))
                     n = n->next;
          } else if (t == PATTR) {
               while (n != NULL && !n->e->find_plabel(s))
                     n = n->next;
          } else if (t == ATEXPR) {
          }; 
          return iterator(t, s, n);
     }
     iterator end() {
          return iterator(END); 
     }

};

typedef smap<characterization *> CharTbl;

// EvSubSysChar

/*
 * EvSubSysTbl:  A class for holding Multiple Event Subsystem or 
 *               characterization.
 *               Use in selection of events
 */
class EvSubSysTbl {
     CharTbl tbl;
public:
     EvSubSysTbl ()    			{                   }
     void reset() {
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if ((*iter) != NULL)
                    tbl.mod_value((*iter)->name, NULL);
          };
     }
     // no duplicate characterization with the same name
     // return 1 if successful, otherwise 0
     int insert(characterization *ch) {
          if (tbl[ch->name] == NULL) {
               tbl.set_value(ch->name, ch);
               return 1;
          } else
               return 0;
     }
     
     void print(ostream &out) {
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if ((*iter) != NULL) 
                    out << (*iter)->name << "  ";
          } 
          out << "\n";
     }
     void merge(EvSubSysTbl &esstbl) {
          CharTbl::iterator iter;
          for (iter = esstbl.tbl.begin(); iter != esstbl.tbl.end(); iter++)
               if ((*iter) != NULL) {
                    insert(*iter);
               };
     }
     int remove(characterization *ch) {
          if (tbl[ch->name] != NULL) {
               tbl.mod_value(ch->name, NULL);
               return 1;
          } else
               return 0;
     }
     int is_valid_op(const char *key) {    
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if (*iter != NULL && (*iter)->is_valid_op(key))
                     return 1;
          }
          return 0;
     }
     EvSet *get_evset(char *s = NULL) {
          EvSet *set = new EvSet; 
          if (s == NULL) {
              CharTbl::iterator iter;
              for (iter = tbl.begin(); iter != tbl.end(); iter++)
                   if (*iter != NULL) 
                         set->merge( (*iter)->evset );
          }
          return set;
     }
     EvSet *get_evset_from_oattr(char *s) {
          EvSet *set = new EvSet;
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if (*iter != NULL) {
                   delete set;
                   set = (*iter)->get_evset_from_oattr(s);
                   if (!set->empty())
                        return set;
               }
          }
          return set;
     }
     EvSet *get_evset_from_pattr(char *s) {
          EvSet *set = new EvSet;
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if (*iter != NULL) {
                   delete set;
                   set = (*iter)->get_evset_from_pattr(s);
                   if (!set->empty())
                        return set;
               }
          }
          return set;
     }
     characterization *operator[] (char *s) {
          return tbl[s];
     }
     WrEvent *get_event(char *s) {
          CharTbl::iterator iter;
          for (iter = tbl.begin(); iter != tbl.end(); iter++) {
               if (*iter != NULL) {
                     WrEvent *e = (**iter)[s];
                     if (e != NULL) return e;
               }
          }
          return NULL;
     }
     WrEvent *is_valid_attr(EvSet *, char *, mlabel **);
     WrEvent *is_valid_rattr(EvSet *, char *, mlabel **);
     Param *is_valid_var(EvSet *, char *, mlabel **);
     source *gen_attr_offset(EvSet *, char *);
     source *gen_rhs(EvSet *set, char *s, source *rhs);
     source *gen_svar_assmt(mlabel *l, source *rhs);
     int special_case(EvSet *, source *, source *, source *, source *);

};

#endif /* _WR_EVENT_H */ 

