/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: type.cc,v 1.61 2001/06/21 17:06:18 stevek Exp $
 */

#include "wr_conf.h"
#include <stdio.h> 
#include <stdlib.h> 

#ifdef WR_UNIX
#include <libio.h>
#include <iostream.h> 
#include <string.h>
#endif
#ifdef WR_WINNT
#include <iostream>
#include <string>
#endif


#include "type.h"
#include "codegen.h"

void wdlerror(const char *s, const char *s1 = "");

extern VarStore vstore;
extern int in_includes;

#define MAXNAME 31
struct btinfo {
    char *name;
    int  size;
};

btinfo tntbl[33] = {
        {"error", 		4},
        {"unknown ", 		4},
        {"tmp ", 		4},
        {"void ", 		4},
        {"char ", 		1},
        {"pd_char_t ", 	1},
        {"signed char ", 	1},
        {"unsigned char ", 	1},
        {"short ",		2},
        {"signed short",  	2},
        {"int ",		4},
        {"signed ",		4},
        {"signed int ",		4},
        {"long ",		4},
        {"long long ",		8},
        {"signed long ",	4},
        {"signed long long ",	8},
        {"unsigned ",		4},
        {"unsigned int ",	4},
        {"unsigned short ",	2},
        {"unsigned long ",	4},
        {"unsigned long long ",	8},
        {"float ",		4},
        {"double ",		8},
        {"long double ",	12},
        {"string ",		4},
        {"databuf_t ",		4},
        {"complex ",		4}
};

btinfo  *typeinfo_tbl = &tntbl[1];
               
Type tmpType(TC_TMP);
Type errType(TC_ERR);
Type voidType(TC_VOID);
Type intType(TC_INT);
Type floatType(TC_FLOAT);
Type charType(TC_CHAR);
Type strType(TC_STR);
Type cstrType(TC_PTR, new PtrTypeInfo(charType));
Type cptrType(TC_PTR, new PtrTypeInfo(charType));
Type carrType(TC_PTR, new ArrayTypeInfo(charType));
Type vptrType(TC_PTR, new PtrTypeInfo(voidType));
 
char * Type::str()

{
     if (tc < TC_ENUM) {
          return typeinfo_tbl[(int) tc].name;
     } else 
          return typeinfo_tbl[MAXNAME - 2].name;

};

void Type::print(ostream& out) const

{
     if (tc < TC_ENUM) {
          out << typeinfo_tbl[(int) tc].name;
          return;
     } else {
          switch(tc) {
          case TC_ENUM:	  
            out << "enum ";
            return;
          case TC_ARRAY:  
            out << "array of " << ((ArrayTypeInfo *)info)->etype;
            return;
	  case TC_STRUCT: 
            out << ((RecTypeInfo *)info)->name;    
            info->print(out);
            return;
	  case TC_UNION:
            out << ((RecTypeInfo *)info)->name;  
            return;
	  case TC_FUNC: 
            out << "func " << ((FuncTypeInfo *)info)->rtype;
            return;
	  case TC_PTR:     
            out << "ptr to " << ((PtrTypeInfo *)info)->etype; 
            return;
	  case TC_NAME:	
            out << ((NameTypeInfo *)info)->name;
            return;
	  case TC_STR:   	
            out << "str";
            return;
	  case TC_DBUF:
            out << "DBUF";
            return;
	  case TC_DBROW: 
            out << "DBROW ";
            return;
          default:	
            out << "default ";
            return;
          }
     }
}

char *Type::get_name()

{
     char *c;
     if (tc < TC_ENUM) {
          return typeinfo_tbl[(int) tc].name;
     } else {
          switch(tc) {
	  case TC_NAME:	
            return ((NameTypeInfo *)info)->name;
          case TC_ARRAY:
            c = (char *) malloc (256);
            strcpy(c, ((ArrayTypeInfo *)info)->etype.get_name());
            strcat(c, " *");
            return c;
	  case TC_STRUCT:	
            return ((RecTypeInfo *)info)->name;  
	  case TC_UNION:
            return ((RecTypeInfo *)info)->name;  
	  case TC_FUNC: 
            return strdup("func");
	  case TC_PTR: 
            c = (char *) malloc (256);
            strcpy(c, ((ArrayTypeInfo *)info)->etype.get_name());
            strcat(c, " *"); 
            return c;
	  case TC_DBROW: 
            return ((DBrowTypeInfo *)info)->name;
          default:	
            return strdup("default ");
          }
     }
}

int Type::fillin_type(Type t) 

{
     if (tc == TC_TMP) { 
          tc = t.tc;
          info = t.info;     
          return 1;
     } else if (tc == TC_ARRAY) {
          return ((ArrayTypeInfo *)info)->etype.fillin_type(t);
     } else if (tc == TC_PTR) {
          return ((PtrTypeInfo *)info)->etype.fillin_type(t);
     } else if (tc == TC_FUNC) {
          return ((FuncTypeInfo *)info)->rtype.fillin_type(t);
     }
     cerr << "Warning in fillin_type \n";
     return 0; 
}

Type Type::type()

{
     if (tc == TC_NAME) {
          Type *t = vstore.get_type(((NameTypeInfo *)info)->name);
          if (t == NULL) {
               wdlerror("Type is not defined: ",((NameTypeInfo*)info)->name);
               exit(1);
          } else
               return *t;
     } else 
          return *this;
}

Type Type::get_elmtype(char *s) const

{
     if (s == NULL) {
         switch(tc) {
         case TC_ARRAY:   return ((ArrayTypeInfo *)info)->etype.type();
         case TC_PTR  :   return ((PtrTypeInfo *)info)->etype.type(); 
         case TC_FUNC :   return ((FuncTypeInfo *)info)->rtype.type(); 
         case TC_NAME :   cerr << ((NameTypeInfo *)info)->name 
                            <<  ": Type ref not define\n";
                       return errType;
	 case TC_DBUF :   return charType;
	 case TC_STR  :   return charType;
         case TC_WSS_CHAR_T : cerr << "Hit get_elmtype- WSS_CHAR_T\n"; return charType;  
         default   :   return errType;
         } 
     } else {
         if (tc == TC_STRUCT || tc == TC_UNION) {
              RecTypeInfo *rti = (RecTypeInfo *) info; 
              VarNode *vn = (*rti->vtbl)[s];
              if (vn != NULL) 
                  return vn->type.type();
         } else if (tc == TC_DBROW) {
              DBrowTypeInfo *dti = (DBrowTypeInfo *) info; 
              VarNode *vn = (*dti->vtbl)[s];
              if (vn != NULL) 
                  return vn->type.type();
              else if ((vn = (*dti->tti->vtbl)[s]) != NULL)
                  return vn->type.type();
         } else if (tc == TC_NAME) {
             cerr << ((NameTypeInfo *)info)->name <<  ": Type ref not define\n";
             return Type(TC_ERR);
         }; 
         return Type(TC_ERR);
     }
}

int Type::is_ptrtorec() const

{
     return tc == TC_PTR && get_elmtype().is_rectype() ;
}

int Type::is_strtype() const

{
  return tc == TC_STR ||
    ((tc == TC_PTR || tc == TC_ARRAY) && get_elmtype().is_chartype());
}

int Type::is_inttype() const

{
     return tc == TC_CHAR || 
            tc == TC_SIGNEDCHAR || 
            tc == TC_UNSIGNEDCHAR ||
            tc == TC_SHORT || 
            tc == TC_SIGNEDSHORT ||
            tc == TC_INT ||
            tc == TC_SIGNED ||
            tc == TC_SIGNEDINT ||
            tc == TC_LONG ||
            tc == TC_LONGLONG ||
            tc == TC_SIGNEDLONG ||
            tc == TC_SIGNEDLONGLONG ||
            tc == TC_UNSIGNED ||
            tc == TC_UNSIGNEDINT ||
            tc == TC_UNSIGNEDSHORT || 
            tc == TC_UNSIGNEDLONG ||
            tc == TC_UNSIGNEDLONGLONG ||
            tc == TC_WSS_CHAR_T;
}

int Type::is_floattype() const

{
     return tc == TC_FLOAT ||
	    tc == TC_DOUBLE ||
	    tc == TC_LONGDOUBLE;
}

int Type::equiv(const Type& t) const

{
     switch (tc) {
     case TC_DBROW :
     case TC_DBTABLE :
     case TC_FUNC  :
     case TC_ARRAY	:
     case TC_PTR	:  return tc == t.tc && get_elmtype().equiv(t.get_elmtype());
     case TC_STRUCT:
     case TC_UNION :
     case TC_ENUM  :  return tc == t.tc && info == t.info;
     case TC_NAME  :
     case TC_DEF   :  cerr << "Fatal error in type.cc\n";
                   exit(0);
     default	:  
         return typeinfo_tbl[(int)tc].size == typeinfo_tbl[(int) tc].size;
     }
}

// return 1 if Ok, otherwise 0
int Type::assmt_chk(const Type& t, char* ebuf)

{
     if (is_inttype() || is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return 1;
          else {
              strcpy(ebuf, "Assignment of non-int type to int type");
              return 0;
          }
     } else if (is_rectype() || tc == TC_ENUM || tc == TC_FUNC){
          if (tc == t.tc && info == t.info) 
              return 1; 
          else {
              strcpy(ebuf, "Incompatible struct types in assignment");
              return 0;
          }
     } else if (tc == TC_PTR) {
          if (t.is_inttype()) {
              return 1; 
          } else if ((t.tc == TC_PTR || t.tc == TC_ARRAY)) {
              if( get_elmtype().equiv(t.get_elmtype()))
                      return 1; 
              if (get_elmtype().tc == TC_VOID || t.get_elmtype().tc == TC_VOID)
                  return 1; 
          }
          strcpy(ebuf, "Incompatible pointer types in assignment");
          return 0;
     } else if (tc == TC_STR) {
          if (t.is_strtype())
              return 1; 
          else {
              strcpy(ebuf, "Assignment of non-str type to string");
              return 0; 
          }
     } else if (tc == TC_DBUF) {
          if (t.tc == TC_DBUF || t.is_strtype())
              return 1;
          else {
              strcpy(ebuf, "Assignment of non-dbuf or non-str type to dbuf");
              return 0; 
          }
     } else if (tc == TC_DBROW) {
          return 0;  // TBD
     } else {
          strcpy(ebuf, "Invalid lhs type in assignment");
          return 0;
     }
};

/* 
 * check type for binary == operators 
 * return 1 if valid, otherwise 0
 */
int Type::equop_chk(const Type& t)

{
     if (is_inttype()) {
         /* Integers can get promoted to pointers in pointer comparisons. */
         if (t.is_inttype() || t.is_floattype())
             return 1;
         if (t.tc == TC_PTR || t.tc == TC_ARRAY)
             return 1;
         return 0;
     } else if (is_floattype()) {
          return t.is_inttype() || t.is_floattype() ;
     } else if (tc == TC_STR) {
          return t.is_strtype();
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
              if (t.is_inttype()) {
                  /* Integers can get promoted to pointers in pointer
                   * comparisons.
                   */
                  return 1;
              } else if ((t.tc == TC_PTR || t.tc == TC_ARRAY)) {
                  if (get_elmtype().equiv(t.get_elmtype()))
                   return 1; 
                  if (get_elmtype().tc == TC_VOID || t.get_elmtype().tc == TC_VOID)
                   return 1; 
              }
              return 0;
          case TC_ENUM:
              return t.tc == TC_ENUM;
          default:
              return 0;
          }
     }
}

/* 
 * check type for binary + operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::addop_chk(const Type& t)

{
     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype()
                  || (t.tc == TC_PTR || t.tc == TC_ARRAY))
              return t;
          else
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
               if (t.is_inttype())
                    /* Pointer math */
                    return *this;
               else if (is_strtype())
                    return t.is_strtype() ? strType : errType;
               else
                    return errType;
          case TC_STR   :
 	       return t.is_strtype() ? strType : errType;
               
          default:
               return errType;
          }
     }
}

/* 
 * check type for binary - operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::subop_chk(const Type& t)

{
     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype()
                  || (t.tc == TC_PTR || t.tc == TC_ARRAY))
              return t;
          else 
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
#ifdef PD_PTRPTRSUB
     } else if (tc == TC_PTR && t.tc == TC_PTR && equop_chk(t)) {
         return *this;
#endif
     } else if (tc == TC_PTR || tc == TC_ARRAY) {
          if (t.is_inttype())
              return *this;
          else
              return errType;
     }
    // How the compilier can get here, I don't know
     return errType;
}

/* 
 * check type for binary *, / operators 
 * return the resulting type if valid, otherwise errType
 */
Type Type::mulop_chk(const Type& t)

{
#ifdef PD_PTRPTRSUB
    /* size - 32 * (i - (u_long *)addr)   ?????
     * And GCC accepts this?
     */
    if (in_includes)
        return *this;
#endif

     if (is_inttype()) {
          if (t.is_inttype() || t.is_floattype())
              return t;
          else 
              return errType;
     } else if (is_floattype()) {
          if (t.is_inttype() || t.is_floattype())
              return *this;
          else
              return errType;
     } else
          return errType;
}

/* 
 * check type for >, >=, <, <= operators 
 * return 1 if valid, otherwise 0
 */
int Type::relop_chk(const Type& t)

{
     //  char/int/float > char/int/float  is allowed
     //  Technically (pointer <relop> int) shoudl be allowed, but that's too
     //  ugly to exist.
     if (is_inttype() || is_floattype()) {
          return t.is_inttype() || t.is_floattype();
     } else {
          switch (tc) {
          case TC_PTR   :
          case TC_ARRAY :
              if ((t.tc == TC_PTR || t.tc == TC_ARRAY) && 
                      get_elmtype().equiv(t.get_elmtype()))
                   return 1; 
              else 
                   return 0;
          case TC_ENUM:
              return t.tc == TC_ENUM;
          default:
              return 0;
          }
     }
}

/* 
 * check type for >> and <<  operators 
 * return 1 if valid, otherwise 0
 */
int Type::shfop_chk(const Type& t)

{
     return is_inttype() && t.is_inttype();
}

/* 
 * check type for % operators 
 * return 1 if valid, otherwise 0
 */
int Type::modop_chk(const Type& t)

{
     return shfop_chk(t);
}

/* 
 * check type for &, |, ^, << operators 
 * return 1 if valid, otherwise 0
 */
int Type::bitop_chk(const Type& t)

{
     return shfop_chk(t);
}

/* 
 * check type for && and || operators 
 * return 1 if valid, otherwise 0
 */
int Type::logop_chk(const Type& t)

{
     return (is_inttype() || is_floattype() 
             || tc == TC_PTR || tc == TC_ARRAY  || tc == TC_ENUM) &&
             (t.is_inttype() || t.is_floattype() 
             || t.tc == TC_PTR || t.tc == TC_ARRAY  || t.tc == TC_ENUM);
}


Type funcType(const Type& t)

{  return Type(TC_FUNC, new FuncTypeInfo(t));  }


#define newvar(name,type) \
    vtbl_tbl[0]->insert((name), new VarNode((name),(type)) )

VarStore::VarStore() 
{
     top = 0; 
     typetbl.insert("databuf_t", new Type(TC_DBUF));
#ifdef PD_BUILTINSUPPORT
     typetbl.insert("__builtin_va_list",
             new Type(TC_PTR, new PtrTypeInfo(voidType)));
#endif
     vtbl_tbl[0] = new VarTbl(100);
     newvar("_children",intType);
     newvar("_cputime",intType);
     newvar("_cwd ",cstrType);
     newvar("_descriptor",intType);
     newvar("_euid",intType);
     newvar("_egid",intType);
     newvar("_gid",intType);
     newvar("_memoryuse",intType);
     newvar("_niceval",intType);
     newvar("_pgid",intType);
     newvar("_pid",intType);
     newvar("_ppid",intType);
#ifndef WR_WINNT
     newvar("_tty",intType);
     /* on NT, "sid" would be a security descriptor.  On Unixen
      * its a session-id (process group leader)
      */
     newvar("_sid",intType);
#endif
     newvar("_progname",cstrType);
     newvar("_path",cstrType);
     newvar("_svgid",intType);
     newvar("_svuid",intType);
#ifdef WR_WINNT
     newvar("_uid",cstrType);
#else
     newvar("_uid",intType);
#endif /* WR_WINNT */
     newvar("_user",cstrType);
     newvar("_curtime",cstrType);
     newvar("_loadave",intType);
     newvar("_numusers",intType);
     newvar("_osname",cstrType);
     newvar("_osvers",intType);
     newvar("_wrvers",intType);
     newvar("WR_SUCCESS",intType);
     newvar("WR_DENY",intType);
     newvar("WR_ALLOW",intType);
     newvar("WR_NONE",intType);
     newvar("WR_NCALLS",intType);

     newvar("wr_skip",Type(TC_FUNC,new FuncTypeInfo(voidType,NULL,IS_LIBFUNC)));

     newvar("_wrp", vptrType);
     newvar("_data", vptrType);
     
#if 0
     newvar("NULL", vptrType);
#warning This is the wrong answer IM(ns)HO.  Talk to me before putting \
         it back in. \
         DSK
#endif

/*
     newvar("wr_printf",funcType(intType));
     newvar("wr_log",funcType(intType));
     newvar("wr_abspath",funcType(cstrType));
     newvar("wr_malloc",funcType(strType));
     newvar("wr_free",funcType(voidType));
     newvar("wr_curproc_malloc",funcType(strType));
     newvar("wr_curproc_free",voidType);
*/

     newvar("wr_arg_replace",funcType(intType));

/*
     newvar("wr_strlen",funcType(intType));
     newvar("wr_strcmp",funcType(intType));
     newvar("wr_strncmp",funcType(intType));
     newvar("wr_strcpy",funcType(cstrType));
     newvar("wr_strchr",funcType(cstrType));
     newvar("wr_strrchr",funcType(cstrType));
     newvar("wr_bcopy",funcType(voidType));
     newvar("wr_strdup",funcType(cstrType));
*/

     newvar("wr_copyin",funcType(intType));
     newvar("wr_copyout",funcType(intType));
     newvar("wr_copyinstr",funcType(intType));
     newvar("wr_substitute",intType);
     
#ifdef PD_BUILTINSUPPORT
     newvar("__builtin_constant_p",funcType(intType));
     newvar("__builtin_next_arg",funcType(vptrType));
#endif

#if 0 /* defined(WR_THREADED) */
     newvar("wr_mutex_init",funcType(voidType));
     newvar("wr_mutex_destroy",funcType(voidType));
     newvar("wr_mutex_enter",funcType(voidType));
     newvar("wr_mutex_exit",funcType(voidType));
#endif
}

void RecTypeInfo::print(ostream& out) 

{
     VarTbl::iterator iter;
     out << "{ ";
     for (iter = vtbl->begin(); iter != vtbl->end(); iter++) {
          VarNode *vn = *iter;
          if (vn != NULL)
              out << vn->type.get_name() << "   " << vn->name << ";  "; 
          else
              cerr << "Null vn\n";
     }
     out << "  }";
};

void TableTypeInfo::print(ostream& out) 
{
     VarTbl::iterator iter;
     out << "TableTypeInfo " << name << " { ";
     for (iter = vtbl->begin(); iter != vtbl->end(); iter++) {
          VarNode *vn = *iter;
          if (vn != NULL)
              out << vn->type.get_name() << "   " << vn->name << ";  "; 
          else
              cerr << "Null vn\n";
     }
     out << "  }";
};

DBrowTypeInfo::DBrowTypeInfo(TableTypeInfo *t, char *s) 
{
    tti = t;
    vtbl = new VarTbl;
    name = strdup(s);  
    FuncTypeInfo *fti = new FuncTypeInfo(Type(TC_INT, this), NULL, IS_DBFUNC);
    FuncTypeInfo *fretv = new FuncTypeInfo(Type(TC_VOID, this), NULL, IS_DBFUNC);
    vtbl->insert("next", new VarNode("next", Type(TC_FUNC, fti)));
    vtbl->insert("start", new VarNode("start", Type(TC_FUNC, fti)));
    vtbl->insert("finish", new VarNode("finish", Type(TC_FUNC, fretv)));
    vtbl->insert("isempty", new VarNode("isempty", Type(TC_FUNC, fti)));
}

Type concat(const Type& t1, const Type& t2)

{
     if (t1.tc == TC_TMP)
	return Type(t2.tc,t2.info);
     if (t2.tc == TC_TMP)
	return Type(t1.tc,t1.info);

     switch (t1.tc) {
     case TC_UNSIGNED:   
          switch (t2.tc) {
          case TC_SHORT    :  return Type(TC_UNSIGNEDSHORT);
          case TC_LONG     :  return Type(TC_UNSIGNEDLONG);
          case TC_LONGLONG :  return Type(TC_UNSIGNEDLONGLONG);
          case TC_INT      :  return Type(TC_UNSIGNEDINT); 
          case TC_CHAR     :  return Type(TC_UNSIGNEDCHAR); 
          default       :  return Type(TC_ERR);
          }
     case TC_SIGNED:
          switch (t2.tc) {
          case TC_SHORT    :  return Type(TC_SIGNEDSHORT);
          case TC_LONG     :  return Type(TC_SIGNEDLONG);
          case TC_LONGLONG :  return Type(TC_SIGNEDLONGLONG);
          case TC_INT      :  return Type(TC_SIGNEDINT); 
          case TC_CHAR     :  return Type(TC_SIGNEDCHAR); 
          default       :  return Type(TC_ERR);
          }
     case TC_SHORT:
          if (t2.tc == TC_INT) 
              return Type(TC_SHORT);
          else 
              return Type(TC_ERR);
     case TC_UNSIGNEDSHORT:
          if (t2.tc == TC_INT)
              return Type(TC_UNSIGNEDSHORT);
          else
              return Type(TC_ERR);
     case TC_SIGNEDSHORT:
          if (t2.tc == TC_INT)
              return Type(TC_SIGNEDSHORT);
          else
              return Type(TC_ERR);
     case TC_LONG:
          switch (t2.tc) {
          case TC_INT     :  return Type(TC_LONG);
          case TC_LONG    :  return Type(TC_LONGLONG);
          case TC_DOUBLE  :  return Type(TC_LONGDOUBLE);
          default      :  return Type(TC_ERR);
          }
     case TC_DBTABLE:
          return Type(TC_DBTABLE,t1.info);
     case TC_UNSIGNEDLONG:
          switch (t2.tc) {
          case TC_INT     :  return Type(TC_UNSIGNEDLONG);
          case TC_LONG    :  return Type(TC_UNSIGNEDLONGLONG);
          default      :  return Type(TC_ERR);
          }
     case TC_SIGNEDLONG:
          switch (t2.tc) {
          case TC_INT     :  return Type(TC_SIGNEDLONG);
          case TC_LONG    :  return Type(TC_SIGNEDLONGLONG);
          default      :  return Type(TC_ERR);
          }


     default:   
	  return Type(TC_ERR);
     }
}

/* generate code to clean up for addon type, e.g., string, buf */
source *Type::gen_final(char *vname)

{
     source *src = new source;  
     if (tc == TC_STR) {
          SourceAppend3(src, "wr_obj_del(", vname, ");");
     } else if (tc == TC_ARRAY) {
          ArrayTypeInfo *ati = (ArrayTypeInfo *) info;
          source *prefix = new source;
          source *body; 
          SourceAppend2(prefix, vname, "[i]");
          body = ati->etype.gen_final(prefix->all());
          if (!body->is_empty()) {
              body->indent("        ");
              SourceAppend6(src, "{   int i;\n",
 	                         "    for (i = 0; i < ", ati->size, "; i++)\n",
                                 body, "}\n");
          }
          delete prefix; delete body;
     } else if (tc == TC_DBUF) {
     } 
     return src;
}

