%{
/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wdl.l,v 1.67 2001/04/07 00:30:15 stevek Exp $
 *
 * Authors: Chris Vance, Mark Feldman
 *
 * Purpose: Scanner for wrapper compiler.
 * 
 */

#include <string.h>
/* #include "codegen.h" */
#include "type.h"

/*
 *		STATE VARIABLES
 */
extern source *symbols; 
void count();

int comment_caller, wql_caller, tbldecl_caller, characterize_caller;
int attribute_caller, attribute_paren_count = 0;
int win_asm_caller, expblock_caller;
int expblock_depth;

int lineno=1;
char *inputfile = 0;
int wql_braces = 0, characterize_braces = 0;

int check_type();

#define MAX_INCLUDE_DEPTH 15
YY_BUFFER_STATE include_stack[MAX_INCLUDE_DEPTH];
int include_stack_ptr = 0;

extern int in_includes;
%}

%x	COMMENT
%x	WQL
%x	INCLUDE
%x      TBLDECL
%x      ATTRIBUTE
%x      WIN_ASM
%x      GCC_EXP_BLOCK
%s      CHARACTERIZE

ID      [_a-zA-Z][a-zA-Z0-9_]*
DEC	[0-9]
PUNC	[.,;:()\\\'#%*+\-<>!/{}\[\]&|=\$?^~]
HEX	[0-9a-fA-F]
EXP	[Ee][+-]?{DEC}+
FS	(f|F|l|L)
IS	(u|U|l|L)*

LINENUM	^#\ {DEC}+\ \".+\".*
PRAGMA  ^#pragma\ .*
%%

    { }

<INITIAL>"({" {
    if (in_includes) {
        expblock_caller = YY_START;
        expblock_depth = 1;
        BEGIN GCC_EXP_BLOCK;
    } else {
        wdlerror("Gcc expression statements not supported ");
        yyterminate();
    }
}

<GCC_EXP_BLOCK>"({" {
    expblock_depth++;
}

<GCC_EXP_BLOCK>"})" {
    if( --expblock_depth == 0 ) {
       BEGIN expblock_caller;
       wdllval.string = strdup("0");
       count();
       return (S_NUMCONST);
    }
}
<GCC_EXP_BLOCK>.	{ /* copy into file */ }

<GCC_EXP_BLOCK><<EOF>>	{
			wdlerror("Gcc Expression block not terminated");
			yyterminate();
		}



<INITIAL,CHARACTERIZE,WQL,TBLDECL>{PRAGMA}   {} /* Ignore #pragma */

<INITIAL,CHARACTERIZE,WQL,TBLDECL>{LINENUM}	{
			char buff[256];
			sscanf(wdltext, "# %d %255s", &lineno, buff);

			/* the directive is immediately followed by a 
			 * newline, don't include this in the counter */
			lineno--;	

			/* don't copy the quotes around buff */
			inputfile=(char *)malloc(strlen(buff)-1*sizeof(char));
			strncpy(inputfile, &buff[1], strlen(buff)-2);
		}

<INITIAL,CHARACTERIZE,WQL,TBLDECL>"/*" {
			comment_caller = YY_START;
			BEGIN COMMENT;
		}
                
<COMMENT>"*/"	{ BEGIN comment_caller; }

<COMMENT>.	{ /* copy into file */ }

<COMMENT><<EOF>>	{
			wdlerror("Comment not terminated");
			yyterminate();
		}

<INITIAL>__attribute((__)?) {
    attribute_caller = YY_START;
    BEGIN ATTRIBUTE;
}

<INITIAL>__asm[^({})]*\( {
    attribute_caller = YY_START;
    attribute_paren_count = 1;
    BEGIN ATTRIBUTE;
}

<ATTRIBUTE>"("  { ++attribute_paren_count; }

<ATTRIBUTE>")"  {
                    --attribute_paren_count;
                    if (!attribute_paren_count) BEGIN attribute_caller;
                }

<ATTRIBUTE>[^()\n]        { }

<ATTRIBUTE><<EOF>> { wdlerror("attribute not terminated"); yyterminate(); }

<INITIAL>__asm[^({})]*\} { count(); return '}';}
<INITIAL>__asm[^({})]*\{ {
    win_asm_caller = YY_START;
    BEGIN WIN_ASM;
}

<WIN_ASM>[^}\n] { }

<WIN_ASM><<EOF>> {wdlerror("inline asm not terminated"); yyterminate(); }

<WIN_ASM>"}" {BEGIN win_asm_caller; }

characterize	{
                    characterize_caller = YY_START;
                    BEGIN(CHARACTERIZE);
                    count(); return K_CHARACTERIZE;
                }
<CHARACTERIZE>\{	{ /* should not match in a quoted string */
				characterize_braces++;
				count(); return '{';
			}
<CHARACTERIZE>\}	{
				characterize_braces--;
				if (characterize_braces == 0) {
					BEGIN characterize_caller;
				}
				count();
				return '}';
			}

wrapper		{ count(); return K_WRAPPER; }
uses		{ count(); return K_USES; }
<INITIAL>includes	{ count(); return K_INCLUDES; }
<INITIAL,CHARACTERIZE,WQL,TBLDECL>float	{ count(); return K_FLOAT; }
nterm		{ count(); return K_NTERM; }
"*nterm"	{ count(); return K_NTERMLIST; }
<CHARACTERIZE>in		{ count(); return K_IN; }
<CHARACTERIZE>out		{ count(); return K_OUT; }
<CHARACTERIZE>vmap		{ count(); return K_VMAP; }
<CHARACTERIZE>copy		{ count(); return K_COPY; }
<CHARACTERIZE>rcopy		{ count(); return K_RCOPY; }
<CHARACTERIZE>handle            { count(); return K_HANDLE; }
<CHARACTERIZE>type		{ count(); return K_TYPE; }
double		{ count(); return K_DOUBLE; }
goto		{ count(); return K_GOTO; }
<WQL,CHARACTERIZE>set		{ count(); return K_SET;  }
switch		{ count(); return K_SWITCH; }
sizeof		{ count(); return K_SIZEOF; }
case		{ count(); return K_CASE; }
default		{ count(); return K_DEFAULT; }
struct		{ count(); return K_STRUCT; }
union		{ count(); return K_UNION; }
enum		{ count(); return K_ENUM; }
if		{ count(); return K_IF; }
else		{ count(); return K_ELSE; }
do		{ count(); return K_DO; }
for		{ count(); return K_FOR; }
while		{ count(); return K_WHILE; }
void		{ count(); return K_VOID; }
return		{ count(); return K_RETURN; }
break		{ count(); return K_BREAK; }
continue	{ count(); return K_CONTINUE; }
__inline__	{ count(); return K___INLINE; }
__inline	{ count(); return K___INLINE; }
_inline	        { count(); return K___INLINE; }
inline	        { count(); return K___INLINE; }
__cdecl 	{ }
_cdecl 	        { }
__declspec\([^)]*\) 	{ }
__stdcall 	{ }
"->"		{ count(); return K_INDIRECT; }
signed		{ count(); return K_SIGNED; }
__signed	{ count(); return K_SIGNED; }   // for parsing stdio.h (Calvin)
__signed__	{ count(); return K_SIGNED; }   // for parsing stdio.h (Calvin)
short		{ count(); return K_SHORT; }
unsigned	{ count(); return K_UNSIGNED; }
<INITIAL,WQL,TBLDECL,CHARACTERIZE>int		{ count(); return K_INT; }
long		{ count(); return K_LONG; }
__int64		{ count(); return K_LONGLONG; }
string		{ count(); return K_STR;  }
<INITIAL,CHARACTERIZE>pd_char_t	{ count(); wdllval.string = strdup(wdltext); return K_WSS_CHAR_T;  }
<TBLDECL,WQL>char			{ count(); return K_WSS_CHAR_T; }
<INITIAL,CHARACTERIZE>char	        { count(); return K_CHAR; }
<TBLDECL>key	{ count(); return K_KEY; }
auto		{ count(); return K_AUTO; }
extern		{ count(); return K_EXTERN; }
register	{ count(); return K_REGISTER; }
static		{ count(); return K_STATIC; }
__declspec\s(\sdllimport\s) { count(); return K_DLLIMPORT; }
global		{ count(); return K_GLOBAL; }
local		{ count(); return K_LOCAL; }
typedef		{ count(); return K_TYPEDEF; }
const		{ count(); return K_CONST; }
__const__	{ count(); return K_CONST; }
volatile	{ count(); return K_VOLATILE; }
delete		{ count(); return K_DELETE; }

"=~"		{ count(); return K_STRMATCH; }
"@@"		{ count(); return K_STRMATCH; }
"+="		{ count(); return K_PLUSEQ; }
"-="		{ count(); return K_MINUSEQ; }
"*="		{ count(); return K_MULTEQ; }
"/="		{ count(); return K_DIVEQ; }
"%="		{ count(); return K_MODEQ; }
"<<="		{ count(); return K_SHFTLEQ; }
">>="		{ count(); return K_SHFTREQ; }
"&="		{ count(); return K_ANDEQ; }
"|="		{ count(); return K_OREQ; }
"^="		{ count(); return K_XOREQ; }
"||"		{ count(); return K_OR; }
"&&"		{ count(); return K_AND; }
<INITIAL,CHARACTERIZE,WQL>"=="	{ count(); return K_EQEQ; }
<INITIAL,CHARACTERIZE,WQL>"!="	{ count(); return K_NOTEQ; }
<WQL>"<>"       { count(); return K_NOTEQ; }
<INITIAL,CHARACTERIZE,WQL>"<="	{ count(); return K_LTEQ; }
<INITIAL,CHARACTERIZE,WQL>">="	{ count(); return K_GTEQ; }
">>"		{ count(); return K_SHFTR; }
"<<"		{ count(); return K_SHFTL; }
"..."		{ count(); return K_ELIPSIS; }
"++"		{ count(); return K_PLUSPLUS; }
"--"		{ count(); return K_MINUSMINUS; }
"::"		{ count(); return K_COLONCOLON; }

<INITIAL,WQL>DBROW	{ count(); return K_DBROW; }
<INITIAL,WQL>DBTABLE	{
                            count();
                            tbldecl_caller = YY_START;
                            BEGIN TBLDECL;
                            return K_DBTABLE;
                        }

"#include"	{ BEGIN INCLUDE; /* TBD: remove this? */}

"op"		{ count(); return K_OP; }
"oattr"		{ count(); return K_OATTR; }
"pattr"		{ count(); return K_PATTR; }
"step"		{ count(); return K_STEP; }

"try"		{ count(); return K_TRY; }
"catch"		{ count(); return K_CATCH; }

"pre"		{ count(); return K_PRE; }
"post"		{ count(); return K_POST; }
"otherwise"	{ count(); return K_OTHERWISE; }

"wr_install"    { count(); return K_INSTALL; }
"wr_uninstall"  { count(); return K_UNINSTALL; }
"wr_activate"   { count(); return K_ACTIVATE; }
"wr_deactivate" { count(); return K_DEACTIVATE; }
"wr_duplicate"  { count(); return K_DUPLICATE; }

wql|WQL		{ 
			wql_caller = YY_START;
			BEGIN(WQL);
			count(); return K_WQL;
		}

<WQL>create|CREATE	{ count(); return K_CREATE; }
<WQL>drop|DROP		{ count(); return K_DROP; }
<WQL>insert|INSERT	{ count(); return K_INSERT; }
<WQL>select|SELECT	{ count(); return K_SELECT; }
<WQL>delete|DELETE	{ count(); return K_DELETE; }
<WQL>update|UPDATE	{ count(); return K_UPDATE; }

<WQL>set|SET		{ count(); return K_SET; }
<WQL>table|TABLE	{ count(); return K_TABLE; }
<WQL>values|VALUES	{ count(); return K_VALUES; }
<WQL>into|INTO		{ count(); return K_INTO; }
<WQL>first|FIRST	{ count(); return K_FIRST; }
<WQL>from|FROM		{ count(); return K_FROM; }
<WQL>where|WHERE	{ count(); return K_WHERE; }
<WQL>order|ORDER	{ count(); return K_ORDER; }
<WQL>by|BY		{ count(); return K_BY; }
<WQL>desc|DESC		{ count(); return K_DESC; }
<WQL>like|LIKE		{ count(); return K_LIKE; }
<WQL>and|AND		{ count(); return K_WQL_AND; }
<WQL>or|OR		{ count(); return K_WQL_OR; }
<WQL>\{			{ /* should not match in a quoted string */
				wql_braces++;
				count(); return '{';
			}
<WQL>\}			{
				wql_braces--;
				if (wql_braces == 0) {
					BEGIN wql_caller;
				}
				count();
				return '}';
			}

<TBLDECL>\{		{ count(); return '{'; }
<TBLDECL>\}		{ count(); BEGIN tbldecl_caller; return '}'; }

<INCLUDE>[ \t]*		/* eat the whitespace */
<INCLUDE>(\"|<)[^ \t\n]+(\"|<) {
		   char *filename = strdup(wdltext);
		   filename++; 
		   filename[strlen(filename)-1] = 0;
                   if ( include_stack_ptr >= MAX_INCLUDE_DEPTH )
                       {
                       fprintf( stderr, "Includes nested too deeply" );
                       exit( 1 );
                       }

                   include_stack[include_stack_ptr++] =
                       YY_CURRENT_BUFFER;

                   wdlin = fopen( filename, "r" );

                   if ( ! wdlin ) {
                       fprintf(stderr, "Couldn't open include file '%s'\n", 
				filename );
		       exit(2);
		   }


                   yy_switch_to_buffer(
                       yy_create_buffer( wdlin, YY_BUF_SIZE ) );

                   BEGIN(INITIAL);
                   }


<INITIAL,CHARACTERIZE,WQL,TBLDECL>L?\"(\\.|[^\\"])*\" { 
                                wdllval.string = strdup(wdltext);
                                count(); return(S_QSTR); 
                        }

<INITIAL,CHARACTERIZE,WQL,TBLDECL>{PUNC}	{ count(); return wdltext[0]; }

0[xX]{HEX}+{IS}?	{ 
				wdllval.string = strdup(wdltext);
                                count(); return(S_NUMCONST);
 			}
0{DEC}+{IS}?		{
				wdllval.string = strdup(wdltext);
                                count(); return(S_NUMCONST);
			}
<CHARACTERIZE,INITIAL,TBLDECL>{DEC}+{IS}?	{
				wdllval.string = strdup(wdltext);
                                count(); return(S_NUMCONST);
			}
<WQL>-?{DEC}+{IS}?	{
				wdllval.string = strdup(wdltext);
                                count(); return(S_NUMCONST);
			}

<CHARACTERIZE,INITIAL,WQL,TBLDECL>L?'(\\.|[^\\'])+'	{ 
                                wdllval.string = strdup(wdltext);
                                count(); return(S_CHARCONST);
                        }

<CHARACTERIZE,INITIAL,WQL,TBLDECL>{DEC}+{EXP}{FS}?	{
			wdllval.string = strdup(wdltext);
                        count(); return S_FLOATCONST; }
<INITIAL,CHARACTERIZE,WQL,TBLDECL>{DEC}*"."{DEC}+({EXP})?{FS}?	{
			wdllval.string = strdup(wdltext); 
			count(); return S_FLOATCONST;  }
<INITIAL,CHARACTERIZE,WQL,TBLDECL>{DEC}+"."{DEC}*({EXP})?{FS}?	{
			wdllval.string = strdup(wdltext);
			count(); return S_FLOATCONST;  }

<INITIAL,CHARACTERIZE,WQL,TBLDECL>{ID}	{ count(); return (check_type()); }

<<EOF>>		{ if ( --include_stack_ptr < 0) 
			yyterminate();
		  else {
			yy_delete_buffer( YY_CURRENT_BUFFER );
			yy_switch_to_buffer(include_stack[include_stack_ptr]);
		  }
		}

<*>[\n]		{ count(); lineno++; }

<*>[ \t]	{ count(); /* do something? */ }

.		{ count(); wdlerror("Character not recognized"); }

%%
int column = 0;
char lastline[2048] = {0}, thisline[2048]; /* TBD: evil hardcoded array */
int lastlinec = 0;

/* The following function should be called for every match above.  A
 * convenient location is just prior to the token return.  It counts
 * the characters in the buffer and stores the characters in two line
 * buffers. It stores the previous input line and the current line (up
 * to the last token received).  These two buffer lines are only used
 * in the error routine; they provide context for the error.
 * Also stores the location of the current input column in the global
 * variable column. */
void
count()
{
	int i;
	for (i = 0; wdltext[i] != '\0'; i++) {
		/* At the end of a line: bump the current line to the
		 * previous line, clear and reset the current line */
		if (wdltext[i] == '\n') {
			thisline[column] = wdltext[i];
			thisline[column+1] = 0;
/* 			printf("LINE: %s", thisline); */
			strcpy(lastline, thisline);
			/* TBD: remove line length constraint? */
// Not on NT...		bzero(thisline, 2048*sizeof(char)); 
			memset(thisline, 0, 2048*sizeof(char)); 
			column = 0;
		}
		/* tabs are assumed to be 8 spaces */
		else if (wdltext[i] == '\t') {
			strcpy(&thisline[column], "        ");
			column += 8 - (column % 8);
		}
		/* All other characters are copied and the column
		 * counter is incremented */
		else {
			thisline[column] = wdltext[i];
			column++;
		}
	}
}

/* Prints the previous input line, the current input line (up to the
 * last token received) and a message from the parameter s.  Uses the
 * column global variable to "point" at the location at which the
 * error occurred. */
void 
wdlerror(const char *s, const char *s1)
{
	thisline[column+1] = 0;
	printf("%s%s",lastline, thisline);
        printf("\n%*s\n%*s %s at line %d of %s\n", column, "^", column, s, s1,
		lineno, inputfile?inputfile:"stdin");

        exit(2);
}

/* Same as wdlerror, except it doesn't abort.
 */
int
wdlwarning(const char *s, const char *s1)
{
	thisline[column+1] = 0;
	printf("%s%s",lastline, thisline);
        printf("\n%*s\nWarning: %*s %s at line %d of %s\n",
                column, "^", column-9, s, s1,
		lineno, inputfile?inputfile:"stdin");
        return 1;
}

int
wdlwrap()
{
        return (1);
}

/* Looks up the symbol in the symbol table.  Symbols are added to the
 * table in typedef statements.  This scanner needs to differentiate
 * between identifiers and typdef'd names in order for C to be
 * parseable by YACC. */
int check_type()
{
	wdllval.string = strdup(wdltext);

        if (vstore.get_type(wdltext)) {
	   return(S_TYPENAME);
        }

        return(S_IDENTIFIER);
}
