%{
/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wql.l,v 1.8 2001/04/05 18:41:36 stevek Exp $
 *
 * Author:
 *
 * Purpose: Scanner for wrapper compiler.
 * 
 */

#include <string.h>

/*
 *		STATE VARIABLES
 */
int wql_caller, comment_caller;

%}

%x	COMMENT
%x	WQL

ID      [_a-zA-Z][a-zA-Z0-9_\-]*
PUNC	[.,;:()\\\'#%*=+\-<>!/{}\[\]]

%%

WQL |
wql		{ 
			wql_caller = YY_START;
			BEGIN(WQL);
			return K_WQL;
		}

<WQL>create	{ return K_WQL_CREATE; 	}
<WQL>drop	{ return K_WQL_DROP; 	}
<WQL>insert	{ return K_WQL_INSERT; 	}
<WQL>select	{ return K_WQL_SELECT;	}
<WQL>delete	{ return K_WQL_DELETE;	}
<WQL>update	{ return K_WQL_UPDATE;	}

<WQL>table	{ return K_WQL_TABLE; 	}
<WQL>values	{ return K_WQL_VALUES; 	}
<WQL>into	{ return K_WQL_INTO; 	}
<WQL>from	{ return K_WQL_FROM; 	}
<WQL>where	{ return K_WQL_WHERE; 	}
<WQL>like	{ return K_WQL_LIKE;	}

<WQL>=		{ return K_EQ;		}
<WQL><=		{ return K_LE;		}
<WQL>>=		{ return K_GE;		}
<WQL><>		{ return K_NE;		}
<WQL>"<"	{ return K_LT;		}
<WQL>>		{ return K_GT;		}
<WQL>and	{ return K_AND; 	}
<WQL>or		{ return K_OR; 	}
<WQL>char	{ return K_CHAR;	}
<WQL>int	{ return K_INT;		}
<WQL>float	{ return K_FLOAT;	}

<INITIAL,WQL>\"[^\"\n]*\" {	/* string values... must look for
				   escaped quotes
				   doesn't handle unterminated strings */
			int n = wqlleng - 2;
			int ctr = 0;

				/* check for escaped "s */
			while (n && (wqltext[n--] == '\\'))
				ctr++;
			if ( (ctr % 2) == 0 ) {
				/* even number of \s, is real " */
				wqllval.string = strdup(wqltext);
				return S_STRING;
			} else {
				/* odd number of \s, " is escaped */
				/* push " back on so can match rest of
				   string */
				yyless(yyleng-1);
				yymore();
			}
		}

<*>{PUNC}	{ return wqltext[0]; }

<INITIAL,WQL>{ID} {
                        wqllval.string = strdup(wqltext);
                        return S_IDENTIFIER;
		}

<WQL>[0-9]+	{
			wqllval.integer = atoi(wqltext); 
			return I_NUMBER;
		}

<*>[ \t\n]	{
				/* do something here */
		}

<*><<EOF>>	{
			yyterminate();
		}

<*>.		{
                        wqlerror("Character not recognized");
		}

%%

int
wqlerror(char *s)
{
	printf("\n'%s' at '%s'  (%d)\n", s, wqltext, YY_START);
	exit(2);
}

int
wqlwrap()
{
        return (1);
}

