%{
/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wql.y,v 1.8 2001/04/05 18:41:36 stevek Exp $
 *
 * author: 
 *
 * Purpose: Parser for wrapper compiler.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <err.h>

int wqlerror(char *s);
int wqllex();
int wqlparse();
%}

%union {
	char *string;
	int  integer;
}

/*
 	All tokens are upper case.  Types start with an upper case
	letter, but <ID> is lower case.
 
 	K_<ID>	-- returns no value

	S_<ID>	-- returns string (char*) value

	I_<ID>	-- returns integer value

	U_<ID>	-- returns user-defined data structure  (mostly used
		   internally to parser as a %type)  I think these
		   will usually be pointers, but don't have to be.
*/

%token	<string>	S_IDENTIFIER, S_STRING
%token	<integer>	I_NUMBER
				/* tokens for WQL */
%token			K_WQL, K_WQL_CREATE, K_WQL_DROP, K_WQL_INSERT,
			K_WQL_SELECT, K_WQL_DELETE, K_WQL_UPDATE,
			K_WQL_TABLE, K_WQL_INTO, K_WQL_VALUES,
			K_WQL_FROM, K_WQL_WHERE, K_WQL_LIKE

				/* these tokens should already be
				   available in parser when we fold
				   things back together */
%token			K_CHAR, K_FLOAT, K_INT
%token			K_EQ, K_GT, K_LT, K_GE, K_LE, K_NE,
			K_AND, K_OR

%type	<string>	S_wql_table
%type	<string>	S_wql_row
%type	<string>	S_wql_selection
%type	<string>	S_wql_crit_list
%type	<string>	S_wql_criteria


%%

stmt_wql	: K_WQL '{' wql_list '}' ';'
		{
			printf("Successly parsed WQL\n");
			return 0;
		};

wql_list	: wql_list wql
		|
		;

wql		: K_WQL_CREATE K_WQL_TABLE S_wql_table wql_col_str ';'
		{	printf("\tcreate table %s\n", $3);	}
		| K_WQL_DROP K_WQL_TABLE S_wql_table ';'
		{	printf("\tdrop table %s\n", $3);	}
		| K_WQL_INSERT K_WQL_INTO S_wql_table
		  K_WQL_VALUES wql_insert_list ';'
		{	printf("\tinsert into table %s\n", $3);	}
		| K_WQL_SELECT S_wql_row K_WQL_FROM S_wql_table 
		  S_wql_selection ';'
		{	
			printf("\tselect into %s from table %s\n", $2,$4);
		}
		| K_WQL_DELETE K_WQL_FROM S_wql_table S_wql_selection ';'
		{	printf("\tdelete from table %s\n", $3);  }
		| K_WQL_UPDATE S_wql_table S_wql_selection ';'
		{	printf("\tnot complete!\n");	}
		|
		{	$$ = NULL;	}
		;

S_wql_selection	: K_WQL_WHERE S_wql_crit_list
		{	$$ = NULL;	}
		|
		{	$$ = NULL;	}
		;

S_wql_crit_list	: S_wql_crit_list K_AND S_wql_criteria
		{	$$ = NULL;	}
		| S_wql_crit_list K_OR S_wql_criteria
		{	$$ = NULL;	}
		| S_wql_criteria
		{	$$ = NULL;	};

S_wql_criteria	: S_wql_col_name wql_rel_op wql_value
		{}
		;

S_wql_col_name	: '.' S_IDENTIFIER
		{};

wql_rel_op	: K_EQ
		| K_GT
		| K_LT
		| K_GE
		| K_LE
		| K_NE
		| K_WQL_LIKE
		;

S_wql_table	: S_IDENTIFIER
		{	$$ = $1;	}
		;

wql_col_str	: '(' wql_col_list ')'

wql_col_list	: wql_col_list ',' wql_col 
		| wql_col
		;

wql_col		: S_IDENTIFIER wql_type
		{};

wql_type	: K_CHAR '(' I_NUMBER ')'
		{}
		| K_FLOAT
		| K_INT
		;

wql_insert_list	: wql_insert_list ',' wql_insert
		| wql_insert
		;

wql_insert	: '(' wql_value_list ')'
		;

wql_value_list	: wql_value_list ',' wql_value
		| wql_value
		;

wql_value	: I_NUMBER
		{}
		| '\'' S_IDENTIFIER '\''
		| S_STRING
		{}
			/* must be extended to include expressions */
		;

%%

#include "lex.wql.c"

extern FILE *wqlin;
int
wr_parse_prototypes(FILE *fp)
{
	wqlin = fp;

	return(wqlparse());
}
