/*! \file wrinit.c
 * Copyright (C) 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrinit.c,v 1.4 2001/02/07 01:30:29 stevek Exp $
 *
 * \brief   Global configuration options
 * \author  Steve Kiernan
 * \version $Revision: 1.4 $
 * \date    $Date: 2001/02/07 01:30:29 $
 */

#include <wr_conf.h>

#include <wss_init.h>

/***************************************************************************\
 * Private globals                                                         *
\***************************************************************************/

static int _wr_init_started = 0;


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

pd_char_t *wss_init_home     = NULL;
pd_char_t *wss_log_path      = NULL;
pd_char_t *wss_db_filename   = NULL;
pd_char_t *wss_inc_path      = NULL;
pd_char_t *wss_wrap_inc_path = NULL;
#ifdef WR_LINUX
int        wss_db_size       = 8; /* DB size in megabytes.  Linux does not have
                                   * pageable kernel memory.
                                   */
#else
int        wss_db_size       = 16; /* DB size in megabytes */
#endif

/*!
 * Table of configuration entries available via wss.conf file
 *
 * \sa WSS_INIT_ENTRY
 */
wss_init_entry_t wss_init_globals[] =
{
   WSS_INIT_ENTRY( "dbpath",      WSS_INIT_TYPE_STRING, &wss_db_filename ),
   WSS_INIT_ENTRY( "dbsize",      WSS_INIT_TYPE_INT,    &wss_db_size ),
   WSS_INIT_ENTRY( "logpath",     WSS_INIT_TYPE_STRING, &wss_log_path ),
   WSS_INIT_ENTRY( "home",        WSS_INIT_TYPE_STRING, &wss_init_home ),
   WSS_INIT_ENTRY( "incpath",     WSS_INIT_TYPE_STRING, &wss_inc_path ),
   WSS_INIT_ENTRY( "wrapincpath", WSS_INIT_TYPE_STRING, &wss_wrap_inc_path ),

   { NULL, 0 , NULL }
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void
wr_init_startup()
{
   if( _wr_init_started ) return;
   _wr_init_started |= 0x01;

   if( ( _wr_init_started & 0x02 ) == 0 )
   {
      wss_init_home     = strdup( PD_MAKESTR(WSS_HOME_PATH) );
      wss_log_path      = strdup( PD_MAKESTR(WSS_LOG_FILENAME) );
      wss_db_filename   = strdup( PD_MAKESTR(WSS_DB_FILENAME) );
      wss_inc_path      = strdup( PD_MAKESTR(WSS_INC_PATH) );
      wss_wrap_inc_path = strdup( PD_MAKESTR(WSS_WRAP_INC_PATH) );
      _wr_init_started |= 0x02;
   }
}

