/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrstat.c,v 1.5 2001/02/23 19:02:29 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Library for getting status from the WSS.
 */

#include <wr_conf.h>

#ifdef LINKAGE
#undef LINKAGE
#endif
#define LINKAGE IMPORT

#include <errno.h>
#include <wrappers.h>

#ifdef WR_WINNT
#include "../libgwmcomm/libgwmcomm.h"
#endif /* WR_WINNT */

#undef LINKAGE
#define LINKAGE EXPORT
#include "libwrstat.h"

#ifdef WR_WINNT

/* Windows NT */
#define syscall(n,c,s,b) wrstatntProcessCommand(c,b,s)
#else /* WR_WINNT */
#define _tgetcwd(a,b) getcwd(a,b)
#if defined(WR_LINUX)
extern long int syscall __P ((long int __sysno, ...));
#else /* WR_LINUX */
extern int syscall( int callnum, ... );
#endif /* WR_LINUX */
#endif /* WR_WINNT */

void EXPORT
wrstatSetup()
{
#ifdef WR_WINNT
   gwmcommSetup();
#endif /* WR_WINNT */
}

/*
 * This function displays the available linkers supported by the WSS.
 */
int EXPORT
wrstatLinkers()
{
   int         error;

   error = WR_wrctl( WRCTL_STAT_LINKERS, 0, 0 );

#ifndef WR_SOLARIS
   if( error < 0 )
   {
      errno = -error;
      error = -1;
   }
#endif /* !WR_SOLARIS */

   return( error );
}

/*
 * This function displays the reference counts for components of the WSS.
 */
int EXPORT
wrstatRefs()
{
   int         error;

   error = WR_wrctl( WRCTL_STAT_REFS, WRCTL_STAT_REFS_ALL, 0 );

#ifndef WR_SOLARIS
   if( error < 0 )
   {
      errno = -error;
      error = -1;
   }
#endif /* !WR_SOLARIS */

   return( error );
}

/*
 * This function displays the memory allocations for components of the WSS.
 */
int EXPORT
wrstatMemory()
{
   int         error;

   error = WR_wrctl( WRCTL_STAT_MEMORY, 0, 0 );

#ifndef WR_SOLARIS
   if( error < 0 )
   {
      errno = -error;
      error = -1;
   }
#endif /* !WR_SOLARIS */

   return( error );
}

