/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: libwrv.h,v 1.43 2001/06/22 04:15:10 stevek Exp $
 *
 * \brief   WRV command definitions, used both in the wrselect program and
 *          in the wr_wrv call  
 * \author  Steve Kiernan
 * \version $Revision: 1.43 $
 * \date    $Date: 2001/06/22 04:15:10 $
 */

#ifndef  __LIBWRV_H
#define  __LIBWRV_H  1

#include <wr_conf.h>

#include <wss_db.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define WREV_EFS_GROUP         PD_MAKESTR("wrappers")

#define WREV_EFS_RAW_STREAM    PD_MAKESTR("raw")
#define WREV_EFS_DB_STREAM     PD_MAKESTR("db")
#define WREV_EFS_MODULE_STREAM PD_MAKESTR("module")


#define WREV_AUDIT_GROUP        PD_MAKESTR("audit")
#define WREV_AUDIT_RAW_STREAM   PD_MAKESTR("raw")

/* Event types */
#define WREV_ET_CREATETAB           PD_MAKESTR("create table")
#define WREV_ET_DROPTAB             PD_MAKESTR("drop table")
#define WREV_ET_UPDATETAB_INSERTROW PD_MAKESTR("insert table row")
#define WREV_ET_UPDATETAB_DELETEROW PD_MAKESTR("delete table row")
#define WREV_ET_UPDATETAB_UPDATEROW PD_MAKESTR("update table row")
#define WREV_ET_WRAPPER_LOADED      PD_MAKESTR("wrapper loaded")
#define WREV_ET_ACTCRIT_LOADED      PD_MAKESTR("activation criteria loaded")
#define WREV_ET_DEACTCRIT_LOADED    PD_MAKESTR("deactivation criteria loaded")
#define WREV_ET_LIBRARY_LOADED      PD_MAKESTR("library loaded")
#define WREV_ET_WRAPPER_UNLOADED    PD_MAKESTR("wrapper unloaded")
#define WREV_ET_ACTCRIT_UNLOADED    PD_MAKESTR("activation criteria unloaded")
#define WREV_ET_DEACTCRIT_UNLOADED  PD_MAKESTR("deactivation criteria unloaded")
#define WREV_ET_LIBRARY_UNLOADED    PD_MAKESTR("library unloaded")


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

#ifndef PD_IN_KERNEL

/*! \brief Table identifier/name pair structure */
typedef struct wrv_table_id
{
   unsigned long        handle; /*!< Table handle                   */
   LPTSTR               name;   /*!< Table name                     */
   struct wrv_table_id *next;   /*!< Pointer to next pair structure */
}
wrv_table_id_t;

/*! \brief Buffer type for communicating with wrv syscall */
typedef char wrv_buffer;


/*
 * Augmented wrapper database structures
 *
 *   These are used so the returned database information
 *   can be structured in array format in additional to
 *   the linked list (mostly for knowing the number of
 *   columns, rows, and values)
 */
typedef struct wrdb_table
{
   wqtab    tab;
   int      numcol_defs;
   int      numrows;
   unsigned long  handle;
}
wrdb_table, *wrdb_table_t;

#define TABLE(t) (&((wrdb_table_t)t)->tab)
#define TABLE_NUMCOLS(t) (((wrdb_table_t)t)->numcol_defs)
#define TABLE_NUMROWS(t) (((wrdb_table_t)t)->numrows)

struct _wrdb_row
{
   wqrow    row;
   int      numvals;
   unsigned long  handle;
};
typedef struct _wrdb_row wrdb_row, *wrdb_row_t;

#define ROW(r) (&((wrdb_row_t)r)->row)
#define ROW_NEXT(r) ((wrdb_row_t)((wrdb_row_t)r)->row.next)
#define ROW_NUMVALS(r) (((wrdb_row_t)r)->numvals)
#define ROW_HANDLE(r)  (((wrdb_row_t)r)->handle)

typedef wqcd   wrdb_coldef;
typedef wqcd_t wrdb_coldef_t;
typedef wqval  wrdb_value;
typedef wqval_t   wrdb_value_t;

#endif /* !PD_IN_KERNEL */

/*! \brief WRV command data handle parameter */
typedef struct _wrv_h
{
   int   handle;           /*!< Handle to item */
}
wrv_h_t;

/*! \brief WRV command table and row handle parameter */
typedef struct _wrv_tr
{
   int   thandle;          /*!< Table handle */
   int   rhandle;          /*!< Row handle   */
}
wrv_tr_t;

/*! \brief WRV command table and row handle, string name, and data parameter
 *
 *  Data is a variable-length block of memory which is located after the
 *  \a name parameter.
 */
typedef struct _wrv_trnd
{
   int   thandle;          /*!< Table handle */
   int   rhandle;          /*!< Row handle   */
   TCHAR name[MAXNAMELEN]; /*!< String name  */
}
wrv_trnd_t;

/*! \brief WRV command table and row data.
 *
 *  Data is a variable-length block of memory which is located after the
 *  \a handle parameter.
 */
typedef struct _wrv_hrd
{
    u_long      handle;         /*!< Table handle */
}
wrv_hrd_t;

#ifdef PD_IN_KERNEL

#define WREV_CREATETAB           wrvevent_type_createtab
#define WREV_DROPTAB             wrvevent_type_droptab
#define WREV_UPDATETAB_INSERTROW wrvevent_type_insertrow
#define WREV_UPDATETAB_DELETEROW wrvevent_type_deleterow
#define WREV_UPDATETAB_UPDATEROW wrvevent_type_updaterow
#define WREV_MODULE_LOADED       wrvevent_type_modloaded
#define WREV_MODULE_UNLOADED     wrvevent_type_modunloaded

extern u_long WSSLINKAGE WREV_CREATETAB;
extern u_long WSSLINKAGE WREV_DROPTAB;
extern u_long WSSLINKAGE WREV_UPDATETAB_INSERTROW;
extern u_long WSSLINKAGE WREV_UPDATETAB_DELETEROW;
extern u_long WSSLINKAGE WREV_UPDATETAB_UPDATEROW;
extern u_long WSSLINKAGE WREV_MODULE_LOADED;
extern u_long WSSLINKAGE WREV_MODULE_UNLOADED;
#  endif /* PD_IN_KERNEL */


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

#ifndef PD_IN_KERNEL
int            LINKAGE  wrvInit                      ( void );
int            LINKAGE  wrvAddActivationCriteria     ( const char * );
int            LINKAGE  wrvAddDeactivationCriteria   ( const char * );
void           LINKAGE  wrvAdminDone                 ( void );

/* Activation and Deactivation criteria */
wrv_buffer     LINKAGE *wrvGetActivationCriteriaByHandle __P(( int handle ));
wrv_buffer     LINKAGE *wrvGetActivationCriteria     __P(( void ));
wrv_buffer     LINKAGE *wrvGetDeactivationCriteriaByHandle __P(( int handle ));
wrv_buffer     LINKAGE *wrvGetDeactivationCriteria   __P(( void ));
const char     LINKAGE *wrvGetParserError            __P(( void ));

/* Module manipulation and information */
wrv_buffer     LINKAGE *wrvGetActiveWrappers         __P(( void ));
wrv_buffer     LINKAGE *wrvGetLibraryByHandle        __P(( int handle ));
wrv_buffer     LINKAGE *wrvGetLibraries              __P(( void ));
wrv_buffer     LINKAGE *wrvGetModuleByHandle         __P(( int handle ));
unsigned long  LINKAGE  wrvGetModuleIdByName         __P(( const char *name ));
wrv_buffer     LINKAGE *wrvGetModules                __P(( void ));
wrv_buffer     LINKAGE *wrvGetWrapperByHandle        __P(( int handle ));
wrv_buffer     LINKAGE *wrvGetWrappers               __P(( void ));
int            LINKAGE  wrvLoadModule                __P(( const char * ));
void           LINKAGE  wrvReleaseActivationCriteria __P(( void * ));
void           LINKAGE  wrvReleaseActiveWrappers     __P(( void * ));
void           LINKAGE  wrvReleaseDeactivationCriteria __P(( void * ));
void           LINKAGE  wrvReleaseLibraries          __P(( void * ));
void           LINKAGE  wrvReleaseModules            __P(( void * ));
void           LINKAGE  wrvReleaseProcesses          __P(( void * ));
void           LINKAGE  wrvReleaseWrappedProcesses   __P(( void * ));
void           LINKAGE  wrvReleaseWrappers           __P(( void * ));
int            LINKAGE  wrvUnloadModule              __P(( int handle ));

/* General information */
int            LINKAGE  wrvIsLoaded                  __P(( void ));
float          LINKAGE  wrvGetVersion                __P(( void ));
wrv_buffer     LINKAGE *wrvGetProcessByPID           __P(( pid_t pid ));
wrv_buffer     LINKAGE *wrvGetWrappedProcesses       __P(( int handle ));
int            LINKAGE  wrvShutdown                  __P(( void ));

/* wrv_buffer accessors */
void           LINKAGE  wrvBufferDump                __P(( wrv_buffer *buf ));
char           LINKAGE *wrvBufferGetString           __P(( wrv_buffer *buf,
                                                            unsigned int row,
                                                            unsigned int col ));
size_t         LINKAGE  wrvBufferGetRowCount         __P(( wrv_buffer *buf ));
size_t         LINKAGE  wrvBufferGetColCount         __P(( wrv_buffer *buf ));


/* Database accessors */
int            LINKAGE  wrvDbAddNullTableRow         __P(( unsigned long handle ));
int            LINKAGE  wrvDbAddTableRow             __P(( unsigned long handle,
                                                           LPTSTR rows[]));
int            LINKAGE  wrvDbDelTableRow             __P(( unsigned long thandle,
                                                           unsigned long rhandle ));
wrdb_coldef_t  LINKAGE  wrvDbGetColumnDefinition     __P(( unsigned long handle,
                                                           int *size ));
wrdb_row_t     LINKAGE  wrvDbGetRows                 __P(( unsigned long thandle ));
wrdb_row_t     LINKAGE  wrvDbGetRowByHandle          __P(( unsigned long thandle,
                                                           unsigned long handles ));
wrdb_row_t     LINKAGE  wrvDbGetRowsByHandle         __P(( unsigned long thandle,
                                                           unsigned long *handles,
                                                           int size ));
wrv_table_id_t LINKAGE *wrvDbGetTableIdByName        __P(( const char *name ));
wrv_table_id_t LINKAGE *wrvDbGetTableIdByHandle      __P(( unsigned long handles ));
wrv_table_id_t LINKAGE *wrvDbGetTableIdsByHandle     __P(( unsigned long *handles,
                                                           unsigned int size ));
wrv_table_id_t LINKAGE *wrvDbGetTableIds             __P(( void ));
void           LINKAGE  wrvDbReleaseColumnDefinition __P(( wrdb_coldef_t cd ));
void           LINKAGE  wrvDbReleaseRows             __P(( wrdb_row_t row ));
void           LINKAGE  wrvDbReleaseTableIds         __P(( wrv_table_id_t *table ));
int            LINKAGE  wrvDbSetTableRowField        __P(( unsigned long thandle,
                                                           unsigned long rhandle,
                                                           const char *field,
                                                           const char *value ));


#ifdef WR_WINNT
/* Windows NT-specific function */
int             LINKAGE wrvntProcessCommand    __P(( DWORD command,
                                                     PVOID pSendBuf,
                                                     UINT sendSize,
                                                     PVOID *pRecvBuf,
                                                     UINT *pRecvSize ));
#endif /* WR_WINNT */

#ifdef WR_UNIX
int             LINKAGE wrvunixProcessCommand  __P(( u_long command,
                                                     void *pSendBuf,
                                                     u_int sendSize,
                                                     void **pRecvBuf,
                                                     u_int *pRecvSize ));
#endif /* WR_UNIX */


#endif /* !PD_IN_KERNEL */

/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#ifndef PD_IN_KERNEL
#ifdef WR_WINNT
#define wrvCallSystem(c)                       wrvntProcessCommand(c,NULL,0,NULL,NULL)
#define wrvCallSystemSendData(c,b,s)           wrvntProcessCommand(c,b,s,NULL,NULL)
#define wrvCallSystemRecvData(c,pb,ps)         wrvntProcessCommand(c,NULL,0,pb,ps)
#define wrvCallSystemSendRecvData(c,b,s,pb,ps) wrvntProcessCommand(c,b,s,pb,ps)
#endif /* WR_WINNT */

#ifdef WR_UNIX
#define wrvCallSystem(c)                       wrvunixProcessCommand(c,NULL,0,NULL,NULL)
#define wrvCallSystemSendData(c,b,s)           wrvunixProcessCommand(c,b,s,NULL,NULL)
#define wrvCallSystemRecvData(c,pb,ps)         wrvunixProcessCommand(c,NULL,0,pb,ps)
#define wrvCallSystemSendRecvData(c,b,s,pb,ps) wrvunixProcessCommand(c,b,s,pb,ps)
#endif /* WR_UNIX */

#define wrvGetModules()                 wrvGetModuleByHandle(0U)
#define wrvGetWrappers()                wrvGetWrapperByHandle(0U)
#define wrvGetLibraries()               wrvGetLibraryByHandle(0U)
#define wrvGetProcesses()               wrvGetWrappedProcesses(0U)
#define wrvGetActivationCriteria()      wrvGetActivationCriteriaByHandle(0U)
#define wrvGetDeactivationCriteria()    wrvGetDeactivationCriteriaByHandle(0U)

/* Database accessors */
#define wrvDbGetRows(thandle)           wrvDbGetRowsByHandle(thandle,0,0)
#define wrvDbGetRowByHandle(thandle,handle)  wrvDbGetRowsByHandle(thandle,&handle,1)
#define wrvDbGetTableIds()              wrvDbGetTableIdsByHandle(0,0)
#define wrvDbGetTableIdByHandle(handle) wrvDbGetTableIdsByHandle(&handle,1)

#endif /* !PD_IN_KERNEL */

#ifdef __cplusplus
}
#endif

#endif /* !__LIBWRV_H */

