/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrvact.cc,v 1.19 2001/04/05 18:41:37 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 */

#ifndef LINKAGE
#define LINKAGE IMPORT
#endif

#include <wr_conf.h>

#ifdef WR_UNIX
#include <sys/param.h>
#include <unistd.h>
#endif /* WR_UNIX */
#ifdef WR_WINNT
#include <io.h>
#endif /* WR_WINNT */
#include <errno.h>

#include "libre.h"
#include "libcrit.h"
#include "libwrmod.h"

#undef  LINKAGE
#define LINKAGE EXPORT

#ifdef WR_FREEBSD
#  define LIB_ERROR -error
#endif
#ifdef WR_SOLARIS
#  define LIB_ERROR errno
#endif
#ifdef WR_LINUX
#  define LIB_ERROR  errno
#endif
#ifdef WR_WINNT
#  define LIB_ERROR GetLastError()
#endif

extern "C" {
int EXPORT wrvAddActivationCriteria( const char *criteria );
int EXPORT wrvAddDeactivationCriteria( const char *criteria );
const char EXPORT *wrvGetParserError();
}

#define ACTIVATE 1
#define DEACTIVATE 0


int EXPORT
wrvAddActivationCriteria( const char *criteria )
{
   Processor *proc;
   int        error;

   error = critParse( (char *)criteria, &proc );
   if( error == 0 )
   {
      char *tmpName;

      tmpName = critMakeTempFile( proc, NULL, ACTIVATE );
      if( tmpName == NULL )
      {
         errno = ENOENT;
         error = -1;
      }
      else
      {
         error = wrmodLoad( tmpName );
         unlink( tmpName );
      }
   }
   else
   {
      if( error < 0 ) errno = LIB_ERROR;
      error = -1;
   }

   delete proc;

   return error;
}

int EXPORT
wrvAddDeactivationCriteria( const char *criteria )
{
   Processor *proc;
   int        error;
   
   error = critParse( (char *)criteria, &proc );
   if( error == 0 )
   {
      char *tmpName;

      tmpName = critMakeTempFile( proc, NULL, DEACTIVATE );
      if( tmpName == NULL )
      {
         errno = ENOENT;
         error = -1;
      }
      else
      {
         error = wrmodLoad( tmpName );
         unlink( tmpName );
      }
   }
   else
   {
      if( error < 0 ) errno = LIB_ERROR;
      error = -1;
   }

   delete proc;

   return error;
}

const char EXPORT *
wrvGetParserError()
{
   return critParserError();
}

