/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrvbuffer.c,v 1.14 2001/02/23 19:02:29 stevek Exp $
 *
 * wrv_buffer accessor routines
 */

#define LINKAGE IMPORT

#include <wr_conf.h>

#include <wsi_types.h>
#include <sys/types.h>
#include <stdio.h>

#undef LINKAGE
#define LINKAGE EXPORT
#include "libwrv.h"

void EXPORT
wrvBufferDump (wrv_buffer *buf)
{
  size_t        nRows;
  size_t        nCols;
  unsigned int  row;
  unsigned int  col;

  size_t  *sizes = (size_t *) buf;
#ifdef WR_WINNT
  int     *array = (int *) &sizes[2];
#else
  char   **array = (char **) &sizes[2];
#endif
  char    *s;

  printf ("**************************************\n");

  nRows = (int)sizes[0];
  nCols = (int)sizes[1];

  printf ("# rows = %d\n", nRows);
  printf ("# cols = %d\n", nCols);

  for (row = 0; row < nRows; row++) {
    for (col = 0; col < nCols; col++) {
#ifdef WR_WINNT
      s = buf + array[row*nCols+col];
#else
      s = array[row*nCols+col];
#endif
      printf ("array[%d][%d] = '%s'\n", row, col, 
         s ? s : "<null>");
    }
  }

  printf ("**************************************\n");

}


char EXPORT *
wrvBufferGetString (wrv_buffer *buf, unsigned int row, unsigned int col)
{
  unsigned int nRows;
  unsigned int nCols;

  size_t  *sizes = (size_t *) buf;
#ifdef WR_WINNT
  int     *array = (int *) &sizes[2];
#else
  char   **array = (char **) &sizes[2];
#endif
  char    *s;

  nRows = sizes[0];
  nCols = sizes[1];

  if (row < nRows && col < nCols)
#ifdef WR_WINNT
    s = buf + array[row*nCols+col];
#else
    s = array[row*nCols+col];
#endif
  else {
    s = NULL;
    printf ("Error grabbing string from array!");
  }

  return s ? s : "<null>";

}


size_t EXPORT
wrvBufferGetRowCount(wrv_buffer *buf)
{
  size_t  *sizes = (size_t *) buf;
  return sizes[0];
}


size_t EXPORT
wrvBufferGetColCount(wrv_buffer *buf)
{
  size_t  *sizes = (size_t *) buf;
  return sizes[1];
}

