/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrvunix.c,v 1.12 2001/02/23 19:02:29 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Wrapper Viewer interface library
 *
 */

#include <wr_conf.h>

#include <wsi_types.h>

#include <sys/syscall.h>
#include <unistd.h>
#include <errno.h>

#include <wrappers.h>
#include <wrerrno.h>

#undef LINKAGE
#define LINKAGE EXPORT

#include "libwrv.h"

#ifdef WR_FREEBSD
#  define LIB_ERROR  -error
#endif
#ifdef WR_SOLARIS
#  define LIB_ERROR  errno
#endif
#ifdef WR_LINUX
#  define LIB_ERROR  errno
#endif


int EXPORT
wrvunixProcessCommand( u_long command, void *pSendBuf, u_int sendSize,
                       void **pRecvBuf, u_int *pRecvSize )
{
   int     error;
   void   *recvBuf  = NULL;
   size_t  recvSize = 0;

   if( pRecvSize )
      recvSize = *pRecvSize;

   if( recvSize < sendSize )
      recvSize = sendSize;

   if( recvSize > 0 )
   {
      if( pRecvBuf && *pRecvBuf && pRecvSize )
      {
         if( *pRecvSize < recvSize )
            recvBuf = realloc( *pRecvBuf, recvSize );
         else
            recvBuf = *pRecvBuf;
      }
      else
         recvBuf = malloc( recvSize );
   }

   if( ( ( pSendBuf == NULL ) && ( pRecvBuf == NULL ) ) || ( recvSize <= 0 ) )
      error = WR_wrv( command, NULL, NULL );
   else
   {
      memcpy( recvBuf, pSendBuf, sendSize );
      error = WR_wrv( command, recvBuf, &recvSize );
   }

   if( error < 0 )
   {
      switch( LIB_ERROR )
      {
         case EAGAIN:
            if( recvBuf )
            {
               recvBuf = realloc( recvBuf, recvSize );
               memcpy( recvBuf, pSendBuf, sendSize );
               error = WR_wrv( command, recvBuf, &recvSize );
            }
            if( error >= 0 )
               break;

            /* errors the second time fall-through to the next case */
         default:
            errno = LIB_ERROR;
            if( recvBuf )
               free( (void *)recvBuf );
            if( pRecvBuf )
               *pRecvBuf = NULL;
            return error;
      }
   }

   if( pRecvBuf )
   {
      *pRecvBuf = recvBuf;
      if( pRecvSize )
         *pRecvSize = recvSize;
   }

   return error;
}

