/* Using "Linux" characterization. */

/* Using "system" characterization. */
/* Using default library */


#define PD_IN_KERNEL
#define WSSLINKAGE IMPORT

#define CUR_THREAD_ID wr_curthreadid()

#define PD_LIST_HEAD(name, type)					\
struct name {								\
	struct type *lh_first;	/* first element */			\
}

#define LIST_ENTRY(type)						\
struct {								\
	struct type *le_next;	/* next element */			\
	struct type **le_prev;	/* address of previous next element */	\
}

#define CONF_SYSCONFDIR    "/usr/local/etc"
#define CONF_LOCALSTATEDIR "/var/gswtk"
#define CONF_LIBDIR        "/usr/local/lib"

#ifndef PD_KERNEL_VERSION
#define __NO_VERSION__          1
#endif
#define __KERNEL__              1
#define MODULE                  1
#define _LOOSE_KERNEL_NAMES     1

#include <linux/types.h>
#include <linux/linkage.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/sched.h>

typedef char          pd_char_t;

#define PD_LINUX
#define PD_UNIX

#define bzero(b,n) memset((b),0,(n))
#define bcopy(s,d,n) memcpy((d),(s),(n))

#ifndef MAXPATHLEN
#define MAXPATHLEN 1024
#endif

#define IMPORT
#define EXPORT

typedef struct task_struct proc_t;

#define PD_MUTEX_PROTECT(linkage,var) spinlock_t var ## _lock

#define WR_LINUX PD_LINUX
#define WR_UNIX PD_UNIX

#define WSS_LOG_FILENAME CONF_LOCALSTATEDIR ## "/wss.log"
#define WSS_DB_FILENAME	 CONF_LOCALSTATEDIR ## "/wss.db"
#define WSS_HOME_PATH    CONF_LOCALSTATEDIR
#define WSS_INIT_FILE	 CONF_SYSCONFDIR ## "/wss.conf"

#define WSS_MUTEX_PROTECT(a)          PD_MUTEX_PROTECT(WSSLINKAGE,a)

typedef struct _wss_ref wss_ref_t;

struct _wss_ref
{
   union
   {
      int                  ref_count;
   }
   un;
};

struct wss_wrapper_instance;

enum _wsi_rw_type {
    RW_READER,
    RW_WRITER
};

typedef enum _wsi_rw_type wsi_rw_type_t;

struct _wsi_rw {
    rwlock_t            lock;
    wsi_rw_type_t       type;
};

typedef struct _wsi_rw wsi_rw_t;

#define WSI_RW_PROTECT(var) wsi_rw_t var ## _rwlock

#define WR_ALLOW   0           /*!< Allow call to continue return code    */
#define WR_SUCCESS 0           /*!< Successful completion return code     */

struct wss_proc
{
   /* Fields protected by global wss_proc_list lock */
   LIST_ENTRY(wss_proc)         wrp_p_list;    /*!< List of all processes    */


   WSS_MUTEX_PROTECT           (wrp);          /*!< Mutex protecting
                                                    wrp_refcount, wrp_flags,
                                                    wrp_nsyscall             */
   /* Fields protected by wrp lock */
   wss_ref_t                    wrp_refcount;  /*!< Reference count for
                                                    structure                */
   int                          wrp_flags;     /*!< Flags                    */
   int                          wrp_nsyscall;  /*!< Reference count for
                                                    calls                    */


   WSI_RW_PROTECT              (wrp_cwd);      /*!< Reader/writer lock
                                                    protecting wrp_cwd       */
   /* Fields protected by wrp_cwd lock */
   pd_char_t                  *wrp_cwd;       /*!< Pointer to current
                                                    working directory of
                                                    process. MALLOC'd memory,
                                                    NULL-terminated          */


   WSS_MUTEX_PROTECT           (wrp_wi_list);  /*!< Mutex protecting
                                                    wrp_wi_list              */
   /* Fields protected by wrp_wi_list lock */
   volatile
   struct wss_wrapper_instance *wrp_wi_list;   /*!< Pointer to list of
                                                    wrappers instances       */


   WSS_MUTEX_PROTECT           (wrp_arg_list); /*!< Mutex protecting
                                                    wrp_arg_list             */
   /* Fields protected by wrp_arglist lock */
   void                        *wrp_arg_list;  /*!< Pointer to stack of
                                                    replaced arguments       */


   WSS_MUTEX_PROTECT           (wrp_viewer);   /*!< Mutex protecting
                                                    wrp_viewer               */
   /* Fields protected by wrp_viewer lock */
   volatile struct wss_wrv     *wrp_viewer;    /*!< Pointer to the wrapper
                                                    viewer structure when
                                                    process is a wrapper
                                                    viewer                   */


   /* Not protected by locks (exists same for life of process)
    *
    * Note: wrp_progname can change during exec, but all threads should no
    *       longer have a pointer to the old area of memory which held the
    *       name of the program pre-exec                                     */
   pid_t                        wrp_pid;       /*!< Process identifier       */
   pd_char_t                  *wrp_path;      /*!< Pointer to full path of
                                                    executable. MALLOC'd
                                                    memory, NULL-terminated  */
   pd_char_t                  *wrp_oldpath;   /*!< Pointer to full path of
                                                    old program (parent).
                                                    MALLOC'd memory,
                                                    NULL-terminated          */
   pd_char_t                  *wrp_progname;  /*!< Pointer to program name
                                                    (no path info). MALLOC'd
                                                    memory, NULL-terminated  */
   proc_t                      *wrp_p;         /*!< Pointer to process
                                                    structure (may be unused)*/
   struct sysentvec            *wrp_sysent;    /*!< Original p_sysent vector */
   void                        *wrp_opaque;    /*!< System-dependent data    */
};


struct wss_wrapper_data;
#define WRLINK_MOD_NAME               __wr_myname
#define WRLINK_MOD_DEPENDENCIES       __wr_mydependencies
#define WRLINK_MOD_INTERCEPT_LIST     __wr_myintercepts
#define WRLINK_MOD_TYPE             __wr_mytype

typedef struct wql_table
{
   /* In order to not polute user-space with kernel-space constructs, we
    * use an opaque pointer which contains the appropriate lock for the
    * table (see struct wql_table_opaque above)
    */
   void               *opaque;
   pd_char_t          *name;
   unsigned int        flags;
   int                 col_count;
   int                 max_rows;  /* TBD */
   int                 row_count; /* TBD */
   struct wql_col_def *col_defs;  /* defines this table's format */
   struct wql_row     *rows;
   struct wql_row     *tail;      /* TBD */
   struct wql_table   *next;
} *wqtab_t;

typedef enum wql_dtype { none_t, pd_char_t_t, int_t, float_t } wql_dtype;
typedef struct wql_col_def
{
   pd_char_t *name;
   wql_dtype   dtype;
   int         max_len;
} *wqcd_t;

typedef struct wql_row
{
   struct wql_val *vals;
   struct wql_row *next;
} *wqrow_t;

typedef struct wql_val
{
   union
   {
      pd_char_t  *cval;
      int         ival;
      float       fval;
   }
   v;
} *wqval_t;

struct wql_dbrow
{
   struct wql_table  results;
   struct wql_row   *cur;
};

enum sxop { gt_op, gte_op, lt_op, lte_op, ne_op, eq_op, true_op };
typedef struct sxval *sxval_t;
typedef struct term_exp *tx_t;
typedef struct sx_node *wqsx_t;

int myversion = 1;
int myhandle = 0;

#if defined(MALLOC_DEBUG)
#define wr_strdup(str) (wr_strdup_debug((str), __FILE__, __LINE__))
#define wr_malloc(size) (wr_malloc_debug((size), __FILE__, __LINE__))
#endif

unsigned int IMPORT wr_curthreadid(void);

#define _pid           wrp->wrp_p->pid
#define _progname      wrp->wrp_progname
#define _uid           wrp->wrp_p->uid

/* Using default library */
int  wr_strlen (const  pd_char_t  * str);
pd_char_t  * wr_abspath (const  pd_char_t  * path);
void  wr_bcopy (const  void  * src,void  * dst,size_t  len);
int  wr_printf (const  pd_char_t *, ...);
int  wr_log (const  pd_char_t  * format, ...);
int  wr_audit_printf (unsigned  long  types,const  char *, ...);
int  wr_copyinstr (void  * uaddr,void  * kaddr,size_t  len,unsigned  int  * done);
int  wr_resolve_event_space (pd_char_t  * name);
pd_char_t  * wr_strdup_debug (const  pd_char_t  * str,pd_char_t  * file,int  line);
void  * wr_malloc_debug (size_t  size,pd_char_t  * file,int  line);
int  wr_arg_replace (struct wss_proc  * wrp,int  handle,int  space,int  code,void  * uap [],void  * new_arg,int  index,size_t  size);
void  wr_free (void  * ptr);
const  pd_char_t  * wr_syscall_name (int  call);
int  wr_substitute (pd_char_t  ** src,pd_char_t  * txt,pd_char_t  * sub);
/* end library */

#ifndef WR_SUPPRESS_WQL

typedef struct wql_dbrow dbrow;

wqtab_t IMPORT wss_db_find_table_by_handle_r( unsigned long handle );

int IMPORT wql_create( pd_char_t *wrapper_name,
		        int instance_id,
		        pd_char_t *table_name,
		        int free_col_defs,
		        struct wql_col_def *col_defs,
                       int col_count,
                       unsigned long *table_id);
int IMPORT wql_drop(	unsigned long table_id );
int IMPORT wql_insert(	unsigned long table_id,
		        int free_vals,
		        wqrow_t vals);
int IMPORT wql_select(	unsigned long table_id,
		        int free_scrit,
		        wqsx_t scrit,
                       int take_first,
		        int order_by,
		        int desc,
                       wqtab_t out_table);

u_long IMPORT wql_find_table_by_names( pd_char_t *wrapper_name, int id,
                                       pd_char_t *table_name );
wqcd_t IMPORT wr_mcol_def( int count, ... );
wqval_t IMPORT wr_mvals( int count, ... );
wqrow_t IMPORT wr_mrow( wqval_t vals, wqrow_t next );
void IMPORT dbrow_finish( struct wql_dbrow *dbrp );

/*
 * See wss_db.c for the reference implimentations.
 */
#define dbrow_start(dbrp) \
((((dbrp)->cur) = (dbrp)->results.rows),((dbrp) != NULL && (dbrp)->cur != NULL))

#define dbrow_next(dbrp) (((dbrp)->cur = (dbrp)->cur->next) != NULL)

#define dbrow_isempty(dbrp) ((dbrp)->results.rows == NULL)

#endif /* WR_SUPPRESS_WQL */

    
/* Mutex stuff */

#define WR_MUTEX_INIT(var)
#define WR_MUTEX_DESTROY(var)
#define WR_MUTEX_ENTER(var)
#define WR_MUTEX_EXIT(var)


static pd_char_t *WRLINK_MOD_DEPENDENCIES="";
pd_char_t EXPORT *get_mod_dependencies()
{                                  
  return WRLINK_MOD_DEPENDENCIES;  
}                                  

WSS_MUTEX_PROTECT(pathnames);
unsigned long pathnames = 0;
static int Linux=0;
static int system=0;

static int WRLINK_MOD_INTERCEPT_LIST[] = 
{
   5, 8, 9, 10, 11, 12, 14,
   15, 16, 18, 21, 30, 33, 38, 39,
   40, 51, 61, 83, 84, 85, 86, 87,
   92, 106, 107, 115, 131, 182, 0
};


pd_char_t * ConvertPath (pd_char_t  * path,const  pd_char_t  * operation,int  pid,pd_char_t  * progname,int  uid){
    dbrow /* ::path_table*/ row = {{0,0,0,0,0,0,0,0},0};
    pd_char_t  * fullname;
    int  changed = 0;
    if ((fullname = wr_abspath (path))==((void  *)0))
        fullname = (wr_strdup_debug ((path),"dbsynthetic.wr",67));
    {
        int wr_retval;
        do {
            wr_retval = 
                ((row.results.col_defs != NULL ? dbrow_finish(&row) : 0),
                wql_select(pathnames,0,
                	NULL,
                	0,-1, 0, 
                	&(row.results)));
                dbrow_start(&row);
            if (wr_retval < 0) break;
        
        } while (0);
    }
    if (!dbrow_isempty(&row))
        {
            do
            {
                if (wr_substitute(&fullname, (row).cur->vals[0].v.cval, (row).cur->vals[1].v.cval))
                    changed++;
            }
            
            while (dbrow_next(&row));
        }
    if (changed)
        {
            wr_audit_printf (1<<4,"User %d running program %.128s redirected from \"%.128s\" to \"%.128s\" on syscall %.10s\n",uid,progname,path,fullname,operation);
            { dbrow_finish(&row);
            return fullname; }
        }
    else {
        wr_free (fullname);
        { dbrow_finish(&row);
        return ((void  *)0); }
    }
    dbrow_finish(&row);
}
/* WIV Functions
 */

#define MYNAME "dbsynthetic"

static pd_char_t *WRLINK_MOD_NAME = MYNAME;
static pd_char_t *myname=MYNAME;

pd_char_t EXPORT **get_mod_name()
{
   return &WRLINK_MOD_NAME;
}

static pd_char_t *WRLINK_MOD_TYPE="wrapper";
pd_char_t EXPORT **get_mod_type()
{
   return &WRLINK_MOD_TYPE;
}


/* The following is in the "(space == Linux)" event subsystem */

static int pre0 (int space, int code, void *uap[], int *retval, struct wss_proc *wrp,struct wss_wrapper_data *data,int args_in_userspace)
{
if ((space == Linux) && ((code==33) || (code==51) || (code==12) || (code==15) || (code==182) || (code==61) || (code==8) || (code==11) || (code==16) || (code==9) || (code==84) || (code==39) || (code==14) || (code==21) || (code==107) || (code==106) || (code==5) || (code==131) || (code==85) || (code==38) || (code==40) || (code==18) || (code==115) || (code==87) || (code==83) || (code==92) || (code==10) || (code==86) || (code==30)))
{   void *uapk[10];
    int wrp_return = WR_ALLOW;
    /* copy in arg $path */
    if (!args_in_userspace) {
        wr_bcopy(uap, uapk, 10 * sizeof(void*));
    } else {
        int done;
        if (uap[ code == 131 ? 1 : 0 ]==NULL)uapk[ code == 131 ? 1 : 0 ]=NULL;
        else {
        uapk[ code == 131 ? 1 : 0 ] = wr_malloc(MAXPATHLEN*sizeof(pd_char_t));
        wr_copyinstr(uap[ code == 131 ? 1 : 0 ], uapk[ code == 131 ? 1 : 0 ], MAXPATHLEN, &done);
        }
    }
    {
        if (((char  *) uapk[ code == 131 ? 1 : 0 ]))
            {
                pd_char_t  * newPath;
                newPath = ConvertPath (((char  *) uapk[ code == 131 ? 1 : 0 ]),(wr_syscall_name(code)),(int )_pid,_progname,(int )_uid);
                if (newPath)
                    {
                        int  uid;
                        uid = _uid;
                        if (!args_in_userspace); else {
                            wr_free(uapk[ code == 131 ? 1 : 0 ]);
                        }
                        ((char  *) uapk[ code == 131 ? 1 : 0 ]) = newPath;
                    }
            }
    }
    preend_0:
    {
        if (!args_in_userspace) {
            wr_log("arg_repl not supported in sysspace.\n");
        } else {
            if( ((pd_char_t *) uapk[ code == 131 ? 1 : 0 ])!=NULL ) {
            (void)wr_arg_replace(wrp, CUR_THREAD_ID, space, code, uap, ((pd_char_t *) uapk[ code == 131 ? 1 : 0 ]), code == 131 ? 1 : 0, sizeof(pd_char_t) * (wr_strlen(((pd_char_t *) uapk[ code == 131 ? 1 : 0 ]))+1));
            wr_free(((pd_char_t *) uapk[ code == 131 ? 1 : 0 ]));
            }
        }
    }
    return wrp_return;
}
return WR_SUCCESS;
}

/* The following is in the "(space == Linux)" event subsystem */

static int pre1 (int space, int code, void *uap[], int *retval, struct wss_proc *wrp,struct wss_wrapper_data *data,int args_in_userspace)
{
if ((space == Linux) && ((code==9) || (code==21) || (code==38) || (code==83)))
{   void *uapk[10];
    int wrp_return = WR_ALLOW;
    /* copy in arg $to */
    if (!args_in_userspace) {
        wr_bcopy(uap, uapk, 10 * sizeof(void*));
    } else {
        int done;
        if (uap[ 1 ]==NULL)uapk[ 1 ]=NULL;
        else {
        uapk[ 1 ] = wr_malloc(MAXPATHLEN*sizeof(pd_char_t));
        wr_copyinstr(uap[ 1 ], uapk[ 1 ], MAXPATHLEN, &done);
        }
    }
    {
        if (((char  *) uapk[ 1 ]))
            {
                pd_char_t  * newPath;
                newPath = ConvertPath (((char  *) uapk[ 1 ]),(wr_syscall_name(code)),(int )_pid,_progname,(int )_uid);
                if (newPath)
                    {
                        int  uid;
                        uid = _uid;
                        if (!args_in_userspace); else {
                            wr_free(uapk[ 1 ]);
                        }
                        ((char  *) uapk[ 1 ]) = newPath;
                    }
            }
    }
    preend_1:
    {
        if (!args_in_userspace) {
            wr_log("arg_repl not supported in sysspace.\n");
        } else {
            if( ((pd_char_t *) uapk[ 1 ])!=NULL ) {
            (void)wr_arg_replace(wrp, CUR_THREAD_ID, space, code, uap, ((pd_char_t *) uapk[ 1 ]), 1, sizeof(pd_char_t) * (wr_strlen(((pd_char_t *) uapk[ 1 ]))+1));
            wr_free(((pd_char_t *) uapk[ 1 ]));
            }
        }
    }
    return wrp_return;
}
return WR_SUCCESS;
}

/* lifecycle function */
int EXPORT wr_uninstall ( )          
{                                    
{
    {
        int wr_retval;
        do {
            wr_retval = 
                wql_drop(pathnames);
            if (wr_retval < 0) break;
        
        } while (0);
    }
}
    WR_MUTEX_DESTROY(pathnames);

    return WR_SUCCESS;
}

/* lifecycle function */
int EXPORT wr_install ( int newhandle )
{
    myhandle = newhandle;
    {
        WR_MUTEX_INIT(pathnames);
        WR_MUTEX_ENTER(pathnames);
        pathnames = wql_find_table_by_names(myname, 0, "pathnames");
        WR_MUTEX_EXIT(pathnames);
        Linux = wr_resolve_event_space("Linux");
        system = wr_resolve_event_space("system");
    }
{
    int  i;
    WR_MUTEX_ENTER(pathnames);
    {
        int wr_retval;
        do {
            wr_retval = 
                wql_create(myname, 0, "pathnames", !NULL, wr_mcol_def(2, wr_strdup("target"), pd_char_t_t, 256,  wr_strdup("replacement"), pd_char_t_t, 256)
                , 2, &pathnames);
            if (wr_retval < 0) break;
            wr_retval = 
                wql_insert(pathnames, !NULL, 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/etc/master.passwd"),  pd_char_t_t, wr_strdup("/etc/passwd")), 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/boot/vmlinuz"),  pd_char_t_t, wr_strdup("/etc/motd")), 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/lib/modules"),  pd_char_t_t, wr_strdup("/tmp")), 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/usr/games"),  pd_char_t_t, wr_strdup("/usr/bin")), 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/usr/tmp"),  pd_char_t_t, wr_strdup("/tmp")), 
		wr_mrow(wr_mvals(2, pd_char_t_t, wr_strdup("/var/tmp"),  pd_char_t_t, wr_strdup("/tmp")), NULL)))))));
            if (wr_retval < 0) break;
        
        } while (0);
    (i) = wr_retval;
    }
    ;
    WR_MUTEX_EXIT(pathnames);
    if (i<0 && i!=-6)
        {
            wr_printf ("Error creating or populating table pathnames.\n");
        }
}

    return WR_SUCCESS;
}

/* lifecycle function */
int EXPORT wr_activate ( struct wss_proc *wrp, struct wss_wrapper_data *data)
{

    return WR_SUCCESS;
}

/* lifecycle function */
int EXPORT wr_duplicate ( struct wss_proc *wrp, struct wss_wrapper_data *data)
{

    return WR_SUCCESS;
}

/* lifecycle function */
int EXPORT wr_deactivate ( struct wss_proc *wrp, struct wss_wrapper_data *data)
{

    return WR_SUCCESS;
}
/* Dispatch routines. */
int EXPORT dispatch_pre(int space, int code, void *uap[],
                        int *retval,struct wss_proc *wrp,
                        struct wss_wrapper_data *data,
                        int args_in_userspace)
{
    int ret = WR_SUCCESS;

    if (ret == WR_SUCCESS)
         ret=pre0(space, code, uap, retval, wrp, data, args_in_userspace);
    if (ret == WR_SUCCESS)
         ret=pre1(space, code, uap, retval, wrp, data, args_in_userspace);
    return ret;
}
int EXPORT dispatch_post(int space, int code, void *uap[],
                        int *retval,struct wss_proc *wrp,
                        struct wss_wrapper_data *data,
                        int args_in_userspace)
{
    int ret = WR_SUCCESS;

    return ret;
}
