/*
 *  Copyright (c) 1999, 2000 Network Associates, Incorporated
 *  All rights reserved.
 *	 
 *  Redistribution and use are governed by the terms detailed in the
 *  license document ("LICENSE") included with the toolkit.
 *
 * $Id: Message.h,v 1.6 2000/10/27 20:41:23 dougk Exp $
 * 
 * File: Message.h
 * 
 * Author: Douglas Kilpatrick
 * 
 * Purpose: Message classes to encapsulate talking to the wrappers
 * event stream.
 * 
 * Classes:
 * 
 *      Source: data source
 *      EventSource: Source subclass that talks to the event manager.
 *      
 *      Message: root message class.
 *      execMessage: subclass for process creation
 *      exitMessage: subclass for process termination
 *      syscallMessage: subclass for syscall execution
 *      quitMessage: subclass for wrapper shutdown
 *
 */

#ifndef MESSAGE_H
#define MESSAGE_H

#ifdef WR_BROKEN_STL
#include "template/list"
#include "template/map"
#else
#include <list>
#include <map>
#endif

#ifndef __LIBEVENTFS_H
extern "C" {
#include "libeventfs.h"
}
#endif

typedef struct audit_data_header header;

class Message {
    protected:
        Message(void) { };
    public:
        virtual bool toEmeraldEvent(void) = 0;
        virtual ~Message() { };
        Message(header *, size_t size) {};
};

class MessageSource {
    private:
        eventfs_event_stream_t      stream;
        bool                    fFile;
    public:
        Message *Next(void);

        ~MessageSource();
        MessageSource(int &argc, char **& argv);
};

#endif /* MESSAGE_H */
