/*
 *  Copyright (c) 2000, Network Associates, Incorporated
 *  All rights reserved.
 *	 
 *  Redistribution and use are governed by the terms detailed in the
 *  license document ("LICENSE") included with the toolkit.
 *
 * $Id: pam_wss.c,v 1.2 2000/04/27 19:34:36 dougk Exp $
 *
 * Author: Jeremy Latt
 *
 * This is a pam module that will give us the authetication information 
 * we want.  It should be added to the pam stack after "pam_pwdb.so", and
 * the "likeauth" option should be added onto the pwdb argument list.
 */

#define PAM_SM_AUTH
#include <wr_conf.h>
#include <syslog.h>
#include <security/pam_modules.h>
#include <time.h>

PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
    int retval, *pretval = &retval, pam;
    char type, *timestr, *username;
    time_t t;
    struct tm tm;
    FILE *log;

    /* Get the username */
    username = "(none)";
    pam_get_item(pamh, PAM_USER, (const void **)&username);
    pam = pam_get_data(pamh, "pwdb_setcred_return", (const void **)&pretval);
    if (pam == PAM_NO_MODULE_DATA) {
        /* we can't get the data */
        fprintf(stdout, "CAN'T GET MODULE DATA!\n");
        return PAM_AUTHINFO_UNAVAIL; /* complain */
    }

    t = time(NULL);
    timestr = ctime(&t);
    switch (retval) { /* type of error to log */
    case PAM_USER_UNKNOWN:
        type = 'U';
        break;
    case PAM_AUTH_ERR:
        type = 'P';
        break;
    case PAM_SUCCESS:
        type = 'S';
        break;
    default:
        type = '?';
        break;
    }

    /* Will normally be a named pipe */
    log = fopen("/var/log/pam_wss.log", "a");
    if (log == NULL)
        log = stderr;
    fprintf(log, "%c\t%s\t%s", type, username, timestr);
    fclose(log);
    return retval;	
}

PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh, int flags, int argc, const char **argv)
{
	/* does nothing, but must be defined for the module */
	return PAM_SUCCESS;
}

