/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: main.cc,v 1.9 2001/04/05 18:41:47 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: Main program for the napload tool.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "utils.h"
#include "policy.h"

char *usage_str = "\
Usage:
napload [-vflh] <policy-filename>\n\
     Exclusive options:
     -v just parse/validate the input file, print brief summary\n\
     -f parse/validate input and write table files in current directory\n\
     -l parse/validate input and load into kernel\n\
     -h this help message\n";

void usage()
{
	fprintf(stderr, "%s\n", usage_str);
	exit(2);
}

typedef enum action { validate, write_files, load_kernel } action_t;

/* Defined in YACC file. */
policy *parse_policy_file(FILE *fp);

/*
 * Invoke the parser, which builds an object containing the policy (if the
 * input data parsed correctly).  Call the object's methods according to
 * the command line parameter.
 */
void parse_policy(action_t action, char *fname)
{
	FILE *file = fopen(fname, "r");
	policy *pol = parse_policy_file(file);

	if (pol) {
		switch (action) {
		case validate:
			pol->print();
			break;
		case write_files:
			pol->write_files();
			break;
		case load_kernel:
			pol->load_kernel();
			break;
		}
	}
	fclose(file);
}

int
main(int argc, char **argv)
{
	char c;
	char *argstr = "v:f:l:h";
	int oflag = 0;

	UT_list<char *> namalist;

	while ((c = getopt(argc, argv, argstr)) != -1) {
		switch(c) {
		case 'v':
			if (oflag) usage();
			parse_policy(validate, optarg);
			oflag++;
			break;
		case 'f':
			if (oflag) usage();
			parse_policy(write_files, optarg);
			oflag++;
			break;
		case 'l':
			if (oflag) usage();
			parse_policy(load_kernel, optarg);
			oflag++;
			break;
		case 'h':
			usage();
			break;
		case '?':
			usage();
		}
	}
	if (! oflag) {
		usage();
	}
}
