/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: utils.h,v 1.6 2001/02/23 19:02:30 stevek Exp $
 *
 * Author: Lee Badger
 *
 * Purpose: Some utility functions snarfed from DTE.
 */

#ifndef _UTILS_H_
#define _UTILS_H_

template <class T> class UT_list;

/*
 *	Heavily used list element.
 */
template <class T> class UT_list_element {
/*friend class UT_list<T>;
friend class UT_named_list<T>;
friend class UT_policy_entity_list<T>; */
public:
	UT_list_element *next;
	T item;
	UT_list_element() {next = NULL;}
	~UT_list_element() {}
	/* destructor needed? */
};

/*
 *	Base list container template.
 */
template <class T> class UT_list {
public:
	void prepend(T item) {
                UT_list_element<T> *element = new UT_list_element<T>;

                element->item = item;
                element->next = head;
                head = element;
                if (!last_element) {
                        last_element = element;
                }
	}
	void append(T item) {
               UT_list_element<T> *element = new UT_list_element<T>;

                element->item = item;
                if (last_element) {
                        last_element->next = element;
                } else {
                        head = element;
                }
                last_element = element;
	}
	/*--------------------------------------------------------------
	 * Purpose:  determine whether or not a list is empty
	 *
	 * Returns: NULL if the list is emtpy
	 *	    !NULL otherwise
	 */
	int empty() {
		if ((int)head) {
			return 0;
		}
		else {
			return(1);
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  determine whether or not an object is in a list
	 *
	 * Returns: !NULL if the object is in the list
	 *	    NULL otherwise
	 */
	int inlist(T item) {
		UT_list_element<T> *cur = head;

		while (cur && (cur->item != item)) {
			cur = cur->next;
		}
		return((int)cur);
	}
	/*--------------------------------------------------------------
	 * Purpose:  return first element of a list.
	 */
	T first() {
		if (head) {
			return(head->item);
		}
		else {
			return(NULL);
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  prepare to iterate through the list using next()
	 */
	void reset_iterator() {
		if (head) {
			iterator = head;
		}
		else {
			iterator = NULL;
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  cancel an iteration, if any.
	 */
	void clear_iterator() {
		iterator = NULL;
	}
	/*--------------------------------------------------------------
	 * Purpose:  poor person's iterator: return next element of
	 *	list, where the current position is kept in the
	 *	iterator state variable.  There can be only _one_
	 *	use of iterator at a time, or confusion will result.
	 *	Also, the list should not be modified during iteration.
	 *
	 * Returns: the next item; or NULL if there is no next
	 *	item, if first() wasn't called in the past to
	 *	set up the iteration through the list, or if the
	 *	current position was removed via remove().
	 */
	T next() {
		T ret;
		if (iterator) {
			ret = iterator->item;
			iterator = iterator->next;
			return(ret);
		}
		else {
			return((T)NULL);
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  same as next(), but the iteration position is not
	 *	advanced.
	 *
	 * Returns: the next item; or NULL if there is no next
	 *	item, if first() wasn't called in the past to
	 *	set up the iteration through the list, or if the
	 *	current position was removed via remove().
	 */
	T current() {
		if (iterator) {
			return(iterator->item);
		}
		else {
			return((T)NULL);
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  move the iteration one position
	 */
	void step() {
		if (iterator) {
			iterator = iterator->next;
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  check the progress of an iteration.
	 *
	 * Returns: NULL if the iteration is not finished,
	 *	1 : iteration is complete: next() will return NULL next time.
	 */
	int iteration_complete() {
		if (iterator) {
			return(NULL);
		}
		else {
			return(1);
		}
	}
	/*--------------------------------------------------------------
	 * Purpose:  Remove an item from a list.  If the item to be
	 *	removed is the next step in an iteration of the list,
	 *	cancel the iteration.
	 *
	 * Returns: NULL on success,
	 *	1 : item not found
	 */
	int remove(T item) {
		UT_list_element<T> *tmp = head;
		UT_list_element<T> *tmp2;

		if (tmp == NULL) {
			return(1);
		}
		if (tmp->item == item) {
			head = head->next;
			if (head == NULL) {
				last_element = NULL;
				iterator = NULL;
			}
			if (tmp == iterator) {
				iterator = NULL;
			}
			delete tmp;
		}
		else {
			while (tmp->next && (tmp->next->item != item)) {
				tmp = tmp->next;
			}
			if (!tmp->next) {
				return(1);
			}
			tmp2 = tmp->next;
			tmp->next = tmp2->next;
			if (tmp2 == last_element) {
				last_element = tmp;
				iterator = NULL;
			}
			if (tmp2 == iterator) {
				iterator = NULL;
			}
			delete tmp2;
		}
		return 0;
	}
	/*--------------------------------------------------------------
	 * Purpose:  Clean out a list.  Does not affect the T objects
	 *	pointed to by the list.  Afterwards, the list is empty
	 *	and ready to be used again.
	 */
	void flush() {
		UT_list_element<T> *cur = head;
		UT_list_element<T> *cur2;

		while (cur) {
			cur2 = cur;
			cur = cur->next;
			delete cur2;
		}
		head = NULL; last_element = NULL; iterator = NULL;
	}
	UT_list() {head = NULL; last_element = NULL; iterator = NULL;}
	~UT_list() {
		flush();
	}
protected:
	UT_list_element<T> *head, *last_element, *iterator;
};

/*
 *	List container template for policy entities, e.g., types and
 *	domains, that have name() and print() member functions.
 */
template <class T> class UT_named_list : public UT_list<T> {
public:
	/*--------------------------------------------------------------
	 * Purpose:  display list contents, with a specified amount
	 *	of detail
	 */
	void print(int detail) {
		UT_list_element<T> *cur = head;

		while (cur) {
			cur->item->print(detail);
			cur = cur->next;
		}
	};
	void insert_ordered(T item) {
		UT_list_element<T> *cur1, *cur2;
		UT_list_element<T> *tuple = new UT_list_element<T>;

		tuple->item = item;
		cur1 = head;
		cur2 = head;

		if (cur2 == NULL) {
			tuple->next = NULL;
			head = tuple;
			last_element = tuple;
		}
		else if (strcmp(cur2->item->name(), tuple->item->name()) > 0) {
			tuple->next = head;
			head = tuple;
		}
		else {
			cur2 = cur2->next;
			while (cur2 &&
			(strcmp(cur2->item->name(), tuple->item->name())< 0)) {
				cur1 = cur1->next;
				cur2 = cur2->next;
			}
			tuple->next = cur2;
			cur1->next = tuple;
		}
	}

	T name_to_obj(char *name) {
		UT_list_element<T> *cur = head;

		while (cur) {
			if (!strcmp(cur->item->name(), name)) {
				return(cur->item);
			}
			cur = cur->next;
		}
		return (T)NULL;
	}
};

#endif _UTILS_H_
