#!/usr/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: parsesyscall.pl,v 1.1 2001/04/12 14:11:01 dougk Exp $
#
# Author: Douglas Kilpatrick
#
# Purpose: This file contains the script to generate the
# syscall => code mapping file (wsi_syscallnames.c).  It requires that the
# linux build tree be in /usr/src/linux, and assumes x86, 2.2.X, and stable
# system call numbers.
#

use strict;

open (SYSCALLS, "/usr/src/linux/arch/i386/kernel/entry.S") || die "entry.S: $!";

# Jump to the syscall table.
while (<SYSCALLS>) {
    last if /^ENTRY\(sys_call_table/;
}

my $i = 0;
my $count = 0;

# We assume the following syscall places are empty.
my %socketcall_map = (
    "socket"            => 239,
    "bind"              => 240,
    "connect"           => 241,
    "listen"            => 242,
    "accept"            => 243,
    "getsockname"       => 244,
    "getpeername"       => 245,
    "socketpair"        => 246,
    "send"              => 247,
    "sendto"            => 248,
    "recv"              => 249,
    "recvfrom"          => 250,
    "shutdown"          => 251,
    "setsockopt"        => 252,
    "getsockopt"        => 253,
    "sendmsg"           => 254,
    "recvmsg"           => 255
);

my @syscall;

while (<SYSCALLS>) {
    while (s/\\\s*$//) {
        my $buff = <SYSCALLS>;
        $_ .= $buff;
    }
    if (m|^\s*\.long\s*SYMBOL_NAME\s*\(\s*old_(\w+)\s*\)|) {
        $syscall[$i] = "old_" . $1;
        $count++;
        $i++;
    }

    if (m|^\s*\.long\s*SYMBOL_NAME\s*\(\s*sys_(\w+)\s*\)|) {
        $syscall[$i] = $1;
        $count++;
        $i++;
    }
# On x86 Linux, all system calls return an int, if they know it or
# not.
}

my $lastsyscall = $i;

# Add in the socketcall_map fake syscalls to the official syscall list now.
foreach (keys %socketcall_map) {
    $syscall[$socketcall_map{$_}] = $_;
}

print "Generating systable.cc\n";

open (OUT, ">systable.cc") || die "systable.cc";

print OUT <<EOF;
/*
 * systable.cc
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */

struct systable {
    char *name;
    int num;
} systable[256] = {
EOF

for( $i=0; $i <= 255; $i++ ) {
	if ($syscall[$i]) {
		print OUT "    { \"$syscall[$i]\", $i },\n";
	}
}

print OUT <<EOF;
    { /*(char*)*/0 , 0 }
};

EOF

close(OUT);

exit;
