/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: ubaduids.c,v 1.6 2001/02/23 19:02:30 stevek Exp $
 *
 * This program demonstrates programatic access to the database.  It
 * updates the uidstop.baduids table, which is created by the uidstop
 * wrapper and is assumed to exist.  
 */

#include <stdio.h>
#include "libwrv.h"

int  main (int argc, char** argv);
void update_baduids (int add, unsigned int uid);
void add_baduid (unsigned long table_handle, unsigned int uid);
void delete_baduid (unsigned long table_handle, unsigned int uid);

/* Simple main program that reads request (add or delete)/uid pairs
 * from stdin and calls update_baduids() to do the real work.
 */

int main (int argc, char ** argv)
{
  int add, uid, n;

  printf ("%s: update_baduids() test program\n\n", argv[0]);
  printf ("On a line, enter 1 to add or 0 to delete followed by the UID.\n");
  printf ("E.g.,\n\n1 200\n0 200\nEnter EOF to quit\n\n");

  while (1) {
    n = scanf ("%d %u", &add, &uid);
    if (n != 2) 
      break;
    printf ("\nCalling update_baduids(%d, %u)\n", add, uid);
    update_baduids (add, uid);
  }
  printf ("\nBye.\n");
  return 0;
}


/* update_baduids is the function a program would call to update the
 * uidstop.baduids table.  It access the table and then calls either
 * add_baduid or delete_baduid to deal with the rows */

void update_baduids (int add, unsigned int uid) {

    wrv_table_id_t *table = (wrv_table_id_t*) NULL;

    table = wrvDbGetTableIdByName("uidstop.baduids");
    if (table && table->handle) {
      if (add)
	add_baduid (table->handle, uid);
      else
	delete_baduid (table->handle, uid);
    }
    else {
      fprintf (stderr, "Wrappers: Null handle in database table list!\n");
    }
    wrvDbReleaseTableIds (table);
}


/* add_baduid takes a valid table handle and a uid and adds a new row
 * containing the uid.  No checking is done, so the uid is added even
 * if it's already in the table. */

void add_baduid (unsigned long table_handle, unsigned int uid) 
{
  char uidstring[sizeof(int)*3+1];
  char *new_row[2];
  
  snprintf(uidstring, sizeof(uidstring), "%u", uid);
  new_row[0] = uidstring;
  new_row[1] = NULL;

  if( wrvDbAddTableRow(table_handle, new_row) < 0 )
     perror( "wrvDbAddTableRow" );
}


/* delete_baduid takes a valid table handle and a uid and removes a
 * single row containing that uid if one is in the table */

void delete_baduid (unsigned long table_handle, unsigned int uid)
{
  wrdb_row_t rows = (wrdb_row_t) NULL;

  rows = wrvDbGetRows(table_handle);
  while (rows) {
    if ((ROW_NUMVALS(rows) == 1) && (ROW(rows)->vals->v.ival == uid))
      break;
    rows = ROW_NEXT(rows);
  }
  if (rows && rows->handle)
    if( wrvDbDelTableRow (table_handle, rows->handle) < 0 )
       perror( "wrvDbDelTableRow" );
}
