/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.h,v 1.21 2001/02/23 23:51:11 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the
 *          system-depenedant parts of the process life-cycle code
 */

#ifndef __WSI_PROCESS_H
#define __WSI_PROCESS_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#include <process.h>
#include <errno.h>

#ifndef __WSS_MUTEX_H
#include <wss_mutex.h>
#endif

#ifndef __WSS_CV_H
#include <wss_cv.h>
#endif


/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

struct proc
{
   pid_t p_pid;
   pid_t p_ppid;
   TCHAR *p_user_name; 
};

struct wsi_proc
{
   HANDLE          hLWMPipe;
   volatile HANDLE hClientPipe;
   DWORD           dwPendingChildren;
};

struct wss_proc;


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_PROC_OPAQUE(wrp) ((struct wsi_proc *)(wrp)->wrp_opaque)

#define wsi_proc_get_curproc()     NULL
#define wsi_proc_get_current_pid() _getpid()
#define wsi_proc_pid(p)         ((p)->p_pid)


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
int  WSSLINKAGE wsi_proc_insyscall          __P(( struct wss_proc *wrp ));
void WSSLINKAGE wsi_proc_fork               __P(( struct wss_proc * ));
void WSSLINKAGE wsi_proc_free               __P(( struct wss_proc *wrp ));
void WSSLINKAGE wsi_proc_new                __P(( struct wss_proc *wrp ));
void WSSLINKAGE wsi_proc_force_syscall_exit __P(( struct wss_proc *wrp ));
int  WSSLINKAGE wsi_proc_suspend            __P(( struct proc *p ));
void WSSLINKAGE wsi_proc_continue           __P(( struct proc *p ));
#endif /* WR_SUPPRESS_FUNCTION_DECL */


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wsi_proc_fork(a,b)

#endif /* !__WSI_PROCESS_H */

