/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_rw.h,v 1.8 2001/02/23 19:02:31 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent reader/writer locks
 *
 */

#ifndef __WSI_RW_H
#define __WSI_RW_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSS_MUTEX_H
#include <wss_mutex.h>
#endif /* __WSS_MUTEX_H */

#ifndef __WSS_CV_H
#include <wss_cv.h>
#endif /* __WSS_CV_H */


#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

struct _wsi_rw
{
   WSS_MUTEX_PROTECT(rw);
   int               rw_active;
   int               rw_reads;
   int               rw_writes;
   WSS_CV_PROTECT   (rw_read);
   WSS_CV_PROTECT   (rw_write);
};

enum _wsi_rw_type
{
   RW_READER,
   RW_WRITER
};

typedef struct _wsi_rw      wsi_rw_t;
typedef enum   _wsi_rw_type wsi_rw_type_t;


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

void WSSLINKAGE wsi_rw_destroy   __P(( wsi_rw_t *rw ));
void WSSLINKAGE wsi_rw_downgrade __P(( wsi_rw_t *rw ));
void WSSLINKAGE wsi_rw_enter     __P(( wsi_rw_t *rw, wsi_rw_type_t type ));
void WSSLINKAGE wsi_rw_exit      __P(( wsi_rw_t *rw ));
void WSSLINKAGE wsi_rw_init      __P(( wsi_rw_t *rw ));


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_RW_PROTECT(var) wsi_rw_t var ## _rwlock
#define WSI_RW_PROTECT_EXPORTED(var) wsi_rw_t WSSLINKAGE var ## _rwlock
#define WSI_RW_EXPORT(var) extern wsi_rw_t WSSLINKAGE var ## _rwlock

#ifndef WR_SUPPRESS_FUNCTION_DECL
#define WSI_RW_INIT(var) wsi_rw_init( &(var ## _rwlock) )
#define WSI_RW_DESTROY(var) wsi_rw_destroy( &(var ## _rwlock) )
#define WSI_RW_ENTER(var,type) wsi_rw_enter( &(var ## _rwlock), (type) )
#define WSI_RW_ENTER_DEBUG(var,type) \
   do { printf( "rw_enter(" ## #var ## "_rwlock=0x%x, " \
                "/thread=0x%x @ %s:%d\n", (uint32_t)&(var ## _rwlock), \
                (uint32_t)curthread, __FILE__, __LINE__ ); \
        wsi_rw_enter( &(var ## _rwlock), (type) ); } while(0)
#define WSI_RW_EXIT(var) wsi_rw_exit( &(var ## _rwlock) )
#define WSI_RW_EXIT_DEBUG(var) \
   do { printf( "rw_exit(" ## #var ## "_rwlock=0x%x)/thread=0x%x @ %s:%d\n", \
                (uint32_t)&(var ## _rwlock), (uint32_t)curthread, __FILE__, \
                __LINE__ ); \
        wsi_rw_exit( &(var ## _rwlock) ); } while(0)
#define WSI_RW_DOWNGRADE(var) wsi_rw_downgrade( &(var ## _rwlock) )
#define WSI_RW_DOWNGRADE_DEBUG(var) \
   do { printf( "rw_downgrade(" ## #var ## "_rwlock=0x%x)/thread=0x%x @ " \
                "%s:%d\n", (uint32_t)&(var ## _rwlock), \
                (uint32_t)curthread, __FILE__, __LINE__ ); \
        wsi_rw_downgrade( &(var ## _rwlock) ); } while(0)
#endif /* WR_SUPPRESS_FUNCTION_DECL */

#ifdef __cplusplus
}
#endif

#endif /* !__WSS_RW_H */

