/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_stdlib.h,v 1.22 2001/02/23 19:02:31 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains standard library macros and prototypes
 */

#ifndef __WSI_STDLIB_H
#define __WSI_STDLIB_H 1

#include <wr_conf.h>

#include <stdio.h>
#include <errno.h>

#include <wsi_types.h>

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

typedef int (*wsi_printf_func)( const pd_char_t *, va_list );


/************************************************************************\
 * Public globals                                                       *
\************************************************************************/

extern wsi_printf_func WSSLINKAGE wsi_printf_fn;


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

int        WSSLINKAGE  wsi_sprintf __P(( pd_char_t *buf,
                                         const pd_char_t *format, ... ));
int        WSSLINKAGE  wsi_printf  __P(( const pd_char_t *format, ... ));
int        WSSLINKAGE  wsi_vprintf __P(( const pd_char_t *format,va_list ap ));
size_t     WSSLINKAGE  wsi_strlen  __P(( const pd_char_t *s ));
int        WSSLINKAGE  wsi_strcmp  __P(( const pd_char_t *s1,
                                         const pd_char_t *s2 ));
int        WSSLINKAGE  wsi_strncmp __P(( const pd_char_t *s1,
                                         const pd_char_t *s2, size_t len ));
pd_char_t WSSLINKAGE *wsi_strcpy  __P(( pd_char_t *dst,
                                         const pd_char_t *src ));
pd_char_t WSSLINKAGE *wsi_strncpy __P(( pd_char_t *dst,
                                         const pd_char_t *src, size_t len ));
#ifndef __CYGWIN__
void       WSSLINKAGE  bcopy       ( const void *src, void *dest, size_t len );
void       WSSLINKAGE  bzero       __P(( void *b, size_t len ));
#endif
int        WSSLINKAGE  copyin      __P(( void *uaddr, void *kaddr,
                                         size_t len ));
int        WSSLINKAGE  copyout     __P(( void *kaddr, void *uaddr,
                                         size_t len ));
pd_char_t WSSLINKAGE  wsi_tolower __P(( const pd_char_t c ));
pd_char_t WSSLINKAGE  wsi_toupper __P(( const pd_char_t c ));

pd_char_t WSSLINKAGE *wsi_strunicode __P(( const char *astr, int *size ));
char       WSSLINKAGE *wsi_strascii   __P(( const pd_char_t *ustr,
                                            int *size ));


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wsi_strlen(a)      _tcslen(a)
#define wsi_strcmp(a,b)    _tcscmp(a,b)
#define wsi_strncmp(a,b,n) _tcsncmp(a,b,n)
#define wsi_strcpy(a,b)    _tcscpy(a,b)
#define wsi_strncpy(a,b,n) _tcsncpy(a,b,n)
#define wsi_strcat(a,b)    _tcscat(a,b)
#define wsi_strncat(a,b,n) _tcsncat(a,b,n)
#define wsi_vsprintf       _vstprintf
#define wsi_sprintf        _stprintf
#define wsi_snprintf       _sntprintf

#ifdef __cplusplus
}
#endif

#endif /* !__WSI_STDLIB_H */

