/*! \file wsi_export.c
 *
 * Copyright (c) 1999, 2000, 2001 Network Associates, Incorporated
 * All rights reserved.
 *	 
 * Redistribution and use are governed by the terms detailed in the
 * license document ("LICENSE") included with the toolkit.
 *
 * $Id: wsi_export.c,v 1.8 2001/02/23 19:02:30 stevek Exp $
 *
 * \brief   Table used to export system-dependent functions from the WSS to
 *          activation criteria and to wrappers
 * \author  Steve Kiernan
 * \version $Revision: 1.8 $
 * \date    $Date: 2001/02/23 19:02:30 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_export.h>
#include <wss_linker.h>

#include <wsi_iface.h>


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

/*! \brief Table which includes all the local symbols we need to export to
 *         wrappers
 * 
 *  In Solaris we use it to deal with builtin math symbols.  It is currently
 *  unused in FreeBSD and Windows.
 */
wss_symbol_t wsi_export_table[] =
{
   WSS_EXPORT_ENTRY("wr_strunicode",      (u_long)wsi_strunicode),
   WSS_EXPORT_ENTRY("wr_strascii",        (u_long)wsi_strascii),
   WSS_EXPORT_ENTRY("wr_seterror",        (u_long)SetLastError),
   WSS_EXPORT_ENTRY("wr_time_get_current_string",   (u_long)wsi_time_get_current_string),
   { NULL, 0, 0 }
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_export_init */
/*! \brief Initialize system-dependent exported functions table
 *
 *  \return 0 (other values reserved for future use)
 */
int EXPORT
wsi_export_init()
{
   return wss_linker_add_symbols( PD_MAKESTR("*WSI WINDOWS NT*"),
                                  wsi_export_table );
}

