/*! \file wsi_module.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_module.c,v 1.13 2001/02/23 19:02:30 stevek Exp $
 *
 * \brief   Handle freeing system-dependent memory used by wr_module structure
 * \author  Steve Kiernan
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:30 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_module.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_mod_delete */
/*! \brief Release resources used by module
 *
 *  \param wrm Pointer to wrapper module structure
 *
 *  \sa wss_mod_delete
 */
void EXPORT
wsi_mod_delete( wss_mod_t *wrm )
{
   if( wrm->wrm_opaque )
      wss_free( (void *)wrm->wrm_opaque );
}

/*  wsi_mod_new */
/*! \brief Allocate memory necessary for system-dependent part of wrapper
 *         module structure
 *
 *  \param wrm Pointer to wrapper module structure
 *  \param lf Pointer to linker file structure
 *
 *  \return 0 on success,
 *  \return ENOMEM if allocation fails
 *
 *  \sa wss_mod_new
 */
int EXPORT
wsi_mod_new( wss_mod_t *wrm, wss_linker_file_t *lf )
{
   wsi_mod_t *wrmi = wss_malloc( sizeof(*wrmi) );
   if( !wrmi )
      return ENOMEM;

   memset( wrmi, 0, sizeof(*wrmi) );
   wrm->wrm_opaque = (void *)wrmi;

   return 0;
}

