/*! \file wsi_path.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_path.c,v 1.14 2001/02/23 19:02:30 stevek Exp $
 *
 * \author  Larry Spector
 *
 * \brief   Functions used to determine a processes current working directory
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>
#include <wsi_path.h>
#include <wss_malloc.h>
#include <wss_utilities.h>


/*  wsi_path_abspath */
/*! \brief Converts the given path to absolute terms
 *
 *  This function is made available to wrappers.
 *
 *  \param path Pointer to string containing the path
 *
 *  \warning wrappers can abuse this
 *
 *  \return Pointer to string containing the path in absolute terms on success,
 *  \return NULL on failure
 */
pd_char_t EXPORT *
wsi_path_abspath( const pd_char_t *path )
{
   pd_char_t *absPath = (pd_char_t *)wss_malloc( MAX_PATH *
                                                   sizeof(pd_char_t) );
   pd_char_t *filePart = NULL;

   DWORD retVal = GetFullPathName( path, MAX_PATH, absPath, &filePart ); 
   if( retVal )
   {
      pd_char_t  shortBuf[MAX_PATH];
      pd_char_t *lowerPath;

      if( GetShortPathName( absPath, shortBuf, MAX_PATH ) != 0 )
      {
         wss_free( absPath );
         absPath = wss_strdup( shortBuf );
      }
      lowerPath = wss_downcase( absPath );
      wss_free(absPath);
      return lowerPath;
   }
   else
   {
      wss_free(absPath);
      return NULL;
   }
}

/*  wsi_path_cwd */
/*! \brief Get the current working directory of a process
 *
 *  \param wrp Pointer to wrapper process structure for process
 *
 *  \return Pointer to string containing the current working directory of
 *          the specified process on success,
 *  \return NULL on failure
 */
pd_char_t EXPORT *
wsi_path_cwd( wss_proc_t *wrp )
{
   pd_char_t *currentDir = (pd_char_t *)wss_malloc( MAX_PATH *
                                                      sizeof(pd_char_t) );

   DWORD retVal = GetCurrentDirectory( MAX_PATH, currentDir );
   if( retVal )
   {
      pd_char_t *lowerDir = wsi_path_abspath( currentDir );
      wss_free(currentDir);
      return lowerDir;
   }
   else
   {
      wss_free(currentDir);
      return NULL;
   }
}

