/*! \file wsi_process.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.c,v 1.14 2001/02/23 19:02:30 stevek Exp $
 *
 * \brief   System-dependent process management functions
 * \author  Steve Kiernan
 * \version $Revision: 1.14 $
 * \date    $Date: 2001/02/23 19:02:30 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_process.h>


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

/*! Flag to indicate that the WSS is unloading */
int wsi_unloading = 0;


/***************************************************************************\
 * Public functions
\***************************************************************************/

/*  wsi_proc_new */
/*! \brief Create the system-dependent opaque data part of the wss_proc
 *         structure
 *
 *  \param wrp Pointer to wrapper process structure
 *
 *  \sa wss_proc_new
 */
void
wsi_proc_new( struct wss_proc *wrp )
{
   wrp->wrp_opaque = wss_malloc( sizeof(struct wsi_proc) );
   memset( (pd_byte_t *)wrp->wrp_opaque, 0, sizeof(struct wsi_proc) );
}

/*  wsi_proc_free */
/*! \brief Free the wss_proc structure system-dependent opaque data
 *
 *  \param wrp Pointer to wrapper process structure
 *
 *  \sa wss_proc_free
 */
void
wsi_proc_free( struct wss_proc *wrp )
{
   if( wrp->wrp_opaque )
      wss_free( (void *)wrp->wrp_opaque );
}

/*  wsi_proc_insyscall */
/*! \brief Returns the number of threads in syscalls for the specified
 *         process
 *
 *  \param wrp Pointer to wrapper process structure
 *
 *  \return Number of threads in syscalls
 */
int
wsi_proc_insyscall( struct wss_proc *wrp )
{
   return 0;
}

/*  wsi_proc_force_syscall_exit */
/*! \brief This is a placeholder for code that actually does make threads
 *         reissue their syscalls
 *
 *  This is obviously TBD.
 *
 *  \param wrp Pointer to wrapper process structure
 */
void
wsi_proc_force_syscall_exit( struct wss_proc *wrp )
{
}

