/*! \file wsi_time.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_time.c,v 1.13 2001/02/23 19:02:30 stevek Exp $
 *
 * \brief   Windows specific routines to get time
 * \author  Steve Kiernan
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:30 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>

#include <wsi_time.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_time_get_current */
/*! \brief Get the current date and time
 *
 *  \return Number of seconds since epoch (Jan 1, 1970) which have elapsed
 */
long EXPORT
wsi_time_get_current(long *usec)
{
   SYSTEMTIME     stEpoch = { 1970, 1, 1, 1, 0, 0, 0, 0 };
   SYSTEMTIME     stTime;
   ULARGE_INTEGER ftEpoch;
   ULARGE_INTEGER ftTime;
   ULARGE_INTEGER ftResult;
   ULARGE_INTEGER ftNano;
   long           seconds;
   long           microseconds;

   SystemTimeToFileTime( &stEpoch, (LPFILETIME)&ftEpoch );

   GetSystemTime( &stTime );
   SystemTimeToFileTime( &stTime, (LPFILETIME)&ftTime );

   /* Number of 100-nanosecond intervals from epoch */
   ftResult.QuadPart = ftTime.QuadPart - ftEpoch.QuadPart;

   /* Grab the modulo 100-nanosecond intervals from conversion to seconds */
   ftNano.QuadPart = ftResult.QuadPart % 10000000;
   microseconds = ftNano.LowPart / 10;

   /* Convert to seconds */
   ftResult.QuadPart /= 10000000;
   seconds = ftResult.LowPart;

   if (usec)
       *usec = microseconds; /* Store microsecond intervals */

   return seconds;
}

/*  wsi_time_format_alloc */
/*! \brief Format time based on epoch into a human-readable form
 *
 *  \param t Time in seconds since epoch
 *
 *  \return Pointer to string containing the number of days, hours,
 *          minutes, and seconds since epoch
 */
pd_char_t EXPORT *
wsi_time_format_alloc( long t )
{
   pd_char_t format[1024];

   wsi_sprintf( format, PD_MAKESTR("Day %05ld %02ld:%02ld"), t / 86400,
                ( t / 3600 ) % 24, ( t / 60 ) % 60 );

   return wss_strdup( format );
}

/*  wsi_getcurrenttimestring */
/*! \brief Get the current system time, and return it as a string
 *
 *  \return Pointer to string containing the current time
 */
pd_char_t EXPORT *
wsi_time_get_current_string( void )
{
  time_t ltime;
  struct tm *gmt;
  pd_char_t *timeStr, *retStr;
  size_t size; 

  time( &ltime );
  gmt = gmtime( &ltime );
  timeStr=_tasctime( gmt );
  retStr = wss_strdup( timeStr );
  size = wsi_strlen( retStr );

  // Nuke the newline that asctime() drops in
  *(retStr + (size - 1))= L'\0';

  return retStr;
}
