/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_file.h,v 1.16 2001/02/23 19:02:35 stevek Exp $
 *
 * Author: Linda Thomas, Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the
 *          file i/o module.
 */

#ifndef __WSI_FILE_H
#define __WSI_FILE_H 1

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif


/************************************************************************\
 * Public structures                                                    *
\************************************************************************/

typedef void *wss_file_t;
typedef int   wss_file_mode_t;


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSS_FILE_INVALID NULL

#define WSS_FILE_SEPARATOR '/'


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
void       WSSLINKAGE  wsi_file_close        __P(( wss_file_t file ));
void       WSSLINKAGE *wsi_file_get_map_addr __P(( wss_file_t file ));
size_t     WSSLINKAGE  wsi_file_get_size     __P(( wss_file_t file,
                                                   int *error ));
int        WSSLINKAGE  wsi_file_is_mapped    __P(( wss_file_t file ));
int        WSSLINKAGE  wsi_file_map          __P(( wss_file_t file,
                                                   size_t size, int mapmode,
                                                   pd_offset_t pos ));
wss_file_t WSSLINKAGE  wsi_file_open         __P(( const pd_char_t *path,
                                                   wss_file_mode_t mode ));
int        WSSLINKAGE  wsi_file_read         __P(( wss_file_t file,
                                                   pd_byte_t *buffer,
                                                   size_t len ));
void       WSSLINKAGE  wsi_file_ref          __P(( wss_file_t file ));
void       WSSLINKAGE  wsi_file_rele         __P(( wss_file_t file ));
int        WSSLINKAGE  wsi_file_seek         __P(( wss_file_t file,
                                                   pd_offset_t offset,
                                                   int whence ));
void       WSSLINKAGE  wsi_file_unmap        __P(( wss_file_t file ));
int        WSSLINKAGE  wsi_file_write        __P(( wss_file_t file,
                                                   pd_byte_t *buffer,
                                                   size_t len ));


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wsi_file_is_mapped(f)     ((wsi_file_get_map_addr(f)==NULL)?TRUE:FALSE)
#define wsi_file_round_to_page(s) round_page(s)
#endif


#endif /* !__WSI_FILE_H */

