/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.h,v 1.24 2001/06/21 21:50:34 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the
 *          system-depenedant parts of the process life-cycle code
 */

#ifndef __WSI_PROCESS_H
#define __WSI_PROCESS_H 1

#include <sys/param.h>
#include <sys/proc.h>
#include <sys/signal.h>
#include <sys/signalvar.h>
#include <sys/sysent.h>


/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

typedef struct wsi_proc wsi_proc_t;

/*! \brief This referred to as the "system-dependent wrappers-specific
 *         process information"
 */
struct wsi_proc
{
   struct sysentvec *sysent;   /*!< Original p_sysent vector */
};

struct wss_proc;


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_PROC_OPAQUE(wrp) ((struct wsi_proc *)(wrp)->wrp_opaque)

#define wsi_proc_get_curproc() (curproc)
#define wsi_proc_get_current_pid() (curproc->p_pid)

#define wsi_proc_pid(p)         ((p)->p_pid)
/*
 * local wss_proc flags
 */
#define WRPF_GET_CWD    WRPF_LOCAL


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

void wsi_proc_new                __P(( struct wss_proc * ));
void wsi_proc_fork               __P(( struct wss_proc *, struct wss_proc * ));
void wsi_proc_free               __P(( struct wss_proc * ));
int  wsi_proc_insyscall          __P(( struct wss_proc * ));
void wsi_proc_force_syscall_exit __P(( struct wss_proc * ));
int  wsi_proc_replace_sysent     __P(( struct wss_proc * ));
void wsi_proc_restore_sysent     __P(( struct wss_proc *, pd_bool_t ));


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wsi_proc_insyscall(wrp) (wrp)->wrp_nsyscall

#endif /* !__WSI_PROCESS_H */

