/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_syscalls.h,v 1.27 2001/06/22 04:09:20 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: System call numbers, commands, and prototypes
 */

#ifndef __WSI_SYSCALLS_H
#define __WSI_SYSCALLS_H   1

#include <sys/types.h>


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

/*
 * WSI syscalls
 */
#define SYS_wrctl 219
#define SYS_wrv   218

/*
 * wrctl syscall commands
 */
#define WRCTL_LOAD         0
#define WRCTL_UNLOAD       1
#define WRCTL_ENABLED      2
#define WRCTL_STAT_LINKERS 3
#define WRCTL_STAT_REFS    4
#define WRCTL_STAT_MEMORY  5

/*
 * Flags for WRCTL_STAT_REFS
 */
#define WRCTL_STAT_REFS_PROCESSES                      0x0001
#define WRCTL_STAT_REFS_MODULES                        0x0002
#define WRCTL_STAT_REFS_LINKER_FILES                   0x0004
#define WRCTL_STAT_REFS_CRITERIA                       0x0010
#define WRCTL_STAT_REFS_LIBRARIES                      0x0020
#define WRCTL_STAT_REFS_WRAPPERS                       0x0040
#define WRCTL_STAT_REFS_WRAPPER_INSTANCES              0x0080
#define WRCTL_STAT_REFS_PROCESSES_DUMP_CHAIN           0x0100
#define WRCTL_STAT_REFS_MODULES_DUMP_CHAIN             0x0200
#define WRCTL_STAT_REFS_LINKER_FILES_DUMP_CHAIN        0x0400
#define WRCTL_STAT_REFS_CRITERIA_DUMP_CHAIN            0x1000
#define WRCTL_STAT_REFS_LIBRARIES_DUMP_CHAIN           0x2000
#define WRCTL_STAT_REFS_WRAPPERS_DUMP_CHAIN            0x4000
#define WRCTL_STAT_REFS_WRAPPER_INSTANCES_DUMP_CHAIN   0x8000

#define WRCTL_STAT_REFS_ALL                            0xffff


/*
 * wrv syscall commands
 */
#define WRV_GET_TABLE_HANDLES                     0
#define WRV_GET_TABLE_COLUMN_DEF                  1
#define WRV_GET_TABLE_ROWS                        2
#define WRV_GET_TABLE_HANDLE_BY_NAME              3
#define WRV_GET_MODULE_BY_ID                      4
#define WRV_GET_WRAPPER_BY_ID                     5
#define WRV_GET_PROCS_BY_WRAPPER_ID               6
#define WRV_GET_ACT_CRIT_BY_ID                    7
#define WRV_GET_LIBRARY_BY_ID                     8
#define WRV_GET_DEACT_CRIT_BY_ID                  9
#define WRV_GET_MODULE_ID_BY_NAME                10
#define WRV_GET_PROC_BY_PID                      11
#define WRV_GET_ACTIVE_WRAPPERS                  12
#define WRV_SET_TABLE_ROW_FIELD                  20
#define WRV_ADD_NULL_TABLE_ROW                   21
#define WRV_DEL_TABLE_ROW                        22
#define WRV_ADD_TABLE_ROW                        23
#define WRV_ADMIN_DONE                           24
#define WRV_GET_VERSION                         200


/************************************************************************\
 * Public structures                                                    *
\************************************************************************/

struct wrctl_uap
{
   int   command;
   int   id;
   void *buffer;
};

struct wrv_uap
{
   int command;      /* IN, select request           */
   void *args;       /* IN,OUT, passed/returned data */
   ssize_t *bufsize; /* IN,OUT                       */
};


#if defined(KERNEL) || defined(_KERNEL)
/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

int         wrctl            __P(( proc_t *p, struct wrctl_uap *uap ));
int         wrv              __P(( proc_t *p, struct wrv_uap *uap ));

const char *wsi_syscall_name __P(( pd_syscall_id_t code ));


/************************************************************************\
 * Public globals                                                       *
\************************************************************************/

/*
 * System call names (may go away: TBD)
 */
extern const char *freebsd_syscallnames[];

#endif /* KERNEL || _KERNEL */

#endif /* __WSI_SYSCALLS_H */

