#!/usr/bin/perl
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: parsesyscall.pl,v 1.18 2001/04/19 17:49:06 stevek Exp $
#
# Author: Chris Vance
#
# Purpose: This file contains the script to generate the syscall 
# interception file (wsi_intercept.c).  It requires a standard FreeBSD 
# kernel build tree, specifically /usr/src/sys/kern/syscalls.master.
# Using this file as input, this script generates function prototypes, 
# The intermediate functions, a sysent table, and a new sysentvec.
#

open (SYSCALLS, "/usr/src/sys/kern/syscalls.master");
open (OUT, ">wsi_intercept.c");

while (<SYSCALLS>) {
	next if (/^\s*\;/);
	next if (/^\s*$/);
	next if (/^\s*\#include/);
	while (s/\\\s*$//) {
		$buff = <SYSCALLS>;
		$_ .= $buff;
	}
	s/MPSAFE\s+//;
	if (m|^\s*(\d+)\s+(\w+)\s+(\w+)\s+\{\s*(\w+)\s+(\w+)\(([^\)]*)\)\;\s*}|) {
#		print "$1 $4 $5 ($6)\n";
		$i = $1;
		if (($2 eq "CPT_NOA" || $2 eq "COMPAT" || $2 eq "STD") && 
			!($5 eq "nosys"))
		{
			$compat[$i] = "$2";
			$retval[$i] = $4;
			$syscall[$i] = $5;
			$args   = $6;
			if ($args eq "void") {
				$argnum[$i] = 0;
			}
			else {
				@args   = split(',', $args);
				$argnum[$i] = $#args + 1;
				if ($args =~ m|off_t|) {
					$argnum[$i]++;
				}
			}
			$count++;
		}
	}
}
$lastsyscall = $i;
print "Generating wsi_intercept.c:  $count interpreted out of $lastsyscall\n";
#print "Processed $count system calls (Ignored ioctl, _syscall, __syscall)\n";
#print "Last syscall is number $lastsyscall\n";

print OUT <<EOF;
/*
 * wsi_intercept.c
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */
#include <wr_conf.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/mount.h>
#include <sys/sysent.h>
#include <sys/sysproto.h>
#include <sys/syscall.h>
#include <sys/signalvar.h>
#include <machine/md_var.h>

#include <wsi_syscalls.h>

#ifdef COMPAT_43
#define compat(n, name) n, (sy_call_t *)name
#else
#define compat(n, name) 0, (sy_call_t *)nosys
#endif

int wsi_dispatch(int, pd_syscall_id_t, struct proc *p, void *uap, int *retval);


EOF

for ($i=0; $i <= $lastsyscall; $i++) {
	if ($syscall[$i]) {
		$syscall[$i] = 'old_' . $syscall[$i]
			if( $compat[$i] ne 'STD' );
		while (defined($declared{$syscall[$i]})) {
			$syscall[$i] .= '_2';
		}
		$declared{$syscall[$i]}++;
		print OUT "$retval[$i] wr_$syscall[$i](struct proc *p, void *uap, int *retval);\n";
	}
}

print OUT "\n\n";

for ($i=0; $i <= $lastsyscall; $i++) {
	if ($syscall[$i]) {
		print OUT "$retval[$i] wr_$syscall[$i](struct proc *p, void *uap, int *retval)\n";
		if ($retval[$i] eq "void") {
			print OUT "{ wsi_dispatch(0,$i, p, uap, retval); }\n\n";
		}
		else {
			if ($retval[$i] eq "int") {
				print OUT "{ return(wsi_dispatch(0,$i, p, uap, retval)); }\n\n";
			}
			else {
				print OUT "{ return(($retval[$i])wsi_dispatch(0,$i, p, uap, retval)); }\n\n";
			}
		}
	}

}

print OUT "struct sysent wr_sysent[] = {\n";
for ($i=0; $i <= $lastsyscall; $i++) {
	if ($syscall[$i]) {
		if ($compat[$i] eq "STD") {
			if ($syscall[$i] eq "mmap")
			{
				print OUT "\t{ $argnum[$i], (sy_call_t *)wr_$syscall[$i] },\n";
			}
			else {
				print OUT "\t{ $argnum[$i], (sy_call_t *)wr_$syscall[$i] },\n";
			}
		}
		else {
			if ($syscall[$i] eq "mmap")
			{
				print OUT "\t{ compat($argnum[$i],wr_$syscall[$i]) },\n";
			}
			else {
				print OUT "\t{ compat($argnum[$i],wr_$syscall[$i]) },\n";
			}
		}
	}
	else {
		print OUT "\t{ 0, (sy_call_t *)nosys },\n";
	}
}
print OUT "};\n";

close(OUT);

open (OUT, ">wsi_syscallnames.c");

print OUT <<EOF;
/*
 * wsi_syscallnames.c
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */
#include <wr_conf.h>

const char *freebsd_syscallnames[] =
{
EOF

for( $i=0; $i <= $lastsyscall; $i++ ) {
	if ($syscall[$i]) {
		print OUT "\t\"$syscall[$i]\",\n";
	}
	else {
		print OUT "\t\"undef\",\n";
	}
}

print OUT <<EOF;
};

EOF

close(OUT);

exit;
