/*! \file wsi_os.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_os.c,v 1.13 2001/02/23 19:02:34 stevek Exp $
 *
 * \brief   Operating system information functions
 * \author  Steve Kiernan 
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:34 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <sys/kernel.h>

#include <wsi_os.h>


/***************************************************************************\
 * External globals                                                        *
\***************************************************************************/

extern pd_char_t *ostype;
extern int         osreldate;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_os_get_name */
/*! \brief Get the name of the OS
 * 
 *  \return Pointer to string containing the name of the OS
 */
pd_char_t *
wsi_os_get_name()
{
   return ostype;
}


/*  wsi_os_get_version */
/*! \brief Get the version of the OS
 *
 *  \return Major version of the OS
 */
int
wsi_os_get_version()
{
   return osreldate;
}


/*  wsi_os_curthreadid */
/*! \brief Return the thread id of the current thread
 *
 *  \return Identifier of the current thread
 */
int
wsi_os_curthreadid()
{
    return 0;
}

/* wsi_os_get_hostname */
/*! \brief Return the name of the host
 *
 * \return String with the host name
 */
pd_char_t *
wsi_os_get_hostname()
{
   return( hostname );
}

