/*! \file wsi_time.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_time.c,v 1.12 2001/02/23 19:02:34 stevek Exp $
 *
 * \brief   System-specific routines to handle time
 * \author  Linda Thomas
 * \version $Revision: 1.12 $
 * \date    $Date: 2001/02/23 19:02:34 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>

#include <sys/time.h>

#include <wsi_time.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_time_get_current */
/*! \brief Return current time
 *
 *  \return Number of seconds since epoch (January 1, 1970)
 */
long
wsi_time_get_current(long *usec)
{
   struct timeval atv;
   microtime( &atv );

   if (usec)
       *usec = atv.tv_usec;
   return( atv.tv_sec );
}


/*  wsi_time_format_alloc */
/*! \brief Convert epoch time to string
 *
 *  \param t Number of seconds since epoch
 *
 *  \return Pointer to string containing the number of days and the time
 */
pd_char_t *
wsi_time_format_alloc( long t )
{
   pd_char_t format[1024];

   wsi_sprintf( format, PD_MAKESTR("Day %05ld %02ld:%02ld"), t / 86400,
                ( t / 3600 ) % 24, ( t / 60 ) % 60 );
 
   return wss_strdup( format );
}

