/*
 * Copyright (C) 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_i386_dispatch.S,v 1.6 2001/06/09 02:51:52 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains the implementation of the Linux system
 *          call dispatch routine.
 *
 * Note: This is written in assembly language in order to be less intrusive
 *       with the system call process.  We leave the stack in the same
 *       layout as normally expected for the system call.  All changes for
 *       parameters to the system call are done directly to the data on
 *       stack.
 */

#define __ASSEMBLY__
#include <linux/sys.h>
#include <linux/linkage.h>
#include <asm/unistd.h>


/*
 * Macro to get the current process' task struct
 */
#define CURRENT(reg) \
	movl $-8182,reg; \
	andl %esp,reg;


#include "wsi_offsets.inc"


/*
 * These are offsets into the wrapper process structure
 */
#ifdef __SMP__
#error Need to find the offsets for wrp_wi_list, wrp_nsyscall, and wrp_opaque
#else
wrp_nsyscall	= 16
wrp_wi_list	= 28
wrp_pid		= 40
wrp_opaque	= 60
#endif


/*
 * These are offsets into the wsi_proc structure
 */
savedip		= 0
code		= 4
retval		= 8
err		= 12
lastwi		= 16
errnum		= 20
wiv		= 24
skip		= 28
wsiproc		= 32


/*
 * Local read-only data
 */
.section .rodata
	ALIGN
.LC0:
	.string "No wss_proc structure!\n"



/*
 * System call dispatch routine
 *
 * On entry: EAX contains the system call number
 *
 *  On exit: EAX contains the return value of the call
 */
ENTRY(wsi_dispatch)
	/*
	 * Play nice with the stack
	 */
	pushl %ebp
	movl %esp,%ebp
	pushl %edi
	pushl %ebx
	pushl %ecx

	movl %eax,%edi                  /* EDI = syscall id (in EAX on entry) */

	/*
	 * Get the PID of the current process
	 */
	CURRENT(%eax)
	movl pid(%eax),%ebx             /* EBX = current->pid */

	/* If current->pid > 4, jump to .LPFIND */
	cmpl $4,%ebx
	jg .LPFIND

.LORIG:
	/*
	 * Call through saved system call
	 */
	popl %ecx
	popl %ebx
	movl %edi,%eax
	popl %edi
	movl %ebp,%esp
	popl %ebp
	jmp *saved_sysent(,%eax,4)
	ALIGN

.LPFIND:
	/*
	 * Lookup wrapper process structure
	 */
	pushl %ebx
	call wss_pfind
	addl $4,%esp

	movl %eax,%ebx                  /* EBX = wrp */

	/*
	 * If wrp is NULL, jump to .LORIG
	 */
	testl %ebx,%ebx
	je .LORIG
	ALIGN

	/*
         * Check for wrappers: if we have any, jump to .LHAVEWRAP
         */
	movl wrp_wi_list(%ebx),%eax
	testl %eax,%eax
	jne .LHAVEWRAP

	/*
	 * Release the wrapper process structure
	 */
	pushl %ebx
	call wsi_proc_rele
	addl $4,%esp

	/*
	 * Jump to .LORIG
	 */
	jmp .LORIG

.LHAVEWRAP:
	/*
	 * Get the wsi_proc structure
	 */
	movl wrp_opaque(%ebx),%eax

	/*
	 * Store address of err in wrd.errnum
	 */
	leal err(%eax),%ecx
	movl %ecx,errnum(%eax)

	/*
	 * Store syscall id in code
	 */
	movl %edi,code(%eax)

	/*
	 * Call the pre-dispatcher
	 *
	 * %EAX = wsi_dispatch_pre(code, args, wrp)
	 */
	pushl %ebx
	leal 8(%ebp),%eax
	pushl %eax
	pushl %edi
	call wsi_dispatch_pre
	addl $12,%esp

	/*
	 * If error (%EAX != 0), jump to .LEXIT
	 */
	testl %eax,%eax
	jne .LEXIT

	/*
	 * wrp->wrp_nsyscall++
	 */
	incl wrp_nsyscall(%ebx)

	/*
	 * Save the return IP which is on the stack in the wsi_proc struct
	 */
	movl wrp_opaque(%ebx),%eax
	movl 4(%ebp),%ecx
	movl %ecx,savedip(%eax)

	/*
	 * Release the wrapper process structure
	 */
	pushl %ebx
	call wsi_proc_rele
	addl $4,%esp

	movl %edi,%eax                  /* EAX = syscall id (restored) */

	/*
	 * Clean up stack
	 */
	popl %ecx
	popl %ebx
	popl %edi
	movl %ebp,%esp
	popl %ebp

	/*
	 * Pop the return IP off the stack
	 */
	addl $4,%esp

	/*
	 * Call through saved system call
	 */
	call *saved_sysent(,%eax,4)
	ALIGN

	/*
	 * Reserve space on the stack for the return IP
	 */
	subl $4,%esp

	/*
	 * Play nice with the stack
	 */
	pushl %ebp
	movl %esp,%ebp
	pushl %edi
	pushl %ebx
	pushl %ecx

	movl %eax,%edi                  /* EDI = retval */

	/*
	 * Get the current PID
	 */
	CURRENT(%ebx)
	movl pid(%ebx),%ebx             /* EBX = current->pid */

	/*
	 * Lookup wrapper process structure
	 */
	pushl %ebx
	call wss_pfind
	movl %eax,%ebx                  /* EBX = wrp */
	addl $4,%esp

	/*
	 * If wss_pfind returned NULL, panic
	 */
	testl %ebx,%ebx
	jne .LOUTSYS

	leal .LC0,%ebx
	pushl %ebx
	call panic

.LOUTSYS:
	/*
	 * Get the wsi_proc struct
	 */
	movl wrp_opaque(%ebx),%ecx      /* ECX = wsiproc */

	/*
	 * Save the return value
	 */
	movl %edi,retval(%ecx)

	/*
	 * Copy the return IP back on the stack
	 */
	movl savedip(%ecx),%eax         /* wsiproc.savedip */
	movl %eax,4(%ebp)

	movl %edi,%eax                  /* EAX = retval */
	movl code(%ecx),%edi            /* EDI = syscall id */

	/*
	 * If EAX > 0, jump to .LZEROERR
	 */
	testl %eax,%eax
	jg .LZEROERR
	negl %eax                       /* EAX = -EAX */
	jmp .LSAVERR
	ALIGN

.LZEROERR:
	xorl %eax,%eax                  /* EAX = 0 */

.LSAVERR:
	/*
	 * Save err in the wsi_proc struct
	 */
	movl %eax,err(%ecx)             /* wsiproc.err = EAX */

	/*
	 * wrp->wrp_nsyscall--
	 */
	decl wrp_nsyscall(%ebx)

	/*
	 * Call the post-dispatcher
	 *
	 * %EAX = wsi_dispatch_post(code, args, wrp)
	 */
	pushl %ebx
	leal 8(%ebp),%eax
	pushl %eax
	pushl %edi
	call wsi_dispatch_post
	addl $12,%esp
.LEXIT:
	/*
	 * Clean up in arg replacement
	 *
	 * wss_arg_donesyscall( wrp, pid, 0, code, args )
	 */
	leal 8(%ebp),%eax
	pushl %eax
	pushl %edi
	xorl %eax,%eax
	pushl %eax
	CURRENT(%eax)
	movl pid(%eax),%eax
	pushl %eax
	pushl %ebx
	call wss_arg_donesyscall
	addl $20,%esp

	/*
	 * Get the wsi_proc struct
	 */
	movl wrp_opaque(%ebx),%eax

	/*
	 * Return the return value stored in the wsi_proc structure
	 */
	movl retval(%eax),%ecx          /* wsiproc.retval */

	/*
	 * Release the wrapper process structure
	 */
	pushl %ebx
	call wsi_proc_rele
	addl $4,%esp

	movl %ecx,%eax                  /* EAX = retval */

	popl %ecx
	popl %ebx
	popl %edi
	movl %ebp,%esp
	popl %ebp
	ret

