/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_fileutil.h,v 1.7 2001/02/23 19:02:38 stevek Exp $
 */

#ifndef __WSI_FILEUTIL_H
#define __WSI_FILEUTIL_H 1

#include <linux/time.h>
#include <linux/socket.h>
#include <linux/in.h>

/*
 * file type values.
 * Don't map to solaris at all, so we really really really fake it.
 */
typedef enum {
    WR_DTYPE_VNODE = 1,                 /* File                         */
    WR_DTYPE_SOCKET,                    /* communications endpoint      */
    WR_DTYPE_PIPE,                      /* unamed pipe                  */
    WR_DTYPE_FIFO,                      /* named pipe                   */
    WR_DTYPE_OTHER                      /* ???                          */
} wsi_fileutil_filetype_t;

/* structure for holding information of a file/inode) */
struct wr_stat {
        dev_t    st_dev;               /* inode's device */
        ino_t    st_ino;               /* inode's number */
        mode_t   st_mode;              /* inode protection mode */
        nlink_t  st_nlink;             /* number of hard links */
        uid_t    st_uid;               /* user ID of the file's owner */
        gid_t    st_gid;               /* group ID of the file's group */
        dev_t    st_rdev;              /* device type */
        struct  timespec st_atimespec;  /* time of last access */
        struct  timespec st_mtimespec;  /* time of last data modification */
        struct  timespec st_ctimespec;  /* time of last file status change */
        off_t    st_size;               /* file size, in bytes */
        u_long   st_gen;                /* file generation number */
};

/* a list of macro associated with wr_stat */
#define WorldReadable(s)        ((s).st_mode & S_IROTH)
#define WorldWritable(s)        ((s).st_mode & S_IWOTH)
#define WorldExecutable(s)      ((s).st_mode & S_IXOTH)

#define GroupReadable(s)        ((s).st_mode & S_IRGRP)
#define GroupWritable(s)        ((s).st_mode & S_IWGRP)
#define GroupExecutable(s)      ((s).st_mode & S_IXGRP)

#define OwnerReadable(s)        ((s).st_mode & S_IRUSR)
#define OwnerWritable(s)        ((s).st_mode & S_IWUSR)
#define OwnerExecutable(s)      ((s).st_mode & S_IXUSR)

#define IsPipe(s)               (((s).st_mode & 0170000) == S_IFIFO)
#define IsDir(s)                (((s).st_mode & 0170000) == S_IFDIR)
#define IsSymlink(s)            (((s).st_mode & 0170000) == S_IFLNK)
#define IsSocket(s)             (((s).st_mode & 0170000) == S_IFSOCK)
#define IsCharDev(s)            (((s).st_mode & 0170000) == S_IFCHR)
#define IsBlkDev(s)             (((s).st_mode & 0170000) == S_IFBLK)
#define IsRegFile(s)            (((s).st_mode & 0170000) == S_IFREG)
#define IsWhiteOut(s)           (((s).st_mode & 0170000) == S_IFWHT)

#define Owner(s)                (s).st_uid

int  wsi_fileutil_stat    __P((const char *path, struct wr_stat *sb));
int  wsi_fileutil_lstat   __P((const char *path, struct wr_stat *sb));
int  wsi_fileutil_fstat   __P((unsigned int fd,  struct wr_stat *sb));
int  wsi_fileutil_fdtype  __P((unsigned int fd));
int  wsi_fileutil_fdfflag __P((unsigned int fd));
 
int          wsi_netutil_get_addr    __P((const struct sockaddr *, int,
                                            char *buffer, int buffer_length));
const char  *wsi_netutil_get_family  __P((const struct sockaddr *, int));
int          wsi_netutil_getsockname __P((int fd, struct sockaddr *s,
                                            int *len));
int          wsi_netutil_getpeername __P((int fd, struct sockaddr *s,
                                            int *len));
unsigned short     wsi_netutil_ntohs(short x);

#endif /* __WSI_FILEUTIL_H */
