/*
 * Copyright (C) 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_hook.h,v 1.2 2001/02/10 22:08:31 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures used by the
 *          Linux kernel hot-patch hook routines
 */

#ifndef __WSI_HOOK_H
#define __WSI_HOOK_H 1

#include <wr_conf.h>


/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

typedef struct call_replacement call_replacement_t;

struct call_replacement
{
   caddr_t  replace;
   caddr_t  orig;
   caddr_t *addr;
};


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

/*
 * Macro to define a call replacement
 */
#define DEFINE_REPLACEMENT(name) \
   extern int wsi_custom_ ## name( struct pt_regs ); \
   call_replacement_t _replace_sys_ ## name = \
   { (void *) wsi_custom_ ## name, NULL, NULL }

#define DECLARE_REPLACEMENT(name) \
   extern call_replacement_t _replace_sys_ ## name

#define REPLACEMENT(name) (_replace_sys_ ## name ## .replace)
#define STORAGEADDR(name) (_replace_sys_ ## name ## .addr)
#define ORIGINAL(name) (_replace_sys_ ## name ## .orig)

#endif /* !__WSI_HOOK_H */

