/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.h,v 1.10 2001/02/23 19:02:38 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the
 *          system-depenedant parts of the process life-cycle code
 */

#ifndef __WSI_PROCESS_H
#define __WSI_PROCESS_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#if defined(PD_IN_KERNEL)
#include <linux/sched.h>
#endif

#if 0
#include <sys/proc.h>
#include <sys/rwlock.h>
#endif

#ifndef __WSS_MUTEX_H
#include <wss_mutex.h>
#endif

#ifndef __WSS_CV_H
#include <wss_cv.h>
#endif

#ifndef __WSS_WRAPPER_H
#include <wss_wrapper.h>
#endif


/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

/*
 * Note: Must keep this struct in sync with wsi_${CPU}_dispatch.S
 */
struct wsi_proc
{
   void                                 *savedip;
   pd_syscall_id_t                       code;
   int                                   retval;
   int                                   err;
   volatile struct wss_wrapper_instance *lastwi;
   struct wss_wrapper_data               wrd;
};

struct wss_proc;

/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_PROC_OPAQUE(wrp) ((struct wsi_proc *)(wrp)->wrp_opaque)
#define wsi_proc_get_curproc()       current
#define wsi_proc_get_current_pid()   current->pid

#define wsi_proc_insyscall(wrp) (wrp)->wrp_nsyscall

#define wsi_proc_pid(p)                 ((p)->pid)
/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
void wsi_proc_free               __P(( struct wss_proc * ));
void wsi_proc_new                __P(( struct wss_proc * ));
void wsi_proc_force_syscall_exit __P(( struct wss_proc * ));
void wsi_proc_fork               __P(( struct wss_proc *, struct wss_proc * ));
#endif /* WR_SUPPRESS_FUNCTION_DECL */

#endif /* !__WSI_PROCESS_H */

