/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_rw.h,v 1.7 2001/04/20 22:48:31 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent reader/writer locks
 *
 */

#ifndef __WSI_RW_H
#define __WSI_RW_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#if PD_LINUX < 0x02004
#ifndef __ASM_SPINLOCK_H
#include <asm/spinlock.h>
#endif
#else
#ifndef __LINUX_SPINLOCK_H
#include <linux/spinlock.h>
#endif
#endif

enum _wsi_rw_type {
    RW_READER,
    RW_WRITER
};

typedef enum _wsi_rw_type wsi_rw_type_t;

struct _wsi_rw {
    rwlock_t            lock;
    wsi_rw_type_t       type;
};

typedef struct _wsi_rw wsi_rw_t;

/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define WSI_RW_PROTECT(var) wsi_rw_t var ## _rwlock
#define WSI_RW_PROTECT_EXPORTED(var) wsi_rw_t var ## _rwlock
#define WSI_RW_EXPORT(var) extern wsi_rw_t var ## rwlock

#ifndef WR_SUPPRESS_FUNCTION_DECL
#define WSI_RW_INIT(var) do { (var ## _rwlock).type = RW_READER; \
                              (var ## _rwlock).lock = RW_LOCK_UNLOCKED;  \
                         } while (0)
#define WSI_RW_DESTROY(var) do { (var ## _rwlock).type = -1; } while (0)
#define WSI_RW_ENTER(var,lock_type) \
    do { \
        if (lock_type == RW_WRITER) \
            write_lock((var ## _rwlock).lock); \
        else \
            read_lock((var ## _rwlock).lock); \
        (var ## _rwlock).type = lock_type; \
    } while (0)
#define WSI_RW_ENTER_DEBUG(var,type) \
   do { printf( "rw_enter(" ## #var ## "_rwlock=0x%x, " \
           "/thread=0x%x @ %s:%d\n", (uint32_t)&(var ## _rwlock), \
           wsi_proc_pid(current), __FILE__, __LINE__ ); \
        WSI_RW_ENTER(var,type); \
   } while (0)

#define WSI_RW_EXIT(var) \
    do { \
        if ((var ## _rwlock).type == RW_WRITER) { \
            (var ## _rwlock).type = RW_READER; \
            write_unlock((var ## _rwlock).lock); \
        } else { \
            read_unlock((var ## _rwlock).lock); \
        } \
    } while (0)

#define WSI_RW_EXIT_DEBUG(var) \
    do { \
        printf( "rw_exit(" ## #var ## "_rwlock=0x%x)/thread=0x%x @ %s:%d\n", \
                (uint32_t)&(var ## _rwlock), wsi_proc_pid(current), __FILE__, \
                __LINE__ ); \
        WSI_RW_EXIT(var); \
    } while (0)

#define WSI_RW_DOWNGRADE(var) \
    do { \
        WSI_RW_EXIT(var); \
        WSI_RW_ENTER(var,RW_READER); \
    } while (0)
#define WSI_RW_DOWNGRADE_DEBUG(var) \
    do { \
        printf( "rw_downgrade(" ## #var ## "_rwlock=0x%x)" \
                "/thread=0x%x @ %s:%d\n", (uint32_t)&(var ## _rwlock), \
                wsi_proc_pid(current), __FILE__, __LINE__ ); \
        WSI_RW_EXIT(var); \
        WSI_RW_ENTER(var,RW_READER); \
    } while (0)
#endif /* WR_SUPPRESS_FUNCTION_DECL */

#endif /* !__WSS_RW_H */

