/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_stdlib.h,v 1.12 2001/02/23 19:02:38 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains standard library macros and prototypes
 */

#ifndef __WSI_STDLIB_H
#define __WSI_STDLIB_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_ERRNO_H
#include "wsi_errno.h"
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#include <stdarg.h>

#ifndef PD_IN_KERNEL
#include <string.h>
#include <strings.h>
#else
#include <linux/string.h>
#include <asm/uaccess.h>
#endif


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
int        WSSLINKAGE  wsi_printf  __P(( const pd_char_t *format, ... ));
size_t     WSSLINKAGE  wsi_strlen  __P(( const pd_char_t *s ));
int        WSSLINKAGE  wsi_strcmp  __P(( const pd_char_t *s1,
                                         const pd_char_t *s2 ));
int        WSSLINKAGE  wsi_strncmp __P(( const pd_char_t *s1,
                                         const pd_char_t *s2, size_t len ));
pd_char_t WSSLINKAGE *wsi_strcpy  __P(( pd_char_t *dst,
                                         const pd_char_t *src ));
pd_char_t WSSLINKAGE *wsi_strncpy __P(( pd_char_t *dst,
                                         const pd_char_t *src, size_t len ));
pd_char_t WSSLINKAGE *wsi_strcat  __P(( pd_char_t *dst,
                                         const pd_char_t *src ));
int        WSSLINKAGE  wsi_vsprintf __P(( pd_char_t *buf,
                                          const pd_char_t *format, va_list ));
pd_char_t WSSLINKAGE  wsi_tolower __P(( pd_char_t c ));
pd_char_t WSSLINKAGE  wsi_toupper __P(( pd_char_t c ));

#endif

/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#if defined(PD_IN_KERNEL)
#define wsi_printf      printk
#else
#define wsi_printf      printf
#define panic(string, args...)    (printf(string, ## args), abort())
#endif
#define wsi_sprintf     sprintf
#define wsi_vsprintf    vsprintf

#if !defined(SUPPRESS_MACROS) && defined(PD_IN_KERNEL)
#define wsi_strncpy     strncpy
#define wsi_strncmp     strncmp
#define wsi_strcpy      strcpy
#define wsi_strlen      strlen
#define wsi_strcat      strcat
#define wsi_strcmp      strcmp
#endif


#if defined(SUPPRESS_MACROS) && !defined(USER_MODE_TEST)
void bcopy(const void *src, void *dest, int n);
void bzero(void *b, int n);
#endif

#ifndef SUPPRESS_MACROS
#ifdef PD_IN_KERNEL
#define copyin(user,driver,n) (copy_from_user(driver,user,n) ? EFAULT : 0)
#define copyout(driver,user,n) (copy_to_user(user,driver,n) ? EFAULT : 0)
#else /* PD_IN_KERNEL */
#define copyin(user,driver,n) bcopy(driver,user,n)
#define copyout(driver,user,n) bcopy(user,driver,n)
#endif
#else /* SUPPRESS_MACROS */
int copyin(const void *user, void *driver, int n);
int copyout(const void *driver, void *user, int n);
#endif
int copyinstr(const char *user, char *driver, int len, size_t *done);

#endif /* !__WSI_STDLIB_H */

