/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: loader.c,v 1.16 2001/02/23 19:02:37 stevek Exp $
 *
 * Author: Douglas Kilpatrick
 *
 * Purpose: Load/unload the WSS from the kernel.
 * 
 */

#include <wr_conf.h>
#include <wss_init.h>
#include <wsi_syscalls.h>
#include <wsi_utilities.h>

#include <linux/kmod.h>

/* Global */
int wsi_unloading = 0;

void wsi_sysevents_init(void);

int wsi_load(void)
{
   int error;

   /* Make sure it doesn't unload. */
   /* Really, we should inc this on the way into a syscall, and 
    * for each process we wrap.
    *
    * Of course, the net effect would be the same, cause someone is
    * always gonna block in select... like the shell that loaded
    * wrappers...  So we still have to come up with a way to signal
    * the wss to unload, and a way to do it safely.  Ug.
    */

#if 0
   /* No need to do this, since we won't load if the module isn't
    * already loaded.
    */
   if ( ( error = request_module("eventfs")) )
       return error;
#endif

   if( ( error = wss_init( WSS_INIT_DB | WSS_INIT_CONF ) ) ) {
       wsi_printf(PD_MAKESTR("Error in wss_init: %d\n"),error);
       return error;
   }

   MOD_INC_USE_COUNT;

   /* Startup library cruft */
   wsi_util_init();

   /* Hook the fork/exec/exit entries.  We don't do the syscalls,
    * we do other patches.
    */
   wsi_hook_init();

   /* Hook all the syscalls. */
   wsi_intercept_init();

   /* Add new syscalls. */
   wsi_syscall_init();

   /* Hook various system events. */
   wsi_sysevents_init();

   /*
    * If we make it to here, print message on console.
    */
   wsi_printf( "Successfully loaded WSS module\n" );

   return 0;
}

void wsi_unload(void)
{
#ifdef FIXED_UNLOAD
    wsi_unloading = 1;
#endif
    wsi_syscall_shutdown();
    wsi_intercept_shutdown();

    /* Unhook our patches. */
    wsi_hook_shutdown();

    /* Library cruft.  Memory balancing mostly */
    wsi_util_shutdown();
}

