#!/usr/bin/perl -w
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: parsesyscall.pl,v 1.28 2001/04/23 22:33:34 stevek Exp $
#
# Author: Douglas Kilpatrick
#
# Purpose: This file contains the script to generate the
# syscall => code mapping file (wsi_syscallnames.c).  It requires that the
# linux build tree be in /usr/src/linux, and assumes x86, 2.2.X, and stable
# system call numbers.
#

use strict;

my $vers = `uname -r`;
my( $linvers, $linrel ) = $vers =~ /^([0-9]+\.[0-9]+)\.?([0-9]*).*$/;

print <<EOF;
Parse Linux System Calls script

Linux Kernel: $linvers
     Release: $linrel

EOF

open (SYSCALLS, "/usr/src/linux/arch/i386/kernel/entry.S") ||
open (SYSCALLS, "/usr/src/linux-$linvers/arch/i386/kernel/entry.S") ||
open (SYSCALLS, "/usr/src/linux-$linvers-$linrel/arch/i386/kernel/entry.S") ||
   die "entry.S: $!";

# Jump to the syscall table.
while (<SYSCALLS>) {
    last if /^ENTRY\(sys_call_table/;
}

my $i = 0;
my $count = 0;

# We assume the following syscall places are empty.
my %socketcall_map = (
    "socket"            => 239,
    "bind"              => 240,
    "connect"           => 241,
    "listen"            => 242,
    "accept"            => 243,
    "getsockname"       => 244,
    "getpeername"       => 245,
    "socketpair"        => 246,
    "send"              => 247,
    "sendto"            => 248,
    "recv"              => 249,
    "recvfrom"          => 250,
    "shutdown"          => 251,
    "setsockopt"        => 252,
    "getsockopt"        => 253,
    "sendmsg"           => 254,
    "recvmsg"           => 255
);

my @syscall;

while (<SYSCALLS>) {
    while (s/\\\s*$//) {
        my $buff = <SYSCALLS>;
        $_ .= $buff;
    }
    if (m|^\s*\.long\s*SYMBOL_NAME\s*\(\s*old_(\w+)\s*\)|) {
        $syscall[$i] = "old_" . $1;
        $count++;
        $i++;
    }

    if (m|^\s*\.long\s*SYMBOL_NAME\s*\(\s*sys_(\w+)\s*\)|) {
        $syscall[$i] = $1;
        $count++;
        $i++;
    }
# On x86 Linux, all system calls return an int, if they know it or
# not.
}

my $lastsyscall = $i;

# Add in the socketcall_map fake syscalls to the official syscall list now.
foreach (keys %socketcall_map) {
    $syscall[$socketcall_map{$_}] = $_;
}

print "Generating wsi_intercept.h\n";

open (OUT, ">wsi_intercept.h") || die "wsi_intercept.h";

print OUT <<EOF;
/*
 * wsi_intercept.h
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */

#ifndef __WSI_INTERCEPT_H
#define __WSI_INTERCEPT_H 1

EOF

foreach (keys %socketcall_map) {
   print OUT <<EOF;
#ifdef __NR_$_
#error __NR_$_ is already defined, this is bad
#endif
EOF
}

print OUT "\n";

foreach (keys %socketcall_map) {
   print OUT <<EOF;
#define __NR_$_ $socketcall_map{$_}
EOF
}

print OUT <<EOF;

#endif /* __WSI_INTERCEPT_H */
EOF

close OUT;


print "Generating wsi_syscallnames.c\n";

open (OUT, ">wsi_syscallnames.c") || die "wsi_syscallnames.c";

print OUT <<EOF;
/*
 * wsi_syscallnames.c
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */
#include <wr_conf.h>
#include <linux/sys.h>

EOF

print OUT <<EOF;

const char *linux_syscallnames[NR_syscalls+1] =
{
EOF

for( $i=0; $i <= 255; $i++ ) {
	if ($syscall[$i]) {
		print OUT "    \"$syscall[$i]\",\n";
	}
	else {
		print OUT "    \"undef\",\n";
	}
}

print OUT <<EOF;
   NULL
};

EOF

close(OUT);

exit;
