#!/usr/bin/perl
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
#############################################################################
#
# binpatch.pl
#
#############################################################################

#
# Open and truncate the binpatch.h file.
#

open( OUTPUT, ">binpatch.h" ) || die "can't open binpatch.h";

#
# Append stub_head to binpatch.h file.
#

open( STUBFILE, "stub_head" ) || die "can't open stub_head";
print OUTPUT <STUBFILE>;
close STUBFILE;


#
# Make SYSMAPFILE a file descriptor to the most appropriate system
# map file.  Some versions of Linux have only one system map file
# called System.map-<kernel version number>.  Others have one system
# map file for an SMP kernel, another for a non-SMP kernel, and a
# symbolic link called System.map pointing to one or the other.  We'll
# set SYSMAPFILE to the shortest filename beginning with "System.map".
#

opendir( BOOTDIR, "/boot" ) || die "can't open /boot";
@allfiles = readdir BOOTDIR;
closedir( BOOTDIR );
@mapfiles = grep( /^System.map/, @allfiles );
@sortedmapfiles = sort @mapfiles;
$themapfile = "/boot/" . $sortedmapfiles[ 0 ];
print "Using kernel symbol offsets found in " . $themapfile . "\n";


#
# Open SYSMAPFILE, convert fields from this format:
#
# 0020c0ac B vt_cons
# 0020c1b0 B spawnpid
# 0020c1b4 B spawnsig
#
# to this format:
#
# #define BP_START_vt_cons  0x0020c0ac
# #define BP_END_vt_cons    0x0020c1af
# #define BP_START_spawnpid 0x0020c1b0
# #define BP_END_spawnpid   0x0020c1b3
# #define BP_START_spawnsig 0x0020c1b4
# #define BP_END_spawnsig   0x0020c1bf
#
# and write these lines to OUTPUT.  Note that each BP_END addresses is
# determined by subtracting 1 from the following BP_START address.  The
# last BP_START and BP_END addresses in the list will be identical.
#

open( SYSMAPFILE, $themapfile ) || die "can't open " . $themapfile . "\n";
@sysmaplines = <SYSMAPFILE>;
close( SYSMAPFILE );

# we're going to process the list in reverse order, to make the end address
# calculations easier.
@sysmaplines = reverse( @sysmaplines );

# calculate the first end address to prime the loop.
$sysmapline = $sysmaplines[ 0 ];
chop $sysmapline;
@sysmapcolumns = split( / +/, $sysmapline );
$endaddress = hex( $sysmapcolumns[ 0 ] ) - 1;
$endaddress = sprintf( "%08x", $endaddress );

foreach $sysmapline (@sysmaplines) {

    chop $sysmapline;
    @sysmapcolumns = split( / +/, $sysmapline );

    # Nothing is as easy as it seems at first.  Some function names in
    # the sysetm map file end in periods followed by numbers, which,
    # if carried into our output, confuses the preprocessor.  For now,
    # we'll skip these names.
    if( grep( /\./, $sysmapcolumns[ 2 ] ) ) {
	next;
    }

    # Not all of the function names in the system map file are unique
    # with respect to each other.  For now, we'll throw out a name if
    # we've seen it before, and move on to the next line.

    if( exists( $seenalready{ $sysmapcolumns[ 2 ] } ) ) {
	next;
    }
    $seenalready{ $sysmapcolumns[ 2 ] } = 1;

    $startline = "#define BP_START_" . $sysmapcolumns[ 2 ] . " 0x" .
	$sysmapcolumns[ 0 ] . "\n";

    $endline = "#define BP_END_" . $sysmapcolumns[ 2 ] . " 0x" .
	$endaddress . "\n";

    $endaddress = hex( $sysmapcolumns[ 0 ] ) - 1;
    $endaddress = sprintf( "%08x", $endaddress );

    push( @outputlines, $endline );
    push( @outputlines, $startline );

}

# put things back into a sensible order again...
@outputlines = reverse( @outputlines );
print OUTPUT @outputlines;

#
# Append stub_tail to binpatch.h file.
#

open( STUBFILE, "stub_tail" ) || die "can't open stub_tail";
print OUTPUT <STUBFILE>;
close STUBFILE;

#
# Close binpatch.h, we're done.
#

close OUTPUT;
