/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * test_mod.c
 *
 * Simple module to test binpatch tools.
 *
 */

#include <linux/module.h>      /* for module macros */
#include <linux/kernel.h>      /* for kernel utility functions */
#include <linux/sched.h>       /* for struct task_struct, current */

#include "binpatch.h"

#define NOT_NOW 1

#if NOT_NOW
int my_do_fork( ulong clone_flags, ulong usp, struct pt_regs *regs )
{
    int (*volatile orig)(ulong, ulong, struct pt_regs *);
    int retval;

    orig = (int (*)(ulong, ulong, struct pt_regs *))
                        (ORIGINAL(do_fork));

    retval = orig(clone_flags, usp, regs);

    printk( KERN_INFO "process %u forks process %u\n",
            current->pid, retval);

    return retval;
}
#else

/* This one works on 2.0 kernels, but not on 2.2 kernels.  Does it
 * overflow the stack?
 */

int my_copy_files( unsigned long clone_flags, struct task_struct *tsk)
{
    int (*volatile orig)(ulong,struct task_struct *);

    printk( KERN_INFO "process %u forks process %u\n", current->pid,
            current->p_cptr->pid );

    orig = (int (*)(ulong,struct task_struct*))(ORIGINAL(copy_files));
    return orig(clone_flags, tsk);
} /* my_copy_files() */
#endif

int my_search_binary_handler(struct linux_binprm *bprm,struct pt_regs *regs)
{
    int (*volatile orig)(struct linux_binprm*, struct pt_regs*);
    int ret_val;

    orig = ((int (*)(struct linux_binprm*, struct pt_regs*))
            (ORIGINAL(search_binary_handler)));
    ret_val = orig(bprm, regs);

    if( ret_val >= 0 ) {
        printk( KERN_INFO "process %u execs %s\n", current->pid,
                bprm->filename );
    }

    return ret_val;
} /* my_search_binary_handler() */


void my_exit_thread( void )
{
    void (* volatile orig)(void) = ((void (*)(void))(ORIGINAL(exit_thread)));
    printk( KERN_INFO "process %u exiting\n", current->pid );
    orig();
} /* my_exit_thread() */


/* Intercept calls to this only from exit.c::release() */
void my_free_uid( struct task_struct *p )
{
    void (* volatile orig)(struct task_struct *) = 
        ((void (*)(struct task_struct *))(ORIGINAL(free_uid)));
    printk( KERN_INFO "process %u freed\n", p->pid );

    orig(p);
} /* my_free_uid() */


/* init_module()
 *
 * in:     nothing
 * out:    nothing
 * return: value      condition
 *         -----      ---------
 *         0          success
 *
 */

int init_module( void )
{
    int count = 0;

#if NOT_NOW
    count += BINPATCH( sys_fork, do_fork, my_do_fork);
    count += BINPATCH( sys_clone, do_fork, my_do_fork);
    count += BINPATCH( sys_vfork, do_fork, my_do_fork);
#else
    count += BINPATCH( do_fork, copy_files, my_copy_files);
#endif
    printk( KERN_CRIT "%d fork calls patched.\n", count );

    count += BINPATCH( do_execve, search_binary_handler, 
            my_search_binary_handler );
    printk( KERN_CRIT "%d fork+exec calls patched.\n", count );

    count += BINPATCH( do_exit, exit_thread, my_exit_thread );
    printk( KERN_CRIT "%d fork+exec+exit calls patched.\n", count );

    count += BINPATCH( release, free_uid, my_free_uid );
    printk( KERN_CRIT "%d fork+exec+exit+exit2 calls patched.\n", count );

    return( 0 );
} /* init_module() */


/* cleanup_module()
 * 
 * in:     nothing
 * out:    nothing
 * return: nothing
 *
 */

void cleanup_module( void )
{
    int count = 0;

#if NOT_NOW
    count += BINUNPATCH( sys_fork, my_do_fork, do_fork);
    count += BINUNPATCH( sys_clone, my_do_fork,  do_fork);
    count += BINUNPATCH( sys_vfork, my_do_fork,  do_fork);
#else
    count += BINUNPATCH( do_fork, my_copy_files, copy_files );
#endif
    count += BINUNPATCH( do_execve, my_search_binary_handler, 
            search_binary_handler );
    count += BINUNPATCH( do_exit, my_exit_thread, exit_thread );
    count += BINUNPATCH( release, my_free_uid, free_uid );


    printk( KERN_INFO "%d calls un-patched.\n", count );
} /* cleanup_module() */



