/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_dispatch.c,v 1.24 2001/02/22 23:09:56 stevek Exp $
 *
 * Author: Steve Kiernan, Doug Kilpatrick
 *
 * This is the main dispatch routine for all syscalls -- Linux style
 */

#include <wr_conf.h>

#include <wss_process.h>
#include <wss_wrapper.h>

#include <wsi_syscalls.h>



/***************************************************************************\
 * Predeclarations and static variables                                    *
\***************************************************************************/

typedef asmlinkage int (*linkagecast)(struct pt_regs);



/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

asmlinkage int
wsi_dispatch_pre( pd_syscall_id_t code, void **uap, wss_proc_t *wrp )
{
   volatile wss_wrapper_instance_t *wi, *wt;

   for( wt = NULL, wi = wrp->wrp_wi_list; wi ; wt = wi, wi = wi->wi_next )
   {
      if( wi->wi_wrapper->w_dispatch_pre )
      {
         int                    inerror;
         wss_wrapper_dispatch_t intercept;
           
         intercept = wi->wi_wrapper->w_dispatch_pre;
#ifdef DEBUG_DISPATCH
         pd_printf(PD_MAKESTR("Pre[%d:%s]: Before w_dispatch_pre\n"),code,
                   wsi_syscall_name(code));
#endif
         WSI_PROC_OPAQUE(wrp)->wrd.wiv = wi->wi_data;
         inerror = (*intercept)( 0, code, uap, &(WSI_PROC_OPAQUE(wrp)->retval),
                                 wrp, &(WSI_PROC_OPAQUE(wrp)->wrd), 1);
#ifdef DEBUG_DISPATCH
         pd_printf(PD_MAKESTR("Pre[%d:%s]: After w_dispatch_pre\n"),code,
                   wsi_syscall_name(code));
#endif
         if( (inerror & WR_MASK1) == WR_DENY )
         {
#ifdef DEBUG_DISPATCH
            pd_printf(PD_MAKESTR("%d; *wrd.errnum = %d"),code,
                      *(WSI_PROC_OPAQUE(wrp)->wrd.errnum));
#endif
            return (inerror & WR_MASK1);
         }
      }
   }
   return 0;
}

asmlinkage int
wsi_dispatch_post( pd_syscall_id_t code, void **uap, wss_proc_t *wrp )
{
   volatile wss_wrapper_instance_t *wi = WSI_PROC_OPAQUE(wrp)->lastwi;

   /*
    * Call post functions for each wrapper.  We perform the
    * "truncated onion method" for failure handling.  Basically
    * if there was a failure, we start with the wrapper that
    * cause the error and work our way back to the first wrapper
    * called.  In this way, after a PRE event fails, no more PRE
    * events run.  POST events start with the failing one and
    * continue with POST events for all prior PREs.
    */
   
   /* mutex_enter? */
   if( wi == NULL )
      wi = wrp->wrp_wi_list;
   for( ; wi; wi = wi->wi_prev )
   {
      if( wi->wi_wrapper->w_dispatch_post )
      { 
         int                    inerror;
         wss_wrapper_dispatch_t intercept;
           
         intercept = wi->wi_wrapper->w_dispatch_post;
#ifdef DEBUG_DISPATCH
         pd_printf(PD_MAKESTR("Post[%d:%s]: Before w_dispatch_post\n"),code,
                   wsi_syscall_name(code));
#endif
         WSI_PROC_OPAQUE(wrp)->wrd.wiv = wi->wi_data;
         inerror = (*intercept)(0, code, uap, &(WSI_PROC_OPAQUE(wrp)->retval),
                                wrp, &(WSI_PROC_OPAQUE(wrp)->wrd), 1);
#ifdef DEBUG_DISPATCH
         pd_printf(PD_MAKESTR("Post[%d:%s]: After w_dispatch_post\n"),code,
                   wsi_syscall_name(code));
#endif
         if ( (inerror & WR_MASK1) == WR_DENY ) {
#ifdef DEBUG_DISPATCH
               /* A little late now to deny, eh? */
               pd_printf(PD_MAKESTR("%d; *wrd.errnum = %d"),code,
                         *(WSI_PROC_OPAQUE(wrp)->wrd.errnum));
#endif
              return (inerror & WR_MASK1);
         }
      }
   }
   return 0;
}

#ifdef DEBUG_DISPATCH
asmlinkage void
wsi_dispatch_print_opaque( wss_proc_t *wrp )
{
   struct wsi_proc *wsiproc = WSI_PROC_OPAQUE(wrp);

   pd_printf( PD_MAKESTR("wsi_proc struct:\n") );
   pd_printf( PD_MAKESTR("   savedip: 0x%8.8x\n"),
              (unsigned int)wsiproc->savedip );
   pd_printf( PD_MAKESTR("      code: %d\n"), wsiproc->code );
   pd_printf( PD_MAKESTR("    retval: %d\n"), wsiproc->retval );
   pd_printf( PD_MAKESTR("       err: %d\n"), wsiproc->err );
   pd_printf( PD_MAKESTR("    lastwi: 0x%8.8x\n"),
              (unsigned int)wsiproc->lastwi );
   pd_printf( PD_MAKESTR("wrd.errnum: 0x%8.8x\n"),
              (unsigned int)wsiproc->wrd.errnum );
   pd_printf( PD_MAKESTR("   wrd.wiv: 0x%8.8x\n"),
              (unsigned int)wsiproc->wrd.wiv );
   pd_printf( PD_MAKESTR("  wrd.skip: %d\n"), wsiproc->wrd.skip );
}
#endif

