/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_intercept.c,v 1.7 2001/02/22 23:08:48 stevek Exp $
 *
 * Author: Doug Kilpatrick, Steve Kiernan
 *
 */

#include <wr_conf.h>

#include <wss_process.h>
#include <wss_malloc.h>
#include <wss_path.h>
#include <wss_module.h>
#include <wss_vm.h>
#include <wsi_stdlib.h>
#include <wsi_syscalls.h>
#include <wsi_db.h>

#include <linux/sys.h>
#include <linux/smp_lock.h>
#include <linux/mm.h>

#include <asm/ptrace.h>
#include <asm/signal.h>
#include <asm/unistd.h>


/***************************************************************************\
 * Externals                                                               *
\***************************************************************************/

extern wss_vmspace_t *wss_db_space;
extern syscallptr sys_call_table[];

extern int wsi_dispatch(int, pd_syscall_id_t, int, int, int, int);
extern asmlinkage int sys_ni_syscall(void);


/***************************************************************************\
 * Predeclarations and static variables                                    *
\***************************************************************************/

syscallptr saved_sysent[NR_syscalls];

static syscallptr orig_sys_umount;
static syscallptr orig_sys_sync;
static syscallptr orig_sys_oldumount;


/***************************************************************************\
 * Private functions                                                       *
\***************************************************************************/

static asmlinkage int
real_sync( void )
{
   if( wss_db_space && wss_db_space->vms_file != WSS_FILE_INVALID )
      wsi_db_flush( wss_db_space );

   lock_kernel();
   fsync_dev( 0 );
   unlock_kernel();
   return 0;
}

static asmlinkage int
real_umount( char *name, int flags )
{
   int (* volatile orig)(char *, int) = (int (*)(char *,int))(orig_sys_umount);

#warning SMP Race condition here.
   static char kname[128]; /* space to put "/var" */
   kname[0] = '\0';
   if( name )
      copy_from_user( name, kname, 127 );

   if( ( name[0] != '\0' ) && wss_db_space &&
       ( wss_db_space->vms_file != WSS_FILE_INVALID ) &&
       !strncmp( wss_db_space->vms_file, kname, strlen(kname) ) )
   {
      wsi_db_flush( wss_db_space );
      wss_vm_destroy( wss_db_space );
      wss_db_space = NULL;
/* XXX what?       wss_vm_shutdown(); */
   }
   return orig( name, flags );
}

static asmlinkage int
real_oldumount( char *name )
{
   return real_umount( name, 0 );
}


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int
wsi_intercept_init( void )
{
   int i;
    
   lock_kernel();
    
   orig_sys_umount = sys_call_table[__NR_umount2];
   orig_sys_sync = sys_call_table[__NR_sync];
   orig_sys_oldumount = sys_call_table[__NR_umount];

   /* Replace syscalls we need to augment with our "real" ones */
   sys_call_table[__NR_umount2] = (syscallptr) real_umount;
   sys_call_table[__NR_sync] = (syscallptr) real_sync;
   sys_call_table[__NR_umount] = (syscallptr) real_oldumount;

   /* Replace all the syscalls with the dispatcher */
   for( i = 1; i < NR_syscalls; i++ )
   {
      switch( i )
      {
         case __NR_socketcall:
         case __NR_sigreturn:
         case __NR_rt_sigreturn:
         case __NR_rt_sigtimedwait:
         case __NR_rt_sigsuspend:
            /* Do not intercept these */
            break;

         default:
            if( sys_call_table[i] != sys_call_table[0] )
            {
               saved_sysent[i] = sys_call_table[i];
               sys_call_table[i] = (syscallptr) wsi_dispatch;
            }
            break;
      }
   }

   unlock_kernel();

   return 0; 
}

int
wsi_intercept_shutdown( void )
{
   int i;

   lock_kernel();

   /* Replace syscalls we need to augment with our "real" ones */
   sys_call_table[__NR_umount2] = orig_sys_umount;
   sys_call_table[__NR_sync] = orig_sys_sync;
   sys_call_table[__NR_umount] = orig_sys_oldumount;

   for( i = 1; i < NR_syscalls; i++ )
   {
      switch( i )
      {
         case __NR_socketcall:
         case __NR_sigreturn:
         case __NR_rt_sigreturn:
         case __NR_rt_sigtimedwait:
         case __NR_rt_sigsuspend:
            /* Not intercepted */
            break;

         default:
            if( sys_call_table[i] != sys_call_table[0] )
               sys_call_table[i] = saved_sysent[i];
            break;
      }
   }
   unlock_kernel();

   return 0;
}

