/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.c,v 1.8 2001/02/23 19:02:37 stevek Exp $
 *
 * Authors: Steve Kiernan
 *
 * Purpose: System-dependent process management functions
 */

#include <wr_conf.h>

#ifndef __WSS_PROCESS_H
#include <wss_process.h>
#endif

#ifndef __WSS_MALLOC_H
#include <wss_malloc.h>
#endif

#ifndef __WSI_STDLIB_H
#include <wsi_stdlib.h>
#endif

/***************************************************************************\
 * External globals                                                        *
\***************************************************************************/

extern const char *original_syscallnames[];


/***************************************************************************\
 * Public functions
\***************************************************************************/

/*
 * Create the system-dependent opaque data part of the wss_proc structure
 */
void
wsi_proc_new( struct wss_proc *wrp )
{
   struct wsi_proc *wsiproc;

   wsiproc = (struct wsi_proc *)wss_malloc( sizeof(*wsiproc) );
   bzero(wsiproc, sizeof(*wsiproc));
   wrp->wrp_opaque = wsiproc;
}

/*
 * Free the wss_proc structure system-dependent opaque data
 */
void
wsi_proc_free( struct wss_proc *wrp )
{
   wss_free( wrp->wrp_opaque );
}

void
wsi_proc_fork( struct wss_proc *parent_wrp, struct wss_proc *wrp )
{
   memcpy( wrp->wrp_opaque, parent_wrp->wrp_opaque, sizeof(struct wsi_proc) );
}

/*
 * This is a placeholder for code that actually does make LWPs reissue
 * their syscalls.  It's obviously TBD.
 */
void
wsi_proc_force_syscall_exit( struct wss_proc *wrp )
{
}

void
wsi_proc_rele( struct wss_proc *wrp )
{
   wss_proc_rele(wrp);
}

