/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_syscall.c,v 1.2 2001/02/16 21:18:54 stevek Exp $
 *
 * Author: Doug Kilpatrick
 *
 */

#include <wr_conf.h>

#include <wsi_syscalls.h>

#include <linux/sys.h>
#include <linux/smp_lock.h>


extern syscallptr sys_call_table[];


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/


/*
 * Return syscall name based on code argument
 * 
 * The table used here is generaged by parsesyscall.pl
 */
const char *
wsi_syscall_name( pd_syscall_id_t code )
{
    if (code >= NR_syscalls) {
        return 0;
    }
    return linux_syscallnames[code];
}

/*
 * Puts our functions in place of all the standard ones.
 */
int
wsi_syscall_init(void)
{
   lock_kernel();

   /* Add our custom syscalls */
   sys_call_table[SYS_wrctl] = (syscallptr)wrctl;
   sys_call_table[SYS_wrv]   = (syscallptr)wrv;
    
   unlock_kernel();
   return 0;
}


int
wsi_syscall_shutdown(void)
{
   lock_kernel();
   sys_call_table[SYS_wrctl] = sys_call_table[0];
   sys_call_table[SYS_wrv]   = sys_call_table[0];
   unlock_kernel();
   return 0;
}

