/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_sysevents.c,v 1.7 2001/02/23 19:02:37 stevek Exp $
 *
 * Author: Douglas Kilpatrick
 *
 * Purpose: This file contains the hooks necessary to support the
 * system_event events.  Currently that only means "coredump", but
 * may be expanded.
 */

#include <wr_conf.h>
#include <wsi_stdlib.h>
#include <wss_wrapper.h>
#include <wss_malloc.h>
#include <wss_path.h>
#include <wss_dispatch.h>
#include <wss_evspace.h>

#include <linux/version.h>
#include <linux/sched.h>


static int (*old_core_dump)(long signr, struct pt_regs * regs);
static int system_event_evspace;


/***************************************************************************\
 * Private functions                                                       *
\***************************************************************************/

static int
core_dump_intercept(long signr, struct pt_regs *regs)
{
    /* Assume code == 1.  This means that this event has to be 
     * first in the space.
     */
    wss_proc_t                 *wrp;
    char                       *canpath;

    wrp = wss_pfind(wsi_proc_pid(current));
    if (!wrp)
        return old_core_dump(signr, regs);

    canpath = wss_path_abspath("./core");

    wss_dispatch(system_event_evspace, 1, wrp, 0, 1, canpath);

    wss_free(canpath);
    wss_proc_rele( wrp );

    return old_core_dump(signr, regs);
}


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/


void
wsi_sysevents_init(void)
{
    /* Add a "system event" event space. */
    system_event_evspace = wss_evspace_resolve("system");

    /* Hook into the coredump commands.
     *
     * We should create a stub function for each bin format
     * so that we can do the coredump for each binformat.
     *
     * As a hack, just do the binfmt for current, which should
     * be the format used most often.
     */
    if (current->binfmt != NULL) {
        old_core_dump = current->binfmt->core_dump;
        current->binfmt->core_dump = core_dump_intercept;
    }
}

int
wsi_sysevents_getevspace(void)
{
    return system_event_evspace;
}

