/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_dispatch.c,v 1.5 2001/02/23 19:02:40 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * This is the main dispatch routine for all syscalls -- Solaris style.
 */

#include <wr_conf.h>

/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*
 * Old-style syscall dispatch routine
 */
int
wsi_dispatch( int code, void *uap, rval_t *retval )
{
   int err;
   /*
    * wsi_dispatch(code, uap, retval)
    * 
    * Find wrappers that wrap the calling process.
    * 
    * do {
    *   Call "pre" functions for each wrapper.
    * until (return value == WR_DENY)
    * if WR_DENY, return appropriate error.
    * Call the actual system call
    * Call "post" functions for each wrapper called in pre in reverse order.
    * 
    * 
    * When done, call
    * wss_arg_donesyscall() to clean up any argument replacements
    * that have been leaked.
    * 
    */
   
   /*
    * Call post functions for each wrapper.  We perform the
    * "truncated onion method" for failure handling.  Basically
    * if there was a failure, we start with the wrapper that
    * cause the error and work our way back to the first wrapper
    * called.  In this way, after a PRE event fails, no more PRE
    * events run.  POST events start with the failing one and
    * continue with POST events for all prior PREs.
    */
   return( err ); /* err is from syscall execution or set by the pre */
}

/*
 * Return syscall name based on code argument
 * 
 * The table used here is generaged by parsesyscall.pl
 */
const char *
wsi_syscall_name( int code )
{
   return original_syscallnames[code];
}

