/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_exported.c,v 1.5 2001/02/23 19:02:40 stevek Exp $
 *
 * Authors: Steve Kiernan
 *
 * Purpose: This file contains the table used to export system-dependent
 * functions from the WSS to activation criteria and to wrappers (mostly used
 * transparently for unresolved libc symbols).
 */

#include <wr_conf.h>

/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

/* This table includes all the local sysbols we need to export to wrappers.
 * 
 * In Solaris we use it to deal with builtin math symbols.  It is unused
 * in FreeBSD.
 */
wss_symbol_t wsi_export_table[] =
{
   { NULL, 0, 0 }
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*
 * Initialize system-depedent exported functions table
 */
int
wsi_export_init()
{
   return 0;
}

/*
 * Lookup system-dependent symbols in table
 */
u_long
wsi_export_lookup( wss_mod_t *wrm, char *name )
{
   int           index;
   wss_symbol_t *sym;

   for( index = 0; ( sym = wsi_export_table + index ); index++ )
   {
      if( !strcmp( name, sym->name ) )
         return sym->addr;
   }

   return 0u;
}

