/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_malloc.c,v 1.5 2001/02/23 19:02:40 stevek Exp $
 *
 * Authors: Douglas Kilpatrick, Steve Kiernan
 *
 * Purpose:
 * This file contains the platform specific part of the malloc routines.
 * 
 */

#include <wr_conf.h>

/***************************************************************************\
 * Forward declarations                                                    *
\***************************************************************************/

void *wsi_proc_malloc __P((struct proc *p, int size));
void  wsi_proc_free   __P((struct proc *p, void *addr, int size));


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*
 * Allocate memory of at least specified size
 */
void *
wsi_malloc( size_t size )
{
   return 0;
}

/*
 * Free allocated memory
 */
void
wsi_free( void *ptr )
{
}


/*
 * Allocates memory in the process space of the current process.
 * This gets used to temporarily store replaced syscall parameters.
 * The size and the address of the memory allocated here will be passed
 * into free, so the allocation mechanism may be simplified.
 * 
 */
void *
wsi_curproc_malloc( size_t size )
{
  return 0;
}

/*
 * Frees the address from the current process's vmspace.
 */
void
wsi_curproc_free( void *addr, size_t size )
{
}

/*
 * Create duplicate of string in kernel memory
 */
char *
wsi_strdup( const char *str )
{
   char *buf;
   buf = (char *)wsi_malloc( strlen( str ) + 1 );
   strcpy( buf, str );
   return( buf );
}

/*
 * Similar to strdup, but allocates memory from the current process's vmspace
 */
char *
wsi_curproc_strdup( const char *str )
{
   char *buf;
   buf = (char *)wsi_curproc_malloc( strlen( str ) + 1 );
   strcpy( buf, str );
   return( buf );
}

