/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_vm.c,v 1.5 2001/02/23 19:02:40 stevek Exp $
 *
 * Authors: Wayne Morrison, others
 *
 * This file contains functions for creating VM used by the WSS.
 *      Currently, it creates a wss.db file in the current directory
 *      and maps it to kernel space for use by the DBMS functions.
 *
 */

#include <wr_conf.h>

#include <wsi_vm.h>


/***************************************************************************\
 * Private macros                                                          *
\***************************************************************************/

#define DBNAME "wss.db"


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

vm_offset_t     wsi_vm_data;
size_t          wsi_vm_size;


/***************************************************************************\
 * Private functions                                                       *
\***************************************************************************/

/*
 * Expand the new wss.db file to the desired number of MB.  We take
 * advantage of the kernel's ability to create sparse files when they
 * are memory mapped to create the file very quickly.  As different
 * parts of the file are written, the kernel dynamically grows it's
 * actual physical representation.
 */
static vnode_t *
get_db_file( size_t size_in_mb )
{
   vnode_t *file = NULL;
   
   return file;
}


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*
 * Allocate a region of VM paged to a file, of the given size.  Create
 * the file, if necessary.  If the file already exists, simply map it.
 *
 * TBD: Set the file size set by lseek'ing past the end of the
 * (initially) empty file (much faster than filling it with writes;
 * hope that ports to kernel space).
 *
 * On success, returns a pointer to the allocated region,
 * On failure, returns NULL
 *    an existing wss.db file is the wrong size
 *    any file operation failed
 *    the mmap call failed
 */

vm_offset_t
wsi_vm_init( size_t size_in_mb )
{
   vnode_t *vnnode;
   caddr_t addr;
    
   wsi_vm_data = 0;
   wsi_vm_size = 0;
    
   vnnode = get_db_file( size_in_mb );
    
   /* mmap the file in
    * addr = ...;
    */
   wsi_vm_data = (vm_offset_t)addr;

   return( wsi_vm_data );
}

/*
 * Unmap the database file
 */
void
wsi_vm_shutdown()
{
}

