/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_fileutil.h,v 1.10 2001/02/23 19:02:43 stevek Exp $
 * 
 */

#ifndef __WSI_FILEUTIL_H
#define __WSI_FILEUTIL_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#include <sys/vnode.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <arpa/inet.h>

/*
 * file type values.
 * Don't map to solaris at all, so we really really really fake it.
 */
typedef enum {
    WR_DTYPE_VNODE = 1,                 /* File                 */
    WR_DTYPE_SOCKET,                    /* network endpoint     */
    WR_DTYPE_PIPE,                      /* unamed pipe          */
    WR_DTYPE_FIFO,                      /* named pipe           */
    WR_DTYPE_OTHER                      /* ???                  */
} wsi_fileutil_filetype_t;

struct	wr_stat {
	dev_t		st_dev; 	/* inodes device                */
	u_longlong_t	st_ino; 	/* node id                      */
	mode_t		st_mode;	/* file access mode             */
	nlink_t		st_nlink;	/* number of references to file */
	uid_t		st_uid;		/* owner user id                */
	gid_t		st_gid;		/* owner group id               */
	dev_t		st_rdev; 	/* device the file represents(fsid) */
        struct timespec st_atimespec;	/* time of last access          */
        struct timespec st_mtimespec;	/* time of last modification    */
        struct timespec st_ctimespec;	/* time file ``created''        */
	u_offset_t	st_size;	/* file size in bytes */
	u_long		st_gen;         /* version code */
#if 0
	long		va_mask;	/* bit-mask of attributes */
	vtype_t		va_type;	/* vnode type (for create) */
	u_long		va_blksize;	/* fundamental block size */
	u_longlong_t	va_nblocks;	/* # of blocks allocated */
#endif
};

/* a list of macro associated with wr_stat */
#define WorldReadable(s)        ((s).st_mode & S_IROTH)
#define WorldWritable(s)        ((s).st_mode & S_IWOTH)
#define WorldExecutable(s)      ((s).st_mode & S_IXOTH)

#define GroupReadable(s)        ((s).st_mode & S_IRGRP)
#define GroupWritable(s)        ((s).st_mode & S_IWGRP)
#define GroupExecutable(s)      ((s).st_mode & S_IXGRP)

#define OwnerReadable(s)        ((s).st_mode & S_IRUSR)
#define OwnerWritable(s)        ((s).st_mode & S_IWUSR)
#define OwnerExecutable(s)      ((s).st_mode & S_IXUSR)

#define IsPipe(s)               ((s).st_rdev == S_IFIFO)
#define IsDir(s)                ((s).st_rdev == S_IFDIR)
#define IsSymlink(s)            ((s).st_rdev == S_IFLNK)
#define IsSocket(s)             ((s).st_rdev == S_IFSOCK)
#define IsCharDev(s)            ((s).st_rdev == S_IFCHR)
#define IsBlkDev(s)             ((s).st_rdev == S_IFBLK)
#define IsRegFile(s)            ((s).st_rdev == S_IFREG)
#define IsWhiteOut(s)           ((s).st_rdev == S_IFWHT)

#define Owner(s)                (s).st_uid


extern int  wsi_fileutil_stat    __P((const char *path, struct wr_stat *sb));
extern int  wsi_fileutil_lstat   __P((const char *path, struct wr_stat *sb));
extern int  wsi_fileutil_fstat   __P((unsigned int fd, struct wr_stat *sb));
extern int  wsi_fileutil_fdtype  __P((unsigned int fd));
extern int  wsi_fileutil_fdfflag __P((unsigned int fd));

extern int  wsi_netutil_get_addr __P((const struct sockaddr *saddrr, int len,
                                       char *buffer, int buffer_length));
extern int  wsi_netutil_getsockname __P((int fd, struct sockaddr *s, int *len));
extern int  wsi_netutil_getpeername __P((int fd, struct sockaddr *s, int *len));
extern const char  *wsi_netutil_get_family __P((const struct sockaddr *, int));


#if 0
/* obtain a path name from an inode or a file descriptor */
string wsi_fileutil_getpath(int fd);
/* rename a file */
int wsi_fileutil_rename(const char *spath, const char *dpath);

/* subr for manipulation files */
int wsi_fileutil_open()
int wsi_fileutil_read()
int wsi_fileutil_write()
int wsi_fileutil_copy(const char *from, const char *to);
int wsi_fileutil_save(const char *from, const char *to);

int wsi_fileutil_close()
#define wsi_netutil_ntohs ntohs


#endif


/************************************************************************\
 * Public netutil functions                                             *
\************************************************************************/

const char *wsi_netutil_get_family(const struct sockaddr *saddr, int len);
int wsi_netutil_get_addr(const struct sockaddr *saddr, int len, char *, int);
int wsi_netutil_getsockname(int fd, struct sockaddr *s, int *len);
int wsi_netutil_getpeername(int fd, struct sockaddr *s, int *len);
in_port_t wsi_netutil_ntohs(in_port_t i);

#endif /* __WSI_FILEUTIL_H */
