/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_hook.h,v 1.9 2001/02/23 19:02:43 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures used by the
 *          Solaris kernel hot-patch hook routine
 */

#ifndef __WSI_HOOK_H
#define __WSI_HOOK_H 1

#include <wr_conf.h>


/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

typedef struct _sym_pair
{
   const pd_char_t *callname;
   caddr_t           replace;
   caddr_t          *save;
   caddr_t           orig;
   int               instances;
}
sym_pair_t;


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define KERN_HOOK_BEGIN(h)  sym_pair_t h ## [] = {
#define KERN_HOOK(f,s)      { #f, (caddr_t)(s), NULL, NULL, 0 },
#define KERN_HOOK_SAVE(f,s) { #f, (caddr_t)(s), (caddr_t *)&(f), NULL, 0 },
#define KERN_HOOK_END       { NULL, NULL, NULL, 0 } }


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

int wsi_hook_init     __P(( void ));
int wsi_hook_shutdown __P(( void ));
int wsi_hook_add      __P(( const pd_char_t *fname, sym_pair_t *syms ));
int wsi_hook_remove   __P(( const pd_char_t *fname ));

#endif /* !__WSS_HOOK_H */

