/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_process.h,v 1.28 2001/02/23 19:02:43 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: This file contains prototypes and structures use by the
 *          system-depenedant parts of the process life-cycle code
 */

#ifndef __WSI_PROCESS_H
#define __WSI_PROCESS_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#include <sys/types.h>
#include <sys/systm.h>
#include <sys/proc.h>
#include <sys/rwlock.h>
#include <errno.h>

#ifndef _SYS_QUEUE_H_
#include <queue.h>
#endif

#ifndef __WSS_MUTEX_H
#include <wss_mutex.h>
#endif

#ifndef __WSS_CV_H
#include <wss_cv.h>
#endif



/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

struct wss_proc;
struct wss_wrapper_instance;

typedef struct _wsi_proc_dispatch_list wsi_proc_dispatch_list_t;
typedef struct _wsi_proc_dispatch      wsi_proc_dispatch_t;
typedef struct _wsi_proc               wsi_proc_t;

struct _wsi_proc_dispatch
{
   LIST_ENTRY(_wsi_proc_dispatch)  next;
   int                             refcount;
   struct wss_wrapper_instance    *instance;
};

struct _wsi_proc_dispatch_list
{
   PD_LIST_HEAD(_wsi_dispatch_list, _wsi_proc_dispatch) list;
};

struct _wsi_proc
{
   wsi_proc_dispatch_list_t dispatchers[NSYSCALL];
};


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#define wsi_proc_get_curproc() (curproc)
#define wsi_proc_get_current_pid() ((int)curproc->p_pidp->pid_id)

#define wsi_proc_pid(p)         ((int)(p)->p_pidp->pid_id)

/*
 * local wss_proc flags
 */
#define WRPF_GET_CWD    WRPF_LOCAL


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
int  wsi_proc_insyscall          __P(( struct wss_proc * ));
void wsi_proc_fork               __P(( struct wss_proc *, struct wss_proc * ));
void wsi_proc_free               __P(( struct wss_proc * ));
void wsi_proc_new                __P(( struct wss_proc * ));
void wsi_proc_force_syscall_exit __P(( struct wss_proc * ));
int  wsi_proc_suspend            __P(( struct proc * ));
int  wsi_proc_suspend_lwp        __P(( kthread_t *, int * ));
void wsi_proc_continue           __P(( struct proc * ));
void wsi_proc_continue_lwp       __P(( kthread_t * ));
#endif /* WR_SUPPRESS_FUNCTION_DECL */


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wsi_proc_fork(a,b)

#endif /* !__WSI_PROCESS_H */

