/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_rw.h,v 1.8 2001/02/23 19:02:43 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Purpose: Macros for system-dependent reader/writer locks
 *
 */

#ifndef __WSI_RW_H
#define __WSI_RW_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef _KERNEL
typedef void cond_t;
#endif /* !_KERNEL */

#ifdef PD_IN_KERNEL
#include <sys/ksynch.h>
#endif /* PD_IN_KERNEL */


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

#ifdef PD_IN_KERNEL
#define WSI_RW_PROTECT(var) krwlock_t var ## _rwlock
#define WSI_RW_PROTECT_EXPORTED(var) krwlock_t var ## _rwlock
#define WSI_RW_EXPORT(var) extern krwlock_t var ## _rwlock

#ifndef WR_SUPPRESS_FUNCTION_DECL
#define WSI_RW_INIT(var) rw_init( &(var ## _rwlock), NULL, RW_DRIVER, NULL )
#define WSI_RW_DESTROY(var) rw_destroy( &(var ## _rwlock) )
#define WSI_RW_ENTER(var,type) rw_enter( &(var ## _rwlock), (type) )
#define WSI_RW_ENTER_DEBUG(var,type) \
   do { printf( "rw_enter(" ## #var ## "_rwlock=0x%x, " \
                "/thread=0x%x @ %s:%d\n", (uint32_t)&(var ## _rwlock), \
                (uint32_t)curthread, __FILE__, __LINE__ ); \
        rw_enter( &(var ## _rwlock), (type) ); } while(0)
#define WSI_RW_EXIT(var) rw_exit( &(var ## _rwlock) )
#define WSI_RW_EXIT_DEBUG(var) \
   do { printf( "rw_exit(" ## #var ## "_rwlock=0x%x)/thread=0x%x @ %s:%d\n", \
                (uint32_t)&(var ## _rwlock), (uint32_t)curthread, __FILE__, \
                __LINE__ ); \
        rw_exit( &(var ## _rwlock) ); } while(0)
#define WSI_RW_DOWNGRADE(var) rw_downgrade( &(var ## _rwlock) )
#define WSI_RW_DOWNGRADE_DEBUG(var) \
   do { printf( "rw_downgrade(" ## #var ## "_rwlock=0x%x)/thread=0x%x @ " \
                "%s:%d\n", (uint32_t)&(var ## _rwlock), \
                (uint32_t)curthread, __FILE__, __LINE__ ); \
        rw_downgrade( &(var ## _rwlock) ); } while(0)

#endif /* WR_SUPPRESS_FUNCTION_DECL */

#else /* PD_IN_KERNEL */

#define WSI_RW_PROTECT(var)
#define WSI_RW_EXPORT(var)
#define WSI_RW_INIT(var)
#define WSI_RW_DESTROY(var)
#define WSI_RW_ENTER(var,type)
#define WSI_RW_ENTER_DEBUG(var,type)
#define WSI_RW_EXIT(var)
#define WSI_RW_EXIT_DEBUG(var)
#define WSI_RW_DOWNGRADE(var)
#define WSI_RW_DOWNGRADE_DEBUG(var)

#endif /* PD_IN_KERNEL */

#endif /* !__WSS_RW_H */

