/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: makebigfile.c,v 1.7 2001/02/23 19:02:42 stevek Exp $
 *
 * Author: Doug Kilpatrick
 *
 * Purpose: Creates a sparse or packed file for use as the wss.db mapped-
 *          database file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <ctype.h>


/***************************************************************************\
 * Macros                                                                  *
\***************************************************************************/

#define FILENAME "wss.db"
#define FILESIZE 1024*1024*16
#define BLOCKSIZE 512


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

const char *usage_str = 
"[-p] [-f filename] [-s size]\n"
"    with no flags, create a 16 MB sparse file called \"wss.db\"\n"
"    -p  create a packed  file. (force each block to be allocated.)\n"
"    -f  use the provided filename instead of \"wss.db\".\n"
"    -s  create a file of the provided size (in bytes)\n"
"    -b  use the provided blocksize instead of 1024 \n"
;


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void
usage( const char *program_name )
{
   fprintf( stderr, "Usage:\n%s %s",program_name, usage_str );
   exit( EXIT_FAILURE );
}

int
main( int argc, char *argv[] )
{
   size_t size = FILESIZE, block=BLOCKSIZE;
   char *name = FILENAME;
   int sparse = 1;
   int i,c;
   FILE *file;

   while( ( c = getopt( argc, argv, "pf:s:b:" ) ) != EOF )
   {
      switch( c )
      {
         case 'p':
            sparse = 0;
            break;
         case 'f':
            name = optarg;
            break;
         case 's':
            size = atoi( optarg );
            if( size > 0x8000000 )
               size = 0x8000000;
            break;
         case 'b':
            block = atoi( optarg );
            break;
         case '?':
         default:
            usage( argv[0] );
      }
   }
    
   if( ( argc - optind ) != 0 )
      usage( argv[0] );

   file = fopen( name,"w+" );
   if( !file )
   {
      fprintf( stderr, "Error opening file %s.\n", name );
      perror( argv[0] );
      return EXIT_FAILURE;
   }
   fseek( file,size-1,SEEK_SET );
   fwrite( "\0",1,1,file );
   if( !sparse )
   {
       fseek( file,0,SEEK_SET );
       for( i = 0; i < ((int)(size/block) - 1); i++ )
       {
          fwrite( "\0",1,1,file );
          fseek( file,block,SEEK_CUR );
       }
   }

   return EXIT_SUCCESS;
}

