#!/usr/local/bin/perl

# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: parsesyscall.pl,v 1.22 2001/04/05 18:41:55 stevek Exp $
#
# Purpose: This file contains the script to generate the syscall 
# interception file (wsi_intercept.c).  It requires a Solaris 2.x
# system, specifically the file /etc/name_to_sysnum.  Using this
# file as input, this script generates function prototypes, 
# the intermediate functions, a sysent table, and a new sysentvec.
#

my $print_syscalls = {
#	"rexit" => 1,
#	"fork" => 2,
#	"vfork" => 119,
#	"fork1" => 143,
#	"lwp_exit" => 160
};

my $custom_syscalls = {
	"uadmin" => 55
};

my $custom_syscalls_c = {
};

open(SYSCALLS, "/etc/name_to_sysnum") || die "/etc/name_to_sysnum: $!";
open(OUT, ">wsi_intercept.c") || die "wsi_intercept.c: $!";

while( <SYSCALLS> ) {
	chomp;
	next if /^\s*$/;
	/^\s*([^\s]+)\s+([^\s]+)\s*$/;
	$lastsyscall = $2;
	$syscall[$2] = $1;
	$count++;
}

close(SYSCALLS);

die "No syscalls found" if( !$count );

print "Generating wsi_intercept.c:  $count interpreted out of $lastsyscall\n";
#print "Processed $count system calls (Ignored ioctl, _syscall, __syscall)\n";
#print "Last syscall is number $lastsyscall\n";

print OUT <<EOF;
/*
 * wsi_intercept.c
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */
#include <wr_conf.h>

#include <sys/systm.h>
#include <sys/syscall.h>

#include <wsi_syscalls.h>

int wsi_dispatch(int, pd_syscall_id_t, void *uap, rval_t *retval);
longlong_t wsi_dispatch_c(int, pd_syscall_id_t, int, int, int, int, int, int); 


EOF

for( $i=0; $i <= $lastsyscall; $i++ ) {
	if( $syscall[$i] ) {
		while( defined( $declared{$syscall[$i]} ) ) {
			$syscall[$i] .= "_2";
		}
		$declared{$syscall[$i]}++;
		print OUT "int wsi_sys_$syscall[$i](void *uap, rval_t *retval);\n";
		print OUT "longlong_t wsi_sys_$syscall[$i]_c(";
                print OUT "int i0, int i1, int i2, int i3, int i4, int i5);\n";
	}
}

print OUT "\n\n";

for( $i=0; $i <= $lastsyscall; $i++ ) {
	if( $syscall[$i] ) {
		print OUT "int wsi_sys_$syscall[$i](void \*uap, rval_t \*retval)\n";
		print OUT "{\n";
		if( exists $custom_syscalls->{$syscall[$i]} ) {
			print OUT "extern int wsi_custom_$syscall[$i](int call, void \*uap, rval_t \*retval);\n";
		}
		if( exists $print_syscalls->{$syscall[$i]} ) {
			print OUT "#ifdef DEBUG_SYSCALLS\n";
			print OUT "printf(\"$syscall[$i]\\n\");\n";
			print OUT "#endif\n";
		}
		if( exists $custom_syscalls->{$syscall[$i]} ) {
			print OUT "return(wsi_custom_$syscall[$i]($i, uap, retval)); }\n";
		}
		else {
			print OUT "return(wsi_dispatch(0, $i, uap, retval)); }\n";
		}

		print OUT "longlong_t wsi_sys_$syscall[$i]_c(int i0, int i1, ";
		print OUT "int i2, int i3, int i4, int i5)\n";
		print OUT "{\n";
		if( exists $custom_syscalls_c->{$syscall[$i]} ) {
			print OUT "extern longlong_t ";
			print OUT "wsi_custom_$syscall[$i]_c(int ";
			print OUT "call, int i0, int i1, int i2, int i3, int ";
			print OUT "i4, int i5);\n";
		}
		if( exists $print_syscalls->{$syscall[$i]} ) {
			print OUT "#ifdef DEBUG_SYSCALLS\n";
			print OUT "printf(\"$syscall[$i] c-style\\n\");\n";
			print OUT "#endif\n";
		}
		if( exists $custom_syscalls_c->{$syscall[$i]} ) {
			print OUT "return(wsi_custom_$syscall[$i]_c($i, ";
			print OUT "i0, i1, i2, i3, i4, i5)); }\n\n";
		}
		else {
			print OUT "return(wsi_dispatch_c(0, $i, i0, i1, i2, ";
                        print OUT "i3, i4, i5)); }\n\n";
		}
	}
}

print OUT <<END;

struct syscallvec new_sysent[NSYSCALL] =
{
END

for( $i=0; $i <= $lastsyscall; $i++ ) {
	if( $syscall[$i] ) {
		print OUT "\t{ wsi_sys_$syscall[$i], wsi_sys_$syscall[$i]_c },\n";
	}
	else {
		print OUT "\t{ NULL, NULL },\n";
	}
}

print OUT <<END;
};

END

close(OUT);

open(OUT, ">wsi_syscallnames.c") || die "wsi_syscallnames.c: $!";

print OUT <<END;
/*
 * wsi_syscallnames.c
 *
 *    File automatically generated by parsesyscalls.pl, do not edit!
 */
#include <wr_conf.h>

const char *solaris_syscallnames[] =
{
END

for( $i=0; $i <= $lastsyscall; $i++ ) {
	if( $syscall[$i] ) {
		print OUT "\t\"$syscall[$i]\",\n";
	}
	else {
		print OUT "\t\"\",\n";
	}
}

print OUT <<END;
};

END

close(OUT);

exit;

