/*! \file wsi_exported.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wsi_export.c,v 1.6 2001/02/23 19:02:42 stevek Exp $
 *
 * \brief   System-dependent functions export table
 * \author  Steve Kiernan
 * \version $Revision: 1.6 $
 * \date    $Date: 2001/02/23 19:02:42 $
 *
 * This file contains the table used to export system-dependent
 * functions from the WSS to activation criteria and to wrappers
 * (mostly used transparently for unresolved libc symbols).
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_iface.h>
#include <wsi_iface.h>
#include <wss_linker.h>

#include "../../linkers/elf32/linker_elf_common.h"
#include "../../linkers/elf32/linker_elf32.h"


/***************************************************************************\
 * External functions                                                      *
\***************************************************************************/

extern unsigned int kobj_getelfsym __P(( char *, void *, int * ));


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

/*! \brief Table of system-dependent functions exported to wrappers */
wss_symbol_t wsi_export_table[] =
{
   WSS_EXPORT_ENTRY( ".urem", 0 ),
   WSS_EXPORT_ENTRY( ".udiv", 0 ),
   WSS_EXPORT_ENTRY( ".umul", 0 ),
   WSS_EXPORT_ENTRY( ".rem",  0 ),
   WSS_EXPORT_ENTRY( ".div",  0 ),
   { NULL, 0, 0 }
};


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wsi_export_init */
/*! \brief Initialize system-depedent exported functions table
 *
 *  \return 0 on success,
 *  \return ENOENT if any symbols were unresolved
 */
int
wsi_export_init()
{
   int i;
   int err = 0;

   for( i = 0; wsi_export_table[i].name; i++ )
   {
      Elf32_Addr addr;
      Elf32_Word size;

      if( ( addr = kobj_getelfsym( wsi_export_table[i].name, NULL,
                                   (int *)&size ) ) )
      {
         wsi_export_table[i].addr = (u_long)addr;
#ifdef DEBUG_EXPORT
         wsi_printf( PD_MAKESTR("wsi_export_init: Resolved symbol '%s' @ "
                                 "0x%x\n"), wsi_export_table[i].name,
                     (u_int)addr );
#endif /* DEBUG_EXPORT */
      }
      else
      {
         wsi_printf( PD_MAKESTR("wsi_export_init: Unable to resolve symbol "
                                 "'%s'\n"), wsi_export_table[i].name );
         err++;
      }
   }

   if( err )
   {
      wsi_printf( PD_MAKESTR("wsi_export_init: [ %d unresolved symbols ]\n"),
                  err );
      return ENOENT;
   }

   return wss_linker_add_symbols( PD_MAKESTR("*WSI SOLARIS SPARC*"),
                                  wsi_export_table );
}

