/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wrimp.cxx,v 1.50.2.1 2001/08/06 23:52:57 stevek Exp $
 *
 * Authors: Steve Kiernan
 *
 * Purpose: Library of functions which can be imported by wrappers
 */

#define WR_SUPPRESS_WDL_STUFF 1
#define WR_SUPPRESS_STAT_STUFF 1

#include <wr_conf.h>

#include <wsi_stdlib.h>
#include <wsi_time.h>
#include <wsi_os.h>
#include <wsi_time.h>
#include <wss_db.h>
#include <wss_process.h>
#include <wss_malloc.h>
#include <wss_arg.h>
#include <wss_strings.h>
#include <wss_path.h>
#include <wss_audit.h>
#include <wss_evspace.h>
#include <wss_log.h>
#include <wss_module.h>
#include <wss_dispatch.h>
#include <wss_utilities.h>
#include <wss_mutex.h>

#include <libwr.h>

#include <stdio.h>
#include <stdarg.h>
#include <tchar.h>

#include "gwm.h"

#define MAXBUFLEN 1024

LPDB_HANDLERS      g_pHandlers = NULL;
WSS_MUTEX_PROTECT( g_pHandlers );


/*
 * Exported functions protoypes
 */
extern "C" {
long EXPORT wr_get_curtime( long *usec );
void EXPORT wr_bcopy( const void *src, void *dst, size_t len );
void EXPORT wr_bzero( void *src, size_t len );
int EXPORT wr_sprintf( pd_char_t *s, const pd_char_t *, ... );
int EXPORT wr_printf( const pd_char_t *format, ... );
int EXPORT wr_copyinstr( void *uaddr, void *kaddr, size_t len,
                         unsigned int *done );
int EXPORT wr_proc_add_wrapper( wss_proc_t *wrp, pd_char_t *name );
int EXPORT wr_proc_del_wrapper( wss_proc_t *wrp, pd_char_t *name );
void EXPORT wr_free( void *ptr );
void EXPORT *wr_malloc( size_t size );
int EXPORT wr_strlen( const pd_char_t *str );
int EXPORT wr_strcmp( const pd_char_t *s1, const pd_char_t *s2 );
pd_char_t EXPORT *wr_strdup( const pd_char_t *str );
pd_char_t EXPORT *wr_strcat( pd_char_t *s, const pd_char_t *append );
pd_char_t EXPORT *wr_strcpy( pd_char_t *s, const pd_char_t *str );
pd_char_t EXPORT *wr_strchr( const pd_char_t *str, int c );
pd_char_t EXPORT *wr_strrchr( const pd_char_t *str, int c );
pd_char_t EXPORT *wr_upcase( const pd_char_t *str );
pd_char_t EXPORT *wr_downcase( const pd_char_t *str );
pd_char_t EXPORT *wr_strunicode( const char *astr, int *size );
void EXPORT wr_seterror( int code );
char EXPORT *wr_strascii( const pd_char_t *ustr, int *size );
int EXPORT wr_match( pd_char_t *src, pd_char_t *pat, pd_char_t **beg,
                     pd_char_t **end );
int EXPORT wr_substitute( pd_char_t **src, pd_char_t *txt, pd_char_t *sub );
pd_char_t EXPORT *wr_abspath( const pd_char_t *path );
int EXPORT wql_create( pd_char_t *wrapper_name, int instance_id,
                       pd_char_t *table_name, int free_col_defs,
                       wqcd_t col_defs, int col_count, u_long *table_id );
int EXPORT wql_drop( u_long table_id );
int EXPORT wql_insert( u_long table_id, int free_vals, wqrow_t vals );
int EXPORT wql_select( u_long table_id, int free_scrit, wqsx_t scrit,
                       int take_first, int order_by, int desc,
                       wqtab_t out_table );
int EXPORT wql_delete( u_long table_id, int free_scrit, wqsx_t scrit,
                       int take_first );
int EXPORT wql_update( u_long table_id, int free_scrit, wqsx_t scrit,
                       int take_first, wqusl_t set_scrit );
void EXPORT wql_free_update_expr( wqusl_t list );
void EXPORT wql_free_criteria( wqsx_t expr );
u_long EXPORT wql_find_table_by_names( pd_char_t *wrapper_name, int id,
                                       pd_char_t *table_name );
wqcd_t EXPORT wr_mcol_def( int count, ... );
wqval_t EXPORT wr_mvals( int count, ... );
wqrow_t EXPORT wr_mrow( wqval_t vals, wqrow_t next );
wqsx_t EXPORT wr_mwqsx_and( wqsx_t left, wqsx_t right );
wqsx_t EXPORT wr_mwqsx_or( wqsx_t left, wqsx_t right );
wqsx_t EXPORT wr_mwqsx_terminal( tx_t terminal );
wqsx_t EXPORT wr_mwqsx_not( wqsx_t expression );
tx_t EXPORT wr_mtx( sxval_t left, sxval_t right, enum sxop sxop );
sxval_t EXPORT wr_msxval_field( int column );
sxval_t EXPORT wr_msxval_lit_int( int i );
sxval_t EXPORT wr_msxval_lit_float( double d );
sxval_t EXPORT wr_msxval_lit_string( const pd_char_t *str );
wqusl_t EXPORT wr_musl( sxval_t left, wquexp_t right, wqusl_t next );
wquexp_t EXPORT wr_muexp( wquexp_t left, wquexp_t right, enum numop numop );
wquexp_t EXPORT wr_muexp_term( sxval_t value );
void EXPORT dbrow_finish( struct wql_dbrow *dbrp );
void EXPORT wr_mutex_init( pd_mutex_t *mtx, const pd_char_t *name );
void EXPORT wr_mutex_destroy( pd_mutex_t *mtx );
void EXPORT wr_mutex_enter( pd_mutex_t *mtx );
void EXPORT wr_mutex_exit( pd_mutex_t *mtx );
void EXPORT wr_file_close( int id, wss_file_t file );
size_t EXPORT wr_file_get_size( int id, wss_file_t file );
void EXPORT *wr_file_get_map_addr( int id, wss_file_t handle );
wss_file_t EXPORT wr_file_open( int id, const pd_char_t*filename,
                                wss_file_mode_t mode );
int EXPORT wr_file_mmap( int id, wss_file_t file,size_t s,
                         int mapmode,pd_offset_t pos );
int EXPORT wr_file_read( int id, wss_file_t file,pd_byte_t *buffer,
                         size_t len );
int EXPORT wr_file_seek( int id, wss_file_t file,pd_offset_t offset,
                         int whence );
pd_byte_t EXPORT *wr_file_read_alloc( int id,wss_file_t file, size_t *len );
int EXPORT wr_file_write( int id, wss_file_t file,pd_byte_t *buffer,
                          size_t len );
void EXPORT wr_audit( unsigned long types,
                      const void* buffer, unsigned long length );
int EXPORT wr_audit_printf( unsigned long types, const pd_char_t *fmt, ...);
void EXPORT *wr_dupin(void *uaddr, size_t size);
int EXPORT wr_curthreadid(void);
int EXPORT wr_resolve_event_space(pd_char_t *name);
int EXPORT wr_log( const pd_char_t *format, ... );
int EXPORT wr_dispatch( int space, int code, wss_proc_t *wrp,
                        int flags, int arg_count, ... );

}



/*
 * Function necessary for DLL
 */
BOOL WINAPI
DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
{
   WSS_MUTEX_INIT( g_pHandlers );
   return TRUE;
}


/*
 * Exported functions implementation
 */
BOOL EXPORT
WRImpRegisterDBHandlers( LPDB_HANDLERS pHandlers )
{
   WSS_MUTEX_ENTER( g_pHandlers );
   if( !g_pHandlers || pHandlers == NULL )
   {
      g_pHandlers = pHandlers;
      WSS_MUTEX_EXIT( g_pHandlers );
      return TRUE;
   }
   WSS_MUTEX_EXIT( g_pHandlers );
   return FALSE;
}

int EXPORT
wr_printf( const pd_char_t *format, ... )
{
   va_list ap;
   int nchars;

   va_start( ap, format );
   nchars = wsi_vprintf( format, ap );
   va_end( ap );

   return nchars;
}

int EXPORT
wr_copyinstr( void *uaddr, void *kaddr, size_t len, unsigned int *done )
{
   if( uaddr == NULL || kaddr == NULL ) *done = 0;

   wsi_strncpy( (pd_char_t *)kaddr, (const pd_char_t *)uaddr, len );
   *done = wsi_strlen( (pd_char_t *)kaddr );

   return 0;
}

int EXPORT
wr_proc_add_wrapper( wss_proc_t *wrp, pd_char_t *name )
{
   return wss_proc_add_wrapper( wrp, name, NULL );
}

int EXPORT
wr_proc_del_wrapper( wss_proc_t *wrp, pd_char_t *name )
{
   return wss_proc_del_wrapper( wrp, name );
}

void EXPORT
wr_free( void *ptr )
{
   wss_free( ptr );
}

void EXPORT *
wr_malloc( size_t size )
{
   return wss_malloc( size );
}

int EXPORT
wr_strlen( const pd_char_t *str )
{
   return wsi_strlen( str );
}

int EXPORT
wr_strcmp( const pd_char_t *s1, const pd_char_t *s2 )
{
   return wsi_strcmp( s1, s2 );
}

pd_char_t EXPORT *
wr_time_get_current_string ( void )
{
  return wsi_time_get_current_string( );
}

pd_char_t EXPORT *
wr_strdup( const pd_char_t *str )
{
   return wss_strdup( str );
}

pd_char_t EXPORT *
wr_strcat( pd_char_t *s, const pd_char_t *append )
{
   return _tcscat( s, append );
}

pd_char_t EXPORT *
wr_strcpy( pd_char_t *s, const pd_char_t *str )
{
   return _tcscpy( s, str );
}

pd_char_t EXPORT *
wr_strchr( const pd_char_t *str, int c )
{
   return _tcschr( str, c );
}

pd_char_t EXPORT *
wr_strrchr( const pd_char_t *str, int c )
{
   return wss_strrchr( str, c );
}

pd_char_t EXPORT *
wr_downcase( const pd_char_t *str )
{
   return wss_downcase( str );
}

pd_char_t EXPORT *
wr_upcase( const pd_char_t *str )
{
   return wss_upcase( str );
}

void EXPORT
wr_seterror( int code )
{
   SetLastError( code );
}

pd_char_t EXPORT *
wr_strunicode( const char *astr, int *size )
{
   return wsi_strunicode( astr, size );
}

char EXPORT *
wr_strascii( const pd_char_t *ustr, int *size )
{
   return wsi_strascii( ustr, size );
}

int EXPORT
wr_arg_replace( wss_proc_t *wrp, int handle, int space, int code, void *uap[],
                void *new_arg, int index, size_t size )
{
   return wss_arg_replace( wrp, handle, space, code, uap, new_arg, index, size );
}

int EXPORT
wr_match( pd_char_t *src, pd_char_t *pat, pd_char_t **beg, pd_char_t **end )
{
   return wss_match(src, pat, beg, end);
}

int EXPORT
wr_substitute( pd_char_t **src, pd_char_t *txt, pd_char_t *sub )
{
   return wss_substitute(src, txt, sub);
}

pd_char_t EXPORT *wr_abspath( const pd_char_t *path )
{
   return wss_path_abspath( path );
}

int EXPORT
wql_create( pd_char_t *wrapper_name, int instance_id, pd_char_t *table_name,
            int free_col_defs, wqcd_t col_defs, int col_count,
            u_long *table_id )
{
   int retcode = -WRE_ERROR;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.CreateTable = (g_pHandlers?g_pHandlers->CreateTable:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.CreateTable )
      retcode = (*dbh.CreateTable)( wrapper_name, instance_id, table_name,
                                    free_col_defs, col_defs, col_count,
                                    table_id );

   return retcode;
}

int EXPORT
wql_drop( u_long table_id )
{
   int retcode = -WRE_NOTFOUND;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.DropTable = (g_pHandlers?g_pHandlers->DropTable:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.DropTable )
      retcode = (*dbh.DropTable)( table_id );

   return retcode;
}

int EXPORT
wql_insert( u_long table_id, int free_vals, wqrow_t vals )
{
   int retcode = -WRE_NOTFOUND;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.InsertTableRow = (g_pHandlers?g_pHandlers->InsertTableRow:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.InsertTableRow )
      retcode = (*dbh.InsertTableRow)( table_id, free_vals, vals );

   return retcode;
}

int EXPORT
wql_select( u_long table_id, int free_scrit, wqsx_t scrit, int take_first,
            int order_by, int desc, wqtab_t out_table )
{
   int retcode = -WRE_NOTFOUND;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.SelectTableRows = (g_pHandlers?g_pHandlers->SelectTableRows:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.SelectTableRows )
      retcode = (*dbh.SelectTableRows)( table_id, free_scrit, scrit,
                                        take_first, order_by, desc, out_table );

   return retcode;
}


int EXPORT
wql_delete( u_long table_id, int free_scrit, wqsx_t scrit, int take_first )
{
   int retcode = -WRE_NOTFOUND;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.DeleteTableRows = (g_pHandlers?g_pHandlers->DeleteTableRows:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.DeleteTableRows )
      retcode = (*dbh.DeleteTableRows)( table_id, free_scrit, scrit,
                                        take_first );

   return retcode;
}

int EXPORT
wql_update( u_long table_id, int free_scrit, wqsx_t scrit, int take_first,
            wqusl_t set_scrit )
{
   int retcode = -WRE_NOTFOUND;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.UpdateTableRows = (g_pHandlers?g_pHandlers->UpdateTableRows:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.UpdateTableRows )
      retcode = (*dbh.UpdateTableRows)( table_id, free_scrit, scrit,
                                        take_first, set_scrit );

   return retcode;
}

void EXPORT
wql_free_update_expr( wqusl_t list )
{
   wss_db_free_wqusl( list );
}

void EXPORT
wql_free_criteria( wqsx_t expr )
{
   wss_db_free_criteria( expr );
}

u_long EXPORT
wql_find_table_by_names( pd_char_t *wrapper_name, int id,
                         pd_char_t *table_name )
{
   u_long retcode = 0;

   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.FindTableByNames = (g_pHandlers?g_pHandlers->FindTableByNames:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.FindTableByNames )
      retcode = (*dbh.FindTableByNames)( wrapper_name, id, table_name );

   return retcode;
}

wqcd_t EXPORT
wr_mcol_def( int count, ... )
{
   va_list ap;
   wqcd_t  retval;

   va_start( ap, count );
   retval = wss_db_mcol_def_va( count, ap );
   va_end( ap );

   return retval;
}

wqval_t EXPORT
wr_mvals( int count, ... )
{
   va_list ap;
   wqval_t retval;

   va_start( ap, count );
   retval = wss_db_mvals_va( count, ap );
   va_end( ap );

   return retval;
}

wqrow_t EXPORT
wr_mrow( wqval_t vals, wqrow_t next )
{
   return wss_db_mrow( vals, next );
}

wqsx_t EXPORT
wr_mwqsx_and( wqsx_t left, wqsx_t right )
{
   return wss_db_mwqsx_and( left, right );
}

wqsx_t EXPORT
wr_mwqsx_or( wqsx_t left, wqsx_t right )
{
   return wss_db_mwqsx_or( left, right );
}

wqsx_t EXPORT
wr_mwqsx_terminal( tx_t terminal )
{
   return wss_db_mwqsx_terminal( terminal );
}

wqsx_t EXPORT
wr_mwqsx_not( wqsx_t expression )
{
   return wss_db_mwqsx_not( expression );
}

tx_t EXPORT
wr_mtx( sxval_t left, sxval_t right, enum sxop sxop )
{
   return wss_db_mtx( left, right, sxop );
}

sxval_t EXPORT
wr_msxval_field( int column )
{
   return wss_db_msxval_field( column );
}

sxval_t EXPORT
wr_msxval_lit_int( int i )
{
   return wss_db_msxval_lit_int( i );
}

sxval_t EXPORT
wr_msxval_lit_float( double d )
{
   return wss_db_msxval_lit_float( d );
}

sxval_t EXPORT
wr_msxval_lit_string( const pd_char_t *str )
{
   return wss_db_msxval_lit_string( str );
}

wqusl_t EXPORT
wr_musl( sxval_t left, wquexp_t right, wqusl_t next )
{
   return wss_db_musl( left, right, next );
}

wquexp_t EXPORT
wr_muexp( wquexp_t left, wquexp_t right, enum numop numop )
{
   return wss_db_muexp( left, right, numop );
}

wquexp_t EXPORT
wr_muexp_term( sxval_t value )
{
   return wss_db_muexp_term( value );
}

void EXPORT
dbrow_finish( struct wql_dbrow *dbrp )
{
   wss_db_dbrow_finish( dbrp );
}

void EXPORT
wr_mutex_init( pd_mutex_t *mtx, const pd_char_t *name )
{
   wss_mutex_init( mtx, (pd_char_t *)name );
}

void EXPORT
wr_mutex_destroy( pd_mutex_t *mtx )
{
   wss_mutex_destroy( mtx );
}

void EXPORT
wr_mutex_enter( pd_mutex_t *mtx )
{
   wss_mutex_enter( mtx );
}

void EXPORT
wr_mutex_exit( pd_mutex_t *mtx )
{
   wss_mutex_exit( mtx );
}

long EXPORT
wr_get_curtime( long *usec )
{
  return wsi_time_get_current( usec );
}

void EXPORT
wr_bcopy( const void *src, void *dst, size_t len )
{
   bcopy( (void *)src, dst, len );
}

void EXPORT
wr_bzero( void *src, size_t len )
{
   bzero( src, len );
}

int EXPORT
wr_sprintf( pd_char_t *s, const pd_char_t *fmt, ... )
{
   va_list ap;

   va_start( ap, fmt );
   int retval = _vstprintf( s, fmt, ap );
   va_end( ap );

   return retval;
}

void EXPORT
wr_audit( unsigned long types, const void * buffer, unsigned long length )
{
   WSS_MUTEX_ENTER( g_pHandlers );
   DB_HANDLERS dbh;
   dbh.AuditPost= (g_pHandlers?g_pHandlers->AuditPost:NULL);
   WSS_MUTEX_EXIT( g_pHandlers );

   if( dbh.AuditPost )
      (*dbh.AuditPost)( types, buffer, length );

   return;
}

int EXPORT
wr_audit_printf( unsigned long types, const pd_char_t *fmt, ... )
{
   va_list ap;
   pd_char_t *buf = (pd_char_t *)wss_malloc(MAXBUFLEN * sizeof(pd_char_t));

   va_start( ap, fmt );
   int retval = _vsntprintf( buf, MAXBUFLEN, fmt, ap );
   va_end( ap );

   if( retval > 0 )
      wr_audit( types, buf, retval );

   wss_free( buf );

   return retval;
}

int EXPORT
wr_copyout( void *kaddr, void *uaddr, size_t len )
{
   return copyout( kaddr, uaddr, len );
}

void EXPORT *
wr_dupin( void *uaddr, size_t size )
{
   return wss_arg_dupin( uaddr, size );
}

int EXPORT 
wr_curthreadid(void)
{
   return wsi_os_curthreadid();
}

int EXPORT 
wr_resolve_event_space(pd_char_t *name)
{
   return wss_evspace_resolve( name );
}

int EXPORT
wr_log( const pd_char_t *format, ... )
{
   va_list ap;
  
   va_start( ap, format );
   int retval =  wss_log_printf( format, ap );
   va_end( ap );
  
   return retval;
}

extern "C" int IMPORT _wss_dispatch_internal __P(( int space, int code,
                                                   wss_proc_t *wrp,
                                                   int flags,
                                                   int arg_count,
                                                   void *uap[] ));

int EXPORT
wr_dispatch( int space, int code, wss_proc_t *wrp,
             int flags, int arg_count, ... )
{
   va_list  ap;
   void    *uap[10];
   int      i;
   int      retval;

   if( !wrp )
      return (1*4|WR_DENY);

   bzero( uap, sizeof(uap) );

   va_start( ap, arg_count );

   if( arg_count > ( sizeof(uap) / sizeof(uap[0]) ) )
       arg_count = ( sizeof(uap) / sizeof(uap[0]) );

   for( i = 0; i < arg_count ; i++ )
      uap[i] = va_arg( ap, void* );
   va_end( ap );

   retval = _wss_dispatch_internal( space, code, wrp, flags, arg_count, uap );

   wss_arg_donesyscall( wrp, wsi_os_curthreadid(), space, code, uap );

   return retval;
}

void EXPORT
wr_file_close( int id, wss_file_t file )
{
   wss_mod_close_file( id, file );
}

size_t EXPORT
wr_file_get_size( int id, wss_file_t file )
{
   return wss_mod_get_file_size( id, file );
}

void EXPORT *
wr_file_get_map_addr( int id, wss_file_t handle )
{
   return wss_mod_get_file_map_addr( id, handle );
}

wss_file_t
EXPORT wr_file_open( int id, const pd_char_t *filename, wss_file_mode_t mode )
{
   return wss_mod_open_file( id, filename, mode );
}

int EXPORT
wr_file_mmap( int id, wss_file_t file, size_t s, int mapmode, pd_offset_t pos )
{
   return wss_mod_map_file( id, file, s, mapmode, pos );
}

int EXPORT
wr_file_read( int id, wss_file_t file, pd_byte_t *buffer, size_t len )
{
   return wss_mod_read_file( id, file, buffer, len );
}

int EXPORT
wr_file_seek( int id, wss_file_t file, pd_offset_t offset, int whence )
{
   return wss_mod_seek_file( id, file, offset, whence );
}

pd_byte_t EXPORT *
wr_file_read_alloc( int id, wss_file_t file, size_t *len )
{
   return wss_mod_read_file_alloc( id, file, len );
}

int EXPORT
wr_file_write( int id, wss_file_t file, pd_byte_t *buffer, size_t len )
{
   return wss_mod_write_file( id, file, buffer, len );
}

