/*! \file wss_file.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_file.h,v 1.13 2001/02/23 19:02:47 stevek Exp $
 *
 * \brief   WSS file operation functions
 * \author  Steve Kiernan
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:47 $
 */

#ifndef __WSS_FILE_H
#define __WSS_FILE_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_FILE_H
#include <wsi_file.h>
#endif


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define WSS_FILE_READ     0x01  /*!< Read mode flag                         */
#define WSS_FILE_WRITE    0x02  /*!< Write mode flag                        */
#define WSS_FILE_APPEND   0x04  /*!< Append mode flag                       */
#define WSS_FILE_CREAT    0x08  /*!< Create file if non-existant mode flag */
#define WSS_FILE_TRUNC    0x10  /*!< Truncate mode flag                     */
#define WSS_FILE_ROOTPRIV 0x20  /*!< Open with root privledged mode flag    */

#define WSS_FILE_SEEK_SET 0     /*!< Set current offset flag                */
#define WSS_FILE_SEEK_CUR 1     /*!< Seek offset from current position flag */
#define WSS_FILE_SEEK_END 2     /*!< Seek offset from end of file flag      */

#define WSS_FILE_MAP_READ  0x01 /*!< Map file readable flag                 */
#define WSS_FILE_MAP_WRITE 0x02 /*!< Map file writable flag                 */
#define WSS_FILE_MAP_ALL   ( WSS_FILE_MAP_WRITE | WSS_FILE_MAP_READ )
                                /*!< Map file readable and writable flag    */


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

#ifndef WR_SUPPRESS_FUNCTION_DECL
void       WSSLINKAGE  wss_file_close       ( wss_file_t handle );
void       WSSLINKAGE *wss_file_get_map_addr( wss_file_t handle );
size_t     WSSLINKAGE  wss_file_get_size    ( wss_file_t handle );
int        WSSLINKAGE  wss_file_is_mapped   ( wss_file_t handle );
int        WSSLINKAGE  wss_file_map         ( wss_file_t handle,
                                              size_t size, int mapmode,
                                              pd_offset_t pos );
wss_file_t WSSLINKAGE  wss_file_open        ( const pd_char_t *filename,
                                              wss_file_mode_t mode );
int        WSSLINKAGE  wss_file_read        ( wss_file_t handle,
                                              pd_byte_t *buffer,
                                              size_t len );
pd_byte_t WSSLINKAGE *wss_file_read_alloc  ( wss_file_t handle,
                                              size_t *len );
void       WSSLINKAGE  wss_file_ref         ( wss_file_t handle );
void       WSSLINKAGE  wss_file_rele        ( wss_file_t handle );
int        WSSLINKAGE  wss_file_seek        ( wss_file_t handle,
                                              pd_offset_t offset,
                                              int whence );
void       WSSLINKAGE  wss_file_unmap       ( wss_file_t handle );
int        WSSLINKAGE  wss_file_write       ( wss_file_t handle,
                                              pd_byte_t *buffer,
                                              size_t len );
#endif


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#define wss_file_close        wsi_file_close
/*!< \brief Close previously-opened file
 *   \sa wsi_file_close */

#define wss_file_get_map_addr wsi_file_get_map_addr
/*!< \brief Get memory-mapped address of mapped file
 *   \sa wsi_file_get_map_addr */

#define wss_file_is_mapped    wsi_file_is_mapped
/*!< \brief 
 *   \sa wsi_file_is_mapped */

#define wss_file_map          wsi_file_map
/*!< \brief Map opened file into memory space
 *   \sa wsi_file_map */

#define wss_file_open         wsi_file_open
/*!< \brief Open or create a file in the filesystem
 *   \sa wsi_file_open */

#define wss_file_read         wsi_file_read
/*!< \brief Read bytes from opened file into specified buffer
 *   \sa wsi_file_read */

#define wss_file_ref          wsi_file_ref
/*!< \brief Acquire reference to file
 *   \sa wsi_file_ref */

#define wss_file_rele         wsi_file_rele
/*!< \brief Release reference to file
 *   \sa wsi_file_rele */

#define wss_file_seek         wsi_file_seek
/*!< \brief Repositions the file offset pointer
 *   \sa wsi_file_seek */

#define wss_file_unmap        wsi_file_unmap
/*!< \brief Unmaps a file previously mapped into the WSS memory space
 *   \sa wsi_file_unmap */

#define wss_file_write        wsi_file_write
/*!< \brief Write bytes from the specified buffer to opened file
 *   \sa wsi_file_write */

#endif /* !__WSS_FILE_H */

