/*! \file wss_iface.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_iface.h,v 1.18 2001/02/23 19:02:47 stevek Exp $
 *
 * \brief   Types and macros for managing iterfacing wrappers at runtime
 * \author  Chris Vance and Lee Badger
 * \version $Revision: 1.18 $
 * \date    $Date: 2001/02/23 19:02:47 $
 *
 * \sa wss_exported.c
 */

#ifndef  __WSS_IFACE_H
#define __WSS_IFACE_H   1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

/*
 * Return codes for event interception.
 */
#define WR_ALLOW   0           /*!< Allow call to continue return code    */
#define WR_SUCCESS 0           /*!< Successful completion return code     */
#define WR_DENY    1           /*!< Deny call from continuing return code */
#define WR_NONE    2           /*!< No action return code                 */
#define WR_MASK1   0x3         /*!< Mask for return code flags            */
#define WR_MASK2   0xfffffffc  /*!< Mask for return value part of code    */


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_symbol wss_symbol_t;
typedef int               wss_id_t;

/*! Structure for exported symbols */
struct wss_symbol
{
   pd_char_t *name;  /*!< Symbol name             */
   u_long     addr;  /*!< Symbol resolved address */
   int        count; /*!< Usage count             */
};


/*
 * Types for generated wrappers.
 * 
 * A string is a string of course.  Char* representation.
 * A databuf_t is a structure that holds data.  buf+len+max size
 * 
 * But the actual buffer first so that "char *foo = buf" type code
 * won't trash anything important
 */
#ifdef WE_REALLY_WANT_THE_STRING_TYPEDEF
/*! String type */
typedef char *string;
#endif

/*! Default wrapper buffer size */
#define WR_BUFSIZE      128

#ifndef WR_IN_WRAPC
/*! Data buffer structure */
struct wr_databuf
{
   char *buf;  /*!< Pointer to buffer */
   int   max;  /*!< Maximum size      */
   int   size; /*!< Current size      */
};

typedef struct wr_databuf *databuf_t;
#endif

#ifdef __cplusplus
}
#endif

#endif /* __WSS_IFACE_H */

