/*! \file wss_init.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_init.h,v 1.14 2001/02/23 19:02:47 stevek Exp $
 *
 * \brief   Prototypes and declarations for handling configuration file
 *          keyword entries and initialization
 * \author  Steve Kiernan
 * \version $Revision: 1.14 $
 * \date    $Date: 2001/02/23 19:02:47 $
 *
 * \sa wss_init.c
 */

#ifndef __WSS_INIT_H
#define __WSS_INIT_H 1

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wsi_init.h>

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************\
 * Public structures and types                                          *
\************************************************************************/

typedef enum wss_init_type    wss_init_type_t;
typedef struct wss_init_entry wss_init_entry_t;

/*! \brief Initialization variable types */
enum wss_init_type
{
   WSS_INIT_TYPE_UNKNOWN,     /*!< Unknown initialization type */
   WSS_INIT_TYPE_STRING,      /*!< String type                 */
   WSS_INIT_TYPE_INT,         /*!< Integer type                */
   WSS_INIT_TYPE_BOOL         /*!< Boolean value type          */
};

struct wss_init_entry
{
   const pd_char_t *key;        /*!< Non-Unicode key name string */
   wss_init_type_t   type;      /*!< Type of storage             */
   void             *storage;   /*!< Pointer to storage          */
};


/************************************************************************\
 * Public macros                                                        *
\************************************************************************/

/*! \brief Macro for creating the initialization variable table
 *
 *  \param n Key name string
 *  \param t Type of storage
 *  \param s Pointer to storage
 */
#define WSS_INIT_ENTRY(n,t,s) { PD_MAKESTR(n), (t), (void *)(s) }

#define WSS_INIT_DB      0x0001 /*!< Flag for wss_init to indicate that the
                                     database code should be initialized    */
#define WSS_INIT_NO_DB   0x0000 /*!< Flag for wss_init to indicate that the
                                     database code should \n NOT be
                                     initialized                            */
#define WSS_INIT_CONF    0x0002 /*!< Flag for wss_init to indicate that the
                                     configuration data should be loaded    */
#define WSS_INIT_NO_CONF 0x0000 /*!< Flag for wss_init to indicate that the
                                     configuration data should \b NOT be
                                     loaded                                 */


/************************************************************************\
 * Public globals                                                       *
\************************************************************************/

extern wss_init_entry_t wss_init_globals[];
extern wss_init_entry_t wsi_init_locals[];


/************************************************************************\
 * Public functions                                                     *
\************************************************************************/

int  WSSLINKAGE wss_init          ( int );
void WSSLINKAGE wss_init_read_conf( void );
int  WSSLINKAGE wss_init_fill_db  ( u_long );


/************************************************************************\
 * Functions implemented as macros                                      *
\************************************************************************/

#define wss_init_read_conf() wsi_init_read_conf()

#ifdef __cplusplus
}
#endif

#endif /* !__WSS_INIT_H */

