/*! \file wss_linker.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_linker.h,v 1.10 2001/06/13 13:27:13 stevek Exp $
 *
 * \brief   Prototypes and structures used by the module linker
 * \author  Steve Kiernan
 * \version $Revision: 1.10 $
 * \date    $Date: 2001/06/13 13:27:13 $
 */

#ifndef __WSS_LINKER_H
#define __WSS_LINKER_H	1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#ifndef __WSS_IFACE_H
#include <wss_iface.h>
#endif

#ifndef __WSS_REF_H
#include <wss_ref.h>
#endif

#ifndef __WSS_FILE_H
#include <wss_file.h>
#endif

#ifndef __WSS_MUTEX_H
#include <wss_mutex.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_linker             wss_linker_t;
typedef union  wss_linker_addr        wss_linker_addr_t;
typedef struct wss_linker_file_depend wss_linker_file_depend_t;
typedef struct wss_linker_file        wss_linker_file_t;
typedef struct wss_linker_file_ops    wss_linker_file_ops_t;
typedef struct wss_linker_ops         wss_linker_ops_t;
typedef wss_linker_addr_t             wss_linker_sym_t;

union wss_linker_addr
{
   u_int32_t  addr32;
   u_int64_t  addr64;
   void      *addr;
};

struct wss_linker_file_depend
{
   wss_linker_file_t        *linker_file_depend_entry;
   wss_linker_file_depend_t *linker_file_depend_next;
};

struct wss_linker_file
{
   wss_ref_t                 linker_file_refcount;
   const pd_char_t          *linker_file_name;
   const pd_char_t          *linker_file_path;
   wss_linker_addr_t         linker_file_image;
   wss_linker_addr_t         linker_file_addr;
   size_t                    linker_file_size;
   wss_linker_file_ops_t    *linker_file_ops;
   void                     *linker_file_data;
   wss_symbol_t             *linker_file_symbols;
   const pd_char_t          *linker_file_dependencies;
   wss_linker_file_depend_t *linker_file_dependency_list;
   wss_linker_file_t        *linker_file_next;
   WSS_MUTEX_PROTECT        (linker_file);
};

struct wss_linker_file_ops
{
   int (*lookup_symbol)           __P(( wss_linker_file_t *, const pd_char_t *,
                                        wss_linker_addr_t * ));
   int (*export_symbols)          __P(( wss_linker_file_t * ));
   int (*finalize)                __P(( wss_linker_file_t * ));
   int (*unload)                  __P(( wss_linker_file_t * ));
};

struct wss_linker
{
   const pd_char_t  *linker_description;
   wss_linker_ops_t *linker_ops;
   wss_linker_t     *linker_next;
};

struct wss_linker_ops
{
   int (*load_file) __P(( const pd_char_t *, wss_linker_file_t * ));
};


/***************************************************************************\
 * Public macros                                                           *
\***************************************************************************/

#define WRLINK_MOD_NAME               __wr_myname
#define WRLINKNAME_MOD                "__wr_myname"
#define WRLINK_MOD_DEPENDENCIES       __wr_mydependencies
#define WRLINKNAME_MOD_DEPENDENCIES   "__wr_mydependencies"
#define WRLINK_MOD_INTERCEPT_LIST     __wr_myintercepts
#define WRLINKNAME_MOD_INTERCEPT_LIST "__wr_myintercepts"


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int  WSSLINKAGE wss_linker_add_symbols            __P(( const pd_char_t *,
                                                        wss_symbol_t * ));
int  WSSLINKAGE wss_linker_construct_dependencies __P(( wss_linker_file_t * ));
void WSSLINKAGE wss_linker_init                   __P(( void ));
int  WSSLINKAGE wss_linker_load                   __P(( const pd_char_t *,
                                                        wss_linker_file_t * ));
int  WSSLINKAGE wss_linker_lookup_multiple        __P(( wss_linker_file_t *,
                                                        int, ... ));
int  WSSLINKAGE wss_linker_register               __P(( const pd_char_t *,
                                                        wss_linker_ops_t * ));
int  WSSLINKAGE wss_linker_search                 __P(( wss_linker_file_t *,
                                                        const pd_char_t *,
                                                        wss_linker_addr_t * ));
int  WSSLINKAGE wss_linker_search_multiple        __P(( wss_linker_file_t *,
                                                        int, ... ));
void WSSLINKAGE wss_linker_file_init              __P(( wss_linker_file_t * ));
int  WSSLINKAGE wss_linker_file_destroy           __P(( wss_linker_file_t * ));
int  WSSLINKAGE wss_linker_file_has_dependency    __P(( wss_linker_file_t *,
                                                        wss_linker_file_t * ));
void WSSLINKAGE wss_linker_file_ref               __P(( wss_linker_file_t * ));
void WSSLINKAGE wss_linker_file_rele              __P(( wss_linker_file_t * ));
int  WSSLINKAGE wss_linker_remove_file            __P(( wss_linker_file_t * ));
void WSSLINKAGE wss_linker_shutdown               __P(( void ));


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#if defined(DEBUG_REFS) && !defined(DISABLE_LINKER_DEBUG_REFS)

#  define wss_linker_file_ref(lf) \
   do { \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) < 1) \
         panic("wss_linker_file_ref: reference count for 0x%x was %d < 1" \
               " [%s:%d]", (unsigned int)(lf), \
               WSS_REF_COUNT((lf)->linker_file_refcount), __FILE__, \
               __LINE__ ); \
      WSS_MUTEX_ENTER((lf)->linker_file); \
      wsi_printf( PD_MAKESTR( \
                  "wss_linker_file_ref: '%s' (0x%x) refcount = %d [%s:%d]\n"), \
                  (lf)->linker_file_name, (unsigned int)(lf), \
                  WSS_REF_INC((lf)->linker_file_refcount), __FILE__, \
                  __LINE__ ); \
      WSS_MUTEX_EXIT((lf)->linker_file); \
   } while(0)

#  define wss_linker_file_rele(lf) \
   do { \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) < 1) \
         panic("wss_linker_file_rele: reference count for 0x%x was %d < 1" \
               " [%s:%d]", (unsigned int)(lf), \
               WSS_REF_COUNT((lf)->linker_file_refcount), __FILE__, \
               __LINE__ ); \
      WSS_MUTEX_ENTER((lf)->linker_file); \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) > 1) { \
         wsi_printf( PD_MAKESTR("wss_linker_file_rele: '%s' (0x%x) ") \
                     PD_MAKESTR("refcount = %d [%s:%d]\n"), \
                     (lf)->linker_file_name, (unsigned int)(lf), \
                     WSS_REF_DEC((lf)->linker_file_refcount), __FILE__, \
                     __LINE__ ); \
         WSS_MUTEX_EXIT((lf)->linker_file); \
      } else { \
         wsi_printf( PD_MAKESTR( \
                    "wss_linker_file_rele: freeing '%s' (0x%x) [%s:%d]\n"), \
                     (lf)->linker_file_name, (unsigned int)(lf), \
                     __FILE__, __LINE__ ); \
         ((lf)->linker_file_ops->unload)(lf); \
      } \
   } while(0)

#else /* DEBUG_REFS */

/*  wss_linker_file_ref */
/*! \brief Obtain reference to linker file structure
 *
 *  \param lf Pointer to linker file structure
 */
#  define wss_linker_file_ref(lf) \
   do { \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) < 1) \
         panic("wss_linker_file_ref: reference count for 0x%x was %d < 1", \
               (unsigned int)(lf), \
               WSS_REF_COUNT((lf)->linker_file_refcount)); \
      WSS_MUTEX_ENTER((lf)->linker_file); \
      WSS_REF_INC((lf)->linker_file_refcount); \
      WSS_MUTEX_EXIT((lf)->linker_file); \
   } while(0)

/*  wss_linker_file_rele */
/*! \brief Release reference to linker file structure
 *
 *  \param lf Pointer to linker file structure
 */

#  define wss_linker_file_rele(lf) \
   do { \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) < 1) \
         panic("wss_linker_file_rele: reference count for 0x%x was %d < 1", \
               (unsigned int)(lf), \
               WSS_REF_COUNT((lf)->linker_file_refcount)); \
      WSS_MUTEX_ENTER((lf)->linker_file); \
      if(WSS_REF_COUNT((lf)->linker_file_refcount) > 1) { \
         WSS_REF_DEC((lf)->linker_file_refcount); \
         WSS_MUTEX_EXIT((lf)->linker_file); \
      } else { \
         ((lf)->linker_file_ops->unload)(lf); \
      } \
   } while(0)

#endif /* DEBUG_REFS */


#ifdef __cplusplus
}
#endif

#endif /* __WSS_LINKER_H */

