/*! \file wss_malloc.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_malloc.h,v 1.20 2001/02/23 19:02:47 stevek Exp $
 * 
 * \brief   Declaration of WSS memory manager services
 * \author  Doug Kilpatrick
 * \version $Revision: 1.20 $
 * \date    $Date: 2001/02/23 19:02:47 $
 *
 * \sa wss_malloc.c
 */

#ifndef __WSS_MALLOC_H
#define __WSS_MALLOC_H  1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

#ifdef MALLOC_DEBUG
void       WSSLINKAGE *wss_malloc_debug                 ( size_t, pd_char_t *,
                                                          int );
pd_char_t WSSLINKAGE *wss_strdup_debug                 ( const pd_char_t *,
                                                          pd_char_t *, int );
void       WSSLINKAGE wss_free_debug                    ( void * );
pd_char_t WSSLINKAGE *wss_malloc_string_allocator_debug( size_t, pd_char_t *,
                                                          int );
#else
pd_char_t WSSLINKAGE *wss_strdup                       ( const pd_char_t * );
pd_char_t WSSLINKAGE *wss_malloc_string_allocator      ( size_t );
#endif /* MALLOC_DEBUG */

void       WSSLINKAGE  wss_malloc_init                  ( void );
void       WSSLINKAGE  wss_malloc_shutdown              ( void );


/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#ifdef MALLOC_DEBUG
#define wss_malloc(size)        wss_malloc_debug((size),PD_MAKESTR(__FILE__), __LINE__)
#define wss_malloc_string(size) wss_malloc_string_allocator_debug((size),PD_MAKESTR(__FILE__), __LINE__)
#define wss_strdup(str)         wss_strdup_debug((str),PD_MAKESTR(__FILE__), __LINE__)
#define wss_free(ptr)           wss_free_debug((ptr))
#define wss_malloc_string_allocator(size) wss_malloc_string_allocator_debug((size),PD_MAKESTR(__FILE__),__LINE__)

#else

/*********
  Can't do this, since we need to have the address
  of wss_malloc, wss_strdup, and wss_free for
  the exported symbols table

#define wss_malloc(size) wsi_malloc(size)
#define wss_strdup(str)  wsi_strdup(str)
#define wss_free(ptr)    wsi_free(ptr)

  We must do the following (ugly, but necessary)
**********/
#define wss_malloc        wsi_malloc
#define wss_malloc_string wss_malloc_string_allocator
#define wss_free          wsi_free

#endif /* MALLOC_DEBUG */

#ifdef __cplusplus
}
#endif

/* Include platform specific parts. */
#include <wsi_malloc.h>

#endif /* __WSS_MALLOC_H */

