/*! \file wss_mutex.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_mutex.h,v 1.16 2001/02/23 19:02:47 stevek Exp $ 
 * 
 * \brief   Declaration of mutex functions for the generated wrappers
 * \author  Douglas Kilpatrick
 * \version $Revision: 1.16 $
 * \date    $Date: 2001/02/23 19:02:47 $
 *
 * \sa wss_mutex.c
 */

#ifndef __WSS_MUTEX_H
#define __WSS_MUTEX_H   1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

/* Include platform specific parts. */
#ifndef __PI_MUTEX_H
#include <pdpi/pi_mutex.h>
#endif


/* Define our mutex macros to set proper PD mutex linkage */
#define WSS_MUTEX_PROTECT(a)          PD_MUTEX_PROTECT(WSSLINKAGE,a)
#define WSS_MUTEX_PROTECT(a)          PD_MUTEX_PROTECT(WSSLINKAGE,a)
#define WSS_MUTEX_PROTECT_EXPORTED(a) PD_MUTEX_PROTECT_EXPORTED(WSSLINKAGE,a)
#define WSS_MUTEX_EXPORT(a)           PD_MUTEX_EXPORT(WSSLINKAGE,a)
#define WSS_MUTEX_POINTER(a)          PD_MUTEX_POINTER(WSSLINKAGE,a)
#define WSS_MUTEX_ADDR(a)             PD_MUTEX_ADDR(WSSLINKAGE,a)
#define WSS_MUTEX_INIT(a)             PD_MUTEX_INIT(WSSLINKAGE,a)
#define WSS_MUTEX_DESTROY(a)          PD_MUTEX_DESTROY(WSSLINKAGE,a)
#define WSS_MUTEX_ENTER(a)            PD_MUTEX_ENTER(WSSLINKAGE,a)
#define WSS_MUTEX_ENTER_DEBUG(a)      PD_MUTEX_ENTER_DEBUG(WSSLINKAGE,a)
#define WSS_MUTEX_EXIT(a)             PD_MUTEX_EXIT(WSSLINKAGE,a)
#define WSS_MUTEX_EXIT_DEBUG(a)       PD_MUTEX_EXIT_DEBUG(WSSLINKAGE,a)
#define WSS_MUTEX_INIT_NOMNG(a,name)  PD_MUTEX_INIT_NOMNG(WSSLINKAGE,a,name)
#define WSS_MUTEX_DESTROY_NOMNG(a)    PD_MUTEX_DESTROY_NOMNG(WSSLINKAGE,a)
#define WSS_MUTEX_ENTER_NOMNG(a)      PD_MUTEX_ENTER_NOMNG(WSSLINKAGE,a)
#define WSS_MUTEX_EXIT_NOMNG(a)       PD_MUTEX_EXIT_NOMNG(WSSLINKAGE,a)


#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

void WSSLINKAGE wss_mutex_init   ( pd_mutex_t *, pd_char_t * );
void WSSLINKAGE wss_mutex_destroy( pd_mutex_t * );
void WSSLINKAGE wss_mutex_enter  ( pd_mutex_t * );
void WSSLINKAGE wss_mutex_exit   ( pd_mutex_t * );

#ifdef __cplusplus
}
#endif

#endif /* !__WSS_MUTEX_H */

