/*! \file wss_utilities.h
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_utilities.h,v 1.10 2001/02/23 19:02:47 stevek Exp $
 *
 * \brief   Collection of tools usefull in muliple parts of the wrappers
 *          system
 * \author  Douglas Kilpatrick
 * \version $Revision: 1.10 $
 * \date    $Date: 2001/02/23 19:02:47 $
 * 
 * Currently just contains red/black trees.  These are used by the Solaris 
 * thread_id -> arg replacement structure and the malloc_debug code.
 *
 * \sa wss_utilites.c
 */

#ifndef __WSS_UTILITIES_H
#define __WSS_UTILITIES_H

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#ifndef __WSI_UTILITIES_H
#include <wsi_utilities.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
int        WSSLINKAGE  wss_atoi     __P(( const pd_char_t * ));
double     WSSLINKAGE  wss_atof     __P(( const pd_char_t * ));
pd_char_t WSSLINKAGE *wss_upcase   __P(( const pd_char_t * ));
pd_char_t WSSLINKAGE *wss_downcase __P(( const pd_char_t * ));
#ifdef __cplusplus
}
#endif

#ifndef WR_SUPPRESS_RBTREE

#define WSS_UTIL_LEFT            1
#define WSS_UTIL_RIGHT           0

/* Inherit from this: or at least make your start the same. */

typedef struct wss_util_rbnode {
    struct wss_util_rbnode *link[2];
    int red;
} wss_util_rbnode_t;

typedef int      (*wss_util_CompFunc) (void *node1, void *node2);
/* Clone func takes NULL to mean create a new one */
/* Clone should call wss_util_init_node on the new node before returning */
typedef void *(*wss_util_CloneFunc)(void *node);
typedef void (*wss_util_FreeFunc)(void *node);
typedef int      (*wss_util_DoFunc)   (void *node, int level);

/* Head is a senteniel.  Does not have any data.  Only child is to the
 * right
 */
typedef struct wss_util_rbtree {
    wss_util_rbnode_t *Head;
    wss_util_CompFunc  Compare;
    wss_util_CloneFunc Clone;
    wss_util_FreeFunc  Free;
} wss_util_rbtree_t;

wss_util_rbtree_t *wss_util_rbtree_new     (wss_util_CompFunc  cf,
                                            wss_util_CloneFunc clone,
                                            wss_util_FreeFunc free);
void               wss_util_rbtree_free    (wss_util_rbtree_t *tree);
wss_util_rbnode_t *wss_util_rbtree_find    (wss_util_rbtree_t *tree,
                                            wss_util_rbnode_t *node);
int                wss_util_rbtree_insert  (wss_util_rbtree_t *tree,
                                            wss_util_rbnode_t *node);
wss_util_rbnode_t *wss_util_rbtree_delete  (wss_util_rbtree_t *tree,
                                            wss_util_rbnode_t *node);
int                wss_util_rbtree_walk    (wss_util_rbtree_t *tree,
                                            wss_util_DoFunc df);

wss_util_rbnode_t *wss_util_rbnode_init    (wss_util_rbnode_t *node);
int                wss_util_rbnode_isempty (wss_util_rbnode_t *node);

#define wss_util_rbnode_isempty(node) (node->link[WSS_UTIL_LEFT] == 0)

#endif /* WR_SUPPRESS_RBTREE */

#endif /* __WSS_UTILITIES_H */
