/*
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_wrvevent.h,v 1.29 2001/02/23 19:02:47 stevek Exp $
 *
 * Author: Steve Kiernan
 *
 * Header file for the alert event component
 */

#ifndef __WSS_WRVEVENT_H
#define __WSS_WRVEVENT_H 1

#ifndef WR_DEFS_INCLUDED
#include <wr_conf.h>
#endif

#ifndef __WSI_TYPES_H
#include <wsi_types.h>
#endif

#include <libwrv.h>

#include <wss_process.h>
#include <wss_db.h>
#include <wss_module.h>

#include <eventfs.h>

#ifdef __cplusplus
extern "C" {
#endif


/***************************************************************************\
 * Public structures and types                                             *
\***************************************************************************/

typedef struct wss_wrv_event wss_wrv_event_t;
typedef wss_wrv_event_t *    wss_wrv_event_queue_t;
typedef void               (*wss_wrv_event_hook_t)( const pd_char_t *, u_long,
                                                    size_t, const void *);

/* Make sure the "wrv_event" object is LAST in the structure.  This
 * is because the wrv_event is a variable length object.
 */
struct wss_wrv_event
{
   struct eventfs_event        event;
};


/***************************************************************************\
 * Privately exported globals                                              *
\***************************************************************************/

u_long WSSLINKAGE WREV_WRAPPER_LOADED;
u_long WSSLINKAGE WREV_ACTCRIT_LOADED;
u_long WSSLINKAGE WREV_DEACTCRIT_LOADED;
u_long WSSLINKAGE WREV_LIBRARY_LOADED;
u_long WSSLINKAGE WREV_WRAPPER_UNLOADED;
u_long WSSLINKAGE WREV_ACTCRIT_UNLOADED;
u_long WSSLINKAGE WREV_DEACTCRIT_UNLOADED;
u_long WSSLINKAGE WREV_LIBRARY_UNLOADED;



/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int             WSSLINKAGE  wss_wrvevent_init             __P(( void ));
wss_wrv_event_t WSSLINKAGE *wss_wrvevent_next             __P(( wss_wrv_event_t *event,
                                                                wss_wrv_event_queue_t *q ));
void            WSSLINKAGE
        wss_wrvevent_post             ( const pd_char_t *group, u_long types,
                                            size_t length, const void *buffer);
void            WSSLINKAGE  wss_wrvevent_post_createtable ( wqtab_t table );
void            WSSLINKAGE  wss_wrvevent_post_deleterow   ( wqtab_t table,
                                                                wqrow_t row );
void            WSSLINKAGE  wss_wrvevent_post_droptable   ( wqtab_t table );
void            WSSLINKAGE  wss_wrvevent_post_insertrow   ( wqtab_t table,
                                                                wqrow_t row );
void            WSSLINKAGE  wss_wrvevent_post_modloaded   ( wss_db_handle_t handle,
                                                                u_long type );
void            WSSLINKAGE  wss_wrvevent_post_modunloaded ( wss_db_handle_t handle,
                                                                u_long type );
void            WSSLINKAGE  wss_wrvevent_post_updaterow   ( wqtab_t table,
                                                                wqrow_t row );
void            WSSLINKAGE  wss_wrvevent_register_hook    ( wss_wrv_event_hook_t hook );
void            WSSLINKAGE  wss_wrvevent_shutdown         ( void );
int             WSSLINKAGE  wss_wrvevent_unregister_hook  ( wss_wrv_event_hook_t hook );

/***************************************************************************\
 * Functions implemented as macros                                         *
\***************************************************************************/

#define _WSS_WRVEVENT_POST_1(evtype, arg1, type1)                       \
do { type1 foo = (arg1);                                                \
    wss_wrvevent_post( WREV_EFS_GROUP, (evtype), sizeof(type1), &foo);  \
} while (0)

#define _WSS_WRVEVENT_POST_2(evtype, arg1, type1, arg2, type2)          \
do { struct { type1 o; type2 t; } s;                                    \
    s.o = (arg1); s.t = (arg2);                                         \
    wss_wrvevent_post( WREV_EFS_GROUP, evtype, sizeof(s), &s); } while (0)


#define wss_wrvevent_post_createtable(table)                            \
    _WSS_WRVEVENT_POST_1(WREV_CREATETAB,                                \
            (wss_db_handle_t)abs2rel(wss_db_space,table),               \
            wss_db_handle_t)

#define wss_wrvevent_post_droptable(table)                              \
    _WSS_WRVEVENT_POST_1(WREV_DROPTAB,                                  \
            (wss_db_handle_t)abs2rel(wss_db_space,table),               \
            wss_db_handle_t)

#define wss_wrvevent_post_insertrow(table,row)                          \
    _WSS_WRVEVENT_POST_2(WREV_UPDATETAB_INSERTROW,                      \
            (wss_db_handle_t)abs2rel(wss_db_space,table),               \
            wss_db_handle_t,                                            \
            (u_long)abs2rel(wss_db_space,row), u_long)

#define wss_wrvevent_post_deleterow(table,row)                          \
    _WSS_WRVEVENT_POST_2(WREV_UPDATETAB_DELETEROW,                      \
            (wss_db_handle_t)abs2rel(wss_db_space,table),               \
            wss_db_handle_t,                                            \
            (u_long)abs2rel(wss_db_space,row), u_long)

#define wss_wrvevent_post_updaterow(table,row)                          \
    _WSS_WRVEVENT_POST_2(WREV_UPDATETAB_UPDATEROW,                      \
            (wss_db_handle_t)abs2rel(wss_db_space,table),               \
            wss_db_handle_t,                                            \
            (u_long)abs2rel(wss_db_space,row), u_long)

#define wss_wrvevent_post_modloaded(handle,type)                        \
    do { u_long __types = 0;						\
         switch(type) {							\
            case WR_MOD_WRAPPER: __types=WREV_WRAPPER_LOADED;break;	\
            case WR_MOD_ACTIVATION: __types=WREV_ACTCRIT_LOADED;break;	\
            case WR_MOD_DEACTIVATION: __types=WREV_DEACTCRIT_LOADED;break; \
            case WR_MOD_LIBRARY: __types=WREV_LIBRARY_LOADED;break; }	\
         _WSS_WRVEVENT_POST_1(__types, handle, wss_id_t);		\
    } while(0)

#define wss_wrvevent_post_modunloaded(handle,type)                      \
    do { u_long __types = 0;						\
         switch(type) {							\
            case WR_MOD_WRAPPER: __types=WREV_WRAPPER_UNLOADED;break;	\
            case WR_MOD_ACTIVATION: __types=WREV_ACTCRIT_UNLOADED;break; \
            case WR_MOD_DEACTIVATION: __types=WREV_DEACTCRIT_UNLOADED;break; \
            case WR_MOD_LIBRARY: __types=WREV_LIBRARY_UNLOADED;break; }	\
         _WSS_WRVEVENT_POST_1(__types, handle, wss_id_t);		\
    } while(0)

#ifdef __cplusplus
}
#endif

#endif /* !__WSS_WRVEVENT_H */

