/*! \file wss_audit.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_audit.c,v 1.21 2001/02/23 19:02:46 stevek Exp $
 *
 * \brief   Message logging facilities for auditing.
 * \author  Douglas Kilpatrick
 * \version $Revision: 1.21 $
 * \date    $Date: 2001/02/23 19:02:46 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <eventfs.h>

#include <wss_audit.h>
#include <wss_malloc.h>

#ifndef WR_WINNT
#warning TBD: redo interface to allow more flexibility in types/group
#endif

/***************************************************************************\
 * Private globals                                                         *
\***************************************************************************/

static volatile int     _wss_audit_initialized = FALSE;

/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

u_long WREV_AUDIT_0             = 0;
u_long WREV_AUDIT_1             = 0;
u_long WREV_AUDIT_2             = 0;
u_long WREV_AUDIT_3             = 0;
u_long WREV_AUDIT_4             = 0;
u_long WREV_AUDIT_5             = 0;
u_long WREV_AUDIT_6             = 0;
u_long WREV_AUDIT_7             = 0;
u_long WREV_AUDIT_8             = 0;
u_long WREV_AUDIT_9             = 0;
u_long WREV_AUDIT_A             = 0;
u_long WREV_AUDIT_B             = 0;
u_long WREV_AUDIT_C             = 0;
u_long WREV_AUDIT_D             = 0;
u_long WREV_AUDIT_E             = 0;
u_long WREV_AUDIT_F             = 0;
u_long WREV_AUDIT_10            = 0;
u_long WREV_AUDIT_11            = 0;
u_long WREV_AUDIT_12            = 0;
u_long WREV_AUDIT_13            = 0;
u_long WREV_AUDIT_14            = 0;
u_long WREV_AUDIT_15            = 0;
u_long WREV_AUDIT_16            = 0;
u_long WREV_AUDIT_17            = 0;
u_long WREV_AUDIT_18            = 0;
u_long WREV_AUDIT_19            = 0;
u_long WREV_AUDIT_1A            = 0;
u_long WREV_AUDIT_1B            = 0;
u_long WREV_AUDIT_1C            = 0;
u_long WREV_AUDIT_1D            = 0;
u_long WREV_AUDIT_1E            = 0;
u_long WREV_AUDIT_1F            = 0;

/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wss_audit_init  */
/*! \brief Initialize the Wrapper Audit Event system
 *
 *  \return 0 on success
 *  \return error code on failure
 */
#define REGISTER_ET(num) \
    EFSPURENAME(eventfs_register_event_type)( WREV_AUDIT_GROUP, \
            PD_MAKESTR("audit stream ") PD_MAKESTR( #num ), \
            & WREV_AUDIT_ ## num)

int EXPORT wss_audit_init(void)
{
   if( !_wss_audit_initialized )
   {
      EFSPURENAME(eventfs_register_group)( WREV_AUDIT_GROUP );
      REGISTER_ET(0);
      REGISTER_ET(1);
      REGISTER_ET(2);
      REGISTER_ET(3);
      REGISTER_ET(4);
      REGISTER_ET(5);
      REGISTER_ET(6);
      REGISTER_ET(7);
      REGISTER_ET(8);
      REGISTER_ET(9);
      REGISTER_ET(A);
      REGISTER_ET(B);
      REGISTER_ET(C);
      REGISTER_ET(D);
      REGISTER_ET(E);
      REGISTER_ET(F);
      REGISTER_ET(10);
      REGISTER_ET(11);
      REGISTER_ET(12);
      REGISTER_ET(13);
      REGISTER_ET(14);
      REGISTER_ET(15);
      REGISTER_ET(16);
      REGISTER_ET(17);
      REGISTER_ET(18);
      REGISTER_ET(19);
      REGISTER_ET(1A);
      REGISTER_ET(1B);
      REGISTER_ET(1C);
      REGISTER_ET(1D);
      REGISTER_ET(1E);
      REGISTER_ET(1F);

      EFSPURENAME(eventfs_register_stream)(
              WREV_AUDIT_GROUP,         /* "audit"                      */
              WREV_AUDIT_RAW_STREAM,    /* "raw"                        */
              ~0,                       /* all 32 bits set in mask      */
              FALSE);                   /* Don't nuke on close          */
      _wss_audit_initialized = TRUE;
   }
   return 0;
}

/*  wss_audit_shutdown  */
/*! \brief Shutdown the Wrapper Audit Event system
 *
 *  \return none
 */
#define REMOVE_ET(num) \
    EFSPURENAME(eventfs_register_event_type)( WREV_AUDIT_GROUP, \
            PD_MAKESTR("audit stream ") PD_MAKESTR( #num ), \
            & WREV_AUDIT_ ## num)

void EXPORT wss_audit_shutdown(void)
{
   if( _wss_audit_initialized )
   {
      _wss_audit_initialized = FALSE;
      EFSPURENAME(eventfs_destroy_stream)(
              WREV_AUDIT_GROUP,
              WREV_AUDIT_RAW_STREAM);

      EFSPURENAME(eventfs_destroy_group)( WREV_AUDIT_GROUP );
   }
}
/*  wss_audit_write */
/*! \brief Posts an event to the specified log.
 *
 * Needs to be a real function so it can be exported to wrappers.
 *
 *  \param logid      Identifier of audit log stream.  Bitmask.
 *  \param buf        Pointer to audit data.
 *  \param length     Length of audit data to log.
 *
 *  \return 0 on success,
 *
 *  \sa wss_wrvevent_post
 */
void WSSLINKAGE wss_audit_write( u_long types, const void *buf, size_t length )
{
    wss_wrvevent_post(WREV_AUDIT_GROUP,types,length,buf);
}
/*  wss_audit_printf */
/*! \brief printf for audit log streams.
 *
 *  Similar to printf, only output goes directly to the audit event queue
 *
 *  \param logid      Identifier of audit log stream.
 *  \param identifier Identifier of audit event.
 *  \param format     printf-style format string.
 *
 *  \return Length of audit log message.
 */
int EXPORT wss_audit_printf( u_long types, const pd_char_t *format, ... )
{
   va_list ap;
   int len;

   static pd_char_t writebuf[2048];
   /* Fixed length buffer.  Bad */
   /* No SMP locking/TLV.   Bad */

   va_start(ap, format);
   len = wsi_vsprintf(writebuf, format, ap);
   va_end(ap);
   if (len > sizeof(writebuf)/sizeof(writebuf[0]))
      panic( "buffer overflow." );
   wss_audit_write(types, writebuf, len);
   return(len);
}

