/*! \file wss_export.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_export.c,v 1.19 2001/06/23 00:07:41 stevek Exp $
 *
 * \brief   Contains the table used to export functions to wrappers
 * \author  Chris Vance, Mark Feldman, Steve Kiernan
 * \version $Revision: 1.19 $
 * \date    $Date: 2001/06/23 00:07:41 $
 */

/* Turn off macros that would hide functions. */

#define SUPPRESS_MACROS 1       


#include <wr_conf.h>

#include <wsi_stdlib.h>

#ifndef SUPPRESS_GLOBAL_EXPORT_TABLE
#include <wss_arg.h>
#include <wss_audit.h>
#include <wss_db.h>
#include <wss_iface.h>
#include <wss_log.h>
#include <wss_malloc.h>
#include <wss_module.h>
#include <wss_mutex.h>
#include <wss_strings.h>
#include <wss_file.h>
#include <wss_evspace.h>
#include <wss_dispatch.h>
#include <wss_utilities.h>
#endif /* !SUPPRESS_GLOBAL_EXPORT_TABLE */
#include <wss_linker.h>

#ifndef SUPPRESS_GLOBAL_EXPORT_TABLE
#include <wsi_iface.h>
#include <wsi_syscalls.h>
#include <wsi_time.h>
#include <wsi_os.h>
#include <wsi_path.h>
#ifndef USER_MODE_TEST
#include <wsi_fileutil.h>
#endif /* USER_MODE_TEST */
#endif /* !SUPPRESS_GLOBAL_EXPORT_TABLE */


/***************************************************************************\
 * External functions                                                      *
\***************************************************************************/

#ifdef PD_BZERO_IS_FN_PTR
void wsi_bzero( void *, size_t );
#endif


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

#ifndef SUPPRESS_GLOBAL_EXPORT_TABLE
/*!
 * Initialize the table of exported functions.  In order for functions
 * to be exported, its prototype must be available (through include
 * files, etc).
 *
 * Fill this out with everything that is going to be required by wrappers.
 */
wss_symbol_t wss_export_table[] =
{
   WSS_EXPORT_ENTRY("wr_log",                  (u_long)wss_log_printf),
   WSS_EXPORT_ENTRY("wr_audit",                (u_long)wss_audit_write),
   WSS_EXPORT_ENTRY("wr_audit_printf",         (u_long)wss_audit_printf),
   WSS_EXPORT_ENTRY("wr_printf",               (u_long)wsi_printf),
   WSS_EXPORT_ENTRY("wr_strcpy",               (u_long)wsi_strcpy),
   WSS_EXPORT_ENTRY("wr_strncpy",              (u_long)wsi_strncpy),
   WSS_EXPORT_ENTRY("wr_strcat",               (u_long)wsi_strcat),
   WSS_EXPORT_ENTRY("wr_atoi",                 (u_long)wss_atoi),
   WSS_EXPORT_ENTRY("wr_upcase",               (u_long)wss_upcase),
   WSS_EXPORT_ENTRY("wr_downcase",             (u_long)wss_downcase),
   WSS_EXPORT_ENTRY("wr_wrapper_loaded",       (u_long)wss_wrapper_loaded),
   WSS_EXPORT_ENTRY("wr_add_new_wrapper",      (u_long)wss_wrapper_add_new),
   WSS_EXPORT_ENTRY("wr_dispatch",             (u_long)wss_dispatch_unsafe),
   WSS_EXPORT_ENTRY("wr_strcmp",               (u_long)wsi_strcmp),
   WSS_EXPORT_ENTRY("wr_strncmp",              (u_long)wsi_strncmp),
   WSS_EXPORT_ENTRY("wr_strlen",               (u_long)wsi_strlen),
#if defined(MALLOC_DEBUG)
   WSS_EXPORT_ENTRY("wr_strdup_debug",         (u_long)wss_strdup_debug),
#else
   WSS_EXPORT_ENTRY("wr_strdup",               (u_long)wss_strdup),
#endif
   WSS_EXPORT_ENTRY("wr_strchr",               (u_long)wss_strchr),
   WSS_EXPORT_ENTRY("wr_strrchr",              (u_long)wss_strrchr),
#if defined(MALLOC_DEBUG)
   WSS_EXPORT_ENTRY("wr_malloc_debug",         (u_long)wss_malloc_debug),
#else
   WSS_EXPORT_ENTRY("wr_malloc",               (u_long)wss_malloc),
#endif
#if defined(MALLOC_DEBUG)
   WSS_EXPORT_ENTRY("wr_free",                 (u_long)wss_free_debug),
#else
   WSS_EXPORT_ENTRY("wr_free",                 (u_long)wss_free),
#endif
   WSS_EXPORT_ENTRY("wr_abspath",              (u_long)wsi_path_abspath),
   WSS_EXPORT_ENTRY("wr_proc_add_wrapper",     (u_long)wss_proc_add_wrapper_blind),
   WSS_EXPORT_ENTRY("wr_proc_del_wrapper",     (u_long)wss_proc_del_wrapper),
   WSS_EXPORT_ENTRY("wr_proc_get_path",        (u_long)wss_proc_get_path),
   WSS_EXPORT_ENTRY("wr_proc_get_cwd",         (u_long)wss_proc_get_cwd),
   WSS_EXPORT_ENTRY("wr_curproc_malloc",       (u_long)wsi_curproc_malloc),
   WSS_EXPORT_ENTRY("wr_curproc_free",         (u_long)wsi_curproc_free),
   WSS_EXPORT_ENTRY("wr_curproc_strdup",       (u_long)wsi_curproc_strdup),
   WSS_EXPORT_ENTRY("wr_arg_replace",          (u_long)wss_arg_replace),
   WSS_EXPORT_ENTRY("wr_arg_copy",             (u_long)wss_arg_copy),
   WSS_EXPORT_ENTRY("wr_get_curtime",          (u_long)wsi_time_get_current),
   WSS_EXPORT_ENTRY("wr_get_osname",           (u_long)wsi_os_get_name),
   WSS_EXPORT_ENTRY("wr_get_osvers",           (u_long)wsi_os_get_version),
   WSS_EXPORT_ENTRY("wr_get_hostname",         (u_long)wsi_os_get_hostname),
   WSS_EXPORT_ENTRY("wr_curthreadid",          (u_long)wsi_os_curthreadid),
#if 0
   WSS_EXPORT_ENTRY("wr_get_wrvers",           (u_long)wss_get_wrvers),
#endif
   WSS_EXPORT_ENTRY("wql_free_criteria",       (u_long)wss_db_free_criteria),
   WSS_EXPORT_ENTRY("wql_free_update_expr",    (u_long)wss_db_free_wqusl),
   WSS_EXPORT_ENTRY("wql_create",              (u_long)wss_create),
   WSS_EXPORT_ENTRY("wql_drop",                (u_long)wss_drop),
   WSS_EXPORT_ENTRY("wql_insert",              (u_long)wss_insert),
   WSS_EXPORT_ENTRY("wql_select",              (u_long)wss_select),
   WSS_EXPORT_ENTRY("wql_delete",              (u_long)wss_delete),
   WSS_EXPORT_ENTRY("wql_update",              (u_long)wss_update),
   WSS_EXPORT_ENTRY("wql_find_table_by_names", (u_long)wss_db_find_table_by_names),
   WSS_EXPORT_ENTRY("wr_mcol_def",             (u_long)wss_db_mcol_def),
   WSS_EXPORT_ENTRY("wr_mrow",                 (u_long)wss_db_mrow),
   WSS_EXPORT_ENTRY("wr_mwqsx_and",            (u_long)wss_db_mwqsx_and),
   WSS_EXPORT_ENTRY("wr_mwqsx_or",             (u_long)wss_db_mwqsx_or),
   WSS_EXPORT_ENTRY("wr_mwqsx_terminal",       (u_long)wss_db_mwqsx_terminal),
   WSS_EXPORT_ENTRY("wr_mwqsx_not",            (u_long)wss_db_mwqsx_not),
   WSS_EXPORT_ENTRY("wr_mtx",                  (u_long)wss_db_mtx),
   WSS_EXPORT_ENTRY("wr_msxval_field",         (u_long)wss_db_msxval_field),
   WSS_EXPORT_ENTRY("wr_msxval_lit_int",       (u_long)wss_db_msxval_lit_int),
   WSS_EXPORT_ENTRY("wr_msxval_lit_float",     (u_long)wss_db_msxval_lit_float),
   WSS_EXPORT_ENTRY("wr_msxval_lit_string",    (u_long)wss_db_msxval_lit_string),
   WSS_EXPORT_ENTRY("dbrow_finish",            (u_long)wss_db_dbrow_finish),
   WSS_EXPORT_ENTRY("wr_musl",                 (u_long)wss_db_musl),
   WSS_EXPORT_ENTRY("wr_muexp",                (u_long)wss_db_muexp),
   WSS_EXPORT_ENTRY("wr_muexp_term",           (u_long)wss_db_muexp_term),
   WSS_EXPORT_ENTRY("wr_mvals",                (u_long)wss_db_mvals),
   WSS_EXPORT_ENTRY("wr_bcopy",                (u_long)bcopy),
#ifdef PD_BZERO_IS_FN_PTR
   WSS_EXPORT_ENTRY("wr_bzero",                (u_long)wsi_bzero),
#else
   WSS_EXPORT_ENTRY("wr_bzero",                (u_long)bzero),
#endif
   WSS_EXPORT_ENTRY("wr_syscall_name",         (u_long)wsi_syscall_name),
   WSS_EXPORT_ENTRY("wr_match",                (u_long)wss_match),
   WSS_EXPORT_ENTRY("wr_substitute",           (u_long)wss_substitute),
   WSS_EXPORT_ENTRY("wr_copyin",               (u_long)copyin),
   WSS_EXPORT_ENTRY("wr_copyout",              (u_long)copyout),
   WSS_EXPORT_ENTRY("wr_copyinstr",            (u_long)copyinstr),
   WSS_EXPORT_ENTRY("wr_dupin",                (u_long)wss_arg_dupin),
   WSS_EXPORT_ENTRY("wr_dupin_ntermlist",      (u_long)wss_arg_dupin_ntermlist),
   WSS_EXPORT_ENTRY("wr_free_ntermlist",       (u_long)wss_arg_free_ntermlist),
   WSS_EXPORT_ENTRY("wr_replace_ntermlist",  (u_long)wss_arg_replace_ntermlist),
   WSS_EXPORT_ENTRY("wr_dupout",               (u_long)wss_arg_dupout),
   WSS_EXPORT_ENTRY("wr_mutex_init",           (u_long)wss_mutex_init),
   WSS_EXPORT_ENTRY("wr_mutex_destroy",        (u_long)wss_mutex_destroy),
   WSS_EXPORT_ENTRY("wr_mutex_enter",          (u_long)wss_mutex_enter),
   WSS_EXPORT_ENTRY("wr_mutex_exit",           (u_long)wss_mutex_exit),
   WSS_EXPORT_ENTRY("wr_sprintf",              (u_long)wsi_sprintf),
#ifndef USER_MODE_TEST
   WSS_EXPORT_ENTRY("wr_get_family",           (u_long)wsi_netutil_get_family),
   WSS_EXPORT_ENTRY("wr_get_addr",             (u_long)wsi_netutil_get_addr),
   WSS_EXPORT_ENTRY("wr_ntohs",                (u_long)wsi_netutil_ntohs),
   WSS_EXPORT_ENTRY("wr_stat",                 (u_long)wsi_fileutil_stat),
   WSS_EXPORT_ENTRY("wr_lstat",                (u_long)wsi_fileutil_lstat),
   WSS_EXPORT_ENTRY("wr_fstat",                (u_long)wsi_fileutil_fstat),
   WSS_EXPORT_ENTRY("wr_fdtype",               (u_long)wsi_fileutil_fdtype),
   WSS_EXPORT_ENTRY("wr_fdfflag",              (u_long)wsi_fileutil_fdfflag),
   WSS_EXPORT_ENTRY("wr_getsockname",          (u_long)wsi_netutil_getsockname),
   WSS_EXPORT_ENTRY("wr_getpeername",          (u_long)wsi_netutil_getpeername),

   WSS_EXPORT_ENTRY("wr_sleep",                (u_long)wsi_util_sleep),
   WSS_EXPORT_ENTRY("wr_wakeup",               (u_long)wsi_util_wakeup),
#endif /* !USER_MODE_TEST */
   
   WSS_EXPORT_ENTRY("wr_resolve_event_space",  (u_long)wss_evspace_resolve),

   WSS_EXPORT_ENTRY("wr_file_close",           (u_long)wss_mod_close_file),
   WSS_EXPORT_ENTRY("wr_file_mmap",            (u_long)wss_mod_map_file),
   WSS_EXPORT_ENTRY("wr_file_mmap_addr",       (u_long)
                                                  wss_mod_get_file_map_addr),
   WSS_EXPORT_ENTRY("wr_file_open",            (u_long)wss_mod_open_file),
   WSS_EXPORT_ENTRY("wr_file_read",            (u_long)wss_mod_read_file),
   WSS_EXPORT_ENTRY("wr_file_read_alloc",      (u_long)wss_mod_read_file_alloc),
   WSS_EXPORT_ENTRY("wr_file_seek",            (u_long)wss_mod_seek_file),
   WSS_EXPORT_ENTRY("wr_file_size",            (u_long)wss_mod_get_file_size),
   WSS_EXPORT_ENTRY("wr_file_write",           (u_long)wss_mod_write_file),
   { NULL, 0, 0 }
};
#endif /* !SUPPRESS_GLOBAL_EXPORT_TABLE */


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

int EXPORT
wss_export_lookup( wss_linker_file_t *lf, const pd_char_t *name,
                   wss_linker_addr_t *addr )
{
   wss_symbol_t *sym;

   if( ( lf == NULL ) || ( lf->linker_file_symbols == NULL ) ||
       ( addr == NULL ) )
      return EINVAL;

   for( sym = lf->linker_file_symbols; sym && sym->name; sym++ )
      if( !wsi_strcmp( name, sym->name ) )
      {
         addr->addr = (void *)sym->addr;
         return 0;
      }

   return ENOENT;
}

int EXPORT
wss_export_noop( wss_linker_file_t *lf )
{
   return 0;
}

