/*! \file wss_file.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_file.c,v 1.13 2001/02/23 19:02:46 stevek Exp $
 *
 * \brief   WSS-level file operations
 * \author  Steve Kiernan
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:46 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_file.h>
#include <wss_malloc.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wss_file_read_alloc */
/*! \brief Read specified number of bytes from file into allocated storage
 *
 *  Specified number of bytes are read from an already-open file, storage
 *  buffer space is allocated via wss_malloc(), and the buffer and read
 *  length are returned.
 *
 *  \param file Handle of already-open file
 *  \param len  Pointer to a size_t variable which contains the buffer length
 *
 *  \return Pointer to the allocated storage buffer on success,
 *  \return NULL on failure (due to invalid parameters or allocation
 *          failure)
 */
pd_byte_t EXPORT *
wss_file_read_alloc( wss_file_t file, size_t *len )
{
   pd_byte_t *buffer;
   int        toRead;
   int        nRead;
   
   if( len == NULL ) return NULL;

   toRead = *len;

   buffer = (pd_byte_t *)wss_malloc( toRead );
   if( buffer == NULL ) return NULL;

   nRead = wss_file_read( file, buffer, toRead );
   *len = nRead;

   return buffer;
}

/*  wss_file_get_size */
/*! \brief Get size of open file
 *
 *  \param handle Handle of already-open file
 *
 *  \return Size of file in bytes on success,
 *  \return 0 on failure
 */
size_t EXPORT
wss_file_get_size( wss_file_t handle )
{
    return wsi_file_get_size( handle, NULL );
}

