/*! \file wss_mutex.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_mutex.c,v 1.13 2001/02/23 19:02:46 stevek Exp $
 * 
 * \brief   Holds functions for locking/unlocking mutexes.
 * \author  Doug Kilpatrick
 * \version $Revision: 1.13 $
 * \date    $Date: 2001/02/23 19:02:46 $
 *
 * LKM code will generally use the WSI macros instead of these functions.
 * The functions contained within this file exist to support generated
 * wrappers.
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>
#include <wss_mutex.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wss_mutex_init */
/*! \brief Initialize a mutex
 *
 *  \param mtx  Pointer to mutex to initialize
 *  \param name Name of mutex
 */
void EXPORT
wss_mutex_init( pd_mutex_t *mtx, pd_char_t *name )
{
#ifdef PD_THREADED
    bzero( mtx, sizeof(*mtx) );
#endif
    WSS_MUTEX_INIT_NOMNG(*mtx,name);
}


/*  wss_mutex_destroy */
/*! \brief Destroy a mutex
 *
 *  \param mtx Pointer to mutex to destroy
 */
void EXPORT
wss_mutex_destroy( pd_mutex_t *mtx )
{
    WSS_MUTEX_DESTROY_NOMNG(*mtx);
}


/*  wss_mutex_enter */
/*! \brief Acquire a mutex
 *
 *  \param mtx Pointer to the mutex to acquire
 */
void EXPORT
wss_mutex_enter( pd_mutex_t *mtx )
{
    WSS_MUTEX_ENTER_NOMNG(*mtx);
}

/*  wss_mutex_exit */
/*! \brief Release hold on a mutex
 *
 *  \param mtx Pointer to the mutex to release
 */
void EXPORT
wss_mutex_exit( pd_mutex_t *mtx )
{
    WSS_MUTEX_EXIT_NOMNG(*mtx);
}

