/*! \file wss_path.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_path.c,v 1.15 2001/06/21 17:06:04 stevek Exp $
 *
 * \brief   Functions used for system-indepedent path manipulations.
 * \author  Steve Kiernan
 * \version $Revision: 1.15 $
 * \date    $Date: 2001/06/21 17:06:04 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_path.h>


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wss_path_rel_to_abspath */
/*! \brief Converts the given relative path to absolute terms based on the
 *         given current working directory
 *
 *  \param cwd Pointer to string containing the current working directory
 *  \param rel Pointer to string containing the path to convert
 *
 *  \return Pointer to string representing the absolute path on success,
 *  \return NULL on failure
 */
pd_char_t EXPORT *
wss_path_rel_to_abspath( pd_char_t const *cwd, pd_char_t const *rel )
{
   pd_char_t *path;
   pd_char_t *abspath;
   int        cwdlen = (cwd==NULL)?0:wsi_strlen( cwd );
   int        rellen = (rel==NULL)?0:wsi_strlen( rel );

#ifdef DEBUG_PATH
   wsi_printf( PD_MAKESTR("wss_path:: cwd = %s, rel = %s\n"), cwd, rel );
#endif

   if( ( cwd == NULL ) && ( rel == NULL ) )
      return NULL;

   if( rel == NULL )
   {
#ifdef DEBUG_PATH
      wsi_printf( PD_MAKESTR("wss_path:: Calling wss_path_abspath( \"%s\" )\n"), cwd );
#endif /* DEBUG_PATH */

      abspath = wss_strdup( cwd );
   }
   else if( ( cwd == NULL ) || ( *rel == WSS_FILE_SEPARATOR ) )
   {
#ifdef DEBUG_PATH
      wsi_printf( PD_MAKESTR("wss_path:: Calling wss_path_abspath( \"%s\" )\n"), rel );
#endif /* DEBUG_PATH */

      abspath = wss_path_abspath( (pd_char_t *)rel );
   }
   else
   {
      path = wss_malloc( ( cwdlen + rellen + 2 ) * sizeof(pd_char_t) );
      memcpy( path, cwd, cwdlen * sizeof(pd_char_t) );
      if( cwd[cwdlen-1] != WSS_FILE_SEPARATOR )
         path[cwdlen++] = WSS_FILE_SEPARATOR;
      memcpy( path + cwdlen, rel, ( rellen + 1 ) * sizeof(pd_char_t) );

#ifdef DEBUG_PATH
      wsi_printf( PD_MAKESTR("wss_path:: Calling wss_path_abspath( \"%s\" )\n"), path );
#endif /* DEBUG_PATH */

      abspath = wss_path_abspath( path );

      wss_free( path );
   }

#ifdef DEBUG_PATH
      wsi_printf( PD_MAKESTR("wss_path:: Returned \"%s\"\n"), abspath );
#endif /* DEBUG_PATH */

   return abspath;
}

