/*! \file wss_ref.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_ref.c,v 1.8 2001/02/23 19:02:46 stevek Exp $
 *
 * \brief   Reference tracking for debugging
 * \author  Steve Kiernan
 * \version $Revision: 1.8 $
 * \date    $Date: 2001/02/23 19:02:46 $
 */

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_ref.h>

#include <eventfs.h>

#ifdef DEBUG_REFS
int EXPORT
wss_ref_debug_inc( wss_ref_t *r, const pd_char_t *file, int line )
{
   int        val;
   wss_ref_t *ent = (wss_ref_t *)wsi_malloc( sizeof(*ent) );

   ent->un.st.ref_file = file;
   ent->un.st.ref_line = line;
   ent->ref_next = r->ref_next;

   val = r->un.ref_count++;
   r->ref_next = ent;

   return val;
}

int EXPORT
wss_ref_debug_dec( wss_ref_t *r, const pd_char_t *file, int line )
{
   wss_ref_t  *ent;

   ent = r->ref_next;
   if( ent )
   {
      r->ref_next = ent->ref_next;
      wsi_free( ent );
      return( --(r->un.ref_count) );
   }

   if( r->un.ref_count <= 0 )
      panic( "wss_ref_debug_dec: called before wss_ref_debug_inc (%s:%d)\n",
             file, line );
   return( --(r->un.ref_count) );
}
#endif /* DEBUG_REFS */

void EXPORT
wss_ref_init( wss_ref_t *r )
{
   bzero( r, sizeof(*r) );
   r->un.ref_count = 1;
}

void EXPORT
wss_ref_destroy( wss_ref_t *r )
{
   if( ( r->un.ref_count > 0 )
#ifdef DEBUG_REFS
       || ( r->ref_next != NULL )
#endif /* DEBUG_REFS */
     )
      panic( "wss_ref_destroy: called on referenced entity" );

   r->un.ref_count = -1;
}

