/*! \file wss_wrvevent.c
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wss_wrvevent.c,v 1.39 2001/02/23 19:02:46 stevek Exp $
 * 
 * \brief   Wrapper viewer event implementation.
 * \author  Steve Kiernan
 * \version $Revision: 1.39 $
 * \date    $Date: 2001/02/23 19:02:46 $
 */

#define LINKAGE

#include <wr_conf.h>

#include <wsi_stdlib.h>

#include <wss_malloc.h>
#include <wss_wrv.h>
#include <wss_wrvevent.h>

#include <wss_debug.h>

#include <eventfs.h>

#ifdef MUTEXLINKAGE
#undef MUTEXLINKAGE
#endif
#define MUTEXLINKAGE EXPORT


/***************************************************************************\
 * Private structures and types                                            *
\***************************************************************************/

struct wss_wrv_event_hook
{
   LIST_ENTRY(wss_wrv_event_hook) h_list;
   wss_wrv_event_hook_t           hook;
};

PD_LIST_HEAD(wss_wrv_event_hook_list, wss_wrv_event_hook);


/***************************************************************************\
 * Private globals                                                         *
\***************************************************************************/

static volatile int                   _wss_wrvevent_initialized = FALSE;
static struct wss_wrv_event_hook_list wss_wrv_event_hooks;
WSS_MUTEX_PROTECT                    (wss_wrv_event_hooks);


/***************************************************************************\
 * Public globals                                                          *
\***************************************************************************/

u_long EXPORT WREV_CREATETAB           = 0;
u_long EXPORT WREV_DROPTAB             = 0;
u_long EXPORT WREV_UPDATETAB_INSERTROW = 0;
u_long EXPORT WREV_UPDATETAB_DELETEROW = 0;
u_long EXPORT WREV_UPDATETAB_UPDATEROW = 0;
u_long EXPORT WREV_WRAPPER_LOADED      = 0;
u_long EXPORT WREV_ACTCRIT_LOADED      = 0;
u_long EXPORT WREV_DEACTCRIT_LOADED    = 0;
u_long EXPORT WREV_LIBRARY_LOADED      = 0;
u_long EXPORT WREV_WRAPPER_UNLOADED    = 0;
u_long EXPORT WREV_ACTCRIT_UNLOADED    = 0;
u_long EXPORT WREV_DEACTCRIT_UNLOADED  = 0;
u_long EXPORT WREV_LIBRARY_UNLOADED    = 0;

/* Warning, the variables used by these defines are not set until
 * startup.  so don't use these defines until after startup.
 */
#define WREV_DB_EVENTS  ( WREV_CREATETAB | WREV_DROPTAB | \
                          WREV_UPDATETAB_INSERTROW | \
                          WREV_UPDATETAB_DELETEROW | \
                          WREV_UPDATETAB_UPDATEROW )
#define WREV_MOD_EVENTS ( WREV_WRAPPER_LOADED | WREV_WRAPPER_UNLOADED | \
			  WREV_ACTCRIT_LOADED | WREV_ACTCRIT_UNLOADED | \
			  WREV_DEACTCRIT_LOADED | WREV_DEACTCRIT_UNLOADED | \
			  WREV_LIBRARY_LOADED | WREV_LIBRARY_UNLOADED )
#define WREV_RAW_EVENTS ( WREV_DB_EVENTS | WREV_MOD_EVENTS )


/***************************************************************************\
 * Public functions                                                        *
\***************************************************************************/

/*  wss_wrvevent_init */
/*! \brief Initialize the Wrapper Viewer Event system
 *
 *  \return 0 on success,
 *  \return error code on failure
 */
int EXPORT
wss_wrvevent_init()
{
   if( !_wss_wrvevent_initialized )
   {
      _wss_wrvevent_initialized = TRUE;

      wss_debug_log( PD_MAKESTR("Registering event streams for WSS\n") );

      EFSPURENAME(eventfs_register_group)( WREV_EFS_GROUP );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_CREATETAB,
                                                &WREV_CREATETAB );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_DROPTAB,
                                                &WREV_DROPTAB );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_UPDATETAB_INSERTROW,
                                                &WREV_UPDATETAB_INSERTROW );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_UPDATETAB_DELETEROW,
                                                &WREV_UPDATETAB_DELETEROW );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_UPDATETAB_UPDATEROW,
                                                &WREV_UPDATETAB_UPDATEROW );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_WRAPPER_LOADED,
                                                &WREV_WRAPPER_LOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_ACTCRIT_LOADED,
                                                &WREV_ACTCRIT_LOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_DEACTCRIT_LOADED,
                                                &WREV_DEACTCRIT_LOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_LIBRARY_LOADED,
                                                &WREV_LIBRARY_LOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_WRAPPER_UNLOADED,
                                                &WREV_WRAPPER_UNLOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_ACTCRIT_UNLOADED,
                                                &WREV_ACTCRIT_UNLOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_DEACTCRIT_UNLOADED,
                                                &WREV_DEACTCRIT_UNLOADED );
      EFSPURENAME(eventfs_register_event_type)( WREV_EFS_GROUP,
                                                WREV_ET_LIBRARY_UNLOADED,
                                                &WREV_LIBRARY_UNLOADED );

      EFSPURENAME(eventfs_register_stream)( WREV_EFS_GROUP, WREV_EFS_RAW_STREAM,
                                        WREV_RAW_EVENTS, FALSE );
      EFSPURENAME(eventfs_register_stream)( WREV_EFS_GROUP, WREV_EFS_DB_STREAM,
                                        WREV_DB_EVENTS, FALSE );
      EFSPURENAME(eventfs_register_stream)( WREV_EFS_GROUP, WREV_EFS_MODULE_STREAM,
                                        WREV_MOD_EVENTS, FALSE );

      WSS_MUTEX_INIT( wss_wrv_event_hooks );
      LIST_INIT( &wss_wrv_event_hooks );
   }
   return 0;
}

/*  wss_wrvevent_shutdown */
/*! \brief Shutdown the Wrapper Viewer Event system
 */
void EXPORT
wss_wrvevent_shutdown()
{
   if( _wss_wrvevent_initialized )
   {
      _wss_wrvevent_initialized = FALSE;

      EFSPURENAME(eventfs_destroy_group)( WREV_EFS_GROUP );

      WSS_MUTEX_DESTROY( wss_wrv_event_hooks );
   }
}

/*  wss_wrvevent_register_hook */
/*! \brief Registers raw event hook function
 *
 *  \param newhook Pointer to hook function
 */
void EXPORT
wss_wrvevent_register_hook( wss_wrv_event_hook_t newhook )
{
   struct wss_wrv_event_hook *h;

   WSS_MUTEX_ENTER( wss_wrv_event_hooks );
   for( h = wss_wrv_event_hooks.lh_first; h; h = h->h_list.le_next )
      if( h->hook == newhook )
         break;

   if( !h )
   {
      h = (struct wss_wrv_event_hook *)wss_malloc( sizeof( *h ) );
      h->hook = newhook;
      LIST_INSERT_HEAD( &wss_wrv_event_hooks, h, h_list );
   }
   WSS_MUTEX_EXIT( wss_wrv_event_hooks );
}

/*  wss_wrvevent_unregister_hook */
/*! \brief Removes function from the list of raw event hooks
 *
 *  \param hook Pointer to hook function
 *
 *  \return TRUE if the function was found and removed,
 *  \return FALSE otherwise
 */
int EXPORT
wss_wrvevent_unregister_hook( wss_wrv_event_hook_t hook )
{
   struct wss_wrv_event_hook *h;
   int                        retval = FALSE;

   WSS_MUTEX_ENTER( wss_wrv_event_hooks );
   for( h = wss_wrv_event_hooks.lh_first; h; h = h->h_list.le_next )
      if( h->hook == hook )
      {
         LIST_REMOVE( h, h_list );
         wss_free( h );
         retval = TRUE;
      }

   WSS_MUTEX_EXIT( wss_wrv_event_hooks );

   return retval;
}

/*  wss_wrvevent_post */
/*! \brief Post a wrapper viewer event
 *
 *  Event, based on the signature, key, and event-specific (extra) data,
 *  is posted to all registered viewers.
 * 
 *  \param group     Group which will receive the event
 *  \param types     Mask of appropriate types for the event
 *  \param length    Length of data packet
 *  \param buffer    Pointer to data packet
 *
 */
void EXPORT
wss_wrvevent_post( const pd_char_t *group, u_long types,
                    size_t length, const void *buffer)
{
   struct wss_wrv_event_hook *h;

   /*
    * Call all raw event hook functions
    */
   WSS_MUTEX_ENTER( wss_wrv_event_hooks );
   for( h = wss_wrv_event_hooks.lh_first; h; h = h->h_list.le_next )
      (*h->hook)( group, types, length, buffer);
   WSS_MUTEX_EXIT( wss_wrv_event_hooks );

   EFSPURENAME(eventfs_post_event_by_types)( group, types, length, buffer);
}

