package CPU::x86::Opcodes;
# 
# Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
# All rights reserved.
#
# This file is part of the NAI Generic Software Wrappers Toolkit
# (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
# distribution.
#
# The GSWTK is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# The GSWTK is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# $Id: Opcodes.pm,v 1.12 2001/04/05 18:41:14 stevek Exp $

use strict;

BEGIN {
   use Exporter;
   use vars qw($VERSION @ISA @EXPORT_OK %EXPORT_TAGS);

   $VERSION = 0.001;
   @EXPORT_OK = qw( @x86_opcodes %x86_index_registers %x86_registers
                    $x86_segments $x86_opcode_functions %x86_base_registers
                    %x86_test_mnemonics );
   %EXPORT_TAGS = ( FIELDS => [ @EXPORT_OK ] );
}

use vars @EXPORT_OK;

sub import { goto &Exporter::import }

@x86_opcodes = (
   [ "AAD", "Imm8", 1,
     "11111111", "11010101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "AAD", "", 2,
     "1111111111111111", "1101010100001010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "AAM", "Imm8", 1,
     "11111111", "11010100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "AAM", "", 2,
     "1111111111111111", "1101010000001010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "AAS", "", 1,
     "11111111", "00111111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "ADC", "Mem,Reg", 2,
     "1111111000000000", "0001000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "ADC", "Reg,Mem", 2,
     "1111111000000000", "0001001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "ADC", "Acc,Imm", 1,
     "11111110", "00010100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "ADC", "Reg,Imm8", 2,
     "1111111000111000", "1000001000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADC", "Mem,Imm8", 2,
     "1111111000111000", "1000001000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADC", "Reg,Imm", 2,
     "1111111000111000", "1000000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADC", "Mem,Imm", 2,
     "1111111000111000", "1000000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADD", "Mem,Reg", 2,
     "1111111000000000", "0000000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "ADD", "Reg,Mem", 2,
     "1111111000000000", "0000001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "ADD", "Acc,Imm", 1,
     "11111110", "00000100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "ADD", "Reg,Imm8", 2,
     "1111111000111000", "1000001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADD", "Mem,Imm8", 2,
     "1111111000111000", "1000001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADD", "Reg,Imm", 2,
     "1111111000111000", "1000000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ADD", "Mem,Imm", 2,
     "1111111000111000", "1000000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "Addr@", "", 1,
     "11111111", "01100111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "AND", "Mem,Reg", 2,
     "1111111000000000", "0010000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "AND", "Reg,Mem", 2,
     "1111111000000000", "0010001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "AND", "Acc,Imm", 1,
     "11111110", "00100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "AND", "Reg,Imm8", 2,
     "1111111000111000", "1000001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "AND", "Mem,Imm8", 2,
     "1111111000111000", "1000001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "AND", "Reg,Imm", 2,
     "1111111000111000", "1000000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "AND", "Mem,Imm", 2,
     "1111111000111000", "1000000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ARPL", "Reg16,Reg16", 2,
     "1111111100000000", "0110001100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "ARPL", "Mem16,Reg16", 2,
     "1111111100000000", "0110001100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BOUND", "Reg16,Mem32", 2,
     "1111111100000000", "0110001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BOUND", "Reg32,Mem64", 2,
     "1111111100000000", "0110001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BSF", "RegWord,MemWord", 3,
     "111111111111111100000000", "000011111011110000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BSR", "RegWord,MemWord", 3,
     "111111111111111100000000", "000011111011110100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BSWAP", "RegWord", 2,
     "1111111111111000", "0000111111001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BT", "MemWord,Imm8", 3,
     "111111111111111100111000", "000011111011101000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BT", "MemWord,RegWord", 3,
     "111111111111111100000000", "000011111010001100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTC", "MemWord,Imm8", 3,
     "111111111111111100111000", "000011111011101000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTC", "MemWord,RegWord", 3,
     "111111111111111100000000", "000011111011101100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTR", "MemWord,Imm8", 3,
     "111111111111111100111000", "000011111011101000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTR", "MemWord,RegWord", 3,
     "111111111111111100000000", "000011111011001100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTS", "MemWord,Imm8", 3,
     "111111111111111100111000", "000011111011101000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "BTS", "MemWord,RegWord", 3,
     "111111111111111100000000", "000011111010101100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CALL", "Near", 1,
     "11111111", "11101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CALL", "MemFar", 2,
     "1111111100111000", "1111111100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CALL", "Far", 1,
     "11111111", "10011010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CALL", "MemNear", 2,
     "1111111100111000", "1111111100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CBW", "", 1,
     "11111111", "10011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CDQ", "", 1,
     "11111111", "10011001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CLC", "", 1,
     "11111111", "11111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CLD", "", 1,
     "11111111", "11111100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CLI", "", 1,
     "11111111", "11111010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CLTS", "", 2,
     "1111111111111111", "0000111100000110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMC", "", 1,
     "11111111", "11110101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMOVcc", "Reg,Reg", 3,
     "111111111111000000000000", "000011110100000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMOVcc", "Reg,Mem", 3,
     "111111111111000000000000", "000011110100000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMP", "Mem,Reg", 2,
     "1111111000000000", "0011100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "CMP", "Reg,Mem", 2,
     "1111111000000000", "0011101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "CMP", "Mem,Imm8", 2,
     "1111111000111000", "1000001000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "CMPB", "Acc,Imm", 1,
     "11111110", "00111100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "CMPB", "Mem,Imm", 2,
     "1111111000111000", "1000000000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "CMPSB", "", 1,
     "11111111", "10100110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMPSD", "", 1,
     "11111111", "10100111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMPSW", "", 1,
     "11111111", "10100111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CMPXCHG", "Mem,Reg", 3,
     "111111111111111000000000", "000011111011000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => [ 8, "000000000000000100000000" ],
       "Reg"        => [ 3, "000000000000000000111000" ]
     }
   ],
   [ "CMPXCHG8B", "Mem64", 3,
     "111111111111111100111000", "000011111100011100001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CPUID", "", 2,
     "1111111111111111", "0000111110100010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CS:", "", 1,
     "11111111", "00101110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CWD", "", 1,
     "11111111", "10011001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "CWDE", "", 1,
     "11111111", "10011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "DAA", "", 1,
     "11111111", "00100111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "DAS", "", 1,
     "11111111", "00101111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "DEC", "Reg", 2,
     "1111111000111000", "1111111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "DEC", "Mem", 2,
     "1111111000111000", "1111111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "DEC", "RegWord", 1,
     "11111000", "01001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "DIV", "Mem", 2,
     "1111111000111000", "1111011000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "DIV", "Reg", 2,
     "1111111000111000", "1111011000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "DS:", "", 1,
     "11111111", "00111110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "ENTER", "Imm16,Imm8", 1,
     "11111111", "11001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "ES:", "", 1,
     "11111111", "00100110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "F2XM1", "", 2,
     "1111111111111111", "1101100111110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FABS", "", 2,
     "1111111111111111", "1101100111100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "ST(n)", 2,
     "1111111111111000", "1101110011000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "", 2,
     "1111111111111111", "1101110011000001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "Mem32", 2,
     "1111111100111000", "1101100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "Mem64", 2,
     "1111111100111000", "1101110000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADD", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADDP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADDP", "", 2,
     "1111111111111111", "1101111011000001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FADDP", "ST(n)", 2,
     "1111111100111000", "1101111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FBLD", "ST(0),Mem80", 2,
     "1111111100111000", "1101111100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FBLD", "Mem80", 2,
     "1111111100111000", "1101111100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FBSTP", "Mem80", 2,
     "1111111100111000", "1101111100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FBSTP", "Mem80,ST(n)", 2,
     "1111111100111000", "1101111100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCHS", "", 2,
     "1111111111111111", "1101100111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCLEX", "", 3,
     "000000001111111111111111", "000000001101101111100010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCMOVcc", "", 2,
     "1111111011100111", "1101101011000001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCMOVcc", "ST(n)", 2,
     "1111111011100000", "1101101011000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCMOVcc", "ST(0),ST(n)", 2,
     "1111111011100000", "1101101011000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "ST(0),ST(n)", 2,
     "1111111100111000", "1101100000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "Mem64", 2,
     "1111111100111000", "1101110000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "", 2,
     "1111111111111111", "1101100011010001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "Mem32", 2,
     "1111111100111000", "1101100000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "ST(n)", 2,
     "1111111100111000", "1101100000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOM", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMI", "ST(n)", 2,
     "1111111111111000", "1101101111110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMI", "ST(0),ST(n)", 2,
     "1111111111111000", "1101101111110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMI", "", 2,
     "1111111111111111", "1101101111110001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMIP", "", 2,
     "1111111111111111", "1101111111110001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMIP", "ST(0),ST(n)", 2,
     "1111111111111000", "1101111111110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMIP", "ST(n)", 2,
     "1111111111111000", "1101111111110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "Mem32", 2,
     "1111111100111000", "1101100000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "ST(n)", 2,
     "1111111100111000", "1101100000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "ST(0),ST(n)", 2,
     "1111111100111000", "1101100000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "Mem64", 2,
     "1111111100111000", "1101110000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOMP", "", 2,
     "1111111111111111", "1101100011011001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FCOS", "", 2,
     "1111111111111111", "1101100111111111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDECSTP", "", 2,
     "1111111111111111", "1101100111110110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDISI", "", 2,
     "1111111111111111", "1101101111100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "", 2,
     "1111111111111111", "1101110011111001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "ST(n)", 2,
     "1111111111111000", "1101110011111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "Mem32", 2,
     "1111111100111000", "1101100000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "Mem64", 2,
     "1111111100111000", "1101110000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIV", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVP", "", 2,
     "1111111111111111", "1101111011111001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVP", "ST(n)", 2,
     "1111111100111000", "1101111000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "", 2,
     "1111111111111111", "1101110011110001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "ST(n)", 2,
     "1111111111111000", "1101110011110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "Mem64", 2,
     "1111111100111000", "1101110000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "Mem32", 2,
     "1111111100111000", "1101100000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVR", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVRP", "", 2,
     "1111111111111111", "1101111011110001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVRP", "ST(n)", 2,
     "1111111100111000", "1101111000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FDIVRP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FENI", "", 3,
     "000000001111111111111111", "000000001101101111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FFREE", "", 2,
     "1111111111111111", "1101110111000001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FFREE", "ST(n)", 2,
     "1111111111111000", "1101110111000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIADD", "Mem32", 2,
     "1111111100111000", "1101101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIADD", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIADD", "Mem16", 2,
     "1111111100111000", "1101111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIADD", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOM", "Mem32", 2,
     "1111111100111000", "1101101000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOM", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOM", "Mem16", 2,
     "1111111100111000", "1101111000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOM", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOMP", "Mem32", 2,
     "1111111100111000", "1101101000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOMP", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOMP", "Mem16", 2,
     "1111111100111000", "1101111000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FICOMP", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIV", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIV", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIV", "Mem32", 2,
     "1111111100111000", "1101101000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIV", "Mem16", 2,
     "1111111100111000", "1101111000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIVR", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIVR", "Mem32", 2,
     "1111111100111000", "1101101000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIVR", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIDIVR", "Mem16", 2,
     "1111111100111000", "1101111000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "Mem64", 2,
     "1111111100111000", "1101111100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "ST(0),Mem64", 2,
     "1111111100111000", "1101111100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "Mem32", 2,
     "1111111100111000", "1101101100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "ST(0),Mem32", 2,
     "1111111100111000", "1101101100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "Mem16", 2,
     "1111111100111000", "1101111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FILD", "ST(0),Mem16", 2,
     "1111111100111000", "1101111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIMUL", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIMUL", "Mem16", 2,
     "1111111100111000", "1101111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIMUL", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIMUL", "Mem32", 2,
     "1111111100111000", "1101101000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FINCSTP", "", 2,
     "1111111111111111", "1101100111110111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FINIT", "", 3,
     "000000001111111111111111", "000000001101101111100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIST", "Mem16,ST(0)", 2,
     "1111111100111000", "1101111100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIST", "Mem16", 2,
     "1111111100111000", "1101111100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIST", "Mem32,ST(0)", 2,
     "1111111100111000", "1101101100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FIST", "Mem32", 2,
     "1111111100111000", "1101101100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem32", 2,
     "1111111100111000", "1101101100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem32,ST(0)", 2,
     "1111111100111000", "1101101100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem16", 2,
     "1111111100111000", "1101111100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem16,ST(0)", 2,
     "1111111100111000", "1101111100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem64", 2,
     "1111111100111000", "1101111100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISTP", "Mem64,ST(0)", 2,
     "1111111100111000", "1101111100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUB", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUB", "Mem32", 2,
     "1111111100111000", "1101101000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUB", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUB", "Mem16", 2,
     "1111111100111000", "1101111000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUBR", "ST(0),Mem32", 2,
     "1111111100111000", "1101101000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUBR", "Mem32", 2,
     "1111111100111000", "1101101000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUBR", "ST(0),Mem16", 2,
     "1111111100111000", "1101111000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FISUBR", "Mem16", 2,
     "1111111100111000", "1101111000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "", 2,
     "1111111111111111", "1101100111000001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "ST(n)", 2,
     "1111111100111000", "1101100100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "ST(0),ST(n)", 2,
     "1111111100111000", "1101100100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "Mem80", 2,
     "1111111100111000", "1101101100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "ST(0),Mem80", 2,
     "1111111100111000", "1101101100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "Mem64", 2,
     "1111111100111000", "1101110100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "ST(0),Mem64", 2,
     "1111111100111000", "1101110100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "ST(0),Mem32", 2,
     "1111111100111000", "1101100100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD", "Mem32", 2,
     "1111111100111000", "1101100100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLD1", "", 2,
     "1111111111111111", "1101100111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDCW", "Mem16", 2,
     "1111111100111000", "1101100100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDENV", "Mem", 2,
     "1111111100111000", "1101100100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDL2E", "", 2,
     "1111111111111111", "1101100111101010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDL2T", "", 2,
     "1111111111111111", "1101100111101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDLG2", "", 2,
     "1111111111111111", "1101100111101100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDLN2", "", 2,
     "1111111111111111", "1101100111101101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDPI", "", 2,
     "1111111111111111", "1101100111101011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FLDZ", "", 2,
     "1111111111111111", "1101100111101110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "ST(n)", 2,
     "1111111111111000", "1101110011001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "Mem64", 2,
     "1111111100111000", "1101110000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "Mem32", 2,
     "1111111100111000", "1101100000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMUL", "", 2,
     "1111111111111111", "1101110011001001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMULP", "ST(n)", 2,
     "1111111100111000", "1101111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMULP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FMULP", "", 2,
     "1111111111111111", "1101111011001001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNCLEX", "", 2,
     "1111111111111111", "1101101111100010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNDISI", "", 2,
     "1111111111111111", "1101101111100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNENI", "", 2,
     "1111111111111111", "1101101111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNINIT", "", 2,
     "1111111111111111", "1101101111100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNOP", "", 2,
     "1111111111111111", "1101100111010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNSAVE", "Mem112", 2,
     "1111111100111000", "1101110100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNSTCW", "Mem16", 2,
     "1111111100111000", "1101100100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNSTENV", "Mem", 2,
     "1111111100111000", "1101100100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNSTSW", "AX", 2,
     "1111111111111111", "1101111111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FNSTSW", "Mem16", 2,
     "1111111100111000", "1101110100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FPATAN", "", 2,
     "1111111111111111", "1101100111110011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FPREM", "", 2,
     "1111111111111111", "1101100111111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FPREM1", "", 2,
     "1111111111111111", "1101100111110101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FPTAN", "", 2,
     "1111111111111111", "1101100111110010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FRNDINT", "", 2,
     "1111111111111111", "1101100111111100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FRSTOR", "Mem112", 2,
     "1111111100111000", "1101110100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FS:", "", 1,
     "11111111", "01100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSAVE", "Mem112", 3,
     "000000001111111100111000", "000000001101110100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSCALE", "", 2,
     "1111111111111111", "1101100111111101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSETPM", "", 2,
     "1111111111111111", "1101101111100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSIN", "", 2,
     "1111111111111111", "1101100111111110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSINCOS", "", 2,
     "1111111111111111", "1101100111111011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSQRT", "", 2,
     "1111111111111111", "1101100111111010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "Mem64", 2,
     "1111111100111000", "1101110100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "ST(n),ST(0)", 2,
     "1111111100111000", "1101110100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "ST(n)", 2,
     "1111111100111000", "1101110100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "", 2,
     "1111111111111111", "1101110111010001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "Mem32,ST(0)", 2,
     "1111111100111000", "1101100100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "Mem32", 2,
     "1111111100111000", "1101100100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FST", "Mem64,ST(0)", 2,
     "1111111100111000", "1101110100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTCW", "Mem16", 3,
     "000000001111111100111000", "000000001101100100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTENV", "Mem", 3,
     "000000001111111100111000", "000000001101100100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem80", 2,
     "1111111100111000", "1101101100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem80,ST(0)", 2,
     "1111111100111000", "1101101100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem64", 2,
     "1111111100111000", "1101110100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem64,ST(0)", 2,
     "1111111100111000", "1101110100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem32", 2,
     "1111111100111000", "1101100100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "Mem32,ST(0)", 2,
     "1111111100111000", "1101100100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "", 2,
     "1111111111111111", "1101110111011001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "ST(n)", 2,
     "1111111111111000", "1101110111011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTP", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110111011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTSW", "AX", 3,
     "000000001111111111111111", "000000001101111111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSTSW", "Mem16", 3,
     "000000001111111100111000", "000000001101110100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "Mem32", 2,
     "1111111100111000", "1101100000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "", 2,
     "1111111111111111", "1101110011101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "ST(n)", 2,
     "1111111111111000", "1101110011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "Mem64", 2,
     "1111111100111000", "1101110000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUB", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBP", "ST(n)", 2,
     "1111111100111000", "1101111000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBP", "", 2,
     "1111111111111111", "1101111011101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "", 2,
     "1111111111111111", "1101110011100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "ST(n)", 2,
     "1111111111111000", "1101110011100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "Mem64", 2,
     "1111111100111000", "1101110000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "ST(0),Mem64", 2,
     "1111111100111000", "1101110000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "Mem32", 2,
     "1111111100111000", "1101100000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "ST(0),Mem32", 2,
     "1111111100111000", "1101100000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBR", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110011100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBRP", "", 2,
     "1111111111111111", "1101111011100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBRP", "ST(n)", 2,
     "1111111100111000", "1101111000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FSUBRP", "ST(n),ST(0)", 2,
     "1111111100111000", "1101111000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FTST", "", 2,
     "1111111111111111", "1101100111100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOM", "ST(0),ST(n)", 2,
     "1111111111111000", "1101110111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOM", "", 2,
     "1111111111111111", "1101110111100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOM", "ST(n)", 2,
     "1111111111111000", "1101110111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOM", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110111100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMI", "ST(0),ST(n)", 2,
     "1111111111111000", "1101101111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMI", "", 2,
     "1111111111111111", "1101101111101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMI", "ST(n)", 2,
     "1111111111111000", "1101101111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMIP", "ST(n)", 2,
     "1111111111111000", "1101111111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMIP", "", 2,
     "1111111111111111", "1101111111101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMIP", "ST(0),ST(n)", 2,
     "1111111111111000", "1101111111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMP", "ST(n)", 2,
     "1111111111111000", "1101110111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMP", "", 2,
     "1111111111111111", "1101110111101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMP", "ST(0),ST(n)", 2,
     "1111111111111000", "1101110111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMP", "ST(n),ST(0)", 2,
     "1111111111111000", "1101110111101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMPP", "ST(0),ST(n)", 2,
     "1111111111111000", "1101101011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMPP", "ST(n),ST(0)", 2,
     "1111111111111000", "1101101011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMPP", "", 2,
     "1111111111111111", "1101111011010101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMPP", "", 2,
     "1111111111111111", "1101101011101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FUCOMPP", "ST(n)", 2,
     "1111111111111000", "1101101011101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FWAIT", "", 1,
     "11111111", "10011011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXAM", "", 2,
     "1111111111111111", "1101100111100101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXCH", "ST(n)", 2,
     "1111111111111000", "1101100111001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXCH", "", 2,
     "1111111111111111", "1101100111001001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXCH", "ST(0),ST(n)", 2,
     "1111111111111000", "1101100111001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXCH", "ST(n),ST(0)", 2,
     "1111111111111000", "1101100111001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "0000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FXTRACT", "", 2,
     "1111111111111111", "1101100111110100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FYL2X", "", 2,
     "1111111111111111", "1101100111110001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "FYL2XP1", "", 2,
     "1111111111111111", "1101100111111001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "GS:", "", 1,
     "11111111", "01100101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "HLT", "", 1,
     "11111111", "11110100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IDIV", "Reg", 2,
     "1111111000111000", "1111011000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "IDIV", "Mem", 2,
     "1111111000111000", "1111011000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "IMUL", "RegWord,Imm", 2,
     "1111111111000000", "0110100111000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IMUL", "RegWord,MemWord", 3,
     "111111111111111100000000", "000011111010111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IMUL", "Reg", 2,
     "1111111000111000", "1111011000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "IMUL", "Mem", 2,
     "1111111000111000", "1111011000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "IMUL", "RegWord,MemWord,Imm8", 2,
     "1111111100000000", "0110101100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IMUL", "RegWord,MemWord,Imm", 2,
     "1111111100000000", "0110100100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IMUL", "RegWord,Imm8", 2,
     "1111111111000000", "0110101111000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IN", "Acc,DX", 1,
     "11111110", "11101100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "IN", "Acc,Imm8", 1,
     "11111110", "11100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "INC", "RegWord", 1,
     "11111000", "01000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INC", "Reg", 2,
     "1111111000111000", "1111111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "INC", "Mem", 2,
     "1111111000111000", "1111111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "INSB", "", 1,
     "11111111", "01101100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INSD", "", 1,
     "11111111", "01101101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INSW", "", 1,
     "11111111", "01101101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INT", "3", 1,
     "11111111", "11001100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INT", "Imm8", 1,
     "11111111", "11001101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INTO", "", 1,
     "11111111", "11001110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INVD", "", 2,
     "1111111111111111", "0000111100001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "INVLPG", "Mem", 3,
     "111111111111111100111000", "000011110000000100111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IRET", "", 1,
     "11111111", "11001111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "IRETD", "", 1,
     "11111111", "11001111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JCXE", "Short", 1,
     "11111111", "11100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JCXZ", "Short", 1,
     "11111111", "11100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JECXE", "Short", 1,
     "11111111", "11100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JECXZ", "Short", 1,
     "11111111", "11100011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "Far", 1,
     "11111111", "11101010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "RegWord", 2,
     "1111111100111000", "1111111100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "MemNear", 2,
     "1111111100111000", "1111111100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "MemFar", 2,
     "1111111100111000", "1111111100101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "Short", 1,
     "11111111", "11101011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "JMP", "Near", 1,
     "11111111", "11101001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "Jcc", "Near", 2,
     "1111111111110000", "0000111110000000",
     { "TestField"  => [ 0, "0000000000001111" ],
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "Jcc", "Short", 1,
     "11110000", "01110000",
     { "TestField"  => [ 0, "00001111" ],
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LAHF", "", 1,
     "11111111", "10011111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LAR", "RegWord,MemWord", 3,
     "111111111111111100000000", "000011110000001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LDS", "Reg16,Mem32", 2,
     "1111111100000000", "1100010100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LDS", "Reg32,Mem64", 2,
     "1111111100000000", "1100010100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LEA", "RegWord,Mem", 2,
     "1111111100000000", "1000110100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LEAVE", "", 1,
     "11111111", "11001001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LES", "Reg32,Mem64", 2,
     "1111111100000000", "1100010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LES", "Reg16,Mem32", 2,
     "1111111100000000", "1100010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "0000000000111000" ],
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LFS", "Reg32,Mem64", 3,
     "111111111111111100000000", "000011111011010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LFS", "Reg16,Mem32", 3,
     "111111111111111100000000", "000011111011010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LGDT", "Mem64", 3,
     "111111111111111100111000", "000011110000000100010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LGS", "Reg32,Mem64", 3,
     "111111111111111100000000", "000011111011010100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LGS", "Reg16,Mem32", 3,
     "111111111111111100000000", "000011111011010100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LIDT", "Mem64", 3,
     "111111111111111100111000", "000011110000000100011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LLDT", "Reg16", 3,
     "111111111111111100111000", "000011110000000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LLDT", "Mem16", 3,
     "111111111111111100111000", "000011110000000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LMSW", "Reg16", 3,
     "111111111111111100111000", "000011110000000100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LMSW", "Mem16", 3,
     "111111111111111100111000", "000011110000000100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOCK", "", 1,
     "11111111", "11110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOCK:", "", 1,
     "11111111", "11110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LODSB", "", 1,
     "11111111", "10101100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LODSD", "", 1,
     "11111111", "10101101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LODSW", "", 1,
     "11111111", "10101101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOOP", "Short", 1,
     "11111111", "11100010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOOPE", "Short", 1,
     "11111111", "11100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOOPNE", "Short", 1,
     "11111111", "11100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOOPNZ", "Short", 1,
     "11111111", "11100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LOOPZ", "Short", 1,
     "11111111", "11100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LSL", "RegWord,MemWord", 3,
     "111111111111111100000000", "000011110000001100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LSS", "Reg32,Mem64", 3,
     "111111111111111100000000", "000011111011001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LSS", "Reg16,Mem32", 3,
     "111111111111111100000000", "000011111011001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LTR", "Reg16", 3,
     "111111111111111100111000", "000011110000000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "LTR", "Mem16", 3,
     "111111111111111100111000", "000011110000000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Mem,Imm", 2,
     "1111111000111000", "1100011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "MOV", "Reg,Mem", 2,
     "1111111000000000", "1000101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "MOV", "Reg32,TRn", 3,
     "111111111111111111000000", "000011110010010011000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "TRn,Reg32", 3,
     "111111111111111111000000", "000011110010011011000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Mem16,Seg", 2,
     "1111111100000000", "1000110000000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Mem,Reg", 2,
     "1111111000000000", "1000100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "MOV", "Reg16,Seg", 2,
     "1111111100000000", "1000110000000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Seg,Reg16", 2,
     "1111111100000000", "1000111000000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Seg,Mem16", 2,
     "1111111100000000", "1000111000000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Reg32,CRn", 3,
     "111111111111111111000000", "000011110010000011000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "CRn,Reg32", 3,
     "111111111111111111000000", "000011110010001011000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "Reg32,DRn", 3,
     "111111111111111111000000", "000011110010000111000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "DRn,Reg32", 3,
     "111111111111111111000000", "000011110010001111000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "000000000000000000111000" ],
       "IndexReg"   => [ 0, "000000000000000000000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOV", "MemOfs,Acc", 1,
     "11111110", "10100010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "MOV", "Acc,MemOfs", 1,
     "11111110", "10100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "MOV", "Reg,Imm32", 1,
     "11110000", "10110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 3, "00001000" ],
       "Reg"        => [ 0, "00000111" ]
     }
   ],
   [ "MOVSB", "", 1,
     "11111111", "10100100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSD", "", 1,
     "11111111", "10100101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSW", "", 1,
     "11111111", "10100101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSX", "RegWord,Mem16", 3,
     "111111111111111100000000", "000011111011111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSX", "RegWord,Reg16", 3,
     "111111111111111100000000", "000011111011111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSX", "RegWord,Reg8", 3,
     "111111111111111100000000", "000011111011111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVSX", "RegWord,Mem8", 3,
     "111111111111111100000000", "000011111011111000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVZX", "RegWord,Mem16", 3,
     "111111111111111100000000", "000011111011011100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MOVZX", "RegWord,Mem8", 3,
     "111111111111111100000000", "000011111011011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "MUL", "Reg", 2,
     "1111111000111000", "1111011000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "MUL", "Mem", 2,
     "1111111000111000", "1111011000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "NEG", "Mem", 2,
     "1111111000111000", "1111011000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "NEG", "Reg", 2,
     "1111111000111000", "1111011000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "NOP", "", 1,
     "11111111", "10010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "NOT", "Mem", 2,
     "1111111000111000", "1111011000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "NOT", "Reg", 2,
     "1111111000111000", "1111011000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "Op@", "", 1,
     "11111111", "01100110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "OR", "Mem,Imm8", 2,
     "1111111000111000", "1000001000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "OR", "Reg,Mem", 2,
     "1111111000000000", "0000101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "OR", "Mem,Reg", 2,
     "1111111000000000", "0000100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "OR", "Acc,Imm", 1,
     "11111110", "00001100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "OR", "Reg,Imm", 2,
     "1111111000111000", "1000000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "OR", "Mem,Imm", 2,
     "1111111000111000", "1000000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "OR", "Reg,Imm8", 2,
     "1111111000111000", "1000001000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "OUT", "Imm8,Acc", 1,
     "11111110", "11100110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "OUT", "DX,Acc", 1,
     "11111110", "11101110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "OUTSB", "", 1,
     "11111111", "01101110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "OUTSD", "", 1,
     "11111111", "01101111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "OUTSW", "", 1,
     "11111111", "01101111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POP", "RegWord", 1,
     "11111000", "01011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POP", "Seg", 2,
     "1111111111000111", "0000111110000001",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POP", "MemWord", 2,
     "1111111100111000", "1000111100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POPA", "", 1,
     "11111111", "01100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POPAD", "", 1,
     "11111111", "01100001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POPF", "", 1,
     "11111111", "10011101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "POPFD", "", 1,
     "11111111", "10011101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSH", "Imm8", 1,
     "11111111", "01101010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSH", "Seg", 2,
     "1111111111000111", "0000111110000000",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "0000000000111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSH", "RegWord", 1,
     "11111000", "01010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSH", "SegOld", 1,
     "11000111", "00000110",
     { "TestField"  => undef,
       "SegmentReg" => [ 3, "00111000" ],
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSH", "Imm", 1,
     "11111111", "01101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHA", "", 1,
     "11111111", "01100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHAD", "", 1,
     "11111111", "01100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHD", "Imm32", 1,
     "11111111", "01101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHF", "", 1,
     "11111111", "10011100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHFD", "", 1,
     "11111111", "10011100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHL", "MemWord", 2,
     "1111111100111000", "1111111100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "PUSHW", "Imm32", 1,
     "11111111", "01101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RCL", "Reg,CL", 2,
     "1111111000111000", "1101001000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCL", "Mem,1", 2,
     "1111111000111000", "1101000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCL", "Reg,1", 2,
     "1111111000111000", "1101000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCL", "Mem,Imm8", 2,
     "1111111000111000", "1100000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCL", "Reg,Imm8", 2,
     "1111111000111000", "1100000000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCL", "Mem,CL", 2,
     "1111111000111000", "1101001000010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Mem,CL", 2,
     "1111111000111000", "1101001000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Reg,Imm8", 2,
     "1111111000111000", "1100000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Mem,Imm8", 2,
     "1111111000111000", "1100000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Reg,1", 2,
     "1111111000111000", "1101000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Mem,1", 2,
     "1111111000111000", "1101000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RCR", "Reg,CL", 2,
     "1111111000111000", "1101001000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RDMSR", "", 2,
     "1111111111111111", "0000111100110010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RDPMC", "", 2,
     "1111111111111111", "0000111100110011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RDTSC", "", 2,
     "1111111111111111", "0000111101010001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "REP", "", 1,
     "11111111", "11110011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "REPE", "", 1,
     "11111111", "11110011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "REPNE", "", 1,
     "11111111", "11110010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "REPNZ", "", 1,
     "11111111", "11110010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "REPZ", "", 1,
     "11111111", "11110011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RET", "FAR", 1,
     "11111111", "11001011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RET", "imm FAR", 1,
     "11111111", "11001010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RET", "NEAR", 1,
     "11111111", "11000011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "RET", "imm NEAR", 1,
     "11111111", "11000010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "ROL", "Reg,1", 2,
     "1111111000111000", "1101000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROL", "Mem,1", 2,
     "1111111000111000", "1101000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROL", "Reg,CL", 2,
     "1111111000111000", "1101001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROL", "Mem,CL", 2,
     "1111111000111000", "1101001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROL", "Reg,Imm8", 2,
     "1111111000111000", "1100000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROL", "Mem,Imm8", 2,
     "1111111000111000", "1100000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Mem,CL", 2,
     "1111111000111000", "1101001000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Reg,CL", 2,
     "1111111000111000", "1101001000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Mem,1", 2,
     "1111111000111000", "1101000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Reg,1", 2,
     "1111111000111000", "1101000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Mem,Imm8", 2,
     "1111111000111000", "1100000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "ROR", "Reg,Imm8", 2,
     "1111111000111000", "1100000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "RSM", "", 2,
     "1111111111111111", "0000111110101010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SAHF", "", 1,
     "11111111", "10011110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SAL", "Mem,1", 2,
     "1111111000111000", "1101000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAL", "Reg,CL", 2,
     "1111111000111000", "1101001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAL", "Mem,CL", 2,
     "1111111000111000", "1101001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAL", "Reg,Imm8", 2,
     "1111111000111000", "1100000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAL", "Mem,Imm8", 2,
     "1111111000111000", "1100000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAL", "Reg,1", 2,
     "1111111000111000", "1101000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SALC", "", 1,
     "11111111", "11010110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SAR", "Reg,CL", 2,
     "1111111000111000", "1101001000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAR", "Mem,CL", 2,
     "1111111000111000", "1101001000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAR", "Reg,Imm8", 2,
     "1111111000111000", "1100000000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAR", "Mem,Imm8", 2,
     "1111111000111000", "1100000000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAR", "Reg,1", 2,
     "1111111000111000", "1101000000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SAR", "Mem,1", 2,
     "1111111000111000", "1101000000111000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Mem,Imm", 2,
     "1111111000111000", "1000000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Reg,Imm", 2,
     "1111111000111000", "1000000000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Mem,Imm8", 2,
     "1111111000111000", "1000001000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Reg,Imm8", 2,
     "1111111000111000", "1000001000011000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Acc,Imm", 1,
     "11111110", "00011100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "SBB", "Reg,Mem", 2,
     "1111111000000000", "0001101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "SBB", "Mem,Reg", 2,
     "1111111000000000", "0001100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "SCASB", "", 1,
     "11111111", "10101110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SCASD", "", 1,
     "11111111", "10101111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SCASW", "", 1,
     "11111111", "10101111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SETcc", "Reg8", 3,
     "111111111111000000111000", "000011111001000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SETcc", "Mem8", 3,
     "111111111111000000111000", "000011111001000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SGDT", "Mem64", 3,
     "111111111111111100111000", "000011110000000100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SHL", "Reg,Imm8", 2,
     "1111111000111000", "1100000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHL", "Mem,CL", 2,
     "1111111000111000", "1101001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHL", "Reg,1", 2,
     "1111111000111000", "1101000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHL", "Mem,1", 2,
     "1111111000111000", "1101000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHL", "Reg,CL", 2,
     "1111111000111000", "1101001000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHL", "Mem,Imm8", 2,
     "1111111000111000", "1100000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHLD", "MemWord,RegWord,Imm8", 3,
     "111111111111111100000000", "000011111010010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SHLD", "MemWord,RegWord,CL", 3,
     "111111111111111100000000", "000011111010010100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SHR", "Reg,Imm8", 2,
     "1111111000111000", "1100000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHR", "Mem,CL", 2,
     "1111111000111000", "1101001000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHR", "Mem,1", 2,
     "1111111000111000", "1101000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHR", "Reg,CL", 2,
     "1111111000111000", "1101001000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHR", "Reg,1", 2,
     "1111111000111000", "1101000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHR", "Mem,Imm8", 2,
     "1111111000111000", "1100000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SHRD", "MemWord,RegWord,Imm8", 3,
     "111111111111111100000000", "000011111010110000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SHRD", "MemWord,RegWord,CL", 3,
     "111111111111111100000000", "000011111010110100000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 3, "000000000000000000111000" ],
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SIDT", "Mem64", 3,
     "111111111111111100111000", "000011110000000100001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SLDT", "Mem16", 3,
     "111111111111111100111000", "000011110000000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SMSW", "Mem16", 3,
     "111111111111111100111000", "000011110000000100100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SS:", "", 1,
     "11111111", "00110110",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STC", "", 1,
     "11111111", "11111001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STD", "", 1,
     "11111111", "11111101",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STI", "", 1,
     "11111111", "11111011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STOSB", "", 1,
     "11111111", "10101010",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STOSD", "", 1,
     "11111111", "10101011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STOSW", "", 1,
     "11111111", "10101011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STR", "Mem16", 3,
     "111111111111111100111000", "000011110000000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "STR", "Reg16", 3,
     "111111111111111100111000", "000011110000000000001000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "SUB", "Reg,Imm8", 2,
     "1111111000111000", "1000001000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SUB", "Mem,Imm8", 2,
     "1111111000111000", "1000001000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SUB", "Reg,Imm", 2,
     "1111111000111000", "1000000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SUB", "Mem,Imm", 2,
     "1111111000111000", "1000000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "SUB", "Mem,Reg", 2,
     "1111111000000000", "0010100000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "SUB", "Reg,Mem", 2,
     "1111111000000000", "0010101000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "SUB", "Acc,Imm", 1,
     "11111110", "00101100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "TEST", "Reg,Mem", 2,
     "1111111000000000", "1000010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "TEST", "Acc,Imm", 1,
     "11111110", "10101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "TEST", "Reg,Imm", 2,
     "1111111000111000", "1111011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "TEST", "Mem,Imm", 2,
     "1111111000111000", "1111011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "TEST", "Mem,Reg", 2,
     "1111111000000000", "1000010000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "VERR", "Reg16", 3,
     "111111111111111100111000", "000011110000000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "VERR", "Mem16", 3,
     "111111111111111100111000", "000011110000000000100000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "VERW", "Reg16", 3,
     "111111111111111100111000", "000011110000000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "VERW", "Mem16", 3,
     "111111111111111100111000", "000011110000000000101000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "WAIT", "", 1,
     "11111111", "10011011",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "WBINVD", "", 2,
     "1111111111111111", "0000111100001001",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "WRMSR", "", 2,
     "1111111111111111", "0000111100110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "XADD", "Mem,Reg", 3,
     "111111111111111000000000", "000011111100000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "000000000000000000000111" ],
       "OpcodeFunc" => [ 6, "000000000000000011000000" ],
       "RegType"    => [ 8, "000000000000000100000000" ],
       "Reg"        => [ 3, "000000000000000000111000" ]
     }
   ],
   [ "XCHG", "Reg,Mem", 2,
     "1111111000000000", "1000011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "XCHG", "AccWord,RegWord", 1,
     "11111000", "10010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "XCHG", "RegWord,AccWord", 1,
     "11111000", "10010000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => [ 0, "00000111" ],
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "XCHG", "Mem,Reg", 2,
     "1111111000000000", "1000011000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "XLAT", "", 1,
     "11111111", "11010111",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => undef,
       "Reg"        => undef
     }
   ],
   [ "XOR", "Mem,Imm", 2,
     "1111111000111000", "1000000000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "XOR", "Mem,Reg", 2,
     "1111111000000000", "0011000000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "XOR", "Reg,Mem", 2,
     "1111111000000000", "0011001000000000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => [ 3, "0000000000111000" ]
     }
   ],
   [ "XOR", "Acc,Imm", 1,
     "11111110", "00110100",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => undef,
       "OpcodeFunc" => undef,
       "RegType"    => [ 0, "00000001" ],
       "Reg"        => undef
     }
   ],
   [ "XOR", "Reg,Imm8", 2,
     "1111111000111000", "1000001000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "XOR", "Mem,Imm8", 2,
     "1111111000111000", "1000001000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ],
   [ "XOR", "Reg,Imm", 2,
     "1111111000111000", "1000000000110000",
     { "TestField"  => undef,
       "SegmentReg" => undef,
       "IndexReg"   => undef,
       "IndexFunc"  => [ 0, "0000000000000111" ],
       "OpcodeFunc" => [ 6, "0000000011000000" ],
       "RegType"    => [ 8, "0000000100000000" ],
       "Reg"        => undef
     }
   ]
);

# rrr : w=0 : w=1 : Registers
%x86_registers = (
   "0" => {
      "000" => "AL",
      "001" => "CL",
      "010" => "DL",
      "011" => "BL",
      "100" => "AH",
      "101" => "CH",
      "110" => "DH",
      "111" => "BH",
   },
   "1" => {
      "000" => "EAX",
      "001" => "ECX",
      "010" => "EDX",
      "011" => "EBX",
      "100" => "ESP",
      "101" => "EBP",
      "110" => "ESI",
      "111" => "EDI",
   }
);

# rrr : Index Register
%x86_index_registers = (
   "000" => "EAX",
   "001" => "ECX",
   "010" => "EDX",
   "011" => "EBX",
   "100" => "ESP",
   "101" => "EBP",
   "110" => "ESI",
   "111" => "EDI",
);

# sss : Segment Registers
$x86_segments = [
   ["ES", "000"],
   ["CS", "001"],
   ["SS", "010"],
   ["DS", "011"],
   ["FS", "100"],
   ["GS", "101"]
];

# oo : Function
$x86_opcode_functions = [
   ["If mmm=110, displacement, else no displacement", "00"],
   ["8-bit signed displacement", "01"],
   ["16-bit signed displacement", "10"],
   ["mmm specifies register", "11"]
];

# bbb : Base Register
%x86_base_registers = (
   "000" => "EAX",
   "001" => "ECX",
   "010" => "EDX",
   "011" => "EBX",
   "100" => "ESP",
   "101" => undef,
   "110" => "ESI",
   "111" => "EDI",
);

# tttn : Condition Test Fields
%x86_test_mnemonics = (
   "0000" => "JO",
   "0001" => "JNO",
   "0010" => "JB",
   "0011" => "JNB",
   "0100" => "JE",
   "0101" => "JNE",
   "0110" => "JNA",
   "0111" => "JA",
   "1000" => "JS",
   "1001" => "JNS",
   "1010" => "JPE",
   "1011" => "JPO",
   "1100" => "JL",
   "1101" => "JGE",
   "1110" => "JLE",
   "1111" => "JG"
);

