%{
/* 
 * Copyright (C) 1999, 2000, 2001  Network Associates, Incorporated (NAI)
 * All rights reserved.
 *
 * This file is part of the NAI Generic Software Wrappers Toolkit
 * (GSWTK).  See ftp://ftp.tislabs.com/pub/wrappers/ for the latest
 * distribution.
 *
 * The GSWTK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * The GSWTK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Id: wdl.y,v 1.239 2001/06/18 15:24:45 stevek Exp $
 *
 * Authors: Chris Vance, Mark Feldman, Calvin Ko, Lee Badger
 *
 * Purpose: Parser for wrapper compiler.
 * 
 * The ISO C Specifications are based on the BNF provided by the K&R book.
 *
 */

#include "wr_conf.h"

// This is an identifier was truncated to '255' characters
// warning that shows up simply by using string and map
// templates together. It can be ignored.
#ifdef WR_WINNT
#pragma warning(disable: 4786) 
#endif

#ifdef WR_BROKEN_STL
#define __PUT_STATIC_DATA_MEMBERS_HERE
#include "template/iterator.h"
#include "template/pair.h"
#include "template/set"
#include "template/list"
#include "template/algo.h"
#else /* WR_BROKEN_STL */
#ifdef WR_UNIX
#include <iterator.h>
#include <pair.h>
#endif
#ifdef WR_WINNT
#include <iterator>
#include <utility>
#endif
#include <set>
#include <list>
#ifdef WR_UNIX
#include <algo.h>
#endif
#ifdef WR_WINNT
#include <algorithm>
#include <vector>
#endif
#endif /* WR_BROKEN_STL */

#include "wr_string.h"
#include "libwdl.h"
#include "codegen.h"
#include "expr.h"
#include "event.h"
#include "stack.h"

#ifdef WR_NOSELECT1ST
#include "template/select1st.h"
#endif

#ifndef YYERROR_VERBOSE
#define YYERROR_VERBOSE 1
#endif

extern int  dflag;                   
extern int  typechk;                 

void wdlerror(const char *s, const char *s1 = "");
int wdlwarning(const char *s, const char *s1 = "");
int wdllex();
int wdlparse();

source *pre_funcs = new source;
source *post_funcs = new source;
set<String, less<String> > space_set;

source *symbols = new source;    /* Used to remember typedef'd ids for lex */
source *event_code = NULL;     
source *space_code = NULL;
source *pre_code = new source;
source *post_code = new source;
source *link_declarations = new source;
source *link_statements = new source;
source *wrapper_decl = new source;   	/* store wrapper local functions and
                                           declaration in wrapper */ 

source *error_recovery_slot = NULL;

EvSet  *event_set;	/*  Used to store the set of events corresponding to 
                            the current <simple_event_expr>                  */

AttrTbl  *attrtbl, *pretbl, *posttbl;

/* Remember if we've seen the following lifecycle code */
int have_install = 0;
int have_activate = 0;
int have_deactivate = 0;
int have_uninstall = 0;
int have_duplicate = 0;

/* Remember whether we are in the pre or post code */
int in_pre = 0;

/* Remember state about sequence we're working on */
int seq_has_pre = 0;
int seq_has_post = 0;
int seq_must;
int seq_once;

int in_sequence_decl = 0;
int in_sequence = 0;
int in_wrapper_action = 0;
int in_wrapper_body = 0;

/* true if in a scope where wrapper instance variables are not defined, 
   e.g. wr_install(), wr_uninstall()   */
int in_no_wiv_scope = 0;		

/* Used for empty event specifications.  We ignore everything else in the
 * function and just parse grammer.
 */
int ignore_event = 0; 

int localfunc_haswiv = 0;	

/* Wrapper instance variables. */
source *wiv_source = NULL;
source *wiv_func_source = new source;
VarTbl wiv_variables;
int curvar_wivvar = 0;

/* Put all global variables in a single block so that we
 * can share them on NT via shared segments.
 * Easier than putting the relevant magic "place in a shared segment"
 * pragma's around every single global variable.
 */

source *global_vars = new source;

/* We use the same parser for wrapper WDL and uses, but
 * there are some differences, so remember if we're in uses
 */
int in_uses = 0;
int in_includes = 0;

characterization *tmpchar = NULL;
characterization *wildchar = new characterization("*");

typedef lnode<source *> SrcNode;

EvSubSysTbl   esstbl;			/* eventsubsystem table      */
EvSubSysTbl   essgrp;			/* eventsubsystem group      */
VarStore      vstore;			/* contain a type table and 
                                           a stack of variable tables */
int           in_essdef = 0;    	/* is parsing event subsystem def */

/* Database compilier globals. */
smap<int>      *table_order = NULL; /* generates col_name => col_num map.*/
StrTbl         *table_rev_map = NULL; /* generates col_num => col_name map */
int             table_elem_count; /* generates column count */
set<int,less<int> > *table_key_fields; /* keeps track of key fields */
int             table_key_field; /* gets set if a query is keyed */
int             table_not_toggle; /* Keeps track of !() nesting. */
stack<int>      table_query_is_keyed;
VarTbl         *table_names = NULL;
const char     *current_table = NULL; /* Keeps track of current table(typechk)*/
source         *creation_instructions = NULL;/*create select scripts for reuse*/
source         *destruction_instructions = NULL;/*clean up those statics*/
source         *init_instructions = NULL;/*Initialize table handles*/
source         *activate_instructions = NULL;/*Initialize table handles*/
source         *deactivate_instructions = NULL;/*Initialize table handles*/
source         *var_decls = NULL; /* declare variables. */
source         *uninstall_body = NULL; /* so we can add stuff to clean up */
source         *install_body = NULL; /* so we can add stuff to init */
source         *activate_body = NULL; /* so we can add stuff to activate*/
source         *deactivate_body = NULL; /* so we can add stuff to de-act*/
source         *duplicate_body = NULL; /* so we can add stuff to duplicate*/
int             variable_generator_count = 0;
int             needs_db_components = 0;


/*
 * Usefull "while I'm here I need to do something else" blocks.
 */
/* This source will be added right before the '}' that ends a block */
stack<source *> post_block_body;
int             post_block_body_count = 0;
stack<int>      last_loop_level;
int             last_function_level = -1;
/* vstore.get_scopelevel() */

/* This source will be added before the current generated statement */
source *pre_statement_block = NULL;
/* This source will be added after the current generated statement */
source *post_statement_block = NULL;
/* This source will be added after declarations are finished. */
source *post_decl_block = NULL;

/* Dependency information. */
list<char*> dependency_list;

#define OP_TYPE    0x001
#define OATTR_TYPE 0x002
#define PATTR_TYPE 0x004

#define IS_PRE     0x008
#define IS_POST    0x010

int char_spec_type = 0; 
int pre_count      = 0;
int post_count     = 0;
int sequence_count = 0;
int step_count     = 0;
stack<int> *seq_block_start = new stack<int>;
stack<int> *seq_in_switch = new stack<int>;
stack<int> *in_switch_stack = new stack<int>;

struct EvNode {
     source *src; 	// contains the source of the if expression
     source *src1;	/* contains the source of the if expression
			 * without the stuff in the STEP statement
			 * used for generating copyin code
			 */
     EvSet  *set;
     EvNode (source *s, EvSet *es, source *s1=NULL) {
          src = s;
          set = es;
          src1 = s1;
     }
};

struct EvSpecNode {
    char *space;
    int code;
    WrEvent *event;
};

struct AssmtNode {
     source *src;
     int    dvflag;   // set if the chain of assmt contain $variable
     int    nested;   // no of assignment in the chain
     AssmtNode(source *s) {  src = s; dvflag = 0;  nested = 0; }
};

typedef pair<int, String> pair_i_S;
typedef set<pair_i_S, less<pair_i_S> > set_pair_i_S;
typedef set<int, less<int> > set_int;

struct pollutionTree {
    set_pair_i_S *polluted;
    source *src;
    pollutionTree() {
        polluted = new set_pair_i_S;
        src = new source;
    }
};

bool check_duplicates(set_pair_i_S &aset)
{
    set_pair_i_S::iterator i;
    /* Duplicate set.  Sorry for the sledgehammer
     * approach
     */
    set_int s;
    for(i=aset.begin();i!=aset.end();++i){
        if (!(s.insert((*i).first).second))
            return true;
    }
    return false;
}

enum wr_lifecycle_types {
    wr_install,
    wr_uninstall,
    wr_activate,
    wr_duplicate,
    wr_deactivate
};

/* VarTbl for system functions.  Haul this out every time we find ourselves
 * in an includes block.
 */
VarTbl *includes_vtbl = NULL;

%}

%union {
	char 	  *string;
        source 	  *src;
	mlabel 	  *lbl;
	Param  	  *para;
        EvNode 	  *evp;
        Expr      *expr;
        Type      *type;
        AssmtNode *assmt;
        SrcNode   *snode;
        EvSpecNode *evsnode;
        pollutionTree *ptree;
        enum wr_lifecycle_types lifecycle_types;
}

%nonassoc LOWER_THAN_ELSE
%nonassoc K_ELSE

%token	<string> S_IDENTIFIER S_TYPENAME
%token	<string> S_QSTR S_NUMCONST S_FLOATCONST S_CHARCONST

%token		 K_CHARACTERIZE K_WRAPPER K_INCLUDES K_LIBRARY K_USES
%token           K_INSTALL K_UNINSTALL K_ACTIVATE K_DEACTIVATE K_DUPLICATE

/* Characterization tokens */

%token  K_NTERM K_NTERMLIST K_IN K_OUT K_COPY K_RCOPY K_HANDLE K_TYPE K_VMAP

/* Used for variable declarations */
%token	K_SHORT K_SHORTINT K_SIGNEDSHORT K_SIGNEDSHORTINT
%token	K_INT K_SIGNEDINT K_SIGNED K_UNSIGNEDINT
%token	K_LONG K_LONGLONG K_SIGNEDLONG 
%token	K_UNSIGNED K_UNSIGNEDINT K_UNSIGNEDSHORT K_UNSIGNEDSHORTINT
%token	K_UNSIGNEDLONG
%token	K_CHAR K_SIGNEDCHAR K_UNSIGNEDCHAR K_STR K_WSS_CHAR_T
%token	K_FLOAT K_DOUBLE K_LONGDOUBLE
%token	K_CONST K_VOLATILE
%token	K_AUTO K_EXTERN K_REGISTER K_STATIC K_TYPEDEF K___INLINE K_GLOBAL K_DLLIMPORT

/* ---------- WDL variable declarations */
%token  K_DBROW K_DBTABLE K_KEY

/* For expressions */
%token	K_OR K_AND K_NOTEQ K_EQEQ K_LTEQ K_GTEQ K_SHFTL K_SHFTR
%token	K_PLUSEQ K_MINUSEQ K_MULTEQ K_DIVEQ K_MODEQ K_SHFTLEQ 
%token	K_SHFTREQ K_ANDEQ K_OREQ K_XOREQ
%token	K_ELIPSIS
%token	K_STRMATCH
%token	K_PLUSPLUS K_MINUSMINUS

/* for statements */
%token	K_GOTO K_BREAK K_CONTINUE K_RETURN K_SWITCH K_CASE K_DEFAULT
%token	K_SIZEOF K_STRUCT K_UNION K_VOID K_ENUM K_IF K_ELSE K_DO K_WHILE
%token	K_FOR K_INDIRECT K_DELETE

/* for wdl events */
%token	K_COLONCOLON K_PRE K_POST
%token	K_OTHERWISE
%token	K_OP K_OATTR K_PATTR K_STEP

/* WQL tokens */
%token	K_WQL K_CREATE K_DROP K_INSERT K_SELECT K_INTO K_VALUES
%token	K_DELETE K_UPDATE K_TABLE K_FROM K_WHERE K_LIKE K_ORDER K_BY
%token	K_WQL_OR K_WQL_AND K_SET K_DESC K_LOCAL K_TRY K_CATCH K_FIRST

/* ---------- Types ---------- */

%type <type>  /* char_type */ floating_type integer_type string_type
/* %type <type>  signed_int_type unsigned_int_type */

%type <lifecycle_types> life_cycle
%type <string> assignment_op enumeration_tag equality_op 
%type <string> RE_op relational_op shift_op 
%type <string> struct_or_union type_qualifier unary_op
%type <string> var_storage_class void_type
%type <string> wql_expression_op
%type <string> wql_select_first

%type <snode>  struct_declarator_list

%type <src> abstract_declarator 
%type <src> characterization compound_stmt compound_stmt_1 
%type <src> conditional_stmt conditioned_event
%type <src> conditioned_event_list 
%type <src> direct_abstract_declarator
%type <src> declaration declaration_list wrapper_declaration_list 
%type <src> declaration_specifiers declaration_specifiers_list declaration_spec
%type <src> declarator direct_declarator
%type <expr> declarator_list
%type <src> enumeration_constant_definition
%type <src> enumeration_definition_list enumeration_type 
%type <src> enumeration_type_definition enumeration_type_reference 
%type <src> event_condition
%type <src> event_sequence event_sequence_spec
%type <src> event_subsys_expr event_subsys_or_expr
%type <src> expression_list 
%type <src> expression_stmt filtered_event
%type <src> function_declarator
%type <src> function_definition
%type <src> local_function_definition local_function_specifier
%type <src> function_specifier
%type <src> id_expr iface_declaration_specifiers 
%type <src> iterative_stmt jump_stmt labeled_stmt
%type <src> life_cycle_decl 
%type <src> parameter_list parameter_type_list
%type <src> parameter_declaration pointer
%type <src> sequence_event
%type <src> sequence_step  sequence_steps
%type <src> simple_event_expr
%type <src> statement statement_list 
%type <src> specifier specifier_list
/* %type <src> specifier_qualifier_list */
%type <src> struct_declaration struct_declaration_body struct_declaration_list
%type <src> struct_or_union_definition
%type <src> struct_declarator 
%type <src> struct_or_union_reference struct_or_union_specifier
%type <src> switch_block switch_stmt memory_management_stmt
%type <src> top_level_declaration type_name 
%type <src> type_specifier type_qualifier_list type_specifier_or_reference
%type <src> inline_spec
%type <src> type_reference uses_stmt type_reference_complete
%type <ptree> wql_and_expression wql_primary_expression wql_expression
%type <ptree> wql_terminal_expression
%type <src> wdl wdl_type wql_column_name
%type <src> wql_table_declaration_body wql_table_column
%type <src> wql_create_stmt wql_delete_stmt wql_drop_stmt
%type <src> wql_insert wql_insert_list
%type <src> wql_insert_stmt wql_null_stmt 
%type <src> wql_orderby wql_select_stmt wql_selection
%type <src> wql_statement wql_select_value
%type <src> wql_statement_list
%type <src> wql_stmt wql_type
%type <src> wql_update wql_update_list
%type <src> wql_update_expression wql_update_primary_expression
%type <src> wql_update_postfix_expression wql_update_unary_expression
%type <src> wql_update_additive_expression wql_update_multiplicative_expression
%type <src> wql_update_function_call wql_update_expression_list
%type <src> wql_update_cast_expression wql_update_expr
%type <src> wql_update_stmt wql_value wql_value_list

%type <src> wr_stmt wr_stmt_list 
%type <src> wrapper wrapper_body 

%type <lbl> markup_stmt markup_stmt_list markup_spec
%type <para> iface_declarator iface_direct_declarator
%type <para> iface_parameter_declaration
%type <para> iface_arg_list iface_arg_declaration

%type <evp> character_spec 
%type <evp> id_and_expr id_or_expr
%type <evp> id_primary_expr identifier_expr  
%type <evp> op_attr_event_and_expr op_attr_event_exp
%type <evp> op_attr_event_primary_expr
%type <evp> seq_attr_event_exp seq_attr_event_and_expr 
%type <evp> seq_attr_event_primary_expr seq_character_spec  
%type <evp> rep_seq_attr_event_exp

%type <expr> additive_expression assignment_expression 
%type <expr> bitwise_and_expression bitwise_or_expression
%type <expr> bitwise_xor_expression conditional_expression
%type <expr> cast_expression constant_expression
%type <expr> equality_expression expression 
%type <expr> function_call 
%type <expr> initialized_declarator initialized_decl_list initializer 
%type <expr> initializer_list logical_and_expression
%type <expr> logical_or_expression multiplicative_expression 
%type <expr> constant varref_expression string_list
%type <expr> postfix_expression primary_expression
%type <expr> relational_expression shift_expression 
%type <expr> op_string_expression string_expression
%type <expr> unary_expression 

%type <expr> markup_expression markup_bitwise_and_expression
%type <expr> markup_bitwise_or_expression markup_bitwise_xor_expression
%type <expr> markup_equality_expression markup_relational_expression
%type <expr> markup_and_expression markup_or_expression
%type <expr> markup_primary_expr

%type <evsnode> event_specifier
%%

/* ================================================================= *
 *                 WDL file  -- everything starts here               *
 * ================================================================= */

wdl_file:       wdl {
                  /* Use already open file or wrapc.out.c */
                  extern char *outfilename;
                  FILE *outfile;
                  if (outfilename == NULL)
                      outfilename = "wrapc.out.c";
                  if ((outfile=fopen(outfilename, "w")) == NULL)
                      wdlerror("Unable to open %s!", outfilename);
                  $1->rewind();
                  while($1->more_text())
                    fprintf(outfile, "%s", $1->next());
		  fclose(outfile);
                }
                ;

wdl:          
                specifier_list wrapper
                {
                    $1->append($2);
                    delete $2;
                    $$=$1;
                    // printf("\nWrapper done!\n");
		}
              |
		wrapper 
                {
                  $$=$1;
		  // printf("\nWrapper done!\n");
		}
                ;

specifier_list:
                specifier_list specifier
		{
                    $1->append($2);
                    delete $2;
		}
              | specifier 
              ;

specifier:      characterization
              | top_level_declaration
              | uses_stmt 
              | error {
                  wdlerror("Error in attempting type-unification recovery.");
              }
              | includes_stmt {
                  $$ = new source;
              }
	      ;

wrapper:	K_WRAPPER S_IDENTIFIER 
                {
                    vstore.newtbl();
                    destruction_instructions = new source;
                    if (!init_instructions)
                        init_instructions = new source;
                    activate_instructions = new source;
                    deactivate_instructions = new source;
                    var_decls = new source;
                    uninstall_body = new source;
                    install_body = new source;
                    activate_body = new source;
                    deactivate_body = new source;
                    duplicate_body = new source;
                    in_wrapper_body = 1;
                } 
                '{' wrapper_body '}' 
                {
                    in_wrapper_body = 0;
                    /* Create wrapper */
                    source *sp = new source;
                    if (!needs_db_components) {
                        sp->append("#define WR_SUPPRESS_WQL 1\n");
                    }
                    /* Add event space variables to link_declarations */
                    set<String, less<String> >::iterator it;
                    for (it = space_set.begin(); it != space_set.end(); ++it) {
#ifdef WR_BROKEN_STL
                        const char *name = (*it).chars();
#else
                        const char *name = (*it).c_str();
#endif
                        /* Global since its set in install */
                        SourceAppend3(global_vars, 
                                "static int ", name, "=0;\n");
                        SourceAppend4(init_instructions,
                                name," = wr_resolve_event_space(PD_MAKESTR(\"",
                                name,"\"));\n");
                    }
                    sp->make_header(link_declarations);

                    sp->append("static pd_char_t *WRLINK_MOD_DEPENDENCIES=");
                    sp->append("PD_MAKESTR(\"");
                    list<char*>::iterator li = dependency_list.begin();
                    for ( ; li != dependency_list.end(); li++ ) {
                        sp->append(*li);
                        sp->append("\\0");
                    }
                    sp->append("\");\n");

                    sp->append("pd_char_t EXPORT *get_mod_dependencies()\n");
                    sp->append("{                                  \n");
                    sp->append("  return WRLINK_MOD_DEPENDENCIES;  \n");
                    sp->append("}                                  \n\n");

                    /* Add in the variable declarations for static
                     * select/update/delete queries.
                     * In fact, we don't want to share these on NT due
                     * to potential race conditions.
                     */
                    sp->append(var_decls);

                    /* Add in the global variable declarations. */
                    SourceAppend12(sp,
                            "#if defined(WR_WINNT)\n",
                            "#pragma data_seg(\".",$2,"\")\n",
                            "#endif\n",
                            global_vars,
                            "#if defined(WR_WINNT)\n",
                            "#pragma data_seg()\n",
                            "#pragma comment(linker, \"/SECTION:.",
                                $2,",RWS\")\n",
                            "#endif\n");

#ifdef WR_WINNT
                    sp->append("/* Disable warnings about unused labels */\n");
                    sp->append("#pragma warning(disable: 4102)\n\n");
                    sp->append("/* List of symbols wrapped by wrapper */\n");
                  
                    sp->append("static pd_char_t *WRLINK_SYMBOL_TABLE[] = \n");
                    sp->append("{\n   PD_MAKESTR(\"\"),\n   ");

                    int nsymbols;
                    int i;

                    for ( i = 0; get_syscallname(i) != NULL; i++) {
                      if( nsymbols++ ) sp->append(",\n   ");
                      sp->append("PD_MAKESTR(\"");
                      sp->append(get_syscallname(i)->c_str());
                      sp->append("\")");
                    }

                    sp->append("\n};\n");
                    sp->append("#define WR_NCALLS (sizeof(WRLINK_SYMBOL_TABLE)/");
                    sp->append("sizeof(pd_char_t *))\n\n");
#endif /* WR_WINNT */

#ifdef WR_UNIX
                    sp->append("static int WRLINK_MOD_INTERCEPT_LIST[] = \n");
                    sp->append("{\n  ");

                    extern IntTbl *codetbl;

                    if( codetbl )
                    {
                       int cols = 0;
                       for( int i = 1; syscallnames[i] != NULL; i++ )
                          if( (*codetbl)[syscallnames[i]] )
                          {
                             if( cols && ( ( cols % 8 ) == 7 ) )
                                sp->append("\n  ");
                             sp->append(" ");
                             sp->append( (*codetbl)[syscallnames[i]] );
                             sp->append(",");
                             cols++;
                          }

                       if( cols && ( ( cols % 8 ) == 7 ) )
                          sp->append("\n  ");
                    }
                    sp->append(" 0\n};\n");
#endif /* WR_UNIX */

                    /* Add wrapper local declaration */
                    sp->append(wrapper_decl);

                    if (sequence_count)
                        sp->gen_seq_step_funcs();
		  
                    /* Add WIV declarations */
                    if (wiv_source != NULL) {
                        wiv_source->indent("    ");
                        SourceAppend3(sp,"struct wr_wiv {\n",
                                wiv_source,
                                "};\n");
                        /* Add in the code to activate_instructions to create
                         * and bzero out the WIV structure.
                         * 
                         * Small issue: This has to be first...
                         */
                        source *new_act_instr = new source;
                        SourceAppend9(new_act_instr,
                                "struct wr_wiv *wr_wiv_foo;\n",
                                "wr_wiv_foo=wr_malloc(sizeof(*wr_wiv_foo));\n",
                                "if (WSS_GET_WIV() != NULL)\n",
                                "    wr_bcopy(WSS_GET_WIV(),\n",
                                "        wr_wiv_foo, sizeof(*wr_wiv_foo));\n",
                                "else\n",
                                "    wr_bzero(wr_wiv_foo,\n",
                                "        sizeof(*wr_wiv_foo));\n",
                                "WSS_SAVE_WIV(wr_wiv_foo);\n"
                        );
                        new_act_instr->append(activate_instructions);
                        delete activate_instructions;
                        activate_instructions = new_act_instr;

                        /* Add in code to deactivate_instructions to free the
                         * WIV structure.
                         * 
                         * Small issues: This has to be last.  Not a problem
                         * yet.
                         */

                        SourceAppend2(deactivate_instructions,
                                "wr_free(WSS_GET_WIV());\n",
                                "WSS_SAVE_WIV(NULL);\n"
                                     );
                    }

                  /* Add WIV Func Definition */
                  sp->gen_comment("WIV Functions\n");
                  sp->append(wiv_func_source);

#ifdef WR_WINNT
                  sp->append("static char *WRLINK_PREPOST_SYMBOLS[] = \n");
                  sp->append("{\n   \"");
                  if (pre_count)
                    sp->append("PRE");
                  if (post_count)
                    sp->append("POST");
                  sp->append("\"");

                  for ( i = 0; get_syscallname(i) != NULL; i++) {
                    sp->append(",\n   ");
                    sp->append("\"");
                    sp->append(get_syscallname(i)->c_str());
                    sp->append("\"");
                  }
                  sp->append(",\n   ");
                  sp->append("\"\"");

                  sp->append("\n};\n\n");

                  sp->append("char EXPORT **wr_getsymbols(void)\n{\n");
                  sp->append("   return (char **)(WRLINK_PREPOST_SYMBOLS);\n");
                  sp->append("}\n\n");

                  sp->append("const pd_char_t *wr_syscall_name(int call)\n{\n");
                  sp->append("   return((const pd_char_t *)(((call<0)||(call>");
                  sp->append(nsymbols);
                  sp->append("))?PD_MAKESTR(\"\"):WRLINK_SYMBOL_TABLE[call]));\n");
                  sp->append("}\n\n");
#endif /* WR_WINNT */

		  sp->gen_ascii($2);
                  sp->append("static pd_char_t *WRLINK_MOD_TYPE=");
                  sp->append("PD_MAKESTR(\"wrapper\");\n");

                  sp->append("pd_char_t EXPORT **get_mod_type()\n");
                  sp->append("{\n");
                  sp->append("   return &WRLINK_MOD_TYPE;\n");
                  sp->append("}\n\n");


		  sp->append($5);
        

                  /* Add the wr_uninstall function now that we know its
                   * contents.
                   */
                  destruction_instructions->indent("    ");
                  init_instructions->indent("    ");
                  sp->gen_wr_uninstall(uninstall_body,destruction_instructions);
                  sp->gen_wr_install(init_instructions, install_body);
                  activate_instructions->indent("    ");
                  deactivate_instructions->indent("    ");
                  activate_body->indent("    ");
                  duplicate_body->indent("    ");
                  deactivate_body->indent("    ");
                  sp->gen_wr_lifecycle("wr_activate ",
                          activate_instructions->all(), activate_body);
                  sp->gen_wr_lifecycle("wr_duplicate ",
                          activate_instructions->all(), duplicate_body);
                  /* Deactivate_destructions is the magic stuff, so that
                   * should probally happen after the user's code. */
                  sp->gen_wr_lifecycle("wr_deactivate ",
                          deactivate_body->all(), deactivate_instructions);

		  /* Add pre_list() function if there are any pre actions */
		  /* Add post_list() function if there are any post actions */
                  if (pre_count || post_count)
                      sp->gen_dispatchers(pre_funcs, post_funcs);

		  vstore.deltop();
		  $$ = sp;
		}
		;

wrapper_body:	  wrapper_declaration_list wr_stmt_list {
                    wrapper_decl->append($1);
		    $$ = $2;
                  }
		| wr_stmt_list
		| wrapper_declaration_list
		| {
		    source *sp = new source;
		    sp->append("/* Empty body!!! */\n");
		    $$ = sp;
		  }
		;

wrapper_declaration_list:
                  declaration wrapper_declaration_list {
                      source *sp = new source;
		      SourceAppend2(sp, $1, $2);
		      $$ = sp;
                      delete $1; delete $2;
                  }
                | local_function_definition wrapper_declaration_list {
                    $1->append($2);
                    delete $2;
                    $$ = $1;
                  }
                | ';' wrapper_declaration_list {
                    $$ = $2;
                }
                | declaration
                | local_function_definition 
                | ';' {
                    source *sp = new source;
                    $$ = sp;
                }
                ;

local_function_definition:
		  local_function_specifier 
		  {   /* assume $1->d_type is a function */ 
                      FuncTypeInfo *fti = (FuncTypeInfo *) $1->d_type->info;
                      if (fti->vtbl != NULL) {
                          vstore.pushtbl(fti->vtbl);
                      }
                      if (!(fti->flag & IS_WI_LOCALFUNC))
                          in_no_wiv_scope = 1;
                      last_function_level = post_block_body_count;
                  }
                  compound_stmt
		  {   
                      FuncTypeInfo *fti = (FuncTypeInfo *) $1->d_type->info;
                      if (fti->vtbl != NULL)
                           vstore.poptbl();
                      VarNode *vn = new VarNode ($1->vname, *$1->d_type);
                      if (!vstore.toptbl()->insert($1->vname, vn)) {
                          wdlerror("Duplicated definition of function");
                      }
                      source *sp = new source;
		      SourceAppend2(sp, $1, $3);
                      in_no_wiv_scope = 0;
                      if (fti->flag & IS_WI_LOCALFUNC) {
                           wiv_func_source->append(sp);
                           $$ = new source;
                      } else 
		           $$ = sp;
                      last_function_level = -1;
		  }
		;

local_function_specifier:
                  declaration_spec declarator 
                  {
                    $2->d_type->fillin_type(*$1->d_type);

                    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
                    sp->d_type = $2->d_type;
                    sp->vname = $2->vname;
		    $$ = sp
                  }
                  ;

wr_stmt_list:	  wr_stmt_list wr_stmt {
                    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
		    $$=sp;
                  }
		| wr_stmt
		;

wr_stmt:	  life_cycle_decl
		| filtered_event
		| event_sequence_spec
		;

includes_list_item:
                  function_definition {
                      // We want the typedef's, and we want the types.
                      // We don't want the include files to make it to the
                      // generated wrapper, and we don't want to see function
                      // declarations.
                      // 
                      // We do want to let the types pollute unfortunately
                      // since we need some inline functions to resolve
                      // other inline functions.
                      
                      VarNode *vn = new VarNode($1->vname, *($1->d_type));
                      vstore.toptbl()->insert($1->vname, vn);
                  }
                | declaration {
                      delete $1;
                  }
                | ';'
                ;

includes_list:
                  includes_list_item {
                  }
                | includes_list includes_list_item {
                  }
                ;

includes_stmt:  K_INCLUDES {
                    in_includes++;in_no_wiv_scope=1;
                    if (includes_vtbl)
                        vstore.pushtbl(includes_vtbl);
                    else
                        vstore.newtbl();
                }
                '{' includes_list '}' ';'
                {
                    includes_vtbl = vstore.poptbl();
                    in_includes--;
                    in_no_wiv_scope = 0;
                }
                ;

uses_stmt:        K_USES S_IDENTIFIER '{' {
		    SourceAppend3(link_declarations, "/* Using library ", $2, 
                                  " */\n");
                    in_uses++;
                  } 
                  declaration_list '}' ';' {
		    source *sp = new source;
		    SourceAppend3(sp, "/* Using library ", $2, " */\n");
		    dependency_list.push_back($2);
		    link_declarations->append($5);
		    in_uses = 0;
		    link_declarations->append("/* end library */\n"); 
		    $$=sp;
                  }
                | K_USES '{' {
		    link_declarations->append("/* Using default library */\n");
                    in_uses++;
                  } 
                  declaration_list '}' ';' {
		    source *sp = new source;
		    SourceAppend1(sp, "/* Using default library */\n");
		    sp->append($4);
		    in_uses = 0;
		    link_declarations->append("/* end library */\n"); 
		    $$=sp;
                  }
                ;


/* ================================================================= *
 *                 WDL Event specifications                          *
 * ================================================================= */
simple_event_expr: 
                  event_subsys_expr K_COLONCOLON op_attr_event_exp 
                  {
		      source *sp = new source;
		      SourceAppend3(sp, "\n/* The following is in the \"", 
				    $1->all(), "\" event subsystem */\n");
		      $$=sp;
                      if (space_code) delete space_code;
                      space_code = $1;
		      event_code = $3->src;
                      event_set = $3->set;
                      if (dflag & 2) event_set->print(cerr);
		  }
		;

event_subsys_expr: 
                  {    essgrp.reset();    }
		  event_subsys_or_expr
                  {    $$ = $2;           }

event_subsys_or_expr: 
                  id_expr
		| event_subsys_or_expr K_OR id_expr  
                  {
                      $$ = new source;
                      SourceAppend5($$,"(",$1,"||",$3,")");
                      delete $1; delete $3;
                  }
                | '*' {
                      $$ = new source;
                      $$->append("(1)");
                      essgrp.merge(esstbl);
                      tmpchar = wildchar;
	 	  }
		;

id_expr:	  S_IDENTIFIER {
                      $$ = new source;
                      SourceAppend3($$,"(space == ",$1,")");
                      space_set.insert($1);
                      tmpchar = esstbl[$1];
                      if (tmpchar == NULL)
                          wdlerror("Characterization is not defined");
                      essgrp.insert(tmpchar);
                  }
		| '!' S_IDENTIFIER {
                      $$ = new source;
                      space_set.insert($2);
                      SourceAppend3($$,"(space != ",$2,")");
                      tmpchar = esstbl[$2];
                      if (tmpchar == NULL)
                          wdlerror("Characterization is not defined");
                      EvSubSysTbl etbl; 
                      etbl.merge(esstbl);
                      etbl.remove(tmpchar);
                      essgrp.merge(etbl);
                  }
		;

op_attr_event_exp: 
		  op_attr_event_and_expr
		| op_attr_event_exp K_OR op_attr_event_and_expr  {
                    source *sp = new source;
		    SourceAppend3(sp, $1->src, " || ", $3->src);

                    EvSet *setp = $1->set;
                    setp->merge(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp); 
		  }
		;

op_attr_event_and_expr: op_attr_event_primary_expr
		| op_attr_event_and_expr K_AND op_attr_event_primary_expr  {
                    source *sp = new source;
		    SourceAppend3(sp, $1->src, " && ", $3->src);

                    EvSet *setp = $1->set;
                    setp->intersect(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp); 
		  }
		;

op_attr_event_primary_expr: 
		  character_spec 
		| '(' op_attr_event_exp ')'  {
		     source *sp = new source;
		     SourceAppend3(sp, "(", $2->src, ")");
		     $$ = new EvNode(sp, $2->set);
                    
		  }
		| '!' op_attr_event_primary_expr  {
                     source *sp = new source;
		     SourceAppend2(sp, "!", $2->src);
                     EvSet *setp = new EvSet(tmpchar->evset);
                     setp->minus(*$2->set);
		     $$ = new EvNode(sp, setp);
		  }
		;

character_spec:	  K_OP {char_spec_type=OP_TYPE} '{' identifier_expr '}'   {
		     $$ = $4;
		  }
		| K_OATTR {char_spec_type=OATTR_TYPE} '{' identifier_expr '}' {
		     $$ = $4;
		  }
		| K_PATTR {char_spec_type=PATTR_TYPE} '{' identifier_expr '}' {
		     $$ = $4;
		  }
		;

identifier_expr:  id_or_expr 
                | '*' {
		    source *sp = new source;
		    sp->append("(1)");          /* true */
                    EvSet *setp = essgrp.get_evset();

                    /* 
                     *   Iterate through all of the symbols in the set
                     *   when we hit a '*' token. This allows us to
                     *   properly generate the list of symbols
                     *   that are used by the wrapper.
                     */
                    EvSet::iterator iter;
                    for (iter = setp->begin(); iter != setp->end(); iter++) {
                      get_syscallcode(*iter);
                    }

                    $$ = new EvNode(sp, setp);
		  }
		;

id_or_expr:	  id_and_expr
                | id_or_expr K_OR id_and_expr {
                    source *sp = new source;
		    SourceAppend5(sp, "((", $1->src, ") || (", $3->src, "))");

                    EvSet *setp = $1->set;
                    setp->merge(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp); 
		  }
		;

id_and_expr:	  id_primary_expr
		| id_and_expr K_AND id_primary_expr {
		    source *sp = new source;
		    SourceAppend5(sp, "((", $1->src, ") && (", $3->src, "))");

                    EvSet *setp = $1->set;
                    setp->intersect(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp); 
		  }
		;

id_primary_expr:  S_IDENTIFIER {
                    source *sp = new source;
		    int i;
		    if (char_spec_type == OP_TYPE) {
		      sp->append("((code==");
                      sp->append(tmpchar->get_code($1));
                      /* Used to be able to do two syscalls with the
                       * same name.  I removed that support in the interests
                       * of my own sanity.  SJK 2/22/99
                       */
		      sp->append("))");
		    }
		    else if (char_spec_type == OATTR_TYPE ||
			     char_spec_type == PATTR_TYPE) {
		      characterization::iterator iter;
		      sp->append("(");
		      for (i=0, iter=(char_spec_type == OATTR_TYPE) ? 
			     tmpchar->begin(characterization::OATTR,$1) :
			     tmpchar->begin(characterization::PATTR,$1);
			   iter != tmpchar->end(); iter++, i++) { 
			if (i) 
			  sp->append(" || ");
			sp->append("(code==");
                        sp->append(tmpchar->get_code((*iter)->name));
                        /* Used to be able to do two syscalls with the
                         * same name.  I removed that support in the interests
                         * of my own sanity.  DSK 11/7/98
                         */
                        /* Now add in any if-clauses. */
                        mlabel *ml;
                        ml = (char_spec_type == OATTR_TYPE) ?
                            (*iter)->find_elabel($1) : 
                            (*iter)->find_plabel($1);
                        if (ml->if_clause != NULL) {
                            sp->append("&&");
                            sp->append(ml->if_clause->get_src());
                        }
			sp->append(")");
		      }
                      if (!i)
                          sp->append("0");
		      sp->append(")");
		    }
		    else {
		      sp->append ("\n/* Something is wrong! */\n");
                      wdlwarning("Internal compilier error ch01");
		    }
                    EvSet *setp = NULL;
		    if (char_spec_type == OP_TYPE) {
                         if (essgrp.is_valid_op($1))
                              setp = new EvSet($1);
                         else
                              wdlerror("identifier is not a valid op");
		    } else if (char_spec_type == OATTR_TYPE) {
                         setp = essgrp.get_evset_from_oattr($1);
                         if (setp->empty())
                              wdlerror("identifier is not a valid oattr");
		    } else if (char_spec_type == PATTR_TYPE) {
                         setp = essgrp.get_evset_from_pattr($1);
                         if (setp->empty()) {
                              wdlwarning("identifier is not a valid pattr");
                              ignore_event = 1;
                         }
                    } else
                         wdlerror("Internal Error #50");
                    $$ = new EvNode(sp, setp);
                  }
		| '(' id_or_expr ')' {
                    source *sp = new source;
		    SourceAppend3(sp, "(", $2->src, ") ");
                    $$ = new EvNode(sp, $2->set);
                  }
		| '!' id_primary_expr {
                    source *sp = new source;
		    SourceAppend3(sp, " !(", $2->src, ")");

                    EvSet *setp = essgrp.get_evset();

                    setp->minus(*$2->set); 
                    delete $2->set;

                    /* 
                     *   Iterate through all of the symbols in the set
                     *   when we hit a '!' token. This allows us to
                     *   properlay generate the list of symbols
                     *   that are used by the wrapper.
                     */
                    EvSet::iterator iter;
                    for (iter = setp->begin(); iter != setp->end(); iter++) {
                      get_syscallcode(*iter);
                    }

		    $$ = new EvNode(sp, setp);
                  }
		;

life_cycle_decl:  life_cycle '(' ')'
                { last_function_level = post_block_body_count; }
                compound_stmt {
                    /* Don't output anything yet.  We'll get it later */
                    source *sp = new source;

                    switch ($1) {
                    case wr_uninstall:
                        uninstall_body = $5;
                        break;
                    case wr_install:
                        install_body = $5;
                        break;
                    case wr_activate:
                        activate_body = $5;
                        break;
                    case wr_duplicate:
                        duplicate_body = $5;
                        break;
                    case wr_deactivate:
                        deactivate_body = $5;
                        break;
                    default:
                        wdlerror("Internal compilier error WIV7");
                    }
                    in_no_wiv_scope = 0;
                    last_function_level = -1;
		    $$=sp;
                  }
		;

life_cycle:       K_INSTALL    {
                    if (have_install++) 
		      wdlerror("Duplicate wr_install()");
                    in_no_wiv_scope = 1;
                    $$=wr_install;}
                | K_UNINSTALL  {
                    if (have_uninstall++) 
                      wdlerror("Duplicate wr_uninstall()");
                    in_no_wiv_scope = 1;
                    $$=wr_uninstall;}
                | K_ACTIVATE   {
                    if (have_activate++) 
                      wdlerror("Duplicate wr_activate()");
                    $$=wr_activate;}
                | K_DEACTIVATE {
                    if (have_deactivate++)
                      wdlerror("Duplicate wr_deactivate()");
                    $$=wr_deactivate;}
                | K_DUPLICATE  {
                    if (have_duplicate++) 
                      wdlerror("Duplicate wr_duplicate()");
                    $$=wr_duplicate;}
                ;

/* ================================================================= *
 *                      WDL Event Sequences                          *
 * ================================================================= */

event_sequence_spec: 
		  event_subsys_expr K_COLONCOLON {
                      if (space_code) delete space_code;
                      space_code = $1;
                      sequence_count++;
                      step_count = 0;
                      in_switch_stack->push(0);
		  }
                  event_sequence {
                      source *sp = new source;
                      SourceAppend4(sp, 
                              "\n/* The following event sequence is in the \"",
                              $1->all(), "\" event subsystem */\n", $4);
                      in_switch_stack->pop();
                      $$=sp;
                      delete space_code;
                      space_code = NULL;
		  }
		;

event_sequence:   
	'{' {in_sequence_decl++;vstore.newtbl();} declaration_list 
        {in_sequence_decl=0; in_sequence++; } sequence_steps '}'
        {
	    in_sequence = 0;
	    vstore.deltop();
	    source *sp = new source;
	    source *tmp;
            SourceAppend7(sp, "\nstatic int sequence", sequence_count-1, 
                    " (int space, int code, ",
                    "void *uap[], int *retval, ",
                    "struct wss_proc *wrp,",
                    "struct wss_wrapper_data *data,",
                    "int args_in_userspace, int ispre)\n{\n");
	    sp->append("   static struct seq_inst *insts = NULL;\n");
	    sp->append("   struct seq_inst *inst;\n");
            $3->indent("       ");
	    SourceAppend7(sp, 
                       "   struct seq_data {\n", $3, "   };\n",
	               "   struct seq_data *my_seq_data;\n",
                       "   void *uapk[10];\n",
	               "   int sequence_return = WR_SUCCESS;\n",
	               "   start_sequence(&insts, sizeof(struct seq_data));\n");
            $5->indent("   ");
	    sp->append($5);
	    SourceAppend4(sp, "   prune_sequences(&insts, ", step_count, ");\n",
	                      "   return sequence_return;\n}\n");
	    $$ = sp;
	    if (seq_has_pre) {
	       seq_has_pre = 0;
	       SourceAppend5(pre_funcs, "    if (ret == WR_SUCCESS)\n",
			    "        ret=sequence", sequence_count-1, 
			    "(space, code, uap, retval, wrp, data, ",
                            "args_in_userspace, 1);\n");
	    }
	    if (seq_has_post) {
 	       seq_has_post = 0;
	       tmp = post_funcs;
 	       post_funcs = new source;
	       SourceAppend5(post_funcs, "    if (ret == WR_SUCCESS)\n",
			    "        ret=sequence", sequence_count-1,
			    "(space, code, uap, retval, wrp, data,",
                            "args_in_userspace, 0);\n");
               post_funcs->append(tmp);
	    }		  
	}
        | '{' sequence_steps '}' {
            source *sp = new source;
            source *tmp;
            SourceAppend7(sp, "\nstatic int sequence", sequence_count-1, 
                    " (int space, int code, ",
                    "void *uap[], int *retval, ",
                    "struct wss_proc *wrp,",
                    "struct wss_wrapper_data *data,",
                    "int args_in_userspace, int ispre)\n{\n");
            sp->append
                ("  static struct seq_inst *insts = NULL;\n");
            sp->append
                ("  struct seq_inst *inst;\n");
            sp->append("start_sequence(&insts, 0));\n");
            sp->append($2);
            SourceAppend3(sp, "  prune_sequences(&insts, ",
                    step_count, ");\n");
            sp->append("  return WR_SUCCESS;\n}\n");
            $$=sp;
            if (seq_has_pre) {
                seq_has_pre = 0;
                SourceAppend5(pre_funcs,"    if (ret==WR_SUCCESS)\n",
                        "        ret=sequence", sequence_count-1, 
                        "(space, code, uap, retval, wrp, data, ",
                        "args_in_userspace, 1);\n");
            }
            if (seq_has_post) {
                seq_has_post = 0;
                tmp = post_funcs;
                post_funcs = new source;
                SourceAppend5(post_funcs,"    if (ret==WR_SUCCESS)\n",
                        "        ret=sequence", sequence_count-1,
                        "(space, code, uap, retval, wrp, data,",
                        "args_in_userspace, 0);\n");
                post_funcs->append(tmp);
            }		  
        }
        | '{' '}' {
            source *sp = new source;
            sp->append("\n/* empty event sequence */\n");
            $$=sp;
        }
        ;

sequence_steps: sequence_steps sequence_step{
		    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
		    $$=sp;
		  }
		| sequence_step
		;

sequence_event:   
        {   pretbl = new AttrTbl(5);
            posttbl = new AttrTbl(5);
            attrtbl = posttbl;
        }
	rep_seq_attr_event_exp 
	conditioned_event_list ';' 
	{ 
	    source *sp = new source;
            AttrTbl::iterator iter;
            source *cpincode = new source;
            source *cpoutcode = new source;
	    if (!pre_code->is_empty()) {
	       seq_has_pre++;
	       pre_count++;
               for (iter = pretbl->begin(); iter != pretbl->end(); iter++) {
                   cpincode->gen_comment("copy in arg $",
                                         (*iter)->m->para->name);
                   cpincode->gen_argcopyin((*iter)->m, (*iter)->ofstr);
                   if ((*iter)->write == 1)
                       if ((*iter)->m->para->is_out())
                           cpoutcode->gen_argcopyout((*iter)->m, 
                                   (*iter)->ofstr);
                       else 
                           cpoutcode->gen_argrepl((*iter)->m, (*iter)->ofstr);
                   else
                       cpoutcode->gen_argfree((*iter)->m, (*iter)->ofstr);
               }
               if (!cpincode->is_empty()) {
	           SourceAppend5(sp, "if (ispre && ", space_code->all(),
                           " && (", $2->src1, ") && !inst->done) {\n");
                   cpincode->indent("      ");
                   SourceAppend4(sp, "   {\n", cpincode, "   }\n", "}\n");
               }
	       SourceAppend5(sp, "if (ispre && ", space_code->all(),
                       " && (", $2->src, ") && !inst->done) {\n");
               sp->append("   inst->done++;\n");
	       if (seq_once) {
	  	   SourceAppend5(sp, 
			"   inst->step_number = ", step_count + 1, ";\n",  
			"   inst->has_run = 0;\n",
			"   inst->try_next = 0;\n");
	       } else {
	  	   SourceAppend4(sp, 
			"   inst->has_run++;\n",
		        "   inst->step_number = ",step_count,";\n");
	       }
               pre_code->indent("   ");
	       sp->append(pre_code);
               if (!cpoutcode->is_empty()) {
                   cpoutcode->indent("      ");
                   SourceAppend3(sp, "   {\n", cpoutcode, "   }\n");
               }
	       pre_code = new source;
	       sp->append("} else\n");
	       if (seq_must && !in_switch_stack->top()) {
	   	   SourceAppend1(sp, "  inst->try_next=inst->has_run;\n");
	       } else {
		   SourceAppend1(sp, "  inst->try_next = 1;\n");
	       }
            }
	    if (!post_code->is_empty()) {
	       seq_has_post++;
	       post_count++;
               for (iter = posttbl->begin(); iter != posttbl->end(); iter++) {
                    cpincode->gen_comment("copy in arg $",
                                          (*iter)->m->para->name);
                    cpincode->gen_argcopyin((*iter)->m, (*iter)->ofstr);
                    if ((*iter)->write == 1)
                        if ((*iter)->m->para->is_out())
                            cpoutcode->gen_argcopyout((*iter)->m,
                                    (*iter)->ofstr);
                        else
                            cpoutcode->gen_argrepl((*iter)->m, (*iter)->ofstr);
                   else
                       cpoutcode->gen_argfree((*iter)->m, (*iter)->ofstr);
               }
               if (!cpincode->is_empty()) {
	           SourceAppend5(sp, "if (!ispre  && ", space_code->all(),
                           "&& (", $2->src1, ") && !inst->done) {\n");
                   cpincode->indent("      ");
                   SourceAppend4(sp, "   {\n", cpincode, "   }\n", "}\n");
               }
	       SourceAppend5(sp, "if (!ispre && ",space_code->all(),
                       " && (", $2->src, 
                       ") && !inst->done) {\n");
               sp->append("   inst->done++;\n");
	       if (seq_once) {
	           SourceAppend5(sp, 
                            "   inst->step_number = ", step_count + 1, ";\n", 
                            "   inst->has_run = 0;\n",
			    "   inst->try_next = 0;\n");
	       } else {
	           SourceAppend4(sp, 
			      "   inst->has_run++;\n",
			      "   inst->step_number = ",step_count,";\n");
	       }
               post_code->indent("   ");
	       sp->append(post_code);
               if (!cpoutcode->is_empty()) {
                   cpoutcode->indent("      ");
                   SourceAppend3(sp, "   {\n", cpoutcode, "   }\n");
               }
	       post_code = new source;
	       SourceAppend1(sp, "} else {\n");
	       if (seq_must && !in_switch_stack->top()) {
                   SourceAppend2(sp, "   inst->try_next=inst->has_run;\n", 
                                     "}\n");
	       } else {
	    	   SourceAppend2(sp, "   inst->try_next = 1;\n", "}\n");
	       }
  	    }
	    $$=sp;
            delete pretbl; delete posttbl;
            attrtbl = NULL;
            pretbl = NULL;
            posttbl = NULL;
        }
	;

sequence_step:	  sequence_event {
                    source *sp = new source;
		    SourceAppend3(sp, "\n", "for (inst = insts; inst ",
			       "!= NULL; inst = inst->next) {\n");
		    if (!step_count) 
		        sp->append("   inst->try_next= inst->done= 0;\n");
		    SourceAppend2(sp, "   if (inst->step_number == ", 
				  step_count);
		    if (step_count)
		        sp->append(" || inst->try_next");
                    $1->indent("       ");
		    SourceAppend6(sp, ") {\n", "       my_seq_data = ", 
				  "(struct seq_data *)inst->inst_vars;\n",
				  $1, "   }\n", "}\n");
		    SourceAppend5(sp, "seqend_", sequence_count-1, "_",
				  step_count, ": /* Target for return */\n");
		    step_count++;
		    $$=sp;
                  }
		| '{' {seq_block_start->push(step_count); 
		       in_switch_stack->push(0);
		      } sequence_steps '}' RE_op {
		    source *sp = new source;
		    sp->gen_tbd("repeating sequence block");
		    (void) seq_block_start->pop();
		    in_switch_stack->pop();
		    step_count++;
		    $$=sp;
		  }
		| K_SWITCH '{' {seq_block_start->push(step_count);
		                in_switch_stack->push(1); }
                  switch_block '}' RE_op {
		    source *sp = new source;
		    SourceAppend4(sp, $4, "\n", "for (inst = insts; ",
			  "inst != NULL; inst = inst->next) {\n");
		    SourceAppend8(sp, "   if (((inst->step_number == ", 
			  seq_block_start->top(), " && inst->has_run) || ",
			  "(inst->step_number > ", seq_block_start->top(), 
			  " && inst->step_number <= ", step_count, 
			  ")) || inst->try_next) {\n");
		    if (seq_once) {
		       SourceAppend5(sp, 
                             "      inst->step_number = ", step_count+1, ";\n",
			     "      inst->try_next = 0;\n",
			     "      inst->has_run = 0;\n");
		    } else {
		       SourceAppend4(sp, 
			     "      inst->step_number = ", 
                                       seq_block_start->top(), ";\n",
			     "      inst->has_run = 0;\n");
		    }
		    SourceAppend1(sp, "   }\n}\n");
		    (void) seq_block_start->pop();
		    in_switch_stack->pop();
		    step_count++;
		    $$=sp;
		  }
		;

switch_block:	switch_block K_CASE sequence_event {
		    source *sp = new source;
		    sp->append($1);
                    SourceAppend3(sp, $1, "\n",
		     "for (inst = insts; inst != NULL; inst = inst->next) {\n");
		    if (!step_count) 
		       sp->append("  inst->try_next= inst->done= 0;\n");
		    SourceAppend2(sp, "   if (inst->step_number == ", 
				  step_count);
		    if (step_count)
		      sp->append(" || inst->try_next");
                    $3->indent("      ");
		    SourceAppend6(sp, ") {\n", "      my_seq_data = ", 
				  "(struct seq_data *)inst->inst_vars;\n",
				  $3, "   }\n", "}\n");
		    SourceAppend5(sp, "seqend_", sequence_count-1, "_",
				  step_count, 
				  ":  /* Target for \"return\" */\n");
		    step_count++;
		    $$=sp;
		  }
		| K_CASE sequence_event{
                    source *sp = new source;
                    SourceAppend2(sp, "\n", 
                     "for (inst = insts; inst != NULL; inst = inst->next) {\n");
		    if (!step_count) 
		      sp->append("  inst->try_next= inst->done= 0;\n");
		    SourceAppend2(sp, "   if (inst->step_number == ", 
				  step_count);
		    if (step_count)
		      sp->append(" || inst->try_next");
                    $2->indent("      ");
		    SourceAppend6(sp, ") {\n", "      my_seq_data = ",
				  "(struct seq_data *)inst->inst_vars;\n",
				  $2, "   }\n", "}\n");
		    SourceAppend5(sp, "seqend_", sequence_count-1, "_",
				  step_count, 
				  ":  /* Target for \"return\" */\n");
		    step_count++;
		    $$=sp;
		  }
		;

rep_seq_attr_event_exp: 
		  {vstore.newtbl();} seq_attr_event_exp RE_op {
		    vstore.deltop();
		    $$ = $2;
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
  		  }
                ;

seq_attr_event_exp: 
  		  seq_attr_event_and_expr
		| seq_attr_event_exp K_OR seq_attr_event_and_expr {
		    source *sp = new source;
		    source *sp1 = new source;
		    SourceAppend3(sp, $1->src, " || ", $3->src);
		    SourceAppend3(sp1, $1->src1, " || ", $3->src1);

		    EvSet *setp = $1->set;
                    setp->merge(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp, sp1);
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		;

seq_attr_event_and_expr: 
		  seq_attr_event_primary_expr
		| seq_attr_event_and_expr K_AND seq_attr_event_primary_expr{
		    source *sp = new source;
		    source *sp1 = new source;
		    SourceAppend3(sp, $1->src, " && ", $3->src);
		    SourceAppend3(sp1, $1->src1, " && ", $3->src1);
                    EvSet *setp = $1->set;
                    setp->intersect(*($3->set));
                    delete $3->set;
		    $$ = new EvNode(sp, setp, sp1); 		    
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		;

seq_attr_event_primary_expr: 
		  seq_character_spec 
		| '(' seq_attr_event_exp ')' {
		    source *sp = new source;
		    source *sp1 = new source;
		    SourceAppend3(sp, "(", $2->src, ")");
		    SourceAppend3(sp1, "(", $2->src1, ")");
		    $$=new EvNode(sp, $2->set, sp1);
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		| '!' seq_attr_event_primary_expr {
		    source *sp = new source;
		    source *sp1 = new source;
		    SourceAppend2(sp, "!", $2->src);
		    SourceAppend2(sp1, "!", $2->src1);
		    EvSet *setp = new EvSet(tmpchar->evset);
		    setp->minus(*$2->set);
		    $$= new EvNode(sp, setp, sp1);
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		;

seq_character_spec: 
		  K_OP {char_spec_type=OP_TYPE} '{' identifier_expr '}' {
		    $$=$4;
                    source *sp = new source;
                    sp->append($$->src->all());
                    $$->src1 = sp;
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		| K_OATTR {char_spec_type=OATTR_TYPE} '{' identifier_expr '}' {
		    $$=$4;
                    source *sp = new source;
                    sp->append($$->src->all());
                    $$->src1 = sp;
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		| K_PATTR {char_spec_type=PATTR_TYPE} '{' identifier_expr '}' {
		    $$=$4;
                    source *sp = new source;
                    sp->append($$->src->all());
                    $$->src1 = sp;
                    event_set = $$->set;
                    if (dflag & 2) event_set->print(cerr);
		  }
		| K_STEP  '(' expression ')' {
		    source *sp = new source;
		    source *sp1 = new source;
                    sp1->append("(1)");
		    SourceAppend3(sp, "(" , $3->code, ")");
		    EvSet *setp = essgrp.get_evset(); /* ??? */
		    $$ = new EvNode (sp, setp, sp1);
		  }
		;


RE_op:		
		  '*' { seq_must = 0; seq_once = 0; $$="*";}
		| '+' { seq_must = 1; seq_once = 0; $$="+";}
		| '?' { seq_must = 0; seq_once = 1; $$="?";}
                |     { seq_must = 1; seq_once = 1; $$=NULL;}
		;


/* ================================================================= *
 *                   Filtering Via Conditionals                      *
 * ================================================================= */

filtered_event:   
	simple_event_expr 
	{  pretbl = new AttrTbl(5);  
	   posttbl = new AttrTbl(5);  
           attrtbl = posttbl;
           in_wrapper_action = 1;
        } 
        conditioned_event_list ';' 
	{
           in_wrapper_action = 0;
           source *tmp;
           source *sp = new source;
	   sp->append($1);
           AttrTbl::iterator iter;
           source *cpincode = new source;
           source *cpoutcode = new source;
           source *retcode = new source;
	   if (!pre_code->is_empty()) {
               for (iter = pretbl->begin(); iter != pretbl->end(); iter++) {
                   cpincode->gen_comment("copy in arg $",
                                          (*iter)->m->para->name);
                   cpincode->gen_argcopyin((*iter)->m, (*iter)->ofstr);
                   if ((*iter)->write == 1)
                       if ((*iter)->m->para->is_out()) 
                            cpoutcode->gen_argcopyout((*iter)->m,
                                                   (*iter)->ofstr);
                       else
                           cpoutcode->gen_argrepl((*iter)->m, (*iter)->ofstr);
                   else
                       cpoutcode->gen_argfree((*iter)->m, (*iter)->ofstr);
               }
               SourceAppend7(sp, "\nstatic int pre", pre_count, 
		   	         " (int space, int code, ",
                                 "void *uap[], int *retval, ",
                                 "struct wss_proc *wrp,",
				 "struct wss_wrapper_data *data,",
                                 "int args_in_userspace)\n{\n");
               if (!cpoutcode->is_empty()) {
                    cpoutcode->indent("    ");
                    SourceAppend7(retcode, "preend_", pre_count, ":\n", "{\n",
                                           cpoutcode, "}\n",
                                           "return wrp_return;\n"); 
               } else {
                    SourceAppend4(retcode, "preend_", pre_count, ":\n", 
                                           "return wrp_return;\n"); 
               }
	       SourceAppend4(pre_funcs, 
                                 "    if (ret == WR_SUCCESS && !data->skip)\n",
	  			 "         ret=pre", pre_count++, 
				 "(space, code, uap, retval, wrp, data,"
                                 " args_in_userspace);\n");
	       if (event_code || space_code) { 
                   SourceAppend5(sp, "if (",
                           space_code ? space_code->all() : "1",
                           " && ", 
                           event_code ? event_code->all() : "1", ")\n");
               }
               
               if (!cpincode->is_empty()) {
                    cpincode->indent("    "); 
                    pre_code->indent("    ");
                    retcode->indent("    ");
                    SourceAppend6(sp, "{   void *uapk[10];\n", 
                                      "    int wrp_return = WR_ALLOW;\n",
                                      cpincode, pre_code, retcode, "}\n");
               } else {
                    pre_code->indent("    ");
                    retcode->indent("    ");
                    SourceAppend5(sp, "{\n", 
                                      "    int wrp_return = WR_ALLOW;\n",
                                      pre_code, retcode, "}\n");
               }
               SourceAppend2(sp, "return WR_SUCCESS;\n", "}\n");
	       pre_code = new source;
	   }
	   if (!post_code->is_empty()) {
               for (iter = posttbl->begin(); iter != posttbl->end(); iter++) {
                    cpincode->gen_comment("copy in arg $",
                                          (*iter)->m->para->name);
                    cpincode->gen_argcopyin((*iter)->m, (*iter)->ofstr);
                    if ((*iter)->write == 1)
                        if ((*iter)->m->para->is_out()) 
                             cpoutcode->gen_argcopyout((*iter)->m,
                                                   (*iter)->ofstr);
                        else
                             cpoutcode->gen_argfree((*iter)->m,
                                                    (*iter)->ofstr);
                   else
                       cpoutcode->gen_argfree((*iter)->m, (*iter)->ofstr);
               }
               SourceAppend7(sp, "\nstatic int post", post_count, 
		   	         " (int space, int code, ",
                                 "void *uap[], int *retval, ",
                                 "struct wss_proc *wrp,",
				 "struct wss_wrapper_data *data,",
                                 "int args_in_userspace)\n{\n");
               if (!cpoutcode->is_empty()) {
                    cpoutcode->indent("    ");
                    SourceAppend7(retcode, "postend_", post_count, ":\n", "{\n",
                                           cpoutcode, "}\n",
                                           "return wrp_return;\n"); 
               } else {
                    SourceAppend4(retcode, "postend_", post_count, ":\n",
                                           "return wrp_return;\n"); 
               }
	       tmp = post_funcs;
	       post_funcs = new source;
	       SourceAppend4(post_funcs, 
                                "    if (ret == WR_SUCCESS && !data->skip)\n",
			        "         ret=post", post_count++, 
                                "(space, code, uap, retval, wrp, data,"
                                " args_in_userspace);\n");
               post_funcs->append(tmp);

	       if (event_code || space_code) { 
                   SourceAppend5(sp, "if (",
                           space_code ? space_code->all() : "1",
                           " && ", 
                           event_code ? event_code->all() : "1", ")\n");
               }

               if (!cpincode->is_empty()) {
                    cpincode->indent("    "); 
                    post_code->indent("    ");
                    retcode->indent("    ");
                    SourceAppend6(sp, "{   void *uapk[10];\n", 
                                      "    int wrp_return = WR_ALLOW;\n",
                                       cpincode, post_code, retcode, "}\n");
               } else {
                    post_code->indent("    ");
                    retcode->indent("    ");
                    SourceAppend5(sp, "{\n", 
                                      "    int wrp_return = WR_ALLOW;\n",
                                      post_code, retcode, "}\n");
               }
               SourceAppend2(sp, "return WR_SUCCESS;\n", "}\n");
	       post_code = new source;
           }
           delete pretbl; delete posttbl; 
           attrtbl = NULL;
           pretbl = NULL;
           posttbl = NULL;
           if (event_code) delete event_code;
           if (space_code) delete space_code;
           event_code = 0; space_code = 0;
	   $$ = sp;
	}
	;

conditioned_event_list: 
                  conditioned_event_list conditioned_event {
                    source *sp = new source;
		    sp->append("conditioned_event_list(1) parsed");
		    $$ = sp;
		    if ($2->get_flags() & IS_PRE)
		      pre_code->append($2);
		    else {
		      source *tmp;
		      tmp = post_code;
		      post_code = $2;
		      $2->append(tmp);
		    }
                  }
		| '<' conditioned_event_list conditioned_event '>' {
                    source *sp = new source;
		    sp->gen_tbd("conditioned_event_list(2) ...");
		    if ($2->get_flags() & IS_PRE)
		      pre_code->append($3);
		    else {
		      source *tmp;
		      tmp = post_code;
		      post_code = $3;
		      $3->append(tmp);
		    }
		    $$ = sp;
                  }
                | conditioned_event {
                    source *sp = new source;
		    sp->gen_tbd("conditioned_vent(3) parsed");
		    $$=sp;
		    if ($1->get_flags() & IS_PRE)
		      pre_code->append($1);
		    else {
		      source *tmp;
		      tmp = post_code;
		      post_code = $1;
		      $1->append(tmp);
		    }
		  }
		;


conditioned_event:
                  K_PRE 
                  /* attrtbl stored all the attr referenced in the block.
                   * In a pre block attrtbl is set to pretbl, other it is
                   * In a post block attrtbl is set to posttbl
                   */ 
                  {  attrtbl = pretbl;  
                     in_pre = 1;
                     // swap pre and post if posttbl not empty
		     if (!posttbl->empty()) {
                          attrtbl = posttbl;
                          posttbl = pretbl;
                          pretbl = attrtbl;
                     }
                     last_function_level = post_block_body_count;
		  } 
                  compound_stmt {
                    in_pre = 0;
		    $3->set_flags(IS_PRE);
                    attrtbl = posttbl;
                    if (ignore_event) {
                        $$ = new source;
                        $$->append("{ }\n");
                        delete $3;
                    } else 
                        $$ = $3;
                    last_function_level = -1;
                    ignore_event = 0;
                  }
                | event_condition K_PRE 
		  {  attrtbl = pretbl; 
                     in_pre = 1;
                     /* swap pre and post if posttbl not empty
                      * the reason is that in parsing event_condition,
                      * we do not know whether this is a pre or post block
                      * $var accessed in the event_condition is put into
                      * the posttbl (since it is the default), therefore,
                      * we swap the pre and post table at this point
                      */
		     if (!posttbl->empty()) {
                          attrtbl = posttbl;
                          posttbl = pretbl;
                          pretbl = attrtbl;
                     }
                     last_function_level = post_block_body_count;
                  } 
		  compound_stmt {
                    source *sp = new source;
                    in_pre = 0;
		    sp->set_flags(IS_PRE);
                    if (ignore_event) {
                        sp->append("if (0) { }\n");
                    } else 
                        SourceAppend4(sp, "if (", $1, ") ", $4);

                    attrtbl = posttbl;
		    $$=sp;
                    last_function_level = -1;
                    delete $4;
                    ignore_event = 0;
                  }
                | K_POST { last_function_level = post_block_body_count; }
                compound_stmt {
		    $3->set_flags(IS_POST);
                    if (ignore_event) {
                        $$ = new source;
                        $$->append("{ }\n");
                        delete $3;
                    } else 
                        $$ = $3;
                    last_function_level = -1;
                    ignore_event = 0;
                  }
                | event_condition K_POST {
                      last_function_level = post_block_body_count; }
                compound_stmt {
                    source *sp = new source;
		    sp->set_flags(IS_POST);
                    if (ignore_event) {
                        sp->append("if (0) { }\n");
                    } else 
                        SourceAppend4(sp, "if (", $1, ") ", $4);

		    $$=sp;
                    last_function_level = -1;
                    delete $4;
                    ignore_event = 0;
                  }
                ;


event_condition:  '(' expression ')' {
                    source *sp = new source;
		    sp->gen_tbd("event_condition(1) ...");
		    sp->append("(");
		    sp->append($2->code);
		    sp->append(")");
		    $$=sp
		  }
		| K_OTHERWISE {
                    source *sp = new source;
		    sp->append("otherwise");
		    $$=sp
		  }
		;



/* ================================================================= *
 *                         C Statements                              *
 * ================================================================= */

statement_list:	  statement_list statement {
                    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
		    $$=sp;
                  }
		| statement
		; 

statement:	  expression_stmt
                {
                    if (pre_statement_block || post_statement_block) {
                        source *sp = new source;
                        if (pre_statement_block) {
                            sp->append(pre_statement_block);
                            delete pre_statement_block;
                            pre_statement_block = NULL;
                        }
                        sp->append($1);
                        if (post_statement_block) {
                            sp->append(post_statement_block);
                            delete post_statement_block;
                            post_statement_block = NULL;
                        }
                        $$ = sp;
                    } else {
                        $$ = $1;
                    }
                }
		| labeled_stmt
		| compound_stmt
		| {last_loop_level.push(post_block_body_count)}
		iterative_stmt {
			$$ = $2;
			last_loop_level.pop();
		}
		| jump_stmt
		| conditional_stmt
		| {last_loop_level.push(post_block_body_count)}
		switch_stmt {
			$$ = $2;
			last_loop_level.pop();
		}
                | memory_management_stmt
		;

memory_management_stmt:
                K_DELETE '$' S_IDENTIFIER ';' {
                    source *sp = new source;
                    /* Yes, limiting things this way is a hidious hack.
                     * No, I'm not implimenting delete for general
                     * primary_expressions's today.
                     * TBD: implement delete for primary_expressions
                     */
                    if (!ignore_event) {
                        if (event_set == NULL)
                            wdlerror("Fatal error: event is NULL");
                        if (event_set->empty()) 
                            wdlerror("Fatal error: event set empty");
                        mlabel *l, *l1, *l2; 
                        WrEvent *ev = essgrp.is_valid_rattr(event_set, $3, &l);
                        WrEvent *ev1 = essgrp.is_valid_attr(event_set, $3, &l1);
                        Param *p  = essgrp.is_valid_var(event_set, $3, &l2);
                        if (p != NULL || ev == NULL) {
                            // p  != NULL ==> $variable is an internal variable
                            // ev == NULL ==> $variable is a return value
                            wdlerror(
                                    "Delete support is limited to parameters.");
                            // All the arg-replacement code is limited
                            // to parameters now that I think about it.
                        } else if (ev1 == NULL) {
                            // $variable is a parameter 
                            // l1 is the mlabel of note.
                            source *ofsrc=essgrp.gen_attr_offset(event_set, $3);
                            sp->gen_argfree(l1,ofsrc->all());
                        } else 
                            wdlerror(
                                    "Attribute is invalid in event", ev1->name);
                    }
                    
		    $$ = sp;
                }
                ;

jump_stmt:	K_GOTO S_IDENTIFIER ';' { 
                    source *sp = new source;
		    SourceAppend3(sp, "goto ", $2, ";\n");
		    $$ = sp;
                }
		| K_CONTINUE ';' { 
                    int i;
                    stack<source *> temp;
                    source *sp = new source;
                    sp->append("{ ");
                    /* Fill up temp with the block's we'll need to duplicate
                     */
                    i = post_block_body_count;
                    while (i-- > last_loop_level.top())
                        temp.push(post_block_body.pop());
                    /* Add in the blocks */
                    while (!temp.is_empty()) {
                        if (!temp.top()->is_empty())
                            sp->append(temp.top()->all());
                        /* And put them back */
                        post_block_body.push(temp.pop());
                    }

		    sp->append("continue; }\n");
		    $$ = sp;
                }
		| K_BREAK ';' { 
                    source *sp = new source;
                    int i;
                    stack<source *> temp, temp2;
                    sp->append("{ ");
                    /* Fill up temp with the block's we'll need to duplicate
                     */
                    i = post_block_body_count;
                    while (i-- > last_loop_level.top())
                        temp.push(post_block_body.pop());
                    /* Add in the blocks */
                    while (!temp.is_empty()) {
                        if (!temp.top()->is_empty())
                            sp->append(temp.top()->all());
                        /* And put them back */
                        post_block_body.push(temp.pop());
                    }

		    sp->append("break; }\n");
		    $$ = sp;
                }
		| K_RETURN expression ';' { 
                    source *sp = new source;
                    source *retsp = new source;
                    source *rvsp = new source; 
                    source *errsp = new source;
                    int spec = 0;

                    if ((in_wrapper_action || in_sequence) && !in_pre)
                        wdlwarning("Returning value from a post-block");

                    char *ret = $2->code->all();
                    /* special handling */
                    if ((in_wrapper_action || in_sequence) && 
                         (spec = essgrp.special_case(event_set, $2->code, 
                                                 retsp, rvsp, errsp)) == 0) { 
                      if (in_pre) {
                         /* generating code for return value */
                         retsp->append(ret); 
                         /* generating code for retval assignment */
                         source *r1 = new source;
                         SourceAppend3(r1, " (", $2->code->all(), ") & WR_MASK2 ");
                         rvsp = essgrp.gen_rhs(event_set, "wrret", r1);
                         /* generating code for errno */
                         mlabel *l;
                         if (essgrp.is_valid_var(event_set, "errno",&l) != NULL) {
                             errsp->append(essgrp.gen_svar_assmt(l, r1));
                         }
                      }
                      else {
                         retsp->append("WR_ALLOW");
                         SourceAppend3(rvsp,"*retval = ",$2->code->all(),";");
                      }
                    }
                    if (!in_wrapper_action && !in_sequence) {
                        int i;
                        stack<source *> temp;
                        sp->append("{ ");
                        /* Fill up temp with the block's we'll need to duplicate
                         */
                        i = post_block_body_count;
                        while (i-- > last_function_level)
                            temp.push(post_block_body.pop());
                        /* Add in the blocks */
                        while (!temp.is_empty()) {
                            if (!temp.top()->is_empty())
                                sp->append(temp.top()->all());
                            /* And put them back */
                            post_block_body.push(temp.pop());
                        }

                        SourceAppend3(sp, "return ", $2->code, "; }\n");
                    } else if (!in_sequence) {
                      if (rvsp->is_empty()) { 
		          SourceAppend4(sp, "{\n", "   wrp_return = ", retsp, 
                                            ";\n");
                          if (in_pre) {
		               SourceAppend4(sp, "   goto preend_", pre_count, 
                                                 ";\n", "}\n");
                          } else {
		               SourceAppend4(sp, "   goto postend_", post_count,
                                                 ";\n", "}\n");
                          }
                      } else  {
                          rvsp->indent("         "); 
                          errsp->indent("         "); 
                          if (!spec) {
                              SourceAppend3(sp,"{  if (", $2->code->all(), 
                                                         " >> 2 != 0) {\n");
		              SourceAppend3(sp, rvsp, errsp, "   }\n");
                          } else {
                              SourceAppend3(sp, "{\n", rvsp, errsp); 
                          }
                          int i;
                          stack<source *> temp;
                          /* Fill up temp with the block's we need to duplicate
                           */
                          i = post_block_body_count;
                          while (i-- > last_function_level)
                              temp.push(post_block_body.pop());
                          /* Add in the blocks */
                          while (!temp.is_empty()) {
                              if (!temp.top()->is_empty())
                                  sp->append(temp.top()->all());
                              /* And put them back */
                              post_block_body.push(temp.pop());
                          }

                          if (in_pre) {
                             SourceAppend7(sp, 
                                  "   wrp_return = ", retsp, ";\n",
                                  "   goto preend_", pre_count, ";\n", 
                                  "}\n");
                          } else {
                             SourceAppend7(sp, 
                                  "   wrp_return = ", retsp, ";\n",
                                  "   goto postend_", post_count, ";\n", 
                                  "}\n");
                          }
                      }
		    } else {
                      if (rvsp->is_empty()) { 
		          SourceAppend3(sp, "{\n   sequence_return = ", retsp, 
                                                  ";\n");
		          SourceAppend5(sp, "   goto seqend_",sequence_count-1,
                                            "_", step_count, ";\n}\n");
                      } else  {
                          rvsp->indent("    "); 
                          errsp->indent("    "); 
                          SourceAppend7(sp,"{\n", rvsp, errsp, "    return ", 
                                            retsp, ";\n", "}\n");
                          if (!spec) {
                               source *sp1 = new source;
                               sp->indent("    ");
                               SourceAppend4(sp1, "{   if (", $2->code->all(),
                                                  " >> 2 != 0)\n", sp);
                          }
		          SourceAppend3(sp, "   sequence_return = ", retsp, 
                                                ";\n");
		          SourceAppend6(sp, "   goto seqend_",sequence_count-1,
				            "_", step_count, ";\n", "}\n");
                      }
		    }
		    $$ = sp;
                }
		| K_RETURN ';' { 
                    int i;
                    stack<source *> temp;
                    source *sp = new source;
                    sp->append("{ ");
                    /* Fill up temp with the block's we'll need to duplicate
                     */
                    i = post_block_body_count;
                    while (i-- > last_function_level)
                        temp.push(post_block_body.pop());
                    /* Add in the blocks */
                    while (!temp.is_empty()) {
                        if (!temp.top()->is_empty())
                            sp->append(temp.top()->all());
                        /* And put them back */
                        post_block_body.push(temp.pop());
                    }

		    sp->append("return; }\n");
		    $$ = sp;
                }
		
iterative_stmt:	  K_DO statement K_WHILE '(' expression ')' ';' {
                    source *sp = new source;
		    SourceAppend6(sp, "do\n", $2, "\n", "while (", 
                                  $5->code, ");\n");
		    $$=sp;
                  }
		| K_WHILE '(' expression ')' statement {
                    source *sp = new source;
                    $5->indent("   ");
		    SourceAppend4(sp, "while (", $3->code, ")\n", $5);
		    $$=sp;
                  }
		| K_FOR '(' expression_stmt expression_stmt ')' statement {
                    source *sp = new source;
                    $6->indent("   ");
		    SourceAppend5(sp, "for (", $3, $4, ")\n", $6);
		    $$=sp;
                  }
		| K_FOR '(' expression_stmt expression_stmt expression ')' 
		statement {
                    source *sp = new source;
                    $7->indent("   ");
		    SourceAppend6(sp, "for (", $3, $4, $5->code, ")\n", $7);
		    $$=sp;
                  }
		| K_TRY statement K_CATCH statement  {
                    source *sp = new source;
		    SourceAppend4(sp, "try ", $2, "catch ", $4);
		    $$=sp;
                  }

		;

conditional_stmt: K_IF '(' expression ')' statement	%prec LOWER_THAN_ELSE {
                    source *sp = new source;
                    $5->indent("    ");
		    SourceAppend4(sp, "if (", $3->code, ")\n", $5);
		    $$ = sp;
                  }
		| K_IF '(' expression ')' statement K_ELSE statement {
                    source *sp = new source;
                    $5->indent("    ");
		    SourceAppend6(sp, "if (", $3->code, ")\n", $5, "else ", $7);
		    $$=sp;
                  }
		;

compound_stmt:	  {  vstore.newtbl();
                      post_block_body.push(new source);
                      ++post_block_body_count; }
	 	  compound_stmt_1
		  {
                      source *sp = new source;
                      sp->append("{\n");
                      $2->indent("    ");
                      sp->append($2);
                      source *pbb = post_block_body.pop();
                      --post_block_body_count;
                      pbb->indent("    ");
                      sp->append(pbb);
                      sp->append("}\n");
                      delete pbb; delete $2;
                      vstore.deltop();  
                      $$ = sp;
		  }

compound_stmt_1:  '{' declaration_list statement_list '}' {
                    if (post_decl_block != NULL) {
                        $2->append(post_decl_block);
                        delete post_decl_block;
                        post_decl_block = NULL;
                    }
                    $2->append($3);
                    $$ = $2;
                    delete $3;
                  }
 		| '{' '}' {
                    source *sp = new source;
		    sp->append("/*Empty block*/");
		    $$=sp;
                  }
		| '{' statement_list '}' {
		    $$=$2;
                  }
		| '{' declaration_list '}' {
                    if (post_decl_block != NULL) {
                        $2->append(post_decl_block);
                        delete post_decl_block;
                        post_decl_block = NULL;
                    }
		    $$=$2;
                  }
		;

labeled_stmt:	S_IDENTIFIER ':' {
                    source *sp = new source;
		    SourceAppend2(sp, $1, ": ");
		    $$=sp;
                  }
		| K_CASE constant_expression ':' {
		    source *sp = new source;
		    SourceAppend3(sp, "case ", $2->code, ": ");
		    $$=sp;
		  }
		| K_DEFAULT ':' {
		    source *sp = new source;
                    sp->append("default: ");
		    $$=sp;
		  }
		;

expression_stmt: ';' {
		    source *sp = new source;
		    sp->append(";\n");
		    $$=sp;
		  }
		| expression ';' {
                    source *sp = new source;
		    SourceAppend2(sp, $1->code, ";\n");
		    $$=sp;
		  }
                | {
                    if (creation_instructions != NULL)
                        wdlerror("Internal compilier error WQL9\n");
                    creation_instructions = new source;
                    if (destruction_instructions == NULL)
                        wdlerror("Internal compilier error WQL10\n");
                    if (var_decls == NULL)
                        wdlerror("Internal compilier error WQL11\n");
                }
                wql_stmt ';' {
                    source *sp = new source;

                    sp->d_type = &intType;
                    creation_instructions->indent("    ");
                    $2->indent("    ");
                    SourceAppend5(sp,"{\n", "    int wr_retval;\n",
                           creation_instructions, $2,  "}\n");

                    $$ = sp;
                    delete $2; delete creation_instructions;
                    creation_instructions = NULL;
                }
		;

switch_stmt:	  K_SWITCH '(' expression ')'
		statement  {
		    source *sp = new source;
		    SourceAppend4(sp, "switch (", $3->code, ")", $5);
		    $$=sp;
		  }
		;

/* ================================================================= *
 *                          Expressions                              *
 * ================================================================= */

expression:	  assignment_expression
		| expression ',' assignment_expression  {
                     SourceAppend2($1->code, ", ", $3->code);
                     Expr *e = $1;
                     while (e->next != NULL) e = e->next;
                     e->next = $3;
		     $$ = $1;
		  }
		;

assignment_expression: 
		unary_expression assignment_op assignment_expression  
        	{   Expr *e = new AssmtExpr($2, $1, $3);
            	    SourceAppend3(e->code, $1->code, $2, $3->code);
	    	    e->type = $1->type;
	            $$ = e;
	 	}
                | unary_expression '='
                {
                    if (creation_instructions != NULL)
                        wdlerror("Internal compilier error WQL9\n");
                    creation_instructions = new source;
                    if (destruction_instructions == NULL)
                        wdlerror("Internal compilier error WQL10\n");
                    if (var_decls == NULL)
                        wdlerror("Internal compilier error WQL11\n");
                }
                wql_stmt
                {
        	    Expr *e = new AssmtExpr("=", $1, 
                                            new ConstExpr("WQL", intType));
                    char buf[256];
                    if ($1->type.assmt_chk(intType, buf) != 1)
                         wdlerror(buf);
                    if ($1->lvalue != 1)
                         wdlerror("Invalid lvalue in assignment");
                    e->type = intType;
                    e->lvalue = 0;

                    $4->indent("    ");
                    creation_instructions->indent("    ");
                    SourceAppend8(e->code,"{\n","    int wr_retval;\n",
                            creation_instructions, $4,
                            "(", $1->code, ") = wr_retval;\n", "}\n");
                
                    $$ = e;

                    delete $1; delete $4; delete creation_instructions;
                    creation_instructions = NULL;
                }
        	| unary_expression '=' assignment_expression  
                  {  Expr *e = new AssmtExpr("=", $1, $3);
                     char buf[256];
                     if ($1->type.assmt_chk($3->type, buf)!=1 && !ignore_event) 
                         wdlerror(buf);
                     if ($1->lvalue != 1)
                         wdlerror("Invalid lvalue in assignment");
                     e->type = $1->type;

		     /* If we assign to or from a $foo variable, we have
		      * likely changed it.  Remember to write the new
		      * version out.
		      */
                     list<SpecIdentExpr *>*e1 = $1->get_siexpr();
                     list<SpecIdentExpr *>*e2 = $3->get_siexpr();
		     list<SpecIdentExpr *>siexpr_list;
		     if (e1 != NULL)
			 siexpr_list.insert(siexpr_list.end(),
				 e1->begin(), e1->end());
		     if (e2 != NULL)
			 siexpr_list.insert(siexpr_list.end(),
				 e2->begin(), e2->end());
		     list<SpecIdentExpr*>::iterator li;
		     for (li=siexpr_list.begin();li!=siexpr_list.end();++li) {
			 if (*li != NULL && (*li)->idtype == Expr::Para
				 && (*attrtbl)[(*li)->name] != NULL)
			     (*attrtbl)[(*li)->name]->write = 1;
                     }
		     /* Done that search */


                     char *lhs = $1->code->all();
                     if ($1->type.tc == TC_STR) {
                          SourceAppend6(e->code, lhs, " = wr_str_assign(", 
                                                 $1, ", ", $3, ")");
                     } else if ($1->type.tc == TC_DBUF) {
                          if ($3->type.is_strtype()) {
                               SourceAppend6(e->code, lhs, 
                                             " = buf_replace_str(",
                                             $1, ", ", $3, ")");
                          } else {
                               SourceAppend6(e->code, lhs," = buf_replace(",
                                             $1, ", ", $3, ")");
                          } 
                     } else {
                          SourceAppend3(e->code, $1, " = ", $3);
                     }
                     free (lhs);
                     $$ = e;
                  }
		| unary_expression K_STRMATCH S_IDENTIFIER '|' 
                  string_expression '|' op_string_expression
                  {   Expr *e = new TrinaryExpr($3, $1, $5, $7);
                      if (!strcmp($3, "m")) {
		          if (!$1->type.is_strtype()) {
                               wdlerror("string or char * expected in lhs"); 
                          } 
                          SourceAppend5(e->code, "wr_match(", $1->code, ", ", 
                                        $5->code, ", &gstr_beg, &gstr_end)");
                          e->type = intType;
                      } else if (!strcmp($3, "s")) {
                          if (!$1->type.is_strtype())
                               wdlerror("string expected in lhs"); 
                          if ($7 == NULL)
                               wdlerror("Error in String Substitute Expr\n");

			  /* Remember to write the new string out. */
			  list<SpecIdentExpr *>*e1 = $1->get_siexpr();
			  if (e1 != NULL) {
			      list<SpecIdentExpr*>::iterator li;
			      for (li = e1->begin(); li != e1->end(); ++li) {
				  if (*li != NULL && (*li)->idtype == Expr::Para
					  && (*attrtbl)[(*li)->name] != NULL)
				      (*attrtbl)[(*li)->name]->write = 1;
			      }
			  }

                          SourceAppend7(e->code, "wr_substitute(&", $1, 
                                        ", ", $5, ", ", $7, ")");
                          e->type = intType;
                      } else {
                          wdlerror("Error in String Substitute Expr\n");
                      }
                      $$ = e;
		  }
		| conditional_expression
		;

		 
string_expression: 
		  additive_expression {
                      if (!$1->type.is_strtype()) {
                          wdlerror($1->str(), ": string or char * expected"); 
                      } 
                      $$ = $1;
                  }
		;

op_string_expression:
		  string_expression '|' {
                       $$ = $1;
                  }
		| {    $$ = NULL;         }
		;


assignment_op:    K_PLUSEQ  {$$="+=";}
                | K_MINUSEQ {$$="-=";}
                | K_MULTEQ  {$$="*=";}
                | K_DIVEQ   {$$="/=";}
                | K_MODEQ   {$$="%=";}
		| K_SHFTLEQ {$$="<<=";} 
                | K_SHFTREQ {$$=">>=";} 
                | K_ANDEQ   {$$="&=";} 
                | K_OREQ    {$$="|=";} 
                | K_XOREQ   {$$="^=";}
		;

unary_expression: postfix_expression
		| '*' cast_expression {
                    Expr *e = new UnaryExpr("*", $2);
                    SourceAppend2(e->code, "*", $2->code);
                    if ($2->type.tc != TC_PTR)
                        wdlerror("Pointer type expected for ", $2->str());
                    e->type = $2->type.get_elmtype(); 
                    e->lvalue = 1;		
		    $$ = e;
                  }
 		| '&' cast_expression {
                    Expr *e = new UnaryExpr("&", $2);
                    SourceAppend2(e->code, "&", $2->code);
                    e->type = Type(TC_PTR, new PtrTypeInfo($2->type));
	            e->lvalue = 0;
		    $$ = e;
                  }
		| unary_op cast_expression {
                    Expr *e = new UnaryExpr($1, $2);
                    SourceAppend2(e->code, $1, $2->code);
                    e->type = $2->type;
	            e->lvalue = 0;
		    $$ = e;
		  }
		| K_PLUSPLUS unary_expression {
                    Expr *e = new UnaryExpr("++", $2);
                    SourceAppend2(e->code, "++", $2->code);
                    e->type = $2->type;
	            e->lvalue = $2->lvalue;		
		    $$ = e;
		  }
		| K_MINUSMINUS unary_expression {
                    Expr *e = new UnaryExpr("--", $2);
                    SourceAppend2(e->code, "--", $2->code);
                    e->type = $2->type;
	            e->lvalue = $2->lvalue;		
		    $$ = e;
		  }
		| K_SIZEOF '(' type_name ')' {
                    Expr *e = new UnaryExpr("sizeof", new IdentExpr($3->all()));
                    SourceAppend3(e->code, "sizeof (", $3, ")");
                    e->type = intType;
	            e->lvalue = 0;		
		    $$ = e;
		  }
		| K_SIZEOF unary_expression {
                    Expr *e = new UnaryExpr("sizeof", $2);
                    SourceAppend2(e->code, "sizeof ", $2->code);
                    e->type = intType;
	            e->lvalue = 0;		
		    $$ = e;
		  }
		;

unary_op:	  '+' {$$="+";}
                | '-' {$$="-";}
                | '~' {$$="~";}
                | '!' {$$="!";}
		;

postfix_expression: primary_expression
                | postfix_expression '[' expression ']' {
                    Expr *e = new ArrayElmExpr($1, $3);
                    if (!in_essdef) {
                       if ($1->type.tc == TC_ARRAY || $1->type.tc == TC_PTR
                          || $1->type.tc == TC_STR) {
                           e->type = $1->type.get_elmtype();
                           /* hack hack hack 
                              This checks whether the postfix_expression
                              is a $ret variable and its type is int * or
                              int[]. If so, generate the correct
                              code retval[$3] instead or *retval[*3]. 
                            */
                           if (e->type == intType &&
                               $1->etype() == Expr::SpecIdent &&
                               ((SpecIdentExpr *)$1)->idtype == Expr::Ret) 
                           {
                                SourceAppend3(e->code, "retval [", $3,"]");
                           } else {
                                SourceAppend4(e->code, $1, " [", $3,"]");
                           }
                       } else if ($1->type.tc == TC_DBUF) {
                           e->type = charType;
                           SourceAppend5(e->code, "*buf_getdataptr(", $1, 
                                                  ", ", $3, ")");
                       } else                
                           wdlerror($1->str(), "is not an array, ptr, or dbuf");
                    } else {
                       SourceAppend4(e->code, $1, " [", $3, "]");
                    }
		    $$ = e;
                  }             
                | function_call {
                    Expr *e = $1;
                    $$ = e;
                }
		| postfix_expression '.' S_IDENTIFIER {
                    Expr *e = new RecElmExpr($1, $3);
                    if (!in_essdef) {
                       if ($1->type.is_rectype()) {
                          e->type = $1->type.get_elmtype($3);
                          if (e->type.tc == TC_ERR)
                              wdlerror($3, "is not a member of struct/union");
                          SourceAppend3(e->code, $1->code, ".", $3);
		       } else if ($1->type.tc == TC_DBUF) {
                          if (!strcmp($3, "size")) 
                              wdlerror($3, "is not an attribute of databuf_t");
                          SourceAppend3(e->code, "buf_size(", $1->code, ")");
                          e->type = intType;
	                  e->lvalue = 0;		
                       } else if ($1->type.tc == TC_DBROW) { 
                          e->type = $1->type.get_elmtype($3);
                          if (e->type.tc == TC_ERR)
                              wdlerror($3, "is not a member of dbrow");
                          else if (e->type.tc == TC_FUNC) {
                              /* three special cases:
                               * isempty(), next() and start().
                               */
                              SourceAppend5(e->code,"dbrow_", $3, "(&", $1,")");
                              /* map to call to dbrow_next(), or
                               * dbrow_isempty(), or dbrow_start() */
                          } else {
                              TableTypeInfo *tti =
                                  ((DBrowTypeInfo *)$1->type.info)->tti;
                              int column = (*tti->order)[$3];
                              SourceAppend6(e->code, "(", $1->code, ")",
                                      ".cur->vals[", column, "].v.");
                              if (e->type.is_inttype()) {
                                  e->code->append("ival");
                              } else if (e->type.is_floattype()) {
                                  e->code->append("fval");
                              } else if (e->type.is_strtype()) {
                                  e->code->append("cval");
                              } else {
                                  wdlerror("Invalid Type.\n");
                              }
                          }
                       } else
                         wdlerror($1->str(), "is not a struct/union/dbrow");
                    }
		    $$ = e;
		  }
		| postfix_expression K_INDIRECT S_IDENTIFIER {
                    Expr *e = new PtrToElmExpr($1, $3);
                    if (!in_essdef) {
                       if ($1->type.is_ptrtorec()) {
                          e->type = $1->type.get_elmtype().get_elmtype($3);
                          if (e->type.tc == TC_ERR)
                              wdlerror($3, "is not a member of struct/union");
                       } else 
                          wdlerror($1->str(), "is not a ptr to struct/union");
                    }
                    SourceAppend3(e->code, $1->code, "->", $3);
		    $$ = e;
		  }
		| postfix_expression K_PLUSPLUS {
                    Expr *e = new PostFixExpr("++", $1);
                    SourceAppend2(e->code, $1->code, "++");
                    e->type = $1->type;
		    $$ = e;
		  }
		| postfix_expression K_MINUSMINUS {
                    Expr *e = new PostFixExpr("--", $1);
                    SourceAppend2(e->code, $1->code, "--");
                    e->type = $1->type;
		    $$ = e;
		  } 
		;

expression_list	: assignment_expression {
                    /* Misuse the paren count to get a size-of-list count */
                    $1->code->paren_count = 1;
                    $$ = $1->code;
                  }
                | expression_list ',' assignment_expression {
                    /* Misuse the paren count to get a size-of-list count */
                    ++$1->paren_count;
                    SourceAppend2($1, ",", $3->code);
		    $$ = $1;
                    delete $3;
                  }		    
		;

primary_expression: 
                  K_OP {
                      /* Hack, since only the parser can recognize that
                       * OP is not a valid token in the current context.
                       */
                    Expr *e = new IdentExpr("op");
                    if (!in_essdef) {
                         VarNode *vn = vstore["op"];
                         if (vn == NULL) 
                              wdlerror("op", ": variable is not declared");
                         e->type = vn->type;
                    }
                    e->code->append("op");
		    $$ = e;
                  }
		  | S_IDENTIFIER {
                    Expr *e = new IdentExpr($1);
                    int seqvar = 0;
                    int wivvar = 0;
                    if (!in_essdef) {
                         VarNode *vn = vstore[$1];
                         if (vn == NULL) 
                              wdlerror($1, ": variable is not declared");
                         /* It might be a WIV variable.  Test for that h
                          * here and if so, generate appropriate code.
                          */
                         if (dflag) {
                              cerr << vn->name << ": " << vn->type << "\n";
                              cerr << vn->name << ":level = " 
                                   << vstore.get_scopelevel(vn->name) << "\n";  
                         }
                         e->type = vn->type;
			 seqvar = vn->seqvar;
                         wivvar = vn->sflag & WIV_VAR;
                    }
		    if (seqvar) {
                        SourceAppend3(e->code, "(my_seq_data->", $1, ")");
                    } else if (wivvar) {
                        if (in_no_wiv_scope) 
                            wdlerror("WIVs not allow in scope", $1);
                        else {
                            SourceAppend3(e->code, "WSS_GET_WIV_VAR(",$1,")");
                        }
                    } else 
		        e->code->append($1);
		    $$ = e;
		  }
		| '$' S_IDENTIFIER { 
                    if (ignore_event) {
                        SpecIdentExpr *e = new SpecIdentExpr($2);
                        e->offset = NULL;
                        e->type = errType;
                        e->idtype = (Expr::IdType)-1;
                        e->label = NULL;
                        $$ = e;
                    } else {
                        if (event_set == NULL)
                            wdlerror("Fatal error: event is NULL");
                        if (event_set->empty()) 
                            wdlerror("Fatal error: event set empty");
                        SpecIdentExpr *e = new SpecIdentExpr($2);
                        mlabel *l, *l1, *l2; 
                        WrEvent *ev = essgrp.is_valid_rattr(event_set, $2, &l);
                        WrEvent *ev1 = essgrp.is_valid_attr(event_set, $2, &l1);
                        Param *p  = essgrp.is_valid_var(event_set, $2, &l2);
                        /* Here, we use sp->offset is to indicate whether the
                         * $var is a para, retval, or a variable. (a hack that
                         * should be changed later) 
                         */ 
                        if (p != NULL) {
                            // $variable is an internal variable
                            e->offset = NULL;
                            e->type = *p->d_type;
                            /* SpecIdentExpr's know how to make their own
                             * source when get_src() is called. */
                            e->label = l2;
                            e->idtype = Expr::InVar;
                        } else if (ev == NULL) {
                            // $variable is a return value
                            e->offset = NULL;
                            WrEvent *ev = essgrp.get_event(*event_set->begin());
                            e->type = *ev->ret->d_type;
                            /* SpecIdentExpr's know how to make their own
                             * source when get_src() is called. */
                            e->label = l;
                            e->idtype = Expr::Ret;
                            if (dflag) 
                                cerr << $2 << " type " << e->type << "\n";
                        } else if (ev1 == NULL) {
                            // $variable is a parameter 
                            WrEvent *ev = essgrp.get_event(*event_set->begin());

                            source *ofsrc=essgrp.gen_attr_offset(event_set, $2);
                            e->offset = ofsrc->all();

                            Type *ptype = ev->find_plabeltype($2);
                            if (ptype->is_ptrtype()) {
                                attrtbl->insert($2, new AttrNode(l1,e->offset));
                            }
                            e->type = *ptype;
                            /* find_plabeltype() takes into consideration the
                             * type() markup specifier (and later string types)
                             */
                            /* SpecIdentExpr's know how to make their own
                             * source when get_src() is called. */
                            e->label = l1;
                            e->idtype = Expr::Para;
                            if (dflag) 
                                cerr << $2 << " type " << e->type << "\n";
                        } else 
                            wdlerror("Attribute is invalid in event",ev1->name);

                        /* Let the SpecIdentExpr generate its code. */
                        e->code = e->get_src();
                        $$ = e;
                    }
                }
		| '$' '#' {
                    Expr *e = new ConstExpr("$#", intType);
                    e->code->append("code"); 
		    $$ = e;
                  }
		| '$' '$' { 
                    Expr *e = new ConstExpr("$$", cstrType);
                    e->code->append("(wr_syscall_name(code))"); 
		    $$ = e;
		  }
		| '(' expression ')' 
                  { source *sp = new source; 
		    SourceAppend3(sp, "(", $2, ")");
                    delete ($2->code);
                    $2->code = sp;
		    $$ = $2;
		  }
		| constant
		;

constant:	 S_NUMCONST  {
                        Expr *e = new ConstExpr($1, intType);
                        e->code->append($1);
#if 0
#error                  IM(ns)HO, This Is The Wrong Answer(tm) \
                        Talk to me before putting this back in. \
                        DSK
                        if (!strcmp($1, "0"))
			    e->type = vptrType;
#endif
			$$ = e;
		  }
		| S_FLOATCONST {
			Expr *e = new ConstExpr($1, floatType);
			e->code->append($1);
			$$ = e;
		  }
		| S_CHARCONST  {
			Expr *e = new ConstExpr($1, charType);
			e->code->append($1);
			$$ = e;
		  }
		| string_list {
                        Expr *e = new ConstExpr(((ConstExpr*)$1)->val,carrType);
                        SourceAppend3(e->code, "PD_MAKESTR(", $1, ")");
			$$ = e;
			delete $1;
                  }
		;

string_list:
                S_QSTR string_list {
                        char *buffer;
                        int len1, len2, len;
                        len1 = strlen($1) - 1 ; /*Trailing ""*/
                        len2 = strlen(((ConstExpr*)$2)->val) - 1; /*leading "*/
                        len  = len1 + len2 + 1 /* '\0' */;
                        buffer = (char*)malloc(len);
                        strncpy(buffer,      $1,                      len1);
                        strncpy(buffer+len1, ((ConstExpr*)$2)->val+1, len2);
                        buffer[len-1] = '\0';

                        Expr *e = new ConstExpr(buffer, carrType);
                        e->code->append(buffer);
			$$ = e;
                        
                        free(buffer);
                        delete $2;
		  }
                | S_QSTR {
                        Expr *e = new ConstExpr($1, carrType);
                        e->code->append($1);
			$$ = e;
		  }
                ;

conditional_expression: 
		  logical_or_expression
		| logical_or_expression '?' expression ':' 
                  conditional_expression 
                  {
                     Expr *e = new TrinaryExpr("?", $1, $3, $5);
		     SourceAppend5(e->code, $1, " ? ", $3, " : ", $5);
                     if (!$1->type.is_inttype()) 
                          wdlerror("int type expected for", $1->str());
                     if (!$3->type.equop_chk($5->type))
                          wdlerror("equal type expected for 2nd &3rd operand");
                     e->type = $3->type;
		     $$ = e;
		  }
		| logical_or_expression '?' ':' conditional_expression 
                  {
                     Expr *e = new TrinaryExpr("?", $1, $1, $4);
		     SourceAppend3(e->code, $1, " ? : ", $4);
                     if (!$1->type.is_inttype() && !$1->type.is_ptrtype()) 
                          wdlerror("int type expected for", $1->str());
                     if (!$1->type.equop_chk($4->type))
                          wdlerror("equal type expected for 2nd &3rd operand");
                     e->type = $1->type;
		     $$ = e;
		  }

		;

logical_or_expression: 
		  logical_and_expression
		| logical_or_expression K_OR logical_and_expression {
                    Expr *e = new BinaryExpr("||", $1, $3);
                    if (!in_essdef) {
                        if (!$1->type.logop_chk($3->type))
                            wdlerror("Invalid operands to binary operator ||");
                        e->type = intType;
                    }
		    SourceAppend3(e->code, $1->code, " || ", $3->code);
		    $$ = e;
		  }
		;

logical_and_expression: 
		  bitwise_or_expression
		| logical_and_expression K_AND bitwise_or_expression {
                    Expr *e = new BinaryExpr("&&", $1, $3);
                    if (!in_essdef && typechk) {
                        if (!$1->type.logop_chk($3->type))
                            wdlerror("Invalid operands to binary operator &&");
                        e->type = intType;
                    }
		    SourceAppend3(e->code, $1->code, " && ", $3->code);
		    $$ = e;
		  }

bitwise_or_expression: 
		  bitwise_xor_expression
		| bitwise_or_expression '|' bitwise_xor_expression  {
                    Expr *e = new BinaryExpr("|", $1, $3);
                    if (!in_essdef) {
                        if (!$1->type.bitop_chk($3->type))
                            wdlerror("Invalid operands to binary operator |");
                        e->type = $1->type;
                    }
		    SourceAppend3(e->code, $1->code, " | ", $3->code);
		    $$ = e;
		  }
		;

bitwise_xor_expression: 
		  bitwise_and_expression
		| bitwise_xor_expression '^' bitwise_and_expression  {
                    Expr *e = new BinaryExpr("^", $1, $3);
                    if (!in_essdef) {
                        if (!$1->type.bitop_chk($3->type))
                            wdlerror("Invalid operands to binary operator ^");
                        e->type = $1->type;
                    }
		    SourceAppend3(e->code, $1->code, " ^ ", $3->code);
		    $$ = e;
		  }
		;

bitwise_and_expression: 
                  equality_expression
		| bitwise_and_expression '&' equality_expression  {
                    Expr *e = new BinaryExpr("&", $1, $3);
                    if (!in_essdef) {
                        if (!$1->type.bitop_chk($3->type))
                            wdlerror("Invalid operands to binary operator &");
                        e->type = $1->type;
                    }
		    SourceAppend3(e->code, $1->code, " & ", $3->code);
		    $$ = e;
		  }
		;

equality_expression: 
                  relational_expression
		| equality_expression equality_op relational_expression  {
                    Expr *e = new BinaryExpr($2, $1, $3);
                    if (!in_essdef) {
                       if (!$1->type.equop_chk($3->type))
                            wdlerror("Invalid operands to binary operator", $2);
                       e->type = intType;
                       if ($1->type.tc == TC_STR || $3->type.tc == TC_STR) {
                           if (!strcmp($2, "!="))
                               e->code->append("!");
                           SourceAppend5(e->code, "wr_str_equ(", $1, ", ", 
                                                                 $3, ")");
		       } else {
                           SourceAppend3(e->code, $1, $2, $3);
                       }
                    } else {
                       SourceAppend3(e->code, $1, $2, $3);
                    } 
		    $$ = e;
		  }
		;

equality_op:	  K_EQEQ {$$="==";}
                | K_NOTEQ {$$="!=";}
		;

relational_expression: 
		  shift_expression
		| relational_expression relational_op shift_expression  {
                    Expr *e = new BinaryExpr($2, $1, $3);
                    if (!in_essdef) {
                       if (!$1->type.relop_chk($3->type))
                            wdlerror("Invalid operands to binary operator", $2);
		       SourceAppend3(e->code, $1, $2, $3);
                       e->type = intType;
                    } else {
		       SourceAppend3(e->code, $1, $2, $3);
                    }
		    $$ = e;
		  }
		;

relational_op:	 '<'     {$$="<";}
                | K_LTEQ {$$="<=";}
                | '>'    {$$=">";}
                | K_GTEQ {$$=">=";}
		;

shift_expression: additive_expression
		| shift_expression shift_op additive_expression  {
                    Expr *e = new BinaryExpr($2, $1, $3);
                    if (!in_essdef) {
                        if (!$1->type.shfop_chk($3->type))
                            wdlerror("Invalid operands to binary operator", $2);
                        e->type = $1->type;
                    }
		    SourceAppend3(e->code, $1, $2, $3);
		    $$ = e;
		  }
		;

shift_op:	  K_SHFTL {$$="<<";}
                | K_SHFTR {$$=">>";}
		;

additive_expression: 
		  multiplicative_expression
		| additive_expression '+' multiplicative_expression  {
                    Expr *e = new BinaryExpr("+", $1, $3);
                    if (!in_essdef) {
                       e->type = $1->type.addop_chk($3->type);
                       if (e->type == errType)
                           wdlerror("Invalid operands to binary operator +");
                       if ($1->type.is_strtype() && $3->type.is_strtype()) {
                           SourceAppend5(e->code, "wr_str_add(", $1, ", ", 
                                                                 $3, ")");
                       } else {
 		           SourceAppend3(e->code, $1, " + ", $3);
                       } 
                    } else {
		       SourceAppend3(e->code, $1, " + ", $3);
                    }
		    $$ = e;
		  }
		| additive_expression '-' multiplicative_expression  {
                    Expr *e = new BinaryExpr("-", $1, $3);
		    SourceAppend3(e->code, $1->code, " - ", $3->code);
                    if (!in_essdef && typechk) {
                       e->type = $1->type.subop_chk($3->type);
                       if (e->type == errType)
                           wdlerror("Invalid operands to binary operator -");
                    }
		    $$ = e;
		  }
		;

multiplicative_expression: cast_expression
		| multiplicative_expression '*' cast_expression  {
                    Expr *e = new BinaryExpr("*", $1, $3);
		    SourceAppend3(e->code, $1->code, " * ", $3->code);
                    if (!in_essdef) {
                       e->type= $1->type.mulop_chk($3->type);
                       if (e->type == errType)
                           wdlerror("Invalid operands to binary operator *");
                    }
		    $$ = e;
		  }
		| multiplicative_expression '/' cast_expression {
                    Expr *e = new BinaryExpr("/", $1, $3);
		    SourceAppend3(e->code, $1->code, " / ", $3->code);
                    if (!in_essdef) {
                       e->type= $1->type.mulop_chk($3->type);
                       if (e->type == errType)
                           wdlerror("Invalid operands to binary operator /");
                    }
		    $$ = e;
		  }
		| multiplicative_expression '%' cast_expression {
                    Expr *e = new BinaryExpr("%", $1, $3);
		    SourceAppend3(e->code, $1->code, " % ", $3->code);
                    if (!in_essdef) {
                        if (!$1->type.modop_chk($3->type))
                            wdlerror("Invalid operands to binary operator %");
                        e->type = $1->type;
                    }
		    $$ = e;
		  }
		;

cast_expression:  unary_expression
                | '(' type_name ')' cast_expression {
                    source *sp = new source;
                    $4->type = *$2->d_type;
		    SourceAppend4(sp, "(", $2, ")", $4);
                    $4->code = sp;
		    $$ = $4;
                  }

type_name:	  declaration_spec abstract_declarator {
                    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
                    if (typechk) {
                        $2->d_type->fillin_type(*$1->d_type);
                        sp->d_type = $2->d_type;
                    }
		    $$ = sp;
                  }
		| declaration_spec
		;

abstract_declarator: pointer
		| direct_abstract_declarator
		| pointer direct_abstract_declarator  {
                     source *sp = new source;
		     SourceAppend3(sp, $1, " ", $2);
                     if (typechk) {
                         $2->d_type->fillin_type(*$1->d_type);
                         sp->d_type = $2->d_type;
                     }
		     $$ = sp;
                  }
		;

direct_abstract_declarator: 
 		  '(' abstract_declarator ')'  {
                    source *sp = new source;
		    SourceAppend3(sp, "(", $2, ")");
                    sp->d_type = $2->d_type;
		    $$=sp;
                  }
		| direct_abstract_declarator '[' constant_expression ']' {
                    source *sp = new source;
		    SourceAppend4(sp, $1, " [", $3, "]");
                    if (typechk) {
                        $1->d_type->fillin_type(Type(TC_ARRAY,
                             new ArrayTypeInfo(tmpType, $3->str())));
                        sp->d_type = $1->d_type;
                    }
		    $$ = sp;
                  }
		| direct_abstract_declarator '[' ']' {
                    source *sp = new source;
		    SourceAppend2(sp, $1, "[]");
                    if (typechk) {
                        $1->d_type->fillin_type
                                 (Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
                        sp->d_type = $1->d_type;
                    }
		    $$ = sp;
                  }
		| direct_abstract_declarator '(' parameter_type_list ')' {
                    source *sp = new source;
		    SourceAppend4(sp, $1, " (", $3, ")");
                    sp->d_type = new Type(TC_FUNC, new FuncTypeInfo(*$1->d_type));
		    $$=sp;
                  }
		| direct_abstract_declarator '(' ')' {
                    source *sp = new source;
		    SourceAppend2(sp, $1, " ()");
                    sp->d_type = new Type(TC_FUNC, new FuncTypeInfo(*$1->d_type));
		    $$=sp;
                  }
		| '[' constant_expression ']' {
                    source *sp = new source;
		    SourceAppend3(sp, "[", $2, "]");
                    sp->d_type = new Type(TC_ARRAY,
                              new ArrayTypeInfo(tmpType, $2->str()));
		    $$=sp;
                  }
		| '[' ']' {
                    source *sp = new source;
		    sp->append("[]");
                    sp->d_type = new Type(TC_ARRAY,new ArrayTypeInfo(tmpType));
		    $$=sp;
                  }
		| '(' parameter_type_list ')' {
                    source *sp = new source;
		    SourceAppend3(sp, "(", $2, ")");
                    sp->d_type = new Type(TC_FUNC, new FuncTypeInfo(intType));
		    $$=sp;
                  }
		| '(' ')' {
                    source *sp = new source;
		    sp->append("()");
                    sp->d_type = new Type(TC_FUNC, new FuncTypeInfo(intType));
		    $$=sp;
                  }
		;

parameter_type_list: parameter_list
		| parameter_list ',' K_ELIPSIS  {
                    source *sp = new source;
		    SourceAppend2(sp, $1, ", ...");
		    $$=sp;
                  }
		;

parameter_list	: parameter_list ',' parameter_declaration   {
                    source *sp = new source;
		    SourceAppend3(sp, $1, ",", $3);
		    $$=sp;
                  }
		| parameter_declaration
		;

parameter_declaration: declaration_spec declarator   {
                    $2->d_type->fillin_type(*$1->d_type);
                    VarNode *vn = new VarNode($2->vname, *$2->d_type);
                    if (!vstore.toptbl()->insert($2->vname, vn)) {
                        wdlerror("Duplicated definition of identifier");
                    }
                    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
		    $$=sp;
                  }
		| declaration_spec abstract_declarator   {
                    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
		    $$=sp;
                  }
                | declaration_spec {
		    source *sp = new source;
		    sp->append($1);
		    $$=sp;
		  }
		;

constant_expression: conditional_expression
		;

/* ================================================================= *
 *                          Declarations                             *
 * ================================================================= */

declaration_list: declaration
                | declaration_list declaration {
		    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
                    delete $1; delete $2;
		    $$=sp;
                  }
		;

declaration:      declaration_spec ';' {
                      if (in_includes) {
                          delete $1;
                          $$ = new source;
                      } else {
                          source *sp = new source;
                          /* if (in_uses) 
                             wdlerror("missing variable or function name");
                           */
                          SourceAppend2(sp, $1, ";\n");
                          $$ = sp;
                      }
                  }
                | declaration_spec initialized_decl_list ';' {
                    Expr *e;
                    for (e = $2; e != NULL; e = e->next) {
                        DeclExpr *de = (DeclExpr *) e;
                        if (typechk) {
                           de->type.fillin_type(*$1->d_type);
                           int sf = curvar_wivvar ? WIV_VAR : 0;
                           curvar_wivvar = 0;
                           sf |= ($1->find("global") ? GLOBAL_TABLE : EMPTY);
                           sf |= ($1->find("local") ? LOCAL_TABLE : EMPTY);
                           if ((sf & GLOBAL_TABLE) && (sf & LOCAL_TABLE)) {
                               wdlerror("Accessing wrapper-instance tables "
                                        "of other wrappers is not allowed.\n");
                           }
                           VarNode *vn = new 
                              VarNode (de->vname, de->type, sf);
			   vn->seqvar = in_sequence_decl; /* is it a seqvar? */
                           if (!vstore.toptbl()->insert(de->vname, vn)) {
                               /* Might be an error.  Might not.
                                * 
                                * If the two identifiers have the exact
                                * same type signature, its not an error.
                                */
                               if (!(*vstore.toptbl())[de->vname]->
                                       type.equiv(vn->type))
                                   wdlerror(
                                           "Duplicated definition"
                                           "of identifier");
                           }
                           /* If its a DBROW variable, add the code to 
                            * clean it up automatically when it goes out
                            * of scope.
                            */
                           if (de->type.tc == TC_DBROW && !in_includes) {
                               post_block_body.top()->append("dbrow_finish(&");
                               post_block_body.top()->append(de->vname);
                               post_block_body.top()->append(");\n");
                           }
                        }
                        if (dflag) {
                           cerr << "Insert Symbol " << de->vname;
                           cerr << "  " << de->type << "\n";
                        }
                    }
		    source *sp = new source;
                    if (in_includes) {
                    } else if (in_uses) {
                       for (e = $2; e != NULL; e = e->next) {
                          DeclExpr *de = (DeclExpr *) e;
                          if (de->type.tc == TC_FUNC) {
                              SourceAppend4(link_declarations, $1, " ",
						               de, ";\n");
                          } else {
                              SourceAppend5(link_declarations, "\t",
			  		     $1, " ", de, ";\n");
                          }
		       }
		    } else if ($1->d_type->tc == TC_DBUF) {  /* databuf_t */
                       sp->append($1);
                       for (e = $2; e != NULL; e = e->next) {
                          DeclExpr *de = (DeclExpr *) e;
			  if (e != $2) sp->append(", ");
			  sp->append(" ");
			  sp->append(de->vname);
                          if (de->e == NULL) {
                              sp->append(" = buf_new()");    
                          } else {
                              SourceAppend3(sp, " = buf_newstr(", de->e, ")");
                          }
                          SourceAppend3(post_block_body.top(),
                                        "buf_del(", de->vname, ");\n");
                       };
		       sp->append(";\n");
                    } else if ($1->d_type->tc != TC_DBTABLE) { /* wrapper */
                       /* Normal variables, + DBROW's */
                       /* If the variable is a WIV, add it to the structure.
                        */
                        DeclExpr *de = (DeclExpr *)$2;
                        char *varName = de->vname;
                        VarNode *vn = vstore[varName];
                        source *destination;
                        if (vn && (vn->sflag & WIV_VAR)) {
                            if (wiv_source == NULL)
                                wiv_source = new source;
                            destination = wiv_source;
                            if (!wiv_variables.insert(varName,vn)) {
                                wdlerror("Duplicate WIV variable.");
                            }
                            // printf("WIV variable %s\n",varName);
                        } else if (vstore.get_scopelevel() == 1) {
                            destination = global_vars;
                        } else {
                            destination = sp;
                        }
                        SourceAppend2(destination, $1, " ");
                        for (e = $2; e != NULL; e = e->next) {
                            de = (DeclExpr *) e;
                            if (e != $2) destination->append(", ");
                            destination->append(de);
                            if ($1->d_type->tc == TC_DBROW) {
                                /* Hidious Hack!
                                 * Assumes that DBROW=foo is never done...
                                 * should be a safe bet considering that
                                 * makes no sence.
                                 */
                                destination->append(" = {{0,0,0,0,0,0,0,0},0}");
                            }
                        } 
                        destination->append(";\n");
		    }
                    else {
                        /* DBTABLE variables get replaced by
                         * unsigned int's of the same name.  The 
                         * DBTABLE holds a reference to the table.
                         * 
                         * Don't initialize the reference.  Create will
                         * initialize it...  
                         * Eventually will add code to initialize them
                         * all in a wrapper init proc if declared in
                         * global space.
                         * 
                         * If not in global space,
                         * Initialize the reference immediately.
                         * 
                         * (Hack Hack Hack Hack Hack)
                         */
                        /* All the variables in the list have the same
                         * type as the first one... so we key off of that.
                         */
                        DeclExpr *de = (DeclExpr *)$2;
                        char *varName = de->vname;
                        VarNode *vn = vstore[varName];
                        if (!vn)
                            wdlerror("Internal Compilier Error:no such var  ",
                                    varName);
                        
                        source *destination;
                        
                        if (!(vn->sflag & LOCAL_TABLE)) {
                            if (vstore.get_scopelevel() == 1)
                                destination = global_vars;
                            else
                                destination = sp;
                        } else {
                            if (wiv_source == NULL)
                                wiv_source = new source;
                            destination = wiv_source;
                        }
                        
                        /* The base type of the table. */
                        destination->append("unsigned long ");
                        
                        for (e = $2; e != NULL; e = e->next) {
                            de = (DeclExpr *) e;
                            varName = de->vname;
                            vn = vstore[varName];
                            if (!vn)
                                wdlerror(
                                    "Internal Compilier Error: no such var2 ",
                                    varName);

                            if (e != $2) destination->append(", ");
                            destination->append(varName);
                            if (!(vn->sflag & LOCAL_TABLE)) {
                                /* Needed for NT's shared segments. */
                                destination->append(" = 0");
                            }
                            
                            /*
                             * Mutex for table.
                             * We are in the middle of a statement here, so we
                             * need to prepend this to destination.
                             * In reverse order for obvious reasons.
                             */
                            destination->prepend(");\n");
                            destination->prepend(varName);
                            destination->prepend("WSS_MUTEX_PROTECT(");


                            /* We can not handle two different types of
                             * table having the same name in different
                             * scopes.
                             */
                            if (table_names == NULL)
                                table_names = new VarTbl;
                            if (!table_names->insert(varName, vn)) {
                                /* If its the same type, that's ok. */
                                VarNode *v2 = (*table_names)[varName];
                                if (v2 != vn
                                        && vn->type.info != v2->type.info)
                                    wdlerror(
                                            "Duplicate definition of "
                                            "table instance.");
                            }
                            if (vstore.get_scopelevel(varName) > 1) {
                                if (!(vn->sflag & LOCAL_TABLE)) {
                                    SourceAppend6(destination,
                                            " = wql_find_table_by_names(",
                                            (vn->sflag & GLOBAL_TABLE) ?
                                                "PD_MAKESTR(\"\")" : "myname",
                                            ", 0,",
                                            "PD_MAKESTR(\"",
                                            varName,
                                            "\")) ");
                                    if (!post_decl_block)
                                        post_decl_block = new source;
                                    SourceAppend3(post_decl_block,
                                            "WR_MUTEX_INIT(",varName,");\n");
                                    /* Clean up mutex */
                                    SourceAppend3( post_block_body.top(),
                                            "WR_MUTEX_DESTROY(",
                                            varName,");\n");
                                } else {
                                    /* Make sure we don't already have
                                     * a copy of the table reference
                                     * in WIV
                                     */
                                    if (!wiv_variables.insert(varName,vn)) {
                                        /* Really we should handle this case.
                                         * but that would require restructuring
                                         * a lot of code and I'm feeling lazy.
                                         */
                                        wdlerror("Duplicate declaration of "
                                                "table name.\n");
                                    }
                                    /* Schedule Initialization of the WIV
                                     * variable for init
                                     */
                                    SourceAppend3(activate_instructions,
                                            "WR_WIV_MUTEX_INIT(",varName,");\n"
                                            );
                                    SourceAppend3(deactivate_instructions,
                                            "WR_WIV_MUTEX_DESTROY(",varName,
                                            ");\n");
                                }
                            } else {
                                /* Schedule initialization */
                                if (!(vn->sflag&LOCAL_TABLE)) {
                                    /* Put it in wr_install */
                                    SourceAppend3(init_instructions,
                                            "WR_MUTEX_INIT(",varName,");\n");
                                    SourceAppend3(init_instructions,
                                            "WR_MUTEX_ENTER(",varName,");\n");
                                    SourceAppend6(init_instructions,
                                            varName,
                                            " = wql_find_table_by_names(",
                                            (vn->sflag & GLOBAL_TABLE) ?
                                                "PD_MAKESTR(\"\")" : "myname",
                                            ", 0, PD_MAKESTR(\"",
                                            varName,
                                            "\"));\n");
                                    SourceAppend3(init_instructions,
                                            "WR_MUTEX_EXIT(",varName,");\n");
                                    SourceAppend3(destruction_instructions,
                                            "WR_MUTEX_DESTROY(",varName,");\n");
                                } else {
                                    /* Don't bother.
                                     * User had better create it before
                                     * using it, and since all WIV
                                     * variables share a structure,
                                     * creating it will initialize it even
                                     * if the use is in a different scope.
                                     * 
                                     * Global+Local tables are not currently
                                     * allowed.
                                     */
                                    SourceAppend3(activate_instructions,
                                            "WR_WIV_MUTEX_INIT(",varName,");\n"
                                            );
                                    SourceAppend3(deactivate_instructions,
                                            "WR_WIV_MUTEX_DESTROY(",varName,
                                            ");\n");
                                }
                            }
                            free(varName);
                        }
                        destination->append(";\n");
                    }
		    $$ = sp;
	          }
		| K_TYPEDEF error ';' {
                    const char *identifier = yylval.string;
                   
                    Type *t1 = vstore.get_type(identifier);
                    if (!t1 && !strcmp(identifier, "pd_char_t") ) {
                        vstore.add_type(identifier,
					error_recovery_slot->d_type);
			t1 = vstore.get_type(identifier);
                    }
                    if (!t1 || (*t1 != *(error_recovery_slot->d_type) ) ) 
			wdlerror("Changed definition of type name");

                    source *sp = new source;
		    $$ = sp;
		    delete error_recovery_slot;
		    error_recovery_slot = NULL;
                }
                | K_TYPEDEF declaration_spec ';' {
                    if (!in_includes)
                        wdlerror("Invalid typedef.");
                    source *sp = new source;
                    $$ = sp;
                }
		| K_TYPEDEF declaration_spec declarator_list ';' {
                    Expr *e;
                    for (e = $3; e != NULL; e = e->next) {
                       DeclExpr *de = (DeclExpr *) e;
		       symbols->append(de->vname);

                       de->type.fillin_type(*$2->d_type);
                       // Hack hack hack
                       if (!strcmp(de->vname, "pd_char_t")) {
                           vstore.add_type("pd_char_t", &cstrType);
                       } else {
                           // fill in the type got in <type_specifier>
                           if (!vstore.add_type(de->vname, &de->type)
                               && (!in_includes
                                       || (*(vstore.get_type(de->vname)) !=
                                           e->type)))
                               wdlerror("Duplicated definition of type name");
                       }
                       if (dflag) {
                           cerr << "Insert typename " << de->vname;
                           cerr << "  " << de->type << "\n";
                       }
                    }
		    source *sp = new source;
                    if (!in_includes) {
                        SourceAppend3(sp, "typedef ", $2, " ");
                        for (e = $3; e != NULL; e = e->next) {
                            DeclExpr *de = (DeclExpr *) e;
                            sp->append(de);
                        } 
                        sp->append(";\n");
                        if (in_uses) {
                            link_declarations->append(sp->all());
                            delete sp;
                            sp = new source;
                        }
                    }
		    $$ = sp;
                  }
	        ;

inline_spec:
        K___INLINE
          {
              source *sp = new source;
              sp->d_type = new Type(TC_TMP);
              $$ = sp;
          }
        ;

declaration_spec:
		declaration_specifiers_list
		| var_storage_class {
                    /* Hack.
                     * If "K_STATIC" && current scope level = 1
                     * then the variable will be a WIV variable.
                     */
                    if (vstore.get_scopelevel() == 1 && !strcmp($1,"static "))
                        curvar_wivvar = 1;
                } declaration_specifiers_list {
                    /* $1 already contains the " " that would be needed */
                    /* Drop the "static" if wiv var */
                    source *sp = new source;
                    if (!curvar_wivvar)
                        sp->append($1);

                    sp->append($3);
                    sp->d_type = $3->d_type;
		    $$ = sp;
		  }
                | var_storage_class {
                    /* Hack.
                     * If "K_STATIC" && current scope level = 1
                     * then the variable will be a WIV variable.
                     * drop the "static" keyword from the source now
                     * while its easy.
                     */
		    source *sp = new source;
                    if (!vstore.get_scopelevel() == 1
                            || strcmp($1, "static ")) {
                        sp->append($1);
                    } else {
                        curvar_wivvar = 1;
                    }
                    sp->d_type = new Type(TC_INT);
		    $$=sp;
		  }
		;

declaration_specifiers_list:
		declaration_specifiers
		| declaration_specifiers_list declaration_specifiers {
		    source *sp = new source;
                    Type *t1, *t2;
                    t1 = $1->d_type;
                    t2 = $2->d_type;
                    if (typechk) {
                         sp->d_type = new Type(concat(*t1,*t2));
                         if (*sp->d_type == errType) {
			     if (error_recovery_slot != NULL)
				 wdlerror("Errors in type specification");
                             error_recovery_slot = $1;
                             YYERROR;
                             // wdlerror("Errors in type specification");
			 }
                    }
		    SourceAppend3(sp, $1, " ", $2);
		    $$ = sp;

		}
		;

declaration_specifiers: 
                  type_qualifier {
		    source *sp = new source;
		    sp->append($1);
                    sp->d_type = new Type(TC_TMP);
		    $$=sp;
		  }
                | inline_spec
		| type_specifier
		| type_specifier_or_reference
		;

type_reference_complete:
                type_reference
                | type_specifier_or_reference
                | type_specifier_or_reference type_specifier_or_reference {
                    /* This is just the wrong place to do this, but so
                     * are all the other places.  This file is just too
                     * big to maintain.
                     */
                    Type *t1, *t2;
                    t1 = $1->d_type;
                    t2 = $2->d_type;
                    $1->d_type = new Type(concat(*t1, *t2));
                    if (*($1->d_type) == errType)
                        wdlerror("Errors in type specification (2)");
                    $1->append($2);
                    delete $2;
                    $$ = $1;
                }
                | type_reference_complete '*' {
                      $1->append(" * ");
                      $1->d_type = new Type(TC_PTR,
                              new PtrTypeInfo(*($1->d_type)));
                  } 
                ;

iface_declaration_specifiers: 
	  	  var_storage_class iface_declaration_specifiers  {
		    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $2->d_type;
		    $$=sp;
		  }
		| type_qualifier iface_declaration_specifiers {
		    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $2->d_type;
		    $$=sp;
		  }
		| type_reference iface_declaration_specifiers {
		    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $2->d_type;
                    if (typechk) {
                         sp->d_type = new Type(concat(*$1->d_type,*$2->d_type));
                         if (*sp->d_type == errType) {
                             wdlerror("Errors in type specification");
                         }
                    }
		    $$ = sp;
		  }
		| type_specifier_or_reference iface_declaration_specifiers {
		    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $2->d_type;
                    if (typechk) {
                         sp->d_type = new Type(concat(*$1->d_type,*$2->d_type));
                         if (*sp->d_type == errType) {
                             wdlerror("Errors in type specification");
                         }
                    }
		    $$ = sp;
		  }

		| var_storage_class {
		    source *sp = new source;
		    sp->append($1);
                    sp->d_type = new Type(TC_INT);
		    $$ = sp;
		  }
		| type_qualifier {
		    source *sp = new source;
		    sp->append($1);
                    sp->d_type = new Type(TC_TMP);
		    $$ = sp;
		  }
		| type_reference
		| type_specifier_or_reference
		; 

var_storage_class: 
		  K_AUTO     {$$="auto ";}
		| K_EXTERN   {$$="extern ";}
		| K_REGISTER {$$="register ";}
		| K_STATIC   {$$="static ";}
                | K_GLOBAL   {$$="global";}
                | K_LOCAL    {$$="local";}
		;

initialized_decl_list: 
		  initialized_declarator 
		| initialized_decl_list ',' initialized_declarator {
                    Expr *e = $1;
                    while (e->next != NULL) e = e->next; 
                    e->next = $3;
                    $$ = $1;
		  }
		;

declarator_list:
                declarator {
                    char *dname = $1->all();
                    Expr *e = new DeclExpr($1, NULL, $1->vname);
                    e->code->append(dname);
                    e->type = *$1->d_type;
		    $$ = e
                }
                | declarator_list ',' declarator {
                    char *dname = $3->all();
                    Expr *er = new DeclExpr($3, NULL, $3->vname);
                    er->code->append(dname);
                    er->type = *$3->d_type;

                    Expr *el = $1;
                    while (el->next != NULL) el = el->next; 
                    el->next = er;
                    $$ = $1;
                }
                ;

initialized_declarator: 
                  declarator {
                    char *dname = $1->all();
                    Expr *e = new DeclExpr($1, NULL, $1->vname);
                    e->code->append(dname);
                    if (vstore.get_scopelevel() == 1
                            && !curvar_wivvar ) {
                        /* the "{"'s allow me to get structures and
                         * arrays' as well.  Hack hack.
                         */
                        e->code->append(" = {0}");
                    }
                    e->type = *$1->d_type;
		    $$ = e
                  }
		| declarator '=' initializer { 
                    char *dname = $1->all();
                    Expr *e = new DeclExpr($1, $3, $1->vname);
		    if (in_uses) 
		      wdlerror("initialization not allowed in uses");
           
                    SourceAppend3(e->code, dname, " = ", $3->code->all());
                    e->type = *$1->d_type;
		    $$ = e
		  }
		;

initializer:	
                  assignment_expression
		| '{' initializer_list '}' {
                    Expr *e = new ListExpr($2);
                    SourceAppend3(e->code, "{", $2, "}\n");
		    $$ = e 
		  }
		| '{' initializer_list ',' '}' {
                    Expr *e = new ListExpr($2);
                    SourceAppend3(e->code, "{", $2, ", }\n");
		    $$ = e
		  }
		;

initializer_list: 
                  initializer_list ',' initializer {
                     SourceAppend2($1->code, ", ", $3->code);
                     Expr *e = $1;
                     while (e->next != NULL) e = e->next;
                     e->next = $3;
		     $$ = $1;
		  }
		| initializer
		;

declarator:	  pointer direct_declarator  {
                    source *sp = new source;
                    SourceAppend3(sp, $1, " ", $2);
                    if (typechk) {
                        $2->d_type->fillin_type(*$1->d_type);
                        sp->d_type = $2->d_type;
                    }
                    sp->vname = $2->vname;
		    $$ = sp
		  }
                | pointer S_TYPENAME {
                    /* Hack for a specific windows header problem */
                    if (!in_includes) {
                        wdlerror("Duplicated typedef.");
                    }
                    source *sp = new source;
                    SourceAppend3(sp, $1, " ", $2);
                    sp->vname = strdup($2);
                    sp->d_type = new Type(TC_TMP);
                    sp->d_type->fillin_type(*$1->d_type);
                    $$ = sp;
                    delete $1;
                }
		| direct_declarator 
		;

direct_declarator: S_IDENTIFIER 
                { 
		  source *sp = new source;
		  sp->append($1);
                  sp->vname = strdup($1);
                  sp->d_type = new Type(TC_TMP);
		  $$=sp;
                  if (esstbl[$1] != NULL)
                      wdlerror("declarator name same as characterization name");
		}
                | K_OP {
		  source *sp = new source;
		  sp->append("op");
                  sp->vname = strdup("op");
                  sp->d_type = new Type(TC_TMP);
		  $$=sp;
                  if (esstbl["op"] != NULL)
                      wdlerror("declarator name same as characterization name");
                }
		| '(' declarator ')' { 
		  source *sp = new source;
		  SourceAppend3(sp, "(", $2, ")");
                  sp->d_type = $2->d_type;
                  sp->vname = $2->vname;
		  $$ = sp;
		}
		| direct_declarator '[' ']' { 
		  source *sp = new source;
		  SourceAppend2(sp, $1, " []");
                  if (typechk) {
                      $1->d_type->fillin_type
                                (Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
                      sp->d_type = $1->d_type;
                  }
                  sp->vname = $1->vname;
		  $$ = sp;
		}
		| direct_declarator '[' constant_expression ']' { 
		  source *sp = new source;
		  SourceAppend4(sp, $1, " [", $3, "]");
                  if (typechk) {
                      $1->d_type->fillin_type(Type(TC_ARRAY,
                                     new ArrayTypeInfo(tmpType, $3->str())));
                      sp->d_type = $1->d_type;
                  }
                  sp->vname = $1->vname;
		  $$ = sp;
		}
                | function_declarator
		;

function_call:    postfix_expression '(' expression_list ')' {
                    Expr *e = new FuncCallExpr($1, NULL);
                    if ($1->type.is_functype())
                        e->type = $1->type.get_elmtype();
                    else 
                        wdlerror("Function expected for ", $1->str()); 
                    FuncTypeInfo *fti = (FuncTypeInfo *) $1->type.info;
                    if (in_includes) {
                    } else if (fti->flag & (IS_LOCALFUNC | IS_LIBFUNC)) {
                        SourceAppend4(e->code, $1, " (", $3, ")");
                    } else if (fti->flag & IS_WI_LOCALFUNC) {
                        if (in_no_wiv_scope)
                            wdlerror("Can't invoke WI function\n");
                        else {
                            SourceAppend5(e->code, $1, " (wrp,", $3, 
                                                       ", data", ")");
                        }
                    } else {
                        wdlerror("Not a local or lib function: ", $1->str()); 
                    }
		    $$ = e;
		  }
		| postfix_expression '(' ')' {
                    Expr *e = new FuncCallExpr($1, NULL);
                    if ($1->type.is_functype())
                        e->type = $1->type.get_elmtype();
                    else 
                        wdlerror("Function expected for ", $1->str());
                    FuncTypeInfo *fti = (FuncTypeInfo *) $1->type.info;
                    if (fti->flag & (IS_LOCALFUNC | IS_LIBFUNC) || in_includes){
                        SourceAppend2(e->code, $1, " ()");
                    } else if (fti->flag & IS_WI_LOCALFUNC) {
                        if (in_no_wiv_scope)
                            wdlerror("Can't invoke WI function\n");
                        else {
                            SourceAppend2(e->code, $1, " (wrp, data)");
                        }
                    } else if (fti->flag & IS_DBFUNC) {
                        e->code->append($1);
                    } else {
                        wdlerror("Not a local or lib function: ", $1->str()); 
                    }
		    $$ = e;
                  }
                | event_specifier '(' expression_list ')' {
                    /* expression_list ->paren_count contains the length
                     * of the list.  I'm misusing the variable but its
                     * not being used here for anything else.
                     * DSK
                     */
                    Expr *e = new FuncCallExpr(new IdentExpr(
                                $1->event->name), NULL);
                    SourceAppend10(e->code,
                            "wr_dispatch(",$1->space,",",$1->code,", wrp, ",
                            /* flags = 0, # other args, other args */
                            "0 , ", $3->paren_count,",",  $3, ")");
		    $$ = e;
                }
                | event_specifier '(' ')' {
                    Expr *e = new FuncCallExpr(new IdentExpr(
                                $1->event->name), NULL);
                    SourceAppend6(e->code,
                            "wr_dispatch(",$1->space,",",$1->code,",",
                            /* flags = 0, 0 other arguments */
                            "0 , 0)");
		    $$ = e;
                }
                ;

event_specifier: S_IDENTIFIER K_COLONCOLON S_IDENTIFIER {
                     characterization *t = esstbl[$1];
                     if (!t)
                         wdlerror("Unknown characterization ",$1);
                     if (!t->is_valid_op($3))
                         wdlerror("Unknown event ",$3);
                     
                     $$ = new EvSpecNode;
                     $$->space = $1;
                     $$->code  = t->get_code($3);
                     $$->event = (*t)[$3];
                 }
                 ;

function_declarator
	:  direct_declarator '(' 
           {   vstore.newtbl();  }
           parameter_type_list ')' {
               VarTbl *vtbl = vstore.poptbl();
               source *sp = new source;
               if (in_wrapper_body) {
                    if (curvar_wivvar) 
                         $1->d_type->fillin_type(Type(TC_FUNC, new FuncTypeInfo(
                                              tmpType,vtbl, IS_WI_LOCALFUNC)));
                    else
                         $1->d_type->fillin_type(Type(TC_FUNC, new FuncTypeInfo(
                                              tmpType,vtbl, IS_LOCALFUNC)));
               } else if (in_uses) {
                    $1->d_type->fillin_type(Type(TC_FUNC, 
                            new FuncTypeInfo(tmpType,vtbl, IS_LIBFUNC)));
               } else {
                    $1->d_type->fillin_type(Type(TC_FUNC, 
                            new FuncTypeInfo(tmpType,vtbl)));
               }
               if (curvar_wivvar) { /* func is a wrapper instance func */
                    curvar_wivvar = 0;
                    SourceAppend5(sp, $1, " (struct wss_proc *wrp, ", $4, 
                                       ", struct wss_wrapper_data *data", ")");
               } else {
                    SourceAppend4(sp, $1, " (", $4, ")");
               }
               sp->d_type = $1->d_type;
               sp->vname = $1->vname;
	       $$=sp
	   }
	|  direct_declarator '(' ')' {
               source *sp = new source;
               if (in_wrapper_body) {
                    if (curvar_wivvar) 
                        $1->d_type->fillin_type(Type(TC_FUNC, new FuncTypeInfo(
                                        tmpType, NULL, IS_WI_LOCALFUNC)));
                    else
                        $1->d_type->fillin_type( Type(TC_FUNC, new FuncTypeInfo(
                                        tmpType, NULL, IS_LOCALFUNC)));
               } else if (in_uses) {
                    $1->d_type->fillin_type(Type(TC_FUNC, 
                               new FuncTypeInfo(tmpType, NULL, IS_LIBFUNC)));
               } else {
                    $1->d_type->fillin_type(
                             Type(TC_FUNC, new FuncTypeInfo(tmpType)));
               }
               if (curvar_wivvar) { /* func is a wrapper instance func */
                    curvar_wivvar = 0;
                    SourceAppend3(sp, $1, "(struct wss_proc *wrp, ", 
                                       "struct wss_wrapper_data *data) ");
               } else {
                    SourceAppend2(sp, $1, " ( )");
               }
               sp->d_type = $1->d_type;
               sp->vname = $1->vname;
	       $$=sp
	   }
	;

function_definition:
		  function_specifier 
                  {
                      FuncTypeInfo *fti = (FuncTypeInfo *)$1->d_type->info;
                      if (fti->vtbl != NULL) {
                          vstore.pushtbl(fti->vtbl);
                      }
                      last_function_level = post_block_body_count;
                  }
                  compound_stmt 
		  {  
                      FuncTypeInfo *fti = (FuncTypeInfo *) $1->d_type->info;
                      if (fti->vtbl != NULL)
                          vstore.poptbl();

                      source *sp = new source;
                      SourceAppend2(sp, $1, $3);
                      sp->d_type = $1->d_type;
                      sp->vname = $1->vname;
                      $$ = sp;
                      last_function_level = -1;
		  }
		;

function_specifier:
                  declaration_spec declarator {
                    $2->d_type->fillin_type(*$1->d_type);
                    source *sp = new source;
		    SourceAppend2(sp, $1, $2);
                    sp->d_type = $2->d_type;
                    sp->vname = $2->vname;
		    $$=sp;
		  }
                | declarator {
                    source *sp = new source;
		    sp->append($1);
                    sp->d_type = &intType;
		    $$=sp;
		  }
                  ;

top_level_declaration:
                  declaration
	        | function_definition 
                ;

pointer:	  '*' {
                    source *sp = new source;
		    sp->append("*");
                    sp->d_type = new Type(TC_PTR, new PtrTypeInfo(tmpType));
		    $$=sp;
                  }
		| '*' pointer {
                    source *sp = new source;
		    SourceAppend2(sp, "*", $2);
                    sp->d_type = new Type(TC_PTR, new PtrTypeInfo(*$2->d_type));
		    $$ = sp;
                  }
		| '*' type_qualifier_list {
                    source *sp = new source;
		    SourceAppend2(sp, "*", $2);
                    sp->d_type = new Type(TC_PTR, new PtrTypeInfo(tmpType));
		    $$ = sp;
                  }
		| '*' type_qualifier_list pointer {
                    source *sp = new source;
		    SourceAppend4(sp, "*", $2, " ", $3);
                    sp->d_type = new Type(TC_PTR, new PtrTypeInfo(*$3->d_type));
		    $$ = sp;
                  }
		;

type_qualifier_list: 
                  type_qualifier_list type_qualifier {
                    source *sp = new source;
		    SourceAppend3(sp, $1, " ", $2);
		    $$=sp;
		  }
		| type_qualifier  {
                    source *sp = new source;
		    sp->append($1);
		    $$=sp;
		  }
		;

type_qualifier:	  K_CONST {$$="const ";}
                | K_VOLATILE {$$="volatile ";}
                | K_DLLIMPORT {$$="__declspec( dllimport )";}
		;

type_specifier_or_reference:
		integer_type {
                    source *sp = new source;
		    sp->append($1->str());
                    sp->d_type = $1;
		    $$ = sp;
		  }
		| floating_type {
                    source *sp = new source;
		    sp->append($1->str());
                    sp->d_type = $1;
		    $$ = sp;
		  }
		| string_type {
	   	    source *sp = new source;
		    sp->append($1->str());
		    sp->d_type = $1;
		    $$ = sp;
		  }
		| S_TYPENAME{
                    source *sp = new source;
		    sp->append($1);
		    sp->append(" ");
                    if (typechk) {
                        sp->d_type = vstore.get_type($1);
		        if (sp->d_type == NULL) 
		   	    wdlerror("Type reference not found for ",$1);
                        $$ = sp;
                    } else {
                        $$ = sp;
                    }
		  }
		| void_type {
                    source *sp = new source;
		    sp->append($1);
                    sp->d_type = &voidType;
		    $$ = sp;
		  }
		| wdl_type {
                    source *sp = new source;
		    sp->append($1);
                    sp->d_type = $1->d_type;
		    $$ = sp;
		  }
		;


type_specifier: enumeration_type
		| struct_or_union_specifier
		;

type_reference:	enumeration_type_reference
		| struct_or_union_reference
		;

wdl_type:	  K_DBROW K_COLONCOLON S_TYPENAME
                  {  source *sp = new source; 
                     SourceAppend4(sp, "dbrow ", "/* ::", $3, "*/"); //TBD: ??
                     sp->d_type = vstore.get_type($3);
                     if (sp->d_type == NULL)
                         wdlerror("Type reference not found for ",$3);
                     if (sp->d_type->tc != TC_DBTABLE)
                         wdlerror("Invalid type reference.");
                     TableTypeInfo *tti= (TableTypeInfo *)(sp->d_type->info);
                     sp->d_type = new Type(TC_DBROW,
                                    new DBrowTypeInfo(tti,$3));
                     $$ = sp;
                     
                     needs_db_components = 1;
                  }
                | K_DBTABLE S_IDENTIFIER
                  {
		      symbols->append($2);
                      Type *t;
                      if (typechk) {
		          vstore.newtbl(); 
                          t = new Type(TC_DBTABLE, 
                                  new TableTypeInfo(vstore.toptbl(), $2));
                          if (!vstore.add_type($2, t))
                             wdlerror("Duplicated definition of table name.");
                          table_order = new smap<int>;
                          table_rev_map = new StrTbl;
                          table_elem_count = 0;
                          table_key_fields = new set<int, less<int> >;
                      }
                      if (esstbl[$2] != NULL)
                          wdlerror("table name same as characterization name");
                  }
                  '{' wql_table_declaration_body '}' {
                      /*
                       * wql_table_declaration_body returns a "list" of 
                       * column definition information.  This has to be
                       * turned into a wr_mcol_def call before we save it.
                       */
                      TableTypeInfo *tti = (TableTypeInfo *)
                                                (vstore.get_type($2)->info);
                      vstore.poptbl();

                      source *sp = new source;
                      SourceAppend5(sp,
                              "wr_mcol_def(",table_elem_count,", ", $5,")\n");
                      tti->wql_create_stmt = sp;
                      sp = new source;
                      sp->d_type = vstore.get_type($2);
                      $$ = sp;
                      tti->order = table_order;
                      tti->num2name = table_rev_map;
                      tti->count = table_elem_count;
                      tti->keyfields = table_key_fields;
                      table_key_fields = NULL;
                      table_order = NULL;
                      table_rev_map = NULL;
                      table_elem_count = 0;
                      
                      delete $5;
                      needs_db_components = 1;
                  }
		;

struct_or_union_specifier:
                  struct_or_union_definition
                | struct_or_union_reference
                ;

struct_or_union_reference:
                  struct_or_union S_IDENTIFIER {
                    char buf[272];
		    source *sp = new source;
		    SourceAppend4(sp, $1, " ", $2, " ");
                    strcpy(buf, $1);
                    strcat(buf, " ");
                    strncat(buf, $2, 256);
                    sp->d_type = vstore.get_type(buf) != NULL 
                                    ? vstore.get_type(buf)
				    : new Type(TC_NAME, new NameTypeInfo(buf));
		    $$ = sp;
                    if (esstbl[$2] != NULL)
                        wdlerror(
                            "struct or union name same as characterization name"
                            );
                  }
                | struct_or_union S_TYPENAME {
                    char buf[272];
		    source *sp = new source;
		    SourceAppend4(sp, $1, " ", $2, " ");
                    strcpy(buf, $1);
                    strcat(buf, " ");
                    strncat(buf, $2, 256);
                    sp->d_type = vstore.get_type(buf) != NULL 
                                    ? vstore.get_type(buf)
				    : new Type(TC_NAME, new NameTypeInfo(buf));
		    $$ = sp;
                    if (esstbl[$2] != NULL)
                        wdlerror(
                            "struct or union name same as characterization name"
                            );

                  }

                ;

struct_or_union_definition:
                  struct_or_union S_IDENTIFIER 
                  {
                      Type *t;
                      if (typechk) {
		          vstore.newtbl(); 
                          char buf[272];
                          strcpy(buf, $1);
                          strcat(buf, " "); 
                          strncat(buf, $2, 256);
                          if (!strcmp($1, "struct"))
                              t = new Type(TC_STRUCT, 
                                         new RecTypeInfo(vstore.toptbl(), buf));
                          else
                              t = new Type(TC_UNION, 
                                         new RecTypeInfo(vstore.toptbl(), buf));
                          int i = vstore.add_type(buf, t);
                          if (!i) 
                             wdlerror("Duplicated definition of struct/union");
                      }
                      if (esstbl[$2] != NULL)
                          wdlerror(
                            "struct or union name same as characterization name"
                            );
                  }
                  '{' struct_declaration_body '}' {
		      source *sp = new source;
                      char buf[272];
                      SourceAppend6(sp, $1, " ", $2, " {\n", $5, "}");
                      if (typechk) {   
                          vstore.poptbl();
                          strcpy(buf, $1); 
                          strcat(buf, " "); 
                          strncat(buf, $2, 256);
                          sp->d_type = vstore.get_type(buf);
                      }
		      $$ = sp;
                  }
                  | struct_or_union S_TYPENAME
                  {
                      Type *t;
                      if (typechk) {
		          vstore.newtbl(); 
                          char buf[272];
                          strcpy(buf, $1);
                          strcat(buf, " "); 
                          strncat(buf, $2, 256);
                          if (!strcmp($1, "struct"))
                              t = new Type(TC_STRUCT, 
                                         new RecTypeInfo(vstore.toptbl(), buf));
                          else
                              t = new Type(TC_UNION, 
                                         new RecTypeInfo(vstore.toptbl(), buf));
                          int i = vstore.add_type(buf, t);
                          if (!i) 
                             wdlerror("Duplicated definition of struct/union");
                      }
                  }
                  '{' struct_declaration_body '}' {
		      source *sp = new source;
                      char buf[272];
                      SourceAppend6(sp, $1, " ", $2, " {\n", $5, "}");
                      if (typechk) {   
                          vstore.poptbl();
                          strcpy(buf, $1); 
                          strcat(buf, " "); 
                          strncat(buf, $2, 256);
                          sp->d_type = vstore.get_type(buf);
                      }
		      $$ = sp;
                  }

		| struct_or_union 
                  {   if (typechk) vstore.newtbl();   }
                  '{' struct_declaration_body '}' {
		      source *sp = new source;
                      SourceAppend4(sp, $1, " {\n", $4, "}");
                      if (typechk) {
                          if (!strcmp($1, "struct"))
                              sp->d_type = new Type(TC_STRUCT, 
                                          new RecTypeInfo(vstore.poptbl())); 
                          else
                              sp->d_type = new Type(TC_UNION, 
                                          new RecTypeInfo(vstore.poptbl())); 
                      }
		      $$ = sp;
                      delete $4;
                  }
                ;

struct_declaration_body: 
		  struct_declaration_list 
                  {   $$ = $1;   }
                  |
                    { $$ = new source; }
                  ;

struct_declaration_list: 
                  struct_declaration
		| struct_declaration_list struct_declaration {
		    source *sp = new source;
                    SourceAppend2(sp, $1, $2);
		    $$=sp;
                  }
		;

struct_declaration: declaration_spec struct_declarator_list ';'  {
                    SrcNode *sn;
                    for (sn = $2; sn != NULL; sn = (SrcNode *)sn->next) {
                        if (typechk) {
                            sn->val->d_type->fillin_type(*$1->d_type);
                            VarNode *vn = new
                                VarNode (sn->val->vname, *sn->val->d_type);
                            if (!vstore.toptbl()->insert(sn->val->vname, vn))
                                wdlerror("Duplicated definition of identifier");
                        }
                        if (dflag) {
                           cerr << "Insert Symbol " << sn->val->vname;
                           cerr << "  " << *sn->val->d_type << "\n";
                        }
                    }
                    source *sp = new source;
                    SourceAppend2(sp, $1, " ");
                    for (sn = $2; sn != NULL; sn = (SrcNode *)sn->next) {
                        if (sn != $2) sp->append(", ");
                        sp->append(sn->val);
                    }
		    sp->append(";\n");
		    $$ = sp;
                  }
                  | error ';'
                    {
                        const char *identifier = yylval.string;
                        source *sp = new source;
			sp->append(identifier);
			sp->vname = strdup(identifier);
			sp->d_type = error_recovery_slot->d_type;
			VarNode *vn = new
				VarNode (sp->vname, *(sp->d_type));
			if (!vstore.toptbl()->insert(sp->vname, vn))
				wdlerror("Duplicated definition of identifier");
                        if (dflag) {
                           cerr << "Insert Symbol " << sp->vname;
                           cerr << "  " << *(sp->d_type) << "\n";
                        }
			SourceAppend3(sp, error_recovery_slot, " ", identifier);
			sp->append(";\n");
			$$ = sp;

			error_recovery_slot->d_type = NULL;
                        delete error_recovery_slot;
			error_recovery_slot = NULL;
                    } 
                  | declaration_spec ';'
                    {
                        $$ = new source;
                        SourceAppend2($$,$1, ";\n");
                        delete $1;
                    }
                  | ';'
                    {
                        $$ = new source;
                    }
		;

/*
specifier_qualifier_list: 
                  type_specifier specifier_qualifier_list {
		    source *sp = new source;
                    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $1->d_type; //  TBD
		    $$=sp;
                  }
		| type_specifier
		| type_qualifier specifier_qualifier_list {
		    source *sp = new source;
                    SourceAppend3(sp, $1, " ", $2);
                    sp->d_type = $2->d_type; //  TBD
		    $$=sp;
                  }
		| type_qualifier {
                    source *sp = new source;
		    sp->append($1);
		    $$=sp;
		  }
		;
*/

struct_declarator_list: 
                  struct_declarator {
                    $$ = new SrcNode($1);
                  }
		| struct_declarator_list ',' struct_declarator 	 {
                    SrcNode *sn = $1;
                    while (sn->next != NULL) sn = (SrcNode *)sn->next; 
                    sn->next = new SrcNode($3);
                    $$ = $1;
                  }
		;

struct_declarator: 
                  declarator
                | ':' constant_expression {
		    static int anonymous_hack = 0;
                    char buf[256];
		    source *sp = new source;
                    SourceAppend2(sp, ":", $2);
                    sprintf(buf, "wr_anony_%d",anonymous_hack++);
                    sp->vname = strdup(buf);
		    sp->d_type = new Type(TC_TMP);
		    $$ = sp;
                }
		| declarator ':' constant_expression {
		    source *sp = new source;
                    SourceAppend3(sp, $1, ":", $3);
                    sp->vname = $1->vname;
                    sp->d_type = $1->d_type;
		    $$ = sp;
                  }
		;

struct_or_union:  K_UNION  {$$="union";}
                | K_STRUCT {$$="struct";}
		;

enumeration_type: enumeration_type_definition
		| enumeration_type_reference
		;

enumeration_type_definition:  
                  K_ENUM '{' enumeration_definition_list '}' {
		    source *sp = new source;
		    SourceAppend3(sp, "enum {", $3, "}\n");
                    sp->d_type = new Type(TC_ENUM, new EnumTypeInfo);
		    $$=sp
		  }
		| K_ENUM enumeration_tag '{' enumeration_definition_list '}' {
                    char buf[255];
		    source *sp = new source;
		    SourceAppend5(sp, "enum ", $2, " {", $4, "}\n");
                    sp->d_type = new Type(TC_ENUM, new EnumTypeInfo);
                    sprintf(buf, "enum %s", $2);
                    vstore.add_type(buf, sp->d_type);
		    $$=sp
		  }
		;

enumeration_type_reference: K_ENUM enumeration_tag {
		    source *sp = new source;
		    SourceAppend2(sp, "enum ", $2);
                    if (typechk) {
                        sp->d_type = vstore.get_type(sp->all());
		        if (sp->d_type == NULL) 
                            wdlerror("Type reference not found for ",$2);
                    }
		    $$ = sp
		  }
		;

enumeration_tag:  S_IDENTIFIER
		;

enumeration_definition_list: 
                enumeration_constant_definition {
                    $$ = new source;
                    $$->append($1);
                    if ($1->vname != NULL) {
                        VarNode *vn = new VarNode($1->vname, *($1->d_type));
                        if (!vstore.gtbl().insert($1->vname, vn)) {
                            if (!vstore.gtbl()[$1->vname]->
                                    type.equiv(vn->type)) {
                                wdlerror("Duplicated definition of identifier");
                            }
                        }
                    }
                    delete $1;
                }
		| enumeration_definition_list ',' enumeration_constant_definition {
                    if ($3->vname != NULL) {
                        VarNode *vn = new VarNode($3->vname, *($3->d_type));
                        if (!vstore.gtbl().insert($3->vname, vn)) {
                            if (!vstore.gtbl()[$3->vname]->
                                    type.equiv(vn->type)) {
                                wdlerror("Duplicated definition of identifier");
                            }
                        }
                    }
		    source *sp = new source;
                    if ($3->vname != NULL) {
                        SourceAppend3(sp, $1, ", ", $3);
                    } else {
                        sp->append($1);
                    }
		    $$=sp;
                    delete $1; delete $3;
		  }
		;

enumeration_constant_definition: 
                  S_IDENTIFIER {
		    source *sp = new source;
		    sp->append($1);
		    $$=sp;
                    sp->vname = strdup($1);
                    sp->d_type = new Type(TC_INT);
                    if (esstbl[$1] != NULL)
                        wdlerror(
                            "enumeration name same as characterization name");
		  }
		| S_IDENTIFIER '=' constant_expression {
		    source *sp = new source;
		    SourceAppend3(sp, $1, " = ", $3);
		    $$=sp;
                    sp->vname = strdup($1);
                    sp->d_type = new Type(TC_INT);
                    if (esstbl[$1] != NULL)
                        wdlerror(
                            "enumeration name same as characterization name");
		  }
                | {
                    source *sp = new source;
                    $$ = sp;
                }
		;

void_type:	  K_VOID {$$="void ";}
		;

integer_type	: K_SHORT          { $$ = new Type(TC_SHORT);		}
		| K_INT      	   { $$ = new Type(TC_INT);     	}
		| K_LONG           { $$ = new Type(TC_LONG);		}     
		| K_LONGLONG       { $$ = new Type(TC_LONGLONG);	}     
                | K_SIGNED   	   { $$ = new Type(TC_SIGNED);		}
                | K_UNSIGNED 	   { $$ = new Type(TC_UNSIGNED);	}
		| K_CHAR           { $$ = new Type(TC_CHAR);		}
		| K_WSS_CHAR_T     { $$ = new Type(TC_WSS_CHAR_T);	}
                ;

floating_type:	  K_DOUBLE         { $$ = new Type(TC_DOUBLE);		}
                | K_FLOAT          { $$ = new Type(TC_FLOAT);		}
		;

string_type:	K_STR		   { $$ = new Type(TC_STR);             }
		;


/* ================================================================= *
 *                          WQL Statements                           *
 * ================================================================= */

wql_stmt:	  K_WQL '{' wql_statement_list '}' {
                      needs_db_components = 1;
                      /*
                       * Assume exterior code of:
                       * "{int wq_retval;
                       * ...
                       * something = wq_retval;
                       * }"
                       */
                      source *sp = new source;
                      $3->indent("    ");
                      SourceAppend4(sp, "do {\n", $3, "\n","} while (0);\n");
                      $$ = sp;
                      delete $3;
                  }
      		| K_WQL '{' '}' {
                    needs_db_components = 1;
                    source *sp = new source;
		    sp->append("wr_retval = 0;\n");
		    $$=sp;
                  }
		;

wql_statement_list: 
                  wql_statement_list wql_statement {
                    source *sp = new source;
                    sp->append($1);
                    $2->indent("    ");
                    SourceAppend3(sp,
                        "wr_retval = \n",
                        $2,
                        "if (wr_retval < 0) break;\n");
                    $$ = sp;
                    delete $1; delete $2;
		  }
		| wql_statement {
                    source *sp = new source;
                    $1->indent("    ");
                    SourceAppend3(sp,
                        "wr_retval = \n",
                        $1,
                        "if (wr_retval < 0) break;\n");
                    $$ = sp;
                    delete $1;
                }
		;

wql_statement:	  wql_create_stmt ';'
		| wql_drop_stmt ';'
		| wql_insert_stmt ';'
		| wql_select_stmt ';' {
                    source *sp = new source;
                    sp->append($1);
                    sp->append(";\n"); /* finish statement */
		    $$ = sp;
                    delete $1;
		  }
		| wql_delete_stmt ';' {
                    source *sp = new source;
                    sp->append($1);
                    sp->append(";\n");
                    $$ = sp;
                    delete $1;
		  }
		| wql_update_stmt ';' {
                    source *sp = new source;
                    sp->append($1);
                    sp->append(";\n"); /* Do semi-colon here. */
		    $$=sp;
                    delete $1;
		  }
		| wql_null_stmt ';' {
                    source *sp = new source;
		    sp->gen_tbd("wql_null_stmt;");
		    $$=sp;
		  }
		;

wql_create_stmt: K_CREATE K_TABLE S_IDENTIFIER
                {
                    source *sp = new source;
                    VarNode *vn = vstore[$3];
                    int wivvar;
                    if (vn == NULL) {
                        wdlerror("Unknown Table", $3);
                    }
                    if (vn->type.tc != TC_DBTABLE) {
                        wdlerror("Wrong type in wql create statement.");
                    }
                    TableTypeInfo *tti = (TableTypeInfo*)vn->type.info;
                    wivvar = (vn->sflag & LOCAL_TABLE);
                    /* Local or global? */
                    sp->gen_wql_create(
                            (vn->sflag & LOCAL_TABLE) ? instance_table :
                            (vn->sflag & GLOBAL_TABLE) ? global_table :
                            wrapper_table,
                            $3,
                            tti->wql_create_stmt,
                            tti->count);
                    $$ = sp;
                    
                    /*
                     * We normally don't bother locking the table reference,
                     * but this time we need to.  wql_create actually modifies
                     * the table reference...
                     */
                    if (!pre_statement_block)
                        pre_statement_block = new source;
                    if (!post_statement_block)
                        post_statement_block = new source;
                    
                    if (wivvar) {
                        SourceAppend3(pre_statement_block,
                                "WR_WIV_MUTEX_ENTER(",$3,");\n");
                        SourceAppend3(post_statement_block,
                                "WR_WIV_MUTEX_EXIT(",$3,");\n");
                    } else {
                        SourceAppend3(pre_statement_block,
                                "WR_MUTEX_ENTER(",$3,");\n");
                        SourceAppend3(post_statement_block,
                                "WR_MUTEX_EXIT(",$3,");\n");
                    }
                }
                ;

wql_drop_stmt:	  K_DROP K_TABLE S_IDENTIFIER {
                    source *sp = new source;
                    VarNode *vn = vstore[$3];
                    if (vn == NULL) {
                        wdlerror("Unknown table.");
                    }
                    if (vn->sflag & GLOBAL_TABLE) {
                        wdlerror("You can not drop global tables.\n");
                    }
                    sp->append("wql_drop(");
                    if (!(vn->sflag & LOCAL_TABLE)) {
                        sp->append($3);
                    } else {
                        SourceAppend3(sp,"WSS_GET_WIV_VAR(",$3,")");
                    }
                    sp->append(");\n");
                    $$ = sp;
		  }
		;

wql_insert_stmt:  K_INSERT K_INTO S_IDENTIFIER
                  { current_table = $3; }
                  K_VALUES wql_insert_list {
			source *sp = new source;
                        VarNode *vn = vstore[$3];
                        if (!vn)
                            wdlerror("Unknown table.");

                        /* wql_insert doesn't distinguish between
                         * wrapper and global tables, so no need to
                         * figure out which it is
                         */
                        sp->gen_wql_insert(!(vn->sflag & LOCAL_TABLE)
                                ? wrapper_table : instance_table, $3, $6);
			$$=sp;
                        current_table = NULL;
		  }
		;

wql_select_stmt:  K_SELECT K_INTO S_IDENTIFIER K_FROM S_IDENTIFIER
                {current_table = $5; table_key_field = 0; table_not_toggle=0;}
                wql_selection wql_select_first wql_orderby {
                    /* With "magic variable"'s case: */
                    source *sp = new source;
                    /* Test that the magic variable is a DBROW */
                    VarNode *vn = vstore[$3];
                    if (vn == NULL)
                        wdlerror("Variable not defined.\n",$3);
                    if (vn->type.tc != TC_DBROW)
                        wdlerror("Variable not a DBROW.\n",$3);
                    /* Clean it out if needed. */
                    SourceAppend5(sp,
                            "((",$3,".results.col_defs != NULL ? "
                            "dbrow_finish(&",$3,") : 0),\n");

                    vn = vstore[$5];
                    if (!vn)
                        wdlerror("Unknown table.");

                    sp->append("wql_select(");
                    /* Table name and don't delete the selection criteria */
                    if (!(vn->sflag & LOCAL_TABLE)) {
                        sp->append($5);
                    } else {
                        SourceAppend3(sp, "WSS_GET_WIV_VAR(", $5, ")");
                    }
                    sp->append(",0,\n");
                    /* The selection criteria */
                    SourceAppend6(sp, "\t", $7, ",\n", "\t", $8, ",");
                    sp->append($9); /* Order by: "mycol, 0,".  Evil */

                    sp->append("\n");
                    SourceAppend3(sp, "\t&(", $3, ".results)");
                    /* Add the semicolon one level higher */
                    SourceAppend4(sp,"));\n","dbrow_start(&",$3,")"); 
		    $$ = sp;
                    
                    /* Free some memory, they are empty anyway */
                    delete $7; delete $9;
		  }
		| K_SELECT K_FROM S_IDENTIFIER
                {current_table = $3; table_key_field = 0; table_not_toggle=0;}
                wql_selection wql_select_first wql_orderby {
                    /* Without "JOIN", sub-queries become much less usefull,
                     * so I'm gonna worry about them.  (That, and the fact
                     * that the proposal doesn't mention supporting them.)
                     */
                    VarNode *vn = vstore[$3];
                    if (!vn)
                        wdlerror("Unknown table.");

                    /* Skipping the "magic variable" case: */
                    source *sp = new source;
                    sp->append("wql_select(\n");
                    /* Table name and don't delete the selection criteria */
                    SourceAppend4(sp, "\t\t", $3, ", 0,\n", "\t\t");
                    SourceAppend5(sp, $5, ",\n", "\t\t", $6, ",");
                    sp->append($7); /* Order by: "mycol, 0,".  Evil */
                    /*
                     * User provided no place for the return structure.
                     * At this point, all he cares about is the number
                     * of rows returned.
                     */
                    sp->append("NULL)");
		    $$ = sp;
                    
                    /* Free some memory, they are empty anyway */
                    delete $5; delete $7;
		  }
		;

wql_orderby:	  K_ORDER K_BY wql_column_name {
                    source *sp = new source;
                    sp->append($3);
                    sp->append(", 0, ");
		    $$ = sp;
                    
                    delete $3;
		  }
		| K_ORDER K_BY wql_column_name K_DESC {
                    source *sp = new source;
                    sp->append($3);
                    sp->append(", 1, ");
		    $$ = sp;
                    
                    delete $3;
		  }
		| {
                    source *sp = new source;
		    sp->append("-1, 0, ");
		    $$ = sp;
		  }
		;
                
wql_select_first:       K_FIRST {
                            $$ = "1";
                        }
                        | {
                            if (table_key_field)
                                $$ = "1";
                            else
                                $$ = "0";
                        }
                        ;

wql_delete_stmt:  K_DELETE K_FROM S_IDENTIFIER
                   {current_table = $3;table_key_field = 0; table_not_toggle=0;}
                    wql_selection wql_select_first {
                    source *sp = new source;
                    sp->append("wql_delete(\n\t\t");
                    
                    VarNode *vn = vstore[$3];
                    if (!vn)
                        wdlerror("Unknown table.");
                    
                    if (!(vn->sflag & LOCAL_TABLE)) {
                        sp->append($3);
                    } else {
                        SourceAppend3(sp, "WSS_GET_WIV_VAR(",$3,")");
                    }

                    /* Table name and please delete the selection criteria */
                    SourceAppend6(sp, ", 0,\n", "\t\t", $5, ",", $6, ")");

		    $$ = sp;
                    
                    delete $5;
                    current_table = NULL;
		  }
		;

wql_update_stmt:  K_UPDATE S_IDENTIFIER
                   {current_table = $2;table_key_field = 0;table_not_toggle=0;}
                    wql_update_expr wql_selection wql_select_first {
                    source *sp = new source;
                    SourceAppend2(sp, "wql_update(\n", "\t\t");
                    
                    VarNode *vn = vstore[$2];
                    if (!vn)
                        wdlerror("Unknown table.");

                    /* Table name and please delete the selection criteria */
                    if (!(vn->sflag & LOCAL_TABLE)) {
                        sp->append($2);
                    } else {
                        SourceAppend3(sp, "WSS_GET_WIV_VAR(", $2, ")");
                    }
                    SourceAppend2(sp, ", 0,\n", "\t\t");
                    SourceAppend5(sp, $5, ",", $6, ",\n", "\t\t");
                    SourceAppend3(sp, $4, "\n", "\t\t)");
		    $$ = sp;
                    
                    /* Free some memory, they are empty anyway */
                    delete $4; delete $5;
                    current_table = NULL;
		  }
		;

wql_null_stmt:  {
                    source *sp = new source;
		    sp->gen_tbd("wql_;");
		    $$=sp;
		  }
		;

wql_update_expr: K_SET wql_update_list {
        source *sp = new source;
        /* Generate variable name. */
        char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
        snprintf(name, sizeof name, "wr_db_%d", variable_generator_count++);
        /* Stash creation instructions. */
        SourceAppend9(creation_instructions,
                "if (",name," == NULL) {\n","\t",name," = ",$2,";\n","}\n");
        /* Stash destruction instructions. */
        SourceAppend10(destruction_instructions,
                "if (", name, " != NULL) {\n","\twql_free_update_expr(",
                name, ");\n","\t", name, "= NULL;\n","}\n");
        /* Stash variable declaration. */
        SourceAppend3(var_decls, "static wqusl *", name, "=0;\n");
        /* Return variable. */
        sp->append(name);
        $$ = sp;
        delete $2;
    }
    ;

wql_update_list: wql_update ',' wql_update_list {
                    /* Hack alert!  wql_update returns something like
                     * "foo, bar", not a complete expression.
                     */
                    source *sp = new source;
                    SourceAppend5(sp, "wr_musl(", $1, ", ", $3, ")");
		    $$=sp;
                    delete $1; delete $3;
		  }
		| wql_update
                {
                    source *sp = new source;
                    SourceAppend3(sp, "wr_musl(", $1, ", NULL) ");
		    $$=sp;
                    delete $1;
                }
		;

wql_update:	wql_column_name '=' wql_update_expression {
                    source *sp = new source;
                    sp->append("wr_msxval_field(");
                    sp->append($1); /* $1 is a source => 3. */
                    sp->append("), ");
                    sp->append($3);
		    $$=sp;
                    if (typechk) {
                        if (!$3->d_type || !$1->d_type)
                            wdlerror("Internal compilier error WQL3");
                        char buf[256] = "";
                        if (!$1->d_type->assmt_chk(*($3->d_type), buf))
                            wdlerror(buf);
                        if (buf[0] != '\0')
                            wdlerror(buf);
                        sp->d_type = $1->d_type;
                    }
                    delete $1; delete $3;
		  }
		;

wql_insert_list:  wql_insert_list ',' wql_insert {
			$1->append($3);
			$1->paren_count++;
			$$ = $1;
		  }
		|
		wql_insert
                {
                    $$ = $1;
                }
		;

wql_insert:	
		{
			table_elem_count = 0;
		} '(' wql_value_list ')' {
			source *sp = new source;
                        /* Check the column count. */
                        VarNode *vn = vstore[current_table];
                        if (!vn || vn->type.tc != TC_DBTABLE || !vn->type.info)
                            wdlerror("Unknown table.");
                        TableTypeInfo *tti=(TableTypeInfo*)vn->type.info;
                        if ($3->paren_count != tti->count) {
                            wdlerror("Incorrect number of columns in insert.");
			}

                        SourceAppend5(sp,
                                "\n\t\twr_mrow(wr_mvals(",
                                tti->count,", ", $3,"), ");
			sp->paren_count = 1;
			$$=sp;
		  }
		;

wql_value_list:  wql_value_list ',' wql_value {
                        $1->append(",  ");
			$1->append($3);
			$1->paren_count++;
			$$ = $1;
		  }
		| wql_value
		;

wql_selection:    K_WHERE wql_expression {
        source *sp = new source;
        /* Generate variable name. */
        char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
        snprintf(name, sizeof name, "wr_db_%d", variable_generator_count++);
        /* Stash creation instructions. */
        SourceAppend9(creation_instructions,
                "if (",name," == NULL) {\n",
                "\t",name," = ",$2->src,";\n",
                "}\n");
        /* Stash destruction instructions. */
        SourceAppend10(destruction_instructions,
                "if (", name, " != NULL) {\n","\twql_free_criteria(",
                name, ");\n","\t", name, "= NULL;\n","}\n");
        /* Stash variable declaration. */
        SourceAppend3(var_decls, "static wqsx *", name, "=0;\n");
        /* Return variable. */
        sp->append(name);
        $$ = sp;
        
        /* Get tti */
        VarNode *vn = vstore[current_table];
        TableTypeInfo *tti = (TableTypeInfo*)vn->type.info;
        /* Flatten $2->polluted. */
        list<int> temp_list;
        transform( $2->polluted->begin(), $2->polluted->end(),
                back_inserter(temp_list),
                select1st<pair<int, String> >());

        if (tti->keyfields->size() > 0 &&
           equal(temp_list.begin(), temp_list.end(), tti->keyfields->begin())) {
            table_key_field = 1;
            //printf("Detected keyed query.\n");
        }
        delete $2->src; delete $2;
    }
    | {
        source *sp = new source;
        sp->append("NULL");
        $$ = sp;
    }
    ;

wql_expression:
                wql_and_expression
                {
                    $$ = $1;
                }
		| wql_expression K_WQL_OR wql_and_expression
                {
                    pollutionTree *pt = new pollutionTree;
                    SourceAppend5(pt->src,
                            "wr_mwqsx_or(",
                            $1->src,
                            ",",
                            $3->src,
                            ")" );

               //   cout << *($1->polluted) <<" Or "<< *($3->polluted) << endl;
                    if (!(table_not_toggle%2)) {
                        /* Or: Intersect $1->polluted and $2->polluted.
                         * If any conflicts, warn.
                         */
                        
                        set_pair_i_S::iterator i=pt->polluted->begin();
                        set_intersection($1->polluted->begin(),
                                $1->polluted->end(),
                                $3->polluted->begin(),
                                $3->polluted->end(),
                                inserter(*(pt->polluted),i));

                   //   cout << "Intersection: " << *(pt->polluted) << endl;
                        if (check_duplicates(*(pt->polluted)))
                            wdlwarning("Questionable \"where\" clause.");
                    } else {
                        /* And: Union $1->polluted and $2->polluted.
                         * If any conflicts, error out.
                         */
                        set_pair_i_S::iterator i=pt->polluted->begin();
                        set_union($1->polluted->begin(),
                                $1->polluted->end(),
                                $3->polluted->begin(),
                                $3->polluted->end(),
                                inserter(*(pt->polluted),i));

                    //  cout << "Union: " << *(pt->polluted) << endl;
                        if (check_duplicates(*(pt->polluted)))
                            wdlerror("Bad \"where\" clause.");
                    }

		    $$=pt;
                    
                    delete $1->src; delete $1; delete $3->src; delete $3;
                }
		;

wql_and_expression:
                wql_primary_expression
                {
                    $$ = $1;
                }
		| wql_and_expression K_WQL_AND wql_primary_expression
                {
                    pollutionTree *pt = new pollutionTree;
                    SourceAppend5(pt->src,
                            "wr_mwqsx_and(",
                            $1->src,
                            ",",
                            $3->src,
                            ")" );
                //  cout << *($1->polluted) <<" And "<< *($3->polluted) << endl;

                    if (table_not_toggle%2) {
                        /* Or: Intersect $1->polluted and $2->polluted.
                         * If any conflicts, warn.
                         */
                        
                        set_pair_i_S::iterator i=pt->polluted->begin();
                        set_intersection($1->polluted->begin(),
                                $1->polluted->end(),
                                $3->polluted->begin(),
                                $3->polluted->end(),
                                inserter(*(pt->polluted),i));
                    //  cout << "Intersection: " << *(pt->polluted) << endl;
                        if (check_duplicates(*(pt->polluted)))
                            wdlwarning("Questionable \"where\" clause.");
                    } else {
                        /* And: Union $1->polluted and $2->polluted.
                         * If any conflicts, error out.
                         */
                        set_pair_i_S::iterator i=pt->polluted->begin();
                        set_union($1->polluted->begin(),
                                $1->polluted->end(),
                                $3->polluted->begin(),
                                $3->polluted->end(),
                                inserter(*(pt->polluted),i));

                    //  cout << "Union: " << *(pt->polluted) << endl;
                        if (check_duplicates(*(pt->polluted)))
                            wdlerror("Bad \"where\" clause.");
                    }

		    $$=pt;
                    
                    delete $1->src; delete $1; delete $3->src; delete $3;
                }
		;

wql_primary_expression:
                wql_terminal_expression
                {
                    source *sp = new source;
                    sp->append("wr_mwqsx_terminal(");
                    sp->append($1->src);
                    sp->append(")");

                    delete $1->src;
                    $1->src = sp;
                    $$ = $1;
                }
		| '(' wql_expression ')'
                {
                    $$ = $2;
                }
		| '!' '(' {++table_not_toggle;} wql_expression ')'
                {
                    source *sp = new source;
                    sp->append("wr_mwqsx_not(");
                    sp->append($4->src);
                    sp->append(")");

                    delete $4->src;

                    $4->src = sp;
                    /* Passthrough polluted. */
		    $$ = $4;
                    --table_not_toggle;
                }
		;

wql_terminal_expression:
                wql_column_name wql_expression_op wql_select_value
                {
                    /* Reduce a wql selection criteria primary expression
                     * to a full blown C expression??  Overkill?
                     * My life becomes easier if "expression" is limited
                     * to WQL expressions, and returns code that generates
                     * a sxval_t object.
                     * So I made that change.
                     * DSK
                     * TBD: make it possible for the wql_select_value to
                     * degenerate into full blown C expression.
                     * (where foo.column < (i) ) might be a popular choice.
                     */
                    /* Return a source object that contains code to create
                     * an tx_t to represent this one simple expression.
                     */
                    pollutionTree *pt = new pollutionTree;
                    SourceAppend8(pt->src,
                            "wr_mtx(",
                            "wr_msxval_field(",
                            $1->all(), /* $1 is a source => 3. */
                            "),",
                            $3->all(), /* $3 is a source that creates a sxval */
                            ",",
                            $2, /* $2 is a string => "gte_op" */
                            ")");
                    
                    /* Keyed tables Hack. */
                    if (
                            ( (!(table_not_toggle%2))&&(strcmp($2,"eq_op")==0))
                            ||
                            ((table_not_toggle%2)&&(strcmp($2,"ne_op")==0))
                            ) {
                        if (current_table) {
                            VarNode *vn = vstore[current_table];
                            TableTypeInfo *tti =(TableTypeInfo*)vn->type.info;
                            char *c = $1->all();
                            int i = atoi(c);
                            if (tti->keyfields->find(i)!=tti->keyfields->end()){
                                String s=$3->all();
                                pair_i_S p(i,s);
                                pt->polluted->insert(p);
                            }
                        }
                    }
                    /* Type-checks: Assuse $3.d_type && $1.d_type is set */
                    if (typechk) {
                        if (!$3->d_type || !$1->d_type)
                            wdlerror("Internal compilier error WQL1");
                        char buf[256] = "";
                        if (!$1->d_type->assmt_chk(*($3->d_type), buf))
                            wdlerror(buf);
                        if (buf[0] != '\0')
                            wdlwarning(buf);
                        pt->src->d_type = &intType;
                    }

		    $$=pt;
                    delete $1; delete $3;
                }
                ;

wql_expression_op: '='      { $$ = "eq_op";}
                | '<'       { $$ = "lt_op";}
                | K_LTEQ    { $$ = "lte_op";}
                | '>'       { $$ = "gt_op";}
                | K_GTEQ    { $$ = "gte_op";}
                | K_NOTEQ   { $$ = "ne_op";}
                | K_LIKE    { $$ = "like_op";}
		;

wql_select_value: wql_column_name
                {
                    source *sp = new source;
                    sp->append("wr_msxval_field(");
                    sp->append($1);
                    sp->append(")");
                    $$ = sp;
                    
                    if (typechk) {
                        if (!current_table) {
                            wdlerror("Internal Compilier Error WQL2.");
                        }
                        sp->d_type = $1->d_type;
                    }
                    delete $1;
                }
                | S_NUMCONST
                {
                    source *sp = new source;
                    sp->append("wr_msxval_lit_int(");
                    sp->append($1);
                    sp->append(")");
                    
                    if (typechk)
                        sp->d_type = &intType;
                    $$ = sp;
                }
                | S_FLOATCONST
                {
                    source *sp = new source;
                    sp->append("wr_msxval_lit_float(");
                    sp->append($1);
                    sp->append(")");
                    if (typechk)
                        sp->d_type = &floatType;
                    $$ = sp;
                }
                | string_list
                {
                    source *sp = new source;
                    char *string = $1->code->all();
                    sp->append("wr_msxval_lit_string(PD_MAKESTR(");
                    sp->append(string);
                    sp->append("))");
                    if (typechk)
                        sp->d_type = &carrType;
                    $$ = sp;
                    
                    free(string);
                    delete $1;
                }
                | S_IDENTIFIER {
                    /* Its a variable. */
                    source *sp = new source;
                    source *varname = new source;
                    char *field = NULL;
                    VarNode *vn = vstore[$1];
                    if (vn == NULL) wdlerror("Variable is not declared.");
                    Type *type = &vn->type;
                    if (type->is_inttype()) {
                        sp->append("wr_msxval_lit_int(");
                        sp->d_type = &intType;
                        field = "v.l.ival";
                    } else if (type->is_floattype()) {
                        sp->append("wr_msxval_lit_float(");
                        sp->d_type = &floatType;
                        field = "v.l.fval";
                    } else if (type->is_strtype()) {
                        sp->append("wr_msxval_lit_string(");
                        sp->d_type = &cstrType;
                        field = "v.l.cval";
                    } else {
                        wdlerror("Invalid type for db operation.");
                    }
                    /* $1 might be a WIV variable.  Test for that here and
                     * generate appropriate code.
                     */
                    int seqvar = vn->seqvar;
                    int wivvar = vn->sflag & WIV_VAR;
                    
                    if (seqvar) {
                        SourceAppend3(varname, "(my_seq_data->",$1,")");
                    } else if (wivvar) {
                        if (in_no_wiv_scope)
                            wdlerror("WIVs not allowed in scope.", $1);
                        else
                            SourceAppend3(varname, "WSS_GET_WIV_VAR(",$1,")");
                    } else {
                        varname->append($1);
                    }
                    SourceAppend2(sp, varname->all(), " )");

                    /* Its also dynamic, and must be reevaluated each time. */
                    /* Generate variable name. */
                    char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                    snprintf(name, sizeof name, "wr_db_%d",
                            variable_generator_count++);
                    /* Stash creation instructions. */
                    SourceAppend17(creation_instructions,
                            "if (", name, " == NULL) {\n","\t",
                            name, " = ", sp, ";\n","} else {\n","\t",
                            name, "->", field , " = (", varname, ");\n","}\n");
                    /* Stash variable declaration. */
                    SourceAppend3(var_decls, "static sxval *", name, "=0;\n");

                    /* Return variable. sp was emptied out..*/
                    sp->append(name);
                    delete varname;
                    $$ = sp;
                }
		| '$' S_IDENTIFIER { 
                    if (ignore_event) {
                        $$ = new source;
                    } else {
                        if (event_set == NULL)
                            wdlerror("Fatal error: event is NULL");
                        if (event_set->empty()) 
                            wdlerror("Fatal error: event set empty");
                        source *sp = new source;
                        mlabel *l, *l1, *l2; 
                        WrEvent *ev = essgrp.is_valid_rattr(event_set, $2, &l);
                        WrEvent *ev1 = essgrp.is_valid_attr(event_set, $2, &l1);
                        Param *p  = essgrp.is_valid_var(event_set, $2, &l2);

                        if (p != NULL) {
                            Type *type;
                            char *field = NULL;
                            type = p->d_type;
                            // $variable is an internal variable
                            if (type->is_inttype()) {
                                sp->append("wr_msxval_lit_int(");
                                sp->d_type = &intType;
                                field = "v.l.ival";
                            } else if (type->is_floattype()) {
                                sp->append("wr_msxval_lit_float(");
                                sp->d_type = &floatType;
                                field = "v.l.fval";
                            } else if (type->is_strtype()) {
                                sp->append("wr_msxval_lit_string(");
                                sp->d_type = &cstrType;
                                field = "v.l.cval";
                            } else {
                                wdlerror("Invalid type for db operation.");
                            }
                            SourceAppend2(sp, "*", p->name);
                            sp->append(")");

                            /* Its dynamic, and must be reevaluated each time.*/
                            /* Generate variable name. */
                            char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                            snprintf(name, sizeof name, "wr_db_%d",
                                    variable_generator_count++);
                            /* Stash creation instructions. */
                            SourceAppend13(creation_instructions,
                                    "if (", name, " == NULL) {\n\t",
                                    name, " = ", sp, ";\n} else {",
                                    name, "->",field," = (*",p->name,");\n}\n");
                            /* Stash variable declaration. */
                            SourceAppend3(var_decls,
                                    "static sxval *",name,"=0;\n");

                            /* Return variable. sp was emptied out..*/
                            sp->append(name);
                        } else if (ev == NULL) {
                            Type *type;
                            char *field = NULL;
                            WrEvent *e = essgrp.get_event(*event_set->begin());
                            type = e->ret->d_type;
                            // $variable is a return value, offset set to "-1"
                            if (type->is_inttype()) {
                                sp->append("wr_msxval_lit_int(");
                                sp->d_type = &intType;
                                field = "v.l.ival";
                            } else if (type->is_floattype()) {
                                sp->append("wr_msxval_lit_float(");
                                sp->d_type = &floatType;
                                field = "v.l.fval";
                            } else if (type->is_strtype()) {
                                sp->append("wr_msxval_lit_string(");
                                sp->d_type = &cstrType;
                                field = "v.l.cval";
                            } else {
                                wdlerror("Invalid type for db operation.");
                            }

                            sp->append("((");
                            sp->append(l->para->type->all());
                            sp->append(") ");
                            sp->append("*retval))");

                            /* Its dynamic, and must be reevaluated each time.*/
                            /* Generate variable name. */
                            char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                            snprintf(name, sizeof name, "wr_db_%d",
                                    variable_generator_count++);
                            /* Stash creation instructions. */
                            SourceAppend13(creation_instructions,
                                    "if (", name, " == NULL) {\n\t",
                                    name, " = ", sp, ";\n} else {",
                                    name, "->", field ,
                                    " = ((",l->para->type->all(),
                                    ") *retval);\n}\n");
                            /* Stash variable declaration. */
                            SourceAppend3(var_decls,
                                    "static sxval *",name,"=0;\n");

                            /* Return variable. sp was emptied out..*/
                            sp->append(name);
                        } else if (ev1 == NULL) {
                            Type *type;
                            char *field = NULL;
                            WrEvent *e = essgrp.get_event(*event_set->begin());
                            type = e->find_plabel($2)->para->d_type;
                            // $variable is a parameter 
                            if (type->is_inttype()) {
                                sp->append("wr_msxval_lit_int(");
                                sp->d_type = &intType;
                                field = "v.l.ival";
                            } else if (type->is_floattype()) {
                                sp->append("wr_msxval_lit_float(");
                                sp->d_type = &floatType;
                                field = "v.l.fval";
                            } else if (type->is_strtype()) {
                                sp->append("wr_msxval_lit_string(");
                                sp->d_type = &cstrType;
                                field = "v.l.cval";
                            } else {
                                wdlerror("Invalid type for db operation.");
                            }

                            source *ofsrc=essgrp.gen_attr_offset(event_set, $2);
                            sp->append("((");
                            sp->append(l1->para->type->all());
                            sp->append(") ");
                            sp->append("uap[ ");
                            sp->append(ofsrc->all());
                            sp->append(" ]))");

                            /* Its dynamic, and must be reevaluated each time.*/
                            /* Generate variable name. */
                            char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                            snprintf(name, sizeof name, "wr_db_%d",
                                    variable_generator_count++);
                            /* Stash creation instructions. */
                            SourceAppend19(creation_instructions,
                                    "if (", name, " == NULL) {\n","\t",
                                    name, " = ", sp, ";\n","} else {\n", "\t",
                                    name, "->", field , " = ((",
                                    l1->para->type->all(), ") uap[ ",
                                    ofsrc,"]);\n","}\n");
                            /* Stash variable declaration. */
                            SourceAppend3(var_decls,
                                    "static sxval *",name,"=0;\n");

                            /* Return variable. sp was emptied out..*/
                            sp->append(name);
                        } else 
                            wdlerror("Attribute is invalid in event",ev1->name);
                        // sp->lvalue = 0;
                        $$ = sp;
                    }
		  }
		| '$' '#' {
                    source *sp = new source;
                    sp->append("wr_msxval_lit_int(code)"); 
                    sp->d_type = &intType;
                    // sp->lvalue = 0;		
                    $$ = sp;

                    /* Its dynamic, and must be reevaluated each time. */
                    /* Generate variable name. */
                    char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                    snprintf(name, sizeof name, "wr_db_%d",
                            variable_generator_count++);
                    /* Stash creation instructions. */
                    SourceAppend13(creation_instructions,
                            "if (", name, " == NULL) {\n","\t",
                            name, " = ", sp, ";\n","} else {\n","\t",
                            name, "->v.l.ival = code;\n",
                            "}\n");
                    /* Stash variable declaration. */
                    SourceAppend3(var_decls, "static sxval *", name, "=0;\n");

                    /* Return variable. sp was emptied out..*/
                    sp->append(name);
                    $$ = sp;
		}
                | '$' '$'
                {
                    source *sp = new source;
                    sp->append("wr_msxval_lit_string(wr_syscall_name(code))"); 
                    sp->d_type = &cstrType;
                    // sp->lvalue = 0;		
                    $$ = sp;

                    /* Its dynamic, and must be reevaluated each time. */
                    /* Generate variable name. */
                    char name[6 + 5 + 1]; /* "wr_db_%-5d\0" */
                    snprintf(name, sizeof name, "wr_db_%d",
                            variable_generator_count++);
                    /* Stash creation instructions. */
                    SourceAppend13(creation_instructions,
                            "if (", name, " == NULL) {\n","\t",
                            name, " = ", sp, ";\n","} else {\n","\t",
                            name, "->v.l.cval = wr_syscall_name(code);\n",
                            "}\n");
                    /* Stash variable declaration. */
                    SourceAppend3(var_decls, "static sxval *", name, "=0;\n");

                    /* Return variable. sp was emptied out..*/
                    sp->append(name);
                    $$ = sp;
                }
            ;

wql_column_name: '.' S_IDENTIFIER  {
                     /* returns a source object that just contains the
                      * name of the field.
                      */
                    int col_num;
                    source *sp = new source;
		    $$=sp;
                    if (!current_table) {
                        wdlerror("Internal Compilier Error: WQL13");
                    }
                    VarNode *vn = vstore[current_table];
                    if (!vn || vn->type.tc != TC_DBTABLE || !vn->type.info)
                        wdlerror("Unknown table.");
                    TableTypeInfo *tti=(TableTypeInfo*)vn->type.info;

                    if (!(vn = (*tti->vtbl)[$2]))
                        wdlerror("Unknown column name.");
                    col_num = (*tti->order)[$2];
                    if ( col_num >= tti->count )
                        wdlerror("Internal Compilier Error: WQL12");

                    sp->d_type = &(vn->type);
                    /*
                     * Inverse the count to correspond with the internal
                     * count.
                     * 
		     * sp->append(tti->count - col_num);
                     */
		    sp->append(col_num);
		  }
		;

wql_value:	  assignment_expression
		{
			source *sp = new source;
                        $1->code->d_type = &($1->type);
			sp->gen_wql_value($1->code);
			$$ = sp;
                        table_elem_count++;
		}
                ;

wql_table_declaration_body : wql_table_column ';'
                | wql_table_declaration_body wql_table_column ';'
                {
                    $1->append(",  ");
                    $1->append($2);
                    $$ = $1;
                    delete $2;
                }
                ;
wql_key_field_opt : K_KEY
                {
                    table_key_fields->insert(table_elem_count);
                }
                |
                ;
                
wql_table_column: wql_type wql_key_field_opt S_IDENTIFIER
                {
			source *sp = new source;
                        /* Returns three arguments minus the trailing comma.
                         * like:
                         * "strdup(\"foo\"), int_t, 0"
                         * The list rule will add the trailing comma if
                         * necessary.
                         */
                        /* wql_type returns a source containing two arguments
                         * without a trailing comma.  Like:
                         * "char_t, 256"
                         */
			/* note: we strdup() the col name so that it can
			   be free'd in wql_create() */
			sp->append("wr_strdup(PD_MAKESTR(\"");
			sp->append($3);
			sp->append("\")),");
			sp->append($1);
                        sp->d_type = $1->d_type;
                        VarNode *vn = new VarNode($3, *$1->d_type);
                        if (!vstore.toptbl()->insert($3, vn))
                            wdlerror("Duplicated definition of field");
                        if (table_order)
                            table_order->insert($3, table_elem_count);
                        if (table_rev_map) {
                            char buf[5]; /* Only 1000 table elements. */
                            snprintf(buf, sizeof buf,"%d",table_elem_count);
                            table_rev_map->insert(buf,$3);
                        }
                        table_elem_count++;
			$$=sp;
                }
                ;

wql_type:	  K_WSS_CHAR_T '(' S_NUMCONST ')' {
		    source *sp = new source;
		    sp->append(" pd_char_t_t, ");
		    sp->append($3);
                    sp->d_type = 
                         new Type(TC_PTR, new PtrTypeInfo(charType));
		    $$=sp;
		  }
		| K_FLOAT {
                    source *sp = new source;
		    sp->append(" float_t, 0");
                    sp->d_type = &floatType;
		    $$=sp;
		  }
		| K_INT {
                    source *sp = new source;
		    sp->append(" int_t, 0");
                    sp->d_type = &intType;
		    $$=sp;
		  }
		;

wql_update_expression : wql_update_additive_expression
    ;
    
wql_update_expression_list : wql_update_expression
    | wql_update_expression_list ',' wql_update_expression
    {
        $1->append(",");
        $1->append($3);
        delete $3;
        $$ = $1;
        /* Only used for function calls, which are not implimented yet. */
    }
    ;

wql_update_additive_expression: wql_update_multiplicative_expression
    | wql_update_additive_expression '+' wql_update_multiplicative_expression
    {
        source *sp = new source;
        sp->append("wr_muexp(");
        SourceAppend3(sp, $1, ", ", $3);
        sp->append(", addition)");
        $$ = sp;
        if (typechk) {
            if (!$3->d_type || !$1->d_type)
                wdlerror("Internal compilier error WQL4");
            sp->d_type = new Type($1->d_type->addop_chk(*$3->d_type));
            if (*sp->d_type == errType)
                wdlerror("Invalid operands to binary operator +");
        }
        delete $1; delete $3;
    }
    | wql_update_additive_expression '-' wql_update_multiplicative_expression
    {
        source *sp = new source;
        sp->append("wr_muexp(");
        SourceAppend3(sp, $1, ", ", $3);
        sp->append(", subtraction)");
        $$ = sp;
        if (typechk) {
            if (!$3->d_type || !$1->d_type)
                wdlerror("Internal compilier error WQL5");
            sp->d_type = new Type($1->d_type->subop_chk(*$3->d_type));
            if (*sp->d_type == errType)
                wdlerror("Invalid operands to binary operator -");
        }
        
        delete $1; delete $3;
    }
    ;
    
wql_update_multiplicative_expression: wql_update_cast_expression
    | wql_update_multiplicative_expression '*' wql_update_cast_expression
    {
        source *sp = new source;
        sp->append("wr_muexp(");
        SourceAppend3(sp, $1, ", ", $3);
        sp->append(", multiplication)");
        $$ = sp;

        if (typechk) {
            if (!$3->d_type || !$1->d_type)
                wdlerror("Internal compilier error WQL6");
            sp->d_type = new Type($1->d_type->mulop_chk(*$3->d_type));
            if (*sp->d_type == errType)
                wdlerror("Invalid operands to binary operator *");
        }
        delete $1; delete $3;
    }
    | wql_update_multiplicative_expression '/' wql_update_cast_expression
    {
        source *sp = new source;
        sp->append("wr_muexp(");
        SourceAppend3(sp, $1, ", ", $3);
        sp->append(", division)");
        $$ = sp;
        
        if (typechk) {
            if (!$3->d_type || !$1->d_type)
                wdlerror("Internal compilier error WQL7");
            sp->d_type = new Type($1->d_type->mulop_chk(*$3->d_type));
            if (*sp->d_type == errType)
                wdlerror("Invalid operands to binary operator /");
        }
        delete $1; delete $3;
    }
    ;
    
wql_update_cast_expression: wql_update_unary_expression
    | '(' type_name ')' wql_update_cast_expression
    {
        source *sp = new source;
        SourceAppend4(sp, "(", $2, ")", $4);
        sp->d_type = $2->d_type;
        // sp->lvalue = $4->lvalue;
        
        $$ = sp;
        delete $2; delete $4;
    }
    ;
    
wql_update_unary_expression: wql_update_postfix_expression
    | unary_op wql_update_cast_expression
    {
        if (!strcmp($1, "-")) {
            source *sp = new source;
            sp->append("wr_muexp(");
            SourceAppend2(sp, $2, ", NULL");
            sp->append(", negation)");
            $$ = sp;

            if (typechk) {
                if (!$2->d_type)
                    wdlerror("Internal compilier error WQL8");
                if (!$2->d_type->is_inttype() && !$2->d_type->is_floattype())
                    wdlerror("Invalid operand to unary operator -");
                sp->d_type = $2->d_type;
            }
            delete $2;
        } else
            $$ = $2;
    }
    ;
    
wql_update_postfix_expression: wql_update_primary_expression
    {
        $$ = $1;
    }
    | wql_update_function_call
    { wdlerror("Function calls not supported yet."); }
    ;
    
wql_update_primary_expression: '(' wql_update_expression ')'
    { $$ = $2 }
    | wql_select_value
    {
        source *sp = new source;
        SourceAppend3(sp, "wr_muexp_term(", $1, ")\n");
        sp->d_type = $1->d_type;
        $$ = sp;
        delete $1;
    }
    ;

wql_update_function_call: wql_update_postfix_expression '(' wql_update_expression_list ')'
    { wdlerror("No function call support in update statements.\n"); }
    | wql_update_postfix_expression '(' ')'
    { wdlerror("No function call support in update statements.\n"); }
    ;
    
    

/* ================================================================= *
 *                         Characterizations                         *
 * ================================================================= */


characterization:  K_CHARACTERIZE S_IDENTIFIER 
                   {   tmpchar = new characterization($2);   
                       in_essdef = 1;
                   }
                   '{' interface_list '}' 
                   {
                        if (esstbl.insert(tmpchar) != 1) {
                            wdlerror( "Redefinition of Characterization %s\n",
                                    $2);
                        }
                        space_set.insert($2);
			source *sp = new source;
                        SourceAppend3(sp,
                                "\n/* Using \"",$2,"\" characterization. */\n");
			$$=sp;
                        in_essdef = 0;
		   }
		;

interface_list:    interface_list interface
		|  interface 
		;

interface: 	   type_reference_complete markup_spec S_IDENTIFIER markup_spec '(' iface_arg_list ')' ';'

                  {   WrEvent *ev = new WrEvent($3);
                      ev->space = tmpchar->name;
                      ev->elabels = $4;
                      ev->set_params($6);
                      ev->ret = new Param("ret", ev, $1);
                      ev->ret->set_labels($2);
                      ev->ret->d_type = $1->d_type;
                      tmpchar->insert($3, ev); 
		      
		      /* I hope this works...
		       * When parcing the markup_spec ($4), we may have
		       * hit a reference to a parameter to the interface.  We
		       * couldnt' know that at the time since we didn't
		       * know the names of the parameters.
		       * 
		       * We could have this problem somewhere in the 
		       * iface_arg_list as well, but I'm gonna ignore that
		       * for now.  Also in the markup_spec for the return
		       * value.
		       * TBD: stop ignoring that.  DSK
		       */
		      
		      /* Go through the $4, looking for any mlabel's with a
		       * "if_clause" not null.  If I find one, search it
		       * for any SpecIdentExpr's.  For all SpecIdentExpr's,
		       * if the idtype is set to -1, it needs to be
		       * initialized.  Do stupid checks at this time as well.
		       * 
		       * For $eli in $4->elabels
		       *    if (eli->if_clause != NULL)
		       *        for $siei in eli->if_clause->get_siexpr()
		       *            if $siei->idtype == -1
		       *                $siei->idtype = get_type($siei->name);
		       *                
		       * mlabel *l;
		       */
		      for (mlabel *ml = ev->elabels ;ml!=NULL;ml=ml->next) {
			  if (ml->if_clause != NULL) {
			      list<SpecIdentExpr *>*sil =
				  ml->if_clause->get_siexpr();
			      list<SpecIdentExpr*>::iterator siei;
			      for (siei=sil->begin();siei!=sil->end();++siei) {
				  if ((*siei)->idtype == -1) {
				      mlabel *l;
				      l = ev->find_plabel((*siei)->name);
				      if (l) {
					  char s[128];
					  snprintf(s,sizeof s,"%d",
						  l->para->offset);
					  (*siei)->offset = strdup(s);
					  (*siei)->label = l;
					  (*siei)->idtype = Expr::Para;
                                          (*siei)->type = *l->para->d_type;
				      } else {
					  wdlerror("Unknown parameter",
						  (*siei)->name);
				      }
				  }
			      }
			  }
		      }
		  }
                |   type_reference_complete markup_spec S_IDENTIFIER markup_spec ';'
                  {
                      Param *p = new Param($3, $1->d_type);
                      if (tmpchar->insert_var($3, p) != 1)
                           wdlerror("Multiple declaration of variable: ", $3);
                      p->set_labels($4);
                  }
		; 

iface_arg_list:   iface_arg_list ',' iface_arg_declaration 
                  {   Param *p = $1; 
                      if (p == NULL)
                          wdlerror("Internal compilier error iface_arg_list1");
                      else {
                          while (p->next != NULL) p = p->next;
                          p->next = $3;
                      }
                      $$ = $1;
                  }
		| iface_arg_declaration  
                  {   $$ = $1;         }
                |
                  {   $$ = NULL;       }
		;

iface_arg_declaration: 
                  iface_parameter_declaration markup_spec 
                  {   $1->set_labels($2);     
                      $$ = $1;
                  }
                ;

iface_parameter_declaration: 
		  iface_declaration_specifiers iface_declarator 
                  {   source *sp = new source;
		      SourceAppend3(sp, $1, " ", $2->type);
                      $2->type = sp;
                      $2->d_type->fillin_type(*$1->d_type);
		      $$ = $2;
		  }
  		| iface_declaration_specifiers abstract_declarator 
                  {   source *sp = new source;
		      SourceAppend3(sp, $1, " ", $2);
		      $$ = new Param;
		      $$->name = strdup("No Name");
                      $$->type->append(sp);
                      $2->d_type->fillin_type(*$1->d_type);
		      $$->d_type = $2->d_type;
		  }
		| iface_declaration_specifiers  
                  {   source *sp = new source;
		      sp->append($1);
		      $$ = new Param;
		      $$->name = strdup("No Name");
                      $$->type->append(sp);
		      $$->d_type = $1->d_type;
		  }
		;

iface_declarator: pointer iface_direct_declarator  
                  {    $2->type->append($1);
                       $2->d_type->fillin_type(*$1->d_type);
		       $$ = $2;
		  }
		| iface_direct_declarator 
		;

iface_direct_declarator: 
		S_IDENTIFIER 
		  {    $$ = new Param; 
                       $$->name = strdup($1);   
		       $$->d_type = new Type(TC_TMP);
                  }
		| '(' iface_declarator ')' 
                  {    $2->type->append("()");
		       $$ = $2;
		  }
		| iface_direct_declarator '[' ']' 
                  {    $1->type->append("[]");
                       $1->d_type->fillin_type
                            (Type(TC_ARRAY,new ArrayTypeInfo(tmpType)));
		       $$ = $1;
		  }
		| iface_direct_declarator '[' constant_expression ']' 
                  {    SourceAppend3($1->type, "[", $3, "]");
                       $1->d_type->fillin_type(Type(TC_ARRAY,
                              new ArrayTypeInfo(tmpType, $3->str())));
                       $$ = $1;
		  }
		;

markup_spec:      '{' markup_stmt_list '}'
                  {    $$ = $2;   }
                |
                  {    $$ = NULL; }
		;

markup_stmt_list: markup_stmt 
                  {    $$ = $1;   } 
                | markup_stmt_list ',' markup_stmt
                  {    mlabel *m = $1; 
                       while (m->next != NULL) m = m->next;
                       m->next = $3;
                       $$ = $1;
                  }
		;

markup_stmt:      S_IDENTIFIER  
                  {    source *sp = new source;
                       sp->append($1);
                       $$ = new mlabel(mlabel::MLT_LABEL, sp);
                  }
                | S_IDENTIFIER '{' markup_stmt_list '}'                  
		  {    source *sp = new source;
                       sp->append($1);
                       $$ = new mlabel(mlabel::MLT_LABEL, sp);
                       $$->son = $3;
                  }
                | K_NTERM       {    $$ = new mlabel(mlabel::MLT_NTERM);       }
                | K_NTERMLIST   {    $$ = new mlabel(mlabel::MLT_NTERMLIST);   }
                | K_IN          {    $$ = new mlabel(mlabel::MLT_IN);          }
                | K_OUT         {    $$ = new mlabel(mlabel::MLT_OUT);         }
                | K_HANDLE
		  {
		       source *sp = new source;
		       sp->append("handle");
		       $$ = new mlabel(mlabel::MLT_HANDLE, sp);
		  }
                | K_VMAP '(' markup_expression ',' markup_expression  ')'
                  {    source *sp = new source;
                       $$ = new mlabel(mlabel::MLT_VMAP, sp, $3, $5);
                  }
                | K_SET '(' markup_expression ')'
                  {    source *sp = new source;
                       SourceAppend4(sp, "SET", "(", $3->code, ")");
                       $$ = new mlabel(mlabel::MLT_SET, sp, $3);
                  }
                | K_COPY '(' markup_expression ',' markup_expression ')' 
                  {    source *sp = new source;
                       SourceAppend6(sp,"COPY","(",$3->code,",",$5->code,")");
                       $$ = new mlabel(mlabel::MLT_COPY, sp, $3, $5);
                  }
                | K_RCOPY '(' markup_expression ',' markup_expression ')' 
                  {    source *sp = new source;
                       SourceAppend6(sp,"RCOPY","(",$3->code,",",$5->code, ")");
                       $$ = new mlabel(mlabel::MLT_RCOPY, sp, $3, $5);
                  }
		| K_TYPE '(' type_reference_complete ')' 
                  {    source *sp = new source;
                       SourceAppend3(sp, "TYPE (", $3, ")"); 
                       sp->d_type = $3->d_type;
                       $$ = new mlabel(mlabel::MLT_TYPE, sp);
                  }
                | K_IF '(' markup_or_expression ')' markup_stmt 
                  {
                      $5->if_clause = $3;
                      $$ = $5;
                  }
		;

markup_expression:
		  varref_expression
		| constant
                | unary_op constant
                  {   source *sp = new source; 
                      SourceAppend2(sp, $1, $2->code);
                      $2->code = sp;
                      ((ConstExpr *) $2)->val = $2->code->all();
                      $$ = $2;
                  }
                | '*'
                  {   Expr *e = new ConstExpr("*", intType);
                      e->code->append("*");
                      $$ = e;
                  }
                | '*' varref_expression 
                  {   Expr *e = new PtrRefExpr($2);
                      SourceAppend2(e->code, "*", $2->code);
                      $$ = e;
                  }

		;
                
markup_or_expression:
	markup_and_expression
	| markup_or_expression K_OR markup_and_expression {
	    Expr *e = new BinaryExpr("||", $1, $3);
	    if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
		if (!$1->type.logop_chk($3->type))
		    wdlerror("Invalid operands to binary operator ||");
	    }
	    /* else { Can't test types... Eeek. } */
	    /* What I'm copying from updates e->code.  I can't yet. */
	    e->type = intType;

	    $$ = e;
	}
	;
	
markup_and_expression:
	markup_bitwise_or_expression
	| markup_and_expression K_AND markup_bitwise_or_expression {
	    Expr *e = new BinaryExpr("&&", $1, $3);
	    if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
		if (!$1->type.logop_chk($3->type))
		    wdlerror("Invalid operands to binary operator &&");
	    }
	    /* else { Can't test types... Eeek. } */
	    /* What I'm copying from updates e->code.  I can't yet. */
	    e->type = intType;
	    
	    $$ = e;
	}
	;

markup_bitwise_or_expression
        :  markup_bitwise_xor_expression
        |  markup_bitwise_or_expression '|' markup_bitwise_xor_expression {
               Expr *e = new BinaryExpr("|", $1, $3);
               if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
                   if (!$1->type.bitop_chk($3->type))
                        wdlerror("Invalid operands to binary operator |");
                   e->type = $1->type;
               } else 
                   e->type = intType;
               $$ = e;
           }    
        ;

markup_bitwise_xor_expression
        :  markup_bitwise_and_expression
        |  markup_bitwise_xor_expression '^' markup_bitwise_and_expression  {
                Expr *e = new BinaryExpr("^", $1, $3);
                if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
                    if (!$1->type.bitop_chk($3->type))
                        wdlerror("Invalid operands to binary operator ^");
                    e->type = $1->type;
                } else
                    e->type = intType;
                $$ = e;
           }
        ;   
        
markup_bitwise_and_expression
        :  markup_equality_expression
        |  markup_bitwise_and_expression '&' markup_equality_expression  {
               Expr *e = new BinaryExpr("&", $1, $3);
               if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
                   if (!$1->type.bitop_chk($3->type))
                        wdlerror("Invalid operands to binary operator &");
                   e->type = $1->type;
               } else
                   e->type = intType;
               $$ = e;
           }
        ;
            
markup_equality_expression
	:  markup_relational_expression
	|  markup_equality_expression equality_op markup_relational_expression {
	      Expr *e = new BinaryExpr($2, $1, $3);
	      if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
	 	 if (!$1->type.equop_chk($3->type))
		    wdlerror("Invalid operands to binary operator", $2);
	      }
	      /* else { Can't test types... Eeek. } */
	      /* What I'm copying from updates e->code.  I can't yet. */
	      e->type = intType;
	    
	      $$ = e;
	   }
	;


markup_relational_expression
	:  markup_primary_expr 
	|  markup_relational_expression relational_op markup_primary_expr {
	      Expr *e = new BinaryExpr($2, $1, $3);
	      if ($1->type.tc != TC_ERR && $3->type.tc != TC_ERR) {
		if (!$1->type.relop_chk($3->type))
		    wdlerror("Invalid operands to binary operator", $2);
	      }
	      /* else { Can't test types... Eeek. } */
	      /* What I'm copying from updates e->code.  I can't yet. */
	      e->type = intType;

	      $$ = e;
	   }
	;

markup_primary_expr:
	S_IDENTIFIER {
	    /* identifier.  Must be argument to the syscall, although
	     * we can't check that yet.
	     * 
	     * Since we can't generate the source now (since any leaf of
	     * the expression might be a parameter reference) we have to
	     * limit ourselves to Expr sub-types that can generate the
	     * correct source when asked to.
	     * 
	     * After each syscall characterization is read, we will go through
	     * looking for these partial structures, and tell the 
	     * SpecIdent expressions about the parameter they represent.
	     */
	    /* Little magic here.  This object just contains its type
	     * (siexpr->etype() == Expr::SpecIdent) and the name of the
	     * spec ident expression.  We will validate the name much
	     * later.
	     */
	    SpecIdentExpr *siexpr = new SpecIdentExpr($1);
	    siexpr->offset = NULL;
	    siexpr->label = NULL;
	    siexpr->idtype = (Expr::IdType)-1;
	    siexpr->type.tc = TC_ERR;
	    $$ = siexpr;
	}
	| constant {
	    $$ = $1;
	    /* $1 is already an Expr type. */
	}
	| '(' markup_or_expression ')' {
	    $$ = $2;
	}
	;

varref_expression: S_IDENTIFIER {
                    Expr *e =  new IdentExpr($1);
                    e->code->append($1);
                    $$ = e;
                  }
                | varref_expression '[' expression ']' {
                    Expr *e = new ArrayElmExpr($1, $3);
                    SourceAppend4(e->code, $1->code, " [", $3, "]");
		    $$ = e;
                  }             
		| varref_expression '.' S_IDENTIFIER {
                    Expr *e = new RecElmExpr($1, $3);
                    SourceAppend3(e->code, $1->code, ".", $3);
		    $$ = e;
		  }
		| varref_expression K_INDIRECT S_IDENTIFIER {
                    Expr *e = new PtrToElmExpr($1, $3);
                    SourceAppend3(e->code, $1->code, "->", $3);
		    $$ = e;
		  }
		;

%%
#include "lex.wdl.c"
extern FILE *wdlin;
extern int lineno;
#if YYDEBUG != 0
extern int wdldebug;
#endif
int
wr_parse_prototypes(FILE *fp)
{
#if YYDEBUG != 0
        wdldebug = 0;
#endif
	wdlin = fp;
	return(wdlparse());
}
